//$Id$

function addValue(object, keySet, value)
{
    /*
     *  author: abdul-zu381 :)
     */

    "use strict"; //No I18N
    keySet = keySet.replace(/\[(\w+)\]/g, '.$1'); // convert indexes to properties
    keySet = keySet.replace(/^\./, ''); // strip a leading dot
    var keys = keySet.split('.');
    var add = function(obj)
    {
        var key = keys.shift();
        if (key === null || keys.length === 0)
        {
            obj[key] = value;
            return false;
        }
        else if (obj[key] === null || typeof obj[key] !== "object")
        {
            obj[key] = !isNaN(parseInt(keys[0])) ? [] : {};
        }
        add(obj[key]);
    };
    add(object);
    return object;
};

function drawChart(data, width, height, plotOnly, id,isMapchart)
{
    var isMulti = (data.metadata.axes.y.length > 1) ? true : false;

    plotOnly = (plotOnly) ? true : false;
    isMulty = (isMulti) ? true : false;

    if (plotOnly)
    {
        data = restToPlotData(data, isMulti);
        data = disableThreshold(data);
    }
    
    /*
     * Change plot mode to svg
     */
    
    addValue(data, "chart.plot.renderer.mode", "svg"); //No I18N
    data = disableAnimation(data);
    
    addValue(data, "canvas.events.onload", function() { //No I18N
        window.canvasLoaded = true;
    });

    /*
     * Chart Holder
     */

    var container = document.getElementById(id);
    container.style.width = width + "px";
    container.style.height = height + "px";

    /* Draw The Chart */
    var chartObj ;
    if(isMapchart )
    {
        chartObj= new $ZC.mapcharts(container, data) ;
        var tileLayer = container._ZCMapTileLayer;
 	tileLayer.on("load",function()
 	{
            window.tilesLoaded=chartObj.chartRendered?true:false;    
 	});
    }
    else
    {
        chartObj = new $ZC.charts(container, data);
    }
    
    window.chartObj = chartObj;
    return chartObj;
}

function restToPlotData(data, isMulti) {

    /*
     * Genetate only the plot.
     */

    addValue(data, 'canvas.border.show', false); //No I18N
    addValue(data, 'canvas.shadow.shadowColor', 'rgba(0,0,0,0)'); //No I18N

    addValue(data, 'canvas.background.alpha', '0'); //No I18N
    addValue(data, 'chart.plot.background.color', 'rgba(0,0,0,0)'); //No I18N
    addValue(data, 'chart.axes.xaxis.grid.color', 'rgba(0,0,0,0)'); //No I18N
    addValue(data, 'chart.axes.yaxis[0].grid.color', 'rgba(0,0,0,0)'); //No I18N
    addValue(data, 'chart.axes.yaxis[0].axisline.color', 'rgba(0,0,0,0)'); //No I18N
    addValue(data, 'chart.axes.xaxis.axisline.color', 'rgba(0,0,0,0)'); //No I18N

    addValue(data, 'chart.marginTop', '0'); //No I18N
    addValue(data, 'chart.marginRight', '0'); //No I18N
    addValue(data, 'chart.marginBottom', '0'); //No I18N
    addValue(data, 'chart.marginLeft', '0'); //No I18N

    addValue(data, 'credits.enabled', 'false'); //No I18N

    addValue(data, 'canvas.title.show', 'false'); //No I18N
    addValue(data, 'canvas.subtitle.show', 'false'); //No I18N
    addValue(data, 'notes.enabled', 'false'); //No I18N

    addValue(data, 'chart.axes.xaxis.show', 'false'); //No I18N

    if (isMulti)
    {
        for (var i = 0; i < data.metadata.axes.y.length; i++) {
            addValue(data, 'chart.axes.yaxis[' + i + '].show', 'false'); //No I18N
        }
    }
    else
    {
        addValue(data, 'chart.axes.yaxis[0].show', 'false'); //No I18N
    }

    addValue(data, 'legend.enabled', 'false'); //No I18N
    return data;
}

function getChartDetails(type)
{
    if (type == null) {
        return
    }
    var chartType,
        isInt = !(isNaN(parseInt(type)));
    var chartType = {};
    chartType.value = (isInt) ? parseInt(type) : ($ZC.charttype[type.toString().toLowerCase()]);
    chartType.name = (isInt) ? $ZC.charttypenames.get(parseInt(type)) : type.toString().toLowerCase();
    return chartType;
}

function disableAnimation(data)
{
    var charttype;
    var chartdata = null;
    if(data.seriesdata && data.seriesdata.chartdata) {
        chartdata = data.seriesdata.chartdata;
    }
    if(chartdata)
    {
        chartdata.forEach(function(seriesdata, i) {
            charttype = chartdata[i].type || data.seriesdata.type;
            addValue(data, "chart.plot.animation.enabled", false); //No I18N
            addValue(data, "chart.plot.plotoptions." + getChartDetails(charttype).name + ".animation.enabled", false); //No I18N
        });
    }
    return data;
}

function disableThreshold(data)
{
    var chartdata = data.seriesdata.chartdata;
    addValue(data, "chart.axes.xaxis.threshold.line.show", 'false'); //No I18N
    addValue(data, "chart.axes.xaxis.threshold.range.show", 'false'); //No I18N

    chartdata.forEach(function(seriesdata, i) {
        addValue(data, "chart.axes.yaxis[" + i + "].threshold.line.show", 'false'); //No I18N
        addValue(data, "chart.axes.yaxis[" + i + "].threshold.range.show", 'false'); //No I18N
    });
    return data;
}
