//$Id$
var ZRElementMask = (function(window,document){

    // el_list - reference document element
    var el_list = document.createElement("div");

    //skip list - to skip particular prop on setAttribute
    var skipList = {
        class : true,
        id : true
    };

    /**
     * Class for ElementMask - end object (meta with wrapped element);
     * @param {mask} attributes
     */
    function ElementMask(mask){
        this.Element = mask.element;
        this._attrs = mask.attributes !== undefined && (typeof mask.attributes) === "object" ? mask.attributes : {}; //No i18N 
        for(var key in this._attrs){
            _key = key.toLowerCase();
            this._attrs[_key] = this._attrs[key]; 
        }
    }

    var el_prototype = ElementMask.prototype;

    /**
     * To set specified value of prop
     * @param {string} prop
     * @param {string} value
     */
    el_prototype.setAttribute = function(prop , value, skip){
    	if((typeof prop) !== "string"){
    		prop = prop.toString();
    	}
    	if((typeof value) !== "string"){
    		value = value.toString();
    	}
        prop = prop.toLowerCase();
        if(skipList[prop] || skip){
            this.Element.setAttribute(prop, value);
        }
        else{
            this._attrs[prop] = value;
        }
    }

    /**
     * To get specified value of prop
     * @param {string} prop
     */
    el_prototype.getAttribute = function(prop){
        prop = prop.toLowerCase();
        if(skipList[prop]){
            return this.Element.getAttribute(prop);
        }
        return this._attrs[prop] === undefined ? null : this._attrs[prop];
    }

    /**
     * To get a cloned copy of Element with mask 
     * @param {boolean} withAttr
     */
    el_prototype.cloneNode = function(withAttr){
        var element = this.Element.cloneNode(withAttr);
        var _attr = {};
        if(withAttr){
            for(var at in this._attrs){
                _attr[at] = this._attrs[at];
            }
        }
        return new ElementMask({element : element, attributes : _attr});
    }
    
    el_prototype.hasAttribute = function(prop){
    	prop = prop.toLowerCase();
    	if(skipList[prop]){
    		return this.Element.hasAttribute(prop);
    	}
    	return prop in this._attrs;
    }
    
    el_prototype.stripAttr = function(){

    }

    /**
     * Setter and getter mapping for existing proto values
     * values include (number || string || null)
     * @param {el_prototype} proto
     * @param {string} prop
     */
    function maskSetter(proto, prop){
        if(!proto[prop]){
            Object.defineProperty(proto, prop,
            {
                set : function(value){
                    this.Element[prop] = value;
                },
                get : function(){
                    return this.Element[prop]
                }
            });
        }
    }

    /**
     * Adding Function - behave as super class model
     * @param {el_prototype} proto
     * @param {string} prop
     */
    function maskFunction(proto, prop){
        if(!proto[prop]){
            proto[prop] = function(){
               return this.Element[prop].apply(this.Element, arguments);
            }
        }
    }

    /**
     * Iterating all properties of el_list and adding to el_prototype
     */
    for(var prop in el_list){
        if(typeof el_list[prop] === "function"){
            maskFunction(el_prototype, prop);
        }
        else{
            maskSetter(el_prototype, prop);
        }
    }
    // return class as module
    return ElementMask;
})(window, document);

(function(window){

    var lazyLoadMap = {}, lazyLoadedModules = {};

    var Builder = null;
    var Util = null;
    var ZRDashboard = null;
    
    

    var initialConfiguration = {
        spacing : 0,
        mode : "editMode" //No i18N
    };

    var modes = {
        "viewMode" : {  //No i18N
            move : false,
            resize : false,
            dropADD : false,
        },
        "editMode" : { //No i18N
            move : true,
            resize : true,
            dropADD : true
        }
    };

    // template cache
    var UFTemplate = null, ZAVMFilterFieldTpl = null;
    
    var ModalDiv = null;
	
	function initViewProperties(layer){
		var properties = layer.properties = {};
		switch(layer.viewType){
			case "AnalysisView" : { //No i18N
				properties.showTitle = 1;
				
				break;
			}
		}
	}

    function initViewHeader(layer){
        var titleDescArea = document.createElement("div");
        var wrapper = document.createElement("div");
        var dlgTitle = document.createElement("div");
        var dlgDescription = document.createElement("div");

        Util.addClass(titleDescArea, "dashChildTitleDesc");
        Util.addClass(wrapper, "dashChildTitleArea");
        Util.addClass(dlgTitle, "dashboardTitleFont");
        Util.addClass(dlgDescription, "dashboardDescFont");

        wrapper.appendChild(dlgTitle);
        wrapper.appendChild(dlgDescription);

        titleDescArea.appendChild(wrapper);

        layer.dashChild.appendChild(titleDescArea);

         Object.defineProperties(layer.dashChild, {
             dlgTitle : {
                 value : dlgTitle
             },
             dlgDesc : {
                 value : dlgDescription
             },
             titleDescArea : {
                 value : titleDescArea
             }
         });
         
         if(layer.properties === undefined){
        	 initViewProperties(layer);
         }
         
         
         if(layer.properties.showTitle === "0"){
        	 //dlgTitle.style.display = "none";
        	 layer.dashChild.setAttribute("showtitle","false", true);
         }
         else{
        	 layer.dashChild.setAttribute("showtitle","true", true);
         }
         
         if(layer.properties.showDesc === "1"){
        	 layer.dashChild.setAttribute("desc","true", true);
        	 layer.dashChild.setAttribute("descLength","true", true);
         }
         else{
        	//dlgDescription.style.display = "none";
        	 layer.dashChild.setAttribute("desc","false", true);
         }
    }

    
    var VIEW = {
        type : "VIEW", //No i18N
		minHeight : 10,
		height : 20,
		minWidth : 10,
		width : 40,
        onAdd : function(){
            var layer = this;
            var dashboard = layer.getMainInstance;
            if(dashboard.isOld){
				var fpx = Math.floor(dashboard.container.clientWidth / 160);
				var extraHeight = Math.round(45 / fpx);
				layer.height = layer.height + extraHeight;
			}
			var totalMultiColors = ZDBDThemes.getMultiColorCount(dashboard.theme);

            var dashChildEl = document.createElement("div");
            dashChildEl.setAttribute("thememulticolortype",dashboard.themeChildCount % totalMultiColors);
            dashChildEl.className = "zd-dashChild dashChild";

            Object.defineProperty(layer, "dashChild",{ //No i18N
                value : new ZRElementMask({element : dashChildEl})
            });
            layer.dashChild.layer = layer;
            layer.dashChild.setAttribute("configid", layer.configId);
            layer.dashChild.setAttribute("showvmfilter", layer.properties.showUF==="0" ? "false" : "true");
            layer.dashChild.setAttribute("excludedashglobalfilters", layer.properties.excludeDashUF==="0" ? "false" : "true");
            
            if(layer.viewType !== "WIDGET"){
            	if(layer.canLoad()){
            		initViewHeader(layer);
            	}
            		
            }else{
            	Util.addClass(layer.dom.container, "zr-widgetChild");
            	dashChildEl.setAttribute("viewtype","WIDGET");
//            	layer.$autoHeight(true);
            }
            
            if(dashboard.editMode === false){
        		layer.addToDashQ(dashboard);
            }
            else{
                fetchCurrentLayer(layer, dashboard)
            }
            
            initLayerBorder();
            var _layerBorder = layerBorder.cloneNode(layer);
            layer.dom.element.appendChild(_layerBorder);
            
            
            
            dashboard.themeChildCount++;

        },
        canLoad : function(){
        	return this.properties.isViewShared && this.properties.isValidFilterCriteria;
        },
        onInsert : function(){
            var layer = this;
            if(layer.viewType === "WIDGET"){
            	Util.addClass(layer.dom.container, "zr-widgetChild");
            	
                var dashChildEl = document.createElement("div");
                dashChildEl.className = "zd-dashChild dashChild widgetChild";

                Object.defineProperty(layer, "dashChild",{ //No i18N
                    value : new ZRElementMask({element : dashChildEl})
                });
                
                layer.dom.container.appendChild(dashChildEl);
                
                initLayerBorder();
                var _layerBorder = layerBorder.cloneNode(true);
                layer.dom.element.appendChild(_layerBorder);
                
                controlOptionSet(layer);
            }
        },
        onResize : function(){
        	if(this.viewType !== "WIDGET"){
        		this.parentEl.view.resize();
        	}
        },
        showEditModeOptions : function(){},
        showViewtModeOptions : function(){},
        
        addToDashQ : function(dashboard){
            var layer = this;
            dashboard.dashQ.push({
                func : fetchLayerFromQ,
                args : [layer, dashboard],
                viewType : ZDBClientViewAPI.TYPES_VALUE_MAP[layer.viewType]
            });
        },
        _refresh : function(icon, event){
        	var view = this.parentEl.view;
        	if(view.type === "WIDGET"){
        		view.fetched = false;
        	}
        	fetchCurrentLayer(this, this.getMainInstance);
        },
        _settings : function(){},
        _edit : function(){
            var layer = this;
            switch(layer.viewType){
                case "WIDGET" :  //No i18N
                    break;
                
            }
        },
        _delete : function(){
        	this.$remove();
        },
        _toggleTitle : function(){
            var layer = this;
            if(layer.properties.showTitle){
                // hide
                layer.properties.showTitle = false;
            }
            else{
                // show
                layer.properties.showTitle = true;
            }
        },
        _toggleDescription : function(){
            var layer = this;
            if(layer.properties.showDescription){
                // hide
                layer.properties.showDescription = false;
            }
            else{
                // show
                layer.properties.showDescription = true;
            }
        },
        _toggleUserFilter : function(){
            var layer = this;
            if(layer.properties.UserFilter){
                // hide
                layer.properties.UserFilter = false;
            }
            else{
                // show
                layer.properties.UserFilter = true;
            }
        },
        _changeChartType : function(icon){
        	//icon.view = this.parentEl.view;
        	//ZDBAnalysisUtil.showChartDlg(icon);
        	DashUtils.showChartTypes_new(icon, this.dashChild.Element, this.parentEl.view);
        },
        _viewUnderlyingData : function(){
        	var view = this.parentEl.view;
        	ZDBVUDataAPI.viewUD(view);
        },
        _export : function(icon, event){
        	if(!ZDBLazyLoadUtil.loadJsAlone("ZDBExportNShare",this,this._export,[icon, event])){
      	  		return;
      	  	}
        	icon.view = this.parentEl.view;
        	ZDBMenu.showDashOptsMenu(icon, event, "Single_View_Export_Menu", "GridMenu"); //No i18N
        },
        _openInNewTab : function(icon, event){
        	var view = this.parentEl.view;
        	var tab = ZDBClientUI.getExistingTab(view.objId);
        	if(tab){
        		ZDBClientUI.selectTab(tab);
                return;
        	}
        	 var tab = ZDBClientUI.createUpdateDummyTab(view.objId, view.displayName, view.type, tab);
             ZDBClientUI.typeDefns[view.type].fetchView(view.objId, tab.viewCtx);
        },
        _moreOpts : function(icon, event){
        	DashUtils.showMoreOptions_new(icon, event, this.dashChild.Element, this.parentEl.view);
        }
    };
    var USERFILTERS = {
		type : "USERFILTERS", //No i18N
        minWidth : 20,
		$resize : false, 
		autoHeight : true,
        onAdd : function(){
            var ufLayer = this;
            var dashboard = this.getMainInstance;
            dashboard.ufLayer = ufLayer;
            
            Util.addClass(ufLayer.dom.container, "zr-ufLayer");

            var UFArea = document.createElement("div");
            Util.addClass(UFArea, "ZR-DashboardUFContainer");
            UFArea.appendChild(UFTemplate.cloneNode(true));
            ZDBUtil.setNamedChildEls(UFArea, dashboard, "elname");
            dashboard.dropAreas = {};
            ufLayer.dom.container.appendChild(UFArea);
            ZAVMFilter.placeVMFilterCriteria(dashboard);
            
            ZDBDashboardAPI.showPanel(dashboard);
            
//        	if(UFTemplate == null){
//        		getUFTemplate();
//        	}
//            var UFElement = ZRDashboardAdapterAPI.getUFTemplate().cloneNode(true);
//            ZDBUtil.setNamedChildEls(UFElement, dashboard);
//            
//            UFArea.appendChild(UFElement);
//
//             dashboard.dropAreas = {};
//             var viewTypeName = dashboard.VMFilterDropArea.getAttribute("VIEWTYPE");
//             dashboard.dropAreas[viewTypeName] = dashboard.VMFilterDropArea;
//
//            ufLayer.dom.container.appendChild(UFArea);
//
////          ZAVMFilter.placeVMFilterDefinition(dashboard);
////			ZAVMFilter.placeVMFilterCriteria(dashboard);
//			ZAVMFilter.showOrHideVMFilterPanel(dashboard, true);
//
//            if(dashboard.uf){
//            	var dropArea = dashboard.VMFiltersHolder;
//                // create existing UF 
//                for(var i=0, j= dashboard.uf.length; i<j; i++){
//                    var field = dashboard.uf[i];
//                    var component = ZRDashboardAdapterAPI.getZAVMFilterFieldTpl().cloneNode(true);
//                    var fieldMask = new ZRElementMask({element : component, attributes :field });
//                    fieldMask.view = dashboard;
////                    ZAVMFilterEdit.dropField(fieldMask, dropArea, dashboard);
//                    ZAVMFilterEdit.showRelComponentDlg(fieldMask, dropArea, true);
////                    var component = ZAVMFilterFieldTpl.cloneNode(true);
////                    var UFmask = new ZRElementMask(component, field);
////                    ZAVMFilter.addVMFilterToDash(UFmask, dashboard);
//                }
//                // bad fix
//                dropArea.style.display = "block";
//                var childs = dashboard.VMFiltersList;
//                var j = childs.length;
//                for(var i=0; i<j; i++){
//                	dropArea.appendChild(childs[i]);
//                }
//                dropArea.appendChild(dashboard.VMFilterResetBtn);
//                
//                dashboard.modified = 0;
//                ZDBMenu.updateToolbar(dashboard);
//                
//            }
        },
        switchMode : function(){},
        validate : function(){},
        enable : function(){},
        disable : function(){},
        markInvalidFilters : function(){},
        addComponent : function(){},
        removeComponent : function(){},
    };
    var SPACER = {
    	type : "SPACER",//No i18N
    	height : 1,
    	autoHeight : true
    };
    var HTML = {
		type : "HTML", //No i18N
		height : 40,
		minWidth : 10,
		width : 30,
		autoHeight : true,
        onAdd : function(){
        	var layer = this;
        	var dashboard = this.getMainInstance;
        	
            var dashChildEl = document.createElement("div");
            dashChildEl.className = "zd-dashChild dashChild zr-textChild";
            dashChildEl.setAttribute("elType","HTML");

            Object.defineProperty(layer, "dashChild",{ //No i18N
                value : new ZRElementMask({element : dashChildEl})
            });
            
            layer.textId = dashboard.textId++;
            // append text from JSON
            dashChildEl.innerHTML = layer.content;
            
            layer.content = ZDBEscapeUtil.encodeForHtml(dashChildEl.innerHTML);
            //console.log(layer.content);
            
            layer.dom.container.appendChild(dashChildEl);
        },
        onInsert : function(){
            var layer = this;
            var dashboard = this.getMainInstance;
            var dashChildEl = document.createElement("div");
            dashChildEl.className = "zd-dashChild dashChild zr-textChild";

            Object.defineProperty(layer, "dashChild",{ //No i18N
                value : new ZRElementMask({element : dashChildEl})
            });
            
            layer.textId = dashboard.textId++;
            
            layer.dom.container.appendChild(dashChildEl);
        },
        _edit : function(){},
        _delete : function(){}
    };

    var layerTypes = [VIEW, USERFILTERS, SPACER, HTML];

    var dashboardPrototype = {
		animatingViewWaiting : 0,
		editMode : false,
		dashLayout : true,
        type : "Dashboard", //No i18N
        ModalDiv : ModalDiv,
        qLimit : 3,
        bgQLimit : 1,
        analysisViewCount : 0,
        initialize : function(response){
            if(!this.initialized){
                
                ZDBClientViewAPI.initFromDiv(this, this.viewComp);

                this.showShiftHelpVMFilter = true;
                this.VMFiltersList = new Array();
                this.vmFilterListCatch = {};
                this.widgetIdMap = {};
                this.isSaveAs = false;
                this.trashWidget = new Array();
                this.themeChildCount = 0;
                this.textId = 0;
                this.editMode = ZDBUtil.isNewView(this);

                this.ajaxResponseText = document.createElement("div");
                this.viewComp.appendChild(this.ajaxResponseText);

                //normalize with response
                if(response !== undefined){
                    this.qLimit = response.qLimit;
                    this.bgQLimit = response.preloadQLimit;
                    this.analysisViewCount = response.analysisViewCount;
                    this.theme = DashTheme(response.themes);
                    this.viewComp.setAttribute("displayName", response.displayName);
                    this.viewComp.setAttribute("description", response.description ? response.description : "");
                    if(response.sliderBounds){
                    	ZASliderComp.sliderRangesForGridView(response.sliderBounds, this);
                    }
                    
                }
                else{
                    this.theme = ZDBDThemes.getDefThemeInst(1);
                    handleNavigation(this.editMode);
                }
                
                
                this.themeChildCount = 0;

                this.dashQ = ZDBDashQ(this);
                this.dashDiv = this.container;
                this.curSize = {width: 0, height: 0};
                this.container.style.overflowY = "scroll"; //No i18N
//                this.container.style.paddingTop = "10px"; //No i18N
                ZDBDThemes.apply(this);               
                Util.addClass(this.viewComp,"zdbdviewmode");
                Object.defineProperty(this, "initialized", {value : true}); //No i18N
            }
        },
        themeFix : function(){
            var multiColors = ZDBDThemes.getMultiColorCount(this.theme);
            var themeChild = 0;
            for(var key in this.layout){
                var layer = this.layout[key];
                if(layer.type === "VIEW"){
                    layer.dashChild.setAttribute("thememulticolortype", themeChild % multiColors, true);
                    themeChild++;
                }
            }
            
        },
        toggleMode : function(){},
        setHidden : function(){
        	handleNavigation(false);
        },
        setVisible : function(){
        	handleNavigation(this.editMode);
        	$$dropAdd.key = this.key;
        },
        save : function(requestOptions){
	    	if(ZDBUtil.isNewView(this)){
				 if(!ZDBCreateViewAPI.getNameForNewView(this,requestOptions)){
					 return;
				 }
			 }
			 this.saveCurrentConfig(requestOptions);
        },
        resize : function(){
        	if(this.loaded){
				this.$resize();
			}
        },
        getViewParams : function(saveAs, oldObjId){
        	var params = (saveAs) ? "&ZDBACTION=SAVEAS&OLDOBJID="+oldObjId : "&ZDBACTION=SAVE";   //No I18N
        	params += "&NEWDASH=true"; //No I18N
    		if(this.objId > -1)
    		{
    			params += "&OBJID=" + this.objId; //No I18N
    		}
    		else
    		{
    			params += "&DBID=" + this.dbId; //No I18N
    			params += ZDBCreateViewAPI.getNewNameInfoAsParams(this);
    		}
    		params += this.viewCtx.getAdditionalParams();
    		return params;
        },
        saveCurrentConfig  : function(requestOptions, saveAs, oldObjId){
			//console.log("currentConfig");
			var url = "/ZDBDashboardDesignEditor.ve?" + this.getViewParams(saveAs, oldObjId); //No i18N
			var xml = getCurrentConfig(this, false, true);
			if(xml == null){
				// alert user - no content in dashboard;
				return;
			}
			AjaxAPI.sendRequest(
			{
				URL : url,
				POSTBODY : xml,
				REQUESTHEADERS:{"Content-type":"text/xml;charset=utf-8"}, //No I18N
                ISNEWVIEW:ZDBUtil.isNewView(this),
                VIEW:this,
            	ONSUCCESSFUNC:ZDBCreateViewAPI.handleViewSave,
            	ONFAILUREFUNC:ZDBCreateViewAPI.handleViewSaveFailure,
            	ORIGREQOPTIONS:requestOptions,
            	OPSTATUSID:this.viewCtx.getLoadIndicator(ZDBI18NMap.g("zrpt.common.loadingdiv.js.saving"))  //No I18N
			});
		},
		appendPermissionsForView : function(){},
        updateRespStatus : function(view){
        	this.dashQ.dequeue();
        	console.log("Resp Status updated");
//        	if(this.loaded){
//				this.$resize();
//			}
        	if(view && view.layer && (view.layer.viewType === "WIDGET" || view.layer.type==="HTML")){
        		view.layer.$fixHeight();
        	}
        },
        setScrollPosition : function(){},
        removeField : function(field,updateGraphArea){
		    ZAVMFilter.removeComponent(field);
		},
        showModalDiv : function(){
			document.body.appendChild(ModalDiv);
		},
		hideModalDiv : function(){
			document.body.removeChild(ModalDiv);
		},
		onDragStart : function(event){
			initDragIcon();
			var objEl = this.currentElement;
			var elId = objEl.getAttribute("objId");
		    var viewInfo = ZDBExpViewHandler.fetchViewListInfo(elId);
		    var objType = viewInfo[elId];
		    
		    dragElement.viewName.innerHTML = viewInfo[elId];
		    dragElement.className = (objType == null) ? "dashDnDMoveDiv" : "dashDnDMoveDiv "+objType+"Obj";
		    dragElement.tabIcon.style.display = (objType == null) ? "none" : "";
		    
			dragElement.style.display = "block";
			dragElement.style.left = event.clientX - 10 + 'px';
			dragElement.style.top = event.clientY - 10 + 'px';
			
		},
		onDrag : function(event){
			dragElement.style.left = event.clientX - 10 + 'px';
			dragElement.style.top = event.clientY - 10 + 'px';
		},
		onDrop : function(event){
			dragElement.style.display = "none";
		},
		removeView : function(){},
		onChangeMode : function(modeName){
			if(modeName === "viewMode"){
				console.log("callBack")
			}
		},
		onBuild : function(){
			clearSpacer(this);
//			emptyExistingSpacer(this);
			console.log("SPACER CLEAR")
		}
    };
    
    function decreaseY(order,y, px){
    	var list = order.y[y];
    	var btm = {};
    	for(var it in list){
    		var item = list[it];
    		item.dom.element.style.marginTop = -px + "px"; //No i18N 
    		btm[item.bottom] = true;
    	}
    	for(var m in btm){
    		decreaseY(order, m, px);
    	}
    }
    
    function emptyExistingSpacer(spacerList, dashboard){
    	var order = dashboard.data.order;
//    	var spacerList = getAllSpacer(order.list);
    	var px = dashboard.data.fPx;
    	for(var i=1; i<=spacerList.length; i++){
    		decreaseY(order, spacerList[i-1].bottom, i*px);
    	}
    }
    
    function sumAll(orderBottom){
    	var width = 0;
    	var points = [];
    	for(var key in orderBottom){
    		width += orderBottom[key].width;
    		points.push(orderBottom[key].x);
    		points.push(orderBottom[key].right);
    	}
    	if(width == 80){
    		return true;
    	}
    	return points;
    }
    
    function checkPt(pt, ptx1, ptx2){
    	return (ptx1 <= pt && pt < ptx2);
    }
    
    function checkPt1(pt, ptx1, ptx2){
		return (ptx1 < pt && pt < ptx2);
	}

    function ptContains(pt1, pt2, pt3, pt4){
    	if(checkPt(pt1, pt3, pt4)){
    		return true;
    	}
    	else if(checkPt1(pt2, pt3, pt4)){
    		return true;
    	}
    	else if(checkPt(pt3, pt1, pt2)){
    		return true;
    	}
    	else if(checkPt1(pt4, pt1, pt2)){
    		return true;
    	}
    	else{
    		return false;
    	}
    }
    function chkPointsMatch(orderTop, points, spacer){
    	for(var k in orderTop){
    		var layer = orderTop[k];
    		for(var i=0; i<points.length; i+=2){
    			var pt1 = points[i];
    			var pt2 = points[i+1];
    			var pt3 = layer.x;
    			var pt4 = layer.right;
    			if(ptContains(pt1, pt2, pt3, pt4)){
    				break;
    			}
    			if(i == points.length - 2){
    				return false;
    			}
    		}
    	}
    	return true;
    }
    
    function checkNremove(order, spacer){
    	var top = spacer.y;
    	var bottom = spacer.bottom;
    	var remove = false;
    	if(order.bottom[top]){
    		var points = sumAll(order.bottom[top]);
			if(points != true){
				remove = chkPointsMatch(order.y[bottom], points, spacer);
			}
			else{
				remove = points;
			}
    	}
    	if(remove){
    		spacer.$remove();
        	return true;
    	}
    	return false;
    }
    
    function getAllSpacer(orderList){
    	var arr = [];
    	for(var k in orderList){
    		if(orderList[k].type === "SPACER"){
    			arr.push(orderList[k]);
    		}    		
    	}
    	return arr;
    }
    
    function clearSpacer(dashboard){
    	var order = dashboard.data.order;
    	var spacerList = getAllSpacer(order.list);
    	var existingSpacer = [];
    	for(var i=0; i<spacerList.length; i++){
			if(!checkNremove(order,spacerList[i])){
				existingSpacer.push(spacerList[i]);
			};
    	}
    	emptyExistingSpacer(existingSpacer, dashboard);
    }

    

    var layerPrototype = {
        onMoveEnd : function(){},
        onResizeStop : function(){},
    };

    var newDashboardUF = {
        type : "USERFILTERS", //No i18N
		x : 0,
		y : 0,
		width : 80,
		height : 2
    };

    

    
    
    var dragElement = false;
    var ic_left = 0;
    var ic_top = 0;
    
    function initDragIcon(){
    	if(!dragElement){
    		dragElement = document.createElement("div");
    		dragElement.className ="dashDnDMoveDiv"; //No i18N
    		
    		var tabIcon = document.createElement("div");
    		tabIcon.style.display = "inline-block";
    		tabIcon.className = "tabIcon moveDivIco";
    		dragElement.appendChild(tabIcon);
    		dragElement.tabIcon = tabIcon;
    		
    		var viewName = document.createElement('div')
    		viewName.style.display = "inline-block";
    		dragElement.appendChild(viewName);
    		dragElement.viewName = viewName;
    		
    		document.body.appendChild(dragElement);
    		
    	}
    }
    
    var layerBorder = false;
    
    function initLayerBorder(){
    	if(!layerBorder){
    		layerBorder = document.createElement("div");
    		layerBorder.className = "zd-ui-border";
    		//top
    		var top = document.createElement("div");
    		top.className = "zd-ui-sides zd-ui-top";
    		layerBorder.appendChild(top);
    		
    		//right
    		var right = document.createElement("div");
    		right.className = "zd-ui-sides zd-ui-right";
    		layerBorder.appendChild(right);

    		//bottom
    		var bottom = document.createElement("div");
    		bottom.className = "zd-ui-sides zd-ui-bottom";
    		layerBorder.appendChild(bottom);

    		//left
    		var left = document.createElement("div");
    		left.className = "zd-ui-sides zd-ui-left";
    		layerBorder.appendChild(left);
    		
    	}
    }
    
    function setTrigger(node, layer, opt){
    	if(opt === "_moreOpts"){
    		if(layer.properties.canShowVUD){
    			node.setAttribute("isVUD", "true");
    		}
    		if(layer.properties.canExport){
    			node.setAttribute("isExp", "true");
    		}
    	}
    	node.onclick = function(event){
    		layer[opt](node, event);
    	}
    }
   
    function createOptionsHolder(layer, editMode){
    	
    	initViewModeIcons();
    	var className = "zd-options zd-viewModeOptions dashviewoptions "+layer.viewType.toLowerCase(); //No i18N
    	var order = viewModeOrder;
    	var icons = viewModeIcons;
    	if(editMode){
    		initEditModeIcons();
    		var options = getEditModeOptions(layer);
    		className = "zd-options zd-editModeOptions"; //No i18N
    		order = editModeOrder;
    		icons = editModeIcons;
    	}
    	else{
    		var options = getViewModeOptions(layer);
    	}
    	//console.log(icons, order);
    	if(options){
    		var holder = document.createElement("div");
    		holder.className = className;
    		var rowHolder = document.createElement("div");
    		rowHolder.className = "dashviewoptionsrow";
    		
    		for(var i=0, j=order.length; i<j; i++){
        		var opt = order[i];
        		if(options[opt]){
        			console.log(opt);
        			var node = icons[opt].cloneNode(true);
        			setTrigger(node, layer, opt);
        			rowHolder.appendChild(node);
        		}
        	}
    		holder.appendChild(rowHolder);
        	layer.dashChild.appendChild(holder);
    	}
    }
    
    var editModeOptions = {
    	_delete : true,
    };
    
    var editModeIcons = false;
    var editModeOrder = ["_refresh", "_edit", "_settings", "_delete"]; //No i18N
    
    function initEditModeIcons(){
    	if(editModeIcons === false){
    		
    		var refreshIcon = document.createElement("div");
    		refreshIcon.className = "optionsIcons refreshOption";

    		var editIcon = document.createElement("div");
    		editIcon.className = "optionsIcons editTextOption";
    		
    		var settingsIcon = document.createElement("div");
    		settingsIcon.className = "optionsIcons settingsOption";
    		
    		var deleteIcon = document.createElement("div");
    		deleteIcon.className = "optionsIcons deleteMenu";
    		
    		editModeIcons = {
				_refresh : refreshIcon,
				_edit : editIcon,
				_settings : settingsIcon,
				_delete : deleteIcon
    		};
    	}
    }
    
    function getEditModeOptions(layer){
    	var options = Object.create(editModeOptions);
    	if(layer.type === "VIEW" && layer.viewType !== "WIDGET"){
    		options._refresh = true;
    		options._settings = true;
    	}
    	else if(layer.type === "VIEW" && layer.viewType === "WIDGET"){
    		options._refresh = true;
    		options._edit = true;
    	}
    	else{
    		options._edit = true;
    	}
    	return options;
    }
    
    
    function addEditModeOptions(layer){
    	initEditModeIcons();
    	var options = getEditModeOptions(layer);
    	
    	var optionsHolder = document.createElement("div");
    	optionsHolder.className = "zd-options zd-editModeOptions";
    	for(var i=0, j=editModeOrder.length; i<j; i++){
    		var opt = editModeOrder[i];
    		if(options[opt]){
    			var node = editModeIcons[opt].cloneNode(true);
    			setTrigger(node, layer, opt);
    			optionsHolder.appendChild(node);
    		}
    	}
    	layer.dashChild.appendChild(optionsHolder);
    }
    
    
    var viewModeIcons = false;
    var viewModeOrder = ["_changeChartType","_openInNewTab","_moreOpts"]; //No i18N
    
    function getViewModeOptsContainer(className, iconClassName){
    	var optCell = document.createElement("div");
    	optCell.className = className;
    	var icon = document.createElement("div");
    	icon.className ="dashioicon "+iconClassName;
    	optCell.appendChild(icon);
    	return optCell;
    }
    
    function initViewModeIcons(){
    	if(viewModeIcons === false){
    		// initialize
    		var chartTypeIcon = getViewModeOptsContainer("dashviewoptionsct", "changecharttypeicon"); //No i18N
    		var openNewTabIcon = getViewModeOptsContainer("dashviewoptionscell", "openinnewtabicon");//No i18N
    		var moreOptionsIcon = getViewModeOptsContainer("dashviewoptionscell moreOptions", "dashviewmoreoptions");//No i18N
    		
    		viewModeIcons = {
				_changeChartType : chartTypeIcon,
				_openInNewTab : openNewTabIcon,
				_moreOpts : moreOptionsIcon
    		};
    	}
    }
    
    function getViewModeOptions(layer){
    	if(layer.type === "VIEW" && layer.viewType !== "WIDGET"){
    		var options = {};
    		if(layer.properties.canRead){
    			options._openInNewTab = true;
    		}
    		switch (layer.viewType){
    			case "AnalysisView" : //No i18N
    				options._changeChartType = true;
    			default :
    				if(layer.properties.canExport || layer.properties.canShowVUD){
    					options._moreOpts = true;
    				}
			}
    		return options;
    	}
    	return null;
    }
    
    function addViewModeOptions(layer, options){
    	
    }
    
    function controlOptionSet(layer){
    	createOptionsHolder(layer);
    	//createOptionsHolder(layer, true);
    }

    function getNewDashboard(viewCtx, dbId, objId, permissionLevel, response){

        var structure = response === undefined ? {
            layout : {
                0 : newDashboardUF
            }
        } : {
            layout : response.layout
        };

        var viewComp = document.createElement("div");
        Util.addClass(viewComp, "ZDBDashboardEditorEl","ZR-Dashboard", "zdbdviewmode");

        var dashboard = new ZRDashboard(viewComp, structure);
        dashboard.viewComp = viewComp;//new ZRElementMask({element : viewComp});

        dashboard.viewComp.setAttribute("objId", objId);
        dashboard.viewComp.setAttribute("dbId", dbId);
        dashboard.viewComp.setAttribute("permLevel", permissionLevel);
        

        dashboard.initialize(response);

        if(objId < 0){
            dashboard.modified = 1;
        }
        else{
        	dashboard.modified = 0;
        }

        ZDBClientViewAPI.initFromDiv(dashboard, viewComp);

        viewCtx.handle(dashboard);
        ZDBMenu.updateToolbar(dashboard);

        // temp code
        $$dropAdd.key = dashboard.key;

        

        return dashboard;
    }
    
    function getWidgetData(dashboard){
    	var xml = "\n<ZDBWidget>"; //No i18N
    	for(var key in dashboard.layout){
    		var layer = dashboard.layout[key];
    		if(layer.type === "VIEW" && layer.viewType === "WIDGET" && dashboard.editMode){
    			var widgetView = layer.parentEl.view;
    			xml +=  "\n"+ WidgetUtils.getConfig(widgetView.objId, widgetView); //No i18N	
    		}
    	}
    	xml += "\n</ZDBWidget>" //No i18N
    	return xml;
    }
    
    function getSaveData(dashboard){
        var layout = "\n<ZRDashboardLayout ID ='layout' layout='"+JSON.stringify(dashboard.layout)+"'></ZRDashboardLayout>";//NO I18N
        var theme = "\n<ZDBDashboardTheme ID='theme' theme='"+JSON.stringify(dashboard.theme.toJSON())+"'></ZDBDashboardTheme>";// NO I18N
        return layout+theme;
    }

    function getCurrentConfig(dashboard, sendDynFltrConfig, sendCompConf){
        sendCompConf = (sendCompConf)?true:false;
        var sendCompleteInfo = (dashboard.permissions.designModify == 1 && ZDBUtil.isViewModified(dashboard))|| sendCompConf;
        
        var saveData = sendCompleteInfo ? getSaveData(dashboard) : "";
        saveData += "\n" + ZDBDashboardUF.getDashBoardDynFltrConfig(dashboard, sendCompleteInfo, sendDynFltrConfig); //No i18N
        saveData += getWidgetData(dashboard);
        
        var xml = "<ZDBDashboard viewConf='"+sendCompleteInfo+"'" + " csrftoken='" + ZDBEscapeUtil.encodeForXML(getCookie("CSRF_TOKEN")) + "'>" + saveData +  "\n</ZDBDashboard>"; //No i18N
        return xml
    }
    
//    function getViewParams(dashboard, saveAs, oldObjId){
//		var params = (saveAs) ? "&ZDBACTION=SAVEAS&OLDOBJID="+oldObjId : "&ZDBACTION=SAVE";   //No I18N
//		if(dashboard.objId > -1)
//		{
//			params += "&OBJID=" + dashboard.objId; //No I18N
//		}
//		else
//		{
//			params += "&DBID=" + dashboard.dbId; //No I18N
//			params += ZDBCreateViewAPI.getNewNameInfoAsParams(dashboard);
//		}
//		params += dashboard.viewCtx.getAdditionalParams();
//		return params;
//	}

//    function saveCurrentConfig(dashboard, request, saveAs, oldobjId){
//        var url = "/ZDBDashboardDesignEditor.ve?" + dashboard.getViewParams(saveAs, oldObjId); //No i18N
//        var xml = getCurrentConfig(dashboard, false, true);
//        AjaxAPI.sendRequest(
//			{
//				URL : url,
//				POSTBODY : xml,
//				REQUESTHEADERS:{"Content-type":"text/xml;charset=utf-8"}, //No I18N
//                ISNEWVIEW:ZDBUtil.isNewView(dashboard),
//                VIEW:dashboard,
//            	ONSUCCESSFUNC:ZDBCreateViewAPI.handleViewSave,
//            	ONFAILUREFUNC:ZDBCreateViewAPI.handleViewSaveFailure,
//            	ORIGREQOPTIONS:requestOptions,
//            	OPSTATUSID:dashboard.viewCtx.getLoadIndicator(ZDBI18NMap.g("zrpt.common.loadingdiv.js.saving"))  //No I18N
//			});
//    }
    
    function getUfTemplate(){
    	UFTemplate = $("ZDBDashVMEl");
        UFTemplate.id = "";

        ZAVMFilterFieldTpl = $("ZAVMFilterFieldTpl");
        ZAVMFilterFieldTpl.id = "";
        
        ZDBAnalysisView.VMFilterFieldTpl = ZAVMFilterFieldTpl;
    }
    var initialized = false;
    
    function init(){
    	if(initialized) {
    		return;
    	}
    	Builder = ZRDashboardLayout;
    	Util = Builder.util;
    	ZRDashboard = Builder.Dashboard;
    	Builder.setInitConfig(initialConfiguration);
    	
    	for(var mode in modes){
            Builder.addMode(mode, modes[mode]);
        }

        layerTypes.forEach(function(type){
            Builder.addLayerType(type);
        });
        ZRDashboardLayout.addCallback("dashboard", dashboardPrototype); //No i18N
        ModalDiv = document.createElement("div");
    	Util.addClass(ModalDiv, "ZR-ModalDiv", "alpha");
    	initialized = true;
    }

    function createView(viewCtx, type, dbId, parentViewId, parentDisplayName, dummyobjId){
        if (!ZDBLazyLoadUtil.loadJsAlone(["ZDBAnalysisViewList","ChartTPVendors","ZRJSChartsList", "ZAVMFilterList", "ZAAnalysisFields", "ZRDashboardViewNEW"], "APPENDTODOC", createView, [viewCtx, type, dbId, parentViewId, parentDisplayName, dummyobjId])){
            return;
        }
        if(!ZDBLazyLoadUtil.loadCssOnly(["CSSLIST_GRIDSTYLES", "CSSLIST_ANALYSISVIEWSTYLES","CSSLIST_DASHBOARDSTYLES", "CSSLIST_VMFILTERSTYLES", "CSSLIST_DASHBOARDLAYOUTSTYLES"], this, createView, [viewCtx, type, dbId, parentViewId, parentDisplayName, dummyobjId])){
            return;
        }
        init();
        
        var url = "/ZDBDashEditUI.cc?"; //NO I18N
        url+= "DBID=" + dbId; //No I18N
        
        if(!ZDBLazyLoadUtil.loadSnippet("VMFilterEdit","/ZAVMFilterEdit.cc?",this,"APPENDTODOC",["zdbjs/vmfilter/ZAVMFilterEdit.js","zdbjs/vmfilter/ZAVMFilterEditUI.js","zdbjs/analysisedit/ZDBAnalysisFilter.js"], createView, [viewCtx, type, dbId, parentViewId, parentDisplayName, dummyobjId])){
			return;
	    }

        if(!ZDBLazyLoadUtil.loadSnippet("ZDBDashEditUI,ZDBDashboardUFTemplate", url, this, "APPENDTODOC", null, null, null, viewCtx.getLoadIndicator(ZDBI18NMap.g("zrpt.common.dashboard.loadingdiv.js.loadingDBDesigner")))){
            return;
        }
//        else if(UFTemplate == null){
////            UFTemplate = $("ZDBDashVMEl");
////            UFTemplate.id = "";
////
////            ZAVMFilterFieldTpl = $("ZAVMFilterFieldTpl");
////            ZAVMFilterFieldTpl.id = "";
////            
////            ZDBAnalysisView.VMFilterFieldTpl = ZAVMFilterFieldTpl;
//        }

        var dashboard = getNewDashboard(viewCtx, dbId, dummyobjId, ZDBClientUI.typeDefns.Dashboard.permLevel);
        ZRDashboardAdapterAPI.configureToolbar(dashboard);
        buildDashboard(dashboard);
        fixMode(dashboard, "editMode"); //No i18N
        //console.log(dashboard.editMode);
//        ZAVMFilter.showOrHideVMFilterPanel(dashboard);
    }

    function fetchView(objId, viewCtx){
        if (!ZDBLazyLoadUtil.loadJsAlone(["ZDBAnalysisViewList","ChartTPVendors","ZRJSChartsList", "ZAVMFilterList", "ZAAnalysisFields", "ZRDashboardViewNEW"], "APPENDTODOC", fetchView, [objId, viewCtx])){
            return;
        }
        if(!ZDBLazyLoadUtil.loadCssOnly(["CSSLIST_GRIDSTYLES", "CSSLIST_ANALYSISVIEWSTYLES","CSSLIST_DASHBOARDSTYLES", "CSSLIST_VMFILTERSTYLES", "CSSLIST_DASHBOARDLAYOUTSTYLES"], this, fetchView,[objId, viewCtx])){
            return;
        }
        init();
//        var url = "/ZDBDashboardUFTemplate.cc?"; //NO I18N
//        url+= "DBID=" + ZDBClientUI.dbId; //No I18N
//
//        if(!ZDBLazyLoadUtil.loadSnippet("ZDBDashboardUFTemplate", url, this, "APPENDTODOC", null, null, null, viewCtx.getLoadIndicator(ZDBI18NMap.g("zrpt.common.dashboard.loadingdiv.js.loadingDBDesigner")))){
//            return;
//        }
//        else if(UFTemplate == null){
//            UFTemplate = $("ZDBDashVMEl");
//            UFTemplate.id = "";
//
//            ZAVMFilterFieldTpl = $("ZAVMFilterFieldTpl");
//            ZAVMFilterFieldTpl.id = "";
//            
//            ZDBAnalysisView.VMFilterFieldTpl = ZAVMFilterFieldTpl;
//        }

        var params = "&OBJID="+objId+viewCtx.getAdditionalParams() + "&STANDALONE=" + ZDBClientUI.standAlone; //No i18N
		    
        AjaxAPI.sendRequest(
        {
            URL : ZDBGlobal.CTX + "/ZDBDashboardDesignEditor.ve?ZDBACTION=FETCH&CHANGESLIDERBOUNDS="+true, //No i18N
            PARAMETERS : params,
            ONSUCCESSFUNC : handleDashboardStructureResponse,
            viewCtx : viewCtx,
            OBJID : objId,
            OPSTATUSID : viewCtx.getLoadIndicator(ZDBI18NMap.g("zrpt.common.dashboard.loadingdiv.js.loadingDashboard")) //No i18N
        });
    }

    function handleDashboardStructureResponse(response, request){
        var viewCtx = request.VIEWCTX;
        var objId = request.OBJID;
        var responseObject = JSON.parse(response.responseText);
        var dbId = ZDBClientUI.dbId;
        var permLevel = responseObject.permLevel;
        
//        if(!ZDBLazyLoadUtil.loadSnippet("VMFilterEdit","/ZAVMFilterEdit.cc?",this,"APPENDTODOC",["zdbjs/vmfilter/ZAVMFilterEdit.js","zdbjs/vmfilter/ZAVMFilterEditUI.js","zdbjs/analysisedit/ZDBAnalysisFilter.js"], handleDashboardStructureResponse, [response, request])){
//			return;
//	    }

        var dashboard = getNewDashboard(viewCtx, dbId, objId, permLevel, responseObject);

        //isOld Check - for migration
        dashboard.isOld = responseObject.isOld;
		
        //uf
        dashboard.uf = responseObject.uf;
        buildDashboard(dashboard);
        
        //if Q is not empty - load the Q
        if(!dashboard.dashQ.isEmpty()){
            dashboard.dashQ.load();
            dashboard.canPreLoad = true;
            ZAVMFilter.preLoadVMFilterIfNeeded(dashboard);
        }
        fixMode(dashboard, "viewMode"); //No i18N
        return dashboard;
	}
	
	function fetchAnalysisViewDetails(dashboard){
		var initGrpArea = (window.ZDBAnalysisView && window.ZDBAnalysisView.initialized) ? false : true;
        var params = "&ZDBACTION=ANALYSISVIEWDETS" + dashboard.viewCtx.getAdditionalParams();//NO I18N
        params += "&DBID=" + dashboard.dbId;//NO I18N
        params += "&OBJID=" + dashboard.objId;//NO I18N
        params += "&INCLUDEGRAPHAREA=" + encodeURIComponent(initGrpArea);//NO I18N
        //console.log(params);
        AjaxAPI.sendRequest({
            URL: ZDBGlobal.CTX + "/ZDBDAnalysisViews.cc", //NO I18N
            PARAMETERS : params,
            ONSUCCESSFUNC : handleAnalysisViewsResponse,
            VIEW : dashboard,
            OPSTATUSID : dashboard.viewCtx.getLoadIndicator()});
        return true;
	}
	
	function handleAnalysisViewsResponse(response, request){
		var lazyLoadList = ["ZDBAnalysisViewList", "ChartTPVendors", "ZRJSChartsList"];//NO I18N
        if(!ZDBLazyLoadUtil.loadJsAlone(lazyLoadList, "APPENDTODOC", handleAnalysisViewsResponse, [response, request])){
            // here analysisView loading is required for chart/pivot/summary/widget view
            return;
        }
        var dashboard = request.VIEW;
        dashboard.ajaxResponseText.innerHTML = response.responseText;
        ZDBAnalysisView.initialize();
        dashboard.dashQ.enableAnalysisReq();
	}
	
	function lazyLoadViewTypeModules(viewType, dashboard, isCallBack){
		// other than this, we can get the dependent js list first and load all necessary js list
        // this will avoid the individual status check cases
        if(lazyLoadedModules[viewType] || !isCallBack && lazyLoadMap[viewType] || viewType < 0) {
            return;
        }
        switch (viewType) {
            case 0://can be replaced with Constants
            case 1:
            case 3:
            case 4:
            case 6:
                if (!ZDBLazyLoadUtil.loadJsAlone("ZDBGridList", "APPENDTODOC", lazyLoadViewTypeModules, [viewType, dashboard, true])){
                    dashboard.dashQ.suspendQ();
                    lazyLoadMap[0] = true;
                    lazyLoadMap[1] = true;
                    lazyLoadMap[3] = true;
                    lazyLoadMap[4] = true;
                    lazyLoadMap[6] = true;
                    return;
                }
                lazyLoadedModules[0] = true;
                lazyLoadedModules[1] = true;
                lazyLoadedModules[3] = true;
                lazyLoadedModules[4] = true;
                lazyLoadedModules[6] = true;
                break;
            case 2:
                var lazyLoadList = ["ZDBAnalysisViewList","ChartTPVendors","ZRJSChartsList"];//NO I18N
                if (!ZDBLazyLoadUtil.loadJsAlone(lazyLoadList, "APPENDTODOC", lazyLoadViewTypeModules, [viewType, dashboard, true])){
                    dashboard.dashQ.suspendQ();
                    lazyLoadMap[viewType] = true;
                    return;
                }
                lazyLoadedModules[viewType] = true;
                break;
            case 8:
                if (!ZDBLazyLoadUtil.loadSnippet('WidgetLayouts', '/WidgetLayouts.cc', this, 'APPENDTODOC', null, lazyLoadViewTypeModules, [viewType, dashboard, true])){
                    dashboard.dashQ.suspendQ();
                    return;
                }
                var lazyLoadList = ["ZDBAnalysisViewList","ChartTPVendors","ZRJSChartsList", "WidgetViewList"];//NO I18N
                if (!ZDBLazyLoadUtil.loadJsAlone(lazyLoadList, 'APPENDTODOC', lazyLoadViewTypeModules, [viewType, dashboard, true])){
                    dashboard.dashQ.suspendQ();
                    lazyLoadMap[viewType] = true;
                    return;
                }
                lazyLoadedModules[viewType] = true;
                break;
        }
        var view = ZDBClientUI.selectedTab.view;
        dashboard = (view instanceof ZRDashboard) ? view : dashboard;
        dashboard.dashQ.resumeQ();
	}
	
	function fetchLayerFromQ(layer, dashboard){
		var dashChild = layer.dashChild;
		if (ZDBClientUI.selectedTab && ZDBClientUI.selectedTab.view.type === 'Dashboard' && !ZDBTabSwitchHandler.isRelevendTab2Exec(dashboard, dashboard.dashQ.resumeQ)) {
            dashboard.dashQ.suspendQ();
            return false;
        }
        if (dashChild && ((dashChild.ctx && dashChild.ctx.view) || dashChild.view)) {
            var childView = dashChild.view || dashChild.ctx.view;
            if(childView.drillDet && window.ZRSDrillDown) {
                ZRSDrillDown.clearDrillDet(childView);
            }
            childView.refreshView();
            if (!(childView.permissions.exportPerm || childView.exportPerm) && dashChild.dashViewOptions.export) {
                dashChild.dashViewOptions.export.parentNode.removeChild(dashChild.dashViewOptions.export);
                delete dashChild.dashViewOptions.export;
            }
            return true;
        }
        fetchCurrentLayer(layer, dashboard);
        return true;
	}
	
	function fetchCurrentLayer(layer, dashboard){
        var dashChild = layer.dashChild;

        var parentEl = document.createElement("div"); //No i18N
        Util.addClass(parentEl, "zd-parDiv", "dashChildPar");

        var parentMask = new ZRElementMask({element : parentEl });

        dashChild.dlgBody =  parentMask;
        dashChild.appendChild(parentEl);

        dashChild.Element.ctx = parentMask.innerViewCtx = new ZDBDashboardCtx(dashboard, dashChild, parentMask);
        
        
        //widget case - on refresh (issue)
        if(layer.parentEl){
        	dashChild.removeChild(layer.parentEl.Element);
        }

        Object.defineProperty(layer, "parentEl", {value : parentMask, writable : true}); //No i18N

        layer.dom.container.appendChild(dashChild.Element);
        
        // add specific controls 
        if(layer.viewType === "WIDGET"){
//        	layer.dashChild.Element.style.height = layer.dashChild.Element.offsetHeight + 20 + "px";
//        	DashUtils.dashChildAutoHeight(layer.dashChild);
//        	layer.dashChild.Element.style.height = "";
        }
        if(layer.canLoad()){
        	if(layer.viewType !== "WIDGET"){
            	controlOptionSet(layer);
            }else{
            	Util.addClass(layer.dom.container, "zr-widgetChild");
            	//dashChildEl.setAttribute("viewtype","WIDGET");
            	layer.dashChild.setAttribute("viewtype","WIDGET", true);
//            	layer.$autoHeight(true);
            }
        	fetchViewByType(layer.viewId, layer.viewType, parentMask.innerViewCtx);
        }
        else{
        	var respSpan = document.createElement("span");
        	respSpan.className ="font14";
        	respSpan.style.cssText="background-color:white;color:blue;font-weight:BOLD;"; //No i18N
        	respSpan.innerText = layer.properties.respContent;
        	parentEl.appendChild(respSpan);
        }
        
    }

    function fetchViewByType(objId, type, viewCtx){
        ZDBClientUI.typeDefns[type].fetchView(objId, viewCtx);
    }
    
    function handleNavigation(editMode){
    	if(ZDBClientUI.standAlone){
    		return
    	}
    	var swipeBar = $("ZDBSwipeBar");
    	  swipeBar.className = (editMode)? "ZDBDashboardPanel": "";
    	  if(ZDBGlobal.ISNEWLAYOUT)
    	  {
    	      var navBar = $("ZDBNavigationBarTD");
    	      navBar.className = (editMode)? "ZRDashboardEdit": "";
    	  }
    	  var showViewsEl = $("ZDBDashShowViewEl");
    	  var navTab = $("ZDBNavigationTab");
    	  var display = "none"; //No I18N
    	  if(editMode)
    	  {
    		  display = "";
    		  jQuery("#ZRSTrash").hide();//NO I18N
    	  }
    	  else
    	  {
    		  jQuery("#ZRSTrash").show();//NO I18N
    	  }
    	  if(!ZDBGlobal.ISNEWLAYOUT)
    	  {
    	            ZDBNavigation.adjustSwipeAreaHeight();
    	  }
    	  showViewsEl.style.display = display;
    	  navTab.style.display = display;
    }
	
	function fixMode(dashboard, modeName){
        if(modeName === "viewMode"){
            dashboard.editMode = false;
            Util.addClass(dashboard.viewComp, "zd-viewMode");
            Util.removeClass(dashboard.viewComp, "zd-editMode");
            
            if(dashboard.loaded){
            	//ZAVMFilter.showOrHideVMFilterPanel(dashboard, true);
            }
            
        }else{
            dashboard.editMode = true;
            var url = "/ZDBDashEditUI.cc?"; //NO I18N
            url+= "DBID=" + ZDBClientUI.dbId; //No I18N
            if(!ZDBLazyLoadUtil.loadSnippet("ZDBDashEditUI", url, this, "APPENDTODOC", null, null, null, dashboard.viewCtx.getLoadIndicator(ZDBI18NMap.g("zrpt.common.dashboard.loadingdiv.js.loadingDBDesigner")))){
                return;
            }
//            else if(UFTemplate == null){
//                UFTemplate = ZRDashboardAdapterAPI.getUFTemplate();
//                UFTemplate.id = "";
//
//                ZAVMFilterFieldTpl = rem$("ZAVMFilterFieldTpl");
//                ZAVMFilterFieldTpl.id = "";
//               
//            }
            
            Util.addClass(dashboard.viewComp, "zd-editMode");
            Util.removeClass(dashboard.viewComp, "zd-viewMode");
            
            //ZDBDashboardControllerAPI.initTextEditor();
            //ZDBDashboardControllerAPI.vmFilterEdit();
            // ZAVMFilter.clearAll(null,dashboard,false);
            if(dashboard.loaded){
            	//ZAVMFilter.showOrHideVMFilterPanel(dashboard);
            }
        }
        handleNavigation(dashboard.editMode);
        //dashboard.$resize();
        dashboard.$changeMode(modeName);
        ZDBMenu.updateToolbar(dashboard);
	}
	
	function switchMode(el, mode){
		var dashboard = ZDBClientUI.getViewObj(el,"view");
        fixMode(dashboard, mode);
	}

    function refreshViews(el, isDashGlobalFiltersApplied, resetQueueAndRefresh){
    	var dashboard = ZDBClientUI.getViewObj(el, "view");
    	dashboard.isViewRendering = false;
		if(!dashboard.dashQ.onsuccess() && !resetQueueAndRefresh){
			return;
		}
		// get all views and add to Q
		for(var key in dashboard.layout){
			var layer = dashboard.layout[key];
			if(layer.type === "VIEW" && (layer.properties.excludeDashUF != "1" || !isDashGlobalFiltersApplied)){
				layer.addToDashQ(dashboard);
			}
		}
		if(dashboard.dashQ.isEmpty()) {
            return;
        }
		dashboard.dashQ.load();
		if(!dashboard.changeSliderBoundaries && dashboard.changedCol != null){
			var component = dashboard.lastAppliedEl;
			var labelName = component.getAttribute("label");
			var action = "FORMATSLIDERVALUE"; //No i18N
			var viewParams = ""; 
		
			if(dashboard.objId > -1){
				viewParams += "&OBJID=" + dashboard.objId; //No i18N
			}
			else{
				viewParams = "&DBID=" + dashboard.dbId;//No i18N
			}
			viewParams += dashboard.viewCtx.getAdditionalParams();
			var doXML = getCurrentConfig(dashboard, false);
			viewParams += "&PARENTVIEWID=" + component.getAttribute("PARENT_OBJ_ID"); //No i18N
			
			var url = "/DashUFAction.ma?VMFILTERLBLNAME=" + encodeURIComponent(labelName) + viewParams;//No i18N
			
			url += "&CHANGESLIDERBOUNDS=" + dashboard.changeSliderBoundaries + "&CHANGEDCOLDETAILS=" + encodeURIComponent(dashboard.changedCol); //No i18N
			url += "&ZDBACTION=" + action;//No i18N
			
			AjaxAPI.sendRequest({
				URL : url,
				POSTBODY : doXML,
				REQUESTHEADERS : {
					"Content-type" : "text/xml; cjarset=utf-8"//No i18N
				},
				VIEW : dashboard,
				COMP : component,
				ACTION : action,
				ONSUCCESSFUNC : ZDBDashboardControllerAPI.handleUFActionResp
			});
		}
    }

    function viewsInDashboard(dashboard){
    	var views = [];
		for(var key in dashboard.layout){
			if(dashboard.layout[key].type === "VIEW"){
				var view = dashboard.layout[key].parentEl.view;
				views.push(view);
			}
		}
		return views;
    }

    function editTheme(){

    }

    function getWidgetLayer(dashboard){
        var widgetLayer = {
            type : "VIEW", //No i18N
            viewType : "WIDGET", //No i18N
            x : 0,
            height : 30,
            width : 80,
            minWidth : 15
        }
        var widget = dashboard.$insertLayer(widgetLayer);
        var dashChild = widget.dashChild;
        var parentEl = document.createElement("div"); //No i18N
        Util.addClass(parentEl, "zd-parDiv", "dashChildPar", "widgetChild");

        var parentMask = new ZRElementMask({element : parentEl});
        dashChild.dlgBody =  parentMask;
        dashChild.appendChild(parentEl);

        Object.defineProperty(widget, "parentEl", {value : parentMask, writable : true}); //No i18N
        
        return widget;
    }
    
    function getHtmlLayer(dashboard){
    	var _htmlLayer = {
			type : "HTML", //No i18N
			x : 0,
			height : 20,
			width : 80,
			minWidth : 10
    	}
    	
    	var htmlLayer = dashboard.$insertLayer(_htmlLayer);
    	var dashChild = htmlLayer.dashChild;
    	return htmlLayer;
    }
    
    var textEditorInitialized = false;
    function initTextEditor(){
		if(textEditorInitialized){
			return;
		}
	    ZE_Init.init();
	    textEditorInitialized = true;
	}

    function buildDashboard(dashboard){
        dashboard.$build();
//        initTextEditor();
        dashboard.loaded = true;
    }
    
    function editTheme(el, event){
        if(!ZDBLazyLoadUtil.loadSnippet('DASHBOARD_THEME_EDITOR', '/DashboardTheme.cc', this, 'APPENDTODOC', ['dashboard/theme/js/ZDBDThemeEditor.js'], editTheme, [el, event])){
            return;
        }
        ZDBDThemeEditor.open(el, event);
    }
    
    function setUFTemplate(el){
    	if(UFTemplate != null){
    		return;
    	}
    	UFTemplate = el;
    	UFTemplate.className = "";
    }
    
    window.ZRDashboardLayoutAPI = {
        lazyLoadedModules : lazyLoadedModules,
        lazyLoadViewTypeModules : lazyLoadViewTypeModules,
        fetchAnalysisViewDetails : fetchAnalysisViewDetails,
        switchMode : switchMode,
        editTheme : editTheme,
        getWidgetLayer : getWidgetLayer,
        getHtmlLayer : getHtmlLayer,
        viewsInDashboard : viewsInDashboard,
        refreshViews : refreshViews,
        getCurrentConfig : getCurrentConfig,
        createView : createView,
        fetchView  : fetchView,
        handleDashboardStructureResponse: handleDashboardStructureResponse,
        markInvalidUserFilters : function(){},
    	init : init,
    	setUFTemplate :setUFTemplate 
    };

    //ZDBClientUI typdefns
    ZDBClientUI.typeDefns["Dashboard_layout"] = {
		fetchView : fetchView,
		createView : createView,
		newName : "zrpt.common.js.newDashboard", // No I18N
	};

})(window);