//$Id$
var ZRDashboard = (function(window){
    
        var controller = {};
    
        var layoutEvents = {}, nodeEvents = {};
    
        var $C = {
            BUILD : "build", INIT : "init",RESIZE : "resize", REMOVE : "remove",//No i18N
            LEFT : "left", TOP : "top", WIDTH : "width", HEIGHT : "height",//No i18N
            PX : "px"//No i18N
        }
        var utilities = {
            isDefined : function(value){
                return value != undefined;
            },
            isUndefined : function(value){
                return value === undefined;
            },
            isNull : function(value){
                return value === null;
            },
            isNullOrUndefined : function(value){
                return this.isUndefined(value) || this.isNull(value);
            },
            getValue : function(value, context, args){
                if((typeof value).toLowerCase() === "function"){
                    return value.apply(context, args);
                }
                return value;
            },
            executeMethod : function(context, method, args){
                if(method && (typeof method).toLowerCase() === "function"){
                    return method.apply(context);
                }
                else if(method && (typeof method).toLowerCase() === "string"){
                    if(context[method]){
                        return this.executeMethod(context, context[method], args);
                    }
                }
            },
            addProps : function(base, child){
                for(var key in child){
                    base[key] = child[key];
                }
            },
            getConstants : function($c){
                this.addProps($C, $c);
                return $C;
            },
            extend : function(parent, child, constructor){
                var $child = Object.create(parent);
                this.addProps($child, child);
                if(this.isDefined(constructor)){
                    Object.defineProperty($child, "$$constructor", {value : constructor});//No i18N
                }
                return $child;
            },
            getMixedChildProto : function(parentClass, childClass){
                return this.extend(parentClass.prototype, childClass.prototype, childClass);
            },
            getNewClass : function(_class){
                return (function newClass(){
                    _class.apply(this, Array.prototype.slice.call(arguments));
                });
            },
            getInheritedClass:  function(_class, userProto){
                var newClass = this.getNewClass(_class);
                newClass.prototype = this.extend(_class.prototype, userProto, newClass);
                return newClass;
            },
            createTypeInheritedClass : function(_class, type, userProto){
                var parentProto = Object.getPrototypeOf(_class.prototype);
                var createdClass = _class[type], added = false;
                if(!createdClass){
                    if(Object.prototype === parentProto){
                        createdClass = _class[type] = this.getInheritedClass(_class, userProto);
                        added = true;
                    }
                    else{
                        var parentClass = parentProto.constructor;
                        if(!parentClass[type]){
                            this.createTypeInheritedClass(parentClass, type);
                        }
                        createdClass = _class[type] = this.getNewClass(_class);
                        createdClass.prototype = this.extend(this.getMixedChildProto(parentClass[type], _class), userProto, createdClass);
                    }
                }
                if(!added){
                    this.addProps(createdClass.prototype, userProto);
                }
            },
            isDirectInstanceOf : function(instance, prototype){
                return prototype === Object.getPrototypeOf(instance);
            }
        };
        var TRANSFORM = "translateX(0px) translateY(0px)", TRANSFORM_SUFFIX = "Transform",	TRANSFORM_LABEL = "transform",//No i18N
        BROWSER_PREFIX = "", VENDOR_PREFIX_MATCH = /-(moz|webkit|ms)-/,//No i18N
        REPLACE_X = /(X)\((.+?)\)/, REPLACE_Y = /(Y)\((.+?)\)/,
        PREFIX_X = "X(", PREFIX_Y = "Y(", SUFFIX_XY = $C.PX + ")",//No i18N
        TransformSupport = false && (function(){
            var cStyle = getComputedStyle(document.body);
            if(TRANSFORM_LABEL in cStyle){
                return true;	
            }
            BROWSER_PREFIX = Array.prototype.slice.call(cStyle).join(' ').match(VENDOR_PREFIX_MATCH)[0];
            var label = BROWSER_PREFIX+TRANSFORM_SUFFIX;
            if(label in cStyle){
                TRANSFORM_LABEL = label;
                return true;
            }
            return false;
        })(),
        actionListener = {}, isDomQ = false, domQ = [], domQCallback = [];

        function removePreviousEntry(index){
            domQ.splice(index,1);
            domQCallback.splice(index,1);
        }

        function addToDomQ(context, method){
            var domQIndex = domQ.indexOf(context);
            var existingMethods = [];
            if(domQIndex >=0 ){
                existingMethods = domQCallback[domQIndex];
                removePreviousEntry(domQIndex);
            }
            domQ.push(context);
            domQCallback.push(existingMethods.push(method));
        }
    
        var DOM = {
            div : "div", svg : "svg", span : "span", style : "style",//No i18N
            x : "x", X : "X", y : "y", Y : "Y",//No i18N
            translate : function(element, type, value){
                var transform = element.style.transform || TRANSFORM;
                switch(type){
                    case this.X :
                    case this.x :{
                        if(!TransformSupport){
                            this.setLeft(element,value);
                            return;
                        }
                        transform.replace(REPLACE_X, PREFIX_X+value+SUFFIX_XY);
                        break;
                    }
                    case this.Y :
                    case this.y :{
                        if(!TransformSupport){
                            this.setTop(element,value);
                            return;
                        }
                        transform.replace(REPLACE_Y, PREFIX_Y+value+SUFFIX_XY);
                        break;
                    }
                }
                element.style.transform = transform;
            },
            setStyle : function(element, key, value){
                element.style[key] = value + $C.PX;
            },
            setLeft : function(element, value){
                this.setStyle(element, $C.LEFT, value);
            },
            setTop : function(element, value){
                this.setStyle(element, $C.TOP, value);
            },
            setWidth : function(element, value){
                this.setStyle(element, $C.WIDTH, value);
            },
            setHeight : function(element, value){
                this.setStyle(element, $C.HEIGHT, value);
            },
            classNames : function(element, classNames){
                classNames = Array.isArray(classNames) ? classNames : [classNames];
                if(element.classList){
                    element.classList.add.apply(element.classList, classNames);
                    return;
                }
                for(var i=0; i<classNames.length; i++){
                    this.className(element, classNames[i], element.classList);
                }
            },
            className : function (element, className, cList){
                if(cList || element.classList){
                    element.classList.add(className);
                }
                else if((" "+element.className+" ").indexOf(" "+className+" ") < 0){
                    element.className = element.className.trim() + " " + className;
                }
            },
            setId : function(element, id){
                element.setAttribute("id", id);
            },
            getNewHtmlElement : function(tagName, css){
                try{
                    var element = document.createElement(tagName);
                    if(css && utilities.isDefined(css.className)){
                        this.className(element, css.className);
                    }
                    if(css && utilities.isDefined(css.id)){
                        this.setId(element,css.id);
                    }
                    return element;
                }
                catch(e){
                    //console.error("Html Element not created", e);
                }
            },
            getNewSvgElement : function(tagName){
                try{
                    return document.createElementNS(d3.ns.prefix.svg, tagName);
                }
                catch(e){
                    //console.error("svg element not created", e);
                }
            },
            pushListener : function(key, context, method){
                if(!actionListener[key]){
                    actionListener[key] = [];
                    actionListener[key].context = context;
                }
                actionListener[key].push(method);
            },
            executeDomEvents : function(timeOut){
                for(var key in actionListener){
                    var item = actionListener[key];
                    item.forEach(function(method){
                        if(timeOut){
                            window.setTimeout(function(){
                                utilities.executeMethod(item.context, method);
                            },timeOut);
                        }
                        utilities.executeMethod(item.context, method);
                    });
                    delete actionListener[key];
                }
            },
            initDomQ : function(){
                isDomQ = true;
            },
            dispatchDomQ : function(){
                domQ.forEach(function(context, index){
                    var methods = domQCallback[index];
                    methods.forEach(function(method){
                        method.call(context);
                    });
                });
                domQ.length = domQCallback.length = 0;
                isDomQ = false;
            }
        }
    
        function outOfBounds(array, value, start, end, higher){
            if(value > array[end]){
                return higher ? (utilities.isDefined(array[end+1]) ? end + 1 : undefined) :  end;
            }
            else if( value < array[start]){
                return higher ? start : (utilities.isDefined(array[start - 1]) ? start - 1 : undefined);
            }
            return true;
        }
    
        function withInBounds(array, value, start, end, equals, higher){
            if(equals){
                if(value == array[start]){ return start; }
                else if(value == array[end]){ return end; }
            }
            if(value == array[start]){
                return higher ? end : (utilities.isDefined(array[start - 1]) ? start - 1 : undefined);
            }
            else if(value == array[end]){
                return higher ? (utilities.isDefined(array[end + 1]) ? end + 1 : undefined) : start;
            }
            else if(value > array[start] && value < array[end]){
                return higher ? end : start;
            }
            
        }
    
        function binarySearch(array, value, equals, higher){
            if(array.length > 1){
                var start = 0, end = array.length - 1;
                while((start+1) !== end){
                    var middle = Math.floor((start + end)/2);
                    var middleValue = array[middle];
                    if(value < middleValue){
                        end = middle;
                    }
                    else{
                        start = middle;
                    }
                }
                var $withInBounds = outOfBounds(array, value, start, end, higher);
                if($withInBounds === true){
                    return withInBounds(array, value, start, end, equals, higher);
                }
                return $withInBounds;
            }
            else if (array.length == 1){
                var arrayValue = array[0];
                if(equals && value == arrayValue){
                    return 0;
                }
                // else if(higher){
                //     return arrayValue > value ? 0 : undefined;
                // }
                // return value < arrayValue ? 0 : undefined;
                else{
                    if(value < arrayValue){
                        return higher ? 0 : undefined;
                    }
                    else if(value > arrayValue){
                        // console.log("HIGHER - ", higher, "RETURN : ",  higher ? undefined : 0, array, value);
                        return higher ? undefined : 0;
                    }
                    // else if(value == arrayValue){
                    //     return 0;
                    // }
                    else{
                        return undefined;
                    }
                }
            }
        }
    
        function BaseObject(primaryKey, parent){
            Object.defineProperties(this,{
                length : {
                    writable : true,
                    value : 0
                },
                primaryKey : {
                    value : primaryKey
                },
                parent : {
                    writable : false,
                    value : parent
                }
            });
        }
    
        Object.defineProperties(BaseObject.prototype, {
            previous : {
                value : function(){
                    var keys = Object.keys(this.parent);
                    var previousIndex = keys.indexOf(this.primaryKey) - 1;
                    return this.parent[keys[previousIndex]];
                }
            },
            next : {
                value : function(){
                    var keys = Object.keys(this.parent);
                    var nextIndex = keys.indexOf(this.primaryKey) + 1;
                    return this.parent[keys[nextIndex]];
                }
            },
            isEmpty : {
                value : function(){
                    return this.length === 0; 
                }
            },
            add : {
                value : function($node){
                    var primaryValue = $node[this.primaryKey];
                    this[primaryValue] = $node;
                    this.length++;
                }
            },
            has : {
                value : function($node){
                    var primaryValue = $node[this.primaryKey];
                    return this[primaryValue] ? true : false;
                }
            },
            equals : {
                value : function($node){
                    var primaryValue = $node[this.primaryKey];
                    return this[primaryValue] && this[primaryValue] === $node;
                }
            },
            remove : {
                value : function($node){
                    var primaryValue = $node[this.primaryKey];
                    delete this[primaryValue];
                    this.length--;
                }
            }
        });
    
        // configuation 
        // Keys
        // primary Key
        // extent Key
        // base - to serialize
        function Order(configuration, structure){
            this.configuration = configuration;
            this.structure = structure;
            configuration.keys.forEach((function(key){
                this[key] = {};
            }).bind(this));
        }
    
        Order.prototype.add = function($node){
            this.configuration.keys.forEach((function(key){
                var baseObject = null;
                if($node[key] in this[key]){
                    baseObject = this[key][$node[key]];
                }
                else{
                    baseObject = this[key][$node[key]] = new BaseObject(this.configuration.primaryKey);
                }
                if(!baseObject.has($node)){
                    baseObject.add($node);
                }
            }).bind(this));
        }
    
        Order.prototype.remove = function($node){
            $return = true;
            this.configuration.keys.forEach((function(key){
                var baseObject;
                if($node[key] in this[key]){
                    baseObject = this[key][$node[key]];
                    if(baseObject.has($node)){
                        baseObject.remove($node);
                    }
                    else{
                        $return = false;
                    }
                    if(baseObject.isEmpty()){
                        delete this[key][$node[key]];
                    }
                }
            }).bind(this));
            return $return;
        }
    
        Order.prototype.serialize = function(){
            this.structure.length = 0;
            
            var baseOrder = this[this.configuration.base];
            var baseKeys = (Object.keys(baseOrder)).sort(function(a,b){return a-b;});
            baseKeys.forEach((function(baseKey){
                var primaryKeys = (Object.keys(baseOrder[baseKey])).sort(function(a,b){return a-b;});
                primaryKeys.forEach((function(primaryKey){
                    var $node = baseOrder[baseKey][primaryKey];
                    this.structure.push($node);
                }).bind(this));
            }).bind(this));
        };

        Order.prototype.clone = function(wrapper, constru){
            // var cloneOrder = new this.constructor(this.configuration, this.structure);
            var cloneOrder = new constru(this.configuration, this.structure);
            this.structure.forEach(function ($node){
                cloneOrder.add(wrapper ? wrapper($node) : $node);
            });
            return cloneOrder;
        }

        var patternStore = {};
    
        function getSelected(mod, blocks){
            return patternStore[blocks] ? patternStore[blocks][mod] : null;
        }
    
        function clearPattern(patternMatcher){
            patternMatcher.pattern = [];
            patternMatcher.extraPattern = [];
        }
    
        function PatternMatcher(pixel, spacing, blocks){
            this.pixel = pixel;
            this.spacing = spacing;
            this.blocks = blocks;
            this.match();
        }
    
        PatternMatcher.prototype.match = function(){
            clearPattern(this);
            var mod = (this.pixel % this.blocks), pixel = this.pixel;
            var selected = getSelected(mod, this.blocks);
            var currentSpace = selected.leftOver;
            while(this.spacing >= currentSpace){
                pixel--; mod--;
                if(mod == -1){
                    mod = this.blocks - 1;
                }
                selected = getSelected(mod, this.blocks);
                currentSpace = selected.leftOver + (this.pixel - pixel);
            }
            this.selection = pixel;
            selected.apply(this);
        }
    
        function getX(x){
            return x;
        }
    
        function getXPlus1(x){
            return x+1;
        }
    
        function getXMinus1(x){
            return x-1;
        }
    
        function getRoundPixel(pixel, blocks){
            return Math.round(pixel/blocks);
        }
    
        function fillPattern(pattern, blocks, remainder, leftOver, blockSize, method, extra){
            var remain = 0 ;
            if(0 != (leftOver %2)){
                leftOver = extra ? getXMinus1(leftOver) : getXPlus1(leftOver);
            }
            if(leftOver > 0){
                var last = Math[method](blocks/leftOver);
                for(var i=2; i<=last; i++){
                    if(i!= last){
                        pattern.push(getX);
                        continue;
                    }
                    pattern.push(extra ? getXPlus1 : getXMinus1)
                }
            }
        }
    
        function getOccupied(blocks, pattern, extraPattern, patternSize, blockSize){
            var completeRepetations = Math.floor(blocks/ pattern.length);
            var occupied = (completeRepetations * patternSize);
            var extraFormations = blocks - (completeRepetations * pattern.length);
            for(var ex=0; ex<extraFormations; ex++){
                extraPattern[ex] = pattern[ex];
                occupied += pattern[ex](blockSize);
            }
            return occupied;
        }
    
        function Pattern(reminder, blocks){
            var pattern = this.pattern = [];
            var extraPattern = this.extraPattern = [];
            var extra = true, method = "ceil", leftOver = 0, occupied = 0;//No i18N
            this.patternSize = 0;
            this.blockSize = Math.round(reminder/blocks);
            var total = this.blockSize * blocks;
            pattern.push(getX);
            if(reminder >= total){
                leftOver = reminder - total;
            }
            else{
                leftOver = total - reminder;
                method = "floor";//No i18N
                extra = false;
            }
            fillPattern(pattern, blocks, reminder, leftOver, this.blockSize, method, extra);
            for(var p=0; p<pattern.length; p++){
                this.patternSize += pattern[p](this.blockSize);
            }
            var occupied = getOccupied(blocks, pattern, extraPattern, this.patternSize, this.blockSize);
            this.leftOver = reminder - occupied;
        }
    
        Pattern.prototype.apply = function(patternMatcher){
            var px = patternMatcher.px = getRoundPixel(patternMatcher.selection, patternMatcher.blocks);
            patternMatcher.patternSize = patternMatcher.extraSize = 0;
            for(var pt = 0; pt<this.pattern.length; pt++){
                patternMatcher.pattern[pt] = this.pattern[pt](px);
                patternMatcher.patternSize += patternMatcher.pattern[pt];
            }
            for(var ex=0; ex<this.extraPattern.length; ex++){
                patternMatcher.extraPattern[ex] = this.extraPattern[ex](px);
                patternMatcher.extraSize += patternMatcher.extraPattern[ex];
            }
            patternMatcher.occupied = (Math.floor(patternMatcher.blocks/patternMatcher.pattern.length) * patternMatcher.patternSize);
            patternMatcher.occupied += patternMatcher.extraSize;
            patternMatcher.leftOver = this.leftOver;
        }
    
        function getTotalPatternPixel(blocks, pattern){
            var pt = 0, ptLen = pattern.length;
            var pixels = 0;
            for(var bl=0; bl<blocks; bl++, pt++){
                if(pt === ptLen){
                    pt = 0;
                }
                pixels += pattern[pt];
            }
            return pixels;
        }

        function getBlockForPixel(pixel, pattern, patternSum){
            var block = Math.floor(pixel/patternSum);
            var leftOver = pixel - (block * patternSum);
            for(var pt=0; pt<pattern.length; pt++){
                if(pattern[pt] < leftOver){
                    block++;
                }
                else if(leftOver >= 0){
                    block += Math.round(pattern[pt]/leftOver);
                }
                leftOver -= pattern[pt];
            }
        }
    
        function addPatternBlockStore(blocks){
            if(patternStore[blocks] === undefined){
                patternStore[blocks] = {};
                for(var i=1; i<=blocks; i++){
                    patternStore[blocks][i % blocks] = new Pattern(i % blocks,blocks);
                }
            }
        }
        
        function styleDom(layout){
        	var wrapper = layout.wrapper;
        	wrapper.style.width = layout.$widthPattern.occupied + "px";
            wrapper.style.borderWidth = (layout.$widthPattern.spacing/2) + "px"; //No i18N
            if(layout.styleEl){
            	wrapper.removeChild(layout.styleEl);
            }
            var style = layout.styleEl = document.createElement("style");
            var styleContent =".ly-node{" + //No i18N
            		"border-width : " + (layout.$widthPattern.spacing/2) + "px;" + //No i18N
            		"}"
            style.innerText = styleContent;
            wrapper.appendChild(style);
        }
        
        function createDOM(layout){
            var wrapper = layout.wrapper = DOM.getNewHtmlElement(DOM.div, {className : layout.wrapperClass});
            
            layout.nodeHolder = DOM.getNewHtmlElement(DOM.div, {className : layout.holderClass});
            layout.nodeHolder.style.position = "relative";
            layout.wrapper.appendChild(layout.nodeHolder);
            layout.container.appendChild(layout.wrapper);
            styleDom(layout);
        }
    
        var $nodeKeys = ["x", "y", "width", "height"];//No i18N
        
        function normalizeNode(key){
            this[key] = this[key];
        }
    
        var $nodeStyle = ["setX", "setY", "setWidth", "setHeight"];//No i18N
    
        function nodeStyle($node){
            $nodeStyle.forEach(function(setter){
                $node[setter]();
            });
        }
        
        function nodeDOM($node){
            var element = DOM.getNewHtmlElement(DOM.div, {className : $node.elementClassName});
//            element.style.boxSizing = "border-box"; //No i18N
//            element.style.position = "absolute";
            var container = DOM.getNewHtmlElement(DOM.div, {className : $node.containerClassName});
            if($node.autoHeight){
                // DOM.className(element,$node.autoHeightClass);
                DOM.className(container,$node.autoHeightClassName);
            }
            element.appendChild(container);
            var dom = {
                element : element,
                container : container
            }
            $node.addProperty("dom", dom);//No i18N
            nodeStyle($node);
        }
    
        function Node($node, layout, noDom){
            for(var key in $node){
                this[key] = $node[key];
            }
            $nodeKeys.forEach(normalizeNode.bind(this));
            Object.defineProperty(this, "Layout", { value : layout });//No i18N
            if(noDom != true){
                nodeDOM(this);
            }
        }
    
        Node.style = nodeStyle;
    
        Node.getTotalPatternPixel = getTotalPatternPixel;
        Node.getBlockForPixel = getBlockForPixel;
    
        Node.prototype.setX = function(){
            //DOM.translate(this.dom.element, DOM.X, this.x * this.Layout.XPixel);
            if(isDomQ){
                addToDomQ(this, this.setX);
                return;
            }
            var pattern = this.Layout.$widthPattern.pattern;
            DOM.translate(this.dom.element, DOM.X, getTotalPatternPixel(this.x, pattern));
        }
    
        Node.prototype.setY = function(){
            if(isDomQ){
                addToDomQ(this, this.setY);
                return;
            }
            if(this.yOffset){
            	DOM.translate(this.dom.element, DOM.Y, (this.y * this.Layout.YPixel) - this.yOffset);
            	return;
            }
            DOM.translate(this.dom.element, DOM.Y, this.y * this.Layout.YPixel);
        }
    
        Node.prototype.setWidth = function(){
            //DOM.setWidth(this.dom.element, this.width * this.Layout.XPixel);
            if(isDomQ){
                addToDomQ(this, this.setWidth);
                return;
            }
            var pattern = this.Layout.$widthPattern.pattern;
            DOM.setWidth(this.dom.element, getTotalPatternPixel(this.width, pattern));
        }
        
        Node.prototype.setHeight = function(){
            if(isDomQ){
                addToDomQ(this, this.setHeight);
                return;
            }
            if(this.hOffset){
            	DOM.setHeight(this.dom.element, (this.height * this.Layout.YPixel) + this.hOffset);
            	return;
            }
            DOM.setHeight(this.dom.element, this.height * this.Layout.YPixel);
        }
        
        Node.prototype.resize = function(noCallBack){
        	nodeStyle($node);
        	if(!noCallBack){
        		fireCallback(this, "onResize", true); //No i18N
        	}
        }
        
        function nodeResize($node){
        	nodeStyle($node);
        	fireCallback($node, "onResize", true); //No i18N
        }

        Node.prototype.reflow = function(){
            
        }

        Node.prototype.addProperty = function(key, value, visible, writable){
            visible = visible ? true : false;
            writable = writable ? true : false; 
            Object.defineProperty(this, key, {value : value, enumerable : visible, writable : writable == true ? true : false});
        }

    
        Node.prototype.elementClassName = "ly-node";//No i18N
        Node.prototype.containerClassName= "ly-container";//No i18N
        Node.prototype.autoHeightClassName= "ly-autoHeight";//No i18N
        Node.prototype.Name = "Node";//No i18N
        
        Object.defineProperties(Node.prototype,{
            right : {
                get : function(){
                    return this.x + this.width;
                }
            },
            bottom : {
                get : function(){
                    return this.y + this.height;
                }
            },
            offsetRight : {
                get : function(){
                    this.offsetX + this.offsetWidth;
                }
            },
            offsetBottom : {
                get : function(){
                    this.offsetY + this.offsetHeight;
                }
            }
        });
    
        Node.prototype.remove = function(){
            var structure = this.Layout.structure;
            var index = Array.prototype.indexOf.call(structure,this);
            var splicedArray = structure.splice(index, 1);
            this.Layout.nodeHolder.removeChild(this.dom.element);
            if(splicedArray.length){
                fireCallback(this, $C.REMOVE, true, Node.prototype);
                return true;
            }
            return false;
        }
        
        function addNodeType(layout, type, userProto){
            var NodeClass = layout.prototype.$Structure.prototype.Node;
            utilities.createTypeInheritedClass(NodeClass, type, userProto);
        }
        
        function Structure(json, layout){
            Object.defineProperties(this,{
                length : {
                    writable : true, 
                    value : 0 
                },
                Layout : {
                    value : layout
                },
                NodeTypes : {
                    value : {}
                }
            });
            this.addNodes(json);
        }
        Structure.prototype.Node = Node;
        Structure.prototype.addNodes = function(nodes){
            if(Array.isArray(nodes)){
                nodes.forEach((this.add).bind(this));
            }
            else{
                for(var key in nodes){
                    this.add(nodes[key]);
                }
            }
        }
    
        Structure.prototype.add = function(node){
            var type = this.Layout.getNodeType(node);
            if(type && !this.Node[type]){
                utilities.createTypeInheritedClass(this.Node, type);
            }
            var $node = type ? new this.Node[type](node, this.Layout) : new this.Node(node, this.Layout);
            this.push($node);
            this.Layout.nodeHolder.appendChild($node.dom.element);
            // var mode = getMode(this.Layout.modeName);
            // adoptModeForInstance(mode.node ,$node);
            return $node;
        }
        
        Structure.prototype.callback = function(method, afterCB){
            this.forEach(function(item){
                if(typeof method === "function"){
                    method(item);
                }
                else{
                    fireCallback(item, method, true);	
                }
                if(utilities.isDefined(afterCB)){
                    afterCB(item);
                }
            });	
        };
    
        Structure.prototype.push = Array.prototype.push;
        Structure.prototype.pop = Array.prototype.pop;
        Structure.prototype.splice = Array.prototype.splice;
        Structure.prototype.forEach = Array.prototype.forEach;
    
        var Configuration = {
            columns : 80,
            rows : 40
        };
    
        function getInheritedConfig(configuration){
            return utilities.extend(Configuration, configuration);
        }
    
        function Layout(container, configuration){
            configuration = this.configuration = getInheritedConfig(configuration);
            addPatternBlockStore(this.configuration.columns);
            this.container = container;
            this.structure = new this.$Structure(configuration.jsonLayout, this);
            //this.YPixel = this.XPixel = getRoundPixel(container.clientWidth, configuration.columns);
//            this.$widthPattern = this.$pattern(this.container.clientWidth, this.configuration.columns);
//            this.YPixel = this.$widthPattern.px;
            fireCallback(this, $C.INIT, false, Layout.prototype);
        }
    
        Layout.prototype.$Structure = Structure;
    
        Layout.prototype.$build = function(json){
            var startWidth = this.container.clientWidth;
        	this.$widthPattern = this.$pattern(this.container.clientWidth, this.configuration.columns);
            this.YPixel = this.$widthPattern.px;
            createDOM(this);
            this.container.style.overflowY = "auto"; //No i18N
            this.structure.addNodes(json);
            var mode = getMode(this.modeName);
            if(utilities.isDefined(mode)){
                adoptModeForInstance(mode.layout,this);
            }
            if(utilities.isDirectInstanceOf(this, Layout.prototype)){
                this.structure.callback("onAdd");//No i18N
            }
            fireCallback(this, $C.BUILD, false, Layout.prototype);
            if(startWidth !== this.container.clientWidth){
                this.$widthPattern = this.$pattern(this.container.clientWidth, this.configuration.columns);
                this.YPixel = this.$widthPattern.px;
        	styleDom(this);
        	this.structure.callback(nodeStyle);
            }
        }
    
        Layout.prototype.$resize = function(force){
        	if(!this.wrapper){
        		return false;
        	}
        	this.$widthPattern = this.$pattern(this.container.clientWidth, this.configuration.columns);
            this.YPixel = this.$widthPattern.px;
        	styleDom(this);
        	this.structure.callback(nodeResize);
        	return true;
        }
    
        Layout.prototype.$changeMode = function(mode){
            adoptMode(mode);
            return this;
        }
    
        Layout.prototype.$getJSON = function(){
            return JSON.stringify(this.structure);
        }
    
        Layout.prototype.$pattern = function(pixel, blocks){
            return new PatternMatcher(pixel, this.getSpacing(), blocks);
        }

        Layout.prototype.getNodeType = function(node){
            return node.type;
        }
    
        Layout.prototype.wrapperClass = "ly-wrapper";//No i18N
        Layout.prototype.holderClass = "ly-holder";//No i18N
    
        var modes = {}

        window.$$modes = modes;
    
        function createMode(name, modules){
            var mode = modes[name] = {};
            for(var key in modules){
                if(utilities.isUndefined(mode[controller[key].scope])){
                    mode[controller[key].scope] = {};
                }
                mode[controller[key].scope][key] = modules[key];
            };
        }
    
        function getMode(name){
            return modes[name];
        }
    
        function removeMode(name){
            if(modes[name]){
                delete modes[name];
            }
        }
    
        function adoptModeForInstance(scope, instance){
            for(var key in scope){
                switch(scope[key]){
                    case "enable" : {//No i18N
                        controller[key].enable(instance);
                        break;
                    }
                    case "disable" : {//No i18N
                        controller[key].disable(instance);
                        break;
                    }
                }
            }
        }
    
        function adoptMode(name, layout){
            var mode = getMode(name);
            if(mode.layout){
                adoptModeForInstance(mode.layout, layout);	
            }
            if(mode.node){
                layout.structure.forEach(function($node){
                    adoptModeForInstance(mode.node, $node);
                });
            }
        }
    
        function fireCallback(context, type, nodeEvent,  prototype){
            var events = (nodeEvent || context instanceof Node) ? nodeEvents : layoutEvents;
            if(utilities.isNullOrUndefined(prototype) || prototype === Object.getPrototypeOf(context)){
                if(utilities.isDefined(events[type])){
                    events[type].forEach(function(callback){
                    	try{
                    		callback(context);
                    	}
                        catch(e){
                        	console.error(e);
                        }
                    });
                }
            }
        }
        
        function addEventListener(type, callback, node){
            var events = node ? nodeEvents : layoutEvents;
            if(utilities.isUndefined(events[type])){
                events[type] = [];
            }
            events[type].push(callback);
        }
        
        controller.DOM = DOM;
        controller.utilities = utilities;
        controller.Order = Order,
        controller.Layout = Layout;
        controller.addEventListener = addEventListener;
        controller.fireCallback = fireCallback;
        controller.binarySearch = binarySearch;
        controller.initialConfiguration = Configuration;
        controller.addNodeType = addNodeType;
        controller.createMode = createMode;
        return controller;
    
})(window);

(function(){
    
    function initialize(){
        var utilities = ZRDashboard.utilities,
        $C = utilities.getConstants(),
        DOM = ZRDashboard.DOM,
        Order = ZRDashboard.Order,
        fireCallback = ZRDashboard.fireCallback,
        binarySearch = ZRDashboard.binarySearch,
        Layout = ZRDashboard.Layout,
        Structure = Layout.prototype.$Structure,
        Node = Structure.prototype.Node,
        getTotalPatternPixel = Node.getTotalPatternPixel,
        orderConfig = {
            keys : ["y", "bottom"], //No i18N
            base : "y", //No i18N
            primaryKey : "x"//No i18N
        };
        
        function getCropped(left, stop, orderX, keys, matched){
            var start = binarySearch(keys, left, false);
            var $start = orderX[keys[start]];
            if(utilities.isUndefined(start) || ($start && $start.right <= left)){
                start = binarySearch(keys, left, true, true);
            }
            for(;start<=stop; start++){
                var $node = orderX[keys[start]];
                matched.push($node);
            }
        }

        function cropNodes(left, right, orderX, matched){
            var keys = Object.keys(orderX);
            var stop = binarySearch(keys, right, false);
            if(utilities.isDefined(stop)){
                var $stop = orderX[keys[stop]];
                if($stop.right > left){
                    if(matched){
                        getCropped(left, stop, orderX, keys, matched);
                    }
                    return $stop;
                }
            }
        }

        utilities.cropNodes = cropNodes;

        function VOrder(configuration, structure){
            Order.call(this, configuration, structure);
        }

        function addAndAlignTop(order, $node, equals){
            order.add($node);
            order.stickToTop($node, equals);
            $node.dom.element.style.outline = "";
        }

        VOrder.prototype = utilities.extend(Order.prototype,{
            stripAllAfter : function(point, key, callback){
                var keys = Object.keys(this[key]);
                var index = binarySearch(keys, point, true, true);
                var list = [];
                for(;index !== false && index>=0 && index<keys.length; index++){
                    var currentPoint = keys[index];
                    var line = this[key][currentPoint];
                    for(var _node in line){
                        var $node = line[_node];
                        this.remove($node);
                        if(callback){
                            callback($node);
                        }
                        list.push($node);
                        $node.dom.element.style.outline = "1px solid yellow";//No i18N
                    }
                }
                return list;
            },
            addAndAlignTop : function(nodes, equals){
                if(Array.isArray(nodes)){
                    for(var _node = 0; _node<nodes.length; _node++){
                        var $node = nodes[_node];
                        addAndAlignTop(this, $node, equals);
                    }
                    return;
                }
                addAndAlignTop(this, nodes, equals);
            },
            stickToTop : function($node, equals){
                this.remove($node);
                var keys = Object.keys(this.bottom);
                var end = binarySearch(keys, $node.y, equals != undefined ? equals : true);
                var selected = null;
                while(utilities.isNullOrUndefined(selected) && !utilities.isNull(end) && end >= 0){
                    var orderX = this.bottom[keys[end]];
                    selected = cropNodes($node.x, $node.right, orderX);
                    end--;
                }
                // $node.y = 0;
                $node.addProperty("oldbottom", $node.bottom, false, true);//No i18N
                if(selected || end == undefined){
                    $node.y = selected ? selected.bottom : 0;
                    $node.setY();
                }
                this.add($node);
            }
        });

        function increaseHeight($node, oldHeight){
            var difference = $node.height - oldHeight;
            function diff($node){
                $node.y += difference;
            }
            var removedNodes = $node.Layout.order.stripAllAfter($node.oldbottom, "y", diff);//No i18N
            $node.Layout.order.add($node);
            $node.Layout.order.addAndAlignTop(removedNodes);
        }

        function decreaseHeight($node){
            var childList = $node.childList();
            childList.forEach(function($node){
                // if($node.childList().length <= 1){
                    $node.Layout.order.stickToTop($node);
                // }
            });
            childList.forEach(decreaseHeight);
        }
        
        function flush_yOffset($node){
        	delete $node.yOffset;
        	delete $node.hOffset;
        }
        
        function add_yOffset_spacer(spacer, hoff){
        	var yLine = spacer.bottom;
            var xOrder = spacer.Layout.order.y[yLine];
        	var list = [];
            utilities.cropNodes(spacer.x, spacer.right, xOrder, list);
            list.forEach(function($node){
            	$node.yOffset = spacer.yOffset;
            	$node.setY();
            	$node.hOffset = spacer.hOffset;
            	$node.setHeight();
            });
        }

        function autoHeight($node){
            if($node.autoHeight){
            	$node.dom.container.style.height = "";
                var content_height = $node.scrollHeight != null ? $node.scrollHeight : $node.dom.container.scrollHeight;
                var includedSpacing = content_height
                if(content_height != 0){
                	includedSpacing += $node.Layout.getSpacing();
                }
                var newHeight = Math.ceil(includedSpacing/$node.Layout.YPixel) || 1;
                var oldHeight = $node.height;
                if(newHeight != oldHeight){
                    $node.Layout.order.remove($node);
                    var point = $node.bottom;
                    $node.height = newHeight;
                    $node.setHeight();
                    $node.oldbottom = point;
                    var tempHOffset = $node.hOffset ? $node.hOffset : 0;
                    $node.hOffset = 0;
                    if(newHeight > oldHeight){
                        increaseHeight($node, oldHeight);
                    }
                    else{
                        $node.Layout.order.add($node);
                        decreaseHeight($node);
                    }
                    $node.hOffset = tempHOffset;
                }
                if(content_height == 0){
                    // hide the content
                    // $node.dom.element.style.visibility = "hidden";
                    $node.$$Spacer = true;
                }
                else{
//                	if($node.type != "SPACER"){
//                		var diffPx = $node.dom.element.offsetHeight - includedSpacing;
//                		$node.dom.element.style.height = includedSpacing + "px";
//                    	function diff($node){
//                    		if(!$node.yOffset){
//                    			$node.yOffset = 0;
//                    		}
//                            $node.yOffset += diffPx;
//                        }
//                        var removedNodes = $node.Layout.order.stripAllAfter($node.bottom, "y", diff);//No i18N
//                        $node.Layout.order.addAndAlignTop(removedNodes);
//                	}
                	if($node.type != "SPACER"){
	            		var diffPx = $node.dom.element.offsetHeight - includedSpacing;
	            		$node.dom.element.style.height = includedSpacing + "px";
	                	function diff($node){
	                		if(!$node.yOffset){
	                			$node.yOffset = 0;
	                		}
	                		$node.yOffset += diffPx;
	                        $node.setY();
	                        if(!$node.hOffset){
	                        	$node.hOffset  = 0;
	                        }
	                        $node.hOffset += diffPx;
	                        $node.setHeight();
	                        if($node.type === "SPACER"){
	                        	add_yOffset_spacer($node);
	                        }
	                    }
	                	var list = [];
	                	var xOrder = $node.Layout.order.y[$node.bottom];
	                	if(xOrder){
	                		utilities.cropNodes($node.x, $node.right, xOrder, list);
		                    list.forEach(diff);
	                	}
	            	}
                	$node.dom.container.style.height = "100%";
                }
            }
        }

        function VNode(node, layout, noDom){
            Node.call(this, node, layout, noDom);
        }

        VNode.prototype = utilities.extend(Node.prototype, {
            setX : function(){
                var pattern = this.Layout.$widthPattern.pattern;
                DOM.translate(this.dom.element, DOM.X, getTotalPatternPixel(this.x, pattern));
            },
            setWidth : function(){
                var pattern = this.Layout.$widthPattern.pattern;
                DOM.setWidth(this.dom.element, getTotalPatternPixel(this.width, pattern));
            },
            remove : function(){
                var removed = Node.prototype.remove.call(this);
                removed = removed & this.Layout.order.remove(this);
                if(removed){
                    var removed = this.Layout.order.stripAllAfter(this.bottom, "y");//No i18N
                    this.Layout.order.addAndAlignTop(removed);
                    this.Layout.order.serialize();
                }
            },
            childList : function(){
                var list = [];
                var line = this.Layout.order.y[this.oldbottom];
                if(line){
                    cropNodes(this.x, this.right, line, list);
                }
                return list;
            },
            parentList : function(){
                var list = [];
                var line = this.Layout.order.bottom[this.y];
                if(line){
                    cropNodes(this.x, this.right, line, list);
                }
                return list;
            },
            $autoHeight : function(){
            	autoHeight(this);
            	ZRDashboard.spacerCallback(this.Layout);
            }
        }, VNode);

        function VStructure(json, layout){
            Structure.call(this, json, layout);
        }

        VStructure.prototype = utilities.extend(Structure.prototype, {
            Node : VNode,
            add : function(node){
                var $node = Structure.prototype.add.call(this, node);
                this.Layout.order.add($node);
                //this.Layout.order.stickToTop($node);
            }
        });
        
        function VLayout(container, configuration){
            Layout.call(this, container, configuration);
            this.order = new VOrder(orderConfig, this.structure);
            // this.$widthPattern = this.$pattern(this.container.clientWidth, this.configuration.columns);
            // this.YPixel = this.$widthPattern.px;
            fireCallback(this, $C.INIT);
        }

        VLayout.prototype = utilities.extend(Layout.prototype, {
            $type : "VLayout",//No i18N
            $build : function(json){
                Layout.prototype.$build.call(this, json);
                this.structure.forEach(this.order.stickToTop.bind(this.order));
                this.structure.callback("onAdd");//No i18N
                this.structure.callback(autoHeight);
                fireCallback(this, $C.BUILD);
            },
            $resize : function(force){
            	var doResize = Layout.prototype.$resize.call(this, force);
            	if(doResize){
            		this.structure.callback(flush_yOffset);
            		this.structure.callback(autoHeight);
            		fireCallback(this, $C.RESIZE);
            	}
            	return doResize;
            },
            $Structure : VStructure
        });

        
        ZRDashboard.VLayout = VLayout;
        ZRDashboard.VNode = VNode;
        ZRDashboard.VOrder = VOrder;
    }

    

    if(window.ZRDashboard){
        initialize();
    }
    
})();

(function(window){
    var SPACER = "SPACER", LESS_THAN = "<", LESS_THAN_EQ = "<=", PX = "px", BUILD = "build", RESIZE ="resize";//No i18N
    var utilities = ZRDashboard.utilities;
    function getAllSpacer(dashboard){
        var array = [];
        for(var key in  dashboard.structure){
            if(dashboard.getNodeType(dashboard.structure[key]) === SPACER || dashboard.structure[key].$$Spacer){
                array.push(dashboard.structure[key]);
            }
        }
        return array;
    }

    function pointCheck(point, low, high, comapare){
        switch(comapare){
            case LESS_THAN_EQ : {
                return low <= point && point < high;
                break;
            }
            case LESS_THAN : {
                return low < point && point < high;
                break;
            }
        }
        
    }

    function containsPointArray(array1, array2){
        for(var i=0; i<array1.length; i++){
            if(pointCheck(array1[i], array2[0], array2[1], i==0 ? LESS_THAN_EQ : LESS_THAN)){
                return true; 
            }
        }
        return false;
    }

    function containsPoint(point1, point2){
        return containsPointArray(point1, point2) || containsPointArray(point2, point1);
    }

    function isLineOrPoints(bottomOrder, blocks, check){
        var width = 0, points = [];
        for(var key in bottomOrder){
            width += bottomOrder[key].width;
            points.push([bottomOrder[key].x, bottomOrder[key].right]);
        }
        return check && width === blocks ? true : points;
    }

    function checkPointsMatch(yOrder, points, spacer){
        for(var key in yOrder){
            var layer = yOrder[key];
            for(var i=0; i<points.length; i++){
                if(containsPoint(points[i],[layer.x, layer.right])){
                    break;
                }
                if(i == points.length-1){
                    return false;
                }
            }
        }
        return true;
    }

    function removeIfNeeded(dashboard, order, spacer, check){
        var y = spacer.y, bottom = spacer.bottom, remove = false;
        if(order.bottom[y]){
            var points = isLineOrPoints(order.bottom[y], dashboard.configuration.columns, check);
            remove = points != true ? checkPointsMatch(order.y[bottom], points, spacer) : points; 
        }
        if(remove || (y==0 && spacer.x == 0 && spacer.width == dashboard.configuration.columns)){
            spacer.remove();
            return true;
        }
        return remove;
    }

    function decreaseY(order, yLine, px){
        var list = order.y[yLine];
        var bottom = {};
        for(var key in list){
            var layer = list[key];
            layer.dom.element.style.marginTop = -px + PX;
            bottom[layer.bottom] = true;
        }
        for(var btm in bottom){
            decreaseY(order, btm, px);
        }
    }

    function hideSpacer(spacerList, dashboard){
        var order = dashboard.order;
        var px = dashboard.YPixel;
        for(var i=1; i<=spacerList.length; i++){
            decreaseY(order, spacerList[i-1].bottom, i*px);
        }
    }

    function decreaseY(order, spacer, top, blocks){
        var yLine = spacer.bottom;
        var xOrder = order.y[yLine];
        var bottom = Infinity;
        var list = [];
        utilities.cropNodes(spacer.x, spacer.right, xOrder, list);
        list.forEach(function($node){
        	order.remove($node);
            bottom = $node.bottom < bottom ? $node.bottom : bottom;
            $node.y = top;
            if(spacer.width != blocks){
            	$node.height++;
            	$node.setHeight();
            }
            order.add($node);
            $node.setY();
        });
//        for(var key in list){
//            var $node = list[key];
//            order.remove($node);
//            bottom = $node.bottom < bottom ? $node.bottom : bottom;
//            $node.y = top;
//            order.add($node);
//            $node.setY();
//        }
        if(bottom != Infinity){
            var removed = order.stripAllAfter(bottom ,"y");//No i18N
            order.addAndAlignTop(removed);
        }
    }

    function hideSpacer(spacerList, dashboard, order, blocks){
        var px = dashboard.YPixel;
        for(var i=0; i<spacerList.length; i++){
            order.remove(spacerList[i]);
            spacerList[i].dom.element.style.visibility = "hidden";
            // decreaseY(order, spacerList[i].bottom, spacerList[i].y);
            decreaseY(order, spacerList[i], spacerList[i].y, blocks);
        }
    }
    
    // orderConfig = {
    //     keys : ["y", "bottom"], 
    //     base : "y", 
    //     primaryKey : "x"
    // };

    function nodeWrapper($node){
//        var wrapperNode = new $node.$$constructor($node, $node.Layout, true);
//        wrapperNode.yOffset = $node.yOffset;
//        wrapperNode.hOffset = $node.hOffset;
//        wrapperNode.dom = $node.dom;
//        // wrapperNode.insideObj = $node;
    	var wrapperNode = Object.create($node);
        return wrapperNode;
    }

    function getNotRemoved(notRemovedList, order){
        var list = [];
        notRemovedList.forEach(function(item){
            list.push(order.y[item.y][item.x]);
        });
        return list;
    }

    function showSpacers(dashboard){
        dashboard.structure.forEach(function($node){
            if(dashboard.getNodeType($node) === SPACER){
                $node.dom.element.style.visibility = "";
            }
            $node.setY();
        });
    }

    function switchModeCallBack(dashboard){
        var mode = dashboard.$getCurrentMode();
        
        //
    }

    function buildCallback(dashboard){
    	if(dashboard.$type !== "VLayout"){
    		return;
    	}
        var order = dashboard.order;
        order.serialize();
        var spacerList = getAllSpacer(dashboard);
        var notRemoved = [];
        // spacerList.forEach(function(spacer) {
        //     if(!removeIfNeeded(dashboard, order, spacer, true)){
        //         notRemoved.push(spacer);
        //     }    
        // });
        //hideSpacer(notRemoved, dashboard);
        
        var newOrder = order.clone(nodeWrapper, ZRDashboard.VOrder);
        var oldYKeys = JSON.stringify(Object.keys(order.y));
        var newYKeys = JSON.stringify(Object.keys(newOrder.y));
        //notRemoved = getNotRemoved(notRemoved, newOrder);
        notRemoved = getNotRemoved(spacerList, newOrder);
        hideSpacer(notRemoved, dashboard, newOrder, dashboard.configuration.columns);
        // setTimeout(function(){
        //     showSpacers(dashboard);
        // }, 2000)
    }

    if(window.ZRDashboard){
    	ZRDashboard.spacerCallback = buildCallback;
        ZRDashboard.addEventListener(BUILD, buildCallback);
        ZRDashboard.addEventListener(RESIZE, buildCallback);
    }

})(window)