//$Id$;

var WidgetUtils = new function ()
{
    this.layouts = {};
    var chartAnimation = window.WidgetQa ? false : true;
    
    var getEl = window.$ ? $ : function(id){document.getElementById(id)};
    var ZDBUtil = window.ZDBUtil ? window.ZDBUtil : {setNamedChildEls : function(){return;}};

    this.getRenderer = function (layout, cinfo) {
        if (layout < 7) {
            return {draw: WidgetUtils.fillChildNode,
                reDraw: function () {
                }, setDimensions: function () {
                }, update: function () {
                }, resize: function (dlgBody) {
                	var paddingPx = dlgBody.innerViewCtx.dashView.dashLayout ? 20 : 0;
                	var layout = dlgBody.children[0];
                	var h_scale = (dlgBody.offsetHeight-paddingPx) / layout.offsetHeight;
                	var w_scale = (dlgBody.offsetWidth-paddingPx) / layout.offsetWidth;
                	var isH_scale = h_scale < 1;
                	var isW_scale = w_scale < 1;
                	var _scale = ""
                	var _translateY = "";
                	var _position = "";
                	var _origin = "";
                	var isMarginLeft = false, isMarginTop = false;
                	
                	if(isH_scale && isW_scale){
                		_scale = "scale("+ ( h_scale > w_scale ? w_scale : h_scale)+")";
            			isMarginLeft = true;
            			isMarginTop = true;
                		_position = "static";
                	}
                	else if(isH_scale){
                		_scale = "scale(" + h_scale + ")";
                		_position = "static";
                		isMarginLeft = true;
            			isMarginTop = true;
                	}
                	else if(isW_scale){
                		_scale = "scale(" + w_scale + ")";
                		_position = "static";
                		isMarginLeft = true;
            			isMarginTop = true;
                	}
                	
                	layout.style.transform = _translateY + _scale;
                	
                	if(isMarginLeft){
                		layout.style.marginLeft = ((dlgBody.offsetWidth-paddingPx) - layout.getBoundingClientRect().width)/2+"px"; //No i18N
                		dlgBody.style.textAlign = "";
                	}else{
                		layout.style.marginLeft = "";
                		dlgBody.style.textAlign = "center";
                	}
                	
                	if(isMarginTop){
                		layout.style.marginTop = ((dlgBody.offsetHeight-paddingPx) - layout.getBoundingClientRect().height)/2+"px";//No i18N
                	}else{
                		layout.style.marginTop = ""
                	}
                	
                	layout.style.transformOrigin = _origin;
                	layout.style.position = _position;
            		
                	
                }, setAnimation: function () {
                }};
        } else {
            var renderer = WidgetCharts(layout, cinfo.DATA);
            renderer.cinfo = cinfo;
            WidgetUtils.setRendererProps(renderer, cinfo.KEYVALMAP);
            return renderer;
        }
    };

    this.setRendererProps = function (renderer, keyValueMap) {
        for (var key in keyValueMap) {
            WidgetUtils.setRendererProp(renderer, key, keyValueMap[key]);
        }
    };

    this.setRendererProp = function (renderer, key, value, reDraw) {
        switch (key) {
            case WidgetChartConstants.LEGEND_COLORS:
                renderer.update(renderer.setDataColors, [value], reDraw);
                renderer.update(renderer.setDataLabelColor, [this.getContrastColor(value)], reDraw);
                break;
            case WidgetChartConstants.LEGEND_BANDWIDTH:
                renderer.update(renderer.setDataBandWidth, [value], reDraw);
                break;
            case WidgetChartConstants.LEVER_MARKER_BANDWIDTH:
                renderer.update(renderer.setLevelMarkerBandWidth, [value], reDraw);
                break;
            case WidgetChartConstants.ORIENTATION:
                renderer.update(renderer.setAxesRotated, [value], reDraw);
                break;
            case WidgetChartConstants.DATA_LABEL:
                renderer.update(renderer.setShowDataLabel, [value], reDraw);
                break;
            case WidgetChartConstants.MAXIMUM_RANGE_COLOR:
            	var length = WidgetUtils.getChartRangeValues(renderer.cinfo).length - 1;
                renderer.update(renderer.setLevelMarkerColors, [value, length], reDraw);
                break;
            case WidgetChartConstants.RANGES_ENABLE:
                //falls through
            case WidgetChartConstants.RANGES_UNITS:
                //falls through
            case WidgetChartConstants.RANGE_ONE:
                //falls through
            case WidgetChartConstants.RANGE_TWO:
                //falls through
            case WidgetChartConstants.RANGE_THREE:
                //falls through
            case WidgetChartConstants.RANGE_FOUR:
                //falls through
            case WidgetChartConstants.RANGE_FIVE:
                if (reDraw) {
                    value = WidgetUtils.getChartRangeValues(renderer.cinfo);
                    renderer.update(renderer.setRangeValues, [value], reDraw);
                }
                break;
            case WidgetChartConstants.RANGE_ONE_COLOR:
                renderer.update(renderer.setLevelMarkerColors, [value, 0], reDraw);
                break;
            case WidgetChartConstants.RANGE_TWO_COLOR:
                renderer.update(renderer.setLevelMarkerColors, [value, 1], reDraw);
                break;
            case WidgetChartConstants.RANGE_THREE_COLOR:
                renderer.update(renderer.setLevelMarkerColors, [value, 2], reDraw);
                break;
            case WidgetChartConstants.RANGE_FOUR_COLOR:
                renderer.update(renderer.setLevelMarkerColors, [value, 3], reDraw);
                break;
            case WidgetChartConstants.RANGE_FIVE_COLOR:
                renderer.update(renderer.setLevelMarkerColors, [value, 4], reDraw);
                break;
            case WidgetChartConstants.TARGET_MARKER_LABEL_ENABLE:
            	renderer.update(renderer.setShowTargetMarkerLabel, [value], reDraw);
            	break;
            case WidgetChartConstants.TARGET_MARKER_SIZE:
                renderer.update(renderer.setTargetMarkerSize, [value], reDraw);
                break;
            case WidgetChartConstants.TARGET_MARKER_COLOR:
                renderer.update(renderer.setTargetMarkerColor, [value], reDraw);
                break;
            case WidgetChartConstants.TARGET_MARKER_DASH_STYLE:
                renderer.update(renderer.setTargetMarkerStrokeStyle, [value], reDraw);
                break;
            case WidgetChartConstants.MIN_MAX_LABEL:
                renderer.update(renderer.setShowMinMaxLabels, [value], reDraw);
                break;
            case WidgetChartConstants.NEEDLE_FILL_COLOR:
                renderer.update(renderer.setNeedleFillColor, [value], reDraw);
                renderer.update(renderer.setNeedleStrokeColor, [WidgetUtils.getDarkerColor(value, -0.3)], reDraw);
                break;
            case WidgetChartConstants.NEEDLE_TYPE:
                renderer.update(renderer.setNeedleType, [value], reDraw);
                break;
            case WidgetChartConstants.NEEDLE_WIDTH:
                renderer.update(renderer.setNeedleWidth, [value], reDraw);
                break;
            default:
                // console.log(key + ": Unknown Handler");
                break;
        }
    };

    this.getChartRangeValues = function (cinfo) {
        var data = cinfo.DATA.DATA; 
        var map = cinfo.KEYVALMAP; 
        var maxRange = parseFloat((data[2]) ? data[2][data[2].length - 1] : 0);
        var rangesData = [];
        var keys = WidgetChartConstants.RANGES_KEYS;
        var units = map[WidgetChartConstants.RANGES_UNITS] || '0';
        var rangesDisabled = map[WidgetChartConstants.RANGES_ENABLE] === "false";
        if (rangesDisabled) {
        	rangesData.push(maxRange);
            return rangesData;
        }
        for (var i = 1; i <= keys.length; i++) {
            var key = WidgetChartConstants.RANGES_KEYS[i - 1];
            if (map[key] == null || map[key]== "") {
                continue;
            }
            var value = parseFloat(isNaN(map[key]) ? 0 : map[key]);
            rangesData[i-1] = (units === '0') ? value * maxRange / 100 : value;
        }
        rangesData.push(maxRange);
        return rangesData;
    };

    this.getContrastColor = function (color)
    {
        function luma(hex) {
            var rgb = hexToRGBArray(hex);
            return (0.2126 * rgb[0]) + (0.7152 * rgb[1]) + (0.0722 * rgb[2]);
        }
        function hexToRGBArray(hex)
        {
            hex = hex.replace('#', '');
            if (hex.length === 3) {
                hex = hex.charAt(0) + hex.charAt(0) + hex.charAt(1) + hex.charAt(1) + hex.charAt(2) + hex.charAt(2);
            } else if (hex.length !== 6) {
                return 255;
            }
            var rgb = [];
            for (var i = 0; i <= 2; i++) {
                rgb[i] = parseInt(hex.substr(i * 2, 2), 16);
            }
            return rgb;
        }
        if (color && !isNaN(color)) {
            color = '#' + WidgetUtils.adjustToHex(parseInt(color));
        }
        return (luma(color) >= 165) ? '#000000' : '#ffffff';//NO I18N
    };

    this.getDarkerColor = function (hex, lum) {
        // validate hex string
        if (!hex) {
            return;
        }
        hex = String(hex).replace(/[^0-9a-f]/gi, '');
        if (hex.length < 6) {
            hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
        }
        lum = lum || 0;
        // convert to decimal and change luminosity
        var rgb = "#", c, i;
        for (i = 0; i < 3; i++) {
            c = parseInt(hex.substr(i * 2, 2), 16);
            c = Math.round(Math.min(Math.max(0, c + (c * lum)), 255)).toString(16);
            rgb += ("00" + c).substr(c.length);
        }
        return rgb;
    };

    this.adjustToHex = function (COLOR) {//conversion int to Hex; $$$REMOVE REDUNDANT METHODS
        var colorVal = COLOR.toString(16);
        var len = colorVal.length;
        var adjust = "";
        for (var i = 0; i < 6 - len; i++)
        {
            adjust += "0";
        }
        return adjust + colorVal;
    };

    this.getChartBandWidth = function (cinfo) {
        return [this.getFromJSON(cinfo.KEYVALMAP, WidgetChartConstants.BULLET_CHART_BANDWIDTH_KEY),
            this.getFromJSON(cinfo.KEYVALMAP, WidgetChartConstants.BULLET_LEVEL_MARKER_BANDWIDTH_KEY)];
    };

    this.getFromJSON = function (object, key) {
        if (!object || !key || !object[key]) {
            return null;
        }
        return object[key];
    };

    this.initialize = function () {
        var layoutsParent = getEl('WidgetLayouts'); //No I18N
        ZDBUtil.setNamedChildEls(layoutsParent, this.layouts, 'layout');
    };

    this.fetchView = function (objId, viewCtx)
    {
        var view = viewCtx.view || WidgetUtils.validateView(objId, viewCtx);
        view.viewCtx = viewCtx;
        viewCtx.view = view;
        view.editMode = view.dashView.editMode;
        WidgetUtils.setVisibilityOfLayout(view, "hidden"); //NO I18N
        if (view.fetched && view.editMode && !view.zacf.isModified())
        {
            WidgetUtils.updateDashChild(view);
            return;
        }
        WidgetFetcher.sendWidgetRequest(2, view);
    };
    
    this.setVisibilityOfLayout = function(view, visibilityState)
    {
    	var dashChildDiv;
    	if (view && view.viewCtx && view.viewCtx.dashChildDiv)
    	{
    		dashChildDiv = view.viewCtx.dashChildDiv;
    		if (dashChildDiv && dashChildDiv.dlgBody && dashChildDiv.dlgBody.children && (dashChildDiv.dlgBody.children.length > 0))
        	{
        		dashChildDiv.dlgBody.children[0].style.visibility = visibilityState;
        	}
    	}
    }

    this.validateView = function (objId, viewCtx)
    {
        var view = ZDBClientUI.getViewObj(viewCtx.dashChildDiv, "view");//No I18N
        var dashView = viewCtx.dashView;
        if (view instanceof WidgetView)
        {
            view.editMode = dashView.editMode;
            return view;
        }
        return new WidgetView(objId, dashView.editMode, dashView);
    };

    this.getConfig = function (objId, view, dashUF)
    {
        var sendCompleteInfo = ((view.permissions.designModify == 1) && (ZDBUtil.isViewModified(view) || view.zacf.isModified()));
        if (sendCompleteInfo) {
            view.zacf.persist();
            view.setModified();
        }
        var xml = this.getZADataConfig(objId, view, dashUF, sendCompleteInfo);
        return xml;
    };

    this.getZADataConfig = function (objId, view, dashUF, sendCompleteInfo) {
        var xmlBuf = new ZDBStrBuf();
        var dispname = (objId > 0) ? objId : "Untitled" + objId;//No I18N
        xmlBuf.sTag("zadata").attr("csrftoken", (getCookie("CSRF_TOKEN"))).a(">\n");//NO I18N
        xmlBuf.sTag("dbobj")//NO I18N
                .attr("dispname", dispname).attr("desc", "").attr("type", "WIDGET")//NO I18N
                .attr("dbId", view.dbId).attr("objId", objId).attr("pvId", view.parentViewId)//NO I18N
                .a(">\n");//NO I18N
        WidgetUtils.getZAAnalysisViewConfig(xmlBuf, view, dashUF, sendCompleteInfo);
        if (view && view.folderId > 0)
        {
            xmlBuf.sTag("zfv").attr("fno", view.folderId).eTag(); //No I18N
        }
        xmlBuf.eNTag("dbobj").eNTag("zadata"); //No I18N
        return xmlBuf.toString();
    };

    this.getZAAnalysisViewConfig = function (xmlBuf, view, dashUF, sendCompleteInfo) {
        joinType = (view.joinType  == null)? 1 :view.joinType;
        xmlBuf.sTag("zaav").a(ZDBEscapeUtil.genXMLAttrs(WidgetConstants.ZAANALYSISVIEWDEFAULTS)).attr("jt", joinType).attr("cinfo", sendCompleteInfo).a(">\n");//NO I18N
        if (sendCompleteInfo)
        {
            ZDBAnalysisConfig.getViewColInstXML(view, xmlBuf, sendCompleteInfo);
            WidgetUtils.getWidgetConfig(view, xmlBuf);
        }
        if (dashUF)
        {
            xmlBuf.a(ZDBDashboardUF.getDashDynFltrConfig(view.viewCtx));
        }
        xmlBuf.eNTag("zaav");//No I18N
    };

    this.getWidgetConfig = function (view, xmlBuf) {
        var showValueAs = view.cinfo.SHOWVALUEAS;
        var settings = view.cinfo.SETTINGS;
        for (var showVal in showValueAs) {
            var showValObj = showValueAs[showVal];
            for (var key in showValObj) {
                var val = showValObj[key];
                if (val === '' || key === 'MODIFIED') {
                    continue;
                }
                xmlBuf.sTag("zawd").attr("pos", showVal).attr("key", key).attr("value", val).a(">\n"); //No I18N
                var styleObj = settings[showVal];
                if (styleObj && styleObj[key]) {
                    WidgetUtils.getSettingsConfig(xmlBuf, styleObj[key]);
                }
                if (key === 'OPERATION') {
                    xmlBuf.a(ZACF.getXMLFromInst(view, showVal));
                }
                xmlBuf.eNTag("zawd"); //No I18N
            }
        }
        xmlBuf.sTag("zawdl").attr("layout", view.layout).a(">\n").eNTag("zawdl"); //No I18N
        WidgetUtils.getKeyValMapConfig(xmlBuf, view.cinfo.KEYVALMAP);
    };

    this.getSettingsConfig = function (xmlBuf, styleObj) {
        xmlBuf.sTag('zawds') //No I18N
                .attr('ff', styleObj['FONT-FAMILY']) //No I18N
                .attr('fs', styleObj['FONT-SIZE'].replace('px', '')) //No I18N
                .attr('fw', styleObj['FONT-WEIGHT']) //No I18N
                .attr('ta', styleObj['TEXTALIGN']) //No I18N
                .attr('clr', CSSFactory.adjustToInt(styleObj.COLOR.replace('#', ''))) //No I18N
                .a(">\n"); //No I18N
        xmlBuf.eNTag('zawds'); //No I18N
    };

    this.getKeyValMapConfig = function (xmlBuf, keyValMap) {
        for (var key in keyValMap) {
            xmlBuf.sTag('zawdkv') //No I18N
                    .attr('key', key)//No I18N
                    .attr('value', keyValMap[key]) //No I18N
                    .a(">\n"); //No I18N
            xmlBuf.eNTag('zawdkv'); //No I18N
        }
    };

    //Insert Widget in Dashboard
    this.updateDashChild = function (view)
    {
        var dashChild = view.viewCtx.dashChildDiv;
        ZDBUtil.setNamedChildEls(dashChild, dashChild);
        WidgetUtils.setDashChildInfo(dashChild, view);
        WidgetUtils.updateDlgBody(view, dashChild);
        return dashChild;
    };

    //Reset dashChild Dimensions
    this.setDashChildInfo = function (dashChild, view)
    {
        var dashView = view.dashView;
        if(!dashView.dashLayout){
        	dashChild.dlgTitle.style.display = "none";//$#Dash#$ Comment for New Layout
            dashChild.dlgDesc.style.display = "none";//$#Dash#$ Comment for New Layout
        }
        dashChild.dlgBody.innerHTML = "";
        dashChild.dlgBody.style.textAlign = 'center';
        dashChild.dlgBody.setAttribute("id", "dash_" + view.objId);
        dashChild.det = null;
        dashChild.view = view;
        dashChild.innerColIdx = 1;
        dashChild.dlgBody.style.display = 'block';
        dashChild.style.cssFloat = 'left';//NO I18N
        var margin = (0.0066 * dashView.dashDiv.offsetWidth + parseInt(dashView.theme.get('child.margin.left') || 0)) + 'px';//NO I18N
        //$#Dash#$ var margin = (0.0066 * dashView.viewComp.offsetWidth + parseInt(dashView.theme.get('child.margin.left') || 0)) + 'px';//NO I18N
        DashUtils.setChildBottomPadding(dashChild, dashView, margin);
        WidgetUtils.setAttrToDiv(dashChild, ["eltype", "objid", "viewtype", "parentviewid"], ["VIEW", view.objId, view.type, view.parentViewId], "", true);//No I18N
    };

    //Append Widget in DashChild
    this.updateDlgBody = function (view, dashChild)
    {
        var childNode = WidgetUtils.getChildNode(view, dashChild.dlgBody);
        //$#Dash#$ dashChild.dlgBody.view = view;
        WidgetUtils.updateDashChildDim(dashChild, childNode);
    };

    //Get Widget Layout
    this.getChildNode = function (view, holder, isPreview)
    {
        var template = document.createElement('div');
        var dashBoardView;
        if (view && view.viewCtx) {
        	var dashBoardView = view.viewCtx.dashView;
        }
        template = WidgetUtils.layouts[view.layout].cloneNode(true);//NO I18N
        ZDBUtil.setNamedChildEls(template, template, 'chart');
        view.renderer = WidgetUtils.getRenderer(view.layout, view.cinfo);
        holder.appendChild(template);
        if (view.layout > 6) {
            var titleDiv = template.titleText || template.holder.titleText;
            var showValDet = view.cinfo.SHOWVALUEAS.LEGEND;
            var innerHTML = showValDet.LABEL;
            var data = view.cinfo.DATA.DATA;
            titleDiv.innerHTML = WidgetUtils.formatLabels(innerHTML, showValDet.OPERATION, data[1], data[0], view.cinfo.DATA.FORMATUTILS);
            var styleObj = view.cinfo.SETTINGS.LEGEND && view.cinfo.SETTINGS.LEGEND.LABEL ? view.cinfo.SETTINGS.LEGEND.LABEL : WidgetConstants.DEFAULTSTYLE.LABELLEGEND;
            WidgetUtils.applyStyle(titleDiv, styleObj);
            view.renderer.setLegendToolTipLabel(titleDiv.innerHTML);
        }
        if (!isPreview) {
            view.renderer.setDimensions(250);
            delete holder.chartSet;
            ZDBUtil.setNamedChildEls(holder, holder, 'chart');
            if(!view.viewCtx.dashChildDiv.isNewView){
            	view.renderer.resize(holder, ((dashBoardView && dashBoardView.dashLayout) ? true : false));
            }
        } else {
            view.renderer.setDimensions(250, 130);
            view.renderer.setAnimation(chartAnimation);
        }
        var isToAnimate = !isPreview
        if (isToAnimate) {
        	var dashChild = view.viewCtx.dashChildDiv;
            isToAnimate = !ZRAnimationUtil.notInVisibleArea(view);
        }
        if (dashBoardView && ZDBDashboardAPI.viewsInDashboard(dashBoardView).length > 30) {
        	isToAnimate = false;
        }
        view.renderer.draw(template, view, isToAnimate, isPreview);
        template.view = view;
        for (var key in view.cinfo.SHOWVALUEAS) {
            var showVal = view.cinfo.SHOWVALUEAS[key];
            showVal.MODIFIED = true;
        }
        return template;
    };

    //Fill Widget Layout with Data
    this.fillChildNode = function (template, view, isToAnimate, isPreview)
    {
        var data = view.cinfo.DATA.DATA;
        var formattedData = data.ZAF || {};
        var columns = view.cinfo.COLUMNS;
        var showValueAs = view.cinfo.SHOWVALUEAS;
        var settings = view.cinfo.SETTINGS;
        var nullGroupLabel = view.cinfo.KEYVALMAP['data.label.null'] || WidgetConstants.NULL_LABEL_TEXT;
        var nullDataLabel = view.cinfo.KEYVALMAP['data.value.null'] || WidgetConstants.NULL_DATA_TEXT;
        var dataLabel = data.GROUP.LABEL;
        var dataLabelFormat = '${' + columns.GROUP.COLUMN_ID + '}';
        var groupLabelFormat = (columns.YAXIS) ? '${' + columns.YAXIS.COLUMN_ID + '}' : '';
        var templateDet = WidgetUtils.getLayoutInfoAsJSON(template);
        var numberAnimationDuration = 0;
        for (var showValKey in showValueAs)
        {
            var showValDivs = templateDet[showValKey];
            var valueDiv = showValDivs.OPERATION;
            if (showValKey.indexOf('kpi') > -1)
            {
            	if (numberAnimationDuration > 0) {
            		numberAnimationDuration += 50;
            	}
                WidgetUtils.fillKPIDiv(valueDiv, view, numberAnimationDuration);
                //Temporary fix for KPI half clipped in preview when text overflow occurs
                if (isPreview && (view.layout > 3 && view.layout < 7)) {
                	var primaryValueSpanEl = valueDiv.parentNode.firstElementChild;
                	if (primaryValueSpanEl.offsetWidth < primaryValueSpanEl.scrollWidth) {
                		valueDiv.parentNode.style.cssFloat = "right"; //NO I18N
                	}
                }
                continue;
            }
            var showValDet = showValueAs[showValKey];
            var showValData = data[showValDet.OPERATION] || {};//NO I18N        
            var labelDiv = showValDivs.LABEL;
            var value = (((data.NODATA + '') !== 'true') && showValData.OPERATION !== null && showValData.OPERATION !== undefined && formattedData[showValKey] && formattedData[showValKey].OPERATION) ? formattedData[showValKey].OPERATION : showValData.OPERATION;
            var label = showValData.LABEL;
            var innerHTML = showValDet.LABEL;
            var settingsDet = settings[showValKey] || {};
            if (valueDiv)
            {
                var valSrc = showValDet.VALUE || value;
                valSrc = WidgetUtils.searchNreplace(valSrc, '${OPERATION}', WidgetConstants.OPRTOSHOWVALDISP[showValDet.OPERATION]);//NO I18N
                valSrc = WidgetUtils.getFormattedValue(groupLabelFormat, valSrc, label || nullGroupLabel, '');
                valSrc = WidgetUtils.getFormattedValue(dataLabelFormat, valSrc, dataLabel, value || nullDataLabel);
                valueDiv.style.display = '';
                valueDiv.innerHTML = (valSrc) ? ZDBEscapeUtil.replaceChar(valSrc, ZDBEscapeUtil.SPECIALCHARS_XML, ZDBEscapeUtil.REPL_SPECIALCHARS_XML) : '';
                var valueStyle = settingsDet.OPERATION;
                if (view.layout > 3 && showValDet.POSITIVE && data.KPI) {
                    var positionKey = data.KPI.OPERATION;
                    if (settingsDet[positionKey]) {
                    	if (valueStyle) {
                    		valueStyle = ZDBUtil.cloneObject(valueStyle);
                    		valueStyle["COLOR"] = settingsDet[positionKey]["COLOR"]; //NO I18N
                    	} else {
                    		valueStyle = settingsDet[positionKey];
                    	}
                    }
                }
                WidgetUtils.applyStyle(valueDiv, valueStyle);
                if (isToAnimate) {
                	if (!showValueAs['kpi']) {
                		var parentDiv = valueDiv.parentNode;
                        parentDiv.style.width = parentDiv.offsetWidth + "px";
                	}
                	view.dashView.animatingViewWaiting++;
                	//Passed Arguments in order (targetElement, duration, digitwiseDuration, delayForAnimation, number, callbackFunciton, ownerObj, args)
                	numberAnimationDuration = WidgetAnimation.animateNumber(showValDivs.OPERATION, 500, false, 120,
                			null, view.dashView.decreaseAnimatingViewCount, view.dashView, []);
                }
            }
            if (labelDiv)
            {
                innerHTML = WidgetUtils.searchNreplace(innerHTML, '${OPERATION}', WidgetConstants.OPRTOSHOWVALDISP[showValDet.OPERATION]);//NO I18N
                innerHTML = WidgetUtils.getFormattedValue(groupLabelFormat, innerHTML, label || nullGroupLabel, '');
                innerHTML = WidgetUtils.getFormattedValue(dataLabelFormat, innerHTML, dataLabel, value || nullDataLabel);
                labelDiv.style.display = '';
                labelDiv.innerHTML = (innerHTML) ? ZDBEscapeUtil.replaceChar(innerHTML, ZDBEscapeUtil.SPECIALCHARS_XML, ZDBEscapeUtil.REPL_SPECIALCHARS_XML) : '';
                WidgetUtils.applyStyle(labelDiv, settingsDet.LABEL);
            }
        }
        return template;
    };

    this.formatLabels = function (format, operation, groupVal, dimVal, formatUtils) {//USE REGEX
        format = format || '';
        format = WidgetUtils.searchNreplace(format, formatUtils.OPERATION, WidgetConstants.OPRTOSHOWVALDISP[operation]);//NO I18N
        format = WidgetUtils.getFormattedValue(formatUtils.YAXISFORMAT || '', format, dimVal || formatUtils['data.label.null'], '');
        format = WidgetUtils.getFormattedValue(formatUtils.GROUPFORMAT, format, formatUtils.GROUP, groupVal || formatUtils['data.value.null']);
        return ZDBEscapeUtil.replaceChar(format, ZDBEscapeUtil.SPECIALCHARS_XML, ZDBEscapeUtil.REPL_SPECIALCHARS_XML);
    };

    //Fill Widget KPI div
    this.fillKPIDiv = function (kpiDiv, view, animDuration)
    {
        if (view.cinfo.SHOWVALUEAS.kpi.SHOWKPI === 'false') {
            return;
        }
        var positionKey = (view.cinfo.DATA.DATA.KPI) ? view.cinfo.DATA.DATA.KPI.OPERATION : 'POSITIVE';//NO I18N
        var fontObj = (view.cinfo.SETTINGS.kpi && view.cinfo.SETTINGS.kpi[positionKey]) ? view.cinfo.SETTINGS.kpi[positionKey] : WidgetConstants.DEFAULTSTYLE[positionKey];
        var kpiIconIdx = parseInt(view.cinfo.SHOWVALUEAS.kpi[positionKey]) - 1;
        kpiDiv.style.display = '';
        kpiDiv.innerHTML = WidgetConstants.COMPARISONINDICATORS[kpiIconIdx];
        var className = WidgetUtils.JSONToCSS(fontObj);
        var classExtension = kpiDiv.getAttribute('class');
        kpiDiv.setAttribute('class', className + ' ' + classExtension);
        if (animDuration > 0) {
        	var direction;
        	var parentKPIDiv = kpiDiv.parentNode;
        	var height = parentKPIDiv.offsetHeight;
        	parentKPIDiv.style.width = parentKPIDiv.offsetWidth + "px";
        	parentKPIDiv.style.height = height + "px";
        	kpiDiv.style.opacity = 0;
        	if (kpiIconIdx > 3) {
        		direction = 'down'; //No I18N
        	} else {
        		direction = 'up'; //No I18N
        	}
        	view.dashView.animatingViewWaiting++;
        	setTimeout(function() {WidgetAnimation.animateKPI(kpiDiv, direction, 1200, height, view.dashView.decreaseAnimatingViewCount, view.dashView, [])}, animDuration);
        }
    };

    //Apply Widget Settings
    this.applyStyle = function (div, style) {
        var className = WidgetUtils.JSONToCSS(style);
        var classExtension = div.getAttribute('class');
        div.setAttribute('class', className + ' ' + classExtension);
        if (style && style["TEXTALIGN"]) {
        	WidgetUtils.updateTextAlignForParent(div, style["TEXTALIGN"]);
        }
    };
    
    //Update DashChild Height 
    this.updateDashChildDim = function (dashChild, childNode)
    {
        childNode.style.display = "inline-block";
        if (dashChild.isCopied) {
        	dashChild.view.dashView.decrementNewWidgetsWaitingForFirstResponse();
        	delete dashChild.isNewView; 
        }
        if (dashChild.isNewView) {
            var width = childNode.offsetWidth + 20;
            dashChild.dlgBody.style.width = width + 'px';//NO I18N
            dashChild.style.width = width + 22 + 'px';//NO I18N
            dashChild.style.height = "";
            var height = childNode.offsetHeight + 'px';//NO I18N
            childNode.style.height = height;
            dashChild.dlgBody.style.height = height;
            ZDBDashScroll.glowDashChild(dashChild);
            dashChild.view.dashView.decrementNewWidgetsWaitingForFirstResponse();
        }
        else{
            if(dashChild.view.layout < 7){
                dashChild.view.renderer.resize(dashChild.dlgBody);
            }
        }
        if(BROWSER.IS_MOBILE_PHONE)
        {
        	dashChild.style.height = "";
        	var height = childNode.offsetHeight + 'px';//NO I18N
            childNode.style.height = height;
            dashChild.dlgBody.style.height = height;
        }
        if (dashChild.modalDiv)
        {
            dashChild.modalDiv.style.height = dashChild.offsetHeight + 'px';//NO I18N;
            dashChild.modalDiv.style.width = width + 22 + 'px';//NO I18N;
        }
        if (dashChild.mobileModalDiv) {
            dashChild.mobileModalDiv.style.height = dashChild.offsetHeight + 'px';//NO I18N;
        }
        var dashView = dashChild.view.viewCtx.dashView;
        ZDBDStructure.updateStructInfo(dashView, null, null, false);

        delete dashChild.isNewView;
        dashChild.det = null;
        if(dashChild.isCopied){
        	WidgetUtils.addSeparator(dashChild);
        	dashChild.scrollIntoView(false);
        	delete dashChild.isCopied;
        }
    };

    //UTIL: search and Replace
    this.searchNreplace = function (srcStr, matchStr, replaceStr)
    {
        replaceStr = (replaceStr == null) ? "" : replaceStr;
        if (!srcStr || !matchStr)
        {
            return srcStr;
        }
        return srcStr.split(matchStr).join(replaceStr);
    };

    //Format Widget Labels
    this.getFormattedValue = function (format, innerHTML, label, value)
    {
        if (!format)
        {
            return innerHTML;
        }
        innerHTML = WidgetUtils.searchNreplace(innerHTML, format + '.LABEL', label);//NO I18N
        if (value)
        {
            innerHTML = WidgetUtils.searchNreplace(innerHTML, format + '.VALUE', value);//NO I18N
        }
        return innerHTML;
    };

    //Update Widget Ids after save
    this.updateWidgetIds = function (oldWidgetIdMap, newWidgetIdMap, dashid)
    {
        for (var id in oldWidgetIdMap)
        {
            var dashChild = oldWidgetIdMap[id];
            var view = dashChild.view;
            view.modified = 0;
            var objId = newWidgetIdMap[id];
            view.objId = objId;
            view.dashboardId = dashid;
            dashChild.setAttribute("objid", objId);
            var parComp = dashChild.ctx.parComp;
            parComp.view = view;
            parComp.setAttribute("id", "dash_" + objId);
        }

    };

    //returns Layout Node details as JSON
    this.getLayoutInfoAsJSON = function (layout)
    {
        var layoutInfoAsJSON = {};
        var els = layout.getElementsByTagName('span');
        for (var i = 0, j = els.length; i < j; i++)
        {
            var curEl = els[i];
            var posDet = curEl.getAttribute('position');
            if (!posDet)
            {
                curEl.parentNode.removeChild(curEl);
                j--;
                i--;
                continue;
            }
            var posKeyDet = curEl.getAttribute('positionKey');
            curEl.style.display = 'none';
            layoutInfoAsJSON[posDet] = layoutInfoAsJSON[posDet] || {};
            layoutInfoAsJSON[posDet][posKeyDet] = curEl;
        }
        return layoutInfoAsJSON;
    };

    //Add Margin between widgets
    this.addSeparator = function (dashChild)
    {
        var prevChild = dashChild.previousElementSibling;
        var nextChild = dashChild.nextElementSibling;
        var marginAdjust = 0.0066 * dashChild.view.dashView.dashDiv.offsetWidth + 'px';//NO I18N
        //$#Dash#$ var marginAdjust = 0.0066 * dashChild.view.dashView.viewComp.offsetWidth + 'px';//NO I18N
        if (nextChild && nextChild.offsetTop === dashChild.offsetTop)
        {
            dashChild.style.marginRight = marginAdjust;
            nextChild.det = null;
            nextChild.oldDet = null;
        }
        if (prevChild && prevChild.offsetTop === dashChild.offsetTop)
        {
            prevChild.style.marginRight = marginAdjust;
            prevChild.det = null;
            prevChild.oldDet = null;
        }
    };

    //Deep copy a object
    this.clone = function (object) {
        var jsonString = JSON.stringify(object);
        return JSON.parse(jsonString);
    };

    this.extend = function (target, other) {//used
        target = target || {};
        for (var prop in other) {
            if (typeof other[prop] === 'object') {
                target[prop] = WidgetUtils.extend(target[prop], other[prop]);
            } else {
                target[prop] = other[prop];
            }
        }
        return target;
    };

    //setAttribute Iterator
    this.setAttrToDiv = function (el, prop, values, exception)//used
    {
        var attrLen = prop.length;
        for (var i = 0; i < attrLen; i++)
        {
            var attr = prop[i];
            var val = values[i];
            el.setAttribute(attr, val);
        }
    };



//	CSSFactory and Settings Utilities

    this.JSONToCSS = function (json)
    {
        if (!json)
        {
            return '';
        }
        var color = CSSFactory.adjustToHex(json.COLOR);
        var cssName = '';
        json["FONT-STYLE"] = json["FONT-WEIGHT"];//NO I18N
        if (json["FONT-FAMILY"] != "Default") {//NO I18N
        	cssName += CSSFactory.validateCSSName('font-family', json["FONT-FAMILY"], json["FONT-FAMILY"].split(' ').join('')) + ' ';//NO I18N
        }
        cssName += CSSFactory.validateCSSName('font-size', json["FONT-SIZE"], 'S' + json["FONT-SIZE"]);//NO I18N
        cssName += ' ' + CSSFactory.validateCSSName('font-weight', json["FONT-WEIGHT"], 'W' + json["FONT-WEIGHT"]);//NO I18N
        cssName += ' ' + CSSFactory.validateCSSName('font-style', json["FONT-STYLE"], 'S' + json["FONT-STYLE"]);//NO I18N
        cssName += ' ' + CSSFactory.validateCSSName('color', color, 'C' + color.replace('#', ''));//NO I18N
        var lineHeight = json["LINE-HEIGHT"] || json["FONT-SIZE"];//NO I18N
        cssName += ' ' + CSSFactory.validateCSSName('line-height', lineHeight, 'LN' + lineHeight);//NO I18N
        return cssName;
    };
    
    this.updateTextAlignForParent = function (div, property)
    {
    	var className = CSSFactory.validateCSSName('text-align', property, 'T' + property); //NO I18N
        div = div.parentNode;
    	var classExtension = div.getAttribute('class');
        div.setAttribute('class', className + ' ' + classExtension);
    }

    this.copyValToDef = function (defFontObj, fontObj, updateDiv)
    {
        if (!fontObj)
        {
            fontObj = {};
            for (var prop in defFontObj)
            {
                fontObj[prop] = defFontObj[prop];
            }
        }
        var prop = updateDiv.getAttribute("propertyName");
        fontObj[prop] = updateDiv.value;
        return fontObj;
    };

};

WidgetUtils.initialize();
