/* $Id$ */
var ZComponents = (function($){
	return {
		isRTL : false,
		userAgent : navigator.userAgent.toLowerCase(),
		zIndex : 1000, // ZIndex can be used for components like dialog.
		$document : $(document),
		$window : $(window),
		sourceDiv : document.createElement('div'),
		sourceText : document.createTextNode(" "),
		hasHandlebars : window.Handlebars ? true : false,
		keys : {},
		keyCode : {
			BACKSPACE: 8,
			COMMA: 188,
			DELETE: 46,
			DOWN: 40,
			END: 35,
			ENTER: 13,
			ESCAPE: 27,
			HOME: 36,
			LEFT: 37,
			NUMPAD_ADD: 107,
			NUMPAD_DECIMAL: 110,
			NUMPAD_DIVIDE: 111,
			NUMPAD_ENTER: 108,
			NUMPAD_MULTIPLY: 106,
			NUMPAD_SUBTRACT: 109,
			PAGE_DOWN: 34,
			PAGE_UP: 33,
			PERIOD: 190,
			RIGHT: 39,
			SPACE: 32,
			TAB: 9,
			UP: 38,
			ALT: 18
		},
		initDeferredComponents : function(componentsMap, timeout, callback){
			var base = this;
			setTimeout(function(){
				$.each(componentsMap, function(ctype, componentGroup){
					$(componentGroup).each(function(i, component){
						base._initComponent($(component), ctype);		
					});
				});
				if(callback){
					callback(componentsMap);
				}
			},(timeout ? timeout : 10000)); // Default is 10 seconds
		},
		init : function(container,callback,attr){
			container = container ? container : $("body");
			var attrSelector = "["+ (attr ? attr : "data-ctype")+"]", // No I18N
				uiComponents = $(attrSelector, container); 
			if(container.is(attrSelector)){
				uiComponents = $(container).add(uiComponents);
			}
			var base = this;
			if(uiComponents.length){
				ZComponents.eachAsync(uiComponents, {
					delay: 20,
					loop: function(i, ele){
						base._initComponent(ele, undefined, attr);
					},
					end: function(){
						base._end(container, uiComponents, callback);
					}
				});
			}else{
				base._end(container, uiComponents, callback);
			}
		},
		eachAsync : function(array,opts){ // will be modified later
			var i = 0, 
			l = array.length, 
			loop = opts.loop || function(){};
		
			ZComponents.whileAsync(
				$.extend(opts, {
					test: function(){ return i < l; },
					loop: function()
					{ 
						var val = array[i];
						return loop.call(val, i++, val);
					}
				})
			);
		},
		whileAsync : function(opts){ // async utility from jquery
			var delay = Math.abs(opts.delay) || 10,
				bulk = isNaN(opts.bulk) ? 500 : Math.abs(opts.bulk),
				test = opts.test || function(){ return true; },
				loop = opts.loop || function(){},
				end  = opts.end  || function(){};
			
			(function(){
				var t = false, 
					begin = new Date();
				while( t = test() ){
					loop();
					if( bulk === 0 || (new Date() - begin) > bulk ){
						break;
					}
				}
				if( t ){
					setTimeout(arguments.callee, delay);
				}else{
					end();
				}
			})();
		},
		_initComponent : function(element,ctype,attr){
			attr = attr ? attr : "data-ctype"; // No I18N
			element = $(element);
			var mainParent = undefined;
			if(element.is(":hidden")){ // No I18N
				mainParent = element.parentsUntil(":visible").last().show(); // No I18N
			}
			ctype = !ctype ? element.attr(attr) : ctype;
			element[ctype]();
			if(mainParent){
				mainParent.hide();
			}
		},
		_end : function(mainDiv, uiComponents, callback){
			$(document).trigger("componentinitialized", mainDiv); // No I18N
			if(callback){
				callback(uiComponents);
			}
		},
		getMetadata : function(element){
			element = $(element);
			if(element && element.length){
				element = element[0]; // $.data requires DOM element to be passed as argument.
				var metadata = $.data(element,"metadata"); // No I18N
				if(metadata){
					return metadata;
				}else{
					var object = {};
					$.each(element.attributes,function(){ // using element.attributes because element.dataset is not supported in IE 9 and 10.
						var name = this.nodeName,
							value = this.nodeValue === "" ? "true" : this.nodeValue ; // No I18N
						if(name === "data"){ // No I18N
							// Retrieving options present in data attribute as a json.
							object = $.extend({},object,ZComponents._getObject(value.indexOf("{") < 0 ? "{" + value + "}" : value)); // No I18N
						}else{
							// data-* attributes and other attributes
							name = name.match(/^data-/) ? name.replace(/^data-/,'') : name; // No I18N
							name = name.replace(/-([a-z])/g, function(g){
								return g[1].toUpperCase(); // replacing the hyphenated names to camelCase
							});
							/* modified the optionName since providing data-inner-html attribute in html element will be converted to innerHtml only. */
							if(name === "innerHtml"){ // No I18N
								name = "innerHTML";  // No I18N
							}
							object[name] = (value === "true" || value === "false") ? (value === "true") : ((value.indexOf("{") !== -1) ? ZComponents._getObject(value.indexOf("{") < 0 ? "{" + value + "}" : value) : value); // No I18N
						}				
					});
					$.data(element,"metadata",object); // No I18N
					return object;
				}
			}
		},
		getTemplate: function(handleBarsFileName){
			var template;
			if(ZComponents.hasHandlebars){
				template = Handlebars.templates[handleBarsFileName];
			}
			return template;
		},
		_getObject : function(data) {
	        if(typeof data !== "string"){ // No I18N
	        	return data;
	        }
	        var modString = data.replace(/'/g,'\"');   // No I18N 
	        data = JSON.parse(modString);
	        return data;
	    },
		_trigger : function(element,options,type,event,data){
			var prop, orig,
				callback = options[ type ];

			data = data || {};
			event = $.Event( event );
			event.type = (options.widgetEventPrefix ? (options.widgetEventPrefix + type) : ""+type).toLowerCase(); // No I18N
			// the original event may come from any element
			// so we need to reset the target on the new event
			event.target = element;
			orig = event.originalEvent;
			if ( orig ) {
				for ( prop in orig ) {
					if ( !( prop in event ) ) {
						event[ prop ] = orig[ prop ];
					}
				}
			}
			element.trigger( event, data );
			//var returnValue = (callback !== undefined && $.isFunction(callback)) ? callback.apply(element, [event].concat(data)) : !(event.isDefaultPrevented());
			var value  = !($.isFunction(callback) &&
				callback.apply(element, [event].concat(data)) === false ||
				event.isDefaultPrevented() );
			return value;
		},
		_changeOptions : function(options,optionName,value){
			if(optionName !== undefined && value !== undefined){
				value = typeof value === "object" ? $.extend(true, {}, options[optionName], value) : value
				options[optionName] = value;
			}
			else if(typeof optionName === "object"){ // No I18N
				$.each(optionName,function(index,value){
					options[index] = value;
				});
			}
			else if(optionName !== undefined && value === undefined){
				return options[optionName];
			}
			else if(optionName === undefined && value === undefined){
				return options;
			}	
		},
		createComponent : function(details){
			if(!ZComponents.documentObject){ // since jquery's document ready & DOMContentLoaded event firing order is unpredictable.
				ZComponents.setProperties(); 
			}
			var parameters = details.parameters,
				returnValue = details.element;
			$.each(details.element,function(index,element){	// allowing multiple elements to perform same operation(initialization or method invokation) on component.
				var	element = $(element);
					data = element.data(details.componentName);
					if(!data){
						var metadata = ZComponents.getMetadata(element);
						if(metadata.override){ // handling the case where two or more components share the same set of attributes.
							var newArray = {};
							for(var obj in metadata){
								if(obj.indexOf(details.componentName) === 0){
									var result = obj.replace(details.componentName,""), // No I18N
										keyName = result[0].toLowerCase()+result.substring(1);
									newArray[keyName] = metadata[obj];
								}
							};
							metadata = newArray;
						}
						var options = $.extend(true,{},metadata.componentOptions,metadata,typeof parameters === "object" && parameters[0]); // No I18N
						if(details.baseClass){
							ZComponents._extend(details.component,details.baseClass);
						}
						options = $.extend(true, {}, (details.baseClass && details.baseClass.prototype.options) ? details.baseClass.prototype.options : {}, details.component.prototype.options, options);
						data = new details.component(element,options);
						element.data(details.componentName,data);
					}
					// Public Methods Invokation
					var array = $.map(parameters,function(value,index){
						return [value];
					});
					array = array.splice(1);
					if(parameters.length && typeof parameters[0] !== "object" && parameters[0] !== undefined && parameters[0].indexOf('_') === -1){
						try{
							if(data[parameters[0]] === undefined && parameters[0] === "widget"){ // No I18N
								return details.element; // returning the element instance for widget method call.
							}
							// handling option method separately because it may contain nested keys.
							returnValue = data[parameters[0]].apply(data,array);
						}catch(error){
							throw error; // method name given is not supported by this component
						}
					}
			});
			return returnValue;
		},
		option : function(instance,params){
			// calling "option" method with no arguments returns the entire options object.(Getter Method)
			// Example: element.zbutton("option")
			if(params.length === 0){
				return $.extend({},instance.options); // extending in order to avoid object overriding.
			}
			var optionName = params[0],
				value = params[1];
			// calling "option" method with object as argument sets one or more options.(Setter Method) 
			// Example: element.zbutton({"text":"Save","disabled":true});
			if(typeof optionName === "object"){ // No I18N
				$.each(optionName,function(i,k){
					ZComponents.option(instance,[i,k]); 
				});
			}else{ // optionName is of type string
				var options = {},currentOption,subkey,
					parts = optionName.split(".");
					key = parts.shift(); // remove the first part of the optionName as it is the actual key.
				if(parts.length){
					currentOption = options[key] = $.extend({}, instance.options[key]);
					for(var i = 0;i < parts.length - 1; i++ ) {
						currentOption[ parts[ i ] ] = currentOption[ parts[ i ] ] || {};
						currentOption = currentOption[ parts[ i ] ];
					}
					subkey = parts.pop();
					if(value === undefined){
						return currentOption[subkey];
					}
					currentOption[subkey] = value;
					return instance.option.apply(instance,[key,options[key]]);
				}else{
					// calling "option" method with only one string argument returns the value of the key.(Getter Method)
					// Example: element.zbutton("option","icon");
					if(value === undefined){
						return instance.options[key];
					}
					// calling "option" method with key and value sets the option with key to the value specified.(Setter Method)
					// This method inturn calls the "option" method of components to reflect the change immediately.
					// Example: element.zbutton("option","text","Save");
					return instance.option.apply(instance,params);
				}
			}
		},
		_extend : function(target,base){
			target.prototype = $.extend({},base.prototype,target.prototype);
			$.each(target.prototype,function(prop,value){
				if ($.type(value) === "function") { // No I18N
					target.prototype[ prop ] = (function(){
						var _super = function() {
							return base.prototype[ prop ].apply( this, arguments );
						};
						return function(){
							this._super = _super;
							return value.apply( this, arguments );
						};
					})();
				}
			});
		},
		Browser : {
			isIE :  function(){
				return (ZComponents.userAgent.indexOf("msie") !== -1 || ZComponents.userAgent.indexOf(".net") !== -1); // No I18N
			},
			isSafari : function(){
				return (ZComponents.userAgent.indexOf("safari") !== -1 && ZComponents.userAgent.indexOf("chrome") === -1); // No I18N
			},
			isFirefox: function(){
				return ZComponents.userAgent.indexOf('firefox') > -1 ? true : false;
			},
			isMac: function(){
				return navigator.platform.match(/(Mac|iPhone|iPod|iPad)/i) ? true : false;
			},
			isWindows: function(){
				return navigator.platform.indexOf("Win") !== -1; // No I18N
			}
		},
		AJAX : {
			transport: function(params, successCallback, failureCallback){
				var request = $.ajax({
					url: params.sourceURL,
					data: params.data,
					type: params.ajaxType,
					dataType: params.dataType
				})
				request.success(successCallback);
				request.fail(failureCallback);
				return request;
			}
		},
		// Component Users can set I18N for all Components at once or individual components separately.
		setI18NKeys : function(moduleName, values, replace){ 
			if(typeof moduleName === "object"){ // No I18N 
				// {"zdialog" : {"close":"Close the Dialog"},"ztooltip":{"close": "Close Text"} }
				$.each(moduleName,function(index,module){
					ZComponents.keys[index] = $.extend({},ZComponents.keys[index],module);
				});
			}else{
			// module name is a string. Format: "zdialog",{ "close" : "Close the Dialog"}, true or false
				if(replace){
					ZComponents.keys[moduleName] = values;
				}else{
					ZComponents.keys[moduleName] = $.extend({},ZComponents.keys[moduleName],values);
				}
			}
		},
		getI18NText : function(moduleName, key, defaultKeys, stringArray){
	        var ii8nText = ZComponents.keys[moduleName] && ZComponents.keys[moduleName][key];
	        if((!ii8nText || ii8nText === "[noi18n] "+ key) && defaultKeys){	// No I18N
	            ii8nText = defaultKeys[ key ];           
	        }
	        if(stringArray){
                for (var i = 0; i < stringArray.length; i++) {
                    ii8nText = ii8nText.replace('{'+ i +'}',stringArray[i]);	// No I18N
                }
            }
	        return ii8nText;
		},
		create : function(initObject){
			var componentName = initObject.ctype.replace(initObject.ctype.charAt(0),""); // No I18N
			methodName = "create"+componentName.charAt(0).toUpperCase()+componentName.substring(1); // No I18N
			return ZComponents[methodName](initObject);	
		},
		setProperties : function(){
			var doc = document.documentElement,
				body = document.body;
			ZComponents.documentObject = {
				height: Math.max(doc.clientHeight,doc.scrollHeight,doc.offsetHeight,body.offsetHeight,body.scrollHeight),
				width: Math.max(doc.clientWidth,doc.scrollHeight,doc.offsetHeight,body.offsetWidth,body.scrollWidth)
			};
			ZComponents.windowObject = {
				height: doc.clientHeight,
				width: doc.clientWidth
			};
			ZComponents.sourceDiv.appendChild(ZComponents.sourceText);
		},
		createIcon: function(element,icon){
			if(icon.indexOf("#") !== 0){ // No I18N
				element.addClass(" "+icon+""); // No I18N
			}else{
				var svgId = icon.split(" ")[0], // No I18N
					customClass = icon.slice(svgId.length).trim();
				element.append($("<svg class='"+customClass+"'><use xlink:href='"+svgId+"'></use></svg>"));
			}
			return element;
		},
		changeIcon: function(element,oldIcon,newIcon){
			if(oldIcon.indexOf("#") !== 0){ // oldIcon is of type class
				element.removeClass(oldIcon);
				element = ZComponents.createIcon(element,newIcon);
			}else{ 
				// old icon is of type svg icon.
				if(newIcon.indexOf("#") !== 0){ // new icon is of type class
					element.find("svg").remove();
					element.addClass(" "+newIcon+"");
				}else{ // new icon is of type svg
					var svgId = newIcon.split(" ")[0], // No I18N
						customClass = newIcon.slice(svgId.length).trim();
					element.find("svg").removeAttr("class").attr("class",customClass);  // No I18N
					element.find("use").attr("xlink:href",svgId);  // No I18N
				}
			}
		},
		encodeHTML : function(str){
			if(!ZComponents.sourceDiv.innerHTML){
				ZComponents.sourceDiv.appendChild(ZComponents.sourceText);
			}
			ZComponents.sourceText.nodeValue = str;
			str = ZComponents.sourceDiv.innerHTML;
			return str;
		}
	}

})(jQuery);
document.addEventListener("DOMContentLoaded", ZComponents.setProperties); // No I18N
window.addEventListener("resize", ZComponents.setProperties); // No I18N