/* $Id$ */
(function($){
	var popoverTemplate = ZComponents.getTemplate("popover.hbs");	// No I18N
	/** @components popover - Popover component is similar to tooltip that displays the content for the target. The difference is Popover can have more content and can be viewed through click, hover, focus etc.
		@eventPrefix zpopover
	*/
	var ZPopover = function(element, options){
		this._initialize(element, options);
		this._bindEvents();
		this._bindBodyEvents();
	}
	ZPopover.prototype = {
		options: {
			"widgetEventPrefix": "zpopover",	// No I18N
			/** @member {string} 
			    [popover.position = bottom] - Indicates the position where the popover need to be placed
			    @allowedValues {('default'|'bottom'|'top'|'left'|'right'|'top-left'|'top-right'|'bottom-left'|'bottom-right')} */	// No I18N	
			"position": "default",	// No I18N
			/** @member {(number|object)} 
		        [popover.offset] - If the value is specified the popover will be relocated from the calculated position. 
                @example <caption> Code examples </caption>  
                $(<activating-element>).zpopover({
					offset: {top: 2, left: 10},
					content: "Here's some amazing popover content."  
                })   */	
			/** @member {number}
					[popover.offset.top] - The displacement value from the calculated top value. For example, if the actual position of popover is calculated as {top: 34px; left: 200px} and if offset.top = 2, then the popover will be displayed at {top: 36px, left: 200px}  */			
			/** @member {number}
					[popover.offset.left] - The displacement value from the calculated left value. For example, if the actual position of popover is calculated as {top: 34px; left: 200px} and if offset.left = 10, then the popover will be displayed at {top: 36px, left: 210px}  */			
			"offset": undefined,	// No I18N
			/** @member {boolean}
			    	[popover.overlay = false] - Overlay for the popover 	*/
			"overlay": false,	// No I18N
			/** @member {(string|dom-selector)}
				    [popover.forElement] - The target for which the popover need to be shown rather than element which initiates the component */		
			"forElement": false,	// No I18N
			/** @member {string}
			        [popover.popOn = 'click'] - Indicates the way with which the popover will be triggered. If 'none' is provided, popover opening and closing should be handled by the user. Multiple triggers can be provided for a single element. Eg. "hover click"
					@allowedValues {('click'|'focus'|'hover'|'none')} 	*/	
			"popOn": "click",	// No I18N
			/** @member {boolean}
			        [popover.insertAfterForElement = false] - Indicates whether the popover need to be placed adjacent to the activating element. */
			"insertAfterForElement": false,	// No I18N
			/** @member {("string")}
			        [popover.appendTo = undefined] - Popover can be placed within this container. */
			"appendTo": undefined,	// No I18N
			/**	 @member {(string|dom-selector)} [popover.title] - Title for the popover */
			"title": undefined,	// No I18N
			/**  @member {(string|dom-selector|object)}
				[popover.content] - Content of the popover. You can pass object if you want to display AJAX content inside popover. */
			/**	@member {string} [popover.content.URL] - The URL from which the popover content need to be retrieved */
			/** @member {string}
				[popover.content.error] - Error Message that need to be displayed if content cannot be retrieved from the URL */			
			/**	@member {string} [popover.content.type] - AJAX request Type */			
 			/** @member {function}
 				[popover.content.success] - The response returned from the ajax request can be processed and passed back to the component.
				@example <caption> Code Examples </caption>
				$(<activating-element>).zpopover({
					content: {
						URL: "ajaxcontent.html",
						success: function(data){
							//data is the response returned from the server
							return data;
						}
					}
				})  */	
			"content": "Popover content appears here.",	// No I18N
			"htmlSafe": false,	// No I18N
			/** @member {boolean}
					[popover.pointer = true] - Indicates whether the arrow need to be shown 
			*/
			"pointer": true,	// No I18N
			/**  @member {boolean|Object} [popover.closeAction] - Contains the popover closing behaviour */
			/**  @member {boolean} [popover.closeAction.button] - Indicates whether close button need to be included in the popover container. */
			/**  @member {string} [popover.closeAction.on] - Indicates whether we need to close popover on scroll and document click  */
			/**  @member {string} [popover.closeAction.scrollContext] - Denotes the element for which the popover should close on scroll. If no value is given scroll event will be attached to document. */
			"closeAction": {	// No I18N
				"button": false,	// No I18N
				"on": "outside-click esc",	// No I18N
				"scrollContext": undefined	// No I18N
			},	
			/**	  @member {string} [popover.messages.close] - Title attribute of close button in Popover */
			"messages": {	// No I18N
				"close": "Close"	// No I18N
			},
			"removeOnClose": false,		// No I18N
			"closeOtherPopovers": false,	// No I18N
			/**	@member {boolean} [popover.detachable] - Specifies whether the popover can be detached from the target and moved across the document.*/
			"detachable": false,	// No I18N
			/**  @member {object} [popover.animation] - Specify the animation effect to use while opening and closing Popover. */
			/** @member {Object} [popover.animation.open] - The animation that will be used when a popover opens. */
			/** @member {(string|Object)} [popover.animation.open.effects] - Effect to be used when opening the popover. */	
			/** @member {Number} [popover.animation.open.duration] - Defines the open animation duration. */
			/** @member {Object} [popover.animation.close] - The animation that will be used whenever closing the popover. */
			/** @member {(String|Object)} [popover.animation.close.effects] - Effect to be used when closing the popover. */
			/** @member {Number} [popover.animation.close.duration] - Defines the close animation duration. */
			"animation": undefined,	// No I18N
			/**  @member {object} [popover.delay] - Used to specify the time after which the popover need to be opened and closed. */
			/** @member {object} [popover.delay.openAfter] - The time after which the popover should be opened. */
			/** @member {object} [popover.delay.closeAfter] - The time after which the popover should be closed. */
			"delay": undefined,	// No I18N
			/**	 @member {boolean}  [popover.autoFocus] - Indicates whether we need to set focus to the elements inside the popover. 	*/	  
			"autoFocus": false,	// No I18N
			/**	 @member {string} [popover.height] - Height of the popover */
			"height": undefined,	// No I18N
			/**	@member {string} [popover.width] - Width of the popover container  */
			"width": undefined,	// No I18N
			/**	  @member {object} [popover.cssClass] - Style for the popover can be specified here. */
			"cssClass": undefined,	// No I18N
			'EFFECTS'      : {   // No I18N
				'slide' : { // No I18N
					"open" : "slideDown", // No I18N
					"close": "slideUp" // No I18N
				},
				'fade' : { // No I18N
					"open" : "fadeIn", // No I18N
					"close" : "fadeOut" // No I18N
				}
			}
		},
		_EVENTS: {
			/** Triggered before the popover is opened.
				@event popover.beforeopen
				@param {object} event eventObject
				@param {object} customObject containing additional information about the popover.*/
			"BEFOREOPEN" : "beforeopen",	// No I18N
			/** Triggered after the popover is opened.
				@event popover.open
				@param {object} event eventObject
				@param {object} customObject containing additional information about the popover. */		
			"AFTEROPEN" : "open",	// No I18N
			/**	Triggered before the popover is closed.
				@event popover.beforeclose
				@param {object} event eventObject
				@param {object} customObject containing additional information about the popover. */		
			"BEFORECLOSE": "beforeclose",	// No I18N
			/**	Triggered after the popover is closed.
				@event popover.close
				@param {object} event eventObject
				@param {object} customObject containing additional information about the popover. */	
			"AFTERCLOSE": "close",	// No I18N
			"CLOSECLICK": "closeclick",	// No I18N
			"CONTENTLOAD": "contentload"	// No I18N
		},
		_targetWithPopover: [],
		_isBodyEventsBinded: false,
        _initialize: function(element, options){
        	var targets = $("[popover]");
        	this.options = options;
        	if(targets.length){
        		for(var i=0;i<targets.length;i++){
        			if($(targets[i]).attr("popover") === element.attr("id")){
        				this.element = $(targets[i]);
        				this._target = $(targets[i]);
        				this.options.content = element;
        				this._initUsingCtype = true;
        			}
        		}
        	}
        	if(!this._initUsingCtype){
        		this.element = element;
        		this._target = element;
				this._targetWithPopover.push(this.element);
        	}else{
				this._targetWithPopover.push(element);
        	}
        	this.options.position = this.options.position === "default" ? (this.options.pointer ? "bottom" : "bottom-right") : this.options.position;	// No I18N      	
        	if(this.options.overlay){
        		this._createOverlay();
        	}
        	if(this.options.content.URL){	
				this._isAjaxContent = true;
			}
        },
        _createOverlay: function(){
			this._overlay = $("<div class='zpopover--overley'></div>").appendTo("body").hide(); 
        },
        _bindEvents: function(){
        	var base = this;
			this._scrollContext = this.options.closeAction.scrollContext ? this.options.closeAction.scrollContext instanceof jQuery ? this.options.closeAction.scrollContext : ((typeof this.options.closeAction.scrollContext === "string" && (this.options.closeAction.scrollContext[0].indexOf("#") !== -1 || this.options.closeAction.scrollContext[0].indexOf(".") !== -1)) ? $(this.options.closeAction.scrollContext) : false) : false; //No I18N
    		if(this._scrollContext){
    			this._scrollContext.off("scroll."+this.options.widgetEventPrefix).on('scroll.'+this.options.widgetEventPrefix, function(orgEvent){	//No I18N
					if(base._generatedPopover && base._generatedPopover.is(":visible")){	//No I18N
						base._closePopover(base, orgEvent);
					}
				})
    		}else if(this.options.closeAction.on.split(' ').indexOf("scroll") > -1){	// No I18N
    			ZComponents.$document.off('scroll.'+this.options.widgetEventPrefix).on('scroll.'+this.options.widgetEventPrefix, function(orgEvent){	// No I18N
    				base._closePopover(base, orgEvent);
    			})
    		}
    		var triggers = this.options.popOn.split(' ');	//No I18N
    		for(var i=0; i<triggers.length;i++){
	    		if(triggers[i] === "hover"){	// No I18N
	        		this._target.on("mouseenter."+this.options.widgetEventPrefix, this._mouseEnterHandler.bind(this))	// No I18N
	        		.on("mouseleave."+this.options.widgetEventPrefix, this._mouseLeaveHandler.bind(this));	// No I18N
	        	}else if(triggers[i] === "click"){	//No I18N
	        		this._target.on("click."+this.options.widgetEventPrefix, this._clickHandler.bind(this));	// No I18N
	        	}else if(triggers[i] === "focus"){	//No I18N
	        		this._target.on("focus."+this.options.widgetEventPrefix, this._focusHandler.bind(this));	// No I18N
	        	}
    		}    	
        },
        _mouseEnterHandler: function(orgEvent){
			this._currentTarget = $(orgEvent.currentTarget);
			this._handleTargetEnter(orgEvent);
        },
        _mouseLeaveHandler: function(orgEvent){
        	var base = this;
			this._mouseLeaveTimeout = setTimeout(function(){
				if(base._generatedPopover){
					var currentPopoverId = base._generatedPopover.attr("id");	//No I18N
					if(!(($(orgEvent.relatedTarget).parents("#"+currentPopoverId).length) || ($(orgEvent.relatedTarget).attr('id') === "#"+currentPopoverId))){	//No I18N
					 	base._closePopover(base, orgEvent);
					}
				}
			},1500)
        },
        _clickHandler: function(orgEvent){
			this._currentTarget = $(orgEvent.currentTarget);
			this._handleTargetClick(orgEvent);
        },
        _focusHandler: function(orgEvent){
			this._clickedOutside = false;
			this._currentTarget = $(orgEvent.currentTarget);
			this._handleTargetEnter(orgEvent);
        },
        _bindBodyEvents: function(){
        	var base = this;
        	var closeOptions = this.options.closeAction.on.split(' ');	//No I18N
        	for(var i=0; i<closeOptions.length;i++){
        		if(closeOptions[i] === "outside-click"){	//No I18N
        			ZComponents.$document.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, this._documentClickHandler.bind(this)); 	//No I18N
        		}else{
        			ZComponents.$document.off("keydown."+this.options.widgetEventPrefix).on("keydown."+this.options.widgetEventPrefix, this._documentKeyDownHandler.bind(this));	//No I18N
        		}
        	}
        },
        _documentClickHandler: function(orgEvent){
      		this._handleBodyClick(orgEvent);
        },
        _documentKeyDownHandler: function(orgEvent){
			if(orgEvent.keyCode === ZComponents.keyCode.ESCAPE){
				this._clickedOutside = true;
				this._closeAllPopover(orgEvent);
			}
        },
        _handleTargetEnter: function(orgEvent){
        	this._closeClicked = false;
        	this._clickedOutside = false;
        	orgEvent === undefined && this.options.overlay && this._overlay.show();
        	if(!this._isAjaxContent){
        		var popover = this._getPopover(orgEvent);
        		popover && this._showPopover(orgEvent);
        	}else{
        		if(!this._popoverOpen && !this._content){
					this._makeAjaxRequest(orgEvent);
				}
				else{
					this._getPopover(orgEvent);
					this.options.htmlSafe ? this._popoverContent.html(this._content) : this._popoverContent.text(this._content);					
					this._showPopover();
				}
        	}
        	!this.options.closeOtherPopovers && this._closeAllPopover(orgEvent);
        	this._autoClosePopover(orgEvent);
        },
        _handleTargetClick: function(orgEvent){
        	this._closeClicked = false;
			this._clickedOutside = false;
			this.options.overlay && this._overlay.show();
			if(!this._isAjaxContent){
				var popover = this._getPopover(orgEvent);
				if(popover !== undefined){ 	//Displays the popover if already created 
					popover.is(":visible") ? this._closePopover(this, orgEvent) : this._showPopover(); 	//No I18N
					//Provides toggle effect
				}
			}else{
				if(!this._popoverOpen && !this._content){
					this._makeAjaxRequest(orgEvent);
				}
				else{
					var popover = this._getPopover(orgEvent);
					if(popover !== undefined){ 	//Displays the popover if already created 
						popover.is(":visible") ? this._closePopover(this, orgEvent) : this._showPopover(); 	//No I18N
						//Provides toggle effect
					}
					this.options.htmlSafe ? this._popoverContent.html(this._content) : this._popoverContent.text(this._content);					
					this._fixPopoverHeight();
				}
			}
			!this.options.closeOtherPopovers && this._closeAllPopover(orgEvent);
			this._autoClosePopover(orgEvent);
        },
        _getPopover: function(orgEvent){
        	var base = this;
        	if(!this._generatedPopover){
        		if(!this.options.closeOtherPopovers){ //Removes the opened popovers if the option is set false
					this._closeAllPopover(orgEvent);
				}
        		if(!ZComponents.Browser.isIE() && ZComponents.hasHandlebars){
        			var popoverData = {}, title, content;
        			popoverData.hasCloseButton = this.options.closeAction.button;
        			popoverData.hasTitle = this.options.title ? true : false;
        			popoverData.htmlSafe = this.options.htmlSafe;
        			if(this.options.title){
        				title = this.options.title;
						title = title instanceof jQuery ? "" : (typeof title === "string" && (title[0].indexOf("#") !== -1 || title[0].indexOf(".") !== -1)) ? "" : title;	// No I18N        			
        				popoverData.title = title;
        			}
        			popoverData.hasArrow = this.options.pointer;
        			if(this._isAjaxContent){
        				popoverData.isAjax = true;
        				popoverData.content = "";	// No I18N  
        				popoverData.ajaxContent = true;
        			}else{
        				content = this.options.content;
						content = content instanceof jQuery ? "" : (typeof content === "string" && (content[0].indexOf("#") !== -1 || content[0].indexOf(".") !== -1)) ? "" :  content;	// No I18N        			
	        			popoverData.content = content;
        			}
        			this._generatedPopover = $(popoverTemplate(popoverData));
					this._arrowPointer = this._generatedPopover.find(".zpopover__pointer"); // No I18N
					this._titleContainer = this._generatedPopover.find(".zpopover__title"); // No I18N
					this._popoverTitle = this._generatedPopover.find(".zpopover__title").find(".zpopover__text"); // No I18N
					this._contentContainer = this._generatedPopover.find(".zpopover__content"); 	// No I18N
					this._popoverContent = this._generatedPopover.find(".zpopover__content").find(".zpopover__text"); // No I18N
					if(title === ""){	// No I18N
						$(this.options.title).appendTo(this._popoverTitle).show();
					}
					if(content === ""){	// No I18N
						$(this.options.content).appendTo(this._popoverContent).show();
					} 
        		}else{
        			this._generatedPopover = $("<div>").addClass("zpopover"); // No I18N
					if(this.options.closeAction.button){
						this._closeButton = $("<div class='zpopover__action'>");	
						var closeContainer = $("<button>").attr("title", this.options.messages.close).addClass("zpopover__close").appendTo(this._closeButton); 	
						var closeIcon = $("<i>").addClass("zpopover__icon").append("<svg><use xlink:href='#close-icon'></use></svg>").appendTo(closeContainer);
						this._closeButton.appendTo(this._generatedPopover); 
					}
					if(this.options.title){
						this._titleContainer = $("<div>").addClass("zpopover__title").appendTo(this._generatedPopover); // No I18N
						this._popoverTitle = $("<div>").addClass("zpopover__text").appendTo(this._titleContainer); // No I18N  
						if(this.options.htmlSafe){
							var title = this.options.title;
							title instanceof jQuery ? title.appendTo(this._popoverTitle).show() : ((typeof title === "string" && (title[0].indexOf("#") !== -1 || title[0].indexOf(".") !== -1)) ? $(title).appendTo(this._popoverTitle).show() :  this._popoverTitle.html(title));	// No I18N
						}else{
							this._popoverTitle.text(this.options.title);
						}	
					}
					this._contentContainer = $("<div>").addClass("zpopover__content").appendTo(this._generatedPopover); // No I18N
					if(this._isAjaxContent){
						this._loaderContainer = $("<div>").addClass("zpopover__loading").appendTo(this._contentContainer); // No I18N
						this._loaderIcon = $("<div>").addClass("zpopover__icon zicon--loader").appendTo(this._loaderContainer);	// No I18N
					}else{
						this._popoverContent = $("<div>").addClass("zpopover__text").appendTo(this._contentContainer); // No I18N  
						if(this.options.htmlSafe){
							var content = this.options.content;
							content instanceof jQuery ? content.appendTo(this._popoverContent).show() : ((typeof content === "string" && (content[0].indexOf("#") !== -1 || content[0].indexOf(".") !== -1)) ? $(content).appendTo(this._popoverContent).show() :  this._popoverContent.html(content));	// No I18N
						}else{
							this._popoverContent.text(this.options.content);
						}
					}
					if(this.options.pointer){
						this._arrowPointer = $("<div>").addClass("zpopover__pointer").appendTo(this._generatedPopover); // No I18N
					}
        		}
        		var appendedTo = this.options.appendTo;
				appendedTo ? appendedTo instanceof jQuery ? this._generatedPopover.appendTo(appendedTo) : ((typeof appendedTo === "string" && (appendedTo[0].indexOf("#") !== -1 || appendedTo[0].indexOf(".") !== -1)) ? this._generatedPopover.appendTo($(appendedTo)) : this._generatedPopover.appendTo($("body"))) : this._generatedPopover.appendTo($("body"));	// No I18N
        		if(this.options.cssClass){
        			this._generatedPopover.addClass(this.options.cssClass);
        		}
        		this._generatedPopover.attr("id",this._target.attr("id")+"_"+this.options.widgetEventPrefix);	// No I18N
        		if(this.options.insertAfterForElement){
					var forElement = this.options.forElement ? this.options.forElement instanceof jQuery ? this.options.forElement : ((typeof this.options.forElement === "string" && (this.options.forElement[0].indexOf("#") !== -1 || this.options.forElement[0].indexOf(".") !== -1)) ? $(this.options.forElement) : false) : this._currentTarget;	// No I18N
        			this._generatedPopover.insertAfter(forElement);
        		}
        		var base = this;
        		this.options.detachable && this._generatedPopover.zdraggable({
        			start: function(orgEvent, ui){
        				base._arrowPointer && base._arrowPointer.hide();	
        			},
        			end: function(orgEvent, ui){
        				if(!base.options.closeAction.button){
        					base._closeButton = $("<div class='zpopover__action'>");		
							var closeContainer = $("<button>").attr("title", base.options.messages.close).addClass("zpopover__close").appendTo(base._closeButton); 		
							var closeIcon = $("<i>").addClass("zpopover__icon").append("<svg><use xlink:href='#close-icon'></use></svg>").appendTo(closeContainer);	
							base._closeButton.prependTo(base._generatedPopover); 
        				}				
						ZComponents.$document.off("click."+base.options.widgetEventPrefix);	// No I18N
						base._initPopoverEvents();
        			}
        		});	
        		this._triggerEvent(this._EVENTS.BEFOREOPEN, this._generatedPopover, orgEvent, this);
        		this._initPopoverEvents();
        		this._showPopover(orgEvent);
        	}else{
        		if(this.options.detachable){
        			if(this.options.pointer){
						this._arrowPointer.show();
						if(this.options.closeAction.on.split(' ').indexOf("outside-click") > -1){	// No I18N
        					!this.options.overlay && ZComponents.$document.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, this._documentClickHandler.bind(this)); 	//No I18N
						}
					}
					!this.options.closeAction.button && this._generatedPopover.find(".zpopover__action").remove();
				}
        		this._triggerEvent(this._EVENTS.BEFOREOPEN, this._generatedPopover, orgEvent, this);
        		return this._generatedPopover;
        	}
        },
        _showPopover: function(orgEvent){
        	var base = this;
        	if(this.options.animation){
        		var animation = this.options.animation,
        			effect = animation.effect ? animation.effect : (animation.open && animation.open.effect),
        			duration = animation.duration ? animation.duration : ( animation.open && animation.open.duration ? animation.open.duration : 400);
				onComplete = function(){
					base._placePopover(orgEvent);
				}
				if(typeof effect === "object"){	// No I18N
					this._generatedPopover.show();
					//this._placePopover(orgEvent);
					this._generatedPopover.animate(effect, duration, onComplete);
				}else{
					this._generatedPopover[this.options.EFFECTS[effect].open](duration, onComplete);
					this._placePopover(orgEvent);	
				}
        	}else if(this.options.delay && this.options.delay.openAfter){
        		if(this._showAfterTimer){
					clearTimeout(this._showAfterTimer); 
				}
				this._showAfterTimer = setTimeout(function(){
					base._generatedPopover.show();
					base._placePopover(orgEvent);
   					//base._generatedPopover.removeClass("fadeOutDown").addClass("fadeInUp");  	// No I18N   		 		
				}, this.options.delay.openAfter);			
        	}else{
   				this._generatedPopover.show(); 
   				this._placePopover(orgEvent); 
   				//this._generatedPopover.removeClass("fadeOutDown").addClass("fadeInUp");  	// No I18N   		 		
        	}
        	this.options.autoFocus && ZFocusHandler.init(this._generatedPopover);  	
        },
        _placePopover: function(orgEvent){
        	this._fixPopoverHeight();
        	if(this._target.css("position") === "fixed"){	// No I18N
        		this._generatedPopover.css({"position": "fixed"});	// No I18N
        	}
        	if(this.options.position === "page-center"){	// No I18N
        		var position = {
					'top': ZComponents.$window.outerHeight() / 2 - this._generatedPopover.outerHeight() / 2 + "px",	// No I18N
					'left': ((ZComponents.$window.width() - this._generatedPopover.outerWidth()) / 2) + ZComponents.$window.scrollLeft() + "px"	// No I18N
				}
				this._generatedPopover.css(position); 
        	}else{
        		this._positionPopover();
        	}
        	this._popoverOpen = true;
        	this._triggerEvent(this._EVENTS.AFTEROPEN, this._generatedPopover, orgEvent, this);      	 
        },
        _positionPopover: function(){
        	if(this.options.position !== "page-center"){	// No I18N
	        	var position = this._getPosition();
	    		if(this.options.offset){
	    			var top, left, arrowleft;
	    			if(typeof this.options.offset === "number"){	// No I18N
	    				top = this.options.offset + position.elementPosition.top;
	    				left = this.options.offset + position.elementPosition.left;
	    			}else{
	    				left = this.options.offset.left ? this.options.offset.left + position.elementPosition.left : position.elementPosition.left;
	    				top = this.options.offset.top ? this.options.offset.top + position.elementPosition.top : position.elementPosition.top;
	    			}
	    			top = top < 0 ? position.elementPosition.top : (top >= ZComponents.documentObject.height ?  ZComponents.documentObject.height - this._generatedPopover.outerHeight() : top);
	    			left = left < 0 ? position.elementPosition.left : (left > ZComponents.documentObject.width ? ZComponents.documentObject.width - this._generatedPopover.outerWidth() : left);
	    			if(this._arrowPointer){
	    				var value = left - position.elementPosition.left;
	    				arrowleft = value < position.arrowPosition.left ? position.arrowPosition.left - value : value - position.arrowPosition.left;
	    				position.arrowPosition.left = arrowleft;
	    			}
	    			position.elementPosition.top = top;
	    			position.elementPosition.left = left;

	    		}
				this._generatedPopover.css(position.elementPosition);
				if(this._arrowPointer){
					this._arrowPointer.css(position.arrowPosition);
				}
			}
        },
        _initPopoverEvents: function(){
        	var base = this;
        	if(this.options.popOn === "hover"){	// No I18N
				this._generatedPopover.off("mouseenter."+this.options.widgetEventPrefix).on("mouseenter."+this.options.widgetEventPrefix, function(orgEvent){ // No I18N
					clearTimeout(base._mouseLeaveTimeout);
					base._handleTargetEnter(orgEvent);
				})
				.off("mouseleave."+this.options.widgetEventPrefix).on("mouseleave."+this.options.widgetEventPrefix, function(orgEvent){ // No I18N
					base._closePopover(base, orgEvent);
				})
			}
        	this._closeButton && this._closeButton.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, function(orgEvent){ // No I18N
				base._triggerEvent(base._EVENTS.CLOSECLICK, undefined, orgEvent, base);
				base._closeClicked = true; //To indicate close button is clicked
				if(base._popoverOpen){
					base._closePopover(base, orgEvent);
				}
			})
        },
        _fixPopoverHeight: function(){
        	if(this.options.width){
        		this._generatedPopover.css({"width": this.options.width+"px"});	// No I18N
        	}
			var children = this._popoverContent && this._popoverContent.children(), value;
			if(this.options.height && this._popoverContent){ //Computes height for content based on title and content children 
				value = this.options.height - this._titleContainer.height() - (children.outerHeight(true) - children.height());
				this._popoverContent.height(value);
			}	
			return value;	
		},
        _makeAjaxRequest: function(orgEvent){
        	var base = this;
			base._getPopover(orgEvent);	
			$.ajax({
				type: this.options.content.type,	
				url: this.options.content.URL,
				success: function(data){
					base._loaderContainer = undefined;
					base._contentContainer.html(""); // No I18N  
					base._popoverContent = $("<div>").addClass("zpopover__text").appendTo(base._contentContainer); // No I18N  
					base._content = $.isFunction(base.options.content.success) ? base.options.content.success(data) : data;
					base.options.htmlSafe ? base._popoverContent.html(base._content) : base._popoverContent.text(base._content);							
					base._fixPopoverHeight();	
					base._positionPopover();
					base._triggerEvent(base._EVENTS.CONTENTLOAD, base._generatedPopover, orgEvent, base);		
				},
				error: function(){ //To display error message if the content cannot be loaded
					base._contentContainer.html(""); // No I18N  
					base._popoverContent = $("<div>").addClass("zpopover__text").appendTo(base._contentContainer); // No I18N   
					base._content = base.options.content.error;	
					base.options.htmlSafe ? base._popoverContent.html(base._content) : base._popoverContent.text(base._content);					
					base._getPopover(orgEvent);
					base._positionPopover();	
				}
			});
        },
        _handleBodyClick: function(orgEvent){
        	var clickedOutside = true;
			this._clickedOutside = false;
			var instance;
			for(var i=0; i<this._targetWithPopover.length;i++){
				instance = this._targetWithPopover[i].data(this.options.widgetEventPrefix);
				if(instance){
					var popoverEle = instance._generatedPopover, target = instance._target;
					if(popoverEle !== undefined){
						var popx1 = popoverEle.offset().left, popy1 = popoverEle.offset().top, popx2 = popoverEle.offset().left + popoverEle.outerWidth(), popy2 = popoverEle.offset().top + popoverEle.outerHeight();
						var popx1a = target.offset().left, popy1a = target.offset().top, popx2a = target.offset().left + target.outerWidth(), popy2a = target.offset().top + target.outerHeight();					
						var clickedInsidePop = orgEvent.pageX >= popx1 && orgEvent.pageX <= popx2 && orgEvent.pageY >= popy1 && orgEvent.pageY <= popy2; 
						var clickedInsideTarget = orgEvent.pageX >= popx1a && orgEvent.pageX <= popx2a && orgEvent.pageY >= popy1a && orgEvent.pageY <= popy2a; 		
						//Prevents closing of popover if clicked inside the popover container or activator
					}
					if(clickedInsidePop || instance._closeClicked || clickedInsideTarget || instance.options.overlay){ //Prevents closing of popover when close button is clicked
						if(popoverEle && popoverEle.is(":visible")){ // No I18N  
							clickedOutside = false;
							instance._closeClicked =  false;
							instance._clickedOutside = false;
							break;
						}					
					}
					instance._closeClicked =  false;
				}
			}
			if(clickedOutside && !instance.options.overlay){
				instance._clickedOutside = true;
				instance._closeAllPopover(orgEvent);
				instance._closeClicked = false;
			}
        },
        _arrowCallback: function(element, direction){
        	this._generatedPopover.removeClass("zpopover--top zpopover--bottom zpopover--left zpopover--right"); // No I18N  
			this._generatedPopover.addClass("zpopover--"+direction); // No I18N  
			this._calloutDirection = direction;
        },
       _getPosition: function(){
        	if(!/^(top|bottom|right|left|top-left|top-right|bottom-left|bottom-right)$/.test(this.options.position)){
				this.options.position = 'bottom';	 //No I18N 
				//Default position
			}
			var base = this;
			var arrowElement = this._generatedPopover.find(".zpopover__pointer");	// No I18N  
			var arrowPointer = arrowElement.length ? arrowElement : false;
			this._forElement = this.options.forElement ? this.options.forElement instanceof jQuery ? this.options.forElement : ((typeof this.options.forElement === "string" && (this.options.forElement[0].indexOf("#") !== -1 || this.options.forElement[0].indexOf(".") !== -1)) ? $(this.options.forElement) : false) : this._currentTarget;	// No I18N
			var popoverPosition = ZPosition.get(this._forElement, this._generatedPopover, { 
					direction: this.options.position,
					bufferSpace : 7, // The space between the element and the popover container 
					positionAlterable: true, 
					arrow: arrowPointer,  
					arrowCallback: this._arrowCallback.bind(base) //Classes for arrow added using the callback
				});
			return popoverPosition;
        },
        _closePopover: function(instance, orgEvent){
        	var toBeClosed;
        	if(!instance._destroyCalled){
        		toBeClosed = instance._triggerEvent(instance._EVENTS.BEFORECLOSE, instance._generatedPopover, orgEvent, instance);
        	}else{
        		toBeClosed = true;
        	}
        	if(toBeClosed){
        		if(instance.options.animation){
        			var base = instance,
					animation = instance.options.animation,
					effect = animation.effect ? animation.effect : (animation.close && animation.close.effect),
					duration = animation.duration ? animation.duration : ( animation.close && animation.close.duration ? animation.close.duration : 400),
					onComplete =  function(){
						base._handlePopoverClose(base, orgEvent);
					};
					if(typeof effect === "object"){  // No I18N
					// custom animation properties
						instance._generatedPopover.animate(effect,duration,onComplete);
					}else{ // predefined effects like slide and fade. 
						instance._generatedPopover[instance.options.EFFECTS[effect].close](duration,onComplete);
					}
        		}else{
        			instance._handlePopoverClose(instance, orgEvent);
        		}
        	}
        	else{
				if(!instance._generatedPopover.is(":visible")){	//No I18N
					instance._popoverOpen = false;
				}
			}   	
        },
        _handlePopoverClose: function(instance, orgEvent){
        	var popoverEle = instance._generatedPopover;
        	instance._autoCloseTimer && clearTimeout(instance._autoCloseTimer);
        	if(instance.options.removeOnClose){
        		var removedPopover = instance._generatedPopover;
    			if(instance._generatedPopover !== undefined){
    				var content = this.options.content;
					if(content instanceof jQuery || content[0].indexOf("#") !== -1 || content[0].indexOf(".") !== -1){
						$(content).appendTo("body").hide();
					}
    				instance._generatedPopover.remove();
    				instance._overlay && instance._overlay.remove();
    				instance._overlay = undefined;
    				instance._generatedPopover = undefined,	instance._arrowPointer = undefined, instance._popoverContent = undefined, instance._contentContainer = undefined,
					instance._popoverTitle = undefined, instance._titleContainer = undefined, instance._closeButton = undefined, instance._loaderContainer = undefined;
    			}
        	}else{
        		if(instance._generatedPopover){
        			//instance._generatedPopover.removeClass("fadeInUp").addClass("fadeOutDown");	//No I18N
	        		//setTimeout(function(){
	        			instance._generatedPopover.hide();
	        		//}, 600);
        		}   		
        	}		
    		instance._popoverOpen = false;		
    		instance._overlay && instance._overlay.hide();
    		if(!instance._destroyCalled){
    			instance._triggerEvent(instance._EVENTS.AFTERCLOSE, popoverEle, orgEvent, instance);
       		}
        },
        _closeAllPopover: function(orgEvent){
        	var clickedOutside = true;
			for(var i=0; i<this._targetWithPopover.length;i++){
				var instance =  this._targetWithPopover[i].data(this.options.widgetEventPrefix);
				//Remove the popover if it is open
				if(instance && instance._popoverOpen){
					//Check whether it is eligible for the remove
					if(instance._clickedOutside){
						instance._closePopover(instance, orgEvent);
					}else{
						if((instance !== this && !instance.options.overlay) || (instance !== this && !instance.options.closeOtherPopovers) || (instance.options.overlay && (orgEvent && orgEvent.keyCode ===  ZComponents.keyCode.ESCAPE))){
							instance._closePopover(instance, orgEvent)
						}
					}
				}
			}
        },
        _autoClosePopover: function(orgEvent){
        	if(this.options.delay && this.options.delay.closeAfter){
        		this.options.delay.closeAfter = this.options.delay.openAfter ? this.options.delay.closeAfter + this.options.delay.openAfter : this.options.delay.closeAfter;
        		this._autoCloseTimer && clearTimeout(this._autoCloseTimer); 
				var base = this;
				this._autoCloseTimer = setTimeout(function(){
					base._closePopover(base, orgEvent);
				}, this.options.delay.closeAfter)
        	}
        },
        //To trigger custom events
		_triggerEvent: function(customEvent, ui, orgEvent, instance){
			var content = {
				'target': instance.element,	// No I18N
				'popover': ui 	// No I18N
			}
			return ZComponents._trigger(instance.element, instance.options, customEvent, orgEvent, content);
		},
		/** Opens the popover for the specified target
			This method does not accept any arguments.
			@function popover.open
			@param {string|object} offset - Optional offset can be provided here.
			@example <caption> Code examples </caption>
			$(".selector").zpopover("open"); */	
		open: function(offset){
			if(offset !== undefined){
				this.options.offset = offset;
			}
			this._currentTarget = this.element;
			if(this.options.popOn === "click"){	// No I18N
				this._handleTargetClick(); //Call this method to provide toggling effect
			}
			//Trigger is manual when the user handles events for opening/closing of popover
			else if(this.options.popOn === "hover" || this.options.popOn === "none"){	// No I18N
				this._handleTargetEnter();
			}
		},
		/** Hides the popover from the screen
			This method does not accept any arguments.
			@function popover.close
			@example <caption> Code examples </caption>
			$(".selector").zpopover("close"); */
		close: function(){
			this._closePopover(this);
		},
		 
		/**	Gets the value currently associated with the specified optionName.
			This signature doesn't accept any arguments.
			@function popover.option
			@param {string} optionName - The name of the option to get.
			@return {string} value.
			@example <caption> Code examples </caption>
			var position = $(".selector").zpopover("option","position"); */		
		/** Sets the value of the option associated with the specified optionName.
			@function popover.option
			@param {string} optionName - The name of the option to get.
			@param {string} value - A value to set for the option.
			@example <caption> Code examples </caption>
			$(".selector").zpopover("option","position","top-left"); */		 
		/**	Gets an object containing key/value pairs representing the current popover options.
			This signature doesn't accept any arguments.
			@function popover.option
			@return {object} options object.
			@example <caption> Code examples </caption>
			var options = $(".selector").zpopover("option"); */	
		
		/** Sets one or more options for the popover.
			@function popover.option
			@param {Object} options - A map of option-value pairs to set.
			@example <caption> Code examples </caption>
			$(".selector").zpopover("option",{"position":"top-right","popOn":"hover"}); */	
		option: function(optionName, value){
			if(typeof optionName === "object"){ // No I18N
				var base = this;
				$.each(optionName,function(i,k){
					base.option(i,k);
				});
			}else if(optionName !== undefined && value !== undefined && value !== this.options[optionName]){
				if(optionName === "popOn"){	//No I18N
					var triggers = this.options.popOn.split(" ");	//No I18N
					for(var i=0; i<triggers.length;i++){
						if(triggers[i] === "hover"){	//No I18N
							this._target.off("mouseenter."+this.options.widgetEventPrefix).off("mouseleave."+this.options.widgetEventPrefix);	//No I18N
						}else{
							this._target.off(triggers[i]+"."+this.options.widgetEventPrefix);	//No I18N
						}
					}
					this.options.popOn = value;
					this._bindEvents();	
				}else if(optionName === "content"){	//No I18N
					this._content && (this._content = false);	
					if(typeof value === "object"){	//No I18N
						if(typeof this.options.content === "object"){	//No I18N
							for(var key in this.options.content ){
								if(value[key] !== undefined){
									this.options.content[key] = value[key];
								}
							}
						}else{
							this.options.content = value;
						}					
						this._isAjaxContent = true;
					}else{
						this.options.content = value;
						this._isAjaxContent = false;
					}
					if(this._generatedPopover && typeof value !== "object"){	//No I18N
						this._popoverContent.empty();
						if(this.options.htmlSafe){
							value instanceof jQuery ? value.appendTo(this._popoverContent).show() : ((typeof value === "string" && (value[0].indexOf("#") !== -1 || value[0].indexOf(".") !== -1)) ? $(value).appendTo(this._popoverContent).show() :  this._popoverContent.html(value));	// No I18N
						}else{
							this._popoverContent.text(value);
						}
					}				
				}
				else if(optionName === "height"){	//No I18N
					this.options.height = value;
					if(this._generatedPopover){
						this._generatedPopover.height(this.options.height);
						this._fixPopoverHeight();	
					}
				}else if(optionName === "width"){	//No I18N
					this.options.width = value;
					this._generatedPopover && this._generatedPopover.width(this.options.width);
				}else if(optionName === "position"){	//No I18N
					this.options.position = (this.options.pointer === false ? (value.indexOf("-") > -1 ? value : ((value === "top" || value === "bottom") ? value+"-right" : value)) : value);
				}
				else if(optionName === "title"){	//No I18N
					this.options.title = value;
					if(this._generatedPopover){
						var title = this._generatedPopover.find(".zpopover__title");	//No I18N
						if(value === false){ //If value is false title should be removed
							title.length && title.remove();
						}else{
							if(!title.length){
								this._titleContainer = $("<div>").addClass("zpopover__title").prependTo(this._generatedPopover); 
								this._popoverTitle = $("<div>").addClass("zpopover__text").appendTo(this._titleContainer);	// No I18N
								if(this.options.htmlSafe){
									value instanceof jQuery ? value.appendTo(this._popoverTitle) : ((typeof value === "string" && (value[0].indexOf("#") !== -1 || value[0].indexOf(".") !== -1)) ? $(value).appendTo(this._popoverTitle) :  this._popoverTitle.html(value));	//No I18N
								}else{
									this._popoverTitle.text(value);
								}
							}else{
								title.find(".zpopover__text").text(value);	//No I18N
							}
						}
					}
				}else if(optionName === "pointer"){		//No I18N
					this.options.pointer = value;
					this.options.position = (value === false ? (this.options.position.indexOf("-") > -1 ? this.options.position : ((this.options.position === "top" || this.options.position === "bottom") ? this.options.position+"-right" : this.options.position)) : this.options.position);
					if(this._generatedPopover){
						var arrow = this._generatedPopover.find(".zpopover__pointer");	//No I18N
						if(value === false){
							arrow.length && arrow.remove();
							this._arrowPointer = undefined;
						}else{
							if(!arrow.length){
								this._arrowPointer = $("<div>").addClass("zpopover__pointer").insertBefore(this._contentContainer);	//No I18N
							}
						}
						this._positionPopover();
					}
				 }else if(optionName === "detachable"){		//No I18N
				 	var base = this;
				 	this.options.detachable = value;
				 	if(value){
				 		this._generatedPopover && this._generatedPopover.zdraggable({
		        			start: function(orgEvent, ui){
		        				base._arrowPointer && base._arrowPointer.hide();
		        			},
		        			end: function(orgEvent, ui){
		        				if(!base.options.closeAction.button){
			        				base._closeButton = $("<div class='zpopover__action'>");	
									var closeContainer = $("<button>").attr("title", base.options.messages.close).addClass("zpopover__close").appendTo(base._closeButton); 	
									var closeIcon = $("<i>").addClass("zpopover__icon").append("<svg><use xlink:href='#close-icon'></use></svg>").appendTo(closeContainer);
									base._closeButton.prependTo(base._generatedPopover); 
								}
								ZComponents.$document.off("click."+base.options.widgetEventPrefix);	//No I18N
								base._initPopoverEvents();
		        			}
		        		});	
				 	}else{
				 		this._generatedPopover && this._generatedPopover.zdraggable("destroy");		//No I18N
				 		this._bindBodyEvents();
				 	}
				 }else if(optionName === "closeAction" && typeof value === "object"){	//No I18N
				 	if(value.button !== undefined){
				 		this.options.closeAction.button = value.button;
				 		if(this._generatedPopover){
				 			var closeButton = this._generatedPopover.find(".zpopover__action");	//No I18N
							if(value.button === false){
								closeButton.length && closeButton.remove();
							}else{
								if(!closeButton.length){
									this._closeButton = $("<div class='zpopover__action'>");
									var closeContainer = $("<button>").addClass("zpopover__close").appendTo(this._closeButton);
									var closeIcon = $("<i>").addClass("zpopover__icon").append("<svg><use xlink:href='#close-icon'></use></svg>").appendTo(closeContainer);
									this._closeButton.prependTo(this._generatedPopover);
									this._initPopoverEvents();
								}
							}
				 		}
				 	}
				 	if(value.scrollContext !== undefined){
				 		this.options.closeAction.scrollContext = value.scrollContext;
				 		if(value.scrollContext){
							var context = value ? value instanceof jQuery ? value : ((typeof value === "string" && (value[0].indexOf("#") !== -1 || value[0].indexOf(".") !== -1)) ? $(value) : false) : false; //No I18N
				    		if(context){
				    			var base = this
				    			context.on('scroll.'+this.options.widgetEventPrefix, function(orgEvent){	//No I18N
									if(base._generatedPopover && base._generatedPopover.is(":visible")){	//No I18N
										base._closePopover(base, orgEvent);
									}
								})
				    		}
						}else if(value.scrollContext === false){
							this._scrollContext.off("scroll."+this.options.widgetEventPrefix);	//No I18N
						}
				 	}
				 	if(value.on !== undefined){
				 		this.options.closeAction.on = value.on;
					    value ? this._bindBodyEvents() : ZComponents.$document.off("."+this.options.widgetEventPrefix); //No I18N
				 	}
				}else if(optionName === "overlay"){	//No I18N
					this.options.overlay = value;
					if(value){
						this._createOverlay();
					}else{
						this._overlay.remove();
						this._overlay = undefined;	
					}
				}else if(optionName === "cssClass"){	//No I18N
					this.options.cssClass && this._generatedPopover.removeClass(this.options.cssClass).addClass(value);				
					this.options.cssClass = value;
				}else{
					return ZComponents._changeOptions(this.options,optionName,value);
				}
			}else{
				return ZComponents._changeOptions(this.options,optionName,value);
			}			
		},
		/** Removes the popover functionality for the element completely. This will return the element back to its pre-init state.
			This method does not accept any arguments.
			@function poover.destroy
			@example <caption> Code examples </caption>
			$(".selector").zpopover("destroy"); */
		destroy: function(){
			this._clickedOutside = this.options.removeOnClose = this._destroyCalled = true; //Indicates the popopver need to be removed when the instance is encountered 
			this._closePopover(this);
			var index = this._targetWithPopover.indexOf(this.element);
			index > -1 ? this._targetWithPopover.splice(index, 1) : false;
			this.element.off("."+this.options.widgetEventPrefix);	// No I18N
			this._target = undefined;
			this.element.removeData(this.options.widgetEventPrefix)
			ZComponents.$document.off("."+this.options.widgetEventPrefix);	// No I18N
			this._destroyCalled = false;
		}
	}
	$.fn.zpopover = function(){
		var data = ZComponents.createComponent({
			'element' : $(this), // No I18N
			'component' : ZPopover, // No I18N
			'componentName' : 'zpopover', // No I18N
			'parameters' : arguments	// No I18N
		});	
		return  data; // data may contain element if component is instantiated or return value from a method
	};
	$.fn.zpopover.Constructor = ZPopover;
})(jQuery);