/* $Id$ */
var ZPosition = (function($, $window, $document){
	// 'use strict';	//No I18N
	var target, element, targetHeight, targetWidth, targetOffset, options, within, elemWidth,
	elemHeight, bufferSpace, availableHeight, availableWidth, win , element, callout, recalculated;
	var defaultOptions = {
		bufferSpace : 0,
		direction: 'top-left', //No I18N
		showWithinTarget: false,
		within: $window,
		arrow:{
			minSpace:0
		},
		customDimensions: false,
		positionAlterable: 'flipallfit' //'fitflip'|'fit'|false //No I18N
	};
	var calloutPositions = {
		'top-left' : 'bottom-right',	//No I18N
		'top-right' : 'bottom-left',	//No I18N
		'bottom-left' : 'top-right',	//No I18N
		'bottom-right' :'top-left',	//No I18N
		'top' :'bottom',	//No I18N
		'bottom' :'top',	//No I18N
		'left' :'right',	//No I18N
		'right' : 'left',    //No I18N
		'left-top' : 'right-bottom',	//No I18N
		'left-bottom' : 'right-top',	//No I18N
		'right-top' : 'left-bottom',	//No I18N
		'right-bottom' : 'left-top'	//No I18N
	};
	var regex = {
		relDirections: /^(top|top-right|top-left|bottom|bottom-right|bottom-left|left|left-top|left-bottom|right|right-top|right-bottom|top-left-corner|top-right-corner|bottom-left-corner|bottom-right-corner)$/,
		withinDirections: /^(top|bottom|left|right|top-left|top-right|bottom-left|bottom-right|center)$/,
		fit: /^(fit|flipfit|flipallfit|true)$/,
		flip: /^(flip|flipfit|flipall|flipallfit|true)$/,
		flipall: /^(flipall|flipallfit|true)$/
	};
	function getDimensions(element){
		if( options.customDimensions ){
			return element;
		}
		if( element.preventDefault ){
			return{
				width:0, height:0, offset: {left: element.pageX, top: element.pageY}
			}
		}
		var element = $(element);
		if( element.nodeType === 9 ){
			return{
				width: element.width(), height: element.height(), offset: {left:0, top:0}
			}
		}
		if( $.isWindow(element) ){
			return{
				width: element.width(), height: element.height(), offset: {left: element.scrollLeft(), top: element.scrollTop()}
			}
		}
		return{
			width: element.outerWidth(), height: element.outerHeight(), offset: element.offset()
		}
	}
	/* Direction if alterable is fixed based on within element*/
	function fixDirection(direction){
		var collisionWidth = elemWidth + bufferSpace ,
			collisionHeight = elemHeight + bufferSpace ,
			dir = direction.split('-'), //No I18N
			changedir = false;
		/* To filp in case of insufficient space
			top-left =>  bottom-left
			bottom-left => top-left
			left-top => right-top
			right-top => left-top
		 */
		if(/^(top|bottom)$/.test(dir[0]) || /^corner$/.test(dir[2])){
			var dirInfo = flip.top(direction, collisionHeight);
			direction = dirInfo.direction;
			if(dirInfo.overTop > 0 && /^(flipall|flipallfit|true)$/.test(options.positionAlterable)){
				var	newDirInfo = flip.left("left", collisionWidth);	//No I18N
				/* check - if space in top/bottom is inefficient left/right space is checked, if available left/right position is taken, else top/bottom position is fixed */
				if(newDirInfo.overLeft <= 0){
					direction = newDirInfo.direction;
				}
			}
		}
		dir = direction.split('-'); //No I18N
		if(/^(left|right)$/.test(dir[0]) || /^corner$/.test(dir[2])){
			var dirInfo = flip.left(direction, collisionWidth);
			direction = dirInfo.direction;
			if(dirInfo.overLeft > 0 && /^(flipall|flipallfit|true)$/.test(options.positionAlterable)){
				var newDirInfo = flip.top("top", collisionHeight);	//No I18N
				/* check - if space in left or right is inefficient, element is displayed in top/bottom */
				direction = newDirInfo.direction;
			}
		}
		return direction;
	}
	/* To filp in case of insufficient space
		top-left =>  top-right
		bottom-left => bottom-right
		left-top => left-bottom
		right-top => right-bottom
	 */
	function fixAlternateDirection(direction){
		var dir = direction.split('-'), //No I18N
		calloutWidthLeft = 0, calloutHeightLeft = 0;
		if(target.preventDefault && callout){
			/* If target is an event, callout must point the mouse pointer, so collisionWidth is reduced*/
			calloutWidthLeft = 2*callout.width + callout.width/2;
			calloutHeightLeft = 2*callout.height + callout.height/2;	
		}
		if(/^(left|right)$/.test(dir[1])){
			// var dirInfo = flip.left(dir[1], elemWidth - targetWidth - calloutHeightLeft);
			var dirInfo = flip.left(dir[1], elemWidth - (dir[0]==='corner'? 0: targetWidth) - calloutHeightLeft);
			direction = direction.replace(dir[1],dirInfo.direction);
		}else if(/^(top|bottom)$/.test(dir[1])){
			var dirInfo = flip.top(dir[1], elemHeight - targetHeight - calloutWidthLeft);
			direction = direction.replace(dir[1], dirInfo.direction);
		}
		return direction;
	}
	/* To initiate callout element*/
	function initCallout(elementDir){
		if(options.arrow){
			var	callout = {};
			var element = options.arrow.element || options.arrow;
			if(typeof element === 'string' || element.jquery || element.nodeType){	//No I18N
				element = $(element);
				if(element.length){
					callout.element = element;
				}
			}
			if(callout.element && callout.element.css('position') !== 'static'){	//No I18N
				callout.centeredToTarget = (/^(center|undefined)$/.test(options.arrow.direction)) && !target.preventDefault ? true : false;
				callout.callback =  $.isFunction(options.arrow.callback)? options.arrow.callback : ($.isFunction(options.arrowCallback)? options.arrowCallback: $.noop );
				callout.direction = calloutPositions[elementDir];
				callout.callback.call(this, callout.element, callout.direction.split('-')[0]);	//No I18N
				callout.height = typeof options.arrow.height === 'number' ? options.arrow.height : (callout.element ?  callout.element.outerHeight() : 0);	//No I18N
				callout.width = typeof options.arrow.height === 'number' ? options.arrow.width : (callout.element ?  callout.element.outerWidth() : 0); //No I18N
				callout.minSpace = options.arrow.minSpace;
				return callout;	
			}else{
				options.arrow = undefined;
			}
		}	
	}
	/* Function to get the callout position for a given direction*/
	function getCalloutPosition(callout){
		var pos = {}, 
			elemWidth1 =  element.innerWidth(),
		    elemHeight1 =  element.innerHeight(),
		    elemOffset = element.offset();
		if( callout.centeredToTarget ){
			callout.direction = callout.direction.split('-')[0]; //No I18N
			if(/^(top|bottom)$/.test(callout.direction)){
				if(targetOffset.left < win.scrollLeft){
					elemWidth1 = targetOffset.left + targetWidth - win.scrollLeft;
					elemOffset.left = win.scrollLeft;
				}else{
					elemWidth1 = targetWidth;
					elemOffset.left = targetOffset.left;
				}
				// elemWidth1 = targetWidth; /*only visible width portion of the target*/
			}else if(/^(left|right)$/.test(callout.direction)){
				if(targetOffset.top < win.scrollTop){
					elemHeight1 = targetOffset.top + targetHeight - win.scrollTop;
					elemOffset.left = win.scrollTop;
				}else{
					elemHeight1 = targetHeight;
					elemOffset.top = targetOffset.top;
				}
				// elemHeight1 = targetHeight; /*only visible height portion of the target*/
			}
		}
		switch(callout.direction){
			case 'top':	//No I18N
				pos = {left:((elemWidth1/2)-(callout.width/2)), top:-(callout.height)};	
				break;
			case 'left':	//No I18N
				pos = {left:-(callout.width), top:((elemHeight1/2)-(callout.height/2))};	
				break;
			case 'right':	//No I18N
				pos = {left:(elemWidth1), top:((elemHeight1/2)-(callout.height/2))};	
				break;
			case 'bottom':	//No I18N
				pos = {left:((elemWidth1/2)-(callout.width/2)), top:(elemHeight1)};	
				break;
 			case 'top-right': 	//No I18N
				pos = {left:(elemWidth1-(callout.width*2)),top: -(callout.height)};	
				break;
			case 'top-left':	//No I18N
				pos = {left:callout.width, top: -(callout.height)};	
				break;
			case 'bottom-left':	//No I18N
				pos = {left:callout.width, top:(elemHeight1)};	
				break;
			case 'bottom-right':	//No I18N
				pos = {left:(elemWidth1-(callout.width*2)), top:(elemHeight1)};	
				break;
			case 'left-top': 	//No I18N
				pos = {left: -(callout.width), top: callout.height}
				break;
			case 'left-bottom': 	//No I18N
				pos = {left: -(callout.width), top: (elemHeight1 - (callout.height * 2))};
				break;
			case 'right-top': 	//No I18N
				pos = {left:(elemWidth1), top: callout.height}
				break;
			case 'right-bottom': //No I18N
				pos = {left:(elemWidth1), top: (elemHeight1 - (callout.height * 2))}
				break;
		}
		pos.left = pos.left + elemOffset.left;
		pos.top = pos.top + elemOffset.top;	
		pos = getCorrectedPosition(pos, callout.element);
		return pos;
	}
	var flip = {
		/* Filp north south position incase of insufficient height in the given direction */
		top: function(direction, collisionHeight){
			var dir = direction.split('-')[0], 
				response = {direction: direction},
				availHeight = getAvailableHeight(direction);
			response.overTop = collisionHeight -  availHeight;
			if(response.overTop > 0){
				var newdir = dir === 'top' ? 'bottom' : 'top'; //No I18N
				var newAvailableHeight = getAvailableHeight(direction.replace(dir, newdir));
				if(newAvailableHeight > availHeight){
					response.direction = direction.replace(dir, newdir);
					response.overTop = collisionHeight - newAvailableHeight;
				}
			}
			return response;
		},
		/* Flip the left right position incase of insufficient width in the given direction*/
		left: function(direction, collisionWidth){
			var dir = direction.split('-'), response = {direction: direction}, finalDir, newdir;
			var availWidth = getAvailableWidth(direction);
			response.overLeft = collisionWidth -  availWidth;
			if(response.overLeft > 0){
				finalDir = dir[0];
				if(dir[2] === 'corner'){
					finalDir = dir[1];
				}
				newdir = finalDir === 'left' ? 'right' : 'left';	//No I18N
				var newAvailableWidth = getAvailableWidth( direction.replace(finalDir, newdir) );
				if(newAvailableWidth > availWidth){
					response.direction = direction.replace(finalDir, newdir);
					response.overLeft = collisionWidth - newAvailableWidth;
				}
			}
			return response;	
		}
	}
	/* Fit the element position based on the viewport*/
	function fitPosition(position){
		var newPosition = $.extend({}, position),
		direction = options.direction,
		collisionWidth = elemWidth + bufferSpace,
		collisionHeight = elemHeight + bufferSpace,
		dir = direction.split('-');	//No I18N
		dir[1] = dir[1] === undefined? (/^(top|bottom)$/.test(dir[0])? 'left' : 'top') : dir[1];	//No I18N
		var offset = within.isWindow? {left: win.scrollLeft, top: win.scrollTop} : within.offset;
		if(/^(top|bottom)$/.test(dir[1]) || target.preventDefault){
			var overTop = within.scrollTop - newPosition.top;
			if( overTop>0 ){
				newPosition.top += overTop;
			}
			var overBottom = ( newPosition.top + elemHeight) - (within.scrollTop + getAvailableHeight().complete);
			if(overBottom > 0){
				newPosition.top -= overBottom;
			}
		}
		if(/^(left|right)$/.test(dir[1]) || target.preventDefault){
			var overLeft = within.scrollLeft - newPosition.left;
			if( overLeft>0 ){
				newPosition.left += overLeft;
			}
			var overRight = ( newPosition.left + elemWidth) - (within.scrollLeft + getAvailableWidth().complete)+1;
			if(overRight > 0){
				newPosition.left -= overRight;
			}
		}
		return newPosition;
	}
	/* Fit the callout position when the elementposition is altered*/
	function fitCalloutPosition(calloutPosition, oldPosition, newPosition){
		var newCalloutPosition = $.extend({}, calloutPosition);
		var diffLeft = newPosition.left - oldPosition.left;
		var diffTop = newPosition.top - oldPosition.top;
		var dir = /*calloutPositions[options.direction].split('-')*/ callout.direction.split('-');	//No I18N
		var newdir;
		if(diffLeft !== 0){
			if(diffLeft < 0 && dir[1] === 'left'){	//No I18N
				newCalloutPosition.left -= diffLeft;
			}else if((diffLeft > 0 && dir[1] === 'right') || (dir[1] === undefined && /^(top|bottom)$/.test(dir[0]))){	//No I18N
				newCalloutPosition.left -= diffLeft;
			}
		}else if(diffTop !== 0){
			if((diffTop > 0 && dir[1] === 'bottom') || (dir[1] === undefined && /^(left|right)$/.test(dir[0]))){	//No I18N
				newCalloutPosition.top -= diffTop;
			}else if(diffTop < 0 && dir[1] === 'top'){	//No I18N
				newCalloutPosition.top += diffTop;
			}
			
			if(newdir){
				calculatePosition(newdir);
			}	
		}
		return newCalloutPosition;
	}
	function calculatePosition(direction){
		var newPosition, position;
		options.direction = direction;
		if( regex.flip.test( options.positionAlterable )){
			options.direction = fixDirection(options.direction);
		}
		if( !/corner/.test( options.direction )){
			callout = initCallout( options.direction );
		}
		if( regex.flipall.test(options.positionAlterable) ){
			options.direction = fixAlternateDirection(options.direction);
		}
		position = fixPosition(options.direction);
		newPosition = position;
		if(regex.fit.test(options.positionAlterable)){
			newPosition = fitPosition( position );
		}
		/* Correcting the border positions */
		/*if( newPosition.top < 0){
			newPosition.top = 0;
		}
		if( newPosition.left < 0){
			newPosition.left = 0;
		}*/
		if( newPosition.top < within.scrollTop ){
			newPosition.top = within.scrollTop;
		}
		if( newPosition.left < within.scrollLeft ){
			newPosition.left = within.scrollLeft;
		}
		var correctedPosition = getCorrectedPosition(newPosition, element);
		if( callout ){
			callout.direction = calloutPositions[options.direction];
			element.css(correctedPosition);
			callout.position = getCalloutPosition(callout);
			var dir = options.direction.split('-'); //No I18N
			var newdir, calloutdir = callout.direction.split('-');	//No I18N
			if(!callout.centeredToTarget){
				callout.position = fitCalloutPosition( callout.position, position, newPosition);
			}
			if(target.preventDefault){
				/*In case of target being an event, tha callout position must be in sync with the pageX and pageY position*/
				if(dir[1] === 'right'){ //No I18N
					correctedPosition.left = correctedPosition.left - (callout.position.left + callout.width/2); 
				}else if(dir[1] === 'left'){	//No I18N
					correctedPosition.left = correctedPosition.left + (elemWidth - callout.position.left - callout.width/2); 
				}else if(dir[1] === 'bottom'){	//No I18N
					correctedPosition.top = correctedPosition.top - (callout.position.top + callout.height/2); 
				}else if(dir[1] === 'top'){	//No I18N
					correctedPosition.top = correctedPosition.top + (elemHeight - callout.position.top - callout.height/2); 
				}	
			}
			var CALLOUTMINSPACE = callout.minSpace;
			if(CALLOUTMINSPACE){
				if(/^(top|bottom)$/.test(calloutdir[0])){
					if(( elemWidth - callout.position.left) <= (CALLOUTMINSPACE + callout.width)){
						callout.position.left = (elemWidth - callout.width - CALLOUTMINSPACE);
					}else if(callout.position.left <= CALLOUTMINSPACE){
						callout.position.left = CALLOUTMINSPACE;
					}	
					if((newPosition.left + callout.position.left) < (targetOffset.left)){
						newdir = 'left';	//No I18N
					}else if((newPosition.left + callout.position.left + callout.width) > (targetOffset.left + targetWidth)){
						newdir = 'right';	//No I18N
					}
				}else{
					if(( elemHeight - callout.position.top) <= (CALLOUTMINSPACE + callout.height)){
						callout.position.top = (elemHeight - callout.height - CALLOUTMINSPACE);
					}else if(callout.position.top <= CALLOUTMINSPACE){
						callout.position.top = CALLOUTMINSPACE;
					}	
					if((newPosition.top + callout.position.top) < (targetOffset.top)){
						newdir = 'top'; //No I18N
					}else if((newPosition.top + callout.position.top + callout.height) > (targetOffset.top + targetHeight)){
						newdir = 'bottom';	//No I18N
					}
				}
				if(newdir && !recalculated && !target.preventDefault && regex.flipall.test(options.positionAlterable)){
					recalculated = true;
				 	correctedPosition = calculatePosition(newdir);
				}	
			}
		}
		return correctedPosition;
	}
	/* Function to get the available height in the given direction*/
	function getAvailableHeight( direction ){
		if( !availableHeight ){
			if( within.isWindow || within.isDocument ){
				availableHeight = { 
									top: targetOffset.top - within.scrollTop,
									bottom: within.height - ( targetOffset.top + targetHeight - within.scrollTop ),
									complete: within.height
								  }
			}else{
				availableHeight = {};
				if( within.offset.top < win.scrollTop ){
					availableHeight.top = targetOffset.top - win.scrollTop;
				}else{
					availableHeight.top = targetOffset.top - within.scrollTop;
				}
				availableHeight.bottom = ( within.offset.top + within.height ) - ( targetOffset.top + targetHeight );
				var diffHeight = ( win.height + win.scrollTop ) - ( within.offset.top + within.height );
				if( diffHeight < 0 ){
					availableHeight.bottom += diffHeight;
				}	
				availableHeight.complete = 	availableHeight.top + availableHeight.bottom + targetHeight;
			}
		}
		if(!direction){
			return availableHeight;
		}
		direction = direction.split('-')[0];	//No I18N
		if(direction === 'top'){	//No I18N
			return availableHeight.top;
		}
		if(direction === 'bottom'){	//No I18N
			return availableHeight.bottom;
		}	
	}
	/* Function to get the available width in the given direction*/
	function getAvailableWidth( direction ){
		if( !availableWidth ){
			if( within.isWindow || within.isDocument ){
				availableWidth = { 
									left: targetOffset.left - within.scrollLeft,
									right: within.width - ( targetOffset.left + targetWidth - within.scrollLeft ),
									complete: within.width
								  }
			}else{
				availableWidth = {};
				if( within.offset.left < win.scrollLeft ){
					availableWidth.left = targetOffset.left - win.scrollLeft;
				}else{
					availableWidth.left = targetOffset.left - within.scrollLeft;
				}
				availableWidth.right = ( within.offset.left + within.width ) - ( targetOffset.left + targetWidth );
				var diffWidth = ( win.width + win.scrollLeft ) - ( within.offset.left + within.width );
				if( diffWidth < 0 ){
					availableWidth.right += diffWidth;
				}	
				availableWidth.complete = 	availableWidth.left + availableWidth.right + targetWidth;
			}
		}
		if(!direction){
			return availableWidth;
		}
		var dir = direction.split('-'),	//No I18N
			finaldir = dir[2] === 'corner'? dir[1]:dir[0];
		if(finaldir === 'left'){	//No I18N
			return availableWidth.left;
		}
		if(finaldir === 'right'){	//No I18N
			return availableWidth.right;
		}
	}
	/* Function to get element position for the given direction*/
	function fixPosition(direction){
		direction = direction || options.direction;
		if(!regex.relDirections.test(direction)){
			options.direction = direction = 'top-left';	//No I18N
		}
		var dir = direction.split('-'); //No I18N
		var basePosition = $.extend({}, targetOffset);
		if(dir[0] === 'top'){ //No I18N
			basePosition.top -=  (elemHeight + bufferSpace);
		}else if(dir[0] === 'bottom'){ //No I18N
			basePosition.top += targetHeight + bufferSpace;
		}else if(dir[0] === 'left'){ //No I18N
			basePosition.left -= (elemWidth + bufferSpace);
		}else if(dir[0] === 'right'){ //No I18N
			basePosition.left += targetWidth + bufferSpace;
		}	
		if(dir[1] === undefined){
			if(dir[0].match(/^left|right$/)){
				basePosition.top = basePosition.top - elemHeight/2 + targetHeight/2; 
			}else{
				basePosition.left = basePosition.left - elemWidth/2 + targetWidth/2;
			}
		}else if(dir[1] === 'top'){	//No I18N
			basePosition.top = basePosition.top + targetHeight - elemHeight;
		}else if(dir[1] === 'left'){	//No I18N
			basePosition.left = basePosition.left - elemWidth + (dir[2] === 'corner'? 0: targetWidth);	//No I18N
		}else if(dir[1] === 'right' && dir[2] === 'corner'){	//No I18N
			basePosition.left = basePosition.left + targetWidth; 
		}
		return basePosition;
	}
	/* Function to get the corrected element position based on the element positioning and relative parent positioning*/
	function getCorrectedPosition(position, element){
		var basePosition = $.extend({}, position);
		var elemPosition = element.css('position'); //No I18N
		if(elemPosition === 'fixed'){	//No I18N
			basePosition.top -= $window.scrollTop();
			basePosition.left -= $window.scrollLeft();
		}else if(elemPosition === 'relative'){ /*issue exists*/ //No I18N  
			element.css({top:0, left:0});
			var elemOffset = element.offset();
			basePosition.top -= elemOffset.top;
			basePosition.left -= elemOffset.left;
		}else{
			var parentEle = element.parent();
			while( parentEle.length && !parentEle.is('html') /*&& !parentEle.is('body')*/ && parentEle.css('position') === 'static' ){ //No I18N
				parentEle = parentEle.parent();
			}
			if(!parentEle.length || parentEle.css('position') === 'static'){ //No I18N
				parentEle = parentEle.length? parentEle.offsetParent() : element.offsetParent();
			}
			if(parentEle.length && !parentEle.is('html')/* && !parentEle.is('body')*/){	//No I18N
				var parentOffset = parentEle.offset() || {left:0, top:0};
				basePosition.top = basePosition.top - parentOffset.top + parentEle.scrollTop() /*If the element is appended to container other than body there is error by 1px*/;
				basePosition.left = basePosition.left - parentOffset.left + parentEle.scrollLeft() ;
			}
		}
		return basePosition;	
	}
	/* Function to get the dimensions of within container*/
	function getWithinDimensions(element){
		element = element.length? element : $window;
		var isWindow =  $.isWindow(element[0]),
		isDocument = !!element[0] && element[0].nodeType === 9,
		offset = (isWindow || isDocument)? { left:0, top:0 } : (element.offset() || {left:0, top:0}),
		scrollLeft = isWindow || isDocument? win.scrollLeft : offset.left < win.scrollLeft? win.scrollLeft : offset.left,
		scrollTop = isWindow || isDocument? win.scrollTop : offset.top < win.scrollTop? win.scrollTop : offset.top,
		/* For window, documentElements.clienWidth is used instead of window.innerWidth ==> because
		   When mouse is connected to lap and scrollBar is shown, window.innerWidth gives (width + scrollbarwidth)
		   documentElement.clientWidth gives width excluding scrollbar width. 	
		*/
		width = isWindow? document.documentElement.clientWidth : isDocument? document.documentElement.scrollWidth : element.outerWidth(),
		height = isWindow? document.documentElement.clientHeight : isDocument? document.documentElement.scrollHeight : element.outerHeight()
		/*
			For window|document, to find with and height, javascript functions are used, becos jquery produces wrong results in some cases
		*/
		return{
			element : element,
			isWindow:  isWindow,
			isDocument: isDocument,
			offset: offset,
			scrollLeft: scrollLeft, 
			scrollTop: scrollTop,
			width: width,
			height: height
		}
	}
	function getPosition(targetEl, elementPositioned ,opt){
		options = $.extend({}, defaultOptions, opt);  //make a copy of options
		/* target elements dimensions*/
		target = targetEl;
		element = elementPositioned;
		if(!options.showWithinTarget){
			var dimension, position, oldDirection = options.direction;
			dimension = getDimensions(targetEl);
			win ={
				scrollLeft : $window.scrollLeft(), scrollTop : $window.scrollTop(), height: document.documentElement.clientHeight, width: document.documentElement.clientWidth
			},
			targetWidth = dimension.width;
			targetHeight = dimension.height;
			targetOffset = dimension.offset;
			/* element to be positioned dimensions*/
			elemWidth = elementPositioned.outerWidth();
			elemHeight = elementPositioned.outerHeight();
			within = getWithinDimensions($(options.within));
			bufferSpace = options.bufferSpace;
			position = calculatePosition(options.direction);
			var response = {
				elementPosition: position || {left:0, top:0}, 
				oldDirection: oldDirection,
				direction: options.direction,
				arrowPosition: callout? callout.position : {left:0,top:0},
				availableHeight: (target.preventDefault && options.positionAlterable === 'fit')? within.height : getAvailableHeight(options.direction),	//No I18N
				availableWidth: getAvailableWidth(options.direction),
				arrowDirection: calloutPositions[options.direction] && calloutPositions[options.direction].split('-')[0]	//No I18N
			}
			return response;
		}else{
			return getWithinTargetPosition();			
		}
	}
	function getWithinTargetPosition(){
		/*Elements with position 'absolute'|'fixed' are preferred to be  */
		target = $(target);
		var isWindow, position = {}, containerProps = {};
		if( !target.length || ( target[0] === window || target[0].nodeType === 9 /*document*/ ) ){
			target = $(window);
			isWindow = true;
		}
		if(!regex.withinDirections.test(options.direction)){
			options.direction = 'top-left';	 //No I18N
		}
		var containerProps = { 
							   width : target.outerWidth() - (isWindow ? 0 :  parseInt(target.css('border-right-width')) + parseInt(target.css('border-left-width'))), //No I18N
							   height : target.outerHeight() - (isWindow ? 0 :  parseInt(target.css('border-top-width')) - parseInt(target.css('border-bottom-width')))	//No I18N
							};
		/*Container Properties*/
		var elementT = element.css('position');	//No I18N
		var targetOffset = isWindow ? {left:0 ,top:0} : (target.offset() || {left:0, top:0});
		if( elementT === 'fixed'){	//No I18N
			if( isWindow ){
				containerProps.offset = {left:0, top:0}
			}else{
				containerProps.offset = targetOffset;
				containerProps.offset.left -= window.scrollX;
				containerProps.offset.top -= window.scrollY; 
			}
		}else{
			containerProps.offset = targetOffset;
		} 
		/*Element Properties*/
		var elementProps = { width:element.outerWidth(), height:element.outerHeight() };
		var centerDirections = ['center','left','right','top','bottom']; // No I18N
		var directionPropertObj = {left:'width',top:'height'}; // No I18N
		for(var dir in directionPropertObj){
			var oppDir = dir === 'left'?'right':'bottom'; // No I18N
			var prop = directionPropertObj[dir];
			if(options.direction.indexOf(dir) !== -1){
				position[dir] = containerProps.offset[dir];
			}else if(options.direction.indexOf(oppDir) !== -1){
				position[dir] = containerProps.offset[dir] + containerProps[prop] - elementProps[prop];
			}else if(centerDirections.indexOf(options.direction) !== -1){
				position[dir] = containerProps.offset[dir] + containerProps[prop]/2 - elementProps[prop]/2;	
			}
		}
		if( elementT !== 'fixed' ){	//No I18N
			position = getCorrectedPosition(position, element);
		}
		return {
			elementPosition : position
		}
	}
	return {
		get: function(targetEl, elementPositioned, opt){
			if( targetEl && ( $(targetEl).length || targetEl.preventDefault || opt.customDimensions ) && $(elementPositioned).length){
				target = element  = recalculated = targetHeight = targetWidth = callout  = targetOffset   = options  = within = elemWidth = elemHeight = bufferSpace =  availableHeight = availableWidth = win = null;
				opt = opt || {};
				var positionObject = getPosition(targetEl, $(elementPositioned), opt);
				target = element  = recalculated = targetHeight = targetWidth = callout  = targetOffset   = options  = within = elemWidth = elemHeight = bufferSpace =  availableHeight = availableWidth = win = null;
				return positionObject;	
			}
		}
	}
})(jQuery, jQuery(window), jQuery(document));
