/* $Id$ */

(function($){
	/** @components tour -  Tour component can be used to explore the new and cool features of your product. 
	   @eventPrefix ztour */
	window.ZTour = function(options){
		this.options = $.extend(true, {}, this.options, options);	
	}
	ZTour.prototype = {
		options: {
			/** @member {string} 
		        [tour.datasource] - Specifies the input for the suggestions that needs to be loaded. */
			'id': undefined,	//No I18N
			/** @member {("navigation"|"slide-highlighter")}
				[tour.type = "navigation"] - Specifies the type of tour you want. "slide-highlighter" option applies for single page tour only. "navigation" type applies to multi page tour. */
			'type': 'navigation',	//No I18N
			/** @member {array}
				[tour.slides] - The various slides for your tour can be given here. */
			'slides': [],	//No I18N
			/** @member {object}
				[tour.navigation] - Settings for navigation type tour is given here. */
			'navigation': {	//No I18N
				/** @member {("icon"|"text"|"bullets"|"icon-bullets"|"text-bullets")}
					[tour.navigation.type = "icon"] - Specifies the type of buttons needed for navigation. */
				'type': 'icon',	//No I18N
				/** @member{("prev_next"|"next"|"all")}
					[tour.navigation.required = "prev_next"] - Specifies the buttons needed for navigation */
				'required': "prev_next"	//No I18N
			},
			/** @member {string}
				[tour.buttons = "close,end"] - Specifies the required buttons apart from the navigation buttons. Buttons should be given as comma seperated values. 
				For eg: 
					buttons: "close,end"
					buttons: "close"
					buttons: "close,end,start" */
			'buttons': 'close,end',	//No I18N
			/** @member {boolean}
				[tour.multiPageTour = false] - Specifies whether the tour has to be navigated across multiple pages. Indicates whether it is a multi page tour. */
			'multiPageTour': false,	//No I18N
			/** @member {boolean}
				[tour.resumeOnReload = true] - Indicates whether the tour has to resume from where user has left. */
			'resumeOnReload': true,	//No I18N
			/** @member {("none"|"current-on-target"|"total-on-slide"|"current-on-slide")}
				[tour.displaySlideNumber = 'none'] - Can be used to indicate whether slide number should be displayed on slide or target. */
			'displaySlideNumber': 'none', 	//No I18N
			'keyboardInteraction': false,	//No I18N
			'keys': {	//No I18N
				'prev': 'Back',	//No I18N
				'next': 'Next',	//No I18N
				'pause': 'Pause',	//No I18N
				'start': 'Start',	//No I18N
				'resume': 'Resume',	//No I18N
				'escTourMessage': 'ESC to exit from Tour',		//No I18N
				'end': 'End'	//No I18N
			 },
			 'widgetEventPrefix': 'ztour'	//No I18N
		},
		_EVENTS: {
			/** Triggered before a slide is shown.
				@event tour.beforeshow 
				@param {object} event eventObject
				@param {object} customObject containing options about the slide */
			'BEFORESHOW' : 'beforeshow',	//No I18N
			/** Triggered before a slide is shown.
				@event tour.aftershow 
				@param {object} event eventObject
				@param {object} customObject containing options about the slide */
			'AFTERSHOW' : 'aftershow',	//No I18N	
			/** Triggered before a slide is shown.
				@event tour.beforehide 
				@param {object} event eventObject
				@param {object} customObject containing options about the slide */
			'BEFOREHIDE': 'beforehide',	//No I18N	
			'AFTERHIDE': 'afterhide',	//No I18N
			/** Triggered when the tour is started.
				@event tour.start
				@param {object} event eventObject
				@param {object} customObject containing options of the tour. */
			'START': 'start',	//No I18N
			/** Triggered when the tour has ended. 
				@event tour.end
				@param {object} event eventObject
				@param {object} customObject containing options of the tour. */
			'END': 'end',	//No I18N
			/** Triggered when the tour stops the tour in between
				@event tour.close
				@param {object} event eventObject
				@param {object} customObject containing options of the tour. */
			'CLOSE': 'close',	//No I18N
			/** Triggered when the slide in a tour has been paused. 
				@event tour.pause 
				@param {object} event eventObject
				@param {object} customObject containing the information about the time that has elapsed and the time that is left out. */
			'PAUSE': 'pause',	//No I18N
			/**	Triggered when the slide in a tour is resumed from the paused state. 
				@event tour.resume
				@param {object} event eventObject
				@param {object} customObject containing the resumed time and the time left for the next slide to display. */
			'RESUME': 'resume',	//No I18N
			/** Triggered when slide is moved forward
				@event tour.next
				@param {object} event eventObject
				@param {object} customObject containing the information about the current slide and the next slide. */
			'NEXT': 'next',	//No I18N
			/** Triggered when the slide is moved backward
				@event tour.prev
				@param {object} event eventObject
				@param {object} customObject containing information about present and previous slide*/
			'PREV': 'prev',	//No I18N
			'RIPPLECLICK': 'hintclick',	//No I18N
			'RIPPLEREMOVE': 'hintremove', 	//No I18N
			'PAGEREDIRECT': 'pageredirect'	//No I18N
		},
		_CONSTANTS: {
			"END": "end",	//No I18N
			"CLOSE": "close",	//No I18N
			"PREVNEXT": "prev_next",	//No I18N
			"NEXT": "next",	//No I18N
			"BULLETS": "bullets",		//No I18N
			"ICONS": "icon",	//No I18N
			"ERRORMESSAGE": "LocalStorage quota exceeded. State storage failed",	//No I18N
			"MULTIPAGEUNSET": "Cannot redirect to next page as you have set multiPage to false",	//No I18N
			"IDERROR": "ID is mandatory if you have a multipage tour"	//No I18N
		},
		_stateValues: ["tourId", "currentStep", "end", "redirect", "isRedirect"],	//No I18N		
		init: function(options){	
			if(options && options.slides === undefined && options.id !== undefined){
				var options = JSON.parse(this._getValue(options.id, options));
				if(options === undefined || options === null){
					return;
				}
				this.options = options;
			}else{
				if(this.options.multiPageTour && !this.options.id){
					throw(this._CONSTANTS.IDERROR);
				}
				if(this.options.multiPageTour){
					this._setValue(this.options.id, this.options);
					this._setValue("tourId", this.options.id);	//No I18N
				}	
			}	
			this.options.multiPageTour && (this.options.resumeOnReload = true);
			this.options.resumeOnReload && (this._localStorage = window.localStorage);	
			this._bindEvents();
			this._inited = true;
			if(this.options.type === "navigation"){		//No I18N
				this._navigation = $("<div class='zpopover__footer'>");
				this._constructNavigationElements();
			}
			if(this.options.progressBar){
				this._progressValue = 100 / this.options.slides.length;
			}
			this._setValue("isRedirect", false);	//No I18N
			typeof this._currentIndex === 'number' && this._showSlide(this._currentIndex);	//No I18N
		},
		_constructNavigationElements: function(){
			if(this.options.buttons.indexOf(this._CONSTANTS.END) > -1){
				this._end = $("<button class='zbutton'><span class='zbutton__text'>"+this.options.keys.end+"</span></button>").appendTo(this._navigation);
			}else{
				this.options.navigation.type === "icon" && this._navigation.addClass("zpopover__invertfooter");	//No I18N
			}
			if(this.options.navigation.type === "icon"){	//No I18N
				this._outerContainer = $("<div class='zpopover__buttonnav'>");
				(this.options.navigation.required.indexOf(this._CONSTANTS.PREVNEXT) > -1) && (this._prev = $("<button class='zpopover__navitem'><i class='zpopover__icon'><svg><use xlink:href='#previous-icon'></use></svg></i></button>").appendTo(this._outerContainer));                                            	
				((this.options.navigation.required.indexOf(this._CONSTANTS.PREVNEXT) > -1) || (this.options.navigation.required.indexOf(this._CONSTANTS.NEXT) > -1)) && (this._next = $("<button class='zpopover__navitem'><i class='zpopover__icon'><svg><use xlink:href='#next-icon'></use></svg></i></button>").appendTo(this._outerContainer));
				this._outerContainer.appendTo(this._navigation);
			}else if(this.options.navigation.type === "text" || this.options.navigation.type === "text-bullets"){	//No I18N
				(this.options.navigation.required.indexOf(this._CONSTANTS.PREVNEXT) > -1) && (this._prev = $("<button class='zbutton'><span class='zbutton__text'>"+this.options.keys.prev+"</span></button>").appendTo(this._navigation));	//No I18N
				((this.options.navigation.required.indexOf(this._CONSTANTS.PREVNEXT) > -1) || (this.options.navigation.required.indexOf(this._CONSTANTS.NEXT) > -1)) && (this._next = $("<button class='zbutton zbutton--primary'><span class='zbutton__text'>"+this.options.keys.next+"</span></button>").appendTo(this._navigation));	//No I18N
			}
			var slideNumberType = this.options.displaySlideNumber;
			if(slideNumberType && slideNumberType === "total-on-slide"){	//No I18N
				this._slideCount = $("<span class='zpopover__slidecount'>").appendTo(this._navigation);
			}
			this._setCurrentSlide();
			this._prevIndex = this._currentIndex;
		},
		start: function(options){		
			this._removeValue("currentStep")._removeValue("redirect")._removeValue("isRedirect");	//No I18N	
			this._currentIndex = undefined;
			if(!this._inited){
				this.options.id && this._removeValue(this.options.id);
				this.options.id && this._removeValue("tourId");	//No I18N
				this.init(options);
			}
			if(this.options.type === "navigation" && typeof this._currentIndex !== 'number'){	//No I18N
				this._setCurrentSlide(0);
				this._prevIndex = this._currentIndex;
				this._showSlide(this._currentIndex);
			}
			if(this.options.type === "slide-highlighter" && !this.options.multiPageTour){	//No I18N
				this._addRippleIcons();
			}
			this._setValue("end", false);	//No I18N
		},
		_addBullets: function(popoverContainer){
			this._bulletsContainer = $("<div class='zpopover__dotnav'>");
			if(this.options.navigation.type === "icon-bullets"){	//No I18N
				this._prev = $("<button class='zpopover__navitem'><i class='zpopover__icon'><svg><use xlink:href='#previous-icon'></use></svg></i></button>").appendTo(this._bulletsContainer);		
			}
			var navItemDiv = $("<div class='zpopover__navitemgroup'>");
			var bulletButtonsList = [];
			var stepValue = this.options.firstSlideAsStart ? 1 : 0;
			for(var i=stepValue; i<this.options.slides.length;i++){
				var bulletButton = $("<button class='zpopover__navitem' title='"+i+"'><i class='zpopover__icon zicon--dot'><svg><use xlink:href='#dot-icon'></use></svg></i></button>");
				bulletButton.data("slideNumber", i);	//No I18N
				if(i === this._currentStep.currentIndex){
					bulletButton.addClass("is-selected");	//No I18N
				}
				bulletButton.on("click."+this.options.widgetEventPrefix, this._bulletClickHandler.bind(this));	//No I18N
				bulletButtonsList[i] = bulletButton[0];
			}
			this._bulletsContainer.append(bulletButtonsList);
			if(this.options.navigation.type === "icon-bullets"){	//No I18N
				this._next = $("<button class='zpopover__navitem'><i class='zpopover__icon'><svg><use xlink:href='#next-icon'></use></svg></i></button>").appendTo(this._bulletsContainer);	
			}
			if((this.options.buttons.indexOf(this._CONSTANTS.END) === -1 && (this.options.navigation.type === "icon-bullets" || this.options.navigation.type === "bullets")) || this._slideCount){	//No I18N
				this._bulletsContainer.insertAfter(popoverContainer.find(".zpopover__content"));
			}else{
				this._bulletsContainer.prependTo(this._navigation);
			}
		},
		_addRippleIcons: function(){
			this._rippleBackdropElement = $("<div class='zpopover--overley'>").appendTo("body").show();	
			this._escElement = $("<div class='zpopover--esccaution'>").text(this.options.keys.escTourMessage).appendTo("body").show();
			var base = this;
			this._escTimer = setTimeout(function(){
				base._escElement.fadeOut();
			}, 3000);
			this._rippleList = [];
			var j=0;
			for(var i=0;i<this.options.slides.length;i++){
				var currentTarget = $(this.options.slides[i].target);
				if(currentTarget.length){
					var hotspot = $("<i class='zicon--ripple'><svg><use xlink:href='#ripple-icon'></use></svg></i>").appendTo("body");
					if(currentTarget.css("position") === "fixed"){	//No I18N
						hotspot.css("position", "fixed");	//No I18N
					}
					var position = ZPosition.get($(this.options.slides[i].target), hotspot, {
						positionAlterable: false,
						showWithinTarget: true,
						direction: "center"	//No I18N
					})
					hotspot.css(position.elementPosition).data("slideNumber", i);	//No I18N
					hotspot.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, this._rippleClickHandler.bind(this));	//No I18N
					this._rippleList[j++] = hotspot;
				} 	
			}
		},
		_bulletClickHandler: function(orgEvent){
			var slideNumber = $(orgEvent.currentTarget).data("slideNumber");		
			if(slideNumber !== undefined && slideNumber !== null){
				this.goTo(slideNumber);
			}
			$(orgEvent.currentTarget).addClass("is-selected");		
		},
		_rippleClickHandler: function(orgEvent){
			var slideNumber = $(orgEvent.currentTarget).data("slideNumber");	
			var currentStep = this.options.slides[slideNumber];
			var eventReturnValue = this._triggerEvent(this._EVENTS.RIPPLECLICK, {currentIndex: slideNumber, clickedHint: $(orgEvent.currentTarget), clickedSlide: currentStep});
			if(!currentStep.highlightStep){
				this._rippleBackdropElement.show();
			}
			if(currentStep.rippleclick && $.isFunction(currentStep.rippleclick)){
				eventReturnValue = currentStep.rippleclick({currentIndex: slideNumber, clickedHint: $(orgEvent.currentTarget), clickedSlide: currentStep});
			}
			if(eventReturnValue || eventReturnValue === undefined){
				this._removeRipple($(orgEvent.currentTarget), slideNumber);
			}
			this._showSlide(slideNumber);
		},
		_removeRipple: function(element, slideNumber){
			if(typeof slideNumber === 'number'){	//No I18N
				element.remove();
				element = undefined;
				var currentStep = this.options.slides[slideNumber];
				this._triggerEvent(this._EVENTS.RIPPLEREMOVE, {currentIndex: slideNumber, visitedSlide: currentStep});
				if(currentStep.rippleremove && $.isFunction(currentStep.rippleremove)){
					currentStep.rippleremove({currentIndex: slideNumber, visitedSlide: this.options.slides[slideNumber]});
				}
			}
		},
		_removeAllRipples: function(){
			if(this._escTimer){
				clearTimeout(this._escTimer);
			}
			for(var i=0;i<this._rippleList.length;i++){
				this._removeRipple(this._rippleList[i], this._rippleList[i].data("slideNumber"));	
			}
			this._rippleList = [];
			this._rippleBackdropElement.remove();
			this._escElement.remove();
			this._escElement = undefined;
			this._rippleBackdropElement = undefined;
		},
		_bindEvents: function(){
			var base = this;
			ZComponents.$document.off("keydown."+this.options.widgetEventPrefix).on("keydown."+this.options.widgetEventPrefix, function(orgEvent){	//No I18N
				if(base.options.keyboardInteraction && orgEvent.keyCode === ZComponents.keyCode.RIGHT){
					orgEvent.preventDefault();
					base._nextSlide();
				}
				if(base.options.keyboardInteraction && orgEvent.keyCode === ZComponents.keyCode.LEFT && (base._prev || base.options.navigation.type.indexOf(base._CONSTANTS.BULLETS) > -1)){
					if(base.options.navigation.type.indexOf(base._CONSTANTS.BULLETS) > -1){
						orgEvent.preventDefault();
						base._prevSlide();
					 }else if(base._prev){
						orgEvent.preventDefault();
						base._prevSlide();
					}
				}
				if(orgEvent.keyCode === ZComponents.keyCode.ESCAPE){
					if(base.options.type === "slide-highlighter"){	//No I18N
						if(!base._currentPopover || (base._currentPopover && !base._currentPopover.is(":visible"))){	//No I18N
							base._setValue("end", true);	//No I18N
							base._removeAllRipples();
						}
					}
				}
			})
			ZComponents.$window.on("resize."+this.options.widgetEventPrefix, function(orgEvent){	//No I18N
				if(base._currentPopover && base._currentPopover.is(":visible") && base._currentStep && base._currentStep.target !== "body"){	//No I18N
					if(base._currentStep.highlightStep){
						base._highlightTarget().done(base._positionPopover());
					}
				}
			})
		},
		_preventDocumentScroll: function(shouldPrevent){
			shouldPrevent ? $("html,body").addClass("h-overlayposition") : $("html,body").removeClass("h-overlayposition");	
		},
		_setCurrentSlide: function(index){
			if(typeof index === 'number'){	//No I18N
				this._currentIndex = index;
				this._setValue('currentStep', index);	//No I18N
			}else{
				this._currentIndex = this._getValue("currentStep") !== null ? Number(this._getValue('currentStep')) : null; 	//No I18N
				this._setValue('currentStep', this._currentIndex);	//No I18N		
			}
		},
		_setValue: function(key, value){
			if(window.localStorage){
				var keyName = this.options.widgetEventPrefix+"-"+key+"-"+(this.options.multiPageTour ? this.options.id : (this.options.id ? this.options.id : "1"));	//No I18N 
				typeof value === 'object' && (value = JSON.stringify(value));	//No I18N
				try{
					window.localStorage.setItem(keyName, value);
				}catch(error){
					if(error.code === DOMException.QUOTA_EXCEEDED_ERR){
						throw (this._CONSTANTS.ERRORMESSAGE);
					}
				}
			}
		},
		_getValue: function(key, options){
			if(window.localStorage){
				var keyName;
				if(options){
					keyName = this.options.widgetEventPrefix+"-"+key+"-"+options.id; 	//No I18N
				}else{
					keyName = this.options.widgetEventPrefix+"-"+key+"-"+(this.options.multiPageTour ? this.options.id : "1"); 	//No I18N
				}
				value = window.localStorage.getItem(keyName);
				if(value === void 0 || value === "null"){	//No I18N
					value = null;
				}
				return value;
			}
		},
		_removeValue: function(key){
			if(window.localStorage){
				var keyName = this.options.widgetEventPrefix+"-"+key+"-"+(this.options.multiPageTour ? this.options.id : "1"); 	//No I18N
				window.localStorage.removeItem(keyName);
			}
			return this;
		},
		_getSlide: function(index){
			if(this.options.slides[index]){
				this._currentStep = $.extend({
					id: "zc-step-"+index,	//No I18N
					next: index === this.options.slides.length - 1 ? -1 : index + 1,
					prev: index - 1,
					currentIndex: index
				}, this.options.slides[index]);
			}
		},
		_handlePopoverBeforeOpen: function(orgEvent, ui){
			orgEvent.stopPropagation();
			if(this._currentStep.beforeshow && $.isFunction(this._currentStep.beforeshow)){
				this._currentStep.beforeshow(orgEvent, ui);	
			}

			var current = this.options.firstSlideAsStart ? this._currentIndex : this._currentIndex + 1,
				totalLength = this.options.firstSlideAsStart ? this.options.slides.length - 1 : this.options.slides.length;
			if(!this._canHaveOtherElements){
				this._start = $("<button class='zbutton zbutton--primary'><span class='zbutton__text'>"+this.options.keys.start+"</span></button>").appendTo(this._navigation).show();	//No I18N
				var base = this;
				this._start.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, function(){	//No I18N
					base._nextSlide();
				})
				this._next && this._next.hide();
				this._prev && this._prev.hide();
				this._slideCount && this._slideCount.hide();
			}else{
				this._next && this._next.show();
				this._prev && this._prev.show();
				this._slideCount && this._slideCount.show();
				this._start && this._start.hide();
			}		
			if(this.options.displaySlideNumber === "current-on-slide"){	//No I18N
				if(this._canHaveOtherElements){
					this._currentSlideContainer = $("<span class='zpopover__steps'>"+current+"</span>");
					var closeButton = ui.popover.find(".zpopover__action");	//No I18N
					closeButton.length ? this._currentSlideContainer.insertAfter(closeButton) : this._currentSlideContainer.prependTo(ui.popover);
				}	
			}
			if(this._canHaveOtherElements && this.options.navigation.type.indexOf(this._CONSTANTS.BULLETS) > -1){
				this._bulletsContainer && this._bulletsContainer.remove() && (this._bulletsContainer = undefined);
				this._addBullets(ui.popover);
			}	
			this._currentStep.arrowPointer = this._currentStep.target === "body" ? ui.popover.find(".zpopover__pointer") : undefined;	//No I18N
			if(this.options.type === "navigation"){	//No I18N
				var pointer = ui.popover.find('.zpopover__pointer'); 
				if(pointer.length){
					this._navigation.insertBefore(pointer);
				}else{
					this._navigation.appendTo(ui.popover);
				}
			}
			this._canHaveOtherElements && this._slideCount && this._slideCount.text(current+"/"+(totalLength));	//No I18N
			if(this._currentStep.delay && this._currentStep.delay.closeAfter){
				if(this.options.navigation.required.indexOf("pause") > -1){
					this._pause && this._pause.remove() && (this._pause = undefined);
					if(this.options.navigation.type === "text" || this.options.navigation.type === "text-bullets"){	//No I18N
						this._pause = $("<button class='zbutton'><span class='zbutton__text'>"+this.options.keys.pause+"</span></button>").insertBefore(this._next);	//No I18N
					}else{
						if(this.options.navigation.type === "icon"){	//No I18N	
							this._pause = $("<button class='zpopover__navitem'><i class='zpopover__icon'><svg><use xlink:href='#pause-icon'></use></svg></i></button>").insertBefore(this._next);
						}
					}		
				}
				this._resumeSlide();
			}else{
				this._pause && this._pause.remove() && (this._pause = undefined);
			}
			if(this._currentStep.coverImage){
				if(this._currentStep.coverImage.indexOf("/") > -1){	//No I18N
					ui.popover.addClass("zpopover--withcover");	//No I18N
					var coverImage = $("<img class='zpopover__image' src="+this._currentStep.coverImage+">").prependTo(ui.popover);		
					this._currentStep.width && coverImage.attr("width", (this._currentStep.width - 4).toString());	//No I18N
					var base = this;
					coverImage.attr("src", this._currentStep.coverImage).off("load."+this.options.widgetEventPrefix).on("load."+this.options.widgetEventPrefix, function(){	//No I18N
						base._currentPopover.show();
						base._fixPopoverHeight();
						base._positionPopover();

					})
				}

			}
			this._bindNavigationEvents();
			if(this._canHaveOtherElements && this.options.progressBar){
				this._addProgressBar(ui.popover);
			}
            this._currentPopover = ui.popover;
		},
		_highlightTarget: function(){		
			var spacing = this._currentStep.highlightTargetSpacing || {x: 20, y: 20},
				spacingX =  spacing.x || 20;
				spacingY = spacing.y || 20;
			if(this.options.type === "slide-highlighter"){	//No I18N
				this._rippleBackdropElement.hide();
			}
			var def = $.Deferred();
			var elementData = {}, currentStep = this._currentStep;
			var currentTarget = $(this._currentStep.target);
			var leftBottom = (ZComponents.$document.height() - ZComponents.$window.height()), 
				topLeft = (currentTarget.offset().left - spacingY).toString() + "px",	//No I18N	
					bottomTop = (currentTarget.offset().top + currentTarget.outerHeight() + spacingX),
				leftRight = (document.body["offsetWidth"] - (currentTarget.offset().left + currentTarget.outerWidth()) + currentTarget.outerWidth() + spacingY),
				rightTop = currentTarget.offset().top - spacingX, 
				rightLeft = currentTarget.offset().left + currentTarget.outerWidth() + spacingY,
				topRight = document.body["offsetWidth"] - ZComponents.$window.width(),
				topBottom = ((ZComponents.Browser.isIE() ? window.innerHeight : document.documentElement.clientHeight)  - (currentTarget.offset().top + currentTarget.outerHeight()) + currentTarget.outerHeight() + spacingX);
			leftBottom = leftBottom > 0 ? -leftBottom : 0;
			topRight = topRight > 0 ? -topRight : 0;
			!this._highlightLeftDiv && (this._highlightLeftDiv = $("<div class='zpopover--highlightbg h-left'>").appendTo("body"));
			this._highlightLeftDiv.css({
				"top": "0px",	//No I18N	
				"left": "0px",	//No I18N	
				"bottom": leftBottom,	//No I18N	
				"right": leftRight.toString() + "px"	//No I18N	
			}).show()

			!this._highlightTopDiv && (this._highlightTopDiv = $("<div class='zpopover--highlightbg h-top'>").appendTo("body"));
			
			this._highlightTopDiv.css({
				"top": "0px",	//No I18N	
				"bottom": topBottom.toString() + "px",	//No I18N	
				"left": topLeft,	//No I18N	
				"right": topRight.toString() + "px"	//No I18N	
			}).show()

			!this._highlightBottomDiv && (this._highlightBottomDiv = $("<div class='zpopover--highlightbg h-bottom'>").appendTo("body"));

			this._highlightBottomDiv.css({
				"top": bottomTop.toString() + "px" ,	//No I18N	
				"bottom": leftBottom,	//No I18N	
				"left": topLeft,	//No I18N	
				"right": (document.body["offsetWidth"] - (currentTarget.offset().left + currentTarget.outerWidth()) - spacingY).toString() + "px"	//No I18N	
			}).show()

			!this._highlightRightDiv && (this._highlightRightDiv = $("<div class='zpopover--highlightbg h-right'>").appendTo("body"));
			this._highlightRightDiv.css({
				"top": rightTop.toString() + "px",	//No I18N	
				"left": rightLeft.toString() + "px",	//No I18N	
				"bottom": leftBottom,	//No I18N	
				"right": topRight.toString() + "px"	//No I18N		
			}).show()
			!this._highlightTopLeftCurve && (this._highlightTopLeftCurve = $("<div class='zpopover--highlightcorner h-topleft'><svg><use xlink:href='#roundedcorner-icon'></use></svg></div>").appendTo("body"));
			this._highlightTopLeftCurve.css({
				"right": (leftRight - 10).toString() + "px",	//No I18N	
				"top" : rightTop.toString() + "px"	//No I18N	
			}).show()
			!this._highlightTopRightCurve && (this._highlightTopRightCurve = $("<div class = 'zpopover--highlightcorner h-topright'><svg><use xlink:href='#roundedcorner-icon'></use></svg></div>").appendTo("body"));
			this._highlightTopRightCurve.css({
				"top": rightTop.toString() + "px",	//No I18N	
				"left": (rightLeft - 10).toString() + "px"	//No I18N
			}).show();
			!this._highlightBottomRightCurve && (this._highlightBottomRightCurve = $("<div class='zpopover--highlightcorner h-bottomright'><svg><use xlink:href='#roundedcorner-icon'></use></svg></div>").appendTo("body"));
			this._highlightBottomRightCurve.css({
				"top" : (bottomTop - 10).toString() + "px",	//No I18N
				"left": (rightLeft - 10).toString() + "px"	//No I18N
			}).show();
			!this._highlightBottomLeftCurve && (this._highlightBottomLeftCurve = $("<div class='zpopover--highlightcorner h-bottomleft'><svg><use xlink:href='#roundedcorner-icon'></use></svg></div>").appendTo("body"));
			this._highlightBottomLeftCurve.css({
				"top": (bottomTop - 10).toString() + "px",	//No I18N
				"right": (leftRight - 10).toString() + "px"	//No I18N
			}).show();
			return def;
		},
		_addProgressBar: function(popoverContainer){
			this._progressBar = $("<div class='zprogress'>").prependTo(popoverContainer);
		},
		_showSlide: function(index){
			if(index >= 0){
				this._getSlide(index);
				this._setCurrentSlide(index);
				this._canHaveOtherElements = (this.options.firstSlideAsStart && this._currentIndex === 0 && this.options.type === "navigation") ? false : true;	//No I18N
				this._currentStep.target = this._currentStep.target ? this._currentStep.target : "body";	//No I18N
				var currentTarget =  $(this._currentStep.target), currentPath;
				currentTarget.off("zpopoverbeforeopen").on("zpopoverbeforeopen", this._handlePopoverBeforeOpen.bind(this));	//No I18N
				currentPath = [document.location.pathname, document.location.hash].join('');	//No I18N
				if(this.options.multiPageTour && this._currentStep.URLToSlide && JSON.parse(this._getValue("isRedirect"))){	//No I18N
					if(this._isRedirect(this._currentStep.URLToSlide, currentPath)){
						this._redirectToTarget(this._currentStep.URLToSlide);	
				 	}
				 	this._setValue("redirect", {URLToSlide: this._currentStep.URLToSlide, index: this._currentIndex});	//No I18N
					this._triggerEvent(this._EVENTS.PAGEREDIRECT, this._currentStep);
				}else{
					if(this._currentStep.URLToSlide && !this.options.multiPageTour){
						throw(this._CONSTANTS.MULTIPAGEUNSET);	
					}
				}
				var redirectPath = JSON.parse(this._getValue("redirect"));	//No I18N
				if(currentTarget.length && ((redirectPath && !this._isRedirect(redirectPath.URLToSlide, currentPath)) || !redirectPath)){
					if(this._canHaveOtherElements){
						if(this._currentStep.next === -1){
							this._next && this._next.addClass("is-disabled")
						}else{
							if(this._prev){
								var prevValue = this.options.firstSlideAsStart ? 0 : -1;
								if(this._currentStep.prev === prevValue){
									this._prev.addClass("is-disabled") 
								}else{
									if(this._prev.hasClass("is-disabled")){
										this._prev.removeClass("is-disabled")
									}
								}
							}
							if(this._next && this._next.hasClass("is-disabled")){
								this._next.removeClass("is-disabled");
							}	
						}
					}
					if(this._currentStep.target !== "body" && this._currentStep.nextOnTargetAction){	//No I18N
						this._bindTargetAction(currentTarget);
					}
					var defaultProp={ popOn: "none",		//No I18N
				                      closeAction: { on: "esc", button: this.options.buttons.indexOf(this._CONSTANTS.CLOSE) > -1 ? true : false }, 	//No I18N
				                      pointer: this._currentStep.target === "body" ? false : true, //No I18N
				                      position: (this._currentStep.target === "body") ? "page-center" : (this._currentStep.position ? this._currentStep.position : "bottom"),	//No I18N
				                      removeOnClose: true
									};
					this._currentStep = $.extend(this._currentStep, defaultProp);
					var metaData = currentTarget.data("zpopover");	//No I18N
					metaData && currentTarget.zpopover("destroy");	//No I18N
					var popover = currentTarget.zpopover(this._currentStep);
					this._currentStep = $.extend(popover.options, this._currentStep);
					var base = this;
					currentTarget.off("zpopoveropen").on("zpopoveropen", function(orgEvent, ui){	//No I18N
						orgEvent.stopPropagation();
						if(base._currentStep.target !== "body" && base._currentStep.highlightStep){	//No I18N
							if(base._backdropElement){
								base._backdropElement.hide();
								base._preventDocumentScroll(false);	
							} 
							base._highlightTarget();	
							if(base._canHaveOtherElements && base.options.displaySlideNumber === "current-on-target"){	//No I18N
								var current = base._currentIndex + 1;
							 	$("<span class='zicon--stepsontarget'>"+current+"</span>").appendTo(base._currentStep.background);
							}
						}else if(base._currentStep.target === "body" && base._currentStep.highlightStep){	//No I18N
							if(!base._backdropElement){
								base._backdropElement = $("<div class='zpopover--overley'></div>").appendTo("body");
								base._preventDocumentScroll(true);
							}else{
								base._backdropElement.show();
								base._preventDocumentScroll(true);
							}
						}else if(!base._currentStep.highlightStep && base._backdropElement && base._backdropElement.is(":visible")){	//No I18N
							base._backdropElement.hide();
							base._preventDocumentScroll(false);
						}
						if(base._currentStep.coverImage === undefined){
							base._fixPopoverHeight();
							base._positionPopover();
						}else{
							base._currentPopover.hide();
						}
						base.options.slides[index] = $.extend(base.options.slides[index], base._currentStep);
						if(base._canHaveOtherElements) {
							var buffer =  $("<div class='zprogress__bar'>").appendTo(base._progressBar).css("width", base._progressValue * ((base._prevIndex < base._currentIndex ? base._currentIndex : (base._prevIndex + 1)))+"%");	//No I18N	
							setTimeout(function(){
								buffer.css("width", base._progressValue * (base._currentIndex + 1)+"%");	//No I18N	
							},10);
						}
						if(base._currentStep.aftershow && $.isFunction(base._currentStep.aftershow)){
							base._currentStep.aftershow(orgEvent, ui);
						}
					}).off("zpopoverclose").on("zpopoverclose", function(orgEvent, ui){	//No I18N
						orgEvent.stopPropagation();
						ui.target.zpopover("destroy");	//No I18N
						if(base.options.type === "navigation"){	//No I18N
							var currentSlide = base.options.slides[base._prevIndex];
							currentSlide.arrowPointer = undefined;	
						}else{
							if(base.options.type === "slide-highlighter"){		//No I18N
								if(base._currentStep.highlightStep && (base._currentStep.target !== "body" && base._currentStep.target !== undefined)){	//No I18N
									base._highlightLeftDiv.hide();
									base._highlightRightDiv.hide();
									base._highlightBottomDiv.hide();
									base._highlightTopDiv.hide();
									base._highlightBottomRightCurve.hide();
									base._highlightBottomLeftCurve.hide();
									base._highlightTopRightCurve.hide();
									base._highlightTopLeftCurve.hide();
									base._rippleBackdropElement.show();
								}
							}
						}
						if(base.options.type !== "slide-highlighter" && orgEvent.keyCode === ZComponents.keyCode.ESCAPE){	//No I18N
							base._endTour();
						}
						base._currentPopover = undefined;		
					})
					this._scrollIntoView(this._currentStep.target, index);			
					this.options.buttons.indexOf(this._CONSTANTS.CLOSE) > -1 && currentTarget.off("zpopovercloseclick").on("zpopovercloseclick", function(orgEvent, ui){	//No I18N
						base._tourClosed = true;
						base._endTour();
					})
				}else{
					if(this.options.skipIfNoElement && currentTarget.length === 0 && ((redirectPath && !this._isRedirect(redirectPath.URLToSlide, currentPath)) || !redirectPath)){
						this._prevIndex < this._currentIndex ? this._nextSlide() : (this._prevIndex > this._currentIndex) && this._prevSlide();
					}
				}
			}			
		},
		_isRedirect: function(path, currentPath){
			return (path) && path !== '' && (({}.toString.call(path) === '[object RegExp]' && !path.test(currentPath)) || ({}.toString.call(path) === '[object String]' && path.replace(/\?.*$/, '').replace(/\/?$/, '') !== currentPath.replace(/\/?$/, '')));	//No I18N
		},
		_redirectToTarget: function(path){
			if($.isFunction(this._currentStep.redirect)){
				this._currentStep.redirect()
			}
			document.location.href = path;
		},
		_bindTargetAction: function(currentTarget){
			var base = this;
			currentTarget.off(this._currentStep.nextOnTargetAction+".reflex"+this.options.widgetEventPrefix).on(this._currentStep.nextOnTargetAction+".reflex"+this.options.widgetEventPrefix, function(orgEvent){	//No I18N
				return function(){
					if(base._currentStep.currentIndex < base.options.slides.length - 1){
						return base._nextSlide();
					}else{
						return base._endTour();	
					} 
				}
			}(this));
		},
		_pauseHandler: function(orgEvent){
			if(this._currentStep.paused){
				if(this.options.navigation.type === "text" || this.options.navigation.type === "text-bullets"){	//No I18N
					$(orgEvent.currentTarget).find('.zbutton__text').text(this.options.keys.pause);		//No I18N	
				}else if(this.options.navigation.type === "icon"){	//No I18N
					$(orgEvent.currentTarget).find('.zpopover__icon').html("").append("<svg><use xlink:href='#pause-icon'></use></svg>");
				}
				this._resumeSlide();
			}else{
				if(this.options.navigation.type === "text" || this.options.navigation.type === "text-bullets"){	//No I18N
					$(orgEvent.currentTarget).find('.zbutton__text').text(this.options.keys.resume);	//No I18N
				}else if(this.options.navigation.type === "icon"){	//No I18N
					$(orgEvent.currentTarget).find(".zpopover__icon").html("").append("<svg><use xlink:href='#play-icon'></use></svg>");
				}
				this._pauseSlide(); 
			}
		},
		_resumeSlide: function(){
			clearTimeout(this._currentStep.timer);
			this._currentStep.paused = false;
			this._currentStep.loadTime = new Date().getTime();
			this._currentStep.durationLeft = this._currentStep.durationLeft || this._currentStep.delay.closeAfter;	
			var base = this;
			this._currentStep.timer = setTimeout(function(){
				if(base.options.type === "navigation"){		//No I18N
					(base._currentStep.currentIndex < base.options.slides.length - 1) ? !base._currentStep._paused && base._nextSlide() : !base._currentStep.paused && base._endTour();
				}
			}, this._currentStep.durationLeft);
			var value = {durationLeft: this._currentStep.durationLeft, instance: this, currentStep: this._currentStep};
			if(this._currentStep.resume && $.isFunction(this._currentStep.resume)){
				this._currentStep.resume(value)
			}
		},
		_pauseSlide: function(){
			clearTimeout(this._currentStep.timer);
			this._currentStep.paused = true;
			this._currentStep.durationLeft -= new Date().getTime() - this._currentStep.loadTime;		
			var base = this;
			$(this._currentStep.target).off("zpopoverbeforeclose").on("zpopoverbeforeclose", function(orgEvent){	
				orgEvent.stopPropagation();
				if(base._currentStep.beforehide && $.isFunction(base._currentStep.beforehide)){
					base._currentStep.beforehide(orgEvent, ui);
				}
				if(base._currentStep.delay && base._currentStep.delay.closeAfter && base._currentStep.paused && !JSON.parse(base._getValue("end"))){	//No I18N
					return false;
				}
			})
			var value = {durationLeft: this._currentStep.durationLeft, instance: this, currentStep: this._currentStep};
			if(this._currentStep.pause && $.isFunction(this._currentStep.pause)){
				this._currentStep.pause(value)
			}
			this._triggerEvent(this._EVENTS.PAUSE, value);
		},
		_positionPopover: function(){
			if(this._currentStep.target === "body" && this._currentPopover){	//No I18N
				this._currentPopover.css({"position" :"fixed"});	//No I18N
			}
			this._currentPopover.addClass("ztour");
			if(this._currentStep.position !== "page-center"){	//No I18N
				var position = this._getPosition(this._currentStep, this._currentPopover);
				this._currentPopover.css(position.elementPosition);
				this._currentPopover.find(".zpopover__pointer").length && this._currentPopover.find(".zpopover__pointer").css(position.arrowPosition);
			}else{
				var top = (ZComponents.$window.outerHeight() / 2 - this._currentPopover.outerHeight() / 2) + ZComponents.$window.scrollTop(), 
					left = ((ZComponents.$window.width() - this._currentPopover.outerWidth()) / 2) + ZComponents.$window.scrollLeft();
				if(top < 0){
					top = 0;
				}else if(top >= ZComponents.documentObject.height){
	           		 top = ZComponents.documentObject.height - this._currentPopover.outerHeight(true);
				}
				if(left < 0){
					left = 0;
				}else if(left >= ZComponents.documentObject.width){ 
	            	left = ZComponents.documentObject.width - this._baseElement.outerWidth(true);
	       		}
        		var position = {
					'top':  top+ "px",	// No I18N
					'left': left+ "px"	// No I18N
				}
				this._currentPopover.css(position); 
			}
			if(this._currentPopover.outerHeight(true) > ZComponents.$window.height()){
				this._currentPopover.css("position", "absolute");	// No I18N
			}
		},
		_scrollIntoView: function(slideTarget , index){
			var rect = $(slideTarget)[0].getBoundingClientRect(), base = this;
			var isInViewPort = (rect.top >= 0 && rect.left >= 0 && rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) && rect.right <= (window.innerWidth || document.documentElement.clientWidth)); 
			if(!isInViewPort && slideTarget !== "body"){	// No I18N
				var offsetTop = $(slideTarget).offset().top, offsetLeft = $(slideTarget).offset().left;
				$("html, body").animate({scrollTop: this._lastScrollTop < offsetTop ? offsetTop + 30 : offsetTop - 30, scrollLeft: this._lastScrollLeft < offsetLeft ? offsetLeft + 30 : offsetLeft - 30 }, 400, "swing").promise().then(function(){		//No I18N			
					base._showContent(index);
				})
				this._lastScrollTop = $(slideTarget).offset().top;
				this._lastScrollLeft = $(slideTarget).offset().left;
			}else{
			 	this._showContent(index);
			}
		},
		_bindNavigationEvents: function(){
			var base = this;
			if(this._canHaveOtherElements){
				this._prev && this._prev.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, function(orgEvent){	//No I18N
					base._prevSlide();
				})
				this._next && this._next.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, function(orgEvent){	//No I18N
					base._nextSlide();
				})
				this._pause && this._pause.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, this._pauseHandler.bind(this));	//No I18N
			}
			this._end && this._end.off("click."+this.options.widgetEventPrefix).on("click."+this.options.widgetEventPrefix, function(orgEvent){	//No I18N			
				base._tourClosed = true;
				base._endTour();
			})
		},
		_showContent: function(index){
			var currentTarget = $(this._currentStep.target);
			currentTarget.zpopover('open');	//No I18N
		},
		_endTour: function(){
			this._clearOptions(true);
			this._clearAutoHide();
			this._highlightLeftDiv = this._highlightRightDiv = this._highlightTopDiv = this._highlightBottomDiv =  this._highlightTopLeftCurve = this._highlightTopRightCurve = this._highlightBottomRightCurve = this._highlightBottomLeftCurve = undefined;
			this._backdropElement && this._backdropElement.remove();
			this._backdropElement = undefined;
			this._preventDocumentScroll(false);
			clearTimeout(this._currentStep.timer);
			var currentTarget = $(this._currentStep.target);
			var metaData = currentTarget.data("zpopover");	//No I18N
			metaData && this._currentPopover && this._currentPopover.is(":visible") && currentTarget.zpopover("close");	//No I18N
			this._inited = false;
			this._prev = undefined, this._navigation = undefined, this._next = undefined, this._slideCount = undefined, this._currentIndex = undefined, this._pause = undefined;
			this._removeValue("currentStep")._removeValue("redirect")._removeValue("isRedirect");	//No I18N
			if(this.options.id){
				this._removeValue(this.options.id)._removeValue("tourId");	//No I18N
			}
			this._setValue("end", true);	//No I18N
			if(this._tourClosed){
				if(this.options.onclose && $.isFunction(this.options.onclose)){
					this.options.onclose(this);
				}
				this._triggerEvent(this._EVENTS.CLOSE);
			}else{
				this._triggerEvent(this._EVENTS.END);
				if(this.options.onend && $.isFunction(this.options.onend)){
					this.options.onend(this);
				}
			}	
		},
		_fixPopoverHeight: function(){
			var documentHeight = ZComponents.$document.height();
			if(this._currentStep.slideHeight === undefined && this._currentPopover.innerHeight() > documentHeight){
				this._currentPopover.height(documentHeight - 50);
				this._currentStep.target === "body" && this._currentPopover.css("overflow", "scroll"); 	//No I18N
			}
			else if(this._currentStep.slideHeight !== undefined){
				var title = this._currentPopover.find(".zpopover__title"), 	// No I18N  
					content = this._currentPopover.find(".zpopover__content").children(),	// No I18N  
					footer = this._currentPopover.find(".zpopover__footer"),	// No I18N  
					dotnav = this._currentPopover.find(".zpopover__dotnav"),	// No I18N  
					progress = this._currentPopover.find(".zprogress");	// No I18N  
					slidecount = this._currentPopover.find(".zpopover__slidecount"),
					buttonnav = this._currentPopover.find(".zpopover__buttonnav")
				var titleHeight = title.length ? title.outerHeight(true) : 0,
					contentHeight = content.outerHeight(true),
					footerHeight = footer.length ? footer.outerHeight(true) : 0,
					dotnavHeight = (dotnav.length && !dotnav.closest('.zpopover__footer').length ) ? dotnav.outerHeight(true) : 0;	// No I18N  
					progressHeight = progress.length ? progress.outerHeight(true) : 0;	
					slidecountHeight = this._end === undefined && slidecount.length ? slidecount.outerHeight(true) : 0,
					buttonnavHeight = this._end === undefined && buttonnav.length ? buttonnav.outerHeight(true) : 0;	 
				var coverImage = this._currentPopover.find(".zpopover__image");	// No I18N  
				var finalImageHeight, finalContentHeight;
				if(coverImage.length){
					var height = this._currentStep.slideHeight - (titleHeight + footerHeight + dotnavHeight + progressHeight + slidecountHeight + buttonnavHeight);
					if(content.outerHeight(true) > (1/3 * height)){
						finalContentHeight = (1/3) * height;
						finalImageHeight = (2/3) * height;
					}else{
						finalImageHeight = height - content.outerHeight(true);
					}
				}else{
					finalContentHeight = this._currentStep.slideHeight - (titleHeight +  footerHeight + progressHeight + dotnavHeight + slidecountHeight + buttonnavHeight) - parseInt(this._currentPopover.css("padding-top")) - parseInt(this._currentPopover.css("padding-bottom"));		// No I18N  	
				}
				finalContentHeight && (this._currentPopover.find(".zpopover__content").css("height", finalContentHeight));		// No I18N  
				finalImageHeight && coverImage.attr("height", (finalImageHeight - (parseInt(coverImage.css("margin-bottom")) + (-parseInt(coverImage.css("margin-top"))))));	// No I18N  
				this._currentPopover.css("height", this._currentStep.slideHeight); 		// No I18N  
			}
		},
		_getPosition: function(currentStep, currentPopover){
			var spacing = currentStep.highlightTargetSpacing || {x: 20, y: 20}
				spacingX = spacing.x || 20,
				spacingY = spacing.y || 20;
			if(!/^(top|bottom|right|left|top-left|top-right|bottom-left|bottom-right)$/.test(currentStep.position)){
				currentStep.position = 'bottom';	 //No I18N 				//Default position
			}
			var base = this, arrowElement = this._currentPopover.find(".zpopover__pointer");	// No I18N  
			var arrowPointer = arrowElement.length ? arrowElement : false;
			var currentTarget = $(currentStep.target);
			var arrowCallback = function(element, direction){
				base._currentPopover.removeClass("zpopover--top zpopover--bottom zpopover--left zpopover--right"); // No I18N  
				base._currentPopover.addClass("zpopover--"+direction); // No I18N  
				base._calloutDirection = direction;
			}
			if(currentStep.forElement){
				if(currentStep.forElement instanceof jQuery){
					this._forElement = currentStep.forElement;
				}else{
					if(typeof currentStep.forElement === "string" && (currentStep.forElement[0].indexOf("#") !== -1 || currentStep.forElement[0].indexOf(".") !== -1)){
						this._forElement = $(currentStep.forElement) 
					}else{
						this._forElement = false;
					}
				}
			}else{
				if(currentStep.highlightStep){
					this._forElement = {
											offset: {
												left: currentTarget.offset().left - spacingY, 
												top : currentTarget.offset().top  - spacingX
											},
											width: currentTarget.outerWidth() + (2*spacingY),
											height: currentTarget.outerHeight() + (2*spacingX)
										};
				}else{
					this._forElement = currentTarget;
				}
			}
			var popoverPosition = ZPosition.get(this._forElement, this._currentPopover, { 
					direction: currentStep.position,
					bufferSpace : 7, // The space between the element and the popover container 
					positionAlterable: true, 
					arrow: arrowPointer,  
					arrowCallback: arrowCallback, //Classes for arrow added using the callback
					customDimensions: currentStep.highlightStep ? true : false
				});
			return popoverPosition;
		},
		_nextSlide: function(orgEvent){
			if(this._currentStep.next > -1){
				this._prevIndex = this._currentIndex;
				var nextStep = this.options.slides[this._currentStep.next],  nextReturnValue;
				nextReturnValue = this._triggerEvent(this._EVENTS.NEXT, orgEvent, {currentStep : this._currentStep, popover: this._currentPopover, nextStep: nextStep});
				if(this._currentStep.onnext && $.isFunction(this._currentStep.onnext)){
					nextReturnValue = this._currentStep.onnext({currentStep : this._currentStep, popover: this._currentPopover, nextStep: nextStep});
				}
				if(nextReturnValue === undefined || nextReturnValue){					
					this._clearOptions();
					this._clearAutoHide();
					clearTimeout(this._currentStep.timer);
					this._showSlide(this._currentStep.next);	
				}
			}
		},
		_clearAutoHide: function(){
			for(var i=0; i< this.options.slides.length;i++){
				clearTimeout(this.options.slides[i].timer);
				this.options.slides[i].timer = undefined;
			}
		},
		_clearOptions: function(tourEnd){
			this._setValue("isRedirect", true);	//No I18N
			var currentTarget = $(this._currentStep.target);
			this._currentStep.nextOnTargetAction && currentTarget.off(this._currentStep.nextOnTargetAction+".reflex"+this.options.widgetEventPrefix);	//No I18N
			if(this._currentStep.highlightStep && this._currentStep.target !== "body" && this._highlightLeftDiv){		//No I18N
				this._highlightLeftDiv[tourEnd ? "remove" : "hide"](); //No I18N
				this._highlightRightDiv[tourEnd ? "remove" : "hide"](); //No I18N
				this._highlightBottomDiv[tourEnd ? "remove" : "hide"](); //No I18N
				this._highlightTopDiv[tourEnd ? "remove" : "hide"](); //No I18N
				this._highlightBottomLeftCurve[tourEnd ? "remove" : "hide"](); //No I18N
				this._highlightBottomRightCurve[tourEnd ? "remove" : "hide"](); //No I18N
				this._highlightTopRightCurve[tourEnd ? "remove" : "hide"](); //No I18N
				this._highlightTopLeftCurve[tourEnd ? "remove" : "hide"](); //No I18N
			}
		},
		_clearTargetAction: function(){
			for(var i=0; i< this.options.slides.length;i++){
				var slide = this.options.slides[i];
				if(slide.nextOnTargetAction){
					$(slide.target).off(slide.nextOnTargetAction+".reflex"+this.options.widgetEventPrefix);	
				}
			}
		},
		_triggerEvent: function(customEvent, orgEvent, ui){
			this.element = $("body");	//No I18N
			var content = {
				'tour': this.element,	// No I18N
				'value': ui 	// No I18N
			}
			return ZComponents._trigger(this.element, this.options, customEvent, orgEvent, content);		
		},
		_prevSlide: function(orgEvent){
			var lastPrevValue = this.options.firstSlideAsStart ?  0 : -1;
			if(this._currentStep.prev > lastPrevValue){
				this._prevIndex = this._currentIndex;
				var prevStep = this.options.slides[this._currentStep.prev], prevReturnValue;	
				prevReturnValue = this._triggerEvent(this._EVENTS.PREV, orgEvent, {currentStep : this._currentStep, popover: this._currentPopover, prevStep: prevStep});
				if(this._currentStep.onprev && $.isFunction(this._currentStep.onprev)){
					prevReturnValue = this._currentStep.onprev({currentStep : this._currentStep, popover: this._currentPopover, prevStep: prevStep})
				}
				if(prevReturnValue === undefined || prevReturnValue){
					this._clearOptions();
					this._clearAutoHide();				
					clearTimeout(this._currentStep.timer);
					this._showSlide(this._currentStep.prev);	
				}			
			}
		},
		_removeSlide: function(index){
			return this.options.slides.splice(index, 1);
		},
		/** Moves slide to the index specified as a parameter
			This method does not accept any arguments.
			@function tour.goTo 
			@param {integer} index - index of the slide. 
			@example <caption> Code examples </caption>
			ztour.goTo(5) */
		goTo: function(index){
			this._prevIndex = this._currentIndex;
			this._setValue("isRedirect", true);	//No I18N
			this._clearOptions();
			this._clearAutoHide();
			clearTimeout(this._currentStep.timer);
			this._showSlide(index);
		},
		/** Moves to the next slide.
			This method does not accept any arguments.
			@function tour.next
			@example <caption> Code examples </caption>
			ztour.next() */
		next: function(){
			this._nextSlide();
		},
		/** This method tells whether the tour has ended.
			This method does not accept any arguments.
			@function tour.prev
			@example <caption> Code examples </caption>
			ztour.prev() */
		prev: function(){
			this._prevSlide();
		},
		/** This method returns the index of the current slide.
			This method does not accept any arguments.
			@function tour.prev
			@return {integer} Index of the slide.
			@example <caption> Code examples </caption>
			ztour.getCurrentSlide() */
		getCurrentSlide: function(){
			return this._currentStep;
		},
		/** This method pauses the current slide if it has timeout.
			This method does not accept any arguments.
			@function tour.pause
			@example <caption> Code examples </caption>
			ztour.pause() */
		pause: function(){
			this._pauseSlide();
		},
		/** This method resumes from the paused state of the current slide.
			This method does not accept any arguments.
			@function tour.resume
			@example <caption> Code examples </caption>
			ztour.resume() */
		resume: function(){
			this._resumeSlide();
		},
		/** Clears local storage values.
			This method does not accept any arguments.
			@function tour.clearStates
			@example <caption> Code examples </caption>
			ztour.clearStates() */
		clearStates: function(){
			this._removeValue("tourId")._removeValue("currentStep")._removeValue("end")._removeValue("isRedirect")._removeValue("redirect");	//No I18N	
			if(this.options.id){	
				this._removeValue(this.options.id);
			}
		},
		removeSlides: function(index){
			if($.isArray(index)){
				for(var i=0;i<index.length;i++){
					this._removeSlide(index[i]);
				}
			}else if(index === parseInt(index, 10)){
				this._removeSlide(index);
			}
		},
		/** This method tells whether the tour has ended.
			This method does not accept any arguments.
			@function tour.hasTourEnded
			@return {boolean} Tour current running status.
			@example <caption> Code examples </caption>
			ztour.hasTourEnded() */
		hasTourEnded: function(){
			return JSON.parse(this._getValue("end"));	//No I18N	
		},
		/** Terminates the running tour.
			This method does not accept any arguments.
			@function tour.endTour
			@example <caption> Code examples </caption>
			ztour.endTour() */
		endTour: function(){
			this._endTour();
		},
		/** Allows user to set the current slide for the tour.
			@function tour.setCurrentSlide
			@param {integer} index - index of the slide which has to be made as current slide . 
			@example <caption> Code examples </caption>
			ztour.setCurrentSlide(6) */
		setCurrentSlide: function(index){
			if(index < this.options.slides.length){
				this._setCurrentSlide(index);
			}
		},
		/** Allows user to add slides to the existing slides.
			@function tour.addSlides
			@param {object} jsonData - the slide that need to be added 
			@param {integer} index - the index where the slide should be appended.  If index is not specified, the slide object will be pushed to the end of the slides list.
			@example <caption> Code examples </caption>
			ztour.addSlides({target: "#element", title: "Welcome to Zoho Components", content: "View our components by exploring our intranet page"}, 1) */
		addSlides: function(jsonData, index){
			var length = this.options.slides.length;
			if($.isArray(jsonData)){
				for(var i=0; i<jsonData.length;i++){
					this.options.slides[length++] = jsonData[i];
				}
			}else if(typeof jsonData === "object"){		//No I18N	
				index !== undefined ? this.options.steps.splice(index,0,jsonData) : this.options.slides[length];
			}
		},
		/** Removes the tour functionality from the page. This will return the page to its pre-init state.
			This method does not accept any arguments.
			@function tour.destroy
			@example <caption> Code examples </caption>
			ztour.destroy() */
		destroy: function(){
			ZComponents.$document.off("."+this.options.widgetEventPrefix);	//No I18N
			ZComponents.$window.off("."+this.options.widgetEventPrefix);	//No I18N
			this._endTour();
			this._removeValue("end");	//No I18N
		},
		option: function(optionName, value){
			if(typeof optionName === "object"){ // No I18N
				for(var key in optionName){
					this.option(key, optionName[key]);
				}
			}else if(optionName !== undefined && value !== undefined && value !== this.options[optionName]){
				if(optionName === "id"){	//No I18N
					this.clearStates();
					this._setValue("tourId", value);	//No I18N
					this.options.id = value;
				}else if(optionName === "multiPageTour"){	//No I18N
					if(value){
						if(this.options.id !== undefined){
							this.options.resumeOnReload = true;	
							this._setValue(this.options.id, this.options);
						}else{
							throw(this._CONSTANTS.IDERROR);
						}
					}else{
						for(var i=0;i<this.options.slides.length;i++){
							if(this.options.slides[i].URLToSlide){
								this.options.slides[i].URLToSlide = undefined;
							}
						}
					}
					this.options.multiPageTour = value;
				}else if(optionName === "buttons"){	//No I18N
					if(value){
						if(this.options.type === "navigation"){	//No I18N
							if(value.indexOf(this._CONSTANTS.END) > -1){
								this._end = $("<button class='zbutton'><span class='zbutton__text'>"+this.options.keys.end+"</span></button>").appendTo(this._navigation);
							}
						}				
					}
					this.options.buttons = value;
				}else if(optionName === "type"){	//No I18N
					this.options.type = value;
					if(value === "slide-highlighter"){	//No I18N
						for(var i=0; i<this._stateValues;i++){
							if(this._stateValues[i] !== "isRedirect" && this._stateValues[i] !== "end"){	//No I18N
								this._removeValue(this._stateValues[i]);
							}
						}
						this._addRippleIcons();	
					}else if(value === "navigation"){	//No I18N
						if(this._rippleBackdropElement){
							this._removeAllRipples();
						}
						this._constructNavigationElements();
					}
				}else if(optionName === "progressBar"){	//No I18N
					if(value){
						this._progressValue = 100 / this.options.slides.length
					}else{
						this._progressBar.remove();
						this._progressBar = undefined;
					}
					this.options.progressBar = value;
				}else if(optionName === "navigation"){	//No I18N
					if(value.required !== undefined && this.options.type === "navigation"){	//No I18N
						var type = value.type !== undefined ? value.type : this.options.navigation.type;
						if(type === "text"){	//No I18N
							if(value.required.indexOf(this._CONSTANTS.PREVNEXT) > -1){
								this._prev = $("<button class='zbutton'><span class='zbutton__text'>"+this.options.keys.prev+"</span></button>").appendTo(this._navigation);
							}
							if(value.required.indexOf(this._CONSTANTS.NEXT) > -1){
								this._next = $("<button class='zbutton zbutton--primary'><span class='zbutton__text'>"+this.options.keys.next+"</span></button>").appendTo(this._navigation)
							}
						}else if(type === "icon"){	//No I18N
							this._outerContainer === undefined && (this._outerContainer = $("<div class='zpopover__buttonnav'>"));
							if(value.required.indexOf(this._CONSTANTS.PREVNEXT) > -1){							
								this._prev = $("<button class='zpopover__navitem'><i class='zpopover__icon'><svg><use xlink:href='#previous-icon'></use></svg></i></button>").appendTo(this._outerContainer);
							}
							if(value.required.indexOf(this._CONSTANTS.NEXT) > -1){
								this._next = $("<button class='zpopover__navitem'><i class='zpopover__icon'><svg><use xlink:href='#next-icon'></use></svg></i></button>").appendTo(this._outerContainer);
							}
						}
						this.options.navigation.type = type;
						this.options.navigation.required = value.required;
					}				
				}else if(optionName === "displaySlideNumber"){	//No I18N
					if(value === "total-on-slide"){		//No I18N
						this._slideCount = $("<span class='zpopover__slidecount'>").appendTo(this._navigation);
					}else if(value === "none"){		//No I18N
						this._slideCount = undefined;
					}
					this.options.displaySlideNumber = value;	
				}else if(optionName === "keys"){	//No I18N
					for(var key in this.options.keys){
						if(value[key] !== undefined){
							this.options.keys[key] = value[key];
						}
						if(this.options.navigation.type === "text"){	//No I18N
							if(key === "prev"){	//No I18N
								this._prev && this._prev.find(".zbutton__text").text(value[key]);
							}else if(key === "next"){	//No I18N
								this._next && this._next.find(".zbutton__text").text(value[key]);
							}else if(key === "end"){	//No I18N
								this._end && this._end.find(".zbutton__text").text(value[key]);
							}
						}else{
							if(key === "escTourMessage"){	//No I18N
								this._escElement && this._escElement.text(value[key]);
							}
						}					
					}
				}else{
					return ZComponents._changeOptions(this.options,optionName,value);
				}
			}else{
				return ZComponents._changeOptions(this.options,optionName,value);
			}
		},
		stepOption: function(optionName, value, index){
			if(this.options.slides[index] !== undefined){
				this.options.slides[index].target === undefined && (this.options.slides[index].target = "body");
				this.options.slides[index][optionName] = value;

				var currentTarget = $(this.options.slides[index].target);
				if(optionName === "nextOnTargetAction"){	//No I18N
					if(value){
						this._bindTargetAction(currentTarget);
					}else{
						currentTarget.off(value+".reflex"+this.options.widgetEventPrefix);	//No I18N
					}
				}else{
					$(this.options.slides.target).zpopover("option", optionName, value);	
				}
			}
		}
	}
	
})(jQuery);
