/* $Id$ */
(function(ZC){
	/** @components menu - Menu allows users to perform an action from the list of choices.
		@eventPrefix zmenu
	*/
	/*
	ARIA attributes
		role - menu
		aria-disabled
		aria-hidden
		aria-checked
		role of menu item - menuitem | menuitemcheckbox | menuitemradio | separator | group
	*/
    var $ = ZC.DOMUtil;
    var menuItemTemplate = ZC.getTemplate("zmenu.hbs"); //No I18N
    var menuItemJsonTemplate = ZC.getTemplate("zmenujson.hbs"); //No I18N
    var hasHandlebars = false;
	if( !ZC.Browser.isIE() && menuItemTemplate && menuItemJsonTemplate  ){
		Handlebars.partials = Handlebars.templates;
        hasHandlebars = true;
	}
	var menuPrototype = {
		options : {
			"displayType"  : "box", // No I18N
			"direction"    : "auto",  // No I18N
			"type"         : "list", // No I18N
			"toggle"       : true, // No I18N
			"contentType"  : "text", // No I18N
			"renderInPlace": false,  // No I18N
			"columnCount"  : undefined,  // No I18N
			"columns"      : 3,  // No I18N
			"search"       : false, // No I18N
			"forElement"   : undefined, // No I18N
			"width"        : undefined, // No I18N
			"height"       : undefined, // No I18N
			"menuClass"    : "", // No I18N
			"className"    : "", // No I18N
 			"animation"    : undefined, // No I18N
			"pointer"      : false, // No I18N
			"checkedRadioItemIconClassName": undefined, // No I18N
			"checkedRadioItemSVGIconId": undefined, // No I18N
			"checkedCheckboxItemIconClassName": undefined, // No I18N
			"checkedCheckboxItemSVGIconId": undefined, // No I18N
			"isCustomHTMLEncoded"  : true, // No I18N
			"htmlSafe" : true, // No I18N
			"offset"       : undefined, // No I18N
			"scroll"	   : "slider", // No I18N
			"constructMenu": true, // No I18N
			"EFFECTS"      : {   // No I18N
				'slide' : { // No I18N
					"open" : "slideDown", // No I18N
					"close": "slideUp" // No I18N
				},
				'fade' : { // No I18N
					"open" : "fadeIn", // No I18N
					"close" : "fadeOut" // No I18N
				}
			},
            "minWidth"      : 200, // No I18N
			"handleAccessKeyNavigation" : true, // No I18N
		},
        ANIMATION_EVENTS: "webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend webkitTransitionEnd msTransitionEnd oTransitionEnd transitionEnd transitionend", // No I18N
		ARIA_ROLES :{
			"default"  : "menuitem", // No I18N
			"custom"   : "menuitem", // No I18N
			"radio"    : "menuitemradio", // No I18N
			"checkbox" : "menuitemcheckbox", // No I18N
			"itemgroup": "group", // No I18N
			"separator": "separator" // No I18N
		},
		/** @member {String} [menu.type = "list"] - The type of menu which specifies whether the menu should be shown as a list or grid.
			@allowedValues {('list'|'grid')}
			@example <caption> Initialize the menu with the "type" option specified: </caption>
    $("menuElement").zmenu({
      type: "grid"
    });
    @example <caption> Get or set the "type" option, after initialization:</caption>
    Getter Method
        var type = $("menuElement").zmenu("getAttribute","type");
    Setter Method
        $(menuElement).zmenu("setAttribute","type","grid");
		 */
		/** @member {String} [menu.contentType = "text"] - The types of all the menu items present in the menu. 
			@allowedValues {('icon'|'text'|'icon-text')}
		@example <caption> Initialize the menu with the "contentType" option specified: </caption>
    $("menuElement").zmenu({
      contentType: "icon"
    });
    @example <caption> Get or set the "contentType" option, after initialization:</caption>
    Getter Method
        var contentType = $("menuElement").zmenu("getAttribute","contentType");
    Setter Method
        $(menuElement).zmenu("setAttribute","contentType","icon");
		*/
		/* @member {Boolean} [menu.renderInPlace = false] - Specifies whether to position menu as an absolute element nearer to the "forElement" or not. If true, the menu will be opened in its current location. (ie. opened as a relative element). 
	@example <caption> Initialize the menu with the "renderInPlace" option specified: </caption>
    $("menuElement").zmenu({
      renderInPlace: true
    });
    @example <caption> Get or set the "renderInPlace" option, after initialization:</caption>
    Getter Method
        var inPlaceMenu = $("menuElement").zmenu("getAttribute","renderInPlace");
    Setter Method
        $(menuElement).zmenu("setAttribute","renderInPlace",true);
		*/
		/** @member {String} [menu.direction = "auto"] - Opening direction of the menu.
			@allowedValues {('top'|'left'|'right'|'bottom'|'top-left'|'top-right'|'bottom-left'|'bottom-right'|'at-cursor'|'auto')}
	@example <caption> Initialize the menu with the "direction" option specified: </caption>
    $("menuElement").zmenu({
      direction: "top"
    });
    @example <caption> Get or set the "direction" option, after initialization:</caption>
    Getter Method
        var direction = $("menuElement").zmenu("getAttribute","direction");
    Setter Method
        $(menuElement).zmenu("setAttribute","direction","top");
		*/
		/** @member {Number} [menu.columns = 3] - The number of columns that the "grid" type menu will have. 
	@example <caption> Initialize the menu with the "columns" option specified: </caption>
    $("menuElement").zmenu({
      columns: 4
    });
    @example <caption> Get or set the "columns" option, after initialization:</caption>
    Getter Method
        var columns = $("menuElement").zmenu("getAttribute","columns");
    Setter Method
        $(menuElement).zmenu("setAttribute","columns",4);
		*/
		/** @member {String | Object} [menu.forElement] - An element or its selector, which will be used as the target for the menu. 
	@example <caption> Initialize the menu with the "forElement" option specified: </caption>
    $("menuElement").zmenu({
      forElement: "#elementID"
    });
    @example <caption> Provide the "forElement" option while showing the menu:</caption>
    $(menuElement).zmenu("show",{"forElement":"#elementID"});
		*/
		/** @member {String} [menu.className] - The CSS class (or the space separated string of classes) to be added to the menu element for additional theming. This option is mostly used while creating menu using JSON. 
	@example <caption> Initialize the menu with the "className" option specified: </caption>
    $("menuElement").zmenu({
      className: "custom-css-class"
    });

    (or)

	ZComponents.createMenu("fileMenu",[
		{"label": "Open"}, {"label": "Save"}
	],{
		className: "custom-css-class",
		displayType: "callout"
	});
		*/
		/** @member {Number} [menu.height] - The height of the menu. 
	@example <caption> Initialize the menu with the "height" option specified: </caption>
    $("menuElement").zmenu({
      height: 400
    });
    @example <caption> Get or set the "height" option, after initialization:</caption>
    Getter Method
        var height = $("menuElement").zmenu("getAttribute","height");
    Setter Method
        $(menuElement).zmenu("setAttribute","height",400);
		*/
		/** @member {Number} [menu.width] - The width of the menu.
	@example <caption> Initialize the menu with the "width" option specified: </caption>
    $("menuElement").zmenu({
      width: 400
    });
    @example <caption> Get or set the "width" option, after initialization:</caption>
    Getter Method
        var width = $("menuElement").zmenu("getAttribute","width");
    Setter Method
        $(menuElement).zmenu("setAttribute","width",400); 
		*/	
		/* @member {Boolean} [menu.toggle = true] - Toggling nature of the menu. */
		/** @member {Boolean} [menu.displayType = "box"] - The display type of the menu. On setting displayType as "callout", the menu will have a pointing arrow towards the forElement.
		@allowedValues {("callout"|"box")}
	@example <caption> Initialize the menu with the "displayType" option specified: </caption>
    $("menuElement").zmenu({
      displayType: "callout"
    });
    @example <caption> Get or set the "displayType" option, after initialization:</caption>
    Getter Method
        var hasPointingArrow = $("menuElement").zmenu("getAttribute","displayType");
    Setter Method
        $(menuElement).zmenu("setAttribute","displayType","callout"); 
		*/
		/* @member {Boolean} [menu.search = false] - Specifies whether the menu should have a search box or not. */
		/** @member {String | Object} [menu.appendTo = "body"] - An element or its selector, which will be used as the container for the menu.
	@example <caption> Initialize the menu with the "appendTo" option specified: </caption>
    $("menuElement").zmenu({
      appendTo: "containerElementID"
    });
    @example <caption> Provide the "appendTo" option while showing the menu:</caption>
    $(menuElement).zmenu("show",{"appendTo":"containerElementID"});
		*/			
		/** @member {String} [menu.checkedRadioItemIconClassName] - The icon to be shown in the radio type menu item whenever it is in checked state. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedRadioItemIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      checkedRadioItemIconClassName: "zicon zicon-bullet"
    });
    @example <caption> Get or set the "checkedRadioItemIconClassName" option, after initialization:</caption>
    Getter Method
        var checkedRadioItemIconClassName = $("menuElement").zmenu("getAttribute","checkedRadioItemIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedRadioItemIconClassName","zicon zicon-bullet");
		*/
		/** @member {String} [menu.checkedRadioItemSVGIconId] - The icon to be shown in the radio type menu item whenever it is in checked state. It contains SVG element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedRadioItemSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      checkedRadioItemSVGIconId: "bulletIconID custom__class"
    });
    @example <caption> Get or set the "checkedRadioItemSVGIconId" option, after initialization:</caption>
    Getter Method
        var checkedRadioItemSVGIconId = $("menuElement").zmenu("getAttribute","checkedRadioItemSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedRadioItemSVGIconId","bulletIconId custom__class");
		*/

		/** @member {String} [menu.checkedCheckboxItemIconClassName] - The icon to be shown in the checkbox type menu item whenever it is in checked state. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedCheckboxItemIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      checkedCheckboxItemIconClassName: "zicon zicon-tick"
    });
    @example <caption> Get or set the "checkedCheckboxItemIconClassName" option, after initialization:</caption>
    Getter Method
        var checkedCheckboxItemIconClassName = $("menuElement").zmenu("getAttribute","checkedCheckboxItemIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedCheckboxItemIconClassName","zicon zicon-tick");
		*/
		/** @member {String} [menu.checkedCheckboxItemSVGIconId] - The icon to be shown in the checkbox type menu item whenever it is in checked state. It contains SVG icon element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedCheckboxItemSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      checkedCheckboxItemSVGIconId: "tickIconID custom__class"
    });
    @example <caption> Get or set the "checkedCheckboxItemSVGIconId" option, after initialization:</caption>
    Getter Method
        var checkedCheckboxItemSVGIconId = $("menuElement").zmenu("getAttribute","checkedCheckboxItemSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedCheckboxItemSVGIconId","tickIconID custom__class");
		*/
		/** @member {String} [menu.arrowIconClassName] - The icon to be shown in the submenu or split menu item. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "arrowIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      arrowIconClassName: "zicon zicon-arrow"
    });
    @example <caption> Get or set the "arrowIconClassName" option, after initialization:</caption>
    Getter Method
        var arrowIconClassName = $("menuElement").zmenu("getAttribute","arrowIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","arrowIconClassName","zicon zicon-arrow");
		*/
		/** @member {String} [menu.arrowSVGIconId] - The icon to be shown in the submenu or split menu item. It contains SVG icon element id followed by space separated custom CSS class name for the SVG element. 
		@newlyAdded
		@example <caption> Initialize the menu with the "arrowSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      arrowSVGIconId: "arrowIconID custom__class"
    });
    @example <caption> Get or set the "arrowSVGIconId" option, after initialization:</caption>
    Getter Method
        var arrowSVGIconId = $("menuElement").zmenu("getAttribute","arrowSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","arrowSVGIconId","arrowIconID custom__class");
		*/

		/** @member {String} [menu.scrollUpArrowIconClassName] - The icon to be shown in the scroller present in the top of the menu when the menu has scrollable content. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollUpArrowIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      scrollUpArrowIconClassName: "zicon zicon__top-scroll"
    });
    @example <caption> Get or set the "scrollUpArrowIconClassName" option, after initialization:</caption>
    Getter Method
        var scrollUpArrowIconClassName = $("menuElement").zmenu("getAttribute","scrollUpArrowIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollUpArrowIconClassName","zicon zicon__top-scroll");
		*/
		/** @member {String} [menu.scrollUpArrowSVGIconId] - The icon to be shown in the scroller present in the top of the menu when the menu has scrollable content. It contains SVG icon element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollUpArrowSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      scrollUpArrowSVGIconId: "scrollIconID custom__class"
    });
    @example <caption> Get or set the "scrollUpArrowSVGIconId" option, after initialization:</caption>
    Getter Method
        var scrollUpArrowSVGIconId = $("menuElement").zmenu("getAttribute","scrollUpArrowSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollUpArrowSVGIconId","scrollIconID custom__class");
		*/
		/** @member {String} [menu.scrollDownArrowIconClassName] - The icon to be shown in the bottom scroller of the menu when the menu has scrollable content. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollDownArrowIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      scrollDownArrowIconClassName: "zicon zicon__bottom-scroll"
    });
    @example <caption> Get or set the "scrollDownArrowIconClassName" option, after initialization:</caption>
    Getter Method
        var scrollDownArrowIconClassName = $("menuElement").zmenu("getAttribute","scrollDownArrowIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollDownArrowIconClassName","zicon zicon__bottom-scroll");
		*/
		/** @member {String} [menu.scrollDownArrowSVGIconId] - The icon to be shown in the bottom scroller of the menu when the menu has scrollable content. It contains SVG icon element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollDownArrowSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      scrollDownArrowSVGIconId: "scrollIconID custom__class"
    });
    @example <caption> Get or set the "scrollDownArrowSVGIconId" option, after initialization:</caption>
    Getter Method
        var scrollDownArrowSVGIconId = $("menuElement").zmenu("getAttribute","scrollDownArrowSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollDownArrowSVGIconId","scrollIconID custom__class");
		*/
		EFFECTS : {   
			'slide' : { // No I18N
				"open" : "slideDown", // No I18N
				"close": "slideUp" // No I18N
			},
			'fade' : { // No I18N
				"open" : "fadeIn", // No I18N
				"close": "fadeOut" // No I18N
			}
		},
		_MENU_CONTENT_TYPES : {
			"text"      : "", // No I18N
			"icon"      : "zmenu--icononly", // No I18N
			"icon-text" : "zmenu--iconwithtext" // No I18N
		},
		_MENU_ITEM_TYPES : {
			"default"   : "zmenu__item", // No I18N
			"separator" : "zmenu__separator", // No I18N
			"splitmenu" : "zmenu__item zmenu--split", // No I18N
			"itemgroup" : "zmenu__title" // No I18N
		},
		_DIRECTIONS : { 
			"menu"    : "bottom-left", // No I18N
			"submenu" : "right-bottom", // No I18N
			"pointer" : "bottom" // No I18N
		},
		_attrNames : ["id","name","data-item-type","disabled","checked","hidden","data-fixed","data-is-footer","data-item-value","data-label","data-icon","data-icon-class-name","data-svg-icon-id","data-text-as-icon","data-shortcutkey","data-shortcut-key","data-hint","data-description","data-menu-id"], // No I18N
		_EVENTS : { 
			/**
					Triggered before showing the menu. To prevent menu show, return false from an event handler.
					@event menu.beforeshow
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu and forElement.
			*/
			"ONBEFORESHOW"         : "beforeshow", // No I18N
			/**
					Triggered before hiding the menu. To prevent hiding the menu, return false from an event handler.
					@event menu.beforehide
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, and left and top position of the menu.
			*/
			"ONBEFOREHIDE"         : "beforehide", // No I18N
			/**
					Triggered after showing the menu.
					@event menu.show
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, and left and top position of the menu.
			*/
			"ONSHOW"               : "show",       // No I18N
			/**
					Triggered after hiding the menu.
					@event menu.hide
					@param {Object} event originalEvent
					@param {Object} data The object, which contains instance of menu and forElement.
			*/
			"ONHIDE"               : "hide",	   // No I18N
			/**
					Triggered while clicking the item present in the sub menu of the main menu.
					@event menu.submenuitemclick
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, clicked menu item of submenu and left and top position of the submenu.
			*/
			"ONSUBMENUMENUITEMCLICKED" : "submenuitemclick", 		// No I18N
			/**
					Triggered before showing the submenu of the currently hovered menu item. To prevent opening the submenu, return false from an event handler.
					@event menu.beforesubmenushow
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONBEFORESUBMENUSHOW"  : "beforesubmenushow",	// No I18N
			/**
					Triggered after showing the submenu of the currently hovered menu item.
					@event menu.submenushow
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONSUBMENUSHOW"        : "submenushow",         // No I18N
			/**
					Triggered before closing the submenu. To prevent closing the submenu, return false from an event handler.
					@event menu.beforesubmenuhide
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONBEFORESUBMENUHIDE"  : "beforesubmenuhide",     // No I18N
			/**
					Triggered after hiding the submenu of the currently hovered menu item.
					@event menu.submenuhide
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONSUBMENUHIDE"        : "submenuhide", // No I18N
			/**
					Triggered while hovering(placing mouse over) an menu item.
					@event menu.itemhover
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, currently hovered menu item, and left and top position of the menu.
			*/
			"ONMENUITEMHOVERED"    : "itemhover", // No I18N
			/**
					Triggered while clicking a menu item.
					@event menu.itemclick
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, clicked menu item, and left and top position of the menu.
			*/
			"ONMENUITEMCLICKED"    : "itemclick",	 // No I18N
			/**
					Triggered before setting focus on the menu while opening the menu. To prevent setting focus, return false from an event handler.
					@event menu.beforefocus
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu and forElement.
			*/
			"ONBEFOREFOCUS" : "beforefocus" // No I18N

		},
		_lastOverlayedMenuItem: undefined,
		_lastOverlayedMenu: undefined,
		_create: function(element,options){
			this.scrollWidth = menuPrototype._scrollWidth === undefined ? menuScrollWidth() : menuPrototype._scrollWidth;
			this.options.appendTo = (options.appendTo !== undefined && options.appendTo.length) ? $(options.appendTo) : undefined;
			if(!options.usingJSON){  	// usingJSON option is used to build the menu using JSON Object.
				this._buildMenu(element);
			}
			if(this.element[0].tagName === "DIV"){ // No I18N
				// this.element.find("ul:first").attr("role","menu"); // No I18N
				element.find("ul:first").addClass("zmenu__list"); // No I18N
			}
			// else{
			// 	this.element.attr("role","menu"); // No I18N
			// }
			element[0].setAttribute("role","menu"); // No I18N
			this._initEvents();
			this._menuTextArray = [];
			if(options.search){
				this._createSearchContainer(element);
			}
			if(options.renderInPlace){
				this._resetMenuTextArray(element);
				this.show();
			}else{
				element[0].style.display = "none"; // No I18N
			}
			element.data("toggle",options.toggle); // No I18N
			// Replacement for focusable method in ui.focusable.js
			element[0].setAttribute("tabindex",0);
			element[0].style.outline = 0; 
			if(options.height){
				// Height is stored in data attribute to support submenu height fixing
				element.data("height", options.height); // No I18N
			}
			if(options.width){
				// width is stored in data attribute to support submenu width fixing
				element.data("width", options.width); // No I18N
			}
			// check for the rtl direction
			if(ZC.isRTL){
				// this.element.addClass("h-rtl");
				this._DIRECTIONS.submenu = "left-bottom"; // No I18N
			}
		},
		/* Building the menu by adding component specific classes and scope classes */
		_buildMenu: function(menu){ 
			var items = menu[0].querySelectorAll("li"); // No I18N
			if(this.options.constructMenu){
				for(var i = 0; i < items.length; i++){
					this._buildMenuItem($(items[i]),menu);
				}
				menu[0].className += " zmenu "+(this.options.type === "grid" ? "zmenu--thumbnail" : this._MENU_CONTENT_TYPES[this.options.contentType])+" "+(this.options.menuClass ? this.options.menuClass : "")+" "+(this.options.className ? this.options.className : ""); // No I18N
			}else{
				for(var i = 0; i < items.length; i++){
					this._handleCheckedandDisabled($(items[i]),this.getData(items[i])); // initial state maintainance for checked,hidden and disabled.
					this._addEventsForMenuItems($(items[i])); // binding events for each menu item.
				}
			}
		},
		_createSearchContainer: function(menu){
			var header = $("<div class='zmenu__header'></div>"), // No I18N
				textBox = $("<input type='text' class='zmenu__textbox' placeholder='Search' />"), // No I18N
				searchIcon = $("<i class='zmenu__icon'></i>"),
				svgEle = ZC.createSVGElement("svg",{ // No I18N
					"class":"zmenu__search","viewBox":"0 0 16 16","width":"100%","height":"100%" // No I18N
				},[
				{"path":{"d":"M15.6,15.6c-0.6,0.6-1.5,0.6-2.1,0l-2.7-2.7C9.7,13.6,8.4,14,7,14c-3.9,0-7-3.1-7-7s3.1-7,7-7c3.9,0,7,3.1,7,7c0,1.4-0.4,2.7-1.1,3.8l2.7,2.7C16.1,14,16.1,15,15.6,15.6z M7,2C4.2,2,2,4.2,2,7s2.2,5,5,5c2.8,0,5-2.2,5-5C12,4.2,9.8,2,7,2z"}} // No I18N
				]),
				base = this;
			searchIcon[0].appendChild(svgEle);
			header.append(textBox,searchIcon);
			this.element.prepend(header);
			menu.data('_styleElement',$("<style id='zmenu-search-filter' type='text/css'>").appendTo("head")); // No I18N
			textBox.on("keydown."+this.componentName,function(event){ // No I18N
				if(event.keyCode === ZC.keyCode.TAB){ // keycode is 9 and 13
					event.preventDefault();
				}
			});
		},
		/* Building menu items present inside the menu */
		_buildMenuItem: function(menuItem,menu){

			var data = this.getData(menuItem),
				hasSubmenu = data.menuId || (menuItem[0].firstElementChild && typeof menuItem[0].firstElementChild === "object" && menuItem[0].firstElementChild.tagName ? menuItem[0].firstElementChild.tagName === "UL" : false), // No I18N
				itemClass = data.className ? data.className : (data.itemCssClass ? data.itemCssClass : ""); // No I18N
			data.isGrid = this.options.type === "grid"; // No I18N
			menuItem[0].className += " "+(this._MENU_ITEM_TYPES[data.itemType] ? this._MENU_ITEM_TYPES[data.itemType] : "zmenu__item")+" "+itemClass; // No I18N
			data.parentMenu = menuItem.parent();
			data.disabled = data.disabled !== undefined ? data.disabled : (menuItem[0].getAttribute("disabled") != null);
			data.hidden   = data.hidden !== undefined ? data.hidden : (menuItem[0].getAttribute("hidden") != null);
			data.checked  = data.checked !== undefined ? data.checked : (menuItem[0].getAttribute("checked") != null);
			if(hasSubmenu){
				data.hasSubmenu = true;
				this._buildSubmenu(menuItem,data);
			}
            menuItem[0].setAttribute("role", this.ARIA_ROLES[data.itemType ?  data.itemType : "default"]);  // No I18N
			if(data.itemType === "separator"){ // No I18N
				this._handleCheckedandDisabled(menuItem,data);
				return;
			}else if(data.itemType === "custom" || data.custom){ // No I18N
				menuItem[0].className += " zmenu__custom"; // No I18N
				if(data.innerHTML){
					var content = data.innerHTML.content,
						replaceArray = data.innerHTML.replaceArray;

					if(content && replaceArray){
						for (var i = 0; i < replaceArray.length; i++) {
	                    	content = content.replace('{'+ i +'}',ZC.encodeHTML(replaceArray[i]));	// No I18N
	                	}
	                	menuItem.html(content);
					}else{
						if(this.options.isCustomHTMLEncoded || this.options.htmlSafe){
							menuItem.html(data.innerHTML); 
						}else{
							menuItem.html(ZC.encodeHTML(data.innerHTML));
						}
					}
					// menuItem.html(data.innerHTML);
				}
				if(data.hasSubmenu){
					this._createArrowIcon(menuItem);
				}
				this._handleCheckedandDisabled(menuItem,data);
				return  this._addEventsForMenuItems(menuItem);
			}
			data.label = data.label ? data.label : menuItem.text().trim();
            data.label = data.ellipsis ? data.label+"..." : data.label; // No I18N
            var shortcutkey = data.shortcutkey || data.shortcutKey;
            data.shortcutkey = window.ZShortCut ?  ZShortCut.getInfo(data.actionName, menuItem, true) : shortcutkey;
			menuItem[0].innerHTML = ""; // No I18N
			var actualItem = menuItem;
			// actualItem is the item to append.
			if(data.action && typeof data.action === "string" && data.action.indexOf("URL:") !== -1){ // No I18N
				data.target = data.target ? data.target : "_blank"; // No I18N
				menuItem.addClass("zmenu--withlink").append("<a class='zmenu__link' href='"+data.action.replace("URL:","")+"' target='"+data.target+"'>");  // No I18N
				actualItem = menuItem.find(".zmenu__link");
			}
			if(!hasHandlebars){
				if(this.options.contentType !== "text"){ // No I18N
					if(data.itemType !== "itemgroup"){ // No I18N
						var isThumbnail = data.isGrid,
							iconContainer = isThumbnail ? undefined : ZM_DOM._iconDiv.cloneNode(),
							iconEle = ZM_DOM._icon.cloneNode();
						iconEle.className = isThumbnail ? "zmenu__image " :"zmenu__icon ";	// No I18N
                        this._handleIcon($(iconEle), undefined, undefined, data.iconClassName, data.SVGIconId, data.icon);
						!isThumbnail && iconContainer.appendChild(iconEle);
						actualItem[0].appendChild(isThumbnail ? iconEle : iconContainer);
					}else if(this.options.contentType === "icon"){ // No I18N
						actualItem[0].appendChild(this._domUpdate(ZM_DOM._text, data.label));
					}
				}
				if(this.options.contentType !== "icon"){ // No I18N
					actualItem[0].appendChild(this._domUpdate(ZM_DOM._text, data.label));
					if(data.hint || data.description){
						actualItem[0].appendChild(this._domUpdate(ZM_DOM._hint, (data.hint ? data.hint : data.description)));
					}else if(data.shortcutkey){
						actualItem[0].appendChild(this._domUpdate(ZM_DOM._key, data.shortcutkey));
					}
				}
				if(data.hasSubmenu){
					this._createArrowIcon(actualItem);
				}
			}else{ // MenuItem using handlebar templates
				data.icon = data.SVGIconId ? "#"+data.SVGIconId : (data.iconClassName ? data.iconClassName : data.icon); // No I18N
				if(data.icon && data.icon.indexOf("#") !== -1){ // No I18N
					data.svgId = data.icon.split(" ")[0]; // No I18N
					data.svgClass = data.icon.slice(svgId.length).trim();
				}
				if(data.itemType !== "itemgroup"){ // No I18N
					data[this.options.contentType === "icon" ? "icon-only" : this.options.contentType ] = this.options.contentType; // No I18N
				}
				actualItem.append(menuItemTemplate(data));
			}
			if(data.icon && data.icon.indexOf("text:") === 0 && this.options.contentType !== "text"){	// No I18N
				menuItem.find("i:first").removeClass(data.icon).text(data.icon.replace("text:",""));	// No I18N
			}
			if(data.fixed || data.isFooter){
				this._addFixedItem(menuItem,menu,data);
			}
            menuItem[0].setAttribute("data-item-type",data.itemType);
			this.options.contentType !== "icon" && menuItem[0].setAttribute("data-label",data.label); // No I18N
			this._handleCheckedandDisabled(menuItem,data);
			this._addEventsForMenuItems(menuItem,data);
		},
		_domUpdate: function(node, text){
			var itemText = node.cloneNode();
			itemText.textContent = text;
			return itemText;
		},
		_createArrowIcon: function(menuItem){
			var iconEle = $("<i class='zmenu__icon h-alignright'/>"),
                svgData = {
                    props: {"class":"zmenu__arrow","viewBox":"0 0 48 48","width":"100%","height":"100%"}, // No I18N
                    nodes: [{
                        "path":{"d":"M17,6l-4,5L27,24,13,38l4,4L36,24Z","class":"cls-1"} // No I18N
                    }]
                };
            this._handleIcon(iconEle, svgData, "arrow", this.options.arrowIconClassName, this.options.arrowSVGIconId);
			menuItem.append(iconEle);
			return iconEle;
		},
		/* Adding the fixed item in the fixed container. */
		_addFixedItem: function(menuItem,menu,data){
			if(data.label && this.options.contentType === "icon" && this.options.type === "grid" && !menuItem.find(".zmenu__text").length){
				menuItem.append($("<span class='zmenu__text'>"+data.label+"</span>"));
			}
			if(menu.find(".zmenu__footer").length){ // No I18N
				menu.find(".zmenu__footer ul").append(menuItem); // No I18N
			}else{
				var fixedArea = $("<div class='zmenu__footer'><ul class='zmenu__list'></ul></div>"); 
				menu.append(fixedArea);
				fixedArea.find("ul").append(menuItem.addClass("is-fixed")); // No I18N
				menu.data("fixedArea",fixedArea.find("ul")); // No I18N
			}			
		},
		_buildSubmenu: function(menuItem,data){
			var menu = this.getSubmenu(menuItem);
			if(!menu){
				return;
			}
			// MenuItem with itemType as "radio" and "checkbox" can also contains splitmenu or submenu.
			menuItem.addClass(data.splitmenu || data.itemType === "splitmenu" ? "zmenu--split": "zmenu--submenu"); // No I18N
			menuItem.data("submenu",menu); // No I18N
			if(!data.menuId){
				$("body").append(menuItem.find(menu)); // No I18N
				this.getData(menuItem).menuId = menu.attr("id"); // No I18N
				menu.zmenu();
			}
		},
		getSubmenu: function(menuItem){
			if(menuItem.data("submenu")){ // No I18N
				return menuItem.data("submenu"); // No I18N
			}
			var submenu = this.getData(menuItem).menuId;
			if(submenu){
				submenu = $("#"+submenu); // No I18N
				if(!submenu.length){
					return undefined;
				}
				if(!submenu.data("zmenu")){ // No I18N
					submenu.zmenu();
				}
				submenu.data("isExternalSubmenu",true);  // No I18N
				return submenu;
			}
			return (menuItem.children("ul:first")); // No I18N
		},
		_handleCheckedandDisabled: function(menuItem,data,optionName){
            var fromSetAttribute = optionName !== undefined; // check is included because disabling the hidden items(made hidden by hide() method of jquery) make the items visible.
			if(fromSetAttribute ? (optionName === "hidden") : (data.hidden !== undefined)){ // No I18N
				this._hideMenuItem(menuItem,!data.hidden);
			}
			if(fromSetAttribute ? (optionName === "disabled") : (data.disabled !== undefined)){ // No I18N
				menuItem[data.disabled ? "addClass" : "removeClass"]("is-disabled"); // No I18N
				menuItem.attr("aria-disabled",data.disabled); // No I18N
				data.disabled ? menuItem.attr("disabled",true) : menuItem.removeAttr("disabled"); // No I18N
			}
            var condition = fromSetAttribute ? (optionName === "checked") : true; // No I18N
			if(condition && (this.options.contentType !== "text" && data.checked !== undefined && (data.itemType === "checkbox" || data.itemType === "radio"))){ // No I18N
				this._checkMenuItem(menuItem,data.checked,data);
			}	
		},
		_initEvents: function(menu){
			var base = this, componentName = this.componentName;
			menu = menu ? $(menu) : $(this.element);
			if((menu.data('initialized') && !menu) || menu.data("isExternalSubmenu")){ // No I18N
				return;
			}
			menu.data('initialized', true);  // No I18N
			menu.on("mouseup."+componentName, function (event) { // No I18N
				base._handleMenuItemClick(event, menu);
			}).on("contextmenu."+componentName,function(event){ // No I18N
				// binding the context menu event to prevent closing the menu while clicking the link item.
				var item = $(event.target).closest(".zmenu__item");
				if(item.length && item.hasClass("zmenu--withlink")){
					event.stopPropagation();
				}
			});
		},
        _handleMenuItemClick: function(event, menu){
            menu = menu || this.element;
            var menuItem = $(event.target);
            if(!menuItem.is("li")){ // No I18N
                menuItem = menuItem.closest("li"); // No I18N
            }
            if(!menuItem || menuItem.length === 0 || menuItem.hasClass("separator")){
                return;
            }
            var data = this.getData(menuItem),
                hasSubmenu = menuItem.data("submenu") && menuItem.data("submenu").length && !menuItem.hasClass("zmenu--split") ? true : false; // No I18N
            if(data.disabled){
                return;
            }
            if(hasSubmenu && data.openOn === "click"){ // No I18N
                var submenu = this.getSubmenu(menuItem), base = this;
                if(submenu.is(":visible")){ // No I18N
                    this.hide(submenu,function(){
                        if(submenu.data("forElement")[0] !== menuItem[0]){ // No I18N
                        // same submenu is used for two or more items
                            base._showSubmenu(menuItem,{"event":event},false,event); // No I18N
                        }
                    });
                }else{
                    this._checkAndHideAllOpenedSubMenus(menu,true);
                    this._showSubmenu(menuItem,{"event":event},true,event); // No I18N
                }
                event.preventDefault(); // preventing the events here since closehandler hide the menu
                event.stopPropagation();
            }
            var click = data.action;
            if(click !== undefined){
                if(typeof click !== "function" && typeof click === "string" && click.indexOf("URL:") === -1){ // No I18N
                    // click = new Function(click);
                    // click.call(event,menuItem,data);
                    this._triggerFunction(click, menuItem, [event, {"menuitem": menuItem, "options": data}]); // No I18N
                    // function names given as strings will be handled here
                }else if(typeof click === "function"){ // No I18N
                    click.call(menuItem,event,{"menuitem": menuItem,"options":data});  // No I18N
                }
            }
            if(this._triggerEvent("beforeitemchecked", event, menu, menuItem, {})){
                if(data.itemType === "checkbox" || data.itemType === "radio"){ // No I18N
                // Uncheck the menuitem if it is of checkbox type and already in checked state.
                    this._checkMenuItem(menuItem,(data.itemType === "checkbox" ? !menuItem.prop("checked") :true),data); // No I18N
                }
            }
            if(this._triggerEvent((menu.data("parent") !== undefined) ?  "ONSUBMENUMENUITEMCLICKED" : "ONMENUITEMCLICKED", event, menu, menuItem, {})){ // No I18N
            // if(!this.options.renderInPlace && !data.doNotTriggerClickEvent && data.itemType !== "checkbox"){ // No I18N
                if(!this.options.renderInPlace){ 
                    !hasSubmenu && this.hide(undefined,undefined,event); // Clicking the item containing the submenu should not close the menu.
                    if(menuItem){
                        if(!menuItem.is(".zmenu--withlink")){ // No I18N
                            event.stopImmediatePropagation();
                            event.preventDefault();
                            event.stopPropagation();
                        }else{
                            menuItem.find(".zmenu__link")[0].click(); // No I18N
                        }
                    }
                }
            }
        },
		/** 
			Opens the menu in the specified direction nearer to the forElement. If direction is at-cursor, original event object should be passed.
			@function menu.show
			@param {Object} [options] - It can contain forElement,direction and event object.
			@param {Object} [menu] -  The menu to show.
			@example <caption> Example: </caption>
$("menuElement").zmenu("show",{"forElement": $("#fileButton")});
		*/
		show: function(options,menu){
            
			options = options ? options : {};
			var appendedTo = $(this.options.appendTo),
				isSubMenu = options.isSubMenu = menu ? true : false,
				menu = menu ? menu : this.element, // Menu is undefined - i.e Its the base menu
				forElement = options.forElement ? $(options.forElement) : $(this.options.forElement),
				isFloating = this._isGridMenu(menu),
				offsetValue = options.offset ? options.offset : (this.options.offset ? this.options.offset : undefined); 
			this._uninitSliders(menu);
            if(!isSubMenu){
                this.options.forElement = options.forElement = forElement;
            }
			if(menu.is(":visible") && !options.mustBeShown){  // No I18N
				if(this.options.toggle && !isSubMenu){ // toggling the menu whenever toggle option is set to true.
					this.hide();
				}
				return;
			}
			if(!isSubMenu){
				if(this.options.handleAccessKeyNavigation){
					this._initRuntimeEvents(); // Keypress events	
				}
				this.element.data("forElement",forElement); // No I18N
			}else{
				menu.data({"forElement":forElement,"parent":forElement}); // No I18N
			}
			if((!this.options.renderInPlace || isSubMenu) && !options.mustBeShown){
				if(appendedTo !== undefined && appendedTo.length ){
					appendedTo.append(menu); 
				}else{
					// submenu is also appended to body in order to support using same submenu for more than one menu items.
					$("body").append(menu);	// No I18N
				}
			}
			options.within = this.options.viewport && $(this.options.viewport).length ? $(this.options.viewport) : appendedTo;
			this.updateLastMenuChain(menu);
			if(menu.data("modified")){ // No I18N
				menu.find("#noresultFound").remove();  // No I18N
				menu.data("_styleElement").empty(); // No I18N
			}
			if(!isFloating && !this.options.renderInPlace){
				menu.width("auto");  // No I18N
			}
			if(!this.options.renderInPlace){
                ZC.bindDocumentEvents && ZC.bindDocumentEvents({
                    element: menu,
                    forElement: forElement,
                    ctype: this.componentName,
                    closeMethodName: "hide" // No I18N
                });
				var originalEvent = options.event ? options.event : {};
                options.isRightClick = options.isRightClick ? options.isRightClick : (options.direction ? ( options.direction === "after-pointer" || options.direction === "at-cursor") : ( this.options.direction === "after-pointer" || this.options.direction === "at-cursor")); // No I18N
				if(!this._triggerEvent(isSubMenu ? "ONBEFORESUBMENUSHOW" : "ONBEFORESHOW", originalEvent, menu, undefined, {"currentTarget": options.isRightClick ? originalEvent.target : forElement})){ // No I18N
					return;
				}
				var base = this;
				menu.css("opacity",0); // No I18N
				menu.css("display","block"); // No I18N
				this._setFocus();
				if(menu.data("width")){ // No I18N
					var menuWidth = parseInt(menu.outerWidth(true)),
						optionWidth = parseInt(menu.data("width")); // No I18N
					if(menuWidth > optionWidth){
						// finding the maximum shortcut key's width
						var maxWidth = menu.find("li").map(function(index,item){
							return ($(item).find(".zmenu__shortcut").length ? parseInt($(item).find(".zmenu__shortcut").outerWidth(true)) : 0);
						});
						maxWidth = Math.max.apply(null, maxWidth);
						if(maxWidth){ // should not set 100% to text span
							menu.find("li:not(li.zmenu__title) .zmenu__text").css("width","calc(100% - "+maxWidth+"px)"); // No I18N			
						}
					}
					menu.width(optionWidth);
				}else if(menu.width() < parseInt(this.options.minWidth) && menu.data("zmenu").options.contentType !== "icon"){ // No I18N
					// min-width for menu. 
                    // Subtract border width if box-sizing is menu's box-sizing is border-box. 1px is added extra by default while setting width. So subtracted 1px.
                    if(menu.css("box-sizing") === "border-box"){ // No I18N
                        menu.width(parseInt(this.options.minWidth) - parseInt(menu.css("border-left-width")) - parseInt(menu.css("border-right-width")) - 1); // No I18N
                    }else{
                        menu.width(parseInt(this.options.minWidth) - 1);
                    }
				}

				// finding the maximum width of li
				/* var maxWidth = menu.find("li").map(function(index,item){
					return ($(item).find(".zmenu__text").length ? parseInt($(item).find(".zmenu__text").outerWidth(true)) : 0) + ($(item).find(".zmenu__shortcut").length ? parseInt($(item).find(".zmenu__shortcut").outerWidth(true)) : 0) + ($(item).find(".zicon--arrow h-alignright").length ? parseInt($(item).find(".zicon--arrow h-alignright").outerWidth(true)) : 0) ;
				});
				maxWidth = Math.max.apply(null, maxWidth)); */
				var hasScroll = false;
				if(isFloating){
					this._fixFloatingMenuWidth(menu); // Fixing the width of grid type menu.
				}
				menu.css("height","auto"); // No I18N 	
	 			menu.find("ul").length && menu.find("ul").css("height","auto"); // No I18N
				if(menu.data("height") <= menu.outerHeight()){ // No I18N
					// Fixing the menu height,which is provided in the option only if it is less than actual height of the menu
					// menu.height(menu.data("height")); // No I18N 
					menu.outerHeight(parseInt(menu.data("height"))); // No I18N 
					hasScroll = true;
				}
				if(!isFloating){ // fixing the width of list type menu.
					var mwidth = menu.innerWidth() + 1 + ( hasScroll ? this.scrollWidth : 0 ); // To consider width in floating numbers, added 1px to innerWidth value.
					menu.width(mwidth);
				}
	            this.refresh(menu);
	            if(options && !options.position){
					options.direction = options.direction ? ( (options.direction === "auto" || options.isRightClick) ? ( (this.options.pointer || this.options.displayType === "callout") ? this._DIRECTIONS.pointer : this._DIRECTIONS.menu) : options.direction) : (isSubMenu ? this._DIRECTIONS.submenu : ( (this.options.direction === "auto" || options.isRightClick) ? ((this.options.pointer || this.options.displayType === "callout") ? this._DIRECTIONS.pointer : this._DIRECTIONS.menu) : this.options.direction)); // No I18N
					options.eventObj = options.event;
	                options.positionAlterable = options.positionAlterable !== undefined ? options.positionAlterable : (isSubMenu ? "flipfit" : (this.options.direction === "auto" || this.options.direction === "after-pointer" || this.options.direction === "at-cursor")); // No I18N
	                /* if direction is after-pointer, we should not place the menu to the left or right of the target */
	                options.positionAlterable = ((this.options.direction === "after-pointer" || this.options.direction === "at-cursor" || options.isRightClick) && !isSubMenu) ? "fit" : options.positionAlterable; // No I18N
	                if(typeof options.position !== "object"){ // No I18N
	                	if((this.options.pointer || this.options.displayType === "callout") && !isSubMenu){ // No I18N
	                		this._createPointerElement();
	                		var base = this; 
	                		options.arrowCallback = function(element,direction){
	                			base._arrowDirection = direction;
	                			element.removeClass("h-top h-bottom h-right h-left").addClass('h-'+direction); // No I18N
	                		};
	                	}
						menu.data("isRightClick",options.isRightClick);  // No I18N
	                	var pos = ZPosition.get( (options.isRightClick && !isSubMenu ? options.event : forElement), menu, $.extend({},{"arrow": {"element": !isSubMenu ? this._pointerElement : null,"callback":options.arrowCallback}},options),this._pointerElement ? this._pointerElement : undefined); // No I18N
						options.availableHeight = pos.availableHeight;
						options.position = pos.elementPosition;
						options.position.direction = pos.direction;
						menu.data("direction",pos.direction);  // No I18N
						if(this._pointerElement && !isSubMenu){
							if(offsetValue){
								if(pos.arrowDirection.match(/^top|bottom$/)){ 
									// adjust left position.
									pos.arrowPosition.left = pos.arrowPosition.left - (offsetValue.left);
								}else{ // adjust top position.
									pos.arrowPosition.top = pos.arrowPosition.top - (offsetValue.top);
								}
							}
							this._pointerElement.css(pos.arrowPosition);
							
						}else if(!this._pointerElement && !offsetValue){
							// Resetting the border radius which was set previously.
							menu.removeClass("zmenu--bottomleft zmenu--bottomright zmenu--topleft zmenu--topright zmenu--top zmenu--bottom zmenu--left zmenu--right"); // No I18N
							if(isSubMenu){
								if(pos.direction === "right-top"){ // No I18N
									menu.addClass("zmenu--bottomleft"); // No I18N
									options.position.left = options.position.left - 1;
								}else if(pos.direction === "right-bottom"){ // No I18N
									menu.addClass("zmenu--topleft"); // No I18N
									options.position.left = options.position.left - 1;
								}else if(pos.direction === "left-top"){ // No I18N
									menu.addClass("zmenu--bottomright"); // No I18N
									options.position.left = options.position.left + 1;
								}else if(pos.direction === "left-bottom"){ // No I18N
									menu.addClass("zmenu--topright"); // No I18N
									options.position.left = options.position.left + 1;
								}
							}else{
								// setting the border radius based on the menu opening direction.
								var topLeft = parseInt(forElement.css("border-top-left-radius")), // No I18N
									topRight = parseInt(forElement.css("border-top-right-radius")), // No I18N
									bottomLeft = parseInt(forElement.css("border-bottom-left-radius")), // No I18N
									bottomRight = parseInt(forElement.css("border-bottom-right-radius")); // No I18N
								if(topLeft <= 5 && topRight <= 5 && bottomLeft <=5 && bottomRight <= 5){
									if(options.isRightClick){
										menu.removeClass("zmenu--bottomleft zmenu--bottomright zmenu--topleft zmenu--topright zmenu--top zmenu--bottom zmenu--left zmenu--right"); // No I18N
									}else if(pos.direction.indexOf("bottom") === 0){ // No I18N

										if(pos.direction === "bottom-right"){ // No I18N
											if(forElement.outerWidth(true) < menu.width()){ 
												// menu.addClass("zmenu--topleft");  // No I18N
												menu.addClass("zmenu--topright");  // No I18N
												forElement.css({"border-bottom-left-radius":"0px","border-bottom-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--top"); // No I18N
												forElement.css({"border-bottom-right-radius":"0px"});	 // No I18N							
											}
										}else if(pos.direction === "bottom-left"){  // No I18N
											if(forElement.outerWidth(true) < menu.width()){
												// menu.addClass("zmenu--topright"); // No I18N
												menu.addClass("zmenu--topleft"); // No I18N
												forElement.css({"border-bottom-left-radius":"0px","border-bottom-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--top"); // No I18N
												forElement.css({"border-bottom-left-radius":"0px"});	 // No I18N	
											}
										}
										options.position.top = options.position.top - 1;
									}else if(pos.direction.indexOf("top") === 0){ // No I18N
										if(pos.direction === "top-right"){ // No I18N
											if(forElement.outerWidth(true) < menu.width()){ 
												// menu.addClass("zmenu--bottomleft"); // No I18N
												menu.addClass("zmenu--bottomright"); // No I18N
												forElement.css({"border-top-left-radius":"0px","border-top-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--bottom"); // No I18N
												forElement.css({"border-top-right-radius":"0px"});	 // No I18N							
											}
										}else if(pos.direction === "top-left"){ // No I18N
											if(forElement.outerWidth(true) < menu.width()){
												// menu.addClass("zmenu--bottomright"); // No I18N
												menu.addClass("zmenu--bottomleft"); // No I18N
												forElement.css({"border-top-left-radius":"0px","border-top-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--bottom"); // No I18N
												forElement.css({"border-top-left-radius":"0px"});	 // No I18N	
											}
										}
										options.position.top = options.position.top + 1;
									}else if(pos.direction.indexOf("left") === 0){ // No I18N
										menu.addClass("zmenu--right"); // No I18N
										forElement.css({"border-top-left-radius":"0px","border-bottom-left-radius":"0px"});	 // No I18N	
									}else if(pos.direction.indexOf("right") === 0){ // No I18N
										menu.addClass("zmenu--left"); // No I18N
										forElement.css({"border-top-right-radius":"0px","border-bottom-right-radius":"0px"}); // No I18N
									}
								}else{ // adjust the top value of the menu by 2 pixels.
									options.position.top = options.position.top + 2;
								}
							}
						}
	                }
	                if(menu.find(".zmenu__textbox").length){ // No I18N
						menu.data("initialDirection",options.position.direction); // No I18N
						var searchDiv = menu.find('.zmenu__search').parent(); // No I18N
						if((options.position.top < $(forElement).position().top)){
							menu.append(searchDiv.addClass("zmenu__footer").removeClass("zmenu__header")); // No I18N
							var pos = ZPosition.get( (options.isRightClick && !isSubMenu ? options.event : forElement), menu, options,this._pointerElement ? this._pointerElement : undefined); 
							options.position = pos.elementPosition;
							options.position.top = options.position.top + 1;
						}else{
							menu.prepend(searchDiv.addClass("zmenu__header").removeClass("zmenu__footer")); // No I18N
						}
					}
					if(offsetValue){
						options.position.top = options.position.top + (parseInt(offsetValue.top) || 0);
						options.position.left = options.position.left + (parseInt(offsetValue.left) || 0);
					}
	            }
               
                if(options.position){
                    if(options.isRightClick){ /* Adjusting two pixel for right click menu's since mouseup events will not be triggered for the clicked portions if we open the menu in the same location as that of cursor. */
                        options.position.top = options.position.top + 2;
                        options.position.left = options.position.left + 2;
                    }
                    menu.css(options.position)
                }
				this._fixMenuUI(menu, options, isSubMenu);
				if(menu.find("ul:first .zmenu__item").length === 0 && menu.find(".zmenu__footer").length){ // No I18N
					menu.find(".zmenu__footer").css("border-top","0px"); // No I18N
				}
				menu.hide();
				menu.css("opacity",""); // No I18N
				this._pointerElement && this._pointerElement.hide();
				this._openWithAnimation(menu,options,originalEvent);
			}else{
				menu.css({"width":"inherit","height":"inherit","position":"relative"}); // No I18N
				this._openWithAnimation(menu,options,originalEvent);
			}
			if(!isSubMenu){
				if(ZC.currentMenu && ZC.currentMenu.get(0) != menu.get(0)){
					ZC.prevMenu = ZC.currentMenu;
				}
				ZC.currentMenu = menu;
			}
		},
		_openWithAnimation: function(menu,options,originalEvent){
			var animationObject = this.options.animation;
			if(options.isSubMenu){
				animationObject = menu.data("zmenu").options.animation; // No I18N
			}
			if(animationObject || options && options.animation){
				var base = this,
					animation = animationObject ? animationObject.open : options.animation.open,
					animationClass = animation.className,
					effect = animation.name,
					duration = animation.duration ? animation.duration : 0,
					completeFunction = function(){
						base._handleMenuOpen(menu,options,originalEvent);
					};
				if(animationClass){
					this._pointerElement && this._pointerElement.show().addClass(animationClass);
					menu.show().addClass(animationClass).on(this.ANIMATION_EVENTS, function(){
                        menu.removeClass(animationClass); // removing the animation class.
						completeFunction && completeFunction();
                        completeFunction = undefined;
						menu.off(base.ANIMATION_EVENTS);
					});
				}else if(typeof effect === "object"){  // No I18N
					// custom animation properties
					this._pointerElement && this._pointerElement.show().animate(effect,duration,completeFunction);
					menu.show().animate(effect,duration,completeFunction);
				}else{ // predefined effects like slide and fade.
					if(this.options.EFFECTS[effect]){
						this._pointerElement && this._pointerElement[this.options.EFFECTS[effect].open](duration,completeFunction);
						menu[this.options.EFFECTS[effect].open](duration,completeFunction);
					}
				}
			}else{
				menu.show();
				this._pointerElement && this._pointerElement.show();
				this._handleMenuOpen(menu,options,originalEvent);
			}
		},
		_handleMenuOpen: function(menu,options,originalEvent){
			this._resetMenuTextArray(menu);
			// if(!options.preventHover && !this.options.preventHover && menu.find("li[data-item-type='radio'],li[data-item-type='checkbox']").length){ // No I18N
			// 	var checkedItems = menu.find("li[data-item-type='radio'][checked],li[data-item-type='checkbox'][checked]");
			// 	checkedItems.length && this._handleMouseOverOnMenuItems($(checkedItems[checkedItems.length - 1]),{},true,true);
			// }
			!this.options.renderInPlace && this._triggerEvent(options.isSubMenu ? "ONSUBMENUSHOW" : "ONSHOW", originalEvent, menu, undefined, {"forElement":options.forElement});  // No I18N
		},
		_createPointerElement: function(){
			if(this._pointerElement){
				return this._pointerElement.removeAttr("class").addClass("zmenu__pointer"); // No I18N
			}
			this._pointerElement = $("<div class='zmenu__pointer' />");
			this.element.prepend(this._pointerElement);
		},
		_isGridMenu: function(menuOrMenuItem, isMenuItem){
			var menu = isMenuItem ? menuOrMenuItem.closest(".zmenu") :  // No I18N
										(menuOrMenuItem? menuOrMenuItem: this.element);
			if(menu.length){
				var type = this.getMenuType(menu);
				return this._isTypeFloating(type, menu);
			}
		},
		/* Binding hover events on menu items */
		_addEventsForMenuItems: function(menuItem){
			var base = this,
				metaData = this.getData(base.element);
			menuItem.hover(function(event, data){
				event.data = data;
				var isdirect = base._isDirectEvent(event),
					mouseOverFunc = function(){
						base._handleMouseOverOnMenuItems(menuItem, event,undefined,true);
					};
				base._clearMouseHoverTimeout();
				if(!isdirect || !base._isToBeDelayed(event)){
					mouseOverFunc();
				}else{
					metaData.mouseOverTimeout = setTimeout(mouseOverFunc,300);
				}
			}, function(event, data) {
				event.data = data;
				var isdirect = base._isDirectEvent(event),
					mouseOutFunc = function(){
						base._handleMouseOutOnMenuItems(menuItem, event);
					};
				if(!isdirect || ( !menuItem.data("submenu") && !metaData.mouseOverTimeout)){
					mouseOutFunc();
				}else{
					metaData.mouseOutTimeout = setTimeout(mouseOutFunc,300);
				}
			});
		},
		/** 
			Hides the menu.
			@function menu.hide
			@param {Object} [menu] - The menu to hide.
			@example <caption> Example: </caption>
$("menuElement").zmenu("hide",menu);
		*/
		hide: function(menu,callback,event){
			if(this.options.renderInPlace){
				return;
			}
			var isSubMenu = menu && menu.length ? true : false,base = this;
			menu = menu ? $(menu) : this.element;
			if(menu.is(":hidden")){ // No I18N
				return;
			}
			var result = this._triggerEvent(isSubMenu ? "ONBEFORESUBMENUHIDE" : "ONBEFOREHIDE", event, menu); // No I18N
			if(result){			
				if(!isSubMenu){
					this._hideAllSubMenus(menu);
					this._destroyRuntimeEvents();	
				}
				$("li.on-hover",menu).each(function(i, menuItem){ // No I18N
					if($(menuItem).is(".zmenu--split")){ // No I18N
						$(menuItem).find(".zmenu__icon.h-alignright").removeClass("on-hover");
					}
					$(menuItem).removeClass("on-hover");  // No I18N
				});
				if(menu.get(0) !== this.element.get(0)){
					this.updateLastMenuChain(menu.data("parent").parent());  // No I18N
				}else{
					this._lastOverlayedMenuItem = undefined;
				}
				this._uninitSliders(menu);	
				this._closeWithAnimation(menu,isSubMenu,callback,event);	
			}
		},
		_closeWithAnimation: function(menu,isSubMenu,callback,event){
			var animation = this.options.animation;
			if(isSubMenu){
				animation = menu.data("zmenu").options.animation; // No I18N
			}
			if(animation && animation.close){
				animation = animation.close;
				var base = this,
					animationClass = animation.className,
					effect = animation.name,
					duration = animation.duration ? animation.duration : 0,
					completeFunction = function(){
						base._handleMenuClose(menu,isSubMenu,callback,event); // hiding the menu on animation complete function
					};
				if(animationClass){
					menu.addClass(animationClass).on(this.ANIMATION_EVENTS, function(){
						menu.removeClass(animationClass);
						menu.hide();
						completeFunction && completeFunction();
						completeFunction = undefined;
						menu.off(base.ANIMATION_EVENTS);
					});
				}else if(typeof effect === "object"){  // No I18N
					// custom animation properties
					var base = this;
					menu.animate(effect,duration,completeFunction);
				}else{ // predefined effects like slide and fade. 
					if(this.options.EFFECTS[effect]){
						menu[this.options.EFFECTS[effect].close](duration,completeFunction);
					} 
				}
				if(isSubMenu){
					menu.data("parent").closest(".zmenu").data("waitingTime",duration); // No I18N
				}
			}else{
				var base = this;
				if(menu.data("waitingTime") > 0){
					window.setTimeout(function(){
						menu.hide();
						base._handleMenuClose(menu,isSubMenu,callback,event);
						menu.data("waitingTime",0);  // No I18N
					},menu.data("waitingTime"));  // No I18N
				}else{
					menu.hide();
					base._handleMenuClose(menu,isSubMenu,callback,event);
				}
				
			}
		},
		_handleMenuClose: function(menu,isSubMenu,callback,event){
			if(isSubMenu){
				var __menu = $(menu.data('parent')).closest(".zmenu");  // No I18N
				this._resetMenuTextArray(__menu);
			}else{
				this._resetMenuTextArray(menu);				
			}
			menu.css({top:"", left:"",height:"", width:""}); // No I18N
			menu.removeClass("zmenu--bottomleft zmenu--bottomright zmenu--topright zmenu--topleft zmenu--bottom zmenu--top zmenu--left zmenu--right"); // No I18N
			$(menu.data("forElement")).css({"border-radius":""}); // No I18N
			if(menu.data("_styleElement") !== undefined){ // No I18N
				menu.data("_styleElement").empty(); // No I18N
			}
			this._triggerEvent(isSubMenu? "ONSUBMENUHIDE": "ONHIDE", event, menu); // No I18N
			callback && callback();
		},
		/* Clearing the preserved mouseover and mouseout timeout values */
		_clearMouseHoverTimeout: function(){
			var metaData = this.getData(this.element);
			clearTimeout(metaData.mouseOverTimeout);
			clearTimeout(metaData.mouseOutTimeout); 
			if(this.element.data("parent") !== undefined){ // No I18N
				var parData = this.getData( this.element.data("parent").closest(".zmenu") ); // No I18N
				clearTimeout(parData.mouseOverTimeout);
				clearTimeout(parData.mouseOutTimeout);
				parData.constantLoc = undefined;
			}
		},
		_isElementInMenu: function(element, menu){
			if(!menu){
				menu = this.element;
			}
			element = $(element)[0];
			return (menu.has(element).length === 1);
		},
		/* Checking whether the submenu hide should be delayed in order to provide inclined mouseover on submenu items */
		_isToBeDelayed: function(e){
			var metaData = this.getData(this.element);
			if(metaData.constantLoc !== undefined){
				var cursorPosition = {x:e.pageX,y:e.pageY},
					decreasingSlope = this._slope(cursorPosition, this.decreasingCorner),
	                increasingSlope = this._slope(cursorPosition, this.increasingCorner);
	            if(decreasingSlope < this.prevDecreasingSlope && increasingSlope > this.prevIncreasingSlope){
	            	return true;
	            }
			}
			return false;
		},
		_scrollToView: function(menuItem, menu, isTop){
			// menu = menu.is("div") ? menu.find("ul:first") : menu; // No I18N
	        var listScrollTop = menu.scrollTop(),
	        	currentItemHeight = menuItem.height(),
	        	currentTopPosition = menuItem.position().top,
	        	listHeight = menu.outerHeight(),sval = listScrollTop;
	        if((listHeight - currentTopPosition) < currentItemHeight) { // scrolling forward in a menu
	         	// sval = listScrollTop + (currentTopPosition - listHeight/2 - currentItemHeight);
	         	sval = listScrollTop + currentTopPosition; // considering the currentTopPosition & listScrollTop alone
	        }
	        else if (currentTopPosition < 0) {  // Scrolling backward
	           	sval = listScrollTop + currentTopPosition - currentItemHeight; // fix for setting zero to scrolltop because of negative values in currentTopPosition.
	        }
	        // menu.animate({scrollTop: sval},{"duration": 100,"easing": "linear"});	// No I18N
	        menu.scrollTop(sval); // animation is removed because in fast repeat keyboards animation produces incorrect results.
		},
		_handleMouseOverOnMenuItems: function(menuItem,event,scrollToView,triggerEvent,fromLoad){
			if(menuItem.is(".is-disabled") || menuItem.is(".zmenu__title")){ // No I18N
				this._lastOverlayedMenuItem && this._lastOverlayedMenuItem.removeClass("on-hover");
				return;
			}
			if(scrollToView){
				this._scrollToView(menuItem, menuItem.closest(".zmenu").is("ul") ? menuItem.closest(".zmenu") : menuItem.closest(".zmenu").find("ul:first")  );	// No I18N
			}
			if(menuItem.is(".on-hover") && this._isMenuOfMenuItemShown(menuItem)){ // No I18N
				this._setLastOverlayedMenuItem(menuItem);
				return;
			}
			if(fromLoad || (triggerEvent && this._triggerEvent("ONMENUITEMHOVERED", event, menuItem.closest(".zmenu"), menuItem))){ // No I18N
				this._checkAndHideAllOpenedSubMenus(menuItem.closest(".zmenu")); // No I18N
				this._setLastOverlayedMenuItem(menuItem);
				this._clearMenuTimeout(); 
				menuItem.addClass("on-hover");
				var isDirectlyFromMouseEvent = event ? this._isDirectEvent(event) : false,
					base = this,
					showSubMenuAndTriggerEvent = function(){
						if(this.getData(menuItem).openOn !== "click" && menuItem.data("submenu") && isDirectlyFromMouseEvent){ // No I18N
							this._checkAndHideAllOpenedSubMenus(menuItem.closest(".zmenu"),true); // No I18N
							this._showSubmenu(menuItem,{"event":event},false,event); // No I18N
						}else{
							this.getData(this.element).constantLoc = undefined; // fix - Not clearing constantLoc creates problem in hoverOnMenuItem after menu is destroyed.
						}
					};
				if(menuItem.is(".zmenu--split")){ // No I18N
					window.setTimeout(function(){ // Hover delay for split menu item
						if(menuItem.hasClass("on-hover")){
							menuItem.find(".zmenu__icon.h-alignright").addClass("on-hover"); // No I18N
							showSubMenuAndTriggerEvent.bind(base)();		
						}
					},150);
				}else{
					showSubMenuAndTriggerEvent.bind(this)();
				}	
			}	
		},
		_showSubmenu: function(menuItem, options, showImmediately,eventObj){
			var base = this;
			if(!options.forElement){
				options.forElement = menuItem;
			}
			var f = function(){
				if(menuItem.is(":visible")){
					var submenu = base.getSubmenu(menuItem);
					options.direction = submenu.data("zmenu").options.submenuDirection ? submenu.data("zmenu").options.submenuDirection : base._DIRECTIONS.submenu; // No I18N
					base.show(options,submenu,false);
					base._saveMenuPoints(eventObj, menuItem); // Save Menu Points when the last menu chain is updated. ie. after show().
				}			
			};
			if(showImmediately){
				f();
			}else{
				base._menuTimeout = setTimeout(f, 100);
			}
		},
		_clearMenuTimeout: function(){
			if(this._menuTimeout){
				clearTimeout(this._menuTimeout);
				this._menuTimeout = undefined;
			}
		},
		_handleMouseOutOnMenuItems: function(menuItem,e){
			var isDirectlyFromMouseEvent = this._isDirectEvent(e);
			if(isDirectlyFromMouseEvent){
				if(!menuItem.parent().has(e.relatedTarget).length){
					if(menuItem.data("submenu") && this._isMenuOfMenuItemShown(menuItem)){
						return;
					}
					this._setLastOverlayedMenuItem(undefined);
				}			
				return;
			}
			var relatedEle = e.data;
			if(this._isMenuOfMenuItemShown(menuItem) && this._isElementInMenu(relatedEle, $(this.getSubmenu(menuItem)))){
				return;
			}
			this._clearMenuTimeout();
			menuItem.removeClass("on-hover"); // No I18N
			if(menuItem.is(".zmenu--split")){ // No I18N
				menuItem.find(".zmenu__icon.h-alignright").removeClass("on-hover"); // No I18N
			}
			if(menuItem.data("submenu")){ 
				var smenu = this.getSubmenu(menuItem);
				if(!smenu || this.getData(menuItem).openOn === "click"){
					return;
				}
				if(!this._isElementInMenu(relatedEle, $(smenu))){
					this.hide(smenu); // hiding the sub menu.
				}
			}
		},
		_isMenuOfMenuItemShown: function(menuItem){
			return (menuItem.data("submenu") && $(this.getSubmenu(menuItem)).is(":visible")); // No I18N
		},
		_isDirectEvent: function(e){
			return !!e.originalEvent;
		},
		_hideMenuItem: function(menuItem,show){
			var menuItem = (typeof menuItem === "string") ? $("#"+menuItem, this.element) : $(menuItem); // No I18N
			if(menuItem.length){
				menuItem[show ? 'show': 'hide'](); // No I18N
				menuItem.attr({"hidden": show ? false : true,"aria-hidden": show ? false : true}); // No I18N
			}
		},
		_checkMenuItem: function(menuItem,checked,data){
			if(data.itemType === "radio"){ // No I18N
				this._handleRadioChecked(menuItem,checked,data);
			}else if(data.itemType === "checkbox"){   // No I18N
				this._handleCheckboxChecked(menuItem,checked,data);
			}
		},
		_checkAndHideAllOpenedSubMenus: function(menu,hideAll,callback){

			var items = menu.find("li");
			for(var i = 0; i < items.length; i++){
				var item = $(items[i]),
					subMenu = this.getSubmenu(item),
					openOnClick = subMenu && this.getData(item).openOn == "click"; // No I18N
				if(subMenu && subMenu.is(":visible") && (!openOnClick || hideAll)){ // No I18N
					this._checkAndHideAllOpenedSubMenus(subMenu,hideAll,callback); // hiding all the submenus of submenu.
					if(subMenu.data("parent") && this.getData(subMenu.data("parent")).openOn === "click"){ // No I18N
						menu.zmenu("hide",subMenu,callback);  // No I18N
						// item.trigger("click"); // No I18N
					}else{
						item.trigger("mouseleave"); // No I18N
					}
				}
			}
		},
		_setLastOverlayedMenuItem: function(menuItem, noTrigger){
			if(this._lastOverlayedMenuItem && menuItem && (this._lastOverlayedMenuItem.get(0) === menuItem.get(0))){
				return;
			}
			if(this._lastOverlayedMenuItem && menuItem && menuItem.length && !this._lastOverlayedMenu.has(menuItem.get(0)).length){ // Split menu case
				var submenu = menuItem.data("submenu"); // No I18N
				if(!submenu || submenu[0] !== this._lastOverlayedMenu[0]){
					var par = this._lastOverlayedMenu.data('parent'); // No I18N
					if(par && par.parent().has(menuItem[0]).length){
						par.trigger("mouseleave", menuItem); // No I18N
					}
				}
			}
			if(this._lastOverlayedMenuItem && !noTrigger){
				this._lastOverlayedMenuItem.trigger("mouseleave", menuItem); // No I18N
			}
			this._lastOverlayedMenuItem = menuItem;
			if(menuItem && menuItem.length){
				var __menu = menuItem.closest(".zmenu"),  // No I18N
					currentMenu = this.element;
				this.updateLastMenuChain(__menu);			
				this.getData(currentMenu)._lastOverlayedMenuItem = menuItem;
				while(currentMenu.data("isSubMenu"))	// No I18N
				{
					var par = this.getData(currentMenu).parent;
					currentMenu = par.closest(".zmenu"); // No I18N
				}
				this.getData(currentMenu)._lastOverlayedMenuItem = menuItem;	
			}
		},
		_fixFloatingMenuWidth: function(menu){
			var options = menu.data("zmenu").options, // No I18N
				type = options.type,
				columns = options.columnCount || options.columns;
			if(!columns || columns === -1){
				var _menu = menu.closest('.zmenu'); // No I18N
				columns = (_menu.data("isExternalSubmenu")) ? (this.getData(_menu).columns ? this.getData(_menu).columns :  this.getData(_menu).columnCount ): this.options.columns; // No I18N
				if(!columns || columns === -1){
					return;
				}
			}
			var items = this._getAllMenuItems(menu, false, false, true),
				itemWidth = $(items[0]).outerWidth(true),
				floatingMenuWidth = 0;
	        	// floatingMenuWidth = itemWidth * columns;

			for (var i = 0; i < columns; i++) {    
				floatingMenuWidth += $(items[i]).outerWidth(true);  
			}
			menu.width(floatingMenuWidth); // setting width here in order to retrieve correct height before menu show.
			if((menu.data("height") <= menu.outerHeight()) || (menu.hasClass("h-scroll_y") || menu.find(".h-scroll_y").length)){ // No I18N
				floatingMenuWidth = floatingMenuWidth + this.scrollWidth;  // adding scroll bar width
				menu.width(floatingMenuWidth);
			}
	        menu.data('width', floatingMenuWidth);	// No I18N
	        if(this.getData(menu).search){
	        	menu.data('container').width(floatingMenuWidth).find('input').width(floatingMenuWidth); // No I18N
	        }
		},
        /*
            Below functions added for Docs backward compatibility
        */
        getItemCount: function(){
            return $(this.element).children('.zmenu__item').length;
        },
		/** 
			Removes the given menu items from a menu.
			@function menu.removeMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("removeMenuItems",["#menuItemId1","#menuItemId2",...]);
		*/
		removeMenuItems: function(menuItems){ // menuItems can be an array of id's or selector containing elements.
			var base = this, item;
			if(!menuItems){
				this.element.empty();
				if(this.element.is("div")){ // No I18N
                    if(this._pointerElement){
                        this.element.append(this._pointerElement);
                    }
					this.element.append($("<ul class='zmenu__list'>"));
				}
			}else{
				for(var i = 0;i < menuItems.length; i++){
					item = $(menuItems[i],this.element);
                    if(item.length){
                        base.removeMenuItem(item);
                    }
				}				
			}
		},
		/** 
			Removes the given menu item from a menu.
			@function menu.removeMenuItem
			@param {(String|Object)} menuItem - Identifier of the item or the item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("removeMenuItem","#menuItemId");
		*/
		removeMenuItem: function(menuItem){
			menuItem = $(menuItem,this.element); 
            if(!menuItem.length){
                return;
            }
			menuItem.remove();
			// Removing the group label if no element of that particular group is visible
			var groups = this.element.find("li.zmenu__title"), // No I18N
				base = this;
			for(var i = 0; i < groups.length; i++){
				var id = $(groups[i]).attr("id"),
					items = base.element.find("li[name="+id+"]:not(.zmenu__title):visible"); // No I18N
				if(!items.length){
					$(groups[i]).remove();
				}
			}
			if(this.element.find(".zmenu__footer") && !this.element.find(".zmenu__footer li").length){ // No I18N
				this.element.find(".zmenu__footer").remove(); // No I18N
			}
			this._fixMenuUI(this.element);
			this._resetMenuTextArray(this.element);
		},
		/** 
			Returns the menu item present in the specified index of the menu.
			@function menu.getMenuItemAt
			@param {Number} index - The index of the menu item to retrieve.
			@param {Object} [menu] - The menu which contains the menu item.
			@param {Boolean} [returnAsObject] - Specifies whether to return the menu item details as a JSON object or not.
			@example <caption> Example: </caption>
var item = $("menuElement").zmenu("getMenuItemAt",2);
		*/
		getMenuItemAt: function(index, menu, returnAsObj){
			menu = menu ? menu : this.element;
			if(returnAsObj){
				return this._getMenuItemAsObject(menu.find(".zmenu__item:eq("+index+")")); // No I18N
			}else{
				return menu.find(".zmenu__item:eq("+index+")"); // No I18N
			}
		},
		_getMenuItemAsObject: function(menuItem){
			var mdata = {},
				data = this.getData(menuItem),
				base = this;
			$(this._attrNames).each(function(i, name){
				var attrVal = menuItem.attr(name) || data[name];
				if(attrVal !== undefined){
					mdata[name] = attrVal;
				}
			});		
			for(var i in data){
				if($.inArray(i, base._attrNames) === -1 && i !== "menuItems"){ // No I18N
					if(data[i] !== undefined){
						mdata[i] = data[i];
					}
				}
			}
			if(mdata.itemType == "custom" || mdata.custom){ // No I18N
				mdata.innerHTML = menuItem[0].innerHTML;
			}else if(mdata.label === undefined){
				mdata.label = menuItem.find(".zmenu__text").length && menuItem.find(".zmenu__text").text();
			}
			mdata.text = undefined; // text is made undefined because text variable affects handlebars template rendering.
			if(menuItem.data("submenu")){
				var subMenu = this.getSubmenu(menuItem);
				if(subMenu){
					mdata.menuItems = this._getMenuAsObject(subMenu);
				}
			}
			return mdata;
		},
		_getMenuAsObject: function(menu){
			menu = menu ? menu : this.element;
			var base = this,menuArr = [],
				menuItems = menu.find("li"); // No I18N
			menuItems.each(function(index, menuItem){
					menuArr.push(base._getMenuItemAsObject($(menuItem)));
			});
			return menuArr;	
		},
		/** 
			Add the menu items whose details are given in the first argument and executes the callback function after adding the menu item into the menu.
			@function menu.addMenuItems
			@param {Array} menuItems - An array containing details of all the menu items to add.
			@param {Object} callbackFunction - Callback function to execute after adding each menu item.
			@example <caption> Example: </caption>
$("menuElement").zmenu("addMenuItems",[{"label":"Save As","icon":"icon-save-as"}], js-function);
		*/
		addMenuItems: function(menuItemArray,itemCallback, appendOnce){ // menuItem details are given as json object.
			var items = [],
				base = this,
				menuItemArr = Object.create(menuItemArray),
				menuItem = undefined, len = menuItemArr.length;
				for(var i = 0;i < len; i++){
					menuItem = base.addMenuItem(menuItemArr[i],itemCallback,true,appendOnce);
					items.push(menuItem);
				}
				if(appendOnce){
					this.element.find("ul.zmenu__list:first").append(items);
				}
			if(this.element.is(":visible") || this.options.renderInPlace){ // No I18N
				this._fixMenuUI(this.element);
				this._resetMenuTextArray(this.element);
			}
			return items;
		},
		/** 
			Adds the menu item using the details given in the first argument and executes the callback function after adding the menu item into the menu.
			@function menu.addMenuItem
			@param {Object} menuItemObject - Object containing details of the menu item.
			@param {Object} callbackFunction - Callback function to execute after adding the item.
			@example <caption> Example: </caption>
$("menuElement").zmenu("addMenuItem",{"label":"Save As","icon":"icon-save-as"}, js-function);
		*/
		addMenuItem: function(menuItemObj,itemCallback,isInternal,appendOnce){
			var menuItem = undefined,
				menu = $(this.element);
			if(typeof menuItemObj === "object"){ // No I18N
				menuItemObj = Object.create(menuItemObj);
			}
			if((menuItemObj.menu && menuItemObj.menu.length && typeof menuItemObj.menu === 'object')){   // No I18N
			// Adding submenu items - menuItemObj.menu will hold the submenu
				if(!$(menuItemObj.menu)[0].tagName === "UL"){ // No I18N
	                menu = $(menuItemObj.menu).children('ul'); // No I18N
	            }else{
			        menu = menuItemObj.menu;
	            }
	        }
			var container = menu[0].tagName === "UL" ? false : true; // No I18N
	        if(!hasHandlebars){ // For applications which doesnot use handlebars.
	        	menuItem = $(this._menuItemUsingElementCreation(menu,menuItemObj,itemCallback));
	        }else{
	      	  	menuItem = this._menuItemUsingTemplate(menu,menuItemObj,itemCallback);
	        }
	        if(!appendOnce){			
				var allItems = menu[0].querySelectorAll("li"); // No I18N
				if(typeof menuItemObj.index !== "undefined" && allItems.length){ // No I18N 
				// Menu is not empty and we are adding a single item at the specified position
					var insertAfter = false;
					if(menuItemObj.index >= allItems.length){
						menuItemObj.index = allItems.length-1;
						insertAfter = true;
					}
					if(menuItemObj.index < 0){
						menuItemObj.index = 0;
					}
					menuItem[insertAfter? "insertAfter": "insertBefore"]($(allItems[menuItemObj.index]));	// No I18N
				}else{
					container ? menu[0].querySelector("ul").appendChild(menuItem[0]) : menu[0].appendChild(menuItem[0]); // No I18N
				}
			}
	        if(typeof menuItemObj.menuItems !== 'object' && itemCallback && menuItemObj.mtype !== "separator"){ // No I18N
	        	itemCallback(menuItem,menuItemObj);	
			}
			var attrName = undefined;
			for(var i in this._attrNames){
				attrName = this._attrNames[i];
				if(menuItemObj[attrName]){
					menuItem[0].setAttribute(attrName, menuItemObj[attrName]+""); // No I18N
				}
			}
			var data = this.getData(menuItem);
			for(var i in menuItemObj){  // Placing Other attributes in data attribute
				if($.inArray(i, this._attrNames) === -1 && i !== "menuItems" && i !== "action"){ // No I18N
					if(menuItemObj[i]){
						data[i] = menuItemObj[i];
					}
				}
			}
			if(menuItemObj.action && typeof menuItemObj.action === "string" && menuItemObj.action.indexOf("URL:") !== -1){ // No I18N
				data.action = menuItemObj.action; 
			}
			if(menuItemObj.checked){
				menuItem.data('checked', menuItemObj.checked); // No I18N
				menuItem.attr({"data-checked":menuItemObj.checked,"aria-checked":menuItemObj.checked}); // No I18N
			}
			if(menuItemObj.hidden){
				menuItem.attr({"data-hidden":menuItemObj.hidden,"aria-hidden":menuItemObj.hidden}); // No I18N
			}
			if(menuItemObj.disabled){
				menuItem.attr({"data-disabled":menuItemObj.disabled,"aria-disabled":menuItemObj.disabled}); // No I18N
			}
			if(menuItemObj.customAttributes){
                if(menuItemObj.customAttributes.data){
                    var attrVal = menuItemObj.customAttributes.data;
                    if(typeof attrVal === "string"){ // No I18N
                        attrVal = JSON.parse(attrVal);
                    }
                    if(attrVal.appData){
                        this.getData(menuItem).appData = attrVal.appData;
                    }
                }
				menuItem.attr(menuItemObj.customAttributes);
			}
			if(!menuItemObj.internal && !hasHandlebars){ // Build the menu items of menu and not submenu items
				this._buildMenuItem(menuItem, menu);
			}else{
				menuItem.attr("role", this.ARIA_ROLES[menuItemObj.itemType ?  menuItemObj.itemType : "default"]); // No I18N
				if(menuItemObj.innerHTML){
					var content = menuItemObj.innerHTML.content,
						replaceArray = menuItemObj.innerHTML.replaceArray;
					if(content && replaceArray){
						for (var i = 0; i < replaceArray.length; i++) {
	                    	content = content.replace('{'+ i +'}',ZC.encodeHTML(replaceArray[i]));	// No I18N
	                	}
	                	menuItem.html(content);
					}else{
						if(this.options.isCustomHTMLEncoded || this.options.htmlSafe){
							menuItem.append(data.innerHTML); 
						}else{
							menuItem.html(ZC.encodeHTML(data.innerHTML));
						}
					}
					if(menuItemObj.hasSubmenu){
						this._createArrowIcon(menuItem);
					}
				}
				this._handleCheckedandDisabled(menuItem,menuItemObj);
			}
			var extAsso = menuItemObj.itemValue;
			if(extAsso){
				menuItem.data('itemValue',extAsso); // No I18N
			}
			var splitButtonData = menuItemObj.splitButtonData;
			if(splitButtonData){
				menuItem.data('relatedButton', splitButtonData); // No I18N
			}
			this.getData(menuItem).action = menuItemObj.action;
			if(menuItemObj.fixed || menuItemObj.isFooter){
				this._addFixedItem(menuItem,menu,data);
			}
			if(!isInternal){
				this._fixMenuUI(menu);
			}
			var className = menuItemObj.className ? menuItemObj.className : (menuItemObj.itemCssClass ? menuItemObj.itemCssClass : ""); // No I18N
			return menuItemObj.isFooter ? null : menuItem.addClass(className);
		},
		_menuItemUsingTemplate: function(menu,menuItemObj,itemCallback){
			menuItemObj.listClass = (menuItemObj.itemType === "itemgroup") ? "zmenu__title" : "zmenu__item"; // No I18N
			menuItemObj.icon = menuItemObj.SVGIconId ? menuItemObj.SVGIconId : (menuItemObj.iconClassName ? menuItemObj.iconClassName : menuItemObj.icon);
			if(menuItemObj.icon){
				if(menuItemObj.icon.indexOf("#") !== -1){ // No I18N
					menuItemObj.svgId = menuItemObj.icon.split(" ")[0]; // No I18N
					menuItemObj.svgClass = menuItemObj.icon.slice(svgId.length).trim();
				}
			}
			menuItemObj.isGrid = this._isGridMenu(menu);
			menuItemObj.isSeparator = (menuItemObj.itemType === "separator") ? true : false; // No I18N
			menuItemObj.isCustom = (menuItemObj.itemType === "custom" || menuItemObj.custom) ? true : false; // No I18N
			if(menuItemObj.itemType === "splitmenu"){ // No I18N
				menuItemObj.listClass += " zmenu--split"; // No I18N
			}
			if(typeof menuItemObj.menuId === "string"){ // No I18N
				menuItemObj.hasSubmenu = true;
				menuItemObj.submenuId = menuItemObj.menuId.substr(1);
			}else if(menuItemObj.menuItems && menuItemObj.menuItems.length){
				menuItemObj.submenuId = menuItemObj.id+"submenu"; // No I18N
				menuItemObj.submenu = $("<ul>").attr('id',menuItemObj.submenuId); // No I18N
				menuItemObj.hasSubmenu = true;
				menuItemObj.contentType = menuItemObj.contentType ? menuItemObj.contentType : "text"; // No I18N
				this.getData(menuItemObj.submenu).contentType = menuItemObj.contentType;		
				var base = this;
				$(menuItemObj.menuItems).each(function(i, _menuItemObj){ // menuItems contain array of menu items details.
					_menuItemObj.menu = menuItemObj.submenu;
					_menuItemObj.internal = true;
					base.addMenuItem(_menuItemObj, itemCallback, undefined);
				});
				menuItemObj.submenu.addClass("zmenu "+this._MENU_CONTENT_TYPES[menuItemObj.contentType ? menuItemObj.contentType : "text"]).zmenu({"usingJSON":true}); // No I18N
				$('body').append(menuItemObj.submenu);
			}
			var menuType = menu.data('zmenu') ? menu.data('zmenu').options.contentType : this.getData(menu).contentType;  // No I18N
			menuItemObj[menuType === "icon" ? "icon-only" : menuType] = menuType; // No I18N
			var menuItem = $(menuItemJsonTemplate(menuItemObj));
			if(menuItemObj.isGroup){
				menuItem.append(menuItemObj.submenu);
			}else if(menuItemObj.menuItems && menuItemObj.menuItems.length){
				menuItem.attr("data-menu-id",menuItemObj.submenuId); // No I18N
				menuItemObj.submenu.data("parent",menuItem); // No I18N
			}
			menuItemObj.label && menuItem.attr("data-label",menuItemObj.label); // No I18N
			this.getData(menuItem).parentMenu = menu;
			if(menuItemObj.icon && menuItemObj.icon.indexOf("text:") === 0 && this.options.contentType !== "text"){  // No I18N
				menuItem.find("i:first").removeClass(menuItemObj.icon).text(menuItemObj.icon.replace("text:","")); // No I18N
			}
			if(!menuItem.is(".zmenu__separator") && !menuItem.is(".zmenu__title")){ // No I18N 
				// Binding events for menu items
				this._addEventsForMenuItems(menuItem);
			}
			if(menuItemObj.action && typeof menuItemObj.action === "string" && menuItemObj.action.indexOf("URL:") !== -1){ // No I18N
				var menuItemContents = menuItem.html();
				menuItemObj.target = menuItemObj.target ? menuItemObj.target : "_blank"; // No I18N
				menuItem.empty().addClass("zmenu--withlink").append($("<a class='zmenu__link' href='"+menuItemObj.action.replace("URL:","")+"' target='"+menuItemObj.target+"'></a>")); // No I18N
				menuItem.find(".zmenu__link").append(menuItemContents);
			}
			if(menuItemObj.hasSubmenu){
				menuItem.data("submenu", menuItemObj.submenu ? menuItemObj.submenu : $(menuItemObj.submenuId) ); // No I18N 
			}
			return menuItem;
		},
		_menuItemUsingElementCreation: function(menu,menuItemObj,itemCallback){
			var menuItem = document.createElement("li");
			menuItemObj.itemType && menuItem.setAttribute("data-item-type",menuItemObj.itemType);
			menuItemObj.name && menuItem.setAttribute("name",menuItemObj.name);
			if(typeof menuItemObj.menuItems === "object"){ // No I18N
				var className = menuItemObj.className ? menuItemObj.className : ( menuItemObj.menuClass ?  menuItemObj.menuClass : ''),
					subMenu = $("<ul>").attr({"id":menuItemObj.id+"submenu","class":className}),
					base = this;
				// $("body").append(subMenu);
				menuItem.appendChild(subMenu[0]);
				// menuItem.data("menuId",menuItemObj.id+"submenu").attr("data-menu-id",menuItemObj.id+"submenu"); // No I18N
				subMenu.data("usingJSON",true); // No I18N
				this.getData(subMenu).contentType = menuItemObj.contentType ? menuItemObj.contentType : this.getData(menu).contentType;
				$(menuItemObj.menuItems).each(function(i, _menuItemObj){ // menuItems contain array of menu items details.
					_menuItemObj.menu = subMenu;
					base.addMenuItem(_menuItemObj, itemCallback, undefined);
				});
			}
			return menuItem;
		},
		/** 
			Returns a jQuery object containing the element visually representing the menu.
			This method does not accept any arguments.
			@function menu.getElement
			@return {Object} element.
			@example <caption> Example: </caption>
var element = $("menuElement").zmenu("getElement");
		*/
		/** 
			Removes the menu component functionality. This will return the element back to its pre-init state.
			This method does not accept any arguments.
			@function menu.destroy
			@example <caption> Example: </caption>
$("menuElement").zmenu("destroy");
		*/
		destroy: function(menu){	
			var menu = menu ? menu : this.element;
			this._clearMenuTimeout();
			this._uninitSliders(menu);
			this.hide();
			if(menu.data("usingJSON") ||  this.options.usingJSON){ // No I18N
				var submenuItems = menu.find("li[submenuitem=true]"); // No I18N
				for(var i = 0; i < submenuItems.length; i++){
					$(submenuItems[i]).data("submenu").zmenu("destroy"); // No I18N
					$(submenuItems[i]).data("submenu").remove(); 
				}
				menu.empty(); // Json menu contents are removed.
			}
            var componentName = this.componentName;
			menu.off("mouseup."+componentName+" keydown.run."+componentName).removeClass('zmenu zmenu--thumbnail '+this._MENU_CONTENT_TYPES[this.options.contentType] +' '+this.options.menuClass+' '+this.options.className).removeData('zmenu').data('initialized',false); // No I18N
			// Destroying separator items
			menu.find('.h-scroll_y').removeClass('h-scroll_y'); // No I18N
			// pointer element is retained in order to avoid pointer element creation each item. 
			if(menu.find(".zmenu__pointer").length){ // No I18N
				menu.find(".zmenu__pointer").remove();
				this._pointerElement = undefined;
			}
			var listEle = menu.find("li");
			for(var i = 0;i < listEle.length; i++){
				var item = listEle[i],
					itemData = this.getData(listEle[i]),
					text = $(item).find(".zmenu__text").text() || itemData.label, // No I18N
					isFixed = $(item).is(".is-fixed"); // No I18N
				$(item).removeData("metadata").empty().removeClass("zmenu__item zmenu__title zmenu__custom zmenu__separator is-disabled is-fixed is-selected").empty().removeAttr('data-label').off('mouseenter mouseleave'); // No I18N
				$(item).text(text);
				isFixed && $(item).appendTo(menu.find("ul:first")); // No I18N
			}
			menu.find(".zmenu__header,.zmenu__footer").remove(); // No I18N
			menu.removeData("fixedArea width height"); // No I18N
			this._menuTextArray = [];
		},
		updateLastMenuChain: function(menu, isExternal){
			this._lastOverlayedMenu = menu;
			if(!isExternal && menu[0] !== this.element[0]){
				var bmenu = this._traverseAndFindBaseMenu();
				bmenu.zmenu("updateLastMenuChain", menu, true); // No I18N
			}
		},
		_traverseAndFindBaseMenu: function(menu){
			var _menu = menu;
			menu = menu ? menu : this.element;
			// if(menu.data("isExternalSubmenu") && !retainParent){  // No I18N
			if(menu.data("parent")){  // No I18N
				var par = menu.data('parent'); // No I18N
				while(par){
					var _menu = par.closest(".zmenu"); // No I18N
					if(_menu.is(":visible")){  // No I18N
						menu = _menu;
						par = menu.data('parent'); // No I18N
					}else{
						par = false;
					}
				};
			}
			return menu.closest(".zmenu"); // No I18N
		},
		/* 
			Triggers the click event on the menu item, which is currently hovered. 
			@function menu.selectHoveredMenuItem
			@example <caption> Example: </caption>
$("menuElement").zmenu("selectHoveredMenuItem");
		*/
		selectHoveredMenuItem: function(){
			var item = this._lastOverlayedMenuItem;
			if(item){
                $(item).trigger("mouseup"); // Trigger method is used since event object is not present. Event object is mandatory in this case to use event.preventDefault and event.stopPropagation methods.
			}
			// this.hide(); Commented because hiding the menu can be prevented using itemclick event.
			return item ? item : false;
		},
		/** 
			Triggers the mouseover event on the menu item. 
			@function menu.hoverMenuItem
			@param {String} menuItem -  Specifies the id of the item or html element to hover.
			@example <caption> Example: </caption>
$("menuElement").zmenu("hoverMenuItem",$("selector"));
		*/
		hoverMenuItem: function(item){
            item = $(item);
            if(!item.length){
                this._lastOverlayedMenuItem && this._lastOverlayedMenuItem.removeClass("on-hover"); // No I18N
                this._lastOverlayedMenuItem = undefined;
                return;
            }
			this._handleMouseOverOnMenuItems($(item),{},true,false,true);
			// item.trigger("mouseover"); // No I18N
		},
		_getAllMenuItems: function(menu, excludeFloatingItems, excludeSubMenuItems, isOnlyVisible, excludeseparator){
			if(!menu || !menu.length){
				menu = this.element;
			}
			return menu.find('.zmenu__item:not([itemType=group], .zc-splitmenu-list-rht'+(excludeseparator? ', .zmenu__separator': '') +(excludeFloatingItems? ', .zmenu--thumbnail': '')+(excludeSubMenuItems? ', li ul[mtype=submenu] li': '')+')'+(isOnlyVisible? ':visible': ''));  // No I18N		
		},
		_handleCheckboxChecked: function(menuItem,checked,data){
			if(this._isGridMenu(menuItem, true)){ 
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else if((data.icon || data.iconClassName || data.SVGIconId) && this.options.contentType !== "text"){  // No I18N
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else{
				var iconEle = menuItem.find("i:first");
				if(iconEle.length){
                    this.options.checkedCheckboxItemIconClassName && iconEle.removeClass(this.options.checkedCheckboxItemIconClassName);
                    if(checked){
                        var svgData = {
                            props: {"class":"zmenu__tick","viewBox":"0 0 16 16","width":"100%","height":"100%"}, // No I18N
                            nodes:[{
                                "polygon": {"points":"14.6,2.4 4.9,12.1 1.2,8.5 0.4,9.3 4.9,13.8 15.4,3.2 "} // No I18N
                            }]
                        };
                        this._handleIcon(iconEle, svgData, "checkedcheckbox", this.options.checkedCheckboxItemIconClassName, this.options.checkedCheckboxItemSVGIconId);
                    }else{
                       iconEle.find("svg").remove(); // No I18N
                    }
				}
			}
			menuItem.prop("checked",checked).attr({"checked":checked,"aria-checked":checked}); // No I18N
		},
		_handleRadioChecked: function(menuItem,checked,data){
			var name = $(menuItem).attr("name"), // No I18N
			    menu = menuItem.closest('ul'), // No I18N
			    base = this,metaInfo = undefined,
			    radios = menu.find("li[role=menuitemradio][name="+name+"]"); // No I18N
		    if(checked){
				radios = radios.filter(function(){
					return $(this).prop('checked');
				});
				radios.each(function(i, radio){
					radio = $(radio);
					metaInfo = base.getData(radio);
					base._fixRadioIcon(radio, false, metaInfo);
				});
		    }		
			base._fixRadioIcon(menuItem, checked, data);
		},
		_fixRadioIcon: function(menuItem,checked,data){
			var isGrid = this._isGridMenu(menuItem, true);
			if(isGrid){
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else if(data.icon || data.iconClassName || data.SVGIconId){
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else{
                var iconEle = menuItem.find("i:first");
                if(iconEle.length){
                    this.options.checkedRadioItemIconClassName && iconEle.removeClass(this.options.checkedRadioItemIconClassName);
                    if(checked){
                        var svgData = {
                            props: {"class":"zmenu__radio","viewBox":"0 0 48 48","width":"100%","height":"100%"}, // No I18N
                            nodes:[{
                               "circle":{"cx":"24.5","cy":"24.5","r":"15.5"} // No I18N
                            }]
                        };
                        this._handleIcon(iconEle, svgData, "checkedradio", this.options.checkedRadioItemIconClassName, this.options.checkedRadioItemSVGIconId);
                    }else{
                       iconEle.find("svg").remove(); // No I18N
                    }
                }
			}
			menuItem.prop("checked",checked).attr({"checked":checked,"aria-checked":checked}); // No I18N
		},
		_initRuntimeEvents: function(){
			var base = this, componentName = this.componentName;
			this.element.off("keydown.run."+componentName).on("keydown.run."+componentName,function(event){ // No I18N
				base.handleKeyCode(event.keyCode,event,undefined);
				if((!event.ctrlKey && !event.metaKey && !event.shiftKey && !event.altKey) || (event.keyCode === ZC.keyCode.UP || event.keyCode === ZC.keyCode.DOWN || event.keyCode === ZC.keyCode.LEFT || event.keyCode === ZC.keyCode.RIGHT)){
					// checking whether the key pressed is a short cut key or not. Shortcutkey events should not be prevented. 
					event.preventDefault();
					event.stopPropagation();
				}		
			});
		},
		handleKeyCode: function(code,event,showSubMenu){
			var _isFloatingMenu = this._isGridMenu();
			event = event ? event : {};
			switch(code){
				case ZC.keyCode.UP:
						if(event.altKey || event.ctrlKey || event.metaKey){ // page up
							var item = this._lastOverlayedMenu.find("li:visible:not('.zmenu__title,.is-disabled'):first"); // No I18N
							this.hoverMenuItem(item,true);
						}else{
							var nextItem = this._getNextMenuItem(true);
							if(nextItem.length){
								this._scrollToView(nextItem,nextItem.closest(".zmenu").is("ul") ? nextItem.closest(".zmenu") : nextItem.closest(".zmenu").find("ul:first") ); // No I18N
								nextItem.trigger('mouseover'); // No I18N
							}
						}					
					break;
				case ZC.keyCode.DOWN:
						if(event.altKey || event.ctrlKey || event.metaKey){ // page down
							var item = this._lastOverlayedMenu.find("li:visible:not('.zmenu__title,.is-disabled'):last"); // No I18N
							this.hoverMenuItem(item,true);
						}else{
							var nextItem = this._getNextMenuItem();
							if(nextItem.length){
								this._scrollToView(nextItem,nextItem.closest(".zmenu").is("ul") ? nextItem.closest(".zmenu") : nextItem.closest(".zmenu").find("ul:first")); // No I18N
								nextItem.trigger('mouseover'); // No I18N
							}
						}
						break;
				case ZC.keyCode.RIGHT:
					if(this._lastOverlayedMenuItem !== undefined){
						var subMenu = this.getSubmenu(this._lastOverlayedMenuItem);
						var menu = this._lastOverlayedMenuItem.closest(".zmenu"); // No I18N
						if((_isFloatingMenu || !subMenu.length && menu.hasClass("zmenu--thumbnail")) && !showSubMenu){ // No I18N - Traversing floating menu items which doesn't have any submenus
							var f = undefined,
								item = this._getNextMenuItem(f, f, f, code);
							this._scrollToView(item,item.closest(".zmenu").is("ul") ? item.closest(".zmenu") : item.closest(".zmenu").find("ul:first") ); // No I18N
							item.trigger("mouseover"); // No I18N
							break;
						}
						if(this._lastOverlayedMenuItem.is(".is-disabled")){ // No I18N
							return;
						}
						if(subMenu && subMenu.length){
							var menu = this.getSubmenu(this._lastOverlayedMenuItem); // Submenu was already extracted -- check again
							if(!menu){
								return;
							}
							this.show({"forElement": this._lastOverlayedMenuItem}, menu); // No I18N
							this._lastOverlayedMenuItem = undefined;
							this._getNextMenuItem(false, true).trigger("mouseover"); // No I18N
						}
					}else if(_isFloatingMenu){
						this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):not(':hidden'):first").trigger("mouseover"); // No I18N
					}
					break;
				case ZC.keyCode.LEFT:
					if(this._lastOverlayedMenuItem !== undefined){ 
						// Fix for split menu left click
						menu = this._lastOverlayedMenuItem.closest(".zmenu"); // No I18N
						// Fix to hide the submenu of floating menu item.
						var parentItem = this._lastOverlayedMenu.data('parent'),  // No I18N
							subMenuShown = parentItem && this._isMenuOfMenuItemShown(parentItem),
							firstMenuItem = menu.find(":visible.zmenu__item").not(".zmenu__separator,.is-disabled,.zmenu__title").filter(":first"), // No I18N
							// _isFirstItem = this._lastOverlayedMenuItem.index() === 0,
                            _isFirstItem = firstMenuItem[0] == this._lastOverlayedMenuItem[0],
							currentMenuFloating = this._isGridMenu(menu);
						if(currentMenuFloating && !_isFirstItem){
							if(!subMenuShown || !_isFirstItem){ 
								var item = this._getNextMenuItem(true, undefined, undefined, code);				
								this._scrollToView(item,item.closest(".zmenu").is("ul") ? item.closest(".zmenu") : item.closest(".zmenu").find("ul:first") ); // No I18N
								item.trigger("mouseover"); // No I18N
								break;
							}
						} 
						if(!this._isMenuMainMenu(this._lastOverlayedMenu)){ // Pressing left from submenu will hide the submenu.
							this._hideLastShownMenu();
						}
					}else if(_isFloatingMenu){
						this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):not(':hidden'):last").trigger("mouseover"); // No I18N
					}
					break;
				case ZC.keyCode.ESCAPE:
					if(!this.options.renderInPlace){
						if(!this._isMenuMainMenu(this._lastOverlayedMenu)){
							this._hideLastShownMenu();
						}else{
							this.hide();
						}
					}
					break;
				case ZC.keyCode.ENTER:
				case ZC.keyCode.SPACE:
					if(this._lastOverlayedMenuItem && this._lastOverlayedMenuItem.data("submenu") && !this._lastOverlayedMenuItem.hasClass("zmenu--split")){ // No I18N
						this.handleKeyCode(ZC.keyCode.RIGHT,event,true);
					}else{	
						if(this._lastOverlayedMenuItem){
                            if(this._lastOverlayedMenuItem.is(".zmenu--withlink")){ // No I18N
                                $(this._lastOverlayedMenuItem).find(".zmenu__link")[0].click(); // No I18N
                            }else{
                                event.target = $(this._lastOverlayedMenuItem); // target is modified since actual target will be menu element.
                                this._handleMenuItemClick(event);
                            }
  						}else{
           					this.hide();
  						}
						// if(this.selectHoveredMenuItem() && !this._lastOverlayedMenuItem  || (this._lastOverlayedMenuItem && !this._lastOverlayedMenuItem.is("[data-item-type='checkbox']"))){ // No I18N
						// 	// checkbox menu should not be closed.
						// 	this.hide();
						// }
					}
					break;
				case ZC.keyCode.PAGE_UP:   var item = this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):first"); // No I18N
													this.hoverMenuItem(item,true);
													break;
				case ZC.keyCode.PAGE_DOWN: var item = this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):last"); // No I18N
													this.hoverMenuItem(item,true);
												    break;
				default:
		            var alphaNumericKey = String.fromCharCode(code);
		            if(this._menuTextArray.length && this.options.handleAccessKeyNavigation){
						this._callMenuitemSearch(alphaNumericKey,(new Date()).getTime());
					}
				break;
			}
		},
		_isDisabled : function(menuItem){
			var disabled = menuItem.attr("disabled")? menuItem.attr("disabled"): false; // No I18N
			return disabled;
		},
		_isMenuMainMenu : function(menu){
			return ($(menu).get(0) === this.element.get(0));
		},
		_getNextFloatingMenuItem : function(currentItemPos, items, isPrev, compareTop, isReverse){
			var next = undefined,
				key = compareTop ? "top" : "left", // No I18N
				total = items.length;
			$.each(items, function(i, item){
				item = items[isReverse ? total-i-1 : i];
				var ipos = $(item).position();
				if(ipos[key] === currentItemPos[key]){
					next = $(item);
					return false;
				}else{
					// Navigation to next row items (example: open office application)
					if(key === "top"){ // No I18N
						next = $(item);
						return false;
					}
				}
			});
			return next;
		},
		_hideLastShownMenu: function(){
			var nextItem = this._lastOverlayedMenu.data('parent'); // No I18N
			this._lastOverlayedMenuItem = nextItem;
			this.hide(this._lastOverlayedMenu);
			var __menu = nextItem.closest(".zmenu"); // No I18N
			this.updateLastMenuChain(__menu);
			this.getData(this.element)._lastOverlayedMenuItem = nextItem;
		},
		_resetMenuTextArray: function(menu){
			this._menuTextArray = [];
	        this._prevSearchKey = ''; // No I18N
	        this._prevKeydownTime = ''; // No I18N
	        this._prevSearchIndex = ''; // No I18N
	        this._prevSearchTimer = undefined; 
			var menuItemSelector = ".zmenu__item:visible:not(.zmenu__title,.is-disabled,.zmenu__separator)";	// No I18N
			if(this.options.renderInPlace){
				menuItemSelector = ".zmenu__item:not(.zmenu__title,.is-disabled,.zmenu__separator)"; 	// No I18N
			}
			var menuItems = menu.find(menuItemSelector);
			for (var i = 0; i < menuItems.length; i++) {
				this._menuTextArray[this._menuTextArray.length] = {"text": $(menuItems[i]).find(".zmenu__text").text() || "","menuitem":$(menuItems[i])}; // No I18N
			}
		},
		_callMenuitemSearch: function(alphaNumericKey,currentTime){
			var sameKeySearch = false;
		    if(this._prevKeydownTime !== "" && (currentTime-this._prevKeydownTime)>700){ // No I18N
		    	sameKeySearch = (this._prevSearchKey == alphaNumericKey) ? true :false;
		        this._prevSearchKey = alphaNumericKey;
		    }
		    else{
		        this._prevSearchKey =  this._prevSearchKey + alphaNumericKey;
		    }
			this._prevKeydownTime = currentTime;    
		   	this._menuitemSearch(sameKeySearch);
		},
		_menuitemSearch: function(sameKeySearch){
			var regExp = new RegExp("^"+(this._prevSearchKey).replace(/[|()\[{.+*?$\\]/g, "\\$0"),"i");	// No I18N 
			// i is used to ignore case
			var searchFromIndex = (sameKeySearch && this._prevSearchIndex < this._menuTextArray.length) ? this._prevSearchIndex + 1 : 0 ;
		    searchFromIndex = (searchFromIndex > 0 && searchFromIndex < this._menuTextArray.length) ? searchFromIndex : 0 ;
	        for(var x = searchFromIndex; x < this._menuTextArray.length; x++){
				if((this._menuTextArray[x].text).match(regExp)){                
					this._prevSearchIndex = x;
					var checkNextMatch = false;
					for(var y = ( x+1 ); y < this._menuTextArray.length; y++){
						if((this._menuTextArray[y].text).match(regExp)){ 
							checkNextMatch = true;
						}
				  	}
				  	if(!checkNextMatch){
				  		 this._prevSearchIndex = -1;
				  	}
				  	this._handleMouseOverOnMenuItems($(this._menuTextArray[x].menuitem),{},true,true);
				  	return;
				}
		  	}
		},
		_getNextMenuItem: function(isPrev, noTrigger, prevItem, keyCode){
			var menu = (this._lastOverlayedMenu && this._lastOverlayedMenu.length !== 0) ? this._lastOverlayedMenu: this.element,
				_isFloatingMenu = this._isGridMenu(menu),	
				item = undefined,
				actualMenu = menu.is("ul") ? menu : menu.find("ul:first"); // No I18N
			prevItem = (prevItem !== undefined)? prevItem : this._lastOverlayedMenuItem;
			if(!prevItem || !prevItem.length){
				if(menu.find(".zmenu__footer").length && !this._lastOverlayedMenuItem){
					item = menu.find(".zmenu__item:visible:not(.zmenu__title,.is-disabled,.zmenu__separator):"+(isPrev? "last": "first")); // No I18N
				}else{
					item = actualMenu.find(".zmenu__item:visible:not(.zmenu__title,.is-disabled,.zmenu__separator):"+(isPrev? "last": "first")); // No I18N
				}
			}else{
				item = prevItem[isPrev? "prevAll" : "nextAll"](".zmenu__item:visible").not(".zmenu__separator,.is-disabled,.zmenu__title"); // No I18N	
				if(_isFloatingMenu){
					var prevPos = prevItem.position(),
						compareTop = (keyCode === ZC.keyCode.LEFT || keyCode === ZC.keyCode.RIGHT);
					item = this._getNextFloatingMenuItem(prevPos, item, isPrev, compareTop);
					if(!item){ // Reached end of row or column in menu
						if(!(menu.find(".zmenu__footer").length && compareTop)){ // No I18N
							item = prevItem[!isPrev? "prevAll" : "nextAll"](".zmenu__item:not(.zmenu__separator,.is-disabled,.zmenu__title):visible"); // No I18N
							prevPos[compareTop ? "left" : "top"] = 0; // No I18N
							item = this._getNextFloatingMenuItem(prevPos, item, isPrev, compareTop, true);	
						}					
					}
				}else{
					item = $(item[0]);
				}
				if(!item || !item.length){
					if(menu.find(".zmenu__footer").length){ // No I18N
						if(!prevItem.closest(".zmenu__footer").length){ // No I18N
							item = menu.find(".zmenu__footer").find(".zmenu__item:visible")[isPrev ? "last" : "first"](); // No I18N
						}else if(prevItem[0] === menu.find(".zmenu__footer").find(".zmenu__item:visible").first()[0]){ // No I18N
							// this._lastOverlayedMenuItem = undefined;
							prevItem.removeClass("on-hover"); // No I18N
							item = this._getNextMenuItem(isPrev, noTrigger, false);
						}else{
							item = this._getNextMenuItem(isPrev, noTrigger, false);
						}
					}else{
						item = this._getNextMenuItem(isPrev, noTrigger, false);
					}
				}
			}
			this._setLastOverlayedMenuItem(item, noTrigger);
			return this._lastOverlayedMenuItem;
		}, 
		_setFocus: function(){
			var base = this;
			var f = function(){
				base.element.focus();
				if(base.element.find(".zmenu__textbox").length){ // No I18N
					base.element.find(".zmenu__textbox").focus(); // No I18N
				}
			};
			if(this._triggerEvent("ONBEFOREFOCUS", {},this.element)){ // No I18N
				setTimeout(f, 100);
			}
		},
		refresh: function(menu){
			menu = menu? menu: this.element;
			this._fixMenuItemsWidth(menu);
		},
		_fixMenuItemsWidth: function(menu, type, referenceMenu){
			if(!menu){
				return;
			}
			type = !type ? this.getMenuType(menu) : type;
			if(this._isTypeFloating(type)){
				return this._fixFloatingMenuWidth(menu);
			}
		},
		getMenuType: function(menu){
			if(menu && menu.data("isExternalSubmenu")){ // No I18N
				return menu.zmenu("getMenuType"); // No I18N
			}else{
				menu = !menu ? this.element : menu;
				var type = this._getMenuTypeFromData(menu);
				if(!type){	
					type = this.options.type;
				}
				return type;
			}
		},
		_getMenuTypeFromData: function(menu){
			var mdata = this.getData(menu);
			return mdata.componentOptions ? mdata.componentOptions.type : mdata.type; 
		},
		_isTypeFloating: function(type, menu){
			if((type && type === "grid") || (menu && (menu.hasClass('zmenu--thumbnail')))){ // No I18N
				return true;
			}else{
				return false;
			}
		},
		_fixMenuUI: function(menu, options, isSubMenu){
			if(!menu.is(":visible")){  // No I18N
				return;
			}
			this._fixMenuHeight(menu,options);
		},
		_fixMenuHeight: function(menu, options){
			var menuHeight = "auto",  // No I18N
				actualMenu = menu.is("ul") ? undefined : menu.find("ul:first");  // No I18N
			menu.height(menuHeight);
			actualMenu && actualMenu.height(menuHeight);
			menu.removeClass("h-scroll_y"); // No I18N
			var positionAlterable = this.options.direction === "auto" || this.options.direction === "after-pointer" || this.options.direction === "at-cursor", // No I18N
				height = menu.outerHeight(),
				top = menu.css('top').split('px')[0] * 1 - $(window).scrollTop(), // No I18N
				windowHeight = ZC.windowObject ? ZC.windowObject.height : $(window).height(), // window object stored on the ZC is undefined sometimes while trying to show the menu on load events.So included $(window).height()
				openingDirection = menu.data("direction"),  // No I18N
				limit = (openingDirection && openingDirection.indexOf("bottom") === 0 || openingDirection && openingDirection.indexOf("top") === 0 ) ? (options && !options.isRightClick ? options.availableHeight : windowHeight ) : windowHeight, // No I18N
				space = limit - (height),
				top = top ? top : 0;
			if(top + height >= limit && (space < 0) || !positionAlterable){
				menuHeight = (options && options.availableHeight && !options.isRightClick) ? options.availableHeight : (windowHeight - top);
			}
			if(menu.data('height')){ // No I18N
				if(options && options.availableHeight){
					menuHeight = (parseInt(menu.data('height')) >= options.availableHeight) ? options.availableHeight : parseInt(menu.data('height')); // No I18N
				}else{
					menuHeight = parseInt(menu.data('height')); // No I18N
				}
			}else if(space > 0 && positionAlterable){
				menuHeight = height;
			}
			// if(menu.is("div")){
			// 	var headerFooter = menu.find(".zmenu__header").outerHeight() + menu.find(".zmenu__footer").outerHeight();
			// 	menu.height(menuHeight);
			// 	menuHeight = menuHeight - headerFooter;
			// 	menu = menu.find("ul:first");
			// }
			var scrollWidth = this._findScrollWidth(),
				menudata = this.getData(menu);
			if(height > menuHeight){
				// menu.height(menuHeight);
				menu.outerHeight(menuHeight);
				if(actualMenu){
					var headerHeight = menu.find(".zmenu__header").length ? (parseInt(menu.find(".zmenu__header").outerHeight(true)) + parseInt(menu.find(".zmenu__header").css("margin-top"))) + parseInt(menu.find(".zmenu__header").css("margin-bottom")) : parseInt(menu.css("padding-top")), // No I18N
						footerHeight = menu.find(".zmenu__footer").length ? (parseInt(menu.find(".zmenu__footer").outerHeight(true)) + parseInt(menu.find(".zmenu__footer").css("margin-top"))) + parseInt(menu.find(".zmenu__footer").css("margin-bottom")): parseInt(menu.css("padding-bottom")),// No I18N
						pointerHeight = menu.find(".zmenu__pointer").length ? parseInt(menu.find(".zmenu__pointer").outerHeight(true)) : 0, // No I18N
						remaining =  headerHeight + footerHeight + pointerHeight; 
					actualMenu.innerHeight(menuHeight - remaining);
					if(this.options.scroll === "slider"){
						this._initSliders(menu);
						actualMenu.addClass("h-overflowhidden");
					}else{
						actualMenu.addClass("h-scroll_y");
					}
					this.options.scroll === "slider" && this._initSliders(menu);  // No I18N
					if(remaining !== 0){
						// menu.height(menuHeight);
						menu.outerHeight(menuHeight);
					}
				}else{
					if(this.options.scroll === "slider"){
						this._uninitSliders(menu);
						this._initSliders(menu);
						menu.addClass("h-overflowhidden");
					}else{
						this._uninitSliders(menu);
	                	menu.addClass("h-scroll_y"); // No I18N
					}
				}
			}else{
				menu.removeClass("h-scroll_y"); // No I18N
			}
		},
		_findScrollWidth: function(){
			var scrollEle = document.createElement("div"),
				styleProps = "width:100px;height:100px;overflow:scroll;";  	 // No I18N
			document.querySelector("body").appendChild(scrollEle);	 // No I18N
			scrollEle.style.cssText = styleProps; 
			var scrollWidth = $(scrollEle).width() - scrollEle.scrollWidth;
			document.querySelector("body").removeChild(scrollEle);	 // No I18N
			return scrollWidth;
		},
		_initSliders: function(menu){
			this._createSlider(menu);
			this._createSlider(menu,true); // create bottom slider.
		},
		_uninitSliders: function(menu){
			menu = menu.is("div") ? menu.find("ul:first") : menu; // No I18N
			var data = this.getData(menu);
			if(data && data.sliders){
				if(data.sliders.top){
					data.sliders.top.remove();
				}
				if(data.sliders.down){
					data.sliders.down.remove();
				}
				data.sliders = null;
			}
			menu.off("mousewheel");	 // No I18N
		},
	/*	Case 1: For menu containing div element as parent without header and footer cases.
		_createSlider: function(menu,isTop){
			var direction = isTop ? 'top': 'bottom',  // No I18N
				data = this.getData(menu), base = this;
			menu = menu.is("ul") ? menu : menu.find("ul");  // No I18N
			if(!data.sliders){
				data.sliders = {};
			}
			var slider = data.sliders[direction];
			if(slider){
				slider.remove();
			}
			slider = $('<li>').addClass('zmenu__scroll zmenu__scroll-'+direction);
			data.sliders[direction] = slider;
			slider.hover(function(e){
				base._startScrolling(menu, isTop);
			}, function(e){
				base._stopScrolling(menu);
			}).hide();
	
			slider.click(function(e){
				var top = menu.scrollTop();
				base._stopScrolling(menu);
				menu.scrollTop(top + (isTop ? -1: 1) * 100);
				base._startScrolling(menu, isTop);
				return false;
			});
	
			var offset = menu.position(),
				width = menu.outerWidth()-2,
				buffer = isTop ? 0 : (menu.height() - 6);
				if(!isTop){
					slider.css({
						'top': offset.top+1+buffer  // No I18N
					}).fadeTo(0, 1);
				}
			var sliderTop = (isTop)? '-3px': '-1px',  // No I18N
				button = $('<button class="zbutton"></button>').css({'top': sliderTop}),
				arrow = (direction === 'top') ? 'n' : 's',   // No I18N
				iconSpan = $('<i class="zbutton__icon zicon--arrow h-floatright arrow-direction-'+arrow+'"></i>');
			$(iconSpan).appendTo($(button));
			$(button).appendTo(slider);
			// slider[isTop ? "insertBefore" : "insertAfter"](menu);
			slider[isTop ? "prependTo" : "appendTo"](menu);   // No I18N
			isTop && slider.hide();
			return slider;
		}, */
		/* Case 2: For Menu which doesn't contain div element as parent. */
		_createSlider: function(menu,isTop){
			menu = menu.is("div") ? menu.find("ul:first"): menu; // No I18N
			var direction = isTop ? 'top': 'down',  // No I18N
				data = this.getData(menu), base = this;
			if(!data.sliders){
				data.sliders = {};
			}
			var slider = data.sliders[direction];
			if(slider){
				slider.remove();
			}
			slider = $('<div>').addClass('zmenu--scroller zmenu--scroller'+direction);
			data.sliders[direction] = slider;
	
			var base = this;
			slider.hover(function(e){
				base._startScrolling(menu, isTop);
			}, function(e){
				base._stopScrolling(menu);
			}).hide();
			slider.click(function(e){
				var top = menu.scrollTop();
				base._stopScrolling(menu);
				menu.scrollTop(top + (isTop ? -1 :  1));
				base._startScrolling(menu, isTop);
				return false;
			});
			/* Binding mousewheel event for the menu to scroll the menu using mouse eventhough sliders are present. */
            var componentName = this.componentName;
			menu.off("mousewheel."+componentName).on("mousewheel."+componentName,function(event){ // No I18N
				var top = menu.scrollTop();
				if(event.originalEvent.wheelDelta > 0){
					menu.scrollTop(top-6);
				}else{
					menu.scrollTop(top+6);
				}
				base._displayScrollElements(menu);
				event.stopPropagation();
				event.preventDefault();
			});
			var	width = menu.width();
			slider.css({"width":width}); // No I18N
			if(!isTop){
				slider.show();
			}
			var	arrow = (direction === 'top') ? 'n' : 's',   // No I18N
				iconSpan = $('<i class="zmenu__icon"></i>'),svgData = undefined;
			if(direction === "top"){ // No I18N
                svgData = {
                    props: {"viewBox":"0 0 16 16","width":"100%","height":"100%"}, // No I18N
                    nodes: [{
                        "path":{"d":"M0,12l8-8l8,8H0z","class":"st0"} // No I18N
                    }]
                };
            }else{
                svgData = {
                    props: {"viewBox":"0 0 16 16","width":"100%","height":"100%"}, // No I18N
                    nodes: [{
                        "path":{"d":"M16,4l-8,8L0,4H16z","class":"st0"} // No I18N
                    }]
                };
            }
            this._handleIcon(iconSpan, svgData, isTop ? "topscroller" : "bottomscroller", isTop ? this.options.scrollUpArrowIconClassName : this.options.scrollDownArrowIconClassName, isTop ? this.options.scrollUpArrowSVGIconId : this.options.scrollDownArrowSVGIconId );
			$(iconSpan).appendTo(slider);
			if(menu.hasClass("zmenu__list")){
				slider.appendTo(menu.parent()); 
				var topValue = menu.height() - slider.outerHeight(true)/2;
				if(!menu.parent().find(".zmenu__footer").length){
					// footer element's margin top is set as 5
					if(this._pointerElement){
						topValue = topValue + Math.ceil(this._pointerElement.outerHeight(true)/2) + parseInt(menu.parent().css("border-bottom-width"));  // No I18N
					}else{
						// without pointer bottom border should be taken into account.
						topValue = topValue - parseInt(menu.parent().css("border-bottom-width")) - parseInt(menu.parent().css("border-top-width"));  // No I18N
					}
				}
				slider.css({"left":"0px","top": isTop ? "0px" : topValue  });  // No I18N	
			}else{
				slider.insertAfter(menu);
				var offset = menu.position(),
				buffer = isTop ? 1 : menu.outerHeight() - slider.outerHeight(true) - 1;
				slider.css({
					'top': offset.top+buffer, // No I18N
					'left': offset.left+1 // No I18N
				});
			}
			return slider;
		},
		_displayScrollElements: function(menu){
			var hasParent = menu.hasClass("zmenu") ? false : true; // No I18N
			// var scrollElement = this.getData(menu.parent().is(".zmenu") ? menu.parent() : menu ).sliders,   // No I18N
			var scrollElement = this.getData( menu ).sliders,  
			// var scrollElement = this.getData( hasParent ? menu.closest(".zmenu") : menu ).sliders,   // No I18N
				scrollTop = menu.scrollTop();
			scrollElement.top[(scrollTop === 0) ? 'hide': 'show']();   // No I18N
			scrollElement.down[(menu[0].scrollHeight-scrollTop <= menu.outerHeight())? 'hide': 'show']();   // No I18N
		},
		_startScrolling: function(menu, isTop){
			var stop = isTop ? 0 : (menu[0].scrollHeight - menu.innerHeight()),
				base = this,
				distance = menu.scrollTop() - stop,
				duration = Math.abs(distance*1000/50); //1000 ms for 50px
			menu.animate({scrollTop: stop}, {
				'duration' : duration,   // No I18N
				'easing'   : 'linear',   // No I18N
				complete   : function(){
					base._displayScrollElements(menu);
				}
			});
		},
		_stopScrolling: function(menu){
			menu.stop();
			this._displayScrollElements(menu);
		},
		_hideAllSubMenus: function(menu,callback){
			var menu = this._traverseAndFindBaseMenu(menu);
			if(menu[0] != this.element[0]){
				 menu.zmenu("hide",undefined,callback); // No I18N
			}else{
				this._checkAndHideAllOpenedSubMenus(menu,true,callback);
			}
		},
		_destroyRuntimeEvents: function(){
			$(this.element).off(".run."+this.componentName); 
		},
		_triggerEvent: function(type, orgEvent, menu, menuItem, dataObj){
			orgEvent = orgEvent ? orgEvent : {};
			menuItem = menuItem ? menuItem : $(orgEvent.target).closest(".zmenu__item");
			var isBase = !menu || (menu.get(0) == this.element.get(0));
			var data = {
				"originalEvent": orgEvent,	// No I18N
                "target": menu.data("forElement"), // No I18N
				"forElement": menu.data("forElement"), // No I18N
				"offset": menu.offset(),  // No I18N
				"menu": menu, // No I18N
				"isBaseMenu": isBase, // No I18N
				"menuitem":menuItem, // No I18N
				"data": menuItem.length ? this._getMenuItemAsObject(menuItem): undefined,  // No I18N
				"itemValue": menuItem.length && menuItem.data("itemValue") // No I18N
			};
			if(dataObj){
				data = $.extend({}, data, dataObj);
			}
			if(menu.data("parent") && type.indexOf("SUBMENU") !== -1){  // No I18N 
				// if external submenu has menu events binded on it
				menu.zmenu("triggerEvent",type.replace("SUBMENU", ""), orgEvent, data);	// No I18N
				var parentMenu = menu.data('parent').closest(".zmenu"); // No I18N
				return parentMenu.zmenu("triggerEvent",type,orgEvent,data); // No I18N
			}
			return this._dispatchEvent(type,orgEvent,data, menu);
		},
		triggerEvent: function(type, orgEvent, data){
			return this._dispatchEvent(type,orgEvent,data);
		},
		/* Computing the slope of given points */
		_slope: function(a,b){
			return (b.y - a.y) / (b.x - a.x);
		},
		/* Computing the triangular region for slanted mouseover on menu items */
		_saveMenuPoints: function(e, menuItem){
			var metaData = this.getData(this.element);
			// metaData.constantLoc = {x:e.pageX, y:e.pageY}; // Current mouse position
			var subMenu = this._lastOverlayedMenu,
				baseMenuOffset = this.element.offset(),
				subMenuOffset = subMenu.offset();
			metaData.constantLoc = {x:baseMenuOffset.left, y:baseMenuOffset.top+menuItem.position().top}; // Current mouse position
			if(baseMenuOffset.left >= subMenuOffset.left){ // Properties of submenu to decide decreasing corner and increasing corner
				var upperRight = {
					x:subMenuOffset.left + subMenu.outerWidth(),
					y:subMenuOffset.top
				};
				var lowerRight = {
					x:subMenuOffset.left + subMenu.outerWidth(),
					y:subMenuOffset.top + subMenu.outerHeight()
				};
				this.increasingCorner = upperRight;
				this.decreasingCorner = lowerRight;
			}
			else{
				var upperLeft = {
					x:subMenuOffset.left,
					y:subMenuOffset.top
				};
				var lowerLeft = {
					x:subMenuOffset.left,
					y:subMenuOffset.top + subMenu.outerHeight()
				};
				this.decreasingCorner = upperLeft;
	            this.increasingCorner = lowerLeft;
	        }
	        this.prevDecreasingSlope = this._slope(metaData.constantLoc, this.decreasingCorner);
	        this.prevIncreasingSlope = this._slope(metaData.constantLoc, this.increasingCorner);
		},
		/** 
			Returns the value currently associated with the specified optionName.
			@function menu.getAttribute
			@param {String} optionName - The name of the option.
			@return {String} value.
			@example <caption> Example: </caption>
var type = $("menuElement").zmenu("getAttribute","type");
		*/
		/** 
			Sets the value for the option with the specified optionName.
			@function menu.setAttribute
			@param {String} optionName - The name of the option.
			@param {String} value - The value for the option.
			@example <caption> Example: </caption>
$("menuElement").zmenu("setAttribute","type","grid");
		*/
		/** 
			Returns an object containing key/value pairs representing the current menu options hash.
			This method does not accept any arguments.
			@function menu.getAttributes
			@return {Object} options object.
			@example <caption> Example: </caption>
var options = $("menuElement").zmenu("getAttributes");
		*/
		/** 
			Sets one or more options for the menu.
			@function menu.setAttributes
			@param {Object} options - A map of option-value pairs to set.
			@example <caption> Example: </caption>
$("menuElement").zmenu("setAttributes",{"type":"grid","columns":4});
		*/
		setAttribute: function(optionName,value){
			if(optionName === "height" && this.options.height !== value){ // No I18N
				this.options.height = parseInt(value);
				this.element.data("height",parseInt(value)); // No I18N
				this._fixMenuHeight(this.element,{});
			}else if(optionName === "width" && this.options.width !== value){ // No I18N
				this.options.width = parseInt(value);
				this.element.data("width",parseInt(value)).width(this.options.width); // No I18N
			}else if(optionName === "contentType" && this.options.contentType !== value){ // No I18N
				if(this.options.type !== "grid"){ // No I18N
					this.element.removeClass(this._MENU_CONTENT_TYPES[this.options.contentType]).addClass(this._MENU_CONTENT_TYPES[value]);
				}
				this.options.contentType = value;
				var items = this._getMenuAsObject(this.element);
				this.element.is("ul") ? this.element.empty() : this.element.find("ul").empty(); // No I18N
				this.addMenuItems(items);
			}else if(optionName === "type" && this.options.type !== value){ // No I18N
				this.element.zmenu('destroy'); // No I18N
				this.options.type = value;
				this.element.zmenu(this.options);
			}else if(optionName === "search" && this.options.search !== value){ // No I18N
				if(value){
					this._createSearchContainer(this.element);
				}else{
					var searchBoxPosition = this.element.find(".zmenu__textbox").parent().attr("class"); // No I18N
					this.element.find("."+searchBoxPosition).remove(); // No I18N
				}
				this._fixMenuHeight(this.element,{});
			}else if((optionName === "pointer" || optionName === "displayType") && this.options.pointer !== value){ // No I18N
				if(value && value !== "box"){ // No I18N
					this._createPointerElement();
				}else{
					this._pointerElement.remove();
					this._pointerElement = undefined;
				}
			}else if(optionName === "renderInPlace" && this.options.renderInPlace !== value){ // No I18N
				if(!value){
					this.element.css({"position":"absolute","width":"auto","height":"auto"}); // No I18N
				}
			}else if((optionName === "menuClass" || optionName === "className") && (this.options.menuClass !== value || this.options.className !== value)){ // No I18N
				this.element.removeClass(this.options.menuClass).addClass(value);
				this.element.removeClass(this.options.className).addClass(value);
			}else if(optionName === "handleAccessKeyNavigation" && this.options.handleAccessKeyNavigation !== value){ // No I18N
				value ? this._initRuntimeEvents() : this._destroyRuntimeEvents(); // Keypress events	
			}else if(optionName === "appendTo"){ // No I18N
                value !== undefined && $(value).length ? this.element.appendTo($(value)) : $("body").append(this.element);
            }else if(optionName === "checkedCheckboxItemSVGIconId" || optionName === "checkedCheckboxItemIconClassName" || optionName === "checkedRadioItemSVGIconId" || optionName === "checkedRadioItemIconClassName"){ // No I18N
                this.options[optionName] = value;
                var childElements = this.element.find("li"), length = childElements.length, menuItem;
                for(var i = 0; i < length; i++){
                    menuItem = $(childElements[i]);
                    this._checkMenuItem(menuItem, menuItem.prop("checked"), this.getData(menuItem)); // No I18N
                }
            }
			this.options[optionName] = value;
		},
		/* 
			Sets the checked state of the given radio or checkbox menu items to true.
			@function menu.checkMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("checkMenuItems",["#menuItemId1","#menuItemId2",...]);
		*/
		checkMenuItems: function(items){
			items = items ? items : this.element.find('[role=menuitemcheckbox],[role=menuitemradio]');
			this.setMenuItemsAttribute(items,"checked",true); // No I18N
		},
		/*
			Sets the checked state of the given radio or checkbox menu items to false.
			@function menu.uncheckMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("uncheckMenuItems",["#menuItemId1","#menuItemId2",...]);
		*/
		uncheckMenuItems: function(items){
			items = items ? items : this.element.find('[role=menuitemcheckbox],[role=menuitemradio]');
			this.setMenuItemsAttribute(items,"checked",false); // No I18N
		},
		/**
			Hides the given menu items by setting its hidden property to true.
			@function menu.hideMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("hideMenuItems",["#menuItemId1","#menuItemId2",...]);	
		*/
		hideMenuItems: function(items){
			items = items ? items : this.element.find('li').filter(function(){  // No I18N
											return $(this).css("display") !== "none"; // No I18N
									});
			this.setMenuItemsAttribute(items,"hidden",true); // No I18N
		},
		/**
			Shows the given menu items by setting its hidden property to false.
			@function menu.showMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("showMenuItems",["#menuItemId1","#menuItemId2",...]);	
		*/
		showMenuItems: function(items){
			items = items ? items : this.element.find('li:not(li[data-hidden=true])'); // No I18N
			this.setMenuItemsAttribute(items,"hidden",false); // No I18N
		},
		/** 
			Set the properties of given menu items.
			@function menu.setMenuItemsAttribute
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@param {String} propName - The name of the property.
			@param {String} value - The value for the property.
			@example <caption> Example: </caption>
$("menuElement").zmenu("setMenuItemsAttribute",["#menuItemId1","#menuItemId2",...],"hidden",true);
		*/
		setMenuItemsAttribute: function(menuItems,optionName,value){
			var base = this;
			$.each(menuItems,function(i,menuItem){
				menuItem = $(menuItem,base.element);
                if(menuItem.length){
    				var data = base.getData(menuItem);
    				if(data.itemType === "itemgroup"){
    					var groupItems = $("[name="+menuItem.attr('name')+"]:not(.zmenu__title)",base.element); // No I18N
    					groupItems.length && base.setMenuItemsAttribute(groupItems,optionName,value);
    				}
    				if((optionName === "icon" || optionName === "iconClassName" || optionName === "SVGIconId") && data[optionName] !== value){ // No I18N
    					if(data.icon || data.iconClass || data.SVGIconId){ // Removing the previous icon
    						menuItem.find(".zmenu__iconcontainer:first").remove();  // No I18N
    					}
    					var iconEle = $("<i class='zmenu__icon'/>");
    					if(value.indexOf("text:") === 0){
    						// giving text as icon
    						menuItem.prepend($("<div class='zmenu__iconcontainer'>").append(iconEle));
    						menuItem.find(".zmenu__icon:first").text(value.replace("text:","")); // No I18N
    					}else{
    						data[optionName] = value;
    						menuItem.prepend($("<div class='zmenu__iconcontainer'>").append(base._handleIcon(iconEle, undefined, undefined, data.iconClassName, data.SVGIconId, data.icon)));
    					}
    				}else if(optionName === "label" && data.label !== value){ // No I18N
                        if(data.ellipsis){
                            value = value+"..." // No I18N
                        }
    					menuItem.find(".zmenu__text").text(value); // No I18N
    				}else if((optionName === "shortcutkey" || optionName === "shortcutKey") && (data.shortcutkey !== value || data.shortcutKey !== value)){ // No I18N
    					if(data.shortcutKey || data.shortcutkey){
    						menuItem.find(".zmenu__shortcut").text(value);  // No I18N
    					}else{
                            value = window.ZShortCut ?  ZShortCut.getInfo(undefined, value) : value;
                            base.getData(menuItem).shortcutkey = base.getData(menuItem).shortcutKey = value;
                            menuItem.append($("<span class='zmenu__shortcut'/>").text(value));    // No I18N
    					}
    				}else if(optionName === "hint" || optionName === "description" && (data.hint !== value || data.description !== value)){ // No I18N
    					if(data.hint || data.description){
    						menuItem.find(".zmenu__hint").text(value);  // No I18N
    					}else{
    						menuItem.append($("<span class='zmenu__hint'/>").text(value));  // No I18N
    					}
    				}else if(optionName === "disabled" || optionName === "checked" || optionName === "hidden"){ // No I18N
    					data[optionName] = value;				
    					base._handleCheckedandDisabled(menuItem,data,optionName);
    				}else if(optionName === "menuId" && data.menuId !== value){ // No I18N
    					data[optionName] = value;
    					if(!value){
    						menuItem.find(".zmenu__icon.h-alignright").remove();
    						menuItem.removeData("submenu"); // No I18N
    						menuItem.off("mouseup.submenuitem.zmenu"); // No I18N
    					}else{
    						menuItem.removeData("submenu"); // No I18N
    						base._buildSubmenu(menuItem,data);
    						base._createArrowIcon(menuItem);
    					}	
    				}else if(optionName === "innerHTML" && data.itemType === "custom"){   // No I18N
    					menuItem.empty().html(value); // should be used only in custom menu item
    				}else if(optionName === "itemType" && data.itemType !== value){ // No I18N
    					if(value === "separator"){ // No I18N
    						menuItem.removeClass("zmenu__item").addClass("zmenu__separator").empty(); // No I18N
    					}else if(value === "default"){ // No I18N
    						menuItem.removeClass("zmenu__title zmenu__custom zmenu__separator").addClass("zmenu__item"); // No I18N
    					}else if(value === "custom"){ // No I18N
    						menuItem.addClass("zmenu__item zmenu__custom"); // No I18N
    					}else if(value === "itemgroup"){ // No I18N
    						menuItem.removeClass("zmenu__item zmenu__custom zmenu__separator").addClass("zmenu__title"); // No I18N
    					}else if(value === "splitmenu"){ // No I18N
    						menuItem.removeClass("zmenu__title zmenu__custom zmenu__separator").addClass("zmenu--split"); // No I18N
    						base._createArrowIcon(menuItem);
    						menuItem.find(".zmenu__shortcut").remove();
    					}
                        menuItem.attr("role", base.ARIA_ROLES[value ?  value : "default"]); // No I18N
    					menuItem.attr("data-item-type",value); // No I18N
                        menuItem[0].setAttribute("role", this.ARIA_ROLES[value ?  value : "default"]); // No I18N           
    				}
    				data[optionName] = value;
                }
    		});
		}
	}
    menuPrototype.updateMenuItems = menuPrototype.setMenuItemsAttribute;
	ZC.registerComponent("zmenu", menuPrototype); // No I18N
	$.extend($.expr[":"],{	// No I18N
		visibleElementsOfGroup:function(element){ // Filter to find the elements that are hidden with display='none'    // No I18N
			return $(element).css("display") !== "none";	// No I18N
		}
	});
	ZC.createMenu = function(menuId,menuDetails,options){ // Menu Created Using JSON - assigning it to global namespace
		if(typeof menuId === "object"){ // No I18N
			menuDetails = menuId.items,
			options = menuId;
			menuId = menuId.id;
		}
		if($("#"+menuId).data("zmenu")){ // No I18N
			return $("#"+menuId); // No I18N
		}
		var	menu = undefined;
		if(document.getElementById(menuId)){
			menu = $(document.getElementById(menuId));
			menu[0].tagName === "DIV" && menu[0].appendChild(document.createElement("ul")); // No I18N
		}else{
			menu = $("<div><ul></ul></div>").attr("id",menuId);	 // No I18N
		}
		var appendTo = options ? options.appendTo : undefined;	
		menu[0].className += " zmenu "+menuPrototype._MENU_CONTENT_TYPES[options.contentType ? options.contentType : "text"]+" "+(options.menuClass ? options.menuClass :"")+" "+(options.className ? options.className :""); // No I18N
		if(options.type && options.type.indexOf("grid") !== -1){ // No I18N
			menu[0].className += " zmenu--thumbnail"; // No I18N
		}
		options.usingJSON = true;
		var appendedTo = document.querySelector("body");	// No I18N
		if( appendTo !== undefined && appendTo.length ){
			appendedTo = $(appendTo)[0];
		}
		appendedTo.appendChild(menu[0]);
		ZC.getOptions(menu).type = options.type;
		menu.zmenu(options);
		menuDetails.length ? menu.zmenu("addMenuItems",menuDetails,options ? options.addItemCallback : undefined) : ""; // No I18N
		return menu;
	}

	// ZC.$document.on("click.zmenu",function(event){ // No I18N
	// 	if(event.which === 3){
	// 		return;
	// 	}
	// 	var target = $(event.target),
	// 		forElement = ZC.currentMenu && ZC.currentMenu.data("forElement"), // No I18N
	// 		inPlaceMenu = ZC.currentMenu && ZC.currentMenu.data("zmenu") && ZC.currentMenu.data("zmenu").options.renderInPlace; // No I18N
	// 	if(!inPlaceMenu){
	// 		if(!target.closest(".zmenu").length && forElement && forElement.length && !target.closest(forElement).length){
	// 			ZC.currentMenu.is(":visible") && ZC.currentMenu.zmenu("hide"); // No I18N
	// 		}else if(!target.closest(".zmenu").length && ZC.prevMenu && ZC.prevMenu.is(":visible")){ // No I18N
	// 			ZC.prevMenu.zmenu("hide"); // No I18N
	// 			ZC.prevMenu = undefined;
	// 		}
	// 	}
	// }).on("zmenuhide",function(event,ui){	// No I18N
	// 	if(ZC.prevMenu && ZC.prevMenu.get(0) === ui.menu.get(0)){
	// 		ZC.prevMenu = undefined;
	// 	}else if(ZC.currentMenu && ZC.currentMenu.get(0) === ui.menu.get(0)){
	// 		ZC.currentMenu = undefined;
	// 	}
	// });
	var ZM_DOM = {};
	function menuScrollWidth(){
		var scrollEle = document.createElement("div");
		ZM_DOM._iconDiv = scrollEle.cloneNode();
		ZM_DOM._iconDiv.className = 'zmenu__iconcontainer';
		ZM_DOM._icon = document.createElement("i");
		document.querySelector("body").appendChild(scrollEle);	// No I18N
		scrollEle.style.cssText = "width:50px;height:50px;overflow:scroll;";	// No I18N
		menuPrototype._scrollWidth = $(scrollEle).width() - scrollEle.scrollWidth;
		$(scrollEle).remove();
		ZM_DOM._text = document.createElement("span");
		ZM_DOM._hint = ZM_DOM._text.cloneNode();
		ZM_DOM._key = ZM_DOM._text.cloneNode();
		ZM_DOM._text.className = 'zmenu__text';
		ZM_DOM._hint.className = 'zmenu__hint';
		ZM_DOM._key.className = 'zmenu__shortcut';
		return menuPrototype._scrollWidth;
	};
})(ZComponents);

/** @member {String} [menu.item-options.icon] - The icon to be shown in the menu item. It can be a sprite css class name or svg element identifier followed by class name to be used in svg element. This option will be deprecated in the future version. So use "iconClassName" and "SVGIconId" instead.
								   Example : "settings-icon" or "#svg-id custom-class"
	@deprecated
*/
/** @member {String} [menu.item-options.iconClassName] - The icon to be shown in the menu item. It can contain one or more space separated sprite css classes.
	@newlyAdded
*/
/** @member {String} [menu.item-options.SVGIconId] - The icon to be shown in the menu item. It contains svg element id followed by space separated css class names to customize SVG icon.
	@newlyAdded
*/

/** @options menu.item-options */
/** @member {String} [menu.item-options.label] - The text to be displayed in the menu item.*/
/** @member {String} [menu.item-options.shortcutKey] - The shortcut key to be displayed in the menu item.*/
/** @member {String} [menu.item-options.description] - The additional description text to be displayed in the menu item.*/
/** @member {String} [menu.item-options.itemType] - The type of the menu item.
	@allowedValues {("default"|"separator"|"radio"|"checkbox"|"custom"|"itemgroup"|"splitmenu")}
*/
/** @member {String | Object} [menu.item-options.itemValue] - The custom value for the menu item.*/
/** @member {String} [menu.item-options.innerHTML] - The content HTML of the menu item of type "custom"(ie. menu item with itemType as "custom").*/
/** @member {String} [menu.item-options.menuId] - The identifier of the menu item's submenu.*/
/** @member {Array} [menu.item-options.menuItems] - Array of menu items object to create the submenu for the menu item.*/
/** @member {Boolean} [menu.item-options.disabled] - Specifies whether the menu item is in disabled state or not.*/
/** @member {Boolean} [menu.item-options.checked] - Specifies whether the menu item is in checked state or not.*/
/** @member {Boolean} [menu.item-options.hidden] - Specifies whether the menu item is in hidden state or not.*/
/** @member {Function} [menu.item-options.action] - The callback to be executed whenever the menu item is clicked.*/
/** @member {Number} [menu.item-options.index] - The index to insert the menu item.*/
/** @member {Boolean} [menu.item-options.isFooter] - Indicates whether to place the menu item as the footer item to the menu or not.*/
/** @member {String} [menu.item-options.className] - The CSS class (or the space separated string of classes) to be added to the menu item for additional theming. This option is mostly used while creating menu using JSON or adding menu items using API methods. 
	@example <caption> Add the menu item with the "className" option specified: </caption>
	$("menuElement").zmenu("addMenuItem",{
	    "label"     : "Print",
	    "icon"      : "printer-icon",
	    "className" : "custom-css-class"
	});
*/
/*	  "icon"  : <#svg-id svg-class|icon-class-name>,  
      "iconText" : <string>,
      "label" : <string>,
      "shortcutKey": <string>,
      "description" : <string>,
      "itemValue" : <string>,
      "itemType": "<default|separator|radio|checkbox|custom|itemgroup|splitmenu|submenu>",
      "innerHTML": <DomElements>,
      "menuId" : <submenuId>,      
      "menuItems" : <[menuItemObject1,menuItemObject2,...]>,      
      "disabled" : <true|false>,
      "checked" : <true|false>,
      "hidden" : <true|false>,
      "action" : <js-function>, */ 
