	/* $Id$ */ 
	(function( ZC ){
		var $ = ZC.DOMUtil;
		/** @components combo-box Combobox is the combination of selectbox and textbox that allows the user to search for an item by typing inside the textbox.
			@eventPrefix zcombobox
		*/
		/** @components select-box Selectbox enables the user to select items from the dropdown list with additional visual effects.
			@eventPrefix zselectbox
		*/
		var selectboxPrototype = {
			options: {
				/** @member {Boolean} [select-box.multiple=false] Specifies whether multiple values can be selected. When multiple  is true, always contentType is set to 'text' */
				multiple:false,
				selectionLimit: 0,
				dataSource: undefined,
		        selectedIndex:undefined,
		        selectedValue:undefined,
				placeholder: 'Select',	//No I18N
		        dropdownList:{
		        	height : undefined,
		        	width :undefined,
		        	appendTo : undefined,
		        	direction : 'auto',	//No I18N
		    		className : undefined,
		    		viewport: undefined
		        },
				autoFill: false,
				searchable: false,
				selectOnClose: false,
				clearSelection: false,
		        searchSettings: {
			       delay: 150,  
			       searchCriteria: 'contains',  //No I18N  
			       caseSensitive: false,             
			       // minKeywordLength: 0,
			       searchByField:'label' //No I18N
			     },
		        disabled: false,
		        contentType: 'text',	//No I18N
		        className:undefined,
		        messages:{
		        	allOptionsSelected: 'All Selected',	//No I18N
		        	noResults:'No Results'	//No I18N
		        },
				eventPrefix:'zselectbox'	//No I18N
			},			
			_EVENTS: {
				/** 
					Triggered before the dropdown list is shown. In order to prevent dropdown list from opening, return 'false' in the callback.
					@event combo-box.beforedropdownlistopen
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combo.
				*/
				'ONBEFORESHOW': 'beforedropdownlistopen',	//No I18N
				/**
					Triggered after the dropdown list is shown.
					@event combo-box.dropdownlistopen
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combo.
				*/
				'ONSHOW': 'dropdownlistopen',	//No I18N
				/**
					Triggered when the selection is changed.
					@event combo-box.change
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combo.
				*/
				'ONCHANGE': 'change',	//No I18N
				/**
					Triggered before the dropdown list is closed. In order to prevent dropdown list from closing, return 'false' in the callback.
					@event combo-box.beforedropdownlistclose
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				'BEFOREHIDE':'beforedropdownlistclose', //No I18N
				/**
					Triggered after the dropdown list is closed.
					@event combo-box.dropdownlistclose
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				'ONHIDE': 'dropdownlistclose',	//No I18N
				/**
					Triggered after an item is clicked.
					@event combo-box.itemclick
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONITEMCLICK: 'itemclick',	//No I18N
				/**
					Triggered when the component is ready for use.
					@event combo-box.init
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONINIT:'init',	//No I18N
				ONNEWVALUE:'newvalue', //No I18N
				/**
					Triggered when the component loses focus.
					@newlyAdded true
					@event combo-box.blur
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONBLUR:'blur', //No I18N
				/**
					Triggered when the component receives focus.
					@newlyAdded true
					@event combo-box.focus
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONFOCUS:'focus', //No I18N
				ONBEFOREOPTIONADD:'beforeoptionadd', //No I18N
				ONOPTIONADD: 'optionadd' //No I18N
			},
			_CLASS : {
				ICON:'zselectbox__icon',	//No I18N
				ICONTEXT: 'zselectbox--icontext', //No I18N
				ICONONLY: 'zselectbox--icononly',	//No I18N
				PLACEHOLDER: 'zselectbox__placeholder', //No I18N
				DROPUP: 'zselectbox--top', //No I18N
				DROPDOWN: 'zselectbox--bottom',	//No I18N
				ACTIVE: 'is-active', //No I18N
				FOCUS: 'has-focus', //No I18N
				DISABLED: 'is-disabled', //No I18N
				EMPTY:'zselectbox__emptyresult', //No I18N
				TEXTBOX:'zselectbox__text' //No I18N
			},
			_create: function( element, options){
				this.frominit = true;
				this._elementWidth = options.width || this.element[0].offsetWidth;
				this.element[0].style.display = 'none';
				this._namespace = '.' + options.eventPrefix; //No I18N
				var base = this;
				['disabled','multiple','title'].forEach( function( attr){ //No I18N
					options[attr] = options[attr] || base.element[0][attr];
				});	
				this._initialize(element, options);
				this._constructComponent();
				this._handleDS();
				this._setInitial();
				this._bindEvents();
				this._trigger( "ONINIT" ); 	//No I18N
				this.frominit = false;
			},
			_initialize: function(element, options){
				/*To handle keydown selection when menu is not opened*/
				this._textArray = [];
				this._prevKey = this._prevTimeStamp = ''; // No I18N
				if( !/^(text|icon|icon-text|custom)$/.test( options.contentType ) ){
					options.contentType = $.zselectbox.prototype.options.contentType;
				}
				if( options.multiple ){
					var limit = +options.selectionLimit;
					options.selectionLimit =  ( isNaN(limit) || limit < 0 ) ? 0 : limit;
					// if( options.contentType === 'icon' ){ //No I18N
						options.contentType = 'text'; //No I18N
					// }
					if( options.selectedValue !== undefined ){
						options.selectedValue = String(options.selectedValue).split(','); //No I18N
					}else if( options.selectedIndex !== undefined ){
						options.selectedIndex = String(options.selectedIndex).split(',') || 0; //No I18N
					}
					this.element[0].setAttribute('multiple', true); //No I18N
				}
			},
			/* Function to construct layout of combo component*/
			_constructComponent: function(){
				if( !this.element[0].id ){
					this.element[0].id = (this._isEditable ?'zcombo_':'zselect_')+Math.floor(Math.random() * 1000000); //No I18N
				}
				if( typeof this.options.menuId === 'string' ){ //No I18N
					this._menuID = this.options.menuId;
				}else{
					this._menuID = this.element[0].id + "Menu"; //No I18N
					$(document.getElementById(this._menuID )).remove();
				}
				var containerId = this.element[0].id+'Container'; //No I18N
				this.container = document.getElementById( containerId );
				if( this.container ){
					this._closeIcon = this.container.querySelector('.zselectbox__close'); //No I18N
					this._arrow = this.container.querySelector('.zselectbox__arrowicon'); //No I18N	
					this._optArea = this.container.querySelector('.zselectbox__selected'); //No I18N
					this.textspan = this._optArea.querySelector('.zselectbox__text'); //No I18N
					this.iconspan = this._optArea.querySelector('.zselectbox__icon'); //No I18N
				}else{
					var baseClass = 'zselectbox ' + ( typeof this.options.className === 'string'? this.options.className : '') + ' ' + (this.element[0].getAttribute('class') ? this.element[0].getAttribute('class') : '');	//No I18N
					this.container = document.createElement('div');	//No I18N
					$(this.container).attr({
						'class':baseClass, //No I18N
						'aria-haspopup': true, //No I18N
						'title': this.element[0].getAttribute('title'), //No I18N
						'aria-owns': this._menuID, //No I18N
						id: containerId,
						'style': this.element[0].getAttribute('style') //No I18N
					});
					this.container.style.display = ''; //No I18N
					if( !this.element[0].style.width && this._elementWidth ){
						this.container.style.width = isNaN( this._elementWidth )? this._elementWidth : (this._elementWidth+'px'); //No I18N
					}
					this.container.innerHTML = "<div class=zselectbox__selected></div> <i class='zselectbox__close'></i> <i class='"+ this._CLASS.ICON +" zselectbox__arrowicon'></i>";
					this._optArea = this.container.children[0];
					this._closeIcon = this.container.children[1];
					this._arrow = this.container.children[2];
					if( this.options.arrowSVGIconId || this.options.arrowIconClassName ){
						this._arrow = ZC.createIcon( $(this._arrow), undefined, this.options.arrowSVGIconId, this.options.arrowIconClassName )[0];
					}else{
						this._arrow.innerHTML = "<svg class='zselectbox__arrow' viewBox='0 0 48 48'>"+ //No I18N
									              "<path class='cls-1' d='M12,19.478L15.571,16l9.286,9.739L34.143,16,37,19.478,24.857,32Z'></path>"+ //No I18N
									            "</svg>";
					}
					if( this.options.closeSVGId || this.options.closeIconClass ){
						this._closeIcon = ZC.createIcon( $(this._closeIcon), undefined, this.options.closeSVGId, this.options.closeIconClass )[0];
					}else{
						this._closeIcon.innerHTML = '<svg viewBox="0 0 16 16"> <path d="M15.1 3.1l-2.2-2.2-4.9 5-4.9-5-2.2 2.2 5 4.9-5 4.9 2.2 2.2 4.9-5 4.9 5 2.2-2.2-5-4.9z"/> </svg>'; //No I18N
					}
					this._closeIcon.style.display = 'none'; //No I18N
					this._constructOptArea();
					this.element[0].parentElement.insertBefore( this.container, this.element[0].nextSibling );
				}
			},
			/*Function to create optArea*/
			_constructOptArea: function(){
				this.container.setAttribute('tabindex', this.element[0].getAttribute('tabindex') || 0 );
				this.container.setAttribute('role','select');
				var html = '';
				if( this.options.contentType.match('icon') && !this.options.multiple ){
					html = html + '<i class='+ this._CLASS.ICON +'></i>';
				}
				if( !/^(icon)$/.test( this.options.contentType) || this.options.multiple ){
					html = html + '<span class=zselectbox__text></span>';
				}
				this._optArea.innerHTML = html;
				this.textspan = this._optArea.querySelector('.zselectbox__text'); //No I18N
				this.iconspan = this._optArea.querySelector('.zselectbox__icon'); //No I18N
				var specialClass = (this.options.contentType === 'icon'? this._CLASS.ICONONLY : this.options.contentType === 'icon-text' && !this.options.multiple ? this._CLASS.ICONTEXT :' '  ).trim();  //No I18N
				this._addClass( this.container, specialClass );
			},
			_createMenu: function(){
				var o = this.options;
				var base = this;
				var menu = document.getElementById( this._menuID );
				var menu_opt = {
					position : o.position,
					contentType: o.multiple ? 'icon-text': o.dropdownList.contentType || o.contentType, //No I18N
					tagName : 'div', //No I18N
					height : o.dropdownList.height,
					width :  o.dropdownList.width,
					appendTo : $(o.dropdownList.appendTo),
					className: (o.dropdownList.className || "") + " zmenu--selectmenu",	//No I18N
					direction : o.dropdownList.direction,
					handleAccessKeyNavigation: o.searchable? false: true,
					scroll: 'default', //No I18N
					viewport: $(o.dropdownList.viewport),
					toggle: false
				}
				if( menu ){
					this._menu = $(menu);
					this._completeMenuLoaded = true;
					menu_opt.constructMenu = false;
					this._menu.zmenu( menu_opt  );
				}else{
					this._menu = ZC.createMenu(this._menuID, {}, menu_opt);
					this._createSearchBox();
				}
				this._menu.attr({'tabindex':-1, 'role':'listbox'}) //No I18N
				  		  .on({
								zmenubeforeshow: function( origEvent ){
									// origEvent.stopImmediatePropagation();
									return base._trigger( "ONBEFORESHOW", origEvent ); 	//No I18N
								},
								zmenubeforehide: function( origEvent ){
									origEvent.stopImmediatePropagation();
									return base._trigger( "BEFOREHIDE", origEvent ); 	//No I18N
								},
								zmenushow: this._onMenuShow.bind(this),
								zmenuhide: this._onMenuHide.bind(this),
								zmenuitemclick: this._onItemClick.bind(this),
								zmenuitemhover: this._onMenuItemHover.bind(this),
								zmenubeforefocus: function(){
									return false;
								}
							}).on('focusout'+this._namespace, this._onBlur.bind(this)) //No I18N
				  		  	  .on('keydown'+this._namespace, function( origEvent ){ //No I18N
				  		  	  	 base._keyPress.call( base, origEvent );
				  		  	  });
			},
			_createSearchBox: function(){
				if( !this._isEditable && this.options.searchable && this._menu ){
					var header = $("<div class='zmenu__header'></div>"); // No I18N
					var textbox = $("<input type='text' class='zmenu__textbox' placeholder='Search' />"); // No I18N
					var searchIcon = $("<i class='zmenu__icon'></i>");
					var svgEle = ZC.createSVGElement("svg",{ // No I18N
						"class":"zmenu__search","viewBox":"0 0 16 16","width":"100%","height":"100%" // No I18N
					},[
					{"path":{"d":"M15.6,15.6c-0.6,0.6-1.5,0.6-2.1,0l-2.7-2.7C9.7,13.6,8.4,14,7,14c-3.9,0-7-3.1-7-7s3.1-7,7-7c3.9,0,7,3.1,7,7c0,1.4-0.4,2.7-1.1,3.8l2.7,2.7C16.1,14,16.1,15,15.6,15.6z M7,2C4.2,2,2,4.2,2,7s2.2,5,5,5c2.8,0,5-2.2,5-5C12,4.2,9.8,2,7,2z"}} // No I18N
					]);
					searchIcon[0].appendChild(svgEle);
					header.append(textbox, searchIcon);
					this._menu.prepend(header);
					this._bindSearchbox(textbox[0]);
				    // textbox.on('keydown'+this._namespace, this._keyPress.bind(this)) //No I18N
				}
			},
			_bindSearchbox:function( textbox ){
				var eventName = isIE9? 'keyup':'input'; //No I18N
				var base = this;
				$(textbox).on( eventName +this._namespace,  function( origEvent ){
					if( origEvent.type === 'input' || !(origEvent.metaKey || origEvent.ctrlKey) && ((origEvent.keyCode >= 186 && origEvent.keyCode <= 192) || (origEvent.keyCode >= 219 && origEvent.keyCode <= 222) || (origEvent.keyCode >= 48 && origEvent.keyCode <= 57) || (origEvent.keyCode >= 65 && origEvent.keyCode <= 90) || (origEvent.keyCode ===  ZC.keyCode.SPACE) || (origEvent.keyCode === ZC.keyCode.DELETE) || (origEvent.keyCode === ZC.keyCode.BACKSPACE))){	//No I18N
						origEvent.type = 'input'; //No I18N
						base._onInput.call( base, textbox, origEvent );
					}
				});
			},
			_onInput:function( textbox, origEvent ){
				var base = this;
				var value = textbox.value;
				var setting = this.options.searchSettings;
				this._activated = true;
				if( this._closeIcon ){
					this._closeIcon.style.display = (($.trim(value) === '' || !this.options.clearSelection) ? 'none':''); //No I18N
				}
				value = value.replace(new RegExp(/\\/gi),"\\\\").replace(new RegExp("'","gi"),"\\'"); //No I18N
				value = setting && setting.caseSensitive ? value : value.toLowerCase();
				clearTimeout( this.inputtextTimer );
				if( setting && setting.delay ){
					this.inputtextTimer = setTimeout( this._searchQuery.bind( this, origEvent, value ), setting.delay );
				}else{
					this._searchQuery( origEvent, value );
				}
			},
			_searchQuery: function( origEvent, query ){
				this._filterList( query );
				this._open( origEvent );
			},
			_filterList: function( query ){
				var base = this;
				var s = this.options.searchSettings || {};
				this._loadDropdown();
				this._clearStyle();
				if( s.searchByField === 'label' ){ //No I18N
					var metadata = ZC.getOptions( this.element[0] );
					/*creating style tag*/
					var styletag = metadata.styletag = document.getElementsByTagName('head')[0].appendChild( document.createElement('style') );
					if( query.length ){
						styletag.textContent = "#"+ this._menuID +" [data-filter-label]:not([data-filter-label"+ (s.searchCriteria === 'startswith'? '^=':'*=') +"'"+ query +"']) {display:none !important;}";	//No I18N
						var g_hidden = [];
						this._queryAll( this._menu[0], '.zmenu__title' ).forEach( function( group ){
							var visibleitems = base._getFilteredNodes(function( node ){
								return $(node).css('display') !== 'none'; //No I18N
							}, base._queryAll( base._menu[0], '.zmenu__item[data-group='+group.id+']' )); //No I18N
							if( !visibleitems.length ){
								g_hidden.push('#'+ group.id);
							}
						});
						if(g_hidden.length){
							styletag.textContent = styletag.textContent +" " +g_hidden.join(',') + " {display:none !important;}"; //No I18N
						}	
					}	
				}else{
					this._hiddenitems = [];
					this._queryAll( this._menu[0],'li:not([hidden])').forEach(function( node ){ //No I18N
						var menuData = ZC.getOptions( node );
						if(!( base._menuitemMatcher( menuData , query )  ) ){
							node.style.display = 'none'; //No I18N
							base._hiddenitems.push( node );
						}	
					});
				}
			},
			_menuitemMatcher: function( menuData, query ){
				if( !menuData || $.trim(query) === ''){ //No I18N
					return true;
				}
				if( menuData.itemType === 'itemgroup' ){ //No I18N
					var childDatas = this._menu.find('[data-group = ' + menuData.customAttributes.id + ']'); //No I18N
					for( var i=0; i< childDatas.length; i++ ){
						var child = childDatas[i];
						if( this._menuitemMatcher( ZC.getOptions( child ), query ) ){
							return true;
						}
					}
				}else{
					var s = this.options.searchSettings;
					var searchKeys = s.searchByField.split(','); //No I18N
					for( var i =0; i< searchKeys.length; i++ ){
						var key = searchKeys[ i ];
						var regex = new RegExp( s.searchCriteria === 'startswith'? '^('+query+')':query , 'g'+(s.caseSensitive?'':'i')); //No I18N
						if(/* menuData.properties && */typeof menuData.properties[key] === 'string' && regex.test( menuData.properties[key] ) ){ //No I18N
							return true;
						}
					}
				}
			},
			_clearStyle: function(){
				var metadata = ZC.getOptions( this.element[0] );
				if( metadata.styletag ){
					$(metadata.styletag).remove();
					metadata.styletag = undefined;
				}
				if( this._hiddenitems && this._hiddenitems.length ){
					$(this._hiddenitems).show();
					this._hiddenitems = undefined;
				}	
				$('.'+ this._CLASS.EMPTY, this._menu ).remove();
			},
			_onMenuHide: function( origEvent ){
				this._removeClass( this.container, this._CLASS.DROPUP + ' '+ this._CLASS.DROPDOWN );
				this.container.setAttribute('aria-expanded', false);
				clearTimeout( this.inputtextTimer );
				this._clearStyle();
				this._updateSTATE();		
				if( this.options.searchable && !this._isEditable ){
					this._menu.find('input.zmenu__textbox').val(''); //No I18N
				}
				this._trigger( "ONHIDE", origEvent );		//No I18N
				// origEvent.stopImmediatePropagation();
			},
			_onMenuShow: function( origEvent ){
				this._activated = true;
				var hoveredItem;
				var visibleItems = this._getFilteredNodes(function( node ){
					return $( node ).css( 'display' ) !== 'none'; //No I18N
				}, this._queryAll( this._menu[0], 'li[data-filter-label]:not([hidden])' )); 
				if( visibleItems.length ){
					visibleItems = this._getFilteredNodes( ':not(.is-disabled)', visibleItems ); //No I18N
					if( origEvent.originalEvent && origEvent.originalEvent.type === 'input' ){ //No I18N
						/* Open dropdown if it contains visible items and hover the first matched items */
						hoveredItem = visibleItems[0]; 
						this._isEditable && this._autoComplete( this.textspan.value, visibleItems[0] );
					}else{
						if( this.options.multiple || !this._selected.length || typeof this._selected === 'string'){
							// hoveredItem = undefined;
							hoveredItem = this._menu[0].querySelector('li.zmenu__item:not([disabled]):not([hidden])');
						}else{ 
							hoveredItem = this._getMenuItemForOption( this._selected )[0];
						}
					}
				}else{
					if( this.options.acceptNewValues ){
						this._menu.hide();
						return;
					}
					$(this._menu[0].querySelector('ul')).append( "<li class='zmenu__item  "+ this._CLASS.EMPTY +"'><span>"+ this._localize('noResults')+"</span></li>" );  //No I18N
				}
				hoveredItem && this._menu.zmenu('hoverMenuItem', hoveredItem); //No I18N
				this.container.setAttribute('aria-expanded', true);
				this._setMenuWidth();	
				this._setState();
				this._menu.find('>ul.zmenu__list').height( this._menu.height() - this._menu.find('.zmenu__header').outerHeight() - this._menu.find('.zmenu__footer').outerHeight( true ) ); //No I18N
				if( this.options.searchable && !this._isEditable ){
					this._menu.find('input.zmenu__textbox').focus(); //No I18N
				}
				this._trigger( "ONSHOW", origEvent ); 	//No I18N
				// origEvent.stopImmediatePropagation();
			},
			_updateSTATE: function(){
				if( !this._activated ){ 
					var o = this.options;
					var itemToBeSelected = o.selectOnClose? this._lastHighlightedItem : ( (o.autoFill || o.autoComplete)? this._lastHoveredItem: undefined );
					if( !o.multiple && itemToBeSelected && itemToBeSelected !== this._selected[0] ){
						this._selectOption( itemToBeSelected, true );
					}else{
						!/(string|number)/.test( typeof this._selected ) && this._updateCombobox();
					}
				}
				if( this._focussed ){
					this.setFocus();
				}
				this._setState();
			},
			_onMenuItemHover: function( origEvent, userdata ){
				var o = this.options;
				this._lastHighlightedItem = this._getOptionForMenuItem( userdata.menuitem[0].id );
						
				if( !o.multiple && origEvent.originalEvent && !origEvent.originalEvent.originalEvent && !userdata.data.isFooter && (o.autoFill || o.autoComplete ) && (!o.searchable || !this._isEditable) ){
					this._lastHoveredItem = this._lastHighlightedItem;
					this._updateCombobox( [this._lastHoveredItem], false, false );
					return true;
				}
				// origEvent.stopImmediatePropagation();
			},
			_setInitial: function( ){
				this._selected = [];
				var o = this.options;
				var defaultOption;
				if( this.element[0].options && this.element[0].options.length ){
					if( o.selectedValue ){
						defaultOption = o.selectedValue;
					}else if( o.selectedIndex !== undefined ){
						if( parseInt( o.selectedIndex ) !== -1 ){
							defaultOption = this._getOptionAt( o.selectedIndex );
						}
					}else{
						defaultOption = $(this._getOptions()).filter(':selected,[data-selected]').toArray() ; //No I18N
					}
					o.selectedIndex = this.element[0].selectedIndex = -1;
					o.selectedValue = undefined;
					this._includePlaceholder();
				}
				this.setValue( defaultOption );
				o.disabled && this._setAttribute('disabled', o.disabled ); //No I18N
			},
			_getOptionForMenuItem: function( itemid ){
				return this.element[0].querySelector('[data-zitemid="'+ itemid +'"]'); //No I18N
			},
			_handleDS: function(){
				var ds = this.options.dataSource;
				if( ds && typeof ds === 'object' ){
					this._addOption( ds );
				}
			},
			_populateMenu: function( options ){
				var results = this._constructResultSet( options );
				if( results.length ){
					//We can part this and show
					var menuObjects = this._constructMenuObject( results );
					if( menuObjects.length ){
						var menuitems = [];
						this._menu.zmenu('addMenuItems', menuObjects, function( item ){ //No I18N
							menuitems.push( item[0] );
						}); 
						return menuitems;
					}
				}
			},
			_isSelected: function( data ){
				var element = this._getOptionForMenuItem( data.id );
				if( element ){
				 	return element.selected;
				}
			},
			_constructMenuObject: function( results ){
				var menuObjects = [];
				var o = this.options;
				var contentType = o.dropdownList.contentType || o.contentType;
				if( results ){
					results = Array.isArray( results ) ? results : [ results ];
					for( var i=0; i< results.length ; i++ ){
						var result = results[i];
						if( typeof result === 'string' ){ //No I18N
							result = { label: result }
						}
						var m_obj = {};
						var attr = m_obj.customAttributes = {};
						if( typeof result.customAttributes === 'object' ){ //No I18N
							attr = $.extend( attr, result.customAttributes );
						}
						var label = String( result.label );
						m_obj.label = label;
						m_obj.hidden = result.hidden;
						m_obj.disabled = result.disabled;
						attr.id = result.id;
						var selected = this._isSelected( result );
						if( !result.options ){
							if( result.isFooter ){
								m_obj.isFooter = true;
								m_obj.innerHTML = result.innerHTML || label;
								m_obj.itemType = 'custom'; //No I18N
							}else if( result.separator ){
								m_obj.itemType = attr.role  = 'separator'; // No I18N
							}else{	
								m_obj.description = result.description;
								m_obj.shortcutkey = result.shortcutkey;
								attr.style = attr.style || result.style;
								attr['data-filter-label'] = o.searchSettings && o.searchSettings.caseSensitive ? label : label.toLowerCase(); //No I18N
								/*Include all supported attributes*/
								attr.title = attr.title || result.title || ( contentType === 'icon'? label : undefined ); //No I18N
								m_obj.className = ( result.className || '' )+( selected? ' is-selected':'' ); //No I18N
								// attr.role = 'option'; //No I18N
								if( contentType === 'custom' ){ //No I18N
								    m_obj.itemType = 'custom'; //No I18N
								    m_obj.replaceWithText = result.replaceWithText;
								    m_obj.innerHTML = this._getCustomContent( result, o.dropdownList.template );
								}
								if( o.multiple ){
									var html = "<div class='zmenu__iconcontainer'><i class='zmenu__icon'></i></div>";
									if( contentType.indexOf('icon') !== -1 ){
										html = html+ "<div class='zmenu__iconcontainer'><i class='zmenu__icon "+ result.icon +"'></i></div>";
									}
									if( contentType === 'custom' ){ //No I18N
										html = html + m_obj.innerHTML;
									}else{
										html = html+ "<span class='zmenu__text'>"+ label +"</span>";
									}
									m_obj.innerHTML = html;
									m_obj.custom = true;
									m_obj.itemType = 'checkbox'; //No I18N
									m_obj.checked = selected;
								}else{
									m_obj.SVGIconId = result.SVGIconId;
									m_obj.iconClassName = result.iconClassName;
									//deprecated
									m_obj.icon = result.icon;
									m_obj.textAsIcon = result.textAsIcon;
								}
								if( this._textArray ){
									this._textArray.push({
										text: label,
										menuid: attr.id
									});	
								}
							}
						}else if( result.options ){
							// attr.role = 'group'; //No I18N
							m_obj.itemType = 'itemgroup'; //No I18N
						}
						menuObjects.push( m_obj );
						if( m_obj.itemType === 'itemgroup' ){ //No I18N
							for( var j=0 ;j< result.options.length;j++ ){
								var suboption = result.options[j];
								var subItemObj = this._constructMenuObject( suboption )[0];
								if( m_obj.disabled ){
									subItemObj.disabled = true;
								}
								if( m_obj.hidden ){
									subItemObj.hidden = true;
								}
								subItemObj.customAttributes['data-group'] = m_obj.customAttributes.id; //No I18N
								menuObjects.push( subItemObj );	
							}
						}
						m_obj.properties = result;
					}
				}	
				return menuObjects;
			},
			/* Function to construct the menu object with which menu can be created*/
			_constructResultSet: function( option ){
				if( option ){
					var results = [];
					option = Array.isArray( option ) ? option : [ option ];
					for( var i=0; i< option.length ; i++ ){
						var opt = option[i];
						var optData = ZC.getOptions( opt );
						if( opt.tagName === 'OPTION' ){ //No I18N
							optData.id = opt.getAttribute('data-zitemid') || 'zoption-'+ (  opt.id || Math.round(Math.random()*100000000) ); //No I18N
							optData.value = opt.value.trim();
						}else if( opt.tagName === 'OPTGROUP' ){ //No I18N
							optData.id = opt.getAttribute('data-zitemid') || 'zoptgroup-'+ (  opt.id || Math.round(Math.random()*100000000) ); //No I18N
							optData.options = this._constructResultSet( this._childs( opt ) );
						}else{
							continue;
						}
						optData.label = optData.label === undefined?  (opt.label || opt.text) : optData.label;
						optData.title = opt.title;
						optData.style = opt.getAttribute('style'); //No I18N
						optData.className = opt.getAttribute('class') || optData.className; //No I18N
						optData.hidden = optData.hidden || opt.hidden;
						optData.disabled = optData.disabled || opt.disabled;
						opt.setAttribute('data-zitemid', optData.id); //No I18N
						optData.element = opt;
						results.push( optData );
					}
					return results;
				}
			},
			_setIcon: function( mData , display ){
				if( this.iconspan ){
					this.iconspan.innerHTML = ''; //No I18N
					this.iconspan.className = this._CLASS.ICON;
					ZC.createIcon( $(this.iconspan), mData.icon, mData.SVGIconId, mData.iconClassName );
					if( display ){
						this.iconspan.style.display = ''; 
					}
				}
			},
			_setText: function( text ){
				if( this.textspan ){
					this._removeClass( this.textspan, this._CLASS.PLACEHOLDER );
					this.textspan.textContent = text;
				}
			},
			/*Function to find whether an element is disabled*/
			_isDisabled: function( element ){	
				return this._matches( element, '.'+this._CLASS.DISABLED );
			},	
			_setMenuWidth: function(){
				var elementWidth = this.container.offsetWidth;
				var direction = this._menu.data('direction') ? this._menu.data('direction').split('-') : undefined; //No I18N
				/* To add class to remove the border radius */
				if( direction ){
					this._addClass( this.container, direction[0] === 'top'? this._CLASS.DROPUP : this._CLASS.DROPDOWN );
				}
				/* To set the width of the menu to the combo width if (menuwidth < combowidth)*/
				if( this._menu.width() <= elementWidth && !/^(left|right)$/.test( direction ) ){
					if( direction ){
						direction = this._menu.data('direction'); //No I18N
						if( /^(bottom-right|top-right|top|bottom)$/.test(direction)  ){
							var left = parseFloat( this._menu.css('left') ); //No I18N
							left = left - ( elementWidth - this._menu.outerWidth() )*( /^(top|bottom)$/.test( direction )? 0.5:1 );
							this._menu.css('left', left); //No I18N
						}
					}
					this._menu.width(elementWidth - 2); //2 is the correction pixel.
				}	
			},
			_bindEvents: function(){
				$(this.container)
					.on('focusin'+this._namespace, this._onFocus.bind(this) ) //No I18N
				  	.on('click'+this._namespace, this._onClick.bind(this) )	//No I18N
				  	.on('keydown'+this._namespace, this._onKeydown.bind(this) ) //No I18N
				  	.on('focusout'+this._namespace, this._onBlur.bind(this) ); //No I18N
				$(this._closeIcon)
					.on('click'+this._namespace, this._onCloseButtonClick.bind(this) ); //No I18N
			},
			_onCloseButtonClick: function( origEvent ){
				this.deselectAllOptions( origEvent );	
				this._includePlaceholder( );	
				this._lastHoveredItem = undefined;
				return false;	
			},
			_onFocus: function( origEvent ){
				if( !this._focussed ){
					this._focussed = true;
					this._setState();
					this._trigger( "ONFOCUS", origEvent ); 	//No I18N
				}
			},
			_onKeydown: function( origEvent ){
				if( !this._isDisabled( this.container ) ){
					this._keyPress( origEvent );
				}
			},
			_onClick: function( origEvent ){
				if( !this._isDisabled( this.container ) ){
					if( this._menu && this._menu.is(':visible') ){
						this._close( origEvent );
					}else{
						this._open( origEvent );
					}
				}
			},
			_onBlur: function( origEvent ){
				clearTimeout( this.focusouttimer );
				this._focussed = false;
				this.focusouttimer = setTimeout( this._closeHandler.bind(this, origEvent), 1 );
			},
			_closeHandler: function( origEvent ){
				var activeElement = $(document.activeElement);
				if( /*!document.hasFocus()||*/!activeElement.closest( $(this.container).add( this._menu )).length ){
					this._focussed = this._activated = false;
					this._close( origEvent );
					this._lastHoveredItem = this._lastHighlightedItem = false;
					this._trigger( "ONBLUR", origEvent ); 	//No I18N
				}else{
					this._focussed = true;
				}
			},
			_onItemClick: function( origEvent, userdata  ){
				if( userdata.data ){
					userdata.data.menuitem = userdata.menuitem;
				}
				var menuData = userdata.data;
				var option = this._getOptionForMenuItem( menuData.menuitem[0].id );
				var trigger = false;
				if( option ){
					if( option.tagName === 'OPTION' ){
						var previousValue = this.options.selectedValue;
						if( menuData.isFooter ){ 
							this._activated = false;
							this._updateCombobox();
						}else if( this.options.multiple ){
							if( menuData.menuitem.prop('checked') ){ 
								if( !this.options.selectionLimit || this._selected.length < this.options.selectionLimit ){
									this._selectOption( option );
									trigger = true;
								}else{
									this._updateMenuItem( menuData.menuitem , false );
								}
							}else{
								option.selected = false;
								this._remove( this._selected, option );
								this._updateCombobox();
								trigger = true;
							}
						}else{
							this._activated = false;
							trigger = this._selected[0] !== option;
							this._selectOption( option );
							this._showDropdown( false, origEvent ); //added -- The menu close is controlled here.
						}
						var metadata = menuData.properties;
						metadata.selected = option.selected;
						trigger && this._trigger("ONCHANGE", origEvent, {previousValue: previousValue, optionData : metadata, menuitem: menuData.menuitem }); 	//No I18N
						this._trigger("ONITEMCLICK", origEvent, {previousValue: previousValue, optionData : metadata, menuitem: menuData.menuitem }); 	//No I18N
					}else{
						this._updateCombobox();
					} 
				}
				return false;
				// else{
					// return false;
				// }
				// this.setFocus();
			},
			_showDropdown: function(show, origEvent){
				show = show === undefined? true: show;
				show && this._loadDropdown();
				if( !show ){
					this._menu.zmenu('hide', undefined, undefined, undefined, origEvent); // No I18N
				}else{
					this._menu.zmenu('show',{ //No I18N
						forElement: this.container,
						mustBeShown:true,
						event:origEvent,
						positionAlterable: 'flipsidefit' //No I18N
					}, undefined);
				}
			},
			_reloadDropdown: function(){
				this._menu && $(this._queryAll(this._menu[0], 'li, .zmenu__footer')).remove(); //No I18N
				this._optArea.innerHTML = "";
				this._completeMenuLoaded = this.textspan = this.iconspan = undefined;
				//nulling textArray for keyAccess
				this._textArray = [];
				this._prevKey = this._prevTimeStamp = ''; // No I18N
				this._removeClass( this.container, this._CLASS.ICONONLY +' ' + this._CLASS.ICONTEXT );
				this._constructOptArea();
				this.setValue( this.options.selectedValue );
				// this._selected = $( this._selected, this.element[0] ).toArray();
				this._updateCombobox();
				this._close();
			},
			_loadDropdown: function(){
				if( !this._menu ){
					this._createMenu();
				}
				if( !this._completeMenuLoaded ){
					this._populateMenu( this._childs( this.element[0] ) );
					this._completeMenuLoaded = true;
				}
			},
			_updateMenuItem: function(menuitems, checked){
				if( checked ){
					menuitems = menuitems.filter(function( item ){
						return !item.checked;
					});
				}
				this._menu && this._menu.zmenu('setMenuItemsAttribute', menuitems, 'checked', checked);	//No I18N		
			},
			_selectOption: function( option, origEvent ){
				if( option ){
					var selected = $(option).filter('option'); //No I18N
					if( this.options.multiple ){
						selected = selected.add( this._selected );
					}
					this._selected = selected.prop('selected', true).toArray(); //No I18N
				}

				this._updateCombobox( this._selected, origEvent, true );
				this._lastHighlightedItem = this._lastHoveredItem = undefined;
			},
			/* Function to update the value in combobox*/
			_updateCombobox: function( selected, origEvent, updatedata ){
				var o = this.options;
				var base = this;
				selected = selected || this._selected;
				if(this._closeIcon){
					this._closeIcon.style.display = 'none'; //No I18N
				} 

				if( selected && selected.length ){
					var prevValue = o.selectedValue;
					var selectedText = ''; //No I18N
					this._removeClass( this.textspan, this._CLASS.PLACEHOLDER  );
					if( o.multiple ){
						selectedText = [];
						selected.forEach( function( opt ){
							var metaData = ZC.getOptions( opt );
							selectedText.push( metaData.label !== undefined ? metaData.label : opt.label.trim() );
						});
						var options = this._queryAll( this.element[0], 'option:not([hidden]):not([data-separator=true]):not([data-is-footer=true])');
						var unselectedOption = this._getFilteredNodes( function(){
							return !this.selected;
						} , options );
						this._setText( unselectedOption.length ? selectedText.join(', ') : this._localize('allOptionsSelected') ); //No I18N
					}else{
						var option = selected[0];
						var metaData = ZC.getOptions( option );
						metaData.label =  metaData.label !== undefined ? metaData.label : option.label.trim();
						selectedText = [ metaData.label ];
						if( o.contentType === 'custom' && !this._isEditable ){ //No I18N
							this.textspan.innerHTML = this._getCustomContent( metaData, o.template );
						}else{
							this._setText( selectedText[0] );
							this._setIcon( metaData , true );
						}
						this.container.setAttribute('aria-activedescendant', option.getAttribute('data-zitemid')); //No I18N	
					}
					if( o.contentType === 'icon' || this.textspan.clientWidth < this.textspan.scrollWidth ){	//No I18N
						this._optArea.setAttribute('title', selectedText.join(', ') );
					}else{
						this._optArea.removeAttribute('title');
					}	
					if( o.clearSelection ){
						this._closeIcon.style.display = ''; //No I18N
					}
				}else{
					this.container.removeAttribute('title');
					this.container.removeAttribute('aria-activedescendant');
					this._setText('');
					this._includePlaceholder();
				}
				if( updatedata || updatedata === undefined ){
					var prevValue = o.selectedValue;
					this._menu && this._menu.find('li').removeClass('is-selected'); //No I18N
					if( selected && selected.length ){
						o.selectedValue = [];
						o.selectedIndex = [];
						selected.forEach(function( opt ){
							o.selectedValue.push( base._getOptionValue( opt ) );
							o.selectedIndex.push( base._getOptionIndex( opt ) );
						});
						o.selectedValue = o.multiple? o.selectedValue : o.selectedValue[0];
						o.selectedIndex = o.multiple? o.selectedIndex : o.selectedIndex[0];
						if( this._menu ){
							var menuitems = this._getMenuItemForOption( this._selected );
							$(menuitems).addClass('is-selected'); //No I18N
						}
					}else{
						o.selectedValue = this.options.multiple? []:''; //No I18N
						o.selectedIndex = this.element[0].selectedIndex  = -1;
					}
					if( origEvent ){
						this._trigger("ONCHANGE",  origEvent.preventDefault? origEvent : undefined, { 'previousValue':prevValue });	//No I18N
					}
				}
			},
			_getOptionValue: function( opt ){
				if( opt ){
					if( opt.value !== '' ){ //No I18N
						return opt.value;
					}else{
						var optionData = ZC.getOptions( opt );
						if( optionData.value !== undefined ){
							return optionData.value;
						}else{
							return optionData.label;
						}
					}	
				}
			},
		    _getCustomContent: function( data, template ){
		   		return ( this._populateTemplate( template || data.innerHTML, data ) || data.label );
			},
			/*
				#prop# is replaced by actual property value
				previously ##prop## was supported, it will be deprecated
			*/
			_populateTemplate: function( template, data ){
				if( typeof template === 'string' ){ //No I18N
					var props = template.match(/#?#[^# ]+#?#/g);
					if( props ){
						props.forEach( function( prop ){
							template = template.replace( new RegExp( prop, 'g' ) ,function( match ){ //No I18N
								return data[match.replace(/#/g,'')]; //No I18N
							});
						});
					}
					return template;
				}
			},
			_insertOption: function( option, menuitem, action, refEle ){
				if( refEle ){
					if( typeof refEle === 'string' &&  !/^(#)/.test( refEle ) ){ //No I18N
						refEle = this._getOption( refEle )[0];
					}else{
						refEle = $(refEle, this.element)[0];
					}
				}
				if( !action || !refEle ){
					action = 'append'; //No I18N
					refEle = this.element;
				}
				$(refEle)[ action ](option);
				var parent = option[0].parentElement;
				if( !parent || !(/^(OPTGROUP|SELECT)$/.test(parent.tagName) ) ){
					action = 'append'; //No I18N
					this.element.append( option );
					parent = option[0].parentElement;
				}
				if( menuitem && menuitem.length ){
					var refItem = this._menu[0].querySelector('ul');
					if( parent.tagName === 'OPTGROUP' ){
		 				this._menu.zmenu( 'setMenuItemsAttribute', menuitem, 'disabled', parent.disabled );	//No I18N
		 				this._menu.zmenu( 'setMenuItemsAttribute', menuitem, 'hidden', parent.hidden  );	//No I18N   
						for( var i=0; i<menuitem.length; i++ ){
		 					menuitem[i].setAttribute( 'data-group', parent.id );
						} 	
					}
					if( action !== 'append' || refEle.tagName === 'OPTGROUP' ){
						if( refEle.tagName === 'OPTGROUP' && (action === 'after' || action === 'append') ){
							var siblings = $(refEle.children).not( option );
							refEle = siblings.length? siblings[siblings.length - 1] : refEle;
							action = 'after'; //No I18N
						}
						refItem = this._getMenuItemForOption( refEle )[0];
					}
					$(refItem)[action]( menuitem );
				}
			},
			_insertOptionAt: function( option, menuitem, index , group ){
				var baseEle = $(group, this.element).filter('optgroup')[0] || this.element[0];
				var refEle = this._queryAll( baseEle, 'option:not([data-separator=true]):not([data-is-footer=true])')[ index ];
				if( !refEle ){
					this._insertOption( option, menuitem, 'append', baseEle );	//No I18N
				}else{
					this._insertOption( option, menuitem, 'before', refEle );	
				}
			},
			_createOption: function( obj ,isGroup){
				var newOptions = [];
				var base = this;
				$.each( (!$.isArray(obj) ? [obj] : obj), function(i, o){
					var option;
					try{
						option = $(o);
					}catch(e){
						option = undefined;
					}
					if( option && option.length && option[0].nodeType ){
						if( /^(OPTGROUP|OPTION)$/.test( option[0].tagName ) ){
							newOptions.push( option[0] );	
						}
					}else{
						o = typeof o === 'string'? { value:o } : o;	//No I18N
						if( o.label === undefined && o.value === undefined && !o.isFooter && !o.separator ){
							return;
						}
						o.label = o.label === undefined? String( o.value ) : String( o.label );
						if( isGroup || o.options ){
							option = $('<optgroup />',{ id: o.id || undefined, label: o.label }); //No I18N	
							if( Array.isArray( o.options ) ){
								option.append( base._createOption( o.options ) );
							}
						}else{
							option = $('<option />',{   value: o.value, 
														id: o.id || undefined
							}).text( o.label );
							if( o.separator ){
								option.attr('data-separator', true); //No I18N
							}else if( o.isFooter ){
								option.attr('data-is-footer', true); //No I18N
							}
						}
						if( o.title !== undefined ){
							option[0].title = o.title;
						}
						if( o.style ){
							option[0].style = o.style;
						}
						option.prop({disabled:o.disabled, hidden: o.hidden, selected: o.selected});
						$.extend( ZC.getOptions( option ), o);
						newOptions.push( option[0] );
					}
				});
				return newOptions;
			},
			_trigger: function(customEvent, originalEvent, data){
			   data = data || {};
			   data.dropdownList = this._menu;
			   data.selectedValue = data.value = this.options.selectedValue || ''; //No I18N
	       	   data[ this._isEditable? 'combobox':'selectbox' ] = $(this.container); //No I18N
		       return this._dispatchEvent( customEvent, originalEvent, data );
			},
			_getOptionIndex: function( option ){
				return  this._getOptions().indexOf( option );
			},
			_getMenuItemForOption: function( option ){
				var menuitems = [];
				if( this._menu ){
					option = Array.isArray( option ) ? option : [option];
					for( var i = 0; i< option.length; i++ ){
						var menuitem = this._menu[0].querySelector('#' + option[i].getAttribute('data-zitemid') );		
						menuitem && this._add( menuitems, menuitem );
					}
				}
				return menuitems;
			},
			_keyPress: function( origEvent ){
				var code = this.currentKeyCode = origEvent.charCode || origEvent.keyCode;
				var isDropdownVisible = this._menu &&  this._menu.is(':visible'); //No I18N
				if( code === ZC.keyCode.ENTER || code === ZC.keyCode.SPACE ){
					if( !isDropdownVisible){
						origEvent.preventDefault();	
						origEvent.stopPropagation();
						this._open();
						return;
					}
					if( this.options.searchable ){
						if( code === ZC.keyCode.SPACE ){
							return;
						}else{
							if( this._menu[0].querySelector('.zselectbox__emptyresult') ){ //No I18N
								this._close();
								return;
							}
						}
					}
					origEvent.preventDefault();	
				}else if( code === ZC.keyCode.DOWN || code === ZC.keyCode.UP ){
					origEvent.preventDefault();
					origEvent.stopPropagation();
					if( !isDropdownVisible ){
						this._open();
						return;
					}
				}else if( code === ZC.keyCode.ESCAPE || (code === ZC.keyCode.TAB && isDropdownVisible) ){
					//In Firefox, pressing ESC shows old value bug --> https://bugzilla.mozilla.org/show_bug.cgi?id=524360 and the event is prevented.
					origEvent.preventDefault();
					if( isDropdownVisible ){
						origEvent.stopPropagation();
					}
					this._close( origEvent );
					return;
				}else if( !isDropdownVisible ){
					if( !this._isEditable && !this.options.multiple && /(text)/.test( this.options.contentType ) ){
						this._loadDropdown();
						var alphaNumericKey = String.fromCharCode(code);
			            if( this._textArray.length ){
							this._selectItemOnKeydown(alphaNumericKey,(new Date()).getTime());
						}
					}
					return;
				}
				this._handleDropdownKeycode(code);
			},
			_reloadTextArray: function(){
				if( this._completeMenuLoaded ){
					var options = this._queryAll( this.element[0] ,'option'); //No I18N
					var t = this._textArray = [];
					options.forEach(function(p){
						t.push({
							text: p.text.trim(),
							menuid: p.getAttribute('data-zitemid')
						});
					});
				}	
			},
			_selectItemOnKeydown: function(searchKey, currentTime){
				var search = function( start, end, pattern ){
					for( var x = start; x < end; x++){
						if((this._textArray[x].text).match(pattern)){
							menuitem = this._menu[0].querySelector('#' + this._textArray[x].menuid  );		
							if(menuitem && !this._matches(menuitem,'.is-disabled,[hidden]') ){
							  	break;
						  	}
						}
				  	}
				  	return x === end ? undefined : menuitem;	
				}.bind(this);
		    	var startIndex = this.options.selectedIndex + 1;
				var timeDiff = currentTime - this._prevTimeStamp;
			    if( !isNaN( timeDiff ) && timeDiff < 300 && this._prevKey !== searchKey ){
		        	searchKey =  (this._prevKey || '') + searchKey;
		        	startIndex--;
			    }
			    var pattern = new RegExp("^"+(searchKey).replace(/[|()[{.+*?$\\]/g, "\\$0"),"i");
		    	if( isNaN(startIndex) || startIndex >= this._textArray.length ){
		    		startIndex = 0;
		    	}
			    var menuitem = search( startIndex, this._textArray.length, pattern );
			    if( !menuitem && startIndex !== 0 ){
			    	menuitem = search( 0, startIndex, pattern );
			    }
			  	if( menuitem ){
					$(menuitem).trigger('mouseup'); //No I18N
			  	}
			  	this._prevKey = searchKey;
				this._prevTimeStamp = currentTime;  
			},
			_handleDropdownKeycode: function(keyCode){
				this._menu.zmenu('handleKeyCode', keyCode);	//No I18N
			},
			_remove: function( array, item ){
				var index = array.indexOf( item );
				if( index > -1 ){
					array.splice(index, 1);
				}	
			},
			_add: function(array, item){
				var index = array.indexOf( item );
				if( index < 0 ){
					array.push( item );
				} 
			},
			//options == should be an array of option nodes
			_removeOption : function( options ){
				if( options && options.length ){
					options = this._getFilteredNodes(':not([data-readonly=true])', options); //No I18N
					if( options.length ){
						this._selected =  $(this._selected).not( options ).toArray();
						$( this._getMenuItemForOption( options ) ).remove();
						$( options ).remove();
						this._updateCombobox();
						this._reloadTextArray();
					}
					return options;
				}
			},
			/*To include placeholder as a span tag for non-editable combo*/
			_includePlaceholder: function(){
				if( this.iconspan ){
					this.iconspan.style.display = 'none';
				}
				if( this.textspan && !this._isEditable ){
					this._addClass( this.textspan, this._CLASS.PLACEHOLDER );
					this.textspan.textContent = this.options.placeholder;
					this.container.setAttribute( 'aria-placeholder', this.options.placeholder ); 
				}
			},
			_getMatchedOption: function( val, groupselector){
				var base = this;
				val = String( val );
				return this._getFilteredNodes( function(){
					return val === String(base._getOptionValue( this ));
				}, this._getOptions( groupselector ));	
			},
			_getOption: function( value, groupselector ){
				var filteredOps = [];
				if( value !== undefined ){
					value = Array.isArray( value ) ? value : [value];
					for( var i=0; i<value.length; i++ ){
						var v = value[i];
						if( typeof v === 'string' || typeof v === 'number' ){ 
							if( /^(#)/.test( v ) ){
								v = this.element[0].querySelector( v );	
							}else{
							   filteredOps = filteredOps.concat( this._getMatchedOption( v , groupselector ) );
							}	
						}else if( v instanceof jQuery ){
							v = v[0];
						}
						if( v && v.nodeType === 1 && ( v.tagName === 'OPTION' || v.tagName === 'OPTGROUP' ) ){
							this._add( filteredOps, v );
						}
					}
				}else if( groupselector !== undefined ){
					return this._getOptions( groupselector );	 		
				}
				return filteredOps;
			},
			_getOptions: function( group ){
				var options = [];
				var selector = 'option:not([data-separator=true]):not([data-is-footer=true])'; //No I18N
				if( group ){
					group = $(group, this.element)[0];
					if( group ){
						options = this._queryAll( group, selector );
					}
				}else{
					options = this._queryAll( this.element[0], selector );
				}
				return options;
			},
			_getOptionAt: function( index, groupselector ){
				index = $.isArray(index)? index : [index];
				var filteredOps = [];
				var options = this._getOptions( groupselector );
				var base = this;
				index.forEach(function( i ){
					base._add( filteredOps, options[i] );
				});
				return filteredOps;
			},
			_reconstructOptArea: function(){
				this._optArea.innerHTML = "";
				this.textspan = this.iconspan = undefined;
				this._removeClass( this.container, this._CLASS.ICONTEXT+' '+ this._CLASS.ICONONLY  );
				this._constructOptArea();
				if( this.options.disabled ){
					this.option('disabled', true, true); //No I18N
				}
				this._updateCombobox();
			},
			_setState: function(){
				if( this._focussed ){
					// this.setFocus();
					if( this._activated ){
						this._addClass( this.container, this._CLASS.ACTIVE );
						this._removeClass( this.container, this._CLASS.FOCUS );
					}else{
						this._removeClass( this.container, this._CLASS.ACTIVE );
						this._addClass( this.container, this._CLASS.FOCUS );
					}	
				}else{
					this._removeClass( this.container, this._CLASS.FOCUS + ' '+  this._CLASS.ACTIVE );
				}
			},
			_setValue: function( value, groupselector, trigger ){
				if( value !== undefined ){
					var options = this._getFilteredNodes( function(){
										if( !this.hidden && !this.disabled ){
											var parent = this.parentElement;
											if( parent && parent.tagName === 'OPTGROUP' && parent.disabled && parent.hidden ){ //No I18N
												return false;
											}
											return true;	
										}
									}, this._getOption( value, groupselector ));
					if( !options.length ){
						!this.options.multiple && this.deselectAllOptions();
						// this._updateCombobox();
						return false;
					}
					if( this.options.multiple ){
						if( this.options.selectionLimit ){
							options = $(options).not( this._selected ).toArray().slice( this.options.selectionLimit - this._selected.length );
						}
						this._updateMenuItem( this._getMenuItemForOption(options), true );	
					}
					this._selectOption( options, trigger );
				}else{
					this._updateCombobox();
				}
			},
			_deselectOption:function( values, groupselector, trigger ){
				if( this._selected.length ){
					var options = $( this._selected ).filter( this._getOption(values, groupselector) );
					if( options.length ){
						options.prop('selected', false); //No I18N
						if( this.options.multiple ){
							this._updateMenuItem( this._getMenuItemForOption( options.toArray() ) , false);
						}
						this._selected = $(this._selected).not(options).toArray();
						this._updateCombobox( undefined, trigger );
					}
				}
			},
			_close: function( origEvent, deactivate){			
				if( deactivate || deactivate === undefined ){
					this._activated = false;
				}
				if( this._menu && this._menu.is(':visible') ){	//No I18N
					this._showDropdown(false, origEvent);
				}else{
					this._updateSTATE();
				}
			},
			_open: function(  origEvent  ){
				this._showDropdown(true, origEvent );
			},
			_addOption: function( obj, arg1, arg2 , isIndex ){
				if( obj ){
					var options = this._createOption(obj);
					if( options.length ){
						var menuitems = this._completeMenuLoaded ? this._populateMenu( options ) : undefined;
						if( menuitems && menuitems.length ){
							menuitems = this._getFilteredNodes(function( n ){
								return !n.closest( '.zmenu__footer' ); //No I18N
							}, menuitems );
						}
						if( isIndex ){
							this._insertOptionAt( options, menuitems, arg1, arg2 );	
						}else{
							this._insertOption( options, menuitems, arg1, arg2 );	
						}
						if( !this.frominit ){
							// this._updateCombobox();
							this._reloadTextArray();
						}
						return options;
					}
				}
			},
			_setOptionAttribute: function( option, optionName, value ){
				if( option ){
					var menuitem = this._getMenuItemForOption( option );
					var optionData = ZC.getOptions( option );
					var menu = $(this._menu);
					menuitem = menuitem.length? $( menuitem[0] ):undefined;
					var oldValue = optionData[optionName];
					optionData[optionName] = value;
					if( option.tagName === 'OPTGROUP' ){ 
						if( /^(label|disabled|hidden)$/.test( optionName ) ){
							menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
							if(/^(disabled|hidden)$/.test( optionName ) ){
								option[optionName] = value;
								this.setOptionAttributes( this._childs(option), optionName, value );
							}
						}
					}else{
						var selected = option.selected;
						if( optionName === 'value' ){	//No I18N
							option.value = value;
							selected && this._updateCombobox();
						}else if( optionName === 'label' ){	//No I18N
							if( typeof value === 'string' ){
								option.innerHTML = value;
								selected && this._updateCombobox();
								if( menuitem ){
									menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
									menuitem[0].setAttribute('data-filter-label', this.options.searchSettings.caseSensitive? value : value.toLowerCase());	//No I18N
								}
							}else{
								value = option.innerHTML;
							}
						}else if( optionName === 'disabled'  || optionName === 'hidden' ){	//No I18N
							option[optionName] = value;
							menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
							value && this.deselectOption( option, undefined, true );
						}else if( optionName === 'readonly' ){ //No I18N
							option.setAttribute('data-readonly', !!value);
						}else if( optionName === 'selected' ){ //No I18N
							if( value ){
								this._setValue( option );
							}else{
								this.deselectOption( option );
							}
							return;
						}else if( optionName === 'customAttributes' ){ //No I18N
							if( typeof value === 'object' ){ //No I18N
								Object.keys( value ).forEach( function( key ){
									if( menuitem && menuitem.length ){
										if( key === 'class' ){ //No I18N
											option['class'] && menuitem.removeClass( option['class'] ); //No I18N
											if( oldValue && oldValue['class'] ){	 //No I18N
												menuitem.removeClass( oldValue['class'] ); //No I18N
											}
											menuitem.addClass( value[key] );
										}else{
											menuitem[0].setAttribute( key, value[key] );
										}
									}
									option.setAttribute( key, value[ key ] );
								});
								optionData.customAttributes = $.extend( oldValue || {}, value );
							}
						}else{
							menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
							selected && this._updateCombobox();
						}
					}			
				}
			},
			_setAttribute: function( optionName, value ){
				var menu = $(this._menu);
				var o = this.options;
				switch(optionName){
					case 'disabled': // No I18N
						this._close();
						o[optionName] = value;
						if( value ){
							 $( this.container ).addClass( this._CLASS.DISABLED ).attr('aria-disabled', true).removeAttr('tabindex').removeClass( this._CLASS.ACTIVE ).removeClass( this._CLASS.FOCUS ); //No I18N
						}else{
							 $( this.container ).removeClass( this._CLASS.DISABLED ).removeAttr('aria-disabled').attr('tabindex', this.element.attr('tabindex') || 0); //No I18N
						}
						break;
					case 'selectedIndex': //No I18N
						this._setAttribute('selectedValue', this._getOptionAt( value )); //No I18N
						break;
					case 'selectedValue': //No I18N
						this.deselectAllOptions();
						this.setValue( value );
						break;
					case 'dataSource': //No I18N
						var selectedValue = this.options.selectedValue; // preserving the selected value since same dataSource can be given with some modifications.
						o[optionName] = value;
						this.removeOption();
						this._selected = [];
						this._handleDS();
						this.setValue(selectedValue);
						break;
					case 'placeholder': //No I18N
						o[ optionName ] = value;
						if( this.textspan && o.selectedIndex === -1 ){
							this._includePlaceholder();
						}
						break;
					case 'contentType': //No I18N
						if( /^(text|icon|icon-text|custom)$/.test( value ) ){
							if( !o.multiple || value === 'text' ){
								o[ optionName ] = value;
								menu.zmenu('setAttribute', 'contentType', o.multiple? 'icon-text': value );	//No I18N
								this._reloadDropdown();	
							}
							break;
						}
					case 'dropdownList': //No I18N
						var keys = Object.keys(value);
						keys.forEach(function(key, index){
							if(/^(width|height|appendTo|className)$/.test(key)){
								o.dropdownList[key] = value[key];
								menu.zmenu('setAttribute', key, value[key]); //No I18N
							}
						});	
						break;
					case 'multiple': //No I18N
						value = !!value;
						if( value !== o[optionName] ){
							o[optionName] = value;
							if( value ){
								this._removeClass( this.container, this._CLASS.ICONONLY );
								this.element[0].setAttribute('multiple', true);
								o.contentType = 'text'; //No I18N
								menu.zmenu('setAttribute', 'contentType', 'icon-text');	//No I18N
							}else{
								this._selected = this._selected.slice(-1);
								this.element[0].removeAttribute('multiple');  //No I18N
								menu.zmenu('setAttribute','contentType',o.dropdownList.contentType || o.contentType ); //No I18N
							}
							this._reloadDropdown();
						}
						break;
					case 'selectionLimit': //No I18N
						value = parseInt( value );
						if( !isNaN( value )  || value >= 0 ){
							o[optionName] = value;
							if( o.multiple && value && this._selected.length > value ){
								this.deselectOption( this._selected.slice( 0, this._selected.length - value ) );
							}
						}
						break;
					case 'autoFill': //No I18N
						o[optionName] = !!value;
						break;
					case 'clearSelection': //No I18N
						o[optionName] = !!value;
						this._updateCombobox();
						break;
					case 'selectOnClose': //No I18N
						o[optionName] = !!value;
						break;
					case 'searchable': //No I18N
						o[optionName] = !!value;
						if( this._menu ){
							this._menu.zmenu('setAttribute','handleAccessKeyNavigation', !value ); //No I18N
							if( value ){
								this._createSearchBox();
							}else{
								this._menu.find('.zmenu__header').remove(); //No I18N
							}
						}
						break;
					case 'searchSettings': //No I18N
						var base = this;
						var keys = Object.keys(value);
						keys.forEach(function(key, index){
							if(/^(delay|searchCriteria|caseSensitive|minKeywordLength)$/.test(key)){
								if(key === 'searchCriteria'){ //No I18N
									if(!/^(startswith|contains)$/.test(value[key]) || ( o.autoComplete ) ){
										return true;
									}
								}else if(key === 'caseSensitive'){//No I18N
									if( base._menu ){
										base._getFilteredNodes( function( n ){
											var label = ZC.getOptions( n ).label;
											n.setAttribute('data-filter-label', value[key]? label : label.toLowerCase());
										}, base._queryAll( base._menu[0], 'li[data-filter-label]' ));
									}
								}
								o.searchSettings[key] = value[key];
							}
						});	
						break;
					default:
				}		
			},
			 /*JQUERY HELPER CLASSES*/
		 	_getFilteredNodes: function( filter, nodeList ){
				nodeList = Array.isArray( nodeList ) ?  nodeList : [nodeList];
				var filteredNodes = [];
				for( var i=0; i<nodeList.length; i++ ){
					var isMatched, n = nodeList[ i ];
					if( n && n.nodeType ){
						if( typeof filter === 'string' ){
							isMatched = this._matches( n, filter );	
						}else if(  typeof filter === 'function' ){ //No I18N
							isMatched = filter.call( n, n );
						}
						if( isMatched ){
							this._add( filteredNodes, n );
						}
					}
				}
				return filteredNodes;
		   },
		   _matches: function(node, selector){
		   		var p = Element.prototype;
				var f = p.matches || p.webkitMatchesSelector || p.mozMatchesSelector || p.msMatchesSelector || function(s) {
					return [].indexOf.call(document.querySelectorAll(s), this) !== -1;
				};
				return f.call(node, selector);
		   },
		   _queryAll: function( node, selector ){
		   	 	return this._toArr(  node.querySelectorAll( selector ) );
		   },
		   _childs: function( node ){
		    	return this._toArr(  node.children );
		   },
		   _toArr: function( list ){
		   		return Array.prototype.slice.call( list, 0 );
		   },
	      _removeClass: function( node, className ){
				if( node && node.nodeType === 1 && typeof className === 'string'  ){ //No I18N
					var nowhitespace = /[^\x20\t\r\n\f]+/g;
					var clsList = className.match( nowhitespace );
					clsList.forEach( function( clazz ){
			   			node.className = node.className.replace( new RegExp("(([ ]|^)"+ clazz +"([ ]|$))" ,'g'), ' '); //No I18N
					});
				}
				return node;
		   },
		  	_addClass: function( node, className ){
				if( node && node.nodeType === 1 && typeof className === 'string' && className ){ //No I18N
					var nowhitespace = /[^\x20\t\r\n\f]+/g;
					var clsList = className.match( nowhitespace );
					var curClassList = node.className.match( nowhitespace ); 
					clsList.forEach( function( clazz ){
						if( curClassList.indexOf( clazz ) === -1 ){
							curClassList.push( clazz );
						}
					});
					node.className = curClassList.join(" ");
				}
				return node;
		   },
				/****PUBLIC FUNCTIONS****/
			/***********************/
			/** Method to set focus to the component.
				@function combo-box.setFocus
				@newlyAdded true
			*/
			setFocus: function(){
				this.container.focus();
				!this._isDisabled( this.container ) && this._onFocus();
			},
			/** Method to close the dropdown list.
				@function combo-box.close
			*/
			close: function(){
				this._close();
			},
			/** Method to open the dropdown list.
				@function combo-box.open
			*/
			open: function(){
				this._open();
			},
			/** 
				Method to re-render the dropdown list.
				@function combo-box.refresh
			*/
			refresh: function(){
				ZC.clearMetadata( this._queryAll( this.element[0], 'option,optgroup' ) ); //No I18N
				this._reloadDropdown();
			},
			/** This method adds item/items to the dropdown.
				@function combo-box.addOption
				@param {Object} items Array of item Properties or HTMLOptions that is/are to be added. 
				@param {String} action 
				Specifies whether to insert the new option after or before or to be appended to the element(option/optgroup) given in the third argument .
				Valid values are 'after', 'before' and 'append'.
				@param {Object|String} selector Selector or DOMElement of the option or optgroup relative to which the new option is placed based on the argument 'action'.
				@returns {Object} Newly added options.
				@example <caption> Option added using item-properties </caption>
	$(selector).zcombobox('addOption',[{
		label:'India',
		value:'IND',
	}], 'after', 'AF')
				@example <caption> Option added using HTML </caption>
	$(selector).zcombobox('addOption',$('<option value='IND' data-icon='flags ind'>India</option>'), 'after', 'AF');
				@example <caption> To insert an option to a group </caption>
	$(selector).zcombobox('g',[{
		label:'FastTrack',
		value:'FT'
	},{
		label:'Titan',
		value:'TT'
	}],'append','#watchBrands')
			*/
			addOption: function(obj, action, refElement){
				return this._addOption.apply( this, arguments );
			},
			/** This method adds option/options to the dropdown at the specified index( Index is Zero based excluding the separators).
				@function combo-box.addOptionAt
				@param {Object} items Array of item Properties or HTMLOptions that is/are to be added. 
				@param {Number} [Index] Specifies the location at which the new item/items is/are inserted.
				@returns {Object} Newly added items.
				@example <caption> Option added using item Properties </caption>
	$(selector).zcombobox('addOption',{
		label:'India',
		value:'IND'
	},4)
				@example <caption> Option added using HTML </caption>
	$(selector).zcombobox('addOptionAt',$('<option value='IND' data-icon='flags ind'>India</option>'),4);
			*/
			addOptionAt: function(obj, index, group){
				return this._addOption( obj, index, group, true );
			},
			/** This method adds groups to the dropdown list.
				@function combo-box.addOptGroup
				@param {Object} group
				Array of Group properties or HTMLOptGroups to be added to the dropdown list. Refer 'group Properties' for the available options.
				@param {String} Value/#option-id Selector of the item or group before which the new group is added. If not mentioned, the new group is appended to the dropdown list.
				@returns {Object} Newly added groups.
				@example <caption> Group added using group-properties </caption>
	$(selector).zcombobox('addOptGroup',{
		id:'BagBrands',
		label:'Bag Brands',
		options:['WildCraft','Fastrack']
	},'#WatchBrands')
			*/
			addOptGroup: function(obj, refElement){
				if(obj){
					var groups = this._createOption( obj , true );
					if( groups.length ){
						var menuitems = this._completeMenuLoaded ? this._populateMenu( groups ) : undefined;
						this._insertOption( groups	,menuitems, 'before', refElement );	//No I18N
					}
					return groups;
				}
			},
			/** This method removes the item/items from the dropdown list given the value/#optionid of the items.
				@function combo-box.removeOption
				@param {String|Object} value Specifies the value/#optionid or array of values/#optionids whose items are to be removed. If not mentioned, all items are removed.
			*/
			removeOption: function(value, groupselector){
				return this._removeOption( ( value === undefined && groupselector === undefined )? this._queryAll( this.element[0], 'option,optgroup' ) : this._getOption(value, groupselector)  );
			},	
			/** This method removes the item/items located at the specified index/indexes.
				@function combo-box.removeOptionAt
				@param {Number|Object} index Specifies position or array of positions of item/items that is/are to be removed. Index is 0 based and the separators are excluded.
			*/
			removeOptionAt: function(index, groupselector){
				return this._removeOption( this._getOptionAt(index, groupselector) );
			},
			/** This method removes the group given the groupselector.
				@function combo-box.removeOptGroup
				@param {String} group-selector Selector of the group to be removed.
			*/
			removeOptGroup: function( groupselector ){
				var g = this._getOption( groupselector );
				var base = this;
				g.forEach(function( _g ){
					base._removeOption( base._childs( _g ) );
					if( !base._childs(_g).length ){
						base._removeOption( [_g] );
					}
				});
			},
			/** Method to select the item that matches given the value/#option-id. This method is valid only when multiple is not set to true.
				@function select-box.setValue
				@param {String} Value  Value/#option-id of the item that is to be selected.
			*/
			/** Method to select the item that matches given the value/#option-id of the option.
				@function combo-box.setValue
				@param {String} Value  Value/#option-id of the item that is to be selected.
			*/
			setValue: function(value, groupselector, trigger){
				// !this.options.multiple && this.deselectAllOptions();
				this._setValue( value, groupselector, trigger );
				return this.options.selectedValue;
			},
			/** Method to select the items that matches the given array of values/#option-ids. Method is valid only when multiple is set to true.
				@function select-box.setValues
				@param {String} Values  Array of values or #option-ids of the items that are to be selected.
			*/
			setValues: function( values, groupselector,trigger ){
				return this.setValue.apply( this, arguments );
			},
			/**
			Method to clear the selection.
				@function combo-box.deselectAllOptions
			*/
			deselectAllOptions: function( trigger ){
				this._deselectOption( this._selected, undefined, trigger );	
			},
			/** Method to deselect a particular item given the value/#option-id.
				@function combo-box.deselectOption
				@param {Object} value  Value/#option-id or array of Values/#option-ids  is/are to be deselected.
			*/
			deselectOption: function( value, groupselector, trigger ){
				this._deselectOption.apply( this, arguments );
			},
			/** This method destroys the component instance.
				@function combo-box.destroy
			*/
			destroy: function(){
				if( this._menu ){
					this._menu.zmenu('destroy'); //No I18N
					this._menu.remove();	
				}
				$(this.container).remove();
				this.element.show().removeData('zcombobox').removeData('zselectbox').removeData('metadata') //No I18N
							.children().removeData('metadata'); //No I18N
			},
			setOptionAttributes: function( options, optionName, value ){
				if( typeof optionName === 'object' && !$.isEmptyObject(optionName) ){	// No I18N
					var base = this;
					$.each(optionName, function( o,v ){
						base.setOptionAttributes(options, o ,v );
					});
				}else if( optionName && value !== undefined ){
					options = this._getOption( options );
					for( var i=0; i<options.length; i++ ){
						if(   this._matches( options[i], ':not([data-readonly=true])' ) ){
							this._setOptionAttribute( options[i], optionName, value );
						}
					}
				}
			},
			setAttribute: function( optionName, value , force){
				if( value !== this.options[optionName] || optionName === 'dataSource'  || force ){
					value = typeof value === 'string'? value.trim() : value; //No I18N
					this._setAttribute( optionName, value );
				}
			},
			/** Method to return the selected value.
				@function combo-box.getValue
				@returns {string} Currently selected value.
			*/
			getValue: function(){
				return this.options.selectedValue;
			},	
			/** Method to return the selected values. Valid only when multiple is set to true.
				@function select-box.getValues
				@returns {string} Currently selected values.
			*/
			getValues: function(){
				if( this.options.multiple ){
					return this.getValue();
				}
			},
			/** Method to return the created component.
				@function select-box.getValues
				@returns {Object} Currently the component.
			*/
		};
		ZC.registerComponent("zselectbox", selectboxPrototype); //No I18N
		var isIE9 = ( ZC.Browser.getIEVersion() > 9 || ZC.Browser.getIEVersion() === -1 ) ? false : true;
		ZC.zcombobox = { }

			/** @member {Object} [combo-box.dataSource = undefined]

				To specify the data source from which dropdown list has to be populated. DataSource is an array of itemProperties. Refer 'itemProperties' list for the list of properties that can be applied to an item.

				@example <caption> To populate using dataSource</caption>

	$(selector).zcombobox({
		contentType : 'custom',
		dataSource:[{
			label:'Mathew',
			icon:'employeeicon mathew_img'
		},{
			label:'James',
			icon:'employeeicon james_img'
		},{
			label:'Rose',
			icon:'employeeicon james_img'
		}]
	});
			*/
	        /** @member {number} [combo-box.selectedIndex = 0] 
	    		Specifies the index of the initially selected option. Index is zero based excluding separators( items with property 'separator' = true ). By default, first option is selected.
	        */
	        /** @member {String} [combo-box.selectedValue = undefined] 
	        	Specifies the initially selected value. SelectedValue is given priority over selectedIndex.
	        */
			/** @member {String} [combo-box.placeholder = "Select"]
				The text displayed when none of the values are selected. 
			*/
			/** @member {Object} 
				combo-box.searchSettings Specifies the search settings. Search is done based on the item label.
				@example <caption>  Search Settings </caption>
	$(selector).zcombobox({
		searchSettings:{
			delay:20,
			searchCriteria: 'contains',
			caseSensitive: true,
			minKeywordLength:3
		}
	});
			*/
			   /**  @member {Number} [combo-box.searchSettings.delay = 150] 
			   		Specifies the search delay.
			   	*/
		       /** @member {String} [combo-box.searchSettings.searchCriteria = 'startswith'] 
		       	   Determines the search method that can be used to match the typed text.
		       	   @allowedValues {('contains'|'startswith')}
		       */              
		       /** @member {Boolean} [combo-box.searchSettings.caseSensitive = false]  
		       	   Specifies whether searching is case sensitive or not. 
		       	*/          
		       /** @member {Number} [combo-box.searchSettings.minKeywordLength = 0] 
		           Specifies the minimum character the user must type in the combobox, before the filtered list is displayed. 
		       */  
	        /** @member {Boolean} [combo-box.autoComplete = 'false'] 
	        	Specifies whether suggestion text is highlighted in the combobox when options are visited( hovered ) using arrow keys.
	        */
	        /** @member {object} combo-box.dropdownList Configurations for the dropdown list. */
	        	/** @member {Number} [combo-box.dropdownList.height = 'auto'] Determines the height of the list. */
	        	/** @member {Number} [combo-box.dropdownList.width = 'auto'] Determines the width of the list. */
	        	/** @member {(Object|String)} [combo-box.dropdownList.appendTo = 'body'] Selector or HTMLElement of the container to which the dropdown list is appended. */
	    		/** @member {String} combo-box.dropdownList.className Custom classname that is added to the dropdown list. */
	    		/** @member {String} [combo-box.dropdownList.direction='auto'] The direction in which the dropdown list opens. If the value is 'auto', it opens in bottom-left direction.
	    			@allowedValue {('top'|'bottom'|'left'|'right'|'bottom-left'|'bottom-right'|'top-left'|'top-right'|'auto')}
	    		*/
	        /** @member  {Boolean} [combo-box.disabled = 'false'] Determines whether the combobox is disabled. */
	        /** @member {String} [combo-box.contentType = 'text'] 
	Determines whether the combobox and the dropdown list has only text or both icon and text. If set to 'custom', the dropdown list can have custom HTML, but the combobox is populated with the label of the selected item.
	If contentType is set to 'custom', label must be given,and the HTML content is given in 'innerHTML' option of each item and any '#label#' in the html is replaced by the item label.
	        	@allowedValues {('text'|'icon-text'|'custom')}
	        	@example <caption> Example using contentType = 'custom' </caption>
	$('#employees').zcombobox({
		contentType:'custom',
		dataSource: [{
			label: 'Mathew',
			innerHTML:'<span class="employee_name">#label#</span><span class='employee_designation'>Designer</span>'
		},{
			label: 'James',
			innerHTML:'<span class='employee_name'>#label#</span><span class='employee_designation'>Developer</span>'
		},{
			label: 'Rose',
			innerHTML:'<span class='employee_name'>#label#</span><span class='employee_designation'>Designer</span>'
		}]	
	});
	        	@example <caption> Example using contentType='icon-text' </caption>
	<select id='employees' data-content-type='icon-text'>
		<option data-icon ='employeeicon mathew_img' >Mathew</option>
		<option data-icon = 'employeeicon james_img'>James</option>
		<option data-icon = 'employeeicon james_img'>Rose</option>
	</select>

	        */
    	/** @member {String} [combo-box.arrowIconClassName] - One or more space separated CSS classes, which contains configuration about the arrow icon to be shown in the menu button. 
			@newlyAdded true
		@example <caption> Example: </caption>
	    $("#country").zcombobox({
	        "arrowIconClassName" : "zicon zicon-arrow-right"
	    });
			*/
	        /** @member {String} [combo-box.arrowSVGIconId] - The SVG element Id followed by space separated CSS class names to replace the default arrow icon. 
			@newlyAdded true
		@example <caption> Initialize the menu button with the "arrowSVGIconId" option specified: </caption>
	    $("#country").zcombobox({
	        "arrowSVGIconId" : "arrowIconID custom__class"
	    });
	        */
			/** @member {String} combo-box.className Custom classname that is added to the combobox component.*/
		var comboboxPrototype =  {
			options: $.extend( {}, selectboxPrototype.options, {
				searchSettings: {
			       delay: 150,  
			       searchCriteria: 'startswith',  //No I18N  
			       caseSensitive: false,             
			       minKeywordLength: 0,
			       searchByField: 'label' //No I18N
			     },
			     autoComplete: false,
		         autoFill: true, 
		         messages:{
		        	noResults:'No Results'	//No I18N
		         },
				 eventPrefix:'zcombobox'	//No I18N
			}),
			_initialize: function( element, options ){
				this._isEditable = true;
				options.searchable = options.multiple = false;
				if( options.autoComplete ){
					options.searchSettings.searchCriteria = 'startswith'; //No I18N
				}
				if( !/^(text|icon-text|custom)$/.test( options.contentType ) ){
					options.contentType = $.zcombobox.prototype.options.contentType;
				}
			},
			_constructComponent: function(){
				this._super();
				this._addClass( this.container, 'zcombobox' ); //No I18N
			},
			_constructOptArea: function(){
				var o = this.options;
				if( this.options.contentType.match('icon') ){
					var i = this.iconspan = document.createElement('i');
					i.className = this._CLASS.ICON;
					this._optArea.appendChild( i );
				}
				var t = this.textspan = document.createElement('input');
				t.className = 'zselectbox__textbox';
				t.setAttribute('placeholder', o.placeholder );
				var tabindex = this.element[0].getAttribute('tabindex');
				if( tabindex !== undefined ){
					t.setAttribute('tabindex', tabindex );
				}
				this._optArea.appendChild( this.textspan );
				this.container.setAttribute('tabindex', -1);
				this._bindSearchbox( this.textspan );
			},
			_onMenuItemHover: function( origEvent, data ){
				if( this._super.apply( this, arguments ) ){
					this.textspan.select();
				}
			},
			_onFocus: function( origEvent ){
				if( !this._focussed ){
					this.textspan.select();
				}
				this._super( origEvent );
			},
			_onClick: function( origEvent ){
				var _super = this._super;
				if( !this._isDisabled( this.container ) && $(origEvent.target).closest( this._arrow ).length  ){
					_super.call( this );
					this.textspan.select();
					this._clearStyle();	
				}
			},
			_bindEvents: function(){
				var base = this;
				this._super();
				var $textspan = $(this.textspan);
				$textspan.on('mousedown'+this._namespace, function( origEvent ){ //No I18N
					if( !base._focussed ){
						$textspan.on('mouseup'+base._namespace, function( origEvent ){ //No I18N
							$textspan.off('mouseup'+base._namespace).select(); //No I18N
							origEvent.preventDefault();
						});
					}
					base._onFocus( origEvent );
				}); 	
			},
			_onCloseButtonClick: function( origEvent ){
				this.deselectAllOptions( origEvent );	
				this._lastHoveredItem = undefined;
				this._setText(''); //No I18N
				if( this._activated ){
					this._onInput( this.textspan, origEvent );
				}
				this.textspan.focus();
				return false;
			},
			_setText: function( text ){
				if( this.textspan ){
					this.textspan.value = text;
				}
			},
			_onInput: function(){
				if( this.iconspan ){
					this.iconspan.style.display = 'none'; //No I18N
				}
				this._super.apply(this, arguments);
				// this._removeClass( this.container, this._CLASS.FOCUS );
				// this._addClass( this.container, this._CLASS.ACTIVE );
			},
			_searchQuery: function( origEvent, query ){
				var s = this.options.searchSettings || {};
				var minKeywordLength = Number(s.minKeywordLength)!=='NaN'? Number(s.minKeywordLength) : 1;	//No I18N
				if( !minKeywordLength || query.length >= minKeywordLength ){
					this._super( origEvent, query );
				}else{
					this._close( origEvent, false );
				}
			},
			_setValue: function( value, groupselector, origEvent ){
				if( this._super( value, groupselector, origEvent ) === false && /(string|number)/.test( typeof value ) ){
					if( this.options.acceptNewValues && value !== this._selected ){
						var previousValue = this._selected;
						this._selected = this.options.selectedValue = value;
						this.options.selectedIndex = this.element.selectedIndex = -1;
						this._setText( value );
						origEvent && this._trigger( "ONCHANGE", origEvent, { previousValue: previousValue } ); 	//No I18N
					}
				}
			},
			_keyPress: function( origEvent ){
				var code = this.currentKeyCode = origEvent.charCode || origEvent.keyCode;
				if( code === ZC.keyCode.ENTER && this.options.acceptNewValues ){
					this._selectMenuItemByContent( this.textspan.value, origEvent );
				}else if( code !== ZC.keyCode.SPACE ){
					this._super( origEvent );
				}
			},
			_selectMenuItemByContent: function( label, origEvent ){
				if( this._menu ){
					var menuitem = this._getItemByLabel( label );
					if( menuitem ){
						$(menuitem).trigger('mouseup'); //No I18N
					}else{
						this._setValue( this.textspan.value, undefined, origEvent );
						!this.options.multiple && this._close();
					}	
				}
				return;
			},
			_getItemByLabel: function( query ){
				if( this.options.searchSettings.searchByField === 'label' ){ //No I18N
					return this._menu.find('li:not([hidden])[data-filter-label="'+ (this.options.searchSettings.caseSensitive? query: query.toLowerCase()) +'"]')[0]; //No I18N	
				}else{
					var base = this;
					this._queryAll( this._menu[0],'li:not([hidden])[data-filter-label]').forEach(function( node ){ //No I18N
						var menuData = ZC.getOptions( node );
						if(!( base._menuitemMatcher( menuData , query )  ) ){
							return node;
						}	
					});
				}
			},
			/*Function to autoComplete Combo*/
			_autoComplete: function( currentText, matchedItem ){			
				if( this.options.autoComplete && currentText.length >= this.options.searchSettings.minKeywordLength && currentText ){
					var keyCode = this.currentKeyCode;
					if( currentText && currentText !== '' && matchedItem ){
						var mData = ZC.getOptions( matchedItem );
						this._setIcon( mData, true );
						if(keyCode !== ZC.keyCode.BACKSPACE && keyCode !== ZC.keyCode.DELETE){
							this._setText( mData.label );
							var start = mData.label.toLowerCase().indexOf(currentText.toLowerCase()) + currentText.length;
							this.textspan.setSelectionRange(start, mData.label.length);
						}
						this._lastHoveredItem = this._getOptionForMenuItem( matchedItem.id );
					}else if( this.iconspan ){
						this.iconspan.style.display = 'none';
					}	
				}
			},
			setFocus: function(){
				if( this._focussed && this._activated ){
					this.textspan.focus();
				}else{
					this.textspan.select();
				}
			},
			_setAttribute: function( optionName, value ){
				if( optionName === 'disabled' ){
					this._close();
					this.options[optionName] = value;
					if( value ){
						$(this.container).addClass( this._CLASS.DISABLED ).attr('aria-disabled', true).removeAttr('tabindex'); //No I18N
						$(this.textspan).prop('disabled', true).removeClass( this._CLASS.ACTIVE ).removeClass( this._CLASS.FOCUS ); //No I18N
					}else{
						$(this.container).removeClass( this._CLASS.DISABLED ).removeAttr('aria-disabled').attr('tabindex', -1); //No I18N
						$(this.textspan).prop('disabled', false); //No I18N
					}
				}else if( optionName === 'placeholder' ){ //No I18N
					this.options[optionName] = value;
					this.textspan.setAttribute('placeholder', value); //No I18N		
				}else if( optionName === 'autoComplete' ){ //No I18N
					this.options[optionName] = !!value;
					if( value ){
						this.options.searchSettings.searchCriteria = 'startswith'; //No I18N
					}
				}else{
					if( /^(multiple|selectionLimit|searchable)$/.test( optionName ) || ( optionName === 'contentType' && !/^(text|icon-text|custom)$/.test(value) ) ){
						return;
					}
					this._super( optionName, value );
				}
			}
		};
		ZC.zcombobox = {
			DEFAULTS: comboboxPrototype.options
		};
	   	ZC.registerComponent("zcombobox", $.zselectbox, comboboxPrototype); //No I18N
	   	var createSelectbox = function( options, baseFunction ){
	  		if(typeof options === 'object'){ //No I18N
				var element = $('#' + options.id); //No I18N
				if( element.length ){
					if(element.data( baseFunction )){
						return element;
					}
				}else{
					element = $('<select>').appendTo( $(options.appendTo).length? options.appendTo:'body' ) // No I18N
										   .attr({
											   	id: options.id,
												style: options.style,
												title: options.title   
											}); 
				}
				element[ baseFunction ](options);
				return element;
			} 	
	   }
	   ZC.createCombobox = function(options){
			createSelectbox( options, 'zcombobox' ); //No I18N
	   }
	   ZC.createSelectbox = function(options){
			createSelectbox( options, 'zselectbox' ); //No I18N
	   }
	})( ZComponents );
	// API
	/** @options combo-box.item-properties This lists properties of each items available in the dropdown list.
		These properties can be set to HTMLOptions as attributes and the custom attributes are prefixed with 'data-'.
		@example <caption> Setting Properties to HTMLOptions </caption>
	<select id='country'>
		<option data-icon='af' value='AF'>Afghanistan</option>
		<option selected data-icon='al' value='AL'>Algeria</option>
		<option data-separator = true></option>
		<option disabled data-icon='Ind' selected=true value='IND'>India</option>
		<option hidden data-icon='us' value='US'>United States</option>
	</select>
	$('#country').zcombobox();
			@example <caption> Setting properties in the datasource </caption>
	<select id='country'></select>
	$('#country').zcombobox({
		dataSource:[{
			label:'Afghanistan',
			value:'AF',
			icon:'af'
		},{
			label:'Algeria',
			value:'AL',
			icon:'al'
		},{
			separator:true
		},{
			label:'India',
			value:'IND',
			icon:'ind'
		},{
			label:'United States'
			value:'US'
			icon:'us'
		}]
	});
		*/
		/** @member {Boolean} [combo-box.item-properties.separator = false] 
			Specifies whether the item is a separator. Separators are not selectable.
		*/
		/** @member {String} [combo-box.item-properties.value = null] 
			Specifies the value.
		*/
		/** @member {String} [combo-box.item-properties.label = null]
			Specifies the label.
		*/
		/** @member {String} [combo-box.item-properties.icon = undefined] 
			Specifies the icon. Use iconClassName / SVGIconId instead.
		    @deprecated
		*/
		/** @member {String} [combo-box.item-properties.SVGIconId = undefined] 
			Specifies SVG element Id followed by space separated CSS class names.
		    @newlyAdded
		*/
		/** @member {String} [combo-box.item-properties.iconClassName = undefined] 
			One or more space separated CSS classes, which configures the icon.
		    @newlyAdded
		*/
		/** @member {String} [combo-box.item-properties.description = undefined]
			Specifies the secondary text for the item. 
		*/
		/** @member {String} [combo-box.item-properties.innerHTML = undefined]
			Specifies the html content when contentType='custom' or when isFooter=true for a particular item. When used as customHTML for contentType='custom, any "#label#" in the content is replaced by item label and items are preferred to be given in datasource rather than in HTML. Refer option 'contentType' for example.
		*/
		/** @member {(String|Object)} [combo-box.item-properties.customData = undefined]
			User required information can be given here. It is passed on to the user when the particular item is selected.	
		*/
		/** @member {String} [combo-box.item-properties.id = undefined]
			Unique id of the item.
		*/
		/** @member {Boolean} [combo-box.item-properties.readonly = false]
		   If set to true , removal and updation of this particular option is prevented.
		*/
		/** @member {Boolean} [combo-box.item-properties.disabled = false]
			If set to true, the item is disabled.
		*/
		/** @member {Boolean} [combo-box.item-properties.hidden = false]
			If set to true, the item is hidden.
		*/
		/** @member {Boolean} combo-box.item-properties.isFooter
			Specifies whether this item is added as footer and the html content of the footer is given in 'innerHTML' property.
		*/
		
	// OPTGROUP OPTIONS
	/** @options combo-box.group-properties This lists properties of each group available in the dropdown list.
		These properties can be set to HTMLOptGroup as attributes inside select tag and the custom attributes are prefixed with 'data-'.
		@example <caption> Groups represented in HTML </caption>
	<select id='laps'>
		<optgroup label="Lenovo" id="lenovolaps">
			<option selected=true value="Lenovo IdeaPad">Lenovo IdeaPad</option>
			<option value="Lenovo UltraBook">Lenovo UltraBook</option>
		</optgroup>
		<optgroup label="HP" id="hplaps">
			<option value="HP Pavilion Notebook">HP Pavilion Notebook</option>
			<option value="HP ENVY Notebook">HP ENVY Notebook</option>
		</optgroup>
	</select>
	$('#laps').zcombobox();
		@example <caption> Groups added using JSON </caption>
	<select id='laps'></select>
	$('#laps').zcombobox({
		dataSource:[{
			label: 'Lenovo',
			id:'lenovolaps',
			options:[{
				label:'Lenovo IdeaPad',
				value:'Lenovo IdeaPad'
			},{
				label:'Lenovo UltraBook',
				value:'Lenovo UltraBook'
			}]
		},{
			label:'HP',
			id:'hplaps',
			options:[{
				label:'HP Pavilion Notebook',
				value:'HP Pavilion Notebook'
			},{
				label:'HP ENVY Notebook',
				value:'HP ENVY Notebook'
			}]
		}]
	});
		*/
		/** @member {String} [combo-box.group-properties.label = null]
			Specifies the group label.
		*/
		/** @member {String} [combo-box.group-properties.id = undefined]
			Unique id of the group.
		*/
		/** @member {Boolean} [combo-box.group-properties.disabled = false]
			If set to true, the group is disabled.
		*/
		/** @member {Boolean} [combo-box.group-properties.hidden = false]
			If set to true, the group is hidden.
		*/
		/** @member {Object} combo-box.group-properties.options 
			Array of items that are to be added to this group.This property is used only when the group is added using JSON.
		*/
	//Updation option
			/** This method is used to update dropdown list items.			
				Properties that can be updated for an item are value, readonly, icon, textAsIcon, label, innerHTML, description, hidden, disabled, customData.
				Properties that can be updated for an item group are hidden, disabled, label.
				@function combo-box.setOptionAttributes
				@param {Object|String} values Specifies the value/#option-id/#optgroup-id of item/group whose properties are to be changed. Inorder to update more than one item, the values are given in array.
				@param property {Object|String} Property to be updated. If more than one property is changed, this param is given as object with <option, value> pair and the third param is omitted.
				@param value {Boolean|String|Object} New property value.
				@example <caption>To update single item </caption>
	$(selector).zcombobox('setOptionAttributes', 'IN','disabled', true)
				@example <caption>To update multiple items </caption>
	$(selector).zcombobox('setOptionAttributes', ['IN','AF'],'disabled', true)
				@example <caption>To update multiple properties for a single item </caption>
	$(selector).zcombobox('setOptionAttributes', IN ,{
								disabled:true,
								label: 'India',
								icon:'flag flag-in'
							});
			*/
	//Update components
	/** Method to set the combobox property.
		@function combo-box.setAttribute
		@param {String} property Name of the property that is to be set.
		@param {String} value New value for the property.
		@example <caption> Example: </caption>
	$(selector).zcombobox('setAttribute','contentType','icon-text');
	*/	
	/** This method sets one or more combobox property.
		@function combo-box.setAttributes
		@param {Object} properties Object contains property-value pairs to be set.
		@example <caption> Example: </caption>
	$(selector).zcombobox('setAttributes',{
						'placeholder':'Select Country',
						'disabled':true 
					});
	*/
	/** This method returns the value of the given property.
		@function combo-box.getAttribute
		@param {String} property Name of the property whose value is to be returned.
		@returns {Object|String} Value of the given property.
		@example <caption> Example: </caption>
	$(selector).zcombobox('getAttribute','disabled')
	*/
	/** This method returns current combobox properties.
		@function combo-box.getAttributes
		@returns {Object} Combobox properties.
		@example <caption> Example: </caption>
	$(selector).zcombobox('getAttributes');
	*/
