/* $Id$ */
(function(ZC){
	var $ = ZC.DOMUtil,
		sortablePrototype = {
		_create: function(element, options){
			this._initialize(element,options);
			this._constructComponent();
		},
		options: {
			'sortableItems': '> *',	// No I18N
			'disabled': false,	// No I18N
			'sortAxis': false,	// No I18N
			'zIndex': 1000,	// No I18N
			'scrollSensitivity': 20,	// No I18N	
			'scrollSpeed': 20,	// No I18N
			'cancel': "input,textarea,button,select,option",	// No I18N
			'leave': null	// No I18N
		},
		_EVENTS: {
			"DRAGSTART": "dragstart",// No I18N
			'MOUSEDRAG': 'drag',	// No I18N
			"DRAGSTOP": "dragstop"	// No I18N
		},
		_initialize: function(element, options){
			this.document = $(this.element[0].ownerDocument);
			this.window = $(this.document[0].defaultView || this.document[0].parentWindow);
		},
		_constructComponent: function(){
			this.saveContainerData = {};
			this.element.addClass("zc-sortable");	// No I18N
			this.refresh();
			this._offsetPos = this.element.offset();
			this._bindMouseEvents();
			this._setHandleClassName();
			this.ready = true;	
		},
		refresh: function(orgEvent){
			this._refreshItems(orgEvent);
			this._setHandleClassName();
			this._refreshPositions();
			return this;
		},
		_handleMouseDown: function(orgEvent){
			var componentName = this.componentName;
			this._dispatchEvent("DRAGSTART", orgEvent.target, orgEvent);  // No I18N
			this._isMouseMoved = false;
			(this._mouseStarted && this._handleMouseUp(orgEvent));
			this._mouseDownEvent = orgEvent;
			var base = this, leftButtonClicked = (orgEvent.which === 1), elementToNeglect = (typeof this.options.cancel === "string" && orgEvent.target.nodeName ? ($(orgEvent.target).closest(this.options.cancel).length) : false); 	// No I18N
			var leaveElem = typeof this.options.leave === "string" ? $(orgEvent.target).hasClass(this.options.leave) ? true : false : false; 	// No I18N
			var isInValidElement = this.options.sortWith ? $(orgEvent.target).closest(this.options.sortWith).length : true;
			if (!isInValidElement || !leftButtonClicked || elementToNeglect || leaveElem || !this._captureMouseAction(orgEvent)) {
				return true;
			}
			if (true === $.data(orgEvent.target, componentName + ".preventClickEvent")) { 	// No I18N
				$.removeData(orgEvent.target, componentName + ".preventClickEvent"); 	// No I18N
			}
			var base = this;
			this.document.on("mousemove." +componentName, function(orgEvent){	// No I18N
				base._handleMouseMove(orgEvent);
			}).on("mouseup." +componentName, function(orgEvent){	// No I18N
				base._handleMouseUp(orgEvent);
			});
			orgEvent.preventDefault();
			return true;
		},
		_handleMouseMove: function(orgEvent){
			this._dispatchEvent("MOUSEDRAG", orgEvent.target, orgEvent);  // No I18N
			if(this._isMouseMoved){
				if(ZC.Browser.isIE() && (!document.documentMode || document.documentMode < 9) && !orgEvent.button) {
					return this._handleMouseUp(orgEvent);
				}
			}
			if(orgEvent.which || orgEvent.button){
				this._isMouseMoved = true;
			}
			if(this._mouseStarted){
				this._onMouseDrag(orgEvent);
				return orgEvent.preventDefault();
			}
			ZC.$document.on("keydown."+this.componentName, this._keydownHandler.bind(this, orgEvent)); // No I18N
			this._mouseStarted = (this._handleMouseMoveStart(this._mouseDownEvent, orgEvent) !== false);
			(this._mouseStarted ? this._onMouseDrag(orgEvent) : this._handleMouseUp(orgEvent));
			return !this._mouseStarted;
		},
		_keydownHandler: function(mouseMoveEvent, orgEvent){
			if(orgEvent.keyCode === ZC.keyCode.ESCAPE){
				this._handleMouseUp(this._mouseDownEvent, true);
			}
			ZC.$document.off("keydown."+this.componentName);  // No I18N
		},
		_dispatchEvent: function(customEvent, ui, orgEvent){
			var content = {
				'value': ui 	// No I18N
			}
			return this._super(customEvent, orgEvent, content);
		},
		_handleMouseMoveStart: function(orgEvent){
			this.currentContainer = this; //revisit
			this._refreshPositions();
			this._draggableElement= this._createDraggableElement();  //revisit
			this._saveDraggableElementSize();
			this._saveMargins();
			this._scrollParent = this._getScrollParent(this._draggableElement);
			this._offsetPos = this._selectedItem.offset();
			this._offsetPos = {
				top: this._offsetPos.top - this._margins.top,
				left: this._offsetPos.left - this._margins.left
			}
			var base = this;
			$.extend(this._offsetPos,{
				click:{
						left: orgEvent.pageX - base._offsetPos.left,
						top: orgEvent.pageY - base._offsetPos.top
					},
				parent: base._getParentOffset(),
				relative: base._getRelativeOffset()
			})
			this._draggableElement.css("position","absolute");	// No I18N
			this._cssPosition = this._draggableElement.css("position");	// No I18N
			this._elementPosition = this._fixPosition(orgEvent);
			this._eventPageX = orgEvent.pageX;
			this._eventPageY = orgEvent.pageY;
			this._formerDomElements = {prev: this._selectedItem.prev()[0], parent: this._selectedItem.parent()[0]};
			this._createPlaceHolder();
			if(this.options.containment){
				this._setContainment();
			}
			if(this.options.zIndex){ //revisit
				if(this._draggableElement.css("zIndex")){	// No I18N
					this._savedZIndex = this._draggableElement.css('zIndex');	// No I18N
				}
				this._draggableElement.css("zIndex", this.options.zIndex);	// No I18N
			}
			if(!this._preserveDraggableElementSize){
				this._saveDraggableElementSize();
			}
			this._draggableElement.addClass("zc-sortable-helper");	// No I18N
			this._onMouseDrag(orgEvent);
			return true;	
		},
		_setContainment: function(){
			if(this.options.containment === "parent"){   // No I18N
				this.options.containment = this._draggableElement[0].parentNode;
			}
			var contaimentElement = $(this.options.containment)[0],
				containmentOffset = $(this.options.containment).offset(),
				isOverFlowHidden = ($(contaimentElement).css("overflow") !== "hidden");  // No I18N 
			this._containment = [containmentOffset.left +  parseInt($(contaimentElement).css("borderLeftWidth")) + parseInt($(contaimentElement).css("paddingLeft")) - this._margins.left,  // No I18N
								containmentOffset.top + parseInt($(contaimentElement).css("borderTopWidth")) + parseInt($(contaimentElement).css("paddingTop")) - this._margins.top,   // No I18N
								containmentOffset.left + (isOverFlowHidden ? Math.max(contaimentElement.scrollWidth, contaimentElement.offsetWidth) : contaimentElement.offsetWidth) - parseInt($(contaimentElement).css("borderLeftWidth")) - parseInt($(contaimentElement).css("paddingRight")) - this._draggableElementSize.width - this._margins.left,  // No I18N
								containmentOffset.top + (isOverFlowHidden ? Math.max(contaimentElement.scrollHeight, contaimentElement.offsetHeight) : contaimentElement.offsetHeight) - parseInt($(contaimentElement).css("borderTopWidth")) - parseInt($(contaimentElement).css("paddingBottom")) - this._draggableElementSize.height - this._margins.top  // No I18N
								];		
		},
		_onMouseDrag: function(orgEvent){
			var scrolled = false, item, intersection, itemElement;
			this._position = this._fixPosition(orgEvent);
			this._changeToAbsolute = this._changePosition("absolute");	// No I18N
			if(!this._lastAbsolutePos){
				this._lastAbsolutePos = this._changeToAbsolute;
			}
			if(orgEvent.pageY - this.document.scrollTop() < this.options.scrollSensitivity) {
				scrolled = this.document.scrollTop(this.document.scrollTop() - this.options.scrollSpeed);
			} else if(this.window.height() - (orgEvent.pageY - this.document.scrollTop()) < this.options.scrollSensitivity) {
				scrolled = this.document.scrollTop(this.document.scrollTop() + this.options.scrollSpeed);
			}
			if(orgEvent.pageX - this.document.scrollLeft() < this.options.scrollSensitivity) {
				scrolled = this.document.scrollLeft(this.document.scrollLeft() - this.options.scrollSpeed);
			}else if(this.window.width() - (orgEvent.pageX - this.document.scrollLeft()) < this.options.scrollSensitivity) {
				scrolled = this.document.scrollLeft(this.document.scrollLeft() + this.options.scrollSpeed);
			}
			this._changeToAbsolute = this._changePosition("absolute");	// No I18N
			if(!this.options.axis || this.options.axis !== "y"){	// No I18N
				this._draggableElement[0].style.left = this._position.left+"px";	// No I18N
			}
			if(!this.options.axis || this.options.axis !== "x"){	// No I18N
				this._draggableElement[0].style.top = this._position.top+"px";	// No I18N
			}
			for(var i= this.items.length - 1;i>=0;i--){
				item = this.items[i];
				itemElement = item.item[0];
				intersection = this._intersectsWithPointer(item);
				if(!intersection){
					continue;
				}
				if(itemElement !==  this._selectedItem[0] && this._placeHolder[intersection === 1 ? "next" : "prev"]()[0] !== itemElement && !$.contains(this._placeHolder[0], itemElement)){	// No I18N
					this._direction = intersection === 1 ? "down" : "up";	// No I18N
					if(this._intersectsWithSides(item) && item.item[0].nodeName !== "INPUT"){	// No I18N
						this._reArrangeItems(orgEvent, item);
					}else{
						break;
					}
					break;
				}
			}
			this._lastAbsolutePos = this._changeToAbsolute;
			return false;	
		},
		_reArrangeItems: function(orgEvent, item, appendTo, hardRefresh){
			if(appendTo){
				appendTo[0].appendChild(this._placeHolder[0]);
			}else{
				var position;
				if(this._direction === "down"){
					position = item.item[0];
				}else{
					position = item.item[0].nextSibling;
				}
				item.item[0].parentNode.insertBefore(this._placeHolder[0], position);
			}
			this._counter = this._counter ? ++this._counter : 1;
			var counter = this._counter, base = this;
			setTimeout(function(){
				base._refreshPositions(!hardRefresh);
			},0);
		},
		_intersectsWithSides: function(item){
			var isOverBottomHalf = this._isOverAxis(this._changeToAbsolute.top + this._offsetPos.click.top, item.top + (item.height/2), item.height),
				isOverRightHalf = this._isOverAxis(this._changeToAbsolute.left + this._offsetPos.click.left, item.left + (item.width/2), item.width),
				verticalDirection = this._getDragVerticalDirection(),
				horizontalDirection = this._getDragHorizontalDirection(), 
				intersectsContainment = false;
			if(!horizontalDirection && !verticalDirection){
				return false;	
			} 
			if(this.floating && horizontalDirection){
				if(this._containment){
					intersectsContainment = ((this._changeToAbsolute.left <= this._containment[0]) || (this._changeToAbsolute.left >= this._containment[2]));	
				}  
	            var value = intersectsContainment || ((horizontalDirection == "right" && isOverRightHalf) || (horizontalDirection == "left" && !isOverRightHalf));  // No I18N
			}else{
				if (this._containment){
					 intersectsContainment = ((this._changeToAbsolute.top <= this._containment[1]) || (this._changeToAbsolute.top >= this._containment[3]));	
				}
	            var value = intersectsContainment || (verticalDirection && ((verticalDirection == "down" && isOverBottomHalf) || (verticalDirection == "up" && !isOverBottomHalf)));  // No I18N
				return value;
			}
		},
		_intersectsWithPointer: function(item){
			var isOnElementHeight = (this.options.axis === "x") || this._isOverAxis(this._changeToAbsolute.top + this._offsetPos.click.top, item.top, item.height, "x"),	// No I18N
				isOnElementWidth = (this.options.axis === "y") || this._isOverAxis(this._changeToAbsolute.left + this._offsetPos.click.left, item.left, item.width, "y"),	// No I18N
				intersectsContainmentL = false,
				intersectsContainmentT = false,
				isOverElement = isOnElementHeight && isOnElementWidth,
				verticalDirection = this._getDragVerticalDirection(),
				horizontalDirection = this._getDragHorizontalDirection();
				if(!isOverElement){
					return false;
				}
				return this.floating ?
				(((horizontalDirection && horizontalDirection === "right") || verticalDirection === "down") ? 2 : 1)	// No I18N
				: (verticalDirection && (verticalDirection === "down" ? 2 : 1));	// No I18N
		},
		_isOverAxis: function(x, reference, size, axis){
			var value = axis === "x" ? this._selectedItem.height() : 0;  // No I18N
			return ((x + value) >= reference) && (x < (reference + size ));
		},
		_getDragVerticalDirection: function(){
			var delta = this._changeToAbsolute.top - this._lastAbsolutePos.top;
			return delta !== 0 && (delta > 0 ? "down" : "up");	// No I18N
		},
		_getDragHorizontalDirection: function(){
			var delta = this._changeToAbsolute.top - this._lastAbsolutePos.top;
			return delta !== 0 && (delta > 0 ? "right" : "left");	// No I18N
		},
		_changePosition: function(pos){
			var mod = pos === "absolute" ? 1 : -1, scroll = this._cssPosition === "absolute" && !(this._scrollParent[0] !== this.document[0] && $.contains(this._scrollParent[0],this._offsetParent[0])) ? this._offsetParent : this._scrollParent, isScrollRootNode;	// No I18N
			if(scroll[0].tagName.toLowerCase() === "html" || scroll[0].tagName.toLowerCase() === "body"){	// No I18N
				isScrollRootNode = true;
			}
			return {
				top: (this._position.top + this._offsetPos.relative.top * mod + this._offsetPos.parent.top*mod - ((this._cssPosition === "fixed" ? - this._scrollParent.scrollTop() : (isScrollRootNode ? 0 : scroll.scrollTop())) * mod)),	// No I18N
				left: (this._position.left + this._offsetPos.relative.left * mod + this._offsetPos.parent.left*mod - ((this._cssPosition === "fixed" ? - this._scrollParent.scrollLeft() : (isScrollRootNode ? 0 : scroll.scrollLeft())) * mod))	// No I18N
			};
		},
		_createPlaceHolder: function(){
			var selectedNodeName = this._selectedItem[0].nodeName.toLowerCase();
			this._placeHolder = $("<"+ selectedNodeName + ">", this.document[0]).addClass(this._selectedItem[0].className+" zc-sortable-placeholder");	// No I18N
			this._placeHolder.css("visibility","hidden");	// No I18N
			this._selectedItem.after(this._placeHolder);
			if(!this._placeHolder.width()) { 
				this._placeHolder.width(this._selectedItem.innerWidth() - parseInt(this._selectedItem.css("paddingLeft")||0, 10) - parseInt(this._selectedItem.css("paddingRight")||0, 10)); 	// No I18N
			}
			if(!this._placeHolder.height()){
				this._placeHolder.height(this._selectedItem.height());
			}
		},
		_fixPosition: function(orgEvent){
			var top, left, pageX = orgEvent.pageX, pageY = orgEvent.pageY, scroll = this._cssPosition === "absolute" && !(this._scrollParent[0] !== this.document[0] && $.contains(this._scrollParent[0],this._offsetParent[0])) ? this._offsetParent : this._scrollParent, isScrollRootNode;	// No I18N
			if(this._containment){
				if(orgEvent.pageX - this._offsetPos.click.left < this._containment[0]){
					pageX = this._containment[0] + this._offsetPos.click.left;
				}
				if(orgEvent.pageY - this._offsetPos.click.top < this._containment[1]){
					pageY = this._containment[1] + this._offsetPos.click.top;
				}
				if(orgEvent.pageX - this._offsetPos.click.left > this._containment[2]){
					pageX = this._containment[2] + this._offsetPos.click.left;
				}
				if(orgEvent.pageY - this._offsetPos.click.top > this._containment[3]){
					pageY = this._containment[3] + this._offsetPos.click.top;
				}
			}
			if(scroll[0].tagName.toLowerCase() === "html" || scroll[0].tagName.toLowerCase() === "body"){	// No I18N
				isScrollRootNode = true;
			}
			return {
				top: pageY - this._offsetPos.click.top - this._offsetPos.relative.top - this._offsetPos.parent.top + (this._cssPosition === "fixed" ? - this._scrollParent.scrollTop() : (isScrollRootNode ? 0 : scroll.scrollTop())),	// No I18N
				left: pageX - this._offsetPos.click.left - this._offsetPos.relative.left - this._offsetPos.parent.left + (this._cssPosition === "fixed" ? - this._scrollParent.scrollLeft() : (isScrollRootNode ? 0 : scroll.scrollTop()))	// No I18N
			}
		},
		_getScrollParent: function(element, includeHidden){
			var position = element.css("position"),	// No I18N
			excludeStaticParent = position === "absolute",  // No I18N	
			overflowRegex = includeHidden ? /(auto|scroll|hidden)/ : /(auto|scroll)/,	
			scrollParent = element.parents().filter(function(){
				var parent = $(this);
				if(excludeStaticParent && parent.css("position") === "static"){	// No I18N
					return false;
				}
				return overflowRegex.test( parent.css( "overflow" ) + parent.css("overflow-y") + parent.css("overflow-x"));	// No I18N
			}).eq(0);
			return position === "fixed" || !scrollParent.length ? $(element[ 0 ].ownerDocument || document) : scrollParent;	// No I18N
		},
		_getParentOffset: function(){
			this._offsetParent = this._draggableElement.offsetParent();
			var po = this._offsetParent.offset();
			if(this.cssPosition === "absolute" && this._scrollParent[0] !== this.document[0] && $.contains(this._scrollParent[0], this._offsetParent[0])) {	// No I18N
				po.left += this._scrollParent.scrollLeft();
				po.top += this._scrollParent.scrollTop();
			}
			if(this._offsetParent[0] === this.document[0].body || (this._offsetParent[0].tagName && this._offsetParent[0].tagName.toLowerCase() === "html" && ZC.Browser.isIE())) {	// No I18N
				po = { top: 0, left: 0 };
			}
			return {
				top: po.top + (parseInt(this._offsetParent.css("borderTopWidth"),10) || 0),	// No I18N
				left: po.left + (parseInt(this._offsetParent.css("borderLeftWidth"),10) || 0)	// No I18N
			};
		},
		_getRelativeOffset: function(){
			if(this.cssPosition === "relative"){	// No I18N
				var p = this._selectedItem.position();
				return {
					top: p.top - (parseInt(this._draggableElement.css("top"),10) || 0) + this._scrollParent.scrollTop(),	// No I18N
					left: p.left - (parseInt(this._draggableElement.css("left"),10) || 0) + this._scrollParent.scrollLeft()	// No I18N
				};
			}else {
				return { top: 0, left: 0 };
			}
		},
		_saveMargins: function(){
			this._margins = {
				left: (parseInt(this._selectedItem.css("marginLeft"),10) || 0),	// No I18N
				top: (parseInt(this._selectedItem.css("marginTop"),10) || 0)	// No I18N
			}
		},
		_createDraggableElement: function(orgEvent){
			var draggableItem = this._selectedItem;
			this._savedCSS = {width: this._selectedItem[0].style.width, height: this._selectedItem[0].style.height, position: this._selectedItem.css("position"), top: this._selectedItem.css("top"), left: this._selectedItem.css("left")};	// No I18N
			draggableItem.width(this._selectedItem.width()).height(this._selectedItem.height());
			return draggableItem;
		},
		_saveDraggableElementSize: function(){
			this._draggableElementSize = {
				width: this._draggableElement.outerWidth(),
				height: this._draggableElement.outerHeight()
			};
		},
		_captureMouseAction: function(orgEvent){
			var _selectedItem = null, base = this;
			this._isPrev = false;
			if(this.reverting){	// revisit
				return false;
			}
			if(this.options.disabled){
				return false;
			}
			this._refreshItems(orgEvent);
			$(orgEvent.target).parents().each(function(){
				if($.data(this,base.componentName+"-item") === base){	// No I18N
					_selectedItem = $(this);
					return false;
				}
			})
			if($.data(orgEvent.target, this.componentName+"-item") === base){	// No I18N
				_selectedItem = $(orgEvent.target);
			}
			if(!_selectedItem){
				return false;
			}
			this._selectedItem = _selectedItem;
			if(this._selectedItem.prev().length){
				this._adjacentElement = this._selectedItem.prev();
				this._isPrev = true;
			}else{
				this._adjacentElement = this._selectedItem.next()
			}
			//this._selectedItem.css("cursor","move");	
			return true;
		},
		_handleMouseUp: function(orgEvent, escKeyPressed){
			var componentName = this.componentName;
			this.document.off("mousemove."+componentName);	// No I18N
			this.document.off("mouseup."+componentName);	// No I18N
			if(this._mouseStarted){
				this._mouseStarted = false;
				if (orgEvent.target === this._mouseDownEvent.target) {
					$.data(orgEvent.target, componentName + ".preventClickEvent", true);	// No I18N
				}
				this._dropDraggedElement(orgEvent, escKeyPressed);
			}
			return false;
		},
		_dropDraggedElement: function(orgEvent, escKeyPressed){
			if(!orgEvent){
				return;
			}
			this._placeElement(orgEvent, escKeyPressed);
		},
		_placeElement: function(orgEvent, escKeyPressed){
			var i;
			this.reverting = false;
			if(escKeyPressed){
				if(this._isPrev){
					this._selectedItem.insertAfter(this._adjacentElement);
				}else{
					this._selectedItem.insertBefore(this._adjacentElement);
				}
			}else{
				if(!this._noFinalSort && this._selectedItem.parent().length){
					this._placeHolder.before(this._selectedItem);
				}	
			}
			
			this._noFinalSort = null;
			if(this._draggableElement[0] === this._selectedItem[0]) {
				for(i in this._savedCSS) {
					if(this._savedCSS[i] === "auto" || this._savedCSS[i] === "static") {	// No I18N
						this._savedCSS[i] = "";	// No I18N
					}
				}
				this._selectedItem.css(this._savedCSS).removeClass("zc-sortable-helper");	// No I18N
			}else {
				this._selectedItem.show();
			}
			if(this._savedZIndex) {
				this._draggableElement.css("zIndex", this._savedZIndex === "auto" ? "" : this._savedZIndex);	// No I18N
			}
			this.dragging = false; // revisit
			this._placeHolder[0].parentNode.removeChild(this._placeHolder[0]);
			this.fromOutside = false;
			this._dispatchEvent("DRAGSTOP", orgEvent.target, orgEvent);  // No I18N
			return !this.cancelHelperRemoval;
		},
		_bindMouseEvents: function(){
			var base = this, componentName = this.componentName;
			this.element.on("mousedown."+componentName, function(orgEvent){	// No I18N
				return base._handleMouseDown(orgEvent);
			})
			this.element.on("click."+componentName, function(orgEvent){	// No I18N
				if($.data(orgEvent.target, componentName+".preventClickEvent") ===  true){	// No I18N
					$.removeData(orgEvent.target, componentName+".preventClickEvent");	// No I18N
					orgEvent.stopImmediatePropagation();
					return false;
				}
			})
			this.started = false; // Revisit
		},
		_refreshItems: function(orgEvent){
			this.items = [];
			this.containers = [this];
			var targetData, item, query, queries = [[$(this.options.sortableItems, this.element), this]];
			for(var i=queries.length - 1;i>=0; i--){
				targetData = queries[i][1];
				query = queries[i][0];
				for(var j=0; j<query.length;j++){
					item = $(query[j]);
					item.data(this.componentName+"-item", targetData);	// No I18N
					this.items.push({
						item: item,
						width: 0, height: 0,
						left: 0, top: 0
					});
				}
			}
		},
		_setHandleClassName: function(){
			this.element.find('.zc-sortable-handle').removeClass('zc-sortable-handle');	// No I18N
			for(var i=0;i< this.items.length; i++){
				var elementsToNeglect = this.options.cancel.split(",");	// No I18N
				var elementPosition = elementsToNeglect.indexOf(this.items[i].item[0].nodeName.toLowerCase());
				if(elementPosition == -1){
					(this.items[i].item).addClass('zc-sortable-handle');	// No I18N
				}
			}
		},
		_refreshPositions: function(fast){
			this.floating = this.items.length ? (this.options.sortAxis === 'x' || this._isFloating(this.items[0].item)) : false;	// No I18N
			var item, itemSize, offsetPos;
			for(var i=this.items.length - 1; i>=0; i--){
				item = this.items[i];
				itemSize = item.item;
				if (!fast) {
					item.width = itemSize.outerWidth();
					item.height = itemSize.outerHeight();
				}
				offsetPos = itemSize.offset();
				item.left = offsetPos.left;
				item.top = offsetPos.top;
			}
			for(var i= this.containers.length - 1;i>=0;i--){
				offsetPos = this.containers[i].element.offset();
				this.containers[i].saveContainerData.left = offsetPos.left;
				this.containers[i].saveContainerData.top = offsetPos.top;
				this.containers[i].saveContainerData.width = this.containers[i].element.outerWidth();
				this.containers[i].saveContainerData.height = this.containers[i].element.outerHeight();
			}
			return this;
		},
		_isFloating: function(item){
			return (item.css('float').toLowerCase() === "left" || item.css('float').toLowerCase() === "right" || item.css('display').toLowerCase() === "table-cell" || item.css("display").toLowerCase() === "inline");	// No I18N
		},
		destroy: function(){
			var componentName = this.componentName;
			this.element.removeData(componentName);	
			this.element.off("mousedown."+componentName).off("click."+componentName);	// No I18N
			this.document.off("mousemove." +componentName).off("mouseup."+componentName);	// No I18N
		},
		disable: function(){
			var componentName = this.componentName;
			this.element.off("mousedown."+componentName).off("click."+componentName);	// No I18N
			this.document.off("mousemove." +componentName).off("mouseup."+componentName);	// No I18N
		},
		enable: function(){
			this._bindMouseEvents();
		}
	}
	ZC.registerComponent("zsortable", sortablePrototype); // No I18N
})(ZComponents);
