/* $Id$ */
(function(ZC){
	/** @components suggest-field -  Assists the user to quickly find the required value from the pre-populated suggestions. 
	    @eventPrefix zsuggestfield 
	*/
	var $ = ZC.DOMUtil,
		suggestfieldPrototype = {
		options: {
			/** @member {(array|object)} 
                 [suggest-field.dataSource] - Specifies the input for the suggestions that needs to be loaded.

                JSONArray - If JSON array is passed as input, it is mandatory to provide data mapping (or)

                 Array - String / Number array can also be passed as input. 	 (or)

                 object - {URL: &lt;string|function&gt;, queryParam: &lt;object|function&gt;, cacheResults: (true|false)} 
	             @example <caption> Example: </caption>  
   dataSource: {
         "URL": "https://api.github.com/search/repositories",
         "type": "GET",
         queryParam: function(params){
                return {
                    "q" : params.query,
                    "page": params.pages  	
                    //If the API has paging for data retrieval, dynamically data is retrieved upon scroll.
                };
          },
          success: function(data, params){
                return {
                    "results": data.items,
                    "more": (params.pages * 30) < data.total_count
                    //Indicates whether data is available for next server request.
                }
            },		    
    } */
			/** @member {string|function}
				[suggest-field.dataSource.URL] - The path from where the suggestion data need to be fetched.*/
			/** @member {object|function}
				[suggest-field.dataSource.queryParam] - The query Parameter that need to be passed to the API to retrieve the associated data. */		
			/** @member {object|function}
				[suggest-field.dataSource.success] - Callback for AJAX request success  . */	
			/** @member {object|function}
				[suggest-field.dataSource.fail] - Callback for AJAX request failure  . */	
			"dataSource": {	// No I18N
				"dataType": "json",	// No I18N
				"type": "GET" // No I18N
			},	
			"searchIcon": false,	// No I18N
			/** @member {object} 
			    [suggest-field.dataMapping] - The mapping for the JSON Data passed should be given here. This field is mandatory for both JSON and API input. The name of the field in the input data should be specified for each of the fields provided below.
		        @example <caption> Example: </caption>  

Data mapping for the sample JSON below:

    object = {
        "id": 3167891,
        "name": {
            "first_name": "Edward",
            "last_name": "Cullen"
        },
        "email": "edward@gmai.com",
        "avatarURL": "http://www.github.com?page=1&q=edward",
        "department": "Training",
        "gender": "Male",
        "country": "India"
    }

    dataMapping: {
        value: "id",
        text: "name.first_name", 
        // When the value should be retrieved from a nested object. The value should be seperated by "." operator
       	informativeText: "email",  (optional)
        image: "avatarURL",   (optional)
        groupLabel: "department" (optional)
    }			
			*/
			/** @member {string}
				[suggest-field.dataMapping.value] - Any unique data field in the JSON object can be given here. */
			/** @member {string}
				[suggest-field.dataMapping.text] - The name(Primary Text) that need to be displayed in each search list item. */
			/** @member {string}
				[suggest-field.dataMapping.informativeText] - The description / any addition information you need to provide for each search list item. */
			/** @member {string}
				[suggest-field.dataMapping.image] - If the user wants to display Image/icon in the suggestions, the JSON key which has image URL / icon should be assigned to the key "image"  */
			/** @member {string}
				[suggest-field.dataMapping.groupLabel] - The field in the JSON that need to be displayed as a group header. Set this value only when you want group header in the suggestions list. */
			"dataMapping": undefined,	// No I18N
			/** @member {boolean}
				[suggest-field.cacheServerResponse = true] - Whether response received from the server/API/JSON file should be cached */
			"cacheServerResponse": true,	// No I18N
			/** @member {boolean}
				[suggest-field.searchKeywordSubsetsInCache = false] - Denotes whether result subset can be retrieved from the previous cached results. 	*/
			"searchKeywordSubsetsInCache": false,	// No I18N
			/** @member {boolean}
				[suggest-field.isResponseFiltered] - The value should be set to true if the user does not want search operation to be handled by the component. If the search result is retrieved from the API, then this value can be set to true. */
			"isResponseFiltered": false,	// No I18N
			/** @member {boolean}
				[suggest-field.isResponsePaginated] - This can be used to load content when paging is available in API. 
		        @example <caption> Example: </caption>  
    $("#input1").zsuggestfield({
        dataMapping: {
            "value": "id",
            "text": "full_name",
            "informativeText": "git_url",
            "image": "owner.avatar_url"
        },
        dataSource: {
            "URL": "https://api.github.com/search/repositories",
            "type": "GET",
            "dataType": "json",
            queryParam: function(params){
               return {
                   "q" : params.query,
                   "page": params.pages
                };
            },
            success: function(data, params){
                return {
                    "results": data.items,
                    "more": (params.pages * 30) < data.total_count
                }
            }	    
        },
        isResponseFiltered: true,
        isResponsePaginated: true,
        lazyLoadData: true,
        className: "container"
    })
			*/
			"isResponsePaginated" : false,	// No I18N
			/** @member {(object)} 
			       [suggest-field.search] - Options associated to search can be specified here. It is MANDATORY to provide "by" field in search for search to be triggered in the component.
			       @example <caption> Example: </caption>  
   $("#demoInput").zsuggestfield({
        search: {
            minKeywordLength: 1,
            by: "name",
            criteria: "startsWith"
        }
   })
			*/
			"search": {			// No I18N
				/** @member {(number)} 
			        	[suggest-field.search.minKeywordLength = 0] - Minimum length of the text that need to be entered to initiate the autocomplete action.*/
				"minKeywordLength": 0,	// No I18N
				"maxKeywordLength": 50,	// No I18N
				/** @member {(string|array)} 
			        	[suggest-field.search.by] - The field name with which search should be carried out. This field can be set if the source data is JSON. If you want search action to be processed for multiple fields, the field names can be passed as "," seperated values. The search will be carried out in the order it appears. It is mandatory to provide "by" if the source data is JSON 
						@example <caption> Example: </caption> 
If search operation has to be carried out on multiple fields in the input JSON, then value should be passed as follows: 

	search: {
	   by: "name,email"
	}


If the input JSON structure contains nested JSON object, then the value should be given as follows: 

	search: {
	   by: "name.first_name,product"
	}
			    */
				"by": undefined,	// No I18N
				/** @member  {{string}}
				  	@allowedValues {("startsWith"|"contains")}
			        	[suggest-field.search.criteria = "contains"] - Filtering mode */	
				"criteria": "contains",	// No I18N
				/** @member {number} 
			        	[suggest-field.search.delay = 100] - The delay after which the search need to be carried out. */
				"delay": 100	// No I18N
			},
			/** @member {string} 
			    [suggest-field.autoComplete] - The value gets auto filled on hover of each suggestion. */
			"autoComplete": false,	// No I18N
			/** @member {string} 
			        [suggest-field.placeholder] - Add the text that need to be displayed as placeholder. */
			"placeholder": false,	// No I18N
			/** @member	{(string)} 
				[suggest-field.isImageSourceSprited = false] -  Indicates whether the image that need to be displayed would be image or sprite CSS class  */	
			"isImageSourceSprited": false,	// No I18N
			/** @member {object} 
			    [suggest-field.dropdownList] - The dropdown options are provided here. 
			    	@example <caption> Example: </caption>
    $("#demoInput").zsuggestfield({
        dataMapping: {
           value: "id",
           name: "name, 
   	    informativeText: "email",
           image: "imageURL",
           groupLabel: "department"
        },			
        search: {
           by: "name"
        },
        drodownList: {
            contentType: "image-text",
            width: 300,
            sortResults: false
        },
        lazyLoadData: false
    })
			*/
			"dropdownList": {	// No I18N
				/** @member {(string)}
			        [suggest-field.dropdownList.contentType = "text"] - The type of menu that need to be displayed. If "image-text-desc" menu type is selected then "informativeText" field in "dataMapping" option need to be given.
			        @allowedValues {("text"|"image-text"|"image-text-desc"|"text-desc")} */
				"contentType": "text",	// No I18N	
				/** @member {String} 
			        [suggest-field.dropdownList.progressIndicatorSVGIconId] - The default SVG Icon provided by the component for the loader can be changed. ID of the user's SVG element should be provided. 
					@newlyAdded true
			    */
				"progressIndicatorSVGIconId": undefined, // No I18N	
				/** @member {String} 
			        [suggest-field.dropdownList.progressIndicatorIconClassName] -   Sprite CSS class can also be provided for the loader icon.
			        @newlyAdded true
			    */
				"progressIndicatorIconClassName": undefined, // No I18N	
				/** @member {number} 
			        	[suggest-field.dropdownList.height = 350 ] - Height of the suggestion menu. */
				"height": 350,	// No I18N
				/** @member {number} 
			        	[suggest-field.dropdownList.width] - Width of the suggestion menu. */
				"width": undefined,	// No I18N
				/** @member {boolean} 
			        	[suggest-field.dropdownList.moreResultsAction = false] - Set this to true if you want more action to be included in the suggestions. If maxDisplayResults option is not set by the user, value for maxDisplayResults will be taken as 10 in this case. 
						@example <caption> Example: </caption>
    $("#demoInput").zsuggestfield({
        drodownList: {
            moreResultsAction: function(){
            	alert("More action clicked")
            },
        },
        lazyLoadData: false
    })
			    */
				"moreResultsAction": false,	// No I18N
				/** @member {(id-string-selector | dom-selector)} 
			        	[suggest-field.dropdownList.appendTo] - Enable this if the suggestions menu need to be appended to a DOM element. */
				"appendTo": undefined,	// No I18N
				/** @member {boolean} 
			        	[suggest-field.dropdownList.sortResults = false] - Enable it when you want to sort the results retrieved for the input. Customize the default text by passing a string if it is needed. */
				"sortResults": true,	// No I18N
				/** @member {(string)} 
			        	[suggest-field.dropdownList.position = "bottom-right"] - The position where the menu need to be placed.				
					@allowedValues {("top"|"bottom"|"left"|"right"|"after-cursor")} */
				"position": 'auto',// No I18N
				/** @member {number} 
			        	[suggest-field.dropdownList.maxDisplayResults] - Number of results that need to be displayed for each matched input */
				"maxDisplayResults": undefined,	// No I18N
				"animation": undefined,	// No I18N
				/** @member {boolean}
			        	[suggest-field.dropdownList.focusFirstItem = true] - First menu item will be focused if set. */
				"focusFirstItem": true,	// No I18N
				/** @member {boolean}
			        [suggest-field.dropdownList.highlightKeyword = false] - Highlight the keyword typed in the input field in the menu displayed. */
				"highlightKeyword": false,	// No I18N
				/** @member {boolean}
			        [suggest-field.dropdownList.menuClassName] - Custom CSS for the menu. */
				"menuClassName": undefined,	// No I18N
				/** @member {boolean} 
		        	[suggest-field.dropdownList.showNoResultsMessage = false] - Enable it when you want to display no results message.
                */	
				"showNoResultsMessage": false	// No I18N
			},
			/** @member {boolean}
			    [suggest-field.showDropDownListOnFocus = false] - Shows Menu upon focus of input field. This option can be enabled when the user wants to show suggestions upon focus of the input field when minKeywordLength is 0.  */
			"showDropDownListOnFocus": false,	// No I18N
			 /** @member {boolean} 
			        [suggest-field.lazyLoadData = false] - Indicates whether data need to be loaded as you scroll.  (On demand) */
			"lazyLoadData": true,	// No I18N
			/** @member {boolean} 
			        [suggest-field.className] - The class name that need to be applied for the input field for which you initialize the component. */
			"className": undefined,	// No I18N
			/** @member {boolean} 
			        [suggest-field.disabled = false] - Enable this option to disable the input field. */
			"disabled": false,	// No I18N
			/**	@member {object}
					[suggest-field.messages] - Text messages that need to be displayed can be provided here.*/
			"messages": {	// No I18N
				/** @member {string}
					[suggest-field.messages.noResults] - Message that need to be displayed when there are no results for the input keyword typed.

	                @example <caption> Example: </caption>

To append typed query at a specified position, #query# acts as a placeholder to append query. Eg. "No results for #query# found." 
    $("#demoInput").zsuggestfield({
        drodownList: {
            showNoResultsMessage: true
        },
        messages: {
             noResults: "No results for #query# found."
        }
    })
				*/
				"noResults": "No results found.",	// No I18N
				/** @member {string}
					[suggest-field.messages.moreAction] - More action text. 
					@example <caption> Example: </caption>
    $("#demoInput").zsuggestfield({
        drodownList: {
            showNoResultsMessage: true,
            moreResultsAction: function(){
                alert("More Action Clicked");
            }
        },
        messages: {
            noResults: "No results for #query# found.",
            moreAction: "See more results for #query#"
        }
    })
				*/
				"moreAction": "See more results",	// No I18N
				/** @member {string} 
					[suggest-field.messages.serverError] - Error message that need to be displayed in case of server output failure. */
				"serverError": "Server request failed. Please try again later"	// No I18N
			}
		},
		_EVENTS: {
			/** Triggered when a suggestion is selected.
				@event suggest-field.listitemselect
				@param {object} event eventObject
				@param {object} customObject containing additional information about the selected item.
			*/
			"ITEMSELECT": "listitemselect",	// No I18N
			/** Triggered when a suggestion is focuse.
				@event suggest-field.listitemfocus
				@param {object} event eventObject
				@param {object} customObject containing additional information about the focused item. */
			"ITEMFOCUS": "listitemfocus",	// No I18N
			/** Triggered before the menu is shown.
				@event suggest-field.dropdownlistbeforeopen
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu. */
			"MENUBEFOREOPEN": "dropdownlistbeforeopen",	// No I18N
			/** Triggered after the menu is shown.
				@event suggest-field.dropdownlistopen
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu */
			"MENUAFTEROPEN": "dropdownlistopen",	// No I18N
			/** Triggered before the menu is closed.
				@event suggest-field.dropdownlistbeforeclose
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu. */
			"MENUBEFORECLOSE": "dropdownlistbeforeclose",	// No I18N
			/** Triggered after the menu is closed.
				@event suggest-field.dropdownlistclose
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu. */
			"MENUAFTERCLOSE": "dropdownlistclose",	// No I18N
			/** Triggered when the value is changed and input field loses focus.
				@event suggest-field.change
				@param {object} event eventObject
				@param {object} customObject containing additional information about the changed values. */
			"CHANGE": "change",	// No I18N
			/** Triggered before the search is performed.
				@event suggest-field.beforesearch
				@param {object} event eventObject
				@param {object} Contains information about the query passed and the associated data.
	            @example <caption> Example </caption>

    $("#input1").on("zsuggestfieldbeforesearch", function(orgEvent, ui){
        console.log(ui.value);
    })
		
Events can be listened through callbacks 

    $("#input1").zsuggestfield({
        dataSource: sourceData, 
        search: {
            by: "name"
        },
        dataMapping: {
            "value": "id",
            "text": "value",
            "informativeText": "email",
            "image": "imageURL"
        },
        beforesearch: function(orgEvent, ui){
            console.log(ui.value);
        }
    })
			*/
			"BEFORESEARCH": "beforesearch",	// No I18N
			/** Triggered after the search is done. This event will be triggered only when the dataSource requires AJAX request.
				@event suggest-field.searchsuccess
				@param {object} event eventObject
				@param {object} Contains information about the query passed and the response received for the query. */
			"SEARCHSUCCESS": "searchsuccess",	// No I18N
			/** Triggered after the search is done. This event will be triggered only when the dataSource requires AJAX request.
				@event suggest-field.searcherror
				@param {object} event eventObject
				@param {object} Contains information about data retrieval failure */
			"SEARCHERROR": "searcherror",	// No I18N
			/** Triggered when the more action is selected.
				@event suggest-field.moreactionclick
				@param {object} event eventObject
				@param {object} customObject containing additional information about the clicked action. */
			"MOREACTIONSELECT": "moreactionclick"	// No I18N
		},
		_CONSTANTS: {
			minKeywordLength: 20,
			maxResults: 10
		},
		_create: function(element, options){
		    this._constructContainer();
			this._initialize(element,options);
			this._bindEvents();
			this._bindMenuEvents();
			this._initSource();
			this._cachedData = {};		
			this.options.isResponsePaginated && (this._params = {pages: undefined});
		},
		_constructContainer: function(){
			this.options.dropdownList.forElement = this.element;
			this._element = this.element;
			this._element.addClass("zsuggestfield-element");  // No I18N
			this._input = this.element;
			this._elementId = (this.element.attr("id") ? this.element.attr("id") : Math.floor(Math.random() * 1000000))+this.componentName;  // No I18N
			this._input.attr({
				"aria-haspopup": "true",	//No I18N
				"aria-expanded": "false",	//No I18N
				"aria-owns": this._elementId+"-menu",	//No I18N
				"role": "search"	//No I18N
			});		
			this.options.className && this._element.addClass(this.options.className);
		},
		_initialize: function(){	
			this._dataMapping = this.options.dataMapping;
			var menu = this.options.dropdownList, isImageSprite = this.options.isImageSourceSprited;
			this.options.placeholder && this.element.attr("placeholder", this.options.placeholder);		// No I18N
			switch(menu.contentType){
				case "image-text-desc": // No I18N
				case "image-text": // No I18N
					this._type = "icon-text"; // No I18N
				break;
				case "text":  // No I18N
				case "text-desc": // No I18N
					this._type = menu.itemType === "checkbox" ? "icon-text" : "text";	// No I18N
				break;
			}
			if(this.options.dataSource instanceof Array && typeof this.options.dataSource[0] === "string"){	// No I18N
				this._type = "text";	// No I18N
			}
			var menuId = (this.element.attr("id") || Math.floor(Math.random() * 1000000))+"-"+this.componentName+"-menu", menuElement = $("#"+menuId);  // No I18N
			if(menuElement.length){
				menuElement.data("zmenu") && menuElement.zmenu("destroy"); // No I18N
				menuElement.remove();
			}
			this._menu = ZC.createMenu(menuId, {}, {forElement: menu.forElement, scroll: "default", contentType: this._type, direction: menu.position !== "after-cursor" ? menu.position : "auto"});	// No I18N
			if(this._type === "icon-text" && menu.contentType !== "text" && menu.contentType !== "text-desc" && !isImageSprite){ // No I18N
				this._menu.addClass("zsuggestfield"); // No I18N
				menu.contentType === "image-text-desc" && this._menu.addClass("zmenu--withhint");	//No I18N
			}
			if(menu.contentType === "image-text" && menu.itemType === "checkbox" && isImageSprite){		// No I18N
				this._menu.addClass("zsuggestfield--withicon"); // No I18N
			}
			if(menu.contentType === "image-text-desc" && menu.itemType === "checkbox" && isImageSprite){	// No I18N
				this._menu.addClass("zsuggestfield zsuggestfield--withiconandhint"); // No I18N
			}
			this._dataMapping.groupLabel && (this.options.lazyLoadData = false);
			(this.options.search.minKeywordLength > this._CONSTANTS.minKeywordLength) && (this.options.search.minKeywordLength = this._CONSTANTS.minKeywordLength);
			(menu.maxDisplayResults === undefined && this.options.lazyLoadData) && (menu.maxDisplayResults = this._CONSTANTS.maxResults);
			menu.appendTo && this._menu.zmenu("option", "appendTo", menu.appendTo);	// No I18N
			menu.menuClassName && this._menu.addClass(menu.menuClassName);
			menu.height !== undefined && this._menu.zmenu("option", "height", menu.height); 	// No I18N
			menu.width !== undefined && this._menu.zmenu("option", "width", menu.width); 	// No I18N
		},
		_bindEvents: function(){
			var componentName = this.componentName;
			this._element.on('keydown.'+componentName, this._keydowHandler.bind(this))	  // No I18N
						 .on('keyup.'+componentName, this._keyupHandler.bind(this)) // No I18N
						 .on('mouseup.'+componentName, this._mouseupHandler.bind(this)) // No I18N
						 .on('paste.'+componentName, this._pasteHandler.bind(this)); // No I18N
			ZC.$document.on("click."+componentName, this._documentClickHandler.bind(this));  // No I18N
		},
		_documentClickHandler: function(orgEvent){
			var target = $(orgEvent.target), value = this._element.val();
			if(target.closest(".zsuggestfield-element")[0] !== this._element[0] && target.closest(".zmenu").attr("id") !== this._menu.attr("id")  && value !== this._oldVal){  // No I18N
				var data = this._currentData ? {} : undefined;
				data = this._getData(this._currentData);				
				this._dispatchEvent("CHANGE", {value: value, data: data}, orgEvent); // No I18N
				this._oldVal = this._element.val();
			}	
		},
		_pasteHandler: function(orgEvent){
			setTimeout(this._searchTimeout.bind(this, this._element.val(), this._element, orgEvent), 200);
		},
		_mouseupHandler: function(orgEvent){	
			var query = this._element.val();
			query = query.substring(0 , this._element[0].selectionStart);
			this._searchTimeout(query, this._element, orgEvent);
		},
		_keydowHandler: function(orgEvent){
			var keyCode = ZC.keyCode;
			if(this._menu !== undefined && !this._menu.is(":visible")){	// No I18N
				if(orgEvent.keyCode === keyCode.DOWN && !this._ajaxSource){
					this._search('', this._element, orgEvent);	// No I18N
				}
				if(orgEvent.keyCode === keyCode.ENTER){
                	ZC.$document.trigger("click");  // No I18N
				}
				return true;
			}
			switch(orgEvent.keyCode){
				case keyCode.UP:
			    case keyCode.DOWN:
				case keyCode.TAB:
                case keyCode.ESCAPE:
             	    orgEvent.preventDefault();
					!this._isLoaderInited && this._menu.zmenu('handleKeyCode',orgEvent.keyCode); // No I18N	
                break;
                case keyCode.ENTER:
                	orgEvent.preventDefault();
					!this._isLoaderInited && this._menu.zmenu('handleKeyCode',orgEvent.keyCode); // No I18N	
                	ZC.$document.trigger("click");  // No I18N
                break;
			}
		},
		_keyupHandler: function(orgEvent){
			var query = this._element.val(), keyCode = ZC.keyCode;
			if((orgEvent.keyCode >= 186 && orgEvent.keyCode <= 192) || (orgEvent.keyCode >= 219 && orgEvent.keyCode <= 222) || (orgEvent.keyCode >= 48 && orgEvent.keyCode <= 57) || (orgEvent.keyCode >= 65 && orgEvent.keyCode <= 90) || (orgEvent.keyCode ===  keyCode.SPACE) || (orgEvent.keyCode ===  keyCode.BACKSPACE) || (orgEvent.keyCode ===  keyCode.DELETE)){		
				this._searchTimeout(query, this._element, orgEvent);
			}
			if(orgEvent.keyCode === keyCode.TAB){
				this._menu.is(":visible") && this._menu.zmenu("selectHoveredMenuItem");	// No I18N
			}
		},
		_bindMenuEvents: function(){
			var base = this;
			this._menu.off("zmenuitemclick").on("zmenuitemclick", this._handleItemClick.bind(this)) 	// No I18N
					  .off('zmenubeforefocus').on('zmenubeforefocus', function(orgEvent, ui){	// No I18N
						return false; })
				  	  .off("zmenuitemhover").on("zmenuitemhover", this._handleItemHover.bind(this))  // No I18N
					  .off("zmenubeforeshow").on("zmenubeforeshow", this._handleMenuBeforeShow.bind(this))  // No I18N
					  .off("zmenushow").on("zmenushow", this._handleMenShow.bind(this))  // No I18N
					  .off("zmenubeforehide").on("zmenubeforehide", this._handleMenuHide.bind(this))  // No I18N
					  .off("zmenuhide").on("zmenuhide", this._handleMenuHide.bind(this)) // No I18N
		},
		_handleMenuBeforeHide: function(orgEvent, ui){
			this._dispatchEvent("MENUBEFORECLOSE", {value: ui}, orgEvent);  // No I18N
		},
		_handleMenuHide: function(orgEvent, ui){
			this._input && this._input.attr("aria-expanded", "false");	//No I18N
			this._dispatchEvent("MENUAFTERCLOSE", {value: ui}, orgEvent);  // No I18N
			orgEvent.stopImmediatePropagation(); 
			//Prevent zmenuhide from triggering when event binded through the element selector
		},
		_handleMenShow: function(orgEvent, ui){
			this._input && this._input.attr("aria-expanded", "true");	//No I18N
			this._dispatchEvent("MENUAFTEROPEN", {value: ui}, orgEvent);  //No I18N
		},	
		_handleMenuBeforeShow: function(orgEvent, ui){
			if(this.options.isImageSourceSprited && this.options.dropdownList.itemType === "checkbox" && (this.options.dropdownList.contentType === "image-text" || this.options.dropdownList.contentType === "image-text-desc")){	// No I18N
				this._menu.removeClass("zmenu--iconwithtext");
			}else if(!this.options.isImageSourceSprited && (this.options.dropdownList.contentType === "image-text" || this.options.dropdownList.contentType === "image-text-desc")){	// No I18N
				this._menu.removeClass("zmenu--iconwithtext");	
			}
			this._dispatchEvent("MENUBEFOREOPEN", {value: ui}, orgEvent);  // No I18N
		},
		_handleItemHover: function(orgEvent, ui){
			var componentName = this.componentName
			if(ui.menuitem.attr("id") === componentName+"noresults" || ui.menuitem.attr("id") === componentName+"loaderror" || ui.menuitem.attr("id") === componentName+"loader"){	// No I18N
				return false;
			}else{
				if(this.options.autoComplete){
					this._handleHover(orgEvent, ui);
				}
				this._dispatchEvent("ITEMFOCUS", {value: ui.menuitem}, orgEvent); // No I18N
			}
		},
		_handleItemClick: function(orgEvent, ui){
			this._handleSelection(orgEvent, ui);
		},
		_mapJSONToData: function(data, valueObj){
			if(valueObj !== undefined && data !== undefined){
				var valueObj = valueObj.split(".");	// No I18N
				for(var i=0;i<valueObj.length;i++){
					data =  data[valueObj[i]];
				}
				return data;
			}	
		},
		_getData: function(currentData){
			var data = {};
			if(currentData){
				for(var key in currentData){
					if(this._dataMapping[key]){
						data[this._dataMapping[key]] = this._mapJSONToData(currentData, key);
					}
				}
			}
			return data;
		},
		_handleHover: function(orgEvent, ui){
			var componentName = this.componentName;
			if(!(ui.menuitem.attr("id") === componentName+"noresults") && !(ui.menuitem.hasClass("zmenu__title")) && !(ui.menuitem.attr("id") === componentName+"moreaction")){	// No I18N
				if(!this._isJSONSource){
					this._element.val(ui.menuitem.data().metadata.label);
				}else{
					this._element.val(ui.menuitem.data().itemValue.text);
				}
			}
		},
		_initSource: function(){
			var dataSource = this.options.dataSource
			if($.isArray(dataSource)){
				this._isJSONSource = typeof dataSource[0] === "object"; 	// No I18N
				this._responseData = this.options.dataSource;
			}else if(typeof dataSource === "object"){	// No I18N
				if(dataSource.URL){
					this._ajaxSource = true;
					this._isJSONSource = true;
				}
			}

			if(!this._isJSONSource){
				var options = this.options;
				this._initialContentType = this.options.dropdownList.contentType;
				options.isImageSourceSprited = options.isResponsePaginated = options.isResponseFiltered = false, options.dropdownList.contentType = "text"; // No I18N
			}
		},
		_searchTimeout: function(query, container, orgEvent){
			this._newQuery = true;
			clearTimeout(this._searchTimer);
			if(this.options.search.delay){
				this._searchTimer = setTimeout(this._search.bind(this, query, container, orgEvent), this.options.search.delay);
			}
		},
		_addLoaderIcon: function(iconElement){
			var dropdown = this.options.dropdownList;
			if(dropdown.progressIndicatorIconClassName || dropdown.progressIndicatorSVGIconId){
				if(dropdown.progressIndicatorIconClassName){
					iconElement.addClass(dropdown.progressIndicatorIconClassName);
				}else{
					var svgConfig = dropdown.progressIndicatorSVGIconId, svgId = svgConfig.split(" ")[0], customClass = svgConfig.slice(svgId.length).trim();
					iconElement.append($("<svg class='"+customClass+"'><use xlink:href='"+svgId+"'></use></svg>"))
				}
			}else{
				var svgElement = ZC.createSVGElement("svg", {"class":"zmenu__svg zmenu__loader zeffects--rotate"}),	// No I18N
				circleElement1 = ZC.createSVGElement("circle", {"class": "circle__track"}),	// No I18N
				circleElement2 = ZC.createSVGElement("circle", {"class": "circle__load"});	// No I18N
				svgElement.appendChild(circleElement1);
				svgElement.appendChild(circleElement2);
				iconElement[0].appendChild(svgElement);
			}
			return iconElement
		},
		_initLoader: function(container, orgEvent){
			this._isLoaderInited = true;
			var iconElement = $("<i class='zmenu__icon h-loading'>");	// No I18N
			iconElement = this._addLoaderIcon(iconElement);
			this._loader = this._menu.zmenu("addMenuItem", {id: this.componentName+"loader",fixed: true, innerHTML: iconElement, itemType: "custom", itemCssClass: "zsuggestfield__nonselectable"});	// No I18N
			this._loader.closest(".zmenu__footer").removeClass("zmenu__footer").addClass("zmenu--loading");		// No I18N
			this._menu.zmenu("show", {"mustBeShown":true, preventHover: true});	// No I18N
			this._makeAjaxRequest(container, orgEvent);
		},
		_search: function(query, container, orgEvent){
			this._filteredResult = [];
			var def = $.Deferred();
			this._currentData = undefined;
			if(this._request){
				this._request.abort();
				this._removeLoader();
				// if(this._filteredResult && this._filteredResult.length){
				// 	this._responseData = this._filteredResult;
				// 	this._filterSuggestions(query);
				// 	this._menu.zmenu("removeMenuItems"); // No I18N
				// 	this._renderMenu(container);
				// }		
				if(this.options.isResponsePaginated){
					this._params && (this._params.pages = undefined);
				}
			}
			if(this.options.lazyLoadData){
				this._pageIndex = 0;
				if(this.options.isResponsePaginated){
					this._params && (this._params.pages = undefined);
				}
			}
			var dataSource = this.options.dataSource; 
			this._params = $.extend(true, {}, this._params, {query: query});
			if(query !== undefined && (query.length >= this.options.search.minKeywordLength) && query.length < this.options.search.maxKeywordLength){
				var returnValue = this._dispatchEvent("BEFORESEARCH", {data: this._params}, orgEvent); // No I18N
				if(this._ajaxSource){
					if(returnValue){
						if(query in this._cachedData){
							this._responseData = this._cachedData[query];	
							if(this.options.isResponseFiltered){
								this._filteredResult = this._responseData;
							}else{
								this._filterSuggestions(query);
							}
							this._renderSuggestions(container, orgEvent);
						}else{
							if(dataSource.queryParam && dataSource.searchKeywordSubsetsInCache && this.options.cacheServerResponse && !this.options.isResponsePaginated){
								var flag = false;
								for(var i=this._params.query.length-1; i>=1; i--){
									var query = this._params.query.slice(0, i);
									if(query in this._cachedData){
										this._responseData = this._cachedData[query];
										flag = true;
										break;
									}
								}
								if(flag){
									this._filterSuggestions(this._params.query);
									this._cachedData[this._params.query] = this._filteredResult;
									this._renderSuggestions(container, orgEvent);
								}else{
									this._menu.zmenu("removeMenuItems");	// No I18N
									this._callLoader(container, orgEvent);
								}
							}
							else{
								this._menu.zmenu("removeMenuItems");	// No I18N
								if(dataSource.queryParam){
									this._callLoader(container, orgEvent);
								}else{
									if(!this._requestInitialized){
										this._callLoader(container, orgEvent);
										this._requestInitialized = true;
									}else{
										this._filterSuggestions(query);
										this._renderSuggestions(container, orgEvent);
									}
								}
							}			
						}
					}else{
						this._menu.is(":visible") && this._hideSuggestions();	 // No I18N
					}
				}else{
					this._responseData = this.options.dataSource;
					this._filterSuggestions(query);
					this._renderSuggestions(container, orgEvent);
				}		
			}else{
				if(typeof query === "string" && query.length === 0 && this.options.search.minKeywordLength === 0){  // No I18N
					if(!this._ajaxSource){
						this._params = {query: query};
						this._filteredResult = this.options.dataSource.slice();	// To prevent mutation of the original array
						this._sortResults();
						this._renderSuggestions(container, orgEvent);
					}else{
						 if(this.options.showDropDownListOnFocus){
						 	this._menu.zmenu("removeMenuItems");	// No I18N
							if(dataSource.queryParam){
								this._callLoader(container, orgEvent);
							}else{
								if(!this._requestInitialized){
									this._callLoader(container, orgEvent);
									this._requestInitialized = true;
								}else{
									this._filteredResult = this._responseData;
									this._renderSuggestions(container, orgEvent);
								}
							}
						 }else{
							this._removeLoader();
							this._menu.is(":visible") && this._hideSuggestions();	// No I18N
						}	
					}
				}else{
					this._removeLoader();
					this._menu.is(":visible") && this._hideSuggestions();	// No I18N
				}			
			}
			this._newQuery = false;
			return def.promise();
		},
		_callLoader: function(container, orgEvent){
			this._initLoader(container, orgEvent);
			this._menu.css("height", "70px");	// No I18N
			this._menu.find(".zmenu--loading").find("ul").css("height", "100%");	 // No I18N
		},
		_handleSelection: function(orgEvent, ui){
			this._oldVal = this.element.val();
			if(!(ui.menuitem.attr("id") === this.componentName+"noresults") && !(ui.menuitem.hasClass("zmenu__title")) && !(ui.menuitem.hasClass("zsuggestfield__nonselectable"))){	// No I18N
				if(ui.menuitem.attr("id") === this.componentName+"moreaction"){	// No I18N
					var action = this.options.dropdownList.moreResultsAction;
					this._dispatchEvent("MOREACTIONSELECT", {value: ui.menuitem, query: this._params.query});  // No I18N
					if(action && $.isFunction(action)){
						action();
					}
				}else{
					//this.options.multiselect && !this.options.multiselect.closeOnSelect && orgEvent.preventDefault();
					if(!this._isJSONSource){
						this._element.val(ui.menuitem.data().metadata.label);
					}else{	
						var itemData = ui.menuitem.data().metadata;
						var result =itemData.itemValue !== undefined ? itemData.itemValue.text : this._mapJSONToData(itemData, this._dataMapping.text);
						this._currentData = itemData.itemValue;
						this._element.val(result);
					}
					var data = {};
					data = this._getData(this._currentData)
					this._dispatchEvent("ITEMSELECT", {value: ui.menuitem, data: data});  // No I18N
					this._element.focus();
				}
			}else{
				this._element.focus();
			}
		},
		_makeAjaxRequest: function(container, orgEvent){
			var dataSource = this.options.dataSource;
			dataSource.sourceURL = typeof dataSource.URL === "function" ? dataSource.URL.call(this.element, this._params) : dataSource.URL;	// No I18N
			dataSource.data = typeof dataSource.queryParam === "function" ? dataSource.queryParam.call(this.element, this._params) : dataSource.queryParam;	// No I18N
			this._requestData(container, orgEvent);
		},
		_successCallback: function(container,orgEvent, data){
			var dataSource = this.options.dataSource;
			this._filteredResult = [];
			this._request = undefined;
			this._isLoaderInited = false;
			this._response = dataSource.success(data, this._params);
			this._responseData = this._response.results;
			this._dispatchEvent("SEARCHSUCCESS", this._responseData, orgEvent);  // No I18N
			if(this._response.more && this._params.pages !== undefined && this.options.lazyLoadData){
				this._cacheResponse(this._responseData);
				this._filteredResult = this._responseData;
				this._totalResponse = this._filteredResult;
				this._spliceResults();
				this._pageIndex = 0;
				var scrollTop = this._menu.find("ul").scrollTop();	// No I18N
				this._removeLoader();
				this._renderMenu(container);
				this._menu.find("ul").scrollTop(scrollTop);	// No I18N
			}else if (this._params.pages === undefined){
				this._cacheResponse(this._responseData);
				if(this.options.isResponseFiltered){
					this._filteredResult = this._responseData;
				}else{
					this._filterSuggestions(this._params.query);
				}
				this._renderSuggestions(container, orgEvent);
			}else{
				this._removeLoader();
				return;
			}
		},
		_spliceResults: function(){
			var maxResults = this.options.dropdownList.maxDisplayResults;
			if(maxResults){
				this._totalPages = Math.ceil(this._filteredResult.length / maxResults);
				this._filteredResult = this._filteredResult.slice(0, maxResults);
			}
		},
		_cacheResponse: function(response){
			var data = this._cachedData[this._params.query];
			if(this.options.cacheServerResponse){
				this._cachedData[this._params.query] = data && data.length ? data.concat(response) : response;
			}
		},
		_removeLoader: function(){
			if(this._loader){
				this._menu.find(".zmenu--loading").remove();	// No I18N
				this._loader = null;
			}
			this._menu.removeData("fixedArea");	// No I18N
		},
		_failureCallback: function(orgEvent, jqXHR, exception){
			if(jqXHR.statusText !== "abort"){	// No I18N
				var dataSource = this.options.dataSource;
				this._isLoaderInited = false;
				this._removeLoader();
				this._menu.zmenu("removeMenuItems");	// No I18N
				this._menu.zmenu("addMenuItem", {"label" : this._localize('serverError'), "index" : 0, id: this.componentName+"loaderror", itemCssClass: "zsuggestfield__nonselectable"}); // No I18N
				this._menu.zmenu("show", {mustBeShown: true, preventHover: true, positionAlterable: true});	// No I18N
				if(dataSource.fail && $.isFunction(dataSource.fail)){
					dataSource.fail(jqXHR, exception);
				}
				this._dispatchEvent("SEARCHERROR", {jqXHR: jqXHR, exception: exception}, orgEvent);		  // No I18N			
			}	
		},
		_requestData: function(container, orgEvent){
			this._request = ZC.AJAX.transport(this.options.dataSource, this._successCallback.bind(this, container, orgEvent), this._failureCallback.bind(this, orgEvent));
		},
		_checkIfItemExists: function(valueToCheck){
			var validPush = false, sourceData = this._filteredResult;
			if(!sourceData.length){
				sourceData.push(valueToCheck);
			}else{
				for(var i=0;i<sourceData.length;i++){
					var sourceKey = this._mapJSONToData(sourceData[i], this._dataMapping.value), valueKey = this._mapJSONToData(valueToCheck, this._dataMapping.value);
					if(sourceKey !== valueKey){
						validPush = true;
					}else{
						validPush = false;
						break;
					}
				}
				validPush && sourceData.push(valueToCheck);
			}			
		},
		_filterSuggestions: function(query){
			//this._filteredResult = [];
			var searchByField, base = this;
			this._isJSONSource && typeof this.options.search.by === "string" && (searchByField = this.options.search.by.split(","));	// No I18N
			if(searchByField){
				switch(this.options.search.criteria){
					case "contains":   	// No I18N
						if(this._isJSONSource){ 
							for(var i=0;i<this._responseData.length;i++){
								for(var j=0;j<searchByField.length;j++){
									var value = this._mapJSONToData(this._responseData[i], searchByField[j]);
									if(value !== undefined){
										value = value.toString();
										if(value.toLowerCase().indexOf(query.toLowerCase()) > -1){
											this._checkIfItemExists(this._responseData[i]);							
										}	
									}							
								}
							}
						}else{
							this._filteredResult = this._responseData.filter(function(item){
								return (item !== undefined && item.toLowerCase().indexOf(query.toLowerCase())) > -1;
							});
						}	
					break;
					case "startsWith":   // No I18N	
						this._getFilteredResult(new RegExp('^'+query, 'gi')); // No I18N
						break;
					case "endsWith":   	// No I18N 	
						this._getFilteredResult(new RegExp(query+'$', 'gi'));  // No I18N
						break;
					case 'equals':   // No I18N
						if(this._isJSONSource){ 
							for(var i=0;i<this._responseData.length;i++){
								for(var j=0;j<searchByField.length;j++){
									var value = this._mapJSONToData(this._responseData[i], searchByField[j]).toString().toLowerCase();
									if(value !== undefined && value === query.toLowerCase()){
										this._checkIfItemExists(this._responseData[i]);							
									}
								}
							}
						}else{
							this._filteredResult = this._responseData.filter(function(item){
								return item !== undefined && item.toString().toLowerCase() === query.toString().toLowerCase();
							});
						}	
					break;
				}
			}
			this._sortResults();
		},
		_getFilteredResult: function(regEx){
			var searchByField, base = this;
			this._isJSONSource && typeof this.options.search.by === "string" && (searchByField = this.options.search.by.split(","));	// No I18N
			if(this._isJSONSource){	
				for(var i=0;i<this._responseData.length;i++){
					for(var j=0;j<searchByField.length;j++){
						var value = this._mapJSONToData(this._responseData[i], searchByField[j]).toString().toLowerCase();
						if(value !== undefined && value.match(regEx) && value.match(regEx).length){
							this._checkIfItemExists(this._responseData[i]);	
						}
					}
				}
			}else{
				this._filteredResult = this._responseData.filter(function(item){
					var value = item.toString().toLowerCase();
					return (value !== undefined && value.match(regEx) && value.match(regEx).length);					
				})
			}
		},
		_sortResults: function(){
			if(this.options.dropdownList.sortResults){
				var base = this;
				if(this._isJSONSource){
					this._filteredResult.sort(function(itemA, itemB) {
						var valueA = base._mapJSONToData(itemA, base.options.dataMapping.text).toLowerCase(), valueB = base._mapJSONToData(itemB, base.options.dataMapping.text).toLowerCase();
				    	return (valueA > valueB) ? 1 : (valueA < valueB) ? -1 : 0;	
					});
				}else{
					this._filteredResult.sort();
				}		
			}
		},
		_lazyLoadContent: function(container, orgEvent){
			if(this._totalPages > 1){
				var base = this;
				this._menu.find("ul").on("scroll."+this.componentName, function(orgEvent){	  // No I18N
					if(!base._request){
						if(base._menu.find("ul").scrollTop() >= (base._menu.find("ul")[0].scrollHeight - base._menu.find("ul").outerHeight())){
							var scrollTop = base._menu.find("ul").scrollTop();
							++base._pageIndex;
							if(base._pageIndex >= base._totalPages){
								if(base.options.isResponsePaginated && base._response && base._response.more !== false){
									base._params.pages++;
									base._initLoader(container, orgEvent);
									base._menu.find(".zmenu--loading").find("ul").css("height", "100%");	 // No I18N
								}
							}else{
								base._filteredResult = base._totalResponse.slice(base._pageIndex * base.options.dropdownList.maxDisplayResults,  base._pageIndex * base.options.dropdownList.maxDisplayResults + base.options.dropdownList.maxDisplayResults);
								base._renderMenu(container);
							}			
							base._menu.find("ul").scrollTop(scrollTop);
						}
					}
				})
			}
		},
		_renderSuggestions: function(container, orgEvent){
			this._focusSet = false;
			if(this.options.isResponsePaginated){
				this._params.pages = 1;
			}		
			this._menu.zmenu("removeMenuItems"); // No I18N
			if(!this.options.allowDuplicateValues && this.options.dropdownList.itemType === "none"){
				this._removeAddedItem && this._removeAddedItem();
			}
			if(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel && this._filteredResult.length){
				this._groupedResult = {};
				for(var i=0; i<this._filteredResult.length;i++){
					var title = this._mapJSONToData(this._filteredResult[i], this._dataMapping.groupLabel);
					if(title in this._groupedResult){
						this._groupedResult[title].push(this._filteredResult[i]); 
					}else{
						this._groupedResult[title] = [];
						this._groupedResult[title].push(this._filteredResult[i]); 
					}
				}
			}
			this._totalResponse = this._filteredResult;
			this._spliceResults();
			this.options.lazyLoadData && this._lazyLoadContent(container); 
			this._renderMenu(container, orgEvent).done(this._handleEnterPressed && this._handleEnterPressed(orgEvent));
		},
		_getItemJSON: function(data){
			var itemImage = this._mapJSONToData(data, this._dataMapping.image), 
				itemText = this._mapJSONToData(data, this._dataMapping.text), 
			    itemValue = this._mapJSONToData(data, this._dataMapping.value),
			    itemSecondaryText = this._mapJSONToData(data, this._dataMapping.informativeText),
			    itemIdentifier = this._mapJSONToData(data, this._dataMapping.itemIdentifier),
			    itemTitle = this._mapJSONToData(data, this._dataMapping.titleText),
			    customValues = this._mapJSONToData(data, this._dataMapping.customValues);
			return {icon: itemImage, name: itemText, key: itemValue, description: itemSecondaryText, identifier: itemIdentifier, customValues: customValues, title: itemTitle};
		},
		_renderMenu: function(container){
			this._lastSelectedMenuItem = null;
			var def = $.Deferred();
			var menuJSON = [], numId = 0, content;
			if((this.options.isImageSourceSprited && (this.options.dropdownList.contentType === "image-text" || this.options.dropdownList.contentType === "image-text-desc"))		// No I18N
				|| (this.options.isImageSourceSprited && this.options.dropdownList.itemType === "checkbox") || (!this.options.isImageSourceSprited && this.options.dropdownList.contentType === "text" && this.options.dropdownList.itemType === "checkbox")){		// No I18N
				if(!this._menu.hasClass("zmenu--iconwithtext")){
					this._menu.addClass("zmenu--iconwithtext");
				}
			}
			if(this._filteredResult && this._filteredResult.length){
				if(!this._isJSONSource){
					for(var i=0; i<this._filteredResult.length;i++){
						var object = {"id": this._filteredResult[i], "label": this._filteredResult[i]};	// No I18N
						this._menu.zmenu("addMenuItem", object);	// No I18N
					}
				}else{		
					if(this.options.dropdownList.contentType !== "text"){	// No I18N						
						if(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel){
							for(var key in this._groupedResult){
								numId++;
								var jsonObj = {"label": key, "itemType": "itemgroup", "id": numId};	// No I18N	
								menuJSON.push(jsonObj);
								for(var i=0; i<this._groupedResult[key].length;i++){
									var itemObj = {};
									var jsonItem = this._getItemJSON(this._groupedResult[key][i]);
									var checkbox = this.options.dropdownList.itemType === "checkbox" ? true : false, type = this.options.dropdownList.contentType;	//No I18N
									if(this.options.dropdownList.contentType === "text-desc"){	// No I18N
										content = "<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>";	// No I18N
									}else{
										content = !this.options.isImageSourceSprited ? "<img src='"+jsonItem.icon+"' class='zsuggestfield__avatar'>" : "<div class='zmenu__iconcontainer'><i class='zmenu__icon "+jsonItem.icon+"'></i></div>";
								    content = type === "image-text" ? content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span></div>" : 
								                     content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>"; // No I18N	
									}				
								    content = checkbox ? "<div class='zmenu__iconcontainer'><i class='zmenu__icon'></i></div>"+content : content;  // No I18N
									itemObj = {
										"itemType": checkbox ? "checkbox" : "custom",// No I18N
										"custom": true,	// No I18N
									    "innerHTML": {content:content, replaceArray:[jsonItem.name, jsonItem.description]}, 	// No I18N
										"isCustomHTMLEncoded": false,	 // No I18N
									    "id": ZC.encodeHTML(jsonItem.key),	// No I18N	
									    "name": jsonObj.id,// No I18N	
										"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}		// No I18N
									}
									menuJSON.push(itemObj);
								}
							}	
							this._menu.zmenu("addMenuItems", menuJSON);	// No I18N
						}
						else{
							var jsonData = {};
							for(var i=0; i<this._filteredResult.length;i++){
								var jsonItem = this._getItemJSON(this._filteredResult[i]);
								var checkbox = this.options.dropdownList.itemType === "checkbox" ? true : false, type = this.options.dropdownList.contentType;	//No I18N
								if(this.options.dropdownList.contentType === "text-desc"){	// No I18N
									content = "<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>";	// No I18N
								}else{
									content = !this.options.isImageSourceSprited ? 
		                       	 		"<img src='"+jsonItem.icon+"' class='zsuggestfield__avatar'>" : "<div class='zmenu__iconcontainer'><i class='zmenu__icon "+jsonItem.icon+"'></i></div>";
									content = type === "image-text" ? content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span></div>": content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>"; // No I18N
								}
								content = checkbox ? "<div class='zmenu__iconcontainer'><i class='zmenu__icon'></i></div>"+content : content;  // No I18N
								jsonData = {
									"itemType" : checkbox ? "checkbox" : "custom", 	// No I18N
									"custom": true,	// No I18N
									"innerHTML": {content: content, replaceArray: [jsonItem.name, jsonItem.description]}, // No I18N
									"id": ZC.encodeHTML(jsonItem.key),	// No I18N
									"isCustomHTMLEncoded": false,  // No I18N
									"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}		// No I18N
								};
								this._menu.zmenu('addMenuItem', jsonData); // No I18N
							}
						}
					}else{
						if(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel){
							for(var key in this._groupedResult){
								numId++;
								var jsonObj = {"label": key, "itemType": "itemgroup", "id": numId};	// No I18N
								menuJSON.push(jsonObj);
								for(var i=0; i<this._groupedResult[key].length;i++){
									var itemObj = {};
									var jsonItem = this._getItemJSON(this._groupedResult[key][i]);
									itemObj = {
										"label": ZC.encodeHTML(jsonItem.name),	// No I18N
										"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}, // No I18N
										"name": jsonObj.id, // No I18N
										"itemType": this.options.dropdownList.itemType === "checkbox" ? "checkbox" : "text", // No I18N
										"id": ZC.encodeHTML(jsonItem.key) // No I18N
									}
									menuJSON.push(itemObj);
								}
							}
							this._menu.zmenu("addMenuItems", menuJSON);	// No I18N
						}
						else{
							var items = [], itemObj = {};
							for(var i=0; i<this._filteredResult.length;i++){
								var jsonItem = this._getItemJSON(this._filteredResult[i]);
								itemObj = {
									"label": ZC.encodeHTML(jsonItem.name), // No I18N
									"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}, // No I18N
								    "itemType": this.options.dropdownList.itemType === "checkbox" ? "checkbox" : "text", // No I18N
								    "id": ZC.encodeHTML(jsonItem.key) // No I18N
								}
								this._menu.zmenu('addMenuItem', itemObj); 	// No I18N
							}
						}
					}						
				}
				var moreAction = this.options.dropdownList.moreResultsAction, text;
				if(moreAction && !this.options.lazyLoadData){
					var moreActionText = this._localize('moreAction'); 	// No I18N
					moreActionText = ZC.encodeHTML(moreActionText);
					 if(typeof moreActionText === "string"){	// No I18N
						if(moreActionText.match(/#query#/g)){	
							text = moreActionText.replace(/#query#/g, this._params.query);
							moreActionText = moreActionText.replace(/#query#/g, "<b class='h-bold'>"+this._params.query+"</b>")	// No I18N										
						}else{
							text = moreActionText;
						}
						var container = $("<span class='zmenu__text'>").html(moreActionText);	
					}
					var item = this._menu.zmenu("addMenuItem", {id: this.componentName+"moreaction", "label": ZC.encodeHTML(text), fixed: true, innerHTML: container, itemType: "custom"});	// No I18N
					item.css("min-height", "inherit");	// No I18N
				}
				this._menu.zmenu("show", {"mustBeShown":true, preventHover: true, positionAlterable: true});	// No I18N
				if(!(this.options.isImageSourceSprited && (this.options.dropdownList.contentType === "image-text-desc") && this.options.dropdownList.itemType === "none") && this.options.dropdownList.contentType !== "text-desc"){		// No I18N
					var totalWidth = 0;
					if(!this.options.isImageSourceSprited){	
						var avatar = $(this._menu.find("li")[this._dataMapping && this._isJSONSource && this._dataMapping.groupLabel ? 1 : 0]).find(".zsuggestfield__avatar");	// No I18N
						if(avatar.length){
							totalWidth = avatar.outerWidth(true);
						}
					}
					var icon = $(this._menu.find("li")[this._dataMapping && this._isJSONSource && this._dataMapping.groupLabel ? 1 : 0]).find(".zmenu__iconcontainer");	// No I18N
					if(icon.length){
						for(var i=0; i<icon.length;i++){
							totalWidth = totalWidth + $(icon[i]).outerWidth(true);
						}		
					}
					totalWidth = totalWidth && totalWidth.toString();
					if(!(this._type === "icon-text" && this.options.dropdownList.itemType === "checkbox" && this.options.isImageSourceSprited && !this.options.dropdownList.contentType === "image-text-desc")){	// No I18N
						totalWidth && this._menu.find(".zmenu__content").css("width", "calc(100% - "+totalWidth+"px)");	// No I18N
					}		
				}
				if(this.options.dropdownList.position === "after-cursor"){	// No I18N
					var pos = this._positionMenuAtCursorPosition(container);
					this._menu.css({top: (pos.top + 15), left: pos.left});
				}
				this.options.dropdownList.highlightKeyword && this._highlightSearchQuery();
				var id,element;
				this._lastSelectedMenuItem && (id = this._lastSelectedMenuItem.attr("id"));		// No I18N
				this._lastSelectedMenuItem && (element = $("#"+id));
				if(element && element.length){
					this._menu.zmenu("hoverMenuItem", element);	// No I18N
					this._lastSelectedMenuItem = null;
				}else{
					if(this.options.dropdownList.focusFirstItem && !this.options.autoComplete){
						if(!this._focusSet){
							this._focusSet = true;
							this._menu.zmenu("hoverMenuItem", this._menu.find("li").eq(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel ? 1 : 0));	// No I18N
						}
					}
				}
				this._removeLoader();			
			}else{
				this._removeLoader();
				if(this.options.dropdownList.showNoResultsMessage){
					var noResultsText = this._localize('noResults');	// No I18N
					noResultsText = ZC.encodeHTML(noResultsText);
					 if(typeof noResultsText === "string"){	// No I18N
						if(noResultsText.match(/#query#/g)){	
							noResultsText = noResultsText.replace(/#query#/g, "<b class='h-bold'>"+this._params.query+"</b>")	
						}
					}
					this._displayNoresults(noResultsText);
					if(this.options.dropdownList.position === "after-cursor"){	// No I18N
						var pos = this._positionMenuAtCursorPosition(container);
						this._menu.css({top: (pos.top+15), left: pos.left});
					}
				}else{
					this._menu.is(":visible") && this._hideSuggestions();	// No I18N
				}
			}	
			this._newQuery = false;	
			return def;
		},
		_highlightSearchQuery: function(){
			var searchBy, menuItems, informativeText, text, groupLabel, componentName = this.componentName;
			if(this._isJSONSource){
				typeof this.options.search.by === "string" && (searchBy = this.options.search.by.split(","));  // No I18N
				informativeText = this._dataMapping.informativeText, text = this._dataMapping.text,
				groupLabel = this._dataMapping.groupLabel; 	
			}
			if(this._isJSONSource && searchBy){
				if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) > -1 && searchBy.indexOf(informativeText) === -1){
					menuItems = this._menu.find(".zmenu__text:not('#"+componentName+"moreaction')");
				}else if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) === -1){
					menuItems = this._menu.find(".zmenu__title").find(".zmenu__text");
				}else if(searchBy.indexOf(groupLabel) === -1 && searchBy.indexOf(text) > -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__item:not('#"+componentName+"moreaction')").find(".zmenu__text,.zmenu__hint")
				}else if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) > -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__text,.zmenu__hint:not('#"+componentName+"moreaction')");
				}else if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) === -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__hint, .zmenu__title > .zmenu__text")
				}else if(searchBy.indexOf(text) === -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__hint")
				}else if(searchBy.indexOf(text) >  -1 && searchBy.indexOf(informativeText) === -1 && searchBy.indexOf(groupLabel) === -1){
					menuItems =this._menu.find(".zmenu__item:not('#"+componentName+"moreaction')").find(".zmenu__text");
				}	
			}else{
				menuItems = this._menu.find(".zmenu__item:not('#"+componentName+"moreaction')").find(".zmenu__text");
			}
			if(menuItems){
				for(var i=0; i<menuItems.length; i++){
					var text = $(menuItems[i]).text();
					text = ZC.encodeHTML(text);
					switch(this.options.search.criteria){
						case "contains":   	// No I18N
						$(menuItems[i]).html(text.replace(new RegExp("(?![^&;]+;)(?!<[^<>]*)("+this._params.query+")(?![^<>]*>)(?![^&;]+;)", "gi"), "<b class='h-bold'>$1</b>")); 	// No I18N
						break;
						case "startsWith":   	// No I18N
						$(menuItems[i]).html(text.replace(new RegExp("(^"+this._params.query+")", "gi"), "<b class='h-bold'>$1</b>"));  	// No I18N
						break;
						case "endsWith":  // No I18N
						$(menuItems[i]).html(text.replace(new RegExp("("+this._params.query+"$)", "gi"), "<b class='h-bold'>$1</b>")); 	// No I18N
						break;
					}
				}
			}
		},
		_positionInputElement : function(domElement){
			var offsetCaretPosition, domElementOffset, pos;
			offsetCaretPosition = domElement.offset(), pos = this._getPositon(domElement);
			offsetCaretPosition = {
				left: offsetCaretPosition.left + pos.left - domElement.scrollLeft(),
				top: offsetCaretPosition.top + pos.top - domElement.scrollTop(),
				height: pos.height
			};
			return offsetCaretPosition;
		},
		_removeSpecialCharacters: function(value){
			value = value.replace(/<|>|`|"|&/g, '?').replace(/\r\n|\r|\n/g, "<br/>");	 // No I18N
		    if(/firefox/i.test(navigator.userAgent)){	
		        value = value.replace(/\s/g, '&nbsp;');	 // No I18N
		    }
	     	return value;
		},
		_duplicateElement : function(domElement){
			var simulatedCss;
			this._duplicateDiv = $("<div>");	
			var cssProperties = ["borderBottomWidth", "borderLeftWidth", "borderRightWidth", "borderTopStyle", "borderRightStyle", "borderBottomStyle", "borderLeftStyle", "borderTopWidth", "boxSizing", "fontFamily", "fontSize", "fontWeight", "height", "letterSpacing", "lineHeight", "marginBottom", "marginLeft", "marginRight", "marginTop", "outlineWidth", "overflow", "overflowX", "overflowY", "paddingBottom", "paddingLeft", "paddingRight", "paddingTop", "textAlign", "textOverflow", "textTransform", "whiteSpace", "wordBreak", "wordWrap"];  // No I18N
	        simulatedCss = {
	        	position: 'absolute',	 // No I18N
	        	left: -9999,
	        	top: 0,
	        	zIndex: -20000
	        };
	        for(var i=0; i<cssProperties.length; i++){
	        	simulatedCss[cssProperties[i]] = domElement.css(cssProperties[i]);
	        }
			this._duplicateDiv.css(simulatedCss);
			return this._duplicateDiv;
		},
		_getPositon : function(domElement){
			var pos = domElement[0].selectionStart, startPos = domElement.val().slice(0, pos), endPos = domElement.val().slice(pos), spanElement,spanRelativePosition, elementWithCaret, dimensions;
			spanElement = "<span style='position: relative; display: inline;'>" +this._removeSpecialCharacters(startPos)+ "</span>";	 // No I18N
			spanElement += "<span id='caret' style='position: relative; display: inline;'>|</span>";	 // No I18N
	  		spanElement += "<span style='position: relative; display: inline;'>" + this._removeSpecialCharacters(endPos) + "</span>";	 // No I18N
	  		this._duplicateElement(domElement);
	  		this._duplicateDiv.html(spanElement);
	  		domElement.after(this._duplicateDiv);
	  		elementWithCaret = this._duplicateDiv.find("#caret");	 // No I18N
	 		spanRelativePosition = elementWithCaret.position();
	 		dimensions = {
	 			left: spanRelativePosition.left,
	 			top: spanRelativePosition.top,
	 			height: elementWithCaret.height()
	 		}
	 		this._duplicateDiv.remove() && (this._duplicateDiv = null);
	 		return dimensions;
		},
		_positionEditableDiv :  function(domElement){
			var sel = window.getSelection(), range = sel.getRangeAt(0), rangeCopy, boundingRect, offsetCaretPosition;
			if(sel){
				if(range.endOffset > -1 && range.endContainer != domElement[0]){
					rangeCopy = range.cloneRange();
					rangeCopy.setStart(range.endContainer, range.endOffset - 1);
					rangeCopy.setEnd(range.endContainer, range.endOffset);
					boundingRect = rangeCopy.getBoundingClientRect();
					offsetCaretPosition = {
						height : boundingRect.height,
						left: boundingRect.left,
						top: boundingRect.top
					};
					rangeCopy.detach();
				}
				if(offsetCaretPosition){
					offsetCaretPosition.top += ZC.$window.scrollTop();
					offsetCaretPosition.left += ZC.$window.scrollLeft();
				}
			}
			return offsetCaretPosition;
		},
		_positionMenuAtCursorPosition: function(container){
		    var offsetCaretPosition;
			offsetCaretPosition = container.is("input") ? this._positionInputElement(container) : this._positionEditableDiv(container)	 // No I18N
			return offsetCaretPosition;
		},
		_hideSuggestions: function(){		
			this._menu !== undefined && this._menu.zmenu("hide");	// No I18N
		},
		_displayNoresults: function(text){
			if(!(this.options.search.minKeywordLength === 0 && this._params.query.length === 0)){
				var container = $("<span class='zmenu__text'>").html(text);	//No I18N
				this._menu.zmenu("removeMenuItem", this.componentName+"moreaction"); // No I18N
				this._menu.zmenu("addMenuItem", {	// No I18N
					label: this._params.query,
					index: 0,
					innerHTML: container,
					itemType: "custom",	// No I18N
					id: this.componentName+"noresults",	// No I18N
					itemCssClass : "zsuggestfield__nonselectable"	// No I18N
				})
				this._menu.zmenu("show", {mustBeShown: true, preventHover: true, positionAlterable: true});	// No I18N
				if(this._menu.hasClass("zmenu--iconwithtext")){
					this._menu.removeClass("zmenu--iconwithtext");
				}
			}else{
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}
		},
		_dispatchEvent: function(customEvent, ui, orgEvent){
			var content = {
				'value': ui  	// No I18N
			};	
			var value = this._super(customEvent, orgEvent, content);
			return content.value.modified !== undefined ? content.value : value;
		},
		/** Opens the suggestion list.
			@function suggest-field.openDropdownList
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("openDropdownList");	*/
		openDropdownList: function(){
			this._menu.zmenu("show", {"mustBeShown":true, preventHover: true, positionAlterable: true});	// No I18N
			this._input.attr("aria-expanded", "true");	//No I18N
		},
		/** Closes the suggestion list.
			@function suggest-field.closeDropdownList
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("closeDropdownList");	*/
		closeDropdownList: function(){
			this._hideSuggestions();
		},
		getValue: function(){
			var data = this._currentData ? {} : undefined;
			data = this._getData(this._currentData);
			return data;
		},
		/** Performs search for the specified input passed as the parameter.
			@function suggest-field.search
			@param {Object} value - Query input for which the search need to be carried out.
			@return {object} Returns the list of results obtained for the specified query.
			@example <caption> Example </caption>
	var results = $(".selector").zsuggestfield("search", "a") */
		search: function(query){
			this._searchTimeout(query, this._element);
		},
		/** Closes the suggestion list.
			@function asuggest-field.clearCache
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("clearCache");	*/
		clearCache: function(){
			this._cachedData = {};
		},
		/** Gets the value currently associated with the specified optionName.
			This signature doesn't accept any arguments.
			@function suggest-field.getAttribute
			@param {string} optionName - The name of the option to get.
			@return {string|object} value.
			@example <caption> Example </caption>
	var position = $(".selector").zsuggestfield("getAttribute","placeholder") */
		/** Sets the value of the option associated with the specified optionName.
			@function suggest-field.setAttribute
			@param {string} optionName - The name of the option to get.
			@param {string} value - A value to set for the option.
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("setAttribute","placeholder","Type a name");	*/
		/** Gets an object containing key/value pairs representing the current autosuggest options.
			This signature doesn't accept any arguments.
			@function suggest-field.getAttributes
			@return {object} options object.
			@example <caption> Example </caption>
	var options = $(".selector").zsuggestfield("getAttributes");*/

		/** Sets one or more options for the autocomplete.
			@function suggest-field.setAttributes
			@param {Object} options - A map of option-value pairs to set.
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("getAttributes",{"placeholder": "Type a name","lazyLoadData":true});*/
		setAttribute: function(optionName, value){
			if(typeof value === "object" && !(value instanceof Array)){  // No I18N
				value = $.extend(true, {}, this.options[optionName], value);
			}
			var oldValue = this.options[optionName];
			this.options[optionName] = value;
			if(optionName === 'className'){	//No I18N
				oldValue && this.element.removeClass(oldValue);
				this.element.addClass(value);
			}else if(optionName === "dataSource"){	// No I18N
				if(typeof value === "object" || value instanceof Array){	// No I18N
					this._cachedData = {};
					var oldSource = this._isJSONSource;
					if(value instanceof Array && typeof oldValue === "object"){	// No I18N
					  	this._isJSONSource =  (typeof value[0] === "object");	// No I18N
						this._ajaxSource = false;
					}else if(typeof value === "object" && oldValue instanceof Array || typeof value === "object"){	// No I18N
						this._ajaxSource = this._isJSONSource = true;
						this._requestInitialized = false;
					}
					if(this._isJSONSource && !oldSource){
						this.options.dropdownList.contentType = this._initialContentType;
					}
				}
			}else if(optionName === "autoComplete"){	//No I18N
				if(value){
					var base = this;
					this._menu.on("zmenuitemhover", function(orgEvent, ui){	// No I18N
						base._handleHover(orgEvent, ui);
						base._dispatchEvent("ITEMFOCUS", {value: ui.menuitem});  // No I18N
					})
				}else{
					this._menu.off("zmenuitemhover");	//No I18N
				}
			}else if(optionName === "search"){	// No I18N
				if(typeof value === "object"){	// No I18N
					if(value.minKeywordLength && value.minKeywordLength < 20){
						this.options.search.minKeywordLength = value.minKeywordLength;
					}else{
						this.options.search.minKeywordLength = oldValue.minKeywordLength;
					}	
				}
			}else if(optionName === "placeholder"){	// No I18N
				value ? this.element.attr("placeholder", value) : this.element.removeAttr("placeholder");	// No I18N
			}else if(optionName === "dropdownList" || optionName === "isImageSourceSprited"){	// No I18N
				if((optionName === 'dropdownList' && value.contentType !== undefined || value.itemType === "checkbox") || (optionName === "isImageSourceSprited")){
					this._menu.removeClass("zsuggestfield zmenu--withhint zmenu--withicon zmenu--withiconandhint zmenu--iconwithtext");
					switch(this.options.dropdownList.contentType){
							case "image-text-desc": // No I18N
							case "image-desc": // No I18N
								this._type = "icon-text"; // No I18N
							break;
							case "text":  // No I18N
							case "text-desc": // No I18N
								if(this.options.dropdownList.itemType === "checkbox"){
									this._type = "icon-text";	// No I18N
									this._menu.addClass("zmenu--iconwithtext");
								}else{
									this._type = "text";	// No I18N
								}
							break;
					}
					this._menu.zmenu("option", "contentType", this._type);	// No I18N
					if(this._type === "icon-text" && this.options.dropdownList.contentType !== "text" && this.options.dropdownList.contentType !== "text-desc" && !this.options.isImageSourceSprited){ // No I18N
						this._menu.addClass("zsuggestfield"); // No I18N
						this.options.dropdownList.contentType === "image-text-desc" && this._menu.addClass("zmenu--withhint");	//No I18N
					}
					if(this.options.dropdownList.itemType === "checkbox" && this.options.isImageSourceSprited){		// No I18N
						if(this.options.dropdownList.contentType === "image-text"){	// No I18N
							this._menu.addClass("zmenu--withicon"); // No I18N
						}
						if(this.options.dropdownList.contentType === "image-text-desc"){	// No I18N
							this._menu.addClass("zsuggestfield zmenu--withiconandhint"); // No I18N
						}
					}
				}
				if(value.appendTo !== undefined){
					this._menu.zmenu("option", "appendTo", value.appendTo);	// No I18N
				}
				if(value.position !== undefined){	
					if(value.position !== "after-cursor"){	// No I18N
						this._menu.zmenu("option", "direction", value.position)	// No I18N
					}
				}
				if(value.height !== undefined){
					this._menu.zmenu("option", "height", value.height)	// No I18N
				}
				if(value.width !== undefined){
					this._menu.zmenu("option", "width", value.width)	// No I18N
				}
			}else if(optionName === "disabled"){	// No I18N
				this.element.attr("disabled", value); // No I18N
				this.element[value ? "addClass" : "removeClass"]("is-disabled");	//No I18N
				this.element.focus();
			}else if(optionName ==="dataMapping"){	//No I18N
				this._dataMapping = value;
			}else if(optionName ==="lazyLoadData"){		//No I18N
				(this.options.dropdownList.maxDisplayResults === undefined && this.options.lazyLoadData) && (this.options.dropdownList.maxDisplayResults = this._CONSTANTS.maxResults);
			}
		},
		updateDataSource: function(newData){
			if(this.options.dataSource instanceof Array && newData instanceof Array){
				var len = this.options.dataSource.length;
				for(var i=len, j=0; i<len+(newData.length);i++){
					this.options.dataSource[i] = newData[j++];
				}
			}
		},
		/** Removes autosuggest functionality completely. This will return the element back to its pre-init state.
			This method does not accept any arguments.
			@function suggest-field.destroy
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("destroy"); */
		destroy: function(){
			var componentName = this.componentName;
			this._element.val("");	//No I18N
			this._oldVal = "";  // No I18N
			this._currentData = {};
			this.element.removeClass("zsuggestfield-element");
			this._isLoaderInited = false;
			this.element.removeData(componentName);
			this._element.off("."+componentName);	//No I18N
			this._menu.zmenu("destroy");	//No I18N
			this._menu.remove();
			this.element.show();
		}
	}
	ZC.registerComponent("zsuggestfield", suggestfieldPrototype); // No I18N
})(ZComponents);
