/* $Id$ */
(function(ZC){
	/** @components token-field - Enables the user to make multiple selections by typing inside a textfield. A token will be added for each selected value. All options of Suggest field component can be accessed here.
	    @eventPrefix ztokenfield 
	*/
	var $ = ZC.DOMUtil,
		tokenfieldPrototype = {
		options: {
			/** @member {object} 
			    [token-field.dataMapping] - The mapping for the JSON Data passed should be given here. This field is mandatory for both JSON and API input. The name of the field in the input data should be specified for each of the fields provided below.
		        @example <caption> Example: </caption>  

Data mapping for the sample JSON below:

    object = {
        "id": 3167891,
        "name": {
            "first_name": "Edward",
            "last_name": "Cullen"
        },
        "email": "edward@gmai.com",
        "avatarURL": "http://www.github.com?page=1&q=edward",
        "department": "Training",
        "gender": "Male",
        "country": "India"
    }

    dataMapping: {
        value: "id",

        // When the value should be retrieved from a nested object. The value should be seperated by "." operator
        text: "name.first_name", 

        itemIdentifier: "email", 

       	informativeText: "email", (optional)

        image: "avatarURL", (optional, only if needed)
        //groupLabel should be used when the user wants a group header for the individual values.

        groupLabel: "department", (optional, only if needed)

        titleText: "email", (optional, only if needed)

        customValues: "gender" (optional, only if needed)
    }			
			*/
			/** @member {string}
				[token-field.dataMapping.titleText] - If the user wishes to have title text for the individual tokens, this value has to be provided. */
			/** @member {string}
				[token-field.dataMapping.itemIdentifier] - The value user provide here will be used for retrieving and removing individual tokens. */
			/** @member {string}
				[token-field.dataMapping.customValues] - If the user has any additional info that should be passed when the selected values are obtained through API. Then in such cases, the JSON key should be passed to the customValues key */
			/** @member {Object} 
			    [token-field.dropdownList] - Dropdown related options are specified here.	*/
			'dropdownList': {	//No I18N
				/** @member {Boolean} 
			    [token-field.dropdownList.itemType] - Indicates whether checkbox kind of menu need to be enabled. 
				@allowedValues 	{("checkbox"|"none")} */				
				"itemType": "none",	// No I18N
				/** @member {Boolean}
					[token-field.dropdownList.closeOnSelect = true] - Whether menu need to be closed once data/value is selected. */
				'closeOnSelect': true	// No I18N
			},
			/** @member {Boolean}
			[token-field.allowDuplicateValues = false] -  Indicates whether duplicate tokens can be included in the token field container. */
			'allowDuplicateValues': false,	// No I18N
			/** @member {Boolean} 
		    [token-field.dataValidation = false] - Performs data validation on the typed input keyword. If the data is invalid, error token is added for the typed input rather than normal token. */
			'dataValidation': false,	// No I18N
			/** @member {Object} 
			    [token-field.tokens] - Token related options are specified here.	*/
			'tokens': {	// No I18N
				/** @member {String} 
			        [token-field.tokens.contentType] - The type of token that need to be created for each selection.
			        @allowedValues {("text"|"image-text")} */
				'contentType': 'text',	// No I18N
				/** @member {String} 
			        [token-field.tokens.removeSVGIconId] - The default SVG Icon provided by the component for token's close icon can be changed. ID of the user's SVG element should be provided. 
					@newlyAdded true */
				"removeSVGIconId": undefined, // No I18N
				/** @member {String} 
			        [token-field.tokens.removeIconClassName] - Sprite CSS class can also be provided for the token's close icon.  
			        @newlyAdded true
			    */
				"removeIconClassName": undefined, // No I18N
				/** @member {Boolean} 
			        [token-field.tokens.sortable=false] - Indicates whether the tokens can have drag and drop functionality */
				'sortable': false,	// No I18N
				/** @member {Boolean} 
			        [token-field.tokens.editable = false] - Indicates whether token value can be edited and updated */
				'editable': false,	// No I18N
				/** @member {String} 
			        [token-field.tokens.editable.pattern = '"#value#" <#id#>'] - The pattern that is used to render the text when a user tries to edit the tokens  */
				/** @member {Boolean} 
					@deprecated true
			        [token-field.tokens.animateOnClose = false] - Denotes whether animation need to be provided for token removal. tokens.animateOnRemove is the replaceement for this option. */
				/** @member {Boolean} 
			        [token-field.tokens.animateOnRemove = false] - Denotes whether animation need to be provided for token removal. 
			        @newlyAdded true
			    */
				'animateOnRemove': false,	// No I18N
				/**	@member {Boolean}
					[token-field.tokens.showCloseButtonOnHover = false] - Denotes whether the remove button on the token should be there by default or should appear as user hovers the token. tokens.showRemoveButtonOnHover is the replacement for this option 
					@deprecated true
				*/
				/**	@member {Boolean}
					[token-field.tokens.showRemoveButtonOnHover = false] - Denotes whether the remove button on the token should be there by default or should appear as user hovers the token. 
					@newlyAdded true
				*/
				'showRemoveButtonOnHover': false	// No I18N
			},
			/** @member {Character}
				[token-field.delimiter = ','] - The delimiter to use for seperating the typed input. */
			'delimiter' : ',',		// No I18N
			/** @member {Number} 
		        [token-field.maxAllowedValues] - Maximum Number of allowed tokens on the tokenfield container. */
			'maxAllowedValues': undefined,	// No I18N
			/**	@member {Array} 
				[token-field.values = []] - Indicates the set of selected data/tokens that appears in the tokenfield. */
			'values': [],	// No I18N
			/** @member {Boolean}
				[token-field.acceptNewValues = false] - Indicates whether the user typed values should be added as a token if the value is not available in the suggestions list. */
			'acceptNewValues': false,	// No I18N
			/** @member {regExp}
				[token-field.newValuePattern = false] - "acceptNewValues" has to be set to true for this option to be effective. A valid regular expression object should be passed to indicate the pattern of input to be accepted. 
				@example <caption> Example </caption>	  
Following Snippet shows how regex should be provided as an option   
	                                    
newValuePattern: new RegExp(/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/) */
			'newValuePattern': undefined,	//No I18N
			/** @member {Object}
				[token-field.messages] - Default text messages */
			'messages': {	// No I18N
				/**  @member {String}
					[token-field.messages.close = 'Close'] - Title text for close button on token. messages.remove is the replacement for this option 
					@deprecated true
				*/
				/** @member {String}
					[token-field.messages.remove = 'Remove'] - Title text for remove button on token 
					@newlyAdded true
				*/
				'remove': 'Remove',	// No I18N
				'maxTokenLimitReached': "Max token limit reached"	// No I18N
			}
		},
		_TOKENFIELDEVENTS: {
			/** Triggered before the token is added.
				@event token-field.beforetokenadd
				@param {Object} event eventObject 
				@param {Object} customObject containing information about the token being added. 
				@example <caption> Example </caption>

    $("#input1").on("ztokenfieldbeforetokenadd", function(orgEvent, ui){
        console.log(ui.value);
    })
		
    Events can be listened through callbacks 

    $("#input1").ztokenfield({
        dataSource: sourceData, 
        search: {
            by: "name"
        },
        dataMapping: {
            "value": "id",
            "text": "value",
            "informativeText": "email",
            "image": "imageURL"
        },
        beforetokenadd: function(orgEvent, ui){
            console.log(ui.value);
        }
    })
			*/
			"BEFORETOKENADD" : "beforetokenadd",	// No I18N
			/** Triggered before the token is removed.
				@event token-field.beforetokenremove
				@param {Object} event eventObject
				@param {Object} customObject containing information about the token being removed. */
			"BEFORETOKENREMOVE": "beforetokenremove",	// No I18N
			/** Triggered after the token is added.
				@event token-field.tokenadd
				@param {Object} event eventObject
				@param {Object} customObject containing information about the token that is added. */
			"AFTERTOKENADD": "tokenadd",	// No I18N
			/** Triggered after the token is removed.
				@event token-field.tokenremove
				@param {Object} event eventObject
				@param {Object} customObject containing information about the token that is removed. */
			"AFTERTOKENREMOVE": "tokenremove",	// No I18N
			/** Triggered when a editable token's value change is submitted or aborted.
				@event token-field.tokenedit
				@param {Object} event eventObject
				@param {Object} customObject containing information about the editable token and the associated value in the token. 
				@example <caption> Example </caption>
	Below example shows how to validate the edited value to a EMAIL ID

	$(<selector>).on("ztokenfieldtokenedit", function(orgEvent, ui){
		var value = ui.value.textboxValue, isInvalid;
		//This value has to be set to true for the modification to be effective on the token
		ui.value.modified = true;
		var name = value.match(/\"(.*)\"/) && value.match(/\"(.*)\"/).pop();
		var value = value.match(/\<(.*)\>/) && value.match(/\<(.*)\>/).pop();
		if(name === null && value === null){
			name = ui.value.textboxValue;
			value = ui.value.textboxValue;
		}
		var regexp = new RegExp(/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/);
		if(ui.value.modified){
			if(value && value.length && !(regexp.test(value))){
				ui.value.itemContent.invalidToken = true;
				isInvalid = true;
			}
			if(!isInvalid){
				if(value !== null){
					if(ui.value.itemContent.invalidToken){
						ui.value.itemContent.itemValue = {};
					}
					ui.value.itemContent["email"] = value;	// No I18N
					ui.value.itemContent["email"] = value;	// No I18N
					ui.value.itemContent["name"] = name ? name : value;	// No I18N
					ui.value.itemContent.invalidToken = false;
				}else{
					ui.value.itemContent.invalidToken = true;
				}
			}	
		}	
	})
			*/
			"TOKENUPDATE": "tokenedit",	// No I18N
			"TOKENDRAG": "tokendrag",	// No I18N
			/** Triggered when token addition fails.
				@event token-field.tokenaddfail
				@param {Object} event eventObject
				@param {Object} customObject containing information about the reason for failure. The failure information is available in a variable called "failureType" */
			"TOKENADDFAIL": "tokenaddfail"	// No I18N	
		},
		_CONSTANTS: {
			'pattern': '"#value#" <#id#>',	// No I18N
			'patterFormatError': "Invalid pattern for token. Pattern should contain #id# or #value# or both",	// No I18N
			'minKeywordLength': 20,	// No I18N
			'maxResults': 10// No I18N
		},
		_create: function(element, options){
			//this._super();
			this._keyCode = ZC.keyCode;
			this._extendEvents();
			this._constructContainer();
			this._initialize();
			this._initSource();	
			this._tokensCollection = [], this._performedStack = [], this._recallStack = [], this._tokensCount = 0, this._cachedData = {};
			!this.options.disabled && this._bindEvents();
			this._uniqueIndex = 0;
			!this.options.disabled && this._bindMenuEvents();
			this.options.isResponsePaginated && (this._params = {pages: undefined});
			if(this.options.values && this.options.values.length){
				this._defaultSelected = true;
				this.addTokens(this.options.values);
				this._defaultSelected = false;
			}
			if(this.options.disabled){
				this.element.attr("disabled", true);	//No I18N
				this.container.addClass("is-disabled");	//No I18N
				this.container.attr("aria-disabled", "true");		//No I18N	
				this._input.attr("disabled", true);	//No I18N
			}
		},
		_constructContainer: function(){
			this._elementId = (this.element.attr("id") ? this.element.attr("id") : Math.floor(Math.random() * 1000000))+"-"+this.componentName;  // No I18N
			this.container = $("<div id='"+this._elementId+"-container' tabindex='-1'>").addClass("ztokenfield");	// No I18N
            this._input = $("<input type='text' role='textbox' aria-haspopup='true' role='textbox' aria-expanded='false' aria-owns='"+this._elementId+"-menu' id='"+this._elementId+"-input' autocomplete='off' tabindex='"+(this.element.attr("tabindex") || 0)+"'>").addClass('ztokenfield__textbox'); // No I18N
			this._input.appendTo(this.container).blur();
			this.container.insertAfter(this.element);
			this.options.className && this.container.addClass(this.options.className);
			this._pasteHelperTextBox = $("<input type='text' style='position:absolute; left: -10000000px' tabindex='-1' id='"+this._elementId+"-copyhelper'>").appendTo(this.container);	// No I18N
			this._pasteRegExp = this._CONSTANTS.pattern;	
			if(this.options.placeholder){	
				this._resizeSearchArea(this._input, this.options.placeholder);	
				this._input.attr("placeholder", this.options.placeholder);	// No I18N	
			}
			this.options.dropdownList.forElement = this.container;	
			!this.options.tokens.showRemoveButtonOnHover && this.container.addClass("ztokenfield--closeshown");		// No I18N
			this._prevContainerHeight = this.container.css("height");	// No I18N
			this.element.hide();
		},
	    _resizeSearchArea: function(textbox, value){
			var duplicateDiv, containerWidth, properties, inputWidth, styleBlock, style, width;
			//Simulate the textbox to calculate width
			styleBlock = "position:absolute; left: -1000px; top: -1000px; display:none;";	// No I18N
			properties = ['font-size', 'font-style', 'font-weight', 'font-family', 'line-height', 'text-transform', 'letter-spacing'];	// No I18N
			for(var i=0; i <properties.length; i++){
				style = properties[i];
				styleBlock += style + ":" + textbox.css(style) + ";"; 	// No I18N
			}
			duplicateDiv =$('<div />', {'style': styleBlock }); 	
			duplicateDiv.text(value !== undefined ? value : textbox.val());
			$('body').append(duplicateDiv);
			var inputWidth;
			containerWidth = this.container.outerWidth();

			if(textbox.attr("id") === "dblclick-input-"+this.componentName){	// No I18N
				inputWidth = duplicateDiv.width() + 30;
				width = inputWidth;
			}else{
				inputWidth = duplicateDiv.width() + 50;
				duplicateDiv.text(textbox.val());
				var inputWidth1 = duplicateDiv.width() + 50;
				duplicateDiv.remove();
				width = inputWidth > inputWidth1 ? inputWidth : inputWidth1; //Checking whether input width is greater or placeholder 
			}			
			if(this.container.is(":visible") && width > (containerWidth - 10)){	// No I18N
				width = containerWidth - 10;
			}
			return textbox.css({'width' : width + 'px'}); 	// No I18N
		},
		_activateInputArea: function(){
			this._inputArea.val(this._inputArea.val());
			this.container.addClass('has-focus');	// No I18N
		},
		_addCustomCloseIcon: function(){
			this._closeButton = $('<button class="ztokenfield__close" tabindex="-1" title='+this._localize('remove')+'></button>');		// No I18N
			var iconElement = $('<i class="ztokenfield__icon"></i>'), tokens = this.options.tokens, svgProps;  // No I18N
			svgProps = {
				props: {"viewBox":"0 0 16 16", "class": "ztokenfield__svg", "width": "100%", "height": "100%"},  // No I18N
				nodes: [
						{"rect": {"x":"1.5","y":"7.4","transform":"matrix(0.7071 -0.7071 0.7071 0.7071 -3.3137 8)","class":"st0","width":"13","height":"1.1"}},  // No I18N
						{"rect": {"x":"1.5","y":"7.4","transform":"matrix(0.7071 0.7071 -0.7071 0.7071 8 -3.3137)","class":"st0","width":"13","height":"1.1"}} // No I18N
					]
			};
			this._handleIcon(iconElement, svgProps, "closebutton", tokens.removeIconClassName, tokens.removeSVGIconId);  // No I18N
			iconElement.appendTo(this._closeButton);
		},
		_addErrorToken: function(value, orgEvent, index, tokenId){
			value = value.trim();
			if(value.length){
				var textbox = this._editTextBox !== undefined ? this._editTextBox : this._input;	
				this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>");	
				this._addCustomCloseIcon();
				this._tokenText = $("<span class ='ztokenfield__text' title='"+value+"'>").text(value);	// No I18N
				this._tokenContainer.addClass("has-error").data("invalidToken", true);	// No I18N
				this._insertToken({"errorValue": value}, textbox, orgEvent, index, tokenId);	// No I18N
				textbox.val("");	// No I18N
			}			
		},
		_checkTokenChange: function(orgEvent){
			if(this._tokensCollection.length > 0 && this._tokensCollection.length === this._lastTokenUpdate.length){
				var current = this._tokensCollection, prev = this._lastTokenUpdate, flag = false;
				for(var i=0; i<current.length;i++){
					for(var j=0; j<prev.length;j++){
						var currentData = $(current[i]).data().metadata, currentKey, prevKey;
						if(currentData !== undefined){
							currentKey = this._mapJSONToData(currentData, this._dataMapping.itemIdentifier);
						}
						var prevData = prev[j];
						if(prevData !== undefined){
							prevKey = this._mapJSONToData(prevData, this._dataMapping.itemIdentifier);
						}
						if(currentKey !== prevKey){
							flag = false;
						}else{
							flag = true;
							break;
						}
					}
				}
				if(!flag){
					var token = {prevTokenList: prev, currentTokenList: current};
					this._dispatchEvent("CHANGE", token, orgEvent);  // No I18N
					this._lastTokenUpdate = [];
					for(var i=0; i<this._tokensCollection.length; i++){
						this._lastTokenUpdate.push(this._tokensCollection[i].data().metadata);
					}
				}
			}else{
				if(this._tokensCollection.length !== this._lastTokenUpdate.length ){
					var token = {prevTokenList: this._lastTokenUpdate, currentTokenList: this._tokensCollection};
					this._dispatchEvent("CHANGE", token, orgEvent);  // No I18N
					this._lastTokenUpdate = [];
					for(var i=0; i<this._tokensCollection.length; i++){
						this._lastTokenUpdate.push(this._tokensCollection[i]);
					}
				}
			}				
		},
		_checkIsContentValid: function(value, keyCode){
			var acceptNewValues = this.options.acceptNewValues,
				pattern = this.options.newValuePattern;
			if(value !== undefined && value.length && acceptNewValues){
				var tokenJSON = {};
				tokenJSON.itemValue = {};
				if(pattern !== undefined && pattern instanceof RegExp && pattern.test(value) || pattern === undefined){
					tokenJSON.itemValue.text = value;
					tokenJSON.itemValue.itemIdentifier = value;
					tokenJSON.itemValue.titleText = value;
					this._addToken(tokenJSON, keyCode, undefined, undefined, true);
					var rawJSON = this._getData(tokenJSON.itemValue);
					if(this.options.dropdownList.acceptNewValues && this.options.dataSource instanceof Array){
						this.options.dataSource.push(rawJSON);
					}
				}else{
					this.options.dataValidation && this._addErrorToken(value, keyCode);
				}
			}
		},
		_handleEnterPressed: function(orgEvent){
			var noResultExists = this._menu.find("#ztokenfieldnoresults").length;
			if(this._enterKeyPressed && this._menu !== undefined && (!this._menu.is(":visible") || noResultExists)){	// No I18N	
				var val = this._input.val();	
				if(this.options.tokens.editable && this._pasteHelperTextBox && $(orgEvent.target).attr("id") === this._pasteHelperTextBox.attr("id")){	// No I18N	
					var activeToken = this.container.find(".is-selected");	 // No I18N
					activeToken.length && activeToken.length === 1 && this._editToken(activeToken, orgEvent);
				}else if(!this.options.acceptNewValues && this.options.dataValidation){
					$(orgEvent.target).attr("id") === this._input.attr("id") && val !== undefined  && val.length && this._addErrorToken(val);		 // No I18N	
					if(this._editTextBox !== undefined){
						this._reformToken(orgEvent);
						this._input.focus();
					}
				}else{
					if(this.options.acceptNewValues){
						if(this._editTextBox && $(orgEvent.target).attr("id") === this._editTextBox.attr("id")){	// No I18N
							this._reformToken(orgEvent);
							this._input.focus();
						}else{
							this._checkIsContentValid(val);
						}
					}else{	
						if(this._editTextBox && $(orgEvent.target).attr("id") === this._editTextBox.attr("id")){	// No I18N
							this._reformToken(orgEvent);
							this._input.focus();
						}	
					}					
				}
				if(noResultExists){
					this._hideSuggestions();
				}
			}
			this._enterKeyPressed = false;				
		},
		_docClickHandler: function(orgEvent){
			var target = $(orgEvent.target), item = target.closest(".zmenu__item");	// No I18N
			if(target.closest(".ztokenfield").attr("id") !==  this.container.attr("id") && target.closest(".zmenu").attr('id') !== this._menu.attr('id')){	// No I18N
				var val = this._input.val();					
			 	this.container.removeClass("has-focus");	// No I18N
				if(!this._lastTokenUpdate.length && this._tokensCollection.length){
					var token = {prevTokenList: this._lastTokenUpdate, currentTokenList: this._tokensCollection};
					this._dispatchEvent("CHANGE", token);  // No I18N
					for(var i=0; i<this._tokensCollection.length; i++){
						this._lastTokenUpdate.push(this._tokensCollection[i].data().metadata);
					}	
				}else{
					this._checkTokenChange(orgEvent);
				}
				this._clickReset = true;			
				if(!this.options.dropdownList.closeOnSelect){					
					!(this._menu[0].contains(item[0])) ? (this._menu.is(":visible") && this._hideSuggestions()) : this._searchTimeout(this._params.query, this._input, orgEvent);	// No I18N
				}else{
					this._menu.is(":visible") && this._hideSuggestions();	// No I18N
				}
				if(this._editTextBox !== undefined && $(orgEvent.target).attr("id") !== this._editTextBox.attr("id")){	// No I18N
					this._reformToken(orgEvent);
					this._menu.is(":visible") && this._hideSuggestions(); // No I18N
				}else{
					if(!(this._menu[0].contains(item[0])) && !this.options.acceptNewValues && this.options.dataValidation && val !== undefined && val.length){
						this._addErrorToken(val);
					}else if(this.options.acceptNewValues){
						this._checkIsContentValid(val);
					}else{
						this._input[0].value = "";
						this._resizeSearchArea(this._input, this.options.placeholder);
					}
				}	
				this.container.find(".is-selected").removeClass("is-selected");	// No I18N		
			}
			if(!this.options.dropdownList.closeOnSelect && this._menu[0].contains(item[0])){
				this._searchTimeout(this._params.query, this._input, orgEvent);
			}
		},
		_keydownHandler: function(orgEvent){
			this._metaKey = false;
			this._checkMaxLimitReach(orgEvent);
			if(this._input.val().length < 1 && !this._editTextBox){	
				var allElements = this.container.find('.is-selected');	// No I18N
				switch(orgEvent.keyCode){
					//Move across tokens left/right when moved with arrow keys
					case this._keyCode.LEFT:		
						var element = $(allElements[0]), ele;
						if(allElements.length && !(orgEvent.shiftKey || orgEvent.metaKey || orgEvent.ctrlKey)){
							allElements.removeClass('is-selected'); // No I18N
							this._clickReset = true;
						}
						if(orgEvent.shiftKey){
							if(!this._clickReset){
								element.length && (ele = element.prev());
							}else{
								if(this._element){
									var prevSelected;
									prevSelected = this._element.prev();
									if(allElements.length > 1){		
										if(prevSelected.hasClass("is-selected")){	// No I18N
											this._element.removeClass("is-selected");	// No I18N
											this._element = prevSelected;
										}else{
											ele = prevSelected;
										}											
									}else{
										element.length && (ele = element.prev());
										prevSelected = undefined;
									}	
								}else{
									element.length && (ele = element.prev());
									prevSelected = undefined;
								}
							}	
							this._clickReset = true;
						}else{
							element.length && (ele = element.prev());
						}
						if(ele !== undefined && ele.length){
							//Gets the latest active element after arrow key operation
							if(ele.hasClass('ztokenfield__token')){ // No I18N   
								this._element = ele.addClass('is-selected'); // No I18N
								this._element.focus();
								this._triggerPasteHelper();						
							}
							else{
								this._input.focus();
							}
						}else{
							if(!prevSelected){
								var ele = this._input.prev();
								if(ele.hasClass('ztokenfield__token')){ // No I18N
									this._element = ele.addClass('is-selected'); // No I18N
									this._element.focus();
									this._triggerPasteHelper();	
								}	
							}				
						}
						this._menu.is(":visible") && this._hideSuggestions();	// No I18N
                        orgEvent.preventDefault();
					break;
					case this._keyCode.RIGHT:
						var length = allElements.length;	
						var element = $(allElements[length - 1]), ele;	
						if(allElements.length && !(orgEvent.shiftKey || orgEvent.metaKey || orgEvent.ctrlKey)){
							allElements.removeClass('is-selected'); // No I18N
							this._clickReset = true;
						}
						if(orgEvent.shiftKey){
							if(!this._clickReset){
								element.length && (ele = element.next());
							}else{
								if(this._element){
									var prevSelected;
									if(allElements.length > 1){
										prevSelected = this._element.next();
										if(prevSelected.hasClass("is-selected")){	// No I18N
											this._element.removeClass("is-selected");	// No I18N
											this._element = prevSelected;
										}else{
											ele = prevSelected;
										}											
									}else{
										element.length && (ele = element.next());
									}	
								}else{
									element.length && (ele = element.next());
								}
							}	
							this._clickReset = true;
						}else{
							element.length && (ele = element.next());
						}
						if(ele !== undefined && ele.length){
							if(ele.hasClass('ztokenfield__token')){ // No I18N
								this._element = ele.addClass('is-selected'); // No I18N		
								this._triggerPasteHelper();			
							}else{
								this._input.focus();
								orgEvent.shiftKey && this.container.find(".is-selected").removeClass("is-selected");		// No I18N
							}
						}
						this._menu.is(":visible") && this._hideSuggestions();	// No I18N
						orgEvent.preventDefault();	
					break;
				}
			}else{
				var query = this._input.val(), cursorPos = this._input[0].selectionStart;
				if(query.length < this.options.search.maxKeywordLength){
						query = orgEvent.keyCode === this._keyCode.LEFT ?
					query.substring(0, cursorPos-1) : 
						orgEvent.keyCode === this._keyCode.RIGHT ? query.substring(0, cursorPos+1) : undefined;
					query !== undefined && this._searchTimeout(query, this._input, orgEvent);
					if(query !== undefined && query.length === cursorPos){
						orgEvent.preventDefault();	
					}
				}
			}
			if(orgEvent.keyCode === this._keyCode.BACKSPACE || orgEvent.keyCode ===  this._keyCode.DELETE){ //Added to prevent default action for backspace when focus is on token
				this._hasInput = false;
				if(this.container.find(".is-selected").length){	// No I18N
					orgEvent.preventDefault();
					return false;
				}else{
					var target = $(orgEvent.target);
					var query = target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && target.val();		// No I18N
					query !== undefined && query.length && (this._hasInput = true);
				}
			}
			if(orgEvent.metaKey || orgEvent.ctrlKey){
				this._metaKey = true;
				var target = $(orgEvent.target);
				if(orgEvent.keyCode === 65 && ($(orgEvent.target).attr("id") === this._pasteHelperTextBox.attr("id") || (target.attr("id") === this._input.attr("id") && this._input.val().length === 0))){	// No I18N
					orgEvent.preventDefault();
				}
				if(this._menu.is(":visible")){
					if(this.options.dropdownList.itemType === "none"){ 
						this._hideSuggestions(); 
					}else{
						var query = target.val();
						this._searchTimeout(query, target, orgEvent);
					}	
				}
				switch(orgEvent.keyCode){
					case 65:
						if(target.attr("id") === this._pasteHelperTextBox.attr("id") || (target.attr("id") === this._input.attr("id") && this._input.val().length === 0)){ 	// No I18N
							var allTokens = this.container.find(".ztokenfield__token"); // No I18N
							for(var i=0; i<allTokens.length;i++){
								$(allTokens[i]).addClass("is-selected").focus();	
							}
							this._triggerPasteHelper();
						}
					break;
					case 90: 
						if(!this._input.val().length && !this._editTextBox){
							orgEvent.preventDefault();
							if(this._performedStack.length){
								var undoAction = this._performedStack.pop();
								if(undoAction && undoAction.action === "Insert"){	
									this._removeToken($("#"+undoAction.tokenId).find(".ztokenfield__close"), orgEvent, undoAction.index);		
								}else if(undoAction && undoAction.action === "Remove"){	// No I18N
									this._addToken(undoAction.tokenData, orgEvent, undoAction.index, undoAction.tokenId);
								}
							}
						}else{
							this._resizeSearchArea(target);
						}							
					break;
					case 89:
						if(!this._input.val().length && !this._editTextBox){
							orgEvent.preventDefault();
							if(this._recallStack.length){
								var redoAction = this._recallStack.pop();
								if(redoAction && redoAction.action === "Remove"){	// No I18N
									this._removeToken($("#"+redoAction.tokenId).find(".ztokenfield__close"), orgEvent, redoAction.index);	
								}else if(redoAction && redoAction.action === "Insert"){	// No I18N
									this._addToken(redoAction.tokenData, orgEvent, redoAction.index, redoAction.tokenId);
								}
							}
						}else{
							this._resizeSearchArea(target);
						}
					break;
					case 88:
						if(target.attr("id") !== this._input.attr("id") && this._activeTokens !== undefined){	// No I18N
							for(var i=0; i<this._activeTokens.length;i++){							
								this._removeToken($("#"+this._activeTokens[i]).find(".ztokenfield__close"));			
							}												
						}
						setTimeout(this._undoTimeoutHandler.bind(this), 500);
					break;
				}	
			}
			if(orgEvent.keyCode === this._keyCode.ESCAPE){
				if(this._editTextBox !== undefined){
					this._escKeyPressed = true;
					this._reformToken(orgEvent);
					this._input.focus();
				}
			}
			if(orgEvent.keyCode === this._keyCode.TAB && this._editTextBox !== undefined){
				this._reformToken(orgEvent);
				this._menu.is(":visible") && this._hideSuggestions(); // No I18N
			}
			if(this._menu !== undefined && (!this._menu.is(":visible") || this._menu.find("#ztokenfieldnoresults").length) && this._tokensCount !== this.options.maxAllowedValues){	// No I18N
				if(orgEvent.keyCode === this._keyCode.DOWN && !this._ajaxSource && !this.container.find(".ztokenfield__token").hasClass("is-selected")){	// No I18N
					this._searchTimeout(this._params.query, this._input, orgEvent);
				}
				if(orgEvent.keyCode === this._keyCode.ENTER){
					orgEvent.preventDefault();
					this._enterKeyPressed = true;	
					if(!this._newQuery){
						this._handleEnterPressed(orgEvent);
					}
				}
				return true;
			}
			if(this._tokensCount !== this.options.maxAllowedValues){
				switch(orgEvent.keyCode){
					case this._keyCode.UP:
				    case this._keyCode.DOWN:
					case this._keyCode.ENTER:
	                case this._keyCode.ESCAPE:
	             	    orgEvent.preventDefault();
	             	    if(!this._newQuery){
							!this._isLoaderInited && this._menu.zmenu('handleKeyCode',orgEvent.keyCode); // No I18N
						}
						if(!this._menu.find(".on-hover").length){
							if(this._editTextBox !== undefined){
								this._reformToken(orgEvent);
							}
						}
	                break;
				}
			}
		},
		_checkMaxLimitReach: function(orgEvent){
			var charInput = $(orgEvent.target).is("input") && $(orgEvent.target).attr("id") !== this._pasteHelperTextBox.attr("id") && !(orgEvent.metaKey || orgEvent.ctrlKey) && ((orgEvent.keyCode >= 186 && orgEvent.keyCode <= 192) || (orgEvent.keyCode >= 219 && orgEvent.keyCode <= 222) || (orgEvent.keyCode >= 48 && orgEvent.keyCode <= 57) || (orgEvent.keyCode >= 65 && orgEvent.keyCode <= 90) || (orgEvent.keyCode ===  this._keyCode.SPACE));  // No I18N
			if(charInput && this.options.maxAllowedValues && this._tokensCount === this.options.maxAllowedValues){
				orgEvent.preventDefault();
			}
		},
		_keyUpHandler: function(orgEvent){
			if(this.element.attr("disabled")){	// No I18N
				return false;
			}
			if($(orgEvent.target).is("input") && $(orgEvent.target).attr("id") !== this._pasteHelperTextBox.attr("id") && !(orgEvent.metaKey || orgEvent.ctrlKey) && ((orgEvent.keyCode >= 186 && orgEvent.keyCode <= 192) || (orgEvent.keyCode >= 219 && orgEvent.keyCode <= 222) || (orgEvent.keyCode >= 48 && orgEvent.keyCode <= 57) || (orgEvent.keyCode >= 65 && orgEvent.keyCode <= 90) || (orgEvent.keyCode ===  this._keyCode.SPACE)) && this._tokensCount !== this.options.maxAllowedValues){		// No I18N	
				this.container.find(".is-selected").removeClass("is-selected");	// No I18N
				this._clickReset = true;
				this._resizeSearchArea($(orgEvent.target), this.options.placeholder ? $(orgEvent.target).attr("placeholder") : undefined);	// No I18N
				var query = $(orgEvent.target).val();
				if(query.charCodeAt(query.length-1) === this.options.delimiter.charCodeAt(0)){
					var isMenuVisible = this._menu.is(":visible"), menuitem;	// No I18N
					if(isMenuVisible){
						menuitem = this._menu.find("li:not(.zmenu__title)");	// No I18N
					}
					if(menuitem && menuitem.length){
						this.options.autoComplete && this._menu.zmenu("hoverMenuItem", $(menuitem[0]));	// No I18N
						this._menu.zmenu("selectHoveredMenuItem");	// No I18N
					}else{
						var query = query.substring(0, query.length - 1);
						if(!this.options.acceptNewValues && this.options.dataValidation){
							query !== undefined && query.length && this._addErrorToken(query);		
							this._editTextBox !== undefined && this._reformToken(orgEvent);
						}else{
							if(this.options.acceptNewValues){
								this._editTextBox ? this._reformToken(orgEvent) : this._checkIsContentValid(query);	
							}else{
								this._editTextBox && this._reformToken(orgEvent);	
							}					
						}
					}
					this.options.dropdownList.closeOnSelect && this._menu.is(":visible") && this._hideSuggestions(); 	// No I18N
				}else{
					if(query.length < this.options.search.maxKeywordLength && !this._metaKey){
						this._newQuery = true;
						this._searchTimeout(query, $(orgEvent.target), orgEvent);
					}
				}
			}
			if(orgEvent.keyCode === this._keyCode.BACKSPACE || (orgEvent.keyCode ===  this._keyCode.DELETE)){
				var target = $(orgEvent.target), query;
				var query = target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && target.val();	// No I18N
				target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && this._resizeSearchArea(target);	// No I18N
				if(query.length < this.options.search.maxKeywordLength){		
					!this._hasInput && query.length === 0 && this._handleTokenRemove($(orgEvent.target), orgEvent); 
					if(query.length >=0 && this._hasInput){
						this._searchTimeout(query, target, orgEvent);
					}
				}else{
					this.container.find(".is-selected").length && this._handleTokenRemove(this._input, orgEvent); 		// No I18N
				}
				if(this.options.values.length === 0 && target.attr("id") === this._input.attr("id") && target.val().length === 0){	// No I18N
					if(this.options.placeholder){
						this._input.attr("placeholder", this.options.placeholder);	// No I18N
						this._resizeSearchArea(this._input, this.options.placeholder);
					}
				}
			}
		},
		_mousedownHandler: function(orgEvent){
			var val = this._editTextBox ? this._editTextBox.val() : this._input.val();
			var token = $(orgEvent.target).closest(".ztokenfield__token"); 
		//	if($(document.activeElement).attr("id") !== this._input.attr("id")){	// No I18N
				if(!(this._editTextBox !== undefined && this._editTextBox.attr("id") !== $(document.activeElement).attr("id"))){	// No I18N
					if(!token.length && val.length < this.options.search.maxKeywordLength){
						this._searchTimeout(val, this._input, orgEvent);
					}
				}
			//}
			if(this._editTextBox !== undefined && $(orgEvent.target).attr("id") !== this._editTextBox.attr("id")){	// No I18N
				this._reformToken(orgEvent);
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}
			if(!(orgEvent.metaKey || orgEvent.ctrlKey || orgEvent.shiftKey)){
				this.container.find(".is-selected").removeClass("is-selected");	// No I18N
				this._clickReset = true;
			}	
			if(token.length){
				token.addClass("is-selected").focus();	// No I18N 	
				this._menu.is(":visible") && this._hideSuggestions(); // No I18N 
			} 
		},
		_focusHandler: function(orgEvent){
			this._reverseTab = false;

			clearTimeout(this._timeout);

			ZC.$window.off("keyup."+this.componentName).on("keyup."+this.componentName, this._windowKeyUpHandler.bind(this)) // No I18N

			this._timeout = setTimeout(this._focusTimeoutHandler.bind(this),200);
			this.container.addClass('has-focus');	// No I18N
		},
		_windowKeyUpHandler: function(orgEvent){
			if(orgEvent.keyCode === this._keyCode.TAB && orgEvent.shiftKey){
				this._reverseTab = true;
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}else if(orgEvent.keyCode === this._keyCode.TAB){
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}
		},
		_pasteEventHandler: function(orgEvent){
			this._pasteTimeout && clearTimeout(this._pasteTimeout);
			this._pasteTimeout = setTimeout(this._pasteHandler.bind(this, orgEvent), 100);
		},
		_contextMenuHandler: function(orgEvent){
			orgEvent.preventDefault();
		},
		_inputFocusHandler: function(orgEvent){
			this.container.find(".is-selected").removeClass("is-selected");		// No I18N
			this.container.addClass('has-focus'); // No I18N
		},
		_inputBlurHandler: function(orgEvent){
			var val = this._input.val();
			if(!this._menu.find("li").first().length){		// No I18N
				if(!this.options.acceptNewValues && this.options.dataValidation && val && val.length){
					this._addErrorToken(this._input.val(),{keyCode: this._keyCode.TAB});
				}else if(this.options.acceptNewValues){
					this._checkIsContentValid(this._input.val(), {keyCode: this._keyCode.TAB});
				}
			}
			this.container.removeClass("has-focus");	// No I18N
		},
		_bindEvents: function(){
			var componentName = this.componentName;

			this._lastTokenUpdate = [];

			ZC.$document.on("click."+componentName, this._docClickHandler.bind(this));	// No I18N

			this.container.on('keydown.'+componentName, this._keydownHandler.bind(this))  // No I18N

			.on('keyup.'+componentName, this._keyUpHandler.bind(this)) // No I18N

			.off('mousedown.'+componentName).on('mousedown.'+componentName, this._mousedownHandler.bind(this))  // No I18N

			.off('focus.'+componentName).on('focus.'+componentName, this._focusHandler.bind(this)) // No I18N

			.off('paste.'+componentName).on('paste.'+componentName, this._pasteEventHandler.bind(this))	// No I18N
				
			.off('contextmenu.'+componentName).on('contextmenu.'+componentName, this._contextMenuHandler.bind(this))	// No I18N
				
			this._input.off('focus.'+componentName).on('focus.'+componentName, this._inputFocusHandler.bind(this)) // No I18N			
				
			.off('blur.'+componentName).on("blur."+componentName, this._inputBlurHandler.bind(this))	// No I18N
				
		},
		_undoTimeoutHandler: function(){
			this._triggerPasteHelper();
			this._input.focus();
		},
		_focusTimeoutHandler: function(){
			!this._reverseTab && this._input.focus();
		},
		_pasteHandler: function(orgEvent){
			var target = $(orgEvent.target);
			var values = target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && target.val();	// No I18N
			values && values.length && this._resizeSearchArea(target, values); 
			var tokenValues = values.split(this.options.delimiter);
			for(var i=0; i<tokenValues.length;i++){
				//if(base._pasteHelperTextBox.val().length){
				var itemName = tokenValues[i].match(/\"(.*)\"/) && tokenValues[i].match(/\"(.*)\"/).pop(),
					itemKey = tokenValues[i].match(/\<(.*)\>/) && tokenValues[i].match(/\<(.*)\>/).pop(), 
					jsonObj = {};
				if(itemKey !== null  && itemName !== null){
					jsonObj.itemValue = {};
					jsonObj.itemValue.itemIdentifier = itemKey;
					jsonObj.itemValue.text = itemName;
					jsonObj.itemValue.titleText = itemKey;
				 	this._addToken(jsonObj, orgEvent, undefined, undefined, true);
				}else{
					this._newQuery = true;
					var value = this._search(tokenValues[i], target).done(this._convertPastedTextToTokens(tokenValues[i].trim()));
					if(value == false){
						return false;
					}
				}
			}	
			if(this._editTextBox !== undefined){
				this._editTextBox.remove();
				this._editTextBox = undefined;
			}
		},
		_extendEvents: function(){
			this._EVENTS = $.extend(true, {}, this._TOKENFIELDEVENTS, this._EVENTS);
		},
		_bindMenuEvents: function(){
			this._super();
			var base = this;
			this._menu.off("zmenubeforeitemchecked").on("zmenubeforeitemchecked", function(orgEvent, ui){	//No I18N
				if(base.options.dropdownList.itemType === "checkbox" && ui.data.checked !== "checked" && base.options.maxAllowedValues && base._tokensCount >= base.options.maxAllowedValues){	//No I18N
					orgEvent.preventDefault();
					base._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"}, orgEvent);	//No I18N
				}
			})
		},
		_convertPastedTextToTokens: function(tokenValue){
			!this._menu.is("visible") && this.options.acceptNewValues && this._checkIsContentValid(tokenValue);	// No I18N
		},
		_handleTokenRemove: function(textbox, orgEvent){
			var element = this.container.find('.is-selected');	// No I18N
			if(!element.length){
				this._element = textbox.prev()
				if(this._element.hasClass('ztokenfield__token')){	// No I18N
					this._element.addClass('is-selected');	// No I18N
					this._removeToken(this._element.find('.ztokenfield__close'), orgEvent);	// No I18N
					textbox.focus();
				}
				if(this._editTextBox !== undefined){
					this._editTextBox.remove();
					this._editTextBox = undefined;
					this._input.focus();
				}
			}else{
				for(var i=0; i<element.length;i++){
					this._removeToken($(element[i]).find('.ztokenfield__close'), orgEvent);	
					textbox.focus();
				}
			}
			this.options.dropdownList.itemType === "none" && this._hideSuggestions();	//No I18N
			//this._searchTimeout('', this._input, orgEvent);	//No I18N
			if(textbox.attr("id") === this._input.attr("id")){	//No I18N
				this._resizeSearchArea(textbox, this.options.placeholder ? textbox.attr("placeholder") : undefined);	//No I18N
			}
		},
		_handleSelection: function(orgEvent, ui){
			if(!(ui.menuitem.attr("id") === this.componentName+"noresults" || ui.menuitem.hasClass("zmenu__title") || ui.menuitem.attr("id") === this.componentName+"loaderror")){	// No I18N
				var textbox = this._editTextBox !== undefined ? this._editTextBox : this._input;						
				if(ui.menuitem.attr("id") === this.componentName+"moreaction"){	// No I18N
					var action = this.options.dropdownList.moreResultsAction;
					if(action && typeof action === "function"){  // No I18N
						action();
					}
					this._dispatchEvent("MOREACTIONSELECT", {value: ui.menuitem, query: this._params.query}, orgEvent);  // No I18N
				}else{
					var itemData = ui.menuitem.data().metadata;
					if(!this.options.dropdownList.closeOnSelect || this.options.dropdownList.itemType === "checkbox"){  // No I18N
						orgEvent.preventDefault();
					}
					this._lastSelectedMenuItem = ui.menuitem;
					if(!($(ui.menuitem).prop("checked")) && this.options.dropdownList.itemType === "checkbox"){	// No I18N
						var itemKey = itemData.itemValue !== undefined ? itemData.itemValue.itemIdentifier : this._mapJSONToData(itemData, this._dataMapping.itemIdentifier);
						this.removeToken(itemKey, orgEvent);
					}else{
						if(!this._isJSONSource){
							return;
						}else{	
							this._dispatchEvent("ITEMSELECT", {value: ui.menuitem}, orgEvent);  // No I18N
							this._selectedItem = this.getData(ui.menuitem);
							if(this._editTextBox){
								var index = this._currentTokenData.metadata ? this._currentTokenData.metadata.index : this._currentTokenData.index;
								var id = this._currentTokenData.metadata ? this._currentTokenData.metadata.id : this._currentTokenData.tokenId;
								this._buildToken(ui.menuitem, textbox, orgEvent, index, id);
							}else{
								this._buildToken(ui.menuitem, textbox, orgEvent);
							}
							if(this._editTextBox !== undefined){
								this._editTextBox.remove();
								this._editTextBox = undefined
							}
						}						
					}	
					if(this.options.dropdownList.itemType === "checkbox" && this.options.dropdownList.position === "after-cursor"){		// No I18N
						this._input.focus();
						var pos = this._positionMenuAtCursorPosition(this._input);
						this._menu.css({top: (pos.top + 15), left: pos.left});	
					}
				}
			}else{
				this._input.focus();
			}
		},
		_handleHover: function(orgEvent, ui){
			if(!(ui.menuitem.attr("id") === this.componentName+"noresults") && !(ui.menuitem.hasClass("zmenu__title")) && !(ui.menuitem.attr("id") === this.componentName+"moreaction")){	// No I18N
				if(!this._isJSONSource){
					this._input.val(ui.menuitem.data().metadata.label);
				}else{
					var value = ui.menuitem.data().itemValue !== undefined ? ui.menuitem.data().itemValue.text : this._mapJSONToData(ui.menuitem.data().metadata, this._dataMapping.text);
					this._input.val(value);
				}
			}else{
				this._input.val(this._params.query);
			}
			this._resizeSearchArea(this._input);
		},
		_removeAddedItem: function(){
			if(this._tokensCollection.length){
				var base = this, addedTokens;
				 addedTokens = this._tokensCollection.map(function(item){
				 	if(item.data("invalidToken") !== true){	// No I18N
					 	var value = item.data().metadata[base._dataMapping.value]; 
						return value;	
					}
				})
				var base = this;
				//Removes already added tokens
				$.each(addedTokens, function(index,item){
					base._filteredResult = $.grep(base._filteredResult , function(val){
				 		var value = base._mapJSONToData(val, base._dataMapping.value);
				 		if(item === undefined || (item !== undefined && value !== undefined && value.toString() !== item.toString())){
				 			return true;
				 		}
					})
				})			
			}	
		},
		_search: function(query, container, orgEvent){
			return	this._super(query, container, orgEvent);	
		},
		_updateCheckedState: function(){
			if(this.options.dropdownList.itemType === "checkbox"){
				var selectedId = [];
				if(this._tokensCollection.length){
					for(var i=0; i<this._tokensCollection.length;i++){
						var tokenData = this._tokensCollection[i].data().metadata, id;
						if(tokenData !== undefined && !tokenData.isNewValue){
							id = this._mapJSONToData(tokenData, this._dataMapping.value);
						}
						this._menu.find("#"+id).length && selectedId.indexOf("#"+id) === -1 && selectedId.push("#"+id); // No I18N 
					}
					selectedId.length && this._menu.zmenu("setMenuItemsAttribute",selectedId,"checked",true); // No I18N
				}
			}
		},
		_renderSuggestions: function(container, orgEvent){
			this._super(container, orgEvent);
			this._updateCheckedState();
			if(this._loader){
				this._loader.closest(".zmenu--loading").remove();	// No I18N
				this._loader = null;
			}
			this._menu.removeData("fixedArea");	// No I18N
		},
		_buildToken: function(menuItem, textbox, orgEvent, currentIndex, currentId){
			var itemKey, itemName, itemTitle, itemImage, maxAllowed = this.options.maxAllowedValues, values;
			if(values = this._selectedItem.itemValue){
				itemKey = values.itemIdentifier;
				itemName = values.text;
				itemTitle = values.titleText;
				itemImage = this.options.tokens.contentType === "image-text" ? values.image : false; // No I18N
			}			
			if(!maxAllowed || (maxAllowed && (this._tokensCount < maxAllowed))){
				this._addCustomCloseIcon();
				this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>"); 
				this._tokenText = $('<span class ="ztokenfield__text">').text(itemName);	// No I18N
				itemTitle !== undefined && this._tokenText.attr("title", itemTitle); // No I18N
				if(itemImage !== false && itemImage !== undefined){
					var tokenImage = !this.options.isImageSourceSprited ? $("<img class='ztokenfield__image' src='"+itemImage+"'>") : 	// No I18N
				    $("<i class='ztokenfield__image'>").addClass(itemImage);	
					tokenImage.appendTo(this._tokenContainer);
				}
				this._insertToken(menuItem, textbox, orgEvent, currentIndex, currentId);
			}else if(maxAllowed && this._tokensCount >= maxAllowed){				
				this._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"}, orgEvent);	//No I18N
			}
		},
		_animateHandler: function(token, orgEvent){
			var base = this;
			token.addClass('on-remove').animate({width: 0}, "fast", function(){	// No I18N
				var tokenToRemove = token.data().metadata;
				var selectedItems = base.options.values;		
				for(var i=0; i < selectedItems.length;i++){
					var itemKey = !tokenToRemove.invalidToken ? base._mapJSONToData(tokenToRemove, base._dataMapping.itemIdentifier) : tokenToRemove.id, 
							tokenToCompare = !tokenToRemove.invalidToken ? base._mapJSONToData(selectedItems[i], base._dataMapping.itemIdentifier) : selectedItems[i].id; 
					if(itemKey === tokenToCompare){
						selectedItems.splice(i,1);
						break;
					}
				}
				token.remove();
				token = null;
				base._checkContainerHeight();
				if(base.options.dropdownList.position === "after-cursor"){	// No I18N	
					base._input.focus();
					var pos = base._positionMenuAtCursorPosition(base._input);
					base._menu.css({top: (pos.top + 15), left: pos.left});
				}	
				if(base.options.placeholder && base.options.values.length === 0 && base._input.val().length === 0){
					base._input.attr("placeholder", base.options.placeholder);	// No I18N
					base._resizeSearchArea(base._input, base.options.placeholder);
				}
				base._dispatchEvent("AFTERTOKENREMOVE", tokenToRemove, orgEvent);  // No I18N				
			});
		},
		_removeToken: function(element, orgEvent, index){
			var canBeRemoved = this._dispatchEvent("BEFORETOKENREMOVE", $(element).parent(), orgEvent);  // No I18N
			var data, invalidToken, token, tokenEncountered = false, matchedId, id;
			if(canBeRemoved && element){
				var tokenToRemove = $(element).parent();
				for(var i=0; i < this._tokensCollection.length; i++){
					if(this._tokensCollection[i].attr("id") === tokenToRemove.attr("id")){	// No I18N
						tokenEncountered = true, matchedId = tokenToRemove.attr("id");	//No I18N
						data = this._tokensCollection[i].data().metadata, invalidToken = this._tokensCollection[i].data("invalidToken"), id = tokenToRemove.attr("id");	// No I18N
						if(this._menu.is(":visible")){		// No I18N
							if(this.options.dropdownList.itemType === "none"){ 
								this._hideSuggestions(); 
							}else{
								if($("#"+id).length && !invalidToken && !data.isNewValue){
									var id = this._mapJSONToData(data, this._dataMapping.value);
									this._menu.zmenu("setMenuItemsAttribute",["#"+id],"checked",false);	// No I18N
									this._lastSelectedMenuItem = $("#"+id);
									this._menu.zmenu("hoverMenuItem", this._lastSelectedMenuItem);	// No I18N
								}
							}
						}		
						invalidToken && (data.invalidToken = invalidToken);
						var token = this._tokensCollection[i];
						if(this.options.tokens.animateOnRemove && ((orgEvent && orgEvent.type !== "dblclick" && orgEvent.key !== "Enter") || orgEvent === undefined)){	// No I18N
							token.animate({ opacity: 0.01 }, "fast", this._animateHandler.bind(this, token, orgEvent));	// No I18N								
						}else{
							this._tokensCollection[i].remove();
							this._tokensCollection[i] = null, tokenToRemove = null, this._element = null;	
							this._checkContainerHeight();
							if(this.options.dropdownList.position === "after-cursor"){	// No I18N	
								this._input.focus();
								var pos = this._positionMenuAtCursorPosition(this._input);
								this._menu.css({top: (pos.top + 15), left: pos.left});
							}							
						}		
						this._tokensCollection.splice(i, 1);
						for(var j = i; j< this._tokensCollection.length; j++){
							this._tokensCollection[j].data().metadata.index = j;
						}
						if(this._editTextBox === undefined && (index === undefined || (orgEvent && (orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 89))){
							this._performedStack.splice(i, 1);		
							this._performedStack.push({"action":"Remove", tokenData: data, tokenId: id, index: i});	// No I18N
						}else if(this._editTextBox === undefined && index !== undefined && (orgEvent && (orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 90)){
							this._recallStack.push({"action": "Insert", tokenData: data, tokenId: id, index: i});		// No I18N
						}
						!invalidToken && this._tokensCount--;
						break;
					}
				}
				if(!this.options.tokens.animateOnRemove || (this.options.tokens.animateOnRemove && orgEvent && (orgEvent.type === "dblclick" || orgEvent.key === "Enter"))){
					var selectedItems = this.options.values, tokenToCompare,
						itemKey = !invalidToken ? this._mapJSONToData(data, this._dataMapping.itemIdentifier) : matchedId;
					for(var i=0; i < selectedItems.length;i++){
							tokenToCompare = selectedItems[i].errorValue === undefined ? this._mapJSONToData(selectedItems[i], this._dataMapping.itemIdentifier) : (data ? data.id : undefined); 
						if(itemKey && tokenToCompare && itemKey === tokenToCompare){
							this.options.values.splice(i,1);
							break;
						}
					}
					if(this.options.placeholder && this.options.values.length === 0 && this._input.val().length === 0){
						this._input.attr("placeholder", this.options.placeholder);	// No I18N
						this._resizeSearchArea(this._input, this.options.placeholder);
					}
					tokenEncountered && (this._dispatchEvent("AFTERTOKENREMOVE", data, orgEvent));	  // No I18N			
				}				
			}
		},
		_bindTokenEvents: function(){
			var base = this;
			if(!this.options.disabled){
				this._closeButton.off('click.'+this.componentName).on('click.'+this.componentName, function(orgEvent){	// No I18N
					orgEvent.stopPropagation();
					base._removeToken(orgEvent.currentTarget, orgEvent);
					base._input.focus();
				})
				this._tokenContainer.off("focus."+this.componentName)	// No I18N
			    .on("focus."+this.componentName, function(orgEvent){	// No I18N
					base.container.addClass("has-focus");	// No I18N
				})
				this._tokenContainer.off("click."+this.componentName).on("click."+this.componentName, function(orgEvent){	// No I18N
								
				})
				this.options.tokens.editable && this._tokenContainer.off("dblclick."+this.componentName).on("dblclick."+this.componentName, function(orgEvent){	// No I18N
					base._editToken($(orgEvent.currentTarget), orgEvent);
				})
			}
		},
		_tokenClickHandler: function(orgEvent){
			var currentTarget = $(orgEvent.currentTarget);
			base._clickReset = false;
			base.container.addClass("has-focus");	// No I18N
			if(!(orgEvent.metaKey || orgEvent.ctrlKey || orgEvent.shiftKey)){
				base.container.find(".is-selected").removeClass("is-selected");	// No I18N
				base._clickReset = true;
			} 
			if(orgEvent.shiftKey){
				var elementsLength = base.container.find(".is-selected").length;	// No I18N
				var firstIndex = $(base.container.find(".is-selected")[0]).data().metadata.index,	
					 lastIndex = $(base.container.find(".is-selected")[elementsLength - 1]).data().metadata.index;	
				for(var i = firstIndex; i<=lastIndex; i++){
					base.container.find("#token-"+i).addClass("is-selected");	// No I18N
				}
				base._clickReset = false;
			}
			currentTarget.addClass("is-selected"); // No I18N
			currentTarget.focus();	
			base._triggerPasteHelper();	
		},
		_triggerPasteHelper: function(){
			var activeTokens = this.container.find(".ztokenfield__token.is-selected");	// No I18N
			var base = this;
			base._activeTokens = [];
			var finalValue = $.map(activeTokens, function(token, j){
				var key, value, data;
				base._activeTokens[j] = $(token).attr("id");	
				data = $(token).data().metadata;
				if($(token).data("invalidToken") !== true && data){	// No I18N
					key =  base._mapJSONToData(data, base._dataMapping.itemIdentifier), value = base._mapJSONToData(data, base._dataMapping.text);				
					var finalValue = base._pasteRegExp.replace(/#value#/g, value).replace(/#id#/g, key);
					return finalValue;	
				}			
			}).join(this.options.delimiter);
			this._pasteHelperTextBox.val(finalValue).select();
		},
		_editToken: function(tokenContainer, orgEvent){
			if(tokenContainer.hasClass("ztokenfield__token")){	// No I18N
				this._editTextBox = $("<input type='text' id='dblclick-input-"+this.componentName+"'>").addClass("ztokenfield__textbox");	// No I18N
				var value, matchFound, itemData, dataValue = tokenContainer.data();	
				dataValue = dataValue.invalidToken;	
				this._tokenOption = this.options.tokens;
				if(dataValue !== true){	
				    itemData = tokenContainer.data().metadata;
				    this._currentTokenData = itemData;
				    this._currentTokenData.tokenId = tokenContainer.attr("id");	// No I18N
				    var tokenKey, tokenValue
				    if(itemData !== undefined){
				    	tokenValue = this._mapJSONToData(itemData, this._dataMapping.text),
				    	tokenKey = this._mapJSONToData(itemData, this._dataMapping.itemIdentifier);
				    }
					try{
						var pattern = this._tokenOption.editable.pattern ? this._tokenOption.editable.pattern : this._CONSTANTS.pattern;
						if(pattern.match(/#value#/g) || pattern.match(/#id#/g)){
							value = pattern.replace(/#value#/g, tokenValue).replace(/#id#/g, tokenKey);
							matchFound = true;
							this._editTextBox.insertBefore(tokenContainer);					
							this._removeToken(tokenContainer.find(".ztokenfield__close"), orgEvent);	// No I18N	
							this._editTextBox.focus()
						}else{
							throw(this._CONSTANTS.patterFormatError);	
						}
					}catch(error){
						throw error;
					}
				}else{
					this._editTextBox.insertBefore(tokenContainer).addClass("invalid"); // No I18N
					value = tokenContainer.find(".ztokenfield__text").text();	// No I18N
					matchFound = true;
					itemData = tokenContainer.data();
					this._currentTokenData = itemData;
					this._currentTokenData.tokenId = tokenContainer.attr("id");	// No I18N
					this._removeToken(tokenContainer.find(".ztokenfield__close"), orgEvent); 	// No I18N
					this._editTextBox.focus()
				}
				if(this._input.attr("placeholder") !== undefined){		// No I18N
					this._input.removeAttr("placeholder");		// No I18N
					this._resizeSearchArea(this._input, undefined);
				}
				if(matchFound){
					matchFound = false;
					this._editTextBox.val(value).select();
					this._resizeSearchArea(this._editTextBox);
					this._input.val("");	// No I18N
					var base = this;
					this._editTextBox.off("focus."+this.componentName).on("focus."+this.componentName, function(orgEvent){	// No I18N
						base.container.addClass("has-focus");	// No I18N
					})
				}

			}
		},
		_insertToken: function(menuItem, textbox, orgEvent, index, tokenId, isNewValue){
			var base = this, metadata, isDuplicate, value = textbox.val();
			if(menuItem !== undefined){
				//Check whether the passed parameter is DOM Element
				metadata = (menuItem[0] && menuItem[0].nodeName) ? menuItem.data().metadata : menuItem;
			}
			metadata && metadata.errorValue === undefined && !this._defaultSelected && !this.options.allowDuplicateValues && (isDuplicate = this._isDuplicateToken(metadata.itemValue !== undefined ? metadata.itemValue : metadata));
			!isDuplicate && (this.options.dropdownList.itemType !== "checkbox" && this.options.dropdownList.closeOnSelect) && textbox.val(""); //No I18N	
			if(this._tokensCollection.length === 0){
				this._input.attr("placeholder") && this._input.removeAttr("placeholder");	 // No I18N
				this._resizeSearchArea(this._input, undefined); 		
			}
			if(!isDuplicate){
				var returnValue = this._dispatchEvent("BEFORETOKENADD", {token: this._tokenContainer, tokenData: metadata ? metadata.itemValue : undefined}, orgEvent); // No I18N
				if(returnValue){
					this._closeButton.appendTo(this._tokenContainer);
					this._tokenText.appendTo(this._tokenContainer);
					tokenId !== undefined ? this._tokenContainer.attr("id", tokenId) :  // No I18N
					this._tokenContainer.attr("id","token-"+this._uniqueIndex++);	// No I18N
					(index || index === 0) ? (index < this._tokensCollection.length ? this._tokenContainer.insertBefore(this._tokensCollection[index]) : this._tokenContainer.insertBefore(textbox)) : this._tokenContainer.insertBefore(textbox);
					if(this.options.tokens.sortable){
						//Hides menu when token is dragged with mouse
						this.container.off("zsortablemousedrag").on("zsortablemousedrag", function(orgEvent){	// No I18N
							base._menu.is(":visible") && base._hideSuggestions();	// No I18N
						})
					}
					var data = this.getData(this._tokenContainer);
					//Adding the remaining metadata to the data attribute of token
					if(metadata){
						if(!metadata.errorValue){
							if(metadata.itemValue){
								for(var i in metadata.itemValue){
									if(i != "id"){
										if(i in this._dataMapping){
											data[this._dataMapping[i]] = metadata.itemValue[i]
										}		
									}
								}	
							}else{
								for(var key in metadata){
									if(i != "id"){
										data[key] = metadata[key];
									}
								}
							}
							if(isNewValue){
								data.isNewValue = isNewValue;
							}
						}else{
							data.errorValue = metadata.errorValue
						}
					}
					data.index = index !== undefined ? index : this._uniqueIndex - 1;
					if(index !== undefined){
						for(var i=index; i<this._tokensCollection.length;i++){
							this._tokensCollection[i].data().metadata.index = (i+1);
						}
					}
					if(this.options.tokens.sortable){
						var data = this.container.data("zsortable");	// No I18N
						data && this.container.zsortable("destroy");	// No I18N
						this.container.zsortable({leave: "ztokenfield__close"});	// No I18N
					}
					this.options.maxAllowedValues && metadata && !metadata.errorValue && this._tokensCount++;
					if(!this._defaultSelected){
						var data = this._tokenContainer.data().metadata, jsonData = {};
						if(data && data.errorValue === undefined){
							for(var key in this._dataMapping){
								this._dataMapping[key] !== undefined && (jsonData[this._dataMapping[key]] = data[this._dataMapping[key]]);
							}
						}else{
							jsonData = data;
							jsonData.type = "error";		// No I18N
						}			
						if(!this.options.values.length){
				    		this.options.values.push(jsonData);
				    	}else{   	
				    		index === undefined ? this.options.values.push(jsonData) : this.options.values.splice(index, 0, jsonData);
				    	}
						this.element.data("selectedTokens", this.options.values);	// No I18N
					}		
					this._dispatchEvent("AFTERTOKENADD", {token: this._tokenContainer}, orgEvent);  // No I18N
					this._checkIfTokenAdded(this._tokenContainer, index);
					this._resizeSearchArea(this._input, undefined); 	
					this._bindTokenEvents();
					orgEvent && orgEvent.keyCode !== this._keyCode.TAB && this._input.focus();
					if(index !== undefined && ((orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 90)){ 
						this._recallStack.push({"action": "Remove", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: index});	// No I18N
						this._performedStack.splice(index, 0, {"action" : "Insert", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: index});	// No I18N
					}else if(index !== undefined && ((orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 89)){
						this._performedStack.push({"action" : "Insert", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: index});	// No I18N
					}else{
						index === undefined && this._performedStack.push({"action" : "Insert", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: this._tokensCollection.length - 1}) ;	// No I18N
					}
					this._checkContainerHeight();	
				}
				
				this._editTextBox !== undefined && (this._tokenContainer.focus().addClass("is-selected"));
				if(this.options.dropdownList.itemType === "checkbox"){
					if(this._menu.is(":visible")){	// No I18N
						value.length && this._searchTimeout(this._params.query, textbox, orgEvent);	
					}
				}
			}else{
				if(this.options.dataValidation){
					this._addErrorToken(menuItem.itemValue ? menuItem.itemValue.text : textbox.val(), orgEvent);
				}
				this._dispatchEvent("TOKENADDFAIL", {failureType: "Duplicate"}, orgEvent);	//No I18N
			}
			this._tokenContainer = null, this._tokenText = null, this._closeButton = null;
		},
		_checkContainerHeight: function(){
			var currentHeight = this.container.css("height");	// No I18N
			if(currentHeight !== this._prevContainerHeight && this._menu.is(":visible")){
				if(this._lastSelectedMenuItem || this._menu.find("li").length){
					this._menu.zmenu("show", {"mustBeShown":true, preventHover: true, positionAlterable: true});	// No I18N
					var item = this._lastSelectedMenuItem ? this._lastSelectedMenuItem : $(this._menu.find("li")[this._dataMapping.groupLabel ? 1 : 0]);
					this._menu.zmenu("hoverMenuItem", item);	// No I18N
				}
			}
			this._prevContainerHeight = currentHeight;
		},
		_reformToken: function(orgEvent){
			var itemData = this._currentTokenData;
			if(!this._escKeyPressed){
				var dataHandled = this._dispatchEvent("TOKENUPDATE", {textboxValue: this._editTextBox.val(), itemContent: this._currentTokenData}, orgEvent); // No I18N
				itemData = (typeof dataHandled === "object") ? dataHandled.itemContent : itemData;	// No I18N
				value = (typeof dataHandled === "object") ? dataHandled.itemContent.invalidToken : itemData.invalidToken;	// No I18N
			}else{
				if(!itemData.invalidToken){
					value = false;
				}else{
					value = true;
				}		
			}
			if(value !== true){	
				var itemKey =  this._mapJSONToData(itemData, this._dataMapping.itemIdentifier), 
					itemValue = this._mapJSONToData(itemData, this._dataMapping.text), 
					itemTitle =  this._mapJSONToData(itemData, this._dataMapping.titleText),
					itemIcon =  this._mapJSONToData(itemData, this._dataMapping.image);
				if(itemValue.length){
					this._addCustomCloseIcon();
					this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>");  
					this._tokenText = $('<span class ="ztokenfield__text">').text(itemValue);	// No I18N
					itemTitle !== undefined && this._tokenText.attr("title", itemTitle);	// No I18N
					if(this.options.tokens.contentType === "image-text" && itemIcon !== false && itemIcon !== undefined){	// No I18N
						var tokenImage = !this.options.isImageSourceSprited ? $("<img class='ztokenfield__image' src='"+itemIcon+"'>") : 	// No I18N
					    $("<i class='ztokenfield__image'>").addClass(itemIcon);	
						tokenImage.appendTo(this._tokenContainer);
					}
					var data = {};
					data.itemValue = {};
					for(var key in this._dataMapping){
						data.itemValue[key] = this._mapJSONToData(itemData, this._dataMapping[key]);
					}
					this._insertToken(data, this._editTextBox, orgEvent, itemData.index, itemData.tokenId);		
					this._performedStack.push({"action" : "Insert", tokenData: itemData, tokenId: itemData.tokenId, index: itemData.index});	// No I18N
				}
			}else{
				if(!this._escKeyPressed){
					var value = this._editTextBox.val();
					if(value.length){
						this._addErrorToken(value, orgEvent, itemData.index, itemData.tokenId);
					}
				}else{
					if(itemData.invalidToken){
						this._addErrorToken(itemData.metadata.errorValue, orgEvent, itemData.index, itemData.tokenId);
					}
				}			
			}	
			this._editTextBox.remove();
			this._editTextBox = undefined;	
			this._escKeyPressed = false;		
			//this._triggerPasteHelper();					
		},
		_checkIfTokenAdded: function(token, index){
			var isExists = false;
	    	if(!this._tokensCollection.length){
	    		this._tokensCollection.push(token);
	    	}else{   	
	    		index === undefined ? this._tokensCollection.push(token) : this._tokensCollection.splice(index, 0, token);
	    	}
		},
		_isDuplicateToken: function(tokenData){
			var isDuplicate = false;
			for(var i=0;i<this.options.values.length;i++){
				if(tokenData.itemIdentifier === this.options.values[i][this._dataMapping.itemIdentifier]){
					isDuplicate = true;
					break;
				}else{
					isDuplicate = false;
				}
			}
			return isDuplicate;
		},
		_addToken: function(value, orgEvent, index, tokenId, isNewValue, target){
			if(value.invalidToken !== true){
				var maxAllowed = this.options.maxAllowedValues;
				if(!maxAllowed || (maxAllowed && (this._tokensCount < maxAllowed))){
					var itemKey = value[this._dataMapping.itemIdentifier] ? value[this._dataMapping.itemIdentifier] : (value.itemValue && value.itemValue.itemIdentifier);
					var itemName = value[this._dataMapping.text] ?  value[this._dataMapping.text] :(value.itemValue && value.itemValue.text);
					var itemTitle = value[this._dataMapping.titleText] ?  value[this._dataMapping.titleText] :(value.itemValue && value.itemValue.titleText);
					var icon = value[this._dataMapping.image] ? value[this._dataMapping.image] : (value.itemValue && value.itemValue.image);
					this._addCustomCloseIcon();
					this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>"); 
					this._tokenText = $('<span class ="ztokenfield__text">').text(itemName);	// No I18N
					itemTitle !== undefined && this._tokenText.attr("title", itemTitle);	// No I18N
					if(this.options.tokens.contentType === "image-text" && icon !== undefined){	// No I18N
						var tokenImage = !this.options.isImageSourceSprited ? $("<img class='ztokenfield__image' src='"+icon+"'>") : 	// No I18N
					    $("<i class='ztokenfield__image'>").addClass(icon);	
						tokenImage.appendTo(this._tokenContainer);
					}
					var textbox = this._editTextBox !== undefined ? this._editTextBox : this._input;	
					this._insertToken(value, textbox, orgEvent, index, tokenId, isNewValue);
				}else if(maxAllowed && this._tokensCount >= maxAllowed){				
					this._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"}, orgEvent);	//No I18N
				}
			}else{
				this._addErrorToken(value.errorValue, orgEvent, index, tokenId);
			}	
		},
		_removeTokens: function(tokens, value){
			for(var j=0; j<tokens.length;j++){
				var metadata = this.getData($(tokens[j]));
				var itemKey = this._mapJSONToData(metadata, this._dataMapping.itemIdentifier);
				if(itemKey == value){
					var tokenItem = $(tokens[j]).find(".ztokenfield__close");	
					this._removeToken(tokenItem);
					if(!this.options.allowDuplicateValues){
						break;
					}
				}
			}
		},
		_getTokens: function(selectedItems, value){
			for(var j=0; j<selectedItems.length;j++){
				var itemKey = this._mapJSONToData(selectedItems[j], this._dataMapping.itemIdentifier);
				if(itemKey === value){
					return selectedItems[j];
				}
			}
		},
		/** Creates multiple tokens with the given data and add it to the container.
			@function token-field.addTokens
			@param {Array} tokenData - The related data for the tokens to be added should be passed as an array.
			@example <caption> Example </caption>
	var token = $(".selector").ztokenfield("addTokens",[{
	        countryName: "India", 
	        flag:"image.jpg", 
	        countryCode: 345, 
	        continentName: "Asia"
	    },
	    {	countryName: "Brazil", 
	        flag:"image1.jpg", 
	        countryCode: 457, 
	        continentName: "South America"
	    }]);*/
		addTokens: function(value){
			if(value && $.isArray(value)){
				for(var i=0; i<value.length;i++){
					var prevIndex = i;
					var returnValue = this.addToken(value[i], i);
					if(returnValue){
						value.splice(i, 1);
					}
					returnValue && (i = prevIndex - 1);
				}
			}
		},
		/** Creates the token with the given data and add it to the tokenfield container.
			@function token-field.addToken
			@param {Object} tokenData - The related data for the token need to be passed as an object
			@example <caption> Example </caption>
var token = $(".selector").ztokenfield("addTokens",{countryName: "India",flag:"image.jpg",countryCode: 345,continentName: "Asia"});*/                                              
		addToken: function(value, index){
			if(value && !$.isArray(value)){
				var tempObj = {};
				tempObj.itemValue = {};
				for(var key in this._dataMapping){				
					tempObj.itemValue[key] = value[this._dataMapping[key]]
				}	
				var shouldPreventAdd,isValidValue;
				if(this.options.dataSource && !this._ajaxSource && value.type !== "error"){
					for(var i=0;i<this.options.dataSource.length;i++){
						if(this.options.dataSource[i][this._dataMapping.itemIdentifier] === tempObj.itemValue.itemIdentifier){
							isValidValue = true;
							break;
						}else{
							isValidValue = false;
						}
					}
					if(!this.options.acceptNewValues){
						if(!isValidValue){
							shouldPreventAdd = true;
						}
					}
				}
				if(value.type !== "error"){	//No I18N
					var maxAllowed = this.options.maxAllowedValues
					if(!shouldPreventAdd && ((maxAllowed && this._tokensCount < maxAllowed) || !maxAllowed)){
						this._addToken(tempObj);
						if(!isValidValue && this.options.acceptNewValues){
							if(this.options.dropdownList.acceptNewValues && this.options.dataSource instanceof Array){
								this.options.dataSource.push(value);
							}
						}
					}else if(maxAllowed && this._tokensCount >= maxAllowed){
						this._defaultSelected && this.options.values.splice(index, 1);
						this._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"});	//No I18N
						return true;
					}else if(shouldPreventAdd){
						this._defaultSelected && this.options.values.splice(index, 1);
						this._dispatchEvent("TOKENADDFAIL", {failureType: "deny new value"});	//No I18N
						return true;
					}
				}else{
					value.type === "error" && this._addErrorToken(value.errorValue);	//No I18N
				}
			}
		},
		/** To retrieve multiple tokens, pass the unqiue key as an array.
			@function token-field.getValues
			@param {Array} searchKey - The unique keys whose associated token need to be retrieved 
			@return {Array} Array of JSON containing information about the passed token keys.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("getValues",["aishwaryalakshmi.p@zohocorp.com","ashwini@zohocorp.com"]);*/
		getValues: function(value){
			var tokens = [];
			//this.container.focus();
			if($.isArray(value) && this.options.values && this.options.values.length){
				for(var i=0;i<value.length;i++){
					tokens.push(this._getTokens(this.options.values, value[i]));
				}
				return tokens;
			}
		},
		getValue: function(value){
			//this.container.focus();
			if(value !== undefined && !$.isArray(value)){
				return this._getTokens(this.options.values, value);	
			}
		},
		/** Retrieves all the tokens in the container.
			This signature doesn't accept any arguments.
			@function token-field.getAllValues
			@return {Object} JSON data of all added tokens.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("getAllValues");*/
		getAllValues: function(){
			return this.options.values;
		},
		/** To remove multiple tokens, pass the unqiue key as an array.
			@function token-field.removeTokens
			@param {array} searchKey - The unique keys whose associated token need to be removed 
			@return {array} value.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("removeTokens",["aishwarya@zohocorp.com","ashwini@zohocorp.com"]);*/
		removeTokens: function(value){
			var tokens = [];
			var allTokens = this.container.find(".ztokenfield__token"); // No I18N 
			if($.isArray(value) && allTokens && allTokens.length){
				for(var i=0;i<value.length;i++){
					this._removeTokens(allTokens, value[i])
				}
			}
			//this._input.focus();		
		},
		removeToken: function(value){
			var tokens = [], allTokens = this.container.find(".ztokenfield__token"); // No I18N 
			if(value !== undefined && !$.isArray(value)){
				this._removeTokens(allTokens, value);
			}
			//this._input.focus();
		},
		/** Removes all the tokens in the container.
			This signature doesn't accept any arguments.
			@function token-field.removeAllTokens
			@return {object} value.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("removeAllTokens"); */
		removeAllTokens: function(){
			var tokens = [], allTokens = this.container.find(".ztokenfield__token"); // No I18N 
			var removedTokenData = []
			for(var i=0; i<allTokens.length;i++){
				var data = $(allTokens[i]).data().metadata, jsonData = {};
				for(var key in this._dataMapping){
					this._dataMapping[key] !== undefined && (jsonData[this._dataMapping[key]] = data[this._dataMapping[key]]);
				}
				removedTokenData[i] = jsonData;
			}
			this._dispatchEvent("BEFORETOKENREMOVE", {tokensToBeRemoved: removedTokenData});  // No I18N
			allTokens.remove();
			this._dispatchEvent("AFTERTOKENREMOVE", {tokensRemoved: removedTokenData});  // No I18N
			this._tokensCollection = [];
			this.options.values = [];
			this._tokensCount = 0;
			//this._input.focus();
		},
		/** Indicates whether any error token exists inside the tokenfield container.
			This signature doesn't accept any arguments.
			@function token-field.hasInvalidTokens
			@return {boolean} value.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("hasInvalidTokens"); */
		hasInvalidTokens: function(){
			var hasErrorToken;
			if(this.options.dataValidation){
				for(var i=0; i<this.options.values.length;i++){
					if(this.options.values[i].type !== undefined && this.options.values[i].type === "error"){
						hasErrorToken = true;
						break;		
					}else{
						hasErrorToken = false;
					}
				}
				if(hasErrorToken){
					return true;
				}else{
					return false;
				}
			}else{
				return false;
			}
		},
		_disableContainer: function(){
			this.element.attr("disabled", true);	//No I18N
			this.container.addClass("is-disabled").attr("aria-disabled", "true");	//No I18N
			this.element.off("."+this.componentName);	//No I18N
			this.container.off("."+this.componentName).off(".zsortable").find("*").off("."+this.componentName).off(".zsortable");	//No I18N
			this._input.attr("disabled", true);	//No I18N
		},
		search: function(query){
			this._input[0].value = query;
			this._resizeSearchArea(this._input);
			this._searchTimeout(query, this._input);
		},
		/** Gets the value currently associated with the specified optionName.
			This signature doesn't accept any arguments.
			@function token-field.getAttribute
			@param {string} optionName - The name of the option to get.
			@return {string|object} value.
			@example <caption> Example </caption>
	var position = $(".selector").ztokenfield("getAttribute","placeholder");*/
		/** Sets the value of the option associated with the specified optionName.
			@function token-field.setAttribute
			@param {string} optionName - The name of the option to get.
			@param {string} value - A value to set for the option.
			@example <caption> Example </caption>
	$(".selector").ztokenfield("setAttribute","placeholder","Type a country"); */
		/** Gets an object containing key/value pairs representing the current tokenfield options.
			This signature doesn't accept any arguments.
			@function token-field.getAttributes
			@return {object} options object.
			@example <caption> Example </caption>
	var options = $(".selector").ztokenfield("getAttributes");*/
		/** Sets one or more options for the tokenfield.
			@function token-field.setAttributes
			@param {Object} options - A map of option-value pairs to set.
			@example <caption> Example </caption>
	$(".selector").ztokenfield("setAttributes",{"placeholder": "Type a country", "lazyLoading":true }); */									
		setAttribute: function(optionName, value){
			var optionsList = ["dataMapping", "lazyLoadData", "search", "dataSource", "autoComplete", "dropdownList", "isImageSourceSprited"];  //No I18N
			if(typeof value === "object" && !(value instanceof Array)){  // No I18N
				value = $.extend(true, {}, this.options[optionName], value);
			}
			var oldValue = this.options[optionName];
			this.options[optionName] = value;
			if(optionName === 'className'){	//No I18N
				oldValue && this.container.removeClass(oldValue);
				this.container.addClass(value);
			}else if(optionName === "values"){  	// No I18N
				this._defaultSelected = true;
				this.addTokens(this.options.values);
				this._defaultSelected = false;
			}else if(optionName === "placeholder"){	// No I18N
				if(value){
					this._input.attr("placeholder", value);	// No I18N	
					this._resizeSearchArea(this._input, this._input.attr("placeholder"));	// No I18N
				}else{
					this._input.removeAttr("placeholder");	// No I18N
				}
			}else if(optionName === "tokens"){	// No I18N
				if(typeof value === "object"){	// No I18N
					if(value.showRemoveButtonOnHover){
						this.container.removeClass("ztokenfield--closeshown");
					}else if(!value.showRemoveButtonOnHover){
						this.container.addClass("ztokenfield--closeshown");
					}
					if(oldValue.sortable){
						var data = this.container.data("zsortable");	//No I18N
						data && this.container.zsortable("destroy");	// No I18N
					}
					if(value.sortable && $.fn.zsortable){
						this.container.zsortable({leave: "ztokenfield__close", "container": "parent"});	//No I18N
					}
				}
			}else if(optionName === "disabled"){	// No I18N
				if(value){
					this._disableContainer();
				}else{
					if(this.element.attr("disabled")){	// No I18N
						this.element.removeAttr("disabled");	//No I18N
						this.container.removeClass("is-disabled");	//No I18N
						this._bindEvents();
						var base = this;
						//Bind removed events
						this.container.on("click."+this.componentName, '.ztokenfield__close', function(orgEvent){	//No I18N
						  	base._removeToken($(orgEvent.currentTarget), orgEvent);
						})
						//Enable sortable if disabled
						if(this.options.tokens.sortable){
							var data = this.container.data("zsortable");	// No I18N
							data && this.container.zsortable("enable");	// No I18N
						}
						this._input.removeAttr("disabled");	//No I18N
						this.container.focus();
						this.container.attr("aria-disabled", "false");		// No I18N
					}
				}
			}else if(optionsList.indexOf(optionName) > -1){	
				this._super(optionName, value);
			}else{
				this._super(optionName, value);
			}		
		},
		/** Removes the tokenfield functionality for the element completely. This will return the element back to its pre-init state.
			This method does not accept any arguments.
			@function token-field.destroy
			@example <caption> Example </caption>
	$(".selector").ztokenfield("destroy");*/
		destroy: function(){
			var componentName = this.componentName;
			this.container.off("."+componentName).off(".zsortable").find("*").off("."+componentName).off(".zsortable").remove(); 	//No I18N
			this.removeAllTokens();
			this._isLoaderInited = false;
			this._menu.zmenu("destroy");	// No I18N
			this._menu.remove();
			this.element.off("."+componentName);	//No I18N
			ZC.$document.off("."+componentName);	//No I18N
			this.container.remove();
			this.options.disabled && this.element.removeAttr("disabled");	//No I18N
			this.element.removeData(componentName).show();
		}
	}
	ZC.registerComponent("ztokenfield", $.zsuggestfield, tokenfieldPrototype);  // No I18N
	ZC.createTokenfield = function(initObject){
		var element = $("<div>").attr({id: initObject.id});	
		element.appendTo("body");	// No I18N
		return element.ztokenfield(initObject);
	}
})(ZComponents);
