/* $Id$ */
(function(ZC){
	/** @components tooltip
		Tooltip is a short description that appears while hovering over toolbar icons, truncated text and other elements.This replaces the native tooltip with rich configurations.Tooltip is shown only for elements with title attribute. It is not shown for disabled form elements(button, input, textarea, select, fieldset, keygen, output).
	*/
    var $ = ZC.DOMUtil;
	var tooltipPrototype = {
		options:{
			/** @member {String} tooltip.title
				Content to be displayed as tooltip. This is given in title attribute.
				@example <caption> Example: </caption>
	<button title='Align Left'><i class='icon alignleft'></i></button>
	$(selector).ztooltip()
			*/
			title: undefined,
			/** @member {Boolean} [tooltip.isMessageHTMLEncoded = false] 
				Specifies whether the tooltip content is encoded. By default, the tooltip content is encoded and displayed as tooltip. When applied in element level, it is prefixed with data-title-*.		
			*/
			isMessageHTMLEncoded: false,
			/** @member {Object|String} [tooltip.selector = undefined]
				Value can be dom-selector|dom-element. Tooltip is shown only for the matched elements. This is not an element level configuration.
				If not mentioned, tooltip is shown to all the elements with title attribute.
				@example <caption>  Example: </caption>
	$(selector).ztooltip({
		selector:':not(.is-disabled)'
	});
			*/
			selector: undefined,
			/** @member {Boolean} [tooltip.displayType = 'box'] Specifies whether the callout is attached to the tooltip. When set to 'box', tooltip will not have a callout. When position = 'at-cursor'|'follow-cursor', displayType is always set to 'box'.
			@allowedValues {('callout'|'box')}
			@example <caption> Property when applied in element level </caption>
	<button title='Align Left' data-title-display-type='callout'><i class='icon alignleft'></i></button>
	$(selector).ztooltip();	
			*/ 
			displayType: "box", //No I18N
			/** @member {String} [tooltip.position = 'at-cursor'] 
	Position at which tooltip is shown.When applied in element level, it is prefixed with data-title-*.	
	'top' - Tooltip is shown to the top of the target.
	'bottom' - Tooltip is shown to the bottom of the target.
	'right' -Tooltip is shown to the right of the target.
	'left' - Tooltip is shown to the left of the target.
	'top-left' - Tooltip is shown to the top of the target, with its left aligned to the left of the target.
	'top-right' - Tooltip is shown to the top of the target, with its right aligned to the right of the target.
	'bottom-left' - Tooltip is shown to the bottom of the target, with its left aligned to the left of the target.
	'bottom-right' - Tooltip is shown to the bottom of the target, with its right aligned to the right of the target.
	'at-cursor' - Tooltip appears at the bottom right position of the cursor.
	'follow-cursor' - Tooltip appears at the bottom right position of the cursor and moves along the cursor. 
	'auto' - Tooltip is shown to the bottom of the target, if space not available, it is shown in the direction with required space.
	 @allowedValues {('auto'|'top'|'bottom'|'right'|'left'|'top-left'|'top-right'|'bottom-left'|'bottom-right'|'at-cursor'|'follow-cursor')}
	 		@example <caption> Property when applied in element level </caption>
	<button title='Align Left' data-title-position='right'><i class='icon alignleft'></i></button>
	$(selector).ztooltip();	
			*/	
			position:'at-cursor',	//No I18N
			/** @member {Object} 
			        tooltip.margin - If the value is specified the tooltip will be relocated from the calculated position based on the given value and the direction of the top. This is not an element level configuration.
	@example <caption> Example: </caption>  
	$(selector).ztooltip({
		margin: {top: 2, left: 10}
	}) */	
			/** @member {Number}
					[tooltip.margin.top=0] - If the tooltip is displayed above the target, its top position is reduced by this value and vice-versa. 
					*/			
			/** @member {Number}
					[tooltip.margin.left=0] - If the tooltip is displayed left to the target, its left position is reduced by this value and vice-versa. 	*/
			margin: {left: 0, top: 0},
			/** @member {Number} [tooltip.showDelay = 500]
				Time interval between mousepointer entering the target and the display of the tooltip.When applied in element level, it is prefixed with data-title-*.	
	        */
	        showDelay: 500,
			/** @member {Number} [tooltip.hideDelay = 300] 
				Time, the tooltip takes to hide when the cursor leaves the target. If set to 0, the tooltip is hidden only for maxDisplayTime.When applied in element level, it is prefixed with data-title-*.	
			*/
			hideDelay: 300,
			/** @member {Number} [tooltip.maxDisplayTime = 5000]
				 Specifies the maximum duration, the tooltip can be shown on the page. Assigning it to 0 disables this feature.When applied in element level, it is prefixed with data-title-*.	
			*/
			maxDisplayTime: 5000,
	        /** @member {Number} [tooltip.reshowDelay = 100]
	    		Specifies the duration before which tooltip is shown when the cursor moves from one target to another within the duration mentioned in hideDelay.By default it is calculated as showDelay/5. If showDelay = 500, reshowDelay = 500/5 = 100ms.When applied in element level, it is prefixed with data-title-*.	
			*/
	        reshowDelay: undefined,
	        /** @member {Number} [tooltip.showShortcutKey = true]
	        	Specifies whether shortcut key registered for the target has to be shown.
			*/
	        showShortcutKey: true,
	        /** @member {Object} tooltip.animation Sets animation for showing and hiding the tooltip. If false, disables this feature.
	        */
	        /** @member {Object} tooltip.animation.show Animation to show the tooltip.
	        */
	        /** @member {String} [tooltip.animation.show.name = 'fadeIn']  Animation effect for showing the tooltip.
	        	@example <caption> To use 'slideDown' effect for tooltip show </caption>
	$(selector).ztooltip({
		animation: {
			show: {
				name: 'slideDown'
			}
		}
	}); */
	        /** @member {String} tooltip.animation.show.className  Animation class for showing the tooltip.
	        	@example <caption> To use animation class for tooltip show</caption>
	$(selector).ztooltip({
		animation:{
			show:{
				className:"animated flipInY"
			}	        	
		}
	});
	        	
	        */
	        /** @member {Number} [tooltip.animation.show.duration=200]  Duration for showing the tooltip. This is not used incase className is used.
	        	@example <caption> SlideDown effect with duration </caption>
	$(selector).ztooltip({
		animation: {
			show: {
				name:'slideDown',
				duration: 300
			}
		}
	});
	        */
	        /** @member {Object} tooltip.animation.hide Animation to hide the tooltip.
	        */
	        /** @member {String} [tooltip.animation.hide.name = 'fadeOut']  Animation effect for hiding the tooltip.
	        	@example <caption> To use 'slideUp' effect for tooltip hide </caption>
	$(selector).ztooltip({
		animation: {
			hide: {
				name: 'slideUp'
			}
		}
	}); 
	*/
	        /** @member {String} tooltip.animation.hide.className  Animation class for hiding the tooltip.
	        	@example <caption> To use animation class for tooltip hide</caption>
	$(selector).ztooltip({
		animation:{
			hide:{
				className:"animated flipDownY"
			}	        	
		}
	});
	        	
	        */
	        /** @member {Number} [tooltip.animation.hide.duration=200]  Duration for hiding the tooltip.
	        	@example <caption>  Example:  </caption>
	$(selector).ztooltip({
		animation: {
			hide: {
				name: 'slideUp',
				duration: 500
			}
		}
	});
	        */
			animation:{
				show:{
					name:'fadeIn' //No I18N
				},
				hide:{
					name:'fadeOut' //No I18N
				}
			},
			/** @member {String} tooltip.className
				Custom class that can be added to the tooltip to give additional styling to the tooltip.When applied in element level, it is prefixed with data-title-*.	
		 		@example <caption> Property applied in element level </caption>
	<button title='Align Left' data-title-class-name='customClass'><i class='icon alignleft'></i></button>
	$(selector).ztooltip();
			*/
			className: undefined,
			/** @member {String} tooltip.innerHTML
				If HTML content is needed in the custom tooltip, HTML String is given here. Every ##title## in the string is replaced with the title content of the element.When applied in element level, it is prefixed with data-title-*.	
				@example <caption> Tooltip with custom HTML -container level configuration</caption>
	$(selector).ztooltip({
		innerHTML: "<span class='customClass'>##title##</span>"
	});
			*/
			innerHTML:undefined,
			/** @member {String} tooltip.pauseOnHover
				This option is currently deprecated. Use 'pauseOnMouseOver' instead.
				@deprecated
			*/
			/** @member {String} tooltip.pauseOnMouseOver
				If the tooltip should say still on hover, or if it has to be made clickable, set pauseOnMouseOver = true. This works only when the user leaves the target and hovers the tooltip before the hideDelay time and the tooltip will be removed only when the mousepointer leaves the tooltip.When applied in element level, it is prefixed with data-title-*.	
			*/
			pauseOnMouseOver:false
		},
		_EVENTS:{
			/** Triggered after the tooltip is shown.
				@event tooltip.show
				@param {Object} event eventObject*/
			ONSHOW:'show',	//No I18N
			/** Triggered after the tooltip is hidden.
				@event tooltip.hide
				@param {Object} event eventObject*/
			ONHIDE:'hide'	//No I18N
		},	
		_ANIMATION: {
			slide:{
				show:'slideDown',	//No I18N
				hide:'slideUp'	//No I18N
			},
			fade:{
				show:'fadeIn',	//No I18N
				hide:'fadeOut'	//No I18N
			},
			end: 'webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend'	//No I18N
		},
		_create: function(){
	    	this._namespace = '.'+this.componentName;	//No I18N
	    	this._tempNmspace = this._namespace+'-temp'; //No I18N
			this._bindEvents();
	    },
	    /* Binding events */
	    _bindEvents: function(){
	    	var base = this;
	    	if( !ZC.ztooltipBinded ){
		    	ZC.ztooltipBinded = true;
	    		ZC.$document.off('keydown'+this._namespace).on('keydown'+this._namespace, function(event){	//No I18N
					if( ZC.ztooltipLastInstance ){
						ZC.ztooltipLastInstance.hide();
		    		}
		    	}).off('mousedown'+this._namespace).on('mousedown'+this._namespace, function( event ){ //No I18N
		    		if( ZC.ztooltipLastInstance && ZC.ztooltipLastInstance.target && !$(event.target).closest( ZC.ztooltipLastInstance.target ).length && !$(event.target).closest( ZC.ztooltipLastInstance.tooltip ).length ){
			    		ZC.ztooltipLastInstance.hide();
		    		}
		    	});	
		    	window.addEventListener('scroll', function(){ //No I18N
		    		if( ZC.ztooltipLastInstance ){
		    			ZC.ztooltipLastInstance.hide();
		    			ZC.ztooltipLastInstance = undefined;
		    		}
		    	}, true);
	    	}
	    	/* Handler to observe enable and disable of buttons */
	    	var stateChangeHandler = function(event, ui){ 
	    		var element = ui.button,
					title = element.attr("title");	// No I18N
				if(element.is("input")){
					element = element.next("label"); // No I18N
				}
				if(ui.disabled){
					element.removeAttr("title");	// No I18N
					element.data("ztooltip", title);	// No I18N
				}else{
					element.attr("title",element.data("ztooltip"));	// No I18N
				}
	    	}
	    	this.element.on('mouseenter'+this._namespace+' mousemove'+this._namespace, function(event){ //No I18N
	    		base._targetEnter.call(base, event);
	    	}).on("zbuttonstatechange", stateChangeHandler).on("zmenubuttonstatechange", stateChangeHandler).on("zsplitbuttonstatechange", stateChangeHandler).on("zuploadbuttonstatechange", stateChangeHandler); // No I18N
	    },	
	    /* Displaying the tooltip when the element is hovered */
	    _targetEnter: function( event , customTarget, opt ){
	    	/* Used to track current mouse pointer position ---> used for tracking tooltip */
	    	var target, custom = false;
	    	var disabledSelector = 'button[disabled], input[disabled], textarea[disabled], fieldset[disabled], select[disabled], keygen[disabled] , output[disabled], option[disabled]';
	    	if( this.target && (this.target.is(':hidden') || this.target.is( disabledSelector ) )){
				this.hide();
			}
	    	if( event ){
	    		target = $(event.target).closest('[title]');
	    		if( !event.originalEvent ){
	    			custom = true;
	    		}else{
		    		this.pageX = event.pageX;
		    		this.pageY = event.pageY;
	    		}
	    	}else{
	    		custom = true;
	    		target = customTarget.closest('[title]');
	    	}
	 		//To check whether the target is eligible for tooltip
	 		//Disabled form elements are omitted becos some browsers doesnt trigger events for disabled elements.
			if( !target.length || !target.attr('title') || ( target.is( disabledSelector ) ) ){
				if( target.is( this.target ) && this.tooltip && !this.tooltip.hasClass('hiding') ){
					this.hide();
				}
				return;
			}
			this.custom = custom;
			var showTip = true;
			if( ( this.options.selector && !target.is( $(this.options.selector, this.element) ) ) ){
				showTip = false;
			}
			//clear previous title.
			this.target && this._clearTitle( true, this.target );
			var titleOptions, base = this;
			this.target = target;
			titleOptions = this._getOptions( this.target );
			//Saving the options of the target so retrival will be easy
			this.targetOpt = $.extend(true, {}, this.options, typeof titleOptions === 'object' && titleOptions, opt);	//No I18N
			//@decprecated
			this.targetOpt.pauseOnMouseOver = this.targetOpt.pauseOnMouseOver || this.targetOpt.pauseOnHover;
	    	!custom && this._clearTitle();
	    	this.target.off( this._tempNmspace )
					   .on( 'mouseleave'+ this._tempNmspace, function(){
					   		base._targetLeave.call(base);
					   });
			if( showTip ){
		    	this._clearHideTimer();
				this._show( event );
			}
	    },	
	    /* To hide the tooltip when the target is left*/
	    _targetLeave: function( immediate ){
	    	this._clearTitle( true );
			this._clearShowTimer();
			if( this.tooltip ){
				this._completeAnimation( this.tooltip );
				this._removeTip( immediate );	
			}
		},
		/* To show the tooltip */
	    _show: function(eventObj){
			/*calculating delay*/
			var targetOpt = this.targetOpt;
			if( targetOpt.showDelay && targetOpt.reshowDelay === undefined){
				this.targetOpt.reshowDelay = targetOpt.showDelay / 5 ;
			}
			//Binding the elements
			var base = this;
			this.target.on('mousedown'+ this._tempNmspace, function(){	
				base._clearShowTimer();
		   		base._removeTip( true ,base.target);
		   	});
	    	this._showTooltip(eventObj);	
	    },
	    _completeAnimation: function( element ){
			element && ( element.finish ? element.finish() : element.stop( true, true ) );
	    	return element;
		},
	    /* Function to stop the show timer */
	    _clearShowTimer: function(){
	    	this._completeAnimation( this.tooltip );
			clearTimeout(this.showTimer)
	    },
	    /* Function to stop the hide timer */
	    _clearHideTimer: function(){
	    	this._completeAnimation( this.tooltip );
	    	clearTimeout(this.hideTimer);
	    },
	    _setTimer: function(target, opt){
	    	if(opt.maxDisplayTime){
	    		var base = this;
				this._clearHideTimer();
				this.hideTimer = setTimeout(function(){
					base._removeTip( true );
				}, opt.maxDisplayTime);
			}	
	    },
		/* Function to show the tooltip*/
	    _showTooltip: function(eventObj){
	    	var base = this,
				opt = this.targetOpt;
			var showFunc = function() {	
				/****CLEAR TITLE FOR LAST TOOLTIP*/
				/*If the current target is not equal to last target, the tite attribute of last target is reset*/
				var animation = opt.animation;
				if( base._lastTip ){
					if( base.tooltip && base.tooltip.is(':visible') ){
						// animation = false;     discuss - whether animation is necessary when the previous tooltip is not yet destroyed.( Becos of quick movement )
						if( base.target.is(base._lastTip) ){
							opt.reshowDelay = undefined;
							animation = false;
						}else{
						    base._trigger( "ONHIDE" , undefined, base.target); //No I18N
						}
					}else{
					    base._trigger( "ONHIDE", undefined, base.target ); //No I18N
					}
				}
				/*When the tooltip is about to be displayed, current target is set as last target*/
				base._lastTip = base.target;
		    	// base._clearHideTimer();
		    	/* validating the position*/
				if( !/^(auto|top|bottom|right|left|top-left|top-right|bottom-left|bottom-right|at-cursor|follow-cursor)$/.test(opt.position)){
					opt.position = $.ztooltip.prototype.options.position;
				}
		    	/*if( opt.pointer === undefined ){
		    		if( !/^(at-cursor|follow-cursor)$/.test(opt.position) || opt.isTargetCustom ){
		    			opt.pointer = true;
		    		}
		    	}*/
		    	/*set displayType = 'box' for at-cursor/follow-cursor*/
		    	if( /^(at-cursor|follow-cursor)$/.test(opt.position) ){
		    		if( !base.custom ){
			    		opt.displayType = 'box'; //No I18N
		    		}else{
		    			opt.position = 'top'; //No I18N
		    		}
		    	}
		        base._createTooltip();
				if(base.tooltip && base.tooltip.length &&(base.target.is(':visible') || base.target.is('option'))){//Chrome & IE says option element is always hidden. So we have to skip option element.//No I18N 
					base._includeContent(opt);
					base._setTimer(base.target, opt);
					if(  opt.position === 'follow-cursor' ){	//No I18N
						var mousemoveDelegate = function(){
							base._positionElement.apply(base, arguments);
							/*When the tooltip is on move, its hidetimer is reset*/
							base._setTimer(base.target, opt);
						}
						base.target.off('mousemove'+base._tempNmspace)	//No I18N
								   .on('mousemove'+base._tempNmspace, mousemoveDelegate); //No I18N
					}
					base._positionElement(eventObj);
					ZC.ztooltipLastInstance = base;
				}
				base._completeAnimation( base.tooltip.hide() );
				base._animate( base.tooltip, animation, 'show','show', function(){	//No I18N
					base._trigger( "ONSHOW", eventObj); //No I18N
				});
			}
			if( this.custom || opt.reshowDelay || opt.showDelay !== undefined ){
				// this.tooltip && this.tooltip.hide(); //removed as it prevents reshowDelay behavior
				this._clearShowTimer();
				this.showTimer = setTimeout(function(){
					showFunc();
				}, ( !this.custom && this.tooltip ) ? opt.reshowDelay : opt.showDelay);
			}else{
				showFunc();
			}
	    },
	    _animate: function( element, animation ,action , alterFn, callback ){
	    	if( animation && typeof animation === 'object'){ //No I18N
	    		var effect, duration, className;
				if( typeof animation === 'string' ){ //No I18N
					effect = this._ANIMATION[ animation ][action];
				}else if( animation.name ){
					if( this._ANIMATION[ animation.name ] ){
						effect = this._ANIMATION[ animation.name ][action];
					}
				}else if( animation[action] ){
					if( typeof animation[action] === 'string'){ //No I18N
						effect = animation[action];
					}else if( animation[action].name ){
						effect = animation[action].name;
					}else if( animation[action].className ){
						className = animation[action].className;
					}
				}
				duration = animation.duration? animation.duration :(animation[action] && animation[action].duration? animation[action].duration : undefined);		
				if(typeof effect === 'object'){	//No I18N
					element.animate( effect, duration ,callback );
				}else if(/^(fadeIn|fadeOut|slideDown|slideUp)$/.test(effect) ){
					element[effect](duration, callback);
				}else if(typeof className === 'string'){ //No I18N
					if(alterFn === 'show'){	//No I18N
						element.show();
					}
				    element.removeClass( element.data('css-effect') ).addClass( className ).data('css-effect', className)	//No I18N
							.off( this._ANIMATION.end )
							.on( this._ANIMATION.end, function(){
									if(alterFn === 'hide'){	//No I18N
										element.hide();
									}
									element.removeClass( className ).removeData('css-effect');	//No I18N
									callback();
							});	
				}else{
					element[ alterFn ]() && callback();	
				}
			}else{
				element[ alterFn ]() && callback();	
			}
	    },
	    _includeContent: function(opt){
	    	var o = this.targetOpt;
			if(typeof o === 'object' && this.tooltipContent && this.tooltipContent.length){	//No I18N
				var content = o.title = o.title || o.content || ''; //No I18N
				if( /*/^(\.|\#)/.test(content) || */content.nodeType || content.jquery ){
					content = $(content).clone().show()[0].outerHTML;
					o.isMessageHTMLEncoded = true;	
				}else if( typeof o.innerHTML === 'string' ){ //No I18N
					content = this._populateTemplate( o.innerHTML, o );
					o.isMessageHTMLEncoded = true;	
				}
				if( o.showShortcutKey && window.ZShortCut ){
					var target = this.target;
					target = target.is('label.zbutton')? target.prev('input') : target;	//No I18N
				    var data = ZC.getOptions(target);
				    var key = ZShortCut.getInfo(data.actionName, target , false);
				    if( key ){
				    	if( o.shortcutKeyCustomHTML ){
				    		key = o.shortcutKeyCustomHTML.replace(/(#key#)/gi, key );
				    	}else{
				    		key = " ("+key+")"; //No I18N
				    	}
				    	content = content + key;
				    }
				}
				this.setContent(o, content );
			}
	    },
	    setContent: function(options, content ){
			if( options.isMessageHTMLEncoded ){
				this.tooltipContent.html(content);
			}else{
				this.tooltipContent.html($('<div>').addClass('ztooltip__text').text(content)); //No I18N
			}
		},
	    _populateTemplate: function( template, data ){
			if( typeof template === 'string' ){ //No I18N
				var props = template.match(/#?#[^# ]+#?#/g);
				if( props ){
					props.forEach( function( prop ){
						template = template.replace( new RegExp( prop, 'g' ) ,function( match ){ //No I18N
							return data[match.replace(/#/g,'')]; //No I18N
						});
					});
				}
				return template;
			}
		},
	    //This function is to position the tooltip and the respective arrow.
		_positionElement: function(eventObj){
			if(this.tooltip){	
				var positionObject = this._getPosition(eventObj);
  				var direction = positionObject.direction.split('-')[0]; //No I18N
				var elementPosition = positionObject.elementPosition;
				this.tooltip.removeClass( 'class' ).addClass('ztooltip ztooltip--'+ direction); //No I18N
	  			if( typeof this.targetOpt.margin === 'object' ){ //No I18N
	  				var margin = {};
	  				margin.top = parseInt( this.targetOpt.margin.top );
	  				margin.left = parseInt( this.targetOpt.margin.left );
	  				if( margin.top === 'NaN' ){ //No I18N
	  					margin.top = 0;
	  				}
	  				if( margin.left === 'NaN' ){ //No I18N
	  					margin.left = 0;
	  				}
	  				if( direction === 'top' ){ //No I18N
						elementPosition.top -= margin.top;	
	  				}else if( direction === 'bottom' ){ //No I18N
						elementPosition.top += margin.top;	
	  				}else if( direction === 'right' ){ //No I18N
						elementPosition.left += margin.left;
	  				}else if( direction === 'left' ){ //No I18N
						elementPosition.left -= margin.left;
	  				}
					// elementPosition.left += parseInt(this.targetOpt.margin.left);
					// elementPosition.top += parseInt(this.targetOpt.margin.top);	
					if(/^(at-cursor|follow-cursor)$/.test( this.targetOpt.position )){
						var maxCorrection = positionObject.availableHeight - this.tooltip.outerHeight();
						elementPosition.left += parseInt(this.targetOpt.margin.left);
						//@constant 18
						elementPosition.top += (parseInt(this.targetOpt.margin.top) + Math.min( 18, maxCorrection)/*There must be some distance between cursor and the tooltip by default*/ );
					}
	  			}
				this.tooltip.css(elementPosition);
				if(this.tooltipCallout){
					this.tooltipCallout.css(positionObject.arrowPosition);
				}	
			}
		},
		//This function is to get the position for the tooltip
		//EventObject can be passed, only when the position is based on the mouse pointer position
		_getPosition: function(eventObj){
			var opt = this.targetOpt;
			var eventTarget, position = this.targetOpt.position, positionAlterable =false;
			var arrowCallback = function(element, direction){
				$(element).removeAttr('class').addClass('ztooltip__pointer h-' + direction); //No I18N
			}
			if(ZPosition && this.tooltip){
				this.tooltip.css({
					left:'auto', //No I18N
					right:'auto'//No I18N
				});
				/* Get the current position of mouse pointer */
				if( /^(at-cursor|follow-cursor)$/.test( opt.position ) || opt.offset ){
					if( opt.offset ){
						this.pageX = parseInt( opt.offset.left );
						this.pageY = parseInt( opt.offset.top );
					}else{
						position = 'bottom-left';	//No I18N
					}
					eventTarget = eventObj;
					eventTarget.pageX = this.pageX;
					eventTarget.pageY = this.pageY;
					positionAlterable = 'flipallfit'; //No I18N
				}else{
					if( opt.position === 'auto' ){ //No I18N
						position = 'bottom'; //No I18N
						positionAlterable = 'flipallfit'; //No I18N
					}
				}
				var tipPosition = ZPosition.get((eventTarget || this.target), this.tooltip,
										{
											direction : position,
											eventObj : eventObj,
											bufferSpace : this.tooltipCallout? 10:0,
											positionAlterable :positionAlterable,
											arrowCallback: arrowCallback,
											arrow: opt.displayType === 'callout'? { //No I18N
														element : this.tooltipCallout,
														direction:true,
														minSpace:8
													} : undefined
										});
			}
			if( !tipPosition ){
				tipPosition = {
					elementPosition: {left:0, top:0},
					arrowPosition: {left:0, top:0}
				}
			}
			return tipPosition;
		},
		/* Function to remove the tip*/
	    _removeTip: function(immediate, target, opt){
	    	opt = opt || this.targetOpt;
	    	if(this.tooltip){
	    		target = target || this.target;
				var base = this;
				var remove = function(){
					base._animate( base._completeAnimation( base.tooltip ), opt.animation, 'hide','hide', function(){	//No I18N
						if( base.tooltip ){
							base.tooltip.remove();
						    base.tooltip = base.tooltipCallout = base.tooltipTitle = base.tooltipContent = null;
						    if( ZC.ztooltipLastInstance && ZC.ztooltipLastInstance.element.is( base.element ) ){
						    	ZC.ztooltipLastInstance = undefined;
						    }
						    base._trigger( "ONHIDE" ); //No I18N
						}	
					});	
				}
				var removeFunc = function(){
					if(base.tooltip){
						base.tooltip.off('mouseleave'); //No I18N
						/* To empty lasttip if the current target is different from lasttip*/
						if(!base.target.is(base._lastTip)){
							base._lastTip = undefined;
						}else if( !immediate && base.targetOpt.pauseOnMouseOver && base.tooltip.hasClass('on-hover') ){  //No I18N
							base.tooltip.on('mouseleave', remove); //No I18N
							return;
						}
						remove();
					}
				}
				/*This has been incuded becos, for all tip targets, its title is reset on targetleave but incase of tip shown using show method, there wont be any targetleave method to clear its title. 
				  This cannot be done for all case becos removeing tip on target click, its title should not be cleared.
				*/
				this.isTargetCustom && this._clearTitle( true, this.target );
				if(immediate || !this.targetOpt.hideDelay){
					removeFunc();
				}else{
					this._clearHideTimer();
					this.hideTimer = setTimeout(function(){
						removeFunc();
					}, opt.hideDelay);
				}	
	    	}
		},

		//To get data by parsing JSON of title attribute
		_getOptions: function(target){
			var titledata = target.attr('title');	//No I18N
			var tooltipdata = {};
			$.each(target[0].attributes,function( ){
				var name = this.nodeName;
				var value = this.value;
				if( name.match(/^data-title-/) ){
					name = name.replace(/^data-title-/,''); 
					name = name.replace(/-([a-z])/g, function(g){
								return g[1].toUpperCase(); // replacing the hyphenated names to camelCase
							});
					name = name.replace(/(html)/gi, "HTML"); //No I18N
					tooltipdata[name] = (value === "true" || value === "false") ? (value === "true") : ((value.indexOf("{") !== -1) ? ZC._getObject(value) : value); // No I18N
				}
			});
			var titleObject;
			if( !isNaN( titledata ) ){
				titleObject = {"content":  titledata };	//No I18N
			}else{
				try{
					titleObject = JSON.parse( titledata );
				}catch(e){
					if( titledata ){
						titleObject = {"content":  titledata };	//No I18N
					}
				}	
			}
			tooltipdata.targetElement = target;
			$.extend(titleObject, tooltipdata);
			return titleObject;
		},
	    //Creating the tooltip
	    _createTooltip: function(){
				if(!this.tooltip){
					this.tooltip = $("<div>").appendTo('body')
											 .hover(function(){
											 	$(this).addClass('on-hover');
											 }, function(){
											 	$(this).removeClass('on-hover');
											 });
					this.tooltipContent = $("<div>").addClass("ztooltip__content").appendTo(this.tooltip);	//No I18N
				}
				this.tooltip.css('pointer-events', this.targetOpt.pauseOnMouseOver? 'auto':'none'); //No I18N
				this.tooltip.attr('class', 'ztooltip '+ (this.targetOpt.className ||''));	//No I18N
				/*	if(this.targetOpt.title){
					if( !this.tooltipTitle ){
						this.tooltipTitle = $("<div>").addClass("ztooltip__title").append($("<span>").addClass("ztooltip__text")).insertBefore( this.tooltipContent ); //No I18N
					}
					this.tooltipTitle.find('.ztooltip__text').html( this.options.isMessageHTMLEncoded ? this.targetOpt.title : ZC.encodeHTML( this.targetOpt.title )); //No I18N
				}else if( this.tooltipTitle ){
					this.tooltipTitle.remove();
					this.tooltipTitle = undefined;
				}*/
				if( this.targetOpt.displayType === 'callout' ){ //No I18N
					if(!this.tooltipCallout){
						this.tooltipCallout = $("<div>").addClass('ztooltip__pointer').appendTo(this.tooltip);	//No I18N
					}
				}else if( this.tooltipCallout ){
					this.tooltipCallout.remove();
					this.tooltipCallout = undefined;	
				}
				return this.tooltip;
	    },
	    //Clearing the title of the target
	    _clearTitle: function(reset, target){
			target = target || this.target;
			if( target && target.length ){
		    	if( reset ){
		    		target.off( this._tempNmspace );
		    		var parents = $( target.data('zparents') );	//No I18N
	    			parents.add( target ).each( function( i, tar ){
	    				tar = $(tar);
	    				var titleData = tar.attr('title') !== undefined ? tar.attr('title') : tar.data('ztitle');	//No I18N
	    				if( titleData !== undefined ){
	    					tar.attr('title', titleData).removeData('ztitle');	//No I18N
		    				tar.removeAttr('aria-label');	//No I18N
	    				}
	    			});
		    		target.removeData('zparents');	//No I18N
		    	}else if( !reset ){
		    		/*Fix to remove title attributes of parent elements having title attribute*/
		    		var parents = target.parents('[title]'); //No I18N
	    			parents.add( target ).each(function( i, tar ){
		    			tar = $(tar);
			    		tar.data('ztitle', tar.attr('title')).removeAttr('title');	//No I18N
		    		});
		    		target.data('zparents', parents); //No I18N	
		    	}
			}
	    },
		/* Function to know the visiblity of the element*/
		_isVisible: function(element){
			if(element && element.length){
				return element.is(':visible') ? true:false;	//No I18N
			}
		},
		_trigger: function( customEvent , origEvent, target){
			var data = {
				target : target || this.target,
				tooltip : customEvent === "ONSHOW"? this.tooltip : undefined  //No I18N
			}
	       return this._dispatchEvent( customEvent, origEvent, data );
		},
		/** Shows the Tooltip for the given target.
			@function tooltip.show
			@param {Object} target Element for which tooltip has to be shown. This 
			element should have title attribute.
			@returns {Object} Returns the element.
		*/
		show: function(target, opt){
			/*To clear previous target*/
			target = $(target);
			if( target.length && target[0].nodeType && target.is(':visible') ){
				this._clearTitle( true , target);
				opt = typeof opt === 'object'? opt : {}; //No I18N
				opt.showDelay = 0;
				this._targetEnter( undefined, target, opt );
			}
			return target;
		},
		/** Hides the current tooltip.
			@function tooltip.hide
			@param {Boolean} immediate Whether tooltip has to be hidden immediately without animation.
		*/
		hide: function( immediate ){
			this._targetLeave( immediate );
		},
		/** Destroys the Tooltip instance.
			@function tooltip.destroy
		*/
		/* Function to destroy the tooltip */
		_destroy: function(){
			if( ZC.ztooltipLastInstance && ZC.ztooltipLastInstance.element.is( this.element ) ){
				ZC.ztooltipLastInstance.target && ZC.ztooltipLastInstance.target.off( this._tempNmspace );
				ZC.ztooltipLastInstance = undefined;
			}
			this.element.off( this._namespace );
			this.hide( true );
			$(this.tooltip).remove();
		},
		option: function(optionName, value){
			var base =  this;
			if(typeof optionName === 'object' && !$.isEmptyObject(optionName)){	// No I18N
				$.each(optionName, function(o,v){
					base.option(o,v);
				});
			}else if(optionName !== undefined && value === undefined){
				return this.options[ value ];
			}else if( optionName !== undefined && value !== undefined ){
				if( typeof value === 'object' ){ //No I18N
					$.extend( this.options[optionName], value );
				}else{
					this.options[optionName] = value;
				}
			}else{
				return Object.create( this.options );
			}
		}	
	}
   ZC.registerComponent("ztooltip", tooltipPrototype); //No I18N
})(ZComponents);
/** Method to set property for tooltip instance property.
	@function tooltip.setAttribute
	@param {String} property Property name to set.
	@param {String} value New value for the property.
	@example <caption> Example: </caption>
$(selector).ztooltip('setAttribute','position','top');
*/	
/** This method sets one or more tooltip instance property.
	@function tooltip.setAttributes
	@param {Object} properties Object contains property-value pairs to be set.
	@example <caption> Example: </caption>
$(selector).ztooltip('setAttributes',{
	position:'top',
	showDelay:100,
	reshowDelay:30
});
*/
/** This method returns the value of the given property.
	@function tooltip.getAttribute
	@param {String} property Name of the property whose value is to be returned.
	@returns {Object|String} Value of the given property.
	@example <caption> Example: </caption>
$(selector).ztooltip('getAttribute','position')
*/
/** This method returns current tooltip instance properties.
	@function tooltip.getAttributes
	@returns {Object} Tooltip instance properties.
	@example <caption> Example: </caption>
$(selector).ztooltip('getAttributes');
*/
