//$Id$
var ZDBChartSort = new function()
{
    // display menu from sort option in tool bar or sort icons in edit mode
    this.displaySortMenu = function(srcEl, event, place, isLegendSort)
    {
        var sortFiles = "zdbjs/sort/ZAChartSortMenu.js"; //No I18N
        if(!ZDBLazyLoadUtil.loadJsAlone(sortFiles, "APPENDTODOC", ZDBChartSort.displaySortMenu,
                                                [srcEl, event, place, isLegendSort])) {
            return;
        }
        ZAChartSortMenu.showSortMenu(srcEl, event, place, isLegendSort);
    }

    // on clicking from drop down menu, update axis sort details and refresh chart
    this.updateSortSetting = function(sortType, sortBy)
    {
        var view = settings.currentViewObj;
        view.customSort = false;
        view.sortVal = sortType;
        view.sortAxis = ZDBChartSort.getSortAxis(view);
        if(sortBy===1) {
            view.sortAxis = ZDBChartSort.getRevAxis(view.sortAxis);
        }
        ZDBChartSort.setAxisSortInList(view);
        view.changeViewRConfMod(true);
    }

    // update current list with chosen sort and remove other sort information
    this.setAxisSortInList = function(view)
    {
        var list = view.currentList;
        for(var i=0; i < list.length; i++)
        {
            var viewType = list[i].getAttribute("viewType");
            if(viewType !== "COLORAXIS")
            {
                list[i].removeAttribute("CUSTOMBASEID");
                list[i].removeAttribute("SORTTYPE");
                if(viewType === view.sortAxis)
                {
                    list[i].setAttribute("SORTTYPE", view.sortVal);
                }
            }
        }
    }

    // on clicking from drop down menu, update legend sort details and refresh chart
    this.updateLegendSort = function(sortType)
    {
        var view = settings.currentViewObj;
        view.legendSortVal = sortType;
        var colorList = ZDBUtil.getElementsWithAttribute(view.currentList, "viewType", "COLORAXIS"); //No I18N
        if (colorList.length > 0)
        {
            colorList[0].setAttribute("SORTTYPE", sortType);
            colorList[0].removeAttribute("CUSTOMBASEID");
        }
        view.legendCustomSort = false;
        view.changeViewRConfMod(true);
    }

    this.getSortAxis = function(view)
    {
        var axisDetails = view.axisDetails;
        if(axisDetails===null || axisDetails === undefined) {return "NONE";} //NO I18N

        var sortAxis = "NONE"; //No I18N
        for(var i=0; i < axisDetails.length; i++)
        {
            if(axisDetails[i].SORTALLOWED)
            {
                sortAxis = axisDetails[i].VIEWTYPE;
                break;
            }
        }
        return sortAxis;
    }

    this.getRevAxis = function(axis)
    {
        if(axis === "XAXIS") {
            return "YAXIS"; //No I18N
        }
        else if(axis === "YAXIS") {
            return "XAXIS"; //No I18N
        }
        return axis;
    }

    this.setSortMenuProp = function(view)
    {
        if(ZDBGlobal.IS_MOBILE)
        {
            return;
        }
        var axisDetails = view.axisDetails;
        var sortMenu = document.getElementById("ZASort");
        var className = "alpha"; //No I18N
        if(!axisDetails)
        {
            ZDBMenu.editCSSClass(sortMenu, className, false);
            return;
        }
        view.isCustSortAllowed = false;
        view.sortAllowedAxis = "", view.sortAxisColId = "", view.clrAxisColId = "";
        var yCount = 0;
        var legSrtAllowed = false;
        for(var i=0; i < axisDetails.length; i++)
        {
            var viewType = axisDetails[i].VIEWTYPE;
            if(viewType==="YAXIS") {
                yCount++;
            }
            else if (viewType === "COLORAXIS") {
                view.clrAxisColId = axisDetails[i].COLUMN_ID;
                view.legAxisOrder = axisDetails[i].ORDER;
                if(axisDetails[i].LEGSORTALLOWED && view.legendPos !== "NONE")
                {
                    legSrtAllowed = true;
                }
            }

            if(axisDetails[i].SORTALLOWED)
            {
                view.sortAllowedAxis = axisDetails[i].VIEWTYPE;
                view.sortAxisColId = axisDetails[i].COLUMN_ID;
                view.isCustSortAllowed = axisDetails[i].CUSTOMSORTALLOWED;
                view.sortAxisOrder = axisDetails[i].ORDER;
            }
        }

        view.chartSelectedEl.isMultiY = view.chartSelectedEl.isMultiY || (yCount > 1);
        view.sortVal = 0;
        var axisCnt = 0;
        var list = view.currentList;
        for(i=0; i < list.length; i++)
        {
            viewType = list[i].getAttribute("viewtype");
            if(viewType==="XAXIS" || viewType==="YAXIS")
            {
                axisCnt++;
                var sortType = list[i].getAttribute("sorttype");
                if(sortType !== null && sortType !== "") {
                    view.sortVal = sortType;
                    view.sortAxis = viewType;
                }
            }
        }

        view.singleAxis = (axisCnt===1);
        ZDBChartSort.setToolTip(view, sortMenu);
        if((view.sortAllowedAxis==="" && !legSrtAllowed) || view.graphSelected.indexOf("MAP") === 0 || view.graphSelected==="TABLE_CHART") //No I18N
        {
            ZDBMenu.editCSSClass(sortMenu, className, false);
        }
        else
        {
            ZDBMenu.editCSSClass(sortMenu, className, true);
        }
    }

    this.setSortElAttr = function(sortMenu, baseClass, status)
    {
        sortMenu.setAttribute("enabled", status);
        sortMenu.className = "iconitem " + baseClass;
        if(!status) {
            sortMenu.className = "iconitem " + baseClass + " alpha";
        }
    }

    this.setToolTip = function(view, sortEls)
    {
        return;
    }

    // add custom sort info and remove chart sort info from list
    this.addCustomSortInfo = function(view)
    {
        if(view.sortAxis == "NONE")
        {
            view.sortAxis = ZDBChartSort.getSortAxis(view);
        }
        var list = view.currentList;
        for(var i=0; i < list.length; i++)
        {
            if(list[i].getAttribute("viewtype")==view.sortAxis)
            {
                list[i].setAttribute("CUSTOMBASEID",view.customBaseId);
            }
            if(list[i].getAttribute("viewtype") !== "COLORAXIS")
            {
                list[i].removeAttribute("SORTTYPE");
            }
        }
    }

    // add chart sort info and remove custom sort info from list
    this.addChartSortInfo = function(view)
    {
        var list = view.currentList;
        for(var i=0; i < list.length; i++)
        {
            if(list[i].getAttribute("viewtype")==view.sortAxis)
            {
                list[i].setAttribute("SORTTYPE",view.sortVal);
            }
            else
            {
                if(list[i].getAttribute("viewtype")!=="COLORAXIS")
                {
                    list[i].removeAttribute("SORTTYPE");
                }
            }
            list[i].removeAttribute("CUSTOMBASEID");
        }
    }

    // set either custom / chart sort info in currentList
    this.setSortInfo = function(view)
    {
        if(view.customSort==true && view.customBaseId!=null)
        {
            var colId = ZDBChartSort.getSortAxisColId(view);
            var csId = view.custSrtColId;
            if(csId!=colId)
            {
                return;
            }
            else
            {
                ZDBChartSort.addCustomSortInfo(view);
            }
        }
        else if(view.sortAxis!=null && view.sortVal!=null)
        {
            ZDBChartSort.addChartSortInfo(view);
        }
    }

    this.getSortAxisColId = function(view)
    {
        var srtAxis = ZDBChartSort.getSortAxis(view);
        if(srtAxis===null) {
            srtAxis = view.sortAxis;
        }
        return ZDBChartSort.getColumnId(view, srtAxis);
    }

    // returns column id of the given axis
    this.getColumnId = function(view, axis)
    {
        var colId = null;
        var list = view.currentList;
        for(var i=0, j=list.length; i<j; i++)
        {
            var viewType = list[i].getAttribute("VIEWTYPE");
            if(viewType === axis)
            {
                colId = list[i].getAttribute("column_id");
                break;
            }
        }
        return colId;
    }

    this.setProperSortAxis = function(view)
    {
        if(view.sortAxis != "NONE")
        {
            var list = view.currentList;
            if(list.length==1)
            {
                var catAxis = list[0].getAttribute("viewType");
                if(view.sortAxis!=catAxis)
                {
                    view.sortAxis=catAxis;
                    view.sortVal=0;
                }
            }
        }
    }

    this.showValueChooser = function(isLegend)
    {
        var view = settings.currentViewObj;
        if(view.viewCtx.isStandAlone() || view.permissions.designModify==0) // shared user
        {
            var list = view.currentList;
            if(isLegend)
            {
                view.legendCustomSort = true;
                for(var i=0; i<list.length; i++)
                {
                    if(list[i].getAttribute("viewtype")==="COLORAXIS")
                    {
                        list[i].removeAttribute("SORTTYPE");
                    }
                }
            }
            else
            {
                view.customSort=true;
                view.sortAxis="NONE"; //NO I18N
                for(i=0; i<list.length; i++)
                {
                    if(list[i].getAttribute("viewtype")!=="COLORAXIS")
                    {
                        list[i].removeAttribute("SORTTYPE");
                    }
                }
            }
            view.changeViewRConfMod(true);
        }
        else // owner
        {
            view.legendCustSort = isLegend;
            ZDBChartSort.showCustomSortDialog(view)
        }
    }

    this.showCustomSortDialog = function(view)
    {
        if(!ZDBLazyLoadUtil.loadJsAlone("ZRCustomSort", "APPENDTODOC", ZDBChartSort.showCustomSortDialog, [view]))
        {
            return;
        }
        ZACustSortEdit.showEditDialog(view);
    }

    // set proper values when refreshing a chart. (set after ZAChartView.ve request)
    this.setSortDetails = function(view)
    {
        view.customSort = false;
        view.legendCustomSort = false;
        view.legendSortAllowed = false; // to show/hide legend sort options in drop down menu
        view.legendCustSortAllowed = false; // to show/hide legend custom sort option in drop down menu
        var axisDetails = view.axisDetails;
        for(var i=0, j=axisDetails.length; i<j; i++)
        {
            var axisDetail = axisDetails[i];
            var custBaseId = axisDetail.CUSTOMBASEID;
            var viewType = axisDetail.VIEWTYPE;
            var colId = axisDetail.COLUMN_ID;
            if(viewType === "COLORAXIS")
            {
                var sortType = axisDetail.SORTTYPE;
                if(axisDetail.LEGSORTALLOWED) {
                    view.legendSortAllowed = true;
                }
                if(axisDetail.LEGCUSTOMSORTALLOWED) {
                    view.legendCustSortAllowed = true;
                }
                var colorList = ZDBUtil.getElementsWithAttribute(view.currentList, "viewType", "COLORAXIS"); //No I18N
                if(sortType)
                {
                    view.legendSortVal = sortType;
                    if (view.legendSortAllowed && colorList.length > 0) {
                        colorList[0]["SORTTYPE"] = sortType;
                    }
                }
                else {
                    view.legendSortVal = null;
                }
                if(custBaseId)
                {
                    view.legendCustomSortId = custBaseId;
                    view.legendCustomSort = true;
                    view.legendCustSrtColId = colId;
                    if (colorList.length > 0) {
                        colorList[0]["CUSTOMBASEID"] = custBaseId;
                    }
                }
            }
            else if(custBaseId) // X or Y-axis
            {
                view.sortAxis = viewType;
                view.custSrtColId = colId;
                view.customBaseId = custBaseId;
                view.customSort = true;
            }
        }
    }
}


// set proper values when opening a saved chart. (set after ZDBAnalysisView.cc request)
ZDBAnalysisView.prototype.setChartSortVal = function(el)
{
    var sortValue = el.getAttribute("SORTTYPE");
    var custBaseId = el.getAttribute("CUSTOMBASEID");
    var viewType = el.getAttribute("VIEWTYPE");
    if(custBaseId)
    {
        var colId = el.getAttribute("COLUMN_ID");
        if(viewType !== "COLORAXIS")
        {
            this.sortAxis = viewType;
            this.customBaseId = custBaseId;
            this.customSort = true;
            this.custSrtColId = colId;
        }
        else
        {
            this.legendCustomSortId = custBaseId;
            this.legendCustomSort = true; // to show tick mark in drop down menu
            this.legendCustSrtColId = colId;
        }
    }
    else if(sortValue!=null)
    {
        this.sortVal = parseInt(sortValue);
        if(viewType !== "COLORAXIS") {
            this.sortAxis = viewType;
        }
        else {
            this.legendSortVal = this.sortVal;
        }
    }
}

ZDBAnalysisView.prototype.resetSortInfo = function()
{
    this.sortVal=0;
    this.legendSortVal=0;
    this.sortAxis="NONE"; //No I18N
}

ZDBAnalysisView.prototype.setSortDetails = function(el)
{
    var elViewType = el.getAttribute("VIEWTYPE");
    var elTypeCurrentList = ZDBUtil.getElementsWithAttribute(this.currentList, "viewtype", elViewType);//No I18N
    var colId = el.getAttribute("vcid");
    for(var i=0; i < elTypeCurrentList.length; i++)
    {
        if(colId == elTypeCurrentList[i].getAttribute("vcid"))
        {
            if(el.getAttribute("sorttype"))
            {
                elTypeCurrentList[i].setAttribute("SORTTYPE", el.getAttribute("sorttype"));
            }
        }
    }
}
