//$Id$
/*
 * propSetting is used for property settings alone
 * propSetForm defines the entire property set
 * propMap is used to map the propValue by propName and PropType
 * propName is the property Name as per DB ----------  need to be defined by the user
 *
 * PropMap are constructed with Three criteria
 * 1. Different property name with same instance eg. chart title, chart desc
 * 2. Different instances with same property name eg. Label
 * 3. Different properties with same property and instance eg. Threshold
 *
 * and also it is extended to handle Custom Properties that can be
 * defined into four categories
 * Integer, String, Color, Font
 *
 * Some js functions are intended only for UI purposes
 * This js is fully concentrated on contructing the PropMap (Property mapping in Client Side)
 */

var PropSettings = new function()
{
    this.STATIC_PROP = "static"; //No I18N
    this.CHART_TITLE =  "chart.title";//No I18N
    this.CHART_DESCRIPTION = "chart.description"; //No I18N
    this.CHART_NULL_VALUE = "chart.nullValue"; //No I18N
    this.CHART_MISSING_VALUE = "chart.missingValue"; //No I18N
    this.PVT_MISSING_VALUE_HIERARCHICAL_MODEL_XAXIS ="chart.hierarchicalModMissValInXAxis"; //No I18N
    this.PVT_MISSING_VALUE_HIERARCHICAL_MODEL_YAXIS ="chart.hierarchicalModMissValInYAxis"; //No I18N
    this.CHART_AXIS_GRIDLINE = "chart.axis.gridLine"; //No I18N
    this.CHART_BACKGROUND_COLOR = "chart.backgroundColor"; //No I18N
    this.CHART_BORDER = "chart.border"; //No I18N
    this.CHART_DEFAULT_CLR="chart.default.clr";//No I18N
    this.CHART_BORDER_OUTLINE = "chart.border.outline";//No I18N
    this.CHART_THRESHOLD_NAME = "chart.threshold"; //No I18N
    this.CHART_THRESHOLD_SCOPE = ".scope"; //No I18N
    this.CHART_THRESHOLD_FUNCTION = ".func"; //No I18N
    this.CHART_THRESHOLD_STYLE = ".style"; //No I18N
    this.CHART_AXIS_LABEL = "chart.axis.label"; //No I18N
    this.CHART_DATALABEL_FONT = "chart.dataLabelFont"; //No I18N
    this.CHART_AXIS_MIN_VALUE = "chart.axis.minValue"; //No I18N
    this.CHART_AXIS_MAX_VALUE = "chart.axis.maxValue"; //No I18N
    this.CHART_AXIS_INTERVAL_VALUE = "chart.axis.intervalValue"; //No I18N
    this.CHART_AXIS_TICK_NUMBERS = "chart.axis.tickNumber"; //No I18N
    this.CHART_X_AXIS_SHOW = "chart.xAxis.show"; //No I18N
    this.CHART_Y_AXIS_SHOW = "chart.yAxis.show"; //No I18N
    this.CHART_X_AXIS_LABELFONT = "chart.xAxis.labelFont"; //No I18N
    this.CHART_Y_AXIS_LABELFONT = "chart.yAxis.labelFont"; //No I18N
    this.CHART_DATALABEL_EXISTS = "chart.isDataLabel"; //No I18N
    this.CHART_DATALABEL_STACKED = "chart.isStackedDataLabel"; //No I18N
    this.CHART_PALETTE = "chart.palette"; //No I18N
    this.CHART_TRANSPARENCY = "chart.transparency"; // No I18N
    this.CHART_WEEKYR_FORMAT = "chart.weekYearFormat"; //No I18N
    this.CHART_X_TICKLABEL_DIRECTION = "chart.xTextDirection"; //No I18N
    this.CHART_X_TICKLABEL_FONT = "chart.xAxis.tickFont"; //No I18N
    this.CHART_Y_TICKLABEL_FONT = "chart.yAxis.tickFont"; //No I18N
    this.CHART_COMBO_LINE = "LINE";//No I18N
    this.CHART_COMBO_SPLINE = "SPLINE";//No I18N
    this.CHART_DEFAULT_ITEMLABEL_PROP = "chart.pie.dataItemLabel";//No I18N
    this.CONVERSION_RATE_EXISTS = "conversion.rate.exists"; //No I18N
    this.FUNNEL_LABEL_PROP = "funnel.conversion.label"; //No I18N
    this.FUNNEL_CONVERSION_PROP = "chart.funnel.conversion"; //No I18N
    this.FUNNEL_DESIGN = "chart.funnel.design"; //No I18N
    this.CHART_UNKNOWN_VALUE = "Unknown"; // No I18N
    this.CHART_AXIS_DRILLDOWN = "chart.axis.drilldown"; // No I18N
    this.CHART_LEGEND_DRILLDOWN = "chart.legend.drilldown"; // No I18N
    this.PACKBUBBLE_DATALABEL_PROP = "packbubble.datalabel"; // No I18N
    this.BAR_COMBOTYPE = 0;
    this.LINE_COMBOTYPE = 1;
    this.SPLINE_COMBOTYPE = 2;

    //various propType used in ZDBPropertySet
    this.STRING_PROPTYPE = "zdbs"; //No I18N
    this.COLOR_PROPTYPE = "zdbc"; //No I18N
    this.FONT_PROPTYPE = "zdbf"; //No I18N
    this.DOUBLE_PROPTYPE = "zdbd"; //No I18N
    this.MULTIVAL_PROPTYPE = "zdbm"; //No I18N
    this.THOLD_PROPTYPE = "zdbt"; //No I18N

    //gradient Types for Plot background
    this.GRADIENT_NORMAL_TYPE = "normal"; //No I18N
    this.GRADIENT_LEFT_RIGHT_TYPE = "leftRight"; //No I18N
    this.GRADIENT_TOP_BOTTOM_TYPE = "topBottom"; //No I18N
    
    this.DEFAULT_CONV_LABEL = "Conversion Rate:"; //No I18N
    this.CONV_LBL_LENGTH = 100;
    this.funnelSpanStyles = ["weightedHeight", "uniformHeight", "weightedWidth"]; //No I18N
    this.funnelSpanHLStyles = ["weightedHeightHL", "uniformHeightHL", "weightedWidthHL"]; //No I18N

    //Map chart Prop types
    this.CHART_MAP_RESETBTNSHOW = "chart.map.resetbtnshow";//No I18N
    this.CHART_MAP_SEARCHOPTSHOW = "chart.map.searchoptshow";//No I18N
    this.CHART_MAP_FILL_OPACITY = "chart.map.fillopacity";//No I18N
    this.CHART_MAP_MARKER_TYPE = "chart.map.markerType";//No I18N
    this.CHART_MAP_SHOWLEGENDBYDEFAULT = "chart.map.showLegendByDefault";//No I18N
    this.CHART_MAP_DEFAULT_OPACITY = "chart.map.defaultOpacity";//No I18N

//    this.handlePropSetResp = function(response,requestOptions)
//    {
//        var div = document.createElement("div");
//        div.innerHTML = response.responseText;
//        document.body.appendChild(div);
//        var el = requestOptions["ELEMENT"];
//        var view = requestOptions.VIEW;
//        PropSettings.showDialog(el, view);
//    }

    this.closePropSet = function(element)//when back button is clicked
    {
    	var view = ZDBClientUI.getViewObj(element, "view");
    	var propSetUtil = view.propSetUtil;
    	ReactDOM.unmountComponentAtNode(propSetUtil.propSetForm.clrPalette);
        view.propSetUtil.status = false;
    };

    this.takeBackUpProp = function(propSetUtil, view)
    {
        //Copying Values
        propSetUtil.preLegTitleModified = view.legendTitleModified;
        propSetUtil.preLegTitleValue = view.legendTitleValue;
        propSetUtil.preLegFilterEnable = view.legendFilterEnable;
        propSetUtil.preLegPos = view.legendPos;
        propSetUtil.preGraphType = view.graphSelected;
        propSetUtil.preSubGraphType = view.subGraphType;
        //Copying References
        propSetUtil.prePropSet = jQuery.extend(true, {}, view.propSet); // for all properties
        propSetUtil.preCurListProp = jQuery.extend(true, {}, view.curListProp); // for all curListProp
        propSetUtil.preLegDetInfo = jQuery.extend(true, [], view.legDetInfo); // for Legend Colors
        if(propSetUtil.clrPltObj)
        {
            propSetUtil.prePltInfo = jQuery.extend(true, {}, ZDBColorPalette.pltInfo);
        }
        view.zacf.setPreview(); // for format and tooltip
        propSetUtil.backUp = true;
    }

    this.clonePropObjects = function(objToClone)
    {
        if(objToClone === null || 'object' !== typeof(objToClone))
        {
            return objToClone;
        }
        if(objToClone.cloneNode)
        {
            return objToClone.cloneNode(true);
        }
        if(objToClone instanceof Array)
        {
            return objToClone.slice();
        }
        else if(objToClone instanceof ZDBFontObject)
        {
            return new ZDBFontObject(objToClone["FONTNAME"],objToClone["FONTSIZE"], objToClone["FONTSTYLE"], objToClone["COLOR"]);
        }
        else if(objToClone instanceof ZDBColorObject)
        {
            var clrObj = new ZDBColorObject();
            clrObj.setColorObject(objToClone["TYPE"],objToClone["STARTCOLOR"], objToClone["ENDCOLOR"]);
            return clrObj;
        }
        else if(objToClone instanceof ZDBTholdObject)
        {
            return new ZDBTholdObject(objToClone["COLID"],objToClone["FMLCOLID"], objToClone["OPERATION"], objToClone["AOT"]);
        }
        var clonedObj = {};
        for(prop in objToClone)
        {
            clonedObj[prop] = PropSettings.clonePropObjects(objToClone[prop]);
        }
        return clonedObj;
    }


    this.restoreBackUpProp = function(view, propSetUtil)
    {
        if(!propSetUtil)
        {
            propSetUtil = view.propSetUtil;
        }
        if(!propSetUtil || propSetUtil.backUp != true)
        {
            return;
        }
        // restoring references
        view.zacf.unSetPreview();
        view.propSet = jQuery.extend(true, {}, propSetUtil.prePropSet); // for all properties
        view.curListProp = jQuery.extend(true, {}, propSetUtil.preCurListProp); // for all curListProp
        view.legDetInfo = jQuery.extend(true, [], propSetUtil.preLegDetInfo); // for Legend Colors
        // restoring values
        view.legendTitleModified = propSetUtil.preLegTitleModified;
        view.legendTitleValue = propSetUtil.preLegTitleValue;
        view.legendFilterEnable = propSetUtil.preLegFilterEnable;
        view.legendPos = propSetUtil.preLegPos;
        view.graphSelected = propSetUtil.preGraphType;
        view.subGraphType = propSetUtil.preSubGraphType;
        ZDBColorPalette.pltInfo = propSetUtil.prePltInfo;
        propSetUtil.resetModified();
        if(view.modified)
        {
            if(view.permissions.designModify)
            {
                view.modified = propSetUtil.prevMod;
                view.viewCtx.updateModified();
                propSetUtil.createPropMap();
            }
        }
        view.loaded = true;//since property has been reset- it is considered as View loaded properly in previous state
        PropSettings.removeBackUpProp(propSetUtil);
    }
    
    this.removePropSet = function(view)
    {
        if(view.drillArrInfo)
        {
            delete view.pathList;
            delete view.drillArrInfo;
        }
        delete view.propSetUtil;
        delete view.propSet;
    }

    this.removeBackUpProp = function(propSetUtil)
    {
        delete propSetUtil.preLegTitleModified;
        delete propSetUtil.preLegTitleValue;
        delete propSetUtil.preLegFilterEnable;
        delete propSetUtil.preLegPos;
        delete propSetUtil.preGraphType;
        delete propSetUtil.preSubGraphType;
        delete propSetUtil.previewMap;
        delete propSetUtil.prevMod;
        delete propSetUtil.backUp;
    }

    this.showDialog = function(el, view)
    {
        if(!loadReact(PropSettings.showDialog, [el, view]))  // for Legend color palette
        {
                    return false;
        }
        if(!view)
        {
            view = ZDBClientUI.getViewObj(el, "view");
        }
        if(!view.loaded)
        {
            setTimeout(function(){PropSettings.showDialog(el, view);},200);
            return;
        }
        var propSetForm = $("ZDBPropSetForm");
        var propSetUtil = view.propSetUtil ? view.propSetUtil : new ZDBPropSetUtil(view);
        view.fetchDrillCols = true;
        propSetUtil.el = el;
        propSetForm = ZDBUtil.getCloneNode(propSetForm);
        document.body.appendChild(propSetForm); // adding cloned node to document
        ZDBUtil.setNamedChildEls(propSetForm,propSetForm);
        propSetUtil.checkPossibleOpt(view);
        propSetUtil.propSetForm = propSetForm;
        propSetForm.propSetUtil = propSetUtil;
        propSetUtil.propSetForm.closeDialog = propSetUtil.closeDialog;//adding  Close function
        propSetUtil.propSetForm.style.display="block";
        jQuery(propSetForm.propSetNav).find("div").click(function(){ZDBInlineDialog.changeTab(this, propSetForm);});
        jQuery(propSetForm.applyBtn).click(function(){PropSettings.refreshPropSet(this);});
        jQuery(propSetForm.cancelBtn).click(function(){ZDBInlineDialog.closeDialog();});
        jQuery(propSetForm.prevLink).click(function(){ZDBPreviewProp.showPreviewGraph(this);});
        jQuery(propSetForm.hidePrevLink).click(function(){ZDBPreviewProp.changeTab(this);});
        jQuery(propSetForm.propResetLink).click(function(){ZDBPreviewProp.resetPreview(this);});
        
        propSetForm.genPropNav.focusEl = propSetForm.chartTitle;
        propSetForm.axisPropNav.focusEl = propSetForm.xAxisLabel;
        var helplink = ZDBClientUI.isCustomDomain ? "" : ZDBI18NMap.g("zrpt.common.analysis.settings.js.help.chart");
        ZDBInlineDialog.showDialog(propSetUtil.propSetForm, { //inline showDialog method is called
            VIEW:view,
            TITLE: ZDBI18NMap.g("zrpt.common.analysis.settings.js.dlgTtl"),
            HELP: helplink,   //NO I18N
            PRECLOSE: PropSettings.closePropSet
        });
        propSetForm.style.display = "";
        var openPossibleOpt = propSetForm.genPropNav;
        var settingType = el.getAttribute("settingType");
        if(settingType === "AxisSettings") //when settings icon near to Axis is clicked
        {
            openPossibleOpt = propSetForm.axisPropNav;
        }
        else if(settingType === "LegendSettings") //when settings icon near to Legend is clicked
        {
            openPossibleOpt = propSetForm.legPropNav;
        }
        else if(settingType === "ThresholdSettings") // when edit threshold menu clicked
        {
            openPossibleOpt = propSetForm.tholdPropNav;
        }
        //changing tab accordingly
        PropSettings.processPossibleOpt(propSetUtil);//setting values in Property Dialog
        ZDBInlineDialog.changeTab(openPossibleOpt, propSetForm);//No I18N
        if(propSetUtil.focusEl)
        {
            ZDBUtil.setFocus(propSetUtil.focusEl, true);
        }
        
        propSetUtil.resetModified();
    }

    this.getLegendModelForChart = function(response,requestOptions)
    {
    	if(!ZDBLazyLoadUtil.loadJsAlone("zdbjs/react/ZRDBColorModel.js","APPENDTODOC",PropSettings.getLegendModelForChart,[response,requestOptions]))//NO I18N
        {
    		return;
        }
    	if(!ZDBLazyLoadUtil.loadCssOnly("ZRDBSettings.css", "APPENDTODOC", PropSettings.getLegendModelForChart,[response,requestOptions]))
    	{
            return;
    	}
    	if(response.responseText)
    	{
    		PropSettings.setLegendChartClrPltInfo(JSON.parse(response.responseText), requestOptions["VIEW"]);
    	}
    	else
    	{
    		PropSettings.setLegendChartClrPltInfo(response, requestOptions["VIEW"]);
    	}
    };
    
    this.setLegendChartClrPltInfo = function(jsonInfo, view)
    {
    	var propSetUtil = view.propSetUtil;
    	ZDBColorPalette.pltInfo  = jsonInfo;
    	PropSettings.setChartColorPaletteInfo(propSetUtil, ZDBColorPalette.pltInfo);
    	var factory = React.createFactory(DBSettingsColorModel);
    	var holder = propSetUtil.propSetForm.clrPalette;
    	propSetUtil.clrPltObj = ReactDOM.render(factory({jsonInfo:ZDBColorPalette.pltInfo,isViewMode:true, propSetUtil: propSetUtil,noTrans:true}),holder);
    }
    
    this.setChartColorPaletteInfo = function(propSetUtil, clrInfo)
    {
    	var chartCurPlt = propSetUtil.getPropValue(PropSettings.CHART_PALETTE, PropSettings.STATIC_PROP, PropSettings.STRING_PROPTYPE);
//    	clrInfo.CHART_TRANS = propSetUtil.getPropValue(PropSettings.CHART_TRANSPARENCY, PropSettings.STATIC_PROP, PropSettings.STRING_PROPTYPE);
		clrInfo.CURPLT = chartCurPlt ? chartCurPlt : clrInfo.DBCOLORCONFIG.CURPLT;
    }
    
    this.onFailureFunc = function(response,options)
    {
    	// console.log("Failed");
    }
    this.refreshPropSet = function(element)
    {
        var view = ZDBClientUI.getViewObj(element,"view");
        if(view.modified)
        {
        	ZRSDrillDown.clearDrillDet(view);
        }
        if(ZDBGlobal.isDynDrillEnabled)
        {
        view.zrsDrillObj = null;
        }
        var propSetUtil = view.propSetUtil;
        if(propSetUtil.isDrillDown) 
        {
            DrilldownConfig.setDrillPathInfo(view);
        }
        propSetUtil.refreshPropSet();
    }
    
    this.removeUnwantedPropEls = function(el)
    {
        if(el)
        {
            var parNode = el.parentNode;
            parNode.removeChild(el);
        }
    }
    
    this.pltReqNeed = function(view)
    {
    	var data = view.chartObj.getData();
    	var reqNeeded = true;
    	var pltSubType = data["ZRCLRPALETTE"]["SUBTYPE"];
    	var pltType = data["ZRCLRPALETTE"]["TYPE"];
    	var chartPlt = pltType + "__" + pltSubType;
    	var pltInfo = ZDBColorPalette.pltInfo;
    	if(!pltInfo|| !pltInfo.PLTS)
    	{
    		return true;
    	}
    	var pltsInfo = pltInfo["PLTS"];
    	var pltTypes = Object.keys(pltsInfo);
    	var pltAvlbl = false;
        pltTypes.forEach(function(plt){
        	if(plt == pltType)
        	{
            var pltSubTypes = Object.keys(pltsInfo[plt]);
            pltSubTypes.forEach(function(subPlt){
                if(subPlt == pltSubType)
                {
                	pltAvlbl = true;
                }
            });
        	}
        });
        reqNeeded = !pltAvlbl || !(ZDBColorPalette.pltInfo.CURPLT == chartPlt) ? true : false;
        return reqNeeded;
    }

    this.sendClrPltReq = function(view)
    {
//    	AjaxAPI.sendRequest({
//            URL: "/reportsapi/db/" + ZDBClientUI.dbId + "/GETDBCOLORMODEL", //No I18N
//            PARAMETERS: "DBID=" + ZDBClientUI.dbId, //No I18N
//            ONSUCCESSFUNC: PropSettings.getLegendModelForChart,
//            ONFAILUREFUNC: PropSettings.onFailureFunc,
//            VIEW: view,
//            LAZYLOADREQ: false
//        });
        
        ZRSAjax.get('/reportsapi/db/' + ZDBClientUI.dbId + '/GETDBCOLORMODEL', 'DBID=' + ZDBClientUI.dbId, {
            successHandler: {
                func: PropSettings.getLegendModelForChart,
                ctx: PropSettings
            },
            failureHandler: {
                func: PropSettings.onFailureFunc,
                ctx: PropSettings
            },
            errorHandler: {
                func: PropSettings.onFailureFunc,
                ctx: PropSettings
            },
            VIEW: view,
            LAZYLOADREQ: false
        });
    };
    
    this.processGeneralProp = function(propSetUtil)
    {
        var generalObj = new ZDBGeneralProp(propSetUtil);
        generalObj.setGeneralPropValues();
        var view = propSetUtil.view;
        if(view.graphSelected === "PIE")
        {
            generalObj.setPieItemLabelVal();
            if(view.subGraphType === "FUNNEL") 
            {
                var funnelObj = new ZAFunnelProp(propSetUtil);
                funnelObj.setFunnelProperties();
                propSetUtil.funnelObj = funnelObj;
            }
            else
            {
            	generalObj.setDataLabelFontDlg(propSetUtil);
            }
            ZACF.setPieInstKey(view, propSetUtil.propSetForm.pieItemLabel.ZACFDlgLink);
        }
        else if(view.subGraphType === "PACKEDBUBBLE" && view.chartSelectedEl.packBubbleAllowed)
        {
            generalObj.setPackBubbleLabelVal();
            generalObj.setDataLabelFontDlg(propSetUtil);
        }
        propSetUtil.generalObj = generalObj;
    }
    
    this.processAxisProp = function(propSetUtil)
    {
        var axisObj = new ZDBAxisProp(propSetUtil);
        axisObj.setAxisPropValues();
        propSetUtil.axisObj = axisObj;
    }
    
    this.processThresholdProp = function(propSetUtil)
    {
        var thresholdObj = new ZDBThresholdProp(propSetUtil);
        thresholdObj.setThresholdPropValues();
        propSetUtil.thresholdObj = thresholdObj;
    }

    this.processBackgroundProp = function(propSetUtil)
    {
        var backgroundObj = new ZDBBackgroundProp(propSetUtil);
        backgroundObj.setBackgroundPropEls();
        propSetUtil.backgroundObj = backgroundObj;
    }

    this.processPossibleOpt = function(propSetUtil)
    {//setting values in Property Dialog
        //methods are called only when the corresponding properties is present.
        var propSetForm = propSetUtil.propSetForm;
        ZDBUtil.setNamedChildEls(propSetForm, propSetForm, "compName", false);
        
        PropSettings.processGeneralProp(propSetUtil);
        
        var view = propSetUtil.view;
        if(propSetUtil.isAxisProp)
        {
            PropSettings.processAxisProp(propSetUtil);
            PropSettings.processBackgroundProp(propSetUtil);
        }
        else
        {
            PropSettings.removeUnwantedPropEls(propSetForm.axisPropNav);
            PropSettings.removeUnwantedPropEls(propSetForm.axisProp);
            PropSettings.removeUnwantedPropEls(propSetForm.bgPropNav);
            PropSettings.removeUnwantedPropEls(propSetForm.backgroundProp);
            delete propSetForm.axisPropNav;
            delete propSetForm.axisProp;
            delete propSetForm.bgPropNav;
            delete propSetForm.backgroundProp;
            delete propSetUtil.isAxisProp;
        }
        
//        if(propSetUtil.isLegend && !view.chartObj)
//        {
//            if(view.legendPos != "NONE")
//            {
//                settings.setLegendProperties(propSetUtil);
//            }
//            else
//            {
//                settings.disableLegendProperties(propSetUtil);
//            }
//            ZALegendSettings.showLegendNames(propSetUtil);
//            if(propSetUtil.isComboProp)
//            {
//                ZAComboChart.setComboChartOpt(propSetUtil);
//            }
//            else
//            {
//                PropSettings.removeUnwantedPropEls(propSetForm.comboPropNav);
//                PropSettings.removeUnwantedPropEls(propSetForm.comboProp);
//                delete propSetForm.comboPropNav;
//                delete propSetForm.comboProp;
//                delete propSetUtil.isComboProp;
//            }
//        }
//        else
        if(propSetUtil.isLegend && view.chartObj)
        {
            if(view.legendPos != "NONE")
            {
                settings.setLegendProperties(propSetUtil);
            }
            else
            {
                settings.disableLegendProperties(propSetUtil);
            }
//            settings.setLegendFormatKey(view);
            var chartObj = view.chartObj;
            var legendModel = chartObj.getLegendModel();
            legendModel.showLegendNames(propSetUtil);
            if(propSetUtil.isComboProp)
            {
                legendModel.setComboChartOpt(propSetUtil);
            }
            else
            {
                PropSettings.removeUnwantedPropEls(propSetForm.comboPropNav);
                PropSettings.removeUnwantedPropEls(propSetForm.comboProp);
                delete propSetForm.comboPropNav;
                delete propSetForm.comboProp;
                delete propSetUtil.isComboProp;
            }
            if(ZDBGlobal.isClrPltEnabled())
        	{
            	if(!ZDBColorPalette.pltInfo || !ZDBColorPalette.pltInfo.PLTS)
            	{
            		PropSettings.sendClrPltReq(view);
            	}
            	else
            	{
            		PropSettings.getLegendModelForChart(ZDBColorPalette.pltInfo, {VIEW: view});
            	}
        	}
        }
        else
        {
            PropSettings.removeUnwantedPropEls(propSetForm.legPropNav);
            PropSettings.removeUnwantedPropEls(propSetForm.legendProp);
            PropSettings.removeUnwantedPropEls(propSetForm.comboPropNav);
            PropSettings.removeUnwantedPropEls(propSetForm.comboProp);
            delete propSetForm.legPropNav;
            delete propSetForm.legendProp;
            delete propSetForm.comboPropNav;
            delete propSetForm.comboProp;
            delete propSetUtil.isLegend;
            delete propSetUtil.isComboProp;
        }

        if(propSetUtil.isMapProp)
        {
            propSetUtil.mapPropObj = new ZDBMapProp(propSetUtil);
        }
        else
        {
            PropSettings.removeUnwantedPropEls(propSetForm.mapPropNav);
            delete propSetForm.mapPropNav;
            delete propSetForm.mapPropObj;
            delete propSetUtil.isMapProp;
        }

        if(propSetUtil.isThreshold)
        {
            PropSettings.processThresholdProp(propSetUtil);
        }
        else
        {
            PropSettings.removeUnwantedPropEls(propSetForm.tholdPropNav);
            PropSettings.removeUnwantedPropEls(propSetForm.thresholdProp);
            delete propSetForm.tholdPropNav;
            delete propSetForm.thresholdProp;
            delete propSetUtil.isThreshold;
        }
        if(propSetUtil.isDrillDown)
        {
            ZAChartDrillView.showDrillSettings(propSetUtil);
        }
        else
        {
            PropSettings.removeUnwantedPropEls(propSetForm.drillDownNav);
            PropSettings.removeUnwantedPropEls(propSetForm.drillDownProp);
            delete propSetForm.drillDownNav;
            delete propSetForm.drillDownProp;
        }
        if(view.graphSelected != "TABLE_CHART")
        {
            ZACF.appendZACFOptions(view, propSetUtil.propSetForm, true);
        }
        else
        {
            PropSettings.removeUnwantedPropEls(propSetForm.formatNav);
            PropSettings.removeUnwantedPropEls(propSetForm.formatProp);
            delete propSetForm.formatNav;
            delete propSetForm.formatProp;
        }
        var prevLink = propSetUtil.propSetForm.prevLink;
        if(prevLink != null)
        {
            prevLink.style.display = (view.graphSelected == "TABLE_CHART") ? "none" : "";
        }
    }

     // to get color from colorprop utility (similar to threshold) for Start,end color and also Foreground and Background color
    this.getColorPropUtil = function(el, type, showConvFont)
    {
        var className = el.className;
        var clsNames = className.split(" ");//space to split the className
        if(clsNames[clsNames.length -1] == "alpha")
        {
            return;
        }
        var currClr = "#"+ el.getAttribute("value");
        ZDBColorPalette.fontEventExist = false;
        if(showConvFont)
        {
            ZDBColorPalette.fontEventExist = true;
            var propSetUtil = ZDBClientUI.getViewObj(el, "propSetUtil");
            ZDBColorPalette.showClrPalette(el, currClr, PropSettings.getTitleForColorPalette(type), propSetUtil.view);
        }
        else if(type == 6)
        {
            var row = ZDBUtil.getParentWithTagName(el, "tr"); //No I18N
            ZDBUtil.setNamedChildEls(row,row,"elname",false);
            ZDBColorPalette.showClrPalette(el, currClr,  PropSettings.getTitleForColorPalette(type));
        }
        else if(type)
        {
            propSetUtil = ZDBClientUI.getViewObj(el, "propSetUtil");
            if(type == 5)
            {
                ZDBColorPalette.fontEventExist = true;
            }
            ZDBColorPalette.showClrPalette(el, currClr, PropSettings.getTitleForColorPalette(type), propSetUtil.view);
            return;
        }
    }

    this.adjustToHex = function(colorIntVal)//conversion int to Hex
    {
        if(colorIntVal == null)
        {
            colorIntVal = 0;
        }
        var colorVal = parseInt(colorIntVal).toString(16) + "";
        var len = colorVal.length;
        var adjust = "";
        for(var i=0 ; i < 6-len; i++)
        {
            adjust += "0" ;
        }
        return adjust+colorVal;
    }

    this.selectFromCombo = function(selectEl, value, prop)
    {
        var childEls = selectEl.getElementsByTagName("option");
        for(var i=0, j=childEls.length; i<j; i++)
        {
            var child = childEls[i];
            if(child.getAttribute(prop) == value)
            {
                child.selected = true;
                break;
            }
        }
        return selectEl;
    }

    this.getTitleForColorPalette = function(type)
    {
        if(type == 1)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseBgClr");
        }
        else if(type == 2)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseStartClr");
        }
        else if(type == 3)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseEndClr");
        }
        else if(type == 4)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseBdrClr");
        }
        else if(type == 5)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseFontClr");
        }
        else if(type == 6)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.threshold.settings.js.dlgTtl");
        }
        else if(type == 7)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseGridLineClr");
        }
        else if(type==8)
        {
                return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseDefaultClr");
        }
        else if(type==9)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseLegendClr");
        }
        else if(type==10)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.convLblClr");
        }
        else if(type==11)
        {
            return ZDBI18NMap.g("zrpt.common.analysis.settings.js.chooseIconClr");
        }
        else if(type == 12)
        	{
        	return "Choose Theme Colour"; //No I18N
        	}
        return null;
    }

    this.removeUnwantedChilds = function(parent,len, childEls)
    {
        if((len && len < 1) || (len == null))
        {
            len = 0;
        }
        if(!childEls)
        {
            childEls= parent.childNodes;
        }
        while(childEls.length > len)
        {
            parent.removeChild(childEls[len]);
        }
    }
    
    this.setLegendPreviewOnPltChange = function(propSetUtil,jsonInfo)
    {
        var view = propSetUtil.view;
    	var legModel = view.chartObj.getLegendModel();
    	var legEls = legModel.getLegendEls();
    	var legLength = legEls.length;
    	var curPlt = jsonInfo["CURPLT"].split("__");//No I18N
    	var pltType = curPlt[0];
    	var plt = curPlt[1];
    	var palettes = jsonInfo["PLTS"];
    	var key = palettes["KEY"];
//    	var dbTrans = palettes[pltType][plt][key.TRANS];
//    	jsonInfo.CHART_TRANS = dbTrans;
    	var clrArr = PropSettings.getCurrColorArrayForLegend(pltType,plt,palettes,legLength);
    	var isResetToPltClrs = propSetUtil.propSetForm.resetKey.style.display === "none";
    	var legendLabelProp = propSetUtil.propSetForm.legendLabelProp;
    	var legendValTr = ZDBUtil.getElsWithTagNAttrWithVal(legendLabelProp,"elname","legendValTr","tr");//No I18N
    	for(var legIdx=0,clrIdx = 0;legIdx<legLength;legIdx++)
    	{
    		if(legEls[legIdx]["filterValue"] == "--NULL--" || legEls[legIdx]["origValue"] == null)
    		{
    			continue;
    		}
    		var legPropEl = legendValTr[legIdx];
    		var legendColor = legPropEl.legendColor;
    		if(legEls[legIdx].isPltClr || isResetToPltClrs)
    	    {
    			var legClr = clrArr[clrIdx];
    			legEls[legIdx].isPltClr = true;
    			legClr = ZRColorModelUtil.convertToHexIfRGB(legClr);
    			var clrVal = ZDBUtil.replaceStr(legClr.toUpperCase(), "#", "");
    			legPropEl.setAttribute("legendclr", clrVal);
    			legPropEl.legendColor.style.backgroundColor = legClr; // hex color
    			legendColor.setAttribute("value", clrVal);
    	    }
    		clrIdx++;
		}
    	if(!isResetToPltClrs && pltType)
		{
		    propSetUtil.showCreateClrPltKeyLink(pltType);
		}
    }
    
    this.getCurrColorArrayForLegend = function(type,plt,palettes,len, inHex)
    {
    	var key = palettes["KEY"];
    	var typePalette = type == "SOLID"? palettes["SOLID"] : type == "ONE"? palettes["ONE"] : palettes["TWO"];//No I18N
    	var currPlt = typePalette[plt];
    	var clrs = currPlt[key.CLRS]
    	var brtns = currPlt[key.BRTNS];
    	var trans = 0;//to display the legend colors without transparency
    	var clrArr, palette;
    	switch(type)
    	{
    		case "SOLID"://No I18N
    		{
    			clrArr = ZCFormatUtil.getLegendColorPalleteForSolid(clrs, len, brtns, trans, inHex);
    			return clrArr;
    		}
    		break;
    		case "ONE"://No I18N
    		{
    			var clr = clrs.substr(0,7);
        		var isRev = currPlt[key.RGRAD];
        		palette = ZRColorModelUtil.getElevenStepColors(clr, isRev);
        		clrArr = ZCFormatUtil.getLegendColorPalleteForOne(palette, len, brtns, trans, inHex);
        		return clrArr;
    		}
    		break;
    		case "TWO"://No I18N
    		{
    			var twoClrs = clrs.split(",");
        		var isRev = currPlt[key.RGRAD];
        		var isMerg = currPlt[key.MGRAD];
        		if(len<=11)
        		{
        			palette = ZRColorModelUtil.getTwoColorsMix(twoClrs[0], twoClrs[1], isMerg, isRev, false);
        		}
        		else
        		{
        			var length = len > 22 ? 22 : len;
        			palette = ZRColorModelUtil.getTwoColorsMixForGT11Clrs(twoClrs[0], twoClrs[1], isMerg, isRev, false, length);
        		}
        		clrArr = ZCFormatUtil.getLegendColorPalleteForTwo(palette, len, brtns, trans, inHex);
        		return clrArr;
    		}
    		break;
    		default: return null;
    	}
    }
}

function ZDBPropSetUtil(view)
{
    this.view = view;
    view.propSetUtil = this;
    this.createPropMap();
}

ZDBPropSetUtil.prototype.checkPossibleOpt = function(view)
{//assigning possible properties for a Chart
    this.isGenProp = true;
    this.isAxisProp = false;
    this.isMapProp = false;
    this.isLegend = false;
    this.isThreshold = false;
    this.isComboProp = false;
    this.isDrillDown = false;
    this.isScatter = false;
    this.isFilled = false;
    this.isBubble = false;
    var graphSelected = view.graphSelected;
    if(graphSelected != "TABLE_CHART")
    {
        if(!(graphSelected == "PIE" || graphSelected == "WEB" ||
                (view.subGraphType === "PACKEDBUBBLE" && view.chartSelectedEl.packBubbleAllowed))) //No I18N
        {
            this.isAxisProp = true;
            var tholdAxisDivEls = view.tholdAxisDiv;
            if(tholdAxisDivEls && tholdAxisDivEls.length >= 1)
            {
                this.isThreshold = true;
            }
        }
        if(view.chartObj && view.chartObj.getLegendModel()) // for JS chart
        {
            this.isLegend = true;
        }
        if(graphSelected.match(/^(MAPSCATTER|MAPBUBBLE|MAPAREA|MAPPIE|MAPPIEBUBBLE)$/))
        {
            this.isMapProp = true;
            this.isAxisProp = false;
            this.isThreshold = false;
            if(graphSelected.match(/^(MAPSCATTER)$/))
            {
                this.isScatter = true;
        }
            if(graphSelected.match(/^(MAPBUBBLE)$/))
            {
                this.isBubble = true;
            }
             if(graphSelected.match(/^(MAPAREA)$/))
            {
                this.isFilled = true;
            }
        }
//        if(!view.chartObj && view.legendType) // for JFreechart
//        {
//            this.isLegend = true;
//        }
        if(graphSelected == "COMBO")
        {
            this.isComboProp = true;
        }
        
        var drillDet = view.drillDet;
        if(drillDet)
        {
            this.isDrillDown = drillDet.isChartDrillable;
        }
    }
}

PropSettings.createDefPropMap = function(view)
{
    var defPropSet = view.defPropSet;
    for(var i=0; i < defPropSet.length; i++)
    {
        var propType = defPropSet[i].PROPTYPE;
        if(propType == PropSettings.FONT_PROPTYPE)
        {
            PropSettings.createDefFontMap(view, defPropSet[i]);
        }
        else if(propType == PropSettings.COLOR_PROPTYPE || propType == PropSettings.THOLD_PROPTYPE)
        {
            continue;
        }
    }
}

PropSettings.createDefFontMap = function(view, defFontPropValue)
{
    var fontName = defFontPropValue.FONTNAME;
    var fontSize = defFontPropValue.FONTSIZE;
    var fontStyle = defFontPropValue.FONTSTYLE;
    var newDefProp = new ZDBFontObject(fontName, fontSize, fontStyle);
    PropSettings.setDefPropMap(view, defFontPropValue.PROPNAME, newDefProp);
}

PropSettings.refreshZACF = function(view, modified) {
    if(modified || view.zacf.isModified()) {
        view.zacf.persist(); 
        view.setModified();
        view.propSetUtil.canReset = true; //for preview
    }  
}

PropSettings.previewZACF = function(view) {
    if(ZDBAnalysisEditViewMode.isColumnsRenamed(view) || view.zacf.isModified()) {
        view.propSetUtil.prevMod = view.modified;
        view.modified = 1;
        view.propSetUtil.canReset = true;
    }
}

ZDBPropSetUtil.prototype.showCreateClrPltKeyLink = function()
{
	var pltType = this.clrPltObj.props.jsonInfo.CURPLT.split("__")[0];
	this.showCreateClrPltKey(pltType);
}

ZDBPropSetUtil.prototype.showCreateClrPltKey = function(pltType)
{
	this.propSetForm.createClrPltKey.style.display = (pltType === "SOLID" && !ZDBGlobal.integrated && ZDBClientUI.isDBOwner ) ? "inline-block" : "none";
}

ZDBPropSetUtil.prototype.closeDialog = function(dlgEl)
{
    var dynamicFieldsTBL = document.getElementById("dynamicFieldsTBL");
    PropSettings.removeUnwantedChilds(dynamicFieldsTBL);
    this.status = false; 
    return true;
}

ZDBPropSetUtil.prototype.refreshPropSet = function()//changing properties and refresh the chart
{
    this.modified = false;
    if(ZDBPropValidator.checkValuesinPropSet(this, true))//validating the properties
    {
        PropSettings.refreshZACF(this.view, ZDBAnalysisEditViewMode.updateRenamedColumns(this.view));
        ZDBInlineDialog.closeDialog();//close that dialog
        this.view.refreshView();
    }
}

ZDBPropSetUtil.prototype.setModified = function()
{
    this.modified = true;
}

ZDBPropSetUtil.prototype.resetModified = function()
{
    this.modified = false;
}

// set the property to the PropMap whenever the chages are done from PropSetForm
ZDBPropSetUtil.prototype.setDoubleProp = function(propEl, initValue, onDelCascade)
{
    if(propEl.getAttribute("propValue") != null)
    {
        propEl.value = propEl.getAttribute("propValue");
    }
    var newValue = propEl.value;
    this.beforeEnterInPropMap(propEl, this.getPropValuefromEl(propEl), newValue, initValue, onDelCascade);
}

ZDBPropSetUtil.prototype.setStringProp = function(propEl, initValue, onDelCascade)
{
    var newValue = null;
    if(propEl.getAttribute("propValue") != null)
    {
        newValue = propEl.getAttribute("propValue");
    }
    else
    {
        newValue = propEl.value;
    }
    this.beforeEnterInPropMap(propEl, this.getPropValuefromEl(propEl), newValue, initValue, onDelCascade);
}

ZDBPropSetUtil.prototype.setColorProp = function(propEl,initValue, onDelCascade)
{
    var colorMapEntry = new ZDBColorObject();
    colorMapEntry["TYPE"]=propEl.getAttribute("TYPE");//No I18N
    colorMapEntry["STARTCOLOR"]=propEl.getAttribute("STARTCOLOR");//No I18N
    colorMapEntry["ENDCOLOR"]=propEl.getAttribute("ENDCOLOR");//No I18N
    var propValue = colorMapEntry;
    this.beforeEnterInPropMap(propEl, this.getPropValuefromEl(propEl), propValue,initValue, onDelCascade);
}

ZDBPropSetUtil.prototype.setFontProp = function(propEl,initValue, onDelCascade)
{
    var fontMapEntry = new ZDBFontObject();
    fontMapEntry["FONTNAME"]=propEl.getAttribute("FONTNAME");//No I18N
    fontMapEntry["FONTSIZE"]=propEl.getAttribute("FONTSIZE");//No I18N
    fontMapEntry["FONTSTYLE"]=propEl.getAttribute("FONTSTYLE");//No I18N
    fontMapEntry["COLOR"]=propEl.getAttribute("COLOR");//No I18N
    var propValue = fontMapEntry;
    this.beforeEnterInPropMap(propEl, this.getPropValuefromEl(propEl), propValue,initValue,onDelCascade);
}

ZDBPropSetUtil.prototype.setTholdProp = function(propEl, initValue, onDelCascade)
{
    var tholdMapEntry = new ZDBTholdObject();
    tholdMapEntry["COLID"] = propEl.getAttribute("COLID"); //No I18N
    tholdMapEntry["FMLCOLID"] = propEl.getAttribute("FMLCOLID"); //No I18N
    tholdMapEntry["OPERATION"] = propEl.getAttribute("OPERATION"); //No I18N
    tholdMapEntry["AOT"] = propEl.getAttribute("AOT"); //No I18N
    var propValue = tholdMapEntry;
    this.beforeEnterInPropMap(propEl, this.getPropValuefromEl(propEl), propValue, initValue, onDelCascade);
}

ZDBPropSetUtil.prototype.checkInPropMapObj = function(propName, instId, propType)
{
    if(!this.checkInPropMap(propName))
    {
        this.createEntryInPropMap(propName);
    }
    if(!this.checkInPropMap(propName,instId))
    {
        this.createEntryInPropMap(propName, instId);
    }
    if(!this.checkInPropMap(propName,instId))
    {
        this.createEntryInPropMap(propName, instId, propType);
    }
}

ZDBPropSetUtil.prototype.setMultiValProp = function(propName, instId, propValue, propType)
{
    var oldValue = this.getPropValue(propName, instId, propType);
    if(oldValue == null)
    {
        this.checkInPropMapObj(propName, instId, propType);
        if(!(propValue instanceof Array))
        {
            this.enterChangesInPropMap(propName, instId, propType, new Array(propValue));
            return;
        }
    }
    else
    {
        if(!(propValue instanceof Array) && oldValue instanceof Array)
        {
            if(oldValue.indexOf(propValue) == -1)
            {
                oldValue.push(propValue);
            }
            this.enterChangesInPropMap(propName, instId, propType, oldValue);
            return;
        }
    }
    var equal = this.checkEqualObj(oldValue, propValue);
    if(!equal && propValue!=null)
    {
        this.enterChangesInPropMap(propName, instId, propType, propValue);
    }
}

ZDBPropSetUtil.prototype.setMultiValFromPropDiv = function(propEl)
{
    var propName = propEl.getAttribute("multivalkey");
    var instId = propEl.getAttribute("instid");
    var propValue = propEl.getAttribute("value");
    this.setMultiValProp(propName, instId, propValue, PropSettings.MULTIVAL_PROPTYPE);
}

ZDBPropSetUtil.prototype.beforeEnterInPropMapWithDet = function(propName, instId, propType, oldValue, newValue, initValue, onDelCascade)
{
	var equal = false;
    equal = this.checkEqualObj(initValue, newValue);

    if(instId == null || instId === 'static')
    {
        instId = 'static';//No I18N
    }
    if(equal)
    {
        if(onDelCascade)
        {
            this.deleteFromPropMap(propName, instId, propType);
        }
        return;
    }
    if(oldValue == null)
    {
        this.checkInPropMapObj(propName, instId, propType);
    }
    equal = this.checkEqualObj(oldValue, newValue);
    if(!equal && newValue!=null)
    {
        this.enterChangesInPropMap(propName, instId, propType, newValue);
    }
}

ZDBPropSetUtil.prototype.beforeEnterInPropMap = function(propEl, oldValue, newValue, initValue, onDelCascade)
{
	var propName = propEl.getAttribute("propname");
    var instId = propEl.getAttribute("instId");
    var propType = propEl.getAttribute("propType");
    this.beforeEnterInPropMapWithDet(propName, instId, propType, oldValue, newValue, initValue, onDelCascade)
}

ZDBPropSetUtil.prototype.checkEqualObj = function(object1, object2)
{
    if(object1 instanceof ZDBColorObject || object1 instanceof ZDBFontObject || object1 instanceof ZDBTholdObject)
    {
        return object2.equals(object1);
    }
    else if(object1 instanceof Array)
    {
        if(object1.length == object2.length)
        {
            return ((object1.toString()) == (object2.toString()));
        }
        return false;
    }
    else
    {
        return object2 == object1;
    }
}

//    ZDBPropSetUtil.prototype.printPropMap = function(propMap) //for debugging - don't delete it
//    {
//        if(propMap == null)
//        {
//            propMap = this.propMap;
//        }
//        for(var propName in propMap)
//        {
//            for(var instId in propMap[propName])
//            {
//                for(var propType in propMap[propName][instId])
//                {
//                    console.log(propName, instId, propType, propMap[propName][instId][propType]);
//                }
//            }
//
//        }
//    }
//
//        $H(PropSettings.propMap).each(function(propName)         //used the concepts of Hash from Prototype.js file --- thirdparty js
//        {
//            $H(propName.value).each(function(instId)
//            {
//                $H(instId.value).each(function(type)
//                {
//                    console.log(propName.key, instId.key , type.key ,type.value);
//                })
//            })
//        });
//    }


ZDBPropSetUtil.prototype.deleteFromPropEl = function(propEl)//deleting the propValue from map by passing propEl.
{
    var propName= propEl.getAttribute("propname");
    var instId = propEl.getAttribute("instId");
    var propType = propEl.getAttribute("propType");
    instId = instId == null ? PropSettings.STATIC_PROP : instId;
    return this.deleteFromPropMap(propName, instId, propType);
}

ZDBPropSetUtil.prototype.deleteFromPropMap = function(propName, instId, propType)//deleting from Map beacuse deleting the property or changing it to default.
{
    if(this.propMap[propName])
    {
        if(instId != null && this.propMap[propName][instId])
        {
            if(propType != null && this.propMap[propName][instId][propType])
            {
                delete this.propMap[propName][instId][propType];
                this.modified = true;
                return;
            }
            else if(!propType)
            {
                delete this.propMap[propName][instId];
                this.modified = true;
                return;
            }
        }
        else if(!instId)
        {
            delete this.propMap[propName];
            this.modified = true;
        }
    }
}

ZDBPropSetUtil.prototype.moveProp = function(from, to)
{
    this.propMap[to] = this.propMap[from];
    delete this.propMap[from];
};

ZDBPropSetUtil.prototype.removePropSet = function()
{
    delete this;
}

ZDBPropSetUtil.prototype.createPropMap = function()
{
    var propSet = jQuery.extend(true, {},this.view.propSet);
    for(var propName in propSet)
    {
        for(var instId in propSet[propName])
        {
    		var newInstId = ZDBAnalysisView.replaceInstKey(this.view, instId);
    		if(newInstId != instId)
    		{
    			propSet[propName][newInstId] = jQuery.extend(true, {}, propSet[propName][instId]);
    			delete propSet[propName][instId];
    			instId = newInstId;
    		}
            for(var propType in propSet[propName][instId])
            {
                if(propType === PropSettings.COLOR_PROPTYPE)
                {
                    if(!(propSet[propName][instId][propType] instanceof ZDBColorObject))
                    {
                        propSet[propName][instId][propType] = new ZDBColorObject(propSet[propName][instId][propType]);
                    }
                }
                else if(propType === PropSettings.FONT_PROPTYPE)
                {
                    if(!(propSet[propName][instId][propType] instanceof ZDBFontObject))
                    {
                        var fontValArr = propSet[propName][instId][propType];
                        propSet[propName][instId][propType] = new ZDBFontObject(fontValArr[0], fontValArr[1], fontValArr[2], fontValArr[3]);
                    }
                }
                else if(propType === PropSettings.THOLD_PROPTYPE)
                {
                    if(!(propSet[propName][instId][propType] instanceof ZDBTholdObject))
                    {
                        var tholdArr = propSet[propName][instId][propType];
                        propSet[propName][instId][propType] = new ZDBTholdObject(tholdArr[0], tholdArr[1], tholdArr[2], tholdArr[3]);
                    }
                }
            }
        }
    }
    this.propMap = propSet;
    this.modified = false;
}

PropSettings.setDefPropMap = function(view, propName, propValue)
{
    if(view.defPropMap)
    {
        if(view.defPropMap[propName])
        {
            view.defPropMap[propName] = propValue;
        }
        else
        {
            view.defPropMap[propName] = new Object();
            PropSettings.setDefPropMap(view, propName, propValue);
        }
    }
    else
    {
        view.defPropMap = new Object();
        PropSettings.setDefPropMap(view, propName, propValue);
    }

}

PropSettings.getDefPropMap = function(view, propName)
{
    if(view.defPropMap[propName])
    {
        return view.defPropMap[propName];
    }
}

ZDBPropSetUtil.prototype.checkInPropMap = function(propName, instId, propType)
{
    if(propName)
    {
        for(var i in  this.propMap)
        {
            if(propName == i)
            {
                if(instId == null)
                {
                    return 1;
                }
                for(var j in this.propMap[propName])
                {
                    if(instId == j)
                    {
                        if(propType == null)
                        {
                            return 1;
                        }
                        for(var k in this.propMap[propName][instId])
                        {
                            if(propType == k)
                            {
                                return 1;
                            }
                        }
                    }
                }
            }
        }
        return 0;
    }
    else
    {
        return 0;
    }
}

ZDBPropSetUtil.prototype.createEntryInPropMap = function(propName, instId, propType)
{
    if(propType != null)
    {
        if(instId != null)
        {
            if(propName != null)
            {
                this.propMap[propName][instId][propType] = new Object();
            }
        }
    }
    else if(instId != null)
    {
        if(propName != null)
        {
            this.propMap[propName][instId] = new Object();
        }
    }
    else if(propName != null)
    {
        if(this.propMap)
        {
            this.propMap[propName] = new Object();
        }
        else
        {
            this.propMap = new Object();
            this.propMap[propName] = new Object();
        }
    }
}

ZDBPropSetUtil.prototype.enterChangesInPropMap = function(propName, instId, propType, propValue)
{
    this.propMap[propName][instId][propType] = propValue;
    this.modified = true;
}

ZDBPropSetUtil.prototype.getPropValue = function(propName, instId, propType)
{
    var propValue = null;
    if(this.propMap && this.propMap[propName] && this.propMap[propName][instId])
    {
        propValue = this.propMap[propName][instId][propType];
    }
    return propValue;
}

ZDBPropSetUtil.prototype.getPropValuefromEl = function(propEl)
{
    var propName= propEl.getAttribute("propname");
    var instId = propEl.getAttribute("instId");
    var propType = propEl.getAttribute("propType");
    instId = instId == null ? PropSettings.STATIC_PROP : instId;
    return this.getPropValue(propName, instId, propType);
}

ZDBPropSetUtil.prototype.setPropMap = function(propEl, initValue, onDelCascade)
{
    var propType = propEl.getAttribute("proptype");
    switch(propType)
    {
        case PropSettings.DOUBLE_PROPTYPE:
            this.setDoubleProp(propEl, initValue, onDelCascade);
            break;
        case PropSettings.STRING_PROPTYPE:
            this.setStringProp(propEl, initValue, onDelCascade);
            break;
        case PropSettings.COLOR_PROPTYPE:
            this.setColorProp(propEl, initValue, onDelCascade);
            break;
        case PropSettings.FONT_PROPTYPE:
            this.setFontProp(propEl, initValue, onDelCascade);
            break;
        case PropSettings.MULTIVAL_PROPTYPE:
            this.setMultiValFromPropDiv(propEl);
            break;
        case PropSettings.THOLD_PROPTYPE:
            this.setTholdProp(propEl, initValue, onDelCascade);
            break;
    }
}

ZDBPropSetUtil.prototype.updateMultiPropVal = function(el)
{
    var view = ZDBClientUI.getViewObj(el,"view");
    var isMultiY = view.chartSelectedEl.isMultiY;
    var legEls = view.chartObj.getLegendModel().getLegendEls();
    for(var i=0, j=legEls.length, k=0; i<j ; i++)
    {
        var curLegEl = legEls[i];
        if(!curLegEl.getAttribute)
        {
            k++;
            continue;
        }
        if(isMultiY)
        {
            if(curLegEl.getAttribute("propValue") && curLegEl.getAttribute("propValue") != PropSettings.BAR_COMBOTYPE)
            {
                var propValue =  view.isComboSplineSelected ? PropSettings.SPLINE_COMBOTYPE : PropSettings.LINE_COMBOTYPE;
                curLegEl.setAttribute("propValue", propValue);
            }
        }
        else if(curLegEl.getAttribute("propName") && curLegEl.getAttribute("propName") != "BAR")
        {
            var legName = ZDBUtil.getElWithTagNAttrWithVal(curLegEl, "lgelname", "legendTxt", "span").innerHTML;//No I18N
            curLegEl.setAttribute("propValue", legName);
            var propName = view.isComboSplineSelected ? PropSettings.CHART_COMBO_SPLINE : PropSettings.CHART_COMBO_LINE;
            var delPropName = view.isComboSplineSelected ? PropSettings.CHART_COMBO_LINE : PropSettings.CHART_COMBO_SPLINE;
            curLegEl.setAttribute("propName", propName);
            this.deleteFromPropMap(delPropName);
        }
    }
    this.updateMultiValInPropMap(curLegEl);
}

ZDBPropSetUtil.prototype.updateMultiValInPropMap = function(curLegEl, toggleSel)
{
    this.modified = false;
    ZAComboChart.setComboProp(this, curLegEl, true, toggleSel);
}

ZDBPropSetUtil.prototype.modifyChartType = function(curLegEl, toggleSel)
{
    var view = this.view;
    var isMultiY = view.chartSelectedEl.isMultiY;
    var legName = ZDBUtil.getElWithTagNAttrWithVal(curLegEl, "lgelname", "legendTxt", "span").innerHTML;//No I18N
    if(curLegEl.getAttribute("propValue") != null && toggleSel)
    {
        curLegEl.removeAttribute("propValue");
    }
    else
    {
        if(isMultiY)
        {
            var propValue =  view.isComboSplineSelected ? PropSettings.SPLINE_COMBOTYPE : PropSettings.LINE_COMBOTYPE;
            curLegEl.setAttribute("propValue", propValue);
        }
        else
        {
            curLegEl.setAttribute("propValue", legName);
            var propName = view.isComboSplineSelected ? PropSettings.CHART_COMBO_SPLINE : PropSettings.CHART_COMBO_LINE;
            var delPropName = view.isComboSplineSelected ? PropSettings.CHART_COMBO_LINE : PropSettings.CHART_COMBO_SPLINE;
            this.deleteFromPropMap(delPropName);
            curLegEl.setAttribute("propName", propName);
        }
    }
    this.updateMultiValInPropMap(curLegEl, toggleSel);
}

//creation of PropSet variables when modified

ZDBPropSetUtil.prototype.createColorPropDiv= function(propSet, propName, instId, propValue)
{
    var colorChild = [];
    if(propValue["TYPE"])
    {
        colorChild[colorChild.length] = propValue["TYPE"];
    }
    else
    {
        colorChild[colorChild.length] = "NORMAL";
    }
    colorChild[colorChild.length] = propValue["STARTCOLOR"];
    colorChild[colorChild.length] = propValue["ENDCOLOR"];
    propSet[propName][instId][PropSettings.COLOR_PROPTYPE] = colorChild;
}

ZDBPropSetUtil.prototype.createFontPropDiv= function(propSet, propName, instId, propValue)
{
    var fontChild = [];
    fontChild[fontChild.length] = propValue["FONTNAME"];
    fontChild[fontChild.length] = propValue["FONTSIZE"];
    fontChild[fontChild.length] = propValue["FONTSTYLE"];
    fontChild[fontChild.length] = propValue["COLOR"];
    propSet[propName][instId][PropSettings.FONT_PROPTYPE] = fontChild;
}

ZDBPropSetUtil.prototype.createTholdPropDiv= function(propSet, propName, instId, propValue)
{
    var tholdChild = [];
    tholdChild[tholdChild.length] = propValue["COLID"];
    tholdChild[tholdChild.length] = propValue["FMLCOLID"];
    tholdChild[tholdChild.length] = propValue["OPERATION"];
    tholdChild[tholdChild.length] = propValue["AOT"];
    propSet[propName][instId][PropSettings.THOLD_PROPTYPE] = tholdChild;
}

ZDBPropSetUtil.prototype.createPropSetDiv = function(view)
{
    if(this.propMap == null)
    {
        return;
    }
    var propSet = PropSettings.clonePropObjects(this.propMap);
    for(var propName in this.propMap)
    {
        for(var instId in this.propMap[propName])
        {
            for(var propType in this.propMap[propName][instId])
            {
                switch(propType)
                {
                    case PropSettings.COLOR_PROPTYPE:
                        this.createColorPropDiv(propSet, propName, instId, this.getPropValue(propName, instId, PropSettings.COLOR_PROPTYPE));
                        break;
                    case PropSettings.FONT_PROPTYPE:
                        this.createFontPropDiv(propSet, propName, instId, this.getPropValue(propName, instId, PropSettings.FONT_PROPTYPE));
                        break;
                    case PropSettings.THOLD_PROPTYPE:
                        this.createTholdPropDiv(propSet, propName, instId,
                                this.getPropValue(propName, instId, PropSettings.THOLD_PROPTYPE));
                        break;
                }
            }
        }
    }
    view.propSet = propSet;
}

ZDBPropSetUtil.prototype.setPropSet = function(view, closeDialog)
{
    this.createPropSetDiv(view);//creating it in Div
    if(closeDialog)
    {
        view.setModified();//set view as modified because properties are changed.
    }
    else
    {
        view.propSetUtil.prevMod = view.modified;
        view.modified = 1;
    }
}

function ZDBTholdObject(colId, fColId, oper, aot)
{
    this.setTholdObject(colId, fColId, oper, aot);
}

ZDBTholdObject.prototype.setTholdObject = function(colId, fColId, oper, aot)
{
    this.COLID = colId;
    this.FMLCOLID = fColId;
    this.OPERATION = oper;
    this.AOT = aot;
}

ZDBTholdObject.prototype.equals = function(oldValue)
{
    if(this.COLID !== oldValue.COLID)
    {
        return false;
    }
    if(this.FMLCOLID !== oldValue.FMLCOLID)
    {
        return false;
    }
    if(this.OPERATION !== oldValue.OPERATION)
    {
        return false;
    }
    if(this.AOT !== oldValue.AOT)
    {
        return false;
    }
    return true;
}


function ZDBFontObject(fontName, fontSize, fontStyle, fontColor)
{
    this.setFontObject(fontName, fontSize, fontStyle, fontColor);
}

ZDBFontObject.prototype.setFontObject = function(fontName, fontSize, fontStyle, fontColor)
{
    this.FONTNAME = fontName;
    this.FONTSIZE = fontSize;
    this.FONTSTYLE = fontStyle;
    this.COLOR = fontColor == null ? 0 : fontColor;
}

ZDBFontObject.prototype.equals = function(oldValue)
{
    if(this.FONTNAME == oldValue.FONTNAME && this.FONTNAME == "Default")
    {
        return true;
    }
    if(this.FONTNAME != oldValue.FONTNAME)
    {
        return false;
    }
    if(this.FONTSIZE != oldValue.FONTSIZE)
    {
        return false;
    }
    if(this.FONTSTYLE != oldValue.FONTSTYLE)
    {
        return false;
    }
    if(this.COLOR != oldValue.COLOR)
    {
        return false;
    }
    return true;
}

ZDBFontObject.prototype.toString = function()
{
    var fontDisp;
    var fontName = this.FONTNAME;
    if(fontName == "Default")
    {
        return fontName ;
    }
    var fontNameLen = fontName.length;
    if(fontNameLen > 10)
    {
        fontDisp = fontName.substr(0, 9) + "..., ";
    }
    else
    {
        fontDisp = fontName + ", ";
    }
    fontDisp += this.FONTSIZE + "px, " + this.FONTSTYLE + ",      " ; //No I18N
    return fontDisp;
}

ZDBFontObject.prototype.toTitle = function()
{
    return this.FONTNAME + ", " + this.FONTSIZE + "px, " + this.FONTSTYLE + ", #" + PropSettings.adjustToHex(this.COLOR);//No I18N
           
}


function ZDBColorObject(init)// Color Object in Js
{
    if(!init)
    {
        this.TYPE = null;
        this.STARTCOLOR = null;
        this.ENDCOLOR = null;
    }
    else if(init instanceof Array && init.length === 3)
    {
        this.TYPE = init[0];
        this.STARTCOLOR = init[1];
        this.ENDCOLOR = init[2];
    }
    else
    {
        switch(init)
        {
            case 'bgColor': //No I18N
                this.setColorObject("normal", "16250871", "16250871"); //No I18N
                break;
            case 'borderClr': //No I18N
                this.setColorObject("normal","10526880", "10526880"); //No I18N
                break;
            case 'gridLineClr': //No I18N
                this.setColorObject("normal","10526880", "10526880"); //No I18N
                break;
            case 'defaultClr'://No I18N
                this.setColorObject("normal","3838697", "3838697"); //No I18N
                break;
            case 'JSDefaultClr'://No I18N
            	this.setColorObject("normal","5745147", "5745147"); //No I18N
                break;
        }
    }
}

ZDBColorObject.prototype.setColorObject = function(type, startColor, endColor)
{
    this.TYPE = type;
    this.STARTCOLOR = startColor;
    this.ENDCOLOR = endColor;
}

ZDBColorObject.prototype.equals = function(oldValue)
{
    if(this.TYPE != oldValue.TYPE)
    {
        return false;
    }
    if(this.STARTCOLOR != oldValue.STARTCOLOR)
    {
        return false;
    }
    if(this.ENDCOLOR != oldValue.ENDCOLOR)
    {
        return false;
    }
    return true;
}

ZDBLazyLoadFuncMap.lazyLoadFuncMap["PropSettings.showDialog"]=PropSettings.showDialog;
