//$Id $

function ZohoCharts(view, chartContainer, data)
{
    var chart = null;
    var chartType = ZCHARTCONST.get(view.graphSelected);
    var subChartType = ZCHARTCONST.get("ST_" + view.subGraphType);
    var modified = false;
    var seriesCount;
    var legendModel = null;
    var isGeoChart = data.geo && data.geo.isGeoChart;
    var previewContainer;
    var previewData;
    var previewChart = null;
    var tholdLabel = null;
    var tooltipFooter;
    var tooltip;

    this.initialize = function()
    {    	
    	ZCFormatUtil.setLegendColor(data);
    	ZCFormatUtil.setLegendFilter(data);
        this.setLegendModel();
        this.setClickMsg();
        this.setDrillEvents();
        this.setTickLblCursorStyle();
        this.setLegendTickMark();
        this.setOverridePoints();
        this.setPlotAsImage();
        this.setThresholdInfo();
        this.setLegendPositionForMobileView();
        this.resetMapState();
    };
    this.resetMapState = function()
    {
        jQuery(view.analysisArea).removeClass("unknownDataChart").removeClass("NoDataChart");
    }
    this.getViewObj = function()
    {
        return view;
    };
    
    this.setPreviewContainer = function(previewDiv)
    {
        previewContainer = previewDiv;
    };
    
    this.setPreviewData = function(previewJSON)
    {
        previewData = previewJSON;
    };
    
    this.setData = function(dataJSON)
    {
        data = dataJSON;
    };
    
    this.showPreview = function()
    {
        if(previewData["error"])
        {
            this.showErrorMsg();
            return;
        }
        ZCFormatUtil.setLegendColor(previewData);
        ZCFormatUtil.setTholdLabelWidth(previewData, null, tholdLabel, previewContainer);
        previewChart = isGeoChart ? $ZC.mapcharts(previewContainer,previewData) : $ZC.charts(previewContainer,previewData);
    };
    
    this.getData = function()
    {
        return data;
    };
    
    this.getAxisInfo = function(isXAxis, multiYCount)
    {
        if(!chart.nochartdata)
        {
            if(this.isHorizontalChart())
            {
                isXAxis = !isXAxis;
                multiYCount = multiYCount ? multiYCount : 0;
            }
            return isXAxis ? chart.axes.x.scale.domain() : chart.axes.y.groups[multiYCount].scale.domain();
        }
        return [0,1];
    };
    
    this.getChartData = function()
    {
        if(data.seriesdata && data.seriesdata.chartdata)
        {
            return data.seriesdata.chartdata;
        }
        return null;
    };
    
    this.chartRendered = function()
    {
    	return chart.chartRendered || chart.breakOnErr;
    }
    
    this.changeRenderer = function()
    {
    	if(chart && !this.chartRendered())
        {
        	chart.systemConf.chart.plot.renderer.mode = "canvas"; //No I18N
        }
    	else if(data.chart.plot)
        {
            if(data.chart.plot.renderer)
            {
                data.chart.plot.renderer.mode = "canvas"; //No I18N
                return;
            }
            else
            {
                data.chart.plot.renderer = {};
                this.changeRenderer();
            }
        }
        else
        {
            data.chart.plot = {};
            this.changeRenderer();
        }
    	this.changeAnimationProp(false);
    	// eventhough chartRendered in canvas, it has a default timeout for animation
    	// that causes slowness
    };
    
    this.changeAnimationProp = function(isEnable)
    {
    	if(chart && !this.chartRendered())
        {
        	chart.systemConf.chart.plot.animation.enabled = isEnable;
        }
    	else if(data.chart)
        {
            if(data.chart.plot)
            {
                if(data.chart.plot.animation)
                {
                    data.chart.plot.animation.enabled = isEnable;
                    return;
                }
                else
                {
                    data.chart.plot.animation = {};
                    this.changeAnimationProp(isEnable);
                }
            }
            else
            {
                data.chart.plot = {};
                this.changeAnimationProp(isEnable);
            }
        }
    };
    
    this.changeChartDraw = function(isEnable, dashView)
    {
    	if(dashView)
        {
            var length = ZDBDashboardAPI.viewsInDashboard(dashView).length;
    		if(length > 25 && (!legendModel || (legendModel && !(legendModel instanceof ZCPieLegendModel))))
            {
                this.changeRenderer();
            }
            else
            {
                this.changeAnimationProp(isEnable);
            }
        }
    	else 
        {
            this.changeAnimationProp(isEnable);
        }
    };
    
    this.printJSONData = function()
    {
        if(ZDBGlobal.troubleshooting)
        {
            console.log(JSON.stringify(data));
        }
    };
    
    this.getChart = function()
    {//for debugging
    	if(ZDBGlobal.troubleshooting)
		{
    		return chart;
		}
		return null;
    }
    
    this.destroy = function()
    {
    	if(chart)
    		{
    			chart.destroy();
    		}
    }
    
    this.setModified = function(status)
    {
        modified = status;
    };
    
    this.isModified = function()
    {
        return modified;
    };
    
    this.getXYValue = function(seriesPos, itemPos, subSeriesPos)
    {
        return getChartXYValue.call(this,chart, seriesPos, itemPos, subSeriesPos);
    };
    
    this.handleChartEvents = function(enable)
    {
        chart.systemConf.chart.plot.events.enabled = enable;
    	var ptrEvts = enable ? null : "none"; //No I18N
    	if (chart.userdata.geo && chart.userdata.geo.isGeoChart) 
    	{    // Geo chart               
    		chart.container.style("pointer-events", ptrEvts); //No I18N        
    	} 
    	else 
    	{                   
    	    if (chart.isAxisCategory || chart.dataObject.isPolarAxisCategory) 
    	    {                      
    	    	chart.eventHandler.trackerElement.style("pointer-events", ptrEvts); //No I18N             
    	    }                   
    	    chart.svg.style("pointer-events", ptrEvts); //No I18N       
    	}
    };
    
    this.isSizePresent = function()
    {
        return !data.metadata.axes.z ? false : true;
    };
    
    this.isHorizontalChart = function()
    {
        return data.chart.axes ? data.chart.axes.rotated : "auto"; //NO i18N
    };
        
    this.clickOnChart = function(pos)
    {
        var view = this.getViewObj();
        var eventEl = {} ;
        if(pos && pos.element)
        {
            eventEl = pos.element;
        }
        else
        {
            var data = view.chartObj.getData();
            eventEl =  isGeoChart ? view.chartObj.getChartType()==25 ? d3.select(chartContainer._ZCMap.getPanes().overlayPane).select("canvas").node() : d3.select(chartContainer._ZCMap.getPanes().overlayPane).select(".chartsvg").node() :  chart.staticPaneSVG.select(".eventhandler").node(); // No I18N
        }
        var containerXY = ZDBUtil.getElementAbsolutePos(chartContainer);
        var opts = {};
        opts.clientX = containerXY.x + pos.x;
        opts.clientY = containerXY.y + pos.y;
        ZDBDomQa.click(eventEl, null, opts, true);
    };
    
    this.redrawNewChartType = function()
    {
        this.getChartObject();
        this.changeChartDraw(!ZRAnimationUtil.notInVisibleArea(view));
        if(isGeoChart)
        {
            this.renderChart(true);
        }
        else
        {
            this.animateChart();
        }
        this.printJSONData();
    };
    
    this.getCurrentChartPltType = function()
    {
    	return data.ZRCLRPALETTE.TYPE;
    };
    
    //Copied from zc.js ---START  -- IMPORTANT
    this.getChartObject = function() 
    {
    	this.printJSONData();
    	if(!document.body.contains(chartContainer)){console.log("No container"); return;}
        var dataObj = d3.select(chartContainer)
            .datum(data); 
        /*
         * Each chart instance is instantiated with chartRendered / breakOnErr flag false
         * chart.initialize is defined in chart-constructor.js
         * */
        dataObj.each(function(data) {
            var chartHolder = d3.select(this);
            chart = new $ZC.charts.get(chartHolder,data); // get chartobj
            chart.showLoading(chartHolder);
        });
    }
    //Copied from zc.js ---END  -- IMPORTANT
    
    this.createChart = function()
    {
        if(this.isError())
        {
            this.showErrorMsg();
            return;
        }
        if(chart === null)
        {
            this.setJSChartDimensions();
            var isGeoChart = data.geo && data.geo.isGeoChart;
            if(!isGeoChart && !ZRAnimationUtil.notInVisibleArea(view))
            {
            	this.getChartObject();
            }
        }
        var dashView = view.viewCtx.dashView;
        if(dashView && dashView.isViewRendering && !isGeoChart)
        {
            var isChartRen = setTimeout(function()
	            {
	                view.chartObj.createChart();
	            }, 30);
            return; // only init for chart alone
        }
        if(!document.body.contains(chartContainer))
        {
            var tabView = view;
            if(dashView)
            {
                tabView =  dashView;
                dashView.isViewRendering = false;
                dashView.animatingViewWaiting--;
            }
            var func = function(){ZDBAnalysisView.createChartCall(view.chartObj)};
            if(!ZDBTabSwitchHandler.isRelevendTab2Exec(tabView,func))
            {
                return;
            }
        }
        if(dashView)
        {
            dashView.isViewRendering = true;
        }
        this.changeChartDraw(!ZRAnimationUtil.notInVisibleArea(view), dashView);
        ZCFormatUtil.setTholdLabelWidth(data, null, tholdLabel, chartContainer);
        if(chart === null)
        {
        	this.renderChart(isGeoChart);
        }
        else
        {
        	this.animateChart();
        }
        if(chart != null && dashView)
        {
            var times = setInterval((function(){
                if(this.chartRendered())
                {
                    view.viewCtx.dashView.isViewRendering = false;
                    view.viewCtx.dashView.animatingViewWaiting--;
                    clearInterval(times);
                }
            }).bind(this), 30);
        }
        
    };
    
    this.animateChart = function()
    {
        if(chart && !this.chartRendered())
        {
            chart.draw();
        }
    };

    this.renderChart = function(isGeo)
    {
        chart = isGeo ? $ZC.mapcharts(chartContainer,data) : $ZC.charts(chartContainer,data);
        if(isGeo)
        {
            chartContainer._ZCMapTileLayer.on("tileerror",function() //NO I18N
            {
                    this.failedTiles = this.failedTiles ? this.failedTiles + 1 : 1;
            });          
            chartContainer._ZCMapTileLayer.on("load",function() //NO I18N
            {                          
                    view.tilesLoaded = true; // Used for qa.
                    var endTime = new Date().getTime();                    
                    this.completTimeTaken = endTime-this.startTime;
                    var additionalParams = new Object();
                    additionalParams.failedTiles  = this.failedTiles;            
                    if(typeof ZRUserAnalyticsObject != "undefined" && ZRUserAnalyticsObject instanceof Object)
                    {
                        ZRUserAnalyticsObject.pushUsageToServer("tileloading", "mapchart", "jscharts",true,additionalParams,this.completTimeTaken);//NO I18N            
                    }
            },view);
            // to handle map control in nodata chart when unknown data is present. Map control has to be shown only in edit mode.
            var view =this.getViewObj();
            view.nochartdata=chart.nochartdata;
            view.hasUnknownGeoData=chart.hasUnknownData;
            var nodataClass = view.nochartdata ? (view.hasUnknownGeoData ? "unknownDataChart" :"NoDataChart"):"";//NO I18N
            jQuery(view.analysisArea).addClass(nodataClass);
            
            if(view.nochartdata)
            {
                if(view.hasUnknownGeoData)
                {
                    jQuery(view.analysisArea).addClass("unknownDataChart");
                }
                else
                {
                    jQuery(view.analysisArea).removeClass("unknownDataChart");
                }
                jQuery(view.analysisArea).addClass("NoDataChart");
            }
            else
            {
                jQuery(view.analysisArea).removeClass("unknownDataChart").removeClass("NoDataChart");
            }
            
            if(this.chartRendered() && (!(chart.nochartdata && !chart.hasUnknownData)))
            {                    
                var zrmap = new ZRMap();
                zrmap.initMapUI(chart);
                zrmap.addUIOptions(chart);
            }
        }
        chart.systemConf.chart.plot.animation.enabled = true;
        // this property will always be there
    };

    this.redrawPossible = function()
    {
        return false;
        var view = this.getViewObj();
        var allowedGraphs = view.chartSelectedEl.allowedGraphs;
        var specialCase = null;
        if(allowedGraphs.indexOf("PIE") >=0 && allowedGraphs.indexOf("COMBO") >=0)
        {// when PIE and COMBO mutually exclusive chart are allowed - when x and color axis are same config
            specialCase = true;
        }
        
        // switching between Stacked chart type
//      var isSplit = !view.metaInfo.MERGE;
        var isStackedBar = view.graphSelected.indexOf("SBR") >= 0 || view.subGraphType.indexOf("SBR") >= 0;
        var isPrevStackedBar = view.oldGraphSelected.indexOf("SBR") >= 0 || (view.oldSubGraphType && view.oldSubGraphType.indexOf("SBR") >= 0);
        isStackedBar = isStackedBar || isPrevStackedBar;
        
        //switching between table chart
        var isNotTableChart = view.graphSelected != "TABLE_CHART" && view.oldGraphSelected != "TABLE_CHART"; // No I18N
        var isNotWebChart = view.graphSelected != "WEB" && view.oldGraphSelected != "WEB"; // No I18N
        
        // others are overlap, plotAsImage, during error case - trying to regenerate
        return !this.isError() && isNotTableChart && isNotWebChart && !isStackedBar && !specialCase && !view.overlap && !view.plotAsImage;
    };
    
    this.legendFilterAll = function(e, selected, chartObj)
    {
    	if(!chartObj.dataObject.data.legend.filter.enabled )
        {
            return;
        }
    	var isPieLeg = legendModel instanceof ZCPieLegendModel;
    	view.isNoLegendSelected = !selected;
        var legendEls = legendModel.getLegendEls();
        var legendCritInfo = view.legendCritInfo;
        if(!legendCritInfo)
        {
            legendCritInfo = [];
            view.legendCritInfo = legendCritInfo;
        }
        if(!legendCritInfo || selected)
        {
            legendCritInfo.length = 0
            view.legendCritInfo = legendCritInfo;
            //removing or initializing legendCriteria
        }
        var dataNotInClient = false;
        if(view.tholdFunc) {
            this.getParamsForDynThold(view);
        }
        for(var i = 0; i<legendEls.length; i++)
    	{
            var curLegEl = legendEls[i];
            var legObj;
            if(view.tholdFunc)
            {
                legObj = {"legId": view.chartSelectedEl.isMultiY ? curLegEl.index : curLegEl.instId, // NO I18N
                    "value": curLegEl.filterValue, "origval": curLegEl.origValue, // NO I18N
                    "subtype": view.legendCritInfo.subType, "operation": view.legendCritInfo.operation}; //NO I18N
            }
            else
            {
                legObj = {"legId": view.chartSelectedEl.isMultiY ? curLegEl.index : curLegEl.instId, // NO I18N
                                        "value": curLegEl.filterValue}; // NO I18N
            }
            if(!selected)
            {
                legendCritInfo[legendCritInfo.length] = legObj;
            //add the legend criteria
            }
            legendEls[i]["filtered"] = !selected;
            if(!legendEls[i]["filtered"] && !dataNotInClient)
            { // anyone legend item - dataNotInClient - then send request to server
    			if(!isPieLeg)
    			{
    	            dataNotInClient = this.getChartData()[i].data[0].length === 0;
				}
                // data values to get from ZohoCharts library is slightly altered as 2D Array
            }
    	}
    	if(isPieLeg)
		{
            legendModel.updatePercentValueSelectAll(selected);        		
		}
        if(selected && (view.overlap || this.sendReqForLegendFilter(view, dataNotInClient)))
        {
            view.refreshView();
            return;
        }
    	chartObj.legend.component.toggleSelectAll();
    };
    
    this.getParamsForDynThold = function(view)
    {
        var axisDetails = view.axisDetails;
        for(var i=0; i< axisDetails.length; i++)
        {
            if(!axisDetails[i] || axisDetails[i].VIEWTYPE !== "COLORAXIS")
            {
                continue;
            }
            else
            {
                view.legendCritInfo.subType = axisDetails[i]["SUBTYPE"];
                view.legendCritInfo.operation = axisDetails[i]["FILTER_OPERATION"];
                break;
            }
        }
    };
    
    this.legendFilter = function(event, data, idx, chartObj)
    {
        if(data.seriesname === "DY_TH" && !data.filterValue) {
            return;
        }
        if(view.editMode)
        {
            var legendEls = legendModel.getLegendEls();
            var curLegEl = legendEls[idx];
            ZDBLegendTools.showLegendMenu(curLegEl, event, view);
            return;
        }
    	else if(!chartObj.dataObject.data.legend.filter.enabled )
        {
            return;
        }
        view.isNoLegendSelected = false;
        var isPieLeg = legendModel instanceof ZCPieLegendModel;
        
        var legendEls = legendModel.getLegendEls();
        var curLegEl = legendEls[idx];
        var excludeCriteria = curLegEl["filtered"];
        if(!excludeCriteria && this.isLastOneInFilter()) {
            return;
        }
        
        curLegEl["filtered"] = excludeCriteria ? false : true;
        var dataNotInClient = false;
        if(!curLegEl["filtered"] && !isPieLeg)
        {
            dataNotInClient = this.getChartData()[idx].data[0].length === 0;
            // data values to get from ZohoCharts library is slightly altered as 2D Array
        }
        var legendCritInfo = view.legendCritInfo;
        if(!legendCritInfo)
        {
            legendCritInfo = [];
            view.legendCritInfo = legendCritInfo;
        }
        var islegfiltered  = false;
        var chcklegObj;
        if(view.tholdFunc)
        {
            this.getParamsForDynThold(view);
            chcklegObj = {"legId": view.chartSelectedEl.isMultiY ? curLegEl.index : curLegEl.instId, //NO I18N
                            "value": curLegEl.filterValue, "origval": curLegEl.origValue, //NO I18N
                            "subtype": view.legendCritInfo.subType, "operation": view.legendCritInfo.operation}; //NO I18N
        }
        else
        {
            chcklegObj = {"legId": view.chartSelectedEl.isMultiY ? curLegEl.index : curLegEl.instId, //NO I18N
                            "value": curLegEl.filterValue, "origval": curLegEl.origValue}; //NO I18N
        }
        for(var i=0;i< legendCritInfo.length;i++ )
        {
            var legObj = legendCritInfo[i];
            if(legObj["legId"] === chcklegObj["legId"] &&  legObj["value"] === chcklegObj["value"] )
            {
                islegfiltered = true ;
                legendCritInfo.splice(i, 1);
                //remove legend filter criteria
                break ;
            }
        }
        if(!islegfiltered)
        {
            legendCritInfo[legendCritInfo.length] = chcklegObj;
            //add legendfilter Criteria
        }
        if(isPieLeg)
        {
        	legendModel.updatePercentValue(idx);
        }
        if(this.sendReqForLegendFilter(view, dataNotInClient))
        {
            view.refreshView();
            return;
        }
        
        var parEl = jQuery(view.chartArea).find(".legseries")[0];
        var legEl = jQuery(parEl).find("li")[idx];
        chartObj.legend.component.toggleItemNUpdateChart(legEl, data, idx, chartObj.systemConf.legend);
    };
  
    this.sendReqForLegendFilter = function(view, dataNotInClient)
    {
        return (view.plotAsImage || view.tholdFunc || dataNotInClient);
    };
    
    this.isLastOneInFilter = function()
    {
        var legendEls = legendModel.getLegendEls();
        for(var i=0, j=0,len = legendEls.length; i<len; i++)
        {
            var newLegend = legendEls[i];
            if(!newLegend["filtered"])
            {
                j++;
            }
        }
        return (j === 1);
    };
    
    //if overriding points, need to refresh whenever chart type is changed / leg filter / resize
    this.setOverridePoints = function()
    {
        if(data.metadata.overlap)
        {
            view.overlap = true;
        }
    };
    
    // if chart is rendered as image, need to refresh whenever chart type is changed
    this.setPlotAsImage = function()
    {
        if(data.chart.plot.renderer && data.chart.plot.renderer.mode === "image")
        {
            view.plotAsImage = true;
        }
    };
    
    // if function is selected for threshold, need to refresh on legend filter
    this.setThresholdInfo = function()
    {
        if(data.metadata.tholdFunc)
        {
            view.tholdFunc = true;
        }
        view.nopercell = data.metadata.nopercell ? true : false;
        if(data.notes && data.notes.blockValues && data.notes.blockValues[0].vAlign === "top"
                            && tholdLabel === null) {
            tholdLabel = data.notes.blockValues[0].htmlEl;
        }
    };
    
    this.setLegendPositionForMobileView = function()
    {
    	var legendData = this.getData().legend;
    	if((ZDBClientUI.fs != "ZC") && BROWSER.IS_MOBILE_PHONE && legendData.filter && legendData.filter.enabled)
    	{
            if(window.orientation == 90 || window.orientation == -90)
            {
                legendData.hAlign = "right";//NO I18N
                legendData.vAlign = "center";//NO I18N
                data.legend.maxHeight = "75%"; // NO I18N
            }
            else
            {
                legendData.hAlign = "center";//NO I18N
                legendData.vAlign = "bottom";//NO I18N
                data.legend.maxHeight = "15%"; //NO I18N
            }
    	}
    };
    
    this.createLegendModel = function()
    {
        var chartData = this.getChartData();
        if(chartType === ZCHARTCONST.get('PIE') || chartType === ZCHARTCONST.get('FUNNEL'))
        {
            return new ZCPieLegendModel(this);
        }
        else if(data.metadata.axes && data.metadata.axes.clr && data.legend)
        {
            return new ZCColorLegendModel(this);
        }
        else if(!view.chartSelectedEl.isMultiY && !data.metadata.axes.clr && data.seriesdata && data.seriesdata.DYN_THRESHOLD_SERIES)
        {
            return null;
        }
        else if(chartData && chartData.length > 1)
        {
            return new ZCMultiYLegendModel(this);
        }
        return null;
    };
    
    this.getLegendModel = function()
    {
        return legendModel;
    };
    
    this.getLegendEls = function()
    {
        if(legendModel)
        {
            return legendModel.getLegendEls();
        }
        return null;
    };
    
    this.getChartType = function()
    {
        return chartType;
    };
    
    // to show show threshold tooltip content, on mouse hover threshold line
    this.customTooltip = function(event, metaInfo, chartObj)
    {
        var chartData = data.seriesdata.chartdata[metaInfo.seriesIndex];
        if(chartData.seriesname === "DY_TH" && chartData.tholdLabel)
        {
            chartObj.dataObject.data.tooltip.footerContentVal = '';
            var yIdx = chartData.fmtColIdx || data.metadata.axes.y[0][0];
            var toldToolTip = [ZDBEscapeUtil.encodeForHtml(chartData.tholdLabel) + " : {{val("+yIdx+")}}"]; //No I18N
            return chartObj.eventHandler.tooltipElement.getTooltipContent(metaInfo.seriesIndex,
                            metaInfo.subseriesIndex, metaInfo.itemIndex,null,metaInfo.actData, toldToolTip);
           
        }
        else
        {
            chartObj.dataObject.data.tooltip.footerContentVal = tooltipFooter;
        }
        return chartObj.eventHandler.tooltipElement.getTooltipContent(metaInfo.seriesIndex,
                            metaInfo.subseriesIndex, metaInfo.itemIndex,null,metaInfo.actData, tooltip);
    };
    
    // to display vud/drill down message in the tool tip
    this.setClickMsg = function()
    {
        var clickText = this.getClickText();
        var overlapDataIndex = ( data && data.metadata && data.metadata.overlapDataIndex) 
                                ? data.metadata.overlapDataIndex : -1;
        if(!data.tooltip)
        {
            data.tooltip = {};
            data.tooltip.content = this.customTooltip;
        }
        if(clickText !== null)
        {
            data.tooltip.footerContentVal = "<span style='line-height: 25px; font-size: 11px;'>" + clickText + "</span><br>";
        }
        else if(overlapDataIndex !== -1)
        {
            data.tooltip.footerContentVal = '';
        }
        else 
        {
            data.tooltip.footerContent = null;
        }
        tooltipFooter = data.tooltip.footerContentVal;
        tooltip = data.metadata.axes.tooltip;
    };
    
    this.getClickText = function()
    {
        var clickText = null;
        var forMobile = (BROWSER.IS_MOBILE) ? "formobile" : "";
        if(!view.editMode && view.permissions.drilldown && (view.drillDet.isNextDrillAllowed   || ((ZDBGlobal.isDynDrillEnabled) && view.drillDet.isChartDrillable && view.drillDet.isClrDynDrillable)))
        {
            clickText = ZDBI18NMap.g("zrpt.common.vudtip.js.lbltip" + forMobile);
            if(view.permissions.vud) {
                clickText = ZDBI18NMap.g("zrpt.common.vudtip.js.drilltip" + forMobile);
            }
        }
        else if(view.permissions.vud)
        {
            clickText = ZDBI18NMap.g("zrpt.common.vudtip.js.datatip" + forMobile);
        }
        return clickText;
    };

    this.setDrillEvents = function()
    {
        var axes = data.chart.axes;
        var evnthandle = BROWSER.IS_MOBILE ? "TAP" : "CLICK";
        	if(this.hasDrillDown())
        	{
        		 if(axes.xaxis.ticklabel.criteria)
                 {
                     this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                     evnthandle, "lblClickJS");//No I18N
                     this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                     'MOUSEOUT', function(e, d) { //No I18N
                                                      this.setTickLabelStyle(e, false);
                                                      }.bind(this));
                     this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                     'MOUSEMOVE', function(e, d) {//No I18N
                                                      this.setTickLabelStyle(e, true);
                                                     }.bind(this));
                 }
                 else if(axes.yaxis[0].ticklabel.criteria)
                 {
                     this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                     evnthandle, "lblClickJS"); //No I18N
                     this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                     'MOUSEOUT', function(e, d) { //No I18N
                                                         this.setTickLabelStyle(e, false);
                                                     }.bind(this));
                     this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                     'MOUSEMOVE', function(e, d) { //No I18N
                                                         this.setTickLabelStyle(e, true);
                                                     }.bind(this));
                 }
        	}
        	else if(view.drillDet.isChartDrillable &&( view.permissions.vud == 1 || (view.permissions.drilldown === 1 && view.drillDet.isXYDynDrillable)))
            {
        	 if(view.DrillPath.axisLabel === "X")
        	 {
        	 this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                    evnthandle, "lblClickJS"); //NO I18N
             this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                    'MOUSEOUT', function(e, d) { //No I18N
                        this.setTickLabelStyle(e, false);
                    }.bind(this));
              this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                    'MOUSEMOVE', function(e, d) { //No I18N
                        this.setTickLabelStyle(e, true);
                    }.bind(this));
        	 }
        	 else if(view.DrillPath.axisLabel === "Y")
             {
        		 if(!axes.rotated)
        		 {
        		 this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                         evnthandle, "lblClickJS"); //NO I18N
                  this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                         'MOUSEOUT', function(e, d) { //No I18N
                             this.setTickLabelStyle(e, false);
                         }.bind(this));
                   this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                         'MOUSEMOVE', function(e, d) { //No I18N
                             this.setTickLabelStyle(e, true);
                         }.bind(this));
        		 }
        		 else
        		 {
        			 this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
        	                    evnthandle, "lblClickJS"); //NO I18N
        	             this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
        	                    'MOUSEOUT', function(e, d) { //No I18N
        	                        this.setTickLabelStyle(e, false);
        	                    }.bind(this));
        	              this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
        	                    'MOUSEMOVE', function(e, d) { //No I18N
        	                        this.setTickLabelStyle(e, true);
        	                    }.bind(this));
 
        		 }

             }
        	}
    };
    
    this.setTickLblCursorStyle = function()
    {
        if(this.hasDrillDown())
        {
                var axes = data.chart.axes;
                if(axes.xaxis.ticklabel.criteria)
                {
                    this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                'CURSOR', "");
                }
                else if(axes.yaxis[0].ticklabel.criteria)
                {
                    this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                                                'CURSOR', "");
                }
        }
        else if(ZDBGlobal.isDynDrillEnabled === true && view.drillDet.isChartDrillable &&( view.permissions.vud == 1 || (view.permissions.drilldown === 1 && view.drillDet.isXYDynDrillable)))
        {
        	 if(view.DrillPath.axisLabel === "X" )
        	 {
               this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                    'CURSOR', ""); //No I18N
        	 }
        	 else if(view.DrillPath.axisLabel === "Y")
        	 {
        		 var axes = data.chart.axes;
        		 if(!axes.rotated)
        		 {

        		 this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'YAXIS', 'ZERO', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                         'CURSOR', ""); //No I18N
        		 }
        		 else
        		 {
        		  this.handleJSONObjVal(data, new Array('CHART_VAL', 'AXES_VAL', 'XAXIS', 'TICKLABEL_VAL', 'EVENTS_VAL'),
                             'CURSOR', ""); //No I18N

        		 }

        	 }
        }

    };
    
    this.setTickLabelStyle = function(event, showUnderLine)
    {
       // Removed to Support VUD in tick Label Functionality.
        /*if(view.editMode) {
            showUnderLine = false;
        }*/
        var el = ZDBUtil.getSourceElement(event);
        el.style.textDecoration = showUnderLine ? "underline" : ""; //No I18N
    };

    // in edit mode, do not show tick mark in legend
    this.setLegendTickMark = function()
    {
        if(data.legend && data.legend.colorBox && data.legend.filter.enabled)
        {
            if(view.editMode)
            {
                data.legend.colorBox.text = "";
                if(!data.legend.selectAllBox)
            	{
                	data.legend.selectAllBox = {};
            	}
                data.legend.selectAllBox["show"] = false;
            }
            else
            {
                data.legend.colorBox.text = "&#10003;";
                if(!data.legend.selectAllBox)
            	{
                	data.legend.selectAllBox = {};
            	}
                data.legend.selectAllBox["show"] = true;
            }
        }
    };

    this.hasDrillDown = function()
    {
        var axes = data.chart.axes;
        if(!axes)
        {
            return false;
        }
        return (axes.xaxis && axes.xaxis.ticklabel && axes.xaxis.ticklabel.criteria ||
                axes.yaxis && axes.yaxis[0].ticklabel && axes.yaxis[0].ticklabel.criteria);
    };
    
    this.getCriteria = function()
    {
        var cr = data.chart.axes.x.ticklabel.criteria;
        if(!cr) {
            cr = data.chart.axes.y[0].ticklabel.criteria;
        }
        return cr;
    };

    this.getColorIdx = function()
    {
        return data.metadata.axes.clr ? data.metadata.axes.clr[0] : null;
    };

    this.getColor = function(idx)
    {
        return chart.seriesColor[idx];
    };
    
    this.resetErrorMsgContainer = function()
    {
        chartContainer.style.paddingLeft = 0 + "px";//No I18N
        chartContainer.style.paddingTop = 0 + "px";//No I18N
    };
        
    this.showErrorMsg = function()
    {
        this.resetErrorMsgContainer();
        this.setJSChartDimensions();
        chartContainer.innerHTML = data["error"];
        chartContainer.style.paddingLeft = ((chartContainer.offsetWidth / 2) -100 ) + "px";//No I18N
        chartContainer.style.paddingTop = ((chartContainer.offsetHeight / 2) -10) + "px";//No I18N
        chartContainer.style.color = "red";//No I18N
    };
    
    this.setConfigData = function(newDataJSON)
    {
        data = newDataJSON;
    };
    
    this.setChartContainer = function(newContainer)
    {
        chartContainer = newContainer;
    };
    
    this.isSizeModified = function()
    {
        var prevWidth = chartContainer.offsetWidth;
        var prevHeight = chartContainer.offsetHeight;
        this.setJSChartDimensions();
        ZCFormatUtil.setTholdLabelWidth(data, null, tholdLabel, chartContainer);
        return !(prevWidth == chartContainer.offsetWidth && prevHeight == chartContainer.offsetHeight);
    };
    
    this.isError = function()
    {
        return data["error"] != null;
    };
    
    this.redrawZohoCharts = function(isDataMod)
    {
        if(chart == null)
        {
            this.createChart();
        }
        else
        {
            if(this.isError())
            {
                this.showErrorMsg();
                return;
            }
            if(this.isSizeModified() || isDataMod)
            {
                this.initialize();
                chart.redraw();
            }
        }
        
    };
    
    this.setJSChartDimensions = function()
    {
        // just to fit the content - as of now fix ----
        if($("ChartViewHolder"))
        {
            $("ChartViewHolder").style.left = "0px";
        }
        this.setChartContainerDim();
        this.setContainerBorderIfNeeded();
    };
    
    this.setContainerBorderIfNeeded = function()
    {
        var chartArea = jQuery(view.chartArea);
        isGeoChart ? chartArea.addClass("mapChartArea") : chartArea.removeClass("mapChartArea")
    }
    this.setChartContainerDim = function()
    {
        var chartWidth = view.workArea.clientWidth;
        var chartHeight = view.workArea.clientHeight;
          // for geo chart - no need of x and y axis setings buttons - so removing that extra space.
        if(view.editMode)
        {
             chartHeight -=isGeoChart ? 5 :25;
             chartWidth -=isGeoChart ? 5 :30;
        }
        //just to fit the content - as of now fix ----
        chartContainer.style.width = chartWidth+"px"; //No I18N
        chartContainer.style.height= chartHeight+"px"; //No I18N
        if(chartWidth * .4 < 300)
    	{
        	if(data.tooltip)
    		{
        		data.tooltip.maxWidth = 300;
    		}
        	else
    		{
        		data.tooltip = {"maxWidth": 300};
    		}
    	}
    };
    
    this.setLegendModel = function()
    {
        if(data.legend)
        {
            legendModel = this.createLegendModel();
            if(legendModel)
            {
                legendModel.init();
                JSChartEventHandler.setLegendOnClickEvent(data, this);
            }
        }   
    };
    
    this.changeChartType = function()
    {
        var graphType = view.graphSelected;
        
        if(graphType === "COMBO")
        {
            var modified = legendModel.makeChangesForComboChart();
            chartType = ZCHARTCONST.get(graphType);
            var subGraphType = "ST_" + view.subGraphType; //No I18N
            subChartType = ZCHARTCONST.get(subGraphType);
            if(modified)
            {
                this.setModified(true);
            }
        }
        else
        {
            this.setRelatedChartTypeJSON();
            this.setRelatedSubGraphTypeJSON();
        }
        this.updateGeochartProp();
        this.setLegendModel();
        if(this.isModified())
        {
            var graphType = ZCHARTCONST.get(view.graphSelected);
            var isGeoChart = view.graphSelected.match(/^(MAPSCATTER|MAPBUBBLE|MAPAREA|MAPPIE|MAPPIEBUBBLE)$/);
            if(isGeoChart && !data.geo)
            {
                ZDBAnalysisView.setModifyAndRefreshView(view);
            }
            else
            {
                if(isGeoChart)
                {
                    this.renderChart(true);
                }
                else
                {
                	this.redrawNewChartType();
                }
                this.printJSONData();
                this.changeMetaInfo();
                ZDBAnalysisView.selectOrEnableGraphTypes(view);
            }           
        }
    };
    
    this.changeMetaInfo = function()
    {
        view.chartSelectedEl.selectedGraph = view.graphSelected;
        view.chartSelectedEl.subGraphType = view.subGraphType;
    };
    
    this.setRelatedChartTypeJSON = function()
    {
        var graphType = view.graphSelected;
        if(!(data && data.seriesdata && data.seriesdata.chartdata) || chartType === ZCHARTCONST.get(graphType))
        {// if there is no data / same chart type
            return;
        }
        if(!ZCHARTCONST.get(graphType))
        {
            data["error"] = "not a defined chart type"; //No I18N
            this.showErrorMsg();
            return;
        }
        chartType = ZCHARTCONST.get(graphType);
//        this.bubbleChartTypeRelatedChanges();
        this.modifyDataSeries(chartType);
        this.pieRelatedChanges();
        this.setModified(true);
    };
    
    this.modifyDataSeries = function(chartTypeParam)
    {
        var series = data.seriesdata.chartdata;
        seriesCount = 0;
        for(var i in series)
        {
            data.seriesdata.chartdata[i]["type"] = parseInt(chartTypeParam);
            seriesCount++;
        }
    };

    this.updateGeochartProp = function()
    {
        var isGeoChart = view.graphSelected.match(/^(MAPSCATTER|MAPBUBBLE|MAPAREA|MAPPIE|MAPPIEBUBBLE)$/);;
        if(isGeoChart)
        {
            data.geo && (data.geo.isGeoChart = true);
        }
        else 
        {
            data.geo && (data.geo.isGeoChart = false);
        }
    };
    
    this.setRelatedSubGraphTypeJSON = function()
    {
        var subGraphType = "ST_" + view.subGraphType; //No I18N
        if(subChartType === ZCHARTCONST.get(subGraphType))
        {
            return;
        }
        if(!ZCHARTCONST.get(subGraphType))
        {
            data["error"] = "not a defined sub-chart type"; //No I18N
            this.showErrorMsg();
            return;
        }
        var prevSubGphType = subChartType;
        subChartType = ZCHARTCONST.get(subGraphType);
        this.funnelChartTypeRelatedChanges(prevSubGphType);
        this.modeRelatedChanges();
        this.markerRelatedChanges();
        this.stackedWebRelatedChanges();
        this.setModified(true);
    };
    
    this.pieRelatedChanges = function()
    {
        if(ZCHARTCONST.get('PIE') === chartType && subChartType !== ZCHARTCONST.get('ST_FUNNEL'))
        {
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'PIE_VAL', 'DATALABELS', 'INNERLABEL'), 'SHOW', true);//No I18N
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'PIE_VAL', 'DATALABELS', 'INNERLABEL'), 'SHOWAS', this.pieDataLabelChanges());//No I18N
        }
    };
    
    this.funnelRelatedChanges = function()
    {
        if(ZCHARTCONST.get('PIE') === chartType && subChartType === ZCHARTCONST.get('ST_FUNNEL'))
        {
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'FUNNEL_VAL', 'DATALABELS', 'INNERLABEL'), 'SHOW', true);//No I18N
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'FUNNEL_VAL', 'DATALABELS', 'INNERLABEL'), 'SHOWAS', this.pieDataLabelChanges());//No I18N
        }
    };
    
    this.funnelChartTypeRelatedChanges = function(prevSubGphType)
    {
            if(subChartType === ZCHARTCONST.get('ST_FUNNEL'))
            {
                this.modifyDataSeries(ZCHARTCONST.get('FUNNEL'));
                this.funnelRelatedChanges();
                this.handleJSONObjVal(data, new Array('NOTES'), 'ENABLED', true); //No I18N
            }
            else
            {
                this.handleJSONObjVal(data, new Array('NOTES'), 'ENABLED', false); //No I18N
            }
            if(prevSubGphType === ZCHARTCONST.get('ST_FUNNEL') && chartType === ZCHARTCONST.get('PIE'))
            {
                this.modifyDataSeries(ZCHARTCONST.get('PIE'));
                this.pieRelatedChanges();
            }
            if(subChartType === ZCHARTCONST.get("ST_RING"))
            {
                this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'PIE_VAL'), 'INNERRADIUS', "65%"); //No I18N
            }
            else
            {
                this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'PIE_VAL'), 'INNERRADIUS'); //No I18N
            }
    };
    
//    this.bubbleChartTypeRelatedChanges = function()
//    {
//            var subGraphType = "ST_" + view.subGraphType; //No I18N
//            if(ZCHARTCONST.get(subGraphType) === ZCHARTCONST.get('ST_BUBBLE'))
//            {
//                chartType = chartType === ZCHARTCONST.get('MAPSCATTER') ? ZCHARTCONST.get('MAPBUBBLE') : ZCHARTCONST.get('BUBBLE');                
//            }
//    };
    
    this.pieDataLabelChanges = function()
    {
        var propSetUtil = view.propSetUtil;
        var dataLabelProp = propSetUtil.getPropValue(PropSettings.CHART_DEFAULT_ITEMLABEL_PROP,
        					PropSettings.STATIC_PROP, PropSettings.DOUBLE_PROPTYPE);
        //dataLabelProp will result as a char / string val
        var yCurrentList = ZDBUtil.getElementsWithAttribute(view.currentList, "viewType","YAXIS");//No I18N
        var isMultiY = yCurrentList.length > 0;
        var formatIDX = isMultiY ? 2 : 1;
        if(!dataLabelProp)
        {
            return "{{per(1)}}"; //No I18N
        }
        switch(parseInt(dataLabelProp))
        {
            case 1:
                return "{{per(1)}}"; //No I18N
            case 2:
                return "{{val("+formatIDX+")}}"; //No I18N
            case 3:
                return "{{per(1)}} ({{val("+formatIDX+")}})"; //No I18N
            default:
                return null;
        }
    };
    
    this.modeRelatedChanges = function()
    {
        var isLineSpline = subChartType === ZCHARTCONST.get('ST_SPLINE') || subChartType === ZCHARTCONST.get('ST_SPLINEWPOINT');
        var isAreaSpline = subChartType === ZCHARTCONST.get('ST_SPAREA') ||  subChartType === ZCHARTCONST.get('ST_SPAREAWPOINT')
                					|| subChartType === ZCHARTCONST.get('ST_STSPAREA') || subChartType === ZCHARTCONST.get('ST_STSPAREAWPOINT');
        var isStep = subChartType === ZCHARTCONST.get('ST_STEP');   
        if(isLineSpline)
        {
			this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'LINE_VAL'), 'MODE', ZCHARTCONST.get('MONOTONE'));//No I18N
        }
        else if(isAreaSpline)
        {
			this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'AREA_VAL'), 'MODE', ZCHARTCONST.get('MONOTONE'));//No I18N
        }
        else if(isStep)
        {
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'LINE_VAL'), 'MODE', ZCHARTCONST.get('STEP_AFTER'));//No I18N
        }
        else
        {
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'LINE_VAL'), 'MODE');//No I18N
        }
    };
    
    this.markerRelatedChanges = function()
    {
        if(subChartType === ZCHARTCONST.get('ST_STEP'))
        {
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'LINE_VAL', 'MARKER'), 'ENABLED', false);//No I18N
        }
        else if(chartType === ZCHARTCONST.get('LINE'))
        {
            var withOutPoints = subChartType === ZCHARTCONST.get('ST_WPOINT') || subChartType === ZCHARTCONST.get('ST_SPLINEWPOINT');
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'LINE_VAL', 'MARKER'), 'ENABLED', !withOutPoints);//No I18N
        }
        else if(chartType == ZCHARTCONST.get('AREA') || chartType == ZCHARTCONST.get('STACKED_AREA'))
        {
            var withPoints = subChartType == ZCHARTCONST.get('ST_POINT') || subChartType == ZCHARTCONST.get('ST_SPAREAWPOINT') || subChartType == ZCHARTCONST.get('ST_STSPAREAWPOINT');
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'AREA_VAL', 'MARKER'), 'ENABLED', withPoints);//No I18N
        }
    };
    
    this.stackedWebRelatedChanges = function()
    {
        if(subChartType == ZCHARTCONST.get('ST_WFILL'))
        {
            this.handleJSONObjVal(data, new Array('CHART_VAL', 'PLOT_VAL', 'PLOTOPTIONS_VAL', 'WEB_VAL'), 'FILLOPACITY', 0);//No I18N
        }
    };
    
    this.handleJSONObjVal = function(curData, arr, key, value)
    {// adding and deleting JSON Obj is done in single method
        if(!curData)
        {
            return;
        }
        if(!arr || arr.length === 0)
        {
            if(value == null)
            {
                if(curData[ZCHARTCONST.get(key)])
                {// if value is NULL, then the property is to be deleted
                    delete curData[ZCHARTCONST.get(key)];
                }
                else
                {
                    //Do Nothing 
                    //No addition / deletion
                }
            }
            else
            {
                curData[ZCHARTCONST.get(key)] = value;
            }
        }
        else
        {
            var jsonKey = ZCHARTCONST.get(arr.shift());
            if(!curData[jsonKey])
            {
                curData[jsonKey] = new Object();
            }
            curData = curData[jsonKey];
            this.handleJSONObjVal(curData, arr, key, value);
        }
    };
}


var ZCHARTCONST = (function () {
    var constants = {
// CHART TYPE CONSTANTS         
            'CONTINGENCT_TYPE' : 0, //No I18N 
            'XYSYMBOLIC_TYPE' : 1, //No I18N
            'XY_TYPE' : 2, //No I18N
            'CATEGORY_CHART' : 3, //No I18N
            'CAT_PIE' : 4, //No I18N
            'MEA_PIE' : 5, //No I18N
            'MEA_BAR' : 6, //No I18N
            'FUNNEL' : 7, //No I18N
            'PIE' : 8, //No I18N
            'BAR' : 9, //No I18N
            'SBR' : 10, //No I18N
            'LINE' : 11, //No I18N
            'SCATTER' : 12, //No I18N
            'CONTINGENCY' : 13, //No I18N
            'AREA' : 14, //No I18N
            'SAREA' : 15, //No I18N
            'WEB' : 16, //No I18N
            'COMBO' : 17, //No I18N
            'BUBBLE' : 20, //No I18N
            'MAPSCATTER' : 23, //No I18N
            'MAPBUBBLE' : 22, //No I18N
            'MAPAREA' : 25, //No I18N
            'MAPPIE' : 26, //No I18N
            'MAPPIEBUBBLE' : 27, //No I18N
// SUB GRAPH TYPE CONSTANTS
            'ST_DEF' : -1, //No I18N
            'ST_STEP' : 1, //No I18N
            'ST_3D' : 2, //No I18N
            'ST_POINT' : 3, //No I18N
            'ST_WPOINT' : 4, //No I18N
            'ST_WFILL' : 5, //No I18N
            'ST_RING' : 6, //No I18N
            'ST_FUNNEL' : 7, //No I18N
            'ST_STEPPOINT' : 8, //No I18N
            'ST_BAR' : 9, //No I18N
            'ST_SBR' : 10, //No I18N
            'ST_BAR3D' : 11, //No I18N
            'ST_SBR3D' : 12, //No I18N
            'ST_SPLINE' : 13, //No I18N
            'ST_PYRAMID' : 13, //No I18N
            'ST_SPLINEWPOINT' : 14,//No I18N
            'ST_SPAREA' : 15, //No I18N
            'ST_SPAREAWPOINT' : 16,//No I18N
            'ST_STSPAREA' : 17, //No I18N
            'ST_STSPAREAWPOINT' :18, //No I18N
            'ST_BUBBLE' :19, //No I18N            
            //
// Value constants
            'CHART_VAL' : 'chart', //No I18N
            'PLOT_VAL' : 'plot', //No I18N
            'PLOTOPTIONS_VAL' : 'plotoptions', //No I18N
            'AXES_VAL' : 'axes', //No I18N
            'XAXIS' : 'xaxis', //No I18N
            'YAXIS' : 'yaxis', //No I18N
            'LEGEND_VAL' : 'legend', //No I18N
            'SELECTALLBOX' : 'selectAllBox', //No I18N
            'TICKLABEL_VAL' : 'ticklabel', //No I18N
            'EVENTS_VAL' : 'events', //No I18N
            'TAP' : 'tap', //No I18N
            'CLICK' : 'click', //No I18N
            'MOUSEOUT' : 'mouseout', //No I18N
            'MOUSEMOVE' : 'mousemove', //No I18N
            'LINE_VAL' : 'line', //No I18N
            'PIE_VAL' : 'pie', //No I18N
            'FUNNEL_VAL' : 'funnel', //No I18N
            'PYRAMID_VAL' : 'pyramid', //No I18N
            'NOTES' : 'notes', //No I18N
            'AREA_VAL' : 'area', //No I18N
            'WEB_VAL' : 'web', //No I18N
            'ENABLED' : 'enabled', //No I18N
            'MODE' : 'mode', //No I18N
            'MONOTONE' : 'monotone', //No I18N
            'STEPAFTER' : 'stepafter', //No I18N
            'STEP_AFTER' : 'step-after', //No I18N
            'MARKER' : 'marker', //No I18N
            'DATALABELS' : 'datalabels', //No I18N
            'INNERLABEL' : 'innerLabel', //No I18N
            'INNERRADIUS' : 'innerRadius', //No I18N
            'SHOW' : 'show', //No I18N
            'SHOWAS' : 'showAs', //No I18N
            'FILLOPACITY' : 'fillOpacity', //No I18N
            'ZERO' : '0', //No I18N
            'CURSOR': 'cursor', //No I18N
    };
    
    return {
        get:function(name) {
            return constants[name];
        }
    };
})();
