//$Id$

var ZDBEditors = new function()
{
    var ctrlPressed = false;
    var keyCode = 0;
    var charCode = 0;
    
    
    this.callGridMove = function(cellObj, keyCode, shiftPressed, ctrlPressed, altPressed, cmdPressed)
    {
        var gridObj = cellObj.colObj.parGrid;
        gridObj.isEdit = false;
        return ZDBKeyEvents.moveFocus(gridObj, cellObj, keyCode, shiftPressed, ctrlPressed, altPressed, cmdPressed);    
    }
     
    //////////////// List Editor
    this.listEditor = function(cellObj)
    {
      var list = ZDBEditors.getEditField('listField');
      list.cellObj = cellObj;
      if(cellObj.colObj.parGrid.type == "TableDesign")
      {
        if(ZDBDesignGrid.isFormulaField(cellObj) || ZDBDesignGrid.isANCType(cellObj))
        { 
          return;
        }
      }
      ZDBEditors.configureListEditor(cellObj,list);
      ZDBEditors.showListEditor(cellObj,list);
      ZDBEditors.setEditOriginal(list);
    }
    
    this.showListEditor = function(cellObj,list)
    {
      var selIndex = list.selectedIndex;
      list.selectedIndex = selIndex;
      ZDBUtil.setTimeout(ZDBUtil.setFocus, null, [list], ZDBGlobal.wait);
    }

    this.setEditOriginal = function(editField)
    {
        editField.editOriginal = ZDBEditors.getValue(editField);
    }
    
    this.getValue = function(editField)
    {
        switch(editField["type"].toLowerCase())
        {
            case "text":
                    {
                        return editField.value;                    
                    }
            case "select-one":
                    {
                        
                        return editField.selectedIndex < 0 ? "" : editField[editField.selectedIndex].value;
                    }
        }
        return "";
    }
    
    this.storeOriginal = function(cellObj)
    {
        if(cellObj.editBackup === null)
        {
         cellObj.editBackup = cellObj.getValue('edit');
        }
    }

    this.restoreOriginal = function(cellObj)
    {
      cellObj.setValue(cellObj.editBackup, null,false, false);
    }

    this.configureListEditor = function(cellObj,list)
    {
        cellObj.rowObj.parGrid.isEdit = true;
        if(list.onkeydown == null)
        {
          list.onkeydown = function(event){return ZDBEditors.onListKeyDown(this,event);};
          list.onkeypress = function(event){return ZDBEditors.onListKeyPress(this,event);};
          list.onchange = function(event){return ZDBEditors.onListItemChange(this,event);};
          list.onblur = function(event){return ZDBEditors.onListBlur(this,event);};
        }
        ZDBEditors.storeOriginal(cellObj);
        list.cellObj = cellObj;
        ZDBEditors.setEditStyle(list, cellObj);
        list.style.lineHeight = list.style.height;
        list.style.position = "absolute"; ///PK: move to class
        var emptyOption = ZDBEditors.getEditField("emptyOption");	 //No I18N
        var colObj = cellObj.colObj;
        if(colObj.type === "FKCOL")
        {
        	list.className = "normalFont12";
        }
        if(colObj.isNullable && colObj.type != "FKCOL")
        {
          ///PK: Define type "FKCOL" for lookup 
          ///PK: Check future list elements
          list.appendChild(emptyOption);
        }
        else if(list.lastChild == emptyOption)
        {
          $("zdbListHolder").appendChild(emptyOption);
        }
        
        if(cellObj.originalValue === ZDBConst.GEO.toString() || cellObj.originalValue === ZDBConst.GEO_NUM.toString())
        {            
            var value = cellObj.rowObj.values[list.gridObj.getColumnIndex("GEOOPT")].originalValue;
            ZDBEditors.selectGeoCurrentOption(list,value);
        }
        else            
        {
        ZDBEditors.selectCurrentOption(list,cellObj.getValue());
        }

    }

    this.onListItemChange = function(listField, event)
    {
        var cellObj = listField.cellObj;
        if(listField.onBlurCustomFunc != null)
        {
          listField.onBlurCustomFunc(event);
        }
        else
        {
          var listObj = listField[listField.selectedIndex];
          if(listObj != null)
          {
              cellObj.oldDispValue = cellObj.displayValue;            
              ZDBEditors.updateOptDetailsIfNeeded(cellObj,listObj);
              cellObj.setValue(listObj.value,listObj.innerHTML, false, false);
          }
        }
        if(cellObj.rowObj.parGrid.type == 'TableDesign')
        {
          ZDBDesignGrid.invokeOptionDlg(listField);
        }
    }

    this.updateOptDetailsIfNeeded = function(cellObj,listObj)
    {
        if(cellObj && cellObj.rowObj.parGrid.type == 'TableDesign' && cellObj.colObj.type=="DATATYPE" && (listObj.value==ZDBConst.GEO || listObj.value==ZDBConst.GEO_NUM))
        {
            var geoOptcellObj = cellObj.rowObj.values[cellObj.rowObj.parGrid.getColumnIndex("GEOOPT")];
            geoOptcellObj.oldgeoRole = geoOptcellObj.originalValue;
            var georole = listObj.getAttribute("georole");                  
            geoOptcellObj.originalValue = georole;
        }        
    }
    this.onListKeyDown = function(listField, event)
    {
        var cellObj = listField.cellObj;
        var gridObj = cellObj.rowObj.parGrid;
        var ev = ZDBUtil.getGridKeyUpEvent(event);
        keyCode = ZDBUtil.getKeyCode(event);
        switch(keyCode)
        {
            case KEY.ENTER:
            case KEY.TAB:
            case KEY.ESC:
            case KEY.LEFTARROW:
            case KEY.RIGHTARROW:
                    listField.blur();
                    ZDBEditors.callGridMove(cellObj, keyCode, ev.shiftPressed, ev.ctrlPressed, ev.altPressed, ev.cmdPressed);
                    ZDBUtil.stopEventBubble(event);
                    return ZDBConst.BROWSER_HANDLE_NO;
        }
        keyCode = 0;
        return ZDBConst.BROWSER_HANDLE_YES;
    }
    
    this.onListKeyPress = function(listField, event)
    {
        var cellObj = listField.cellObj;
        if(listField.onBlurCustomFunc != null)
        {
          listField.onBlurCustomFunc(event);
        }
        else
        {
          var selectedIndex = listField.selectedIndex;           
          if(selectedIndex >=0)
          {
              var listObj = listField[selectedIndex];
              cellObj.setValue(listObj.value,listObj.innerHTML, false, false);
          }
        }
        return true;
    }
    
    this.onListBlur = function(listField, event)
    {
      var cellObj = listField.cellObj;
      var original = cellObj.original;
      var gridObj = cellObj.rowObj.parGrid;
      gridObj.isEdit = false;
     
      if(listField.selectedIndex < 0)
      {
        ZDBEditors.restoreOriginal(cellObj);
      }
      else
      {
        if(listField.onBlurCustomFunc != null)
        {
          listField.onBlurCustomFunc(event);
        }
        else
        {
          var listObj = listField[listField.selectedIndex];
          if(listObj != null)
          {
            cellObj.setValue(listObj.value,ZDBEscapeUtil.decodeForHtml(listObj.innerHTML),true, true, listField.clearError);

          }
        }
      }
      ZDBEditors.hideElements([listField]);
      return true;
    }


        //////Date Editor
    
     this.dateEditor = function(cellObj)
      {
        var gridObj = cellObj.rowObj.parGrid;
        gridObj.editorActive = "dateInput"; //No I18N
        var dateField = ZDBEditors.getEditField("dateInput"); //No I18N
        var dateButton = ZDBEditors.getEditField("dateButton"); //No I18N
        ZDBEditors.storeOriginal(cellObj);
        dateField.cellObj = cellObj;
        dateField.gridObj = gridObj;
        dateButton.gridObj = gridObj;
        ///PK dateField.value = cell.original.data;
        dateField.value = cellObj.getValue('edit'); //No I18N
        ZDBEditors.setEditStyle(dateField, cellObj);
        ZDBEditors.setAndShowDateIcon(dateButton, cellObj);
        //To be moved - check Calendar.show();
        var format = "%d %b, %Y %H:%M:%S.%f";//No I18N
        var showTime = true;
        
        var dateFormat = cellObj.colObj.dateFormat;
        if(dateFormat)
        {
          //User date format not applied for the date format with time zone 
          // and date format without day
          if(!(ZDBUtil.containsAnyValue(dateFormat,["Z","MILLI"])) && ZDBUtil.containsAllValues(dateFormat,["dd"]))
          {
            format = ZDBUtil.calendarDateFormat(dateFormat);
            cellObj.colObj.userDateFmt = true;
          }
        }

        Calendar.setup(
        {
            inputField  :   "dateInput",      // id of the input field //No I18N
            ifFormat    :   format,       // format of the input field
            showsTime   :   showTime,            // will display a time selector
            button      :   "dateButton",   // trigger for the calendar (button ID) //No I18N
            singleClick :   true,           // double-click mode
            date        :   dateField.value, 
            weekNumbers :   false,
            range       :   [1000,9999],
            step        :   1,                // show all years in drop-down boxes (instead of every other year as default)
            onClose     :   function(){this.hide(); this.destroy(); ZDBUtil.setTimeout(ZDBUtil.setFocus, null, [dateField], ZDBGlobal.wait);}
        }
        );
          ZDBUtil.attachEvent($("dateButton"),"click",ZDBEditors.delHelpIcon,false);
        ///PK: Remove the try catch
        try
        {
          ZDBEditors.setEditorFocus(cellObj, dateField);
        }
        catch(e)
        {
            // Catch IE error dialogs, if any!
        }
      }  

        this.onDateButtonMouseDown = function(dateButton, event) // PK
        {
            var dateField = ZDBEditors.getEditField("dateInput"); //No I18N
            dateField.buttonClick = true;
            return true;
        }

        this.delHelpIcon = function()
        {
          var tableEl = window.calendar.table;
          var tr = tableEl.firstChild.firstChild;   //Select <tr> el
          var tdEls = tr.getElementsByTagName("td");
          var titleEl = tdEls[1];
          if(titleEl && titleEl.className == "title")
          {
            titleEl.setAttribute("colSpan",6);
          }
          if(tdEls[0] && tdEls[0].firstChild.innerHTML == "?")
          {
            tr.removeChild(tdEls[0]);                 //Removing '?' element
          }
        }
        
        this.onDropDownBtnPress = function(clickStatus)
        {
          var inputField = ZDBEditors.getEditField("singleLineCharInput"); //No I18N
          var sbObj = inputField.sbObj;
          if(sbObj)
          {
            sbObj.isIconClick = clickStatus;
            sbObj.inputEl.focus();
          }
          return true;
          
        }
     
      this.onDateFocus = function(dateField, event)
      {
        dateField.gridObj.isEdit = true;
        dateField.buttonClick = false;
      }
     
      this.onDateBlur = function(dateField, event)
      {
        if(dateField.buttonClick)
        {
            dateField.buttonClick = false;
            return false;            
        }
        var data = dateField.value;
        var gridObj = dateField.gridObj;
        data = trimAll(data);
        var cellObj = dateField.cellObj;
        var dateButton = ZDBEditors.getEditField("dateButton");  //No I18N
        ZDBEditors.hideElements([dateButton, dateField]);
        var colObj = cellObj.colObj;
        gridObj.setHighlightRowCol(cellObj,false);
        gridObj.isEdit = false;
        var childIndex = 0;
        if(cellObj.cellEl)
        	{
        	cellObj.setValue(data, null, true, true);
        var tempRow=cellObj.cellEl.parentNode;
  	  while( (tempRow.previousSibling) != null ) 
  		  {
  		  tempRow=tempRow.previousSibling;
  	    childIndex++;
  		  }
  	  gridObj.adjustRowHeight(childIndex);
  	  if(gridObj.dArea.scrollTop == 0)
  		  {
  	  gridObj.setFillerHeight(true);
  		  }
        	}
		    return true;
      }


  this.getEditField = function(editorId)
  {
    if(this[editorId] == null)
    { 
      this[editorId] = document.getElementById(editorId);          
    }
    return this[editorId];
  }
  
  this.setEditStyle = function(editorField, cellObj)
  {
    var colObj = cellObj.colObj;
    var rowObj = cellObj.rowObj;
    var gridObj = colObj.parGrid;
    var cellElPos = gridObj.getCellElPosInGrid(cellObj);
    var parentArea = colObj.freeze ? gridObj.rhArea : gridObj.dArea; 
    if(parentArea != editorField.parentNode)
    {
        parentArea.appendChild(editorField);
    }
    var diff = 4;
    var dS = editorField.style;
    dS.left = cellElPos.cellLeft + "px"; 
    dS.top = cellElPos.cellTop  + "px";
    if(editorField.type == 'select-one')
    {
        dS.width = colObj.width + "px";
        dS.height = rowObj.height + "px";
    }
    else
    {
        dS.width = (colObj.width - diff) + "px";
        dS.height = (rowObj.height - diff)+ "px";
    }
    dS.display = "block";
  }
  
  this.setDropDownStyle = function(el,cellObj)
  {
    var colObj = cellObj.colObj;
    var gridObj = colObj.parGrid;
    var parentArea = colObj.freeze ? gridObj.rhArea : gridObj.dArea;
    var curCellPos = gridObj.getCellElPosInGrid(cellObj);
    if(parentArea != el.parentNode)
    {
        parentArea.appendChild(el);
    }
    var elStyle = el.style;
    elStyle.left = curCellPos.cellLeft + colObj.width - 20 + "px"; //width of the drop down icon
    elStyle.top = curCellPos.cellTop  + "px";
    elStyle.display = "block";
  }

  
  this.setEditorWidth = function(editorField, cellObj)
  {
    var colObj = cellObj.colObj;
    var rowObj = cellObj.rowObj;
    var gridObj = colObj.parGrid;
    var minWidth = colObj.width;
    var maxWidth = (colObj.freeze ? gridObj.tcWidth : (gridObj.chWidth + gridObj.chArea.scrollLeft)) - colObj.left - 10; //PK: 10 grace width  
    var value = editorField.value;
    var dummyDIV = ZDBGlobal.dummyDIV;
    dummyDIV.innerHTML = "";
    var dummyDIVstyle = dummyDIV.style;
    dummyDIV.innerHTML = (ZDBEscapeUtil.encodeForHtml(value)).replace(/\n/g,"<BR>") + "PK"; /// "PK" two more character //No I18N
    var dWidth = dummyDIV.offsetWidth;
    if(dWidth > maxWidth)
    {
      dWidth = maxWidth;
    }
    if(dWidth < minWidth)
    {
      dWidth = minWidth-3;
    }
    editorField.style.width = dWidth + "px";
    if(colObj.type == 'MULTI_LINE')
    {
      var dHeight = dummyDIV.offsetHeight;
      var minHeight = rowObj.height;
      var maxHeight = (gridObj.rhHeight + gridObj.rhArea.scrollTop) - rowObj.top - 10;
      if(dHeight > maxHeight)
      {
        dHeight = maxHeight;
      } 
      if(dHeight < minHeight)
      {
        dHeight = minHeight;
      }
      editorField.style.height = dHeight + "px";
    }
    if(colObj.type == 'REFCOL')
    {
      var dropDownEl = ZDBEditors.getEditField('LCDropDown'); //No I18N
      dropDownEl.style.left = colObj.left + dWidth - 20 + "px"; //width of the drop down icon
    }

  } 
  

   this.setInputHeight = function(editorField, cellObj)
   {
      var height = parseInt(editorField.style.height,10);
      if(!height)
      {
        height = cellObj.rowObj.height;
      }
      editorField.style.height = height + "px";
   }



  this.hideElements = function(elList) // ZDBUtil ????
  {
    try{
      if(!elList)
      {
          return;
      }
      else if(!elList.length)
      {
          elList.style.display = 'none';
          document.body.appendChild(elList);
      }
      else
      {
          var len = elList.length;
          for(var k=0;k<len;++k)
          {
              elList[k].style.display = 'none';
              document.body.appendChild(elList[k]);
          }
      }
    }
    catch(excp){
      //Error occurred in append child.
    }
  }
    
    this.setAndShowDateIcon = function(dateIcon, cellObj)
    {

      var gridObj = cellObj.colObj.parGrid;
      var rowObj = cellObj.rowObj;
      var colObj = cellObj.colObj;
      var cellPos = gridObj.getCellElPosInGrid(cellObj);
      var left = 0;
      if(colObj.freeze)
      {
        gridObj.rhArea.appendChild(dateIcon);
        left = cellPos.cellLeft + colObj.width + 2;
      }
      else
      {
        gridObj.dArea.appendChild(dateIcon);
        left = cellPos.cellLeft;//colObj.left;
        var lastCol = ZDBDataGrid.getLastVisibleColumn(gridObj.colHeaders, gridObj.colHeaders.length-1, false);
        if(lastCol == colObj && (left + colObj.width + 10) >= gridObj.chWidth)
        {
          left -= 20;
        }
        else
        {
          left += colObj.width;
        }
      }
      var dS = dateIcon.style;
      var top =  cellPos.cellTop;//rowObj.top; 
      dS.position = "absolute";
      dS.left = left + "px";
      dS.top = top + "px";
      dS.display = "block";
    }





  this.selectCurrentOption = function(listObj,curValue)
  {
    for(var i = 0; i < listObj.length; i++)
    {
      if(listObj[i].value == curValue)
      {
        listObj[i].selected = true;
        listObj.zdbIndex = listObj.selectedIndex = i;
        return;
      }
    }
    listObj.selectedIndex = -1;
  }
  this.selectGeoCurrentOption = function(listObj,curValue)
  {
    for(var i = 0; i < listObj.length; i++)
    {
      if(listObj[i].getAttribute("georole")=== curValue)
      {
        listObj[i].selected = true;
        listObj.zdbIndex = listObj.selectedIndex = i;
        return;
      }
    }
    listObj.selectedIndex = -1;
  }


  this.setEditor = function(cellObj, inputID, data)
  {
      var editorField = ZDBEditors.getEditField(inputID);
      var cellEl = cellObj.cellEl;
      editorField.cellObj = cellObj;
      cellObj.colObj.parGrid.isEdit = true;
      cellObj.colObj.parGrid.editorActive = inputID;
      ZDBEditors.setEditStyle(editorField, cellObj);
      ZDBEditors.storeOriginal(cellObj);
      ZDBEditors.setEditorFocus(cellObj, editorField);
      if(data)
      {          
        var sb = new ZDBSuggestionBox(editorField, data , true, false, false, true);
        var dropDownEl = ZDBEditors.getEditField('LCDropDown'); //No I18N
        ZDBEditors.setDropDownStyle(dropDownEl, cellObj);
      }
      else
      {
        editorField.sbObj = null;
      }
  }
  
  this.getAutoListData = function(cellObj)
  {
    //PK: Temp
    
    var colObj = cellObj.colObj;
    var gridObj = colObj.parGrid;
    var rowData = gridObj.rowData;
    var rowLen = rowData.length;
    var list = [];
    var curRow, cellObjs, cellObj, cellLen, curValue;
    for(var i=0;i<rowLen;++i)
    {
      if(rowData[i].objType != ZDBConst.DATA)
      {
        continue;
      }
      cellObjs = rowData[i].values;
      cellLen = cellObjs.length;

      for(j=0;j<cellLen;++j)
      {
        cellObj = cellObjs[j];
        if(cellObj.colObj == colObj)
        {
          curValue = cellObj.displayValue;
          if(curValue && ZDBUtil.indexOf(list, curValue, null, true) < 0)
          {
            list.push(curValue);
          }
          break;
        }
      }
    }
    return list;
  }
    
  this.setEditorFocus = function(cellObj, editorField)
  {
    var value = cellObj.getValue('edit'); //No I18N
    editorField.value = value;
    ZDBEditors.setEditOriginal(editorField);
    ZDBEditors.setEditorWidth(editorField, cellObj);
    ZDBUtil.setTimeout(ZDBUtil.setFocus, null, [editorField],5);
    ZDBUtil.setTimeout(ZDBUtil.setCaretToEnd, null, [editorField],5);
  }

      this.onTextAreaKeyDown = function(editorField, event)
      {
        var cellObj = editorField.cellObj;
        var ev = ZDBUtil.getGridKeyUpEvent(event);
        var gridObj = cellObj.colObj.parGrid;
        keyCode = ZDBUtil.getKeyCode(event);  
        if(keyCode == KEY.ENTER)
        {
          var value = editorField.value;
          if(editorField.id != "multiLineInput")
        	  {
          cellObj.setValue(value,null, true, true);
        	  }
          return ZDBConst.BROWSER_HANDLE_YES;
        }  
        else
        {
          ZDBEditors.onEditorKeyDown(editorField, event);
        }
      }


    this.onEditorKeyDown = function(editorField, event)
    {       
      var cellObj = editorField.cellObj;
      var gridObj = cellObj.colObj.parGrid;
      var cell = cellObj.cellEl;
      var ev = ZDBUtil.getGridKeyUpEvent(event);
      keyCode = ZDBUtil.getKeyCode(event);  
      if(ev.altPressed || ev.ctrlPressed || ev.cmdPressed) 
      {
          return ZDBKeyEvents.onKeyDown(event);
      }
      
      var sbObj = editorField.sbObj;
      if(sbObj)
      {
        switch(ZDBUtil.getKeyCode(event))
        {
          case KEY.UPARROW:
            return ZDBConst.BROSWER_HANDLE_YES;
          case KEY.DOWNARROW:
            return ZDBConst.BROSWER_HANDLE_YES;
        }
      }
      
      switch(keyCode)
      {
          case KEY.LEFTARROW:
          case KEY.RIGHTARROW:
                {
                  return ZDBConst.BROSWER_HANDLE_YES;
                }
          case KEY.DOWNARROW:
          case KEY.UPARROW:
                {
                  if(editorField.tagName.toLowerCase() == 'textarea')
                  {
                      ///PK: For any text area - up/down arrow works as usual i.e., no cell up/down
                      return ZDBConst.BROSWER_HANDLE_YES;
                  }
                }
          case KEY.ENTER:
          case KEY.TAB:              
                  ZDBSuggestionBox.suggestBlur(event); //PK: Hack
                  editorField.blur();
                  return ZDBEditors.callGridMove(cellObj, keyCode, ev.shiftPressed, ev.ctrlPressed, ev.altPressed, ev.cmdPressed);
          case KEY.ESC:
                  editorField.value = cellObj.editBackup;
                  editorField.blur();
                  event.stopPropagation();  //In Design View, editing a data cell and pressing esc closes the dialog
                  return ZDBEditors.callGridMove(cellObj, keyCode, ev.shiftPressed, ev.ctrlPressed, ev.altPressed, ev.cmdPressed);
          case KEY.BACKSPACE:
                  ZDBEditors.setEditorWidth(editorField, cellObj);
                  return ZDBConst.BROSWER_HANDLE_YES;
          case KEY.DEL:
                  ZDBEditors.setEditorWidth(editorField, cellObj);
                  return ZDBConst.BROSWER_HANDLE_YES;
     }
     return ZDBConst.BROSWER_HANDLE_YES;
    }
    
    
      
    this.onEditorKeyPress = function(editorField, event)
    {
      var charCode = ZDBUtil.getGridKeyPressEvent(event).charCode;
      var cellObj = editorField.cellObj;
      var cell = cellObj.cellEl;
      if(ZDBUtil.isPrintable(charCode))
      {
          var val = editorField.value + String.fromCharCode(charCode);
          if(editorField.id != "multiLineInput")
        	  {
          cellObj.setValue(val, null, false, false);
        	  }
          if(editorField.value == "")
        	  {
        	  var gridObj = cellObj.colObj.parGrid;
        	  if(gridObj.dArea.scrollTop == 0)
        		  {
        	  gridObj.setFillerHeight(true);
        		  }
        	  }
      }
      return ZDBKeyEvents.onKeyPress(event);
    }

     this.onEditorKeyUp = function(editorField, event)
     {
        var keycode = ZDBUtil.getKeyCode(event);
        var sbObj = editorField.sbObj;
        if(sbObj)
        {
          switch(keycode)
          {
            case KEY.SHIFT:
                break; ///PK???????
            case KEY.UPARROW:
            case KEY.DOWNARROW:
              return ZDBConst.BROWSER_HANDLE_NO;
            case KEY.BACKSPACE:
            case KEY.DEL:
              sbObj.requestSuggestions(false);
              break;
            default:
              sbObj.requestSuggestions(true);
          }
        }
        var cellObj = editorField.cellObj;
        ZDBEditors.setEditorWidth(editorField, cellObj);
        return ZDBConst.BROWSER_HANDLE_YES;
     }


    this.onEditorBlur = function(editorField, event)
    {
      var cellObj = editorField.cellObj;
      var colObj = cellObj.colObj;
      var data = editorField.value;
      var typeObj = colObj.typeObj;
      var gridObj = colObj.parGrid;
      if(event) // in one case (deleteEfficientyly) oneditorblur is triggered explicitely where there will be no event 
    	  {
      if(ZDBSuggestionBox.suggestBlur(event) == false)
      {
        return ZDBConst.BROWSER_HANDLE_YES;
      }
    	  }
      gridObj.editorActive = undefined;
      if(colObj.type.toUpperCase() == 'DATE' || colObj.type.toUpperCase() == 'DATE_AS_DATE')
      {
          return ZDBEditors.onDateBlur(editorField, event);
      }
      else if (typeObj.editorType && typeObj.editorType.toLowerCase() == 'list')
      {
          return ZDBEditors.onListBlur(editorField, event);
      }
      if(!gridObj.isEdit)
    	  {
    	  return;
    	  }
      cellObj.setValue(data,null,true,true);
      var childIndex = 0;
      if(cellObj.cellEl)
    	  {
      var tempRow=cellObj.cellEl.parentNode;
	  while( (tempRow.previousSibling) != null ) 
		  {
		  tempRow=tempRow.previousSibling;
	    childIndex++;
		  }
	  gridObj.adjustRowHeight(childIndex);
	  if(gridObj.dArea.scrollTop == 0)
		  {
	  gridObj.setFillerHeight(true);
		  }
    	  }
      editorField.value = "";
      cellObj.editBackup = null; 
	    gridObj.isEdit = false;
      ZDBEditors.hideElements([editorField]);
      return ZDBConst.BROWSER_HANDLE_YES;
    }
}

//// End of Editors
/// Must Remove unnecessary Fns
var ZDBColumnType = 
  {
    DUMMY:
    {
      dataValFn:function(){return true;},
      valFn:function(){return true;},
      editFn:ZDBUtil.dummyFunc,
      //filterFn:ZDBUtil.dummyFunc,
      isHtml:false
    },

    PLAIN : 
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.plainData,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }},
      //filterFn : ZDBFilterEditors.plainFilterEditor,
      isHtml : true,
      dispVal : "Plain Text",
      typeClass: "plainIcon"
    },
     
    MULTI_LINE : 
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.multilineData,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "multiLineInput");
      }},
      //filterFn : ZDBFilterEditors.plainFilterEditor,
      isHtml : true,
      dispVal : "Multi Line Text",
      typeClass: "multilineIcon"
    },

    EMAIL : 
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.emailData,
      valFn: ZDBValidator.isEmail,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }},
      //filterFn : ZDBFilterEditors.plainFilterEditor,
      isHtml : true,
      dispVal : "E-mail",
      typeClass: "mailIcon"
    },
    
    NUMBER:
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.numberData,
      valFn :  ZDBValidator.isNumber,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }},
      //filterFn : ZDBFilterEditors.numericFilterEditor,
      convertFn : ZDBConvert.trimData,
      typeValue : function(value) { return parseFloat(value); },
      dispVal : "Number",
      typeClass: "numberIcon"
    },

    POSITIVE_NUMBER:
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.positiveData,
      valFn :  ZDBValidator.isPositive,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }},
      //filterFn : ZDBFilterEditors.numericFilterEditor,
      convertFn : ZDBConvert.trimData,
      typeValue : function(value) { return parseFloat(value); },
      dispVal : "Positive Number",
      typeClass: "positiveIcon"
    },

    AUTO_NUMBER:
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.autoNumFn,
      valFn :  ZDBValidator.isPositive,
      editFn :  function(cellObj) {ZDBEditors.setEditor(cellObj, "singleLineCharInput");},
      //filterFn : ZDBFilterEditors.numericFilterEditor,
      convertFn : ZDBConvert.trimData,
      isHtml:true,
      typeValue : function(value) { return parseFloat(value); },
      dispVal : "Auto Number",
      typeClass: "autoNumberIcon"
    },

    DECIMAL_NUMBER:
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.decimalData,
      valFn :  ZDBValidator.isDecimal,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");}
      },
      //filterFn : ZDBFilterEditors.numericFilterEditor,
      convertFn : ZDBConvert.trimData,
      typeValue : function(value) { return parseFloat(value); },
      dispVal : "Decimal Number",
      typeClass: "decimalIcon"
    },
    CURRENCY:
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.currencyData,      
      valFn :  ZDBValidator.isCurrency,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }},
      //filterFn : ZDBFilterEditors.numericFilterEditor,
      convertFn : ZDBConvert.toCurrency,
      typeValue : function(value) { return parseFloat(value); },
      dispVal : "Currency",
      typeClass: "currencyIcon"
    },

    PERCENT:
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.percentData,
      valFn :  ZDBValidator.isPercent,
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }},
      //filterFn : ZDBFilterEditors.numericFilterEditor,
      convertFn : ZDBConvert.toPercent,
      typeValue : function(value) { return parseFloat(value); },
      dispVal : "Percent",
      typeClass: "percentageIcon"
      
    },

    DATE : 
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.DateData,
      valFn :  ZDBValidator.isDate,
      editFn :  ZDBEditors.dateEditor,
      //filterFn : ZDBFilterEditors.plainFilterEditor,
      convertFn : ZDBConvert.toDate,
      typeValue : function(value) { return new Date(Date.parse(value)); },
      dispVal : "Date",
      typeClass: "dateIcon"
    },
    
    BOOLEAN:
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn : ZDBDisplay.booleanData,
      editFn :  ZDBEditors.listEditor,
      editorType : "list",
      dispVal : "Decision Box",
      typeClass: "booleanIcon"
    },

    NAVIG : 
    {
      dataValFn: ZDBValidator.dataValFn,
      valFn :  ZDBUtil.dummyFunc, 
      dispFn :ZDBDisplay.UrlData,
      editFn :  ZDBUtil.dummyFunc,
      isHtml : true
    },
    
    URL :   	
    { 	    
      dataValFn: ZDBValidator.dataValFn, 	  
      dispFn: ZDBDisplay.urlData, 	     
      valFn: ZDBValidator.isUrl, 	       
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }},
      //filterFn : ZDBFilterEditors.plainFilterEditor, 	      
      isHtml : true, 	 
      dispVal : "URL", 	 
      typeClass: "urlIcon"  
    },
    
    REFCOL :
    {
      valfunc:null,
      editFn:AllowedValPopup.showRefVals,
      typeValue:function(value, zdbcell){return zdbcell.displayValue;},
      isHtml: true 
    },

    RELCOL :
    {
      valfunc:null,
      editFn:AllowedValPopup.showRefVals,
      typeValue:function(value, zdbcell){return zdbcell.displayValue;},
      isHtml: true 
    },
    
GEO : 
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.plainData,      
      editFn :  function(cellObj) {
      if(cellObj.colObj.lookupType !=undefined && cellObj.colObj.lookupType == ZDBConst.RELCOL)
      {
        AllowedValPopup.showRefVals(cellObj);
      }
      else
      {
        ZDBEditors.setEditor(cellObj, "singleLineCharInput");
      }}, //NO I18N      
      isHtml : true,
      dispVal : "Geo", //NO I18N
      typeClass: "geoIcon" //NO I18N
    },
GEO_NUM : 
    {
      dataValFn: ZDBValidator.dataValFn,
      dispFn: ZDBDisplay.plainData,      
      editFn :  function(cellObj) {ZDBEditors.setEditor(cellObj, "singleLineCharInput");}, //NO I18N      
      isHtml : true,
      dispVal : "Geo", //NO I18N
      typeClass: "geoIcon" //NO I18N
    }
    
  };

 ZDBColumnType.DATE_AS_DATE = ZDBColumnType.DATE
