//$Id$

var ZAChartDrillView = new function()
{
    // display drilldown path in settings dialog
    this.showDrillSettings = function(propSetUtil)
    {
        var view = propSetUtil.view,
            showCyclicErrMsg = true,
            drillPath = view.DrillPath,
            pathLists = view.pathList,
            pathInstances = new Array();
        for(var i=0; i<pathLists.length; i++)
        {
            var pathList = pathLists[i];
            if(pathList)
            {
                pathInstances[i] = DrillPath.getDrillPathInstance();
                pathInstances[i].nodes = new Array();
                pathInstances[i].linkedList = pathList;
                pathInstances[i].field = i;
                pathInstances[i].tblBodyId = DrilldownConstants.DIV_TBODY_IDS[i];
                
                if(i === DrilldownConstants.AXIS_PATH_IDX) 
                {
                    pathInstances[i].label = drillPath.axisLabel;
                    pathInstances[i].displayLabel();
                }
                
                pathInstances[i].origPathList = jQuery.extend(true, {}, pathList);
                pathInstances[i].displayPath(view);
                pathInstances[i].checkAndEnable(propSetUtil, view);
                if(!pathInstances[i].linkedList.freeze) {
                    showCyclicErrMsg = false;
                }
            }
        }
        if(pathInstances.length > 0 && showCyclicErrMsg) {
            jQuery('#cyclicErrorDiv').show(); //No I18N
            ZAChartDrillView.clearErrorMessage(pathInstances);
        }
        view.pathInstances = pathInstances;
        view.pathInstances.chartDimNodes = drillPath.chartDimNodes;
        view.pathInstances.aggNodes = drillPath.aggNodes;
        view.pathInstances.aggFilterNodes = drillPath.aggFilterNodes;
        view.pathInstances.dimFilterNodes = drillPath.dimFilterNodes;
    };
    
    this.clearErrorMessage = function(pathInstances)
    {
        var len = pathInstances.length;
        for(var i=0; i<len; i++) {
            pathInstances[i].setErrorMesg('');
        }
    };
    
    this.resetDrillInfo = function(propSetUtil)
    {
        var view = propSetUtil.view;
        var pathInsts = view.pathInstances;
        if(!pathInsts){return;}
        var origPathLists = new Array();
        for(var i=0; i<pathInsts.length; i++)
        {
            var pathInst = pathInsts[i];
            var origList = null;
            if(pathInst)
            {
                origList = pathInst.origPathList;
                pathInst.removeFrom($(pathInst.tblBodyId), 0);
                if(i === DrilldownConstants.AXIS_PATH_IDX) {
                    pathInst.resetLabel();
                }
                pathInst.linkedList = origList;
            }
            origPathLists.push(origList);
        }
        view.pathList = origPathLists;
        DrilldownConfig.setDrillPathInfo(view);
    };
    
    // send the column ids which you want to hide in the drop down list
    this.getPrevDrillParams = function(el)
    {
        var tdEl = jQuery(el).closest("td")[0];
        ZDBUtil.setNamedChildEls(tdEl, tdEl);
        var compEl = tdEl.compElContainer,
            colId = compEl.getAttribute("column_id"),
            operation = compEl.getAttribute("operation");
        
        var view = ZDBClientUI.ZDBViewEl.view;
        var pathInstances = view.pathInstances;
        var pathIdx = ZAChartDrillView.getSelectedPathIdx(el);
        var pathList = view.pathList[pathIdx];
        var currentNode = pathList.head;
        var prevDrillInfo = ZAChartDrillView.getPrevDrillColInfo(currentNode, colId, operation);
        var colIds = "{\"DATA\":" + JSON.stringify(prevDrillInfo.drillColIds) + "}"; //No I18N
        var params = "&DRILLCOLIDS=" + colIds; //No I18N

        var isMultiTable = pathInstances.chartDimNodes ? true : false;
        if(isMultiTable)
        {
            var selNode = view.pathInstances.selectedNode;
            var selPos = ZAChartDrillView.getSelPosition(pathList, selNode);
            if(selPos === -1) {
                selPos = pathList.size();
            }
            var drillTblIds = ZAChartDrillView.getPrevTableIds(pathList, selPos);
            var tblIds = "{\"DATA\":" + JSON.stringify(drillTblIds) + "}"; //No I18N
            params += "&DRILLTBLIDS=" + tblIds; //No I18N
            
            var chartDimNodes = JSON.parse(pathInstances.chartDimNodes);
            params += "&CHARTDIMNODES=" + "{\"DATA\":" + JSON.stringify(chartDimNodes) + "}"; //No I18N
            
            var aggNodes = JSON.parse(pathInstances.aggNodes);
            params += "&CHARTAGGNODES=" + "{\"DATA\":" + JSON.stringify(aggNodes) + "}"; //No I18N
            
            var aggFilterNodes = JSON.parse(pathInstances.aggFilterNodes);
            params += "&AGGFILTERNODES=" + "{\"DATA\":" + JSON.stringify(aggFilterNodes) + "}"; //No I18N
            
            var dimFilterNodes = JSON.parse(pathInstances.dimFilterNodes);
            params += "&DIMFILTERNODES=" + "{\"DATA\":" + JSON.stringify(dimFilterNodes) + "}"; //No I18N
            
            if(pathList.delNode) {
                params += "&DELTBLOBJID=" + pathList.delNode; //No I18N
            }
        }
        params += ZDBUtil.isNewView(view) ? "&PARENTVIEWID="+view.parentViewId : "&OBJID="+view.objId; //NO I18N
        return params + "&TBLOBJID=" + prevDrillInfo.objId; //No I18N
    };
    
    // get unique table ids from start of path to the node selected by user
    this.getPrevTableIds = function(pathList, selPos)
    {
        var tblIds = new Array();
        var currentNode = pathList.head;
        var i=0;
        while (currentNode && i < selPos)
        {
            var data = currentNode.data;
            var tblId = data[DrilldownConstants.COL_POBJID];
            if(tblIds.indexOf(tblId) < 0) {
                tblIds.push(tblId);
            }
            currentNode = currentNode.next;
            i++;
        }
        return tblIds;
    };
    
    this.getSelPosition = function(pathList, node)
    {
        var pos = 0;
        ZDBUtil.setNamedChildEls(node, node);
        var compEl = node.compElContainer;
        var colId = compEl.getAttribute("column_id");
        if(colId === "") {return -1;}
        var colOpr = compEl.getAttribute("operation");
        var currentNode = pathList.head;
        while (currentNode)
        {
            var data = currentNode.data;
            if(data[DrilldownConstants.COL_ID] === colId &&
                        data[DrilldownConstants.COL_OPER] === colOpr) {
                break;
            }
            pos++;
            currentNode = currentNode.next;
        }
        return pos;
    };
    
    this.getPrevDrillColInfo = function(currentNode, colId, operation)
    {
        var objId = "";
        var drillColIds = new Array();
        var retObj = new Object();
        while (currentNode)
        {
            var data = currentNode.data;
            if(data[DrilldownConstants.COL_ID] === colId &&
                        data[DrilldownConstants.COL_OPER] === operation) {
                break;
            }
            else {
                drillColIds.push(data[DrilldownConstants.COL_ID]);
            }
            objId = data[DrilldownConstants.COL_POBJID];
            currentNode = currentNode.next;
        }
        retObj.drillColIds = drillColIds;
        retObj.objId = objId;
        return retObj;
    };
    
    // display dropdown list on clicking columns in the drill path
    this.fetchDrillColumns = function(el, event)
    {
        jQuery($("chartDrillDiv")).remove();
        var view = ZDBClientUI.getViewObj(el, "view");
        if(view.fetchDrillCols)
        {
            if(!ZDBLazyLoadUtil.loadSnippet("DrillSettings", "/ZDBDrillDown.cc?", this, "APPENDTODOC",
        			["ZDBAnalysisViewList","ChartTPVendors","ZRJSChartsList", "zdbjs/grid/ZDBCommunication.js","ZAAnalysisFields"], //No I18N
                    ZAChartDrillView.fetchDrillColumns, [el, event])) 
            {
                  return;
            }
            urls = ["CSSLIST_GRIDSTYLES", "CSSLIST_ANALYSISVIEWSTYLES", "CSSLIST_VMFILTERSTYLES"];//NO I18N
            if(!ZDBLazyLoadUtil.loadCssOnly(urls, "APPENDTODOC", 
            		ZAChartDrillView.fetchDrillColumns,[el,event])){
            	return;
            }
            if(!ZFontload.preLoadFonts(ZDBGlobal.fontFamilyArr, ZAChartDrillView.fetchDrillColumns,[el,event]))
            {
                return;
            }
            view.fetchDrillCols = false;
            var compElContainer = DOMUtils.getSelfOrParentWithAttrValue(el, "elname", "compElContainer"); //No I18N
            var attrs = [compElContainer, "PLAIN", compElContainer.getAttribute("column_id"), event, true]; //No I18N
            var selNode  = jQuery(compElContainer).parent()[0];
            view.pathInstances.selectedNode = selNode;
            view.pathInstances.selectedPathIdx = ZAChartDrillView.getSelectedPathIdx(selNode);
            DrillDownSettings.fetchNUpdateDrillComboList(compElContainer.getAttribute("objid"),
                                            DrillDownSettings.showComboList, attrs, true, el);
        }
    };
    
    this.getSelectedPathIdx = function(compEl)
    {
        var tBodyId = jQuery(compEl).closest("tbody")[0].id;
        return DrilldownConstants.DIV_TBODY_IDS.indexOf(tBodyId);
    };
    
    // handle drill down check box
    this.handleEnableDrilldown = function(chkBox)
    {
        var view = ZDBClientUI.getViewObj(chkBox, "view");
        var len = view.pathList.length;
        var propVal = chkBox.checked + "";
        for(var i=0; i<len; i++)
        {
            if(view.pathList[i])
            {
                var divTBodyId = view.pathList[i].divTBodyId;
                var pathDiv = jQuery($(divTBodyId)).parent().parent().parent()[0];
                view.pathList[i].disable = !chkBox.checked;
                var checkBox = jQuery(pathDiv).find("input")[0];
                checkBox.checked = chkBox.checked;
                checkBox.setAttribute("propValue", propVal);

                ZAChartDrillView.showOrHideDiv(checkBox.checked, pathDiv);
            }
        }
    };

    // handle drill path check boxes
    this.handleCheckBox = function(chkBox, type)
    {
        var view = ZDBClientUI.getViewObj(chkBox, "view");
        var drillPathDiv = jQuery(chkBox).parent().parent()[0];
        var status = chkBox.checked;
        ZAChartDrillView.showOrHideDiv(chkBox.checked, drillPathDiv);
        var propVal = chkBox.checked + "";
        if(chkBox.checked)
        {
            var otherType = type ^ 1;
            var otherList = view.pathList[otherType];
            if(otherList && otherList.disable) {
                status = false;
            }
        }
        chkBox.setAttribute("propValue", propVal);
        view.pathList[type].disable = !chkBox.checked;
        $("drillCheck").checked = status;
    };

    //modify linked list as per user selection
    this.updateLinkedList = function(list, pos, selData)
    {
        if(pos)
        {
           list.removeFrom(pos);
        }
        if(selData[DrilldownConstants.COL_ID] !== "") {
            list.add(selData);
        }
    };
    
    this.disablePath = function(drillChkBox, propVal)
    {
        drillChkBox.setAttribute("propValue", propVal);
        
        var disableDiv = jQuery(drillChkBox).parent().parent()[0];
        ZAChartDrillView.hideDiv(disableDiv);

        drillChkBox.checked = false;
        $("drillCheck").checked = false;
    };
    
    this.showOrHideDiv = function(selected, pathDiv)
    {
        var childEls = DOMUtils.getChildElsWithAttr(pathDiv, "remove", "y"); //No I18N
        if(selected && childEls.length > 0) {
            pathDiv.removeChild(childEls[0]);
        }
        else if(!selected && childEls.length === 0) {
            ZAChartDrillView.hideDiv(pathDiv);
        }
    };

    this.hideDiv = function(drillPathDiv)
    {
        var divEl = document.createElement("div");
        divEl.className = "dataLblCover";
        divEl.style.marginLeft = 80 + "px"; //No I18N
        divEl.style.width = 700 + "px";
        var height = 5 + jQuery(jQuery(drillPathDiv).find("tbody")[0]).children("tr").length *   //No I18N
                                        DrilldownConstants.ROW_HEIGHT;
        divEl.style.height = height + "px";
        divEl.innerHTML = "&nbsp;"; //No I18N
        divEl.setAttribute("remove", "y");
        drillPathDiv.appendChild(divEl);
    };
    
    // set column details in selected list and drill details in previous list
    this.setColumnDetails = function(listEl, compEl, refTblId, view)
    {
        var colId = listEl.getAttribute("colId");
        var dispName = listEl.getAttribute("listElcolname");
        if(view.multiTable)
        {
            dispName = (colId === "") ? "None" : jQuery(listEl).parent().parent().find("div").attr("tip") + "." + //No I18N
                            dispName;
        }
        compEl.displaySelected.innerHTML = ZDBEscapeUtil.encodeForHtml(dispName);
        compEl.displaySelected.setAttribute("value", ZDBEscapeUtil.encodeForHtmlDispChar(dispName));
        compEl.displaySelected.setAttribute("tip", ZDBEscapeUtil.encodeForHtmlDispChar(dispName));
        
        var colName = listEl.getAttribute("colname");
        var op = listEl.getAttribute("operation");
        var aot = listEl.getAttribute("addOnType");
        ZDBUtil.setNamedChildEls(compEl, compEl);
        var oldColId = compEl.getAttribute("column_id");
        var oldOp = compEl.getAttribute("operation");
        if(oldColId !== colId || oldOp !== op)
        {
            if(view.chartObj) {
                view.chartObj.drillSelected = true;
            }
            view.setModified();
        }
        jQuery(compEl).attr({COLUMN_ID:colId, COLADDONTYPE: aot, OPERATION: op,
                COLDISPNAME: colName, DRILLCOLID:colId, DRILLTBLID:refTblId ? refTblId : "",
                DRILLCOLADDONTYPE:aot, DRILLCOLOPERATION:op, DRILLDISPCOLNAME:colName});


        var prevColumns = jQuery(compEl).closest("td").prev(); //No I18N
        var prevColumn;
        if(prevColumns.length === 0)
        {
            var prevTds = jQuery(jQuery(compEl).closest("tr").prev()[0]).children("td"); //No I18N
            prevColumn = prevTds[DrilldownConstants.NODES_PER_ROW - 1];
        }
        else
        {
            prevColumn = prevColumns[0];
        }
        ZDBUtil.setNamedChildEls(prevColumn, prevColumn);
        var containerEl = prevColumn.compElContainer;
        jQuery(containerEl).attr({DRILLCOLID:colId, DRILLTBLID:refTblId ? refTblId : "",
                    DRILLCOLADDONTYPE:aot, DRILLCOLOPERATION:op, DRILLDISPCOLNAME:colName});
    };

    // get the Last node which has none
    this.getLastNode = function(drillTBody)
    {
        var trEls = drillTBody.getElementsByTagName("tr");
        var lastTr = trEls[trEls.length-1];
        var tdEls = lastTr.getElementsByTagName("td");
        return ZDBUtil.getCloneNode(tdEls[tdEls.length-1]);
    };
};
