//$Id$
map_series_renderer.scatter = function(parent, series, seriesIndx, color, pointDetails, update) {
        
        var dataset = map_util.getDataset();
        var type = $ZC.DataProcessor.helpers.getChartType(series.type, "numeric");//NO I18N
        var scale = map_util.getScale();
        var latLngArray = [];
        var className, markerOpt; 
        var chartObj = map_util.getChartObject();
        var isExport = chartObj.userdata.geo.isExport;
        
        var markerType = map_util.getMarkerType() ? map_util.getMarkerType() : "default" ;//NO I18N
                
        className = 'scatterseries_' + seriesIndx ; //NO I18N
        if (type == geoscatter || type == geofilled) {
            markerOpt = "geoscatter"; //NO I18N
        } else if (type == geobubble) {
            markerOpt = "geobubble"; //NO I18N
        }
        var markerOptions = map_util.getMarkerOptions(markerOpt);
        var extraOptions = {
            "color": color, // series color//NO I18N
            "symbolSize": {//NO I18N
                "inner": 0.5, // Percentage of inner marker size//NO I18N
                "outer": 1 // Percentage of outer marker size//NO I18N
            }
        };
        var points = [];
        
        series.data.forEach(function(subgroup, index) {
            if (subgroup && subgroup.constructor == Array) {
                subgroup.forEach(function(point, index) {
                    var xVal = dataset.getActualX(point)
                    var latLng = dataset.getLatLng(point);
                    if (latLng) {
                        if( point[1] )
                        {
                            latLngArray.push(latLng);
                            points.push(point);
                            pointDetails.push(point);
                            map_util.addPlotPoint({
                                "xVal": xVal,//NO I18N
                                "latLng": latLng,//NO I18N
                                "color":color,//NO I18N
                                "size":map_util.getScale() ? map_util.getScale()(dataset.getZ(point)) : 0//NO I18N
                            });
                        }
                    } else {
                        map_renderer_util.unknowndata.add(xVal);
                    }
                });
            }
        });
        parent.attr("class",className);//NO I18N
        var scatterGroups = parent.selectAll('g').data(points); //NO I18N
        scatterGroups.enter().append('g').attr('class','scatter'); //NO I18N
        scatterGroups.exit().remove();
        scatterGroups
            .each(function(d) {
                if (type == geobubble) {
                    var z = scale(dataset.getZ(d));
                    markerOptions.outerFillOpacity = map_util.getOpacityValue();
                    markerOptions.size = z / 10;
                }
               type != geobubble && markerType != "default" ?  addCustomMarker(d3_select(this),extraOptions,markerType) : addSVGmarker(d3_select(this), null, markerOptions, extraOptions);//NO I18N
        });
        parent.selectAll("g").data(latLngArray);//NO I18N
       
       
      if(!update && !isExport)
      {
          switch(type)
          {
              case geoscatter : markerType != "default"  ? animateMarker(scatterGroups) : animateScatter(scatterGroups);//NO I18N
                                break;
              case geobubble  : animateBubble(scatterGroups); 
                                break;
          }
      }
                
         function animateScatter(scatterGroups)
         {
             var delay = 5;
             var durationTime = scatterGroups[0].length > 100 ? 750 : 1000;
             scatterGroups
                .each(function(d,i){
                     d3_select(this).selectAll("path")//NO I18N
                                       .style("opacity","0.1")//NO I18N
                                       .transition()
                                       .delay(function()
                                        {
                                           return scatterGroups[0].length < 100 ?  delay + ( i == 0 ? (i+1) * 2 : i * 2) : delay = delay + 0.5 ;
                                        })
                                       .duration(durationTime)
                                       .style("opacity","1");//NO I18N
                });
         }
         
         function animateBubble(scatterGroups)
         {
             scatterGroups
                .each(function(d,i){ 
                     d3_select(this).selectAll("path")//NO I18N
                                       .attr("transform","scale(0)")//NO I18N
                                       .transition()
                                       .duration(1000)
                                       .ease("cubic-out")//NO I18N
                                       .attr("transform","scale(1)")//NO I18N
                });
         }
         
         function animateMarker(scatterGroups)
         {
           var delay = 2.5;
           var opacity;
           scatterGroups
                .each(function(d,i){         
                    var translate = d3.transform(d3.select(this).select("path").attr("transform")).translate;//NO I18N
                    d3_select(this)
                               .selectAll('*:not(text)')//NO I18N
                               .style("opacity",function()//NO I18N
                               {
                                   opacity = this.getAttribute("opacity");
                                   return "0";
                               })
                               .attr("transform","translate(" + translate[0] + "," + (translate[1] - 20) + ")")//NO I18N
                               .transition()
                               .delay(function()
                                {
                                    
                                    scatterGroups[0].length < 100 ?  delay = delay + 2.5 : delay = delay + 0.5;
                                       return delay ;
                                })
                               .duration(1000)
                               .ease("bounce")//NO I18N
                               .style("opacity",function()//NO I18N
                                {
                                    return opacity;

                                })
                               .attr("transform","translate(" + translate[0] + "," + translate[1] + ")");  //NO I18N
                });
         }
            
    };