/*$Id$*/

/* 
    Don't invoke the method which is marked as "PRIVATE". Later it will be moved to JS class (ES6)
*/

/*** Components **/

/*** Preloader  **/

// Reports Loading Animation
// Can be used in different sizes: By default, normal will be applied
// Optionally, type can be specified as "xl", "medium", "small" and "xs"
var preloader = new function() {
    var self = this;

    // ***PRIVATE*** Creates the view template and adds to the DOM
    // Reference will be maintained
    self.init = function() {
        if (!self.view) { self.view = dom(ZRSStringTemplateCreator.createTemplateForData('preloader', 'components')); } // No I18N
        dom(document.body).append(self.view);
    };

    // Takes two arguments: container and type. container is the place where we want to show the preloader
    // and type defines the size of the preloader animation
    self.show = function(container, type, text) {
        var loader = self.view.clone(true);
        type && loader.first('.reports-preloader').addClass(type); // No I18N
        !container.first('.reports-preloader') && container.append(loader); // No I18N
        text ? loader.first('.preloader-text').text(text).show() : loader.first('.preloader-text').hide(); // No I18N
        loader.dFadeIn();
    };

    // Takes a argument: container. container is the place where we are showing the preloader
    // Will hide the preloader from the container
    self.hide = function(container) {
        var loader = container.first('.reports-preloader-cont').dFadeOut(); // No I18N
        setTimeout(function() { loader.remove(); }, 1000);
    };
};

/** Sidepane Component **/

// Simply the dialog from the right hand side.
// Usually, it creates two sidepanes: primary and secondary. 
// Primary is the one which will be served every time when the dialog is requested
// Secondary will be come into the picture when the primary dialog is already active. Basically the another layer.

var sidepane = new function() {

    var self = this;

    self.view, self.primary, self.secondary = null;

    // ***PRIVATE***Initializes the view and creates the view template
    // And both primary and secondary views will be added to the DOM
    // References will be maintained
    self.init = function() {
        // Constructs it only if it is not created already
        if (!self.view) { self.view = dom(ZRSStringTemplateCreator.createTemplateForData('sidepane', 'components')); } // No I18N
        self.primary = self.view.clone(true);
        self.secondary = self.view.clone(true);
        self.secondary.first('.sidepane-container').addClass('secondary'); // No I18N

        // Appends to the body
        dom(document.body).append(self.primary);
        dom(document.body).append(self.secondary);
    };

    // Show method takes the JSON argument which comprises of: header1, body1, footer1, header2, body2 and footer2
    // Except header1 and body1, everything is optional
    // header1, body1 and footer1 are the primary dialog contents. 
    // Where header2, body2 and footer2 are the dialog contents which we can use to drill into the dialog
    self.show = function(params) { self.primary.isActive() ? self.set(self.secondary, params) : self.set(self.primary, params); };

    // ***PRIVATE***
    // Just sets the content to both the primary and secondary dialogs.
    // Adds the escape key handler (Dialog will be hidden when the escape key is triggered)
    self.set = function(view, params) {
        params.size && view.addClass(params.size);
        if (params.setOnCompletion) {
            // Shows the preloader
            preloader.show(sidepane.getActivePane().first('.sidepane-container'), null, ZDBI18NMap.g('zrpt.chart.dyndrilldown.js.loading')); // No I18N
            // Shows the view
            view.dFadeIn();

            setTimeout(function() {
                preloader.hide(sidepane.getActivePane().first('.sidepane-container')); // No I18N
                self.constructView(view, params);
            }, 300);
        } else {
            self.constructView(view, params);
            // Shows the view
            view.dFadeIn();
        }
    };

    self.constructView = function(view, params) {
        params.header1 && view.first('[data-header-1]').text(params.header1); // No I18N
        params.body1 && view.first('[data-body-1]').empty().append(params.body1); // No I18N
        params.footer1 ? view.first('[data-footer-1]').empty().append(params.footer1).show() : view.first('[data-footer-1]').hide(); // No I18N
        !params.footer1 && view.first('.sidepane-layer-1 .panel').addClass('no-footer'); // No I18N

        params.header2 && view.first('[data-header-2]').text(params.header2); // No I18N
        params.body2 && view.first('[data-body-2]').empty().append(params.body2); // No I18N
        params.footer2 ? view.first('[data-footer-2]').empty().append(params.footer2).show() : view.first('[data-footer-2]').hide(); // No I18N
        !params.footer2 && view.first('.sidepane-layer-2 .panel').addClass('no-footer'); // No I18N

        if(params.onClose) { self.onClose = params.onClose; }

        // Opt for preventing the form from closing without saving the changes
        if(params.alertOnAccidentClose) {
            self.unsaved = false;
            var inputEls = [];
            params.body1 && inputEls.push(params.body1.find('input'));
            params.body2 && inputEls.push(params.body2.find('input'));
            self.handleInputChange(inputEls);
        } else {
            self.unsaved = false;
        }

        // Adds the help doc to the sidepane
        if (params.helpdoc) {
            var helpIcon = view.first('.QHelpIcon').replaceSelf(); // No I18N
            helpIcon.show().on('click', function() { // No I18N
                window.open(params.helpdoc, '_blank');
            });
        }

        // Adds the sticky header options
        if (params.stickyHeaderOptions) {
            view.first('[data-body-1]').on('scroll', function(event) { // No I18N
                var targetEl = event.currentTarget;
                var stickyEl = params.stickyHeaderOptions.element;
                if (targetEl.scrollTop >= params.stickyHeaderOptions.top) {
                    stickyEl.addClass('sticky-header');
                    stickyEl.parent().toDOM().style.marginTop = '40px'; // No I18N
                } else {
                    stickyEl.removeClass('sticky-header');
                    stickyEl.parent().toDOM().style.marginTop = '0';
                }
            });
        }
        
         // No I18N
        // Adds the escape key handler
        ev.addEscHandle(self.hide);
    };

    self.handleInputChange = function (els) {
        for(var i = 0; i < els.length; i++) { for(var j = 0; j < els[i].length; j++) { dom(els[i][j]).on('change', function() { self.unsaved = true; }); } }
    };

    // Hides the sidepane dialog
    self.hide = function() {
        if(self.unsaved) { self.showConfirmDialog(); return; }
        self.close();
    };

    // Closes the active sidepane.
    self.close = function() {
        (self.secondary && self.secondary.isActive()) ? self.secondary.dFadeOut(): self.primary.dFadeOut();
        self.slideIn();
        self.hideConfirmDialog();
        PushBar.hide();
        self.onClose && self.onClose();
    };

    // Returns the refernece of the active sidepane
    self.getActivePane = function() {
        if (self.secondary && self.secondary.isActive()) { return self.secondary; }
        return self.primary;
    };

    // Drills into the dialog and shows the contents which is passed as "header2", "body2" and "footer2"
    self.slideIn = function() {
        var activePane = self.getActivePane();
        if (!activePane) return;
        var layer1 = activePane.first('[data-layer-1]'); // No I18N
        layer1.toDOM().style.right = 0;
        activePane.first('[data-layer-2]').toDOM().style.right = '-' + layer1.width() + 'px'; // No I18N
    };

    // Drills out from the contents which is passed as "header2", "body2" and "footer2" dialog and shows the dialog content: "header1", "body1" and "footer1"
    self.slideOut = function() {
        var activePane = self.getActivePane();
        var layer1 = activePane.first('[data-layer-1]'); // No I18N
        layer1.toDOM().style.right = layer1.width() + 'px'; // No I18N
        activePane.first('[data-layer-2]').toDOM().style.right = 0; // No I18N

        // Adds the escape key handling
        ev.addEscHandle(self.slideIn);

        // Hides the pushbar if visible
        PushBar.hide();
    };

    self.showConfirmDialog = function(options) {
        var dlg = self.getActivePane().first('.confirm-dlg-cont'); // No I18N
        dlg.first('[data-message]').text(options ? options.message : ZDBI18NMap.g('zr.share.js.unsaved_changes')); // No I18N

        var positiveButton = dlg.first('.greenBtn').replaceSelf(); // No I18N
        positiveButton.on('click', function() { (options && options.onProceed) ? options.onProceed(options.extras) : sidepane.close(); }); // No I18N
        dlg.dFadeIn();
    };

    self.hideConfirmDialog = function() { self.getActivePane().first('.confirm-dlg-cont').dFadeOut(); }; // No I18N
};

/** Tab Component */
// Tab construction made easy on this component.
var tab = new function() {

    var self = this;

    // Tab construction will happen here. Takes a json argument which comprises of: switches and contents
    // Where switches are the array of tab titles
    // Where contents are the array of the body contents as dom object (I mean:: dom object is dom_helper's object)
    self.construct = function(params) {
        if (!self.view) { self.view = dom(ZRSStringTemplateCreator.createTemplateForData('tab', 'components')); } // No I18N
        var tab = self.view.clone(true);
        self.callback = params.callback;
        self.extras = params.extras;
        var switches = tab.first('.tab-switches'); // No I18N
        var contents = tab.first('.tab-contents'); // No I18N
        for (var i = 0; i < params.switches.length; i++) {
            var tabHead = dom(ZRSStringTemplateCreator.createTemplateForData('tabHead', 'components')); // No I18N
            var template = '<li class="enable-transition" onclick="tab.change(event);"></li>';
            var swtch = dom.new('li', { attributes: { class: 'enable-transition' }, text: params.switches[i], events: { click: self.change } }); // No I18N
            switches.append(swtch);
            // Makes the first element as active by default
            if (i == 0) { swtch.addClass('active'); }
        }
        for (var i = 0; i < params.contents.length; i++) {
            var template = '<li class="enable-transition"></li>';
            var content = dom.new('li', { attributes: { class: 'enable-transition' } }); // No I18N
            content.append(params.contents[i]);
            contents.append(content);
            // Makes the first element as active by default
            if (i == 0) { content.addClass('active'); }
        }
        tab.append(switches);
        tab.append(contents);
        return tab;
    };

    // ***PRIVATE*** Handles the user click event
    // Changes the tab and shows the corresponding content
    self.change = function(event) {
        self.callback && self.callback(tabIndex, self.extras);
        var targetEl = dom(event.currentTarget);
        var currentTab = targetEl.dParent('tabs'); // No I18N
        var tabContents = currentTab.first('.tab-contents').toDOM(); // No I18N
        var tabIndex = targetEl.getIndex();

        var activeSwitch = currentTab.first('.tab-switches > li.active'); // No I18N
        activeSwitch.removeClass('active');
        targetEl.addClass('active');

        var activeContent = currentTab.first('.tab-contents > li.active'); // No I18N
        activeContent.removeClass('active');
        dom(tabContents.childNodes[tabIndex]).addClass('active');

        var textbox = dom(tabContents.childNodes[tabIndex]).first('textarea') || dom(tabContents.childNodes[tabIndex]).first('input[type=text]');
        setTimeout(function(){ if(textbox) { textbox.focus(); } }, 300);
    };

};

/** Email Bubble Component */

// It is the extended version of the text input which takes the contact as a input.
// contact can be email (or) group
var bubble = new function() {

    var self = this;
    self.view = null;
    self.keys = { BACKSPACE_KEY: 8, TAB_KEY: 9, ENTER_KEY: 13, CTRL_KEY: 17, LEFT_KEY: 37, TOP_KEY: 38, RIGHT_KEY: 39, BOTTOM_KEY: 40, BOLD_KEY: 66, COPY_KEY: 67, ITALIC_KEY: 73, UNDERLINE_KEY: 85, PASTE_KEY: 86, CUT_KEY: 88, CMD_KEY: 91 };

    // ***PRIVATE*** Constructs the email bubble view and keeps the reference
    self.construct = function() {
        if (!self.view) { self.view = dom(ZRSStringTemplateCreator.createTemplateForData('bubble', 'components')); } // No I18N
        return self.view.clone(true);
    };

    // ***PRIVATE*** Creates new contact if not present already
    self.new = function(targetEl) {
        var info = targetEl.val().split(',');
        for(var i = 0; i < info.length; i++) {
            var email = info[i].trim();
            if (info[i] != '') {
                var contact = ContactsAPI.getMatch(email);
                self.add(contact, dom(targetEl));
            }
        }
        targetEl.val('');
    };

    // ***PRIVATE*** Adds the new contact to the email textarea component
    self.add = function(contact, targetEl) {
        var parent = targetEl.dParent('contact-textarea'); // No I18N

        var matchingEl = parent.first('.bubble[data-id="' + contact.id + '"]'); // No I18N
        if (!matchingEl) {
            var bubbleEl = self.construct();
            bubbleEl.attr('data-id', contact.id).attr('data-name', contact.name).attr('data-zuid', contact.zuid); // No I18N

            bubbleEl.first('.bubble-icon').text(contact.name[0]).dFadeOut().hide(); // No I18N
            bubbleEl.first('.bubble-value').text(contact.name); // No I18N
            bubbleEl.first('.bubble-close').on('click', self.remove); // No I18N

            if (self.isItGroup(contact.id)) {
                var bubbleImg = bubbleEl.first('img').hide().dParent('bubble-pic-cont'); // No I18N
                bubbleImg.first('.bubble-icon').html('<svg class="svgIcons" style="height: 18px;width: 14px; stroke-width: 1px;"><use class="gr" xlink:href="#ZR-group"></use></svg>').dFadeIn().show(); // No I18N
            } else {
                var contactEl = bubbleEl.first('img').attr('src', ContactsAPI.getImgURL(contact.zuid)); // No I18N
                contactEl.toDOM().onerror = function(event) {
                    var img = dom(event.currentTarget);
                    var bubbleImg = img.hide().dParent('bubble-pic-cont'); // No I18N
                    bubbleImg.first('.bubble-icon').dFadeIn().show(); // No I18N
                };
            }

            parent.prepend(bubbleEl, targetEl.getIndex());
            self.markIfInvalid(bubbleEl, contact.id);
            targetEl.val('');
            parent.append(targetEl);
        } else {
            Contacts.msgEl.dFadeIn();
            matchingEl.addClass('highlight');
            setTimeout(function() {
                Contacts.msgEl.dFadeOut();
                matchingEl.removeClass('highlight');
            }, 3000);
        }

        var bubbleCount = parent.find('.bubble').length;
        var copyOption = parent.first('.copy-option'); // No I18N
        copyOption && (bubbleCount > 0 ? copyOption.dFadeIn() : copyOption.dFadeOut());
        setTimeout(function() { self.focusInputEl(parent); }, 300);
    };

    // ***PRIVATE*** The contact will be marked as invalid if invalid email
    self.markIfInvalid = function(bubbleEl, email) { (!Helper.isValidEmail(email)) && (bubbleEl.addClass('error').attr('onmouseout', 'ZDBToolTip.hide();').attr('onmousemove', 'ZDBToolTip.show(this,event);').attr('tip', email + ' ' + ZDBI18NMap.g('zr.share.js.invalid_email'))); }; // No I18N

    // ***PRIVATE*** The method will return true if the id is group id. False otherwise
    self.isItGroup = function(id) { return !isNaN(id); };

    // ***PRIVATE*** All the key events will be monitered and shows the suggesstion box
    // And adjusts the hidden input element for every key press event
    self.listenKeys = function(event) {
        var keyCode = event.which || event.keyCode;
        var targetEl = dom(event.currentTarget);
        ZDBAutoFillContacts.autolist(event.currentTarget, event, 'dropdown', ZDBAutoFillContacts.matchSample); // No I18N
        self.adjustFieldWidth(targetEl.toDOM());
    };

    // ***PRIVATE*** Listens the key events, On ENTER_KEY, the contact will be created. On BACK_SPACE, contact will be removed
    self.createOnKeyUp = function(event) {
        var keyCode = event.which || event.keyCode;
        var targetEl = dom(event.currentTarget);
        ZDBAutoFillContacts.autolist(targetEl.toDOM(), event, 'dropdown', ZDBAutoFillContacts.matchSample); // No I18N
        var bubbleEls = targetEl.dParent('contact-textarea').find('.bubble'); // No I18N
        if ((keyCode == self.keys.ENTER_KEY || event.key == ',') && targetEl.val().trim() != '') {
            self.new(targetEl);
        } else if (keyCode == self.keys.BACKSPACE_KEY && targetEl.val() == '') {
            if (bubbleEls.length > 0) {
                targetEl.dParent('contact-textarea').append(targetEl); // No I18N
                var lastEl = dom(bubbleEls[bubbleEls.length - 1]);
                (lastEl.hasClass('ready-to-remove')) ? ((lastEl.remove()) || (bubbleEls[bubbleEls.length - 2]) && dom(bubbleEls[bubbleEls.length - 2]).addClass('ready-to-remove')) : lastEl.addClass('ready-to-remove'); // No I18N
                self.focusInputEl(targetEl.dParent('contact-textarea')); // No I18N
            }
        } else {
            (bubbleEls[bubbleEls.length - 1]) && dom(bubbleEls[bubbleEls.length - 1]).removeClass('ready-to-remove');
        }
        self.adjustFieldWidth(targetEl.toDOM());
    };

    self.adjustFieldWidth = function(targetEl) { targetEl.style.width = ((targetEl.value.length + 1) * 7) + 'px'; };

    // ***PRIVATE*** Listens the blur event. if the input element blurs, the contact will be created from the content of input element
    // And hides the contact suggestion
    self.createOnBlur = function(event) {
        var targetEl = dom(event.currentTarget);
        ZDBAutoFillContacts.hidepopup('dropdown'); // No I18N
        if (targetEl.val() != '') {
            self.new(targetEl);
            targetEl.toDOM().style.width = '10px';
        }
        bubble.blurInputEl(targetEl.dParent('contact-textarea')); // No I18N
    };

    // ***PRIVATE*** Removes the contact element
    self.remove = function(event) {
        event.stopPropagation();
        var el = dom(event.currentTarget);
        var textArea = el.dParent('contact-textarea'); // No I18N
        ZDBToolTip.hide();
        el.dParent('bubble').remove(); // No I18N
        self.focusInputEl(textArea);
    };

    // Extract the contacts as JSON from the component
    // JSON will be comprised of: 'users', 'groups' and 'warnings'
    // Where 'users' is the array of email address, 'groups' is the array of group ids and 'warnings' is the count of invalid entries
    self.extractContacts = function(el) {
        var contactEls = el.find('.bubble');
        var users = [], groups = [], warnings = 0;
        for (var i = 0; i < contactEls.length; i++) {
            var contactEl = dom(contactEls[i]);
            var id = contactEl.attr('data-id'); // No I18N
            isNaN(id) ? users.push(id) : groups.push(id);
            (!Helper.isValidEmail(id)) && warnings++;
        }
        return { 'users': users, 'groups': groups, 'warnings': warnings }; // No I18N
    };

    // ***PRIVATE*** Returns the suggestion array which eliminates the duplicates
    // All the contacts which is already provided will be eliminated from the suggestion.
    self.getSuggesstionExculdeArr = function(el) {
        var contactEls = el.find('.bubble');
        var arr = [];
        for (var i = 0; i < contactEls.length; i++) {
            var contactEl = dom(contactEls[i]);
            var id = contactEl.attr('data-id'); // No I18N
            isNaN(id) ? arr.push(id) : arr.push(contactEl.attr('data-name')); // No I18N
        }
        return arr;
    };

    // ***PRIVATE*** Returns the count of the contact
    self.getCount = function(el) { return el.find('.bubble').length; };

    // ***PRIVATE*** Enables the edit mode for the contact
    // Onclick, edit will be triggered.
    self.edit = function(el) {
        el = dom(el);
        var value = isNaN(el.attr('data-id')) ? el.attr('data-id') : el.attr('data-name'); // No I18N
        var inputEl = el.dParent('contact-textarea').first('input'); // No I18N
        if (inputEl.element) {
            inputEl.val(value);
            inputEl.toDOM().style.width = (inputEl.val().length * 7) + 'px';
            el.replace(inputEl);
        }
        ZDBToolTip.hide();
    };

    // ***PRIVATE*** Focuses the bubble component
    self.focusInputEl = function(el) {
        var targetEl = dom(el);
        targetEl.addClass('focus');
        var inputEl = targetEl.first('input'); // No I18N
        inputEl.focus();

        // Just to handle the blur when DOM changes happening
        setTimeout(function() { inputEl.focus(); }, 1000);

        self.activeBubble = targetEl;

        // To keep the auto suggestion up-to date
        ContactsAPI.fillAutoSuggest(ContactsAPI.json, self.getSuggesstionExculdeArr(targetEl));
    };

    // ***PRIVATE*** Blurs the bubble component
    self.blurInputEl = function(el) { el.removeClass('focus').first('input').blur(); }; // No I18N

    self.clear = function(targetEl) { dom(targetEl).dFind('.bubble').remove(); }; // No I18N

    // ***PRIVATE*** Returns the active bubble component
    self.getActiveBubble = function() { return self.activeBubble; };

    self.addCopyOption = function(targetEl) {
        var copyOption = dom(ZRSStringTemplateCreator.createTemplateForData('copyOption', 'components'));  // No I18N
        dom(targetEl).append(copyOption);
    };

    // ***PRIVATE*** All the emails and group ids will be coped to clipboard when clicked on "Copy" button
    self.copyToClipboard = function(event) {
        var textArea = dom(event.currentTarget).dParent('contact-textarea'); // No I18N
        var contacts = self.extractContacts(textArea);
        var text = contacts.users.concat(contacts.groups).join(',');
        try {
            ZDBClientUI.pasteArea.value = text;
            ZDBClientUI.pasteArea.select();
            if (document.execCommand('copy')) { ZDBMsgBox.showSuccess(ZDBI18NMap.g('zrpt.common.share.shareview.js.copied_to_clipboard')); }
        } catch (err) {}
    };

    // ***PRIVATE*** Takes care of the contact element ondrag event
    self.handleContactDrag = function (element, event) {
        var el = dom(element);
        self.draggedContact = dom(element).attr('data-id'); // No I18N

        // Keeps the track of current dragged contact
        self.draggedContact = { id: el.attr('data-id'), name: el.attr('data-name'), zuid: el.attr('data-zuid') }; // No I18N
    };

    // ***PRIVATE*** Adds visual cue when the drag event started on contact element
    self.handleContactDragStart = function (element, event) { dom(element).addClass('is-dragging'); };

    // ***PRIVATE*** Removes visual cue when the drag event stopped on contact element
    self.handleContactDragEnd = function (element, event) { dom(element).removeClass('is-dragging'); };

    // ***PRIVATE*** Focus will be added to the textarea when the contact element dragged over.
    self.handleDragOver = function (element, event) {
        event.preventDefault && event.preventDefault();
        event.stopPropagation && event.stopPropagation();
        dom(element).addClass('focus').addClass('on-drag');
    };

    // ***PRIVATE*** Focus will be removed from the textarea when the contact leaves.
    self.handleDragLeave = function (element, event) {
        event.preventDefault && event.preventDefault();
        event.stopPropagation && event.stopPropagation();
        dom(element).removeClass('focus').removeClass('on-drag');
    };

    // ***PRIVATE*** Add contact will be triggered when the contact is dropped on the textarea
    self.handleContactDrop = function (element, event) {
        event.preventDefault && event.preventDefault();
        event.stopPropagation && event.stopPropagation();
        var textEl = dom(element);
        textEl.removeClass('on-drag');
        self.focusInputEl(textEl);
        bubble.add(self.draggedContact, bubble.getActiveBubble().first('input')); // No I18N
        self.blurInputEl(textEl);
    };

};

/*** Filter Criteria Component ***/

// Filter criteria dialog is just replaced with this JS component
// So, the multiple occurances of the HTML can be eliminated

var fc = new function() {

    var self = this;
    // Column Type VS CSS Class Map
    self.columnClassMap = { 'PLAIN': 'plain', 'GEO': 'geo', 'MULTI_LINE': 'multiline', 'EMAIL': 'email', 'NUMBER': 'number', 'POSITIVE_NUMBER': 'positive', 'DECIMAL_NUMBER': 'decimal', 'GEO_NUM': 'geo', 'CURRENCY': 'currency', 'PERCENT': 'percentage', 'DATE': 'date', 'BOOLEAN': 'boolean', 'URL': 'url', 'AUTO_NUMBER': 'autoNumber', 'DATE_AS_DATE': 'date' }; // No I18N
    // Array of Variables
    self.globalVariables = ['system.login.email', 'system.login.firstname', 'system.login.lastname', 'system.login.fullname']; // No I18N

    // Takes the view type and constructs the Filter Criteria for that and returns the view
    self.construct = function(viewType) {
        self.view = dom(ZRSStringTemplateCreator.createTemplateForData('filterCriteria', 'components')); // No I18N
        self.columnContainer = self.view.first('#columns'); // No I18N
        self.variableContainer = self.view.first('#variables'); // No I18N
        self.fcTextArea = self.view.first('#fcTextArea'); // No I18N
        self.toggleViewType(viewType);
        return self.view;
    };

    // ***PRIVATE*** Filter criteria view will be adjusted based on the viewType
    // If multiple, shows the "Not supported" message, shows the corresponding otherwise.
    self.toggleViewType = function(viewType) {
        if (viewType == 'Multiple') {
            self.view.first('.fc-cont').hide(); // No I18N
            self.view.first('.fc-not-supported').show(); // No I18N
        } else {
            self.view.first('.fc-cont').show(); // No I18N
            self.view.first('.fc-not-supported').hide(); // No I18N
        }
    };

    // Fills the view with the provided JSON value.
    // JSON comprises of: objId, viewInfo:object, variables:array
    self.fill = function(json) {
        self.objId = json.objId;
        self.constructColumns(json.viewInfo);
        self.columnContainer.addClass('is-active');

        var variables = json.variables;
        for (var i = 0; i < self.globalVariables.length; i++) {
            variables.push(self.globalVariables[i]);
        }
        self.constructVariables(json.variables);
    };

    // ***PRIVATE*** Constructs the column list for the filter criteria
    self.constructColumns = function(viewInfo) {
        if (!viewInfo) { return; }
        self.columnContainer.html('');
        for (var i = 0; i < viewInfo.length; i++) {
            var info = viewInfo[i];
            //var template = '<li onclick="fc.toggleList(event);"><span class="strong-text gray"><svg class="svgIcons" style="height: 18px; width: 14px;"><use class="gr" xlink:href="#ZR-group"></use></svg></span></li>';
            var li = dom.new('li', { events: { click: self.toggleList } }); // No I18N
            var span = dom.new('span', { attributes: { class: 'strong-text gray' } }); // No I18N
            var icon = dom.new('span', { html: '<svg class="svgIcons enable-transition" style="height: 10px;width: 10px;display: inline-block;stroke-width: 2px;margin: 0 5px 0 -4px;"><use class="gr" xlink:href="#ZR-downArrow"></use></svg>' }); // No I18N
            span.append(icon);
            span.append(document.createTextNode(info.viewName));
            li.append(span);

            var columnsUl = dom.new('ul', { attributes: { class: 'opts-cont-column' } }); // No I18N
            li.append(columnsUl);

            for (var j = 0; j < info.columns.length; j++) {
                var column = info.columns[j];
                var template = '<li onclick="fc.addColumn(event);"><i class="fc-icon"></i><span></span></li>';
                var c_li = dom.new('li', { events: { click: self.addColumn } }); // No I18N
                var c_icon = dom.new('i', { attributes: { class: 'fc-icon ' + self.columnClassMap[column.type] } }); // No I18N
                var c_text = dom.new('span', { text: column.name }); // No I18N
                c_li.append(c_icon);
                c_li.append(c_text);
                columnsUl.append(c_li);
            }
            self.columnContainer.append(li);
        }
    };

    // ***PRIVATE*** Toggles the columns list based on the table.
    self.toggleList = function(event) {
        var el = dom(event.currentTarget);
        var li = el.first('ul'); // No I18N
        li.toggle();

        if (li.toDOM().style.display == '' || li.toDOM().style.display == 'block') {
            el.toDOM().querySelector('.svgIcons').style.transform = 'rotate(0deg)'; // No I18N
        } else {
            el.toDOM().querySelector('.svgIcons').style.transform = 'rotate(-90deg)'; // No I18N
        }
    };

    // ***PRIVATE*** Adds the column to the filter criteria text area
    self.addColumn = function(event) {
        event.stopPropagation();
        var element = event.currentTarget;
        var columnName = '"' + element.textContent + '"'; // No I18N
        var tableName = '"' + element.parentNode.previousElementSibling.textContent + '"'; // No I18N
        self.setFCTextArea(tableName + '.' + columnName);
    };

    // ***PRIVATE*** Constructs the variable list
    self.constructVariables = function(variables) {
        if (!variables) { return; }
        self.variableContainer.html('');
        for (var i = 0; i < variables.length; i++) {
            var li = dom.new('li', { events: { click: self.addVariable } }); // No I18N
            li.append(document.createTextNode(variables[i]));
            self.variableContainer.append(li);
        }
    };

    // ***PRIVATE*** Adds the variable to the filter criteria text area
    self.addVariable = function(event) {
        event.stopPropagation();
        var element = event.currentTarget;
        var variableName = element.textContent;
        self.setFCTextArea("${" + variableName + "}");
    };

    // ***PRIVATE*** Sets the text for the filter criteria text area and focuses it
    self.setFCTextArea = function(value) {
        self.fcTextArea.val(self.fcTextArea.val() + value);
        self.fcTextArea.focus();
    };

    // ***PRIVATE*** Hanles the click event on the tabs: "Columns" & "Variables"
    self.loadTab = function(element) {
        var element = dom(element);
        var index = element.getIndex();
        dom(element.toDOM().parentNode).first('.is-active').removeClass('is-active'); // No I18N
        element.addClass('is-active');
        var container = dom(element.parent('fc-options-cont')); // No I18N
        var contentParent = container.first('.opts-cont'); // No I18N
        var activeParent = contentParent.first('.is-active'); // No I18N
        activeParent.removeClass('is-active');
        dom(contentParent.children()[index]).addClass('is-active');
    };

    // ***PRIVATE*** Will be invoked when the "Preview" is clicked
    // Sends the request to the server with the provided filter criteria
    self.requestPreview = function() {
        var filterCriteria = self.fcTextArea.val();
        var viewType = ZDBClientUI.selectedTab.view.type;
        if (filterCriteria == null || filterCriteria == '') {
            ZDBAlertAPI.showMsg(ZDBI18NMap.g('zrpt.common.share.shareview.js.empty_filter_criteria_in_preview'));
            return;
        }
        filterCriteria = encodeURIComponent(filterCriteria);
        var params = 'ZDBACTION=VALIDATECRITERIA&OBJID=' + self.objId + '&ZDBFILTERCRITERIA=' + filterCriteria + '&ISDASHBOARD=' + ((viewType == 'Dashboard') ? 'true' : 'false'); // No I18N
        AjaxAPI.sendRequest({
            URL: '/ZDBSecurityAction.ma', METHOD: 'GET', // No I18N
            PARAMETERS: params,
            ONSUCCESSFUNC: self.preview,
            FILTERCRITERIA: filterCriteria,
            OBJID: self.objId,
            OPSTATUSID: 'ZDBFCPreviewProcess' // No I18N
        });
    };

    // ***PRIVATE*** Shows the preview of the view for the given filter criteria
    self.preview = function(response, requestOptions) {
        var filterCriteria = requestOptions['FILTERCRITERIA'];
        var objId = requestOptions['OBJID'];
        var previewIframe = $('ZDBPreviewIFrame');
        if (!previewIframe) {
            var previewCont = ZRSStringTemplateCreator.createTemplateForData('fcPreview', 'components'); // No I18N
            document.body.append(previewCont);
            previewIframe = previewCont.querySelector('iframe'); // No I18N
        }
        previewIframe.src = ZDBGlobal.REPORTS_CONTEXT_NAME + '/ZDBDataSheetView.cc?OBJID=' + objId + '&STANDALONE=true&REMTOOLBAR=true&SEARCHBOX=false&INCLUDETITLE=true&INCLUDEDESC=true&FCPREVIEW=true&ZOHO_CRITERIA=' + filterCriteria;
        var titleText = ZDBI18NMap.g('zrpt.common.share.shareview.js.previewwindow');
        ZDBDialogUtil.showDialog($('ZDBPreviewDiv'), { MODAL: true, DONTREMOVE: true, ADDFRAME: true, title: titleText, SHADOW: true, ONCLOSE: self.closePreview, ONESC: ZDBDialogUtil.closeDialog });
    };

    // ***PRIVATE*** Closes the filter criteria preview dialog
    self.closePreview = function() { $('ZDBPreviewIFrame').src = ''; return true; };
};

/** PushBar Component **/

// Pushbar is again the sidepane (Dialog from the right side)
// Useful to show the minimal information when the sidepane is already active
// For e.g., to show the contact list

var PushBar = new function() {

    var self = this;

    // Constructs the HTML skeleton and shows the pushbar
    // If already created, It just shows
    self.show = function(title, body) {
        if (!self.view) { self.construct(); }
        self.title.text(title);
        self.body.empty().append(body);
        return self.view;
    }

    // Constructs the HTML Skeleton and cahce it and initiates the element references
    self.construct = function() {
        self.view = dom(ZRSStringTemplateCreator.createTemplateForData('pushbar', 'components')); // No I18N
        self.title = self.view.first('#pushbarTitle'); // No I18N
        self.body = self.view.first('#pushbarBody'); // No I18N
    };

    // Hides the Pushbar
    self.hide = function() {
        self.view && self.view.removeClass('is-active'); // No I18N 
        sidepane.getActivePane().removeClass('has-pushbar'); // No I18N 
    };
};

/*** Accordion Component */

// Just the accordion component. Expands the focused item, others collapsed

var Accordion = new function() {

    var self = this;

    // Constructs the view template and takes the JSON argument: json
    // json argument comprises of: Array of json object { 'title' and 'content' }
    self.construct = function(json) {
        if (!self.view && !self.accGroup) {
            self.view = dom.new('div', { attributes: { class: 'accordion ZRSUnselectable' } }); // No I18N
            self.accGroup = dom(ZRSStringTemplateCreator.createTemplateForData('accGroup', 'components')); // No I18N
        }
        var accordion = self.view.clone(true);

        if (json.search) {
            self.search = json.search;
            self.accSearch = dom(ZRSStringTemplateCreator.createTemplateForData('accSearch', 'components')); // No I18N
            accordion.append(self.accSearch);

            var txtSearch = self.accSearch.first('input'); // No I18N
            txtSearch.on('keydown', function(event) {
                event.keyCode == 27 && txtSearch.val('');
                event.stopPropagation();
            });
            self.searchMethod = function() {
                var noMatchEl = dom.new('span', { i18n: 'zrpt.zpinteg.js.nomatchfound', attributes: { class: 'gray js-nomatch-el', style: 'margin: 10px 0 10px 31px; display: inline-block;' } }); // No I18N
                var key = txtSearch.val().toLowerCase();
                var group = accordion.find('.acc-group');
                for(var i = 0; i < group.length; i++) {
                    var items = dom(group[i]).find('.' + json.search.itemClass);
                    var matchCount = 0;
                    for (var j = 0; j < items.length; j++) {
                        var item = dom(items[j]);
                        var matchFound = false;
                        for (var k = 0; k < json.search.attributes.length; k++) {
                            var value = item.attr(json.search.attributes[k]).toLowerCase();
                            if (value.indexOf(key) != -1) { matchFound = true; }
                        }
                        if (matchFound) {
                            item.show();
                            matchCount++;
                        } else {
                            item.hide();
                        }
                    }
                    dom(group[i]).first('[data-search-result]').text(' (' + matchCount + ')'); // No I18N
                    (matchCount == 0 && items.length > 0) ? !dom(group[i]).first('.js-nomatch-el') && dom(group[i]).first('.acc-content').append(noMatchEl.clone(true)) : dom(group[i]).first('.js-nomatch-el') && dom(group[i]).first('.js-nomatch-el').remove(); // No I18N
                }
                var searchIcon = self.accSearch.first('.ZRSSearchIcon'); // No I18N
                key.length == 0 ? searchIcon.removeClass('cancel-acc-search') : searchIcon.addClass('cancel-acc-search');
            };
            txtSearch.on('keyup', self.searchMethod);
            self.accSearch.first('.ZRSSearchIcon').on('click', function() { txtSearch.val('').focus(); self.searchMethod(); }); // No I18N
        } else {
            self.accSearch = false;
        }

        var data = json.data;
        for (var i = 0; i < data.length; i++) {
            var accGroup = self.accGroup.clone(true);
            accGroup.first('[data-title]').text(data[i].title);
            var accContent = accGroup.first('[data-content]'); // No I18N
            accContent.empty();
            if(data[i].content) {
                accContent.append(data[i].content);
            } else {
                accContent.i18n('zr.share.js.fetching_contacts'); // No I18N
                var callbackData = { index: i, view: accContent };
                accGroup.on('click', function(callbackData) {
                    if(!this.view.hasClass('loaded')) {
                        data[this.index].callbackForContent(this);
                    }
                }.bind(callbackData));
            }
            accordion.append(accGroup);
        }

        self.freeFlow = json.freeFlow;
        return accordion;
    };

    // On title click,  expands the corresponding accordion group.
    self.showGroup = function(el) {
        el = dom(el);
        var accordion = el.dParent('accordion'); // No I18N
        var parentNode = accordion.parent();
        var accTitles = accordion.find('.acc-title');
        var computedHeight = self.freeFlow ? parentNode.height() : (parentNode.height() - ((accTitles[0].offsetHeight * accTitles.length) + 24));

        var selectedGroup = dom(el.parent('acc-group')); // No I18N
        var activeGroup = accordion.first('.acc-group.is-active'); // No I18N
        if (activeGroup) {
            activeGroup.removeClass('is-active');
            activeGroup.first('.acc-content').toDOM().style.maxHeight = '0'; // No I18N
        }
        if (activeGroup && activeGroup.toDOM() == selectedGroup.toDOM()) {
            selectedGroup.removeClass('is-active'); // No I18N
            selectedGroup.first('.acc-content').toDOM().style.maxHeight = '0'; // No I18N
        } else {
            selectedGroup.first('.acc-content').toDOM().style.maxHeight = computedHeight - 23 + 'px'; // No I18N
            selectedGroup.toggleClass('is-active'); // No I18N
        }
    };

    self.setContent = function(params) {
        params.view.addClass('loaded').empty().append(params.content).dParent('acc-group').removeClass('is-active'); // No I18N
        self.showGroup(params.view);
    };

    self.clearSearch = function () {
        if(self.accSearch) {
            var txtSearch = self.accSearch.first('input'); // No I18N
            txtSearch.val('');
            self.searchMethod();
        }
    };
};

/*** Contacts API */

// The place to handle all the contacts: Zoho Contacts, Google Contacts, Shared Contacts and Groups
var ContactsAPI = new function() {

    var self = this;
    self.dataRequested = false;

    // ***PRIVATE*** Initializes the ContactsAPI, Lazyloads the AutoSuggesstion JS
    // Takes the callbacks argument, where callbacks is the array of method references
    // Once lazyload is done, callbacks will be invoked and API will be invoked to get all the contacts
    self.init = function(callbacks) {
        if (!ZDBLazyLoadUtil.loadJsAlone('ZDBExportNShare', 'APPENDTODOC', self.init, [callbacks])) { return; } //NO I18N
        if (!self.dataRequested) {
            self.dataRequested = true;
            AjaxAPI.sendRequest({ URL: '/reportsapi/db/' + ZDBClientUI.dbId + '/GETCONTACTS?REFETCH=true', METHOD: 'GET', ONSUCCESSFUNC: self.set, CALLBACKS: callbacks });
        } else {
            self.invokeCallbacks(callbacks);
        }
    };

    self.refetch = function(callbacks) {
        self.dataRequested = false;
        self.init(callbacks);
    };

    // ***PRIVATE*** Invokes all the methods which is specified in the callbacks array
    self.invokeCallbacks = function(callbacks) { for (var i = 0; callbacks && i < callbacks.length; i++) { callbacks[i](); } };

    // ***PRIVATE*** sets the contacts as json
    self.set = function(response, request) {
        self.json = JSON.parse(response.responseText).data;
        self.initAutoSuggest();
        self.invokeCallbacks(request.CALLBACKS);
    };

    // ***PRIVATE*** Initializes the auto suggestion and fill the auto suggestion list
    self.initAutoSuggest = function() {
        ZDBAutoFillContacts.setUsers(self.getAutoFillValues());
        self.fillAutoSuggest(self.json);
    };

    // ***PRIVATE*** fills the auto suggestion list
    self.fillAutoSuggest = function(json, excludeArr) {
        if (!json) return;
        if (!excludeArr) { excludeArr = []; }
        var list = [];
        for (var arr in json) {
            var arr = self.getAsArray(json[arr]);
            for (var i = 0; i < arr.length; i++) {
                ((list.indexOf(arr[i]) == -1) && (excludeArr.indexOf(arr[i]) == -1)) && list.push(arr[i]);
            }
        };
        ZDBAutoFillContacts.fillMyUsers({ responseText: list.join(',') });
    };

    // ***PRIVATE*** Returns the auto fill values
    self.getAutoFillValues = function() {
        var values = [];
        for (list in self.json) {
            for (var i = 0; i < self.json[list].length; i++) {
                var data = self.json[list][i];
                isNaN(data.id) ? values.push(data.id) : values.push(data.name);
            }
        }
        return values;
    };

    // ***PRIVATE*** Converts the contacts json object into contact array
    self.getAsArray = function(arr) {
        var outputArr = [];
        for (var i = 0; i < arr.length; i++) { outputArr.push(isNaN(arr[i].id) ? arr[i].id : arr[i].name); }
        return outputArr;
    };

    // ***PRIVATE*** Gets the matches with the given value
    self.getMatch = function(value) {
        for (list in self.json) {
            for (var i = 0; i < self.json[list].length; i++) {
                var data = self.json[list][i];
                var matchVal = isNaN(data.id) ? data.id : data.name;
                if (matchVal == value) { return data; }
            }
        }
        return { id: value, name: value, zuid: '-1' };
    };

    // Returns the contacts image url
    self.getImgURL = function(zuid) { return GlobalNotification.CONTACTS_IMG_BASE_URL + (zuid ? zuid : '-1') + '&nps=404'; };  // No I18N
};

/** Contacts Component ***/

var Contacts = new function() {

    var self = this;
    self.templateConstructed = false;

    self.get = function(refetch) {
        refetch ? ContactsAPI.refetch([self.construct]) : ContactsAPI.init([self.construct]);
        if (!self.templateConstructed) {
            self.view = PushBar.show('Contacts List', dom.new('div', { attributes: { class: 'gray fetching-contacts' }, i18n: 'zr.share.js.fetching_contacts' })); // No I18N
            document.body.appendChild(self.view.toDOM());
        }
        if(refetch) { self.templateConstructed = false; self.view.first('#pushbarBody').empty().append(dom.new('div', { attributes: { class: 'gray fetching-contacts' }, i18n: 'zr.share.js.fetching_contacts' })); } // No I18N
    };

    self.construct = function() {
        if (!self.template) {
            self.template = dom(ZRSStringTemplateCreator.createTemplateForData('contact', 'components')); // No I18N
        }
        if (!self.templateConstructed) {
            var accordion = Accordion.construct({
                data: [
                    { title: ZDBI18NMap.g('zr.share.js.shared_contacts'), content: self.constructContactGroup(ContactsAPI.json.sharedContacts, 'shared', ContactsAPI.json) }, // No I18N
                    { title: ZDBI18NMap.g('zr.share.js.zoho_contacts'), content: self.constructContactGroup(ContactsAPI.json.zohoContacts, 'zoho', ContactsAPI.json) }, // No I18N
                    { title: ZDBI18NMap.g('zr.share.js.google_contacts'), content: self.constructContactGroup(ContactsAPI.json.googleContacts, 'google', ContactsAPI.json) }, // No I18N
                    { title: ZDBI18NMap.g('zr.share.js.groups'), content: self.constructContactGroup(ContactsAPI.json.groups, 'groups', ContactsAPI.json) } // No I18N
                ],
                search: { itemClass: 'zr-contact', attributes: ['data-id', 'data-name'] } // No I18N
            });

            self.view.first('#pushbarBody').empty().append(accordion); // No I18N
            Accordion.showGroup(accordion.first('.acc-title')); // No I18N
            self.templateConstructed = true;

            var template = '<div class="bubble-already-added enable-transition">{{i18n:zr.share.js.already_added}}</div>';
            self.msgEl = dom.new('div', { attributes: { class: 'bubble-already-added enable-transition' }, i18n: 'zr.share.js.already_added' }); // No I18N
            self.view.append(self.msgEl);
        }
    };

    self.constructContactGroup = function(jsonArray, type, rawJSON) {
        if (jsonArray.length == 0) {
            var spanEl = dom.new('span', { attributes: { style: 'margin: 10px 0 10px 30px; display: inline-block;' } }); //No I18N
            var noDataEl = dom.new('span', { attributes: { class: 'gray' } }); //No I18N
            (type == 'groups') ? noDataEl.i18n('zrpt.common.js.nogroups') : noDataEl.i18n('zrpt.common.js.nousersavail'); //No I18N
            
            var anchorEl = dom.new('a', { attributes: { class: 'ZRSLink', style: 'padding-left: 10px;' } }); //No I18N
            if (type == 'google') {
                if (!rawJSON.googleContactsStatus && rawJSON.gContactsErrMsg == "GOOGLE_AUTH_REQUIRED") {
                    anchorEl.i18n('zr.share.js.authenticate_google').on('click', self.authenticateGoogle); //No I18N
                    spanEl.append(anchorEl);
                    return spanEl;
                } else {
                    anchorEl.i18n('zrpt.share.sharenewusr.addmycontact.js.signinasdiffguser').on('click', self.signInAsDifferentUser); //No I18N
                    return spanEl.append(noDataEl).append(anchorEl);
                }
            }
            spanEl.append(noDataEl);
            return spanEl;
        }

        var ul = dom.new('ul', { attributes: { class: 'zr-contacts-list' } }); // No I18N
        for (i = 0; i < jsonArray.length; i++) {
            var contactEl = self.template.clone(true);
            contactEl.attr('data-id', jsonArray[i].id).attr('data-name', jsonArray[i].name).attr('data-zuid', jsonArray[i].zuid ? jsonArray[i].zuid : '-1');
            
            if (isNaN(jsonArray[i].id)) {
                contactEl.first('[data-id]').text(jsonArray[i].id);

                // Contact Image related code snippet
                var contactImgEl = contactEl.first('img').toDOM(); // No I18N
                contactImgEl.src = ContactsAPI.getImgURL(jsonArray[i].zuid);

                contactImgEl.onerror = function(event) {
                    var img = dom(event.currentTarget);
                    var conImg = img.hide().dParent('contact-pic-holder'); // No I18N
                    conImg.first('.no-image').show(); // No I18N
                };
                contactEl.first('[data-name]').text(jsonArray[i].name);
                contactEl.first('.no-image').text(jsonArray[i].name[0].toUpperCase()).hide();
            } else {
                contactEl.first('[data-id]').hide(); // No I18N

                var picHolder = contactEl.first('img').hide().dParent('contact-pic-holder'); // No I18N
                picHolder.first('.no-image').html('<svg class="svgIcons" style="height: 18px; width: 14px;color: #ffffff; stroke-width: 1px;"><use class="gr" xlink:href="#ZR-group"></use></svg>').dFadeIn().show(); // No I18N

            }
            contactEl.first('[data-name]').text(jsonArray[i].name);
            contactEl.on('click', self.add); // No I18N
            contactEl.on('dragstart', function(event) { // No I18N
                 // See, Its a hack. To make the draggable functionality to work on FireFox, Need to set some data.
                event.dataTransfer.setData('foo', 'bar'); // No I18N
            });
            ul.append(contactEl);
        }
        if (type == 'google') {
            var anchorEl = dom.new('a', { attributes: { class: 'ZRSLink', style: 'padding: 10px 0px 10px 16px; display: inline-block;' }, i18n: 'zrpt.share.sharenewusr.addmycontact.js.signinasdiffguser' }); //No I18N
            anchorEl.on('click', self.signInAsDifferentUser);
            return dom.new('div').append(anchorEl).append(ul); // No I18N
        }
        return ul;
    };

    self.add = function(event) {
        var contactEl = dom(event.currentTarget);
        bubble.add({ id: contactEl.attr('data-id'), name: contactEl.attr('data-name'), zuid: contactEl.attr('data-zuid') }, bubble.getActiveBubble().first('input')); // No I18N
    };

    self.show = function() {
        self.get();
        self.view.addClass('is-active');
        sidepane.getActivePane().addClass('has-pushbar');
    };

    self.hide = function() {
        dom.hide(self.template);
        sidepane.getActivePane().removeClass('has-pushbar');
    };

    self.toggle = function() {
        self.view.toggleClass('is-active'); // No I18N
        sidepane.getActivePane().toggleClass('has-pushbar'); // No I18N
        Accordion.clearSearch();
    };

    self.authenticateGoogle = function() {
        var authURL = ZDBGlobal.GAPP_EMBED_GADGET_URL + "/gcontacts?auth=true&zservice=" + ZDBGlobal.GAPP_ZSERVICE + "&border=no&serURL=" + ZDBGlobal.REPORTS_SERVER_URL + ZDBGlobal.REPORTS_CONTEXT_NAME + encodeURIComponent("/integ/GResponseRedirect.jsp?gauth_type=contactsapi"); // No I18N
        ZDBGoogleInteg.authGoogle(authURL);
    };

    self.onGoogleAuthenticationPass = function() { self.get(true); };

    self.onGoogleAuthenticationFail = function() { self.get(true); };

    self.signInAsDifferentUser = function(event) { AjaxAPI.sendRequest({URL: '/ZDBSecurityAction.ma?ZDBACTION=ZDBREVOKEGTOKEN&GTYPE=contactsapi', ONSUCCESSFUNC: self.handleRevokeGAuthTokenSuccess, ONFAILUREFUNC: self.handleRevokeGAuthTokenFailure, GOOGLETYPE:"contactsapi", EVENT:event, OPSTATUSID:"ZDBMainLoader" }); }; // No I18N

    self.handleRevokeGAuthTokenSuccess = function (response, request) { self.authenticateGoogle(); };

    self.handleRevokeGAuthTokenFailure = function () { ZDBAlertAPI.showMsg(ZDBI18NMap.g("zrpt.common.share.shareview.js.errInGAuth")) };

};

var Helper = new function() {

    var self = this;

    self.compare = function (obj1, obj2) {
        for (var p in obj1) {
            if (obj1.hasOwnProperty(p) !== obj2.hasOwnProperty(p)) return false;
            switch (typeof (obj1[p])) {
                case 'object': if (!Object.compare(obj1[p], obj2[p])) return false; break;
                case 'function': if (typeof (obj2[p]) == 'undefined' || (p != 'compare' && obj1[p].toString() != obj2[p].toString())) return false; break;
                default: if (obj1[p] != obj2[p]) return false;
            }
        }
        for (var p in obj2) { if (typeof (obj1[p]) == 'undefined') return false; }
        return true;
    };

    // ***PRIVATE*** The contact will be validated against the email regex.
    // Returns true if the email is valid. false, otherwise.
    self.isValidEmail = function(email) { return !isNaN(email) || /^(([^<>()\[\]\.,;:\s@\"]+(\.[^<>()\[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i.test(email); };

};

// Initializers 
sidepane.init();
preloader.init();
ZDBClientUI.loadBaseSVGIcons();