//$Id$
//ignorei18n_start

var GlobalNotification = new function () {

    var self = this;

    self.templateConstructed = false;
    self.dataLoaded = false;
    self.initial_load = true;

    self.viewTypeIcon = ['Table','Report','AnalysisView','Pivot','SummaryView','TableView','QueryTable','Dashboard']; // No I18N
    self.colorCodes = ['#78eae1', '#cc8cea', '#ffd505', '#8396e2', '#fa72c1', '#b7e84f', '#26c6e2', '#f9b06f', '#ffdb46', '#898989', '#f77b7c', '#03d5c7', '#3c5491', '#df61d5', '#6fcae3']; // No I18N
    
    self.currentColorCodeIndex = 0;
    self.colorCodeMap = {};

    // Constants for the types of Notifications
    self.NOTI_SHARE = 1;
    self.NOTI_GROUP = 2;
    self.NOTI_SCHEDULAR = 3;
    self.NOTI_SERVICE = 5;

    // Date Constants
    self.DAY_IN_MILLISECONDS = 1000 * 60 * 60 * 24;
    self.DAYS_IN_WEEK = 7;
    self.months = { 0 : 'zrpt.dialoginfo.monthinterval.js.jan',
    				1 : 'zrpt.dialoginfo.monthinterval.js.feb',
    				2 : 'zrpt.dialoginfo.monthinterval.js.mar',
    				3 : 'zrpt.dialoginfo.monthinterval.js.apr',
    				4 : 'zrpt.dialoginfo.monthinterval.js.may',
    				5 : 'zrpt.dialoginfo.monthinterval.js.june',
    				6 : 'zrpt.dialoginfo.monthinterval.js.july',
    				7 : 'zrpt.dialoginfo.monthinterval.js.aug',
    				8 : 'zrpt.dialoginfo.monthinterval.js.sep',
    				9 : 'zrpt.dialoginfo.monthinterval.js.oct',
    				10 : 'zrpt.dialoginfo.monthinterval.js.nov',
    				11 : 'zrpt.dialoginfo.monthinterval.js.dec'};
    self.days = { 0 : 'zrpt.dialoginfo.weeklyinterval.js.sunday',
    			  1 : 'zrpt.dialoginfo.weeklyinterval.js.monday',
    			  2 : 'zrpt.dialoginfo.weeklyinterval.js.tuesday',
    			  3 : 'zrpt.dialoginfo.weeklyinterval.js.wednesday',
    			  4 : 'zrpt.dialoginfo.weeklyinterval.js.thursday',
    			  5 : 'zrpt.dialoginfo.weeklyinterval.js.friday',
    			  6 : 'zrpt.dialoginfo.weeklyinterval.js.saturday'}; // No I18N
    self.monthname = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', 'SEP', 'OCT', 'NOV', 'DEC']; // No I18N

    self.NOTIFICATION_TYPE=1;
    self.ANNOUNCEMENT_TYPE=2;




    // All initialization will happen here. It populates the existing notifications and settings
    self.init = function () {
        var holder = dom(ZDBUtil.templateCreator('notificationCont'));
        var mainCont = dom('#ZDBMainHeader');
        mainCont.append(holder);

        if(!self.cardTemplate) {
            self.cardTemplate = dom(ZDBUtil.templateCreator('infoCardContainer'));
        }


        // Dom elements related to notification module
        self.container = mainCont.first('#notificationCont');
        self.notifications = self.container.first('#notifications');
        self.announcements = self.container.first('#announcements');
        self.badge = mainCont.first('#notificationBadge');
        self.rippleEffect = mainCont.first('.notification-ripple');
        self.gnBadge = self.container.first("#generalNotfIconBadgeText");
        self.annmtBadge = self.container.first("#annmtNotfIconBadgeText");
        self.card = self.container.first('#notificationCard');
        self.settings = self.container.first('#notificationSettings');
        self.freezeLayer = self.container.first('#settingsFreezeLayer');
        self.noNotificationEl = self.container.first('#noNotificationEl');

        self.templateConstructed = true;
        self.currentTab = self.notifications;

        self.unreadAnnouncementIds=[];
    };

    // Requests the global notifications from the server and constructs the template
    self.get = function () {
        if(!self.templateConstructed) {
            self.init();
        }

        self.freezeLayer.addClass('is-visible').i18n('zr.notification.js.fetching');
        var reqURL = '/reportsapi/GETGLOBALNOTIFICATIONS';
//        AjaxAPI.sendRequest({URL: reqURL, ONSUCCESSFUNC: self.setJSON});

        ZRSAjax.get(reqURL, null, {
            successHandler: {
                func: self.setJSON,
                ctx: self
            },
            noLoader: true
        });
        self.unreadAnnouncementIds.clear();   //empties the unread announcement ID list before populating
    };

    // On Success, sets the json object, and initiates the Notification component
    self.setJSON = function (rawJSON) {
        //on live notification comes we have to empty the container before append
        self.notifications.empty();
        self.announcements.empty();

        self.freezeLayer.removeClass('is-visible').text('');
        self.dataLoaded = true;
//        var rawJSON = JSON.parse(response.responseText);
        self.json = rawJSON.notifications;
        self.userTimeZone = rawJSON.userTimeZone;
        self.CONTACTS_IMG_BASE_URL = rawJSON.contactsImgBaseUrl + '/file?t=user&height=60&width=60&exp=360000&ID='; // No I18N
        //self.CONTACTS_IMG_BASE_URL = 'https://contacts.csez.zohocorpin.com/file?fs=thumb&exp=360000&ID='; // No I18N

        self.preferences = rawJSON.preferences;
        
        for (var key in self.preferences) {
            var el = self.settings.first('[name=' + key + ']');
            el.element.checked = self.preferences[key];
            self.persistPreference(el.toDOM(), true);
        }

        // Populates the existing notification
        self.populate();

        // Shows the unread count
        self.showBadge(self.getUnreadCount());
        typeof ZRCRMPlus !== 'undefined' && ZRCRMPlus.showBadge(self.getUnreadCount()[0]);
        typeof ZRCRMPlus !== 'undefined' && ZRCRMPlus.onloadChanges('{"GETGLOBALNOTIFICATIONS":true}');

        //for handling live announcements
        if(self.container.hasClass('is-visible')) {
            self.markAsRead(self.currentTab.element.id=='notifications'?self.NOTIFICATION_TYPE:self.ANNOUNCEMENT_TYPE);
            //if either notifications or announcements is empty
           self.noNotificationEl.toDOM().style.display = (self.currentTab.toDOM().childElementCount == 0) ? 'block' : 'none';
           if(self.card.hasClass('is-visible')) {
                 self.card.removeClass('is-visible');
           }
        }
    };

    // Iterates the JSON data and adds the notification element into the component
    self.populate = function () {
        if(self.json.length == 0) {
            self.noNotificationEl.toDOM().style.display = 'block';
        } else {
            for (var i = 0; i < self.json.length; i++) {
                var data = self.json[i];
                self.new(data, i);
            }
        }

    };

    // Sends the request to server to mark all the notifications as read
    self.markAsRead = function (type) {
        var unReadCount=self.getUnreadCount();
        if(unReadCount[0] > 0) {
            var reqURL="";
            if(unReadCount[1] > 0 && type == self.NOTIFICATION_TYPE){
                reqURL = '/reportsapi/MARKNOTIFICATIONSREAD';
            }
            if(unReadCount[2]>0 && type == self.ANNOUNCEMENT_TYPE){
                reqURL = '/reportsapi/SETNOTIFICATIONACCESSEDTIME';
            }
            if(reqURL!=""){
            ZRSAjax.post(ZDBGlobal.CTX + ZDBGlobal.REPORTS_CONTEXT_NAME + reqURL, null, {
                successHandler: {
                    func: self.resetUnreadCount,
                    ctx: self
                },
                noLoader: true,
                TYPE:type
            });
            }
        }
    };

    // Need to reset the unread count since we have updated the server
    self.resetUnreadCount = function (response, request) {
        for(var i = 0; i < self.json.length; i++) {
            var data=self.json[i];
            var type='';
            var mustRead = false;
            if(data.type==self.NOTI_SERVICE){
                type=self.ANNOUNCEMENT_TYPE;
                mustRead=data.mustRead;
            }
            else{
                type=self.NOTIFICATION_TYPE;
            }
            if(type==request["TYPE"] && !mustRead){
                self.markDataAsRead(data);
            }
        }
    };

    // Returns the number of unread notification
    self.getUnreadCount = function () {
        var gnCount=0;
        var annmtCount =0;
        var mustReadAnnmtCount =0;
        for (var i = 0; i < self.json.length; i++) {
            if(!self.json[i].isRead) {
                if(self.json[i].type==self.NOTI_SERVICE){
                    (self.json[i].mustRead)?mustReadAnnmtCount++:annmtCount++;
                }
                else{
                    gnCount++;
                }
            }
        }
        return [annmtCount+gnCount+mustReadAnnmtCount,gnCount,annmtCount,mustReadAnnmtCount];
    };

    // Marks the JSON data as read
    self.markDataAsRead = function (data) {
        data.isRead = true;
        self.removeAnnouncementIdFromUnreadAnnouncement(data.messageId);
    };

    self.removeAnnouncementIdFromUnreadAnnouncement = function(mId){
        var index = self.unreadAnnouncementIds.indexOf(mId);
        if(index != -1){
            self.unreadAnnouncementIds.splice(index,1);
        }
    };

    // Creates new notification elment from the json object
    self.new = function (data, index, needDuplicatesCheck, prepend) {
        self.noNotificationEl.toDOM().style.display = 'none';
        if(needDuplicatesCheck) {
            var duplicateIndex = self.isDuplicate(data);
            if(duplicateIndex != -1) {
                index = duplicateIndex;
                self.json[index] = data;
                var notiDOM = self.notifications.toDOM();
                for(var i = 0; i < notiDOM.childNodes.length; i++) {
                    var el = notiDOM.childNodes[i];
                    if(el.getAttribute('data-index') == index) {
                        el.remove();
                        break;
                    }
                }
            } else {
                self.json.push(data);
            }
        }

        var el = dom(self.cardTemplate.element.cloneNode(true));

        if(!data.isRead) {
            if(data.type==self.NOTI_SERVICE){
                el.first('.extra-indicator').addClass('unread-announcements');
                if(data.mustRead){
                 el.first('.extra-indicator').addClass('must-read');
                }
                self.unreadAnnouncementIds.push(data.messageId);
            }
            else{
                el.addClass('unread');
            }
        }
        if(data.type ==self.NOTI_SHARE) {
            var viewCount = data.viewsInfo.length;
            if(data.time!=null){
            var dataTime = (ZDBGlobal.IS_ONPREMISE_BUILD)? self.getElapsedTime(data.time) : self.getElapsedTime(data.time) + ', ' + self.getOrgName(data);
            }
           var info = {
                'data-name': data.fromDisplayName,
                'data-msg-1': ' ' + ZDBI18NMap.g('zr.notification.js.shared') + ' ',
                'data-count': ((viewCount == 1) ? ZDBI18NMap.g('zr.notification.js.view') : viewCount + ' ' + ZDBI18NMap.g('zr.notification.js.views')),
                'data-msg-2': ' ' + ZDBI18NMap.g('zr.notification.js.with') + ' ' + (data.isSharedToGroup ? data.groupName : ZDBI18NMap.g('zr.notification.js.you'))
                                 + ' ' + ZDBI18NMap.g('zr.notification.js.in') + ' ' + data.dbName + ' ' + ZDBI18NMap.g('zr.notification.js.database'),
                'data-time': dataTime
            };
        } else if (data.type == self.NOTI_GROUP) {
            var info = {
                'data-name' : data.fromDisplayName,
                'data-msg-1' : ' ' + ZDBI18NMap.g('zr.notification.js.added_you') + ' ',
                'data-count' : data.groupName + ' ' + ZDBI18NMap.g('zr.notification.js.grouptext'),
                'data-msg-2' : ' ' + ZDBI18NMap.g('zr.notification.js.in') + ' ' + data.dbName + ' ' + ZDBI18NMap.g('zr.notification.js.database'),
                'data-time': dataTime
            }
        } else if (data.type == self.NOTI_SCHEDULAR) {
            // Schedular Notification
        } else if (data.type == self.NOTI_SERVICE) {
            var info = {
            };
        }

        if(data.type == self.NOTI_SHARE || data.type == self.NOTI_GROUP) {
            var imageContainer = el.first('.image-container'); // No I18N
            self.setContactImage(imageContainer, data);
        }

        if(data.type==self.NOTI_SERVICE) {
             el.first('.image-container').element.style.display="none";
             el.first('.extra-indicator-text').element.innerHTML=data.message.announcements.length;
             var dateContainer = el.first('.an-date'); //No I18N
             var userTime = data.time + GlobalNotification.userTimeZone;
             var userDate = new Date(userTime);
             dateContainer.first('.annmt-day').element.innerHTML =  ("0" + userDate.getUTCDate()).slice(-2);
             dateContainer.first('.month').element.innerHTML = self.monthname[userDate.getUTCMonth()];
             dateContainer.first('.year').element.innerHTML = userDate.getUTCFullYear();

            if(data.message.titlestyle!= undefined && data.message.descriptionstyle!=undefined){   //check for old JSON object
            //adding font size and color class for announcement title
            if( data.message.titlestyle.size!=-1){
                el.first('.announcement-title').addClass("font".concat(data.message.titlestyle.size-1)); }
            if(data.message.titlestyle.color!='default'){
                el.first('.announcement-title').element.style.color = data.message.titlestyle.color;  }

            //adding font size and color for announcement description
            if(data.message.descriptionstyle.size!=-1){
                el.first('.announcement-desc').addClass("font".concat(data.message.descriptionstyle.size-1)); }
            if(data.message.descriptionstyle.color!='default'){
                el.first('.announcement-desc').element.style.color = data.message.descriptionstyle.color; }
            }
            jQuery(el.first('.announcement-title').toDOM()).append(data.message.title);
            jQuery(el.first('.announcement-desc').toDOM()).append(data.message.description);
        }
        else{
            el.first('.extra-indicator').element.style.display='none';
        }


        el.attr('data-index', index);
        dom.fill(el.element, info);
        if(data.type == self.NOTI_SERVICE){
            self.announcements.append(el);
        }
        else{
            prepend ? self.notifications.prepend(el) : self.notifications.append(el);
        }
        el.on('click', self.toggleCard);
    };

    self.getOrgName = function (data) {
        var orgName = data.orgName;
        if(orgName) {
            return (orgName.length > 25) ? orgName.substr(0, 25) + '...' : orgName;
        }
        return '';
    };

    // Sets the contact image, On error: It will show the first letter of the contact
    self.setContactImage = function(imageContainer, data) {
        var image = imageContainer.first('img').toDOM();
        if(data.hasContactImage)
        {
            var reqURL = self.CONTACTS_IMG_BASE_URL + data.fromZuId + '&nps=404'; // No I18N
            image.src = reqURL;
        }
        else
        {
            dom(image).hide();
            var colorCode = '';
            if(self.colorCodeMap[data.fromZuId] != undefined) {
                colorCode = self.colorCodes[self.colorCodeMap[data.fromZuId]];
            } else {
                colorCode = self.colorCodes[self.currentColorCodeIndex];
                self.colorCodeMap[data.fromZuId] = self.currentColorCodeIndex;
                self.currentColorCodeIndex = (++self.currentColorCodeIndex % self.colorCodes.length);
            }
            var noImageEl = imageContainer.first('.no-image'); // No I18N
            noImageEl.text(data.fromDisplayName[0].toUpperCase());
            noImageEl.attr('style', 'background: ' + colorCode); // No I18N
            noImageEl.show();
        }
    };

    // Checks the duplicate in notification. Will be removed when it found.
    self.removeDuplicateIfFound = function (data) {
        for(var i = 0; i < self.json.length - 1; i++) {
            var info = self.json[i];
            if(data.type == self.NOTI_SHARE && info.type == self.NOTI_SHARE && data.fromZuId == info.fromZuId && data.viewsInfo.length == info.viewsInfo.length) {
                var matchCount = 0;
                for(var j = 0; j < data.viewsInfo.length; j++) {
                    if(self.contains(info.viewsInfo, data.viewsInfo[j].viewId)) {
                        matchCount++;
                    }
                }
                // Removes the duplicate element since the match found
                if(data.viewsInfo.length > 0 && matchCount == data.viewsInfo.length) {
                    self.notifications.element.children[i].remove();
                    return;
                }
            } else if (data.type == self.NOTI_GROUP && info.type == self.NOTI_GROUP && data.dbId == info.dbId && data.groupId == info.groupId) {
                self.notifications.element.children[i].remove();
                return;
            }
        }
    };

    self.isDuplicate = function (data) {
        for(var i = 0; i < self.json.length; i++) {
            var info = self.json[i];
            if(data.type == self.NOTI_SHARE && info.type == self.NOTI_SHARE && data.fromZuId == info.fromZuId && data.viewsInfo.length == info.viewsInfo.length) {
                var matchCount = 0;
                for(var j = 0; j < data.viewsInfo.length; j++) {
                    if(self.contains(info.viewsInfo, data.viewsInfo[j].viewId)) {
                        matchCount++;
                    }
                }
                // Removes the duplicate element since the match found
                if(data.viewsInfo.length > 0 && matchCount == data.viewsInfo.length) {
                    return i;
                }
            } else if (data.type == self.NOTI_GROUP && info.type == self.NOTI_GROUP && data.dbId == info.dbId && data.groupId == info.groupId) {
                self.notifications.element.children[i].remove();
                return i;
            }
        }
        return -1;
    };

    // Returns true if the value found in the array. False, otherwise.
    self.contains = function (array, value) {
        var i = array.length;
        while (i--) {
            if (array[i].viewId === value) {
                return true;
            }
        }
        return false;
    };

    // toggles the Notification component
    self.toggle = function (el) {
        /*if(self.dataLoaded) {
            self.markAsRead();
        }*/

        if(self.container == undefined || self.json == undefined)
        {
            return;
        }
        self.container.toggleClass('is-visible');
        self.hideBadge();
        self.rippleEffect.removeClass('is-visible');

        //handle which tab to load first
        self.handleTabsOnToggle();

        if(self.container.hasClass('is-visible')) {
            ev.addEscHandle(self.hide);
        }
        ZRUserAnalyticsAttacher.attachUAToAnnouncement();   //for handling analytics

    };

    self.handleTabsOnToggle = function(){
        var unreadCount = self.getUnreadCount();
        var totalCount=unreadCount[0];
        var notificationsCount = unreadCount[1];
        var announcementsCount = unreadCount[2]+unreadCount[3];

        if(announcementsCount>0){
            //show announcements tab
            self.switchTab(undefined,'service-notification');
        }
        else{
            //show notifications tab
            self.switchTab(undefined);
        }

    };

    // toggles the notification preferences
    self.toggleSettings = function (event) {
        self.container.first('.panel-1').toggleClass('inactive'); // No I18N
        self.settings.toggleClass('is-visible');
        self.card.removeClass('is-visible');
        var currentSelection = self.currentTab.first('.current');
        if(currentSelection) {
            currentSelection.removeClass('current');
        }
    };

    // On success, it reflects on UI
    self.applyPreferencesInUI = function () {
        for (var key in self.preferences) {
            var el = self.settings.first('[name=' + key + ']');
            el.element.checked = self.preferences[key];
            self.persistPreference(el.toDOM(), true);
        }
    };

    // toggles the notification card
    self.toggleCard = function (event) {
        var el = dom(event.currentTarget);
        self.constructCard(el);
        if(el.hasClass('current')){
            self.card.removeClass('is-visible');
            el.removeClass('current');
        } else {
            var pos = self.getRelativePosition(el.toDOM());
            var beforeEl = self.card.first('.ZRSPointerRightBefore').toDOM();
            var afterEl = self.card.first('.ZRSPointerRightAfter').toDOM();
            if((pos.y + self.card.toDOM().offsetHeight) <= self.container.toDOM().offsetHeight) {
                self.card.toDOM().style.top = pos.y + 'px';
                beforeEl.style.top = '20px';
                afterEl.style.top = '20px';
            } else {
                self.card.toDOM().style.top = (pos.y - self.card.element.offsetHeight + el.element.offsetHeight) + 'px';
                beforeEl.style.top = (self.card.element.offsetHeight - el.element.offsetHeight + 20) + 'px';
                afterEl.style.top = (self.card.element.offsetHeight - el.element.offsetHeight + 20) + 'px';
            }
            self.card.addClass('is-visible');
        }
        self.removeElementSelection();
        if(self.card.hasClass('is-visible')) {
            el.addClass('current');
        }
        el.removeClass('unread');
        el.first('.extra-indicator').removeClass('unread-announcements').removeClass('must-read');

        //handling must read announcement
        var index = el.attr('data-index');
        var data = self.json[index];
        if(data.type == self.NOTI_SERVICE && data.mustRead && !data.isRead)
        {
            var params="MSGID="+data.messageId;
            AjaxAPI.sendRequest({ URL: ZDBGlobal.CTX + ZDBGlobal.REPORTS_CONTEXT_NAME + "/reportsapi/SETMUSTREADANNOUNCEMENTREAD" ,PARAMETERS:params ,ONSUCCESSFUNC: self.setMustReadAnnouncementRead,INDEX:index });
        }
        self.removeAnnouncementIdFromUnreadAnnouncement(data.messageId);


        ZRUserAnalyticsAttacher.attachUAToSubAnnouncementLinks();    //handling UA in links
    };

    self.setMustReadAnnouncementRead = function(response,request) {
        self.json[request["INDEX"]].isRead=true;

        //handling announcement unread count badge
        var count=self.getUnreadCount()[3];
        count>0?self.annmtBadge.addClass('is-visible').text(count):self.annmtBadge.removeClass('is-visible');

    };

    self.removeElementSelection = function () {
        var currentEl = GlobalNotification.currentTab.first('li.current');
        if(currentEl) { currentEl.removeClass('current'); }
    };

    // Constructs the notification card
    self.constructCard = function (notificationEl) {
        var imgContainer = self.card.first('.image-container');
        var index = notificationEl.attr('data-index');
        var data = self.json[index];
        var cardHeader = self.card.first('#cardHeader');
        var cardBody = self.card.first('#cardBody');


        self.constructCardHeader(data, cardHeader);
        self.constructCardBody(data, cardBody, index);
        var cardFooter = self.card.first('#cardFooter');

        if(data.type == self.NOTI_SERVICE){
            self.card.first('.notification-card').element.style.display="none";
            self.card.element.style.paddingBottom='0px';
            if(data.message.footer)
            {
                self.card.element.style.borderBottom='36px solid #f7f7f7';
                cardFooter.element.style.display='';
                cardFooter.element.style.marginBottom='-26px';
                cardFooter.element.style.paddingTop='10px';
                cardFooter.first('#cardFooterText').toDOM().textContent = data.message.footertext;
                cardFooter.first('#cardFooterText').attr('data-index',index);
            }
            else
            {
                self.card.element.style.paddingBottom='10px';
                self.card.element.style.borderBottom='10px';
                cardFooter.element.style.display='none';
            }
        }
        else{
            self.card.element.style.paddingBottom='10px';
            self.card.first('.notification-card').element.style.display="";
            self.card.element.style.borderBottom='';
            cardFooter.element.style.display='none';
            self.setContactImage(imgContainer, data);
        }

    };

    // Constructs the header for the notification card
    self.constructCardHeader = function (data, cardHeader) {
        cardHeader.empty();
        if (data.type == self.NOTI_SHARE) {
            var nameEl = dom.new('span', { text: data.fromDisplayName, attributes: { class: 'strong-text' } });
            var viewsCount = data.viewsInfo.length;
            var actionEl = dom.new('span', { text: ' ' + ZDBI18NMap.g('zr.notification.js.shared') + ' ' });
            var countEl = dom.new('span', {
                text: (viewsCount > 1 ? viewsCount + ' ' + ZDBI18NMap.g('zr.notification.js.views') : ZDBI18NMap.g('zr.notification.js.view')) + ' with ' + (data.isSharedToGroup ? data.groupName : ZDBI18NMap.g('zr.notification.js.you')) + ' ' + ZDBI18NMap.g('zr.notification.js.in') + ' '
            });
            var dbLink = dom.new('a', {
                attributes: {
                    class: 'ZRSLink',
                    href: ZDBGlobal.CTX + ZDBGlobal.REPORTS_CONTEXT_NAME + '/ZDBDataSheetView.cc?DBID=' + data.dbId,
                    target: typeof CrmPlusImpl !== 'undefined' && CrmPlusImpl.isLoadedInCrmplus() ? '_blank' : '_self'
                },
                text: data.dbName
            });
            var txtEl = dom.new('span', { text: ' ' + ZDBI18NMap.g('zr.notification.js.database') });
            cardHeader.append(nameEl);
            cardHeader.append(actionEl);
            cardHeader.append(countEl);
            cardHeader.append(dbLink);
            cardHeader.append(txtEl);
        } else if (data.type == self.NOTI_GROUP) {
            var nameEl = dom.new('span', { text: data.fromDisplayName, attributes: { class: 'strong-text' }  });
            var actionEl = dom.new('span', { text: ' ' + ZDBI18NMap.g('zr.notification.js.added_you') + ' ' });
            var groupNameEl = dom.new('span', { text: data.groupName, attributes: { class: 'strong-text' }});
            var countEl = dom.new('span', { text: ' ' + ZDBI18NMap.g('zr.notification.js.grouptext') + ' ' + ZDBI18NMap.g('zr.notification.js.in') + ' ' });
            var dbLink = dom.new('a', {
                attributes: {
                    class: 'ZRSLink',
                    href: ZDBGlobal.CTX + ZDBGlobal.REPORTS_CONTEXT_NAME + '/ZDBDataSheetView.cc?DBID=' + data.dbId,
                    target: typeof CrmPlusImpl !== 'undefined' && CrmPlusImpl.isLoadedInCrmplus() ? '_blank' : '_self'
                },
                text: data.dbName
            });
            var txtEl = dom.new('span', { text: ' ' + ZDBI18NMap.g('zr.notification.js.database') });
            cardHeader.append(nameEl);
            cardHeader.append(actionEl);
            cardHeader.append(groupNameEl);
            cardHeader.append(countEl);
            cardHeader.append(dbLink);
            cardHeader.append(txtEl);
        } else if (data.type == self.NOTI_SCHEDULAR) {
            var nameEl = dom.new('span', { attributes: { class: 'strong-text' }, text: data.fromDisplayName });
            var actionEl = dom.new('span', { text: ' ' + ZDBI18NMap.g('zr.notification.js.schedular_failed') });
            cardHeader.append(nameEl);
            cardHeader.append(actionEl);
        } else if( data.type == self.NOTI_SERVICE ){

            var nameEl = dom.new('span', { attributes: { class: 'strong-text' }, text: data.fromDisplayName });
            var titleEl = dom.new('span', { attributes: { class: 'strong-text' }, text: ' '+data.message.title });
            var descEl = dom.new('span', { attributes: { class: 'strong-text' }, text: ' '+data.message.description });
            cardHeader.append(nameEl);
            cardHeader.append(titleEl);
            cardHeader.append(descEl);
        }

    };

    // Constructs the body for the notification card
    self.constructCardBody = function (data, cardBody, index) {
        cardBody.empty();
        if(data.type == self.NOTI_SHARE) {
            cardBody.show();
            var ul = dom.new('ul');
            for(var i = 0; i < data.viewsInfo.length; i++) {
                var view = data.viewsInfo[i];
                var li = dom.new('li');
                var iconCont = dom.new('div', { attributes: { class: 'card-icon-cont' } });
                var iconEl = dom.new('span', { attributes: { class: 'viewTypeIcon ' + self.viewTypeIcon[view.viewType] } });
                var viewCont = dom.new('div');
                var viewNameEl = dom.new('span', {
                    attributes: { class: 'ZRSLink', 'data-index': index, 'view-index': i },
                    text: view.displayName,
                    events: { click: self.loadView }
                });
                var viewDescEl = dom.new('div', {
                    attributes: { class: 'no-break', style: 'clear:both; padding: 5px 0;' },
                    text: view.description ? view.description : ''
                });
                var sharedOnEl = dom.new('div', {
                    attributes: { class: 'gray font10', style: 'margin-top: 5px;' },
                    text: self.getElapsedTime(view.sharedTime)
                });
                iconCont.append(iconEl);
                iconCont.append(viewNameEl);
                viewCont.append(viewDescEl);
                viewCont.append(sharedOnEl);
                li.append(iconCont);
                li.append(viewCont);
                ul.append(li);
            }
            cardBody.append(ul);
        } else if(data.type == self.NOTI_SERVICE){
            cardBody.show();

            var ul = dom.new('ul');
            for(var i = 0; i < data.message.announcements.length; i++) {
                var feature = data.message.announcements[i];
                var li = dom.new('li');
                var iconCont = dom.new('div', { attributes: { class: 'card-icon-cont' } });
                //var iconEl = dom.new('span', { attributes: { class: 'ndTypeIcon '} });
                var viewCont = dom.new('div',{attributes:{ style:'margin-top:8px' }});
                var viewNameEl = dom.new('span', {
                    attributes:{class:'sub-ann-title no-break',style:'margin-right:-10px'}
                });


                jQuery(viewNameEl.toDOM()).append(feature.title);


                var featureImgStyle="display:none";
                if(feature.imagelink != ""){
                    featureImgStyle="width:320px;padding-top:8px";
                }
                var featureImg = dom.new('img',{attributes:{src:feature.imagelink,style:featureImgStyle+""}});
                var featCont = dom.new('div',{attributes:{ style:'margin-top:8px ; margin-right:-10px' }});
                var viewDescEl = dom.new('div', {
                    attributes: { class: 'no-break', style: 'clear:both;text-align:justify;' }
                });
                jQuery(viewDescEl.toDOM()).append(feature.description);


                if(data.message.subannouncementstitlestyle!=undefined && data.message.subannouncementsdescstyle!=undefined){    //check for old JSON object
                if(data.message.subannouncementstitlestyle.size!=-1){
                    viewNameEl.addClass("font".concat(data.message.subannouncementstitlestyle.size-1)); }
                if(data.message.subannouncementstitlestyle.color!='default'){
                    viewNameEl.element.style.color=data.message.subannouncementstitlestyle.color; }

                if(data.message.subannouncementsdescstyle.size!=-1){
                    viewDescEl.addClass("font".concat(data.message.subannouncementsdescstyle.size-1)); }
                if(data.message.subannouncementsdescstyle.color!='default'){
                    viewDescEl.element.style.color = data.message.subannouncementsdescstyle.color; }
                }

                var learnMoreEl = dom.new('div', {
                    attributes: { class: 'ZRSLink no-break', 'data-index': index, 'feature-index': i , style:'text-align:right' },
                    text:' '+feature.linkholder ,
                    events: {click: self.loadHelpPage}
                });
                //iconCont.append(iconEl);
                iconCont.append(viewNameEl);
                featCont.append(viewDescEl);
                featCont.append(learnMoreEl);
                li.append(iconCont);
                li.append(featureImg);
                li.append(featCont);
                ul.append(li);
            }

            cardBody.append(ul);
        } else {
            var elapsedTimeEl = dom.new('div', {
                attributes: { class: 'gray', style: 'margin-top: 5px;' },
                text: self.getElapsedTime(data.time)
            });
            cardBody.append(elapsedTimeEl);
        }
    };

    self.getElapsedTime = function (timestamp) {
        var elapsedTime = '';
        var userTime = timestamp + GlobalNotification.userTimeZone;
        var userDate = new Date(userTime);
        var currDate = new Date();

        var diffDays = self.getDifferenceInDays(userDate, currDate);
        if(diffDays < self.DAYS_IN_WEEK) {
            var i18nKey = (diffDays == 0) ? 'zrpt.za.analysis.filter.js.today' : ((diffDays == 1) ? 'zrpt.za.analysis.filter.js.yesterday' : self.days[userDate.getUTCDay()]); // No I18N
            var day = ZDBI18NMap.g(i18nKey);
            var time = self.extractTime(userDate);
            elapsedTime = day + ' ' + ZDBI18NMap.g('zr.notification.js.at') + ' ' + time; // No I18N
        } else {
            var month = ZDBI18NMap.g(self.months[userDate.getUTCMonth()]);
            var time = self.extractTime(userDate);
            elapsedTime = month + ' ' + userDate.getUTCDate() + ' ' + ZDBI18NMap.g('zr.notification.js.at') + ' ' + time; // No I18N;
        }
        return elapsedTime;
    };

    self.extractTime = function (date) {
        var hours = date.getUTCHours();
        hours = (hours < 10) ? '0' + hours : hours; // No I18N
        var minutes = date.getUTCMinutes();
        minutes = (minutes < 10) ? '0' + minutes : minutes; // No I18N
        var time = hours + ':' + minutes;
        return time;
    };

    self.getDifferenceInDays = function (date1, date2) {
        var utc1 = Date.UTC(date1.getFullYear(), date1.getMonth(), date1.getDate());
        var utc2 = Date.UTC(date2.getFullYear(), date2.getMonth(), date2.getDate());
        return Math.floor((utc2 - utc1) / self.DAY_IN_MILLISECONDS);
    };

    self.loadView = function (event) {
        var el = dom(event.currentTarget);
        var index = el.attr('data-index');
        var viewIndex = el.attr('view-index');
        var data = self.json[index];
        var view = data.viewsInfo[viewIndex];
        var url = ZDBGlobal.CTX + ZDBGlobal.REPORTS_CONTEXT_NAME
                + '/ZDBDataSheetView.cc?DBID=' + data.dbId
                + '&OBJID=' + view.viewId;
        if(ZDBClientUI.dbId == undefined) {
        	if(typeof CrmPlusImpl !== 'undefined' && CrmPlusImpl.isLoadedInCrmplus()){
        		window.open(url, '_blank');
        		window.focus();
        	}else{
        		window.location = url;
        	}
        } else if (ZDBClientUI.dbId != data.dbId) {
            window.open(url, '_blank');
            window.focus();
        } else {
            var tab = ZDBClientUI.getExistingTab(view.viewId);
            if(tab != null)
            {
                ZDBClientUI.selectTab(tab);
                self.hide();
                return;
            }
            ZDBClientUI.fetchView(view.viewId, data.dbId, view.displayName, self.viewTypeIcon[view.viewType]);
            ZDBNavigation.refreshExplorer();
            self.hide();
        }
    };

    self.loadHelpPage = function(event) {
        var el = dom(event.currentTarget);
        var url="#";
        var index = el.attr('data-index');
        var data = self.json[index];

        if(el.element.id=="cardFooterText"){
            url=data.message.footerlink;
        }
        else{
            var featureIndex = el.attr('feature-index');
            var feature = data.message.announcements[featureIndex];
            url=feature.linkvalue;
        }
        window.open(url,'_blank');
    }

    // Creates the new notification element when the new server event is triggered
    self.emit = function (data) {
        self.new(data, self.json.length, true, true);
        self.showBadge(self.getUnreadCount());
        typeof ZRCRMPlus !== 'undefined' && ZRCRMPlus.showBadge(self.getUnreadCount()[0]);
        if(self.container.hasClass('is-visible')) {
            self.markAsRead(self.currentTab.element.id=='notifications'?self.NOTIFICATION_TYPE:self.ANNOUNCEMENT_TYPE);
        }
        if(self.card.hasClass('is-visible')) {
            self.card.removeClass('is-visible');
            self.currentTab.first('.current').removeClass('current');
        }
    };

    // Shows the badge with number of unread notifications
    self.showBadge = function (notfCount) {
        var count = notfCount[0];
        var gnCount = notfCount[1];
        var annmtCount = notfCount[2]+notfCount[3];

        if(count > 0) {
            self.badge.addClass('is-visible').text(count);
            (gnCount > 0) ? self.gnBadge.addClass('is-visible').text(gnCount) : self.gnBadge.removeClass('is-visible');
            (annmtCount >0) ? self.annmtBadge.addClass('is-visible').text(annmtCount) : self.annmtBadge.removeClass('is-visible');
            if(annmtCount>0){
                self.rippleEffect.addClass('is-visible');
            }
        }
        else{
            self.badge.removeClass('is-visible');
            self.rippleEffect.removeClass('is-visible');
            self.gnBadge.removeClass('is-visible');
            self.annmtBadge.removeClass('is-visible');
        }
    };

    // Hides the badge
    self.hideBadge = function () {
        self.badge.removeClass('is-visible');
    };

    // Handles the scroll, for lazy load. Whenever it reaches the end of the list,
    // It will invoke the loadMore method
    self.handleScroll = function (event) {
        var el = event.currentTarget;
        if(el.scrollTop + el.offsetHeight >= el.scrollHeight) {
            self.loadMore();
        }
        if(self.card.hasClass('is-visible')) {
            self.card.removeClass('is-visible');
            self.currentTab.first('.current').removeClass('current');
        }
    };

    // Need to be implmented (for Lazy Loading)
    self.loadMore = function () {
    };

    self.hide = function () {
        self.container.removeClass('is-visible');
        self.container.first('.panel-1').removeClass('inactive'); // No I18N
        self.container.removeClass('service-notification');
        self.card.removeClass('is-visible');
        self.settings.removeClass('is-visible');
        self.freezeLayer.removeClass('is-visible');
        self.removeElementSelection();
        self.showBadge(self.getUnreadCount());
        typeof ZRCRMPlus !== 'undefined' && ZRCRMPlus.showBadge(self.getUnreadCount()[0]);
    };

    // closes the notification component
    self.close = function (event) {
        if(event.target == $('notificationCont') || event.target == $('closeNotfIcon')) { self.hide(); return; }

        if(event.currentTarget != self.notifications.toDOM() &&  event.currentTarget != self.announcements.toDOM() && event.currentTarget!=self.card.toDOM()) {
            self.card.removeClass('is-visible');
            self.removeElementSelection();
        }
    };


    // Persists the notification preference on change
    self.persistPreference = function (targetEl, resend) {
        var el = dom(targetEl);
        var params = '';
        if(el.attr('data-parent')) {
            var children = self.settings.find('[data-child-of=' + targetEl.name + ']');
            for(var i = 0; i < children.length; i++)  {
                children[i].checked = targetEl.checked;
                params += ((i != 0) ? '&' : '') + children[i].name + '=' + children[i].checked;
            }
        } else if(el.attr('data-child-of')) {
            var parent = self.settings.first('[name=' + el.attr('data-child-of') + '');
            var total = self.settings.find('[data-child-of=' + parent.attr('name') + ']').length;
            var checked = self.settings.find('[data-child-of=' + parent.attr('name') + ']:checked').length;
            parent.toDOM().checked = (total == checked);
            params = targetEl.name + '=' + targetEl.checked;
        } else {
            params = targetEl.name + '=' + targetEl.checked;
        }
        if(!resend) {
            var reqURL = '/reportsapi/SETNOTIFICATIONPREFERENCES?' + params;
//            AjaxAPI.sendRequest({ URL:ZDBGlobal.CTX + ZDBGlobal.REPORTS_CONTEXT_NAME + reqURL, ONFAILUREFUNC: self.backToOldState, TARGETEL: targetEl });
            ZRSAjax.post(ZDBGlobal.CTX + ZDBGlobal.REPORTS_CONTEXT_NAME + reqURL, null, {
                failureHandler: {
                    func: self.backToOldState,
                    ctx: self
                },
                TARGETEL: targetEl,
                noLoader: true
            });
        }
    };

    self.backToOldState = function(response, request) {
        request.TARGETEL.checked = !(request.TARGETEL.checked);
        self.persistPreference(request.TARGETEL, true);
    };

    self.getRelativePosition = function(elem) {
        var box = elem.getBoundingClientRect();

        var body = document.body;
        var docEl = document.documentElement;

        var scrollTop = window.pageYOffset || docEl.scrollTop || body.scrollTop;
        var scrollLeft = window.pageXOffset || docEl.scrollLeft || body.scrollLeft;

        var clientTop = docEl.clientTop || body.clientTop || 0;
        var clientLeft = docEl.clientLeft || body.clientLeft || 0;

        var top  = box.top +  scrollTop - clientTop;
        var left = box.left + scrollLeft - clientLeft;

        return { x: Math.round(left), y: Math.round(top) };
    };

    self.switchTab = function (event,tab) {
        self.card.removeClass('is-visible');
        self.removeElementSelection();

        self.currentTab = self.notifications;
        var tabTitle = ZDBI18NMap.g('zr.notification.js.title');
        var tabType = self.NOTIFICATION_TYPE;
        if(tab) {

            self.container.addClass(tab);
            self.currentTab = self.announcements;
            tabTitle = ZDBI18NMap.g('zr.announcement.js.title');
            tabType = self.ANNOUNCEMENT_TYPE;
            if(event){
                self.gnBadge.removeClass('is-visible');
            }
            setTimeout(function(){
                var count=self.getUnreadCount()[3];
                count>0?self.annmtBadge.addClass('is-visible').text(count):self.annmtBadge.removeClass('is-visible');
            },5000);

            if(self.getUnreadCount()[0]>0){  //checking if any unread announcements present
                ZRUserAnalytics.immediatePushToServer("VisitAnnouncementTab","Announcement","Support",null,{unreadAnnouncementIdList:self.unreadAnnouncementIds});
            }
        }else {

             self.container.removeClass('service-notification');
            if(event){
                var count=self.getUnreadCount()[3];
                count>0?self.annmtBadge.addClass('is-visible').text(count):self.annmtBadge.removeClass('is-visible');
            }
            setTimeout(function(){self.gnBadge.removeClass('is-visible');},5000);
        }

        self.markAsRead(tabType);
        self.noNotificationEl.toDOM().style.display = (self.currentTab.toDOM().childElementCount == 0) ? 'block' : 'none';
        self.container.first('.notification-type').text(tabTitle);
    };

};
//ignorei18n_end
