//$Id$
/** DOM Helper Scripts **/

var DEBUG = false;

var dom = function(selector) {
    return new dom.init(selector);
};

dom.init = function(selector) {
    if (Array.isArray(selector)) {
        this.element = selector;
    } else if (selector.type && selector.type == 'dom') {
        return selector;
    } else if (dom.isHTMLElement(selector)) {
        this.element = selector;
    } else {
        this.element = [];
        var nodes = document.querySelectorAll(selector);
        if (nodes.length == 1) { this.element = nodes[0]; }
        else { for (var i = 0; i < nodes.length; i++) { this.element.push(dom(nodes[i])); } }
    }
};

dom.init.prototype.type = 'dom';

dom.construct = function(template) {
    var parent = document.createElement('div');
    this.constructHTML(template, parent);
    return new dom.init(parent.childNodes[0]);
};

dom.constructHTML = function(obj, parent) {
    if (this.isHTMLElement(obj) && parent) {
        parent.appendChild(obj);
        return;
    }
    var element = document.createElement(obj.tagname);
    if (!parent) {
        parent = element;
    }
    if (obj.children) {
        for (var i = 0; i < obj.children.length; i++) {
            this.constructHTML(obj.children[i], element);
        }
    }
    for (var prop in obj.attributes) {
        element.setAttribute(prop, obj.attributes[prop]);
    }
    for (var event in obj.events) {
        element.addEventListener(event, obj.events[event]);
    }
    if (obj.text || obj.i18n) {
        var txt = (obj.text) ? obj.text : ZDBI18NMap.g(obj.i18n);
        var textNode = document.createTextNode(txt);
        element.appendChild(textNode);
    } else if (obj.html) {
        element.innerHTML = obj.html;
    }
    parent.appendChild(element);
};

dom.fill = function(template, json) {
    for (key in json) {
        var attrName = key;
        var value = json[attrName];
        var elements = template.querySelectorAll('[' + attrName + ']');
        if (elements.length > 0) {
            for (var i = 0; i < elements.length; i++) {
                var el = dom(elements[i]);
                // Checks if the value is valid dom
                if (this.isHTMLElement(value)) {
                    el.empty();
                    el.append(value);
                } else {
                    var textNode = document.createTextNode(value);
                    el.text('');
                    el.append(textNode);
                }
            }
        }
    }
};

dom.new = function(elname, json) {
    var element = document.createElement(elname);
    if (json) {
        if (json.attributes) {
            for (key in json.attributes) {
                element.setAttribute(key, json.attributes[key]);
            }
        }

        if (json.text || json.i18n) {
            var txt = (json.text) ? json.text : ZDBI18NMap.g(json.i18n);
            var textNode = document.createTextNode(txt);
            element.appendChild(textNode);
        } else if (json.html) {
            element.innerHTML = json.html;
        }

        if (json.events) {
            for (var event in json.events) {
                element.addEventListener(event, json.events[event]);
            }
        }
    }
    return dom(element);
};

dom.isHTMLElement = function(obj) {
    try {
        return obj instanceof HTMLElement;
    } catch (e) {
        // For all older browsers
        return (typeof obj === 'object') && (obj.nodeType === 1) && (typeof obj.style === 'object') && (typeof obj.ownerDocument === 'object');
    }
};

dom.init.prototype.eq = function(index) {
    return Array.isArray(this.element) ? this.element[index] : this.element;
};

dom.init.prototype.hasClass = function(className) {
    if (this.throwNotSupported()) return;
    return this.element.className.match(new RegExp('(\\s|^)' + className + '(\\s|$)'));
};

dom.init.prototype.addClass = function(className) {
    var isArray = Array.isArray(this.element);
    if (isArray) {
        for(var i = 0; i < this.element.length; i++) {
            if(!this.element[i].hasClass(className)){
                this.element[i].className += ' ' + className;
            }
        }
    } else {
        if (!this.hasClass(className)) {
            this.element.className += ' ' + className;
        }
    }
    return this;
};

dom.init.prototype.removeClass = function(className) {
    var isArray = Array.isArray(this.element);
    var regex = new RegExp('(\\s|^)' + className + '(\\s|$)');
    if (isArray) {
        for(var i = 0; i < this.element.length; i++) {
            if (!this.element[i].hasClass(className)) { this.element.className = this.element.className.replace(regex, ' '); }
        }
    } else {
        // Can do like this also, But need to check the support
        // element.classList.remove(className);
        if (this.hasClass(className)) { this.element.className = this.element.className.replace(regex, ' '); }
    }
    return this;
};

dom.init.prototype.toggleClass = function(className) {
    var isArray = Array.isArray(this.element);
    if (isArray) {
        for(var i = 0; i < this.element.length; i++) {
            (this.element[i].hasClass(className)) ? this.element[i].removeClass(className): this.element[i].addClass(className);
        }
    } else {
        (this.hasClass(className)) ? this.removeClass(className): this.addClass(className);
    }
    return this;
};

dom.init.prototype.append = function(child) {
    if (this.throwNotSupported()) return;
    if (child instanceof dom.init) {
        child = child.element;
    }
    this.element.appendChild(child);
    return this;
};

dom.init.prototype.prepend = function(child, index) {
    if (this.throwNotSupported()) return;
    if (child instanceof dom.init) {
        child = child.element;
    }
    if (index) {
        this.element.insertBefore(child, this.element.childNodes[index]);
    } else {
        this.element.insertBefore(child, this.element.childNodes[0]);
    }
    return this;
};

dom.init.prototype.empty = function() {
    if (this.throwNotSupported()) return;
    while (this.element.firstChild) this.element.removeChild(this.element.firstChild);
    return this;
};

dom.init.prototype.first = function(selector) {
    if (this.throwNotSupported()) return;
    var el = this.element.querySelector(selector);
    return ((el) ? (new dom.init(el)) : null);
};

dom.init.prototype.find = function(selector) {
    if (this.throwNotSupported()) return;
    return this.element.querySelectorAll(selector);
};

dom.init.prototype.dFind = function(selector) {
    if (this.throwNotSupported()) return;
    var nodes = this.find(selector);
    var element = [];
    for(var i = 0; i < nodes.length; i++) {
        element.push(dom(nodes[i]));
    }
    return dom(element);
};

dom.init.prototype.filter = function(selector) {
    // Not supported yet
};

dom.init.prototype.show = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) {
        for(var i = 0; i < this.element.length; i++) {
            this.element[i].toDOM().style.display = '';
        }
    } else {
        this.element.style.display = '';
    }
    return this;
};

dom.init.prototype.hide = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) {
        for(var i = 0; i < this.element.length; i++) {
            this.element[i].toDOM().style.display = 'none';
        }
    } else {
        this.element.style.display = 'none';
    }
    return this;
};

dom.init.prototype.toggle = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) {
        for(var i = 0; i < this.element.length; i++) {
            this.element[i].toDOM().style.display = (this.element[i].toDOM().style.display == '') ? 'none' : '';
        }
    } else {
        this.element.style.display = (this.element.style.display == '') ? 'none' : '';
    }
    return this;
};

dom.init.prototype.index = function(child) {
    if (this.throwNotSupported()) return;
    for (var i = 0; element = this.element.previousElementSibling; i++);
    return i;
};

dom.init.prototype.parent = function(className) {
    if (this.throwNotSupported()) return;
    if (!className) {
        return dom(this.element.parentNode);
    }
    var element = this.element;
    while ((element = element.parentElement) && !dom(element).hasClass(className));
    return element;
};

dom.init.prototype.closest = function(elName) {
    if (this.throwNotSupported()) return;
    var element = this.element;
    while ((element = element.parentElement) && (element.tagName.toLowerCase() != elName));
    return dom(element);
};

dom.init.prototype.dParent = function(className) {
    return dom(this.parent(className));
};

dom.init.prototype.text = function(value) {
    if (this.throwNotSupported()) return;
    if (value != undefined) {
        this.element.textContent = '';
        this.element.appendChild(document.createTextNode(value));
        return this;
    } else {
        return this.element.textContent;
    }
};

dom.init.prototype.html = function(value) {
    if (this.throwNotSupported()) return;
    if (value != undefined) {
        this.element.innerHTML = value;
        return this;
    } else {
        return this.element.innerHTML;
    }
};

dom.init.prototype.i18n = function(value) {
    if (this.throwNotSupported()) return;
    return this.text(ZDBI18NMap.g(value));
};

dom.init.prototype.attr = function(attribute, value) {
    if (this.throwNotSupported()) return;
    if (value != undefined) {
        this.element.setAttribute(attribute, value);
        return this;
    } else {
        return this.element.getAttribute(attribute);
    }
};

dom.init.prototype.removeAttr = function(attribute) {
    if (this.throwNotSupported()) return;
    this.element.removeAttribute(attribute);
    return this;
};

dom.init.prototype.remove = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) { for(var i = 0; i < this.element.length; i++) { this.element[i].toDOM() && this.element[i].toDOM().remove(); } }
    else { this.element && this.element.remove(); }
};

dom.init.prototype.css = function(json) {
    if (this.throwNotSupported()) return;
    for (var key in json) { this.element.style[key] = json[key]; }
    return this;
};

dom.init.prototype.length = function() {
    return Array.isArray(this.element) ? this.element.length : 1;
};

dom.init.prototype.isNull = function() {
    if (this.throwNotSupported()) return;
    return (this.length > 1) ? false : true;
};

dom.init.prototype.on = function(event, method) {
    var isArray = Array.isArray(this.element);
    if (isArray) { for(var i = 0; i < this.element.length; i++) { this.element[i].toDOM().addEventListener(event, method); } }
    else { this.element.addEventListener(event, method); }
    return this;
};

dom.init.prototype.toDOM = function() {
    if (this.throwNotSupported()) return;
    return this.element;
};

dom.init.prototype.getIndex = function() {
    if (this.throwNotSupported()) return;
    var el = this.element;
    for (var i = 0; el = el.previousElementSibling; i++);
    return i;
};

dom.init.prototype.fadeOutRemove = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) { for(var i = 0; i < this.element.length; i++) { this.element[i].toDOM().remove(); } }
    else { this.element.remove(); }
};

dom.init.prototype.children = function() {
    if (this.throwNotSupported()) return;
    return this.element.children;
};

dom.init.prototype.height = function(value) {
    if (this.throwNotSupported()) return;
    if (value != undefined) {
        this.element.style.height = value + 'px';
        return this;
    } else {
        return this.element.offsetHeight;
    }

};

dom.init.prototype.width = function(value) {
    if (this.throwNotSupported()) return;
    if (value != undefined) {
        this.element.style.width = value + 'px';
        return this;
    } else {
        return this.element.offsetWidth;
    }
};

dom.init.prototype.val = function(value) {
    if (this.throwNotSupported()) return;
    if (value != undefined) {
        this.element.value = value;
        return this;
    } else {
        return this.element.value;
    }
};

dom.init.prototype.clone = function(cloneChildren) {
    if (this.throwNotSupported()) return;
    return dom(this.element.cloneNode(cloneChildren ? true : false));
};

dom.init.prototype.checked = function(state) {
    if (this.throwNotSupported()) return;
    if (state != undefined) {
        this.element.checked = state;
        return this;
    } else {
        return this.element.checked;
    }
};

dom.init.prototype.focus = function() {
    if (this.throwNotSupported()) return;
    this.element.focus();
    return this;
};

dom.init.prototype.blur = function() {
    if (this.throwNotSupported()) return;
    this.element.blur();
    return this;  
};

dom.init.prototype.dFadeIn = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) { for(var i = 0; i < this.element.length; i++) { this.element[i].addClass('is-active'); } }
    else { this.addClass('is-active'); }
    return this;
};

dom.init.prototype.dFadeOut = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) { for(var i = 0; i < this.element.length; i++) { this.element[i].removeClass('is-active'); } }
    else { this.removeClass('is-active'); }
    return this;
};

dom.init.prototype.isActive = function() {
    if (this.throwNotSupported()) return;
    return this.hasClass('is-active'); // No I18N
};

dom.init.prototype.remEvent = function(event, method) {
    var isArray = Array.isArray(this.element);
    if (isArray) { for(var i = 0; i < this.element.length; i++) { this.element[i].toDOM().removeEventListener(event, method); } }
    else { this.element.removeEventListener(event, method); }
};

dom.init.prototype.click = function() {
    if (this.throwNotSupported()) return;
    this.element.click();
    return this;
};

dom.init.prototype.replace = function(newNode) {
    if (this.throwNotSupported()) return;
    if (newNode.type && newNode.type == 'dom') {
        newNode = newNode.element;
    }
    this.element.parentElement.replaceChild(newNode, this.element);
    return dom(newNode);
};

dom.init.prototype.replaceSelf = function() {
    if (this.throwNotSupported()) return;
    var duplicate = this.clone(true);
    this.replace(duplicate);
    return duplicate;
};

dom.init.prototype.fadeOut = function() {
    var isArray = Array.isArray(this.element);
    if (isArray) { for(var i = 0; i < this.element.length; i++) { this.element[i].addClass('enable-transition').addClass('zr-fade-out'); } }
    else { this.addClass('enable-transition').addClass('zr-fade-out'); }
    return this;
};

dom.init.prototype.prev = function() {
    if (this.throwNotSupported()) return;
    return dom(this.element.previousElementSibling);
};

dom.init.prototype.next = function() {
    if (this.throwNotSupported()) return;
    return dom(event.currentTarget.nextElementSibling);
};

dom.init.prototype.throwNotSupported = function() {
    if (Array.isArray(this.element) && DEBUG) {
        console.log('This action not supported on multiple nodes') || console.trace(); // No I18N
    }
    return Array.isArray(this.element);
};
