(function(global) {
    
    "use strict";
    
    var tp = global.tooltip || {},
        themes = {},
        clone = global.utils.simpleClone || function(obj) {
            return JSON.parse(JSON.stringify(obj))
        };

    function merge(target, object) {
        if (typeof target !== 'object') {
            target = {};
        }
        for (var p in object) {
            if(object.hasOwnProperty(p)) {
                if (typeof object[p] === "object") {
                    target[p] = merge(target[p], object[p]);

                } else {
                    target[p] = object[p];
                }
            }
        }
        return target;
    }


    var defaultTheme = {
        "borderStyle": {
            "borderStyle": "solid",
            "borderRadius": "0",
            "borderWidth": "4 1 1 1",
            "backgroundColor": "white",
            "fontColor": "#001",
            "padding": 0
        },
        "header": {
            "show": true,
            "orient": "left",
            "borderColor": "",
            "size": 7,
            "radial": false,
            "showBorder": false,
            "vAlign": "middle",
            "hAlign": "center",
            "contentBackground": "inherit",
            "background": "inherit",
            "fontColor": "inherit"
        },
        "content": {
            "show": true,
            "background": "inherit",
            "fontColor": "inherit",
            "fontSize": "1em"
        }
    };
    
    /**
     *
     * @param {String} orient
     * @returns {boolean}
     */
    
    var isVertical = function(orient) {
        return !(orient == "left" || orient == "right");
    };
    
    
    /**
     *
     * @param {Array} array
     * @param {Number} delimiter
     * @returns {Array}
     */
    
    var splitArray = function(array, delimiter) {
        var base = [];
        var _tmp = [];
        array.forEach(function(e) {
            if (e == delimiter) {
                if (_tmp.length) {
                    base.push(_tmp);
                }
                _tmp = [];
                base.push([delimiter])
            } else if (typeof e === "string") {
                if (_tmp.length) {
                    base.push(_tmp);
                }
                base.push([e]);
                _tmp = [];
            } else {
                _tmp.push(e);
            }
        });
        if (_tmp.length) {
            base.push(_tmp);
        }
        return base;
    };
    
    /**
     *
     * @param {Array|Number|String} item
     * @param {String}type
     * @returns {string}
     */
    
    var patternBuilder = function(item, type) {
        var multi = Array.isArray(item);
        var pattern = type === "column" ? "{{col(-d)}} " : "{{val(-d)}} ";
        var stringBuilder = "";
        if (multi) {
            item.forEach(function(index) {
                stringBuilder = stringBuilder.concat(pattern.replace(/-d/ig, index))
            });
        } else {
            stringBuilder = stringBuilder.concat(pattern.replace(/-d/ig, item))
        }
        return stringBuilder;
    };
    
    /**
     *
     * @param {String} name
     * @param {Array} items
     * @param {Number} parentWidth
     * @param {Number|String} staticMaxWidth
     * @returns {{body: string, maxWidth: number}}
     */
    
    var htmlBuilder = function(name, items, parentWidth, staticMaxWidth) {
        
        var fullContent = '<table style="border-spacing: 0; display: block; max-width: -mxw-;"> <tbody><tr>',
            roundBox = "",
            contentTable = "";
        
        var theme = themes[name];
        var orient = theme.header.orient;
        staticMaxWidth = staticMaxWidth && staticMaxWidth;
        
        // max width
        var width_ratio = 0.5;
        var headerMaxWidth = 0,
            contentMaxWidth = 0;
        var containerMaxWidth;
        
        if (theme.header.show && !isVertical(orient)) {
            containerMaxWidth = staticMaxWidth || parentWidth * 0.50;
            headerMaxWidth = containerMaxWidth * 0.32;
            contentMaxWidth = containerMaxWidth * 0.68;
        } else {
            containerMaxWidth = contentMaxWidth = staticMaxWidth || parentWidth * width_ratio;
        }
        contentMaxWidth -= 30;
        var contentLeftWidth = contentMaxWidth * 0.4,
            contentRightWidth = contentMaxWidth * 0.6;
        
        if (theme.header.show) {
            var size = theme.header.size * 10 + "px",
                borderColor = theme.header.borderColor || "{{clr}}",
                radial = theme.header.radial,
                vAlign = theme.header.vAlign,
                hAlign = theme.header.hAlign,
                fontColor = theme.header.fontColor,
                bg1 = theme.header.contentBackground,
                bg2 = theme.header.background,
                radius = 0;
            
            
            var item = items.shift();
            var itemIsString = typeof item === "string" || typeof item[0] === "string";
            
            if (!isVertical(orient)) {
                var em1 = 1,
                    em2 = 1.5;
                var totalEm = itemIsString ? em2 : em1 + em2;
                var sharePadding = (totalEm / 2) + "em";
                var innerPadding = "calc(50% - 2px - " + sharePadding + ");" + "-webkit-calc(50% - 2px - " + sharePadding + ")" + "-moz-calc(50% - 2px - " + sharePadding + ")";
                var styles = "",
                    style1 = "";
                
                if (!radial) {
                    innerPadding = "0px";
                    styles = "padding: 0;display: table;";
                    style1 = "display: table-cell";
                } else {
                    radius = "50%";
                }
                
                var border = theme.header.showBorder ? (orient === "left" ? "border-right: 1px solid " + borderColor : "border-left: 1px solid " + borderColor) : "";
                var roundBoxStart = ' <td style="padding: 5px; -brd-; background: -bg2-; vertical-align: -va-;"> <div style="max-width: -mxw-;color: -cl1-; background: -bg1-;padding: 0px; padding-top: -pdt-; padding-bottom: -pdb-; border-width: 1px;border-color: -c-;border-style: solid;-webkit-border-radius: -r-;-moz-border-radius: -r-;border-radius: -r-;font-weight: bold;min-width: -s-; ' + styles + '">' +
                    '<div style="max-width: -mxw-; overflow: hidden; white-space: nowrap; vertical-align: -va-; -ms-text-overflow: ellipsis;text-overflow: ellipsis;text-align: -ha-;' + style1 + '">';
                var roundBoxContent = '<span style="font-size: 1em">-col-</span> <br> <span style="font-size: 1.5em">-val-</span>';
                var roundBoxEnd = '</div> </div></td>';
                
                
                if (itemIsString) {
                    item = typeof item === "string" ? item : item[0];
                    roundBox = roundBoxStart + '<span style="font-size: 1.5em">' + item + '</span>' + roundBoxEnd;
                } else {
                    roundBox = roundBoxStart + roundBoxContent + roundBoxEnd;
                    roundBox = roundBox.replace(/-col-/ig, patternBuilder(item, "column"))
                        .replace(/-val-/ig, patternBuilder(item, "value"))
                }
            } else {
                var borderBottom = "border-bottom: 2px solid {{clr}}; padding-bottom: 5px; margin-bottom: 5px;";
                if (itemIsString) {
                    roundBox = '<tr style="color: -cl1-; font-size: 1.35em; font-weight: bold"><td colspan="2" style="max-width: -mxwl-; vertical-align: top;padding-right: 10px; position: relative;' + borderBottom + '">' + item + '</td>';
                } else {
                    roundBox = '<tr style="color: -cl1-; font-size: 1.35em; font-weight: bold"><td style="max-width: -mxwl-; vertical-align: top;padding-right: 10px; position: relative;' + borderBottom + '">-col-<span style="position:absolute; right: 0; top: 0;">:</span></td> <td style="padding-left: 10px;max-width: -mxwr-; vertical-align: top;' + borderBottom + '">-val-</td></tr>';
                    roundBox = roundBox.replace(/-col-/ig, patternBuilder(item, "column"))
                        .replace(/-val-/ig, patternBuilder(item, "value"))
                }
            }
            
            roundBox = roundBox.replace(/-s-/ig, size)
                .replace(/-c-/ig, borderColor)
                .replace(/-r-/ig, radius)
                .replace(/-brd-/ig, border)
                .replace(/-va-/ig, vAlign)
                .replace(/-ha-/ig, hAlign)
                .replace(/-pdt-/ig, innerPadding)
                .replace(/-pdb-/ig, innerPadding)
                .replace(/-bg1-/ig, bg1)
                .replace(/-bg2-/ig, bg2)
                .replace(/-cl1-/ig, fontColor)
                .replace(/-mxw-/ig, headerMaxWidth + "px")
        }
        
        if (theme.content.show) {
            
            var showHeaderContent = theme.header.show && isVertical(orient);
            
            var fc = theme.content.fontColor,
                bg = theme.content.background,
                fz = theme.content.fontSize;
            
            contentTable = '<td style="padding: 5px 10px;color: -fc-; background: -bg-"> <table style="display: block;border-spacing: 0; line-height: ; font-size: -fz-;">';
            
            if (showHeaderContent) {
                contentTable += roundBox;
            }
            
            var column = '<tr><td style="max-width: -mxwl-; vertical-align: top;padding-right: 10px; position: relative;">-col-<span style="position:absolute; right: 0; top: 0;">:</span></td> <td style="padding-left: 10px;max-width: -mxwr-; vertical-align: top;">-val-</td></tr>';
            
            items.forEach(function(item) {
                if (typeof item === "string" || typeof item[0] === "string") {
                    item = typeof item === "string" ? item : item[0];
                    contentTable += "<tr><td colspan='2' style='max-width: -mxwstr-'>" + item + "</td></tr>"
                } else {
                    contentTable += column.replace(/-col-/ig, patternBuilder(item, "column")).replace(/-val-/ig, patternBuilder(item, "value"));
                }
            });
            
            contentTable += '</tr></table> </td>';
            contentTable = contentTable.replace(/-fc-/ig, fc)
                .replace(/-bg-/ig, bg)
                .replace(/-mxwl-/ig, contentLeftWidth + "px")
                .replace(/-mxwr-/ig, contentRightWidth + "px")
                .replace(/-fz-/ig, fz)
        }
        
        if (orient === "left") {
            if (!isVertical(orient)) {
                fullContent += roundBox;
            }
            fullContent += contentTable;
        } else {
            fullContent += contentTable;
            if (!isVertical(orient)) {
                fullContent += roundBox;
            }
            
        }
        fullContent += '</tr> </tbody></table>';
        fullContent = fullContent.replace(/-mxw-/ig, containerMaxWidth + "px");
        
        return {
            "body": fullContent,
            "maxWidth": containerMaxWidth
        };
    };
    
    /**
     *
     * @param {String} name
     * @param {Object} chartObj
     * @returns {Object|null}
     */
    
    tp.getTheme = function(name, chartObj) {
        name = name.trim();
        if (!themes[name]) {
            return null
        }
        
        var axes = chartObj.userdata.metadata.axes,
            data = clone(axes.tooltip),
            xAxisOrder = axes.x[0],
            mergedAxes = (function() {
                return !axes.y.every(function(item) {
                    return item.length == 1;
                });
            })(),
            isMulti = chartObj.ismultiy || mergedAxes;
        
        if (isMulti) {
            data = splitArray(data, xAxisOrder);
        }
        
        var content = htmlBuilder(name, data, chartObj.canvasarea.width, chartObj.systemConf.tooltip.maxWidth);
        
        var generatedTheme = clone(themes[name]);
        generatedTheme.body = content.body;
        generatedTheme.maxWidth = content.maxWidth;
        return generatedTheme;
    };
    
    tp.setTheme = function(name, data) {
        if (arguments.length !== 2) {
            return false;
        }
        themes[name] = merge(data, clone(defaultTheme));
    };
    
    // create some theme
    
    tp.setTheme("theme1", {
        "header": {
            "radial": true
        }
    });
    tp.setTheme("theme2", {
        "header": {
            "orient": "right",
            "radial": true
        }
    });
    
    tp.setTheme("theme3", {
        "header": {
            "orient": "top",
            "fontColor": "{{clr}}"
        }
    });
    
    tp.setTheme("theme4", {
        "header": {
            "show": false
        }
    });
    
    tp.setTheme("theme5", {
        "header": {
            "show": true,
            "showBorder": true,
            "fontColor": "{{clr}}",
            "borderColor": "transparent",
            "hAlign": "left",
            "vAlign": "top"
        }
    });
    
    global.tooltip = tp;
})($ZC);