//$Id$
$ZC.annotation = function() {
};
$ZC.annotation.draw = function(chart, notesArray) {
    "use strict";
    
    var notes = chart.systemConf.notes,
        notesEnabled = defined(notes) && notes.enabled === true;
    
    if (notesEnabled) {
        var isPolar = chart.dataObject.isPolarAxisCategory,
            chartAnnotations = pick(notes.chartValues, []);
        
        chartAnnotations.forEach(function(d) {
            $ZC.annotation.addNotes(chart, {
                "coordinate": [d.x, d.y],
                "conf": d
            });
        });
        
        if (chart.isAxisCategory || isPolar) {
            for (var i = 0, len = chart.seriesdata.length; i < len; i++) {
                var renderer = chart.renderer[i];
                if (chart.plot.renderer.mode === "image") {
                    renderer_helpers.setRendererProp(chart, renderer, chart.seriesdata, i, {
                        image: true
                    });
                }
                var rp = chart.rendererConf[i].params,
                    xy;
                var xscale = rp.xscale,
                    yscale = rp.yscale,
                    stacked = rp.stacked,
                    index = rp.datasetIndex,
                    axisrotated = rp.axisrotated,
                    yaxiscolumnorder = rp.yaxiscolumnorder;
                var categories = (chart.dataset.getDataType("x") == ORDINAL_DATATYPE) ? xscale.domain() : chart.dataObject.getCategories("x");
                
                var type = chart.chartTypes[index].value;
                if (isBarChartType(type)) {
                    xy = function(dt, i, j) {
                        var commonOption = renderer.commonOption || getBarCommonOption(chart.rendererConf[index], index);
                        return [chart.dataObject.getTransRectValue("x", dt, i, j, commonOption),
                            chart.dataObject.getTransRectValue("y", dt, i, j, commonOption)
                        ];
                    };
                } else {
                    xy = function(dt) {
                        var dxy = isPolar ? [chart.plotarea_approx.x, chart.plotarea_approx.y] : [0, 0];
                        return [(chart.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, dt, "x") + dxy[0]),
                            (chart.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, dt, "y") + dxy[1])
                        ];
                    }
                }
                notesArray = $ZC.annotation.preprocessAnnotations(chart.seriesdata, index, xy, notesArray, chart, rp, categories);
            }
        }
        notesArray.forEach(function(d) {
            $ZC.annotation.addNotes(chart, d);
        });
    }
    notesArray = [];
    return notesArray;
};
$ZC.annotation.addNotes = function(chart, args) {
    var noteConf = chart.systemConf.notes,
        chartWidth = chart.canvasarea.width,
        chartHeight = chart.canvasarea.height;
    var coordinate = args.coordinate,
        basedOn = args.basedOn || "chart",
        annotationData = args.data,
        val = args.conf || {},
        disabled = args.disabled;
    var displayMode = pick(val.type, noteConf.type, "stickyNote");
    
    
    var id = (basedOn == "plot") ?
        jsonStringify(annotationData) :
        ((basedOn == "block") ?
            generateIdByPropValue(val, "") :
            jsonStringify(coordinate));
    id = (basedOn != "plot" && (val.id != null) ? val.id : basedOn + id.replace(/[`~!@#$%^&*( )_|+\-=?;:'",.<>\{\}\[\]\\\/]/gi, ''));
    // check again
    val.id = id;
    //
    
    var container = appendEle(chart.notes, "div", [1], "#", "id", basedOn);
    container
        .style({
            "width": "0",
            "height": "0"
        });
    
    var annotation = appendEle(container, "div", [args], "#", "id", id);
    annotation
        .style({
            "display": (disabled ? "none" : null),
            "position": "absolute",
            "left": "0", // Position at left:0 & top:0. Else on redraw the annotation may be wrapped as it's previous position is out of div's current position. (e.g: threshold-02.json)
            "top": "0"
        });
    
    var annotationRenderer = $ZC.annotation[(displayMode === "stickyNote") ? "stickyNote" : "customNote"](),
        conf = mergeJSON(noteConf.options[displayMode], (args.conf || {}), true);
    annotationRenderer
        .chartObj(chart)
        .conf(conf);
    
    applyFont(annotation, [conf, chart.systemConf.canvas, $ZCG], true);
    annotation.call(annotationRenderer);
    
    var ele = annotation.node();
    var BW = (displayMode === "stickyNote") ? mathMax(parseFloat(annotation.selectAll(".head").style("width")), parseFloat(annotation.selectAll(".body").style("width"))) : 0,
        BH = (displayMode === "stickyNote") ? (parseFloat(annotation.selectAll(".head").style("height")) + parseFloat(annotation.selectAll(".body").style("height"))) : 0;
    
    coordinate = coordinate.map(function(d, i) {
        var val = (basedOn === "plot") ?
            (d + chart.plotarea[(i == 0) ? "left" : "top"]) //plot annotation
            :
            invokeFunction(d, chart, val, ele);
        return isNaN(+val) //Contains '%'
            ?
            perToPx(d, ((i == 0) ? chartWidth : chartHeight), 0) :
            val;
    });
    
    var transform = (basedOn === "plot" || basedOn === "block") ? val.transform : null;
    coordinate = invokeFunction(transform, chart, val, ele, coordinate) || coordinate;
    
    annotationRenderer.coordinate(coordinate);
    coordinate = getCoordinateWithinBound(coordinate[0], coordinate[1], BW, BH, chartWidth, chartHeight);
    
    annotation
        .style({
            "left": coordinate[0] + "px",
            "top": coordinate[1] + "px"
        });
    
    // check again
    var dt = (basedOn == "plot") ? annotationData : id;
    var index = getAnnotationIndex(noteConf, dt, basedOn);
    if (index == -1) {
        saveAnnotation(noteConf, dt, "", args.coordinate, basedOn);
    }
    //
    return annotation;
};
//Preprocessor for plot annotations to get the coordinate
$ZC.annotation.preprocessAnnotations = function(seriesdata, idx, coordinate, notesArray, chartObj, rp, categories) {

    if(chartObj.destroying) {
        return;
    }

    var notes = chartObj.systemConf.notes;
    
    if (!defined(notes) || notes.enabled === false) {
        return notesArray;
    }
    
    var notesConf = notes.plotValues,
        isAxisCat = chartObj.isAxisCategory,
        isNonAxisMultiSeriesCat = chartObj.isNonAxisMultiSeriesCategory,
        dataset = rp.dataset,
        stacked = rp.stacked,
        plotarea = chartObj.plotarea,
        yaxiscolumnorder = rp.yaxiscolumnorder,
        curSeries = seriesdata[idx];
    
    var curData = (chartObj.dataObject.isHierarchy ? curSeries : curSeries.data),
        serName = ((isAxisCat || isNonAxisMultiSeriesCat) && (seriesdata.length > 1)) ? curSeries.seriesname : null,
        chartType = chartObj.chartTypes[idx].name,
        seriesnames = chartObj.dataObject.seriesNames || (chartObj.dataObject.seriesNames = chartObj.seriesdata.map(function(d) {
                return d.seriesname;
            })); //Collect the series names

    var filteredNotes = (isAxisCat && (seriesdata.length > 1)) ? notesConf.filter(function(d) {
            var s = d.data[d.data.length - 1];
            return (serName == s) || (seriesnames.indexOf(s) == -1);
        }) : notesConf;
    
    filteredNotes.forEach(function(curNote, i) {
        var found = false,
            nData = curNote.data;
        for (var subseries = 0, slen = (isAxisCat ? curData.length : 1);
             (subseries < slen && !found); subseries++) {
            for (var j = 0, len = curData[subseries].length;
                 (j < len && !found); j++) {
                var dt = curData[subseries][j];
                var data = generateDataForAnnotation(dt, serName);
                if (jsonStringify(data) == jsonStringify(nData)) {
                    found = true;
                    var pos = coordinate(dt, subseries, j);
                    var disabled = (isAxisCat && !isWithin(pos, plotarea)) // Exclude if the data is out of domain
                        ?
                        true :
                        ((isAxisCat || isNonAxisMultiSeriesCat) ? curSeries.disabled : dt.disabled);
                    pushNotes(pos, data, curNote, disabled);
                }
            }
        }
        if (isAxisCat && !found && ((chartType === "bar") ? (categories.indexOf(dataset.getX(nData)) != -1) : true)) { //Non data points
            found = true;
            var dt = generateDataForAnnotation(nData);
            if (stacked) {
                dt.y = dataset.getY(nData, yaxiscolumnorder);
                dt.y0 = 0;
            }
            var pos = coordinate(dt, 0, j);
            var disabled = (isAxisCat && !isWithin(pos, plotarea)) // Exclude if the data is out of domain
                ?
                true :
                ((isAxisCat && (seriesdata.length > 1) && (seriesnames.indexOf(nData[nData.length - 1]) == -1)) //Non data points in multi series chart
                    ?
                    false :
                    curSeries.disabled);
            pushNotes(pos, dt, curNote, disabled);
        }
        if (!found && (chartType === "sunburst")) { // Sunburst parent arc
            var parentData = [serName, curSeries.value];
            if (jsonStringify(parentData) === jsonStringify(nData)) {
                pushNotes(coordinate(curSeries), parentData, curNote, curSeries.disabled);
            }
        }
    });
    return notesArray;
    
    function isWithin(coordinate, bound) {
        var x = coordinate[0],
            y = coordinate[1];
        return (x >= 0) && (y >= 0) && (x <= bound.width) && (y <= bound.height);
    }
    
    function pushNotes(coordinate, data, conf, disabled) {
        notesArray.push({
            "coordinate": coordinate,
            "data": data,
            "conf": conf,
            "disabled": disabled,
            "basedOn": "plot"
        });
    };
};

$ZC.annotation.addBlockAnnotations = function(chart) {
    var notes = chart.systemConf.notes;
    
    if (!defined(notes) || notes.enabled === false) {
        return;
    }
    
    var defaultAnnotationOptions = $ZC.setDefaultOptions("notes", chart.systemConf, chart.globalProp);
    var blockAnnotations = chart.systemConf.notes.blockValues,
        plotarea = chart.plotarea,
        drawnAtBottom, drawnAtTop;
    var PW = plotarea.width,
        PH = plotarea.height,
        PL = plotarea.left,
        PT = plotarea.top,
        PB = plotarea.bottom;
    blockAnnotations.forEach(function(d) {
        var hAlign = d.hAlign,
            vAlign = d.vAlign;
        var isRight = (hAlign === "right"),
            isLeft = (hAlign === "left"),
            isTop = (vAlign === "top"),
            isBottom = (vAlign === "bottom");
        
        if (!((drawnAtTop && isTop) || (drawnAtBottom && isBottom))) {
            var xFunc = function(chart, val, ele) {
                    var bound = ele.getBoundingClientRect();
                    var x = isRight ? (PL + PW) : (isLeft ? PL : (PL + (PW / 2))),
                        dx = isRight ? bound.width : (isLeft ? 0 : (bound.width) / 2);
                    return mathMax((x - dx), 0);
                },
                yFunc = function(chart, val, ele) {
                    var bound = ele.getBoundingClientRect();
                    var y = isTop ? PT : (PT + PH),
                        dy = isTop ? 0 : bound.height;
                    return (y - dy);
                };
            
            var note = $ZC.annotation.addNotes(chart, {
                "coordinate": [xFunc, yFunc],
                "conf": d,
                "basedOn": "block"
            });
            var bound = note.node().getBoundingClientRect();
            if (bound.height > (PH * 0.1)) {
                note.remove();
            } else {
                var bHei = bound.height;
                plotarea.height = PH - bHei;
                if (isTop) {
                    plotarea.top = PT + bHei;
                } else if (isBottom) {
                    plotarea.bottom = PB + bHei;
                }
            }
            
            drawnAtTop = drawnAtTop || isTop;
            drawnAtBottom = drawnAtBottom || isBottom;
        }
    });
};