//$Id$
/*
 * z-index check
 * textarea resize
 * While opening a note, check within the plot
 * dragging
 */
$ZC.annotation.stickyNote = function() {
    var chartObj, conf, coordinate;
    
    function annotation(selection) {
        selection.each(function(args) {
            var container = d3_select(this);
            var noteConf = chartObj.systemConf.notes,
                chartWidth = chartObj.canvasarea.width,
                chartHeight = chartObj.canvasarea.height,
                notesWrap = chartObj.notes;
            
            var basedOn = args.basedOn || "chart",
                annotationData = args.data,
                disabled = args.disabled,
                xy = args.coordinate;
            
            var xInPercent = (typeof xy[0] === "string" && xy[0].indexOf("%") > -1),
                yInPercent = (typeof xy[1] === "string" && xy[1].indexOf("%") > -1);
            
            var text = conf.text,
                colorIndex = notesWrap.selectAll(".head").size(),
                bgColor = conf.color,
                open = (conf.mode === "open"),
                bodyWidth = validateNullVal(conf.bodyWidth, 100),
                bodyHeight = validateNullVal(conf.bodyHeight, 100),
                headWidth = validateNullVal(conf.headWidth, 15),
                headHeight = validateNullVal(conf.headHeight, 15),
                toggle = open ? 0 : 1;
            bgColor = (bgColor instanceof Array) ? bgColor[colorIndex % bgColor.length] : bgColor;
            
            //Drag behaviour
            var drag = d3_drag()
                .on("drag", function(d, i) {
                    if (d3.event.defaultPrevented) return;
                    // d3.event.sourceEvent.stopPropagation();
                    var parentEle = d3_select(this.parentNode);
                    var x = parseFloat(parentEle.style("left")) + d3.event.x,
                        y = parseFloat(parentEle.style("top")) + d3.event.y,
                        w = mathMax(((toggle % 2 != 0) ? 0 : bodyWidth), headWidth),
                        h = ((toggle % 2 != 0) ? 0 : bodyHeight) + headHeight;
                    x = (x > (chartWidth - w)) ?
                        (chartWidth - w) :
                        ((x < 0) ? 0 : x);
                    y = (y > (chartHeight - h)) ?
                        (chartHeight - h) :
                        ((y < 0) ? 0 : y);
                    coordinate[0] = x, coordinate[1] = y;
                    var leftVal = x + "px",
                        topVal = y + "px";
                    
                    parentEle
                        .style({
                            "left": leftVal,
                            "top": topVal
                        });
                    if (basedOn != "plot") {
                        var iden = parentEle.attr("id");
                        var index = getAnnotationIndex(noteConf, iden, basedOn);
                        if (index != -1) {
                            noteConf.chartValues[index].x = xInPercent ? ((x / chartWidth * 100) + "%") : x;
                            noteConf.chartValues[index].y = yInPercent ? ((y / chartHeight * 100) + "%") : y;
                        }
                    }
                });
            
            container
                .attr("class", "group")
                .style({
                    "width": 0,
                    "height": 0,
                    "z-index": 2
                });
            
            // Draw text area
            var textarea = appendEle(notesWrap, "textarea", [1]);
            textarea
                .style({
                    "width": bodyWidth + "px",
                    "height": bodyHeight + "px",
                    "position": "absolute",
                    "border-color": "transparent",
                    "border-radius": (conf.bodyBorderRadius || 0) + "px",
                    "background-color": "transparent",
                    "overflow": "hidden",
                    "display": "none",
                    "resize": "none", //Need to implement textarea resize
                    "z-index": 2,
                    "outline": "none"
                })
                .on("blur", function(d, i) {
                    var baseEle = textarea.attr("eleID");
                    textarea.style("display", "none");
                    if (baseEle != null) {
                        var parentEle = d3_select("#" + baseEle);
                        var bodyEle = parentEle.selectAll(".body");
                        
                        bodyEle.style("color", null);
                        
                        parentEle.style("z-index", 2);
                        
                    }
                })
                .on("input", function() {
                    var curEle = d3_select(this);
                    if (curEle.style("display") === "none") {
                        return;
                    }
                    var parentEle = d3_select("#" + curEle.attr("eleID"));
                    var noteEle = parentEle.selectAll(".body");
                    var iden = parentEle.attr("id"),
                        txt = textarea[0][0].value,
                        datum = parentEle.datum();
                    var basedOn = datum.basedOn || "chart",
                        annotationData = datum.data;
                    noteEle.text(txt);
                    var dt = (basedOn == "plot") ? annotationData : iden;
                    var index = getAnnotationIndex(noteConf, dt, basedOn);
                    if (index == -1) {
                        saveAnnotation(noteConf, dt, txt, xy, basedOn);
                    } else {
                        noteConf[basedOn + "Values"][index].text = txt;
                    }
                })
                .text(text);
            
            // Draw head
            var head = appendEle(container, "div", [1], ".", "class", "head", {
                "background-color": bgColor
            });
            head
                .style({
                    "width": headWidth + "px",
                    "height": headHeight + "px",
                    "border": conf.border,
                    "border-radius": (conf.headBorderRadius || 0) + "px",
                    "z-index": "auto"
                });
            // .call(drag);

            var click = function(d, i) {
                if (d3.event.defaultPrevented) return;
                var close = (toggle % 2 == 0);
                var bodyEle = d3_select(this.nextSibling),
                    parentEle = d3_select(this.parentNode);
                textarea.on("blur")();

                if (!close) {
                    var xy = getCoordinateWithinBound(parseFloat(parentEle.style("left")), parseFloat(parentEle.style("top")), bodyWidth, (headHeight + bodyHeight), chartWidth, chartHeight);
                    parentEle
                        .style({
                            "left": xy[0] + "px",
                            "top": xy[1] + "px"
                        });
                } else {
                    parentEle
                        .transition()
                        .delay(250)
                        .duration(0)
                        .style({
                            "left": coordinate[0] + "px",
                            "top": coordinate[1] + "px"
                        });
                }

                bodyEle
                    .style("color", null)
                    .transition()
                    .ease("linear")
                    .style({
                        "width": (close ? "0px" : (bodyWidth + "px")),
                        "height": (close ? "0px" : (bodyHeight + "px")),
                        "display": (close ? null : "block")
                    });

                bodyEle
                    .transition()
                    .delay(250)
                    .style("display", (close ? "none" : null));

                notesWrap.selectAll(".group").style("z-index", 1);
                parentEle.style("z-index", 2);

                toggle++;
            };
            var mouseHandlers = {
                "click": click
            },
            touchHandlers = {
                "tap": click
            };
            attachevents(head, mouseHandlers, touchHandlers);
            
            //Draw body
            var body = appendEle(container, "div", [1], ".", "class", "body", {
                "background-color": bgColor
            });
            body
                .style({
                    "width": !open ? "0" : bodyWidth + "px",
                    "height": !open ? "0" : bodyHeight + "px",
                    "display": !open ? "none" : null,
                    "border": conf.border,
                    "overflow": "hidden",
                    "box-shadow": conf.shadow,
                    "border-radius": (conf.bodyBorderRadius || 0) + "px",
                    "z-index": "auto"
                })
                .text(text);

                var click = function(d, i) {
                    var curEle = d3_select(this),
                        parentEle = d3_select(this.parentNode);
                    textarea.on("blur")();
                    textarea
                        .style({
                            "display": null,
                            "left": parentEle.style("left"),
                            "top": (parseFloat(parentEle.style("top")) + headHeight) + "px",
                            "color": parentEle.style("color"),
                            "font-size": parentEle.style("font-size"),
                            "font-family": parentEle.style("font-family"),
                            "font-weight": parentEle.style("font-weight"),
                            "font-style": parentEle.style("font-style")
                        })
                        .attr("eleID", parentEle.attr("id"));
                    textarea[0][0].value = curEle.text();
                    textarea[0][0].focus();
                    
                    curEle.style("color", "transparent");
                    
                    notesWrap.selectAll(".group").style("z-index", 1);
                    parentEle.style("z-index", 2);
                    textarea.style("z-index", 3);
                };
                var mouseHandlers = {
                    "click": click
                },
                touchHandlers = {
                    "tap": click
                };
            attachevents(body, mouseHandlers, touchHandlers);
            
        });
    }
    
    annotation.chartObj = function(_) {
        if (!arguments.length) {
            return chartObj;
        }
        chartObj = _;
        return annotation;
    };
    annotation.conf = function(_) {
        if (!arguments.length) {
            return conf;
        }
        conf = _;
        return annotation;
    };
    annotation.coordinate = function(_) {
        if (!arguments.length) {
            return coordinate;
        }
        coordinate = _;
        return annotation;
    };
    return annotation;
};