//$Id$
/**
 *
 * @returns {chart} area and arearange renderer.
 */
svg_plot_renderer.area = function() {
    var commonRendererProp, xscale, yscale, dataset, yaxiscolumnorder, index = 0,
        stacked = false,
        axisrotated = false,
        redraw = false,
        line = svg_plot_renderer.line();
    var area = [],
        color, labelQueue, commonPlotOptions, systemconf, hasNegValue, chartObj,
        transitionDuration;
    var shiftInfo, shiftDiff,
        dataProcessorHelpers = $ZC.DataProcessor.helpers;
    
    function chart(selection) {
        selection.each(function(data) {
            //renderer config values
            var rp = commonRendererProp.params;
            chartObj = rp.chartObj,
                xscale = rp.xscale,
                yscale = rp.yscale,
                stacked = rp.stacked,
                index = rp.datasetIndex,
                axisrotated = rp.axisrotated,
                commonPlotOptions = rp.commonPlotOptions,
                color = rp.color,
                labelQueue = rp.labelQueue,
                hasNegValue = rp.hasNegValue,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset,
                systemconf = chartObj.systemConf
            
            var defsLocation = chartObj.defsLocation,
                isPolar = chartObj.dataObject.isPolarAxisCategory;
            
            var currentdata = data[index];
            var plotoptions = currentdata.plotoptions;
            var currentseriesdata = currentdata.data;
            var charttype = chartObj.chartTypes[index].name;

            var gvp = $ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions);
            var interpolation = gvp("mode"),
                fillOpacity = gvp("fillOpacity"),
                strokeWidth = gvp("strokeWidth"),
                strokeColor = gvp("strokeColor"),
                markerEnabled = defined(plotoptions) ? $ZC.utils.getValueOfPlotOptions(plotoptions.marker, commonPlotOptions.marker)("enabled") : commonPlotOptions.marker.enabled;
            var borderEnabled = ((strokeWidth != null && strokeWidth != "" && pInt(strokeWidth) != 0) ||
            (strokeColor != null && strokeColor != "") || (markerEnabled));
            
            //Animation variables
            var animationOptions = getAnimationConf(commonPlotOptions, plotoptions, chartObj, charttype);
            var isAnimationEnable = animationOptions.enabled,
                easingType = animationOptions.easingType,
                animationType = animationOptions.type;
            transitionDuration = animationOptions.duration;
            animationType = (animationType == "vertPathTweenAll") ? "vertPathTween" : animationType;
            
            var datatypeX = chartObj.dataset.getDataType("x")
                datatypeY = chartObj.dataset.getDataType("y", yaxiscolumnorder),

                isCatNullX = dataProcessorHelpers.isCatNull(dataset, "x"),
                isCatNullY = dataProcessorHelpers.isCatNull(dataset, "y", yaxiscolumnorder),

                xCategories = (datatypeX == ORDINAL_DATATYPE) ? xscale.domain() : null,
                yCategories = (datatypeY == ORDINAL_DATATYPE) ? yscale.domain() : null,

                categories = (datatypeX == ORDINAL_DATATYPE && !defined(chartObj.dataObject.hierarchical)) ? xCategories : chartObj.dataObject.getCategories("x");

            var yReversed = systemconf.chart.axes.yaxis[yaxiscolumnorder].reversed;
            var polarBaseIdx = (isPolar && yReversed && !stacked) ? yscale.domain().length - 1 : 0;
            if (datatypeX != ORDINAL_DATATYPE && categories.length == 0 && isAnimationEnable && animationType == "pointTween") {
                categories = chartObj.dataObject.findXCategories();
            }
            
            var isRangeChart = (charttype) == "arearange";
            var yrangePos = (isRangeChart) ? 1 : null;
            
            chart.seriesdataAfterUpdate = [];
            currentseriesdata.forEach(function(d, i) {
                var serData;
                if (datatypeX == ORDINAL_DATATYPE || datatypeY == ORDINAL_DATATYPE) {
                    serData = chartObj.dataObject.updateSeriesdatabyCategories(d3_values(d), xCategories, yCategories, yaxiscolumnorder);
                } else {
                    serData = d;
                }

                if (chartObj.brush && chartObj.brush.extent) {
                    var domain = chartObj.axes.x.scale.domain();
                    var ticks = chartObj.axes.x.ticks.tickValueObj.ticks;
                    var startVal = (datatypeX == TIME_DATATYPE) ? ticks[0].getTime() : (datatypeX == ORDINAL_DATATYPE) ? categories.indexOf(domain[0]) : domain[0];
                    var endVal = (datatypeX == TIME_DATATYPE) ? domain[1].getTime() : (datatypeX == ORDINAL_DATATYPE) ? categories.indexOf(domain[domain.length - 1]) : domain[1];
                    var accessor = (datatypeX == ORDINAL_DATATYPE) ? function(d) {
                            return categories.indexOf(chartObj.dataset.getX(d))
                        } : function(d) {
                            return chartObj.dataset.getX(d)
                        }
                    serData = dataProcessorHelpers.filterDataToViewPort(serData, startVal, endVal, accessor);
                }

                chart.seriesdataAfterUpdate[i] = serData;
                if (chartObj.systemConf.chart.axes.xaxis.reversed) {
                    serData = serData.slice(0).reverse();
                }
                shiftDiff = (shiftInfo != null) ? (((shiftDiff == null) || (shiftDiff == 0)) ? chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[serData.length - 1], "x") -
                        chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[serData.length - 2], "x") : shiftDiff) : 0
                area[i] = isPolar ?
                    d3_areaRadial()
                        .angle(function(d, j) {
                            return xscale(dataset.getX(d));
                        })
                        .outerRadius(function(d, j) {
                            var OR = (stacked) ? yscale(d.y0 + d.y) : (yscale(dataset.getY(d, yaxiscolumnorder)) + ((yscale.rangeBand != null) ? yscale.rangeBand() / 2 : 0));
                            return mathMax(OR, yscale.range()[polarBaseIdx]);
                        })
                        .innerRadius(function(d, j) {
                            var IR;
                            if (stacked) {
                                IR = (index == 0) ? mathMin(yscale.range()[0], yscale(hasNegValue)) : yscale(d.y0);
                            } else {
                                IR = (yscale(yscale.domain()[polarBaseIdx]) + ((yscale.rangeBand != null) ? yscale.rangeBand() / 2 : 0));
                            }
                            return mathMax(IR, yscale.range()[polarBaseIdx]);
                        })
                        .interpolate(interpolation) :
                    d3_area()
                        .defined(function(d, j) {
                            if (stacked) {
                                return !isCatNullX(dataset.getX(d));
                            }
                            return !isCatNullY(dataset.getY(d, yaxiscolumnorder)) && !isCatNullX(dataset.getX(d));
                        })
                        .x(function(d, j) {
                            var x = chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos)
                            if (shiftInfo != null) {
                                var transX = (shiftInfo != null) ?
                                    Math.abs((chartObj.dataObject.getXYvalue(axisrotated, stacked, chart.oldXscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos) -
                                    chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos))) :
                                    null;
                                x = x + transX;
                            }
                            return x;
                        })
                        .y(function(d, j) {
                            return chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos);
                        })
                        .interpolate(interpolation)
                        .tension(commonPlotOptions.tension || 0.7);
                
                if (!isPolar) {
                    if (axisrotated) {
                        area[i].x0(function(d, j) {
                            if (stacked) {
                                return ((index == 0) && (i == 0) && (j == 0)) ? mathMax(yscale.range()[0], yscale(hasNegValue)) : yscale(d.y0);
                            } else {
                                if (isRangeChart) {
                                    return chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, 0);
                                } else {
                                    return (yscale.rangeBand != null) ? (yscale.range()[0] + yscale.rangeBand()) : mathMax(yscale.range()[0], yscale(hasNegValue));
                                }
                            }
                        });
                    } else {
                        area[i].y0(function(d, j) {
                            if (stacked) {
                                return ((index == 0) && (i == 0) && (j == 0)) ? mathMin(yscale.range()[0], yscale(hasNegValue)) : yscale(d.y0);
                            } else {
                                if (isRangeChart) {
                                    return chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, 0);
                                } else {
                                    return (yscale.rangeBand != null) ? (yscale.range()[0] + yscale.rangeBand()) : mathMin(yscale.range()[0], yscale(hasNegValue));
                                }
                            }
                        });
                    }
                }
            });
            
            var container = d3_select(this);
            chart.update = function() {
                container.transition().call(chart);
            };
            var series = container.selectAll('g.areaseries_' + index).data([currentdata]);
            series.enter().insert('g', (isPolar ? ':first-child' : null)).attr('class', 'areaseries_' + index);
            
            if (borderEnabled || markerEnabled) {
                line
                    .commonRendererProp(commonRendererProp)
                    .shiftInfo(shiftInfo);
                line.chartRendered = false;
                
                if (isRangeChart) {
                    line.yrangePos(1)
                    var baseLine = svg_plot_renderer.line()
                        .commonRendererProp(commonRendererProp)
                        .shiftInfo(shiftInfo)
                        .yrangePos(0);
                    baseLine.chartRendered = false;
                    container.call(baseLine);
                }
                container.call(line);
                
            } else {
                container.selectAll("g.lineseries_" + index + ",g.scatterseries_" + index).remove();
            }
            
            currentseriesdata.forEach(function(d, i) {
                var serData = chart.seriesdataAfterUpdate[i];
                var baseData = serData;
                var basedataShift, ShiftedData;
                
                if (isAnimationEnable) {
                    if (animationType == "pointTween") {
                        baseData = serData.map(function(d, i) {
                            return Object.create(serData[0])
                        });
                    } else {
                        var idx = (animationType == "horiPathTween") ? dataset.xidx : dataset.yidx(yaxiscolumnorder);
                        var scale = (animationType == "horiPathTween") ? xscale : yscale;
                        var baseIdx = (isPolar && yReversed && !stacked) ? scale.domain().length - 1 : 0;
                        baseData = serData.map(function(d, i) {
                            var s = Object.create(d);
                            if (stacked && animationType != "horiPathTween") {
                                s.y0 = (isPolar && (animationType === "strokeTween")) ? mathMin(0, scale.domain()[0]) : 0;
                                s.y = (!isPolar && (hasNegValue != null) && (scale(hasNegValue) != null)) ? mathMax(hasNegValue, scale.domain()[0]) : scale.domain()[0];
                            } else {
                                if (isRangeChart && animationType == "vertPathTween") {
                                    var yArr = new Array(2);
                                    var y0 = chartObj.dataset.getY(d, yaxiscolumnorder, 0);
                                    yArr[0] = y0;
                                    yArr[1] = (s[idx] == null) ? null : (((hasNegValue != null) && (scale(hasNegValue) != null)) ? mathMax(hasNegValue, y0) : y0);
                                    s[idx] = yArr
                                } else {
                                    s[idx] = (s[idx] == null) ? null : ((!isPolar && (hasNegValue != null) && (scale(hasNegValue) != null)) ? mathMax(hasNegValue, scale.domain()[0]) : scale.domain()[baseIdx]);
                                }
                                
                            }
                            return s;
                        })
                    }
                    basedataShift = d3_values(baseData)
                }
                
                var ishideable = currentdata.disabled;
                var subseries = appendEle(series, "g", function(d) {
                    return ishideable ? [] : [d];
                }, ".", "class", ('subseries_' + i));
                var path = subseries.selectAll("path.area").data(function(d) {
                    return ishideable ? [] : [d];
                });
                
                var pathEnter = path.enter().append("path")
                    .datum(baseData)
                    .attr("class", "area")
                    .style('opacity', fillOpacity)
                    .style('fill', color)
                    .attr("d", area[i])
                
                if (isAnimationEnable) {
                    if (animationType == "pointbypoint" || animationType == "pointTween") {
                        interTween(baseData, basedataShift, serData, categories, animationType, easingType, transitionDuration, rp, function(d) {
                            return {
                                "d": area[i](d)
                            };
                        }, pathEnter);
                    } else if (pathEnter.node() != null && animationType == "strokeTween") { //For stacked chart, check again
                        pathEnter
                            .transition()
                            .duration(0)
                            .delay(transitionDuration)
                            .attr("d", function(d) {
                                return area[i](d);
                            })
                    } else {
                        pathEnter
                            .transition()
                            .duration(transitionDuration)
                            .ease(easingType)
                            .attrTween("d", pathTween)
                    }
                }
                
                function pathTween() {
                    var interpolate = d3_interpolate(area[i](baseData), area[i](serData));
                    return function(t) {
                        return interpolate(t);
                    };
                }
                var gradientDim = isPolar ? [chartObj.plotarea_approx.radius, chartObj.plotarea_approx.radius] : [(axisrotated ? yscale(yscale.domain()[0]) : xscale(xscale.domain()[1])), (axisrotated ? xscale(xscale.domain()[1]) : yscale(yscale.domain()[0]))];
                var fillColor = chart.fillColorFunc = getFillColor(chartObj, index, gradientDim);

                // var ShiftedData = d3_values(serData);
                // var ShiftedData_bef = [];
                if (shiftInfo != null) {
                    // ShiftedData = d3_values(serData);
                    // var co = shiftInfo.count;
                    // for (var k = 0; k <= co; k++) {
                    //     ShiftedData.splice(k, 0, shiftInfo.point)
                    // }
                    //
                    // for (var h = 0; h < serData.length; h++) {
                    //     ShiftedData_bef[h] = []
                    //     ShiftedData_bef[h][0] = serData[h][0]
                    //     ShiftedData_bef[h][1] = (h == 0) ? shiftInfo.point[1] : serData[h - 1][1]
                    // }
                    // ShiftedData_bef.push(["dummmy", serData[serData.length - 1][1]])
                    
                    if (redraw) {
                        // var markerWidth = (chartObj.dataObject.totalnoofdatas <= maxLimit) && markerEnabled
                        var clipX = xscale(serData[0][0]) + ((xscale.rangeBand != null) ? xscale.rangeBand() / 2 : 0);
                        var clipWid = chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[serData.length - 1], "x") -
                            chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[0], "x");

                        var properties = {
                            name: "rect",
                            id: chartObj.id,
                            x: clipX,
                            y: 0,
                            width: clipWid,
                            height: chartObj.plotarea.height
                        },
                        args = {
                            chartObj: chartObj
                        };
                        var clipID = effects.get(svg_clips, "rect", properties, args);
                        series
                            .attr('clip-path', "url(" + defsLocation + "#" + clipID + ")");
                    }
                    
                }
                
                var dataUpdate = serData;
                var pathUpdate = path.datum(dataUpdate)
                    .style({
                        "opacity" : fillOpacity,
                        "fill" : fillColor(index,null,null,color)
                    });
                

                if (shiftInfo != null) {
                    var transX = (shiftInfo != null) ?
                        Math.abs((chartObj.dataObject.getXYvalue(axisrotated, stacked, chart.oldXscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos) -
                        chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos))) :
                        null;
                    pathUpdate
                        .datum(dataUpdate)
                        .attr("d", area[i])
                        .attr("transform", null)
                        .transition()
                        .duration(transitionDuration)
                        .ease(easingType)
                        .attr("transform", function() {
                            return ((transX == null) ? null : "translate(-" + transX + ",0)")
                        })
                    
                } else {
                    if (pathEnter.node() != null && animationType == "strokeTween") {
                        pathUpdate
                            .datum(dataUpdate)
                            .transition()
                            .duration(0)
                            .delay(transitionDuration)
                            .attr("d", function(d) {
                                return area[i](d);
                            })
                    } else {
                        pathUpdate
                            .datum(dataUpdate)
                            .transition()
                            .delay(((animationType == "pointbypoint" || animationType == "pointTween") && pathEnter.node() != null) ? transitionDuration : 0)
                            .duration(transitionDuration)
                            .ease(easingType)
                            .attr("d", area[i])
                    }
                }
                
                path.exit().remove();
                
            });
            if (!(borderEnabled || markerEnabled)) {
                if (isRangeChart) {
                    labelQueue.push({
                        "renderer": chart,
                        "charttype": "scatter",
                        "chartObj": chartObj,
                        "args": [currentdata, index, 0, 1]
                    });
                    labelQueue.push({
                        "renderer": chart,
                        "charttype": "scatter",
                        "chartObj": chartObj,
                        "args": [currentdata, index, 0, 0]
                    }); //last argument in args for yrangePos
                } else {
                    labelQueue.push({
                        "renderer": chart,
                        "charttype": "scatter",
                        "chartObj": chartObj,
                        "args": [currentdata, index, 0]
                    });
                }
            }
        });
        chart.renderedTimer = setTimeout(function() {
            chart.chartRendered = true;
            redraw = true;
        }, transitionDuration);
        chart.oldXscale = xscale;
        return chart;
    }
    chart.commonRendererProp = function(_) {
        if (!arguments.length) {
            return commonRendererProp;
        }
        commonRendererProp = _;
        return chart;
    };
    
    chart.shiftInfo = function(_) {
        if (!arguments.length) {
            return shiftInfo;
        }
        shiftInfo = _;
        return chart;
    };
    
    return chart;
}