//$Id$
var colorscale_helpers = {};
colorscale_helpers.getProcessedColorRange = function(range, domain, options, chartObj) {
	var userStops = options.stops,
		userRanges = options.ranges,
		processed_range = [];
	if (userStops && userStops.length) {
		var prevStopIndex = 0;
		domain.forEach(function(d, i) {
			var stopIndex = userStops.indexOf(d);
			if (stopIndex === -1) {
				stopIndex = prevStopIndex;
			}
			processed_range.push(range[stopIndex % range.length]);
			prevStopIndex = stopIndex;
		});
	} else if (userRanges && userRanges.length) {
		userRanges = getRawRanges(chartObj, true);
		var isOrdinal = chartObj.dataset.getDataType("clr") === ORDINAL_DATATYPE;
		var i = 0,
			check = function(r, j) {
				if ((r.length === 1) ? (isOrdinal ? (r[0] === domain[i]) : (r[0] === domain[i+1])) : (r[0] === domain[i] && r[1] === domain[i+1])) {
					rangeIndex = j;
					return true;
				}
			};

		for (var len=domain.length; i<len; (isOrdinal ? i++ : i+=2)) {
			var rangeIndex = -1;
			userRanges.some(check);
			if (rangeIndex === -1 || userRanges[rangeIndex].disabled) {
				processed_range.push(null);
			} else {
				processed_range.push(range[rangeIndex % range.length]);
			}
		}
	} else {
		processed_range = range;
	}

	// interpolateRGBA is not available in d3.v3. So we consider the rgba color as rgb. Remove this once we switch to d3.v4
	// ***START
	processed_range = processed_range.map(function(d){
		return (typeof d === "string") ? rgb2hex(d) : d;
	});
	// ***END

	return processed_range;
};

colorscale_helpers.getColorDomain = function(chartObj) {
	var options = chartObj.systemConf.legend.colorBand,
		userStops = options.stops,
		userRanges = options.ranges,

		minmax = colorscale_helpers.getMinMaxClr(chartObj),
		min = pick(options.minRange, minmax[0]) || 0, //min may be undefined|null //Test_695.json
		max = pick(options.maxRange, minmax[1]) || 0; //max may be undefined|null //Test_695.json

	if (userStops && userStops.length) {
		var copy_userStops = d3_values(userStops);
		copy_userStops.sort(d3_ascending);
		var filteredStops = copy_userStops.filter(function(d) {
			var condition = true;
			if (defined(options.minRange)) {
				condition = condition && (d >= options.minRange);
			}
			if (defined(options.maxRange)) {
				condition = condition && (d <= options.maxRange);
			}
			return condition;
		});

		if (min < filteredStops[0] || !filteredStops.length) {
			filteredStops.splice(0, 0, min);
		}
		if (max > filteredStops[filteredStops.length - 1]) {
			filteredStops.splice(filteredStops.length, 0, max);
		}
		return filteredStops; //Add percentage support
	} else if (userRanges && userRanges.length) {
		var copy_userRanges = getRawRanges(chartObj),
			final_ranges = [],
			isOrdinal = chartObj.dataset.getDataType("clr") === ORDINAL_DATATYPE;

        if (isOrdinal) {
        	final_ranges = copy_userRanges;
        } else {
        	if (copy_userRanges[0].length === 1) {
        		var m = min;
		        copy_userRanges = copy_userRanges.map(function(d) {
		        	var newRange = [m, d[0]];
		        	m = newRange[1];
		        	return newRange;
		        });
        	}
        	final_ranges = copy_userRanges;
        	final_ranges.splice(0, 0, [-Infinity, -MAX_SAFE_INTEGER]);
        	final_ranges.splice(final_ranges.length, 0, [MAX_SAFE_INTEGER, Infinity]);
        }
		var merged_userRanges = d3_merge(final_ranges);
		return merged_userRanges;

	}
	return [min, max];
};

/*
 * @param {Object} chartObj
 * @returns {Array} minmax
 */
colorscale_helpers.getMinMaxClr = function(chartObj) {
	var seriesdata = chartObj.seriesdata,
		colorValuesArray = [];

	seriesdata.forEach(function(series) {
		if (!series.disabled) {
			var curSeriesdata = series.data || series;
			colorscale_helpers.pushColorValues(chartObj, curSeriesdata, colorValuesArray)
		}
	});
	return d3_extent(colorValuesArray);
};

/*
 * @param {Object} chartObj
 * @param {Object} root
 * @param {Array} colorValuesArray
 */
colorscale_helpers.pushColorValues = function(chartObj, root, colorValuesArray) {
	if (root && isArray(root) && isArray(root[0])) {
		root.forEach(function(child) {
			colorscale_helpers.pushColorValues(chartObj, child, colorValuesArray);
		});
	} else {
		var clrValue = chartObj.dataset.getClr(root);
		colorValuesArray.push(clrValue);
	}
};

colorscale_helpers.isQuantileColorScale = function(chartObj) {
	if (chartObj.axes.clr) {
		return chartObj.axes.clr.scaleType === "quantile";
	} else {
		var dataObject = chartObj.dataObject;
		//check for hierarchy chart
		return defined(chartObj.dataset.clridx) && !(dataObject.noofAreaEnabledSeries || dataObject.isPolarAxisCategory || (dataObject.allChartTypes && dataObject.allChartTypes.indexOf($ZC.charttype.line) > -1));
	}
};

colorscale_helpers.quantileScale = function() {
  var domain = [],
      range = [],
      quantiles = [];

  function scale(x) {
  	var index;
    quantiles.forEach(function(d, i) {
    	if (x >= d[0] && x <= d[1]) {
    		index = i;
    	}
    });
    return defined(index) ? range[index] : null;
  }

  scale.domain = function(_) {
  	if (arguments.length) {
  		domain = _;;
   		for(var i=0; i<domain.length; i+=2) {
  			quantiles.push([domain[i], domain[i + 1]]);
  		}
  		return scale;
  	}
  	return domain;
  };

  scale.range = function(_) {
    if (arguments.length) {
  		range = _;
  		return scale;
  	}
  	return range;
  };

  scale.copy = function() {
    return colorscale_helpers.quantileScale()
        .domain(domain)
        .range(range);
  };
  return scale;
};

function getRawRanges(chartObj, dont) {
	var userRanges = chartObj.systemConf.legend.colorBand.ranges;

	var copy_userRanges = simpleClone(userRanges),
		isOrdinal = chartObj.dataset.getDataType("clr") === ORDINAL_DATATYPE;

	if (!isArray(copy_userRanges[0])) {
		copy_userRanges = copy_userRanges.map(function(d) {
			return [d];
		});
	}

	if (!isOrdinal && !dont) {
		copy_userRanges.sort(function(a, b) {
	           return d3_ascending(a[0], b[0]);
	       });
	}
	return copy_userRanges;
}
