//$Id$
$ZC.scale.constructor = function(chartObj, orient, scaleFunc, eventscaleFunc) {
    var scale;
    var defaultscaleFunc = $ZC.scale.getScale,
        defaulteventscaleFunc = $ZC.scale.getEventScale;

    scaleFunc = pick(scaleFunc, defaultscaleFunc);
    eventscaleFunc = pick(eventscaleFunc , defaulteventscaleFunc);

    switch (orient) {
        case 'left':
            scale = (chartObj.axes.rotated) ? getxscale() : getyscale();
            break;
        case 'bottom':
            scale = (chartObj.axes.rotated) ? getyscale() : getxscale();
            break;
    }
    return scale;
    
    function getxscale() {

        var processor = chartObj.dataObject,
            hierarchical = processor.hierarchical,
            levels = 0,
            extent = (chartObj.brush) ? chartObj.brush.extent : null,
            xaxes = chartObj.axes.x;

        chartObj.axes.x.scale = scaleFunc(chartObj, chartObj.dataObject, chartObj.plotarea, chartObj.axes.x.orient, 0, chartObj.dataset, "x", extent);
        chartObj.axes.x.eventscale = (eventscaleFunc == null) ? chartObj.axes.x.scale : eventscaleFunc(chartObj, chartObj.dataObject, chartObj.plotarea, chartObj.axes.x.orient, null, chartObj.dataset, "x"); //No I18N

        if (defined(hierarchical)) {

            chartObj.axes.x.rangeScale = chartObj.axes.x.scale.copy();
            if (hierarchical.groupByDiscreteTime) {
                chartObj.axes.x.rangeScale.domain(processor.getDomain(processor.getCategories("x")));
            }

            levels =  hierarchical.finalDepth;
            hierarchical.setRanges(chartObj.axes.x.rangeScale); // init all axes
            for (var i = 0; i < levels; i++) {
                if(!xaxes.levels[i]){
                    xaxes.levels[i] = {};
                }
                xaxes.levels[i].scale =  scaleFunc(chartObj, processor, chartObj.plotarea, chartObj.axes.x.orient, i + 1, chartObj.dataset, "x", extent, null);
            }
        }

        return [{
            scale: chartObj.axes.x.scale
        }].concat(xaxes.levels);
    }
    
    function getyscale() {
        var splityaxes = chartObj.axes.y.count;
        var yaxes = chartObj.axes.y;
        for (var i = 0; i < splityaxes; i++) {
            if (!yaxes.groups[i]) {
                yaxes.groups[i] = {};
            }
            yaxes.groups[i].scale = scaleFunc(chartObj, chartObj.dataObject, chartObj.plotarea, chartObj.axes.y.orient, i, chartObj.dataset, "y");
            yaxes.groups[i].eventscale = (eventscaleFunc == null) ? yaxes.groups[i].scale : eventscaleFunc(chartObj, chartObj.dataObject, chartObj.plotarea, chartObj.axes.y.orient, i, chartObj.dataset, "y");
        }
        return chartObj.axes.y.groups;
    }
};

$ZC.scale.getScale = function(chartObj, dataObj, plotarea, orient, index, dataset, type, extent, outerPadding) {

    var scale, domain;
    var systemconf = chartObj.systemConf;
    var datatype = dataset.getDataType(type, index),
        axis_data = getAxisOptions(),
        hierarchical = dataObj.hierarchical;
    
    /*
     * If hasCustomMinRange or hasCustomMaxRange is present, the correction should not be applied to the scale domain
     * Test cases :- Test_535.json, Test-536.json
     */
    var hasCustomMinRange = defined(axis_data.minRange),
        hasCustomMaxRange = defined(axis_data.maxRange);
    
    var chartType = chartObj.chartTypes[0].name;
    var commonPlotOptions = systemconf.chart.plot.plotoptions[chartType];
    var isPolar = (chartType === "dial") || (chartType === "web" && type === "x"),
        isHeatmap = (chartType === "heatmap"),
        hasColorScale = type === "clr" && colorscale_helpers.isQuantileColorScale(chartObj);
    
    var range = getRange(),
        padding = getPadding(),
        outerPadding = getOuterPadding();

    switch (datatype) {
        case 'numeric':
            if (extent) {
                domain = extent;
            } else if (type === "x") {
                domain = dataObj.getMinMaxX(0);
            } else if (type === "y") {
                domain = dataObj.getMinMaxY(index, outerPadding);
            } else if(type === "z"){
                domain = dataObj.getMinMaxZ();
            } else {
                domain = colorscale_helpers.getColorDomain(chartObj);
                range = colorscale_helpers.getProcessedColorRange(range, domain, axis_data, chartObj);
            }

            var actualStart = domain[0],
                actualEnd = domain[1];
            if (domain[0] == "NoData" && domain[1] == "NoData") { //for addorremoveaxis
                domain = [0, 0];
            }
            if (extent || type == "x") {
                var diff = (domain[1] - domain[0]);
                var correction = (diff == 0) ? chartObj.plotarea.width / 2 : (diff * outerPadding);
                domain[0] = hasCustomMinRange ? domain[0] : domain[0] - correction;
                domain[1] = hasCustomMaxRange ? domain[1] : domain[1] + correction;
            }
            
            var scaleMode = (type === "clr" && axis_data.ranges && axis_data.ranges.length) ? colorscale_helpers.quantileScale : d3_scaleLinear;
            scale = scaleMode()
                .domain(domain)
                .range(range);

            var actualRange = [scale(actualStart), scale(actualEnd)];
            scale.actualRange = actualRange;
            break;
        case 'ordinal':
            if (extent) {
                domain = extent;
            } else if(type === "clr") {
                domain = colorscale_helpers.getColorDomain(chartObj);
                range = colorscale_helpers.getProcessedColorRange(range, domain, axis_data, chartObj);
            } else {
                var categories = dataObj.getCategories(type, index);
                domain = dataObj.getDomain(categories, true)
            }

            var d3_outerPadding = 0;
            if (outerPadding) {
                d3_outerPadding = $ZC.scale.getOuterPadding(range, domain.length, padding, outerPadding);
                if (chartType !== "bullet") {
                    d3_outerPadding += dataObj.extendOuterpadding(domain, range, systemconf, padding, d3_outerPadding, type, index);
                }
            }

            scale = d3_scaleOrdinal()
                .domain(domain);

            if (type === "clr") {
                scale.range(range);
            } else {
                scale.rangeBands(range, padding, d3_outerPadding);
            }
            // hierarchical.setRanges(scale);
            break;
        
        case 'time':
            var column = dataset.getColumn(type, index),
                metaConf = column[column.datatype],
                subfunction = pick(metaConf && metaConf.subfunction, "YEAR"),
                useUTC = pick(metaConf && metaConf.useUTC, false);

            if (extent) {
                domain = extent;
            } else if(type === "x") {
                domain = dataObj.getMinMaxX(0);
            } else {
                domain = dataObj.getMinMaxY(index, 0)
            }
            
            var start = new Date(domain[0]),
                end = new Date(domain[1]),
                actualStart,
                actualEnd;
            
            /*
             * Below condition for only zoom purpose. becauze this function reset the time to its base value
             * So, when any chart is zoomed , that time no need set the time to it's default value.
             */
            
            if (!systemconf.chart.axes[type + "axis"].disableBaseValue) {
                var dateInterval = new date_utils(metaConf);
                dateInterval.useUTC = useUTC;
                dateInterval.getBaseValue(start, subfunction, false)
                dateInterval.getBaseValue(end, subfunction, true);
            }
            actualStart = start, actualEnd = end;
            
            if (dataObj.isBubbleEnabled) {
                var correctionOverBubbles = dataObj.overBubbles([start.getTime(), end.getTime()], type, (type == "y" ? index : null));
                start = new Date(start.getTime() - correctionOverBubbles[0]);
                end = new Date(end.getTime() + correctionOverBubbles[1]);
            }

            var diff = (end.getTime() - start.getTime());
            var outerPaddingCorrection = (diff == 0) ? chartObj.plotarea.width / 2 : (diff * outerPadding);
            
            var correctionOverDatalabels = (outerPadding == 0) ? 0 : dataObj.getCorrectionOverDataLabels([start.getTime(), end.getTime()], "x");
            start = new Date(hasCustomMinRange ? domain[0] : start.getTime() - outerPaddingCorrection - correctionOverDatalabels);
            end = new Date(hasCustomMaxRange ? domain[1] : end.getTime() + outerPaddingCorrection + correctionOverDatalabels);
            
            domain = [start, end]
            
            scale = d3_scaleTime()
                .domain(domain)
                .range(range);
            var actualRange = [scale(actualStart), scale(actualEnd)];
            scale.actualRange = actualRange;
            break;

        case "ordinalNonLinear": {
            // init the scale with the leaf scale;

            var categories = dataObj.getCategories(type, index);
            domain = dataObj.getDomain(categories);
            var rangeList = (hierarchical.rangeList) ? hierarchical.rangeList[index - 1]:null;

            scale = $ZC.nonLinearOrdinalScale()
                .domain(domain)
                .rangeExtent(range)
                .rangeBands(rangeList);
        }
    }
    scale.outerPadding = outerPadding;
    return scale;
    
    function getRange() {
        var axisReversed = axis_data.reversed;

        if (isPolar) {
            return axisReversed ?
                [validateNullVal(commonPlotOptions.endAngle, 2 * mathPI), validateNullVal(commonPlotOptions.startAngle, 0)] :
                [validateNullVal(commonPlotOptions.startAngle, 0), validateNullVal(commonPlotOptions.endAngle, 2 * mathPI)];
        } else if (type === "clr") {
            return chartObj.seriesColor;
        } else if (type === "z") {
            if (isHeatmap) {
                var size = 1;
                return [perToPx(commonPlotOptions.minSize, size, perToPx("25%", size)), perToPx(commonPlotOptions.maxSize, size, perToPx("100%", size))];
            }

            var zChartName;
            if (dataObj.isCombinationalChart) {
                chartObj.chartTypes.some(function(d){
                    var name = d.name;
                    if (name === "bubble" || name === "bubblepie") {
                        zChartName = name;
                        return true;
                    }
                });
            } else {
                zChartName = chartType;
            }

            var zConf = systemconf.chart.plot.plotoptions[zChartName] || {},
                dim = mathMin(plotarea.width, plotarea.height),
                size = validateNullVal(zConf.size, 1);
            return [perToPx(zConf.minSize, dim, perToPx("2.5%", dim)) * size, perToPx(zConf.maxSize, dim, perToPx("25%", dim)) * size]
        } else {
            var borderSize = dataObj.getPlotBorder();
            return axisReversed ?
                    ((orient === "bottom" || orient == "top") ?
                        [getPlotSize(chartObj, "width", plotarea) - borderSize.right, borderSize.left] :
                        [borderSize.top, getPlotSize(chartObj, "height", plotarea) - borderSize.bottom]) :
                    ((orient === "bottom" || orient == "top") ?
                        [borderSize.left, getPlotSize(chartObj, "width", plotarea) - borderSize.right] :
                        [getPlotSize(chartObj, "height", plotarea) - borderSize.bottom, borderSize.top])
        }
    }

    function getPadding() {
        if (isPolar || isHeatmap || (type != "x")) {
            return 0;
        } else {
            return validateNullVal(commonPlotOptions.padding, (dataObj.noofBarEnabledSeries ? 0.2 : 0.9)); //default value is 0.2 (Bar, Bullet, Gantt) or 0.9 (Others)
        }
    }

    function getOuterPadding() {
        if (isPolar || hasColorScale || isHeatmap) {
            return 0;
        } else if (type === "y") {
            return validateNullVal(axis_data.outerPadding, 0.05);
        } else if (outerPadding != null) {
            return outerPadding
        } else {
            return validateNullVal(commonPlotOptions.outerPadding, 0.02);
        }
    }

    function getAxisOptions() {
        if (type === "x") {
            return systemconf.chart.axes.xaxis;
        } else if (type === "clr") {
            return systemconf.legend.colorBand;
        } else {
            return systemconf.chart.axes.yaxis[index || 0];
        }
    }
};

$ZC.scale.getEventScale = function(chartObj, dataObj, plotarea, orient, index, dataset, type) {
    var datatype = dataset.getDataType(type, index);
    var scale;
    switch (datatype) {
        
        default:
            scale = $ZC.scale.getScale(chartObj, dataObj, plotarea, orient, index, dataset, type); //No I18N ;
    }
    return scale;
};

$ZC.scale.invert = function(datatype, mousePos, scale, axisrotated, type, axisReversed) {
    var actualPos;
    switch (datatype) {
        case 'numeric':
        case 'time':
            var point = (type == "y") ? (axisrotated ? mousePos.x : mousePos.y) : (axisrotated ? mousePos.y : mousePos.x);
            actualPos = scale.invert(point);
            break;
        case 'ordinal':
            var ordinalRange = scale.range();
            var rangeBand = scale.rangeBand();
            var diffRange = (ordinalRange.length > 1) ? mathAbs(ordinalRange[1] - ordinalRange[0]) : 0;
            var actualPadding = mathAbs(diffRange - rangeBand);
            ordinalRange = ordinalRange.map(function(d) {
                return d - (actualPadding / 2)
            });
            var rangeReversed = (axisrotated && !axisReversed) || (!axisrotated && axisReversed);
            var bisector = d3_bisector((type == "y") ? (rangeReversed ? d3_ascending : d3_descending) : (rangeReversed ? d3_descending : d3_ascending));
            var bisectVal = (type == "y") ? (axisrotated ? mousePos.x : mousePos.y) : (axisrotated ? mousePos.y : mousePos.x);
            var xbisect = bisector.left(ordinalRange, bisectVal, 0, ((axisrotated && axisReversed) ? ordinalRange.length : ordinalRange.length - 1))
            var toCmp1 = (axisrotated) ? (bisectVal) : (ordinalRange[xbisect]);
            var toCmp2 = (axisrotated) ? (ordinalRange[xbisect - 1]) : (bisectVal);
            actualPos = (xbisect > 0) ? (toCmp1 > toCmp2 ? xbisect - 1 : xbisect) : xbisect;
            break;
    }
    return actualPos;
};

$ZC.scale.barBandwidth = function(chartObj) {
    var systemconf = chartObj.systemConf,
        dataObj = chartObj.dataObject;
    var m = dataObj.isstacked() ?
        (dataObj.groupedByStack.keys().length || 1) // length = 0 for continuous chart with ordinal scale
        :
        d3_sum(chartObj.seriesdata.map(function(d) {
            if (dataObj.isBarEnabled(d)) {
                return d.data.length;
            }
        })); // no of series
    var temp = chartObj.chartTypes[0];
    var chartType = temp.name;
    var isBullet = temp.value == $ZC.charttype.bullet,
        isHeatmap = isHeatMapChartType(temp.value);

    var plotBorderOptions = systemconf.chart.plot.border;
    var borderSize = plotBorderOptions.show === true ? plotBorderOptions.size : 0;
    var padding = (chartType == "gantt" || chartType == "bullet" || chartType == "bar") ? systemconf.chart.plot.plotoptions[chartType].padding || 0.2 : 0.2;
    var interPadding = (m == 1) ? 0 : (chartType == "bar" || chartType == "gantt") ? systemconf.chart.plot.plotoptions[chartType].interPadding || 0 : 0;
    
    var outerPadding = systemconf.chart.plot.plotoptions[chartType].outerPadding;
    outerPadding = ((outerPadding != null) && (outerPadding !== "")) ? outerPadding : 0.02;
    var n = chartObj.axes.x.scale.domain().length //chart.dataObject.getCategories("x").length;
    var range = !chartObj.axes.rotated ? [borderSize, getPlotSize(chartObj, "width", chartObj.plotarea) - borderSize] : [getPlotSize(chartObj, "height", chartObj.plotarea) - borderSize, borderSize],
        barscale1, barscale2;
    
    outerPadding = $ZC.scale.getOuterPadding(range, n, padding, outerPadding);
    outerPadding += dataObj.extendOuterpadding(d3_range(n), range, systemconf, padding, outerPadding, "x");
    
    chartObj.barscale1 = barscale1 = chartObj.axes.x.scale;
    chartObj.barscale2 = barscale2 = d3_scaleOrdinal()
        .domain(d3_range(m))
        .rangeBands([0, barscale1.rangeBand()], interPadding);
    
    //setting bandwidth if the actual bandwidth is greater than the maximum bar bandwidth
    barscale2.rangeBands([0, barscale1.rangeBand()], interPadding);
    var maxBandWidth = (dataObj.noofBarEnabledSeries && !isBullet) ? systemconf.chart.plot.plotoptions[dataObj.noofGanttEnabledSeries ? "gantt" : isHeatmap ? "heatmap" : "bar"].maxBandWidth || barscale1.rangeBand() : 200;
    if (!isBullet && (barscale2.rangeBand() > maxBandWidth)) {
        var outspace = (barscale1.rangeBand() - (m * maxBandWidth)) / 2;
        barscale2.rangeBands([outspace, (barscale1.rangeBand() - outspace)], interPadding);
    }
    return [barscale1, barscale2]
};

$ZC.scale.getOuterPadding = function(range, domainLength, padding, outerPadding) {
    var rangeDiff = range[1] - range[0];
    var pad = outerPadding * rangeDiff;
    var step = (rangeDiff - (2 * pad)) / (domainLength - padding + (2 * 0));
    return pad / step;
};

/**
 *
 * @param scale linear scale
 * @param points tick values
 * @returns {{outerPaddingInPixel, rangeBand: number}}
 */

$ZC.scale.getContinuousRangeBand = function(scale, points, index) {

    var firstPointSpan = scale(points[0]),
        currentPointSpan = (index && points[index]) ? scale(points[index]) : firstPointSpan,
        nextPointSpan = (index && points[index+1]) ? scale(points[index+1]) : scale(points[1]),
        rangeBand = nextPointSpan - currentPointSpan,
        ranges = scale.range(),
        // for axes rotated or axes reversed.
        correction = rangeBand < 0 ? -1 : 1;

    return {
        "outerPaddingInPixel": correction * mathMin(mathAbs(firstPointSpan - ranges[0]), mathAbs(rangeBand / 2)),
        "rangeBand": rangeBand / 2
    }
};
$ZC.scale.getDiscreteRangeBand = function(scale){
    var rangeBand = scale.rangeBand(),
        rangeExtent = scale.rangeExtent(),
        outerPadding = (scale.range()[0] - rangeExtent[0])+(rangeBand/2),
        diffBand = (scale.domain().length > 1) ? scale.range()[1] - scale.range()[0] - rangeBand : 0;
    return {
        "outerPaddingInPixel": mathMin(outerPadding,value_1),
        "rangeBand": ((rangeBand/2) + (diffBand/2))
    }

}
$ZC.scale.getRangeBand = function(scale, points, index){
    var rb;
    if(scale.rangeBand){
        rb = $ZC.scale.getDiscreteRangeBand(scale);
    }else{
       rb = $ZC.scale.getContinuousRangeBand(scale, points, index);
    }
    return (mathAbs(rb.rangeBand)+mathAbs(rb.outerPaddingInPixel));
}

function getCategoryStep(rangeband, padding){
    return (rangeband/(1-padding));
}

function getOuterPaddingInPixel(plotSize, count, padding, userOuterPadding, step){
    var outerPadding = $ZC.scale.getOuterPadding(plotSize, count, padding, userOuterPadding),
        outerPaddingPx = 2 * (plotSize[0] + step * outerPadding);
    return outerPaddingPx;
}