//$Id$
/**
 * @returns {chart} bar renderer.
 */
svg_plot_renderer.bar = function() {
    var commonRendererProp, bandwidth1, bandwidth2, yscale, barDatasetIndex,
        isredraw = false, shiftInfo, transitionDuration, bulletEvent;
    
    function chart(selection) {
        selection.each(function(data) {
            var container = d3_select(this);
            chart.update = function() {
                container.transition().call(chart);
            };
            // chart.getGanttLevelMarkerProp = getGanttLevelMarkerProp();
            //renderer config values
            var rp = commonRendererProp.params,
                chartObj = rp.chartObj,
                xscale = rp.xscale,
                stacked = rp.stacked,
                index = rp.datasetIndex,
                axisrotated = rp.axisrotated,
                commonPlotOptions = rp.commonPlotOptions,
                color = rp.color,
                labelQueue = rp.labelQueue,
                hasNegValue = rp.hasNegValue,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset,
                systemconf = chartObj.systemConf,
                plotarea = chartObj.plotarea,
                dataObject = chartObj.dataObject,
                threshold = rp.threshold,
                colorOverlay = rp.colorOverlay;
            yscale = rp.yscale;
            var yaxisReversed = systemconf.chart.axes.yaxis[rp.yaxiscolumnorder || 0].reversed;
            var xAxisReverseed = chartObj.systemConf.chart.axes.xaxis.reversed;

            var defsLocation = chartObj.defsLocation;
            
            var currentdata = data[index];
            var plotoptions = currentdata.plotoptions;
            var currentseriesdata = currentdata.data;
            var charttype = chartObj.chartTypes[index].name;
            var gvp = $ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions),
                gradientDim = [(axisrotated ? yscale.range()[0] : xscale.range()[1]), (axisrotated ? xscale.range()[1] : yscale.range()[0])],
                patternPallete = (chartObj.patternPallete && chartObj.patternPallete.length),
                noofbarDatas = chartObj.dataObject.noofBarEnabledSeries,
                isBullet = (charttype == "bullet"),
                isGantt = (charttype == "gantt"),
                tooltipEnabled = pick(chartObj.systemConf.tooltip.enabled, true),
                disabledBulletEvents = getPropVal(commonPlotOptions, "events.enabled") === false,
                LevelMarkerEventsEnabled = pick(getPropVal(commonPlotOptions, "levelMarker.events.enabled"), true),
                targetMarkerEventsEnabled = pick(getPropVal(commonPlotOptions, "targetMarker.events.enabled"), true);
            
            
            // for bullet events
            if (isBullet && !defined(chartObj.events.bullet)) {
                bulletEvent = chartObj.events.bullet = {};
                bulletEvent.levelmarker = [];
                bulletEvent.marker = [];
                bulletEvent.point = [];
            }
            bulletEvent = chartObj.events.bullet;
            
            var datatypeY = chartObj.dataset.getDataType("y", yaxiscolumnorder),

                xCategories = xscale.domain(),
                yCategories = (datatypeY == ORDINAL_DATATYPE) ? yscale.domain() : null,

                categories = defined(dataObject.hierarchical) ? dataObject.getCategories("x") : xCategories;
            
            var series = appendEle(container, "g", [currentdata], ".", "class", ('barseries_' + index));
            var x = 0;
            
            if (!bandwidth1 || !bandwidth2) { //Test_515.json (Combinational chart(bar + line) where all the bar series are disabled)
                isredraw = true;
                chart.chartRendered = true;
                return;
            }
            
            var levelMarker = (plotoptions && plotoptions.levelMarker) ? mergeJSON(commonPlotOptions.levelMarker, plotoptions.levelMarker, true) : commonPlotOptions.levelMarker,
                borderOptions = (plotoptions && plotoptions.border) ? mergeJSON(commonPlotOptions.border, plotoptions.border, true) : commonPlotOptions.border;
            var multiColoring = gvp("multiColoring") === true && !chartObj.legendarea.show;
            
            //Animation variables
            var animationOptions = getAnimationConf(commonPlotOptions, plotoptions, chartObj, "bar");
            var isAnimationEnable = animationOptions.enabled,
                easingType = animationOptions.easingType,
                animationType = animationOptions.type,
                exitDuration = isAnimationEnable ? 100 : 0,
                duration;
            transitionDuration = isredraw ? (isAnimationEnable ? 300 : 0) : animationOptions.duration;
            
            /**
             *
             * @type {{stacked, categories, axisrotated, bandwidth1, bandwidth2, barDatasetIndex, padding, yscale, hasNegValue, yaxiscolumnorder, addRangeBand, barWidth, levelMarkerWidth, levelMarkerPadding}|*}
             */
            var commonOption = getBarCommonOption(chartObj.rendererConf[index], index);
            var barWidth = commonOption.barWidth,
                levelMarkerWidth = commonOption.levelMarkerWidth,
                barPadding = commonOption.padding,
                levelMarkerPadding = commonOption.levelMarkerPadding;

            var stackGroup = chartObj.userdata.seriesdata.stackGroup;
            var actualStackGroupIndex = (stackGroup) ? getStackGroupIndex(index, stackGroup) : 0;
            
            chart.getParamValue = function() {
                return chartObj.dataObject.getTransRectValue.apply(chartObj.dataObject, toArray(arguments));
            };
            var addRangeBand = commonOption.addRangeBand;
            
            var targetMarker, targetMarkerSeries,
                levelMarkerColor = levelMarker.color; //Saving the levelMarker colors, helps in multi categories bullet if each categories dont have same no of levels
            levelMarker.overlayRange = "below";
            
            if (isBullet || isGantt) {
                var markerPropertyName = isGantt ? "marker" : "targetMarker";
                targetMarker = (plotoptions && plotoptions[markerPropertyName]) ? mergeJSON(commonPlotOptions[markerPropertyName], plotoptions[markerPropertyName], true) : commonPlotOptions[markerPropertyName];
                targetMarkerSeries = appendEle(chartObj.staticPaneSVG.selectAll("g.threshold"), "g", (targetMarker.enabled === true && !currentdata.disabled ? [index] : []), ".", "class", (isGantt ? "markerSeries" + index : "markerSeries"));
            }
            
            var startRadius = {
                    tlx: 0,
                    tly: 0,
                    trx: 0,
                    trY: 0,
                    brx: 0,
                    bry: 0,
                    blx: 0,
                    bly: 0
                },
                borderRadius = startRadius,
                borderSize = 0,
                borderColor = null,
                borderStyle = null,
                levelMarkerRadius = startRadius;
            if (defined(borderOptions) && isTrue(borderOptions.show)) {
                borderSize = borderOptions.size;
                borderColor = borderOptions.color;
                borderStyle = borderOptions.style;
                borderRadius = chart.getParamValue("radius", null, null, null, commonOption, borderOptions.radius);
            }
            
            var dashStyle = getDashArray(borderStyle, (borderSize == null ? 2 : borderSize));
            var fillColor = chart.fillColorFunc = getFillColor(chartObj, index, gradientDim),
                strokeColor = getStrokeColor(chartObj, index);
            
            //Save seriesdata reference without null points , that can also be used in zcevents
            chart.seriesdataAfterUpdate = []
            currentseriesdata.forEach(function(d, i) {
                chart.getGanttLevelMarkerProp = getGanttLevelMarkerProp;
                var valuey;
                var startx, starty, startwid, starthei, endx, endy, endwid, endhei;
                var ishideable = currentdata.disabled;

                var serData = chartObj.dataObject.updateSeriesdatabyCategories(d3_values(d), xCategories, yCategories, yaxiscolumnorder);

                if (xAxisReverseed) {
                    serData = serData.slice(0).reverse();
                }

                chart.seriesdataAfterUpdate[i] = serData;

                var ShiftedData; // = d3_values(d);
                if (shiftInfo != null) {
                    ShiftedData = d3_values(d);
                    var co = shiftInfo.count;
                    for (var ii = 0; ii <= co; ii++) {
                        ShiftedData.splice(ii, 0, [null, null])
                    }
                    
                    if (isredraw) {
                        var clipX = xscale(dataset.getX(d[0]));
                        var properties = {
                            name: "rect",
                            id: chartObj.id,
                            x: clipX,
                            y: 0,
                            width: chartObj.plotarea.width,
                            height: chartObj.plotarea.height
                        },
                        args = {
                            chartObj: chartObj
                        };
                        var clipID = effects.get(svg_clips, "rect", properties, args);
                        series.attr('clip-path', "url(" + defsLocation + "#" + clipID + ")");
                    }
                }
                var dataUpdate = (shiftInfo != null) ? ShiftedData : chart.seriesdataAfterUpdate[i];
                duration = ((animationType == "verticalAll") || isredraw) ? transitionDuration : (transitionDuration / (dataUpdate.length * noofbarDatas));
                
                var showLevelMarker = levelMarker.enabled === true &&
                    (isBullet || (stacked && noofbarDatas > 0) || !ishideable) && !chartObj.dataObject.isStackedPercentChart &&
                    ((stackGroup) ? chartObj.dataObject.groupedByStack.has(actualStackGroupIndex) : true); // If specified stackgroup is disabled, remove it
                var subseries_levelMarker = (stacked ? container : series)
                    .selectAll(stacked ? 'g.levelMarkerSeries_' + actualStackGroupIndex : 'g.subseries_levelMarker_' + i)
                    .data(function(d) {
                        return showLevelMarker ? [d] : [];
                    });
                var subseries = series.selectAll('g.subseries_' + i).data(function(d) {
                    return ishideable ? [] : [d];
                });
                
                //Removing unnecessary 'g' on redraw ((stacked bar to bar) or (bar to stacked bar))
                (!stacked ? container : series)
                    .selectAll(!stacked ? 'g.levelMarkerSeries_' + actualStackGroupIndex : 'g.subseries_levelMarker_' + i)
                    .remove();
                
                subseries_levelMarker.exit()
                    .selectAll('path.levelMarker')
                    .transition()
                    .duration(exitDuration)
                    .attr("d", function(d, j) {
                        if (isGantt) {
                            var path = d3_select(this).attr("d"); // If the category is no more available at the categoryStore, We get the x,y from the 'd' attr by parsing
                            var points = parseSVGPath(path);
                            var x = points[0].moves[0],
                                y = points[3].moves[1];
                            
                            var prop = getGanttLevelMarkerProp(d, i, j, commonOption);
                            var exitx = prop.x || x,
                                exity = (prop.y + (axisrotated ? 0 : prop.height * (yaxisReversed ? -1 : 1))) || y,
                                exitwid = axisrotated ? 0 : prop.width,
                                exithei = axisrotated ? prop.height : 0;
                            return roundedRect(exitx, exity, exitwid, exithei, borderRadius, yaxisReversed, axisrotated);
                        }
                        var exitx = chart.getParamValue("x", d, i, j, commonOption) || parseSVGPath(d3_select(this).attr("d"))[0].moves[0],
                            exity = yscale(yscale.domain()[0]),
                            exitwid = axisrotated ? 0 : chart.getParamValue("width", d, i, j, commonOption),
                            exithei = axisrotated ? chart.getParamValue("height", d, i, j, commonOption) : 0;
                        return roundedRect(exitx, exity, exitwid, exithei, borderRadius, yaxisReversed, axisrotated);
                    })
                    .remove();
                
                subseries.exit()
                    .selectAll("path.zcbar")
                    .transition()
                    .duration(exitDuration)
                    .attr("d", function(d, j) {
                        var path = d3_select(this).attr("d"); // If the category is no more available at the categoryStore, We get the x,y from the 'd' attr by parsing
                        var points = parseSVGPath(path);
                        var x = points[0].moves[0],
                            y = points[3].moves[1],
                            exitx = chart.getParamValue("x", d, i, j, commonOption) || x,
                            exity = (axisrotated ? chart.getParamValue("y", d, i, j, commonOption) : isGantt ? chart.getParamValue("y", d, i, j, commonOption, null, 0) :
                                        (stacked ? yscale(d.y0) : ((hasNegValue != null) ? yscale(hasNegValue) : (yscale(0) != null ? mathMin(yscale(0), yscale.range()[0]) : yscale.range()[0]))) + (addRangeBand() * 2)) || y,
                            exitwid = axisrotated ? 0 : chart.getParamValue("width", d, i, j, commonOption),
                            exithei = axisrotated ? chart.getParamValue("height", d, i, j, commonOption) : 0;
                        return roundedRect(exitx, exity, exitwid, exithei, borderRadius, yaxisReversed, axisrotated);
                    })
                    .remove();
                
                
                subseries.enter().append('g').attr('class', 'subseries_' + i);
                
                var subseries_levelMarkerZindex = (isGantt) ? subseries_levelMarker.enter().append("g") : subseries_levelMarker.enter().insert('g', ':first-child');
                
                subseries_levelMarkerZindex.attr('class', stacked ? 'levelMarkerSeries_' + actualStackGroupIndex : 'subseries_levelMarker_' + i);
                
                var rect_levelMarker = subseries_levelMarker.selectAll("path.levelMarker").data(dataUpdate, function(d) {
                    return (isGantt) ? d : dataset.getX(d);
                });
                
                rect_levelMarker.enter().append("path").attr("class", "levelMarker");
                rect_levelMarker
                    .style({
                        "fill": function(d, j) {
                            if (isBullet) {
                                if (!currentdata.disabled) {
                                    if (LevelMarkerEventsEnabled && tooltipEnabled) {
                                        // get the points and save in bulletEvent.points
                                        var rectangles = getBulletLevelMarkerRect(d, levelMarker, "y");
                                        var baseIndex = j * rectangles.length;
                                        bulletEvent.levelmarker = bulletEvent.levelmarker.concat(rectangles);
                                    }
                                    
                                    
                                    if (!disabledBulletEvents && tooltipEnabled) {
                                        // save the data point here to avoid loops
                                        var point = getPos(d);
                                        point.data = d;
                                        point.seriesPos = index;
                                        point.subseriesindex = i;
                                        point.itemPos = j;
                                        point.type = "point";
                                        point.yaxiscolumnorder = yaxiscolumnorder;
                                        bulletEvent.point = bulletEvent.point.concat(point);
                                    }
                                }
                                
                            }
                            if (isGantt) {
                                return levelMarker.color ||
                                    (multiColoring ? chartObj.seriesColor[j % chartObj.seriesColor.length] : color);
                            }

                            var baseClr = (multiColoring ? chartObj.seriesColor[j % chartObj.seriesColor.length] : color);
                            var clr = (isArray(levelMarker.color) ? levelMarker.color[0] : levelMarker.color) ; 
                            if(patternPallete){
                                var prop = (isGantt) ? getGanttLevelMarkerProp(d, i, j, commonOption) : getBulletLevelMarkerProp(d, commonOption);
                                var x = prop.x,
                                    y = prop.y,
                                    wid = prop.width,
                                    hei = prop.height;
                               clr =  fillColor((multiColoring ? j : index), null,null,clr,null,null,{"x":x, "y": y, "width":wid, "height":hei}); 
                            }

                            return getBulletLevelMarkerGradient(d, levelMarker, chartObj, baseClr, levelMarkerColor) || clr || baseClr;
                        },
                        "fill-opacity": levelMarker.fillOpacity
                    })
                    .attr("d", function(d, j) {
                        var prop = (isGantt) ? getGanttLevelMarkerProp(d, i, j, commonOption) : getBulletLevelMarkerProp(d, commonOption);
                        var x = prop.x,
                            y = prop.y,
                            wid = prop.width,
                            hei = prop.height;
                        if (isGantt) {
                            return roundedRect(x, y, wid, hei, levelMarkerRadius, yaxisReversed, axisrotated);
                        } else {
                            return (axisrotated) ? roundedRect(y, x, wid, hei, borderRadius, yaxisReversed, axisrotated) : roundedRect(x, y, wid, hei, borderRadius, yaxisReversed, axisrotated)
                        }
                    });
                
                levelMarker.color = levelMarkerColor; //Saving back the levelMarker colors, helps in multi categories bullet if each categories dont have same no of levels
                rect_levelMarker.exit().remove();
                
                if (isBullet) {
                    addMarker(targetMarker, "");
                    addTargetLabel.call(chartObj, targetMarker, chart, dataUpdate);
                } else if (isGantt) {
                    addMarker(targetMarker.start, 0);
                    addMarker(targetMarker.end, 1);
                }
                
                var rect = subseries
                    .selectAll("path.zcbar")
                    .attr("class", "zcbar")
                    .data(dataUpdate, function(d) {
                        return (isGantt) ? d : dataset.getX(d); // For gantt chart we should support different timeline data for the same category
                    });
                
                
                var rectEnter = rect.enter().append("path")
                    .style('fill-opacity', gvp("fillOpacity"))
                    .attr({
                        "class": "zcbar",
                        "y": function(d, j) {
                            //for stacked animation should starts from baseline i.e data[0].data[0][0].y0
                            return (yscale.rangeExtent) ?
                                yscale.rangeExtent()[axisrotated ? 0 : 1] :
                                (axisrotated ? ((stacked) ? (hasNegValue != null) ? yscale(hasNegValue) : yscale.range()[0] : ((hasNegValue != null) ? yscale(hasNegValue) : yscale.range()[0])) :
                                    ((stacked) ? ((hasNegValue != null) ? yscale(hasNegValue) : yscale.range()[0]) : ((hasNegValue != null) ? yscale(hasNegValue) : (yscale(0) != null ? mathMin(yscale(0), yscale.range()[0]) : yscale.range()[0]))));
                        },
                        "height": 0,
                        "d": function(d, j) {
                            var v = dataset.getX(d);
                            startx = (stacked) ? bandwidth1(v) : bandwidth1(v) + bandwidth2(barDatasetIndex + i)
                            starty = (yscale.rangeExtent) ?
                                yscale.rangeExtent()[axisrotated ? 0 : 1] :
                                (axisrotated ? ((stacked) ? (hasNegValue != null) ? yscale(hasNegValue) : yscale.range()[0] : ((hasNegValue != null) ? yscale(hasNegValue) : yscale.range()[0])) :
                                    ((stacked) ? ((hasNegValue != null) ? yscale(hasNegValue) : yscale.range()[0]) : ((hasNegValue != null) ? yscale(hasNegValue) : (yscale(0) != null ? mathMin(yscale(0), yscale.range()[0]) : yscale.range()[0]))));
                            startwid = (axisrotated) ? 0 : barWidth
                            starthei = (axisrotated) ? barWidth : 0
                            return (axisrotated) ? roundedRect(starty, startx, startwid, starthei, startRadius, yaxisReversed, axisrotated) : roundedRect(startx, starty, startwid, starthei, startRadius, yaxisReversed, axisrotated)
                        }
                    });
                
                var rectUpdate = rect
                    .attr({
                        "stroke": function(d, j) {
                            return strokeColor((multiColoring ? j : index), d, borderColor);
                        },
                        "stroke-width": borderSize,
                        "stroke-dasharray": dashStyle
                    })
                    .style({
                        "fill-opacity": gvp("fillOpacity"),
                        "fill": function(d, j) {
                            if(patternPallete){
                               return fillColor((multiColoring ? j : index), d,null,null,null,null,{"x":chart.getParamValue("x", d, i, j, commonOption), "y": chart.getParamValue("y", d, i, j, commonOption),
                                                                                                        "width": chart.getParamValue("width", d, i, j, commonOption), "height": chart.getParamValue("height", d, i, j, commonOption)}); 
                            }
                            return fillColor((multiColoring ? j : index), d);
                        }
                    });
                if (shiftInfo == null) {
                    rect
                        .transition()
                        .duration((isredraw) ? duration : 0)
                        .delay(function(d, j) {
                            return (animationType == "verticalAll" || isredraw) ? 0 : (((j * noofbarDatas) + index) * duration);
                        })
                        
                        .attr("d", function(d, j) {
                            var curele = d3_select(this);
                            var starthei = parseFloat(curele.attr("height"));
                            endx = axisrotated && !isGantt ? parseFloat(curele.attr("y")) : chart.getParamValue("x", d, i, j, commonOption)
                            endy = axisrotated ? chart.getParamValue("y", d, i, j, commonOption) : (isGantt && !starthei) ? chart.getParamValue("y", d, i, j, commonOption, null, 0) : parseFloat(curele.attr("y")) // Why (isGantt && !starthei)? -> rectEnter animation should start from y0
                            endwid = axisrotated ? parseFloat(curele.attr("height")) : chart.getParamValue("width", d, i, j, commonOption)
                            endhei = axisrotated ? chart.getParamValue("height", d, i, j, commonOption) : starthei
                            return roundedRect(endx, endy, endwid, endhei, borderRadius, yaxisReversed, axisrotated);
                        })
                        
                        .each("end", function(d, k) {
                            d3_select(this).transition()
                                .delay(function(d, j) {
                                    return (animationType == "verticalAll" || isredraw) ? 0 : (((k * noofbarDatas) + index) * duration);
                                })
                                .duration(duration)
                                .ease(easingType)
                                .attr("d", function(d) {
                                    endx = chart.getParamValue("x", d, i, k, commonOption)
                                    endy = chart.getParamValue("y", d, i, k, commonOption)
                                    endwid = chart.getParamValue("width", d, i, k, commonOption)
                                    endhei = chart.getParamValue("height", d, i, k, commonOption)
                                    return roundedRect(endx, endy, endwid, endhei, borderRadius, yaxisReversed, axisrotated);
                                })
                        })
                        .attr({
                            "y": function(d, j) {
                                var param = (axisrotated) ? "x" : "y"
                                return chart.getParamValue(param, d, i, j, commonOption);
                            },
                            "height": function(d, j) {
                                var param = (axisrotated) ? "width" : "height"
                                return chart.getParamValue(param, d, i, j, commonOption);
                            }
                        });
                    
                } else {
                    rect
                        .transition()
                        .duration((isredraw) ? duration : 0)
                        .attr({
                            "d": function(d, j) {
                                endx = chart.getParamValue("x", d, i, j, commonOption);
                                endy = chart.getParamValue("y", d, i, j, commonOption);
                                endwid = chart.getParamValue("width", d, i, j, commonOption);
                                endhei = chart.getParamValue("height", d, i, j, commonOption);
                                return roundedRect(endx, endy, endwid, endhei, borderRadius, yaxisReversed, axisrotated)
                            },
                            "y": function(d, j) {
                                var param = (axisrotated) ? "x" : "y";
                                return chart.getParamValue(param, d, i, j, commonOption);
                            },
                            "height": function(d, j) {
                                var param = (axisrotated) ? "width" : "height";
                                return chart.getParamValue(param, d, i, j, commonOption);
                            }
                        });
                }
                
                rect.exit()
                    .transition()
                    .duration(duration)
                    .remove();
                
                if (i > 0 && stacked) {
                    var subdividerseries = appendEle(series, "g", (ishideable ? [] : [1]), ".", "class", ('subdividerseries_' + i));
                    var dividerbar = subdividerseries
                        .selectAll("rect")
                        .data(dataUpdate, function(d) {
                            return dataset.getX(d)
                        });
                    dividerbar
                        .enter().append("rect")
                        .style({
                            "fill": "white",
                            "opacity": 0.7,
                            "shape-rendering": "crispedges"
                        })
                        .attr({
                            "width": getDividerParam("width"),
                            "height": getDividerParam("height"),
                            "x": function(d, j) {
                                return getDividerParam("x", d, i, j, commonOption);
                            },
                            "y": function(d, j) {
                                return getDividerParam("y", d, i, j, commonOption)
                            }
                        });
                    dividerbar
                        .transition()
                        .duration(duration)
                        .ease(easingType)
                        .style({
                            "fill": "white",
                            "opacity": 0.7,
                            "shape-rendering": "crispedges"
                        })
                        .attr({
                            "width": getDividerParam("width"),
                            "height": getDividerParam("height"),
                            "x": function(d, j) {
                                return getDividerParam("x", d, i, j, commonOption);
                            },
                            "y": function(d, j) {
                                return getDividerParam("y", d, i, j, commonOption)
                            }
                        });
                    dividerbar.exit().remove();
                }
                if (isGantt) {
                    
                    var dependencyConfig = commonPlotOptions.dependency,
                        showDependency = dependencyConfig.enabled === true;
                    
                    var dependencies = null;
                    var dependenceSeries = series
                        .selectAll('g.dependencySeries')
                        .data(showDependency ? [d] : []);
                    
                    var seriesDpn = dependenceSeries.enter().insert('g', ':first-child');
                    dependenceSeries.attr('class', 'dependencySeries');
                    dependenceSeries.exit().remove()
                    if (showDependency && defined(dependencyConfig.sourceIndex) && defined(dependencyConfig.targetIndex)) {
                        dependencies = getDependencies(dataUpdate, +dependencyConfig.sourceIndex, +dependencyConfig.targetIndex);
                    }
                    if (dependencies) {
                        var symbolConf = dependencyConfig.marker,
                            linkConf = dependencyConfig.link,
                            linkPadding = defined(linkConf.padding) ? linkConf.padding : 1,
                            symbolPadding = defined(symbolConf.padding) ? symbolConf.padding : linkPadding
                        symbolPadding = mathMin(linkPadding, symbolPadding)
                        // dependency links
                        
                        var linkGroup = appendEle(dependenceSeries, "g", [1], '.', "class", "links");
                        var arrowGroup = appendEle(dependenceSeries, "g", [1], '.', "class", "symbol");
                        
                        // links
                        var path = appendEle(linkGroup, "path", dependencies, ".", "class", "link");
                        path
                            .attr("d", function(d, j) {
                                var details = getDependencyDetails(d);
                                return getDependencyPathString(details.type, details.end, details.start)(new Array(6));
                            })
                            .attr("stroke", function(d) {
                                var startTime = dataset.getY(d.source, yaxiscolumnorder)[0];
                                var endTime = dataset.getY(d.target, yaxiscolumnorder)[1];
                                var error = startTime < endTime;
                                return error ? linkConf.errorStyle : linkConf.strokeColor
                            })
                            .attr("stroke-width", linkConf.strokeWidth)
                            .attr("fill", "none");
                        
                        // symble
                        
                        var markerEnabled = dependencyConfig.marker.enabled === true;
                        var arrows = appendEle(arrowGroup, "g", markerEnabled ? dependencies : [], ".", "class", "arrow");
                        
                        if (dependencyConfig.marker.enabled) {
                            arrows.each(function(d, j) {
                                var symbolType = symbolConf.symbol,
                                    temp;
                                var markerID, mSize = {
                                    "inner": 0.4,
                                    "outer": 0.7
                                };
                                var self = d3_select(this);
                                self.attr("transform", function(d) {
                                    var details = getDependencyDetails(d),
                                        x, y;
                                    var type = details.type
                                    if (type == "SS" || type == "ES") {
                                        x = axisrotated ? details.start.x - symbolPadding : details.start.x + details.start.width / 2;
                                        y = axisrotated ? details.start.y + details.start.height / 2 : details.start.y + details.start.height + symbolPadding
                                        temp = axisrotated ? "triangle-right" : "triangle-up"
                                    } else if (type == "EE" || type == "SE") {
                                        x = axisrotated ? details.start.x + details.start.width + symbolPadding : details.start.x + details.start.width / 2;
                                        y = axisrotated ? details.start.y + details.start.height / 2 : details.start.y - symbolPadding
                                        temp = axisrotated ? "triangle-left" : "triangle-down"
                                    }
                                    return "translate(" + x + "," + (y) + ")";
                                })
                                addSVGmarker(self, null, symbolConf, {
                                    "symbolSize": mSize,
                                    "color": null,
                                    "symbol": temp
                                });
                            })
                        }
                    }
                }
                
                function getDependencyDetails(d) {
                    return {
                        "start": getPos(d.source),
                        "end": getPos(d.target),
                        "type": dependencyConfig.link.type.toUpperCase() || "ES"
                    }
                }
                
                function getDependencyPathString(type, startPosition, endPosition) {
                    var padding = linkPadding,
                        markerPadding = symbolPadding;
                    type = type.toUpperCase();
                    
                    var x = null,
                        y = null,
                        pos = null,
                        crucialCurve = isCrucialCurve(type, startPosition, endPosition);
                    
                    return d3_line()
                        .x(function(d, j) {
                            switch (j) {
                                case 0:
                                case 1:
                                case 2: {
                                    pos = startPosition;
                                    if (j == 0) {
                                        if (type == "SS" || type == "SE") {
                                            x = axisrotated ? pos.x : pos.x + pos.width / 2
                                        } else if (type == "EE" || type == "ES") {
                                            x = axisrotated ? pos.x + pos.width : pos.x + pos.width / 2
                                        }
                                    } else if (j == 1) {
                                        if (type == "SS" || type == "SE") {
                                            x = axisrotated ? pos.x - padding : x
                                        } else if (type == "EE" || type == "ES") {
                                            x = axisrotated ? pos.x + pos.width + padding : x
                                        }
                                        
                                    } else {
                                        var dist;
                                        if (type == "SS") {
                                            dist = startPosition.x - endPosition.x;
                                            dist = dist < 0 ? 0 : dist
                                            x = axisrotated ? pos.x - dist - padding : x
                                            
                                        } else if (type == "EE") {
                                            dist = (startPosition.x + startPosition.width) - (endPosition.x + endPosition.width);
                                            dist = dist > 0 ? 0 : dist
                                            x = axisrotated ? pos.x + pos.width + padding - dist : x;
                                        } else if (type == "SE") {
                                            if (axisrotated) {
                                                x = pos.x - padding;
                                                y = crucialCurve ? pos.y + pos.height : endPosition.y;
                                            } else {
                                                x = xAxisReverseed ? x - pos.width : x + pos.width;
                                            }
                                        } else if (type == "ES") {
                                            if (axisrotated) {
                                                x = pos.x + padding + pos.width;
                                                y = crucialCurve ? pos.y + pos.height : endPosition.y;
                                            } else {
                                                x = xAxisReverseed ? x - pos.width : x + pos.width;
                                            }
                                        }
                                    }
                                    break;
                                }
                                default: {
                                    if (axisrotated) {
                                        y = null
                                    }
                                    pos = endPosition
                                    if (j == 3) {
                                        if (axisrotated && crucialCurve) {
                                            y = startPosition.y + startPosition.height;
                                        }
                                        if (type == "SS" || type == "EE") {
                                            x = axisrotated ? x : pos.x + pos.width / 2;
                                        } else if (type == "SE") {
                                            x = axisrotated ? pos.x + pos.width + padding : x
                                        } else if (type == "ES") {
                                            x = axisrotated ? pos.x - padding : x
                                        }
                                        
                                    } else if (j == 4) {
                                        if (type == "SE") {
                                            x = axisrotated ? pos.x + pos.width + padding : pos.x + pos.width / 2
                                        } else if (type == "ES") {
                                            x = axisrotated ? x : pos.x + pos.width / 2
                                        }
                                    } else {
                                        if (type == "SS" || type == "ES") {
                                            x = axisrotated ? pos.x - markerPadding : x
                                        } else if (type == "EE" || type == "SE") {
                                            x = axisrotated ? pos.x + pos.width + markerPadding : x
                                        }
                                    }
                                }
                            }
                            return x
                        })
                        .y(function(d, j) {
                            if (axisrotated) {
                                return (y || pos.y) + (pos.height / 2)
                            }
                            
                            if (j == 0) {
                                if (type == "SS" || type == "SE") {
                                    y = startPosition.y + startPosition.height
                                } else if (type == "EE" || type == "ES") {
                                    y = startPosition.y
                                }
                            } else if (j == 1) {
                                if (type == "SS" || type == "SE") {
                                    y = y + padding
                                } else if (type == "EE" || type == "ES") {
                                    y = startPosition.y - padding
                                }
                                
                            } else if (j == 2) {
                                if (type == "SS") {
                                    y = (startPosition.y + startPosition.height < endPosition.y + endPosition.height) ? endPosition.y + endPosition.height + padding : y
                                } else if (type == "EE") {
                                    y = (startPosition.y > endPosition.y) ? endPosition.y - padding : y;
                                }
                            } else if (j == 3) {
                                if (type == "SE") {
                                    y = crucialCurve ? endPosition.y - padding : y
                                } else if (type == "ES") {
                                    y = crucialCurve ? endPosition.y + endPosition.height + padding : y
                                }
                                
                            } else if (j == 4) {
                                y = y + 0;
                            } else {
                                if (type == "SS" || type == "ES") {
                                    y = endPosition.y + endPosition.height + markerPadding
                                } else if (type == "EE" || type == "SE") {
                                    y = endPosition.y - markerPadding
                                }
                            }
                            return y
                        })
                        .interpolate("linear");
                }
                
                function isCrucialCurve(type, start, end) {
                    var direct = false;
                    switch (type) {
                        case "SS":
                        case "EE": {
                            direct = true
                            break;
                        }
                        case "SE": {
                            direct = axisrotated ? start.x > (end.x + end.width) : start.y + start.height < end.y
                            break;
                        }
                        case "ES": {
                            direct = axisrotated ? (start.x + start.width) < end.x - linkPadding * 2 : start.y > end.y + end.height
                        }
                    }
                    return !direct;
                }
                
                function getDependencies(source, dependencyIndex, searchIndex) {
                    var allDependencies = [];
                    source.forEach(function(dd) {
                        var dependencies = [],
                            currentDependency = dd[dependencyIndex];
                        source.forEach(function(data) {
                            if (data !== dd && defined(currentDependency) && isArray(currentDependency)) {
                                if (currentDependency.indexOf(data[searchIndex]) !== -1) {
                                    allDependencies.push({
                                        "source": dd,
                                        "target": data
                                    })
                                }
                            }
                        });
                    });
                    if (allDependencies.length) {
                        return allDependencies;
                    }
                };
                
                function getPos(d) {
                    var width, height, x, y;
                    x = chart.getParamValue("x", d, i, null, commonOption)
                    y = chart.getParamValue("y", d, i, null, commonOption)
                    width = chart.getParamValue("width", d, i, null, commonOption)
                    height = chart.getParamValue("height", d, i, null, commonOption)
                    return {
                        "x": x,
                        "y": y,
                        "width": width,
                        "height": height
                    }
                }
                
                function getDividerParam(paramType, d, i, j, cp) {
                    switch (paramType) {
                        case "x":
                            return (axisrotated) ? yscale(d.y0) : chart.getParamValue("x", d, i, j, cp);
                        case "y":
                            return (axisrotated) ? chart.getParamValue("y", d, i, j, cp) : yscale(d.y0);
                        case "width":
                            return (axisrotated) ? 1 : barWidth;
                        case "height":
                            return (axisrotated) ? barWidth : 1;
                    }
                }
                
                function getBulletLevelMarkerProp(d, cp) {
                    var level = d[levelMarker.dataindex] || [],
                        hasLevels = defined(levelMarker.dataindex),
                        min,
                        max;
                    if (level != null && !(level instanceof Array)) { //If level marker is a single value and in non-array format, make it as array
                        level = [level];
                    }
                    
                    if (hasLevels && !level.length) {
                        min = max = 0;
                    } else {
                        min = d3_min(level);
                        max = d3_max(level);
                    }
                    
                    var v = dataset.getX(d);
                    var x = (stacked ? (bandwidth1(v) + (bandwidth2(cp.stackGroupIndex) || 0)) : bandwidth1(v) + bandwidth2(barDatasetIndex + i)) + levelMarkerPadding;
                    var y = hasLevels ?
                            (axisrotated ? yscale(mathMin(0, min)) : yscale(mathMax(0, max))) :
                            ((axisrotated && (hasNegValue != null)) ? yscale(hasNegValue) : 0),
                        wid = (axisrotated) ?
                            (hasLevels ? (yscale(mathMax(0, max)) - yscale(mathMin(0, min))) : ((hasNegValue != null) ? (plotarea.width - yscale(hasNegValue)) : plotarea.width)) :
                            levelMarkerWidth,
                        hei = (axisrotated) ?
                            levelMarkerWidth :
                            (hasLevels ? (yscale(mathMin(0, min)) - yscale(mathMax(0, max))) : ((hasNegValue != null) ? yscale(hasNegValue) : plotarea.height));
                    return {
                        "x": x,
                        "y": y,
                        "width": wid,
                        "height": hei
                    }
                }
                
                function addMarker(markerOption, id) {
                    var bandwidth = isBullet ? xscale.rangeBand() : levelMarkerWidth;
                    var size = validateNullVal(markerOption.size, (levelMarkerWidth - (2 * validateNullVal(markerOption.padding, barPadding / 2))));
                    var target = targetMarkerSeries.selectAll("g.markers" + id).data([d], function(d) {
                        return [dataset.getX(d)];
                    })
                    
                    // var markerWidth = ((markerOption.strokeWidth == null || markerOption.strokeWidth === "") ? 1 : JSON.parse(markerOption.strokeWidth));
                    var markerWidth = pick(markerOption.strokeWidth, 1);
                    
                    target.enter().append("g")
                    target.exit().remove();
                    target.attr({
                        "class": "markers" + id,
                        "transform": "translate(" + plotarea.left + "," + plotarea.top + ")",
                        "stroke-width": markerWidth,
                        "stroke-dasharray": getDashArray(markerOption.dashStyle, validateNullVal(markerOption.outerStrokeWidth, 1)),
                        "stroke-linecap": getLineCap(markerOption.lineCap, markerOption.dashStyle),
                        "fill": markerOption.color,
                        "stroke": markerOption.color
                    });
                    
                    var marker = target.selectAll("g").data(dataUpdate, function(d) {
                        return dataset.getX(d);
                    });
                    marker.enter().append("g");
                    var markerID, mSize = markerOption.symbol === "line" ? {
                                "inner": 0,
                                "outer": 1
                            } : {
                                "inner": 0.6,
                                "outer": 1
                            },
                        customMarkerSize = markerOption.size;
                    
                    marker.each(function(d, j) {
                        var m = d3_select(this);
                        var clr = multiColoring ? chartObj.seriesColor[j % chartObj.seriesColor.length] : color;
                        m.attr({
                                "transform": function(d, k) {
                                    var x = 0,
                                        y = 0;
                                    if (isGantt) {
                                        var position = getMarkerPosition(d, i, k, commonOption);
                                        if (id === 0) {
                                            x = position.x + position.width/2;
                                            y = position.y  - (markerOption.size*10)/2;
                                        } else {
                                            x = position.x + position.width / 2;
                                            y = position.y + position.height + (markerOption.size*10)/2;
                                        }
                                        return "translate(" + (axisrotated ? y : x) + "," + (axisrotated ? x : y) + ")"
                                    } else {
                                        x = (bandwidth1((dataset.getX(d))) + (bandwidth / 2));
                                        y = yscale(d[markerOption.dataindex]);
                                        return (d[markerOption.dataindex] == null) ? null : "translate(" + (axisrotated ? y : x) + "," + (axisrotated ? x : y) + ") rotate(" + (axisrotated ? 90 : 0) + ")";
                                    }
                                }
                            })
                            .style("display", function(d) {
                                return (d[markerOption.dataindex] == null) && isBullet ? "none" : null;
                            });
                        markerOption.size = defined(customMarkerSize) ?
                            (isBullet ? mathMin(size, (bandwidth / 10)) : size) :
                            (size / 10)
                        addSVGmarker(m, null, markerOption, {
                            symbolSize: mSize,
                            "color": isGantt ? clr : null
                        });
                        
                        // getBulletMarkerRect
                        if (tooltipEnabled && isBullet && targetMarkerEventsEnabled) {
                            var markerPos = getBulletMarkerRect(this, d, markerOption);
                            markerPos.seriesPos = index;
                            markerPos.data = d;
                            markerPos.type = "marker";
                            markerPos.yaxiscolumnorder = yaxiscolumnorder;
                            markerPos.currentStage = d[markerOption.dataindex];
                            bulletEvent.marker.push(markerPos)
                        }
                    })
                    markerOption.size = customMarkerSize; //reset with actual value
                    
                    if (isGantt) {
                        chart.markerProperties = {
                            "subseriesIndex": i
                        };
                    }
                }
                
                // returns marker position relative to the event rectangle
                
                function getBulletMarkerRect(el, d, markerOption) {
                    var elRect = bbox(el);
                    
                    var x = bandwidth1((dataset.getX(d))),
                        y = yscale(d[markerOption.dataindex]),
                        width = elRect.width,
                        height = mathMax(elRect.height, markerOption.outerStrokeWidth || 0, markerOption.innerStrokeWidth || 0);
                    x += (xscale.rangeBand() - elRect.width) / 2;
                    y -= height / 2;
                    if (height < 5) {
                        height = 5;
                        y -= 2.5;
                    }

                    // The target marker bound might be very small. triggeringOffset extends its boundary for event
                    var eventOptions = markerOption.events || {};
                    var triggerOffset = eventOptions.triggerOffset || 0;
                    x -= (triggerOffset / 2);
                    y -= (triggerOffset / 2);
                    width += triggerOffset;
                    height += triggerOffset;

                    return {
                        x: axisrotated ? y : x,
                        y: axisrotated ? x : y,
                        width: axisrotated ? height : width,
                        height: axisrotated ? width : height
                    };
                }
                
                function getBulletLevelMarkerRect(d, levelMarker, type) {
                    
                    if (d.isDummy) {
                        return [];
                    }
                    
                    var scale = (type == "x") ? xscale : yscale,
                        x = bandwidth1(dataset.getX(d)) + levelMarkerPadding,
                        y,
                        width = xscale.rangeBand(),
                        height,
                        datatype = chartObj.dataset.getDataType(type, 0),
                        formatConf,
                        temp = [],
                        rectangles = [],
                        originalValues = d[levelMarker.dataindex] || [],
                        col;
                    
                    if (originalValues != null && !(originalValues instanceof Array)) { //If level marker is a single value and in non-array format, make it as array
                        originalValues = [originalValues];
                    }
                    var steps = originalValues.slice(0);
                    
                    if (datatype == TIME_DATATYPE) {
                        var column = chartObj.dataset.getColumn(type, 0);
                        formatConf = defined(column.time) ? column.time : null
                    }
                    
                    steps.push(0); // add for get the base.
                    steps = steps.map(function(d) {
                        return (datatype == TIME_DATATYPE) ? generateDateVal(d, null, formatConf).getTime() : d;
                    });
                    steps = !isRotatedChart(type, axisrotated) ? steps.sort(d3_ascending) : steps.sort(d3_descending);
                    steps.forEach(function(target) {
                        var val = scale(target) + (defined(scale.rangeBand) ? scale.rangeBand() / 2 : 0);
                        temp.push(val);
                    });
                    steps.splice(steps.indexOf(0), 1); // remove
                    
                    col = temp.length - 1;
                    while (col--) {
                        height = temp[col + 1] - temp[col];
                        
                        if (height > 0) {
                            y = temp[col];
                            
                        }
                        y = temp[col];
                        if (height > 0) {
                            rectangles.push({
                                x: axisrotated ? y : x,
                                y: axisrotated ? x : y,
                                width: axisrotated ? height : width,
                                height: axisrotated ? width : height,
                                currentStage: steps[col],
                                seriesPos: index,
                                data: d,
                                yaxiscolumnorder: yaxiscolumnorder,
                                type: "levelmarker"
                            });
                        }
                    }
                    return rectangles;
                }
                
                function getMarkerPosition(d, i, j, commonOption) {
                    var width, height, x, y;
                    x = chart.getParamValue(axisrotated ? "y" : "x", d, i, j, commonOption)
                    y = chart.getParamValue(axisrotated ? "x" : "y", d, i, j, commonOption)
                    width = chart.getParamValue(axisrotated ? "height" : "width", d, i, j, commonOption)
                    height = chart.getParamValue(axisrotated ? "width" : "height", d, i, j, commonOption)
                    return {
                        "x": x,
                        "y": y,
                        "width": width,
                        "height": height
                    }
                }
                
                chart.getMarkerPosition = getMarkerPosition;
                
                function getGanttLevelMarkerProp(d, i, j, cp) {
                    var percentComplete = d[levelMarker.dataindex];
                    var yVal = dataset.getY(d, yaxiscolumnorder);
                    var startVal = yVal[0],
                        endVal = yVal[1],
                        maxRange = cp && cp.yscale.range()[1];
                    var diff = endVal - startVal;
                    var marginIncrease = (diff * percentComplete) / 100
                    var finalVal = cp.yscale(startVal + marginIncrease);
                    
                    var xPadding = (axisrotated) ? 0 : cp.levelMarkerPadding,
                        yPadding = (axisrotated) ? cp.levelMarkerPadding : 0;


                    var x, y, wid, hei;
                    x = chartObj.dataObject.getTransRectValue("x", d, i, j, cp) + xPadding;
                    var basevalue = cp.hasNegValue == null ? cp.yscale(dataset.getY(d, cp.yaxiscolumnorder, 0)) : cp.yscale(cp.hasNegValue);
                    if (cp.axisrotated) {
                        y = chart.getParamValue("y", d, i, j, cp) + yPadding;
                        wid = (cp.stacked) ? (mathAbs(cp.yscale(d.y + d.y0) - cp.yscale(d.y0))) + cp.addRangeBand(cp.yscale) :
                            (mathAbs((maxRange - basevalue) - (maxRange - finalVal)) || 0) + cp.addRangeBand(cp.yscale);
                        hei = cp.levelMarkerWidth || cp.barWidth;
                    } else {
                        y = finalVal;
                        wid = cp.levelMarkerWidth || cp.barWidth;
                        hei = cp.stacked ? mathAbs(((basevalue) - cp.yscale(d.y + d.y0)) - ((basevalue) - cp.yscale(d.y0))) + cp.addRangeBand(cp.yscale) :
                            mathAbs(basevalue - finalVal) + cp.addRangeBand(cp.yscale);
                    }
                    return {
                        "x": x,
                        "y": y,
                        "width": wid,
                        "height": hei
                    }
                }
            })
            
            labelQueue.push({
                "renderer": chart,
                "charttype": "bar",
                "chartObj": chartObj,
                "args": [currentdata, index]
            });
            
        });
        chart.renderedTimer = setTimeout(function() {
            isredraw = true;
            chart.chartRendered = true;
        }, (2 * transitionDuration + 10));
        return chart;
    }
    
    chart.commonRendererProp = function(_) {
        if (!arguments.length) {
            return commonRendererProp;
        }
        commonRendererProp = _;
        return chart;
    };
    
    chart.bandwidth1 = function(_) {
        if (!arguments.length) {
            return bandwidth1;
        }
        bandwidth1 = _;
        return chart;
    };
    
    chart.bandwidth2 = function(_) {
        if (!arguments.length) {
            return bandwidth2;
        }
        bandwidth2 = _;
        return chart;
    };
    
    chart.barDatasetIndex = function(_) {
        if (!arguments.length) {
            return barDatasetIndex;
        }
        barDatasetIndex = _;
        return chart;
    };

    chart.shiftInfo = function(_) {
        if (!arguments.length) {
            return shiftInfo;
        }
        shiftInfo = _;
        return chart;
    };
    
    return chart;
};