//$Id$
//Returns the chart object after setting all the required properties and methods


(function(exports) {

    "use strict";

    // TODO renaming exports or $ZC
    exports.charts.get = function(chartHolder, data) {
        return new Chart(chartHolder, data);
    };

    function Chart(container, data) {
        this.container = container;
        this.userdata = data;
        this.dataObject = null;
        this.dataset = null;
        this.svg = null;
        this.canvasarea = {};
        this.chartarea = {};
        this.legendarea = {};
        this.plotarea = {};
        this.seriesColor = [];
        this.axes = {};
        this.renderer = [];
        this.subrenderer = [];
        this.ismultiy = null;
        this.plotarea_approx = null;
        this.nochartdata = false;
        this.chartRendered = false;
        this.plot = {};
        this.events = {};
        this.legend = {
            component: null
        };

        this.availableWidth = null;
        this.availableHeight = null;
        this.initialize();
    }

    Chart.prototype.initialize = function() {

        var chart = this;

        try {

            var data = this.userdata,
                dataProcessorHelpers = chart.dataProcessorHelpers = $ZC.DataProcessor.helpers;


            chart.chartRendered = false;
            chart.chartFontProp = {};
            chart.chartHeight = parseFloat(chart.container.style("height"));
            chart.chartWidth = parseFloat(chart.container.style("width"));

            chart.createHelperElements();
            chart.handleHiddenContainer();

            Chart.helpers.initializeChartHolders(chart);
            chart.setCanvasDimension();

            chart.axes.x = {
                levels: []
            };

            chart.axes.y = {
                groups: []
            };

            // get all chart types
            chart.chartTypes = dataProcessorHelpers.getAllChartTypes(data, "object");
            chart.globalProp = setGlobalPropByUserdata(data, {
                "width": chart.canvasarea.width,
                "height": chart.canvasarea.height
            }, chart.chartFontProp);

            chart.themeoptions = chart.getThemeOptions(chart.globalProp);

            (!data.metadata) ? $ZC.setDefaultOptions("metadata", data) : null; // If metadata is unavailable, get default metadata

            chart.processUserData(chart.themeoptions);
            chart.setChartDimension();

            chart.defsLocation = getDefsLocation();
            var svg = chart.svg;
            chart.legendHighlight = new $ZC.legendHighlight(chart, chart.svg);
            svg.append('defs');
            Chart.helpers.constructChartGroups(chart);


            CURRENT_CHART_COUNT++;
            chart.id = "ZC_" + CURRENT_CHART_COUNT;

        } catch (err) {
            // console.error(err.stack);
            // console.log("Oops! Sorry something went wrong to render the chart...")
            // console.log("Error message : " + err.message);
            // console.log(err.stack);

            chart.breakOnErr = true;
            if (isObjectPropDefined(chart.systemConf, "canvas.events")) {
                invokeFunction(chart.systemConf.canvas.events.onerror, d3.event, chart, err);
            }
        }
    };

    /**
     *
     */

    Chart.prototype.createHelperElements = function() {

        // create the base container first

        if (defined(BASE_HELPER_CONTAINER)) {
            return;
        }

        var parent = d3_select(document.body);
        var baseContainer = BASE_HELPER_CONTAINER = appendEle(parent, "div", [1], ".", "class", "__baseZC_Container__");
        baseContainer
        .style({
            "position": "absolute",
            "left": BASE_CONTAINER_POSITION + PX,
            "top": BASE_CONTAINER_POSITION + PX
        });

        var svg = $ZCG.patternSVG = appendEle(baseContainer, "svg", [1], ".", "class", "svgPattern");
        svg.attr({
            "version": "1.1",
            "xmlns": d3.ns.prefix.svg
        });
        appendEle(svg, "defs", [1]);
    };

    /**
     * Handles if the container is not visible or a virtual element in document.
     * @param {Boolean} normalize to restore the cloned element
     */

    Chart.prototype.handleHiddenContainer = function(normalize) {

        var chart = this,
            container = chart.container;

        if (normalize && defined(chart.originalContainer)) {

            chart.originalContainer.node().appendChild(chart.d3container.node()); // place the cloned element under the original element.
            chart.container.remove(); // remove the cloned element.
            chart.container = chart.originalContainer; // restore the tween element.

            delete chart.isHiddenCoainer; // delete the temp variable.
            delete chart.originalContainer; // delete the temp variable.

        } else {

            var containerNode = container.node();

            if (isHidden(containerNode)) {

                var containerClone = containerNode.cloneNode(0); // clone the parent node.
                BASE_HELPER_CONTAINER.node().appendChild(containerClone); // append to visible element.
                containerClone.style.setProperty("display", "block", 'important'); // ensure its properly visible.

                chart.isHiddenCoainer = true;
                chart.originalContainer = container; // save the original container
                chart.container = d3.select(containerClone); // set pointer to our cloned element as main container.
            }
        }
    };

    /**
     * @returns {void}
     */

    Chart.prototype.draw = function() {
        this.render(false);
    };

    /**
     *
     * @param {boolean} update
     */

    Chart.prototype.render = function(update) {

        var chart = this;

        try {
            chart.updateCanvasArea();

            var systemConfig = chart.systemConf;
            chart.applyChartProperties();
            if (chart.nochartdata) {
                chart.hideLoading(chart.container);
                chart.handleNoData(systemConfig.noDataHandler);
                chart.chartRendered = true;
                stateChangeListener(chart, systemConfig.canvas.events, "chart", (update) ? "onreload" : "onload");
            } else {
                chart.getChartSeriesColor();
                chart.isAxisCategory = chart.dataObject.isAxisCategory;
                chart.isNonAxisMultiSeriesCategory = chart.dataObject.isNonAxisMultiSeriesCategory;
                chart.xymultiRenderer = chart.isAxisCategory;
                $ZC.legend.render(chart, update);

                if ($ZC.annotation) { //Adding block level annotations
                    $ZC.annotation.addBlockAnnotations(chart);
                }

                //brushing
                if (isObjectPropDefined(chart.systemConf.chart, "scroll") && chart.systemConf.chart.scroll.enabled) {
                    $ZC.setDefaultOptions("scroll", chart.systemConf.chart, chart.globalProp);
                    chart.brush = new svg_brush(chart, "x");
                    // chart.svg.call(chart.brush);
                }

                addDefaultScroll.initialize(chart);

                // Axis renderer
                stateChangeListener(chart, systemConfig.chart.axes.events, null, "init");

                if (chart.isAxisCategory) {
                    $ZC.axis.render(chart, chart.chartWidth, chart.chartHeight, update)
                } else if (chart.dataObject.isPolarAxisCategory || chart.dataObject.hasLevelMarker) {
                    var type = (chart.dataObject.hasLevelMarker) ? "y" : "x";
                    $ZC.axis.polar.render(chart, type);
                }
                chart.zscale = (defined(chart.dataset.zidx) && !chart.dataObject.isHierarchy) ? $ZC.scale.getScale(chart, chart.dataObject, chart.plotarea, chart.axes.x.orient, null, chart.dataset, "z") : null;

                stateChangeListener(chart, systemConfig.chart.axes.events, null, ((update) ? "onreload" : "onload"));

                chart.updatePlotareaProp();
                chart.drawPlotareaClippath();

                if (chart.plot.renderer) {
                    chart.plot.renderer.draw(update);
                } else {
                    var renderer = chart.plot.renderer = $ZC.plot.render(chart, update);
                    renderer.draw(false);
                }

                if (chart.isAxisCategory) {
                    chart.drawThreshold(update);
                }
                chart.handleEvent(update);
                drawBorderBox(chart.wrap.selectAll("rect.dummy-rect-filter"), {
                    width: chart.plotarea.width,
                    height: chart.plotarea.height
                });
                // stateChangeListener(chart, systemConfig.canvas.events, "chart", (update)?"onreload":"onload");
            }
        } catch (err) {
            // console.error(err.stack);
            //    console.log("Oops! Sorry something went wrong to render the chart...")
            //    console.error("Error message : " + err.message);
            //    console.warn(err.stack);
            chart.breakOnErr = true;
            if (isObjectPropDefined(systemConfig, "canvas.events")) {
                invokeFunction(systemConfig.canvas.events.onerror, d3.event, chart, err);
            }
        }
    };


    /**
     *
     */

    Chart.prototype.updateCanvasArea = function() {
        var chart = this,
            systemConfig = chart.systemConf;

        chart.container.selectAll("div.d3container")
            .style({
                "position": "relative",
                "width": chart.container.style("width"),
                "height": chart.container.style("height")
                // "-webkit-transform" : "scale(1)",
                // "transform" : "scale(1)"
            });

        var canvas = chart.container.selectAll("div.canvas-component");
        canvas.style({
            "position": "relative",
            "width": chart.canvasarea.width + "px",
            "height": chart.canvasarea.height + "px"
        });

        var canvasProp = chart.container.selectAll("div.canvas-property");
        canvasProp.style({
            "position": "absolute",
            "width": chart.canvasarea.width + "px",
            "height": chart.canvasarea.height + "px"
        });
        chart.updateCommonBBSProp(canvasProp, null, systemConfig.canvas, "canvas");
    };

    /**
     * // BBS -->Border,Background,Shadow for canvas area, chartarea
     * @param ele
     * @param subEle
     * @param data
     * @param type
     */

    Chart.prototype.updateCommonBBSProp = function(ele, subEle, data, type) {

        var chart = this,
            systemConfig = chart.systemConf;

        var border = data.border || {};
        var background = data.background || {};
        var shadow = data.shadow || {},
            borderEle;
        //background
        ele.style({
            "background-color": background.color,
            "opacity": background.alpha
        });

        var bgImg = null, useGradient = getPropVal(background, "gradients.show") === true;
        if (useGradient) {
            var gradient, gradientColors = "";
            if (!defined(background.gradients.useCSS)) {
                if (defined(background.gradients.stopColor)) {
                    background.gradients.stopColor.map(function(d, i) {
                        var offset = (defined(background.gradients.stopOffset) && defined(background.gradients.stopOffset[i])) ? " " + background.gradients.stopOffset[i] : "";
                        gradientColors = gradientColors + ((i === 0) ? (d + " " + offset) : ("," + d + offset))
                    })
                }
                if (background.gradients.type === "radial") {
                    gradient = "radial-gradient(" + gradientColors + ")"
                } else if (background.gradients.type === "linear") {
                    gradient = "linear-gradient(" + gradientColors + ")"
                }
            } else {
                gradient = background.gradients.useCSS
            }


            if ($Browser.WEBKIT) {
                bgImg = "-webkit-" + gradient;
            } else if ($Browser.FIREFOX) {
                bgImg = "-moz-" + gradient;
            } else if ($Browser.IE) {
                bgImg = "-ms-" + gradient;
                var filter = (defined(background.gradients.stopColor)) ? ((background.gradients.type === "linear") ? "progid:DXImageTransform.Microsoft.gradient(startColorstr=" + (background.gradients.stopColor[0] || "black") + ", endColorstr=" + (background.gradients.stopColor[1] || "white") + ")" :
                    "progid:DXImageTransform.Microsoft.Alpha(opacity=100, finishopacity=0, style=2)") : null;
                ele.style("filter", filter);
            }

        }

        if (defined(background.imageUrl)) {
            bgImg = "url(" + background.imageUrl + ")";
        }
        if (!(($Browser.IE === 1) && ($Browser.VERSION.indexOf("9.") === 0))) {
            ele.style({
                "background-image": bgImg,
                "background-repeat": background.imageRepeat,
                "opacity": background.alpha
            })
        }

        //border
        if (border.show === true && type === "canvas") {

            borderEle = (type === "plotarea") ? subEle : ele;

            //border size validation
            var borderSize = (type === "canvas") ? validateValWithinLimit(systemConfig.canvas.border, "size", 0, 10, chart.canvasarea.width, chart.canvasarea.height) :
                validateValWithinLimit(systemConfig.chart.plot.border, "size", 0, 10, chart.plotarea.width, chart.plotarea.height);
            borderSize = pick(borderSize, 2);
            borderEle.style({
                "border": borderSize + "px " + border.style + " " + border.color,
                "border-radius": pInt(border.radius) + "px"
            });
            borderEle.node().border = {
                "size": borderSize,
                "clr": border.color
            }; //bkpinfo
        } else {
            borderEle = (type === "plotarea") ? subEle : ele;
            if (type !== "plotarea") {
                borderEle.style({
                    "border": ""
                })
            }
        }

        //shadow
        if (shadow.show === true) { // only for canvas area
            var shadowstyle = shadow.x + "px " + shadow.y + "px " + shadow.blur + "px " + shadow.color;
            // var shadowstyle = [shadow.x, shadow.y, shadow.blur, shadow.color].join(PX);

            if (shadow.inset === true) {
                shadowstyle += " inset";
            }

            ele.style({
                "box-shadow": shadowstyle
            })
        } else {
            ele.style("box-shadow", "");
        }
    };

    /**
     *
     * @param baseSystemConf
     * @param update
     */

    Chart.prototype.processUserData = function(baseSystemConf, update) {

        var chart = this,
            systemConfig,
            series, chartdata;

        chart.systemConf = systemConfig = mergeJSON(baseSystemConf, chart.userdata, true, true);
        series = getPropVal(systemConfig, "seriesdata") || {};
        chart.seriesdata = chartdata = series.chartdata;

        if (!update) {
            stateChangeListener(chart, systemConfig.canvas.events, "chart", "init");
        }

        chart.nochartdata = !defined(chartdata) || (defined(chartdata) && chartdata.length < 1);

        if (chart.nochartdata) {
            chart.seriesdata = null
        } else {
            chart.dataset = new $ZC.dataset.get(systemConfig, chart);
            chart.preProcessor = new $ZC.PreProcessor(chart);
            chart.dataObject = chart.getDataObject();
            chart.dataset.setDataObj(chart.dataObject);

            chart.nochartdata = chart.dataObject.totalnoofdatas < 1;
            chart.axes.rotated = chart.dataObject.getAxisRotated();

            if(chart.axes.rotated) {
                chart.axes.x.orient = "left";
                chart.axes.y.orient = "bottom";
            } else {
                chart.axes.x.orient = "bottom";
                chart.axes.y.orient = "left";
            }
            chart.ismultiy = chart.getAxesCount("y") > 1;
            chart.ismultix = chart.getAxesCount("x") > 1;
        }
    };

    /**
     *
     * @param width
     * @param height
     */

    Chart.prototype.setCanvasDimension = function(width, height) {
        var chart = this,
            svg = chart.svg,
            width_style, height_style;

        if (!width && !height) {
            width_style = svg.style("width");
            height_style = svg.style("height");
            //need to change validateNullval under zcutils and to use this for width_style
            width_style = (!defined(width_style) || width_style === "auto") ? 960 : parseFloat(width_style);
            height_style = (!defined(height_style) || height_style === "auto") ? 400 : parseFloat(height_style);
        }

        var availableWidth = chart.availableWidth = (width || width_style || 960),
            availableHeight = chart.availableHeight = (height || height_style || 400);

        this.canvasarea = {
            left: 0,
            top: 0,
            right: 0,
            bottom: 0,
            width: availableWidth,
            height: availableHeight
        };
    };

    /**
     *
     * @param globalProp
     * @returns {*}
     */

    Chart.prototype.getThemeOptions = function(globalProp) {

        var chart = this,
            overallTypes = $ZC.charttypenames.values(),
            allSeriesTypes = chart.dataProcessorHelpers.getAllChartTypes(chart.userdata, "string"),
            theme = null,
            userTheme,
            defaultTheme;

        if (chart.userdata.canvas) {
            theme = chart.userdata.canvas.theme;
        }

        defaultTheme = new $ZC.defaultTheme(globalProp);

        overallTypes.map(function(d, i) {
            if (allSeriesTypes.indexOf(d) !== -1) {
                $ZC.setDefaultOptions(d, defaultTheme.chart.plot.plotoptions, globalProp);
            }
        });


        if (defined($ZC[theme])) {
            userTheme = new $ZC[theme](globalProp);
            userTheme = mergeJSON(defaultTheme, userTheme, true, true);
        } else {
            userTheme = defaultTheme;
        }
        return userTheme;
    };

    /**
     *
     *
     * @returns {*}
     */

    Chart.prototype.getDataObject = function() {
        var chart = this,
            dataProcessor;

        if ($ZC.chartcategory.hierarchy.indexOf(chart.chartTypes[0].value) > -1) {
            dataProcessor = new $ZC.hierarchymodel.get(chart);
        } else {
            dataProcessor = new $ZC.DataProcessor(chart.systemConf, chart.dataset, chart);
        }
        chart.seriesdata = dataProcessor.isHierarchy ? simpleClone(chart.seriesdata) : chart.seriesdata;
        return dataProcessor;
    };

    /**
     *
     * @param type
     * @returns {number}
     */
    Chart.prototype.getAxesCount = function(type) {

        var chart = this,
            count = 1,
            metadata = chart.systemConf.metadata;

        if (type === "x") {
            var columnIndex = chart.dataset.getColumnIdx(type),
                originalDataType = metadata.columns[columnIndex].datatype,
                formatConfig = pick(metadata.columns[columnIndex][originalDataType], {}),
                formatFunction = formatConfig.subfunction;

            if (defined(chart.dataObject.hierarchical)) {
                count = chart.dataObject.hierarchical.finalDepth + 1;
            } else if (isArray(formatConfig.groupby) && formatConfig.groupby.length) {
                count = formatConfig.groupby.length + 1;
            } else if(formatConfig.groupby === "AUTO") {
                count = $ZC.HierarchicalTimeDataAdapter.getAutoGroupOrder(formatFunction).length + 1;
            }
            chart.axes[type].count = count;
        } else {

            var metadataY = metadata.axes.y;
            count = chart.axes[type].count = metadataY.length;
        }
        return count;
    };

    Chart.prototype.setChartDimension = function() {
        var chart = this,
            systemConfig = chart.systemConf;
        var borderOptions = systemConfig.canvas.border;
        var borderSize_canvas = borderOptions.show ? validateValWithinLimit(borderOptions, "size", 0, 10, chart.canvasarea.width, chart.canvasarea.height) : 0;
        chart.chartarea = {
            left: chart.canvasarea.left + borderSize_canvas,
            top: chart.canvasarea.top + borderSize_canvas,
            right: chart.canvasarea.right - borderSize_canvas,
            bottom: chart.canvasarea.bottom - borderSize_canvas,
            width: chart.canvasarea.width - (2 * borderSize_canvas),
            height: chart.canvasarea.height - (2 * borderSize_canvas)
        };
        chart.plotarea = Object.create(chart.chartarea)
    };

    /**
     *
     */

    Chart.prototype.applyChartProperties = function() {
        var chart = this,
            systemConfig = chart.systemConf,
            chartHeight = chart.chartHeight;

        var title = systemConfig.canvas.title,
            subtitle = systemConfig.canvas.subtitle,
            credits = systemConfig.credits || {};
        var showTitle = title.show && dimension_based_intelligence(chart, "title", chartHeight),
            showSubtitle = subtitle.show && dimension_based_intelligence(chart, "subtitle", chartHeight),
            showCredits = credits.enabled === true && dimension_based_intelligence(chart, "credits", chartHeight);
        if (showCredits) {
            var defaultCreditOptions = $ZC.setDefaultOptions("credits", chart.systemConf, chart.globalProp);
            // mergeJSON(chart.systemConf.credits, defaultCreditOptions, true, true);
            chart.drawTitle(chart.systemConf.credits, "credits");
        }
        if (showTitle) {
            (title.vAlign === "bottom" && subtitle.vAlign === "bottom" && subtitle.show) ? chart.drawTitle(systemConfig.canvas.subtitle, "subtitle") : chart.drawTitle(systemConfig.canvas.title,
                "title");
        }
        if (showSubtitle) {
            (subtitle.vAlign === "bottom" && title.vAlign === "bottom" && title.show) ? chart.drawTitle(systemConfig.canvas.title, "title") : chart.drawTitle(systemConfig.canvas.subtitle,
                "subtitle");
        }
        chart.plotarea = Object.create(chart.chartarea);
        chart.container.selectAll("g.plotareaBorder-property").selectAll("line", "path").style("border", ""); //reset plot area border color
        var plotOptions = systemConfig.chart;
        var mLeft = parseFloat(plotOptions.marginLeft),
            mRight = parseFloat(plotOptions.marginRight),
            mTop = parseFloat(plotOptions.marginTop),
            mBottom = parseFloat(plotOptions.marginBottom);
        // If margin values were greater than chart container width and height, reset to 0
        var mLeft_corr = (mLeft + mRight) > chart.plotarea.width ? 0 : mLeft,
            mRight_corr = (mLeft + mRight) > chart.plotarea.width ? 0 : mRight,
            mTop_corr = (mTop + mBottom) > chart.plotarea.height ? 0 : mTop,
            mBottom_corr = (mTop + mBottom) > chart.plotarea.height ? 0 : mBottom;
        chart.plotarea.left += mLeft_corr;
        chart.plotarea.right += mRight_corr;
        chart.plotarea.bottom += mBottom_corr;
        chart.plotarea.top += mTop_corr;
        chart.plotarea.width = chart.plotarea.width - mLeft_corr - mRight_corr;
        chart.plotarea.height = chart.plotarea.height - mBottom_corr - mTop_corr;
    };

    /**
     *
     * @param data
     * @param type
     */
    Chart.prototype.drawTitle = function(data, type) {
        var chart = this,
            svg = chart.svg,
            systemConfig = chart.systemConf,
            floating = defined(data.x) && defined(data.y),
            location = window.location;

        var margin = {
            top: validateNullVal(data.marginTop, 0),
            right: validateNullVal(data.marginRight, 0),
            bottom: validateNullVal(data.marginBottom, 0),
            left: validateNullVal(data.marginLeft, 0)
        };

        var titlearea = svg.selectAll("g.chart" + type);
        var titleAlign = (floating) ? {
            x: data.x + margin.left,
            y: data.y + margin.top,
            dy: "1",
            textAnchor: (data.hAlign === "left") ? "start" : (data.hAlign === "right") ? "end" : "middle"
        } : Chart.helpers.align(data.hAlign, data.vAlign, chart.chartarea, margin);

        // var titleBox = appendEle(titlearea, "rect", [data], ".", "class", (type+"Box"));
        var imageUrl = data.imageUrl;
        var isTextBox = !defined(imageUrl);
        var element = (isTextBox) ? (data.text === "" ? "none" : "text") : "image";

        if (element !== "none") {
            var title = appendEle(titlearea, element, [data], ".", "class", type);
            if (defined(data.href)) {
                var click = function() {
                    var isValidURL = validateURL(data.href);
                    if (isValidURL) {
                        var target = data.target;
                        if (target === "newtab" || target === "newwindow") {
                            window.open(data.href, 'popUpWindow', (target === "newwindow") ? "status=yes" : "");
                        } else {
                            location.href = data.href;
                        }
                    } else {
                        Console.log("invalid URL for credits")
                    }
                };
                var mouseHandlers = {
                        "click": click
                    },
                    touchHandlers = {
                        "tap": click
                    };
                attachevents(title, mouseHandlers, touchHandlers);
            }

            if (isTextBox) {
                var userdataCanvasFontColor = chart.userdata.canvas ? chart.userdata.canvas.fontColor : null;

                title
                    .attr({
                        "x": titleAlign.x,
                        "y": titleAlign.y,
                        "dy": titleAlign.dy + "em",
                        "text-anchor": titleAlign.textAnchor
                    })
                    .style({
                        "cursor": defined(data.href) ? "pointer" : "auto",
                        "display": "block"
                    })
                    .text(decodeHTML(data.text));

                if (data.tooltip === true) {
                    addSVGtitleEle(title)
                }

                applyFont(title, [{
                    fontColor: (type === "subtitle" && !defined(data.fontColor) && !defined(userdataCanvasFontColor)) ? "#CCCCCC" : data.fontColor || userdataCanvasFontColor
                }, data, systemConfig.canvas, $ZCG]);
            } else {
                title
                    .attr({
                        "x": titleAlign.x + (data.hAlign === "right" ? -data.imageWidth : 0),
                        "y": titleAlign.y + (data.vAlign === "bottom" ? -data.imageHeight : 0),
                        "width": data.imageWidth,
                        "height": data.imageHeight,
                        "xlink:href": data.imageUrl
                    })
                    .style({
                        "cursor": defined(data.href) ? "pointer" : "auto",
                        "display": "block"
                    });
            }

            //validate for long text value
            var titlenode = bbox(title.node());
            var textBounds = {
                    "left": floating ? data.x + margin.left : titlenode.x,
                    "top": floating ? data.y + margin.top : titlenode.y,
                    "width": titlenode.width,
                    "height": titlenode.height
                },
                edge = {
                    "left": margin.left,
                    "top": margin.top,
                    "right": chart.canvasarea.width - margin.right,
                    "bottom": chart.canvasarea.height
                };
            trimText(title.node(), decodeHTML(data.text), textBounds, edge, "bottom", "..", {
                "left": 5,
                "right": 5,
                "top": 5,
                "bottom": 5
            });

            if (!floating) {
                var nodeHeight = titlenode.height + margin.bottom + margin.top;
                if (data.vAlign === "top") {
                    chart.chartarea.top += nodeHeight;
                    chart.chartarea.height -= nodeHeight;
                }
                if (data.vAlign === "bottom") {
                    chart.chartarea.height -= nodeHeight;
                }
            }
        }

        titlearea.selectAll((element === "none") ? "text,image" : ((element === "image") ? "text" : "image"))
            .style("display", "none");
    };

    /**
     *
     */

    Chart.prototype.getChartSeriesColor = function() {
        var chart = this,
            systemConfig = chart.systemConf,
            colors = [],
            legend_data = systemConfig.legend;

        if (legend_data.colors) {
            if (!isArray(legend_data.colors)) {
                var legendColors = [];
                for (var key in legend_data.colors) {
                    if (legend_data.colors.hasOwnProperty(key)) {
                        legendColors.push(legend_data.colors[key]);
                    }
                }
                colors = legendColors;
            } else {
                colors = legend_data.colors;
            }

        } else if (legend_data.imagePallete) {
            chart.patternImageSizes = [];
            var imagePallete = $ZC.legutil.imagePallete.get(legend_data.imagePallete);
            chart.imagePallete = imagePallete;

            var defaultSize = (legend_data.imagePallete === "stripes") ? null : 50;
            var tempImage = appendEle($ZCG.patternSVG, "image", [1], ".", "class", "tempImage");

            imagePallete.map(function(d, i) {
                tempImage
                    .attr("xlink:href", d);

                var bb = bbox(tempImage.node());

                chart.patternImageSizes[i] = [defaultSize || bb.width || 3, defaultSize || bb.height || 3];
            });
            tempImage.remove();

        } else if (legend_data.colorPallete) {
            if (legend_data.colorPallete.type === "monochrome") {
                colors = $ZC.legutil.getMonochromeColorPallette(legend_data.colorPallete.options.monochrome);
            } else {
                colors = $ZC.legutil.colorpallete.get(legend_data.colorPallete.options.multicolor) || $ZC.legutil.colorpallete.get("flat-ui-colors");
            }
        }

        if (legend_data.patterns) {
            chart.patternPallete = legend_data.patterns;
        } else if (legend_data.patternPallete) {
            var patternPallete = $ZC.legutil.patternPallete.get(legend_data.patternPallete);
            if (!defined(patternPallete) && isArray(legend_data.patternPallete) && legend_data.patternPallete.length) {
                patternPallete = legend_data.patternPallete;
            }
            chart.patternPallete = patternPallete;
        }
        /*
         * setting seriesdata color
         */
        if (defined(colors)) {
            chart.seriesColor = colors.slice(0, colors.length);
            chart.seriesdata.map(function(d, i) {
                if (defined(d.color)) {
                    chart.seriesColor[i] = d.color;
                }
            })
        }
    };

    /**
     *
     */
    Chart.prototype.updatePlotareaProp = function() {
        var chart = this,
            systemConfig = chart.systemConf,
            container = chart.container,
            plotareaProp = container.selectAll("div.plotarea-property"),
            plotareaBorderProp = chart.staticPaneSVG.selectAll("g.plotareaBorder-property");

        var styles = {
            "position": "absolute",
            "left": chart.plotarea.left + "px",
            "top": chart.plotarea.top + "px",
            "width": chart.plotarea.width + "px",
            "height": chart.plotarea.height + "px",
            "pointer-events": "none",
            "box-sizing": "border-box"
        };
        // for adding prefix css
        styles[$Browser.prefix.css + "box-sizing"] = "border-box";

        container.selectAll("div.plotarea-property")
            .style(styles);
        plotareaBorderProp.attr({
            "transform": "translate(" + chart.plotarea.left + "," + chart.plotarea.top + ")"
        });
        chart.updateCommonBBSProp(plotareaProp, plotareaBorderProp, systemConfig.chart.plot, "plotarea");
    };

    /**
     *
     */

    Chart.prototype.drawPlotareaClippath = function() {
        var chart = this,
            systemConfig = chart.systemConf,

            //If the plot is zoomed, dont consider the marker size for clipPath
            zoomedX = isObjectPropDefined(systemConfig, "chart.zoom.x") || defined(chart.scrolledX),
            zoomedY = isObjectPropDefined(systemConfig, "chart.zoom.y") || defined(chart.scrolledY),
            axisrotated = chart.dataObject.getAxisRotated(),
            ms = getMarkerSize(chart),//ms -> Marker Size

            x = ((axisrotated ? zoomedY : zoomedX) ? 0 : -ms.left),
            y = ((axisrotated ? zoomedX : zoomedY) ? 0 : -ms.top),
            width = (chart.plotarea.width + ((axisrotated ? zoomedY : zoomedX) ? 0 : (ms.left + ms.right))),
            height = (chart.plotarea.height + ((axisrotated ? zoomedX : zoomedY) ? 0 : (ms.top + ms.bottom)));

        var properties = {
                name: "rect",
                id: chart.id,
                x: x,
                y: y,
                width: width,
                height: height
            },
            args = {
                chartObj: chart
            };
        var clipID = effects.get(svg_clips, "rect", properties, args);

        chart.wrap.selectAll("g.seriesholder")
            .attr('clip-path', "url(" + chart.defsLocation + "#" + clipID + ")");
        chart.staticPaneSVG.selectAll("g.labelholder,g#zcscatterhighlight,g#zcareahighlight,path#zcbarhighlight")
            .attr('clip-path', "url(" + chart.defsLocation + "#" + clipID + ")");

    };

    /**
     *
     * @param update
     */

    Chart.prototype.drawThreshold = function(update) {
        if ($ZC.axis.threshold) {
            $ZC.axis.threshold(this, update)
        }
    };

    /**
     *
     * @param update
     */
    Chart.prototype.handleEvent = function(update) {
        var chart = this,
            systemConfig = chart.systemConf;

        if (!update) {
            var eventgroup = chart.staticPaneSVG.selectAll("g.eventgroup")
                .style({
                    "pointer-events": "visible"
                });


            var areahighlight = eventgroup.append("g")
                .attr("id", "zcareahighlight");

            areahighlight.append("path").attr("id", "areapath");

            var barhighlight = eventgroup.append("g")
                .attr("id", "zcbarhighlight");

            barhighlight.append("path").attr("id", "barpath");

            if (systemConfig.tooltip.crosshair) {
                var crosshairEle = appendEle(eventgroup, "g", [1], ".", "class", "crosshair");
                crosshairEle.style("visibility", "hidden");
                appendEle(crosshairEle, "line", [1], ".", "class", "x-crosshair");
                appendEle(crosshairEle, "line", [1], ".", "class", "y-crosshair");
            }
            var scatterhighlight = eventgroup.append("g")
                .attr("id", "zcscatterhighlight");

            if (chart.dataObject.isHierarchy) {
                eventgroup.append("g")
                    .attr("id", "zchierarchyhighlight")
                    .style({
                        "opacity": 0,
                        "pointer-events": "none"
                    });
            }

            chart.eventHandler = $ZC.events.mouseevents(chart, chart.tooltip.element, systemConfig, chart.plotarea, chart.dataset, chart.axes.x.scale, chart.barscale1, chart.barscale2, chart.axes.y.groups,
                eventgroup, chart.seriesColor, chart.labelHighlighter);
            chart.eventHandler.initializeListener();
        } else {
            chart.eventHandler.updateParams(chart);
        }
        if (chart.xymultiRenderer || chart.dataObject.isPolarAxisCategory) {
            drawBorderBox(chart.eventHandler.trackerElement, chart.plotarea);
        }
        addDefaultScroll(chart);
    };

    /**
     *
     */

    Chart.prototype.redraw = function() {
        var chart = this,
            systemConfig = chart.systemConf;

        chart.svg
            .style({
                "width": chart.container.style("width"),
                "height": chart.container.style("height")
            });
        chart.staticPaneSVG
            .style({
                "width": chart.container.style("width"),
                "height": chart.container.style("height")
            });
        chart.resetdata(systemConfig);
        Chart.helpers.constructChartGroups(chart);
        chart.render(true);
    };

    /**
     *
     * @param newSystemConf
     */

    Chart.prototype.resetdata = function(newSystemConf) {
        var chart = this;
        chart.chartHeight = parseFloat(chart.container.style("height"));
        chart.chartWidth = parseFloat(chart.container.style("width"));
        chart.setCanvasDimension();
        chart.chartTypes = chart.dataProcessorHelpers.getAllChartTypes(newSystemConf, "object");
        chart.processUserData(newSystemConf, true);
        chart.setChartDimension();
        chart.globalProp = setGlobalPropByUserdata(newSystemConf, {
            "width": chart.canvasarea.width,
            "height": chart.canvasarea.height
        }, chart.chartFontProp);
        chart.events = {};
        if (chart.dataObject) {
            chart.dataObject.formattedData = {
                x: [],
                y: []
            };
        }
    };

    /**
     *
     * @param series
     * @param i
     * @param axisorder
     * @param updateAllAxes
     */

    Chart.prototype.addRemoveSeries = function(series, i, axisorder, updateAllAxes) {

        var chart = this,
            systemConfig = chart.systemConf;

        chart.dataObject.formattedData = {
            x: [],
            y: []
        };
        chart.dataObject.initialize("addorremoveseries");

        stateChangeListener(chart, systemConfig.chart.axes.events, null, "init");
        if (chart.isAxisCategory) {
            var brushExtent = chart.brush && chart.brush.extent;
            chart.updateZscale();
            if (updateAllAxes) {
                chart.axes.y.groups.forEach(function(d, i) {
                    chart.updateScale([i, 0], "y");
                })
            } else {
                chart.updateScale(axisorder, "y");
            }

            chart.updateScale(0, "x", brushExtent);
            if(chart.dataObject.hierarchical){
                chart.axes.x.rangeScale = chart.axes.x.scale.copy();
                if(chart.dataObject.hierarchical.groupByDiscreteTime) {
                    chart.axes.x.rangeScale.domain(d3.range(chart.axes.x.scale.domain().length));
                }
                chart.dataObject.hierarchical.setRanges(chart.axes.x.rangeScale); // initialize ranges
                chart.axes.x.levels.forEach(function(d,i){
                    chart.updateScale(axisorder, "x",null,i+1);
                })
            }
            chart.drawThreshold(true);
            chart.drawPlotareaClippath();
            chart.updatePlotareaProp();
        } else if (chart.dataObject.isPolarAxisCategory) {
            $ZC.axis.polar.render(chart, "x");
        }

        stateChangeListener(chart, systemConfig.chart.axes.events, null, "onreload");
        // update the events
        chart.events = {};
        chart.plot.renderer.draw(true);
        if (chart.isAxisCategory || chart.dataObject.isPolarAxisCategory) {
            drawBorderBox(chart.eventHandler.trackerElement, chart.plotarea);
            chart.eventHandler.updateParams(chart);
            addDefaultScroll(chart, "addorremoveseries");
        }
    };

    /**
     *
     * @param container
     */

    Chart.prototype.showLoading = function(container) {
        var chart = this,
            systemConfig = chart.systemConf;

        var data = systemConfig.loader;
        if (data.enabled === true && data.type !== "none") {
            var loader = appendEle(this.container.selectAll("div.d3container"), "div", [1], ".", "class", "loading");
            loader
                .style("position", "absolute")
                .html(Chart.helpers.getLoader(chart, data));
            applyFont(loader, [data, systemConfig.chart, systemConfig.canvas, $ZCG], true);
            var node = loader.node();
            loader.style({
                "left": ((chart.chartarea.width / 2) - (node.offsetWidth / 2)) + "px",
                "top": ((chart.chartarea.height / 2) - (node.offsetHeight / 2)) + "px"
            })
        }
    };

    Chart.prototype.hideLoading = function() {
        this.container.selectAll("div.loading").remove();
    };


    /**
     * Add data point to single series or multiple series
     * @param seriesidx
     * @param point
     * @param shift
     */
    Chart.prototype.pushDataPoint = function(seriesidx, point, shift) {
        var chart = this,
            isComboChart = chart.dataObject.isCombinationalChart,
            orderedSeriesdata = (isComboChart) ? renderer_helpers.arrangeSeriesRenderingOrder(chart.seriesdata) : chart.seriesdata;

        // If series indices were not of ARRAY type, then convert it to array and process
        if (!(Array.isArray(seriesidx))) {
            var seriesidx_arr = [],
                point_arr = [];
            seriesidx_arr.push(seriesidx);
            point_arr.push(point);
            seriesidx = seriesidx_arr;
            point = point_arr
        }

        seriesidx.forEach(function(d, i) {
            var currOrdereddata = orderedSeriesdata[d],
                curSeriesidx = (isComboChart) ? currOrdereddata.seriesidx : d,
                series = chart.seriesdata[curSeriesidx],
                renderer = chart.renderer[curSeriesidx],
                shiftPoint, shiftInfo

            series.data[0].push(point[i]);

            if (shift) {
                shiftPoint = series.data[0][0];
                var shiftedData = series.data[0].shift();
                if (defined(renderer.shiftInfo())) {
                    shiftInfo = renderer.shiftInfo();
                    shiftInfo.count++;
                } else {
                    shiftInfo = {};
                    shiftInfo.count = 0;
                }
                shiftInfo.point = shiftPoint;
                shiftInfo.shiftedData = shiftedData;
                renderer.shiftInfo(shiftInfo)
            } else {
                renderer.shiftInfo(null)
            }

            if (chart.ismultiy) {
                chart.updateScale(series, "y");
            }
        });

        chart.dataObject.initialize();
        chart.updateScale(null, "x");
        if (!chart.ismultiy) {
            chart.updateScale(chart.seriesdata[0], "y");
        }
        chart.plot.renderer.draw(true);
        chart.drawThreshold(true);
        chart.eventHandler.updateParams(chart);
    };

    /**
     *
     * @param seriesidx
     */

    Chart.prototype.popDataPoint = function(seriesidx) {
        var chart = this,
            series = chart.seriesdata[seriesidx];

        series.data.forEach(function(d) {
            d.pop()
        });
        chart.dataObject.initialize();
        chart.updateScale(series, "x");
        chart.updateScale(series, "y");
        chart.plot.renderer.draw(true);
        chart.drawThreshold(true);
        chart.eventHandler.updateParams(chart);
    };

    /**************** Zooom ********************/

    /**
     *
     * @param type
     */
    Chart.prototype.zoomON = function(type) {
        var chart = this;
        chart.changeZoomType(type);
        var zoomObj = chart.zoom;
        if (!zoomObj) {
            chart.eventHandler.initializeZoomListener();
        }
    };

    /**
     *
     */
    Chart.prototype.zoomOFF = function() {
        var chart = this;
        var zoomObj = chart.zoom;
        if (zoomObj) {
            zoomObj.reset();
        }
        chart.changeZoomType("none");
    };

    /**
     *
     * @param seriesidx
     * @param range
     */
    Chart.prototype.zoomX = function(seriesidx, range) {
        var chart = this;
        chart.changeZoomType("x");
        if (!chart.zoom) {
            chart.eventHandler.initializeZoomListener();
        }
        var zoomObj = chart.zoom;
        zoomObj.zoomed(null, seriesidx, range);
    };

    /**
     *
     * @param seriesidx
     * @param range
     */
    Chart.prototype.zoomY = function(seriesidx, range) {
        var chart = this;
        chart.changeZoomType("y");
        if (!chart.zoom) {
            chart.eventHandler.initializeZoomListener();
        }
        var zoomObj = chart.zoom;
        zoomObj.zoomed(null, seriesidx, null, range);
    };

    /**
     *
     * @param seriesidx
     * @param xrange
     * @param yrange
     */
    Chart.prototype.zoomXY = function(seriesidx, xrange, yrange) {
        var chart = this;
        chart.changeZoomType("xy");
        if (!chart.zoom) {
            chart.eventHandler.initializeZoomListener();
        }
        var zoomObj = chart.zoom;
        zoomObj.zoomed(null, seriesidx, xrange, yrange);
    };

    /**
     *
     * @param type
     */
    Chart.prototype.changeZoomType = function(type) {
        var chart = this,
            systemConfig = chart.systemConf;

        if (!systemConfig.chart.zoom) {
            $ZC.setDefaultOptions("zoom", systemConfig.chart, chart.globalProp);
        }
        systemConfig.chart.zoom.type = type;
    };

    /**
     *
     * @param data
     * @param ele
     */
    Chart.prototype.handleNoData = function(data, ele) {

        var chart = this,
            systemConfig = chart.systemConf,
            nodatawrap = pick(ele, chart.wrap.selectAll("g.nodatahandler"));

        if (defined(data.htmlEl)) {
            var customHandler = appendEle(chart.container.selectAll("div.canvas-component"), "div", [1], ".", "class", "nodatahandler");
            customHandler
                .style("position", "absolute")
                .html(data.htmlEl);

            var node = customHandler.node();
            customHandler.style({
                "left": validateNullVal(data.x, (chart.availableWidth - node.offsetWidth) / 2) + "px",
                "top": validateNullVal(data.y, (chart.availableHeight - node.offsetHeight) / 2) + "px"
            });
        } else if (defined(data.imageUrl)) {
            var image = appendEle(nodatawrap, "image", [1]);
            image
                .attr({
                    "x": data.x || (chart.chartarea.width / 2) - 50,
                    "y": data.y || (chart.chartarea.height / 2) - 50,
                    "width": 100,
                    "height": 100,
                    "xlink:href": data.imageUrl
                });
        } else if (defined(data.text)) {

            var text = appendEle(nodatawrap, "text", [data], ".", "class", "nodata");
            text
                .attr({
                    "x": data.x || chart.chartarea.width / 2,
                    "y": data.y || chart.chartarea.height / 2,
                    "text-anchor": "middle"
                })
                .text(decodeHTML(data.text || "No Data"));
            applyFont(text, [data, systemConfig.canvas, $ZCG]);
        }
    };

    /**
     *
     */
    Chart.prototype.destroy = function() {
        var chart = this;

        if (chart.eventHandler) {
            clearTimeout(chart.eventHandler.mouseouttimer);
            clearTimeout(chart.eventHandler.mousemovetimer);
            clearTimeout(chart.eventHandler.tooltipElement.timeout);
            chart.eventHandler.tooltipElement.destroy();
        }
        chart.container.datum(null);
        chart.d3container.remove();
        // var prop;
        // for(prop in chart) {
        //     delete chart[prop];
        // }
        chart.destroying = true;
    };

    /**
     *
     * @param data
     * @returns {*}
     */



    /************************** Helpers ****************/

    Chart.helpers = {

        initializeChartHolders: function(chart) {

            var container = chart.container,
                width_style, height_style;

            var d3container = container.selectAll("div.d3container");
            if (!d3container.empty()) {
                // If user going to create a new chart in the same container again, destroy the previous chart object
                var prevChartObj = d3container.datum();
                if (prevChartObj && prevChartObj.destroy) {
                    prevChartObj.destroy();
                }

                d3container.remove();
            }

            width_style = pick(parseFloat(container.style("width")), 960);
            height_style = pick(parseFloat(container.style("height")), 400);

            d3container = chart.d3container = appendEle(container, "div", [chart], ".", "class", "d3container");
            var styles = {
                "width": width_style + "px",
                "height": height_style + "px",
                "position": "relative",
                "left": "0px",
                "top": "0px",
                "box-sizing": "border-box",
                "-moz-user-select": "none",
                "-webkit-user-select": "none",
                "-ms-user-select": "none",
                "user-select": "none"
            };
            // for adding prefix css
            styles[$Browser.prefix.css + "box-sizing"] = "border-box";
            d3container.style(styles);

            //CANVASPROP - canvas area properties-border,background,shadow
            appendEle(d3container, "div", [1], ".", "class", "canvas-property");
            //PLOTAREA PROP - plotarea background prop
            appendEle(d3container, "div", [1], ".", "class", "plotarea-property");

            // Overview container for scroll
            appendEle(d3container, "div", [1], ".", "class", "zcoverview");

            //main canvas area - svg
            var canvas = appendEle(d3container, "div", [1], ".", "class", "canvas-component");
            canvas.style("-webkit-tap-highlight-color", "transparent");

        var tooltipElement = appendEle(BASE_HELPER_CONTAINER, "div", [1], "#", "id", "zctooltip" + CURRENT_CHART_COUNT);
        tooltipElement[0][0].isTooltipContainer = true;
        
        chart.tooltip = {
            "element": tooltipElement
        };
        chart.tooltip.element
            .style({
                "position": "fixed",
                "display": "inline-block",
                "visibility": "hidden",
                "white-space": "nowrap",
                "background-color": "white",
                "border": "1px solid",
                "-webkit-tap-highlight-color": "transparent"
            });

        chart.svg = appendEle(canvas, "svg", [1], ".", "class", "svgarea");
        chart.svg
            .style({
                "width": width_style + "px",
                "height": height_style + "px",
                "position": "absolute",
                "top": 0,
                "left": 0
            });
        
        

            var staticPane = d3_select(chart.svg.node().parentNode).datum(chart)
                .style({
                    "width": chart.container.style("width"),
                    "height": chart.container.style("height")
                });
            chart.staticPaneSVG = staticPane.append("svg")
                .attr("class", "static")
                .style({
                    "width": chart.container.style("width"),
                    "height": chart.container.style("height"),
                    "position": "absolute",
                    "top": 0,
                    "left": 0,
                    "pointer-events": "none"
                });

            chart.legendarea.element = canvas.append("div").attr("class", "legendarea");
            chart.notes = canvas.append("div").attr("class", "zcnotes");
        },

        removeChartHolders: function(chart) {
            var container = chart.container;
            container.select("div.d3container") //No I18N
                .remove();

            container.select("div.zc-staticPane") //No I18N
                .remove();
            container.select("div.zc-chartPane") //No I18N
                .remove();
        },

        /**
         *
         * @param hAlign
         * @param vAlign
         * @param box
         * @param margin
         * @returns {{}}
         */

        align: function(hAlign, vAlign, box, margin) {
            var attr = {};
            //hAlign
            if (hAlign === "center") {
                attr.x = box.width / 2 + (margin.left);
                attr.textAnchor = "middle";
            } else if (hAlign === "left") {
                attr.x = (box.left) + (margin.left);
                attr.textAnchor = "start";
            } else if (hAlign === "right") {
                attr.x = box.width + box.left - (margin.right);
                attr.textAnchor = "end";
            }
            //vAlign
            if (vAlign === "top") {
                attr.y = (box.top) + (margin.top);
                attr.dy = 1;
            } else if (vAlign === "bottom") {
                attr.y = box.height + box.top - (margin.bottom);
                attr.dy = -0.21;
            }
            return attr;
        },

        getLoader: function(chart, data) {
            var htmlEl = data.htmlEl,
                imageUrl = data.imageUrl,
                type = data.type,
                text = data.text,
                colors = data.colors;
            if (htmlEl) {
                return htmlEl;
            } else if (imageUrl) {
                return "<img src=" + imageUrl + " height=" + data.imageHeight + " width=" + data.imageWidth + ">";
            } else {
                var html = "",
                    ele = null;
                if (type) {
                    var loaderDiv = chart.container.selectAll("div.loading");
                    colors = (colors && colors.length) ? colors : null;
                    ele = ZCloader(loaderDiv, {
                        "type": type,
                        "colors": colors
                    });

                    var XMLS = new XMLSerializer();
                    html = XMLS.serializeToString(ele); // Converting DOM node into a string
                }
                if (text) {
                    html += "<p style=position:relative;display:inline-block>" + text + "</p>";
                }
                return html;
            }
        },

        constructChartGroups: function(chart) {

            var svg = chart.svg,
                systemConfig = chart.systemConf,
                chartHeight = chart.chartHeight,
                chartWidth = chart.chartWidth;
            //Title area
            var istitle = systemConfig.canvas.title.show === true && dimension_based_intelligence(chart, "title", chartHeight);
            appendEle(svg, "g", groupdata(istitle), ".", "class", "charttitle");

            //Subtitle area
            var issubtitle = systemConfig.canvas.subtitle.show === true && dimension_based_intelligence(chart, "subtitle", chartHeight);
            appendEle(svg, "g", groupdata(issubtitle), ".", "class", "chartsubtitle");
            //Credit Area
            var iscredits = systemConfig.credits && systemConfig.credits.enabled === true && dimension_based_intelligence(chart, "credits", chartHeight);
            appendEle(svg, "g", groupdata(iscredits), ".", "class", "chartcredits");

            var wrap = chart.wrap = appendEle(svg, "g", [systemConfig], ".", "class", "xychart");

            if (chart.nochartdata) {
                appendEle(wrap, "g", [1], ".", "class", "nodatahandler");
            } else {
                appendEle(wrap, "g", [0], ".", "class", "nodatahandler");
                var chartgroup = appendEle(wrap, "g", [chart.seriesdata], ".", "class", "chartgroup");

                var isxaxis = systemConfig.chart.axes.xaxis.show === true;
                //xaxisgroup
                appendEle(chartgroup, "g", groupdata(isxaxis), ".", "class", "x-axis");
                //yaxisgroup
                appendEle(chartgroup, "g", [chart.seriesdata], ".", "class", "y-axis");

                // Underlay threshold - So that threshold range will render first and then plot will be rendered
                appendEle(chartgroup, "g", [chart.seriesdata], ".", "class", "threshold");

                //series group
                var seriesgroup = appendEle(chartgroup, "g", [chart.seriesdata], ".", "class", "seriesholder");
                chart.filterEffectsID = getChartEffects(systemConfig);
                seriesgroup.attr("filter", (chart.filterEffectsID ? "url(" + chart.defsLocation + "#" + chart.filterEffectsID + ")" : null));

                var onlyBarChart = chart.dataObject.noofBarEnabledSeries && !chart.dataObject.isCombinationalChart; // For bar charts, the element should be before the highlighting element
                if (!onlyBarChart) {
                    //labelgroup
                    var labelgroup = appendEle(chart.staticPaneSVG, "g", [chart.seriesdata], ".", "class", "labelholder");
                }

                //thgroup
                appendEle(chart.staticPaneSVG, "g", [chart.seriesdata], ".", "class", "threshold");


                //plotarea border properties,as border has to be drawn over axis tickmarks // also below the event rect
                var plotBorder = appendEle(chart.staticPaneSVG, "g", [1], ".", "class", "plotareaBorder-property");
                var isplotBorder = systemConfig.chart.plot.border.show === true;
                appendEle(plotBorder, "path", groupdata(isplotBorder), ".", "class", "plotBorderPath");

                //eventgroup
                appendEle(chart.staticPaneSVG, "g", [1], ".", "class", "eventgroup");

                if (onlyBarChart) {
                    //labelgroup
                    var labelgroup = appendEle(chart.staticPaneSVG, "g", [chart.seriesdata], ".", "class", "labelholder");
                    labelgroup
                        .style("pointer-events", "none");
                }

                // Dummy rect added for filter - ISSUE : http://www.inkscapeforum.com/viewtopic.php?t=18611
                appendEle(seriesgroup, "rect", [1], ".", "class", "dummy-rect-filter");

                var isContinousLegend = legend_helpers.isContinousLegend(chart);

                var legendarea = appendEle(svg, "g", isContinousLegend ? [1] : [], ".", "class", "legendarea");
                if (isContinousLegend) {
                    chart.legendarea.element = legendarea;
                }


            }

            function groupdata(show) {
                return show ? [1] : [];
            }
        }
    };

    /**
     *
     * @type {Chart}
     */
    exports.Chart = Chart;

})($ZC);