//$Id$
/*
 * Main method to generate a chart or charts
 * containers,datas param can be an array if there have to be multiple chart generation.
 */
$ZC.charts = function(containers, datas, mapObj) {
    var isMulti = (containers.constructor === Array && datas.constructor === Array);
    if (isMulti) {
        return getMultiCharts(containers, datas, mapObj);
    } else {
        var chartObj = getChartObject(containers, datas);
        if (chartObj) {
            chartObj.draw();
        }
        return chartObj;
    }
};

/*
 * This is the actual chart object init method
 * here the data is binded with respective selected container
 * Note: Only the chart object is initialized here, but the chart generation or rendering will not happen.
 * */
function getChartObject(container, data) {
    if (!document.body.contains(container)) {
        Console.warn("No container");
        return null;
    }
    var dataObj = d3_select(container)
        .datum(data);
    var chart = null;
    
    /*
     * Each chart instance is instantiated with chartRendered flag false
     * chart.initialize is defined in chart-constructor.js
     * */
    dataObj.each(function(data) {
        var chartHolder = d3_select(this);
        chart = new $ZC.charts.get(chartHolder, data); // get chartobj
        chart.showLoading(chartHolder);
    });
    return chart;
}
/*
 * when it is multi charts instantiation, the chart rendering is queued here.
 */
function getMultiCharts(containers, datas) {
    
    if (!(containers.constructor === Array && datas.constructor === Array)) {
        return;
    }
    var chartObjs = [];
    for (var i in containers) {
        if (containers.hasOwnProperty(i)) {
            var container = containers[i];
            var data = datas[i];
            var chartObj = getChartObject(container, data);
            chartObjs.push(chartObj);
        }
    }
    var executedChartObjs = [];
    //unexecutedChartObjs are reversed just for the convenience of using Array.pop() method to remove first rendered chart object.
    var clonedObjs = chartObjs.slice();
    var unexecutedChartObjs = clonedObjs.reverse();
    drawEfficiently(unexecutedChartObjs, executedChartObjs, chartObjs);
    return chartObjs;
}


function drawEfficiently(unexecutedChartObjs, executedChartObjs, chartObjs) {
    
    var len = unexecutedChartObjs.length,
        chartObj = unexecutedChartObjs[len - 1];
    
    // all charts are rendered
    if (!len) {
        chartObjs.chartRendered = true;
    }
    
    if (len > 0 && (chartObj.drawStarted != true) && document.body.contains(chartObj.container.node())) {
        chartObj.draw();
        chartObj.drawStarted = true;
        checkStatusNCallNext(unexecutedChartObjs, executedChartObjs, chartObjs);
    }
    
}

/*
 * Executed/rendered chart objects are popped and pushed to the executed object list.
 * Waits till next chart object is rendered.
 * */
function checkStatusNCallNext(unexecutedChartObjs, executedChartObjs, chartObjs, timer) {
    var len = unexecutedChartObjs.length,
        chartObj = unexecutedChartObjs[len - 1];
    
    if (chartObj.chartRendered || chartObj.breakOnErr) {
        executedChartObjs.push(unexecutedChartObjs.pop());
        drawEfficiently(unexecutedChartObjs, executedChartObjs, chartObjs);
    } else {
        if (timer != null) {
            clearTimeout(timer);
        }
        
        var timer = setTimeout(function() {
            checkStatusNCallNext(unexecutedChartObjs, executedChartObjs, chartObjs, timer);
        }, 10)
    }
}