//$Id$
/*
 * this file contains methods to extract the required forms of data from the user input
 */
$ZC.dataset.get = function(data, chartObj) {
    var metadata = data.metadata,
        columns = metadata.columns,
        axes = metadata.axes;
    var seriesdata = data.seriesdata.chartdata;
    var yaxisorderBySeries,

        // data processor helpers

        dataProcessorHelpers = $ZC.DataProcessor.helpers,
        getParsedYAxisOrder = dataProcessorHelpers.getParsedYAxisOrder;

    
    axes = sortAxesObject();
    var dataObject,
        columnInfo_clr = columns[axes.clr] || {};
    var colTypeOrder = columns.map(function(d, i) {
        return getColTypeOrder(i);
    });
    
    var dataset = {
        xidx: columns[axes.x].dataindex,
        yidx: function(order) {
            var columnidx = (order instanceof Array) ? axes.y[order[0]][order[1] || 0] : axes.y[order][0];
            return columns[columnidx].dataindex;
        },
        zidx: (columns[axes.z] || {}).dataindex,
        labelidx: (axes.label != null) ? ((columns[axes.label] != null) ? columns[axes.label].dataindex : axes.label) : null,
        tooltipidx: function(index) {
            return (axes.tooltip != null) ? columns[index].dataindex : null
        },
        clridx: columnInfo_clr.levelindex ? null : columnInfo_clr.dataindex
    };
    dataset.setDataObj = function(obj) {
        dataObject = obj;
    };
    dataset.getX = function(d) {
        if(defined(dataObject) && dataObject.hierarchical && !dataObject.hierarchical.groupByTime) {
            return d.index
        } else {
            return this.getActualX(d);
        }
        // return this.getActualX(d)
    },
        dataset.getY = function(d, yaxiscolumnorder, range) {
            /*
             * Meta data for y should be in array
             * Axis order may be array of objects or an object
             */
            try {
                var curYidx = this.yidx(yaxiscolumnorder);
                var yVal = (range == null) ? d[curYidx] : d[curYidx][range]
                return yVal;
            } catch (err) {
                
            }
            
        },
        dataset.getZ = function(d) {
            return d[this.zidx];
        },
        dataset.getClr = function(d) {
            return d[this.clridx];
        },
        dataset.getDataLabelValue = function(d) {
            return d[this.labelidx];
        },
        dataset.getGroupByValues = function(d, indices) {
            if (indices == null || indices.length == 0) {
                return null;
            }
            return indices.reduce(function(prev, cur, i) {
                return (defined(d[cur])) ? prev + d[cur] : prev
            }, "");
        }
    
    dataset.getActualX = function(d) {
        return d[this.xidx];
    };

        dataset.getDataType = function(type, index, seriestypes) {
            seriestypes = seriestypes || dataset.seriesTypes;

            if(type === "x" && index > 0) {
                return "ordinalNonLinear";
            }

            var datatype = null;
            if (metadata.axes[type] != null) {
                var columnidx = this.getColumnIdx(type, index);
                var columnInfo = columns[columnidx];
                datatype = columnInfo.datatype;
                //ordinal cases:
                var ordinalType = (type == "x") && (dataObject ? (dataObject.isPolarAxisCategory || dataObject.noofBarEnabledSeries) : ((seriestypes.indexOf($ZC.charttype.bar) != -1) || (seriestypes.indexOf($ZC.charttype.stkdbar) != -1) || (seriestypes.indexOf($ZC.charttype.bullet) != -1) || (seriestypes.indexOf($ZC.charttype.gantt) != -1) || (seriestypes.indexOf($ZC.charttype.web) != -1)));
                var isHeatmap = (seriestypes.indexOf($ZC.charttype.heatmap) != -1) && (type === "x" || type === "y");
                datatype = (ordinalType || columnInfo.useDiscrete || isHeatmap) ? ORDINAL_DATATYPE : datatype;
            }
            return datatype;
        }

    dataset.getColumnIdx = function(type, order) {
        var columnidx;
        var axes = metadata.axes[type];
        if (type == "y") {
            columnidx = (order instanceof Array && (order.length == 2)) ? axes[order[0]][order[1]] : axes[order][0];
        } else if (type == "tooltip") {
            columnidx = axes[order];
        } else if(type == "x" && order >0){
            metadata.columns.filter(function(d,i){
                if(d.levelindex == order){
                    columnidx = i;
                }
            })
            columnidx = columnidx || axes[0];
        } else {
            columnidx = (axes == null) ? null : axes[0];
        }
        return columnidx;
    };
    
    dataset.getColumn = function(type, yaxiscolumnorder) {
        var columnidx = this.getColumnIdx(type, yaxiscolumnorder);
        return columns[columnidx];
    };
    
    dataset.getColumnName = function(colidx, toDecode) {
        return ((toDecode != false) ? decodeHTML : encodeHTML)(columns[colidx].columnname);
    };
    
    dataset.getSeriesIdx = function(colidx) {
        
        var seriesidx = -1
        var typeandOrder = colTypeOrder[colidx],
            type = (typeandOrder != null) ? typeandOrder.type : null;
        
        if (type == "y" && (axes.y.length > 1 || axes.y[0].length > 1)) { //check for multi y case(only for multiy case colidx and series order are mapped)
            var order = typeandOrder.order;
            yaxisorderBySeries = yaxisorderBySeries || seriesdata.map(function(d, i) {
                return isUndefined(d.yaxiscolumnorder) ? "0" : d.yaxiscolumnorder.toString();
            });

            seriesidx = yaxisorderBySeries.indexOf(order.toString()); //yaxisorder should be 2d array
        }
        return seriesidx;
    };

    dataset.getCustomizedVal = function(chartObj, colidx, seriespos, subseriespos, itempos, d, type, color, toDecode) {
        if (isUndefined(toDecode)) {
            toDecode = !/(tooltip|legend)/.test(type);
        }

        return colidx.toString().replace(CUSTOM_VALUE_PATTERN, function() {
            var matchedString = arguments[0],
                valIndex = arguments[1], //INDEX value of {{val(INDEX)}}
                colIndex = arguments[2], //INDEX value of {{col(INDEX)}}
                perIndex = arguments[3], //INDEX value of {{per(INDEX)}}
                splitPerIndex = arguments[4]; //INDEX value of {{splitper(INDEX)}}
            var idx = valIndex || colIndex || perIndex || splitPerIndex;
            if (matchedString.indexOf("{{clr}}") != -1) { // theme color based on series color
                return color
            }
            if (columns[idx] != null) {
                var dataPoint = (type === "stackLabels") ? d : dataset.getDataPoint(chartObj, idx, seriespos, subseriespos, itempos, d);
                if (dataPoint != null) {
                    var operation = matchedString.match(/[a-z]+/gi)[0];
                    switch (operation) {
                        case "col":
                            return dataset.getColumnName(colIndex, toDecode); //return columnname
                        case "val":
                            return dataset.getDataPointValue(chartObj, dataPoint, valIndex, seriespos, false, type, subseriespos, toDecode); //return value
                        case "per":
                        case "splitper":
                            return dataset.getDataPointValue(chartObj, dataPoint, perIndex||splitPerIndex, seriespos, operation, type, subseriespos, toDecode); //return the percentage of value (percentage of Y value is only supported now)
                    }
                } else {
                    return "";
                }
            } else {
                return matchedString;
            }
        });
    };
    
    dataset.getDataPoint = function(chartObj, colidx, seriespos, subseriespos, itempos, d) {
        //mappedSeriesidx refers seriespos for multiY charts
        //seriesdataAfterUpdate refers seriesdata without null data,till now handled only in bar chart
        colidx = +colidx;
        var dataPoint = null,
            mappedSeriesidx = (colidx == null) ? seriespos : this.getSeriesIdx(colidx),
            colInfo = columns[colidx] || {};
        var curSeriesPos = (mappedSeriesidx != -1 && mappedSeriesidx == seriespos) ?
            mappedSeriesidx :
            ((mappedSeriesidx == -1) ? seriespos : null),
            hierarchyCondition = chartObj.dataObject.isHierarchy ? ((axes.clr && axes.clr[0] === colidx && d.depth === 1) || (chartObj.systemConf.metadata.axes.y[0].indexOf(colidx) !== -1) || (d.children && (colInfo.levelindex === d.depth-1 || (isArray(colInfo.levelindex) && colInfo.levelindex.indexOf(d.depth-1) !== -1))) || (!d.children && (colInfo.levelindex == null || colInfo.dataindex != null))) : true;

        if (curSeriesPos != null && hierarchyCondition) {
            var renderer = chartObj.renderer[(chartObj.isAxisCategory || chartObj.dataObject.isPolarAxisCategory) ? curSeriesPos : 0];
            var seriesdataAfterUpdate = (renderer != null && renderer.seriesdataAfterUpdate != null) ?
                renderer.seriesdataAfterUpdate[subseriespos] :
                seriesdata[curSeriesPos].data[subseriespos];
            dataPoint = d || seriesdataAfterUpdate[itempos];
        }
        return dataPoint;
    };
    
    dataset.getDataPointValue = function(chartObj, dataPoint, colidx, seriespos, percent, type, subseriespos, toDecode) {
        if (colidx != null) {
            colidx = +colidx;
        }

        if (isUndefined(toDecode)) {
            toDecode = !/(tooltip|legend)/.test(type);
        }

        var colInfo = columns[colidx] || {},
            datatype = colInfo.datatype,
            levelindex = colInfo.levelindex,
            dataindex = colInfo.dataindex;
        var format = colInfo[datatype];
        if (dataPoint != null) {
           if ((axes.clr != null || type === "legend") && colidx == axes.clr && !(chartObj.dataObject.isHierarchy && type !== "legend" && colInfo.dataindex != null)) { //Eventhough clrIdx is not specified in metadata, the seriesname will be returned correctly
                if (defined(this.clridx) && colorscale_helpers.isQuantileColorScale(chartObj)) {
                    if (type === "legend") {
                        var arraySeperator = "-";
                        return getFormattedValue(format, dataPoint, datatype, chartObj, type, toDecode, arraySeperator);
                    }
                    return getFormattedValue(format, this.getClr(dataPoint), datatype, chartObj, type, toDecode);
                } else {
                    return getFormattedValue(format, seriesdata[seriespos].seriesname, datatype, chartObj, type, toDecode);
                }
            } else if (chartObj.dataObject.isHierarchy && (levelindex === dataPoint.depth-1 || (isArray(levelindex) && levelindex.indexOf(dataPoint.depth-1) !== -1))) {
                var val = !dataPoint.children && (colInfo.dataindex != null) ? dataPoint[dataindex] : dataPoint.seriesname;
                return getFormattedValue(format, val, datatype, chartObj, type, toDecode);
            } else if(levelindex != null && chartObj.dataObject.hierarchical){
                var parentData = chartObj.dataObject.hierarchical.getParentByIndex(dataPoint.index, levelindex-1);
                if(parentData && parentData.value){
                    return getFormattedValue(format, parentData.value, datatype, chartObj, type, toDecode);
                }else{
                    return ""
                }

            }
            else {
                if (percent) {
                    var total = (datatype == NUMERIC_DATATYPE) ? getTotal(chartObj, seriespos, subseriespos, colidx, dataPoint, percent) : null;
                    if (total != null) {
                        var val = ((type == "stackLabels") ? dataPoint : ((chartObj.dataObject.isHierarchy && defined(dataPoint.value)) ? dataPoint.value : chartObj.dataObject.isStackedPercentChart ? dataPoint.y : dataPoint[dataindex])) / total * 100;
                        val = val || 0; //if total=0 and y=0, the 'val' holds NaN
                        return getFormattedValueByPercent(format, val, datatype, chartObj, type);
                    } else {
                        return "";
                    }
                }
                var yidx = this.getColumnIdx("y", getParsedYAxisOrder(chartObj.seriesdata[seriespos].yaxiscolumnorder));
                var arraySeperator = (colidx == yidx) ? " - " : undefined;
                return getFormattedValue(format, ((chartObj.dataObject.isHierarchy && colidx == yidx && defined(dataPoint.value)) ? dataPoint.value : (type == "stackLabels") ? dataPoint : dataPoint[dataindex]), datatype, chartObj, type, toDecode, arraySeperator);
            }
        }
        return null;
    };
    
    dataset.getMetaInfoObj = function(chartObj, data, seriespos) {
        var axes = chartObj.systemConf.metadata.axes,
            columns = chartObj.systemConf.metadata.columns,
            seriesdata = chartObj.systemConf.seriesdata.chartdata,
            seriespos = (chartObj.isAxisCategory || chartObj.isNonAxisMultiSeriesCategory) ? seriespos : 0,
            dataWithMetadata = [];
        var yindex = seriesdata[seriespos].yaxiscolumnorder || [0, 0]; //yaxiscolumnorder is not mandatory
        var yColIdx = (yindex instanceof Array) ? axes.y[yindex[0]][yindex[1] || 0] : axes.y[yindex][0];
        if (data instanceof Array) {
            data.forEach(function(d, i) {
                var colidx = columns.indexOf(columns.filter(function(c, j) {
                    return (c.dataindex == i) && ((dataset.yidx(yindex) != i) || (dataset.yidx(yindex) == i && yColIdx == j));
                })[0]);
                dataWithMetadata.push(getColObj(colidx, d));
            });
        }

        if (axes.clr != null) {
            var clrColIdx = (axes.clr instanceof Array) ? axes.clr[0] : axes.clr;
            dataWithMetadata.push(getColObj(clrColIdx, seriesdata[seriespos].seriesname));
        }
        return dataWithMetadata;
        
        function getColObj(colidx, value) {
            var obj = simpleClone(columns[colidx] || {});
            obj.value = value;
            obj.columnindex = colidx;
            return obj;
        }
    };

    dataset.processMetaData = function() {

        var legendIndex = getPropVal(metadata, "axes.clr");

        if(defined(legendIndex)) {
            legendIndex = legendIndex[0];
        }

        function process(metadata, i) {
            var type;

            if(i === legendIndex) {
                return
            }

            if(!defined(metadata.datatype)) {
                var data = dataset.getDataByColumnIndex(metadata.dataindex);
                type = dataset.findDataType(data);
                metadata.datatype = type;
            }
            if (defined(type) && !defined(metadata[type])) {
                metadata[type] = {};
            }
        }
        columns.forEach(process);
    };

    dataset.getDataByColumnIndex = function(index) {
        var data = null, found = false;
        chartObj.seriesdata.some(function(series) {
            return series.data.some(function(subSeries) {
                return subSeries.some(function(point) {
                    if(defined(point)) {
                        found = defined(point[index]);
                        if(found) {
                            data = point[index];
                        }
                        return found;
                    }
                })
            })
        });
        return data;
    };

    dataset.findDataType = function(data) {
        if (typeof data === "number") {
            if (data > 365 * 24 * 3600 * 1000) {
                return TIME_DATATYPE;
            }
            return NUMERIC_DATATYPE
        }
        return ORDINAL_DATATYPE;
    };

    // init
    dataset.processMetaData();
    // handle meta data.

    function getTotal(chartObj, seriespos, subseriespos, colidx, dataPoint, showAs) {
        colidx = (+colidx);
        var yaxes = axes.y,
            index = -1,
            isBubblePie = chartObj.chartTypes[seriespos].value === $ZC.charttype.bubblepie;
        if (chartObj.dataObject.isstacked(chartObj.chartTypes[seriespos].value) && !chartObj.dataObject.isStackedPercentChart) {
            return getStackedTotal(chartObj, seriespos, dataPoint, showAs);
        }

        if (yaxes != null) {
            if (yaxes[0] instanceof Array) {
                yaxes.forEach(function(d, i) {
                    index = ((index == -1) && (d.indexOf(colidx) > -1)) ? i : index;
                });
            } else {
                index = (yaxes.indexOf(colidx) > -1) ? 0 : -1;
            }
            if (index != -1 || isBubblePie) {
                var total;
                if (isBubblePie) {
                    var X = dataset.getX(dataPoint);
                    var pieRenderer = chartObj.subrenderer[subseriespos || 0][X];
                    total = pieRenderer.total;
                } else if (chartObj.isAxisCategory || chartObj.dataObject.isPolarAxisCategory) {
                    var domain = chartObj.axes.y.groups[index].scale.domain();
                    total = domain[1];
                } else {
                    if (chartObj.isNonAxisMultiSeriesCategory) {
                        total = dataPoint.parent.value;
                    } else {
                        total = chartObj.renderer[0].total;
                    }
                }
                return (!total && (total !== 0)) ? null : total;
            }
        }
        return null;
    }



    function getColTypeOrder(colidx) {
        var axesType, order = [];
        for (var type in axes) {
            if (axes.hasOwnProperty(type)) {
                var val = (type == "y") ? d3_merge(axes[type]) : axes[type];
                if (val.indexOf(colidx) != -1) {
                    if (type == "y") {
                        axes[type].map(function(d, i) {
                            var index = d.indexOf(colidx)
                            if (index != -1) {
                                order.push(i);
                                order.push(index)
                            }
                        })
                    }
                    axesType = type;
                    return {
                        "type": axesType,
                        "order": order
                    };
                }
            }
        }
    }
    
    function sortAxesObject() {
        var newaxes = {},
            keysSorted = Object.keys(axes).sort(function(a, b) {
                return (b.length > a.length) ? -1 : (b.length < a.length) ? 1 : 0; // (a.length > b.length) is incorrect in SAFARI
            });
        keysSorted.forEach(function(d) {
            newaxes[d] = axes[d];
        });
        return newaxes;
    }
    
    return dataset;
};

function getStackedTotal(chartObj, seriesPos, d, showAs) {
    var stackGroupIdx = 0,
        rp = chartObj.renderer[seriesPos].commonRendererProp().params,
        dataset = chartObj.dataset,
        X = dataset.getX(d),
        Y = dataset.getY(d, rp.yaxiscolumnorder||0),
        total;

    if (chartObj.userdata.seriesdata.stackGroup) {
        stackGroupIdx = getStackGroupIndex(seriesPos, chartObj.userdata.seriesdata.stackGroup);
    }

    var curStackExtent = chartObj.dataObject.groupedByStack.get(stackGroupIdx).stackExtent
        .filter(function(d) {
            return d[0] === X;
        });

    if (showAs === "percent" || showAs === "per") {
       total = curStackExtent
            .reduce(function(prev, cur) {
                return prev + mathAbs(cur[1]);
            }, 0);
    } else {
       total = curStackExtent[(Y < rp.hasNegValue && curStackExtent[1]) ? 1 : 0][1];
    }
    return total;
}

