(function(win) {

    /**
     *
     * @param data
     * @param dataSet
     * @param chartInstance
     * @constructor
     */

    function DataProcessor(data, dataSet, chartInstance) {

        this.data = data;
        this.dataset = dataSet;
        this.chartObj = chartInstance;

        // support variables
        this.values = [];
        this.minmaxY = [];
        this.groupedDataBySplitAxes = [];
        this.groupedDataByCombo = d3_map();
        this.stackeddata = [];
        this.stackedChart = null;
        this.allChartTypes = null;
        this.hasNegativeValue = [];
        this.baseline = [];
        this.xRange = null;
        this.yRange = [];
        this.domainCorrections = {
            plotHeight: null,
            byDatalabels: {},
            forBubbles: {}
        };
        this.xaxisData = data.chart.axes.xaxis;
        this.yaxesData = data.chart.axes.yaxis;
        this.qualitativeRanges = null;
        this.groupedByStack = d3_map();
        this.ganttSeries_count = 0;
        this.categoriesStore = null;
        this.plotBorder = null;
        this.globalChartType = null;

        this.hierarchical = chartInstance.preProcessor.hierarchical;
        this.processedseriesData = chartInstance.seriesdata;
        this.initialize();
    }

    var DataProcessorProto = DataProcessor.prototype;

    /**
     *
     * @return {Array}
     */

    DataProcessorProto.getSeriesData = function() {
        var processor = this,
            chart = this.chartObj;

        if (defined(processor.hierarchical) && processor.hierarchical.categories) {
            // should be updated in the chartObj.seriesdata;
            var temp = processor.hierarchical.getSeries();
            chart.seriesdata = temp;
            return temp;
        }
        return chart.seriesdata;
    };

    /**
     * @returns {Array}
     */

    DataProcessorProto.getMinMaxZ = function() {
        return d3_extent(d3_merge(this.categoriesStore.z));
    };

    /**
     *
     * @param {Object} series is chart series
     * @returns {boolean}
     */

    DataProcessorProto.isBarEnabled = function(series) {
        var helpers = DataProcessor.helpers,
            cType = helpers.getChartType((series.type || this.globalChartType), NUMERIC_DATATYPE);
        return !series.disabled && isBarChartType(cType);
    };

    /**
     * Returns if the axis rotated
     * @returns {boolean}
     */

    DataProcessorProto.getAxisRotated = function() {
        var axes = this.data.chart.axes;
        return axes && axes.rotated === true;
    };

    /**
     *
     * @param {Number} index
     * @returns {*}
     */

    DataProcessorProto.containsNegativeData = function(index) {
        return this.hasNegativeValue[index];
    };

    /**
     *
     * @param seriesdata
     * @param categories
     * @returns {*}
     */
    DataProcessorProto.updateSeriesdatabyCategories = function(seriesdata, xCategories, yCategories, yaxiscolumnorder) {
        var dataSet = this.dataset,
            helpers = DataProcessor.helpers,
            isCatNullX = helpers.isCatNull(dataSet, "x"),
            isCatNullY = helpers.isCatNull(dataSet, "y", yaxiscolumnorder);
        seriesdata = seriesdata.filter(function(d) {
            var show = true;

            if (xCategories) {
                var x = dataSet.getX(d);
                show = show && !isCatNullX(x) && (xCategories.indexOf(x) > -1);
            }

            if (yCategories) {
                var y = dataSet.getY(d, yaxiscolumnorder);
                show = show && !isCatNullY(y) && (yCategories.indexOf(y) > -1);
            }

            return show;
        });
        return seriesdata;
    };

    /**
     *
     * @returns {*}
     */
    DataProcessorProto.getPlotBorder = function() {

        var processor = this,
            data = processor.data,
            borderSize = processor.plotBorder,
            plotBorderOptions = data.chart.plot.border;

        // var borderSize = validateValWithinLimit(plotBorderOptions, "size", 0, 10, chart.plotarea.width, chart.plotarea.height);

        var temp = 0;//(borderSize == 1) ? 0 : borderSize;
        if (plotBorderOptions.show === true) {
            borderSize = {};
            borderSize.top = borderSize.right = borderSize.bottom = borderSize.left = temp;
        }
        return borderSize;
    };

    /**
     *
     * @param domain
     * @param type
     * @param index
     * @returns {*}
     */

    DataProcessorProto.overBubbles = function(domain, type, index) {

        var processor = this,
            domainCorrections = processor.domainCorrections,
            xAxis = processor.xaxisData,
            yAxis = processor.yaxesData,
            chart = processor.chartObj,
            id = type + (defined(index) ? index : ""),
            plotHeight = chart.plotarea.height,
            chartObj = processor.chartObj

        // For bubble charts in which minrange and maxrange is not defined, extend the domain if the bubbles go out of the plotarea

        if (defined(domainCorrections.forBubbles[id]) && domainCorrections.plotHeight === plotHeight) {
            return domainCorrections.forBubbles[id];
        }


        domainCorrections.plotHeight = plotHeight;

        var isXAxis = (type === "x"),
            orient = chartObj.axes[type].orient,
            min = 0,
            max = chartObj.plotarea[(orient === "left") ? "height" : "width"],
            range = [min, max],
            axisConf = (isXAxis ? xAxis : yAxis[index]),
            scale = d3_scaleLinear().domain(domain).range(range);

        var updatedMinMax = processor.updateMinMaxForBubble(range, scale, type, index);

        min = (!defined(axisConf.minRange)) ? scale.invert(updatedMinMax[0]) : min;
        max = !defined(axisConf.maxRange) ? scale.invert(updatedMinMax[1]) : max;

        domainCorrections.forBubbles[id] = [(domain[0] - min), (max - domain[1])];
        return domainCorrections.forBubbles[id];
    };

    /**
     *
     * @param stacked
     * @param chartType
     * @param obj
     */

    DataProcessorProto.setGroupDataComboKey = function(stacked, chartType, obj) {
        var processor = this,
            data = processor.data,
            groupedByStack = processor.groupedByStack,
            groupedDataByCombo = processor.groupedDataByCombo,
            key = 0;

        if (stacked && isBarChartType(chartType)) {
            if (data.seriesdata.stackGroup) {
                key = getStackGroupIndex(obj.si, data.seriesdata.stackGroup);
            }

            if (!groupedByStack.has(key)) {
                groupedByStack.set(key, [obj]);
            } else {
                var lastData = groupedByStack.get(key);
                groupedByStack.set(key, lastData.concat([obj]));
            }
            groupedByStack.get(key).stackLabelIndex = obj.si;
        }

        /*
         * Grouping by chart type
         * chartType | chartType_stackGroupIdx (e.g): 10 | 10_0
         */

        var comboKey = defined(key) ? (chartType + "_" + key) : chartType;

        if (!groupedDataByCombo.has(comboKey)) {
            groupedDataByCombo.set(comboKey, [obj]);
        } else {
            var lastdat = groupedDataByCombo.get(comboKey);
            groupedDataByCombo.set(comboKey, lastdat.concat([obj]));
        }
    };

    /**
     *
     * @param curdata
     * @param stacked
     * @param yDatatype
     * @param chartType
     * @param isZenabled
     * @param yaxiscolumnorder
     */

    DataProcessorProto.setExtremes = function(curdata, stacked, yDatatype, chartType, isZenabled, yaxiscolumnorder) {

        var processor = this,
            dataset = processor.dataset,
            zCat = processor.categoriesStore.z;

        if (((yDatatype === NUMERIC_DATATYPE || yDatatype === TIME_DATATYPE) && !stacked) || isZenabled) {
            if (!curdata.disabled && isArray(curdata)) {
                var mergedSubgroup = d3_merge(curdata);
                if (isZenabled) {
                    var zExtremes = d3_extent(mergedSubgroup, function(d) {
                        return dataset.getZ(d)
                    });
                    zCat.push(zExtremes);
                }
                if (yDatatype === NUMERIC_DATATYPE || yDatatype === TIME_DATATYPE) {
                    var yExtremes;
                    if (chartType == $ZC.charttype.arearange || chartType == $ZC.charttype.gantt) {
                        var mergedYranges = d3_merge(mergedSubgroup.map(function(d) {
                            return dataset.getY(d, yaxiscolumnorder)
                        }));
                        yExtremes = d3_extent(mergedYranges);
                    } else {
                        yExtremes = d3_extent(mergedSubgroup, function(d) {
                            return dataset.getY(d, yaxiscolumnorder)
                        });
                    }
                    processor.seriesExtermes[yaxiscolumnorder].push(yExtremes);
                }
            }
        }
    };

    /**
     *
     * @param state
     */

    DataProcessorProto.initialize = function(state) {

        var processor = this,
            domainCorrections = processor.domainCorrections,
            xaxisData = processor.xaxisData,
            yaxesData = processor.yaxesData,
            allChartTypes = processor.allChartTypes,
            data = processor.data,
            chartObj = processor.chartObj,
            globalChartType = processor.globalChartType = data.seriesdata && data.seriesdata.type,

            noofYaxis = data.metadata.axes.y.length,
            metadata = data.metadata,
            axes = metadata.axes,
            stacked = processor.isstacked(),


            groupedDataBySplitAxes = processor.groupedDataBySplitAxes = [],
            groupedByStack = processor.groupedByStack = d3_map(),
            dataset = processor.dataset,
            categoriesStore = processor.categoriesStore = {},
            plotBorder,
            processedseriesData = processor.processedseriesData,
            helpers = DataProcessor.helpers;

        if (processor.hierarchical && !processor.hierarchical.groupByTime && defined(state)) {
            processor.hierarchical.update();
        }

        processor.xRange = null;
        processor.yRange = [];
        processor.qualitativeRanges = null;
        processor.groupedDataByCombo = d3_map();

        domainCorrections.plotHeight = null;
        domainCorrections.forBubbles = {};

        dataset.seriesTypes = [];

        processor.isBubbleEnabled = false;
        processor.noofBarEnabledSeries = 0;
        processor.noofAreaEnabledSeries = 0;
        processor.noofGanttEnabledSeries = 0;
        processor.totalnoofdatas = 0;
        processor.lastCategoryValue = 0;
        processor.isCombinationalChart = d3_set();
        processor.isAxisCategory = false;
        processor.isNonAxisMultiSeriesCategory = false;
        processor.isContinousChart = true;
        processor.isPolarAxisCategory = false;
        processor.hasLevelMarker = false;
        processor.isStackedPercentChart = false;

        categoriesStore.x = [];
        categoriesStore.z = [[0]];

        categoriesStore.subseries = [];
        dataset.hasNullFormatter = {};

        plotBorder = this.plotBorder = {};
        plotBorder.left = plotBorder.top = plotBorder.right = plotBorder.bottom = 0;

        // axisrotated = processor.getAxisRotated();
        // orient = xaxisData.orient || (axisrotated ? "left" : "bottom");
        // plotBorder[orient] = 0;//(xaxisData.show === true) && xaxisData.axisline && xaxisData.axisline.show === true && validateNullVal(xaxisData.axisline.strokeWidth, 0) || 0;

        processor.seriesExtermes = []; // finding series extremes for each series

        var yaxesGroups = chartObj.axes.y.groups;

        for (var i = 0; i < noofYaxis; i++) {

            categoriesStore["y" + i] = [];
            processor.seriesExtermes[i] = [];
            (!defined(yaxesData[i])) ? (yaxesData[i] = {}) : null;

            // Create yaxis group and mark the axis is disabled initially
            yaxesGroups[i] = yaxesGroups[i] || {};
            yaxesGroups[i].disabled = true;

            // var curYaxisData = (i > 0) ? mergeJSON(yaxesData[i], yaxesData[0], false) : yaxesData[i]; //CHANGE
            // var orient = curYaxisData.orient || (axisrotated ? "bottom" : (i ? "right" : "left"));
            // plotBorder[orient] = 0;//(curYaxisData.show === true && curYaxisData.axisline && curYaxisData.axisline.show === true && validateNullVal(curYaxisData.axisline.strokeWidth, 0)) || 0
        }

        if (isArray(processedseriesData)) {

            var xColumn = axes.x ? dataset.getColumn("x") : null,
                xDatatype = xColumn ? xColumn.datatype : null,
                isZenabled = defined(axes.z),
                groupbyDataIndices = helpers.groupByDataIndices(metadata);


            allChartTypes = processor.allChartTypes = allChartTypes || helpers.getAllChartTypes(data);

            // TODO have to rename of this function
            var temp = function(d) {
                return allChartTypes.indexOf(d) !== -1
            };

            processor.isDiscrete = $ZC.chartcategory.discrete.some(temp);
            processor.isHierarchy = $ZC.chartcategory.hierarchy.some(temp);

            //sorting of data
            var xColPos = data.metadata.axes.x;
            var datatype_x = data.metadata.columns[xColPos].datatype;

            processor.needXCategoriesSorting = false;
            var needXCategoriesSorting = false;

            processedseriesData.forEach(function(d, i) {

                var disabled = d.disabled,
                    curdata = d.data;
                var is3DArray = helpers.is3D(curdata);
                // var chart = chartObj.chartTypes[i],
                //     chartType = chart.value;
                var chartType = helpers.getChartType((d.type || globalChartType), NUMERIC_DATATYPE);
                var cType = helpers.getChartType((d.type || globalChartType), "string");
                var yaxiscolumnorder = helpers.getParsedYAxisOrder(d.yaxiscolumnorder);
                var yCat = categoriesStore["y" + yaxiscolumnorder];
                var yDatatype = dataset.getDataType("y", yaxiscolumnorder);

                dataset.seriesTypes.push(chartType);

                processor.isCombinationalChart.add(chartType);
                processor.isBubbleEnabled = processor.isBubbleEnabled || (!disabled && isBubbleChartType(chartType));
                processor.pieWithAxis = processor.pieWithAxis || (!disabled && chartType === $ZC.charttype.bubblepie);
                processor.noofBarEnabledSeries += (isBarChartType(chartType) && !disabled) ? 1 : 0;
                processor.noofAreaEnabledSeries += ((chartType == $ZC.charttype.area || chartType == $ZC.charttype.stkdarea) && !disabled) ? 1 : 0;
                processor.noofGanttEnabledSeries = processor.ganttSeries_count += ((chartType === $ZC.charttype.gantt) && !disabled) ? 1 : 0;


                processor.isContinousChart = processor.isContinousChart && (chartType == $ZC.charttype.line || isAreaChartType(chartType) || isPointChartType(chartType));
                processor.hasLevelMarker = processor.hasLevelMarker || (chartType == $ZC.charttype.bullet || chartType == $ZC.charttype.dial);

                processor.isAxisCategory = processor.isAxisCategory || (($ZC.chartcategory.axis).indexOf(chartType) >= 0);
                processor.isNonAxisMultiSeriesCategory = processor.isNonAxisMultiSeriesCategory || (($ZC.chartcategory.nonaxismultiseries).indexOf(chartType) >= 0);
                processor.isPolarAxisCategory = processor.isPolarAxisCategory || (chartType == $ZC.charttype.web);


                // convert data to 3d array even it is disabled;
                if (state !== "addorremoveseries" && !is3DArray && isArray(curdata)) {
                    curdata = processor.convertTo3DArray(d, groupbyDataIndices, chartType);
                }

                if (!disabled && isArray(curdata)) {

                    categoriesStore["y" + yaxiscolumnorder].enabled = true;
                    yaxesGroups[yaxiscolumnorder].disabled = false;

                    var obj = {
                        "si": i,
                        "d": curdata
                    };

                    //Grouping by split axes
                    if (!groupedDataBySplitAxes[yaxiscolumnorder]) {
                        groupedDataBySplitAxes[yaxiscolumnorder] = [obj];
                    } else {
                        groupedDataBySplitAxes[yaxiscolumnorder] = groupedDataBySplitAxes[yaxiscolumnorder].concat([obj]);
                    }

                    // Grouping by stack
                    processor.setGroupDataComboKey(stacked, chartType, obj);

                    var actXdatatype = dataset.getDataType("x", null, allChartTypes),
                        actYdatatype = dataset.getDataType("y", yaxiscolumnorder, allChartTypes),
                        yColumn = dataset.getColumn("y", yaxiscolumnorder);

                    // debugger
                    if (!defined(dataset.hasNullFormatter.x)) {
                        dataset.hasNullFormatter.x = !((actXdatatype === ORDINAL_DATATYPE) ? !isObjectPropDefined(xColumn[xDatatype], "nullformatting") : true);
                    }

                    if (!defined(dataset.hasNullFormatter["y" + yaxiscolumnorder])) {
                        dataset.hasNullFormatter["y" + yaxiscolumnorder] = !((yDatatype === ORDINAL_DATATYPE) ? !isObjectPropDefined(yColumn[yColumn.datatype], "nullformatting") : true);
                    }

                    //For stacked data need to iterate each data for collecting X and groupbyval
                    //Categories store update
                    processor.storeCategories(curdata, stacked, groupbyDataIndices, chartType, actXdatatype, yDatatype, yaxiscolumnorder);

                    //Numeric store extremes update
                    processor.setExtremes(curdata, stacked, yDatatype, chartType, isZenabled, yaxiscolumnorder);

                    //totalNo of datas
                    processor.totalDataCount(curdata);
                }
            });

            var userCategories = getPropVal(chartObj.userdata, "chart.axes.xaxis.categories");

            if (processor.isAxisCategory && datatype_x !== ORDINAL_DATATYPE && processor.needXCategoriesSorting && (!defined(userCategories) || (defined(userCategories) && userCategories.length === 0))) {
                categoriesStore.x.sort(d3_ascending);
            }

            //sort xcat, xoriginalcat
            processor.xymultiRenderer = processor.isAxisCategory;
            processor.isCombinationalChart = processor.isCombinationalChart.values().length > 1;
            processor.groupedByStack = groupedByStack;

            if (stacked) {
                var seriesTypes = dataset.seriesTypes,
                    plotoptions = processor.data.chart.plot.plotoptions;

                processor.isStackedPercentChart = ((processor.noofBarEnabledSeries && !processor.hasLevelMarker) && (plotoptions.bar.stacked.showAs === "percent")) ||
                    (processor.noofAreaEnabledSeries && (plotoptions.area.stacked.showAs === "percent")) ||
                    ((seriesTypes.indexOf($ZC.charttype.web) > -1) && (plotoptions.web.stacked.showAs === "percent")) ||
                    ((seriesTypes.indexOf($ZC.charttype.line) > -1) && (plotoptions.line.stacked.showAs === "percent"));
            }
            if (processor.pieWithAxis && (state !== "addorremoveseries")) {
                processor.bubblepieData = processor.processBubblePieData();
            }
        }
    };

    /**
     *
     * @return {Array}
     */

    DataProcessorProto.processBubblePieData = function(baseChartType) {
        // baseChartType = bubblepie | geobubblepie | geopie
        baseChartType = baseChartType || $ZC.charttype.bubblepie;

        var processor = this,
            processedseriesData = processor.processedseriesData,
            chartObj = processor.chartObj,
            dataset = processor.dataset;

        var pieData = [],
            dataindex = validateNullVal(processor.data.chart.plot.plotoptions[$ZC.charttypenames.get(baseChartType)].dataindex, 3),
            categories = processor.findXCategories(true);

        processedseriesData.forEach(function(d, i) {
            var chartTypeNumeric = chartObj.chartTypes[i].value;
            var ishideable = d.disabled || (chartTypeNumeric !== baseChartType),
                currentseriesdata = d.data;
            currentseriesdata.forEach(function(d, subseries) {
                pieData[subseries] = pieData[subseries] || [];
                categories.forEach(function(cat, i) {
                    var dt = d.filter(function(d) {
                        return dataset.getX(d) === cat
                    })[0];
                    if (!dt) {
                        var v;
                        processedseriesData.some(function(seires, si) {
                            var currentseriesdata = seires.data;
                            return currentseriesdata[subseries].some(function(d, i) {
                                var hasX = (dataset.getX(d) === cat);
                                if (hasX) {
                                    v = d.slice(0);
                                    v[dataindex] = null;
                                }
                                return hasX;
                            })
                        });
                        dt = v;
                    }

                    if (dt) { // if missing subseries available
                        dt.disabled = ishideable;
                        pieData[subseries][i] = pieData[subseries][i] || [];
                        pieData[subseries][i].push(dt);
                    }

                });
            });
        });
        return pieData;
    };

    /**
     *
     * @param conf
     * @return {Array}
     */

    DataProcessorProto.getThresholdValues = function(conf) {
        var values = [];
        if (defined(conf.threshold)) {
            each(conf.threshold, function(threshold) {
                if (threshold.show === true && threshold.values.length) {
                    values = values.concat(threshold.values)
                }
            });
        }
        return values;
    };

    /**
     *
     * @param {Array} categories
     * @param {Boolean} leaf
     * @return {*}
     */

    DataProcessorProto.getDomain = function(categories, leaf) {
        if (this.hierarchical && !(this.hierarchical.groupByDiscreteTime && leaf)) {
            return d3.range(categories.length)
        } else {
            return categories;
        }
    };

    /**
     *
     * @param range
     * @param scale
     * @param type
     * @param index
     * @return {[number,number]}
     */

    DataProcessorProto.updateMinMaxForBubble = function(range, scale, type, index) {

        var context = this,
            processedseriesData = context.processedseriesData,
            groupedDataBySplitAxes = context.groupedDataBySplitAxes,
            dataset = context.dataset,
            chartObj = context.chartObj,
            helpers = DataProcessor.helpers;

        var min = range[0],
            max = range[1],
            isXaxis = (type == "x"),
            serData = (isXaxis ? processedseriesData : groupedDataBySplitAxes[index]) || []; //serData = null on multi-y legend filter Test-502.json
        serData.forEach(function(series) {
            var currentdata = (isXaxis ? series.data : series.d),
                disabled = ((isXaxis ? series : processedseriesData[series.si]).disabled),
                yaxiscolumnorder = helpers.getParsedYAxisOrder((isXaxis ? series : processedseriesData[series.si]).yaxiscolumnorder);
            if (!disabled) {
                var isCatNullX = helpers.isCatNull(dataset, "x"),
                    isCatNullY = helpers.isCatNull(dataset, "y", yaxiscolumnorder);

                currentdata.forEach(function(subgroup) {
                    subgroup.forEach(function(data) {
                        var val = (isXaxis ? dataset.getX(data) : dataset.getY(data, index)),
                            z = dataset.getZ(data) || chartObj.zscale.domain()[0];

                        //If data is null, dont consider
                        if (!(!data || data.isDummy || isCatNullX(dataset.getX(data)) || isCatNullY(dataset.getY(data, yaxiscolumnorder)))) {

                            var zpx = chartObj.zscale(z),
                                valpx = scale(val) + ((scale.rangeBand != null) ? scale.rangeBand() / 2 : 0);
                            // if(isNaN(valpx)) debugger

                            var radPlus = valpx + (zpx / 2),
                                radMinus = valpx - (zpx / 2);

                            if (radPlus > max) {
                                max = radPlus;
                            }

                            if (radMinus < min) {
                                min = radMinus;
                            }
                        }
                    });
                });
            }
        });

        return [min, max];
    };


    /**************** Data Processor Helper Functions  **************/

    DataProcessor.helpers = {

        /**
         *
         * @param dataset
         * @param type
         * @param index
         * @return {Function}
         */

        isCatNull: function(dataset, type, index) {
            var noNullFormat = dataset.hasNullFormatter && !dataset.hasNullFormatter[type + (defined(index) ? index : "")];
            return function(cat) {
                return !defined(cat) && noNullFormat;
            }
        },


        /**
         *
         * @param data
         * @return {boolean}
         */

        is3D: function(data) {
            return isArray(data) && isArray(data[0]) && isArray(data[0][0]);
        },

        getParsedYAxisOrder: function(yaxiscolumnorder) {
            return isArray(yaxiscolumnorder) ? yaxiscolumnorder[0] : yaxiscolumnorder || 0;
        },

        /**
         *
         * @param type
         * @param chartTypeBy
         * @param index
         * @param data
         * @return {{name: *, value: *}}
         */
        getChartType: function(type, chartTypeBy, index, data) {
            if (type == null && index == null) {
                return
            }

            var seriesdata = data && data.seriesdata.chartdata,
                chartName,
                chartValue;
            if (defined(index)) {
                type = pick((seriesdata[index] || seriesdata[0]).type, data.seriesdata.type);
            }
            if (isNaN(+type)) {
                chartName = type.toLowerCase();
                chartValue = $ZC.charttype[chartName];
            } else {
                chartValue = +type;
                chartName = $ZC.charttypenames.get(chartValue);
            }
            return chartTypeBy === "string" ? chartName : chartTypeBy === NUMERIC_DATATYPE ? chartValue : {
                "name": chartName,
                "value": chartValue
            };
        },

        /**
         *
         * @param data
         * @param chartTypeBy
         */

        getAllChartTypes: function(data, chartTypeBy) {
            var helpers = this;
            var globalChartType = data.seriesdata && data.seriesdata.type;

            return pick(getPropVal(data, "seriesdata.chartdata"), []).map(function(a) {
                var temp = helpers.getChartType(pick(a.type, globalChartType), pick(chartTypeBy, NUMERIC_DATATYPE));
                if (!chartTypeBy && chartTypeBy === "object") {
                    return {}
                }
                return temp;
            })
        },

        /**
         *
         * @param array
         * @param startVal
         * @param endVal
         * @param accessor
         */

        filterDataToViewPort: function(array, startVal, endVal, accessor) {
            return array.filter(function(d) {
                var val = accessor(d);
                if (val >= startVal && val <= endVal) {
                    return d;
                }
            });
        },

        /**
         *
         * @param priorities
         * @param defaultValue
         * @return {number|*}
         */

        priorityBasedValue: function(priorities, defaultValue) {
            var value = null,
                size = priorities.length,
                i = 0;

            defaultValue = pick(defaultValue, 0)
            while (i < size && !defined(value)) {
                value = (priorities[i] != null) ? priorities[i] : null;
                i++
            }
            return pick(value, defaultValue)
        },

        groupByDataIndices: function(metadata) {

            /*
             * returns an array of groupby dataindices
             */
            var columns = metadata.columns,
                groupbyCol = metadata.axes.groupby || [];
            return groupbyCol.map(function(d) {
                return columns[d].dataindex;
            });
        }
    };


    $ZC.DataProcessor = DataProcessor;

})(window, $ZC);

