
/**
 *
 * @param domain
 * @param range
 * @param systemconf
 * @param padding
 * @param outerpadding
 * @param type
 * @param index
 * @returns {number}
 */

$ZC.DataProcessor.prototype.extendOuterpadding = function(domain, range, systemconf, padding, outerpadding, type, index) {

    var processedseriesData = this.processedseriesData,
        groupedDataBySplitAxes = this.groupedDataBySplitAxes,
        chartObj = this.chartObj;

    var outerPadding = 0;
    if (dataLabelEnabled(chartObj) || this.isBubbleEnabled) {
        if (range[0] > range[1]) {
            range = [range[1], range[0]];
        }
        var scale = d3_scaleOrdinal()
                .domain(domain)
                .rangeBands(range, padding, outerpadding),
            updatedMinMax = range;
        if (this.isBubbleEnabled) {
            var minmax = this.updateMinMaxForBubble(range, scale, type, index),
                correction = mathMax((range[0] - minmax[0]), (minmax[1] - range[1])) * 0.5;
            updatedMinMax = [(minmax[0] - correction), (minmax[1] + correction)];
        } else if ($ZC.labelRenderer) {
            var serData = (type == "x") ? processedseriesData : groupedDataBySplitAxes[index],
                maxFontSize = getDataLabelsSize(type, serData, chartObj),
                pad = scale.range()[0] - scale.rangeExtent()[0],
                maxFontSize = mathMax(0, (maxFontSize - pad));
            updatedMinMax = [(range[0] - maxFontSize), (range[1] + maxFontSize)];
        }
        outerPadding = calculateOuterpadding(scale, padding, outerpadding, updatedMinMax);
    }
    return outerPadding;
};

function dataLabelEnabled(chartInstance) {

    var systemConfig = chartInstance.systemConf,
        totalDataCount = chartInstance.dataObject.totalnoofdatas,
        allowedLabelCount = +systemConfig.canvas.intelligence.dataCount.datalabels;

    if (totalDataCount > allowedLabelCount) {
        return false;
    }

    if (getPropVal(systemConfig.chart, "plot.datalabels.show") === true) {
        return true;
    }
    var seriesData = chartInstance.seriesdata,
        len = seriesData.length;

    while (len--) {
        var commonPlotOptions = systemConfig.chart.plot.plotoptions[chartInstance.chartTypes[len].name],
            datalabels = commonPlotOptions.datalabels;

        if(defined(seriesData[len].plotoptions)) {
            datalabels = getMergedDatalabelOptions(commonPlotOptions, seriesData[len].plotoptions);
        }
        if(isDatalabelEnabled(datalabels, chartInstance)) {
            return true;
        }
    }
    return false;
}

function calculateOuterpadding(scale, padding, outerpadding, updatedMinMax) {
    var rangeExtent = scale.rangeExtent ? scale.rangeExtent() : scale.range(),
        domain = scale.domain(),
        rangeDiff = (rangeExtent[1] - rangeExtent[0]),
        min = updatedMinMax[0],
        max = updatedMinMax[1];
    var correction = mathMax((max - rangeExtent[1]), -(min));
    if (!correction) {
        return 0;
    }
    var step = rangeDiff / (domain.length - padding + 2 * outerpadding);
    var outerpad = step * outerpadding;
    var newouterpad = outerpad + correction;
    var newstep = (rangeDiff - (2 * newouterpad)) / (domain.length - padding);
    var newouterpadding = newouterpad / newstep;
    return newouterpadding - outerpadding;
}