;

/**
 *
 * @param {Chart} chartInstance
 * @constructor
 */

function HierarchicalDataAdapter(chartInstance) {
    this.chart = chartInstance;
    this.originalSeries = chartInstance.seriesdata;
    this.initialize();
}


HierarchicalDataAdapter.prototype = {


    initialize: function() {

        var layout = this;

        layout.hierarchyLayout = true;
        layout.finalDepth = 0;
        layout.dataCount = 0;
        layout.categories = [];
        layout.leafNodes = [];
        layout.series = [];
        layout.categoriesByDepth = [];
        layout.globalChartType = layout.originalSeries.type;

    },

    /**
     *
     */
    setDepth: function() {
        var layout = this,
            categories = layout.categories,
            depths = layout.categoriesByDepth = [],
            leaves = layout.leafNodes = [],
            leafCategories = layout.leafCategories = [],
            categoriesList = layout.categoriesList = [],
            pointProperty = layout.dataType === TIME_DATATYPE || layout.groupByDiscreteTime ? "point" : "value";


        doDepth(categories, 0);

        /**
         *
         * @param {array} cats
         * @param {number} depth
         * @param {object} parent
         */

        function doDepth(cats, depth, parent) {

            if (depth > layout.finalDepth) {
                layout.finalDepth = depth;
            }

            cats.forEach(function(c) {

                if (defined(parent)) {
                    c.parent = parent;
                }

                if (!c.leaf) {
                    if (!depths[depth]) {
                        depths[depth] = [];
                        categoriesList[depth] = [];
                    }
                    categoriesList[depth].push(c[pointProperty]);
                    depths[depth].push(c);
                    doDepth(c.children, depth + 1, c)
                } else {
                    c.index = layout.dataCount++;
                    if (isArray(c.data)) {
                        c.data.forEach(function(d) {
                            d.index = c.index;
                        });
                    }
                    leaves.push(c);
                    leafCategories.push(c[pointProperty]); // store the values for reducing loops
                }
            });
        }
        categoriesList.reverse();
    },

    /**
     *
     */

    getCategories: function() {
        return this.categories;
    },

    /**
     *
     * @param depth
     * @returns {*}
     */
    getCategoriesByDepth: function(depth) {
        if (!defined(depth)) {
            depth = 0
        }

        var categories = {};
        categories.categories = categories.children = null;
        if (depth === this.finalDepth) {
            return this.leafNodes;
        }
        else if (depth < this.finalDepth) {
            categories.categories = this.categoriesByDepth[depth];
            categories.children = this.categoriesByDepth[depth].map(function(d) {
                return d.children;
            })
        }
        return categories
    },

    /**
     *
     * @param {number} itemIndex
     * @param {number} parentDepthIndex
     * @returns {object}
     */

    getParentByIndex: function(itemIndex, parentDepthIndex) {

        var layout = this,
            leaf = layout.leafNodes[itemIndex],
            parent = null,
            count = layout.finalDepth + 1;

        while (count--) {
            if (leaf.depth === parentDepthIndex) {
                parent = leaf;
                break
            } else if (leaf.depth === 0) {
                break;
            }
            leaf = leaf.parent;
        }
        return parent;
    },

    /**
     *
     * @param {function} leafScale
     * @returns {Array}
     */

    setRanges: function(leafScale) {


        var layout = this,
            isOrdinal = defined(leafScale.rangeBand),
            depth = layout.finalDepth,
            innerPadding = 0,
            outerPadding = 0,
            points = layout.leafNodes,
            firstPoint,
            lastPoint,
            ranges = leafScale.range();

        if (isOrdinal) {
            var span = isOrdinal && leafScale.rangeBand(),
                rangeExtent = leafScale.rangeExtent();

            firstPoint = points[0];
            lastPoint = points[points.length - 1];
            innerPadding = (ranges[1] - (ranges[0] + span)) / 2;
            outerPadding = ranges[0] - rangeExtent[0];
            outerPadding = mathMin(outerPadding, innerPadding);

        } else {

            firstPoint = points[0];
            lastPoint = points[points.length - 1];

            var continuousRangeBand = $ZC.scale.getContinuousRangeBand(leafScale, layout.leafCategories);
            innerPadding = continuousRangeBand.rangeBand / 2;
            outerPadding = continuousRangeBand.outerPaddingInPixel;
        }

        layout.scaleList = [];
        // layout.categoriesList = [];
        layout.rangeList = [];

        function setRange(d, i) {
            if (isLeafParent(d)) {
                d.rangeBand = getRangeBand(children[i]);
                tempRange = (isOrdinal ? leafScale(d.children[0].index) : leafScale(d.children[0].point)) - getSpan(d.children[0], true);
                d.range = [tempRange, tempRange + d.rangeBand];

            } else {
                d.rangeBand = d.children.reduce(sumLeaf, 0);
                d.range = [d.children[0].range[0], d.rangeBand + d.children[0].range[0]]
            }

            tempRangeBandList.push(d.rangeBand);
            tempRangeList.push(d.range);
        }

        while (depth--) {
            var obj = layout.getCategoriesByDepth(depth),
                children = obj.children,
                categories = obj.categories,
                tempRange, tempRangeBandList = [], tempRangeList = [];

            categories.forEach(setRange);
            // layout.rangeBandList.push(tempRangeBandList);
            layout.rangeList.push(tempRangeList);
        }

        /**
         *
         * @param d
         * @returns {*|boolean}
         */
        function isLeafParent(d) {
            return defined(d.children[0]) && d.children[0].leaf === true;
        }

        /**
         *
         * @param a
         * @param point
         * @returns {Number}
         */
        function sumLeaf(a, point) {
            if (point.leaf) {
                return a + span + getSpan(point, true) + getSpan(point);
            }
            return a + point.rangeBand;
        }

        /**
         *
         * @param children
         * @returns {*}
         */

        function getRangeBand(children) {
            if (layout.categoryTree) {
                return children.reduce(sumLeaf, 0);
            } else {

                var start = children[0],
                    end = children[children.length - 1],
                    pointProperty = "point",
                    barWidth = span;

                if (layout.groupByContinuousTime) {

                    barWidth = 0;

                    if (defined(layout.tickValues[start.index + 1])) {
                        var startTick = layout.tickValues[start.index],
                            nextTick = layout.tickValues[start.index + 1],
                            r = leafScale(nextTick) - leafScale(startTick);

                        innerPadding = r / 2;
                    }
                } else {
                    pointProperty = "index";
                }

                return (leafScale(end[pointProperty]) - leafScale(start[pointProperty])) + getSpan(start, true) + getSpan(end) + barWidth;
            }
        }

        /**
         *
         * @param {Object} point
         * @param {boolean} startSide
         * @returns {number}
         */

        function getSpan(point, startSide) {
            if ((startSide && point === firstPoint) || (!startSide && point === lastPoint)) {
            // if((firstPoint === point || lastPoint === point)) {
                return outerPadding;
            } else {
                return innerPadding
            }
        }

        return layout.scaleList;
    }
};

/* ********************************** Static Functions ****************************/

/**
 *
 * @param {Object} obj
 * @param {String} key
 * @return {Array}
 */

HierarchicalDataAdapter.getValuesByKey = function(obj, key) {
    var mapper = function(a) {
        return a[key];
    };
    return obj.map(mapper)
};

/**
 *
 * @param arr
 * @return {boolean}
 */

HierarchicalDataAdapter.is3D = function(arr) {
    return isArray(arr) && isArray(arr[0])
};

/**
 *
 * @param series
 * @return {boolean|*}
 */

HierarchicalDataAdapter.isArrayOfSeries = function(series) {
    return HierarchicalDataAdapter.is3D(series.data) && defined(series.seriesname)
};

/**
 *
 * @param series
 * @return {*|boolean}
 */

HierarchicalDataAdapter.hasInnerSeries = function(series) {
    return isArray(series.data) && typeof series.data[0] === "object" && defined(series.data[0].seriesname);
};

/**
 *
 * @param arr
 * @return {{}}
 */

HierarchicalDataAdapter.keysObject = function(arr) {
    var t = {};
    if (arr && arr.length) {
        arr.forEach(function(el) {
            t[el.value] = el;
        });
    }
    return t;
};

/**
 *
 * @param arr
 * @return {Array|null}
 */

HierarchicalDataAdapter.getValues = function(arr) {
    if (!defined(arr)) {
        return null;
    }
    return arr.map(function(d) {
        return d.value;
    })
};

$ZC.HierarchicalDataAdapter = HierarchicalDataAdapter;
