(function() {


    "use strict";


    /**
     *
     * @param {Chart} chartInstance
     * @constructor
     */
    function HierarchicalOrdinalDataAdapter(chartInstance) {
        HierarchicalDataAdapter.call(this, chartInstance);
        this.dataType = ORDINAL_DATATYPE;
        this.categoryTree = true;
        this.update(true);
    }


    /**
     *
     * @type {{process: process, getSeries: getSeries, searchHierarchy: searchHierarchy}}
     */

    var hierarchicalOrdinalDataAdapter = {

        update: function(init) {

            if(!init) {
                this.initialize();
            }

            this.process();
            this.setDepth();
        },

        process: function() {

            var layout = this;
            layout.originalSeries.forEach(this.searchHierarchy, this);
        },


        getSeries: function(seriesIndex) {
            return defined(seriesIndex) ? this.series[seriesIndex] : this.series;
        },

        /**
         *
         * @param series
         * @param seriesIndex
         */
        searchHierarchy: function(series, seriesIndex) {

            var layout = this,
                chart = layout.chart,
                newSeries = layout.series[seriesIndex],
                column = pick(chart.dataset.getColumn("x"), {}),
                dataType = column.datatype,
                dataIndex = pick(column.dataindex, 0),
                seriesDisabled = series.disabled,
                hasNullFormatter = defined(column[dataType]) && defined(column[dataType].nullformatting),
                hasCategory = function(data) {
                    return hasNullFormatter || defined(data);
                };

            // create a new series if not exists
            if (!layout.series[seriesIndex]) {
                newSeries = layout.series[seriesIndex] = simpleClone(series);
                newSeries.data = [];
            }

            // start processing recursively
            search(series.data, 0, layout.categories);


            /**
             * @param {Object} series (current series)
             * @param depth depth of the children
             * @param children array to be stored the children
             */

            function search(series, depth, children) {

                var objKey = HierarchicalDataAdapter.keysObject(children);

                series.forEach(function(currentSeries) {

                    var hasChildren = HierarchicalDataAdapter.hasInnerSeries(currentSeries),
                        isArrayOfSeries = HierarchicalDataAdapter.isArrayOfSeries(currentSeries),
                        entry = null;

                    if (!objKey[currentSeries.seriesname]) {
                        entry = {
                            value: isArrayOfSeries || hasChildren ? pick(currentSeries.seriesname, "") : "",
                            depth: depth,
                            children: []
                        };
                        if(!seriesDisabled) {
                            children.push(entry);
                        }

                    } else {
                        entry = objKey[currentSeries.seriesname];
                    }

                    if (hasChildren) {
                        if (!objKey[currentSeries.seriesname]) {
                            objKey[currentSeries.seriesname] = entry;
                        }
                        search(currentSeries.data, depth + 1, objKey[currentSeries.seriesname].children);
                        if (entry.children.length === 0) {
                            removeElementFromArray(children, entry);
                        }

                    } else {

                        // convert data to 3d array
                        if (!HierarchicalDataAdapter.is3D(currentSeries.data)) {
                            currentSeries.data = [currentSeries.data];
                        }

                        var childrenEntries = HierarchicalDataAdapter.keysObject(entry.children);

                        currentSeries.data.forEach(function(d) {

                            var tp = {
                                value: d[dataIndex],
                                depth: depth + 1,// final depth,
                                leaf: true,
                                data: [d]
                            };

                            if (hasCategory(d[dataIndex])) {

                                // #ZC875
                                if(!seriesDisabled) {
                                    if (!childrenEntries[d[dataIndex]]) {
                                        entry.children.push(tp);
                                        childrenEntries[d[dataIndex]] = tp;

                                    }
                                    else if (childrenEntries[d[dataIndex]].leaf) {
                                        childrenEntries[d[dataIndex]].data.push(d);
                                    }

                                    // TODO need to verify.
                                    else if (childrenEntries[d[dataIndex]].children) {
                                        childrenEntries[d[dataIndex]].children.push(tp)
                                    }
                                }

                                // push the data to series
                                newSeries.data.push(d);
                            }

                        });

                        // remove the hierarchy if all is empty value
                        if (entry.children.length === 0) {
                            removeElementFromArray(children, entry);
                        }
                    }
                });
            }
        }
    };


    // inherit from HierarchicalDataAdapter
    inherit(HierarchicalOrdinalDataAdapter, HierarchicalDataAdapter);

    // extend all the functions
    extend(HierarchicalOrdinalDataAdapter.prototype, hierarchicalOrdinalDataAdapter);

    $ZC.HierarchicalOrdinalDataAdapter = HierarchicalOrdinalDataAdapter;

})();