/**
 *
 * @param index
 * @param correctionFactor
 * @returns {*}
 */

$ZC.DataProcessor.prototype.getMinMaxY = function(index, correctionFactor) {

    var processor = this,
        yAxisConfig = processor.yaxesData,
        hasNegativeValue = processor.hasNegativeValue,
        baseline = processor.baseline,
        yRange = processor.yRange,
        chart = processor.chartObj,
        dataSet = processor.dataset,
        stacked = processor.isstacked(),
        yAxisData = pick(yAxisConfig[index], {}),
        dataType = dataSet.getDataType("y", index),
        ganttSeriesCount = processor.ganttSeries_count,
        minY = yAxisData.minRange,
        maxY = yAxisData.maxRange,
        correction = 0,
        minMaxDataY = yRange[index],
        temp;

    index = pick(index, 0);
    baseline[index] = getPropVal(yAxisData, "baseline.show") === true ? yAxisData.baseline.value : null;

    if (isObjectPropDefined(chart.systemConf, "chart.zoom.y")) {
        var zoomedY = chart.systemConf.chart.zoom.y,
            zoomedYMinRange = zoomedY.minRange,
            zoomedYMaxRange = zoomedY.maxRange,
            baselineValue = baseline[index];
        hasNegativeValue[index] = (defined(baselineValue) && !stacked && baselineValue >= zoomedYMinRange && baselineValue <= zoomedYMaxRange) ? baselineValue : (zoomedYMinRange < 0) ? 0 : null;

        return [zoomedYMinRange, zoomedYMaxRange];

    }

    // TODO need to check
    // if (defined(minY) && defined(maxY)) {
    //     return [minY, maxY]
    // }

    correctionFactor = pick(correctionFactor, 0.05);
    if (!defined(yRange[index])) { // save the min and max of y at yRange

        if (stacked) {
            yRange[index] = updateMinMaxforStackedData(baseline[index], processor);
            temp = yRange[index][index];
        } else {
            yRange[index] = updateminMax(index, processor);
            temp = yRange[index];
        }

        var yAxis = pick(getPropVal(chart.userdata, "chart.axes.yaxis[" + index + "]"), {});
        if(defined(temp)) {
            temp = d3_extent(temp.concat(processor.getThresholdValues(yAxis)));
        }

        var levelMarkerMinMax;
        if (processor.hasLevelMarker || (processor.noofBarEnabledSeries && !ganttSeriesCount)) {
            levelMarkerMinMax = processor.overLevelMarker(temp)
        }

        if (stacked) {
            yRange[index][index] = pick(levelMarkerMinMax, temp);
        } else {
            yRange[index] = pick(levelMarkerMinMax, temp);
        }
        minMaxDataY = yRange[index];
    }


    if (!defined(minMaxDataY) || (stacked && !defined(minMaxDataY[index]))) {
        return ["NoData", "NoData"]
    }

    var extentY = stacked ? minMaxDataY[index] : minMaxDataY;

    if (dataType === TIME_DATATYPE) {
        hasNegativeValue[index] = null;
    } else {
        hasNegativeValue[index] = (defined(baseline[index]) && !stacked) ? baseline[index] : (yAxisData.minRange < 0 || extentY[0] < 0) ? 0 : null;
    }


    var hasDefaultBase = (processor.noofBarEnabledSeries || processor.noofAreaEnabledSeries) && !processor.hasLevelMarker && (hasNegativeValue[index] != null ? (hasNegativeValue[index] > 0) : (dataType == NUMERIC_DATATYPE));

    //If extentY = [undefined, undefined], don't change the extent with '0'

    if ((dataType === NUMERIC_DATATYPE) && (processor.noofBarEnabledSeries || processor.noofAreaEnabledSeries) && ((extentY[1] - extentY[0]) === 0)) {
        extentY = d3_extent(d3_merge([extentY, [0]])); // Test_511.json & Test_512.json
    }

    if (processor.isBubbleEnabled && dataType !== TIME_DATATYPE && !defined(yAxisData.minRange) && !defined(yAxisData.maxRange)) {

        //if 'hasDefaultBase' is present, apply that. so that we can adjust the bubbles within range.
        extentY[0] = hasDefaultBase ? 0 : extentY[0];

        temp = processor.overBubbles(extentY, "y", index);
        extentY = [(extentY[0] - temp[0]), (extentY[1] + temp[1])]
    }

    var diff = (extentY[1] - extentY[0]);

    if (diff === 0) {
        diff = 1;
    }

    if (!(processor.isStackedPercentChart || processor.hasLevelMarker)) {
        correction = diff * correctionFactor;
    }

    if (!defined(minY)) {
        minY = hasDefaultBase ? 0 : extentY[0] - correction;
    }

    if (!defined(maxY)) {
        maxY = extentY[1] + correction
    }

    var correctionOverDatalabels = 0;

    // no need to find correction datalabel,if correction factor is 0
    if (dataType !== "time" && correctionFactor !== 0) {
        correctionOverDatalabels = this.getCorrectionOverDataLabels([minY, maxY], "y", index);
    }


    if (!hasDefaultBase) {
        minY = (minY - correctionOverDatalabels) || 0;
    }

    maxY = (maxY + correctionOverDatalabels) || 0;
    return [minY, maxY];
};

function updateminMax(index, model) {
    // var yValues = categoriesStore["y"+index];
    var yValues = d3_merge(model.seriesExtermes[index]);
    var minmaxY = (yValues.length != 0) ? d3_extent(yValues) : (yValues.enabled) ? [0, 0] : null; // yvalues.enabled for referring that axis is visible or not.If the axis is visible,but all the data contains 0 or null,we dont want to remove axis
    return minmaxY;
}

function updateMinMaxforStackedData(baseline, context) {

    //

    var dataset = context.dataset,
        data = context.data,
        chartObj = context.chartObj;

    var processedseriesData = context.processedseriesData,
        groupedDataByCombo = context.groupedDataByCombo,
        groupedDataBySplitAxes = context.groupedDataBySplitAxes,
        categoriesStore = context.categoriesStore,
        groupedByStack = context.groupedByStack,
        dataObject = chartObj.dataObject,
        helpers = $ZC.DataProcessor.helpers;


    //
    var seriesdata = processedseriesData;
    var baseValue = 0; //(baseline == null) ? 0 : parseFloat(baseline)

    var grouped = (context.isCombinationalChart || data.seriesdata.stackGroup) ? groupedDataByCombo.values() : groupedDataBySplitAxes;
    var xCategories = dataObject.getDomain(context.getCategories("x")),
        xCategoriesLength = xCategories.length,
        groupByValues = categoriesStore.subseries,
        isCatNullX = helpers.isCatNull(dataset, "x"),
        minmaxY = [],
        lastSeriesY = [];

    var stackeddata = grouped.map(function(d, i) {
        var currType = groupedDataByCombo.keys()[i];
        var split = currType.split("_");
        var chartType = +split[0],
            groupIndex = defined(split[1]) ? +split[1] : i;
        var isStkd = context.isstacked(chartType);

        // y0 arrarys
        var y0Positive = [],
            y0Negative = [];

        var dt = d3_merge(d.map(function(da, j) {
            var seriesIndex = da.si;
            var yaxiscolumnorder = helpers.getParsedYAxisOrder(processedseriesData[seriesIndex].yaxiscolumnorder);

            if (isStkd && !isBarChartType(chartType)) { // CHANGE
                // add dummy group for missed groupbyvalues
                var curSubseriesVal = da.d.map(function(d) {
                    return d.groupByVal
                })
                for (var i = 0; i < groupByValues.length; i++) {
                    var key = groupByValues[i];
                    var havSubSeriesidx = curSubseriesVal.indexOf(key);
                    if (havSubSeriesidx < 0 && key != null) {
                        var duplicate = [];
                        da.d.splice(i, 0, duplicate);
                    }
                }
            }

            var dt = da.d.map(function(d, k) {
                if (isStkd && !isBarChartType(chartType)) { // CHANGE
                    var xMappingFunction = function(d) {
                        return dataset.getX(d)
                    };
                    var currentkeys = d.map(xMappingFunction);
                    for (var i = 0, len = xCategoriesLength; i < len; i++) {
                        var key = xCategories[i];
                        var haveKey = currentkeys.indexOf((key));
                        //aaranging order of data as per x order;
                        if (haveKey >= 0 && haveKey != i) {
                            d.splice(i, 0, d[haveKey]); //copying , Moving unordererd data to correct position i
                            d.splice(haveKey + 1, 1); // removing the copied data from old position havekey
                            currentkeys = d.map(xMappingFunction); // Rearranging the currentkeys
                        }
                        if (typeof currentkeys[i] != 'undefined' && isCatNullX(currentkeys[i])) {
                            d.splice(i, 1);
                        }
                        if (haveKey < 0) {
                            var duplicate = []
                            var order = helpers.getParsedYAxisOrder(processedseriesData[da.si].yaxiscolumnorder);
                            duplicate.splice(dataset.xidx, 0, (key));
                            duplicate.splice(dataset.yidx(order), 0, 0); //dummy data add as 0
                            duplicate.isDummy = true;
                            d.splice(i, 0, duplicate);
                            currentkeys = d.map(xMappingFunction); // Rearranging the currentkeys
                        }
                    }

                    // If the series has duplicate x values, just remove that. Test_586.json, Test_587.json
                    if (xCategoriesLength != d.length) {
                        d.splice(xCategoriesLength, (d.length - xCategoriesLength));
                    }
                }
                return d.map(function(d) {
                    var x = dataset.getX(d),
                        y = dataset.getY(d, yaxiscolumnorder);
                    var xCategoryIndex = xCategories.indexOf(x);
                    var y0ArrayToBeUpdated = (y < 0) ? y0Negative : y0Positive; // If y-value is (-ve) update on y0Negative, Else update on y0Positive array
                    var y0 = y0ArrayToBeUpdated[xCategoryIndex];

                    if (isStkd) {
                        y0 = defined(y0) ? y0 : baseValue;
                        d.y0 = y0;
                        d.y = y;
                        lastSeriesY[xCategoryIndex] = y0 + y;
                        y0ArrayToBeUpdated[xCategoryIndex] = y0 + y; // Update the y0 array
                    } else {
                        if (defined(y0)) {
                            y0ArrayToBeUpdated[xCategoryIndex] = ((y < 0) ? mathMin : mathMax)(y0, y);
                        } else {
                            y0ArrayToBeUpdated[xCategoryIndex] = y;
                        }
                    }

                    return d;
                });
            });
            return dt;
        }));

        minmaxY[i] = d3_extent(d3_merge([y0Positive, y0Negative, [0]]));

        // Save the stack extent
        // If stacklabel.show = true | datalabels.showAs = 'percent' | metadata.axes.tooltip = ['{{per(1)}}']
        var chartName = $ZC.charttypenames.get(chartType);
        if (isStkd && (isStacklabelEnabled(data.chart.plot.plotoptions[chartName].datalabels.stackLabels, chartObj) || isShowAsPercentage(chartName, chartObj) || PERCENT_PATTERN.test(data.metadata.axes.tooltip))) {
            if (!groupedByStack.has(groupIndex)) { // groupedByStack is available for bar chart only
                groupedByStack.set(groupIndex, {});
            }
            groupedByStack.get(groupIndex).stackExtent = updateStackExtent(xCategories, y0Positive, y0Negative);
        }

        return dt;
    });

    minmaxY = (groupedDataBySplitAxes.length > 1) ? minmaxY : [d3_extent(d3_merge(minmaxY))]

    if (context.isStackedPercentChart) {
        stackeddata.map(function(data) {
            return data.map(function(series) {
                return series.map(function(d) {
                    var xCategoryIndex = xCategories.indexOf(dataset.getX(d));
                    var max = lastSeriesY[xCategoryIndex] || 0;
                    if (isNaN(max) != true && max != null && max != 0) {
                        d.y0 /= max / 100;
                        d.y /= max / 100;
                    }
                    return d;
                })
            })

        });
        minmaxY[0][0] = 0, minmaxY[0][1] = 100;
    }
    return minmaxY;
}

function updateStackExtent(xCategories, pos, neg) {
    var stackValues = [];
    xCategories.forEach(function(x, j) {
        if (pos[j] >= 0) {
            stackValues.push([x, pos[j]]);
        }
        if (neg[j] < 0) {
            stackValues.push([x, neg[j]]);
        }
    });
    return stackValues;
}