;

(function(win) {


    "use strict";

    /**
     *
     * @param chart
     * @constructor
     */

    function PreProcessor(chart) {
        this.chart = chart;
        this.data = chart.systemConf;
        this.initialize();
    }

    PreProcessor.prototype.initialize = function() {

        var preProcessor = this,
            chart = preProcessor.chart,
            data = preProcessor.data,
            xColumnIndex = data.metadata.axes.x,
            xDataType = data.metadata.columns[xColumnIndex].datatype,
            dataProcessorHelpers = $ZC.DataProcessor.helpers,

            isDiscrete = $ZC.chartcategory.discrete.some(function(d) {
                return chart.chartTypes.indexOf(d) !== -1
            }),
            chartType = dataProcessorHelpers.getChartType(null, null, 0, chart.systemConf),
            chartValue = chartType.value,
            chartName = chartType.name;


        // check if the data has hierarchy
        var isAllowedHierarchyCharts = isBarChartType(chartValue) || isAreaChartType(chartValue) || isScatterChartType(chartValue)
            || isPointChartType(chartValue) || chartName === "line";

        if (this.isHierarchyLayout() && isAllowedHierarchyCharts) {
            preProcessor.hierarchical = new $ZC.HierarchicalOrdinalDataAdapter(this.chart);
            // update the chart data
            preProcessor.chart.seriesdata = this.hierarchical.getSeries();
        }

        // process the time data
        preProcessor.processTimeData();

        // sort if defined
        if ($ZC.requireXDataSorting && (xDataType === NUMERIC_DATATYPE || (xDataType === TIME_DATATYPE && !isDiscrete))) {
            preProcessor.sortData(xColumnIndex, xDataType);
        }
    };

    /**
     * Checks if the series has hierarchy
     * @return {boolean}
     */

    PreProcessor.prototype.isHierarchyLayout = function() {
        var processor = this,
            chart = processor.chart,
            series = chart.systemConf.seriesdata.chartdata;

        function hasHierarchy(series) {
            return HierarchicalDataAdapter.hasInnerSeries(series);
        }

        return series.some(hasHierarchy);
    };

    /**
     *
     */

    PreProcessor.prototype.processTimeData = function() {
        var processor = this,
            chart = processor.chart,
            timeDataInfo,
            timeDataIndices,
            timeDataInputFormat,
            hasTimeInputFormat,
            series = chart.seriesdata;

        timeDataInfo = PreProcessor.helpers.time.getDataInfo(processor.data);
        timeDataIndices = timeDataInfo.indices;
        timeDataInputFormat = timeDataInfo.formatConf;

        hasTimeInputFormat = timeDataInputFormat.some(function(d) {
            return defined(d) && defined(d.inputformat);
        });

        if (timeDataIndices && timeDataIndices.length && hasTimeInputFormat) {
            series.forEach(function(d, i) {
                var currentData = d.data;
                var is3DArray = $ZC.DataProcessor.helpers.is3D(currentData);
                if (isArray(currentData)) {
                    currentData.forEach(function(subgroup, j) {
                        if (is3DArray && isArray(subgroup)) {
                            subgroup.forEach(function(dataPoint, k) {
                                PreProcessor.helpers.time.applyFormats(dataPoint, timeDataInfo);
                            });
                        }
                        if (!is3DArray) {
                            PreProcessor.helpers.time.applyFormats(subgroup, timeDataInfo);
                        }
                    });
                }
            });
        }
    };


    /**
     *
     * @param xColPos
     * @return {*}
     */

    PreProcessor.prototype.sortData = function(xColPos) {

        var processor = this,
            chart = processor.chart,
            data = processor.data,
            series = chart.seriesdata;

        series.forEach(function(currentSeries) {
            var currentData = currentSeries.data,
                dataIndex = data.metadata.columns[xColPos].dataindex;

            if ($ZC.DataProcessor.helpers.is3D(currentData)) {
                if (isArray(currentData)) {
                    currentData.forEach(function(subgroup) {
                        PreProcessor.helpers.linear.sort(subgroup, dataIndex);
                    });
                }
            } else {
                PreProcessor.helpers.linear.sort(currentData, dataIndex);
            }
        });
        return data;
    };


    /**************** Pre Processor Dependency Functions  **************/

    PreProcessor.helpers = {
        time: {},
        ordinal: {},
        numeric: {},
        linear: {}
    };

    /************ Time Helpers Start **************/


    PreProcessor.helpers.time = {

        /**
         *
         * @param data
         * @param isDiscrete
         * @return {{indices: Array, formatConf: Array}}
         */

        getDataInfo: function(data, isDiscrete) {
            var xColumnIndex = data.metadata.axes.x,
                columnIndexes = d3_merge([xColumnIndex, d3_merge(data.metadata.axes.y)]),
                regexTimeStamp = /timestamp/i,
                timeFormatConfig,
                inputFormat,
                column,
                timeDataIndices = [],
                formatConf = [];

            columnIndexes.map(function(d) {

                column = pick(data.metadata.columns[d], {});
                timeFormatConfig = pick(column.time, {});

                if (column.datatype === "time" && (d !== xColumnIndex && !isDiscrete)) {

                    inputFormat = timeFormatConfig.inputformat;

                    if (!regexTimeStamp.test(inputFormat) || inputFormat === "function" || typeof win[inputFormat] === "function") {
                        formatConf.push(timeFormatConfig);
                    } else {
                        formatConf.push(null);
                    }
                    timeDataIndices.push(data.metadata.columns[d].dataindex);
                }
            });

            return {
                "indices": timeDataIndices,
                "formatConf": formatConf
            };
        },

        /**
         *
         * @param d
         * @param info
         * @return {*}
         */

        applyFormats: function(d, info) {
            var indices = info.indices,
                format = info.formatConf;

            indices.map(function(index, j) {
                if (defined(d[index])) {
                    if (isArray(d[index])) {
                        d[index] = d[index].map(function(d) {
                            return (generateDateVal(d, null, format[j], true)).getTime();
                        })
                    } else {
                        d[index] = (generateDateVal(d[index], null, format[j], true)).getTime();
                    }
                }
            });
            return d;
        }
    };

    /************ Time Helpers End **************/

    /************ Linear Helpers Start **************/

    PreProcessor.helpers.linear = {

        /**
         *
         * @param array
         * @param dataIndex
         */

        sort: function(array, dataIndex) {
            return array.sort(function(a, b) {
                a = a[dataIndex];
                b = b[dataIndex];
                return a < b ? -1 : a > b ? 1 : a >= b ? 0 : NaN;
            });
        }
    };

    /************ Linear Helpers End **************/


    $ZC.PreProcessor = PreProcessor;

})(window);