//$Id$
/*
 * this is an extended utility to the data parser to process the data with supporting methods like
 * getCategories, getMin or Max of a series values etc.,
 */
$ZC.hierarchymodel = {}
$ZC.hierarchymodel.get = function(chartObj) {
    var data = chartObj.systemConf,
        dataset = chartObj.dataset,
        processedseriesData = chartObj.seriesdata,
        dataProcessorHelpers = $ZC.DataProcessor.helpers;

    var model = {
        data: data
    };

    model.getTransRectValue = function (param, d, i, j, cp, radius, yrangePos) {
        var priorityBasedValue = dataProcessorHelpers.priorityBasedValue;
        switch (param) {
            case "radius": {
                var parsedVal = {}
                var splittedStr = (radius != null) ? radius.toString().split("/") : null
                var rx = (splittedStr != null && splittedStr[0] != null) ? splittedStr[0].split(" ") : [],
                    ry = (splittedStr != null && splittedStr[1] != null) ? splittedStr[1].split(" ") : rx;
                /*
                 * index 0-->topleft,  1-->topright,  2-->bottomright,  3-->bottomleft
                 */
                parsedVal.tlx = pInt(priorityBasedValue([rx[0]]))
                parsedVal.tly = pInt(priorityBasedValue([ry[0], rx[0]]))
                
                parsedVal.trx = pInt(priorityBasedValue([rx[1], rx[0]]))
                parsedVal.trY = pInt(priorityBasedValue([ry[1], ry[0], rx[1], rx[0]]))
                
                parsedVal.brx = pInt(priorityBasedValue([rx[2], rx[0]]))
                parsedVal.bry = pInt(priorityBasedValue([ry[2], ry[0], rx[2], rx[0]]))
                
                parsedVal.blx = pInt(priorityBasedValue([rx[3], rx[1], rx[0]]))
                parsedVal.bly = pInt(priorityBasedValue([ry[3], ry[1], rx[3], rx[0]]))
                
                return parsedVal
            }
        }
    };
    model.getAxisRotated = function() {
        var axes = model.data.chart.axes;
        return axes && axes.rotated === true;
    };

    model.containsNegativeData = function() {
        return false;
    };

    model.isstacked = function() {
        return false;
    };    

    model.getPlotBorder = function() {
        var plotBorderOptions = data.chart.plot.border;
        var borderSize = plotBorderOptions.show === true
            ? validateValWithinLimit(plotBorderOptions, "size", 0, 10, chartObj.plotarea.width, chartObj.plotarea.height)
            : 0;
        borderSize = {
                "left": borderSize,
                "top": borderSize,
                "right": borderSize,
                "bottom": borderSize
            };
        return borderSize;
    };

    function dataToArray(obj){
        return obj.map(function(d){
            var arr;
            if (d.data && d.data[0] && d.data[0].constructor === Array) {
                arr = d.data;
                var leafColors = d.leafColors;
                if (leafColors) {
                    arr.forEach(function(d, i) {
                        d.color = leafColors[i %leafColors.length];
                    });
                }
            } else if (d.data){
                model.hasHierarchyData = true;
                arr = dataToArray(d.data);
            } else {
                arr = d;
            }

            for (var attr in d) {
                if (d.hasOwnProperty(attr) && attr !== "data") {
                    arr[attr] = d[attr];
                }
            }

            return arr;
        });
    }

    function processHierachyData(parent) {
        parent.forEach(function(d, i) {
            d.level = (parent.level != null) ? (parent.level + "_" + i) : (i + "");

            if (isArray(d[0])) {
                processHierachyData(d);
            } else {
                var seriesIndex = +d.level.split("_")[0],
                    currentdata = data.seriesdata.chartdata[seriesIndex],
                    yaxiscolumnorder = dataProcessorHelpers.getParsedYAxisOrder(currentdata.yaxiscolumnorder);

                if (!currentdata.disabled) {
                    var yExtremes = d3_extent([d], function(d) {
                                        return dataset.getY(d, yaxiscolumnorder)
                                    });

                    model.seriesExtermes[0].push(yExtremes);
                    model.totalnoofdatas++;
                }
            }
        });
    }

    model.initialize = function(state) {
        globalChartType = data.seriesdata && data.seriesdata.type;
        dataset.seriesTypes = [];
        model.totalnoofdatas = 0;
        model.isNonAxisMultiSeriesCategory = true;
        model.isHierarchy = true;
        model.seriesExtermes = [[]];
        
        if (processedseriesData != null && isArray(processedseriesData)) {
            if (state != "addorremoveseries") {
                chartObj.seriesdata = processedseriesData = dataToArray(processedseriesData);
            }

            processHierachyData(processedseriesData);
            processedseriesData.forEach(function(d, i) {
                var chartType = dataProcessorHelpers.getChartType((d.type || globalChartType), NUMERIC_DATATYPE);
                dataset.seriesTypes.push(chartType);
            });
        };
    }
    
    model.initialize();
    return model;
};

function getChildrenCount(dt, count) {
    if (!count) {
        count = 0;
    }

    dt.forEach(function(d) {
        if (isArray(d[0])) {
            count = getChildrenCount(d, count);
        } else {
            count++;
        }
    });
    return count;
}

