//$Id$
var canvas_gradients = (function() {
    var types = {};
    
    var create = function(context, patternName, properties, offset) {
        if (/threshold/ig.test(patternName)) {
            return types[patternName](context, properties, offset);
        } else {
            var safari = $Browser.SAFARI;
            $Browser.SAFARI = 0;
            var stop = gradients.getStops({}, properties, properties.color, patternName);
            var gradient = types[patternName](context, properties, stop);
            $Browser.SAFARI = safari;
            return gradient;
        }
    };
    var canvasGradientFill =  types.canvasGradientFill = function(properties, gradientID, offset, returnSVGimage, base64, chartObj){
        return (properties && properties.type) ? canvas_gradients[properties.type](properties, gradientID, offset, returnSVGimage, base64, chartObj) : null;
    }
    
    var linear = types.linear = function(properties, gradientID, offset, returnSVGimage, base64, chartObj) {
        var context = chartObj.context,
            dimension = properties.dimension,
            spread = properties.spreadMethod,
            stops = getStops({}, properties, properties.color, "linear");
        var dim = (spread === "repeat" || spread === "reflect") && ($Browser.IS_IOS || $Browser.SAFARI) ?
            mathMax(mathAbs(+properties.x2 - +properties.x1, mathAbs(+properties.y2) - +properties.y1)) :
            100;
        
        var xScale = d3_scaleLinear().domain([0, 100]).range([0, dimension[0]]),
            yScale = d3_scaleLinear().domain([0, 100]).range([0, dimension[1]]),
            x0 = xScale(+(properties.x1 || 0) * 100 / dim),
            x1 = xScale(+(properties.x2 || 0) * 100 / dim),
            y0 = yScale(+(properties.y1 || 0) * 100 / dim),
            y1 = yScale(+(properties.y2 || 0) * 100 / dim);
        
        var gradient = context.createLinearGradient(x0, y0, x1, y1);
        addStop(gradient, stops, properties.opacity);
        return gradient;
    };
    
    var radial = types.radial = function(properties, gradientID, offset, returnSVGimage, base64, chartObj) {
        var context = chartObj.context,
            dimension = properties.dimension,
            spread = properties.spreadMethod,
            stops = getStops({}, properties, properties.color, "radial");
            x = (dimension[0] / 100 * properties.x || 0),
            y = (dimension[0] / 100 * properties.y || 0),
            radius = ((spread === "repeat" || spread === "reflect") && ($Browser.IS_IOS || $Browser.SAFARI)) ? dimension[0] : (dimension[0] / 100 * properties.radius);
        
        var gradient = context.createRadialGradient(x, y, 0, x, y, radius);
        addStop(gradient, stops, properties.opacity);
        return gradient;
    };
    
    var thresholdLinear = types.thresholdLinear = function(properties, gradientID, offset, returnSVGimage, base64, chartObj) {
        var context = chartObj.context;
        var stop = {
            "opacity": [],
            "offset": [],
            "color": []
        };
        offset.forEach(function(d) {
            stop.opacity.push(1);
            stop.offset.push(d.offset);
            stop.color.push(d.color);
        });
        
        var gradient = context.createLinearGradient(properties.x1, properties.y1, properties.x2, properties.y2);
        addStop(gradient, stop, properties.opacity);
        return gradient;
    };
    
    var addStop = function(gradient, stop, stopOpacity) {
        var n = stop.offset.length,
            i = -1;
        stopOpacity = stopOpacity || 1;
        while (++i < n) {
            var opacity = mathAbs(1 - (stop.opacity[i] + stopOpacity)),
                color = canvasUtils.rgba(stop.color[i], opacity);
            gradient.addColorStop(parseFloat(stop.offset[i]) / 100, color);
        }
    };
    
    var global = {};
    global.create = create;
    global.linear = linear;
    global.radial = radial;
    global.thresholdLinear = thresholdLinear;
    global.canvasGradientFill = canvasGradientFill;
    return $ZC.canvasGradient ? $ZC.canvasGradient : global;
})();