//$Id$
var svg_gradients = function(){};

svg_gradients.get = function(gradient, color, dimension, offset, chartObj, defaultOpacity) {
    /**
     * @param chartObj (optional)
     * @param defaultOpacity (optional) // for canvas.
     */
    
    var gradientTypeName = gradient.type;
    if (!gradientTypeName || gradientTypeName === "none") {
        return;
    }
    var mode = chartObj ? chartObj.plot.renderer.mode : "svg";
    
    var properties = gradient.options[gradientTypeName] || gradient.options;
    var propObj = simpleClone(properties);
    propObj.color = (color != null) ? color : propObj.color;
    propObj.dimension = (dimension != null) ? dimension : propObj.dimension;
    propObj.defsLocation = chartObj ? chartObj.defsLocation : "";
    propObj.id = chartObj.id;
    switch (mode) {
        case "canvas": {
            propObj.pointOpacity = defaultOpacity || 1;
            return canvasGradient.create(chartObj.context, gradientTypeName, propObj, offset);
            break;
        }
        default: {
            return effects.get(svg_gradients,gradientTypeName, propObj, {"offset":offset});
            break;
        }
    }
};
svg_gradients.svgGradientFill = function(properties, gradientID, offset, returnSVGimage, base64, chartObj){
        return (properties && properties.type) ? svg_gradients[properties.type](properties, gradientID, offset, returnSVGimage, base64, chartObj) : null;
}
svg_gradients.radial = function(properties, gradientID, type, returnSVGimage, base64, chartObj) {
    var svg = $ZCG.patternSVG,
        color = properties.color,
        dimension = properties.dimension,
        spread = properties.spreadMethod;
    var x = (dimension[0] / 100 * (+properties.x || 0)),
        y = (dimension[0] / 100 * (+properties.y || 0)),
        radius = ((spread === "repeat" || spread === "reflect") && ($Browser.IS_IOS || $Browser.SAFARI)) ? dimension[0] : (dimension[0] / 100 * (+properties.radius));
    var gradient = svg.select("defs").append("radialGradient")
        .attr({
            "id": gradientID,
            "gradientUnits": "userSpaceOnUse",
            "spreadMethod": spread,
            "cx": x + "px",
            "cy": y + "px",
            "r": radius + "px"
        });
    addStop(gradient, properties, color, "radial");
    return gradientID;
};
svg_gradients.linear = function(properties, gradientID, type, returnSVGimage, base64, chartObj) {
    var svg = $ZCG.patternSVG,
        color = properties.color,
        dimension = properties.dimension,
        spread = properties.spreadMethod,
        x1 = (+properties.x1 || 0),
        x2 = (+properties.x2 || 0),
        y1 = (+properties.y1 || 0),
        y2 = (+properties.y2 || 0);
    var dim = (spread === "repeat" || spread === "reflect") && ($Browser.IS_IOS || $Browser.SAFARI) ?
        mathMax(mathAbs(x2 - x1), mathAbs(y2 - y1)) :
        100;
    var min = (/(pie|dial|bubblepie|web)/.test(type)) ? (-dimension[0]) : 0;
    var xscale = d3_scaleLinear().domain([0, 100]).range([min, dimension[0]]),
        yscale = d3_scaleLinear().domain([0, 100]).range([min, dimension[1]]);
    x1 = x1 * 100 / dim,
        x2 = x2 * 100 / dim,
        y1 = y1 * 100 / dim,
        y2 = y2 * 100 / dim;
    var gradient = svg.select("defs").append("linearGradient")
        .attr({
            "id": gradientID,
            "gradientUnits": properties.gradientUnits,
            "spreadMethod": spread
        });
    if (properties.gradientUnits == "userSpaceOnUse") {
        gradient.attr({
            "x1": xscale(x1) + "px",
            "x2": xscale(x2) + "px",
            "y1": yscale(y1) + "px",
            "y2": yscale(y2) + "px"
        });
    } else {
        gradient.attr({
            "x1": x1 + "%",
            "x2": x2 + "%",
            "y1": y1 + "%",
            "y2": y2 + "%"
        });
    }
    addStop(gradient, properties, color, "linear");
    return gradientID;
};

svg_gradients.thresholdLinear = function(properties, gradientID, offset) {
    var svg = $ZCG.patternSVG;
    var gradient = svg.select("defs").append("linearGradient")
        .attr({
            "id": gradientID,
            "gradientUnits": "userSpaceOnUse",
            "x1": properties.x1,
            "x2": properties.x2,
            "y1": properties.y1,
            "y2": properties.y2
        });
    var stop = gradient.selectAll("stop").data(offset);
    
    var stopEnter = stop.enter().append("stop")
        .attr("offset", function(d) {
            return d.offset;
        })
        .attr("stop-color", function(d) {
            return d.color;
        });
    var stopUpdate = stop.attr("offset", function(d) {
            return d.offset;
        })
        .attr("stop-color", function(d) {
            return d.color;
        });
    stop.exit().remove();
    return gradientID;
}

function addStop(gradient, properties, color, type) {
    "use strict";
    var stops = getStops(gradient, properties, color, type);
    for (var i = 0; i < stops.offset.length; i++) {
        gradient.append("stop")
            .attr("offset", stops.offset[i])
            .style({
                "stop-color": stops.color[i],
                "stop-opacity": stops.opacity[i]
            });
    }
};

function getStops(gradient, properties, color, type) {
    "use strict";
    var stopoffset = properties.stopOffset,
        stopopacity = properties.stopOpacity,
        stopcolor = properties.stopColor,
        colorgamma = properties.colorGamma || [1, 0],
        spread = properties.spreadMethod,
        radius = (type == "radial") ? (+properties.radius) : mathMax(mathAbs((+properties.x2 || 0) - (+properties.x1 || 0)), mathAbs((+properties.y2 || 0) - (+properties.y1 || 0)));
    stopoffset = stopoffset || ["0%", "100%"];
    stopcolor = ((typeof stopcolor === "function") ? stopcolor(color) : stopcolor) || stopoffset.map(function(d, i) {
            return getColorCopy(color, (colorgamma[i] || 0), "brighter", d3_hsl);
        });
    stopopacity = stopopacity || [1, 1];
    
    if ((spread === "repeat" || spread === "reflect") && ($Browser.IS_IOS || $Browser.SAFARI)) {
        var newStopoffset = [],
            newStopColor = [],
            newStopOpacity = [],
            step = 0,
            repeat = 0;
        while (step < 100) {
            for (var i = 0, len = stopoffset.length; i < len; i++) {
                newStopoffset.push(((parseFloat(stopoffset[i]) / 100 * radius) + step) + "%");
                newStopOpacity.push(((repeat % 2) != 0) ? stopopacity[stopopacity.length - 1 - i] : stopopacity[i]);
                newStopColor.push(((repeat % 2) != 0) ? stopcolor[stopcolor.length - 1 - i] : stopcolor[i]);
            }
            step += radius;
            repeat += ((spread === "reflect") ? 1 : 0);
        }
        stopoffset = newStopoffset;
        stopcolor = newStopColor;
        stopopacity = newStopOpacity;
    }
    return {
        "offset": stopoffset,
        "color": stopcolor,
        "opacity": stopopacity
    }
};