//$Id$
var svg_patterns = function() {};

svg_patterns.imageFill = function(properties, patternID) {
    var w = properties.width,
        h = properties.height,
        svg = $ZCG.patternSVG;
    svg.select("defs").append("pattern")
        .attr({
            "patternUnits": "userSpaceOnUse",
            "id": patternID,
            "width": w,
            "height": h,
            "x": properties.x || 0,
            "y": properties.y || 0
        })
        .append('image')
        .attr({
            "xlink:href": properties.name,
            "x": 0,
            "y": 0,
            "width": w,
            "height": h
        });
    
    return patternID;
}


svg_patterns.svgPatternFill = function(properties, patternID, returnSVGimage, base64, patternUnits) {
    var patternEle;

    if(properties.ele != null){
       patternEle =  generateSVGPatternByHTML(properties.ele,properties,patternID,patternUnits);
    }else{
       patternEle =  generateSVGPatternByProp(properties,patternID,patternUnits);
    }

    if (returnSVGimage) {
        var xml = "<svg xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' width='" + patternEle.w + "' height='" + patternEle.h + "'>" + new XMLSerializer().serializeToString(patternEle.node.node()) + "<path d='" + getSVGsymbol("square")(mathMin(patternEle.w, patternEle.h)) + "' transform='translate(" + (patternEle.w / 2) + "," + (patternEle.h / 2) + ")' style='fill: " + "url(#" + patternEle.id + ")'></path> </svg>";
        return {
            url :  ($Browser.FIREFOX || base64) ? ('data:image/svg+xml;base64,' + btoa(xml)) : ('data:image/svg+xml;utf8,' + xml),
            width : patternEle.w,
            height : patternEle.h,
            id :patternEle.id
        };
    }

    return patternEle;
}


function generateSVGPatternByHTML(ele,properties,patternID,patternUnits){
    var pattern_svg = $ZCG.patternSVG;
    var bgClr = properties.bgClr || properties.bgClr_base;
    var customele = invokeFunction(ele, bgClr);
    
    properties.ele = customele.replace(/\{\{clr\}\}/g, bgClr);

    var patternNode = pattern_svg.select("defs").append("pattern")
        .attr({
            "patternUnits": patternUnits || "userSpaceOnUse",
            "id": patternID,
            "x": properties.x || 0,
            "y": properties.y || 0
        })
        .html(properties.ele);

    var bb;
    if ($Browser.FIREFOX) {
        pattern_svg.append("g").html(properties.ele).each(function() {
            bb = bbox(this);
        }).remove()
    } else {
        bb = bbox(patternNode.node().firstChild);
    }

    var w = bb.width,
        h = bb.height;
    patternNode
        .attr({
            "width" : w,
            "height" : h
        });

    return {"node" : patternNode, "w":w, "h":h, "id":patternID};
}
function generateSVGPatternByProp(properties,patternID,patternUnits){
    var pattern_svg = $ZCG.patternSVG;
    var bgClr_base = properties.bgClr || properties.bgClr_base, 
        stroke_base = properties.stroke || properties.stroke_base;
    var regExp = /\{\{([a-z]+)\}\}/;

    var symbolTypes = (properties.type || "").split(","),
        noOfSymbols = symbolTypes.length,
        size = (properties.size || "").split(","),
        strokeWidth = (properties.strokeWidth || "").split(","),
        fillColor = bgClr_base.split(","),
        stroke = stroke_base.split(","),
        fillOpacity = (properties.fillOpacity || "").split(","),
        strokeOpacity = (properties.strokeOpacity || "").split(",")
        translate = properties.translate ? properties.translate.split(",") : null;
        size = properties.width ? getPatternSize(properties,size) : size;

    var patternNode = pattern_svg.select("defs")
        .append("pattern")
        .attr({
            'id': patternID,
            'patternUnits': patternUnits || 'userSpaceOnUse',
            'width': size[0],
            'height': size[0],
            'x' : properties.x || 0,
            'y' : properties.y || 0
        });
    
    for(var i=0; i<noOfSymbols; i++){
        var pattern_path = getSVGsymbol(symbolTypes[i])((size[i] || size[0])),
            stroke_cur = isNull(stroke[i] || stroke[0]) ?  properties.stroke_base : (stroke[i] || stroke[0]),
            fill_cur = isNull(fillColor[i] || fillColor[0]) ?  properties.bgClr_base : (fillColor[i] || fillColor[0]);
            stroke_cur = (regExp.test(stroke_cur)) ? getColorCopy(properties.stroke_base, 1, stroke_cur.replace(regExp, '$1') ,d3_hsl) : stroke_cur;
            fill_cur = (regExp.test(fill_cur)) ? getColorCopy(properties.bgClr_base, 1, fill_cur.replace(regExp, '$1') ,d3_hsl) : fill_cur;
            trans_cur = (size[i] || size[0]) / 2;
            if(translate){
                var subTranslate = translate[i] || translate[0];
                trans_cur += parseFloat(subTranslate);
            }
                
        patternNode
        .append('path')
        .attr({
            'transform': 'translate(' + trans_cur + ',' + trans_cur + ')',
            'd': pattern_path,
            'stroke': stroke_cur,
            'fill' : fill_cur,            
            'stroke-width': strokeWidth[i] || strokeWidth[0],
            'stroke-opacity': strokeOpacity[i] || strokeOpacity[0],
            'fill-opacity' : fillOpacity[i] || fillOpacity[0]
        });
    }
    

    return {"node" : patternNode, "w":size[0], "h":size[0], "id":patternID};
}

function getSVGImage(patternEle,base64){
    var xml = "<svg xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' width='" + patternEle.w + "' height='" + patternEle.h + "'>" + new XMLSerializer().serializeToString(patternEle.node.node()) + "<path d='" + "M-5,-5 L5,-5 L5,5 L-5,5 Z" + "' transform='translate(" + (patternEle.w / 2) + "," + (patternEle.h / 2) + ")' style='fill: " + "url(#" + patternEle.id + ")'></path> </svg>";
    return {
        url :  ($Browser.FIREFOX || base64) ? ('data:image/svg+xml;base64,' + btoa(xml)) : ('data:image/svg+xml;utf8,' + xml),
        width : patternEle.w,
        height : patternEle.h
    };
}
