/**
 *
 * @param {Number} type chart type
 * @return {boolean}
 */
function isAreaChartType(type) {
    // return [$ZC.charttype.area, $ZC.charttype.stkdarea, $ZC.charttype.arearange].indexOf(type) > -1;
    return type === $ZC.charttype.area || type === $ZC.charttype.stkdarea || type === $ZC.charttype.arearange;
    
}

/**
 *
 * @param {Number} type chart type
 * @return {boolean}
 */

function isBarChartType(type) {
    return type === $ZC.charttype.bar || type === $ZC.charttype.stkdbar || type === $ZC.charttype.bullet || type === $ZC.charttype.gantt || isHeatMapChartType(type);
}

/**
 *
 * @param {Number} type chart type
 * @return {boolean}
 */

function isScatterChartType(type) {
    return $ZC.chartcategory.dotPlot.indexOf(type) != -1;
}


/**
 *
 * @param {Number} type chart type
 * @return {boolean}
 */

function isBubbleChartType(type) {
    return type === $ZC.charttype.bubble || type === $ZC.charttype.bubblepie;
}

/**
 *
 * @param {Number} type chart type
 * @return {boolean}
 */

function isPointChartType(type) {
    return isScatterChartType(type) || type === $ZC.charttype.bubblepie;
}

function  isSunburstChartType(type) {
    return type === $ZC.charttype.sunburst;
}

function  isHeatMapChartType(type) {
    return type === $ZC.charttype.heatmap;
}

function  isPieChartType(type) {
    return $ZC.chartcategory.piePlot.indexOf(type) != -1;
}

/**
 *
 * @param node
 * @return {Array}
 */
function getMousePosition(node, event) {
    // For touch devices d3.mouse(node) will return incorrect values, if the document is scrolled. So that pageXOffset and pageYOffset is added.
    var mouse = d3_mouse(node);
    if (event.touches != null && !$Browser.ANDROID && $Browser.uiwebview) { //d3.mouse(node) considers pageXOffset and pageYOffset in android devices
        mouse[0] += window.pageXOffset;
        mouse[1] += window.pageYOffset;
    }
    return mouse;
}

/**
 *
 * @param {*} a
 * @param {*} b
 * @return {boolean}
 */

function equals(a, b) {
    return a === b;
}

/**
 *
 * @param {Object } chart current chart Object
 * @param {Array} point data point
 * @param seriesIndex series index of the corresponding data
 * @param commonOption commonOption for chart support
 * @return {{x, y, width, height}}
 */

function getBarPosition(chart, point, seriesIndex, commonOption) {
    var getTransRectValue = function() {
        return chart.dataObject.getTransRectValue.apply(chart.dataObject, toArray(arguments));
    };
    return {
        x: getTransRectValue("x", point, seriesIndex, 0, commonOption),
        y: getTransRectValue("y", point, seriesIndex, 0, commonOption),
        width: getTransRectValue("width", point, seriesIndex, 0, commonOption),
        height: getTransRectValue("height", point, seriesIndex, 0, commonOption)
    }
}

/**
 *
 * @param {Object} chartObj
 * @param {Array} dataPoint
 * @param {Scale} xscale
 * @param {Scale} yscale
 * @param {Number} yaxiscolumnorder
 * @return {{x, y, width, height}}
 */

function getHeatCellPosition(chartObj, dataPoint, xscale, yscale, yaxiscolumnorder) {
    var dataset = chartObj.dataset,
        commonPlotOptions = chartObj.systemConf.chart.plot.plotoptions.heatmap,
        symbolPath = commonPlotOptions.symbol && getSVGsymbol(commonPlotOptions.symbol),
        xValue = dataset.getX(dataPoint),
        yValue = dataset.getY(dataPoint, yaxiscolumnorder),
        zValue = defined(dataset.zidx) ? chartObj.zscale(dataset.getZ(dataPoint) || chartObj.zscale.domain()[0]) : 1,
        xBand = xscale.rangeBand(),
        yBand = yscale.rangeBand(),
        x, y, w, h;

    if (chartObj.axes.rotated) {
        w = yBand * zValue;
        h = xBand * zValue;

        if (symbolPath) {
            w = h = mathMin(w, h);
        }

        y = xscale(xValue) + (xBand - h) / 2;
        x = yscale(yValue) + (yBand - w) / 2;
    } else {
        w = xBand * zValue;
        h = yBand * zValue;

        if (symbolPath) {
            w = h = mathMin(w, h);
        }

        x = xscale(xValue) + (xBand - w) / 2;
        y = yscale(yValue) + (yBand - h) / 2;
    }

    return {
        x: x,
        y: y,
        width: w,
        height: h
    };
}

/**
 *
 * @param {Array} dataPoint
 * @return {{x, y, width, height}}
 */

function getTreemapPosition(dataPoint) {
    return {
        x: dataPoint.x || 0,
        y: dataPoint.y || 0,
        width: dataPoint.dx,
        height: dataPoint.dy
    };
}

/**
 *
 * @param {Array} dataPoint
 * @return {{x, y, width, height}}
 */

function getPackedbubblePosition(dataPoint) {
    var w = h = 2 * dataPoint.r;
    return {
        x: (dataPoint.x || 0) - (w / 2),
        y: (dataPoint.y || 0) - (h / 2),
        width: w,
        height: h
    };
}

// get tooltip position for both ordinal and continuous scale
// TODO revamp continuous scale tooltip position.
function getTooltipPosition(event, coordinates, x, y, chartObj, point, axisrotated, sharedTooltip, sharedTooltipInfo, isOrdinalChart, isPolar) {
    var tooltipPos = null,
        plotarea = chartObj.plotarea,
        plotarea_approx = chartObj.plotarea_approx;
    
    if (sharedTooltip && crosshair_enabled()) {
        if (axisrotated) {
            tooltipPos = {
                actLeft: x + plotarea.left,
                actTop: coordinates.y + plotarea.top,
                top: coordinates.y,
                left: isOrdinalChart ? plotarea.width : coordinates.x
            }
        } else {
            // common for both
            tooltipPos = {
                top: 0,
                actTop: plotarea.top + y
            };
            
            if (isOrdinalChart) {
                tooltipPos.left = coordinates.x;
                tooltipPos.actLeft = coordinates.x + plotarea.left;
            } else {
                tooltipPos.left = x;
                tooltipPos.actLeft = plotarea.left + x;
            }
        }
    } else {
        if (isOrdinalChart) {
            // common for both
            tooltipPos = {
                left: coordinates.x,
                top: coordinates.y
            };
            
            if (axisrotated) {
                tooltipPos.actLeft = x + plotarea.left;
                tooltipPos.actTop = coordinates.y + plotarea.top;
            } else {
                tooltipPos.actLeft = coordinates.x + plotarea.left;
                tooltipPos.actTop = y + plotarea.top;
            }
        } else {
            tooltipPos = {
                left: x,
                top: y,
                actLeft: (plotarea.left + x),
                actTop: (plotarea.top + y)
            }
        }
    }
    var dLeft = coordinates.x - event.clientX,
        dTop = coordinates.y - event.clientY;
    
    tooltipPos.dLeft = dLeft + plotarea.left;
    tooltipPos.dTop = dTop + plotarea.top;
    tooltipPos.left -= dLeft;
    tooltipPos.top -= dTop;
    tooltipPos.dLeft = dLeft + plotarea.left;
    tooltipPos.dTop = dTop + plotarea.top;
    
    // add dependencies in helper variable
    
    tooltipPos.helpers = {
        "seriesindex": point.seriesPos,
        "subseriespos": isArray(point.subseriesindex) ? point.subseriesindex[point.seriesPos] : point.subseriesindex,
        "itempos": point.itemPos,
        "color": chartObj.seriesColor,
        "tooltipSharedInfo": sharedTooltipInfo,
        "tooltipShared": sharedTooltip
    };
    
    
    if (isPolar) {
        tooltipPos.dLeft += plotarea_approx.x;
        tooltipPos.dTop += plotarea_approx.y
    }
    
    return tooltipPos
}

function getColumnIdxBasedOnDataindex(chartObj, options) {
    //check for multi yaxis
    var columns = chartObj.userdata.metadata.columns,
        columnidx = columns.filter(function(d) {
            return d.dataindex === options.dataindex;
        });
    return columnidx[0];
}
