//$Id$

function getFormattedValue(colInfo, value, dataType, chartObj, type, toDecode, arraySeperator, levelIndex) {
    if (!dataType) {
        return encodeOrDecode(value);
    }
    if (value == null) {
        return (colInfo && (colInfo.nullformatting != null)) ? encodeOrDecode(invokeFunction(colInfo.nullformatting, value, dataType, colInfo, type, chartObj)) : value;
    }
    var format = colInfo ? (colInfo[type + "format"] || colInfo.format) : null;
    if ((!format || !Object.keys(format).length)) {

        format = getDefaultFormat(dataType, colInfo);
    }
    return (value.constructor == Array) ? //For partition charts, array of values will come here to format
        value.map(function(d) {
            return applyFormat(d);
        }).join(defined(format.arraySeperator) ? format.arraySeperator : arraySeperator) :
        applyFormat(value);
    
    function applyFormat(value) {
        if (dataType === TIME_DATATYPE && !format.customFormat) {
            value = generateDateVal(value, format, colInfo);
        }
        if(levelIndex && format.customFormat){
            colInfo = simpleClone(colInfo);
            colInfo.levelindex  = levelIndex;
        }
        var formattedValue = (typeof format === "function") ?
            format(value, dataType, colInfo) :
            (format.customFormat ?
                invokeFunction(format.customFormat, value, dataType, colInfo, type, chartObj) :
                generateFormatSpecifier(format, value, colInfo, dataType, levelIndex, type));
        return encodeOrDecode(formattedValue);
    }
    
    function encodeOrDecode(val) {
        return (toDecode != false) ?
            decodeHTML(val) :
            encodeHTML(val);
    }
}

function getDefaultFormat(type, colInfo) {
    var defaultFormat = {};
    switch (type) {
        case "time":
            if(colInfo && colInfo.groupby){
                defaultFormat = {}
            }else if(colInfo && colInfo.subfunction){
                defaultFormat = getTimeSubfunctionFormatter(colInfo.subfunction, colInfo.useUTC)
            }else{
                defaultFormat.specifier = "YYYY";
            }

            break;
        case "numeric":
            defaultFormat.thousandSeperator = ",";
            break;
        case "percent":
            defaultFormat.decimalPlaces = 1;
            defaultFormat.suffix = "%";
            defaultFormat.trailingZeros = false;
    }
    return defaultFormat;
}

function generateFormatSpecifier(format, value, colInfo, dataType, levelIndex,type) {
    var formattedValue;
    switch (dataType) {
        case "time": {
            // if useUTC is true use utc time formatter else use local time formatter
            var d3Format = (colInfo && colInfo.useUTC === true) ? d3_timeFormatUTC : d3_timeFormat;
            var specifier = format.specifier;

            if((defined(specifier) && specifier.constructor == Array) || (defined(colInfo) && defined(colInfo.groupby) && isArray(colInfo.groupby))){
                //For multiple line lables with the given array of specifier
                formattedValue = [];
                specifier = specifier || [null];
                if(defined(levelIndex)){
                    specifier = (defined(specifier)) ? [specifier[levelIndex]] : [null];
                }
                specifier = (isArray(specifier)) ? specifier : [specifier]
                // specifier = (defined(levelIndex)) ? [specifier[levelIndex]] : specifier; //for timegroupby with specifier index
                specifier.forEach(function (d,i) {
                    var datetimespecifier = generated3TimeSpecifier(d),
                        formatter = generateTimeFormatter(d3Format,datetimespecifier, colInfo, levelIndex);
                        formattedValue[i] = (formatter) ? formatter(value) : value;
                })
                if(type == "datalabels"){
                    formattedValue = formattedValue && formattedValue.toString();
                }
            }else{
                var datetimespecifier = generated3TimeSpecifier(specifier);
                formattedValue = (format.specifier == "auto") ?
                    value //if format specifier is auto val should be timeScale.tickFormat()(d)
                    : ((datetimespecifier != null) ? d3Format(datetimespecifier)(value) : value);
            }

            break;
        }
        case "ordinal": {
            var lc = format.letterCase;
            value += ""; //toString
            formattedValue = lc ? ((lc === "uppercase") ? value.toUpperCase() : ((lc === "lowercase") ? value.toLowerCase() : (value[0].toUpperCase() + value.substring(1).toLowerCase()))) : value;
            break;
        }
        default: {
            var currency = format.currency;
            var numericspecifier = generated3NumericSpecifier(format);
            value = (format.type == "decimal" || (format.decimalPlaces === 0)) ? parseFloat(value) : value; //Decimal places = 0
            formattedValue = numericspecifier(value);

            if (defined(format.decimalPlaces) && /(e|s)/gi.test(getd3NumericFormatterType(format))) {
                formattedValue = formattedValue.replace(/[^a-z]+/i, function() {
                    return generated3NumericSpecifier(format, "." + format.decimalPlaces + "f")(arguments[0]);
                });
            }

            if (format.signEnabled === false) { //1) null -> shows sign for -ve only 2) true -> shows sign for +ve and -ve 3) false -> no sign shown for both +ve and -ve
                formattedValue = formattedValue.substring(1, formattedValue.length);
            }
            
            if (!validateNullVal(format.trailingZeros, true)) { // Remove trailing zeros
                var unitSuffix = "";
                formattedValue = formattedValue.replace(/[a-z]+(\+|-)*(\d)*$/i, function() {
                    unitSuffix = arguments[0];
                    return "";
                });

                formattedValue = formattedValue.replace(/\.(\d)*0*$/gi, function() {
                    var matchedString = parseFloat(arguments[0]);
                    return (!matchedString) ? "" : ("." + matchedString.toString().split(".")[1]);
                }) + unitSuffix;
            }

            formattedValue = applySeperators(format, formattedValue);
            
            if (currency != null) { //Add the currency symbol
                formattedValue = /(\+|\-)/.test(formattedValue[0]) ? (formattedValue[0] + currency + formattedValue.slice(1)) : (currency + formattedValue);
            }
        }
    }
    if(dataType == ORDINAL_DATATYPE || dataType == NUMERIC_DATATYPE){ //apply prefix or suffix
        formattedValue = ((format.prefix != null) ? format.prefix : "") + formattedValue + ((format.suffix != null) ? format.suffix : ""); //Adding prefix and suffix
    }

    return formattedValue;
}

function getTimeSubfunctionFormatter(subfunctype, useUTC) {
    useUTC = useUTC === true;
    var formatter = useUTC ? d3_timeFormatUTC : d3_timeFormat;
    switch (subfunctype) {
        case "YEAR":
        case "year":
            return formatter("%Y");
        case "MONTH":
        case "month":
            return formatter("%B");
        case "ABSDAY":
            return formatter("%a");
        case "DAY":
        case "day":
            return formatter("%e");
        case "WEEKDAY":
            return formatter("%A");
        case "WEEK":
        case "week":
            return formatter("Week %W");
        case "ABSHOUR":
            return formatter("%I");
        case "ABSSECOND":
            return formatter("%S");
        case "ABSMINUTE":
        case "MINUTE":
            return formatter("%M");
        case "MILLISEC":
            return formatter("%L");
        case "HOUR":
            return formatter("%H hrs");
        case "DATETIME":
            return formatter("%d %b %Y %H:%M:%S");
        case "DATE":
            return formatter("%x");
        case "TIME":
            return formatter("%X");
        case "TIMESTAMP":
            return formatter("%x %X hrs");
        case "ABSMONTH":
            return formatter("%b %Y");
        case "ABSWEEK":
            return formatter("W" + "%U" + " " + "%Y");
        case "ABSQUARTER":
            return (function(d) {
                var month = useUTC ? d.getUTCMonth() : d.getMonth();
                var year = useUTC ? d.getUTCFullYear() : d.getFullYear();
                var quarter = mathFloor(month / 3) + 1;
                return formatter("Q" + quarter + " " + year);
            });
        case "QUARTER":
            return (function(d) {
                var month = useUTC ? d.getUTCMonth() : d.getMonth();
                var quarter = mathFloor(month / 3) + 1;
                return formatter("Q" + quarter);
            });
        case 100:
            return formatter("%d %b %Y %H:%M:%S %L");
        default:
            throw "UNKNOWN TYPE 111: " + subfunctype;
    }
}

function generateDateVal(value, format, colInfo, toProcess) {
    //var dateValue = (format != null && format.specifier != null)? d3.time.format(format.specifier).parse(value):new Date(value.toString());
    var inputformat = colInfo ? colInfo.inputformat : null;
    var dateValue = new Date(value.toString());
    var customFunc = (typeof inputformat === "function") ? inputformat :
        ((typeof window[inputformat] === "function") ? window[inputformat] : null);
    if (!(value instanceof Date) && inputformat != null && (customFunc || inputformat.toLowerCase() != "timestamp")) {
        value = (!customFunc && !(typeof value == String)) ? value.toString() : value;
        var formatter = (colInfo && colInfo.useUTC === true) ? d3_timeFormatUTC : d3_timeFormat;
        var parsedVal = customFunc ? customFunc(value, colInfo) : formatter(generated3TimeSpecifier(inputformat)).parse(value);
        dateValue = (parsedVal == null) ? dateValue : new Date(parsedVal);
    }
    if (isNaN(dateValue)) {
        dateValue = new Date(+value);
    }
    return dateValue;
}

function getFormattedValueByPercent(colInfo, value, dataType, chartObj, type) {
    colInfo = colInfo || {};
    type = ((type && colInfo[type + "percentformat"]) ? type : "") + "percent";

    if (!colInfo[type + "format"]) {
        colInfo.percentformat = getDefaultFormat("percent");
    }

    return getFormattedValue(colInfo, value, dataType, chartObj, type);
}

//Format Utils
var timeSpecifiers = {
        "YYYY": "%Y",
        "MMMM": "%B",
        "EEEE": "%A",
        "EEE": "%a",
        "MMM": "%b",
        "YY": "%y",
        "MM": "%m",
        "EE": "%U",
        "dd": "%d",
        "HH": "%H",
        "hh": "%I",
        "mm": "%M",
        "ss": "%S",
        "a": "%p",
        "MILLI": "%L",
        "z": "%Z"
    },
    numericSpecifiers = {
        "decimal" : "d",
        "exponent": "e",
        "float": "f",
        "binary": "b",
        "octal": "o",
        "hexadecimal": "x",
        "SIprefix": "s"
    };

function generated3TimeSpecifier(specifier) {
    if (specifier) {
        for (var spec in timeSpecifiers) {
            var index = specifier.indexOf(spec);
            if ((index >= 0) && (specifier[index - 1] != "%")) {
                specifier = specifier.replace(spec, timeSpecifiers[spec]);
            }
        }
    }
    return specifier;
}

function  generateTimeFormatter(d3Format, specifier, colInfo, levelIndex) {
    var formatter;
    if(specifier){
        formatter = d3Format(specifier);
    }else if(colInfo.groupby){
        var subFuncType = (levelIndex == 0 || !defined(levelIndex)) ? colInfo.subfunction : colInfo.groupby[levelIndex-1];
        formatter = getTimeSubfunctionFormatter(subFuncType, colInfo.useUTC);
    }
    return formatter;
}

function generated3NumericSpecifier(format, defaultSpecifier) {
    var sign = (defined(format.signEnabled) ? "+" : ""),
        zeroPad = ((format.zeroPadding != null && format.zeroPadding.enabled) ? ("0" + format.zeroPadding.digits) : ""),
        thousandSep = ((format.thousandSeperator != null) ? "," : ""),
        type = getd3NumericFormatterType(format),
        decPlaces = (format.decimalPlaces != null) ? (/(e|s)/gi.test(type) ? ($Browser.IE ? ".20" : "") : ("." + format.decimalPlaces)) : ""; //#ZC898
    var formatSpecifier = sign + zeroPad + thousandSep + (defaultSpecifier || (decPlaces + type));
    return d3_format(formatSpecifier);
}

function getd3NumericFormatterType(format) {
    var decimalPlaces = format.decimalPlaces,
        type = format.type;
    
    return numericSpecifiers[type] ||
        ((type === "decimal" || !defined(decimalPlaces)) ?
            "" :
            "f");
}

function applySeperators(format, formattedValue) {
    var thSep = format.thousandSeperator,
        deSep = format.decimalSeperator,
        split = formattedValue.split(/\./);
    return ((thSep != null && thSep != ",") ? split[0].replace(/,/g, thSep) : split[0]) +
        (!split[1] ? "" : (deSep != null ? deSep : ".") + split[1]);
}

/**
 * Returns absolute years, months, days till seconds from timestamp
 * @param {Number} ms date timestamp
 * @returns {{seconds: number, minute: number, hour: number, day: number, month: number, years: number}}
 */

function millisecondToAbsoluteTime(ms) {
    var seconds = mathFloor(ms / 1000),
        minute = mathFloor(seconds / ( 60)),
        hour = mathFloor(minute / (60)),
        day = mathFloor(hour / (24)),
        month = mathFloor(day / (30.436854166666667)),
        years = mathFloor(day / 365.24225);

    return {
        seconds: seconds,
        minute: minute,
        hour: hour,
        day: day,
        month: month,
        years: years
    }
}

/**
 *
 * @param milliseconds
 * @returns {{subFunction: string, specifier: *}}
 */

function getAutoTimeSubFunction(milliseconds) {

    var absoluteTime = millisecondToAbsoluteTime(milliseconds),
        subFunction = "YEAR";

    if(absoluteTime.years > 1) {
        subFunction = "YEAR";
    } else if(absoluteTime.month > 1) {
        subFunction = "ABSMONTH";
    }
    else if(absoluteTime.day > 1) {
        subFunction = "ABSDAY";
    }
    else if(absoluteTime.hour > 1) {
        subFunction = "ABSHOUR";
    }
    else if(absoluteTime.minute > 1) {
        subFunction = "ABSMINUTE";

    } else {
        subFunction = "ABSSECOND";
    }

    return {
        subFunction: subFunction
    };
}