//$Id$
/**
 *
 * @returns {chart} heatmap renderer.
 */

 //hierarchy series name on the top like fusion chart
svg_plot_renderer.heatmap = function() {
	var isredraw = false,
		commonRendererProp;

	function chart(selection) {
		selection.each(function(data) {
			//renderer config values
            var rp = commonRendererProp.params;
            var chartObj = rp.chartObj,
                xscale = rp.xscale,
                yscale = rp.yscale,
                stacked = rp.stacked,
                index = rp.datasetIndex,
                axisrotated = rp.axisrotated,
                commonPlotOptions = rp.commonPlotOptions,
                color = rp.color,
                labelQueue = rp.labelQueue,
                hasNegValue = rp.hasNegValue,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset,
                systemconf = chartObj.systemConf;

			var currentdata = data[index];
			var currentseriesdata = currentdata.data;

			//Animation variables
			var animationOptions = getAnimationConf(commonPlotOptions, null, chartObj, "heatmap");
			var isAnimationEnable = animationOptions.enabled,
				animationType = animationOptions.type,
				easingType = animationOptions.easingType,
				transitionDuration = isredraw ? (isAnimationEnable ? 200 : 0) : animationOptions.duration;

			var eventDisabled = (validateNullVal(getEventConf(null, commonPlotOptions, chartObj, "enabled"), true) == false);
			var cursor = (commonPlotOptions.events) ? commonPlotOptions.events.cursor || "pointer" : systemconf.chart.plot.events.cursor || "pointer";

			var fillColor = chart.fillColorFunc = getFillColor(chartObj, index),
				strokeColor = getStrokeColor(chartObj, index),
				borderRadius = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, commonPlotOptions.borderRadius),
				symbolPath = commonPlotOptions.symbol && getSVGsymbol(commonPlotOptions.symbol),

                xCategories = xscale.domain(),
                yCategories = yscale.domain();

			var container = d3_select(this);
			var series = appendEle(container, "g", [1], ".", "class", ('heatmapseries_' + index));
			chart.seriesdataAfterUpdate = [];

			currentseriesdata.forEach(function(d, i) {
				var serData = chart.seriesdataAfterUpdate[i] = chartObj.dataObject.updateSeriesdatabyCategories(d3_values(d), xCategories, yCategories, yaxiscolumnorder);
				var subseries = appendEle(series, "g", [1], ".", "class", ('subseries_' + i));
				var path = appendEle(subseries, "path", serData);

				path
					.style({
						"fill": function(d, i) {
							var pos = getHeatCellPosition(chartObj, d, xscale, yscale, yaxiscolumnorder);
							return fillColor(index, d, null, null, null, [pos.width, pos.height]);
						},
						"fill-opacity": commonPlotOptions.fillOpacity,
						"stroke": function(d) {
							return strokeColor(index, d, commonPlotOptions.strokeColor);
						},
						"stroke-width": commonPlotOptions.strokeWidth,
						"stroke-opacity": commonPlotOptions.strokeOpacity
					})
					.attr("transform", function(d) {
						var pos = getHeatCellPosition(chartObj, d, xscale, yscale, yaxiscolumnorder);
						return "translate(" + (pos.x + pos.width / 2) + "," + (pos.y + pos.height / 2) + ")"
					})
					.attr("d", function(d) {
						var pos = getHeatCellPosition(chartObj, d, xscale, yscale, yaxiscolumnorder);
						if (symbolPath) {
							return symbolPath(mathMin(pos.width, pos.height));
						} else {
							return roundedRect(-pos.width / 2, -pos.height / 2, pos.width, pos.height, borderRadius);
						}
					});

	            //animation
	            if (!isredraw) {
	            	cancelAnimation(path);//REFER: heatmap redraw- animation issue
		            path
		            	.attr("opacity", 0) //#ZC892
		            	.transition()
		            	.duration(transitionDuration)
		            	.ease(easingType)
		            	.each("start", function() {
		            		d3_select(this)
		            			.attr("opacity", null); //#ZC892
		            	})
						.call((animationType === "sizing") ? scaleTween : (animationType === "fade") ? fadeTween : flipTween);
	            }

            });

            if ($ZC.labelRenderer) {
            	labelQueue.push({
            		"renderer": chart,
            		"charttype": "hierarchy",
            		"chartObj": chartObj,
            		"args": [currentdata, index, null, null, null, d3_merge(chart.seriesdataAfterUpdate)]
            	});
            }

			chart.renderedTimer = setTimeout(function() {
				isredraw = true;
				chart.chartRendered = true;
			}, (transitionDuration + 17));
		});
		return chart;
	}
	chart.commonRendererProp = function(_) {
		if (!arguments.length) {
			return commonRendererProp;
		}
		commonRendererProp = _;
		return chart;
	};
	return chart;
};
