//$Id$
function getBoundBoxColor(actualColor, properties, chartObj, data, index) {
    var data_length = data.length;
    return actualColor || function(d, k) {
            var clrIndex = properties.reverseColor ? (data_length - 1 - k) : ((chartObj.isAxisCategory || chartObj.isNonAxisMultiSeriesCategory || chartObj.dataObject.isPolarAxisCategory) && !properties.multiColoring) ? index : k;
            return getRawColor(chartObj, clrIndex, ((d && d.data) ? d.data : d), index);
        };
}

function getMergedBoundBoxOptions(dataLabels, chartObj) {
    return mergeJSON(chartObj.systemConf.chart.plot.datalabels.boundBox, dataLabels.boundBox, true);
}

function isBoundBoxEnabled(dataLabels, chartObj) {
    return getValByPriority([dataLabels.boundBox || {}, chartObj.systemConf.chart.plot.datalabels.boundBox || {}], "show") === true;
}

function getBoundBoxPadding(dataLabels, chartObj) {
    return validateNullVal(getValByPriority([dataLabels.boundBox || {}, chartObj.systemConf.chart.plot.datalabels.boundBox || {}], "padding"), 2);
}

function getBoundBoxStrokeWidth(dataLabels, chartObj) {
    return validateNullVal(getValByPriority([dataLabels.boundBox || {}, chartObj.systemConf.chart.plot.datalabels.boundBox || {}], "strokeWidth"), 1);
}

function getSizeToBeAdded(defaultWidth, padding, radius, hookWidth) {
    var datalabels_min_width = (2 * (radius || 0)) + hookWidth;
    return mathMax((datalabels_min_width - defaultWidth - (2 * padding)), 0);
}

/*
 * Returns the options for boundbox hook
 * @param {Object} hookOptions
 * @return {Object} obj which contains enabled, width, height
 */
function getBoundBoxHookConf(hookOptions) {
    hookOptions = hookOptions || {};
    var hookEnabled = hookOptions.enabled === true;
    var obj = {};
    
    obj.enabled = hookEnabled;
    obj.width = hookEnabled ?
        (defined(hookOptions.width) ? +hookOptions.width : 10) :
        0;
    obj.height = hookEnabled ?
        (defined(hookOptions.height) ? +hookOptions.height : 4) :
        0;
    return obj;
}

/*
 * Returns the SVG path description for bound box
 * @param {Number} x
 * @param {Number} y
 * @param {Number} w
 * @param {Number} h
 * @param {Number} r
 * @param {Number} padding
 * @param {Number} hookPosition
 * @param {Object} hookOptions
 * @return {String} SVG path
 */
function getBoundBoxPathDescription(x, y, w, h, r, padding, hookPosition, hookOptions, adjustVal) {
    var hookEnabled = hookOptions.enabled,
        hookWidth = hookOptions.width,
        hookHeight = hookOptions.height;
    
    var path = roundedRect(x, y, w + (2 * padding), h + (2 * padding), r);
    if (hookEnabled && hookPosition) {
        var positionIndex = (hookPosition === "bottom") ? 7 :
            ((hookPosition === "top") ? 3 :
                ((hookPosition === "right") ?
                    5 : 1));
        var parsedPath = parseSVGPath(path);
        var coordinates = getHookCoordinate(hookPosition, parsedPath, positionIndex);
        addPointToSVGPathPoints(parsedPath, positionIndex, coordinates);
        
        path = createSVGPathFromPoints(parsedPath);
    }
    
    function getHookCoordinate(mode, parsedPath, positionIndex) {
        var horizontalx1 = parsedPath[positionIndex - 1].moves[(positionIndex - 1) ? 5 : 0],
            horizontalx2 = parsedPath[positionIndex].moves[0],
            verticaly1 = parsedPath[positionIndex - 1].moves[(positionIndex - 1) ? 6 : 1],
            verticaly2 = parsedPath[positionIndex].moves[1];
        var horizontalMid = (horizontalx1 + horizontalx2) / 2,
            verticalMid = (verticaly1 + verticaly2) / 2
        y = parsedPath[positionIndex].moves[1],
            x = parsedPath[positionIndex].moves[0];
        
        if (defined(adjustVal) && adjustVal) {
            var flip = (adjustVal > 0) ? 1 : -1,
                dist = (mode === "bottom") ?
                    (horizontalx1 - horizontalx2) :
                    (horizontalx2 - horizontalx1),
                adj;
            
            if ((((dist - hookWidth) / 2) - (adjustVal * flip)) > 0) {
                adj = adjustVal;
            } else {
                adj = (dist - hookWidth) / 2 * flip;
            }
            horizontalMid -= adj;
        }
        
        switch (mode) {
            case "bottom":
                return [
                    [(horizontalMid - (hookWidth / 2)), y],
                    [horizontalMid, (y + hookHeight)],
                    [(horizontalMid + (hookWidth / 2)), y]
                ];
            case "top":
                return [
                    [(horizontalMid + (hookWidth / 2)), y],
                    [horizontalMid, (y - hookHeight)],
                    [(horizontalMid - (hookWidth / 2)), y]
                ];
            case "left":
                return [
                    [x, (verticalMid - (hookWidth / 2))],
                    [(x - hookHeight), verticalMid],
                    [x, (verticalMid + (hookWidth / 2))]
                ];
            case "right":
                return [
                    [x, (verticalMid + (hookWidth / 2))],
                    [(x + hookHeight), verticalMid],
                    [x, (verticalMid - (hookWidth / 2))]
                ];
        }
    }
    
    return path;
}

/*
 * Generate and return SVG path from the given array of points
 * @param {Array of points} points
 * Each point should be,
 * {
 *      "function" : OPERATOR, (e.g) : "M" | "L" | "A" etc
 *      "moves" : [point1, point2, ...pointN]
 * }
 * @return {String} SVG path
 */
function createSVGPathFromPoints(points) {
    return points.reduce(function(prev, cur) {
        return prev + cur.function.toUpperCase() + ((cur.moves && cur.moves.join(" ")) || "");
    }, "");
}

/*
 * Add all the LineTo coordinates at the given index
 * @param {Array} array
 * @param {Number} index
 * @param {Array} coordinate [[x1, y1], [x2, y2], ...[xN, yN]]
 */
function addPointToSVGPathPoints(array, index, coordinates) {
    coordinates.forEach(function(d) {
        array.splice(index, 0, {
            "function": "l",
            "moves": d
        });
    });
}