//$Id$
datalabelsBoundBox_handler.canvas = function() {
    var series;
    
    function shape(selection, index) {
        selection.each(function() {
            if (!series) {
                return;
            }
            var properties = series.properties,
                chartObj = properties.rp.chartObj,
                rgba = canvasUtils.rgba;
            var context = chartObj.labelHolderContext;
            if (!defined(context)) {
                return false;
            }
            
            if (!isBoundBoxEnabled(properties.dataLabels, chartObj)) {
                return;
            }
            
            var boundboxOptions = getMergedBoundBoxOptions(properties.dataLabels, chartObj);
            var hookOptions = getBoundBoxHookConf(boundboxOptions.hook);
            var dataUpdate = properties.dataUpdate,
                rotation = properties.rotation || 0,
                padding = getBoundBoxPadding(properties.dataLabels, chartObj),
                r = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, boundboxOptions.radius),
                strokeWidth = defined(boundboxOptions.strokeWidth) ? +boundboxOptions.strokeWidth : 1,
                strokeDashArray = getDashArray(boundboxOptions.dashStyle, strokeWidth),
                strokeLineCap = getLineCap(boundboxOptions.lineCap, boundboxOptions.dashStyle),
                heightAdjustScale = d3_scaleLinear();
            
            series.forEach(function(subSeries) {
                subSeries.forEach(function(item, k) {
                    var display = item.display;
                    if (display === true) {
                        var x = item.X,
                            y = item.Y,
                            dy = (item.actualDY !== undefined) ? item.actualDY : parseFloat(item.DY) || 0,
                            h = item.height,
                            w = item.width;
                        
                        var strokeColor = getBoundBoxColor(boundboxOptions.strokeColor, properties, chartObj, dataUpdate, index);
                        if (typeof strokeColor === "function") {
                            strokeColor = strokeColor(dataUpdate && dataUpdate[k], k);
                        }
                        
                        var fillColor = getBoundBoxColor(boundboxOptions.fillColor, properties, chartObj, dataUpdate, index);
                        if (typeof fillColor === "function") {
                            fillColor = fillColor(dataUpdate && dataUpdate[k], k);
                        }
                        
                        context.save();
                        if (defined(rotation) && rotation !== 0) {
                            var tx = 0,
                                ty = 0;
                            if (chartObj.axes.rotated) {
                                tx = -1 * dy * h;
                            } else {
                                ty = ((item.ANCHOR === "middle") ?
                                        (w / 2) :
                                        ((item.ANCHOR === "end") ? -w : 0)) + padding;
                                tx = -heightAdjustScale.domain([-0.3, 0.9]).range([h, 0])(dy) - padding;
                            }
                            
                            context.translate(x + tx, y + ty);
                            context.rotate(canvasUtils.getRadianAngle(rotation));
                        } else {
                            x = ((item.ANCHOR === "middle") ?
                                    (x - (w / 2)) :
                                    ((item.ANCHOR === "end") ? (x - w) : x)) - padding;
                            y = y - heightAdjustScale.domain([-0.3, 1]).range([h, 0])(dy) - padding;
                            
                            context.translate(x, y);
                        }
                        
                        context.fillStyle = rgba(fillColor, boundboxOptions.fillOpacity);
                        context.strokeStyle = rgba(strokeColor, boundboxOptions.strokeOpacity);
                        context.lineWidth = strokeWidth;
                        if (strokeDashArray && defined(context.setLineDash)) {
                            context.setLineDash(strokeDashArray);
                            context.lineCap = strokeLineCap;
                        }
                        
                        context.beginPath();
                        context.translate(-x, -y);
                        canvasUtils.boundBoxWithHook(context, getBoundBoxPathDescription(x, y, w, h, r, padding, item.hookPosition, hookOptions, item.adjustVal));
                        context.fill();
                        context.stroke();
                        context.closePath();
                        context.restore();
                    }
                });
            });
        });
    }
    
    shape.args = function(_) {
        if (!arguments.length) {
            return series;
        }
        series = _;
        return shape;
    };
    
    return shape;
};