//$Id$
dataLabel_handler.canvas = {};
dataLabel_handler.canvas.initialize = function(chartObj) {
    
    var computed_properties = [],
        area_range_properties = [],
        stackLabel_properties = [],
        isStackedLabels = false;
    
    return function() {
        var handlers = {};
        
        handlers.add = function(c, series, subSeries, yrangePos, stackLabel) {
            if (stackLabel) {
                isStackedLabels = true;
            }
            var queue = stackLabel ? stackLabel_properties : yrangePos ? area_range_properties : computed_properties;
            queue[series] = queue[series] || [];
            queue[series][subSeries] = queue[series][subSeries] || [];
            queue[series][subSeries].push.apply(queue[series][subSeries], c);
            return queue[series];
        };
        
        handlers.update = function(id, x, y, display, DY, hookPosition) {
            var index = id.match(/\d+/g),
                seriesIndex = index[0],
                subSeriesIndex = index[1],
                itemIndex = index[2],
                yRangePos = 0;
            if (index[3]) {
                yRangePos = +index[3];
            }
            var queue = isStackedLabels ? stackLabel_properties : yRangePos ? area_range_properties : computed_properties;
            var d = queue[seriesIndex][subSeriesIndex][itemIndex];
            d.X += x;
            d.Y += y;
            d.display = d.display && !display;
            if (hookPosition) {
                d.hookPosition = hookPosition;
            }
        };
        
        handlers.getQueue = function(series, yRangePos, stackLabel) {
            var queue = stackLabel ? stackLabel_properties : (yRangePos ? area_range_properties : computed_properties);
            return queue[series];
        };
        
        handlers.draw = function() {
            var context = chartObj.labelHolderContext;
            if (!defined(context)) {
                return false;
            }
            
            var align = function(anchor) {
                switch (anchor.toLocaleLowerCase()) {
                    case "start": {
                        return "left";
                    }
                    case "middle": {
                        return "center";
                    }
                    case "end": {
                        return "right";
                    }
                        return anchor.toLocaleLowerCase();
                }
            };
            
            var drawLabel = function(series, i) {
                var properties = series.properties,
                    rotation = properties.rotation,
                    dataLabels = properties.dataLabels,
                    renderer = defined(i) ? chartObj.renderer[i] : null;
                var fontStyle = canvasUtils.getFontStyles(chartObj, properties.dataLabels, properties.color),
                    fontSize = parseFloat(fontStyle["font-size"]),
                    
                    canvasFont = fontStyle["font-size"] + " " + fontStyle["font-family"];
                series.forEach(function(subSeries, subSeriesIndex) {
                    var dataUpdate = (renderer && renderer.seriesdataAfterUpdate) ? renderer.seriesdataAfterUpdate[subSeriesIndex] : null;
                    subSeries.forEach(function(item, k) {
                        context.save();
                        var x, y, dy, height, display;
                        display = item.display;
                        if (display === true) {
                            x = item.X;
                            y = item.Y;
                            dy = parseFloat(item.DY || 0);
                            height = item.height;
                            dy = (item.tspanInfo ? fontSize : height) * dy;

                            if (defined(rotation) && rotation !== 0) {
                                if (chartObj.axes.rotated) {
                                    dy = -1 * dy
                                }
                                context.translate(x + dy, y);
                                context.rotate(canvasUtils.getRadianAngle(rotation))
                            } else {
                                context.translate(x, y + dy);
                            }

                            if (defined(item.ANCHOR)) {
                                context.textAlign = align(item.ANCHOR);
                            }

                            if (item.tspanInfo) {
                                item.tspanInfo.forEach(function(a){
                                    var userStyle = a.style,
                                        font = canvasFont,
                                        fSize,
                                        fill;

                                    if (userStyle) {
                                        font = (userStyle["font-size"] || fontStyle["font-size"]) + " " + (userStyle["font-family"] || fontStyle["font-family"])
                                        fill = userStyle.fill;
                                        fSize = parseFloat(userStyle["font-size"]);
                                    }

                                    context.translate(a.dx || 0, (fSize || fontSize) * a.dy);
                                    createText(k, a.text, font, fill, dataUpdate);
                                    context.translate(-a.dx || 0, 0);
                                });
                            } else {
                                createText(k, item.text, canvasFont, null, dataUpdate);
                            }
                            context.restore();
                        }
                    })
                })

                function createText(k, textContent, font, fill, dataUpdate) {
                    var rawColor = getRawColor(chartObj, properties.multiColoring ? k : i, dataUpdate && dataUpdate[k], i);
                    var color = getLabelFontColor(chartObj.systemConf, rawColor, dataLabels, chartObj);

                    context.font = font;
                    context.fillStyle = fill || color || fontStyle.color;
                    context.fillText(textContent || "", 0, 0);
                }
            };

            // stack labels
            var groupedByStack = chartObj.dataObject.groupedByStack;
            var keys = chartObj.userdata.seriesdata.stackGroup || groupedByStack.keys();
            keys.forEach(function(d, i) {
                if (isStackedLabels && groupedByStack.has(i)) {
                    var group = groupedByStack.get(i);
                    drawLabel(stackLabel_properties[group.stackLabelIndex], group.stackLabelIndex);
                }
            });

            // data labels
            computed_properties.forEach(function(series, i) {
                drawLabel(series, i);
                if (defined(area_range_properties[i])) {
                    drawLabel(area_range_properties[i], i);
                }
            });
            chartObj.labelHandler = null
        };
        return handlers;
    }();
};