//$Id$
function getFontSizeByHierarchy(conf, chartObj) {
    var systemconf = chartObj.systemConf;
    return +($ZC.utils.getVal(getValByPriority([conf, systemconf.chart.plot.datalabels, systemconf.chart, systemconf.canvas], "fontSize")));
}

function getFontFamilyByHierarchy(config, chartInstance) {
    var systemConf = chartInstance.systemConf;
    return $ZC.utils.getVal(getValByPriority([config, systemConf.chart, systemConf.canvas, $ZCG], "fontFamily"));
}

function getMergedDatalabelOptions(commonPlotOptions, plotoptions, chartObj, index) {
    var options = (plotoptions && plotoptions.datalabels) ?
        mergeJSON(commonPlotOptions.datalabels, plotoptions.datalabels, true) :
        commonPlotOptions.datalabels;

    if (chartObj) {
        overrideDatalabelShowAs(options, chartObj, index);
    }

    return options;
}

function applyDatalabelFonts(labelseries, dataLabels, chartObj, seriesColor, inner, seriesIndex, isStackLabel) {
    var systemconf = chartObj.systemConf;

    if(isArray(seriesColor) && defined(seriesIndex)) {
        seriesColor = seriesColor[seriesIndex];
    }

    return applyFont(labelseries, [{
        fontColor: getLabelFontColor(systemconf, seriesColor, dataLabels, chartObj, inner, seriesIndex, isStackLabel)
    }, dataLabels, systemconf.chart.plot.datalabels, systemconf.chart, systemconf.canvas, $ZCG]);
}

function getLabelFontColor(systemconf, seriesColor, datalabelOptions, chartObj, inner, seriesIndex, isStackLabel) {
    
    /**
     * @param inner is boolean, means function being call from inner-label
     */
    
    var fontColor = datalabelOptions.fontColor;
    if (typeof fontColor === "function") {
        var userColor = invokeFunction(fontColor, seriesColor, chartObj);
        if (defined(userColor)) {
            return userColor;
        }
    }

    var globalDatalabelOptions = getPropVal(chartObj.systemConf, "chart.plot.datalabels") || {},
        dataLabelColor = globalDatalabelOptions.fontColor,
        chart = chartObj.chartTypes[seriesIndex || 0],
        chartName = chart.name,
        chartNumber = chart.value,
        imagePalletUsed = defined(systemconf.legend.imagePallete),
        stacked = chartObj.dataObject.isstacked(chartNumber),
        color,
        isStackedBar = isBarChartType(chartNumber) && stacked && isStackLabel !== true,
        isHeatmap = isHeatMapChartType(chartNumber);


    // for constant color
    if (fontColor === UNDEFINED && dataLabelColor === UNDEFINED) {
        if (inner && imagePalletUsed) {
            return "#444444"; // always returns default color
        } else if ((inner && /(pie|funnel|sunburst|pyramid|packedbubble)/ig.test(chartName)) || isStackedBar || isHeatmap) {
            return getContrastColor(seriesColor);
        }
    }
    
    color = (!imagePalletUsed && ((!defined(fontColor) && dataLabelColor === null) || (fontColor === null))) ? seriesColor : fontColor;
    return color || pick(globalDatalabelOptions.fontColor, systemconf.chart.fontColor, systemconf.canvas.fontColor, $ZCG.fontColor);
}

function doOverlap(rect1, rect2) {
    return !((rect1.x1 > rect2.x2) || (rect2.x1 > rect1.x2) || (rect1.y1 > rect2.y2) || (rect2.y1 > rect1.y2));
}

function isLabelWithinBound(labelbound, plotbound) {
    return !((labelbound.x1 < plotbound.x1) || (labelbound.y1 < plotbound.y1) || (labelbound.x2 > plotbound.x2) || (labelbound.y2 > plotbound.y2));
}

function getDataLabelsSize(type, serData, chartObj) {
    if (chartObj.dataObject.hasLevelMarker || !serData) {
        return 0;
    } // serData = undefined -> On legend filter, multi yaxes chart (numeric axis + ordinal axis)
    var fontSizeArray = [],
        systemconf = chartObj.systemConf,
        axisrotated = chartObj.dataObject.getAxisRotated(),
        dim = (axisrotated ? chartObj.plotarea.width : chartObj.plotarea.height) * 0.75,
        plotLevel = systemconf.chart.plot.datalabels,
        chartLevel = systemconf.chart,
        canvasLevel = systemconf.canvas;
    var dummyText = chartObj.staticPaneSVG.selectAll("g.labelholder").selectAll("text.dummyText").data([1]);
    dummyText.enter().append("text").attr("class", "dummyText").text("qtyplkjhgfd").style("fill", "none");
    serData.forEach(function(series, i) {
        var curSeriesdata = ((type == "x") ? series : chartObj.seriesdata[series.si]);
        if (!curSeriesdata.disabled) {
            var plotoptions = curSeriesdata.plotoptions,
                cType = chartObj.chartTypes[(type === "x" ? i : series.si)].value,
                commonPlotOptions = systemconf.chart.plot.plotoptions[$ZC.charttypenames.get(cType)];
            
            var dataLabels = getMergedDatalabelOptions(commonPlotOptions, plotoptions);
            
            var showlabel = isDatalabelEnabled(dataLabels, chartObj),
                padding = mathMin(+(dataLabels.padding || 0), dim);
            
            if (showlabel) {
                var fontSize = +($ZC.utils.getVal(getValByPriority([dataLabels, plotLevel, chartLevel, canvasLevel], "fontSize")));
                dummyText.style("font-size", fontSize + "px");
                fontSize = bbox(dummyText.node()).height;
                fontSize += ((type == "y") ? mathMax(0, padding) : 0) +
                    ((plotLevel.handleOverlapping && !axisrotated && type == "y") ? fontSize : 0);
                fontSizeArray.push(fontSize);
            }
        }
    });
    
    if ((type == "y") && chartObj.dataObject.noofBarEnabledSeries && chartObj.dataObject.isstacked() && !chartObj.dataObject.isStackedPercentChart) {
        var stackedLabels = systemconf.chart.plot.plotoptions.bar.datalabels.stackLabels;
        var showStackLabels = stackedLabels ? stackedLabels.show: false;
        
        
        if (showStackLabels) {
            var padding = mathMin(+(stackedLabels.padding || 0), dim);
            var fontSize = +($ZC.utils.getVal(getValByPriority([stackedLabels, plotLevel, chartLevel, canvasLevel], "fontSize")));
            dummyText.style("font-size", fontSize + "px");
            fontSize = bbox(dummyText.node()).height + mathMax(0, padding);
            fontSizeArray.push(fontSize);
        }
    }
    dummyText.remove();
    var maxFontSize = (fontSizeArray.length > 0) ? mathMax.apply(null, fontSizeArray) : 0;
    return maxFontSize;
}

function dataLabelTextContent(chartObj, rp, conf, type, total, seriespos, yrangePos) {
    var yColumn = chartObj.seriesdata[seriespos].yaxiscolumnorder || [0, 0]; //We need the yaxis order and column order. But, rp.yaxiscolumnorder - returns only the yaxis order
    var chartType = chartObj.chartTypes[seriespos],
        dataProcessorHelpers = $ZC.DataProcessor.helpers;
    rp = rp || {
            stacked: false,
            commonPlotOptions: chartObj.systemConf.chart.plot.plotoptions[chartType.name],
            yaxiscolumnorder: dataProcessorHelpers.getParsedYAxisOrder(yColumn)
        };
    var dataset = chartObj.dataset,
        yaxiscolumnorder = rp.yaxiscolumnorder,
        stacked = rp.stacked,
        showAs = conf.showAs,
        columns = chartObj.systemConf.metadata.columns,
        haslabelidx = type == "datalabels" && dataset.labelidx != null,
        isBubblePie = (chartType.value === $ZC.charttype.bubblepie),
        piedataindex = validateNullVal(rp.commonPlotOptions.dataindex, 3);
    var labelidx = (haslabelidx && columns[dataset.labelidx] != null);
    showAs = (haslabelidx && columns[dataset.labelidx] == null) ? dataset.labelidx : showAs;
    var axisType = labelidx ? "label" : ((showAs == "x") ? "x" : ((showAs == "z" && dataset.zidx != null) ? "z" : "y"));
    var datatype = columns[dataset.getColumnIdx(axisType, yColumn)].datatype;
    var formatConf = dataset.getColumn(axisType, yColumn)[datatype];
    var formattingFunc = (!labelidx && (showAs == "percent" || showAs === "splitpercent")) ? getFormattedValueByPercent : getFormattedValue;
    var isShowAsDefined = ["x" , "y", "z", "percent", "splitpercent"].indexOf(showAs) != -1,
        newShowAs;

    if (!labelidx && !isShowAsDefined) {
        newShowAs  =  getTspanTemplate(showAs);
    }

    var textContent = function(d) {
        if (labelidx) {
            var value = dataset.getDataLabelValue(d);
            return formattingFunc(formatConf, value, datatype, chartObj, "datalabels");
        } else if (isShowAsDefined) {
            var value = null;
            switch (showAs) {
                case "percent" :
                case "splitpercent" :
                    var t = total,
                        v = (type == "stackLabels") ? d : (stacked ? d.y : (isBubblePie ? d[piedataindex] : (d.value || dataset.getY(d, yaxiscolumnorder, yrangePos))));

                    if (stacked && !chartObj.dataObject.isStackedPercentChart && type == "datalabels") {
                        t = getStackedTotal(chartObj, seriespos, d, showAs);
                    } else if (typeof total === "function") {
                        t = total(d);
                    }

                    t *= 0.01;
                    value = v / t;
                    value = value || 0//if t=0 and v=0, the 'value' holds NaN
                    break;
                case "x" :
                    value = dataset.getX(d);
                    break;
                case "z" :
                    if (dataset.zidx != null) {
                        value = dataset.getZ(d);
                    }
                    break;
                default :
                    value = (type == "stackLabels") ? d : (chartObj.dataObject.isHierarchy ? (d.value || dataset.getY(d, yaxiscolumnorder)) : dataset.getY(d, yaxiscolumnorder, yrangePos));
            }
            return formattingFunc(formatConf, value, datatype, chartObj, type);
        } else {
            return dataset.getCustomizedVal(chartObj, newShowAs, seriespos, null, null, d, type, null, (textContent.contentType !== "html"));
        }
    };

    //Find the content type of the data labels
    textContent.contentType = (!labelidx && !isShowAsDefined && (isArray(showAs) || showAs.indexOf("<tspan") != -1)) ? "html" : "text";
    textContent.multiLine = (textContent.contentType === "html") && isArray(showAs) && (showAs.length > 1);
    return textContent;
}

function isDatalabelEnabled(dataLabels, chartObj) {

    if(datacount_based_intelligence(chartObj, "datalabels")) {
        return defined(chartObj.dataset.labelidx) || getValByPriority([dataLabels, chartObj.systemConf.chart.plot.datalabels], "show") === true;
    }
    return false;
    // return datacount_based_intelligence(chartObj, "datalabels") ?
    //     (defined(chartObj.dataset.labelidx) || getValByPriority([dataLabels, chartObj.systemConf.chart.plot.datalabels], "show") === true) : false;
}

function isStacklabelEnabled(stackLabels, chartObj) {
    return chartObj.dataObject.noofBarEnabledSeries && chartObj.dataObject.isstacked() && !chartObj.dataObject.isStackedPercentChart && stackLabels && stackLabels.show;
}

function isShowAsPercentage(chartName, chartObj) {
    var datalabelOptions = chartObj.systemConf.chart.plot.plotoptions[chartName].datalabels || {},
        userDatalabelsOptions = isObjectPropDefined(chartObj.userdata, "chart.plot.plotoptions."+chartName+".datalabels", true) || {},
        showAs = userDatalabelsOptions.showAs || datalabelOptions.showAs;
    return showAs === "percent" || showAs === "splitpercent" || PERCENT_PATTERN.test(showAs);
}

function callTrimText(labelText, chartObj, tranform, fontSize, edge) {
    var dx = +tranform[0],
        dy = +tranform[1],
        plotarea = chartObj.plotarea,
        margin = {
            "left": -fontSize,
            "right": -fontSize,
            "top": 0,
            "bottom": 0
        },
        pattern = "..",
        orient = "bottom";
    
    edge = edge || {
            "left": -dx,
            "top": -dy,
            "right": plotarea.width - dx,
            "bottom": plotarea.height - dy
        };
    labelText
        .each(function() {
            var bb = bbox(this);
            var textBound = {
                "left": bb.x,
                "top": bb.y,
                "width": bb.width,
                "height": bb.height
            };
            trimText(this, this.textContent, textBound, edge, orient, pattern, margin);
        });
}

function overrideDatalabelShowAs(options, chartObj, index) {
    /* If defaulttheme.showAs ='y' && userdata.showAs = ['<tspan>{{val(0)}}</tspan>', '<tspan style="fill:red">{{val(1)}}</tspan>'], merged sysconf will have "y" only.
     * The following will update the userdata's showAs if presents.
     */
    var chartName = chartObj.chartTypes[index].name;
    var userDatalabels = isObjectPropDefined(chartObj.userdata, "chart.plot.plotoptions."+chartName+".datalabels", true) || {},
        userInnerLabel,
        userParentLabel,
        userStackLabel,
        systemConfInnerLabel,
        systemConfParentLabel,
        systemConfStackLabel;

    if (userDatalabels.showAs) {
        options.showAs = userDatalabels.showAs || options.showAs;
    }

    if (userInnerLabel = userDatalabels.innerLabel) {
        systemConfInnerLabel = options.innerLabel;
        systemConfInnerLabel.showAs = userInnerLabel.showAs || systemConfInnerLabel.showAs;

        if (userParentLabel = userInnerLabel.parentLabel) {
            systemConfParentLabel = options.innerLabel.parentLabel;
            systemConfParentLabel.showAs = userParentLabel.showAs || systemConfParentLabel.showAs;
        }
    } else if (userStackLabel = userDatalabels.stackLabels) {
        systemConfStackLabel = options.stackLabels;
        systemConfStackLabel.showAs = userStackLabel.showAs || systemConfStackLabel.showAs;
    }
}
