//$Id$
$ZC.labelRenderer = function() {};
$ZC.labelRenderer.drawLabelsFromQueue = function(labelQueue, array, chartObj) {
    chartObj.labelHandler = null;
    if (chartObj.plot.renderer.mode !== "image") {
        var dataObj = chartObj.dataObject;

        if (chartObj.isAxisCategory) {
            var drawStackLabel = function() {
                var groupedByStack = dataObj.groupedByStack;
                groupedByStack.keys().forEach(function(d, i) {
                    var value = groupedByStack.get(d);
                    var si = value.stackLabelIndex;
                    if ((!dataObj.noofBarEnabledSeries && !dataObj.isCombinationalChart) || (si == null)) {
                        return;
                    }

                    var obj = {
                        renderer: chartObj.renderer[si],
                        chartObj: chartObj,
                        args: [{
                            data: [value.stackExtent]
                        }, si, "stackLabels", i],
                        charttype: "bar"
                    };
                    $ZC.labelRenderer.drawLabel(obj);
                });
            };
            drawStackLabel();
        }
        
        labelQueue.forEach(function(d, i) {
            $ZC.labelRenderer.drawLabel(d);
        });
        
        //handle overlap - start
        var currentPlotBound = getPlotBound(chartObj);
        var length = array.length,
            plotbound = {
                x1: currentPlotBound.left,
                y1: currentPlotBound.top,
                x2: currentPlotBound.right,
                y2: currentPlotBound.bottom
            };
        
        var pointTravel = function() {
            var isContinousChart = dataObj.isContinousChart;
            array = isContinousChart ? array.sort(function(a, b) {
                    return d3_ascending(a.x1, b.x1);
                }) : array;
            for (var i = 1; i < length; i++) {
                var bool = 0,
                    overlap = false,
                    testCase = 0,
                    radius = (array[i].pointRadius || 0),
                    x1 = array[i].x1,
                    y1 = array[i].y1,
                    x2 = array[i].x2,
                    y2 = array[i].y2,
                    w = (array[i].x2 - array[i].x1),
                    h = (array[i].y2 - array[i].y1),
                    previousLabels = array.slice(0, i),
                    currentLabel,
                    isHidden,
                    newHookPosition;
                previousLabels = isContinousChart ? previousLabels.reverse() : previousLabels;
                do {
                    //testCase 0 : actual position, testCase 1 : top, testCase 2 : right, testCase 3 : bottom, testCase 4 : left
                    switch (testCase) {
                        case 0:
                            currentLabel = array[i];
                            isHidden = (currentLabel.overlap === "hidden") || dataObj.isBubbleEnabled;
                            break;
                        case 1:
                            currentLabel.y1 = (y1 - h / 2),
                                currentLabel.y2 = (y2 - h / 2);
                            newHookPosition = "bottom";
                            break;
                        case 2:
                            currentLabel.x1 = (((x1 + x2) / 2) + radius),
                                currentLabel.x2 = (((x1 + x2) / 2) + radius + w),
                                currentLabel.y1 = (y2 + 1),
                                currentLabel.y2 = (y2 + h + 1);
                            newHookPosition = "left";
                            break;
                        case 3:
                            currentLabel.x1 = x1,
                                currentLabel.x2 = x2,
                                currentLabel.y1 = (y2 + (2 * (radius + 1))),
                                currentLabel.y2 = (y2 + (2 * (radius + 1)) + h);
                            newHookPosition = "top";
                            break;
                        case 4:
                            currentLabel.x1 = (((x1 + x2) / 2) - radius - w),
                                currentLabel.x2 = (((x1 + x2) / 2) - radius),
                                currentLabel.y1 = (y2 + 1),
                                currentLabel.y2 = (y2 + h + 1);
                            newHookPosition = "right";
                            break;
                    }
                    for (var index = i - 1; index >= 0; index--) {
                        overlap = doOverlap(previousLabels[index], currentLabel);
                        if (overlap) {
                            bool++;
                            break;
                        }
                    }
                } while ((++testCase < 5) && overlap);
                if (bool != 0) {
                    var isWithInPlot = dataObj.isPolarAxisCategory ? true : isLabelWithinBound(currentLabel, plotbound), //For web chart, within plot boundary check is not necessary
                        translateFunc = array[i].translateFunction,
                        id = array[i].id;
                    if (!isWithInPlot || bool == 5 || isHidden) {
                        translateFunc(id, null, null, true);
                        array[i].x1 = array[i].y1 = array[i].x2 = array[i].y2 = 0;
                    } else {
                        var translateX = (currentLabel.x1 - x1),
                            translateY = (currentLabel.y1 - y1);
                        translateFunc(id, translateX, translateY, null, undefined, newHookPosition);
                    }
                }
            }
        };
        
        var stepTravel = function() {
            var axisrotated = dataObj.getAxisRotated();
            array.sort(function(a, b) {
                return axisrotated ? d3_descending(a.y1, b.y1) : d3_ascending(a.x1, b.x1);
            });
            for (var i = 0; i < length; i++) {
                var bool = false,
                    overlappingCount = 0,
                    translateValue = 0,
                    x = array[i].x1,
                    y = array[i].y1,
                    currentLabel = array[i],
                    previousLabels = (array.slice(0, i)).sort(function(a, b) {
                        return axisrotated ? d3_descending(a.x1, b.x1) : d3_ascending(a.y1, b.y1);
                    });
                var isHidden = (currentLabel.overlap === "hidden");
                for (var index = i - 1; index >= 0 && overlappingCount < 2; index--) {
                    var overlap = doOverlap(previousLabels[index], currentLabel);
                    if (overlap) {
                        if (axisrotated) {
                            translateValue = previousLabels[index].x2 - currentLabel.x1;
                            currentLabel.x2 += translateValue;
                            currentLabel.x1 += translateValue;
                        } else {
                            translateValue = previousLabels[index].y1 - currentLabel.y2;
                            currentLabel.y2 += translateValue;
                            currentLabel.y1 += translateValue;
                        }
                        overlappingCount++;
                    }
                    bool = bool || overlap;
                }
                if (bool) {
                    var isWithInPlot = isLabelWithinBound(currentLabel, plotbound),
                        translateFunc = array[i].translateFunction,
                        id = array[i].id;
                    if (!isWithInPlot || overlappingCount > 1 || isHidden) {
                        translateFunc(id, null, null, true);
                        array[i].x1 = array[i].y1 = array[i].x2 = array[i].y2 = 0;
                    } else {
                        var translateX = (currentLabel.x1 - x),
                            translateY = (currentLabel.y1 - y);
                        translateFunc(id, translateX, translateY);
                    }
                }
            }
        };
        
        var slotTravel = function() {
            if (!length) {
                return;
            }
            var rightHalf = [],
                leftHalf = [],
                TOP = array.TOP,
                BOTTOM = array.BOTTOM,
                rightCount = 0,
                leftCount = 0,
                prevLeft, prevRight,
                rightSlot = [],
                leftSlot = [],
                rightCo = 0,
                leftCo = 0,
                totalRightSpaceNeeded = 0,
                totalLeftSpaceNeeded = 0,
                MARGIN = 5,
                slotCount;
            
            array.forEach(function(d) {
                var isLeft = d.isLeft;
                if (isLeft) { // Collect left half labels
                    var added = addToHalf(d, slotCount, TOP, BOTTOM, prevLeft, leftHalf, leftCount, totalLeftSpaceNeeded, true);
                    slotCount = added.totalslot,
                        prevLeft = added.prev,
                        leftCount = added.count,
                        totalLeftSpaceNeeded = added.need;
                } else { // Collect right half labels
                    var added = addToHalf(d, slotCount, TOP, BOTTOM, prevRight, rightHalf, rightCount, totalRightSpaceNeeded);
                    slotCount = added.totalslot,
                        prevRight = added.prev,
                        rightCount = added.count,
                        totalRightSpaceNeeded = added.need;
                }
            });
            
            if (totalRightSpaceNeeded || totalLeftSpaceNeeded) {
                if (array.actualType === "funnel") {
                    rightHalf.reverse();
                }
                rightCo = addToSlot(rightHalf, rightCo, rightSlot, TOP);
                leftHalf.reverse();
                leftCo = addToSlot(leftHalf, leftCo, leftSlot, TOP);
                
                arrange(rightSlot, TOP, slotCount, rightCo);
                arrange(leftSlot, TOP, slotCount, leftCo);
                
                array.forEach(function(d) {
                    chartObj.labelHandler.update(d.id, d.DX, (d.translate || 0), d.hide, undefined, d.overlappingHookPosition);
                })
                
                labelQueue.forEach(function(d) {
                    var index = d.args[1];
                    var queue = chartObj.labelHandler.getQueue(index);
                    if (queue && queue.properties && queue.properties.onOverlap) {
                        queue.properties.onOverlap();
                    }
                });
            } else {
                labelQueue.forEach(function(d, i) {
                    var index = d.args[1];
                    var queue = chartObj.labelHandler.getQueue(index);
                    if (queue && queue.properties && queue.properties.onNonOverlap) {
                        queue.properties.onNonOverlap();
                    }
                });
                array.forEach(function(d) {
                    chartObj.labelHandler.update(d.id, 0, 0, null, d.actualDY);
                });
            }
        };
        
        ((dataObj.isCombinationalChart || dataObj.isContinousChart || dataObj.isPolarAxisCategory) ?
            pointTravel :
            ((dataObj.noofBarEnabledSeries) ?
                stepTravel :
                slotTravel))();
        //handle overlap - end
        
        // remove the temp node.
        chartObj.staticPaneSVG.select(".temp_label").remove();
        
        if (chartObj.labelHandler) {
            var labelHolder = chartObj.staticPaneSVG.selectAll("g.labelholder"),
                mode = chartObj.plot.renderer.mode;
            labelQueue.forEach(function(d) {
                var index = d.args[1],
                    yRangePos = dataObj.isAxisCategory ? d.args[3] : null,
                    innerLabels = /(pieInnerLabels|funnelInnerLabels)/.test(d.charttype);
                var queue = chartObj.labelHandler.getQueue(index, yRangePos, null, innerLabels);
                var datalabelsBoundBox_renderer = datalabelsBoundBox_handler[mode]()
                    .args(queue);
                labelHolder
                    .call(datalabelsBoundBox_renderer, index, innerLabels);
            });
            chartObj.labelHandler.draw();
        }
        
    }
};

$ZC.labelRenderer.drawLabel = function(obj) {
    var chartObj = obj.chartObj,
        renderer = obj.renderer,
        charttype = obj.charttype,
        args = obj.args;
    var labelHolder = chartObj.staticPaneSVG.selectAll("g.labelholder");
    var handler = chartObj.labelHandler;
    
    if (!chartObj.labelHandler) {
        var mode = chartObj.plot.renderer ? chartObj.plot.renderer.mode : "svg";
        handler = chartObj.labelHandler = dataLabel_handler[mode].initialize(chartObj);
        handler.chartType = charttype;
    }
    var labelrenderer = $ZC.labelRenderer[charttype]();
    
    labelrenderer
        .renderer(renderer)
        .args(args);
    labelHolder
        .call(labelrenderer, handler);
    
};

function getPlotBound(chartObj) {
    var l = (-chartObj.scrolledX || 0),
        t = (-chartObj.scrolledY || 0);
    return {
        left: l,
        top: t,
        right: getPlotSize(chartObj, "width"),
        bottom: getPlotSize(chartObj, "height")
    };
}