//$Id$
datalabelsBoundBox_handler.svg = function() {
    var series,
        properties,
        chartObj;
    
    function shape(selection, index, innerLabels) {
        selection.each(function() {
            
            if (series) {
                properties = series.properties;
                chartObj = properties.rp.chartObj;
            }
            var showBoundBox = series && isBoundBoxEnabled(properties.dataLabels, chartObj);
            
            var container = d3_select(this);
            var boundboxSeries = appendEle(container, "g", [1], ".", "class", ((innerLabels ? "innerboundboxseries_" : "boundboxseries_") + index));
            if (properties && properties.rangeChart) {
                boundboxSeries = appendEle(boundboxSeries, "g", [1], ".", "class", ('rangePos_' + properties.yrangePos));
            }
            var boundboxSubSeries = appendEle(boundboxSeries, "g", (showBoundBox ? series : []));
            
            if (!showBoundBox) {
                return;
            }
            
            var seriesTranslation = chartObj.dataObject.isPolarAxisCategory ?
                "translate(" + chartObj.plotarea_approx.x + "," + chartObj.plotarea_approx.y + ")" :
                (!chartObj.isAxisCategory ?
                    properties.translation :
                    null);
            boundboxSeries
                .attr("transform", seriesTranslation);
            
            var boundboxOptions = getMergedBoundBoxOptions(properties.dataLabels, chartObj);
            var hookOptions = getBoundBoxHookConf(boundboxOptions.hook);
            var globalDataUpdate = properties.dataUpdate,
                rotation = properties.rotation || 0,
                padding = getBoundBoxPadding(properties.dataLabels, chartObj),
                r = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, boundboxOptions.radius),
                strokeColor = getBoundBoxColor(boundboxOptions.strokeColor, properties, chartObj, globalDataUpdate, index),
                fillColor = getBoundBoxColor(boundboxOptions.fillColor, properties, chartObj, globalDataUpdate, index),
                strokeWidth = defined(boundboxOptions.strokeWidth) ? +boundboxOptions.strokeWidth : 1,
                strokeDashArray = getDashArray(boundboxOptions.dashStyle, strokeWidth),
                strokeLineCap = getLineCap(boundboxOptions.lineCap, boundboxOptions.dashStyle),
                heightAdjustScale = d3_scaleLinear(),
                plotarea = chartObj.plotarea;
            
            var trans = properties.trans || [0, 0];
            var edge = {
                "left": -trans[0] + plotarea.left,
                "right": plotarea.width - trans[0] - plotarea.left
            };
            
            boundboxSubSeries
                .attr("class", function(d, i) {
                    return "subseries_" + i;
                })
                .each(function(d, i) {
                    var dataUpdate = d.dataUpdate || globalDataUpdate;
                    var boundbox = appendEle(d3.select(this), "path", dataUpdate);
                    boundbox
                        .attr({
                            "d": function(dt, j) {
                                var point = d[j];
                                var x = point.X - padding,
                                    y = point.Y - padding,
                                    w = point.width,
                                    h = point.height,
                                    dy = (point.actualDY !== undefined) ? point.actualDY : parseFloat(point.DY) || 0,
                                    anchor = point.ANCHOR,
                                    hookPosition = point.hookPosition;
                                x = (anchor === "middle") ?
                                    (x - (w / 2)) :
                                    ((anchor === "end") ? (x - w) : x);
                                y = y - heightAdjustScale.domain([-0.3, 0.9]).range([h, 0])(dy);
                                
                                if (!chartObj.isAxisCategory && !chartObj.dataObject.isPolarAxisCategory) {
                                    if (x < edge.left) {
                                        w -= (edge.left - x + (strokeWidth / 2) + padding);
                                        x = edge.left + (strokeWidth / 2);
                                    }
                                    if ((x + w + (strokeWidth / 2) + padding) > edge.right) {
                                        w -= (x + w - edge.right + (strokeWidth / 2) + (padding * 2));
                                    // if ((x + w) > edge.right) {
                                    //     w -= (x + w + padding - edge.right);
                                    }
                                    x += (boundboxOptions.strokeWidth || 1) * ((hookPosition === "right") ? 0.5 : (hookPosition === "left") ? -0.5 : 0);
                                }
                                return getBoundBoxPathDescription(x, y, w, h, r, padding, hookPosition, hookOptions, point.adjustVal);
                            },
                            "transform": function(dt, j) {
                                var point = d[j];
                                return "rotate(" + rotation + " " + point.X + " " + point.Y + ")";
                            },
                            "id": function(dt, j) {
                                return "boundbox" + d[j].ID;
                            }
                        })
                        .style({
                            "display": function(dt, j) {
                                return (d[j].display === false) ? "none" : null;
                            },
                            "fill": fillColor,
                            "fill-opacity": boundboxOptions.fillOpacity,
                            "stroke": strokeColor,
                            "stroke-opacity": boundboxOptions.strokeOpacity,
                            "stroke-width": boundboxOptions.strokeWidth,
                            "stroke-dasharray": strokeDashArray,
                            "stroke-linecap": strokeLineCap
                        });
                });
        });
    }
    
    shape.args = function(_) {
        if (!arguments.length) {
            return series;
        }
        series = _;
        return shape;
    };
    
    return shape;
};