;
dataLabel_handler.svg = {};
dataLabel_handler.svg.initialize = function(chartObj) {
    var computed_properties = [],
        area_range_properties = [],
        stackLabel_properties = [],
        innerLabel_properties = [];
    var container = chartObj.staticPaneSVG.select(".labelholder"),
        isStackedLabels = false,
        isPackedBubble = chartObj.chartTypes[0].value == $ZC.charttype.packedbubble,
        isHierarchy = chartObj.chartTypes[0].value == $ZC.charttype.treemap;
        // isPackedBubble = getChartType((chartObj.seriesdata[0].type || chartObj.systemConf.seriesdata.type), NUMERIC_DATATYPE) == $ZC.charttype.packedbubble;

    return function() {
        var handlers = {};
        handlers.add = function(c, series, subSeries, yrangePos, stackLabel, innerLabel) {
            if (stackLabel) {
                isStackedLabels = true;
            }
            var queue = stackLabel ? stackLabel_properties : (innerLabel ? innerLabel_properties : (yrangePos ? area_range_properties : computed_properties));
            queue[series] = queue[series] || [];
            queue[series][subSeries] = queue[series][subSeries] || [];
            queue[series][subSeries].push.apply(queue[series][subSeries], c);
            queue[series][subSeries].dataUpdate = c.dataUpdate; //Each subseries may have different dataUpdates
            return queue[series];
        };
        
        handlers.update = function(id, x, y, display, DY, hookPosition) {
            var index = id.match(/\d+/g),
                seriesIndex = index[0],
                subSeriesIndex = index[1],
                itemIndex = index[2],
                yRangePos = +index[3] || 0;
            var queue = isStackedLabels ? stackLabel_properties : yRangePos ? area_range_properties : computed_properties;
            var d = queue[seriesIndex][subSeriesIndex][itemIndex];
            d.X += x;
            d.Y += y;
            d.display = d.display && !display;
            if (DY !== undefined) {
                d.DY = DY;
            }
            if (hookPosition) {
                d.hookPosition = hookPosition;
            }
        };
        
        handlers.getQueue = function(series, yRangePos, stackLabel, innerLabel) {
            var queue = stackLabel ? stackLabel_properties : (innerLabel ? innerLabel_properties : (yRangePos ? area_range_properties : computed_properties));
            return queue[series];
        };
        
        handlers.draw = function() {
            if (chartObj.isAxisCategory) {
                var groupedByStack = chartObj.dataObject.groupedByStack;
                var keys = chartObj.userdata.seriesdata.stackGroup || groupedByStack.keys();
                keys.forEach(function(d, i) {
                    if (isStackedLabels && groupedByStack.has(i)) {
                        var group = groupedByStack.get(i);
                        drawLabel(stackLabel_properties[group.stackLabelIndex], 1, i, "stacklabel");
                    } else {
                        appendEle(container, "g", [], ".", "class", "stacklabel_" + i);
                    }
                });

                /*
                 * bar19.json - Remove the stack labels if all the bar series are disabled at legend filter
                 */
                if (!keys.length) {
                    appendEle(container, "g", [], "#", "id", "stacklabels");
                }
            }
            
            chartObj.seriesdata.forEach(function(s, i) {
                var data = computed_properties[i];
                
                drawLabel(data, s, i);
                if (defined(area_range_properties[i])) {
                    drawLabel(area_range_properties[i], s, i)
                }
                
                if (!(chartObj.isAxisCategory && !chartObj.dataObject.pieWithAxis) && !chartObj.dataObject.isPolarAxisCategory && !chartObj.dataObject.hasLevelMarker && !isPackedBubble && !isHierarchy) {
                    if (!chartObj.dataObject.pieWithAxis) {
                        drawConnector(data, s, i);
                    }
                    drawLabel(innerLabel_properties[i], s, i, "innerlabel");
                }
                
            });
            // chartObj.labelHandler = null;
        };
        return handlers;
    }();
    
    function drawLabel(series, seriesData, i, groupName) {
        groupName = groupName || "label";
        var isStackedLabels = (groupName === "stacklabel"),
            isInnerLabel = (groupName === "innerlabel");
        
        var labelseries = appendEle(container, "g", (series ? [1] : []), ".", "class", (groupName + "_" + i));
        if (!series) {
            return;
        }
        
        var properties = series.properties;
        var globalDataUpdate = properties.dataUpdate,
            dataLabels = properties.dataLabels,
            rp = properties.rp,
            rotation = properties.rotation,
            isRangeChart = properties.rangeChart,
            yrangePos = properties.yrangePos,
            reverseColor = properties.reverseColor,
            isPolar = chartObj.dataObject.isPolarAxisCategory,
            isAxisCategory = chartObj.isAxisCategory,
            clip = (dataLabels.textOverlap === "clip"),
            contentType = properties.contentType || "text",
            isMultiLine = properties.isMultiLine,
            fontSize = getFontSizeByHierarchy(dataLabels, chartObj),
            clipseries;
        var data_length = (isAxisCategory || isPolar) ? null : globalDataUpdate.length,
            isScatterType = (properties.type === "scatter"),
            hasColorScale = colorscale_helpers.isQuantileColorScale(chartObj);
        
        var dataLevelFontColor = null;
        if (hasColorScale || rp.threshold || !((isAxisCategory || isPolar || chartObj.dataObject.isNonAxisMultiSeriesCategory) && !properties.multiColoring)) {
            dataLevelFontColor = function(d, k) {
                var clrIndex = reverseColor ? (data_length - 1 - k) : ((isAxisCategory || isPolar || chartObj.dataObject.isNonAxisMultiSeriesCategory) && !properties.multiColoring) ? i : k;
                var clr = getRawColor(chartObj, clrIndex, ((d && d.data) ? d.data : d), i);
                return getLabelFontColor(chartObj.systemConf, clr, dataLabels, chartObj, isInnerLabel, i);
            }
        }
        
        if (clip) {
            var defs = ($Browser.SAFARI ? chartObj.svg : $ZCG.patternSVG).selectAll("defs");
            clipseries = appendEle(defs, "g", [1], ".", "class", "textClipPaths");
        }
        
        if (isRangeChart) {
            labelseries = appendEle(labelseries, "g", [1], ".", "class", ('rangePos_' + yrangePos));
        }
        
        if (isStackedLabels) {
            labelseries
                .attr("id", "stacklabels");
        }
        
        labelseries
            .attr({
                "text-anchor": (isScatterType ? "middle" : null),
                "transform": properties.translation
            });
        applyDatalabelFonts(labelseries, dataLabels, chartObj, rp.color, isInnerLabel, i, isStackedLabels); //change rp.color
        
        if (isPolar) {
            labelseries
                .attr("transform", "translate(" + chartObj.plotarea_approx.x + "," + chartObj.plotarea_approx.y + ")")
        }
        
        series.forEach(function(subSeries, j) {
            var dataUpdate = subSeries.dataUpdate || globalDataUpdate; //Each subseries may have different dataUpdates (Axis charts)
            var subseries = appendEle(labelseries, "g", [1], ".", "class", ("subseries_" + j));
            var labelText = appendEle(subseries, "text", dataUpdate);
            labelText[contentType](function(d, k) {
                return subSeries[k].text;
            });
            if (isMultiLine) {
                labelText
                    .each(function(d, k) {
                        translateTspan(d3.select(this), subSeries[k].X);
                    });
            }

            labelText
                .style({
                    "display": function(d, k) {
                        return (subSeries[k].display === false) ? "none" : null;
                    },
                    "fill": dataLevelFontColor,
                    "pointer-events": "none"
                })
                .attr({
                    "x": function(d, k) {
                        return subSeries[k].X;
                    },
                    "y": function(d, k) {
                        return subSeries[k].Y;
                    },
                    "dy": function(d, k) {
                        return subSeries[k].DY;
                    },
                    "text-anchor": function(d, k) {
                        return subSeries[k].ANCHOR;
                    },
                    "id": function(d, k) {
                        return subSeries[k].ID;
                    },
                    "transform": !defined(rotation) ? null : function(d, k) {
                            return "rotate(" + rotation + " " + subSeries[k].X + " " + subSeries[k].Y + ")"
                        },
                    "clip-path": !clip ? null : function(d, k) {
                            var bb = bbox(this);
                            var properties = {
                                name: "textClip",
                                id: chartObj.id,
                                x: bb.x,
                                y: bb.y,
                                width: subSeries[k].width,
                                height: bb.height
                            },
                            args = {
                                chartObj: chartObj
                            };
                            var clipID = effects.get(svg_clips, "rect", properties, args);
                            return "url(" + chartObj.defsLocation + "#" + clipID + ")";
                        }
                });
            
            if (!isAxisCategory && !isHierarchy && !isPolar && !chartObj.dataObject.hasLevelMarker && !isPackedBubble && !isStackedLabels && !isInnerLabel) {
                if (contentType === "html") {
                    var boundBoxPadding = isBoundBoxEnabled(dataLabels, chartObj) ? (getBoundBoxPadding(dataLabels, chartObj) + getBoundBoxStrokeWidth(dataLabels, chartObj)) : 0;

                    labelText
                        .each(function(d, k) {
                            if (subSeries[k].display !== false) {
                                var ele = d3.select(this);
                                var tspanInfo = getTspanBound(ele, fontSize);
                                trimAllTspan(ele, chartObj, properties.trans, properties.txtHgt, null, tspanInfo, (subSeries[k].isLeft ? boundBoxPadding : 0));

                                if (subSeries[k].ANCHOR === "middle") {
                                    var TX = (subSeries[k].width - bbox(this).width) / 2;
                                    var flip = subSeries[k].isLeft ? 1 : -1;
                                    var newX = subSeries[k].X + (TX * flip);

                                    ele.attr("x", newX);
                                    translateTspan(ele, newX);
                                }
                            }
                        });
                } else {
                    callTrimText(labelText, chartObj, properties.trans, properties.txtHgt);
                }
            }
        });
    }
    
    function drawConnector(series, seriesData, i) {
        
        var linkseries = appendEle(container, "g", (!series || series.properties.showlabel === "circular" ? [] : [1]), ".", "class", ("link_" + i));
        if (!series) {
            return;
        }
        
        var properties = series.properties,
            dataUpdate = properties.dataUpdate,
            rp = properties.rp,
            isSunburst = properties.isSunburst,
            connectorPath = properties.connectorPath;
        var lineConf = properties.dataLabels.line,
            color = rp.color,
            data_length = dataUpdate.length;
        
        linkseries
            .attr("transform", properties.translation)
            .style({
                "stroke": lineConf.strokeColor,
                "stroke-width": lineConf.strokeWidth,
                "fill": "none"
            });
        
        series.forEach(function(subSeries, j) {
            var subseries = appendEle(linkseries, "g", [1], ".", "class", ("subseries_" + j));
            var labelText = appendEle(subseries, "path", dataUpdate);
            labelText
                .style({
                    "display": function(d, k) {
                        return (subSeries[k].display === false) ? "none" : null;
                    },
                    "stroke": function(d, k) {
                        if (isSunburst || lineConf.strokeColor) {
                            return null;
                        }
                        return getRawColor(chartObj, (properties.reverseColor ? (data_length - 1 - k) : k), ((d && d.data) ? d.data : d), i);
                    }
                })
                .attr({
                    "d": connectorPath,
                    "id": function(d, k) {
                        return "link" + i + "_" + j + "_" + k;
                    }
                });
        });
    }
};