//$Id$
function addTargetLabel(targetMarkerOptions, renderer, dataUpdate) {
    var targetLabelOptions = targetMarkerOptions.label || {};
    var chartObj = this,
        dataset = this.dataset,
        plotarea = this.plotarea,
        systemconf = this.systemConf,
        commonOption = renderer.commonOption,
        rp = renderer.commonRendererProp().params,
        axisrotated = rp.axisrotated,
        yscale = rp.yscale,
        rangeBand = rp.xscale.rangeBand(),
        yaxisOrient = systemconf.chart.axes.yaxis[0].orient || ((this.axes.rotated) ? "bottom" : "left");
    
    var targetDataIndex = targetMarkerOptions.dataindex,
        showlabel = targetMarkerOptions.enabled === true && targetLabelOptions.enabled === true && defined(targetDataIndex);
    
    var targetLabelGroup = appendEle(this.staticPaneSVG.selectAll("g.threshold"), "g", (showlabel ? [1] : []), ".", "class", "targetLabel");
    targetLabelGroup
        .attr("transform", "translate(" + plotarea.left + "," + plotarea.top + ")");
    
    if (!showlabel) {
        return;
    }
    
    var markerSize = validateNullVal(targetMarkerOptions.size, (commonOption.levelMarkerWidth - (2 * validateNullVal(targetMarkerOptions.padding, commonOption.padding / 2))) / 10) * 10,
        hAlign = targetLabelOptions.hAlign,
        vAlign = targetLabelOptions.vAlign,
        padding = validateNullVal(targetLabelOptions.padding, 2),
        markerSizeAdjust = ((targetMarkerOptions.symbol === "line") ? validateNullVal(targetMarkerOptions.outerStrokeWidth, 1) : markerSize) / 2;
    
    if (axisrotated) {
        hAlign = hAlign || "center";
        vAlign = vAlign || ((yaxisOrient === "bottom") ? "top" : "bottom");
    } else {
        hAlign = hAlign || ((yaxisOrient === "left") ? "right" : "left");
        vAlign = vAlign || "center";
    }
    
    var columns = this.userdata.metadata.columns;
    var columnidx = columns.filter(function(d) {
        return d.dataindex === targetMarkerOptions.dataindex;
    });
    var targetColumn = (columnidx.length) ? columnidx[0] : columns[dataset.getColumnIdx("y", 0)];
    var datatype = targetColumn.datatype;
    var formatConf = targetColumn[datatype];

    var adjust = {
        left: 0,
        right: 0,
        top: 0,
        bottom: 0
    };
    if (axisrotated) {
        adjust.top = adjust.bottom = plotarea.adjust;
    } else {
        adjust.left = adjust.right = plotarea.adjust;
    }
    if (systemconf.chart.axes.yaxis[0].show === true) {
        adjust[yaxisOrient] = 0;
    }

    var rectBound = chartObj.svg.node().getBoundingClientRect(),
        rectBoundLeft = rectBound.left + chartObj.plotarea.left,
        rectBoundTop = rectBound.top + chartObj.plotarea.top,
        currentPlotBound = getPlotBound(chartObj),
        left = currentPlotBound.left - adjust.left,
        top = currentPlotBound.top - adjust.top,
        right = currentPlotBound.right + adjust.left + adjust.right,
        bottom = currentPlotBound.bottom + adjust.top + adjust.bottom;
    
    applyFont(targetLabelGroup, [targetLabelOptions, systemconf.chart, systemconf.canvas, $ZCG]);
    var targetLabelText = targetLabelGroup.selectAll("text").data(dataUpdate, function(d) {
        return dataset.getX(d);
    });
    targetLabelText.enter().append("text");
    targetLabelText.exit().remove();
    
    targetLabelText
        .each(function(d) {
            var targetValue = d[targetDataIndex];
            if (!defined(targetValue)) {
                return;
            }

            var xValue = dataset.getX(d),
                X, Y, DY, ANCHOR,
                TEXT = defined(targetLabelOptions.text) ?
                    dataset.getCustomizedVal(chartObj, targetLabelOptions.text, 0, null, null, d, "targetLabel") :
                    getFormattedValue(formatConf, targetValue, datatype, chartObj, "targetLabel"),
                isNeg = (targetValue < 0),
                flip = isNeg ? -1 : 1;
            var x = commonOption.bandwidth1(xValue) + (rangeBand / 2),
                y = yscale(targetValue);
            
            if (axisrotated) {
                X = y;
                Y = x;
                if (vAlign === "bottom") {
                    Y += (markerSize / 2);
                    if (hAlign === "center") {
                        DY = "1em";
                    }
                } else if (vAlign === "center") {
                    DY = "0.3em";
                } else if (vAlign === "top") {
                    Y -= (markerSize / 2);
                    if (hAlign === "center") {
                        DY = "-0.3em";
                    } else {
                        DY = "1em";
                    }
                }
                
                if (hAlign === "center") {
                    ANCHOR = "middle";
                    Y -= (vAlign === "center") ?
                        0 :
                        ((vAlign === "top") ?
                            (padding * flip) :
                            -(padding * flip));
                } else {
                    if (hAlign === "right") {
                        X += ((padding + markerSizeAdjust) * flip);
                        ANCHOR = isNeg ? "end" : null;
                    } else {
                        X -= ((padding + markerSizeAdjust) * flip);
                        ANCHOR = isNeg ? null : "end"
                    }
                }
                
            } else {
                X = x;
                Y = y;
                if (hAlign === "right") {
                    X += (markerSize / 2);
                } else if (hAlign === "center") {
                    ANCHOR = "middle";
                } else if (hAlign === "left") {
                    X -= (markerSize / 2);
                    ANCHOR = "end";
                }
                
                if (vAlign === "center") {
                    X += (hAlign === "center") ?
                        0 :
                        ((hAlign === "right") ?
                            padding :
                            -padding);
                    DY = "0.3em";
                } else if (vAlign === "top") {
                    Y -= ((padding + markerSizeAdjust) * flip);
                    DY = "-0.3em";
                } else {
                    Y += ((padding + markerSizeAdjust) * flip);
                    DY = "1em";
                }
            }
            
            d3_select(this)
                .attr({
                    "transform": "translate(" + X + "," + Y + ")",
                    "text-anchor": ANCHOR,
                    "dy": DY
                })
                .text(TEXT);

            // label should be within container
            var b = this.getBoundingClientRect(),
                leftPos = b.left - rectBoundLeft,
                rightPos = leftPos + b.width,
                topPos = b.top - rectBoundTop,
                bottomPos = topPos + b.height;

            var newX = X + ((leftPos < left)
                    ? (left - leftPos)
                    : ((rightPos > right) ? -(rightPos - right) : 0)),
                newY = Y + ((topPos < top)
                    ? (top - topPos)
                    : ((bottomPos > bottom) ? -(bottomPos - bottom) : 0));

            if (newX != X) {
                X = newX;
            }

            if (newY != Y) {
                Y = newY
            }

            d3_select(this)
             .attr("transform", "translate(" + X + "," + Y + ")");
        });
}