//$Id$
function getTspanTemplate(showAs) {
    var newShowAs = "";
    if (isArray(showAs)) {
        showAs.forEach(function(d, i) {
            newShowAs += "<tspan class='zcTspan'" + (i ? " dy='1.1em'" : "") + ">" + updateTspanTemplate(d) + "</tspan>";
        });
    } else {
        var tagIndex = showAs.indexOf("<tspan");
        if (tagIndex != -1) {
            newShowAs = "<tspan class='zcTspan'>" + updateTspanTemplate(showAs) + "</tspan>";
        } else {
            newShowAs = showAs;
        }
    }
    return newShowAs;
}

function updateTspanTemplate(showAs) {
    var str = "",
        lastIndex = 0,
        endTagLength = 8,
        t = {
            "<tspan": [],
            "</tspan>": []
        };

    showAs.replace(/<tspan|<\/tspan>/g, function() {
        t[arguments[0]].push(arguments[1]);
    });

    t["<tspan"].forEach(function(d, i) {
        if (lastIndex != d) {
            str += "<tspan>" + showAs.substring(lastIndex, d) + "</tspan>";
        }
        str += showAs.substring(d, lastIndex = t["</tspan>"][i] + endTagLength);
    });

    if (lastIndex != showAs.length) {
        str += "<tspan>" + showAs.substring(lastIndex, showAs.length) + "</tspan>";
    }

    return str;
}

function translateTspan(ele, x) {
    ele.selectAll("tspan.zcTspan")
        .attr("x", x);
}

function trimAllTspan(labelText, chartObj, tranform, fontSize, edge, b, boundboxPadding) {
    var dx = +tranform[0],
        dy = +tranform[1],
        plotarea = chartObj.plotarea,
        margin = {
            "left": -fontSize,
            "right": -fontSize,
            "top": 0,
            "bottom": 0
        },
        pattern = "..",
        orient = "bottom";
    boundboxPadding = boundboxPadding || 0;

    edge = edge || {
            "left": -dx + boundboxPadding,
            "top": -dy,
            "right": plotarea.width - dx - boundboxPadding,
            "bottom": plotarea.height - dy
        };

    labelText
        .each(function() {
            var children = this.childNodes,
                childCount = 0,
                parentBBox = bbox(this),
                parentWidth = this.getComputedTextLength();

            if ((parentBBox.x + parentWidth) > edge.right) {
                parentWidth -= ((parentBBox.x + parentWidth) - edge.right + boundboxPadding);
            }
            if (parentBBox.x < edge.left) {
                parentWidth -= (edge.left - parentBBox.x + boundboxPadding);
            }

            for (var i=0; i<children.length; i++) {
                var node = children[i],
                    c = node.childNodes,
                    totalWidth = node.getComputedTextLength(),
                    textContent = ""

                //collect all text contents
                for(var j=0; j<c.length; j++) {
                    var curTextContent = c[j].textContent;
                    textContent += curTextContent;
                    b[childCount + j].textLength = curTextContent.length;
                }

                // trim the text
                if (parentWidth < totalWidth) {
                    var bb = b[childCount].bbox;
                    var textBound = {
                        "left": bb.x,
                        "top": bb.y,
                        "width": totalWidth,
                        "height": bb.height
                    };
                    trimText(c[0], textContent, textBound, edge, orient, pattern, margin, null, null, false);

                    // distribute the text contents to the respective tspan
                    var innerHTML = c[0].innerHTML;
                    if (innerHTML === "..") {
                        innerHTML = "";
                    }

                    for(var j=0, start=0; j<c.length; j++) {
                        var curTextLength = b[childCount + j].textLength;
                        c[j].innerHTML = innerHTML.substring(start, start + curTextLength);
                        start += curTextLength;
                    }
                }
                childCount += c.length - 1;
            }

        });
}

function getTspanBound(label, fontSize) {
    var c = label.selectAll(function() {
                return this.childNodes;
            }),
        anchor = label.attr("text-anchor"),
        X = +label.attr("x"),
        Y = +label.attr("y"),
        o = [];

    c.each(function(){
        var ele = d3.select(this),
            nodeDY = parseFloat(ele.attr("dy") || 0),
            children = this.childNodes,
            totalWidth = this.getComputedTextLength(),
            oldRef = (anchor === "middle") ? (-totalWidth / 2) : (anchor == "end" ? -totalWidth : 0);

        for(var i=0; i<children.length; i++) {
            var currentWidth = children[i].getComputedTextLength();
            var info = {
                dy : i ? 0 : nodeDY,
                dx : oldRef + ((anchor === "middle") ? (currentWidth / 2) : 0),
                bbox : {x: X + oldRef,
                        y: Y + (i ? 0 : nodeDY) * fontSize,
                        width: children[i].getComputedTextLength(),
                        height: fontSize
                    },
                style : children[i].style,
                text : children[i].textContent
            }
            o.push(info);
            oldRef += currentWidth;
        }
    });
    return o;
}
