//$Id$
var legendBrush;
(function(){
    var MIN_VALUE = 0.000000000001;
	legendBrush = function(chartObj, options) {
        var position = options.position,

            dimension = legend_helpers.getColorBandDimension(chartObj),
            width = dimension[0],
            height = dimension[1],

            legendOptions = chartObj.systemConf.legend,
            colorbandOptions = legendOptions.colorBand,

            orient = legend_helpers.getOrient(legendOptions),
            datatype = chartObj.dataset.getDataType("clr"),

            userRanges = colorbandOptions.ranges,
            userRanges_length = userRanges && userRanges.length,
            initialFilterRange = chartObj.systemConf.legend.filter.range,

            scale = chartObj.axes.clr.scale,
            scale_domain = scale.domain(),

            isHorizontal = orient === "bottom" || orient === "top",
            range = isHorizontal ? [0, width] : [height, 0],

            brushScale,
            rangeBand,
            rangeBandBy2,
            brushScale_domain,
            initialExtent,
            singleDomainContinuous;

            if (userRanges_length) {
                brushScale = d3_scaleOrdinal()
                    .domain((datatype === ORDINAL_DATATYPE) ? scale_domain : getRawRanges(chartObj))
                    .rangeBands(range, 0, 0);

                rangeBand = brushScale.rangeBand();
                rangeBandBy2 = rangeBand / 2;
            } else {
                var domain = [scale_domain[0], scale_domain[scale_domain.length - 1]];
                if (domain[0] === domain[1]) { // If the domain is [10, 10], need to change the domain //Test_694.json
                    singleDomainContinuous = [-1, 1];
                    domain = [-MAX_SAFE_INTEGER, MAX_SAFE_INTEGER];
                }

                brushScale = d3_scaleLinear()
                    .domain(domain)
                    .range(range);
            }

        brushScale_domain = brushScale.domain();

        if (userRanges_length) {
            if (initialFilterRange) {
                initialExtent = getExtentForOrdinalScale(initialFilterRange, brushScale, scale_domain, userRanges, datatype);
            } else {
                var start = brushScale(brushScale_domain[brushScale_domain.length - 1]),
                    end = brushScale(brushScale_domain[0]);
                initialExtent = [start + rangeBandBy2 - MIN_VALUE, end + rangeBandBy2 + MIN_VALUE];
            }

            initialExtent.sort(d3_ascending);
        } else {
            initialExtent = initialFilterRange || singleDomainContinuous || brushScale.domain();
        }

        var brushOptions = {
            element: chartObj.legendarea.element,
            orient: orient,
            scale: brushScale,
            initialExtent: initialExtent,
            brushOptions: legendOptions.filter,
            dimension: dimension,
            onBrush: function(component) {
                if (singleDomainContinuous) {
                    return;
                }

                var extent = component.extent();
                extent = getExtentWithinRange(extent, (userRanges_length ? range : domain));

                if (userRanges_length) {
                    extent = getDomainForOrdinalScale(extent, brushScale, scale_domain, userRanges, datatype, isHorizontal);
                }

                chartObj.systemConf.legend.filter.range = extent;
                chartObj.addRemoveSeries();

                brushObj.context.selectAll(".resize")
                    .style("fill", function(d, i) {
                        return scale(extent[i ? 0 : extent.length - 1]);
                    })
            },
            onEnd: function(component) {
                var new_extent;
                if (userRanges_length) {
                    var filterRange = chartObj.systemConf.legend.filter.range;
                    new_extent = getExtentForOrdinalScale(filterRange, brushScale, scale_domain, userRanges, datatype);
                } else if (singleDomainContinuous) {
                    new_extent = singleDomainContinuous;
                }

                if (new_extent) {
                    component.extent(new_extent);
                    brushObj.context
                        .transition()
                        .call(component);
                }
            }
        };

        var brushObj = new brushComponent(chartObj, brushOptions);
        brushObj.context.attr("transform", "translate(" + position[0] + "," + position[1] + ")");

        brushObj.context.selectAll(".resize")
            .style("fill", function(d, i) {
                var extent = userRanges_length ? getDomainForOrdinalScale(initialExtent, brushScale, scale_domain, userRanges, datatype, isHorizontal) : initialExtent;
                return scale(extent[mathMax(0, extent.length - 1 - i)]);
            });
	};

    function getDomainForOrdinalScale(extent, brushScale, scale_domain, userRanges, datatype, isHorizontal) {
        var brushScale_range = d3_values(brushScale.range()).sort(d3_ascending),
            brushScale_domain = brushScale.domain(),
            rangeBand = brushScale.rangeBand(),
            startIndex, endIndex;

        // Find start and end index
        brushScale_range.some(function(d, i) {
            if (extent[0] >= d && extent[0] <= (d + rangeBand)) {
                var diff = extent[0] - d;
                if (diff <= 0.5 && diff >= 0) { // antialiasing artifacts
                    startIndex = mathMax(i - 1, 0);
                } else {
                    startIndex = i;
                }
            }
            if (extent[1] >= d && extent[1] <= (d + rangeBand)) {
                var diff = d + rangeBand - extent[1];
                if (diff <= 0.5 && diff >= 0) { // antialiasing artifacts
                    endIndex = mathMin(i + 1, brushScale_range.length - 1);
                } else {
                    endIndex = i;
                }
            }
            return defined(startIndex) && defined(endIndex);
        });

        var domain = brushScale_domain;
        if (datatype !== ORDINAL_DATATYPE && userRanges[0].length === 1) { // If ranges are single element like [50, 60, 70] (e.g: http://localhost/#/demoCharts/heatmap5), need to make it as range --> [[min, 50], [50, 60], [60, 70]]
            domain = getPairs(scale_domain, domain);
        }

        if (!isHorizontal) {
            var temp = startIndex;
            startIndex = brushScale_range.length - 1 - endIndex;
            endIndex = brushScale_range.length - 1 - temp;
        }

        domain = domain.slice(startIndex, endIndex + 1, 0);
        if (datatype === ORDINAL_DATATYPE) {
            extent = domain;
        } else {
            extent = d3_extent(d3_merge(domain));
            extent = updateFilterRange(extent, scale_domain);
        }
        return extent;
    }

    function getExtentForOrdinalScale(filterRange, brushScale, scale_domain, userRanges, datatype) {
        var rangeBandBy2 = brushScale.rangeBand() / 2,
            extent;

        if (datatype === ORDINAL_DATATYPE) {
            extent = [brushScale(filterRange[0]) + rangeBandBy2, brushScale(filterRange[filterRange.length - 1]) + rangeBandBy2];
        } else {
            var brushScale_domain = brushScale.domain(),
                startIndex, endIndex; //start and end index of userRange
            filterRange = updateFilterRange(filterRange, scale_domain);

            brushScale_domain.forEach(function(d, i) {
                if (filterRange[0] >= d[0]) {
                    startIndex = (userRanges[0].length === 1) ? i + 1 : i;
                }
                if (!defined(endIndex) && filterRange[1] <= d[0]) {
                    endIndex = (userRanges[0].length === 1) ? i : i - 1;
                }
            });

            startIndex = mathMax(0, validateNullVal(startIndex, 0));
            endIndex = mathMax(0, validateNullVal(endIndex, brushScale_domain.length - 1));
            extent = [brushScale(brushScale_domain[startIndex]) + rangeBandBy2, brushScale(brushScale_domain[endIndex]) + rangeBandBy2];
        }

        if (extent[0] === extent[1]) {
            extent[0] -= MIN_VALUE;
            extent[1] += MIN_VALUE;
        }
        return extent;
    }

    function getPairs(scale_domain, domain) {
        var min = scale_domain[2],
            max = scale_domain[scale_domain.length - 1 - 2],
            pairs = [];
        for(var i=0; i<domain.length; i++) {
            pairs.push([min, domain[i][0]]);
            min = domain[i][0];
        }
        pairs.push(min, max);
        return pairs;
    }

    function isDistinct(array, value) {
        return array.indexOf(value) === array.lastIndexOf(value);
    }

    function updateFilterRange(filterRange, scale_domain) {
        // If [[10, 20], [20, 30]] is the user ranges (20 is present twice), the filterRange is should be modified
        // Refer legend-brush-1

        if (!isDistinct(scale_domain, filterRange[1])) {
            filterRange[1] -= MIN_VALUE;
        }
        return filterRange;       
    }

    function getExtentWithinRange(extent, range) { //Refer: legend filter issue
        if (range[1] > range[0]) {
            extent[1] = mathMin(extent[1], range[1]);
            extent[0] = mathMax(extent[0], range[0]);
        } else {
            extent[1] = mathMax(extent[1], range[1]);
            extent[0] = mathMin(extent[0], range[0]);
        }
        return extent;
    }

})();