//$Id$
(function() {
	$ZC.legend.colorAxis = function(element, chartObj, colorAxisOptions) {
		var legendOptions = chartObj.systemConf.legend,
			colorbandOptions = legendOptions.colorBand,
			minmaxlabelsOptions = colorAxisOptions.minmaxlabels,

			dimension = legend_helpers.getColorBandDimension(chartObj),
			width = dimension[0],
			height = dimension[1],

			mode = colorbandOptions.type,
			orient = legend_helpers.getOrient(legendOptions),

			//Formatter
			columnInfo = chartObj.dataset.getColumn("clr"),
			datatype = columnInfo.datatype,
			formatConf = columnInfo[datatype],
			arraySeperator = "-",

			axisComp = $ZC.axis.get(),
			scale = chartObj.axes.clr.scale,
			scale_domain = scale.domain(),
            tickValues = minmaxlabelsOptions.show && minmaxlabelsOptions.excludeIntervals ? [] : scale_domain,
			range = (orient === "bottom" || orient === "top") ? [0, width] : [height, 0],

			align = (minmaxlabelsOptions.show && minmaxlabelsOptions.excludeIntervals && orient === "top") ? [0, 0] : (orient === "bottom" || orient === "top") ? [0, height] : [width, 0], //Test_692.json
			isDiscrete = colorbandOptions.ranges && colorbandOptions.ranges.length,
            userLabels = colorbandOptions.labels,
			posScale;

            chartObj.axes.clr.ticks = $ZC.axis.ticks(chartObj, colorAxisOptions, "clr", 0, orient, chartObj.chartarea); //dummy axisComp needs this

		if (isDiscrete) {
			scale_domain = getRawRanges(chartObj);
			var isOrdinal = chartObj.dataset.getDataType("clr") === ORDINAL_DATATYPE;
			if (mode === "continuous" && !isOrdinal) {
				scale_domain = d3_merge(scale_domain);
				if (colorbandOptions.minRange !== scale_domain[0]) {
					scale_domain.splice(0, 0, colorbandOptions.minRange);
				}

				posScale = d3_scaleOrdinal()
					.domain(scale_domain)
					.rangePoints(range, 0, 0);
			} else {
				posScale = d3_scaleOrdinal()
					.domain(scale_domain)
					.rangeBands(range, 0, 0);
			}
		} else {
            if (tickValues[0] === tickValues[1]) { //Test_694.json
                var rangeDiff = mathAbs(range[0] - range[1]);
                range = [rangeDiff / 2, rangeDiff / 2];
            }

			posScale = d3_scaleLinear()
				.domain([scale_domain[0], scale_domain[scale_domain.length - 1]])
				.range(range);
		}

        tickValues = minmaxlabelsOptions.show && minmaxlabelsOptions.excludeIntervals ? [] : scale_domain;

		axisComp
			.scale(posScale)
			.tickSize(validateNullVal(colorAxisOptions.tickmark.size, 6), 0)
			.majorTicks(getMajorTicks(orient, width, height))
			.orient(orient)
			.systemconf(chartObj.systemConf)
			.chartObj(chartObj)
			.tickValues(tickValues)
			.tickFormat(function(d, i) {
                if (userLabels && userLabels.length) {
                    var labelIndex = legend_helpers.getLabelIndex(d, colorbandOptions);
                    if (defined(labelIndex)) {
                        return userLabels[labelIndex];
                    }
                }
                return getFormattedValue(formatConf, d, datatype, chartObj, "legend", null, arraySeperator);
            });

		element
			.datum(colorAxisOptions)
			.attr("transform", "translate(" + align[0] + "," + align[1] + ")")
			.call(axisComp, "clr");

		element.selectAll("g.tick")
			.style("display", null); //important on redraw

        applyAxislineProperties(element, colorAxisOptions.axisline);

		element
			.call(ZC_legend_minmaxLabels, dimension, colorAxisOptions, chartObj, orient);
	};

    function getMajorTicks(orient, w, h) {
        if (orient == "bottom") {
            return [0, -h]
        } else if (orient == "top") {
            return [0, h]
        } else if (orient == "left") {
            return [w, 0]
        } else if (orient == "right") {
            return [-w, 0]
        }
    }

    $ZC.legend.colorAxis.handleOverlapNOverlap = function (axisGroup, axisGroupPosition, position, chartObj, chartarea) {
        var minmaxlabels = axisGroup.selectAll("g.zchart-minmaxlabels").selectAll("text"),
            ticklabels = axisGroup.selectAll("g.tick").selectAll("text"),

            minmaxlabelsBoundArray = handleOverflowNGetBound(minmaxlabels, axisGroupPosition, position, chartObj, chartarea);

        handleOverlap(ticklabels, axisGroupPosition, position, minmaxlabelsBoundArray);
    }

    function handleOverflowNGetBound(minmaxlabels, axisGroupPosition, position, chartObj, chartarea) {
    	var legendOptions = chartObj.systemConf.legend,
            orient = legend_helpers.getOrient(legendOptions),

    		minmaxlabelsBoundArray = [],
    		bound = getAvailableLegendBound(chartObj, chartarea),
            edge = {
	            "left": bound.x1,
	            "top": bound.y1,
	            "right": bound.x2,
	            "bottom": bound.y2
	        },
	        margin = {
	            "left": 5,
	            "right": 5,
	            "top": 5,
	            "bottom": 5
	        },
	        pattern = "..";

        minmaxlabels
        .each(function(d, i) {
            var pos = getMinmaxLabelBound(this, axisGroupPosition, position);

            var textBound = {
                "left": pos.x1,
                "top": pos.y1,
                "width": pos.width,
                "height": pos.height
            };
            trimText(this, this.textContent, textBound, edge, "bottom", pattern, margin);
            if (this.textContent === pattern) {
                d3_select(this).text("");
            }
            minmaxlabelsBoundArray[i] = getMinmaxLabelBound(this, axisGroupPosition, position);
        });

        if (orient === "bottom" || orient === "top") {
            if (minmaxlabelsBoundArray.length && doOverlap(minmaxlabelsBoundArray[0], minmaxlabelsBoundArray[1])) {
                var bandWidth = minmaxlabelsBoundArray[1].x2 - minmaxlabelsBoundArray[0].x1;
                minmaxlabels
                    .each(function(d, i) {
                        var pos = getMinmaxLabelBound(this, axisGroupPosition, position);
                        var textBound = {
                            "left": pos.x1,
                            "top": pos.y1,
                            "width": pos.width,
                            "height": pos.height
                        };
                        var e = simpleClone(edge);
                        if (i === 0) {
                            e.right = pos.x1 + bandWidth / 2;//need to change
                        } else {
                            e.left = pos.x2 - bandWidth / 2;//need to change
                        }

                        trimText(this, this.textContent, textBound, e, "bottom", pattern, margin);
                        if (this.textContent === pattern) {
                            d3_select(this).text("");
                        }
                        minmaxlabelsBoundArray[i] = getMinmaxLabelBound(this, axisGroupPosition, position);
                    });
            }
        }

        return minmaxlabelsBoundArray
    }

    function getMinmaxLabelBound(element, axisGroupPosition, position, parentPos) {
        parentPos = parentPos || [0, 0];
        var bb = bbox(element);
        var pos = {
            x1: bb.x + axisGroupPosition[0] + position[0] + parentPos[0],
            y1: bb.y + axisGroupPosition[1] + position[1] + parentPos[1],
            width: bb.width,
            height: bb.height
        };
        pos.x2 = pos.x1 + bb.width;
        pos.y2 = pos.y1 + bb.height;
        return pos;
    }

    function handleOverlap(ticklabels, axisGroupPosition, position, minmaxlabelsBoundArray) {
    	var ticklabelsBoundArray = [];
        var count = 0;
        ticklabels
        .each(function() {
            var parentPos = d3_transform(d3_select(this.parentNode).attr("transform")).translate;
            ticklabelsBoundArray.push(getMinmaxLabelBound(this, axisGroupPosition, position, parentPos));
            var overlap;

            for (var index = count - 1; index >= 0 && !overlap; index--) {
                overlap = doOverlap(ticklabelsBoundArray[index], ticklabelsBoundArray[count]);
            }
            if ((minmaxlabelsBoundArray.length ? (doOverlap(ticklabelsBoundArray[count], minmaxlabelsBoundArray[0]) || doOverlap(ticklabelsBoundArray[count], minmaxlabelsBoundArray[1])) : false) || overlap) {
                d3_select(this.parentNode).style("display", "none");
                ticklabelsBoundArray[count] = {
                    x1: 0,
                    y1: 0,
                    x2: 0,
                    y2: 0
                };
            }
            count ++;
        });
    }

    function getAvailableLegendBound(chartObj, chartarea) {
        var legendOptions = chartObj.systemConf.legend,
            orient = legend_helpers.getOrient(legendOptions),

            hAlign = legendOptions.hAlign,
            vAlign = legendOptions.vAlign,

            w = chartarea.width - legendOptions.marginLeft - legendOptions.marginRight,
            h = chartarea.height - legendOptions.marginTop - legendOptions.marginBottom,
            l = chartarea.left + legendOptions.marginLeft,
            t = chartarea.top + legendOptions.marginTop,

            widthRatio = (orient === "top" || orient === "bottom") ? ZC_LEGEND_maxWidthHoriRatio : ZC_LEGEND_maxWidthVertRatio,
            heightRatio = (orient === "top" || orient === "bottom") ? ZC_LEGEND_maxHeightHoriRatio : ZC_LEGEND_maxHeightVertRatio,

            maxWidth = perToPx(legendOptions.maxWidth || widthRatio, w),
            maxHeight = perToPx(legendOptions.maxHeight || heightRatio, h),

            bound = {};

        if (orient === "bottom") {
            bound.y1 = t + h - maxHeight;
            if (hAlign === "left") {
                bound.x1 = l;
            } else if (hAlign === "right") {
                bound.x1 = l + w - maxWidth;
            } else {
                bound.x1 = l + (w - maxWidth) / 2;
            }
        } else if (orient === "top") {
            bound.y1 = t;
            if (hAlign === "left") {
                bound.x1 = l;
            } else if (hAlign === "right") {
                bound.x1 = l + w - maxWidth;
            } else {
                bound.x1 = l + (w - maxWidth) / 2;
            }
        } else if (orient === "right") {
            bound.y1 = t;
            bound.x1 = l + w - maxWidth;
        } else {
            bound.y1 = t;
            bound.x1 = l;
        }

        bound.x2 = bound.x1 + maxWidth;
        bound.y2 = bound.y1 + maxHeight;
        return bound;
    }

    function applyAxislineProperties(container, axislineOptions) {
        var element = container.selectAll("path.domain");
        if (!axislineOptions.show) {
            element.remove();
        }
    }

})();