//$Id$
(function() {
	var getClr = legend_helpers.getClr,
		isDisabled = legend_helpers.isDisabled,
		hexToRGB = legend_helpers.hexToRGB;

	$ZC.legend.setColorBoxBackground = function(elements, chartObj, states) {
		var legendOptions = chartObj.systemConf.legend,
			colorBoxOptions = legendOptions.colorBox,
			itemDisabledOptions = legendOptions.itemDisabledStyle,
			disabledColorBoxOptions = itemDisabledOptions.colorBox || {},

			imagePallete = legendOptions.imagePallete ? $ZC.legutil.imagePallete.get(legendOptions.imagePallete) : null,
			useImagePallete = defined(imagePallete) && imagePallete.length > 0,
			patternPallete = chartObj.patternPallete,
			patternPalleteLength = patternPallete && patternPallete.length,
			useSVGImage = !chartObj.patternPallete && (defined(legendOptions.shapes) || colorBoxOptions.shape !== "square"),

			fontSize = +($ZC.utils.getVal((legendOptions.fontSize) || chartObj.systemConf.canvas.fontSize)),
			colorBoxSize = legend_helpers.getFilterBoxSize(colorBoxOptions, fontSize),
			colorBoxWidth = colorBoxSize[0],
			colorBoxHeight = colorBoxSize[1],

			background, border,
			backgroundColor, borerColor,
			borderRadius,
			backgroundSize = null,
			self, disabled;

		elements.each(function(d, i) {
			self = d3_select(this);
			disabled = isDisabled(d, states.allSelected, states.isFirstClickAfterDeselection);

			borerColor = getClr(disabled, pick(colorBoxOptions.strokeColor, getColorByIndex(chartObj, i)), itemDisabledOptions);
			background = backgroundColor = getClr(disabled, pick(colorBoxOptions.fillColor, getColorByIndex(chartObj, i)), itemDisabledOptions);
			borderRadius = colorBoxOptions.borderRadius;

			if (useImagePallete) {
				if (disabled) {
					background = pick(itemDisabledOptions.color, "silver");
				} else {
					background = "url(" + (imagePallete[i % imagePallete.length]) + ")";
					var patternSize = chartObj.patternImageSizes && chartObj.patternImageSizes;
					backgroundSize = patternSize[i] ? (patternSize[i % patternSize.length][0] + PX + " " + patternSize[i % patternSize.length][1] + PX) : null;
				}
			} else if (useSVGImage) {
				background = getsvgImage(colorBoxWidth, colorBoxHeight, borerColor, background, chartObj, i);
				borderRadius = null;
			} else if (patternPalleteLength) {
				var svgPattern = patternPallete[i % patternPalleteLength];
				var prop = patternThemes[svgPattern] ? patternThemes[svgPattern] : addPatternThemeByEle(svgPattern);
				prop.bgClr_base = backgroundColor
				prop.stroke_base = backgroundColor
				prop.width = colorBoxWidth
				var patternID = "zp" + generateIdByPropValue(prop, "legelegendndFill");
				var p = svg_patterns.svgPatternFill(prop, patternID, true, true, "objectBoundingBox");
				var patternWidth = p.width,
					patternHeight = p.height;
				background = "url(" + p.url + ")";
				backgroundSize = ((patternWidth * 2 < colorBoxWidth) ? patternWidth : (colorBoxWidth / 2)) + PX + " " + ((patternHeight * 2 < colorBoxHeight) ? patternHeight : (colorBoxHeight / 2)) + PX;
				border = colorBoxOptions.strokeWidth + PX + "solid " + borerColor;
			} else {
				border = colorBoxOptions.strokeWidth + PX + "solid " + borerColor;
			}
			self.style({
					"background": background,
					"background-size": backgroundSize,
					"color": disabled ? disabledColorBoxOptions.fontColor : colorBoxOptions.fontColor || "white",
					"border": border,
					"border-radius": defined(borderRadius) ? (borderRadius + PX) : null
				})
				.html(disabled ? disabledColorBoxOptions.text || "" : colorBoxOptions.text);
		})
	};

	function getsvgImage(width, height, strokeColor, fillColor, chartObj, index) { //SVG data uri as div background
		strokeColor = hexToRGB(strokeColor);
		fillColor = hexToRGB(fillColor);
		var legendOptions = chartObj.systemConf.legend,
			colorBoxOptions = legendOptions.colorBox,
			symbolsArray = legendOptions.shapes,
			symbol = (symbolsArray ? symbolsArray[index % symbolsArray.length] : null) || colorBoxOptions.shape;
		var xml = "<svg xmlns='http://www.w3.org/2000/svg' xmlns:xlink='http://www.w3.org/1999/xlink' width='" + width + "' height='" + height + "'>";
		xml += !symbol && $ZC.legend.getHTML4ChartTypeSymbol && !colorscale_helpers.isQuantileColorScale(chartObj) ?
			$ZC.legend.getHTML4ChartTypeSymbol(width, height, fillColor, chartObj, index) :
			getHTML4SVGSymbol(width, height, strokeColor, fillColor, symbol, colorBoxOptions);
		xml += " </svg>";
		return 'url("' + 'data:image/svg+xml;' + ($Browser.IE ? 'base64' : 'utf8') + ',' + ($Browser.IE ? btoa(xml) : xml) + '")';
	}

	function getHTML4SVGSymbol(width, height, strokeColor, fillColor, symbol, colorBoxOptions) {
		var symbolpath = getSVGsymbol(symbol || "square")(10);
		return "<path d='" + symbolpath + "' transform='translate(" + (width / 2) + "," + (height / 2) + ") scale(" + (width / (10 + (2 * validateNullVal(colorBoxOptions.strokeWidth, 1)))) + ")' style='stroke: " + hexToRGB(strokeColor) + "; fill: " + hexToRGB(fillColor) + "; fill-opacity: " + (colorBoxOptions.fillOpacity) + "; stroke-width: " + validateNullVal(colorBoxOptions.strokeWidth, 1) + "px; stroke-opacity: " + (colorBoxOptions.strokeOpacity) + ";'></path>";
	}
})();