//$Id$
(function() {
    $ZC.legend.continuous = function(){};
    $ZC.legend.continuous.get = function() {
        "use strict";
        var dispatch = d3_dispatch('legendClick', 'legendDblclick', 'legendMouseover', 'legendMouseout'),
            chartObj,
            systemConf,
            color,
            chartarea,

            getLegendData = legend_helpers.getLegendData,
            getLegendDataLength = legend_helpers.getLegendDataLength;

        function chart(selection) {
            selection.each(function(data) {
                var legData = getLegendData(chartObj),
                    legData_length = getLegendDataLength(chartObj, legData),
                    legendOptions = chartObj.systemConf.legend,
                    colorbandOptions = legendOptions.colorBand,
                    colorAxisOptions = getColorAxisOptions(legendOptions),

                    hAlign = data.hAlign,
                    vAlign = data.vAlign,

                    strokeWidth = validateNullVal(colorbandOptions.strokeWidth, 1) / 2,
                    padding = getParsedValue(colorbandOptions.padding),
                    hideAxis = colorAxisOptions.show === false,

                    container = d3_select(this),
                    legBBox;

                var background = appendEle(container, "path", [1], ".", "class", "zchart-background");
                var title = appendEle(container, "g", [1], ".", "class", "zcheader");
                var bandGroup = appendEle(container, "path", [1], ".", "class", "colorband");
                var axisGroup = appendEle(container, "g", (hideAxis ? [] : [1]), ".", "class", "legend-axis");
                background.style("display", "none");//important
                container.selectAll(".brush").style("display", "none");//important REFER: legend-colorband-right-redraw

                drawTitle(title, systemConf);
                $ZC.legend.colorBand(bandGroup, chartObj);
                $ZC.legend.colorAxis(axisGroup, chartObj, colorAxisOptions);

                var axisGroupTranslate = hideAxis ? [0, 0] : d3.transform(axisGroup.attr("transform")).translate;
                var info = translateGroupsNgetInfo(axisGroup, bandGroup, title, chartObj, colorAxisOptions, hAlign, vAlign, axisGroupTranslate, strokeWidth);

                var position = getLegendPosition();

                if (!hideAxis) {
                    $ZC.legend.colorAxis.handleOverlapNOverlap(axisGroup, info.axisGroupPosition, position, chartObj, chartarea);
                    info = translateGroupsNgetInfo(axisGroup, bandGroup, title, chartObj, colorAxisOptions, hAlign, vAlign, axisGroupTranslate, strokeWidth);
                    position = getLegendPosition();
                }

                container.selectAll(".brush").style("display", null);//important
                var legendBrushObj = new legendBrush(chartObj, {position: info.bandGroupPosition});
                position = getLegendPosition();

                container
                    .attr("transform", "translate(" + position[0] + "," + position[1] + ")");

                legBBox = bbox(container.node());
                drawBackground(background, chartObj, legBBox);
                background.style("display", null);//important

                //Filter effects
                if(chartObj.filterEffectsID){
                    container.style("filter" , chartObj.filterEffectsID ? "url(#"+chartObj.filterEffectsID+")" : null);
                }

                function getLegendPosition() {
                    legBBox = bbox(container.node());
                    var legwid = legBBox.width,
                        floating = defined(legendOptions.x) && defined(legendOptions.y),
                        x, y;

                    if (floating) {
                        return [legendOptions.x - legBBox.x + padding.left, legendOptions.y - legBBox.y + padding.top];
                    }

                    if (vAlign != "center") {
                        if (hAlign == "right") {
                            x = chartarea.left + chartarea.width - legwid - data.marginRight - legBBox.x - padding.right;
                        } else if (hAlign == "left") {
                            x = chartarea.left + data.marginLeft + padding.left - legBBox.x;
                        } else {
                            x = chartarea.width / 2 - legwid / 2;
                        }

                        if (vAlign === "top") {
                            y = chartarea.top + padding.top + data.marginTop - legBBox.y;
                        } else {
                            y = chartarea.top + chartarea.height - padding.bottom - data.marginBottom - legBBox.height - legBBox.y;
                        }
                    } else {
                        if (hAlign == "right") {
                            x = chartarea.left + chartarea.width - legwid - data.marginRight - legBBox.x - padding.right;
                        } else {
                            x = chartarea.left + data.marginLeft + padding.left - legBBox.x;
                        }

                        y = chartarea.top + data.marginTop + padding.top - legBBox.y;
                    }
                    return [x, y];
                }

            });
            return chart;
        }
        
        chart.dispatch = dispatch;

        chart.chartarea = function(_) {
            if (!arguments.length) {
                return chartarea;
            }
            chartarea = _;
            return chart;
        };

        chart.systemconf = function(_) {
            if (!arguments.length) {
                return systemConf;
            }
            systemConf = _;
            return chart;
        };

        chart.chartObj = function(_) {
            if (!arguments.length) {
                return chartObj;
            }
            chartObj = _;
            return chart;
        };

        chart.color = function(_) {
            if (!arguments.length) {
                return color;
            }
            color = _;
            return chart;
        };

        return chart;
    };

    function drawTitle(element, systemConf) {
        var titleOptions = systemConf.legend.title;

        var title = appendEle(element, "text", titleOptions.show ? [1] : [], ".", "class", "legendtitle");
        applyFont(element, [titleOptions, systemConf.canvas, $ZCG]);
        title
            .text(decodeHTML(titleOptions.text))
            .attr("dy", "0.91em");

    }

    function drawBackground(element, chartObj, legBBox) {
        var legendOptions = chartObj.systemConf.legend,
            backgroundOptions = legendOptions.background,

            colorBandOptions = legendOptions.colorBand,
            padding = getParsedValue(colorBandOptions.padding),

            borderOptions = legendOptions.border,
            showBorder = borderOptions.show,
            strokeWidth = pick(borderOptions.strokeWidth, 1),
            strokeDashArray = getDashArray(borderOptions.style, strokeWidth),
            strokeLineCap = getLineCap(borderOptions.lineCap, borderOptions.style),
            borderRadius = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, (showBorder ? borderOptions.radius : undefined));

        element
            .attr({
                "d": roundedRect(legBBox.x - padding.left, legBBox.y - padding.top, legBBox.width + padding.left + padding.right, legBBox.height + padding.top + padding.bottom, borderRadius)
            })
            .style({
                "stroke": borderOptions.color,
                "stroke-width": showBorder ? borderOptions.size : 0,
                "stroke-dasharray": strokeDashArray,
                "stroke-linecap": strokeLineCap,
                "fill": backgroundOptions.color || "transparent",
                "fill-opacity": backgroundOptions.alpha
            });
            //shadow
    }

    function updatePositionByStrokeWidth(orient, strokeWidth, bandGroupPosition, axisGroupPosition) {
        if (orient === "top") {
            bandGroupPosition[1] += strokeWidth;
        } else if (orient === "bottom") {
            axisGroupPosition[1] += strokeWidth;
        } else if (orient === "left") {
            bandGroupPosition[0] += strokeWidth;
        } else {
            axisGroupPosition[0] += strokeWidth;
        }
    }

    function getAxisGroupBBox(orient, axisGroup) {
        var selection = axisGroup.selectAll("g.zchart-minmaxlabels,line.majorTicks");
        selection.style("display", "none");
        var axisGroupBBox = simpleClone(bbox(axisGroup.node()));

        selection.style("display", null);

        return axisGroupBBox;
    }

    function getBandGroupPosition(systemConf, minmaxlabelsOptions, orient, hAlign, vAlign, axisGroupBBox, titleBBox, minmaxlabelsBBox) {
        var align = minmaxlabelsOptions.align;
        var bandGroupPosition = [];

        // get x position
        if (orient === "left") {
            if (align === "right") {
                bandGroupPosition[0] = axisGroupBBox.width;
            } else if (align === "left") {
                bandGroupPosition[0] = mathMax(axisGroupBBox.width, minmaxlabelsBBox.width);
            } else {
                bandGroupPosition[0] = mathMax(axisGroupBBox.width, minmaxlabelsBBox.width/2);
            }
        } else if (orient === "right") {
            if (align === "right") {
                bandGroupPosition[0] = 0;
            } else if (align === "left") {
                bandGroupPosition[0] = minmaxlabelsBBox.width;
            } else {
                bandGroupPosition[0] = minmaxlabelsBBox.width/2;
            }
        } else {
            if (align === "top" || align === "bottom") {
                bandGroupPosition[0] = 0;
            } else {
                bandGroupPosition[0] = - minmaxlabelsBBox.x;
            }
        }

        // get y position
        bandGroupPosition[1] = titleBBox.height + getTitleMargin(systemConf);
        if (orient === "left" || orient === "right") {
            bandGroupPosition[1] -= mathMin(0, minmaxlabelsBBox.y);
        } else if (orient === "bottom" && align === "top") {
            bandGroupPosition[1] += minmaxlabelsBBox.height;
        }
        if (vAlign === "top") {
            bandGroupPosition[1] += axisGroupBBox.height;
        }
        return bandGroupPosition;
    }

    function getAxisGroupPosition(systemConf, colorAxisOptions, orient, hAlign, vAlign, axisGroupBBox, titleBBox, minmaxlabelsBBox, axisGroupTranslate) {
        var minmaxlabelsOptions = colorAxisOptions.minmaxlabels,
            align = minmaxlabelsOptions.align;
        var axisGroupPosition = [];

        // get x position
        if (orient === "left") {
            if (align === "right") {
                axisGroupPosition[0] = axisGroupBBox.width;
            } else if (align === "left") {
                axisGroupPosition[0] = mathMax(axisGroupBBox.width, minmaxlabelsBBox.width);
            } else {
                axisGroupPosition[0] = mathMax(axisGroupBBox.width, minmaxlabelsBBox.width/2);
            }
        } else if (orient === "right") {
            if (align === "right") {
                axisGroupPosition[0] = axisGroupTranslate[0];
            } else if (align === "left") {
                axisGroupPosition[0] = axisGroupTranslate[0] + minmaxlabelsBBox.width;
            } else {
                axisGroupPosition[0] = axisGroupTranslate[0] + minmaxlabelsBBox.width/2;
            }
        } else {
            if (align === "top" || align === "bottom") {
                axisGroupPosition[0] = axisGroupTranslate[0];
            } else {
                axisGroupPosition[0] = axisGroupTranslate[0] - minmaxlabelsBBox.x;
            }
        }

        // get y position
        axisGroupPosition[1] = titleBBox.height + getTitleMargin(systemConf);
        if (orient === "left" || orient === "right") {
            axisGroupPosition[1] -= mathMin(0, minmaxlabelsBBox.y);
        } else if (orient === "bottom" && align === "top") {
            axisGroupPosition[1] += minmaxlabelsBBox.height;
        }

        if (orient === "top") { //#ZC844
            axisGroupPosition[1] += axisGroupBBox.height;
        } else {
            axisGroupPosition[1] += axisGroupTranslate[1];
        }

        axisGroupPosition = updateAxisPositionByAxisline(axisGroupPosition, colorAxisOptions.axisline, orient);

        return axisGroupPosition;
    }

    function updateAxisPositionByAxisline(axisGroupPosition, axislineOptions, orient) {
        if (!axislineOptions.show) {
            var strokeWidth = validateNullVal(axislineOptions.strokeWidth, 1),
                adjust = strokeWidth / 2;

            if (orient === "left") {
                axisGroupPosition[0] += adjust;
            } else if (orient === "right") {
                axisGroupPosition[0] -= adjust;
            } else if (orient === "bottom") {
                axisGroupPosition[1] -= adjust;
            } else {
                axisGroupPosition[1] += adjust;
            }
        }
        return axisGroupPosition;
    }

    function getTitleMargin(systemConf) {
        var titleOptions = systemConf.legend.title;
        if (titleOptions.show) {
            return defined(titleOptions.margin) ? titleOptions.margin : 3;
        }
        return 0;
    }

    function getParsedValue(padding) {
        var t, r, b, l;
        if (defined(padding)) {
            var parsedPadding = padding.toString().split(" "),
                priorityBasedValue = $ZC.DataProcessor.helpers.priorityBasedValue;

            t = pInt(priorityBasedValue([parsedPadding[0], 0]));
            r = pInt(priorityBasedValue([parsedPadding[1], parsedPadding[0], 0]));
            b = pInt(priorityBasedValue([parsedPadding[2], parsedPadding[0], 0]));
            l = pInt(priorityBasedValue([parsedPadding[3], parsedPadding[1], parsedPadding[0], 0]));
        } else {
            t = 0;
            r = 0;
            b = 0;
            l = 0;
        }

        return {
            top: t,
            right: r,
            bottom: b,
            left: l
        };
    }

    function getColorAxisOptions(legendOptions) {
        var defaultColorAxisOptions = {
            "label": {},
            "grid": {
                color: "transparent"
            },
            "ticklabel": {
                fontColor: legendOptions.fontColor,
                fontFamily: legendOptions.fontFamily,
                fontSize: legendOptions.fontSize,
                fontStyle: legendOptions.fontStyle,
                fontWeight: legendOptions.fontWeight,
                // marginBottom: 5,
                // marginLeft: 5,
                // marginRight: 5,
                // marginTop: 5,
                // maxHeight : "",
                // maxWidth : "",
                // rotation: 0,
                // textOverflow: "ellipsis",
                tooltip: legendOptions.tooltip
            },
            "minmaxlabels": { //excludeIntervals, show, padding, align, text, fontProperties
                padding: 2,
                align: "center",
                fontSize: legendOptions.fontSize
            },
            "tickmark": {
                color: "#B0B0B0",
                size: 3
            },
            "axisline": {
                show: false
            }
        };
        var colorAxisOptions = legendOptions.colorAxis || {};
        colorAxisOptions = mergeJSON(defaultColorAxisOptions, colorAxisOptions, true);
        return colorAxisOptions;
    }

    function translateGroupsNgetInfo(axisGroup, bandGroup, title, chartObj, colorAxisOptions, hAlign, vAlign, axisGroupTranslate, strokeWidth) {
        var systemConf = chartObj.systemConf,
            titleBBox = bbox(title.node()),
            orient = legend_helpers.getOrient(chartObj.systemConf.legend),
            axisGroupBBox = getAxisGroupBBox(orient, axisGroup),
            minmaxlabelsBBox = bbox(axisGroup.selectAll("g.zchart-minmaxlabels").node()),
            bandGroupPosition = getBandGroupPosition(systemConf, colorAxisOptions.minmaxlabels, orient, hAlign, vAlign, axisGroupBBox, titleBBox, minmaxlabelsBBox),
            axisGroupPosition = getAxisGroupPosition(systemConf, colorAxisOptions, orient, hAlign, vAlign, axisGroupBBox, titleBBox, minmaxlabelsBBox, axisGroupTranslate);

        updatePositionByStrokeWidth(orient, strokeWidth, bandGroupPosition, axisGroupPosition); //update position based on strokeWidth

        bandGroup.attr("transform", "translate(" + bandGroupPosition[0] + "," + bandGroupPosition[1] + ")");
        axisGroup.attr("transform", "translate(" + axisGroupPosition[0] + "," + axisGroupPosition[1] + ")");

        return {
            axisGroupPosition: axisGroupPosition,
            bandGroupPosition: bandGroupPosition
        }
    }

})();
