//$Id$
$ZC.legend.discrete = function(){};
$ZC.legend.discrete.get = function() {
    "use strict";
    var chartarea,
        legendWidth = 0,
        color,
        legendSeriesWidth = 0,
        dispatch = d3_dispatch('legendClick', 'legendDblclick', 'legendMouseover', 'legendMouseout'),
        scrollbarwid = 20,
        wrap, systemconf,
        chartObj,

        getLegendData = legend_helpers.getLegendData,
        getLegendDataLength = legend_helpers.getLegendDataLength,
        isHiddenItem = legend_helpers.isHiddenItem,
        getClr = legend_helpers.getClr,
        isDisabled = legend_helpers.isDisabled;

    
    function chart(selection) {
        selection.each(function(data) {
            var legData = getLegendData(chartObj),
                legData_length = getLegendDataLength(chartObj, legData),

                colorBoxOptions = data.colorBox,
                itemDisabledOptions = data.itemDisabledStyle,

                priorityArray = [data, systemconf.canvas, $ZCG],
                floating = ((data.x != null && data.x !== "") && (data.y != null && data.y !== "")),
                fontSize = +($ZC.utils.getVal((data.fontSize) || systemconf.canvas.fontSize)),
                fontColor = getValByPriority(priorityArray, "fontColor"),

                colIdx = chartObj.dataset.getColumnIdx((chartObj.xymultiRenderer || chartObj.isNonAxisMultiSeriesCategory) ? "clr" : "x"),
                showAs = data.showAs,
                colorBoxPosition = colorBoxOptions.position || "left",

                expandable = data.expandable || {},
                legendExpandable = expandable.show  === true,
                redrawing = !!chartObj.renderer.length,

                colorBoxSize = legend_helpers.getFilterBoxSize(colorBoxOptions, fontSize),
                colorBoxWidth = colorBoxSize[0],
                colorBoxHeight = colorBoxSize[1],

                hAlign = data.hAlign,
                vAlign = data.vAlign,

                states = chartObj.legend.states = chartObj.legend.states || {
                    allSelected: true,
                    isFirstClickAfterDeselection: false
                };

            //text-overflow properties
            var item_textOverflow_properties;
            if (data.textOverflow === "wrap") {
                item_textOverflow_properties = {
                    "word-wrap": "break-word",
                    "white-space": "normal",
                    "vertical-align": "top"
                };
            } else {
                item_textOverflow_properties = {
                    "text-overflow": "ellipsis",
                    "vertical-align": "middle"
                };
            }
            
            var container = d3_select(this);
            chart.update = function() {
                container.transition().call(chart);
            };
            wrap = container
                .style({
                    "display": "block",
                    "position": "absolute",
                    "left": null, //Reset left and top position. so that floating legend won't get trimmedon redraw (legend-03.json)
                    "top": null
                })
                .selectAll('div.legendgroup')
                .data([data]);
            var gEnter = wrap.enter().append('div').attr('class', 'legendgroup');
            
            //shadow properties
            var shadow = data.shadow;
            if (shadow.show === true) {
                var shadowstyle = shadow.x + "px " + shadow.y + "px " + shadow.blur + "px " + shadow.color;
                shadowstyle = shadow.inset ? shadowstyle + " inset" : shadowstyle;
                container.style("box-shadow", shadowstyle)
            } else {
                container.style("box-shadow", null);
            }
            
            //Background properties
            var rgb = d3_rgb(data.background.color.toString());
            container.style("background-color", "rgba(" + rgb.r + "," + rgb.g + "," + rgb.b + "," + data.background.alpha + ")");
            
            //Border Properties
            var border = data.border;
            if (border.show === true) {
                container.style({
                    "border": (+border.size) + "px " + border.style + " " + border.color,
                    "border-radius": border.radius + "px"
                });
            } else {
                container.style("border", null);
            }
            //Filter effects
            if(chartObj.filterEffectsID){
                container.style("filter" , chartObj.filterEffectsID ? "url(#"+chartObj.filterEffectsID+")" : null);
            }
            
            //Legend title
            var title_data = data.title;
            var titleMargin = defined(title_data.margin) ? title_data.margin : 3;
            
            var header = appendEle(wrap, "ul", ((title_data.show === true) ? [title_data] : []), ".", "class", "zcheader");
            var headerLI = appendEle(header, "li", [1], ".", "class", "zcheader");
            header
            .style({
                    "white-space": "normal",
                    "overflow": "hidden",
                    "width": "",
                    "list-style-type": "none",
                    "margin": 0,
                    "padding": 0
                });

            headerLI
                .style({
                    "padding" : shorthand_px(titleMargin),
                    "white-space": "nowrap",
                    "overflow": "hidden",
                    "text-align": "left",
                    "text-overflow": "ellipsis",
                    "vertical-align": "middle"
                });

            var selectAllObj = $ZC.legend.selectAllBox(headerLI, chartObj, states);
            var selectAllFilter = selectAllObj.element,
                selectAllBoxOptions = selectAllObj.options,
                selectAllFilterSize = selectAllObj.size,
                disabledOpacity = selectAllBoxOptions.disabledOpacity;
            legend_helpers.fadeChart(states.allSelected, chartObj, disabledOpacity)

            var legtitle = appendEle(headerLI, "span", [1], ".", "class", "legendtitle");
            legtitle
                .text(decodeHTML(title_data.text))
                .style(("padding-"+colorBoxPosition), selectAllBoxOptions.show ? (data.textPadding) + "px" : null)
                .style(("margin-"+colorBoxPosition), selectAllBoxOptions.show ? selectAllFilterSize + "px" : null);
            applyFont(legtitle, [title_data, systemconf.canvas, $ZCG], true);
 
            selectAllFilter
                .style("margin", function() {
                    var top = Math.max(0, ((legtitle.node().offsetHeight - selectAllFilter.node().offsetHeight) / 2)) + "px",
                        bottom = "0px",
                        left = ((colorBoxWidth - selectAllFilterSize) / 2) + "px",
                        right = ((colorBoxWidth - selectAllFilterSize) / 2) + "px";
                    return top + " " + right + " " + bottom + " " + left;
                });

            headerLI
                .style("line-height", function(){
                    var selectAllFilterHeight = selectAllBoxOptions.show ? selectAllFilter.node().offsetHeight : 0;
                    var headerLineHeight = Math.max(legtitle.node().offsetHeight, selectAllFilterHeight);
                    return headerLineHeight + "px"
                });

            var legseries = appendEle(wrap, "ul", [data], ".", "class", "legseries");
            legseries
                .style({
                    "white-space": "normal",
                    "overflow-y": "hidden",
                    // "cursor": cursor,
                    "width": "",
                    "list-style-type": "none",
                    "margin": 0,
                    "padding": 0,
                    "-webkit-overflow-scrolling" : "touch",

                    // #ZC758
                    // https://stackoverflow.com/questions/5256533/
                    "font-size": "0px"
                });

            var showMoreElement;
            
            if (legendExpandable) {
                legseries.style("overflow", "hidden");
                showMoreElement = appendEle(wrap, "div", [1], ".", "class", "expandable");
                showMoreElement.style({
                    "color": expandable.fontColor,
                    "font-size": $ZC.utils.getVal(expandable.fontSize) + "px",
                    "font-family": expandable.fontFamily || getValByPriority(priorityArray, "fontFamily"),
                    "text-align": expandable.textAlign,
                    "cursor": "pointer",
                    "display": "block",
                    "padding": data.itemMargin + "px",
                    "overflow": "hidden",
                    "text-overflow": "ellipsis",
                    "white-space": "nowrap"
                });
                showMoreElement.html(invokeFunction(expandable.text, 0, chartObj) || (0 + " more...")); // for pre height calculation
                
            } else {
                if (!wrap.select(".expandable").empty()) {
                    wrap.selectAll(".expandable").remove();
                    legseries.style("overflow-y", "hidden");
                    legseries.style("overflow-x", "hidden");
                }
            }
            
            var series = legseries.selectAll("li.series").data(legData);
            var seriesEnter = series.enter().append("li").attr("class", "series");
            seriesEnter.append("div").attr("class", "legendfilterbox");
            seriesEnter.append("span").attr("class", "legendlabel");
            series.exit().remove();
            
            series
                .style({
                    "color": function(d) {
                        return getClr(isDisabled(d, states.allSelected, states.isFirstClickAfterDeselection), fontColor, itemDisabledOptions);
                    },
                    "position": "relative",
                    "white-space": "nowrap",
                    "padding": (+data.itemMargin) + "px",
                    "overflow": "hidden",
                    "text-align": "left",
                    "line-height": defined(colorBoxOptions.size) ? mathMax(fontSize, colorBoxHeight) + "px" : ((($Browser.OSX || $Browser.LINUX) && $Browser.FIREFOX) ? "1.2" : "normal"),
                    "font-size": fontSize + "px",
                    "width": (data.itemWidth == null || data.itemWidth == "") ? null : data.itemWidth + "px",
                    "height": (data.itemHeight == null || data.itemHeight == "") ? null : data.itemHeight + "px",
                    "font-family": getValByPriority(priorityArray, "fontFamily"),
                    "font-weight": getValByPriority(priorityArray, "fontWeight"),
                    "display": function(d) {
                        return isHiddenItem(d) ? "none" : ((!floating && vAlign != "center") ? "inline-block" : "block");
                    }
                })
                .style(item_textOverflow_properties)
                .style("vertical-align", "top");

            $ZC.legend.events(chartObj, series, states, disabledOpacity);

            var legendlabel = series.select("span.legendlabel");
            legendlabel
                .style(("padding-"+colorBoxPosition), (data.textPadding) + "px")
                .style(("margin-"+colorBoxPosition), colorBoxWidth + "px")
                .html(function(d, i) {
                    if (colorscale_helpers.isQuantileColorScale(chartObj)) {
                        var colorBandOptions = data.colorBand,
                            userLabels = colorBandOptions.labels;

                        if (userLabels && userLabels.length) {
                            return userLabels[i];//check with unorders ranges input
                        }
                    }

                    var seriesPos = (chartObj.xymultiRenderer || chartObj.isNonAxisMultiSeriesCategory) ? i : 0,
                        itemPos = (chartObj.xymultiRenderer || chartObj.isNonAxisMultiSeriesCategory) ? 0 : i;
                    return showAs ?
                        chartObj.dataset.getCustomizedVal(chartObj, showAs, seriesPos, 0, itemPos, d, "legend") :
                        chartObj.dataset.getDataPointValue(chartObj, d, colIdx, seriesPos, null, "legend");
                })
                .attr("title", function() {
                    return data.tooltip ? this.textContent : null;
                });
            
            var ImpFontProp = {
                "fontColor": function(d) {
                    return getClr(isDisabled(d, states.allSelected, states.isFirstClickAfterDeselection), fontColor, itemDisabledOptions);
                },
                "textDecoration": function(d) {
                    return (isDisabled(d, states.allSelected, states.isFirstClickAfterDeselection) && itemDisabledOptions.strikeout) ? "line-through" : null;
                }
            };
            applyFont(legendlabel, [ImpFontProp, data, systemconf.canvas, $ZCG], true);

            var legend_label_height = legseries.select("span.legendlabel").node().offsetHeight;

            var margin_top = 0;
            if (defined(colorBoxOptions.size) || data.textOverflow === "wrap") {
                margin_top = (mathMax(fontSize, colorBoxHeight) - colorBoxHeight) / 2;
            } else if (legend_label_height > colorBoxHeight) {
                margin_top = (legend_label_height - colorBoxHeight) / 2;
            }
            
            var legendFilter = series.select("div.legendfilterbox");
            
            legendFilter
                .style({
                    "width": colorBoxWidth + "px",
                    "height": colorBoxHeight + "px",
                    "float": colorBoxPosition,
                    "margin-top": margin_top + "px", // ISSUE:ZC429 -> line-height will not be applied to float element
                    "border-color": "transparent",
                    "background-repeat": "repeat",
                    "line-height": (colorBoxHeight - colorBoxOptions.strokeWidth) + "px",
                    "text-align": "center",
                    "font-family": colorBoxOptions.fontFamily,
                    "font-size": (colorBoxOptions.fontSize || fontSize) + "px",
                    "position" : "absolute",
                    "display" : "inline-block",
                    "box-sizing": "border-box"
                });

            // background for colorBox
            $ZC.legend.setColorBoxBackground(legendFilter, chartObj, states);


            var itemTempHeight = series.node().offsetHeight,
                legendExpanded = chartObj.legend.expanded;
            
            var itemWidths = [];
            var maxItemWidth = data.itemWidth != null ? (+data.itemWidth) : null
            var textNodeWid, filterNodeWid;
            //Itemwidth calculate
            series.each(function(d) {
                var legendfilter = d3_select(this)
                    .select('div.legendfilterbox');
                var legendText = d3_select(this)
                    .select('span.legendlabel');
                filterNodeWid = filterNodeWid || legendfilter.node().getBoundingClientRect().width;
                textNodeWid = legendText.node().getBoundingClientRect().width;
                var computedTextLength = filterNodeWid + textNodeWid;
                itemWidths.push(maxItemWidth || (computedTextLength + 2 * (+data.itemMargin)));
            });
            
            maxItemWidth = data.itemWidth != null ? maxItemWidth : d3_max(itemWidths);
            var maxItemHeight = data.itemHeight || (mathMax(legend_label_height, colorBoxHeight) + (2 * (+data.itemMargin)));
            
            legseries.style("overflow-y", ((legendExpandable && legendExpanded) || !legendExpandable) === true ? "auto" : "hidden");
            legseries.style("overflow-x", "hidden");
            var itemsPerRow = 0,
                maxItemToFit;
            
            var columnWidths = [];
            legendWidth = 0

            if (floating) {
                var maxHeight = perToPx((data.maxHeight || ZC_LEGEND_maxHeightVertRatio), chartarea.height);
                if (legendExpandable) {
                    maxHeight = (mathFloor(maxHeight / itemTempHeight) * itemTempHeight);
                }
                
                var dimension = chartObj.legendarea.element.node().getBoundingClientRect();
                
                var maxX = mathMin(chartObj.canvasarea.width - dimension.width, data.x),
                    maxY = mathMin(chartObj.canvasarea.height - (mathMin(dimension.height, maxHeight)), data.y);
                
                container.style('left', maxX + "px");
                container.style('top', maxY + "px");
                legseries.style("max-height", maxHeight + "px");
            } else {
                var maxWidth = perToPx(data.maxWidth || ((vAlign != "center") ? ZC_LEGEND_maxWidthHoriRatio : ZC_LEGEND_maxWidthVertRatio), chartarea.width);
                wrap
                    .style("max-width", maxWidth + "px");
                header
                    .style("max-width", maxWidth + "px");

                if (vAlign != "center") {
                    var maxHeight = perToPx((data.maxHeight || ZC_LEGEND_maxHeightHoriRatio), chartarea.height);
                    var availableLegWidth = maxWidth - (+data.marginLeft) - (+data.marginRight) - (data.border.show ? (2 * data.border.size) : 0);
                    if (legendExpandable) {
                        maxHeight = (mathFloor(maxHeight / itemTempHeight) * itemTempHeight);
                        if(maxHeight < itemTempHeight) {
                            maxHeight = itemTempHeight;
                        }
                    }

                    while (legendWidth < availableLegWidth && itemsPerRow < itemWidths.length) {
                        columnWidths[itemsPerRow] = itemWidths[itemsPerRow];
                        legendWidth += itemWidths[itemsPerRow];
                        itemsPerRow++;
                    }

                    var sumFunc = function(prev, cur) {
                            return prev + cur;
                        };
                    while (legendWidth > availableLegWidth && itemsPerRow > 1) {
                        columnWidths = [];
                        itemsPerRow--;
                        
                        for (var k = 0; k < itemWidths.length; k++) {
                            // if (maxItemWidth > (columnWidths[k % itemsPerRow] || 0)) {
                            columnWidths[k % itemsPerRow] = itemWidths[k % itemsPerRow];
                            // }
                        }
                        
                        legendWidth = columnWidths.reduce(sumFunc);
                    }
                    var moduloRem = itemWidths.length % itemsPerRow;
                    var moduloQuo = (itemWidths.length - moduloRem) / itemsPerRow;
                    var noOfRows = (moduloRem == 0) ? moduloQuo : moduloQuo + 1;

                    legseries.style({
                        "height": ((noOfRows * maxItemHeight) + 2) + "px",
                        "max-height": maxHeight + "px",
                        "max-width": maxWidth + "px"
                    });

                    var scrollWid = (legseries.node().scrollHeight > legseries.node().offsetHeight) ? scrollbarwid : 0;
                    legendSeriesWidth = legendWidth + 2 + scrollWid;
                    legseries.style("width", legendSeriesWidth + "px");
                    var legalign = align(data, container);

                    maxItemToFit = getVisibleItemCount(legseries);
                    if(defined(showMoreElement) && legData_length <= maxItemToFit) {
                        appendEle(wrap, "div", [], ".", "class", "expandable");
                        legalign = align(data, container);
                    }
                    container.style({
                        "left": (legalign.x) + "px",
                        "top": (legalign.y) + "px"
                    });
                } else {
                    itemsPerRow = 1;
                    var maxHeight = perToPx((data.maxHeight || ZC_LEGEND_maxHeightVertRatio), chartarea.height);
                    
                    if (legendExpandable) {
                        maxHeight = (mathFloor(maxHeight / itemTempHeight) * itemTempHeight);
                    }
                    
                    legseries
                        .style({
                            "max-height" : maxHeight + "px",
                            "max-width" : maxWidth + "px"
                        });

                    var scrollWid = (legseries.node().scrollHeight > legseries.node().offsetHeight) ? scrollbarwid : 0;
                    legendSeriesWidth = maxItemWidth + 2 + scrollWid;

                    maxItemToFit = getVisibleItemCount(legseries);
                    if (defined(showMoreElement) && legData_length <= maxItemToFit) {
                        appendEle(wrap, "div", [], ".", "class", "expandable");
                    }

                    legseries.style({
                        "width": legendSeriesWidth + "px",
                        "height": null
                    });
                    var legwid = container.node().getBoundingClientRect().width;
                    if (hAlign == "right" || hAlign == "center") {
                        container.style('left', (chartarea.width - legwid - (+data.marginRight)) + "px");
                        container.style('top', (chartarea.top + (+data.marginTop)) + "px");
                    } else {
                        container.style('left', (+data.marginLeft) + "px");
                        container.style('top', (chartarea.top + (+data.marginTop)) + "px");
                    }
                }
            }

            if (legendExpandable) {
                var visibleItemCount = pick(maxItemToFit, getVisibleItemCount(legseries));
                if (legData_length > visibleItemCount) {
                    showMoreElement.style({
                        "width": (legendSeriesWidth) + "px"
                    });
                    
                    var showMoreText = invokeFunction(expandable.text, legData_length - visibleItemCount, chartObj) || (legData_length - visibleItemCount + " more...");
                    
                    if (!legendExpanded) {
                        showMoreElement.html(showMoreText);
                    } else if (redrawing && legendExpanded) {
                        showMoreElement.html(function() {
                            return invokeFunction(expandable.ontoggle, this, legendExpanded, chartObj) || "Show less..";
                        });
                    }
                    
                    showMoreElement.on("click", function() {
                        legendExpanded = chartObj.legend.expanded = !chartObj.legend.expanded;
                        var updatedText = invokeFunction(expandable.ontoggle, this, legendExpanded, chartObj);
                        if (defined(updatedText)) {
                            updatedText = updatedText + "";
                        }
                        
                        if (legendExpanded) {
                            this.innerHTML = updatedText || "Show less..";
                            legseries.style("overflow-y", "auto");
                        } else {
                            legseries.node().scrollTop = 0;
                            visibleItemCount = getVisibleItemCount(legseries);
                            this.innerHTML = updatedText || invokeFunction(expandable.text, legData_length - visibleItemCount, chartObj) || (legData_length - visibleItemCount + " more...");

                            legseries.style("overflow-y", "hidden");
                        }
                    })
                } else {
                    showMoreElement.remove();
                }
            }
        });
        return chart;
    }
    
    chart.dispatch = dispatch;
    
    chart.chartarea = function(_) {
        if (!arguments.length) {
            return chartarea;
        }
        chartarea = _;
        return chart;
    };
    
    chart.systemconf = function(_) {
        if (!arguments.length) {
            return systemconf;
        }
        systemconf = _;
        return chart;
    };
    
    chart.chartObj = function(_) {
        if (!arguments.length) {
            return chartObj;
        }
        chartObj = _;
        return chart;
    };
    
    chart.color = function(_) {
        if (!arguments.length) {
            return color;
        }
        color = _;
        return chart;
    };

    function align(data, legNode) {
        var hAlign = data.hAlign,
            vAlign = data.vAlign,

            bb = legNode.node().getBoundingClientRect(),
            legwid = bb.width,
            leghei = legNode.node().offsetHeight;

        var align = {};
        if (hAlign == "left") {
            align.x = chartarea.left + (+data.marginLeft);
        } else if (hAlign == "center") {
            align.x = chartarea.width / 2 - legwid / 2;
        } else if (hAlign == "right") {
            align.x = chartarea.width - legwid - (+data.marginRight);
        }
        if (vAlign == "top") {
            align.y = chartarea.top + (+data.marginTop);
        } else if (vAlign == "bottom") {
            align.y = chartarea.height + chartarea.top - leghei - (+data.marginBottom);
        }
        return align;
    }
    
    function getVisibleItemCount(legseries) {

        var leg = legseries.node(),
            elements = leg.children,
            count = 0;

        for (var ii = 0, ll = elements.length; ii < ll; ii++) {
            if (elementIsVisible(elements[ii], leg)) {
                count++;
            } else {
                break;
            }
        }
        return count;
    }

    return chart;
};