//$Id$
(function() {
	var getLegendData = legend_helpers.getLegendData,
		getLegendDataLength = legend_helpers.getLegendDataLength,
		isHiddenItem = legend_helpers.isHiddenItem,
		isDisabled = legend_helpers.isDisabled,
		isAllEnabled = legend_helpers.isAllEnabled,
		getClr = legend_helpers.getClr,
		updateData = legend_helpers.updateData;

	$ZC.legend.events = function(chartObj, element, states, disabledOpacity) {
		var legendComponent = chartObj.legend.component,
			legendOptions = chartObj.systemConf.legend,
			itemDisabledOptions = legendOptions.itemDisabledStyle,
			colorBoxOptions = legendOptions.colorBox,

			priorityArray = [legendOptions, chartObj.systemConf.canvas, $ZCG],
			priorityArrayOnhover = [legendOptions.itemHoverStyle, legendOptions, chartObj.systemConf.canvas, $ZCG],
			fontColor = getValByPriority(priorityArray, "fontColor"),

			legendFilter,
			selectAllFilter,

			legData,
			legData_length;

		/*
		 * legend event variables
		 */
		var events = chartObj.systemConf.legend.events;
		var onClickFn, onDblClickFn, onMouseOverFn, onMouseOutFn,
			onTapFn, onTapHoldFn, onTapHoldEndFn,
			cursor, eventDisabled;
		if (events != null) {
			eventDisabled = (validateNullVal(events.enabled, true) == false);
			onClickFn = events.click;
			onDblClickFn = events.doubleclick;
			onMouseOverFn = events.mouseover;
			onMouseOutFn = events.mouseout;
			cursor = events.cursor;
			onTapFn = events.tap;
			onTapHoldFn = events.taphold;
			onTapHoldEndFn = events.tapholdend;
		}

		if (!eventDisabled) {
			element
				.style("cursor", cursor || "pointer");

			var eventAttacherOptions = {
				chartObj: chartObj
			};
			var mousemove = function(d, i) {
					clickedTime = null;
					if (onMouseOverFn == null) {
						defaultOnMouseOverFunc(this, d, i);
					} else {
						var userFunction = chartObj.zcEventType ? onTapHoldFn : onMouseOverFn;
						invokeFunction(userFunction, d3.event, d, i, chartObj);
					}
				},
				mouseout = function(d, i) {
					if (onMouseOutFn == null) {
						defaultOnMouseOutFunc(this, d, i);
					} else {
						var userFunction = chartObj.zcEventType ? onTapHoldEndFn : onMouseOutFn;
						invokeFunction(userFunction, d3.event, d, i, chartObj);
					}
				},
				click = function(d, i) {
					var isTouchEvent = d3.event && d3.event.touches;
					if (isTouchEvent) {
						clickedTime = null;
						docTapFunction(chartObj);
					}
					if (onClickFn == null) {
						defaultOnClickFunc(this, d, i);
					} else {
						var userFunction = chartObj.zcEventType ? onTapFn : onClickFn;
						invokeFunction(userFunction, d3.event, d, i, chartObj);
					}
				},
				dblclick = function(d, i) {
					if (onDblClickFn == null) {
						var dummy = true;
						//default double click function
					} else {
						invokeFunction(onDblClickFn, d3.event, d, i, chartObj);
					}
				};
			var mouseHandlers = {
					"mousemove": mousemove,
					"mouseout": mouseout,
					"click": click,
					"dblclick": dblclick
				},
				touchHandlers = {
					"taphold": mousemove,
					"tapholdend": mouseout,
					"tap": click,
					"dbltap": dblclick
				};
			attachevents(element, mouseHandlers, touchHandlers, eventAttacherOptions);
		}

		function defaultOnClickFunc(ele, d, i) {
			legendComponent.toggleItemNUpdateChart(ele, d, i, legendOptions);
		}

		function defaultOnMouseOverFunc(ele, d, i) {
			legendComponent.highlightSeries(ele, d, i, legendOptions, chartObj, true);
			legendComponent.highlightItem(ele, d, i, legendOptions, true);
		}

		function defaultOnMouseOutFunc(ele, d, i) {
			legendComponent.highlightSeries(ele, d, i, legendOptions, chartObj);
			legendComponent.highlightItem(ele, d, i, legendOptions);
		}

		legendComponent.filterSeriesNUpdate = function(d, i, chartObj) {
			if (!legData) { //If legend is disabled, legData will not be available
				legData = getLegendData(chartObj);
				legData_length = getLegendDataLength(chartObj, legData);
			}
			var clickeddata = legData[i];
			var disabledEle = legData.filter(function(d, j) {
				if (!states.allSelected && !states.isFirstClickAfterDeselection) {
					if (!isHiddenItem(d)) {
						d.disabled = !states.allSelected;
					}
					updateData(d, j, chartObj)
					legend_helpers.toggleChartEvents(chartObj, true);
					legend_helpers.fadeChart(true, chartObj, disabledOpacity);

					var tooltip = chartObj.eventHandler.tooltipElement;
					tooltip.hide(tooltip.element);
				}
				return d.disabled;
			});
			var isAlldisabled = (disabledEle.length == legData_length - 1) && !clickeddata.disabled;
			if (!isAlldisabled) {
				clickeddata.disabled = !clickeddata.disabled;
				if(chartObj.dataObject.hierarchical) {
	                chartObj.systemConf.seriesdata.chartdata[i].disabled = clickeddata.disabled;
	            }
				updateData(clickeddata, i, chartObj)
				chartObj.addRemoveSeries(clickeddata, 0, clickeddata.yaxiscolumnorder, (!states.allSelected && !states.isFirstClickAfterDeselection));
			}
			if (!states.allSelected && !states.isFirstClickAfterDeselection) {
				states.isFirstClickAfterDeselection = true;
			}
		};

		legendComponent.toggleItemNUpdateChart = function(ele, d, i, legendOptions) {
			var legendContainer = chartObj.legendarea.element;
			legendFilter = legendFilter || legendContainer.selectAll("div.legendfilterbox");
			selectAllFilter = selectAllFilter || legendContainer.selectAll("div.selectAll");

			if (legendOptions.filter.enabled) {
				legendComponent.filterSeriesNUpdate(d, i, chartObj);
				var filteredEle = d3_select(ele),
					fontClr = getClr(d.disabled, fontColor, itemDisabledOptions),
					txtDec;
				if (d.disabled) {
					legendComponent.highlightItem(ele, d, i, legendOptions, false, true); //Reset the hoverstyle for disabled series; strictApply=true
					if (itemDisabledOptions.strikeout) {
						txtDec = "line-through";
					}
				}

				filteredEle.selectAll(".legendlabel")
					.style({
						"color": fontClr,
						"text-decoration": txtDec
					});
				// update the colorBox
				$ZC.legend.setColorBoxBackground(legendFilter, chartObj, states);
				if (d3.event.touches == null) {
					defaultOnMouseOverFunc(ele, d, i);
				}
			}
			if (!states.allSelected) {
				states.allSelected = !states.allSelected;
			}
			if (!selectAllFilter.empty()) {
				selectAllFilter
					.html(isAllEnabled(legData, states.allSelected) ? colorBoxOptions.text || "&#10003;" : (itemDisabledOptions.colorBox || {}).text || "");
			}
		};

		legendComponent.highlightItem = function(ele, d, i, legendOptions, onLegHover, strictApply) {
			if (strictApply || !isDisabled(d, states.allSelected, states.isFirstClickAfterDeselection)) {
				if (legendOptions.itemHoverStyle) {
					var filteredEle = d3_select(ele);
					filteredEle.style("background-color", (onLegHover ? legendOptions.itemHoverStyle : legendOptions).backgroundColor);
					applyFont(filteredEle.selectAll("span"), (onLegHover ? priorityArrayOnhover : priorityArray), true);
				}
			}
		};

		legendComponent.highlightSeries = function(ele, d, i, legendOptions, chartObj, onLegHover) {
			chartObj.highlightContext = null;
			if (colorscale_helpers.isQuantileColorScale(chartObj)) {
				var highlightObj = chartObj.legendHighlight;
				var filterIndex = i;
				if (onLegHover) {
					if (!d.disabled) {
						chartObj.seriesdata.forEach(function(d, j) {
							highlightObj.highlightSelectedSeries(legendOptions, j, filterIndex);
							highlightObj.highlightUnselectedSeries(legendOptions, j, filterIndex);
						});
					}
				} else {
					chartObj.seriesdata.forEach(function(d, j) {
						highlightObj.reset(legendOptions, j, filterIndex, (chartObj.seriesdata.length-1 === j));
					});
				}
			} else {
				var highlightObj = chartObj.legendHighlight;
				if (onLegHover) {
					if (!d.disabled) {
						highlightObj.highlightSelectedSeries(legendOptions, i);
						highlightObj.highlightUnselectedSeries(legendOptions, i);
					}
				} else {
					highlightObj.reset(legendOptions, i);
				}
			}
		};
	};
})();
