//$Id$
var legend_helpers = {};
legend_helpers.getLegendData = function(chartObj) {

	// Continuous color - ranges
	if (colorscale_helpers.isQuantileColorScale(chartObj)) {
		var userRanges = chartObj.systemConf.legend.colorBand.ranges;
		if (userRanges && userRanges.length) {
			return userRanges;
		}
	}

	if (chartObj.xymultiRenderer || chartObj.isNonAxisMultiSeriesCategory) {
		return chartObj.seriesdata;
	} else {
		return chartObj.seriesdata[0].data[0];
	}
};

legend_helpers.getLegendDataLength = function(chartObj, legData) {
	if (chartObj.xymultiRenderer || chartObj.isNonAxisMultiSeriesCategory) {
		return legData.filter(function(d) {
			return !legend_helpers.isHiddenItem(d);
		}).length;
	} else {
		return legData.length;
	}
};

legend_helpers.isHiddenItem = function(d) {
	return d.hideInLegend;
};

legend_helpers.getClr = function(disabled, defaultClr, itemDisabledStyle) {
	return disabled ?
		itemDisabledStyle.color || (!itemDisabledStyle.color || itemDisabledStyle.strikeout ? defaultClr : "silver") :
		defaultClr;
};

legend_helpers.updateBubblePieData = function(d, i, chartObj) {
	if (chartObj.dataObject.pieWithAxis) {
		d.data.forEach(function(subseries) {
			var isBubblePie = (chartObj.chartTypes[i].value === $ZC.charttype.bubblepie);
			subseries.forEach(function(dataPoint) {
				dataPoint.disabled = d.disabled || !isBubblePie;
			});
		});
	}
};

legend_helpers.updateHierarchyData = function(d, i, chartObj) {
	if (chartObj.dataObject.isHierarchy) {
		chartObj.systemConf.seriesdata.chartdata[i].disabled = d.disabled;
	}
};

legend_helpers.toggleChartEvents = function(chartObj, enable) {
	var val = enable ? null : "none",
		elements = [];
	if (!chartObj.svg) {
		elements.push(chartObj.container);
	} else {
		if (chartObj.isAxisCategory || chartObj.dataObject.isPolarAxisCategory) {
			elements.push(chartObj.eventHandler.trackerElement);
		}
		elements.push(chartObj.svg);
	}
	elements.forEach(function(d) {
		d.style("pointer-events", val);
	})
};

legend_helpers.fadeChart = function(enable, chartObj, opacity) {
	chartObj.container.selectAll("g.chartgroup,svg.static,canvas.zc-canvas,div.zcnotes")
		.style("opacity", enable ? null : opacity);
};

legend_helpers.isDisabled = function(d, allSelected, isFirstClickAfterDeselection) {
	return (!allSelected && !isFirstClickAfterDeselection) ?
		!allSelected :
		d.disabled;
};

legend_helpers.isAllEnabled = function(legData, allSelected) {
	if (!allSelected) {
		return allSelected;
	}
	var disabledEle = legData.filter(function(d) {
		return d.disabled;
	});
	return !disabledEle.length;
};

legend_helpers.getFilterBoxSize = function(colorBox, fontSize) {
	var colorBoxSize = defined(colorBox.size) ? +colorBox.size : fontSize,
		colorBoxWidth = defined(colorBox.width) ? +colorBox.width : colorBoxSize,
		colorBoxHeight = defined(colorBox.height) ? +colorBox.height : colorBoxSize;
	return [colorBoxWidth, colorBoxHeight];
};

legend_helpers.updateData = function(d, i, chartObj) {
	if (colorscale_helpers.isQuantileColorScale(chartObj)) {
		chartObj.axes.clr.scale = $ZC.scale.getScale(chartObj, chartObj.dataObject, null, null, null, chartObj.dataset, "clr");
	} else {
		legend_helpers.updateBubblePieData(d, i, chartObj); //For bubblepie chart, it is mandatory to mark every data point to be disabled if the series is disabled
		legend_helpers.updateHierarchyData(d, i, chartObj); //For hierarchy chart, chartObj.seriesdata reference is not equal to chartObj.systemConf.seriesdata.chartdata reference as the chartObj.seriesdata is simplecloned. So, storing disable information at systemconf is mandatory
		chartObj.dataObject.data = chartObj.systemConf;
	}
};

legend_helpers.isContinousLegend = function(chartObj) {
	var colorBandOptions = chartObj.systemConf.legend.colorBand,
		isQuantileScale = colorscale_helpers.isQuantileColorScale(chartObj);

	if (!isQuantileScale) {
		return false;
	} else {
		var userRanges = colorBandOptions.ranges;
		if (userRanges && userRanges.length) {
			var type = colorBandOptions.type;
			return type === "continuousDiscrete" || type === "continuous";
		} else {
			return true;
		}
	}
};

legend_helpers.getColorBandDimension = function(chartObj) {
	var chartarea = chartObj.chartarea,
		legendOptions = chartObj.systemConf.legend,
		colorbandOptions = legendOptions.colorBand,

		orient = legend_helpers.getOrient(legendOptions),
		isHorizontal = orient === "bottom" || orient === "top",

		maxWidth = legendOptions.maxWidth,
		maxHeight = legendOptions.maxHeight,

		w = colorbandOptions.width,
		h = colorbandOptions.height;

	if (isHorizontal) {
		maxWidth = maxWidth || ZC_LEGEND_maxWidthHoriRatio;
		maxHeight = maxHeight || ZC_LEGEND_maxHeightHoriRatio;
		w = w || "40%";
		h = h || 14;
	} else {
		maxWidth = maxWidth || ZC_LEGEND_maxWidthVertRatio;
		maxHeight = maxHeight || ZC_LEGEND_maxHeightVertRatio;
		w = w || 14;
		h = h || "70%";
	}

	maxWidth = perToPx(maxWidth, chartarea.width);
	maxHeight = perToPx(maxHeight, chartarea.height);
	w = perToPx(w, chartarea.width);
	h = perToPx(h, chartarea.height);

	w = mathMin(w, maxWidth);
	h = mathMin(h, maxHeight);

	return [w, h];
};

legend_helpers.getOrient = function(legendOptions) {
	var hAlign = legendOptions.hAlign,
		vAlign = legendOptions.vAlign,
		orient = "right";

	if (vAlign === "bottom") {
		orient = "bottom";
	} else if (vAlign === "top") {
		orient = "top";
	} else if (hAlign === "left") {
		orient = "left";
	}

	return orient;
};

legend_helpers.getLabelIndex = function(value, colorbandOptions) {
    var userStops = colorbandOptions.stops,
        userRanges = colorbandOptions.ranges,
        labelIndex;

    if (userStops && userStops.length) {
        userStops.some(function(d, i) {
            var condition = d === value;
            if (condition) {
                labelIndex = i;
                return true;
            }
        });
    } else if (userRanges && userRanges.length) {
        userRanges.some(function(d, i) {
            var condition;
            if (isArray(d)) {
                if (isArray(value)) {
                   condition = d[0] === value[0] && d[1] === value[1];
                } else {
                    condition = d[0] === value;
                }
            } else {
                condition = d === value;
            }
            if (condition) {
                labelIndex = i;
                return true;
            }
        });
    }
    return labelIndex;
};

legend_helpers.hexToRGB = function(color) { //Converting hex colors to rgb
	if (color && (color.toString().indexOf("#") == -1)) {
		return color;
	}
	var bigint = pInt(d3_rgb(color).toString().replace(/#/g, ""), 16);
	return "rgb(" + ((bigint >> 16) & 255) + "," + ((bigint >> 8) & 255) + "," + (bigint & 255) + ")";
};

legend_helpers.filterSelectionByColor = function(selection, selectedSeries, filterIndex, chartObj) {
	if (defined(filterIndex)) {
		var filterValue = chartObj.systemConf.legend.colorBand.ranges[filterIndex],
			colorScale = chartObj.axes.clr.scale,
			isOrdinal = chartObj.dataset.getDataType("clr") === ORDINAL_DATATYPE,
			condition;
		if (filterValue.length === 1 && !isOrdinal) {
			var domain = d3_values(colorScale.domain());
			domain.splice(0, 2);
			filterValue = [domain[filterIndex * 2], domain[filterIndex * 2 + 1]];
		}
		return selection.filter(function(d) {
			var dt = d.data || d;
			var clr = chartObj.dataset.getClr(dt);
			condition = isWithinColorRange(clr, chartObj, colorScale);
			if (isOrdinal) {
				condition = condition && clr === filterValue[0];
			} else {
				var colorByIndex = getColorByIndex(chartObj, filterIndex);
				colorByIndex = (typeof colorByIndex === "string") ? rgb2hex(colorByIndex) : colorByIndex; // Need to remove in d3.v4
				condition = condition && (clr >= filterValue[0] && clr <= filterValue[1]);

				if (clr === filterValue[0]) {
					condition = condition && (colorByIndex === colorScale(clr));
				}
				if (clr === filterValue[1]) {
					condition = condition && (colorByIndex === colorScale(clr));
				}
			}
			return selectedSeries ? condition : !condition;
		});
	} else {
		return selection;
	}
};
