//$Id$
$ZC.legendHighlight = function(chartObj, container) {
    var legendHighlight = {},
        origProp = {},
        defsLocation = chartObj.defsLocation,
        userdata = chartObj.userdata,

        // data processor helpers

        dataProcessorHelpers = $ZC.DataProcessor.helpers,
        getChartType = dataProcessorHelpers.getChartType,
        filterSelectionByColor = legend_helpers.filterSelectionByColor;
    
    /*
     * Highlight effects for Bar,pie,funnel,web,area,line,scatter
     * patterns--> diagonalfill,strokeEnlarge,fadeout
     * diagonal fill not supported for line,scatter
     * fadeout: fillopacity except line and scatter, strokeopacity for line and scatter
     */
    legendHighlight.highlightSelectedSeries = function(legdata, index, filterIndex) {
        legendHighlight.baseClr = getColorByIndex(chartObj, (defined(filterIndex) ? filterIndex : index));

        switch (chartObj.plot.renderer.mode) {
            case "svg": {
                highlightSVGSeries(legdata, index, filterIndex);
                break;
            }
            case "canvas": {
                highlightCanvasSeries(chartObj, legdata, index, filterIndex);
                break;
            }
            default: {
                highlightNonSVGSeries(legdata, index);
            }
        }
    };
    
    legendHighlight.highlightUnselectedSeries = function(legdata, index, filterIndex) {
        var unselSeriesObj = getUnselectedSeries(index, filterIndex);
        var unselectedSeries = unselSeriesObj.unselectedSeries;
        var indices = unselSeriesObj.indices;
        var charttypes = unselSeriesObj.charttypes;
        var length = unselectedSeries.length;
        for (var i = 0; i < length; i++) {
            var chartType = $ZC.charttypenames.get(charttypes[i]);
            var unselecteffect = this.getLegendHighlightEffect(indices[i], "unselectedSeries");
            var prop = legendHighlight.getEffect(unselecteffect, chartType, null, null, indices[i]);
            applyProp(unselectedSeries[i], prop, indices[i], "unselect");
        }
    };
    
    legendHighlight.reset = function(legdata, index, filterIndex, resetOriginalProp) {
        switch (chartObj.plot.renderer.mode) {
            case "svg": {
                resetSelectedSeries(legdata, index, filterIndex);
                resetUnselectedSeries(legdata, index, filterIndex);
                break;
            }
            case "canvas": {
                resetCanvasSeries();
                break;
            }
        }


        if (resetOriginalProp !== false) { //Test_662.json
            origProp = {}; //Deleting the stored style values, will help you to know whether the element is already highlighted are not.
        }
    };
    
    legendHighlight.getLegendHighlightEffect = function(index, seriesType, plotOrLegend) {
        var chartName = getChartType(null, "string", index, userdata);
        var plotOptions = (chartObj.isAxisCategory || chartObj.isNonAxisMultiSeriesCategory) ? chartObj.seriesdata[index].plotoptions : chartObj.seriesdata[0].plotoptions;
        var commonPlotOptions = chartObj.systemConf.chart.plot.plotoptions[chartName];
        var ef = (plotOptions && plotOptions.legendHighlightEffect && commonPlotOptions.legendHighlightEffect) ?
            ($ZC.utils.getValueOfPlotOptions(plotOptions.legendHighlightEffect, commonPlotOptions.legendHighlightEffect)(seriesType)) :
            (commonPlotOptions.legendHighlightEffect && commonPlotOptions.legendHighlightEffect[seriesType]);
        var legendEffect = chartObj.systemConf.legend.highlightEffect[seriesType];
        if (plotOrLegend === "plot") {
            return ef || (legendEffect && legendEffect !== "none" ? legendEffect : defaultEffect(chartName));
        } else {
            return ef || legendEffect || defaultEffect(chartName);
        }
    };
    
    
    legendHighlight.getEffect = function(effect, chartName, baseClr, additionalStyles, index) {
        baseClr = baseClr || legendHighlight.baseClr;

        var userEffect = invokeFunction(effect, chartObj, baseClr, patternThemes);
        if (userEffect && userEffect.constructor == Object){
            return userEffect;
        }
        effect = (effect === null || invalidEffect(effect, chartName)) ? defaultEffect(chartName) : effect;
        var eStyle = {};

        if (chartObj.patternPallete && /(flip|moody|illuminate)/.test(effect)) {
            var svgPattern = chartObj.patternPallete[(index || 0) % chartObj.patternPallete.length];
            var prop = patternThemes[svgPattern] ? patternThemes[svgPattern] : addPatternThemeByEle(svgPattern);

            if (prop.ele) {
                // If custom pallete is given, moody effect will be applied
                prop.bgClr_base = getColorCopy(baseClr, 1, "darker", d3_hsl);
            } else {
                var colorCopy = (effect === "flip")
                            ? baseClr
                            : getColorCopy(baseClr, 1, ((effect === "moody") ? "darker" : "brighter"), d3_hsl);

                // if(isUndefined(prop.bgClr_base)) {
                    prop.bgClr_base = colorCopy;
                // }
                // if(isUndefined(prop.stroke_base)) {
                    prop.stroke_base = colorCopy;
                // }

                if (effect === "flip") {
                    prop = simpleClone(prop); // To avoid inplace change in the pattern object

                    //swapping bgClr_base and stroke_base
                    var temp = prop.bgClr_base;
                    prop.bgClr_base = prop.stroke_base;
                    prop.stroke_base = temp;

                    //swapping bgClr and stroke
                    var copy_bgClr = stringifiedArrayReverse(prop.bgClr),
                        copy_stroke = stringifiedArrayReverse(prop.stroke);

                    prop.bgClr = copy_stroke;
                    prop.stroke = copy_bgClr;
                }
            }

            var patternId = effects.get(svg_patterns,"svgPatternFill",prop);
            eStyle.fill = "url(" + defsLocation + "#" + patternId + ")";
        } else if (/(saturate|grayscale|blur|glow|shadow)/.test(effect)) {
            var filterType = /(saturate|grayscale)/.test(effect) ? "colorMatrix" : "dropShadow",
                options;
            switch (effect) {
                case "saturate":
                    options = {
                        "type": "saturate",
                        "values": "0.3"
                    };
                    break;
                case "grayscale":
                    options = {
                        "type": "matrix",
                        "values": ".33 .33 .33 0 0 .33 .33 .33 0 0 .33 .33 .33 0 0 0 0 0 1 0"
                    };
                    break;
                case "blur":
                    options = {
                        "blur": 6,
                        "source": "SourceGraphic"
                    };
                    break;
                case "glow":
                    options = {
                        "blur": 3,
                        "color": "gold"
                    };
                    break;
                case "shadow":
                    options = {
                        "dx": 5,
                        "dy": 5,
                        "blur": 3
                    };
            }
            var filterId = effects.get(svg_filters,filterType, options);
            eStyle.filter = "url(" + defsLocation + "#" + filterId + ")";
        } else if (/(moody|illuminate)/.test(effect)) {
            var ismoody = (effect === "moody");
            eStyle.fill = (baseClr.indexOf("url") != -1) ? (ismoody ? "grey" : "silver") : getColorCopy(baseClr, 0.6, (ismoody ? "darker" : "brighter"), d3_hsl);
        } else if (effect === "strokeEnlarge") {
            eStyle["stroke-width"] = 3;

            if (additionalStyles) {
               for(var a in additionalStyles) {
                    eStyle[a] = additionalStyles[a];
                }
            }

        } else if (effect === "fadeOut") {
            eStyle["fill-opacity"] = eStyle["stroke-opacity"] = 0.3;
        } else if (effect !== "none") {
            var prop = patternThemes[effect];
            if (prop) {
                prop.bgClr_base = baseClr;
                prop.stroke = "transparent,white,white"; //need to check
                var patternId = effects.get(svg_patterns,"svgPatternFill",prop);
                eStyle.fill = "url(" + defsLocation + "#" + patternId + ")";
            } else {
                eStyle.fill = baseClr;
            }
        }
        return eStyle;
    };
    
    function resetSelectedSeries(legdata, index, filterIndex) {
        legendHighlight.selSeriesObj = getSelectedSeries(index, filterIndex);
        var selecteffect = legendHighlight.getLegendHighlightEffect(index, "selectedSeries");
        var selectedSeries = legendHighlight.selSeriesObj.selectedSeries;
        var charttypes = legendHighlight.selSeriesObj.charttypes;
        var length = selectedSeries.length;
        for (var i = 0; i < length; i++) {
            var chartType = $ZC.charttypenames.get(charttypes[i]);
            var prop = legendHighlight.getEffect(selecteffect, chartType, null, null, index);
            resetProp(selectedSeries[i], prop, index, "select");
        }
    }
    
    function resetUnselectedSeries(legdata, index, filterIndex) {
        legendHighlight.unselSeriesObj = getUnselectedSeries(index, filterIndex);
        var unselectedSeries = legendHighlight.unselSeriesObj.unselectedSeries;
        var indices = legendHighlight.unselSeriesObj.indices;
        var charttypes = legendHighlight.unselSeriesObj.charttypes;
        var length = unselectedSeries.length;
        for (var i = 0; i < length; i++) {
            var chartType = $ZC.charttypenames.get(charttypes[i]);
            var unselecteffect = legendHighlight.getLegendHighlightEffect(indices[i], "unselectedSeries");
            var prop = legendHighlight.getEffect(unselecteffect, chartType, null, null, indices[i]);
            resetProp(unselectedSeries[i], prop, indices[i], "unselect");
        }
    }
    
    function resetCanvasSeries() {
        chartObj.container.select(".highlight-canvas").style("visibility", "hidden");
    }
    
    function getSelectedSeries(index, filterIndex) {
        var seriesHolder = container.selectAll("g.seriesholder");
        var type = getChartType(null, "object", index, userdata).value;
        var charttypes = [],
            selectedSeries = [];
        collectSelectedSeries(type, index);
        return {
            "selectedSeries": selectedSeries,
            "charttypes": charttypes
        };
        
        function collectSelectedSeries(type, index) {
            /*
             * This function adds the selected series to the selectedSeries array and respective chart type to the charttypes array.
             * Scatter chart = outer scatter series + inner scatter series.
             * Area chart = area series + line series.
             * Line chart = line series + scatter series.(for IE and SAFARI)
             */
            var series;
            if (isHeatMapChartType(type)) {
                var heatmapSeries = seriesHolder.selectAll("g.heatmapseries_" + index);
                series = heatmapSeries.selectAll("path");
            } else if (isBarChartType(type)) {
                var barSeries = seriesHolder.selectAll("g.barseries_" + index);
                series = barSeries.selectAll("path.zcbar");
            } else if (type == $ZC.charttype.line) {
                var lineSeries = seriesHolder.selectAll("g.lineseries_" + index);
                series = lineSeries.selectAll("path.line");
            } else if (isAreaChartType(type) || type == $ZC.charttype.web) {
                var areaSeries = seriesHolder.selectAll("g.areaseries_" + index);
                series = areaSeries.selectAll("path.area");
            } else if (isScatterChartType(type)) {
                var scatterSeries = seriesHolder.selectAll("g.scatterseries_" + index);
                series = scatterSeries.selectAll("g.scatter");
                series = filterSelectionByColor(series, true, filterIndex, chartObj);

                var outerScatters = series.selectAll("path.outer");
                var innerScatters = series.selectAll("path.inner");

                selectedSeries.push(outerScatters, innerScatters);
                charttypes.push(type, type);
                return;
            } else if (isPieChartType(type)) {
                var pieSeries = seriesHolder.selectAll("g.piegroup");
                series = pieSeries.selectAll(defined(filterIndex) ? "path" : "path#arc_" + index);
            } else if (type == $ZC.charttype.funnel) {
                var funnelSeries = seriesHolder.selectAll("g.funnelgroup");
                var trapezoidgroup = funnelSeries.selectAll("g.trapezoidgroup");
                series = trapezoidgroup.selectAll(defined(filterIndex) ? "path" : "path#funnel" + index);
            } else if (type == $ZC.charttype.pyramid) {
                var pyramidSeries = seriesHolder.selectAll("g.pyramidgroup");
                series = pyramidSeries.selectAll(defined(filterIndex) ? "g" : "g#section_" + index);
            } else if ($ZC.chartcategory.hierarchy.indexOf(type) != -1) {
                var burstseries = seriesHolder.selectAll("g.hierarchychart");
                series = burstseries.selectAll(".series_" + index);
            }

            series = filterSelectionByColor(series, true, filterIndex, chartObj);

            selectedSeries.push(series);
            charttypes.push(type);
            
            if (isAreaChartType(type) || type == $ZC.charttype.web) {
                collectSelectedSeries($ZC.charttype.line, index);
            }
            if (type == $ZC.charttype.line && ($Browser.IE || $Browser.SAFARI)) {
                collectSelectedSeries($ZC.charttype.scatter, index);
            }
        }
    }
    
    function getUnselectedSeries(index, filterIndex) {
        var unSelectedSeries = [],
            indices = [],
            charttypes = [];
        var seriesHolder = container.selectAll("g.seriesholder");
        collectUnselectedSeries(null, index);
        return {
            "unselectedSeries": unSelectedSeries,
            "indices": indices,
            "charttypes": charttypes
        };
        
        function collectUnselectedSeries(subcharttype, index) {
            /*
             * This function adds the unselected series to the selectedSeries array, respective chart type to the charttypes array, respective index to the indices array.
             * Scatter chart = outer scatter series + inner scatter series.
             * Area chart = area series + line series.
             * Line chart = line series + scatter series.(for IE and SAFARI)
             */
            var series, type;
            if ((chartObj.isAxisCategory && !chartObj.dataObject.pieWithAxis) || chartObj.isNonAxisMultiSeriesCategory) {
                chartObj.seriesdata.forEach(function(d, i) {
                    if ((d.disabled != true) && (defined(filterIndex) ? (i === index) : (i !== index))) {
                        type = subcharttype || getChartType(null, "object", i, userdata).value;
                        var selectionIndex = defined(filterIndex) ? index : i;
                        if (isHeatMapChartType(type)) {
                            var heatmapSeries = seriesHolder.selectAll("g.heatmapseries_" + selectionIndex);
                            series = heatmapSeries.selectAll("path");
                        } else if (isBarChartType(type)) {
                            var barSeries = seriesHolder.selectAll("g.barseries_" + selectionIndex);
                            series = barSeries.selectAll("path.zcbar");
                        } else if (type == $ZC.charttype.line) {
                            var lineSeries = seriesHolder.selectAll("g.lineseries_" + selectionIndex);
                            series = lineSeries.selectAll("path.line");
                        } else if (isAreaChartType(type) || type == $ZC.charttype.web) {
                            var areaSeries = seriesHolder.selectAll("g.areaseries_" + selectionIndex);
                            series = areaSeries.selectAll("path.area");
                        } else if (isScatterChartType(type)) {
                            var scatterSeries = seriesHolder.selectAll("g.scatterseries_" + selectionIndex);
                            series = scatterSeries.selectAll("g.scatter");
                            series = filterSelectionByColor(series, false, filterIndex, chartObj);

                            var outerScatters = series.selectAll("path.outer");
                            var innerScatters = series.selectAll("path.inner");

                            unSelectedSeries.push(outerScatters, innerScatters);
                            charttypes.push(type, type);
                            indices.push(selectionIndex, selectionIndex);
                            return;
                        } else if ($ZC.chartcategory.hierarchy.indexOf(type) != -1) {
                            var burstseries = seriesHolder.selectAll("g.hierarchychart");
                            series = burstseries.selectAll(".series_" + selectionIndex);
                        }

                        series = filterSelectionByColor(series, false, filterIndex, chartObj);

                        unSelectedSeries.push(series);
                        indices.push(i);
                        charttypes.push(type);
                    }
                    
                })
            } else {
                type = subcharttype || getChartType(null, "object", index, userdata).value;
                var prefix_ID,
                    processFunc = function(d, i) {
                    var ele = d3_select(this),
                        unselect = defined(filterIndex)
                                        ? !filterSelectionByColor(ele, false, filterIndex, chartObj).empty()
                                        : (this.getAttribute("id") != prefix_ID + index);
                    if (unselect) {
                        unSelectedSeries.push(ele);
                        indices.push(i);
                        charttypes.push(type);
                    }
                    return unselect;
                };

                if (type == $ZC.charttype.pie || type == $ZC.charttype.bubblepie) {
                    prefix_ID = "arc_";
                    series = seriesHolder.selectAll("g.piegroup").selectAll("path").filter(processFunc);
                } else if (type == $ZC.charttype.funnel) {
                    prefix_ID = "funnel";
                    series = seriesHolder.selectAll("g.trapezoidgroup").selectAll("path").filter(processFunc);
                } else if (type == $ZC.charttype.pyramid) {
                    prefix_ID = "section_";
                    var pyramidGroup = seriesHolder.selectAll("g.pyramidgroup");
                    series = pyramidGroup.selectAll("g").filter(processFunc);
                }
            }
            
            if (isAreaChartType(type) || type == $ZC.charttype.web) {
                collectUnselectedSeries($ZC.charttype.line, index);
            }
            if (type == $ZC.charttype.line && ($Browser.IE || $Browser.SAFARI)) {
                collectUnselectedSeries($ZC.charttype.scatter, index);
            }
        }
    }
    
    function applyProp(series, prop, index, type) {
        if (prop == null || prop.constructor != Object || series == null || series.node() == null) {
            return
        }
        var key = getKey(series, index, type),
            currentOrigProp = {},
            multipleStyles = hasMultipleStyles(index);

        for (var attr in prop) {
            if (prop.hasOwnProperty(attr)) {
                if (multipleStyles) {
                    currentOrigProp[attr] = currentOrigProp[attr] || [];
                    series.each(function(){
                        currentOrigProp[attr].push(d3.select(this).style(attr));
                    });
                } else {
                    currentOrigProp[attr] = series.style(attr);
                }

                var val = $ZC.utils.getVal(prop[attr]);
                if ($Browser.SAFARI || $Browser.IE) {
                    series
                        .style(attr, val)
                        .attr(attr, val);
                } else {
                    series
                        .style(attr, val);
                }
            }
        }
        origProp[key] = origProp[key] || currentOrigProp; //If the style values are already stored, don't store again. If u store again, the origProp[key] object will be replaced with the highlighting props instead of containing the style values of the original element.
    }
    
    function resetProp(series, prop, index, type) {
        if (prop == null || prop.constructor != Object || series == null || series.node() == null) {
            return
        }
        
        if (chartObj.plot.renderer.mode !== "svg") {
            setSeriesVisible(series, false);
            return;
        }
        
        var key = getKey(series, index, type),
            currentOrigProp = origProp[key],
            multipleStyles = hasMultipleStyles(index);

        for (var attr in currentOrigProp) {
            if (currentOrigProp.hasOwnProperty(attr)) {
                var val = currentOrigProp[attr],
                    actualVal = function() {
                        return multipleStyles ? val[count++] : val;
                    },
                    count = 0;
                if ($Browser.SAFARI || $Browser.IE) {
                    series
                        .style(attr, actualVal)
                        .attr(attr, actualVal);
                } else {
                    series
                        .style(attr, actualVal);
                }
            }
        }
    }
    
    function getKey(series, index, selectionType) {
        /*
         * This function returns a unique key that is generated from the localName, class, id of the series element and index.
         * The key will be used to store the style values of the element on legend highlighting. The stored values will be applied to the element on unhighlighting.
         */
        var node = series.node();
        var localName = node.localName,
            className = node.getAttribute("class"),
            idName = node.getAttribute("id");
        var KEY = localName + "_" + index;
        if (className != null) {
            KEY += "_" + className;
        }
        if (idName != null) {
            KEY += "_" + idName;
        }
        if (selectionType !== null) {
            KEY += "_" + selectionType;
        }
        return KEY;
    }
    
    function setSeriesVisible(series, visible) {
        series.style("visibility", (visible ? "visible" : "hidden"));
    }
    
    function defaultEffect(chartName) {
        if (hasPatternPallete(chartObj)) {
            return /(area|arearange)/.test(chartName) ?
                "moody" :
                "strokeEnlarge";
        }

        return /(bar|bullet|gantt|area|arearange|pie|funnel|pyramid|web|sunburst|treemap|heatmap)/.test(chartName) ?
            "hatchfill" :
            "strokeEnlarge";
    }
    
    function invalidEffect(effect, chartName) {
        return /(line|scatter)/.test(chartName) && /(((\D)+fill$)|illuminate|moody|flip)/.test(effect);
    }
    
    function highlightSVGSeries(legdata, index, filterIndex) {
        legendHighlight.selSeriesObj = getSelectedSeries(index, filterIndex);
        var selecteffect = legendHighlight.getLegendHighlightEffect(index, "selectedSeries");
        var selectedSeries = legendHighlight.selSeriesObj.selectedSeries;
        var charttypes = legendHighlight.selSeriesObj.charttypes;
        var length = selectedSeries.length;
        
        for (var i = 0; i < length; i++) {
            var chartType = $ZC.charttypenames.get(charttypes[i]);
            var prop = legendHighlight.getEffect(selecteffect, chartType, null, null, index);
            applyProp(selectedSeries[i], prop, index, "select");
        }
    }
    function highlightCanvasSeries(chartObj, legdata, index, filterIndex){
        if (!chartObj.chartRendered) {
            return
        }
        
        var context = chartObj.highlightContext || (chartObj.highlightContext = updateCanvasContext(chartObj, "highlight-canvas")),
            data = chartObj.systemConf,
            effect = chartObj.legendHighlight.getLegendHighlightEffect(index, "selectedSeries");
        if (effect === "none") {
            return;
        }
        var chart = chartObj.chartTypes[index],
            chartName = chart.name,
            chartType = chart.value,
            renderer = createCanvasRenderer(chartName),
            currentSeries = chartObj.seriesdata[index],
            pattern = null;
        
        renderer_helpers.setRendererProp(chartObj, renderer, currentSeries, index, {
            "labelQueue": []
        });
        
        switch (chartName) {
            case "bar":
            case "area":
            case "heatmap":
            {
                var fillOpacity = chartObj.systemConf.chart.plot.plotoptions[chartName].fillOpacity;
                var prop = patternThemes[effect];
                if (prop) {
                    if (defined(filterIndex)) {
                        pattern = getCanvasPattern(prop);
                    } else {
                        pattern = getCanvasPattern(prop)(legendHighlight.baseClr, fillOpacity);
                    }
                }
                break;
            }
            case "scatter":
            case "bubble":
            case "line":
            {
                pattern = chartObj.legendHighlight.getEffect(effect, chartType);
            }
        }
        drawCanvas(chartObj, currentSeries, renderer, context, pattern, filterIndex);
    }

    function getCanvasPattern(prop) {
        return function(baseClr, fillOpacity) {
            prop.bgClr_base = canvasUtils.rgba(baseClr, fillOpacity);
            prop.stroke = "transparent,white,white"; //need to check
            return effects.get(canvas_patterns,"canvasPatternFill",prop);
        };
    }
    
    function highlightNonSVGSeries(legdata, index) {
        
        var selecteffect = legendHighlight.getLegendHighlightEffect(index, "selectedSeries");
        if (selecteffect === "none") {
            return;
        }
        
        legendHighlight.baseClr = getColorByIndex(chartObj, index);
        
        var chartType = chartObj.chartTypes[index];
        var cName = chartType.name;
        var cTypeNumeric = chartType.value;
        
        var selectedSeriesObj = getSelectedSeries(index);
        
        if (selectedSeriesObj.selectedSeries[0].length > 0) {
            setSeriesVisible(selectedSeriesObj.selectedSeries[0], true);
            return;
        }
        
        if (!chartObj.chartRendered) return;
        
        /*
         * Disable animation during the chartObj drwaing proccess and disable the datalabels also.
         */
        
        chartObj.systemConf.chart.plot.animation.enabled = false;
        chartObj.systemConf.chart.plot.plotoptions[cName].animation.enabled = false;
        if (chartObj.seriesdata[index].plotoptions && chartObj.seriesdata[index].plotoptions.animation) {
            chartObj.seriesdata[index].plotoptions.animation.enabled = false;
        }
        
        var renderer = createSVGRenderer(cName);
        var currOrdereddata = chartObj.systemConf.seriesdata.chartdata[index]
        var barDatasetIndex = 0;
        if (cTypeNumeric === $ZC.charttype.bar || cTypeNumeric === $ZC.charttype.stkdbar) {
            for (var i = 0; i < index; i++) {
                chartType = chartObj.chartTypes[i];
                cTypeNumeric = chartType.value;
                if ((chartObj.seriesdata[i].disabled == undefined || !chartObj.seriesdata[i].disabled) && isBarChartType(cTypeNumeric)) {
                    barDatasetIndex = barDatasetIndex + chartObj.seriesdata[i].data.length;
                }
            }
        }
        renderer_helpers.setRendererProp(chartObj, renderer, currOrdereddata, index, {
            "labelQueue": [],
            "barDatasetIndex": barDatasetIndex
        });
        drawSVG(chartObj, currOrdereddata, renderer);
        
        highlightSVGSeries(legdata, index);
        return true;
    }

    function hasMultipleStyles(seriesIndex) {
        var chartType = getChartType(null, NUMERIC_DATATYPE, seriesIndex, userdata);

        return $ZC.chartcategory.hierarchy.indexOf(chartType) != -1 ||
            colorscale_helpers.isQuantileColorScale(chartObj) ||
            isBubbleChartType(chartType) ||
            chartObj.rendererConf[chartObj.isAxisCategory ? seriesIndex : 0].thresholdType;
    }

    return legendHighlight;
};