//$Id$
(function() {
    var defaultSelectAllOptions = {
        show: false,
        strokeColor: "#262626",
        strokeWidth: 1,
        fillColor: "transparent",
        fontColor: "#262626",
        disabledOpacity: 0.3,
        tooltip: {
            enabled: true,
            color: "red",
            content: "Select atleast one legend"
        },
        events: {
            click: null
        }
    };
    var getLegendData = legend_helpers.getLegendData,
        isDisabled = legend_helpers.isDisabled,
        isAllEnabled = legend_helpers.isAllEnabled,
        getClr = legend_helpers.getClr,
        toggleChartEvents = legend_helpers.toggleChartEvents,
        updateData = legend_helpers.updateData;

    $ZC.legend.selectAllBox = function(container, chartObj, states) {
        defaultSelectAllOptions.events.click = chartObj.legend.component.toggleSelectAll = toggleSelectAll;
        var legData = getLegendData(chartObj),

            legendOptions = chartObj.systemConf.legend,
            colorBox = legendOptions.colorBox,
            selectAllBoxOptions = mergeJSON(defaultSelectAllOptions, legendOptions.selectAllBox, true),
            itemDisabledOptions = legendOptions.itemDisabledStyle,
            disabledColorBox = itemDisabledOptions.colorBox || {},

            priorityArray = [legendOptions, chartObj.systemConf.canvas, $ZCG],
            fontSize = +($ZC.utils.getVal((legendOptions.fontSize) || chartObj.systemConf.canvas.fontSize)),
            fontColor = getValByPriority(priorityArray, "fontColor"),
            imagePallete = legendOptions.imagePallete ? $ZC.legutil.imagePallete.get(legendOptions.imagePallete) : null,

            colorBoxSize = legend_helpers.getFilterBoxSize(colorBox, fontSize),
            colorBoxWidth = colorBoxSize[0],
            colorBoxHeight = colorBoxSize[1],
            selectAllFilterSize = Math.min(colorBoxWidth, colorBoxHeight) - ((legendOptions.imagePallete || (!defined(legendOptions.shapes) || colorBox.shape === "square")) ? 0 : +colorBox.strokeWidth),

            svgNode = chartObj.container.node(),
            selectAllBoxTooltipOptions = selectAllBoxOptions.tooltip,
            disabledOpacity = selectAllBoxOptions.disabledOpacity,

            defaultTooltipColor,
            legendlabel,
            legendFilter;

        states.allSelected = defined(selectAllBoxOptions.selected) ? selectAllBoxOptions.selected : states.allSelected;

        var selectAllFilter = appendEle(container, "div", [1], ".", "class", "selectAll");
        selectAllFilter
            .style({
                "display": selectAllBoxOptions.show ? "inline-block" : "none",
                "width": selectAllFilterSize + "px",
                "height": selectAllFilterSize + "px",
                "border": selectAllBoxOptions.strokeWidth + "px solid " + selectAllBoxOptions.strokeColor,
                "background-color": selectAllBoxOptions.fillColor,
                "float": "left",
                "position": "absolute",
                "box-sizing": "border-box",
                "text-align": "center",
                "line-height": selectAllFilterSize + "px",
                "font-family": colorBox.fontFamily,
                "font-size": (colorBox.fontSize || (fontSize - 2)) + "px",
                "color": function(d) {
                    return selectAllBoxOptions.fontColor || (isDisabled(d, states.allSelected, states.isFirstClickAfterDeselection) ? disabledColorBox.fontColor : colorBox.fontColor);
                }
            })
            .html(isAllEnabled(legData, states.allSelected) ? (colorBox.text || "&#10003;") : (disabledColorBox.text || ""));

        container
            .style("cursor", selectAllBoxOptions.show ? "pointer" : null);

        var mouseHandlers = {
                "click": selectBoxClickFunction
            },
            touchHandlers = {
                "tap": selectBoxClickFunction
            };
        if (selectAllBoxOptions.show) {
            attachevents(container, mouseHandlers, touchHandlers);
        } else {
            detachevent(container, null, mouseHandlers);
            detachevent(container, null, touchHandlers);
        }

        if (chartObj.eventHandler) {
            var tooltip = chartObj.eventHandler.tooltipElement;
            tooltip.hide(tooltip.element);
        }

        return {
            element: selectAllFilter,
            options: selectAllBoxOptions,
            size: selectAllFilterSize
        };

        function selectBoxClickFunction() {
            if (!(states.allSelected && !isAllEnabled(legData, states.allSelected))) {
                states.allSelected = !states.allSelected;
            }
            states.isFirstClickAfterDeselection = false;
            selectAllFilter
                .html(states.allSelected ? colorBox.text || "&#10003;" : disabledColorBox.text || "");
            var userFunction = selectAllBoxOptions.events[chartObj.zcEventType || "click"];
            invokeFunction(userFunction, d3.event, states.allSelected, chartObj);
        }

        function toggleSelectAll() {
            filterAllAxesNUpdate();
            applyStyleToFilterAndLabel();
            toggleChartEvents(chartObj, states.allSelected);
            legend_helpers.fadeChart(states.allSelected, chartObj, disabledOpacity);

            if (selectAllBoxTooltipOptions.enabled) {
                var tooltip = chartObj.eventHandler.tooltipElement;
                if (!defaultTooltipColor) {
                    defaultTooltipColor = tooltip.element.selectAll("div#tooltipcontent").style("color");
                }
                var tooltip_data = chartObj.systemConf.tooltip;
                var tooltipBorderWidth = shorthand_px(defined(tooltip_data.borderWidth) ? tooltip_data.borderWidth : 1)
                if (!states.allSelected) {
                    var coordinate = getMousePosition(svgNode, d3.event);
                    coordinate = {
                        x: coordinate[0],
                        y: coordinate[1]
                    };
                    var evt = getEventObject(d3.event);
                    var position = getTooltipPosition(evt, coordinate, 0, 0, chartObj, {}, false, false, false, true, false);
                    var dLeft = coordinate.x - evt.clientX,
                        dTop = coordinate.y - evt.clientY;
                    position.dLeft = dLeft;
                    position.dTop = dTop;
                    position.helpers.color = selectAllBoxTooltipOptions.color;
                    position.helpers.backgroundColor = selectAllBoxTooltipOptions.backgroundColor;
                    position.helpers.borderColor = selectAllBoxTooltipOptions.borderColor;
                    applyTooltipStyle(tooltip, "center", null, (selectAllBoxTooltipOptions.fontColor || defaultTooltipColor));
                    tooltip.show(chartObj, position, selectAllBoxTooltipOptions.content, false, position.helpers);
                    setTimeout(function() {
                        var tooltip = chartObj.eventHandler.tooltipElement;
                        applyTooltipStyle(tooltip, null, tooltipBorderWidth, defaultTooltipColor);
                        tooltip.hide(tooltip.element);
                    }, 4000);
                } else {
                    var tooltip = chartObj.eventHandler.tooltipElement;
                    applyTooltipStyle(tooltip, null, tooltipBorderWidth, defaultTooltipColor);
                    tooltip.hide(tooltip.element);
                }
            }
        }

        function filterAllAxesNUpdate() {
            if (!legData) { //If legend is disabled, legData will not be available
                legData = getLegendData(chartObj);
            }
            if (states.allSelected) {
                legData.forEach(function(d, j) {
                    d.disabled = !states.allSelected;
                    updateData(d, j, chartObj);
                });
                chartObj.addRemoveSeries(legData, 0, [0, 0], true);
            }
        }

        function applyStyleToFilterAndLabel() {
            if (!legendFilter) {
                var legendContainer = chartObj.legendarea.element;
                legendFilter = legendContainer.selectAll("div.legendfilterbox");
                legendlabel = legendContainer.selectAll("span.legendlabel");
            }

            $ZC.legend.setColorBoxBackground(legendFilter, chartObj, states);
            var fontClr, txtDec;
            if (states.allSelected) {
                fontClr = fontColor,
                    txtDec = null;
            } else {
                fontClr = getClr(true, fontColor, itemDisabledOptions),
                    txtDec = (JSON.parse(itemDisabledOptions.strikeout) ? "line-through" : null);
            }
            legendlabel
                .style({
                    "color": fontClr,
                    "text-decoration": txtDec
                });
        }

        function applyTooltipStyle(tooltip, align, borderWidth, contentColor) {
            tooltip.element
                .attr("align", align)
                .style({
                    "border-width": borderWidth
                });
            tooltip.element.selectAll("div#tooltipcontent")
                .style("color", contentColor);
        }
    }
})();