//$Id$
(function() {
	var hexToRGB = legend_helpers.hexToRGB;
	function getChartTypeSymbol(chartName) {
		return function(size) {
			switch (chartName) {
				case "bar":
				case "gantt":
				case "bullet":
					return "M" + (-size * 0.5) + " " + (size * 0.5) +
						" L" + (-size * 0.5) + " " + (-size * 0.2) +
						" L" + (-size * 0.25) + " " + (-size * 0.2) +
						" L" + (-size * 0.25) + " " + (size * 0.5) + "Z " +
						" M" + (-size * 0.125) + " " + (size * 0.5) +
						" L" + (-size * 0.125) + " " + (-size * 0.5) +
						" L" + (size * 0.125) + " " + (-size * 0.5) +
						" L" + (size * 0.125) + " " + (size * 0.5) + "Z " +
						" M" + (size * 0.25) + " " + (size * 0.5) +
						" L" + (size * 0.25) + " " + (size * 0.1) +
						" L" + (size * 0.5) + " " + (size * 0.1) +
						" L" + (size * 0.5) + " " + (size * 0.5) + "Z";
				case "area":
				case "arearange":
					return "M" + (-size * 0.5) + " " + (size * 0.5) +
						" L" + (-size * 0.5) + " " + (size * 0.28) +
						" L" + (-size * 0.24) + " " + (size * 0.08) +
						" L" + (-size * 0.13) + " " + (size * 0.19) +
						" L" + (size * 0.07) + " " + (-size * 0.14) +
						" L" + (size * 0.23) + " " + (size * 0.012) +
						" L" + (size * 0.5) + " " + (-size * 0.5) +
						" L" + (size * 0.5) + " " + (size * 0.5) + "Z";
				case "pie":
				case "bubblepie":
				case "sunburst":
					return "M" + (size * 0.045) + " " + (-size * 0.045) +
						" L" + (size * 0.5) + " " + (-size * 0.045) +
						" A " + (size * 0.455) + " " + (size * 0.455) + " 0 0 0 " + (size * 0.045) + " " + (-size * 0.5) + "Z" +
						" M" + (-size * 0.045) + " " + (size * 0.045) +
						" L" + (-size * 0.045) + " " + (-size * 0.41) +
						" A " + (size * 0.455) + " " + (size * 0.455) + " 1 1 0 " + (size * 0.41) + " " + (size * 0.045) + "Z";
				case "funnel":
					return "M" + (-size * 0.5) + " " + (-size * 0.5) +
						" L " + (size * 0.5) + " " + (-size * 0.5) +
						" L " + (size * 0.15) + " " + (size * 0.2) +
						" L " + (size * 0.15) + " " + (size * 0.5) +
						" L " + (-size * 0.15) + " " + (size * 0.5) +
						" L " + (-size * 0.15) + " " + (size * 0.2) + "Z";
				case "pyramid":
					return "M0 " + (-size * 0.5) +
						" L" + (size * 0.23) + " " + (-size * 0.06) +
						" L" + (-size * 0.23) + " " + (-size * 0.06) + "Z" +
						" M" + (-size * 0.5) + " " + (size * 0.5) +
						" L" + (size * 0.5) + " " + (size * 0.5) +
						" L" + (size * 0.27) + " 0" +
						" L" + (-size * 0.27) + " 0Z";
				case "line":
				case "web":
					return getSVGsymbol("line")(size);
				case "scatter":
				case "bubble":
				case "packedbubble":
				case "line":
				default:
					return getSVGsymbol(chartName)(size);
			}
		};
	}

	$ZC.legend.getHTML4ChartTypeSymbol = function(width, height, fillColor, chartObj, index) {
		var seriesIndex = chartObj.isAxisCategory || chartObj.dataObject.isPolarAxisCategory ? index : 0,
			chartName = chartObj.chartTypes[seriesIndex].name,
			commonPlotOptions = chartObj.systemConf.chart.plot.plotoptions[chartName],
			plotOptions = chartObj.seriesdata[seriesIndex].plotoptions,
			symbolpath = getChartTypeSymbol(chartName)(width);

		switch (chartName) {
			case "line":
			case "web":
				var strokeWidth = height * 0.15,
					markerOptions = (plotOptions && plotOptions.marker) ? mergeJSON(commonPlotOptions.marker, plotOptions.marker, true) : commonPlotOptions.marker,
					dashProperties = getDashProperties(plotOptions, commonPlotOptions, strokeWidth),
					markerProperties = "";
				var styles = "stroke:" + fillColor + ";" +
					"stroke-width:" + strokeWidth + ";";

				if(dashProperties) {
					styles += dashProperties;
				} else if(markerOptions.enabled != false){
					var outerRatio = (chartName === "web") ? 0.5 : 0.65,
						innerRatio = (chartName === "web") ? 0.25 : 0.45;

					markerProperties = getMarkerProperties(markerOptions, width, height, fillColor, "outer", mathMin(width, height)  * outerRatio);
					markerProperties += getMarkerProperties(markerOptions, width, height, fillColor, "inner", mathMin(width, height)  * innerRatio);
				}

				return "<path d='" + symbolpath + "' transform='translate(" + (width / 2) + "," + (height / 2) + ")' style='" + styles + "'></path>"
					+ markerProperties;

			case "scatter":
			case "bubble":
				var markerOptions = (plotOptions && plotOptions.marker) ? mergeJSON(commonPlotOptions.marker, plotOptions.marker, true) : commonPlotOptions.marker;

				return getMarkerProperties(markerOptions, width, height, fillColor, "outer", mathMin(width, height)  * 0.65)
					+ getMarkerProperties(markerOptions, width, height, fillColor, "inner", mathMin(width, height)  * 0.45);
			case "packedbubble":
				return getMarkerProperties(commonPlotOptions, width, height, fillColor, null, mathMin(width, height) - (commonPlotOptions.strokeWidth || 1));			
			default:
				var styles = "fill:" + fillColor + ";";
				return "<path d='" + symbolpath + "' transform='translate(" + (width / 2) + "," + (height / 2) + ")' style='" + styles + "'></path>"
		}
	};

	function getDashProperties(plotOptions, commonPlotOptions, strokeWidth) {
		var dashStyle = getValByPriority([plotOptions, commonPlotOptions], "dashStyle"),
			dashArray = getDashArray(dashStyle, strokeWidth*0.75),
			styles;

		if (dashArray && dashArray.length) {
			var strokeLineCap = getLineCap(getValByPriority([plotOptions, commonPlotOptions], "lineCap"), dashStyle)
			styles = "stroke-dasharray:" + dashArray +";" +
				"stroke-linecap:" + strokeLineCap + ";";
		}

		return styles;
	}

	function getMarkerProperties(markerOptions, width, height, color, markerType, size) {
		var symbolpath = getChartTypeSymbol(markerOptions.symbol)(size);

		var styles = "stroke:" + hexToRGB(markerOptions[markerType ? markerType + "StrokeColor" : "strokeColor"] || color) + ";" +
			"stroke-width:" + markerOptions[markerType ? markerType + "StrokeWidth" : "strokeWidth"] + ";" + 
			"stroke-opacity:" + markerOptions[markerType ? markerType + "StrokeOpacity" : "strokeOpacity"] + ";" + 
			"fill:" + hexToRGB(markerOptions[markerType ? markerType + "FillColor" : "fillColor"] || color) + ";" +
			"fill-opacity:" + markerOptions[markerType ? markerType + "FillOpacity" : "fillOpacity"] + ";";

		return "<path d='" + symbolpath + "' transform='translate(" + (width / 2) + "," + (height / 2) + ")' style='" + styles + "'></path>"
	}

})();