//$Id$
canvas_plot_renderer.line = function() {
    "use strict";
    var commonRendererProp, shiftInfo, yrangePos, scatter = canvas_plot_renderer.scatter(),
        dataProcessorHelpers = $ZC.DataProcessor.helpers;
    
    function chart(context, series, pattern) {
        var rp = commonRendererProp.params,
            chartObj = rp.chartObj,
            xscale = rp.xscale,
            yscale = rp.yscale,
            stacked = rp.stacked,
            index = rp.datasetIndex,
            axisrotated = rp.axisrotated,
            commonPlotOptions = rp.commonPlotOptions,
            color = rp.color,
            labelQueue = rp.labelQueue,
            yaxiscolumnorder = rp.yaxiscolumnorder,
            dataset = rp.dataset,
            systemconf = chartObj.systemConf;
        
        var currentdata = series;
        var plotoptions = currentdata.plotoptions;
        var gvp = $ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions),
            gradientDim = axisrotated ? [yscale.range()[0], xscale.range()[1]] : [xscale.range()[1], yscale.range()[0]],
            canvasarea = chartObj.canvasarea;
        var tempChart = chartObj.chartTypes[index],
            charttype = tempChart.name,
            chartTypeNumeric = tempChart.value,
            isAreaChart = chartTypeNumeric != $ZC.charttype.line;
        var interpolation = gvp("mode"),
            dashStyle = gvp("dashStyle"),
            strokeWidth = validateNullVal(gvp("strokeWidth"), 2),
            strokeOpacity = gvp("strokeOpacity"),
            strokeColor = gvp("strokeColor"),
            markeroptions = (plotoptions ? mergeJSON(commonPlotOptions.marker, plotoptions.marker, true) : commonPlotOptions.marker);
        
        var strokeLineCap = getLineCap(gvp("lineCap"), dashStyle),
            strokeDashArray = getDashArray(dashStyle, strokeWidth),
            markerEnabled = (datacount_based_intelligence(chartObj, "marker") && dimension_based_intelligence(chartObj, "marker", mathMin(canvasarea.width, canvasarea.height))) ? markeroptions.enabled : false;
        
        var datatypeX = chartObj.dataset.getDataType("x"),
            datatypeY = chartObj.dataset.getDataType("y", yaxiscolumnorder),

            isCatNullX = dataProcessorHelpers.isCatNull(dataset, "x"),
            isCatNullY = dataProcessorHelpers.isCatNull(dataset, "y", yaxiscolumnorder),

            xCategories = (datatypeX == ORDINAL_DATATYPE) ? xscale.domain() : null,
            yCategories = (datatypeY == ORDINAL_DATATYPE) ? yscale.domain() : null,

            categories = (datatypeX == ORDINAL_DATATYPE && !defined(chartObj.dataObject.hierarchical)) ? xCategories : chartObj.dataObject.getCategories("x");

        if (defined(pattern)) {
            context.lineWidth = pattern["stroke-width"];
        } else {
            context.lineWidth = strokeWidth;
        }
        if (!series.disabled) {
            chart.seriesdataAfterUpdate = [];
            context.save();
            currentdata.data.forEach(function(d, i) {
                context.beginPath();
                var serData;
                if (datatypeX == ORDINAL_DATATYPE || datatypeY == ORDINAL_DATATYPE) {
                    serData = chartObj.dataObject.updateSeriesdatabyCategories(d3_values(d), xCategories, yCategories, yaxiscolumnorder);
                } else {
                    serData = d;
                }

                chart.seriesdataAfterUpdate[i] = serData;
                var line = canvas_drawer.line()
                    .defined(function(d) {
                        var nullData = !isCatNullY(dataset.getY(d, yaxiscolumnorder)) && !isCatNullX(dataset.getX(d));
                        if (chartTypeNumeric === $ZC.charttype.stkdarea) {
                            return true
                        }
                        return !d.isDummy && nullData;
                    })
                    .x(function(d) {
                        var x = chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos);
                        if (isNaN(x) && shiftInfo != null) {
                            var diff = chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[1], "x", null, yrangePos) -
                                chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos);
                            x = chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[serData.length - 1], "x", null, yrangePos) + diff;
                        }
                        return x;
                        
                    })
                    .y(function(d) {
                        return chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos);
                    })
                    .context(context)
                    .interpolate(interpolation)(d);
                if (strokeDashArray.length && typeof strokeDashArray === "object" && defined(context.setLineDash)) {
                    context.setLineDash(strokeDashArray);
                    context.lineCap = strokeLineCap;
                }
                var args = {"offset":"line", "dimension":gradientDim, chartObj:chartObj},
                    prop =  getGradientProp(commonPlotOptions.gradients,gradientDim,(strokeColor || chartObj.seriesColor[index % chartObj.seriesColor.length]),strokeOpacity),
                    gradientID = (!isAreaChart ? effects.get(canvas_gradients, "canvasGradientFill", prop, args) || color : color);
                context.strokeStyle = gradientID ? gradientID  : canvasUtils.rgba(strokeColor || color, strokeOpacity);
                context.stroke();
            });
            context.restore();
            
            if (markerEnabled) {
                scatter
                    .commonRendererProp(commonRendererProp)
                    .shiftInfo(shiftInfo);
                scatter.chartRendered = false;
                scatter(context, series);
            }
        }
        
        if (!(markerEnabled)) {
            var markerSize = markerEnabled ? (((markeroptions.size || strokeWidth) * 0.7 * 10) + strokeWidth / 2) : 0;
            labelQueue.push({
                "renderer": chart,
                "charttype": "scatter",
                "chartObj": chartObj,
                "args": [currentdata, index, markerSize]
            });
        }
        chart.chartRendered = true;
        return chart;
    }
    
    chart.commonRendererProp = function(_) {
        return (!arguments.length) ? commonRendererProp : (commonRendererProp = _) && chart;
    };
    chart.shiftInfo = function(_) {
        return (!arguments.length) ? shiftInfo : (shiftInfo = _) && chart;
    };
    return chart;
};