//$Id$
/**
 *
 * @type {chart} line renderer.
 */
svg_plot_renderer.line = function() {
    var commonRendererProp, xscale, yscale, index, dataset, axisrotated, yaxiscolumnorder, color;
    var line = [],
        scatter = svg_plot_renderer.scatter();
    var series, redraw = false,
        path, strokeDashArray, marqueeEnabled, marqueeDuration, marker;
    var stacked, labelQueue, commonPlotOptions, systemconf, hasNegValue, chartObj;
    var oldData, shiftInfo, yrangePos;
    var transitionDuration, animationType;
    
    function chart(selection) {
        selection.each(function(data) {
            //renderer config values
            var rp = commonRendererProp.params;
            chartObj = rp.chartObj,
                xscale = rp.xscale,
                yscale = rp.yscale,
                stacked = rp.stacked,
                index = rp.datasetIndex,
                axisrotated = rp.axisrotated,
                commonPlotOptions = rp.commonPlotOptions,
                color = rp.color,
                labelQueue = rp.labelQueue,
                hasNegValue = rp.hasNegValue,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset,
                systemconf = chartObj.systemConf
            
            var defsLocation = chartObj.defsLocation;
            
            var currentdata = data[index];
            var plotoptions = currentdata.plotoptions;
            var currentseriesdata = currentdata.data;
            var tempChart = chartObj.chartTypes[index],
                charttype = tempChart.name,
                chartTypeNumeric = tempChart.value,
                dataProcessorHelpers = $ZC.DataProcessor.helpers;
            var gvp = $ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions),
                isAreaChart = chartTypeNumeric != $ZC.charttype.line,
                isRangeChart = (charttype == "arearange"),
                isPolar = chartObj.dataObject.isPolarAxisCategory,
                gradientDim = axisrotated ? [yscale.range()[0], xscale.range()[1]] : [xscale.range()[1], yscale.range()[0]],
                canvasarea = chartObj.canvasarea;
            var interpolation = gvp("mode"),
                dashStyle = gvp("dashStyle"),
                strokeWidth = validateNullVal(gvp("strokeWidth"), 2),
                strokeOpacity = gvp("strokeOpacity"),
                strokeColor = gvp("strokeColor"),
                callScatterRenderer = $Browser.IE || $Browser.SAFARI || interpolation == "step-before" || interpolation == "step-after" || rp.thresholdType, // IE & SAFARI don't support marker updation. So, render the scatter chart instead of using marker.
                markeroptions = ((plotoptions && plotoptions.marker) ? mergeJSON(commonPlotOptions.marker, plotoptions.marker, true) : commonPlotOptions.marker);
            
            //Animation variables
            var animationOptions = getAnimationConf(commonPlotOptions, plotoptions, chartObj, charttype);
            var isAnimationEnable = animationOptions.enabled,
                easingType = animationOptions.easingType;
            transitionDuration = animationOptions.duration;
            animationType = animationOptions.type;
            animationType = (animationType == "vertPathTweenAll") ? "vertPathTween" : animationType;
            
            var blinkConf = (plotoptions && plotoptions.animation && plotoptions.animation.blink) ?
                mergeJSON(commonPlotOptions.animation.blink, plotoptions.animation.blink, true) :
                commonPlotOptions.animation.blink;
            
            var markerEnabled = (datacount_based_intelligence(chartObj, "marker") && dimension_based_intelligence(chartObj, "marker", mathMin(canvasarea.width, canvasarea.height))) ? markeroptions.enabled : false,
                strokeLineCap = getLineCap(gvp("lineCap"), dashStyle);
            strokeDashArray = getDashArray(dashStyle, strokeWidth);
            
            var marqueeOptions = (plotoptions && plotoptions.animation && plotoptions.animation.marquee) ?
                mergeJSON(commonPlotOptions.animation.marquee, plotoptions.animation.marquee, true) :
                commonPlotOptions.animation.marquee;
            marqueeEnabled = marqueeOptions.enabled;
            marqueeDuration = marqueeOptions.duration;
            
            var gradientOptions = (!isAreaChart && plotoptions && plotoptions.gradients) ?
                mergeJSON(commonPlotOptions.gradients, plotoptions.gradients, true) :
                commonPlotOptions.gradients;

            var datatypeX = chartObj.dataset.getDataType("x")
                datatypeY = chartObj.dataset.getDataType("y", yaxiscolumnorder),

                isCatNullX = dataProcessorHelpers.isCatNull(dataset, "x"),
                isCatNullY = dataProcessorHelpers.isCatNull(dataset, "y", yaxiscolumnorder),

                xCategories = (datatypeX == ORDINAL_DATATYPE) ? xscale.domain() : null,
                yCategories = (datatypeY == ORDINAL_DATATYPE) ? yscale.domain() : null,

                categories = (datatypeX == ORDINAL_DATATYPE && !defined(chartObj.dataObject.hierarchical)) ? xCategories : chartObj.dataObject.getCategories("x");

            var yReversed = systemconf.chart.axes.yaxis[yaxiscolumnorder].reversed;
            if (datatypeX != ORDINAL_DATATYPE && categories.length == 0 && isAnimationEnable && animationType == "pointTween") {
                categories = chartObj.dataObject.findXCategories();
            }
            
            chart.seriesdataAfterUpdate = [];
            currentseriesdata.forEach(function(d, i) {
                var serData;
                if (datatypeX == ORDINAL_DATATYPE || datatypeY == ORDINAL_DATATYPE) {
                    serData = chartObj.dataObject.updateSeriesdatabyCategories(d3_values(d), xCategories, yCategories, yaxiscolumnorder);
                } else {
                    serData = d;
                }

                chart.seriesdataAfterUpdate[i] = serData;
                var curInterpolation = ($Browser.FIREFOX && (serData.length == 1)) ? "linear-closed" : interpolation; //Firefox fails to render path marker if 'd' has only 'M(x) (y)' ZC509

                line[i] = isPolar ?
                    d3_lineRadial()
                        .defined(function(d, j) {
                            return !d.isDummy;
                        })
                        .radius(function(d, j) {
                            var adjustFactor = 0.1;// #ZC717 -> if path contains 'L0,0L0,0L0,0Z'
                            var R = (stacked) ? yscale(d.y + d.y0) : (yscale(dataset.getY(d, yaxiscolumnorder)) + ((yscale.rangeBand != null) ? yscale.rangeBand() / 2 : 0));
                            return mathMax(R, yscale.range()[(yReversed && !stacked) ? yscale.domain().length - 1 : 0] + adjustFactor);
                        })
                        .angle(function(d, j) {
                            return xscale(dataset.getX(d));
                        })
                        .interpolate(curInterpolation) :
                    d3_line()
                        .defined(function(d, j) {
                            var nullData = !isCatNullY(dataset.getY(d, yaxiscolumnorder)) && !isCatNullX(dataset.getX(d));
                            if (chartTypeNumeric === $ZC.charttype.stkdarea) {
                                return true
                            }
                            return !d.isDummy && nullData;
                        })
                        .x(function(d, j) {
                            var x = chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos)
                            if (shiftInfo != null) {
                                var transX = (shiftInfo != null) ?
                                    Math.abs((chartObj.dataObject.getXYvalue(axisrotated, stacked, chart.oldXscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos) -
                                    chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos))) :
                                    null;
                                x = x + transX;
                            }
                            return x;
                        })
                        .y(function(d, j) {
                            return chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos);
                        })
                        .interpolate(curInterpolation)
                        .tension(commonPlotOptions.tension || 0.7);
            });
            
            var container = d3_select(this);
            chart.update = function() {
                container.transition().call(chart);
            };
            series = appendEle(container, "g", [currentdata], ".", "class", ('lineseries_' + index));
            if (isRangeChart) {
                series = appendEle(series, "g", [currentdata], ".", "class", ('rangePos_' + yrangePos));
            }
            
            var markerID,
                mSize = isPolar ? {
                        "inner": 0.32,
                        "outer": 0.56
                    } : {
                        "inner": 0.4,
                        "outer": 0.7
                    };
            if (markerEnabled && !callScatterRenderer) {
                markeroptions.color = currentdata.color || chartObj.seriesColor[index % chartObj.seriesColor.length];
                markerID = generateIdByPropValue(markeroptions, "marker") + generateIdByPropValue(blinkConf, "blink");
                marker = addSVGmarker(series, markerID, markeroptions, {
                    "color": markeroptions.color,
                    symbolSize: mSize,
                    strokeWidth: strokeWidth
                });
                delete markeroptions.color;
            }
            
            if (markerEnabled && !callScatterRenderer && blinkConf.enabled) {
                blinkSVGsymbols(marker.selectAll(".outer,.inner"), blinkConf);
            }
            
            if (markerEnabled && callScatterRenderer) {
                scatter
                    .commonRendererProp(commonRendererProp)
                    .shiftInfo(shiftInfo)
                    .yrangePos(yrangePos);
                scatter.chartRendered = false;

                container.call(scatter);
            } else {
                container.selectAll("g.scatterseries_" + index).remove();
            }
            
            currentseriesdata.forEach(function(d, i) {
                var ishideable = currentdata.disabled;
                if (chartObj.brush && chartObj.brush.extent) {
                    var domain = chartObj.axes.x.scale.domain();
                    var ticks = chartObj.axes.x.ticks.tickValueObj.ticks;
                    var startVal = (datatypeX == TIME_DATATYPE) ? domain[0].getTime() : (datatypeX == ORDINAL_DATATYPE) ? categories.indexOf(domain[0]) : domain[0];
                    var endVal = (datatypeX == TIME_DATATYPE) ? domain[1].getTime() : (datatypeX == ORDINAL_DATATYPE) ? categories.indexOf(domain[domain.length - 1]) : domain[1];
                    var accessor = (datatypeX == ORDINAL_DATATYPE) ? function(d) {
                            return categories.indexOf(chartObj.dataset.getX(d))
                        } : function(d) {
                            return chartObj.dataset.getX(d)
                        }
                    chart.seriesdataAfterUpdate[i] = dataProcessorHelpers.filterDataToViewPort(chart.seriesdataAfterUpdate[i], startVal, endVal, accessor);
                }
                var serData = chart.seriesdataAfterUpdate[i];
                if (chartObj.systemConf.chart.axes.xaxis.reversed) {
                    serData = serData.slice(0).reverse();
                }
                var baseData = serData;
                
                var basedataShift, ShiftedData;
                
                var subseries = appendEle(series, "g", function(d) {
                    return ishideable ? [] : [d];
                }, ".", "class", ('subseries_' + i));
                
                path = subseries.selectAll("path.line").data(function(d) {
                    return ishideable ? [] : [d];
                });
                if (isAnimationEnable && (animationType != "strokeTween")) {
                    if (animationType == "pointTween") {
                        
                        //get baseData for starting animation
                        baseData = (oldData != null) ? oldData.map(function(d, j) {
                                return Object.create(oldData[0])
                            }) : serData.map(function(d, i) {
                                return Object.create(serData[0])
                            });
                        
                    } else {
                        var idx = (animationType == "horiPathTween") ? dataset.xidx : dataset.yidx(yaxiscolumnorder);
                        var scale = (animationType == "horiPathTween") ? xscale : yscale;
                        var baseIdx = (isPolar && yReversed && !stacked) ? scale.domain().length - 1 : 0;
                        baseData = serData.map(function(d, i) {
                            var s = Object.create(d);
                            if (stacked && animationType != "horiPathTween") {
                                s.y0 = 0;
                                s.y = (!isPolar && (hasNegValue != null) && (scale(hasNegValue) != null)) ? mathMax(hasNegValue, scale.domain()[0]) : scale.domain()[0];
                            } else {
                                if (isRangeChart && animationType == "vertPathTween") {
                                    var yArr = new Array(2);
                                    var y0 = chartObj.dataset.getY(d, yaxiscolumnorder, 0);
                                    yArr[0] = y0;
                                    yArr[1] = (s[idx] == null) ? null : (((hasNegValue != null) && (scale(hasNegValue) != null)) ? mathMax(hasNegValue, y0) : y0);
                                    s[idx] = yArr
                                } else {
                                    s[idx] = (s[idx] == null) ? null : ((!isPolar && (hasNegValue != null) && (scale(hasNegValue) != null)) ? mathMax(hasNegValue, scale.domain()[0]) : scale.domain()[baseIdx]);
                                }
                            }
                            return s;
                        });
                    }
                    basedataShift = d3_values(baseData);
                }
                

                if (shiftInfo != null) {

                    if (redraw) {
                        var clipX = xscale(serData[0][0]) + ((xscale.rangeBand != null) ? xscale.rangeBand() / 2 : 0);
                        var clipWid = chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[serData.length - 1], "x", null, yrangePos) -
                            chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos)

                        var properties = {
                            name: "rect",
                            id: chartObj.id,
                            x: clipX,
                            y: 0,
                            width: clipWid,
                            height: chartObj.plotarea.height
                        },
                        args = {
                            chartObj: chartObj
                        };
                        var clipID = effects.get(svg_clips, "rect", properties, args);
                        series
                            .attr('clip-path', "url(" + defsLocation + "#" + clipID + ")");
                    }
                    
                }
                
                
                var pathEnter = path.enter().append("path").attr("class", "line")
                    .datum(baseData, function(d) {
                        return dataset.getX(d)
                    })
                    .attr("d", line[i])
                    .style('stroke', currentdata.color || color)
                    .style('stroke-width', strokeWidth)
                    .style('stroke-opacity', (strokeOpacity == null || strokeOpacity === "") ? 1 : strokeOpacity)
                
                
                if (isAnimationEnable) {
                    if (animationType == "pointbypoint" || animationType == "pointTween") {
                        interTween(baseData, basedataShift, serData, categories, animationType, easingType, transitionDuration, rp, function(d) {
                            return {
                                "d": line[i](d)
                            };
                        }, pathEnter);
                    } else if (animationType == "strokeTween") {
                        strokeTween(pathEnter, easingType, transitionDuration);
                    } else {
                        pathEnter
                            .transition()
                            .duration(transitionDuration)
                            .ease(easingType)
                            .attrTween("d", pathTween);
                    }
                    
                }
                
                var pathUpdate = path.datum(serData, function(d) {
                        return dataset.getX(d)
                    })
                    .attr({
                        "id": "line_" + index,
                        "stroke-dasharray": strokeDashArray
                    })
                    .style({
                        "stroke-linecap": strokeLineCap,
                        "stroke-width": strokeWidth,
                        "stroke-opacity": validateNullVal(strokeOpacity, 1),
                        "fill": "none",
                        "stroke": function(d, i) {
                            if(gradientOptions.type){
                                var args = {"offset":charttype};
                                gradientOptions = getGradientProp(gradientOptions,gradientDim,strokeColor || color,strokeOpacity);
                                var gradientID = isAreaChart ? null : effects.get(svg_gradients, "svgGradientFill", gradientOptions, args);
                                return "url(" + defsLocation + "#" + gradientID + ")";
                            }
                            
                            return (strokeColor || color);
                        }
                    });
                
                var dataUpdate =  serData;
                


                if (shiftInfo != null) {
                    var transX = (shiftInfo != null) ?
                        Math.abs((chartObj.dataObject.getXYvalue(axisrotated, stacked, chart.oldXscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos) -
                        chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, serData[0], "x", null, yrangePos))) :
                        null;
                    pathUpdate
                        .datum(dataUpdate)
                        .attr("d", line[i])
                        .transition()
                        .duration(transitionDuration)
                        .ease(easingType)
                        .attr("transform", function() {
                            return ((transX == null) ? null : "translate(-" + transX + ",0)")
                        });

                    resetDashProp(path);
                } else if (pathEnter.node() == null) {
                    path.datum(dataUpdate)
                        .attr("transform", null)
                        .transition()
                        // .delay(((isAreaChart && ((animationType != "pointTween") || (animationType == "pointTween" && redraw != false && currentdata.disabled != false))) ? 0 : transitionDuration))
                        .duration(transitionDuration)
                        .ease(easingType)
                        .attr("d", line[i]);
                    
                    /*
                     * Reset stroke-dasharray & stroke-dashoffset
                     * If animation.type = "strokeTween" & calling redraw before the animation completes
                     */
                    resetDashProp(path);
                }
                
                var mID = (markerEnabled && !callScatterRenderer) ? "url(" + defsLocation + "#" + markerID + ")" : null;
                path
                    .attr({
                        "marker-end": mID,
                        "marker-start": mID,
                        "marker-mid": mID
                    });
                
                path.exit().remove();
                
                function pathTween() {
                    var interpolate = d3_interpolate(line[i](baseData), line[i](serData));
                    return function(t) {
                        return interpolate(t);
                    };
                }
            });
            if (!(callScatterRenderer && markerEnabled)) {
                var markerSize = markerEnabled ? (((markeroptions.size || strokeWidth) * mSize.outer * 10) + strokeWidth / 2) : 0;
                labelQueue.push({
                    "renderer": chart,
                    "charttype": "scatter",
                    "chartObj": chartObj,
                    "args": [currentdata, index, markerSize, yrangePos]
                });
            }
        });
        chart.renderedTimer = setTimeout(function() {
            if (marqueeEnabled) {
                applyMarquee(series.selectAll("#line_" + index), marqueeDuration, strokeDashArray, chart);
            }
            chart.chartRendered = true;
            redraw = true;
        }, (!redraw) ? (transitionDuration + 20) : (2 * transitionDuration + 20));
        
        chart.oldXscale = xscale;
        return chart;
    }
    
    //chart.prototype = new $ZC.renderer();
    function strokeTween(path, easingType, duration) {
        path
            .style({
                "stroke-dasharray": function() {
                    return this.getTotalLength() + " " + this.getTotalLength()
                },
                "stroke-dashoffset": function() {
                    return this.getTotalLength()
                }
            })
            .transition()
            .duration(duration)
            .ease(easingType)
            .style("stroke-dashoffset", 0)
            .each("end", function() {
                resetDashProp(d3_select(this));
            });
    }
    
    function resetDashProp(path) {
        path
            .style({
                "stroke-dasharray": strokeDashArray,
                "stroke-dashoffset": null
            });
    }
    
    chart.commonRendererProp = function(_) {
        if (!arguments.length) {
            return commonRendererProp;
        }
        commonRendererProp = _;
        return chart;
    };
    
    chart.shiftInfo = function(_) {
        if (!arguments.length) {
            return shiftInfo;
        }
        shiftInfo = _;
        return chart;
    };
    
    chart.yrangePos = function(_) {
        if (!arguments.length) {
            return yrangePos;
        }
        yrangePos = _;
        return chart;
    };
    return chart;
};