//$Id$
/**
 *
 * @returns {chart} packedbubble renderer.
 */
svg_plot_renderer.packedbubble = function() {
    var pack = d3_pack(),
        isredraw = false,
        commonRendererProp,
        bubbleGroup,
        path,
        chartObj,
        commonPlotOptions,
        mode,
        seriesGroup = [],
        zchighlighterPosition,
        highlightGroup,
        highlightEle,
        multiColoring,
        valueFunction,
        includeDisabledSeries;
    
    function chart(selection) {
        selection.each(function(data) {
            //renderer config values
            var rp = commonRendererProp.params;
            chartObj = rp.chartObj;
            var color = rp.color,
                notesArray = rp.notesArray,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset,
                systemconf = chartObj.systemConf,
                labelQueue = rp.labelQueue;
            
            commonPlotOptions = rp.commonPlotOptions;
            var fillColor = chart.fillColorFunc = getFillColor(chartObj, 0),
                strokeColor = getStrokeColor(chartObj, 0);
            
            var cloneddata = data.filter(function(d, i) {
                    d.seriesIndex = i;
                    seriesGroup[i] = [];
                    return !d.disabled;
                }),
                chartWidth = chartObj.plotarea.width,
                chartHeight = chartObj.plotarea.height,
                gvpFn = $ZC.utils.getValueOfPlotOptions,
                isSingleSeries = (cloneddata.length === 1),
                isPatternPalleteEnabled = hasPatternPallete(chartObj) || chartObj.patternPallete,
                notesOptions = chartObj.systemConf.notes,
                notesEnabled = notesOptions && notesOptions.enabled === true && notesOptions.plotValues,
                index;
            mode = commonPlotOptions.mode;
            multiColoring = commonPlotOptions.multiColoring === true && !chartObj.legendarea.show;
            
            var NegativeAsAbsolute = commonPlotOptions.showAsNegative === "absolute",
                interPadding = (+commonPlotOptions.interPadding || 0),
                outerPadding = (+commonPlotOptions.outerPadding || 0),
                diameter = mathMin(chartWidth, chartHeight) - outerPadding;

            zchighlighterPosition = [(chartObj.plotarea.left + (chartWidth - diameter) / 2), (chartObj.plotarea.top + ((chartHeight - diameter) / 2))];
            valueFunction = function(d) {
                if (!includeDisabledSeries && isDisabled(d)) {
                    return 0;
                }
                var y = dataset.getY(d, yaxiscolumnorder) || 0;
                if (y < 0) {
                    return NegativeAsAbsolute ? mathAbs(y) : 0;
                }
                return y;
            };
            
            var eventDisabled = (validateNullVal(getEventConf(null, commonPlotOptions, chartObj, "enabled"), true) == false);
            var cursor = (commonPlotOptions.events) ? commonPlotOptions.events.cursor || "pointer" : systemconf.chart.plot.events.cursor || "pointer";
            
            //Animation variables
            var animationOptions = getAnimationConf(commonPlotOptions, null, chartObj, "packedbubble");
            var isAnimationEnable = animationOptions.enabled,
                animationType = animationOptions.type,
                easingType = animationOptions.easingType,
                transitionDuration = isredraw ? (isAnimationEnable ? 200 : 0) : animationOptions.duration;

            //Transition variables
            var initial_transform = function(d) {
                    var transform = d3_select(this).attr("transform");
                    return (transform != null) ? transform : "translate(" + diameter / 2 + "," + diameter / 2 + ")";
                },
                final_transform = function(d) {
                    return "translate(" + d.x + "," + d.y + ")";
                },
                initial_r = function(d) {
                    var r = d3_select(this).attr("r");
                    return (r != null) ? r : 0;
                },
                final_r = function(d) {
                    return d.r;
                },
                final_fillOpacity = function(d, i) {
                    return commonPlotOptions.fillOpacity;
                },
                final_strokeOpacity = function(d, i) {
                    return commonPlotOptions.strokeOpacity;
                },
                isFade = !isredraw && (animationType === "fade" || animationType === "fadeAll");

            //Initializing pack layout
            pack
                .size([diameter, diameter])
                .padding(interPadding);

            bubbleGroup = appendEle(selection, "g", [1], ".", "class", "hierarchychart");
            bubbleGroup
                .attr("transform", "translate(" + ((chartWidth - diameter) / 2) + "," + ((chartHeight - diameter) / 2) + ")")
                .style("cursor", cursor);
            path = appendEle(bubbleGroup, "circle", getPackNodeDescendants(data, true));

            path
                .attr({
                    "class": function(d) {
                        return "series_" + getSeriesPos(d);
                    }
                })
                .style("visibility", function(d) {
                    return isDisabled(d) ? "hidden" : null;
                });

            path = path.filter(function(d) {
                return !isDisabled(d);
            });
            path = path.data(getPackNodeDescendants(cloneddata));

            path
                .attr({
                    "transform": isFade ? final_transform : initial_transform,
                    "r": isFade ? final_r : initial_r
                })
                .style({
                    "fill": function(d, i) {
                        d.idx = i;
                        //Test_654.json
                        d.r = d.r || 0;
                        d.x = d.x || 0;
                        d.y = d.y || 0;
                        d.data.parent = d.parent;

                        if (notesEnabled) {
                            d.data.x = d.x;
                            d.data.y = d.y;
                        }

                        if (!d.children && d.parent) {// Test_655.json
                            seriesGroup[getSeriesPos(d)].push(d);
                        }

                        if (!d.depth) {
                            d.arcColor = (mode == "flat" || multiColoring || isSingleSeries) ? "none" : "grey";
                        } else if (mode !== "flat" && d.depth === 1 && multiColoring) {
                            d.arcColor = "grey";
                        } else {
                            d.arcColor = color[(multiColoring ? getItemPos(d) : getSeriesPos(d)) % color.length];
                        }

                        var baseClr = getBaseClr(d);
                        return d.fillColor = (!d.depth && isPatternPalleteEnabled) ? "none" : fillColor((multiColoring ? getItemPos(d) : getSeriesPos(d)), d.data, null, baseClr);
                    },
                    "fill-opacity": isFade ? 0 : final_fillOpacity,
                    "stroke-opacity": isFade ? 0 : final_strokeOpacity,
                    "stroke": function(d) {
                        if (commonPlotOptions.strokeColor) {
                            return commonPlotOptions.strokeColor;
                        } else if (d.children) {
                            return d.arcColor;
                        } else {
                            return strokeColor((multiColoring ? getItemPos(d) : getSeriesPos(d)), d.data);
                        }
                    },
                    "stroke-width": function(d) {
                        return commonPlotOptions.strokeWidth;
                    }
                });
            
            //Animation
            var total = path.size(),
                dur = (!isredraw && animationType === "fade") ? (transitionDuration / total) : transitionDuration,
                del = (animationType === "fade") ? dur : 0;
            
            path
                .transition()
                .delay(function(d, i) {
                    return isredraw ? 0 : (i * del);
                })
                .duration(dur)
                .ease(isredraw ? "linear" : easingType)
                .attr({
                    "transform": final_transform,
                    "r": final_r
                })
                .style({
                    "fill-opacity": final_fillOpacity,
                    "stroke-opacity": final_strokeOpacity
                });
            
            //Events
            if (!eventDisabled) {
                var mousemove = function(d, i) {
                        var args = getEventArgs(d, i);
                        chartObj.eventHandler.mouseMove(args);
                    },
                    mouseout = function(d, i) {
                        chartObj.eventHandler.mouseOut(chart, d, getSeriesPos(d));
                    },
                    click = function(d, i) {
                        var args = getEventArgs(d, i);
                        chartObj.eventHandler.click(chart, d, getSeriesPos(d), getItemPos(d), args);
                    },
                    dblclick = function(d, i) {
                        if (!chart.chartRendered) {
                            return;
                        }
                        var dt = $ZC.annotation ? generateDataForAnnotation(d.data, ((data.length > 1) ? d.parent.parent.seriesname : null)) : d.data;
                        var coordinates = [d.x, d.y];
                        coordinates = [(coordinates[0] + ((chartWidth - diameter) / 2)), (coordinates[1] + ((chartHeight - diameter) / 2))];
                        var args = {
                            "coordinate": coordinates,
                            "data": dt,
                            "basedOn": "plot"
                        };
                        chartObj.eventHandler.doubleclick(d, getSeriesPos(d), args);
                    },
                    eventAttacherOptions = {
                    target: "plot",
                    touchleave: true,
                    chartObj: chartObj
                };
                var mouseHandlers = {
                    "mousemove": mousemove,
                    "mouseout": mouseout,
                    "click": click,
                    "dblclick": dblclick
                },
                touchHandlers = {
                    "singletouchmove": mousemove,
                    "touchend": mouseout,
                    "tap": click,
                    "dbltap": dblclick
                };
                attachevents(path.filter(function(d) {
                    return !d.children;
                }), mouseHandlers, touchHandlers, eventAttacherOptions);
            }
            
            chart.renderedTimer = setTimeout(function() {
                //Annotation preprocessing
                if ($ZC.annotation) {
                    data.forEach(function(series, i) {
                        notesArray = $ZC.annotation.preprocessAnnotations(data, i, function(dt, k, j) {
                            var c = [dt.x, dt.y];
                            return [(c[0] + ((chartWidth - diameter) / 2)), (c[1] + ((chartHeight - diameter) / 2))];
                        }, notesArray, chartObj, rp);
                    });
                }
                
                if ($ZC.labelRenderer) {
                    var transform = [(chartWidth - diameter) / 2, (chartHeight - diameter) / 2],
                        totalFunc = function(d) {
                            return d.parent.value
                        };
                    data.forEach(function(d, i) {
                        labelQueue.push({
                            "renderer": chart,
                            "charttype": "hierarchy",
                            "chartObj": chartObj,
                            "args": [data[i], i, null, null, totalFunc, seriesGroup[i], transform]
                        });
                    });
                }
                
                isredraw = true;
                chart.chartRendered = true;
            }, (transitionDuration + 17));

            function getEventArgs(d, i) {
                var ev = getEventObject(d3.event);
                return {
                    "renderer": chart,
                    "seriesPos": getSeriesPos(d),
                    "subseriesPos": 0,
                    "itemPos": getItemPos(d),
                    "coordinates": [ev.pageX, ev.pageY],
                    "element": this,
                    // "color": d.arcColor,
                    "data": d
                };
            }
        });
        return chart;
    }
    
    chart.commonRendererProp = function(_) {
        if (!arguments.length) {
            return commonRendererProp;
        }
        commonRendererProp = _;
        return chart;
    };
    
    chart.defaultMouseOverFunc = function(d, seriesPos, itemPos) {
        if (!highlightGroup) {
            highlightGroup = chartObj.staticPaneSVG.selectAll("g#zchierarchyhighlight");
            highlightEle = appendEle(highlightGroup, "path", [1]);
        }

        var refElement = d3_select(path[0][d.idx]);
        var refElementPosition = d3_transform(refElement.attr("transform")).translate;
        var highlightEffect = chartObj.legendHighlight.getLegendHighlightEffect(seriesPos, "selectedSeries", "plot");
        var currentColor = getRawColor(chartObj, (multiColoring ? itemPos : seriesPos), d.data, seriesPos);

        highlightGroup
            .style("opacity", null);
        highlightEle
            .attr({
                "transform" : "translate(" + (zchighlighterPosition[0] + refElementPosition[0]) + "," + (zchighlighterPosition[1] + refElementPosition[1]) + ")",
                "d" : getSVGsymbol("circle")(+refElement.attr("r") * 2)
            })
            .style(chartObj.legendHighlight.getEffect(highlightEffect, "packedbubble", (/(illuminate|moody|flip)/.test(highlightEffect) ?  currentColor : "transparent"), {fill: "transparent", stroke: currentColor}, (multiColoring ? itemPos : seriesPos)));
    };
    
    chart.defaultMouseOutFunc = function(d, i) {
        highlightGroup
            .style("opacity", 0);
    };
    
    function getNodes(root) {
        var leafNodes = [];
        
        function recurse(node, prevID) {
            var child = getChildren(node);
            if (child) {
                child.forEach(function(c, i) {
                    var ID = defined(prevID) ? (prevID + "_" + i) : (c.seriesIndex + "");
                    parseID(c, ID);
                    recurse(c, ID);
                });
            } else {
                parseID(node, prevID);
                if (mode == "flat") {
                    leafNodes.push(node);
                }
            }
        }
        
        function parseID(node, ID) {
            var indices = ID.split(/_/);
            node.seriesPos = +indices[0];
            node.itemPos = +indices[indices.length - 1];
        }
        
        recurse(root);
        return (mode == "flat") ? leafNodes : root;
    }
    
    function getChildren(node) {
        var depth = node.level && node.level.split("_").length;
        if (!chartObj.dataObject.hasHierarchyData && depth == 1 && node.length < 2 && node[0] && isArray(node[0][0])) {
            return node[0];
        } else if (isArray(node[0])) {
            if (isArray(node[0][0])) {
                return node;
            } else {
                return node.filter(function(d) {
                    return valueFunction(d) > 0;
                });
            }
        } else {
            return null;
        }
    }

    function getSeriesPos(d) {
        var level = d.level || (d.data && d.data.level);
        if (level) {
            return +level.split("_")[0];
        }
    }

    function getItemPos(d) {
        return d.data.itemPos;
    }

    function getBaseClr(d) {
        return d.children ? d.arcColor : null;
    }

    function isDisabled(d) {
        var seriesIndex = getSeriesPos(d);
        return defined(seriesIndex) && chartObj.seriesdata[seriesIndex].disabled;
    }

    function getPackNodeDescendants(data, include) {
        includeDisabledSeries = include;
        var root = d3.hierarchy(getNodes(data), getChildren)
            .sum(valueFunction)
            .sort(d3_ascending);

        pack(root);
        return root.descendants();
    }
    
    return chart;
};