//$Id$
var canvas_plot_renderer = {},
    canvas_drawer = {};

$ZC.plot.canvasRenderer = function(chartObj, args) {
    "use strict";
    
    var canvasRenderer = {};
    var systemconf = chartObj.systemConf,
        helpers = $ZC.DataProcessor.helpers;
    
    
    canvasRenderer.draw = function(update) {
        /**
         * Disable legendHighlight
         */
        //systemconf.legend.highlightEffect.selectedSeries = "none";
        
        chartObj.container.select(".highlight-canvas").style("visibility", "hidden");
        
        chartObj.context = updateCanvasContext(chartObj, "zc-canvas");
        chartObj.labelHolderContext = updateCanvasContext(chartObj, "zc-labelholder-canvas", window.PointerEvent ? chartObj.container.selectAll("div.legendarea").node() : null);
        args.barDatasetIndex = 0;
        
        if (chartObj.systemConf.legend.enabled) {
            chartObj.svg.select(".seriesholder").selectAll("*:not(.zmarea)").remove(); // remove all except zoom are
        }
        
        if (chartObj.dataset.getDataType("x") == ORDINAL_DATATYPE && chartObj.isAxisCategory) {
            $ZC.scale.barBandwidth(chartObj);
        }
        
        var type = null;
        chartObj.chartRendered = false;
        var isComboChart = chartObj.dataObject.isCombinationalChart;
        var orderedSeriesdata = (isComboChart) ? renderer_helpers.arrangeSeriesRenderingOrder(chartObj) : chartObj.seriesdata;
        
        chartObj.seriesdata.forEach(function(d, i) {
            var currOrdereddata = orderedSeriesdata[i];
            var curSeriesidx = (isComboChart) ? currOrdereddata.seriesidx : i;
            
            if (!chartObj.isAxisCategory && (curSeriesidx > 0)) {
                return;
            }
            type = helpers.getChartType((currOrdereddata.type || chartObj.userdata.seriesdata.type), "string");
            
            if (chartObj.renderer[curSeriesidx] == null) {
                chartObj.renderer[curSeriesidx] = createCanvasRenderer(type);
                chartObj.rendererConf[curSeriesidx] = new $ZC.plot.rendererConf();
            }
            chartObj.renderer[curSeriesidx].chartRendered = false;
            
            if (i == 0) {
                stateChangeListener(chartObj, systemconf.chart.plot.events, "plot", "init");
                chartObj.hideLoading(chartObj.container);
            }
            
            renderer_helpers.setRendererProp(chartObj, chartObj.renderer[curSeriesidx], currOrdereddata, curSeriesidx, args);
            drawCanvas(chartObj, currOrdereddata, chartObj.renderer[curSeriesidx]);
        });
        renderer_helpers.renderCompletedEvents(chartObj, update, args);
        
        
        if (!update) {
            
            /**
             * Fade Effect
             */
            var commonPlotOptions = chartObj.systemConf.chart.plot.plotoptions[type];
            var animationOptions = getAnimationConf(commonPlotOptions, chartObj.seriesdata[0].plotoptions, chartObj, type);
            var animationDuration = defined(animationOptions.duration) ? animationOptions.duration : 400;
            if (!defined(animationOptions.enabled) || animationDuration === 0) {
                return
            }
            
            var totalSeries = chartObj.seriesdata.length;
            chartObj.context.canvas.style.opacity = 0;
            
            var fadeTimeout = setInterval(function() {
                var allRendered = true;
                if (chartObj.renderer.length === totalSeries) {
                    for (var i = 0; i < chartObj.renderer.length; i++) {
                        if (!chartObj.renderer[i].chartRendered) {
                            allRendered = false;
                            break
                        }
                    }
                }
                
                if (allRendered) {
                    clearInterval(fadeTimeout);
                    d3_select(chartObj.context.canvas)
                        .transition()
                        .duration(animationDuration)
                        .style("opacity", 1);
                }
            }, 1000 / 60);
        }
    };
    
    return canvasRenderer;
};

function createCanvasRenderer(chartName) {
    switch (chartName) {
        case "bullet": {
            chartName = "bar";
            break;
        }
        case "arearange": {
            chartName = "area";
            break;
        }
        case "bubble": {
            chartName = "scatter";
            break;
        }
    }
    return new canvas_plot_renderer[chartName]();
}

function drawCanvas(chartObj, renderdata, renderer, context, styles, filterIndex) {
    
    
    context = defined(context) ? context : chartObj.context;
    var wrap = chartObj.svg.selectAll('g.xychart');
    transformWrap(wrap, "seriesholder", {
        x: chartObj.plotarea.left,
        y: chartObj.plotarea.top
    });
    transformWrap(chartObj.staticPaneSVG, "labelholder", {
        x: chartObj.plotarea.left,
        y: chartObj.plotarea.top
    });
    renderer(context, renderdata, styles, filterIndex);
    
    function transformWrap(wrap, selection, translate) {
        return (wrap.select('g.' + selection)
            .attr('transform', 'translate(' + (translate.x) + ',' + (translate.y) + ')'));
    }
}

function updateCanvasContext(chartObj, selector, prevElement){
    var parentNode = chartObj.svg.node().parentNode;
    var canvas = chartObj.svg.node().parentNode.querySelector("canvas." + selector);
    var width = chartObj.plotarea.width,
        height = chartObj.plotarea.height;
    
    if (!canvas) {
        canvas = document.createElement("canvas");
        canvas.className = selector;
        canvas.setAttribute("actual-width", width);
        canvas.setAttribute("actual-height", height);
        canvas.style.position = "absolute";
        canvas.style.pointerEvents = "none";
        parentNode.insertBefore(canvas, prevElement || chartObj.staticPaneSVG.node());
    }
    
    canvas.width = width;
    canvas.height = height;
    canvas.style.left = mathRound(chartObj.plotarea.left) + "px";
    canvas.style.top = mathRound(chartObj.plotarea.top) + "px";
    canvas.style.visibility = "visible";
    var context = canvas.getContext("2d");
    if (window.devicePixelRatio) {
        canvas.style.width = mathFloor(width) + "px";
        canvas.style.height = mathFloor(height) + "px";
        canvas.width = width * window.devicePixelRatio;
        canvas.height = height * window.devicePixelRatio;
        context.scale(window.devicePixelRatio, window.devicePixelRatio);
    } else {
        window.devicePixelRatio = 1;
    }
    context.clearRect(0, 0, width, height);
    return context;
}