//$Id$
$ZC.plot.imageRenderer = function(chartObj, args) {
    var imageRenderer = {};
    
    function renderImage() {
        
        chartObj.chartRendered = false;
        
        // place the seriesholder
        
        var wrap = chartObj.svg.selectAll('g.xychart');
        transformWrap(wrap, "seriesholder", {
            x: chartObj.plotarea.left,
            y: chartObj.plotarea.top
        });
        
        transformWrap(wrap, "labelholder", {
            x: chartObj.plotarea.left,
            y: chartObj.plotarea.top
        });
        
        function transformWrap(wrap, selection, translate) {
            return (wrap.select('g.' + selection)
                .attr('transform', 'translate(' + (translate.x) + ',' + (translate.y) + ')')) //No I18N
        }
        
        var seriesholder = chartObj.svg.select(".seriesholder");
        
        /*
         * Hide all the threshold
         */
        
        chartObj.svg.selectAll(".threshold").style({
            // "visibility": "hidden",
            "opacity": 0
        });
        
        /*
         * If image source is available use that and generate the plot 
         */
        
        var imageSource = chartObj.systemConf.chart.plot.renderer.image.source;
        if (imageSource != "auto") {
            seriesholder.selectAll("*:not(.zmarea)").remove(); // remove all except zoom are
            createImageObj(imageSource);
            return;
        }
        
        seriesholder.selectAll("*:not(.zmarea)").remove(); // remove all except zoom area
        chartObj.showLoading(chartObj.container);
        
        var actualCType = chartObj.seriesdata[0].type || chartObj.userdata.seriesdata.type;
        var cType = getChartType(actualCType, "string");
        var sysConf = chartObj.systemConf;
        var dataLebelFontSize = sysConf.chart.plot.plotoptions[cType].datalabels.fontSize;
        dataLebelFontSize = (typeof dataLebelFontSize === 'function') ? dataLebelFontSize() : dataLebelFontSize;
        sysConf.chart.plot.plotoptions[cType].datalabels.fontSize = dataLebelFontSize;
        
        var imagedata = JSON.parse(JSON.stringify(sysConf));
        imagedata.chart.plot.renderer.mode = "svg";
        
        
        /**
         * check the datalabels color
         */
        
        if (!isObjectPropDefined(chartObj.userdata, "chart.plot.datalabels.fontColor")) {
            delete imagedata.chart.plot.datalabels.fontColor
        }
        
        if (!isObjectPropDefined(chartObj.userdata, "chart.plot.plotoptions." + cType + ".datalabels.fontColor")) {
            delete imagedata.chart.plot.plotoptions[cType].datalabels.fontColor
        }
        
        // request aarthi
        
        var chartData = {
            "url": chartObj.systemConf.chart.plot.renderer.image.exportUrl,
            "type": "POST",
            "data": {
                "DATA": encodeURIComponent(JSON.stringify(imagedata)),
                "HEIGHT": mathRound(chartObj.plotarea.height),
                "WIDTH": mathRound(chartObj.plotarea.width)
            }
        };
        
        if (chartObj.xhr) {
            chartObj.xhr.abort();
        }
        
        var data = urlize(chartData.data);
        var xhr = d3_xhr(chartData.url)
            .header("Content-type", "application/x-www-form-urlencoded")
            .post(data, function(error, text) {
                seriesholder.selectAll("image").remove();
                var response = JSON.parse(text.responseText);
                if (response.rendered) {
                    chartObj.hideLoading(chartObj.container);
                    createImageObj(response.serverPath);
                }
                delete chartObj.xhr;
            });
        chartObj.xhr = xhr;
        
        function urlize(data) {
            var temp = "";
            var keys = Object.keys(data);
            var i = 0;
            
            for (var key in data) {
                temp += key + "=" + data[key];
                if (data.hasOwnProperty(keys[i + 1])) {
                    temp += "&";
                }
                i++;
            }
            return temp;
        }
        
        function createImageObj(url) {
            if (chartObj.systemConf.chart.plot.renderer.image.addDimensionInSource === true) {
                var symble = (url.indexOf("?") > -1) ? "&" : "?";
                url += symble + "WIDTH=" + chartObj.plotarea.width + "&HEIGHT=" + chartObj.plotarea.height;
            }
            
            var transitionDuration = 300;
            
            var img = seriesholder.append("svg:image")
                .attr("xlink:href", url)
                .attr("width", mathRound(chartObj.plotarea.width))
                .attr("height", mathRound(chartObj.plotarea.height))
                .attr("x", 0)
                .attr("y", 0)
                .style("opacity", 0)
                .attr("onload", function() {
                    chartObj.hideLoading(chartObj.container);
                    chartObj.svg.selectAll(".threshold").transition().style("opacity", "1").duration(transitionDuration);
                })
                .on("error", function() {
                    chartObj.chartRendered = false;
                    chartObj.hideLoading(chartObj.container);
                    throw new ReferenceError("Invalid image source.");
                })
                .transition()
                .style("opacity", "1")
                .duration(transitionDuration)
                .each("end", function() {
                    chartObj.chartRendered = true;
                })
        }
    }
    
    imageRenderer.draw = function(update) {
        
        chartObj.systemConf.legend.highlightEffect.selectedSeries = "none";
        chartObj.systemConf.loader.enabled = true;
        
        if (chartObj.dataset.getDataType("x") == ORDINAL_DATATYPE && chartObj.isAxisCategory) {
            $ZC.scale.barBandwidth(chartObj);
        }
        
        args.barDatasetIndex = 0;
        var type;
        chartObj.chartRendered = false;
        var isComboChart = chartObj.dataObject.isCombinationalChart;
        var orderedSeriesdata = (isComboChart) ? renderer_helpers.arrangeSeriesRenderingOrder(chartObj, chartObj.seriesdata) : chartObj.seriesdata
        
        // draw the image;
        renderImage();
        chartObj.seriesdata.forEach(function(d, i) {
            var currOrdereddata = orderedSeriesdata[i];
            var curSeriesidx = (isComboChart) ? currOrdereddata.seriesidx : i;
            
            if (!chartObj.isAxisCategory && (curSeriesidx > 0)) {
                return;
            }
            
            var chartType = getChartType(null, "object", curSeriesidx, chartObj.userdata),
                cType = chartType.name;
            
            if (chartObj.renderer[curSeriesidx] == null) {
                type = getChartType((currOrdereddata.type || chartObj.userdata.seriesdata.type), NUMERIC_DATATYPE);
                chartObj.renderer[curSeriesidx] = createSVGRenderer(cType);
                chartObj.rendererConf[curSeriesidx] = new $ZC.plot.rendererConf();
            } else {
                chartObj.renderer[curSeriesidx].commonOption = null;
            }
            renderer_helpers.setRendererProp(chartObj, chartObj.renderer[curSeriesidx], currOrdereddata, curSeriesidx, args);
            chartObj.renderer[curSeriesidx].chartRendered = false;
            if (i == 0) {
                stateChangeListener(chartObj, chartObj.systemConf.chart.plot.events, "plot", "init"); //No I18N
            }
        });
        renderer_helpers.renderCompletedEvents(chartObj, update, args);
    };
    return imageRenderer;
};