//$Id$
$ZC.plot.render = function(chartObj) {
    var mode = chartObj.systemConf.chart.plot.renderer.mode.toLowerCase();
    var chartType = chartObj.chartTypes[0];
    var render = {
            "mode": (!chartObj.isAxisCategory || chartObj.dataObject.pieWithAxis || chartType.value === 24 || isHeatMapChartType(chartType.value)) ? "svg" : mode.toLocaleLowerCase()
        },
        args = {};
    
    args.labelQueue = [];
    args.notesArray = [];
    args.labelArray = [];
    
    if (!chartObj.rendererConf) {
        chartObj.rendererConf = [];
    }
    
    if (typeof $ZC.plot[render.mode + "Renderer"] === "function") {
        render[render.mode] = $ZC.plot[render.mode + "Renderer"](chartObj, args);
    } else {
        throw new SyntaxError("" + render.mode + "Renderer not found");
    }
    
    render.draw = function(update) {
        chartObj.chartRendered = false;
        this[this.mode].draw(update);
    };
    return render;
};

renderer_helpers.renderCompletedEvents = function(chartObj, update, args) {
    var systemconf = chartObj.systemConf,
        isDialChart = (chartObj.dataObject.hasLevelMarker && !chartObj.isAxisCategory),
        DoWaitForChartRendered = ((chartObj.isAxisCategory || chartObj.dataObject.isPolarAxisCategory) && !chartObj.dataObject.pieWithAxis) || isDialChart,
        renderer = chartObj.renderer,
        k,
        axesRendered = false;
    // var labelHolder = chartObj.staticPaneSVG.select(".labelholder").style("opacity", 0);

    if (chartObj.isAxisCategory || chartObj.dataObject.isPolarAxisCategory) {
        setTimeout(function() {
            axesRendered = true;
        }, chartObj.axes.animationDuration + 17);
    } else {
        axesRendered = true;
    }
    
    var checkRendered = function() {
        if (isHiddenDocument()) {
            chartObj.chartRendered = true;
        }

        if(chartObj.destroying) {
            return true;
        }

        k = renderer.length;
        while (k--) {
            if (renderer[k].chartRendered !== true || !axesRendered) {
                return false;
            }
            args.notesArray = renderer[k].commonRendererProp().notes();
        }
        proceed();
        chartObj.chartRendered = true;
        clearInterval(timer);
        return true;
    };
    
    function proceed() {
        // labelHolder.transition().duration(250).style("opacity", 1);
        stateChangeListener(chartObj, systemconf.chart.plot.events, "plot", (update) ? "onreload" : "onload");
        callDatalableAndAnnotationRenderer(chartObj, args, !(DoWaitForChartRendered && update));
        chartObj.handleHiddenContainer(true);
        stateChangeListener(chartObj, systemconf.canvas.events, "chart", (update) ? "onreload" : "onload");
    }
    
    var timer = setInterval(checkRendered, (1000 / 60));
    
    if (DoWaitForChartRendered) {
        callDatalableAndAnnotationRenderer(chartObj, args, (isDialChart || update));
    }
};

function callDatalableAndAnnotationRenderer(chartObj, args, enable) {
    /*
     * It will call datalabels renderer and annotation renderer
     */
    if (enable) {
        if ($ZC.labelRenderer) {
            $ZC.labelRenderer.drawLabelsFromQueue(args.labelQueue, args.labelArray, chartObj);
        }
        args.labelArray = [];
        args.labelQueue = [];
        args.notesArray = $ZC.annotation ? $ZC.annotation.draw(chartObj, args.notesArray) : [];
    }
}

renderer_helpers.arrangeSeriesRenderingOrder = function(chartObj) {
    var seriesdata = chartObj.seriesdata,
        dataProcessorHelpers = $ZC.DataProcessor.helpers;

    if (chartObj.dataObject.isCombinationalChart) {
        var order = {
                defaultType: 0,
                area: 0,
                bar: 1,
                line: 2,
                bubble: 3,
                bubblepie: 4,
                scatter: 5
            },
            globalChartType = chartObj.userdata.seriesdata.type;
        
        var seriesdata_cpy = simpleClone(seriesdata);
        seriesdata_cpy.map(function(d, i) {
            d.seriesidx = i;
            return d
        });
        seriesdata_cpy.sort(function(a, b) {
            var type1 = dataProcessorHelpers.getChartType((a.type || globalChartType), "string"),
                type2 = dataProcessorHelpers.getChartType((b.type || globalChartType), "string");
            var order1 = (order[type1] == null) ? order.defaultType : order[type1],
                order2 = (order[type2] == null) ? order.defaultType : order[type2],
                seriesidx1 = a.seriesidx,
                seriesidx2 = b.seriesidx;
            return order1 < order2 ? -1 : order1 > order2 ? 1 //Arrange by chart type
                    : (seriesidx1 < seriesidx2 ? -1 : seriesidx1 > seriesidx2 ? 1 : 0); //Arrange by series index
        });
        return seriesdata_cpy;
    }
    return seriesdata;
};


renderer_helpers.setRendererProp = function(chartObj, renderer, seriesdata, index, args) {
    var chartType = chartObj.chartTypes[index];
    var cType = chartType.name;
    var cTypeNumeric = chartType.value;
    var systemconf = chartObj.systemConf,
        helpers = $ZC.DataProcessor.helpers;
    
    setCommonRendererProp(renderer, seriesdata, index, cType, cTypeNumeric);
    renderer.commonRendererProp(chartObj.rendererConf[index])
    var yaxiscolumnorder = chartObj.seriesdata[index].yaxiscolumnorder;
    var yscindex = helpers.getParsedYAxisOrder(yaxiscolumnorder);
    
    renderer.chartRendered = false;
    
    if (chartObj.xymultiRenderer || chartObj.dataObject.isPolarAxisCategory) {
        var yaxis = chartObj.axes.y.groups[yscindex];
        var ysc = yaxis.scale;
        
        chartObj.rendererConf[index]
            .xscale(chartObj.axes.x.scale)
            .yscale(ysc)
            .datasetIndex(index);
        
        args.labelArray = chartObj.rendererConf[index].params.labelArray;
        args.labelQueue = chartObj.rendererConf[index].params.labelQueue;
    }
    if (isBarChartType(cTypeNumeric) && !isHeatMapChartType(cTypeNumeric)) {
        renderer
            .bandwidth1(chartObj.barscale1)
            .bandwidth2(chartObj.barscale2)
        if (!args.image) {
            renderer.barDatasetIndex(args.barDatasetIndex);
        }
        if (seriesdata.disabled == null || !seriesdata.disabled) {
            args.barDatasetIndex = args.barDatasetIndex + chartObj.seriesdata[index].data.length;
        }
    }
    
    function setCommonRendererProp(renderer, seriesdata, index, cType, actualCType) {
        var seriesClr = chartObj.seriesdata[index].color || chartObj.seriesColor[index % chartObj.seriesColor.length];
        var yaxiscolumnorder = chartObj.seriesdata[index].yaxiscolumnorder;
        var yscindex = helpers.getParsedYAxisOrder(yaxiscolumnorder);
        var commonProp = chartObj.rendererConf[index];
        commonProp
            .chartObj(chartObj)
            .dataset(chartObj.dataset)
            .axisrotated(chartObj.dataObject.getAxisRotated())
            .yaxiscolumnorder(yscindex)
            .hasNegValue(chartObj.dataObject.containsNegativeData(yscindex))
            .stacked(chartObj.dataObject.isstacked(actualCType))
            .commonPlotOptions(chartObj.systemConf.chart.plot.plotoptions[cType])
            .color((cType == "bubblepie") ? chartObj.seriesColor : (chartObj.isAxisCategory) ? checkNgetColorOverlayData(renderer, seriesdata, seriesClr, index, yscindex, cType) : (chartObj.dataObject.isPolarAxisCategory) ? seriesClr : chartObj.seriesColor)
            .notes(args.notesArray)
            .updateLabelArrayAndLabelQueue(args.labelArray, args.labelQueue);
    }

    function checkNgetColorOverlayData(renderer, seriesdata, seriesClr, index, yscindex, cType) {
        var thOverlay_data_x = systemconf.chart.axes.xaxis.threshold.coloroverlay;
        var thOverlay_data_y = (systemconf.chart.axes.yaxis[yscindex] != null && systemconf.chart.axes.yaxis[yscindex].threshold != null) ? systemconf.chart.axes.yaxis[yscindex].threshold.coloroverlay : null;
        var showColorOverlay = (thOverlay_data_x.show) || (thOverlay_data_y != null && thOverlay_data_y.show);
        if (!showColorOverlay) {
            return seriesClr
        } else {
            var overlayData = ((thOverlay_data_y != null && thOverlay_data_y.show) ? thOverlay_data_y : thOverlay_data_x.show ? thOverlay_data_x : null);
            var negThreshType = ((thOverlay_data_y != null && thOverlay_data_y.show) ? "y" : thOverlay_data_x.show ? "x" : null);
            var colorOverlay = (showColorOverlay) ? $ZC.axis.threshold.overlay(chartObj, "continous") : null,
                axisOrder = (negThreshType === "y") ? yscindex : 0;

            chartObj.rendererConf[index]
                .threshold(showColorOverlay ? overlayData : null)
                .thresholdType(negThreshType)
                .colorOverlay(showColorOverlay ? $ZC.axis.threshold.overlay(chartObj, "discrete", overlayData) : null);

            return (cType == "bar" || cType == "scatter" || cType == "bubble") ? seriesClr : colorOverlay(seriesClr, overlayData, axisOrder, negThreshType)
        }
        
    }
};

function getBarCommonOption(conf, index) {
    var rp = conf.params,
        stacked = rp.stacked,
        chartObj = rp.chartObj,
        yscale = rp.yscale,
        dataProcessorHelpers = $ZC.DataProcessor.helpers,
        barDataSetIndex = chartObj.renderer[index].barDatasetIndex ? chartObj.renderer[index].barDatasetIndex() : 0,
        commonPlotOptions = rp.commonPlotOptions,
        charttype = dataProcessorHelpers.getChartType((chartObj.seriesdata[index].type || chartObj.userdata.seriesdata.type), "string"),
        isBullet = (charttype == "bullet"),
        groupedByStack = chartObj.dataObject.groupedByStack,
        levelMarkerOptions = commonPlotOptions.levelMarker || {};
    
    
    var wid = chartObj.barscale2.rangeBand() || 0,
        levelMarkerWidth = wid,
        barWidth = +(commonPlotOptions.bandWidth || null) || (isBullet ? (wid / 3) : wid);
    
    
    var levelMarkerBandWidth = +(levelMarkerOptions.bandWidth || wid),
        levelMarkerMaxBandWidth = +(levelMarkerOptions.maxBandWidth || wid),
        maxBandWidth = +(commonPlotOptions.maxBandWidth || wid); //setting bandwidth if the actual bandwidth is greater than the maximum bar bandwidth
    
    levelMarkerWidth = mathMin(levelMarkerBandWidth, levelMarkerMaxBandWidth, wid);
    barWidth = mathMin(barWidth, maxBandWidth, wid);
    
    var barPadding = mathMax(0, (wid - barWidth) / 2),
        levelMarkerPadding = mathMax(0, (wid - levelMarkerWidth) / 2);
    
    
    var addRangeBand = function() {
        return yscale.rangeBand ? (yscale.rangeBand() / 2) : 0;
    };
    
    var commonOption = {
        "stacked": stacked,
        "categories": rp.xscale.domain(),
        "axisrotated": rp.axisrotated,
        "bandwidth1": chartObj.barscale1,
        "bandwidth2": chartObj.barscale2,
        "barDatasetIndex": barDataSetIndex,
        "padding": barPadding,
        "yscale": yscale,
        "hasNegValue": rp.hasNegValue,
        "yaxiscolumnorder": rp.yaxiscolumnorder,
        "addRangeBand": addRangeBand,
        "barWidth": barWidth,
        "levelMarkerWidth": levelMarkerWidth,
        "levelMarkerPadding": levelMarkerPadding,
        "stackGroupIndex": (chartObj.userdata.seriesdata.stackGroup) ?
            getStackGroupIndex(index, groupedByStack.keys(), function(d) {
                var value = groupedByStack.get(d);
                return value.map(function(d) {
                    return d.si;
                });
            }) :
            0
    };
    chartObj.renderer[index].commonOption = commonOption;
    return commonOption;
}

function getStackGroupIndex(index, array, accessor) {
    return d3_max(array, function(d, j) {
        var value = (accessor) ? accessor(d, j) : d;
        return (value.indexOf(index) != -1) ? j : -1;
    });
}