/**
 *
 * @param chartObj
 * @param args
 * @returns {{}}
 */
$ZC.plot.svgRenderer = function(chartObj, args) {
    var svgRenderer = {};
    var systemconf = chartObj.systemConf,
        helpers = $ZC.DataProcessor.helpers;

    svgRenderer.draw = function(update) {
        
        if (chartObj.dataset.getDataType("x") == ORDINAL_DATATYPE && chartObj.isAxisCategory) {
            $ZC.scale.barBandwidth(chartObj);
        }
        args.barDatasetIndex = 0;
        
        var type, isRendererCalled = [];
        chartObj.chartRendered = false;
        var isComboChart = chartObj.dataObject.isCombinationalChart;
        var orderedSeriesdata = (isComboChart) ? renderer_helpers.arrangeSeriesRenderingOrder(chartObj) : chartObj.seriesdata
        var bubblePieRendererIndex;
        
        chartObj.seriesdata.forEach(function(d, i) {
            var currOrdereddata = orderedSeriesdata[i]
            var curSeriesidx = (isComboChart) ? currOrdereddata.seriesidx : i;
            
            if ((curSeriesidx > 0) && !chartObj.xymultiRenderer && !chartObj.dataObject.isPolarAxisCategory) {
                return;
            }
            
            type = helpers.getChartType((currOrdereddata.type || chartObj.userdata.seriesdata.type), NUMERIC_DATATYPE);
            if (type == $ZC.charttype.bubblepie) { //Only one bubblepie renderer will be called eventhough it has so many series. Share that renderer to all the bubblepie series
                if (bubblePieRendererIndex != null) {
                    chartObj.renderer[curSeriesidx] = chartObj.renderer[bubblePieRendererIndex];
                    chartObj.rendererConf[curSeriesidx] = new $ZC.plot.rendererConf();
                    renderer_helpers.setRendererProp(chartObj, chartObj.renderer[curSeriesidx], currOrdereddata, curSeriesidx, args);
                    return;
                } else {
                    chartObj.bubblePieRendererIndex = bubblePieRendererIndex = curSeriesidx;
                }
            }
            
            var animated = [],
                chartName = $ZC.charttypenames.get(type),
                plotoptions = currOrdereddata.plotoptions,
                commonPlotOptions = systemconf.chart.plot.plotoptions[chartName];
            var animationOptions = getAnimationConf(commonPlotOptions, plotoptions, chartObj, chartName);
            var isAnimationEnable = animationOptions.enabled,
                animationType = animationOptions.type;
            
            if (chartObj.renderer[curSeriesidx] == null) {
                chartObj.renderer[curSeriesidx] = createSVGRenderer(chartName)
                chartObj.rendererConf[curSeriesidx] = new $ZC.plot.rendererConf();
            }
            chartObj.renderer[curSeriesidx].chartRendered = false;
            
            if (i == 0) {
                stateChangeListener(chartObj, systemconf.chart.plot.events, "plot", "init");
                chartObj.hideLoading(chartObj.container);
            }
            if (isAnimationEnable && !/(allseries|pointTween|vertPathTweenAll|fadeAll|verticalAll|sizing)/.test(animationType) && (type == $ZC.charttype.scatter || type == $ZC.charttype.area || type == $ZC.charttype.arearange || type == $ZC.charttype.bubble) && !update) {
                isRendererCalled[curSeriesidx] = false;
                animated[curSeriesidx] = setInterval(function() {
                    if (chartObj.renderer[curSeriesidx].chartRendered) {
                        clearInterval(animated[curSeriesidx]);
                    }
                    //isRendererCalled - every renderer should be called only once not at every 10 seconds. All the renderers except first one should wait until previous renderer is rendered
                    if (!isRendererCalled[curSeriesidx] && ((curSeriesidx == 0) || (curSeriesidx > 0 && chartObj.renderer[curSeriesidx - 1].chartRendered))) {
                        renderer_helpers.setRendererProp(chartObj, chartObj.renderer[curSeriesidx], currOrdereddata, curSeriesidx, args);
                        drawSVG(chartObj, currOrdereddata, chartObj.renderer[curSeriesidx]);
                        isRendererCalled[curSeriesidx] = true;
                    }
                }, 10);
            } else {
                renderer_helpers.setRendererProp(chartObj, chartObj.renderer[curSeriesidx], currOrdereddata, curSeriesidx, args);
                drawSVG(chartObj, currOrdereddata, chartObj.renderer[curSeriesidx]);
            }
        });
        renderer_helpers.renderCompletedEvents(chartObj, update, args);
    };
    return svgRenderer;
};

var createSVGRenderer = function(chartName) {
    switch (chartName) {
        case "gantt":
        case "bullet": {
            chartName = "bar";
            break;
        }
        case "web":
        case "arearange": {
            chartName = "area";
            break;
        }
        case "bubble":{
            chartName = "scatter";
            break;
        }
    }
    return new svg_plot_renderer[chartName]();
};

var drawSVG = function(chartObj, renderdata, renderercollection, update) {
    if (update) {
        renderercollection.update();
    } else {
        clearTimeout(renderercollection.renderedTimer);
        var wrap = chartObj.svg.selectAll('g.xychart').data([chartObj.systemConf])
        transformWrap(wrap, "seriesholder", {
            x: chartObj.plotarea.left,
            y: chartObj.plotarea.top
        })
            .datum(chartObj.seriesdata)
            .call(renderercollection)
        
        if (chartObj.dataObject.isPolarAxisCategory) {
            wrap.selectAll("g.seriesholder").selectAll(function() {
                    return this.childNodes;
                })
                .attr("transform", "translate(" + chartObj.plotarea_approx.x + "," + chartObj.plotarea_approx.y + ")");
        }
        
        transformWrap(chartObj.staticPaneSVG, "labelholder", {
            x: chartObj.plotarea.left,
            y: chartObj.plotarea.top
        })
    }
    
    function transformWrap(wrap, selection, translate) {
        return (wrap.select('g.' + selection)
            .attr('transform', 'translate(' + (translate.x) + ',' + (translate.y) + ')')) //No I18N
    }
};