//$Id$
$ZC.axis.polar = function() {
    var scale, chartObj, radius, columnIndices, tickFormat, tickValues,
        columnInfo;
    
    function axis(g) {
        g.each(function(data) {
            var g = d3_select(this);
            var systemconf = chartObj.systemConf,
                axisline = data.axisline,
                tickmark = data.tickmark,
                subtickmark = data.subtickmark || {},
                grid = data.grid,
                ticklabel = data.ticklabel,
                events = ticklabel.events,
                mousemoveFn, mouseoutFn, clickFn, dblclickFn, cursor;
            if (events != null) {
                mousemoveFn = events.mousemove;
                mouseoutFn = events.mouseout;
                clickFn = events.click;
                dblclickFn = events.doubleclick;
                cursor = events.cursor;
            }
            
            var tickOrient = data.tickOrient,
                axislineSize = axisline.show ? validateNullVal(axisline.strokeWidth, 1) : 0,
                gridSize = validateNullVal(grid.strokeWidth, 1),
                ticklabelPad = validateNullVal(ticklabel.marginLeft, 5),
                tickmarkSize = validateNullVal(tickmark.size, 3),
                subtickmarkSize = validateNullVal(subtickmark.size, 2),
                subticksCount = (+subtickmark.count || 0);
            
            var ticks = tickValues || (scale.ticks ? scale.ticks() : scale.domain()),
                subticks = d3_svg_axisSubdivide(scale, ticks, subticksCount),
                range = scale.rangeExtent ? scale.rangeExtent() : scale.range();
            
            var tick = g.selectAll(".tick").data(ticks, scale);
            var tickEnter = tick.enter().insert("g", ".domain").attr("class", "tick");
            tick.exit().remove();
            
            var subticksGroup = g.selectAll(".subticks").data([1]);
            subticksGroup.enter().insert("g", "g").attr("class", "subticks");
            subticksGroup
                .style("stroke", subtickmark.color);
            var subtick = appendEle(subticksGroup, "line", subticks);
            
            var path = appendEle(g, "path", (axisline.show ? [1] : []), ".", "class", "domain");
            path
                .attr("d", function() {
                    var path = d3_arc()
                        .innerRadius(radius - 1)
                        .outerRadius(radius)
                        .startAngle(range[0])
                        .endAngle(range[1])(1);
                    return "M" + path.split(/L|M/gi)[1];
                })
                .style({ //Axis line properties
                    "stroke": axisline.color,
                    "fill": "none",
                    "stroke-width": axislineSize,
                    "stroke-dasharray": getDashArray(axisline.style, axislineSize),
                    "stroke-linecap": getLineCap(axisline.lineCap, axisline.style),
                    "pointer-events": "none"
                });
            
            tickEnter.append("line").attr("class", "tickmark");
            tickEnter.append("line").attr("class", "grid");
            tickEnter.append("text");
            var text = tick.select("text"),
                line = tick.select("line.tickmark"),
                gridLine = tick.select("line.grid");
            
            text
                .style("cursor", cursor || "pointer")
                .text(function(d) {
                    return tickFormat(d);
                });
            applyFont(text, [ticklabel, systemconf.chart, systemconf.canvas, $ZCG]);
            
            columnInfo = {
                "data": columnIndices.map(function(d) {
                    return mergeJSON(chartObj.systemConf.metadata.columns[d], {
                        "columnindex": d
                    }, true);
                })
            };

            var mousemove = function(d) {
                    callback(mousemoveFn, d3.event, d);
                },
                mouseout = function(d) {
                    callback(mouseoutFn, d3.event, d);
                },
                click = function(d) {
                    callback(clickFn, d3.event, d);
                },
                dblclick = function(d) {
                    callback(dblclickFn, d3.event, d);
                };
            var mouseHandlers = {
                    "mousemove": mousemove,
                    "mouseout": mouseout,
                    "click": click,
                    "dblclick": dblclick
                },
                touchHandlers = {
                    "singletouchmove": mousemove,
                    "touchend": mouseout,
                    "tap": click,
                    "dbltap": dblclick
                };
            attachevents(text, mouseHandlers, touchHandlers);
            
            var tickRad, textRad, tickmarkRad, subtickmarkRad;
            switch (tickOrient) {
                case "inner":
                    tickRad = radius - axislineSize / 2;
                    textRad = -(tickmarkSize + ticklabelPad);
                    tickmarkRad = -tickmarkSize;
                    subtickmarkRad = -subtickmarkSize;
                    break;
                default:
                    tickRad = radius + axislineSize / 2;
                    textRad = tickmarkSize + ticklabelPad;
                    tickmarkRad = tickmarkSize;
                    subtickmarkRad = subtickmarkSize;
            }
            tick
                .attr("transform", function(d) {
                    var xy = polarToXY(tickRad, scale(d));
                    return "translate(" + xy[0] + "," + xy[1] + ")";
                });
            text
                .attr({
                    "dy": function(d) {
                        var a = d0_360(scale(d));
                        return (tickOrient == "inner") ?
                            ((!((a > mathPI / 2) && (a < 3 * mathPI / 2))) ? -getDY(a) + "em" : null) :
                            (((a > mathPI / 2) && (a < 3 * mathPI / 2)) ? getDY(a) + "em" : null);
                    },
                    "text-anchor": function(d) {
                        var a = d0_360(scale(d));
                        return (a % mathPI == 0) ?
                            "middle" :
                            (((tickOrient == "inner" && a < mathPI) || (tickOrient != "inner" && a > mathPI)) ? "end" : null);
                    },
                    "transform": function(d) {
                        var xy = polarToXY(textRad, scale(d));
                        return "translate(" + xy[0] + "," + xy[1] + ")";
                    }
                });
            line
                .attr({
                    "x2": function(d) {
                        return polarToXY(tickmarkRad, scale(d))[0];
                    },
                    "y2": function(d) {
                        return polarToXY(tickmarkRad, scale(d))[1];
                    }
                })
                .style({
                    "stroke": tickmark.color,
                    "stroke-width": tickmark.strokeWidth
                }); //tickmark properties
            
            gridLine
                .attr({
                    "x2": function(d) {
                        return polarToXY(-radius, scale(d))[0];
                    },
                    "y2": function(d) {
                        return polarToXY(-radius, scale(d))[1];
                    }
                })
                .style({
                    "stroke": grid.color,
                    "stroke-width": gridSize,
                    "stroke-dasharray": getDashArray(grid.style, gridSize),
                    "stroke-linecap": getLineCap(grid.lineCap, grid.style)
                }); //grid properties
            
            subtick
                .attr({
                    "x2": function(d) {
                        return polarToXY(subtickmarkRad, scale(d))[0];
                    },
                    "y2": function(d) {
                        return polarToXY(subtickmarkRad, scale(d))[1];
                    },
                    "transform": function(d) {
                        var xy = polarToXY(tickRad, scale(d));
                        return "translate(" + xy[0] + "," + xy[1] + ")"
                    }
                });
            
        });
    }
    
    axis.scale = function(x) {
        if (!arguments.length) return scale;
        scale = x;
        return axis;
    };
    axis.chartObj = function(x) {
        if (!arguments.length) return chartObj;
        chartObj = x;
        return axis;
    };
    axis.radius = function(x) {
        if (!arguments.length) return radius;
        radius = x;
        return axis;
    };
    axis.columnIndices = function(_) {
        if (!arguments.length) {
            return columnIndices;
        }
        columnIndices = _;
        return axis;
    };
    axis.tickValues = function(_) {
        if (!arguments.length) {
            return tickValues;
        }
        tickValues = _;
        return axis;
    };
    axis.tickFormat = function(_) {
        if (!arguments.length) {
            return tickFormat;
        }
        tickFormat = _;
        return axis;
    };
    return axis;
    
    function d3_svg_axisSubdivide(scale, ticks, m) {
        subticks = [];
        if (m && ticks.length > 1) {
            var extent = d3_scaleExtent(scale.domain()),
                subticks, i = -1,
                n = ticks.length,
                d = (ticks[1] - ticks[0]) / ++m,
                j, v;
            while (++i < n) {
                for (j = m; --j > 0;) {
                    if ((v = +ticks[i] - j * d) >= extent[0]) {
                        subticks.push(v);
                    }
                }
            }
            for (--i, j = 0; ++j < m && (v = +ticks[i] + j * d) < extent[1];) {
                subticks.push(v);
            }
        }
        return subticks;
    }
    
    function d3_scaleExtent(domain) {
        var start = domain[0],
            stop = domain[domain.length - 1];
        return start < stop ? [start, stop] : [stop, start];
    }
    
    function callback(func, e, d) {
        if (func != null) {
            columnInfo.data = columnInfo.data.map(function(dt) {
                dt.value = d;
                return dt;
            });
            invokeFunction(func, e, columnInfo, chartObj);
        }
    }
};