//$Id$
/**
 *
 * @returns {chart} pyramid renderer.
 */
svg_plot_renderer.pyramid = function() {
    var MAXWIDTH_DATALABELS = 0.3;
    var cfg = {};
    var animationType;
    var commonRendererProp, dataset, yaxiscolumnorder, isredraw = false;
    var chartWidth, chartHeight, commonPlotOptions, systemconf, notesArray, isPatternPalleteEnabled, defsLocation;
    var height1, total, b, h, side1, side2, side3, side4, hScale, color, heightArray, chartObj, pyramidGroup, gradientDim, pyramidsections, dataLength, data_backup,
        datalabelPadding,
        fillColor;
    
    function chart(selection) {
        selection.each(function(data) {
            //renderer config values
            var rp = commonRendererProp.params;
            chartObj = rp.chartObj,
                commonPlotOptions = rp.commonPlotOptions,
                color = rp.color,
                notesArray = rp.notesArray,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset;
            systemconf = chartObj.systemConf;
            var labelQueue = rp.labelQueue;
            
            defsLocation = chartObj.defsLocation;
            
            chartWidth = chartObj.plotarea.width,
                chartHeight = chartObj.plotarea.height;
            var minD = mathMin(chartWidth, chartHeight);
            
            datalabelPadding = commonPlotOptions.datalabels.padding;
            var showlabel = isDatalabelEnabled(commonPlotOptions.datalabels, chartObj);
            if (showlabel) {
                /*
                 *If chart width is lesser than height, datalabels are hidden as default padding is 10 and there is no space to show datalabels.
                 * So 30% of the width is allocated for datalabels and the default padding will be changed to 5
                 */
                var minDatalabelsWidth = chartWidth * MAXWIDTH_DATALABELS,
                    availableDatalabelsWidth = chartWidth - minD;
                if (availableDatalabelsWidth < minDatalabelsWidth) {
                    datalabelPadding = cfg.datalabelPadding = 5;
                    minD -= mathMax((minDatalabelsWidth - availableDatalabelsWidth), 0);
                }
            }
            cfg.width = minD;
            cfg.height = minD * 4 / 5;
            
            var plotoptions = data[0].plotoptions,
                gvp = $ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions);
            isPatternPalleteEnabled = systemconf.legend.imagePallete || hasPatternPallete(chartObj);
            
            var strokeWidth = gvp("strokeWidth"),
                strokeOpacity = gvp("strokeOpacity"),
                threeDimension = gvp("threeDimension"),
                fillOpacity = gvp("fillOpacity");
            
            //Animation variables
            var animationOptions = getAnimationConf(commonPlotOptions, plotoptions, chartObj, "pyramid");
            var isAnimationEnable = animationOptions.enabled,
                easingType = animationOptions.easingType,
                transitionDuration = isredraw ? (isAnimationEnable ? 200 : 0) : animationOptions.duration;
            animationType = animationOptions.type;
            
            var eventDisabled = (validateNullVal(getEventConf(plotoptions, commonPlotOptions, chartObj, "enabled"), true) == false);
            var cursor = eventDisabled ? null : getEventConf(plotoptions, commonPlotOptions, chartObj, "cursor") || "pointer";
            
            var container = d3_select(this);
            pyramidGroup = appendEle(container, "g", [data], ".", "class", "pyramidgroup");
            pyramidGroup
                .attr("transform", "translate(" + ((chartWidth / 2) - (cfg.width / 2)) + "," + ((chartHeight / 2) - (cfg.height / 2)) + ")")
                .style("cursor", cursor);
            data_backup = data = data[0].data[0];
            dataLength = data.length;
            heightArray = [];
            height1 = cfg.height;
            b = cfg.width, h = (threeDimension ? (cfg.height / 6) : 0);
            var theta = 25;
            var x = mathSQRT(mathPow((h / mathSin(theta * mathPI / 180)), 2) - mathPow(h, 2));
            b = b - x;
            side1 = d3_scaleLinear()
                .range([(b + x) / 2, 0])
                .domain([0, height1]);
            side2 = d3_scaleLinear()
                .range([(b + x) / 2, b])
                .domain([0, height1]);
            side3 = d3_scaleLinear()
                .range([(b + x) / 2, (b + x)])
                .domain([0, height1]);
            side4 = d3_scaleLinear()
                .range([(b + x) / 2, x])
                .domain([0, height1]);
            hScale = d3_scaleLinear()
                .range([0, h])
                .domain([0, height1 - h / 2]);
            gradientDim = [cfg.width, cfg.height];
            fillColor = chart.fillColorFunc = getFillColor(chartObj, 0, gradientDim);
            var strokeColor = getStrokeColor(chartObj, 0);
            
            var noofNonZeroData = 0;
            chart.total = total = data.reduce(function(prev, cur) {
                return prev + (cur.disabled ? 0 : (noofNonZeroData++, mathAbs(dataset.getY(cur, yaxiscolumnorder) || 0)))
            }, 0);
            
            var previous = 0;
            data.forEach(function(d, i) {
                heightArray.push({
                    "height": ((previous + (d.disabled ? 0 : mathAbs(dataset.getY(d, yaxiscolumnorder)))) / total * (height1 - h / 2)) || 0,
                    "previousHeight": (previous / total * (height1 - h / 2)) || 0
                });
                previous += d.disabled ? 0 : mathAbs(dataset.getY(d, yaxiscolumnorder));
            });
            pyramidsections = appendEle(pyramidGroup, "g", data);
            pyramidsections
                .attr("id", function(d, i) {
                    return "section_" + (dataLength - 1 - i);
                })
                .style({
                    "stroke": function(d, i) {
                        return strokeColor(dataLength - 1 - i, data[dataLength - 1 - i], commonPlotOptions.strokeColor);
                    },
                    "stroke-opacity": strokeOpacity,
                    "stroke-width": strokeWidth,
                    "fill-opacity": fillOpacity,
                    "fill": function(d, i) {
                        return fillColor(dataLength - 1 - i, data[dataLength - 1 - i]);
                    }
                })
                .each(function(d, i) {
                    var currentGroup = d3_select(this);
                    var pyramid = appendEle(currentGroup, "path", [d, d, d, d]);
                    pyramid.attr("class", "section_" + (dataLength - 1 - i)); //For touch events, class will help to get index
                });
            
            if (!eventDisabled) {
                var mousemove = function(d, j) {
                        var ele = this.parentNode;
                        var i = pInt(d3_select(ele).attr("id").replace(/section_/gi, ""));
                        var args = getEventArgs(ele, i);
                        chartObj.eventHandler.mouseMove(args);
                    },
                    mouseout = function(d, j) {
                        var ele = this.parentNode;
                        var i = pInt(d3_select(ele).attr("id").replace(/section_/gi, ""));
                        chartObj.eventHandler.mouseOut(chart, data[dataLength - 1 - i], dataLength - 1 - i);
                    },
                    click = function() {
                        var ele = this.parentNode;
                        var i = pInt(d3_select(ele).attr("id").replace(/section_/gi, ""));
                        var args = getEventArgs(ele, i);
                        chartObj.eventHandler.click(chart, data[i], 0, i, args);
                    },
                    dblclick = function(d, j) {
                        var ele = this.parentNode;
                        var i = pInt(d3_select(ele).attr("id").replace(/section_/gi, ""));
                        var dt = $ZC.annotation ? generateDataForAnnotation(data[i], null) : data[i];
                        var height = heightArray[i].previousHeight;
                        var coordinates = [(side1(height + hScale(height) / 2) + ((chartWidth / 2) - (cfg.width / 2))), ((hScale(height) / 2 + height) + ((chartHeight / 2) - (cfg.height / 2)))];
                        var args = {
                            "coordinate": coordinates,
                            "data": dt,
                            "basedOn": "plot"
                        };
                        chartObj.eventHandler.doubleclick(d, i, args);
                    },
                    eventAttacherOptions = {
                        target: "plot",
                        touchleave: true,
                        chartObj: chartObj
                    };
                var mouseHandlers = {
                    "mousemove": mousemove,
                    "mouseout": mouseout,
                    "click": click,
                    "dblclick": dblclick
                },
                touchHandlers = {
                    "singletouchmove": mousemove,
                    "touchend": mouseout,
                    "tap": click,
                    "dbltap": dblclick
                };
                attachevents(pyramidsections.selectAll("path"), mouseHandlers, touchHandlers, eventAttacherOptions);
            }
            
            var prev = 0;
            pyramidsections
                .each(function(d, i) {
                    var pyramid = pyramidGroup.selectAll("#section_" + i).selectAll("path");
                    if (isredraw) {
                        pyramid
                            .transition()
                            .duration(transitionDuration)
                            .ease("linear")
                            .attr("d", function(d, j) {
                                return getPath(i, j);
                            });
                    } else {
                        var yVal = dataset.getY(d, yaxiscolumnorder);
                        var dur = (animationType == "vertical") ? ((((d.disabled || !yVal) ? 0 : mathAbs(yVal)) / total) * transitionDuration) : transitionDuration
                        prev += (d.disabled || !yVal) ? 0 : mathAbs(yVal);
                        var del = (animationType == "vertical") ? ((((total - prev) / total) * transitionDuration)) : 0;
                        pyramid
                            .transition()
                            .duration(dur)
                            .delay(del)
                            .ease(easingType)
                            .attrTween("d", function(dt, j) {
                                return tween(d, i, j)
                            });
                    }
                });
            chart.renderedTimer = setTimeout(function() {
                if ($ZC.labelRenderer) {
                    var args = [chartObj.seriesdata, 0, data, noofNonZeroData, 0, total, cfg, calculatePointsForLabelPlacement(data)];
                    labelQueue.push({
                        "renderer": chart,
                        "charttype": "funnel",
                        "chartObj": chartObj,
                        "args": args
                    });
                    labelQueue.push({
                        "renderer": chart,
                        "charttype": "funnelInnerLabels",
                        "chartObj": chartObj,
                        "args": args
                    });
                }
                
                //Annotation preprocessing
                if ($ZC.annotation) {
                    notesArray = $ZC.annotation.preprocessAnnotations(chartObj.seriesdata, 0, function(dt, k, j) {
                        var height = heightArray[j].previousHeight;
                        return [(side1(height + hScale(height) / 2) + ((chartWidth / 2) - (cfg.width / 2))), ((hScale(height) / 2 + height) + ((chartHeight / 2) - (cfg.height / 2)))];
                    }, notesArray, chartObj, rp);
                }
                
                isredraw = true;
                chart.chartRendered = true;
            }, (transitionDuration + 10));

            function getEventArgs(ele, i) {
                var ev = getEventObject(d3.event);
                return {
                    "renderer": chart,
                    "seriesPos": 0,
                    "subseriesPos": 0,
                    "itemPos": i,
                    "color": color[i % color.length],
                    "coordinates": [ev.pageX, ev.pageY],
                    "element": ele,
                    "data": data[i]
                };
            }
        });
    }
    
    chart.commonRendererProp = function(_) {
        if (!arguments.length) {
            return commonRendererProp;
        }
        commonRendererProp = _;
        return chart;
    };
    
    chart.defaultMouseOverFunc = function(d, seriesPos, itemPos) {
        if (isPatternPalleteEnabled) {
            pyramidsections.style("opacity", function(d, i) {
                return (dataLength - 1 - i == itemPos) ? null : 0.2;
            });
        } else {
            pyramidsections
                .style("fill", function(d, i) {
                    return fillColor(dataLength - 1 - i, data_backup[dataLength - 1 - i], ((dataLength - 1 - i == itemPos) ? 0.5 : null));
                });
        }
    };
    chart.defaultMouseOutFunc = function(d, i) {
        if (isPatternPalleteEnabled) {
            pyramidsections.style("opacity", null);
        } else {
            pyramidsections
                .style("fill", function(d, i) {
                    return fillColor(dataLength - 1 - i, data_backup[dataLength - 1 - i]);
                });
        }
    };
    
    function constructSide1(height, previous) {
        return "M" + side1(height + hScale(height) / 2) + " " + (hScale(height) / 2 + height) +
            " L" + side2(height + hScale(height) / 2) + " " + (hScale(height) / 2 + height) +
            " L" + side2(previous + hScale(previous) / 2) + " " + (hScale(previous) / 2 + previous) +
            " L" + side1(previous + hScale(previous) / 2) + " " + (hScale(previous) / 2 + previous) +
            " L" + side1(height + hScale(height) / 2) + " " + (hScale(height) / 2 + height);
    }
    
    function constructSide2(height, previous) {
        return "M" + side2(height + hScale(height) / 2) + " " + (hScale(height) / 2 + height) +
            " L" + side3(height + hScale(height) / 2) + " " + (-hScale(height) / 2 + height) +
            " L" + side3(previous + hScale(previous) / 2) + " " + (-hScale(previous) / 2 + previous) +
            " L" + side2(previous + hScale(previous) / 2) + " " + (hScale(previous) / 2 + previous) +
            " L" + side2(height + hScale(height) / 2) + " " + (hScale(height) / 2 + height);
    }
    
    function constructSide3(height, previous) {
        return "M" + side3(height + hScale(height) / 2) + " " + (-hScale(height) / 2 + height) +
            " L" + side4(height + hScale(height) / 2) + " " + (-hScale(height) / 2 + height) +
            " L" + side4(previous + hScale(previous) / 2) + " " + (-hScale(previous) / 2 + previous) +
            " L" + side3(previous + hScale(previous) / 2) + " " + (-hScale(previous) / 2 + previous) +
            " L" + side3(height + hScale(height) / 2) + " " + (-hScale(height) / 2 + height);
    }
    
    function constructSide4(height, previous) {
        return "M" + side4(height + hScale(height) / 2) + " " + (-hScale(height) / 2 + height) +
            " L" + side1(height + hScale(height) / 2) + " " + (hScale(height) / 2 + height) +
            " L" + side1(previous + hScale(previous) / 2) + " " + (hScale(previous) / 2 + previous) +
            " L" + side4(previous + hScale(previous) / 2) + " " + (-hScale(previous) / 2 + previous) +
            " L" + side4(height + hScale(height) / 2) + " " + (-hScale(height) / 2 + height);
    }
    
    function getPath(i, j, base) {
        var height = heightArray[i].height;
        var baseHgt = (base == "down" ? height : heightArray[i].previousHeight);
        switch (j) {
            case 0:
                return constructSide3(height, baseHgt);
            case 1:
                return constructSide4(height, baseHgt);
            case 2:
                return constructSide1(height, baseHgt);
            case 3:
                return constructSide2(height, baseHgt);
        }
    }
    
    function tween(d, i, j) {
        var interpolate = d3_interpolate(getPath(((animationType == "vertical") ? i : (dataLength - 1)), j, "down"), getPath(i, j));
        return function(t) {
            return interpolate(t);
        };
    }
    
    function calculatePointsForLabelPlacement(data) {
        var trape = [];
        data.forEach(function(d, i) {
            var idx = data.length - 1 - i;
            var height = heightArray[idx].height,
                previous = heightArray[idx].previousHeight;
            var midHeight = (height + previous) / 2;
            var bottom_right = {
                    x: side3(height + hScale(height) / 2),
                    y: (-hScale(height) / 2 + height)
                },
                top_right = {
                    x: side3(previous + hScale(previous) / 2),
                    y: (-hScale(previous) / 2 + previous)
                },
                bottom_left = {
                    x: side1(height + hScale(height) / 2),
                    y: (hScale(height) / 2 + height)
                },
                top_left = {
                    x: side1(previous + hScale(previous) / 2),
                    y: (hScale(previous) / 2 + previous)
                };
            var rightX = (bottom_right.x + top_right.x) / 2,
                rightY = (bottom_right.y + top_right.y) / 2,
                leftX = (bottom_left.x + top_left.x) / 2,
                leftY = (bottom_left.y + top_left.y) / 2,
                midX = (side1(midHeight + hScale(midHeight) / 2) + side2(midHeight + hScale(midHeight) / 2)) / 2;
            
            trape.push({
                "rightx": rightX + datalabelPadding,
                "leftx": leftX - datalabelPadding,
                "y": rightY,
                "y0": height + hScale(height) / 2,
                "midX": midX,
                "leftY": leftY
            });
        });
        return trape;
    }
    
    return chart;
};