//$Id$
/**
 *
 * @returns {chart} scatter, bubble renderer.
 */
svg_plot_renderer.scatter = function() {
    var chartObj, xscale, yscale, index, yaxiscolumnorder, color, stacked, dataset, axisrotated,
        labelQueue, commonPlotOptions, systemconf, hasNegValue, redraw = false,
        transitionDuration;
    var shiftInfo, commonRendererProp,
        yrangePos,
        dataProcessorHelpers = $ZC.DataProcessor.helpers;
    function chart(selection) {
        selection.each(function(data) {
            
            //renderer config values
            var rp = commonRendererProp.params;
            chartObj = rp.chartObj,
                xscale = rp.xscale,
                yscale = rp.yscale,
                stacked = rp.stacked,
                index = rp.datasetIndex,
                axisrotated = rp.axisrotated,
                commonPlotOptions = rp.commonPlotOptions,
                color = rp.color,
                labelQueue = rp.labelQueue,
                hasNegValue = rp.hasNegValue,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset,
                systemconf = chartObj.systemConf;
            
            var defsLocation = chartObj.defsLocation;
            
            var xReversed = systemconf.chart.axes.xaxis.reversed,
                yReversed = systemconf.chart.axes.yaxis[yaxiscolumnorder].reversed;
            
            var container = d3_select(this);
            chart.update = function() {
                container.transition().call(chart);
            };
            var currentdata = data[index];
            var plotoptions = currentdata.plotoptions;
            
            var chartType = chartObj.chartTypes[index].value;
            var chartName = chartObj.chartTypes[index].name;

            var isBubbleChart = (chartType == $ZC.charttype.bubble),
                scatterOrBubble = (isScatterChartType(chartType)),
                isPolar = chartObj.dataObject.isPolarAxisCategory;
            
            var currentseriesdata = currentdata.data;
            var series = appendEle(container, "g", [currentdata], ".", "class", ('scatterseries_' + index));
            if (chartName === "arearange") {
                series = appendEle(series, "g", [currentdata], ".", "class", ('rangePos_' + yrangePos));
            }
            
            //Animation variables
            var animationOptions = getAnimationConf(commonPlotOptions, plotoptions, chartObj, chartName);
            var isAnimationEnable = animationOptions.enabled,
                easingType = animationOptions.easingType,
                animationType = animationOptions.type;
            transitionDuration = animationOptions.duration;
            
            var markerOptions = (plotoptions && plotoptions.marker) ?
                mergeJSON(commonPlotOptions.marker, plotoptions.marker, true) :
                commonPlotOptions.marker;
            var symboltype = markerOptions.symbol,
                symbolsize = markerOptions.size;
            
            if (!scatterOrBubble) {
                symbolsize = ((symbolsize == null || symbolsize == "") ?
                        (pInt((plotoptions != null) ? $ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions)("strokeWidth") : commonPlotOptions.strokeWidth) || 2) :
                        symbolsize) * (isPolar ? 0.56 : 0.7);
                animationType = (animationType == "vertPathTween" || animationType == "vertPathTweenAll") ? "verticalAll" : animationType;
                animationType = (animationType == "horiPathTween") ? "horizontalAll" : animationType;
                color = currentdata.color || color;
            } else {
                symbolsize = validateNullVal(symbolsize, 1);
                animationType = (animationType == "fadeAll") ? "fade" : animationType;
            }
            
            var symbolpath = getSVGsymbol(symboltype, symbolsize),
                multiColoring = scatterOrBubble ? ($ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions)("multiColoring") && !chartObj.legendarea.show) : false,
                outersize = symbolsize * 10,
                initial_outerFillOpacity = (animationType == "fade") ? 0 : markerOptions.outerFillOpacity,
                initial_outerStrokeOpacity = (animationType == "fade") ? 0 : markerOptions.outerStrokeOpacity,
                initial_innerFillOpacity = (animationType == "fade") ? 0 : markerOptions.innerFillOpacity,
                initial_innerStrokeOpacity = (animationType == "fade") ? 0 : markerOptions.innerStrokeOpacity,
                hasImage = (markerOptions.imageUrl != null && markerOptions.imageUrl !== ""),
                imageConf = {
                    "name": markerOptions.imageUrl
                },
                outerDashArray = getDashArray(markerOptions.outerDashStyle, markerOptions.outerStrokeWidth),
                innerDashArray = getDashArray(markerOptions.innerDashStyle, markerOptions.innerStrokeWidth),
                outerStrokeLineCap = getLineCap(markerOptions.outerLineCap, markerOptions.outerDashStyle),
                innerStrokeLineCap = getLineCap(markerOptions.innerLineCap, markerOptions.innerDashStyle);

            var fillColor = chart.fillColorFunc = getFillColor(chartObj, index, null, (scatterOrBubble ? null : true)),
                strokeColor = getStrokeColor(chartObj, index);
            
            chart.seriesdataAfterUpdate = [];
            var datatypeX = chartObj.dataset.getDataType("x"),
                datatypeY = chartObj.dataset.getDataType("y", yaxiscolumnorder),

                isCatNullX = dataProcessorHelpers.isCatNull(dataset, "x"),
                isCatNullY = dataProcessorHelpers.isCatNull(dataset, "y", yaxiscolumnorder),

                xCategories = (datatypeX == ORDINAL_DATATYPE) ? xscale.domain() : null,
                yCategories = (datatypeY == ORDINAL_DATATYPE) ? yscale.domain() : null,

                categories = (datatypeX == ORDINAL_DATATYPE && xscale.domain != null && !defined(chartObj.dataObject.hierarchical)) ? xCategories : chartObj.dataObject.getCategories("x");
            
            currentseriesdata.forEach(function(d, i) {
                
                var dataUpdate;
                if (datatypeX == ORDINAL_DATATYPE || datatypeY == ORDINAL_DATATYPE) {
                    dataUpdate = chartObj.dataObject.updateSeriesdatabyCategories(d3_values(d), xCategories, yCategories, yaxiscolumnorder);
                } else {
                    dataUpdate = d;
                }

                if (chartObj.brush && chartObj.brush.extent) {
                    var domain = chartObj.axes.x.scale.domain();
                    var ticks = chartObj.axes.x.ticks.tickValueObj.ticks;
                    var startVal = (datatypeX == TIME_DATATYPE) ? domain[0].getTime() : (datatypeX == ORDINAL_DATATYPE) ? categories.indexOf(domain[0]) : domain[0];
                    var endVal = (datatypeX == TIME_DATATYPE) ? domain[1].getTime() : (datatypeX == ORDINAL_DATATYPE) ? categories.indexOf(domain[domain.length - 1]) : domain[1];
                    var accessor = (datatypeX == ORDINAL_DATATYPE) ? function(d) {
                            return categories.indexOf(chartObj.dataset.getX(d))
                        } : function(d) {
                            return chartObj.dataset.getX(d)
                        }
                    dataUpdate = dataProcessorHelpers.filterDataToViewPort(dataUpdate, startVal, endVal, accessor);
                }
                
                chart.seriesdataAfterUpdate[i] = dataUpdate;
                var ishideable = currentdata.disabled;
                var serData, baseData, basedataShift;
                
                if (animationType == "pointbypoint" || animationType == "pointTween") {
                    serData = dataUpdate;
                    if (xReversed) {
                        serData = serData.slice(0).reverse();
                    }
                    baseData = serData;
                    if (animationType == "pointTween") {
                        baseData = serData.map(function(d, i) {
                            return Object.create(serData[0])
                        });
                    } else if (animationType == "pointbypoint") {
                        var baseIdx = (isPolar && yReversed && !stacked) ? yscale.domain().length - 1 : 0;
                        baseData = serData.map(function(d, i) {
                            var s = Object.create(d);
                            var baseY = (!isPolar && (hasNegValue != null) && (yscale(hasNegValue) != null)) ? mathMax(hasNegValue, yscale.domain()[0]) : yscale.domain()[baseIdx];
                            if (stacked) {
                                s.y0 = 0;
                                s.y = baseY;
                            }
                            s[dataset.yidx(yaxiscolumnorder)] = baseY;
                            return s;
                        });
                    }
                    basedataShift = d3_values(baseData);
                }
                
                var subseries = appendEle(series, "g", function(d) {
                    return ishideable ? [] : [dataUpdate];
                }, ".", "class", ('subseries_' + i));
                var scatter = subseries.selectAll('g.scatter').data(function(d) {
                    return d;
                });
                var scatterEnter = scatter.enter().append('g').attr("class", "scatter");
                var noScatters = scatter.size();
                var duration = (animationType != "onebyone") ? transitionDuration : transitionDuration / noScatters,
                    pathData = function(d, i) {
                        return (chartType !== $ZC.charttype.stkdarea) && (!(d) || d.isDummy || isCatNullX(dataset.getX(d)) || isCatNullY(dataset.getY(d, yaxiscolumnorder))) ?
                            [] :
                            [{
                                color: (multiColoring ? chartObj.seriesColor[i % chartObj.seriesColor.length] : color),
                                index: multiColoring ? i : index,
                                data: d
                            }];
                    };
                
                var outermarkerpath = appendEle(scatterEnter, "path", pathData, ".", "class", "outer");
                outermarkerpath
                    .style({
                        "fill-opacity": initial_outerFillOpacity,
                        "stroke-opacity": initial_outerStrokeOpacity
                    });
                
                var innermarkerpath = appendEle(scatterEnter, "path", (hasImage ? [] : pathData), ".", "class", "inner");
                innermarkerpath
                    .style({
                        "fill-opacity": initial_innerFillOpacity,
                        "stroke-opacity": initial_innerStrokeOpacity
                    });
                
                if (isAnimationEnable) {
                    if ((animationType == "pointbypoint" || animationType == "pointTween")) {
                        interTween(baseData, basedataShift, serData, categories, animationType, easingType, transitionDuration, rp, function(d) {
                            return {
                                "transform": function(dt, i) {
                                    return "translate(" + getInitialX(d[i]) + "," + getInitialY(d[i]) + ")";
                                }
                            }
                        }, scatterEnter, "scatter");
                    } else {
                        scatterEnter
                            .attr("transform", function(d, i) {
                                return "translate(" + getInitialX(d) + "," + getInitialY(d) + ")" + ((hasImage && animationType === "sizing") ? " scale(0)" : "");
                            })
                            .transition()
                            .delay(function(d, i) {
                                return (animationType == "onebyone") ? (i * duration) : 0;
                            })
                            .duration(duration)
                            .ease(easingType)
                            .attr("transform", function(d) {
                                return "translate(" + chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos) + "," + chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos) + ")";
                            });
                    }
                }
                var scatterUpdate = scatter;
                // if(scatterEnter.node() == null || !isAnimationEnable){ //doubt : Why scatterEnter.node() == null?
                scatterUpdate
                    .transition()
                    .delay(function(d, i) {
                        return ((animationType == "pointbypoint" || animationType == "pointTween") && scatterEnter.node() != null) ?
                            transitionDuration :
                            ((animationType == "onebyone") ? (i * duration) : 0);
                    })
                    .duration(duration)
                    .ease(easingType)
                    .attr("transform", function(d) {
                        return "translate(" + chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos) + "," + chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos) + ")";
                    });
                // }
                
                var outermarkerpathUpdate = scatterUpdate.selectAll("path.outer")
                    .attr("d", function(d, i) {
                        var previousSize = d3_select(this.parentNode).attr("size");
                        var s = (previousSize != null) ? previousSize : ((animationType == "sizing" && !hasImage) ? 0 : getSymbolSize(d.data));
                        return symbolpath(s);
                    })
                    .style({
                        "stroke-width": markerOptions.outerStrokeWidth,
                        "fill": function(d, i) {
                            var size = getSymbolSize(d.data);
                            if (hasImage) {
                                imageConf.width = imageConf.height = parseFloat(size);
                                imageConf.x = imageConf.y = -parseFloat(size) / 2;
                                return "url(" + defsLocation + "#" + effects.get(svg_patterns,"imageFill", imageConf) + ")";
                            }

                            var radius = size / 2;
                            return fillColor(d.index, d.data, null, markerOptions.outerFillColor,null, (scatterOrBubble ? [radius, radius] : null));
                        },
                        "stroke": function(d, i) {
                            return strokeColor(d.index, d.data, markerOptions.outerStrokeColor);
                        },
                        "stroke-dasharray": outerDashArray,
                        "stroke-linecap": outerStrokeLineCap
                    });
                
                var innermarkerpathUpdate = scatterUpdate.selectAll("path.inner")
                    .attr("d", function(d, i) {
                        var previousSize = d3_select(this.parentNode).attr("size");
                        var s = (previousSize != null) ? previousSize : ((animationType == "sizing" && !hasImage) ? 0 : getSymbolSize(d.data));
                        return symbolpath(s / 2);
                    })
                    .style({
                        "stroke-width": markerOptions.innerStrokeWidth,
                        "fill": function(d, i) {
                            var radius = getSymbolSize(d.data) / 4;
                            return fillColor(d.index, d.data, null, markerOptions.innerFillColor,null, (scatterOrBubble ? [radius, radius] : null));
                        },
                        "stroke": function(d, i) {
                            return strokeColor(d.index, d.data, markerOptions.innerStrokeColor);;
                        },
                        "stroke-dasharray": innerDashArray,
                        "stroke-linecap": innerStrokeLineCap
                    });
                
                outermarkerpathUpdate.transition().duration(duration).ease(easingType)
                    .attr("d", function(d) {
                        return symbolpath(getSymbolSize(d.data));
                    })
                    .style({
                        "fill-opacity": markerOptions.outerFillOpacity,
                        "stroke-opacity": markerOptions.outerStrokeOpacity
                    });
                innermarkerpathUpdate.transition().duration(duration).ease(easingType)
                    .attr("d", function(d) {
                        return symbolpath(getSymbolSize(d.data) / 2);
                    })
                    .style({
                        "fill-opacity": markerOptions.innerFillOpacity,
                        "stroke-opacity": markerOptions.innerStrokeOpacity
                    });
                
                var blinkConf = ((plotoptions && plotoptions.animation && plotoptions.animation.blink) ?
                            mergeJSON(commonPlotOptions.animation.blink, plotoptions.animation.blink, true) :
                            commonPlotOptions.animation.blink) || {},
                    blinkEnabled = blinkConf.enabled;
                if (blinkEnabled) {
                    blinkSVGsymbols(scatter.selectAll(".outer,.inner"), blinkConf);
                }
                
                scatter.attr("size", function(d) {
                    return getSymbolSize(d);
                }); //Saving the size, makes redraw animation cool
                scatter.exit().remove();
                
                function getInitialX(d) {
                    switch (animationType) {
                        case "verticalAll":
                            var baseX = ((hasNegValue != null) && (yscale(hasNegValue) != null)) ? yscale(mathMax(hasNegValue, yscale.domain()[0])) : yscale.range()[0];
                            return axisrotated ? baseX : chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos);
                        case "horizontalAll":
                            var baseX = (((hasNegValue != null) && (xscale(hasNegValue) != null)) ? xscale(mathMax(hasNegValue, xscale.domain()[0])) : xscale.range()[0]) + ((xscale.rangeBand != null) ? xscale.rangeBand() / 2 : 0);
                            return axisrotated ? chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos) : baseX;
                        case "fade":
                        case "sizing":
                        case "pointTween":
                        case "pointbypoint":
                        case "strokeTween":
                            return chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "x", null, yrangePos);
                        default:
                            var range = (axisrotated ? yscale : xscale).range();
                            var pos = ((axisrotated && yReversed) || (!axisrotated && xReversed)) ? range.length - 1 : 0;
                            return range[pos];
                    }
                }
                
                function getInitialY(d) {
                    switch (animationType) {
                        case "verticalAll":
                            var baseY = ((hasNegValue != null) && (yscale(hasNegValue) != null)) ? yscale(mathMax(hasNegValue, yscale.domain()[0])) : yscale.range()[0];
                            return axisrotated ? chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos) : baseY;
                        case "horizontalAll":
                            var baseX = (((hasNegValue != null) && (xscale(hasNegValue) != null)) ? xscale(mathMax(hasNegValue, xscale.domain()[0])) : xscale.range()[0]) + ((xscale.rangeBand != null) ? xscale.rangeBand() / 2 : 0);
                            return axisrotated ? baseX : chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos);
                        case "fade":
                        case "sizing":
                        case "pointTween":
                        case "pointbypoint":
                        case "strokeTween":
                            return chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "y", null, yrangePos);
                        default:
                            var range = (axisrotated ? xscale : yscale).range();
                            var pos = ((axisrotated && xReversed) || (!axisrotated && yReversed)) ? range.length - 1 : 0;
                            return range[pos];
                    }
                }
                
                function getSymbolSize(d) {
                    return chartType !== $ZC.charttype.stkdarea && (isCatNullX(dataset.getX(d)) || isCatNullY(dataset.getY(d, yaxiscolumnorder))) ?
                        0 :
                        isBubbleChart ? chartObj.dataObject.getXYvalue(axisrotated, stacked, xscale, yscale, yaxiscolumnorder, d, "z", chartObj.zscale, yrangePos) : outersize;
                }

                //Assign renderer functions
                chart.getSymbolSize = getSymbolSize;
            });
            if ($ZC.labelRenderer != null) {
                var markerSize;
                if (scatterOrBubble || isBubbleChart) {
                    markerSize = outersize + (markerOptions.outerStrokeWidth);
                } else {
                    var strokeWidth = validateNullVal($ZC.utils.getValueOfPlotOptions(plotoptions, commonPlotOptions)("strokeWidth"), 2);
                    markerSize = ((markerOptions.size || strokeWidth) * outersize * 0.5) + strokeWidth / 2;
                }
                labelQueue.push({
                    "renderer": chart,
                    "charttype": "scatter",
                    "chartObj": chartObj,
                    "args": [currentdata, index, markerSize, yrangePos]
                });
            }
            
            chart.renderedTimer = setTimeout(function() {
                redraw = true;
                chart.chartRendered = true;
            }, (transitionDuration));
        });
        return chart;
    }

    chart.commonRendererProp = function(_) {
        if (!arguments.length) {
            return commonRendererProp;
        }
        commonRendererProp = _;
        return chart;
    };
    chart.shiftInfo = function(_) {
        if (!arguments.length) {
            return shiftInfo;
        }
        shiftInfo = _;
        return chart;
    };

    chart.yrangePos = function(_) {
        if (!arguments.length) {
            return yrangePos;
        }
        yrangePos = _;
        return chart;
    };
    return chart;
}