//$Id$
var ZC_brushComponent_getSliderOptions;
(function(){
    brushComponent.prototype.drawSlider = function(){
        var chartObj = this.chartObj,
            options = this.options,
            container = this.context,

            orient = options.orient,
            isHorizontal = orient === "top" || orient === "bottom",
            dimension = options.dimension,

            brushOptions = ZC_brushComponent_getSliderOptions(dimension, options.brushOptions),
            type = brushOptions.type,
            sliderOptions = brushOptions[type],

            rotate = isHorizontal? "" : " rotate(-90)",
            filterID = brushOptions.shadow;

        if (filterID) {
            filterID = getFilterID(filterID, chartObj);
        }

        var element = container.selectAll(".resize");
        element.selectAll("*").remove();

        switch(type) {
            case "lever":
                var r = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, sliderOptions.radius);
                element.append("path")
                    .attr({
                        "d": roundedRect(-sliderOptions.width / 2, -sliderOptions.height / 2, sliderOptions.width, sliderOptions.height, r),
                        "filter": filterID
                    })
                    .style({
                        "fill": sliderOptions.fillColor,
                        "stroke": sliderOptions.strokeColor
                    });

                element.append("path")
                    .attr({
                        "d": "M" + (-sliderOptions.stripePadding / 2) + " " + (-sliderOptions.stripeHeight / 2) + " L" + (-sliderOptions.stripePadding / 2) + " " + (sliderOptions.stripeHeight / 2) + " M" + (sliderOptions.stripePadding / 2) + " " + (-sliderOptions.stripeHeight / 2) + " L" + (sliderOptions.stripePadding / 2) + " " + (sliderOptions.stripeHeight / 2)
                    })
                    .style({
                        "stroke": sliderOptions.stripeColor
                    });
                break;
            case "triangleUp":
                element.append("path")
                    .attr({
                        "d": getSVGsymbol("triangle-down")(sliderOptions.size),
                        "filter": filterID
                    })
                    .style("fill", sliderOptions.fillColor);

                //transparent path to handle resize on the color band
                var r = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, 0);
                element.append("path")
                    .attr("d", roundedRect(-sliderOptions.size / 2, sliderOptions.size / 2, sliderOptions.size, dimension[isHorizontal ? 1 : 0], r))
                    .style("fill", "transparent");
                break;
            case "triangleDown":
                element.append("path")
                    .attr({
                        "d": getSVGsymbol("triangle-up")(sliderOptions.size),
                        "filter": filterID
                    })
                    .style("fill", sliderOptions.fillColor);

                //transparent path to handle resize on the color band
                var r = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, 0);
                element.append("path")
                    .attr("d", roundedRect(-sliderOptions.size / 2, -sliderOptions.size / 2 - dimension[isHorizontal ? 1 : 0], sliderOptions.size, dimension[isHorizontal ? 1 : 0], r))
                    .style("fill", "transparent");
                break;
            case "hollowCircle":
                element.append("path")
                    .attr({
                        "d": getSVGsymbol()(sliderOptions.size),
                        "filter": filterID
                    })
                    .style({
                        "fill": sliderOptions.outerFillColor,
                        "stroke": sliderOptions.outerStrokeColor
                    });
                element.append("path")
                    .attr({
                        "d": getSVGsymbol()(sliderOptions.size * 0.4)
                    })
                    .style({
                        "fill": sliderOptions.innerFillColor,
                        "stroke": sliderOptions.outerStrokeColor
                    });
                break;
            case "circle":
                element.append("path")
                    .attr({
                        "d": getSVGsymbol()(sliderOptions.size),
                        "filter": filterID
                    })
                    .style({
                        "fill": sliderOptions.outerFillColor,
                        "stroke": sliderOptions.outerStrokeColor,
                        "stroke-width": sliderOptions.outerStrokeWidth
                    });
                element.append("path")
                    .attr({
                        "d": getSVGsymbol()(sliderOptions.size * 0.3)
                    })
                    .style({
                        "fill": sliderOptions.innerFillColor,
                        "stroke": sliderOptions.outerStrokeColor
                    });
                break;
            case "balance":
                // var r = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, sliderOptions.radius),
                //     k1 = container.selectAll(".n"),
                //     k2 = container.selectAll(".s");

                // k1.append("path")
                //     .attr({
                //         "d": roundedRect(-sliderOptions.width / 2, -sliderOptions.height / 2, sliderOptions.width, sliderOptions.height, r),
                //         "filter": filterID
                //     })
                //     .style({
                //         "fill": sliderOptions.fillColor,
                //         "stroke": sliderOptions.strokeColor
                //     });

                // k2.append("path")
                //     .attr({
                //         "d": roundedRect(-sliderOptions.width / 2, -sliderOptions.height / 2, sliderOptions.width, sliderOptions.height, r),
                //         "filter": filterID
                //     })
                //     .style({
                //         "fill": sliderOptions.fillColor,
                //         "stroke": sliderOptions.strokeColor
                //     });
                break;
        }

        var position = getSliderPosition(type, dimension, isHorizontal, sliderOptions);
        element.selectAll("path")
            .attr("transform", "translate(" + position[0] + "," + position[1] + ")" + rotate);
    }

    ZC_brushComponent_getSliderOptions = function(dimension, userOptions) {
        var defaultSliderOptions = {
            "type": "lever",
            "shadow": "0 0 2px rgba(0,0,0,0.2)",
            "lever": {
                height: 26,
                width: 9,
                radius: 2,
                stripeHeight: 8,
                stripePadding: 2,
                fillColor: "white",
                strokeColor: "#DADADA",
                stripeColor: "#DADADA"
            },
            "triangleUp": {
                size: 15,
                fillColor: "#4C4C4C"
            },
            "triangleDown": {
                size: 15,
                fillColor: "#4C4C4C"
            },
            "hollowCircle": {
                size: 25,
                outerFillColor: "white",
                outerStrokeColor: "#DADADA",
                innerStrokeColor: "#DADADA"
            },
            "circle": {
                size: 25,
                outerStrokeColor: "white",
                outerStrokeWidth: 2,
                innerFillColor: "white"
            },
            "balance": {
                height: 26,
                width: 9,
                radius: 2,
                stripeHeight: 8,
                fillColor: "white",
                strokeColor: "#DADADA",
                stripeColor: "#DADADA"
            }
        };
        var sliderOptions = userOptions.slider || {};
        sliderOptions = mergeJSON(defaultSliderOptions, sliderOptions, true);
        return sliderOptions;
    }

    function getFilterID(filterID, chartObj) {
        var parsedShadowOptions = filterID.split(" "),
            shadowOptions = {
                dx: parseFloat(parsedShadowOptions[0]),
                dy: parseFloat(parsedShadowOptions[1]),
                color: parsedShadowOptions[3],
                blur: parseFloat(parsedShadowOptions[2]),
                x: "-50%",
                y: "-50%",
                width: "200%",
                height: "200%"
            };
        filterID = effects.get(svg_filters, "dropShadow", shadowOptions);
        filterID = "url(" + chartObj.defsLocation + "#" + filterID + ")";
        return filterID;
    }

    function getSliderPosition(type, dimension, isHorizontal, sliderOptions) {
        var width = dimension[0],
            height = dimension[1],
            x, y;
        switch(type) {
            case "lever":
            case "balance":
                if (isHorizontal) {
                    x = 0;
                    y = height / 2;
                } else {
                    x = width / 2;
                    y = 0;
                }
                break;
            case "triangleUp":
                if (isHorizontal) {
                    x = 0;
                    y = -sliderOptions.size / 2;
                } else {
                    x = -sliderOptions.size / 2;
                    y = 0;
                }
                break;
            case "triangleDown":
                if (isHorizontal) {
                    x = 0;
                    y = height + sliderOptions.size / 2;
                } else {
                    x = width + sliderOptions.size / 2;
                    y = 0;
                }
                break;
            case "hollowCircle":
                if (isHorizontal) {
                    x = 0;
                    y = height / 2;
                } else {
                    x = width / 2;
                    y = 0;
                }
                break;
            case "circle":
                if (isHorizontal) {
                    x = 0;
                    y = height / 2;
                } else {
                    x = width / 2;
                    y = 0;
                }
                break;
        }
        return [x, y];
    }

})();