/* SCROLL in MOBILE need to implement
 SCROLL for type y need to implement
 */
var svg_brush = function(chartObj, type, axisorder) {
    
    var scrollData = chartObj.systemConf.chart.scroll;
    var datatype, axisRotated, orient,
        isScrollNeed;
    var defaultOptions = {
        slider: {
            fillColor: "#B3B6B7",
            cursor: "pointer"
        },
        tracker: {
            fillColor: "#ECF0F1",
            cursor: "pointer"
        },
        resizer: {
            fillColor: "none"
        },
        scrollHeight: 10
    };
    
    var defaultScrollLength = 20,
        defaultItemSize = 15,
        overviewContext,
        isOverviewEnabled = (scrollData.overview || {}).enabled,
        isResizeInitiated,

        resizeStartedPosition;
        chartObj.isBrushing = null;
    
    function brush(selection) {
        axisRotated = chartObj.dataObject.getAxisRotated();
        orient = (axisRotated) ? "left" : "bottom";
        var scale = chartObj.axes.x.scrollScale = $ZC.scale.getScale(chartObj, chartObj.dataObject, chartObj.plotarea, orient, 0, chartObj.dataset, type, null, 0)
        
        var categoryPosInScreen = getCategoryPosScreen(chartObj, chartObj.plotarea);
        var preScrollBarLength = getScrollBarLength(scale, categoryPosInScreen);
        isScrollNeed = Math.abs(preScrollBarLength - chartObj.plotarea[axisRotated ? "height" : "width"]) >= 1;

        if (!isScrollNeed) {
            return;
        }

        brush.component = d3_brush();
        brush.component
            .on("brush", function() {
                chartObj.isBrushing = true;
                brush.scrollNrender();
            })
            .on("brushstart", function(event) {
                chartObj.isBrushing = true;
                isResizeInitiated = false;
            })
            .on("brushend", function() {
                chartObj.isBrushing = false;
                // brush.updateExtentPosition();
            })
        if (!brush.context) {
            brush.context = appendEle(chartObj.svg, "g", [1], ".", "class", "brush");
        }

        if (axisRotated) {
            chartObj.plotarea.width -= scrollData.height || defaultOptions.scrollHeight;
            brush.context.attr("transform", "translate(" + (chartObj.plotarea.left + chartObj.plotarea.width) + "," + (chartObj.plotarea.top) + ")");
            
        } else {
            chartObj.plotarea.bottom += scrollData.height || defaultOptions.scrollHeight;
            chartObj.plotarea.height -= scrollData.height || defaultOptions.scrollHeight;
            brush.context.attr("transform", "translate(" + chartObj.plotarea.left + "," + (chartObj.plotarea.top + chartObj.plotarea.height) + ")");
        }
        
    }
    
    brush.initialize = function(plotarea) {
        if (!isScrollNeed) {
            return;
        }

        //scroll scale
        datatype = chartObj.dataset.getDataType(type);
        orient = (axisRotated) ? "left" : "bottom";
        setScrollScale(chartObj, plotarea, type);
        brush.categoryPosInScreen = getCategoryPosScreen(chartObj, plotarea);
        //set Initialize Extent
        
        brush.extent = []; //need array initialize optimize
        brush.context.call(brush.component);
        
        var initialExtent = (axisRotated) ?
            [{
                "y": plotarea.height,
                "x": 0
            }, {
                "y": brush.categoryPosInScreen,
                "x": 0
            }] :
            [{
                "x": 0,
                "y": 0
            }, {
                "x": brush.categoryPosInScreen,
                "y": 0
            }];
        brush.setExtent(initialExtent);
        
        // Renders the wholeview
        if (isOverviewEnabled) {
            renderOverviewContext();
        }
        
        //Navigator style
        styleBrushNavigator();
    };
    
    brush.scrollNrender = function() {
        brush.updateExtentPosition();
        chartObj.updateScale(chartObj.seriesdata[0], type, brush.extent);
        chartObj.drawThreshold(true);
        chartObj.drawPlotareaClippath();
        chartObj.plot.renderer.draw(true);
        chartObj.eventHandler.updateParams(chartObj);
    };
    brush.updateExtentPosition = function() {
        var extent = brush.component.extent();
        var mousePos = (datatype == ORDINAL_DATATYPE) ?
            (axisRotated) ? [extent[1], extent[0]] : [extent[0], extent[1]] //need to change x,y
            : [chartObj.axes.x.scrollScale(extent[0]), chartObj.axes.x.scrollScale(extent[1])];
        
        var categoryPosInScreen = brush.categoryPosInScreen,
            clicked = (mousePos[0] === mousePos[1]);
        isResizeInitiated = isResizeInitiated || clicked;
        var scrollBarLength  = brush.scrollBarLength || getScrollBarLength(chartObj.axes.x.scrollScale,categoryPosInScreen);
        if (isResizeInitiated) {
            if (!clicked) {
                var scrollBarOffset = scrollBarLength * (axisRotated ? -3/2 : 1/2);
                mousePos[0] += mousePos[1] - resizeStartedPosition[1] + scrollBarOffset;
                mousePos[1] = mousePos[0] + scrollBarLength;
            }

            mousePos[0] -= (scrollBarLength / 2);
            mousePos[1] -= (scrollBarLength / 2);

            // user clicked on the brush background, width will be zero, offset x by half width
            if (axisRotated) {
                mousePos[1] = (mousePos[1] < 0) ? 0 : mousePos[1];
                mousePos[0] = mousePos[1] + scrollBarLength;
                
                mousePos[0] = (mousePos[0] > chartObj.plotarea.height) ? chartObj.plotarea.height : mousePos[0];
                mousePos[1] = mousePos[0] - scrollBarLength;
            } else {
                mousePos[0] = (mousePos[1] > chartObj.plotarea.width) ? mousePos[0] - (mousePos[1] - chartObj.plotarea.width) : ((mousePos[0] < 0) ? 0 : mousePos[0]);
                mousePos[1] = mousePos[0] + scrollBarLength;
                
                mousePos[0] = ((mousePos[0] + scrollBarLength) > chartObj.plotarea.width) ? mousePos[0] - (mousePos[1] - chartObj.plotarea.width) : mousePos[0];
                mousePos[1] = mousePos[0] + scrollBarLength;
            }

            if (clicked) {
                resizeStartedPosition = mousePos;
            }
        }
        
        var newExtent = (axisRotated) ?
            [{
                "y": mousePos[0],
                "x": 0
            }, {
                "y": mousePos[1],
                "x": 0
            }] :
            [{
                "x": mousePos[0],
                "y": 0
            }, {
                "x": mousePos[1],
                "y": 0
            }];
        brush.setExtent(newExtent); //need to change x,y
        brush.scrollBarLength = mathAbs(mousePos[1] - mousePos[0]); // Save the resized scrollbar length
        
        // If the brush is clicked, redraw the brush
        if (isResizeInitiated) {
            brush.context.call(brush.component);
            removeResizer();
        }
    };
    
    brush.setExtent = function(mousePos) {
        var customExtent = getCustomExtent(mousePos);
        brush.extent = customExtent;
        
        var curMousePos = (axisRotated) ? [mousePos[0].y, mousePos[1].y] : [mousePos[0].x, mousePos[1].x]
        if (datatype == ORDINAL_DATATYPE) {
            brush.component.extent(curMousePos); //need to change x,y
            
        } else {
            brush.component.extent(customExtent);
        }
    };
    
    return brush;
    
    function getCustomExtent(mousePos) {
        var customExtent;
        var x1 = $ZC.scale.invert(datatype, mousePos[0], chartObj.axes.x.scrollScale, axisRotated, type);
        var x2 = $ZC.scale.invert(datatype, mousePos[1], chartObj.axes.x.scrollScale, axisRotated, type);
        if (datatype == ORDINAL_DATATYPE) {
            customExtent = chartObj.axes.x.scrollScale.domain().slice(x1, x2 + 1, 0);
        } else {
            customExtent = [x1, x2];
        }
        return customExtent;
    }
    
    function styleBrushNavigator() {
        // Translate the brush context
        var translate = d3_transform(brush.context.attr("transform")).translate,
            tx, ty;
        if (axisRotated) {
            tx = chartObj.plotarea.left + chartObj.plotarea.width;
            ty = chartObj.plotarea.top;
        } else {
            tx = chartObj.plotarea.left;
            ty = translate[1];
        }
        brush.context.attr("transform", "translate(" + tx + "," + ty + ")");
        
        if (isOverviewEnabled) {
            overviewContext
                .style({
                    "left": tx + "px",
                    "top": ty + "px"
                });
        }
        
        var scrollDIM = {}; // height | width for extent and background
        scrollDIM[(axisRotated) ? "width" : "height"] = scrollData.height || defaultOptions.scrollHeight;
        
        applyStyle(brush.context.selectAll("rect.extent"), scrollData.slider, defaultOptions.slider, scrollDIM);
        applyStyle(brush.context.selectAll("rect.background"), scrollData.tracker, defaultOptions.tracker, scrollDIM);
        applyStyle(brush.context.selectAll(".resize").selectAll("rect"), scrollData.resizer, defaultOptions.resizer, scrollDIM);
        
        removeResizer();
    }
    
    function applyStyle(ele, conf, defaultConf, scrollDIM) {
        conf = conf || {};
        ele
            .style({
                "visibility": "visible",
                "cursor": conf.cursor || defaultConf.cursor,
                "fill": conf.fillColor || defaultConf.fillColor,
                "opacity": conf.fillOpacity,
                "stroke-width": conf.strokeWidth,
                "stroke": conf.strokeColor
            })
            .attr({
                "rx": conf.radius,
                "ry": conf.radius
            })
            .attr(scrollDIM);
    }
    
    function removeResizer() {
        var resizerData = scrollData.resizer || {};
        if (!resizerData.enabled) {
            brush.context.selectAll(".resize").remove();
        }
    }
    
    function setScrollScale(chartObj, plotarea, type) {
        
        var scale = chartObj.axes.x.scrollScale = $ZC.scale.getScale(chartObj, chartObj.dataObject, plotarea, orient, 0, chartObj.dataset, type, null, 0)
        if (type == "x" && !axisRotated) {
            brush.component.x(scale);
        } else {
            brush.component.y(scale);
        }
    }
    
    function getCategoryPosByCount(plotarea, padding, outerPadding, numVisibleCategories){
        var scrollScale = chartObj.axes.x.scrollScale,
            baseValue = (axisRotated) ? plotarea.height : plotarea.width,
            xCat = chartObj.dataObject.getCategories("x"),
            xCat = (xCat && xCat.length) ? xCat : chartObj.dataObject.findXCategories(),
            actNumCategories = xCat.length;
            numVisibleCategories = (numVisibleCategories > actNumCategories) ? actNumCategories : numVisibleCategories;
        var categoryPosValue = (scrollScale) ? getActualScreenValue(scrollScale, numVisibleCategories-1, xCat[numVisibleCategories-1]) : mathFloor(((baseValue - outerPadding) / actNumCategories)*numVisibleCategories);
        return categoryPosValue;
    }
    function getCategoryPosByThickness(plotarea, padding, outerPadding,thickness){
        var scrollScale = chartObj.axes.x.scrollScale,
            baseValue = (axisRotated) ? plotarea.height : plotarea.width;
        var step = getCategoryStep(thickness, padding);
        var numVisibleCategories = getVisibleNumCategories(chartObj, plotarea, outerPadding, step);

        if(scrollScale && scrollScale.rangeExtent){
            var range = scrollScale.rangeExtent();
            outerPadding = getOuterPaddingInPixel(scrollScale.rangeExtent(), numVisibleCategories, padding, outerPadding, step);
        }else{
            outerPadding = 2 * (step * outerPadding);
        }
        var numVisibleCategories = getVisibleNumCategories(chartObj, plotarea, outerPadding, step);
        var categoryPosValue = getCategoryPosByCount(plotarea,padding,outerPadding,numVisibleCategories);
        return categoryPosValue;
    }

    function getNumScreens(chartObj, numVisibleCategories) {
        var xCat = chartObj.dataObject.getCategories("x");
        xCat = (xCat && xCat.length) ? xCat : chartObj.dataObject.findXCategories();
        var actNumCategories = xCat.length;
        var numOfScreen = (actNumCategories > numVisibleCategories) ? actNumCategories / numVisibleCategories : 1;
        return numOfScreen;
    }
    
    function getCategoryPosScreen(chartObj, plotarea) {
        var baseValue = (axisRotated) ? plotarea.height : plotarea.width;
        var numVisibleCategories = scrollData.visibleCategories;
        var chartType = chartObj.chartTypes[0].name;
        var padding = (chartType == "gantt" || chartType == "bullet" || chartType == "bar") ? chartObj.systemConf.chart.plot.plotoptions[chartType].padding || 0.2 : 0.2;
        var outerPadding = chartObj.systemConf.chart.plot.plotoptions[chartType].outerPadding || 0.02;
        var categoryPosValue;

        if(numVisibleCategories){
            categoryPosValue = getCategoryPosByCount(plotarea, padding, outerPadding, numVisibleCategories)
        }else{
            categoryPosValue = getCategoryPosByThickness(plotarea,padding,outerPadding,scrollData.categoryThickness||defaultItemSize)
        }
        // categoryPos = getScrollBarLength(chartObj.axes.xaxis.scrollScale,categoryPos)
        return categoryPosValue;
    }
    function getActualScreenValue(scrollScale,index,value){
        if(scrollScale.rangeExtent){
            return scrollScale.range()[index];
        }else{
            return scrollScale(value);
        }
    }
    function getScrollBarLength(scrollScale, categoryPos){
        if(scrollScale.rangeExtent){
            if(axisRotated){
                return scrollScale.rangeExtent()[1]-categoryPos;
            }else{
                return categoryPos+scrollScale.rangeBand();
            }
        }else{
            if(axisRotated){
                return scrollScale.range()[0]-categoryPos;
            }else{
                return categoryPos;
            }
            
        }
    }
    
    function getVisibleNumCategories(chartObj, plotarea, outerPadding, step) {
        return (axisRotated) ? mathFloor((plotarea.height - outerPadding) / step) : mathFloor((plotarea.width - outerPadding) / step);
    }
    
    function getOuterPadding(datatype, actScale, scrollScale, plotarea, numVisibleCategories) {
        var actDomain = actScale.domain()[0],
            scrollDomain = scrollScale.domain()[0],
            outerPadding;
        if (datatype == TIME_DATATYPE) {
            outerPadding = 2 * actScale(scrollDomain.getTime());
        } else if (datatype == NUMERIC_DATATYPE) {
            outerPadding = 2 * actScale(scrollDomain);
        } else if (datatype == ORDINAL_DATATYPE) {
            // $ZC.scale.getOuterPadding(actScale.rangeExtent(), actScale.domain().length, padding, outerPadding)
            // var rangeBand = actScale.rangeBand();
            outerPadding = actScale.outerPadding;
        }
        return outerPadding;
    }
    
    function renderOverviewContext() {
        var plotarea = chartObj.plotarea,
            userdata = chartObj.userdata;
        
        // scrollData.tracker = {"fillColor" : "transparent"};
        userdata.chart.scroll.enabled = false;
        userdata.chart.axes.xaxis.show = false;
        userdata.chart.axes.yaxis[0].show = false;
        userdata.canvas.title.show = false;
        userdata.canvas.subtitle.show = false;
        (userdata.chart.plot.renderer) ?
            (userdata.chart.plot.renderer.mode = "canvas") :
            (userdata.chart.plot) ? userdata.chart.plot.renderer = {
                    mode: "canvas"
                } : userdata.chart.plot = {
                    renderer: {
                        mode: "canvas"
                    }
                };
        userdata.chart.marginTop = 0;
        userdata.chart.marginBottom = 0;
        userdata.chart.marginLeft = 0;
        userdata.chart.marginRight = 0;
        userdata.canvas.intelligence = {};
        (userdata.credits) ? userdata.credits.show = false : null;
        (userdata.chart.plot.datalabels) ? userdata.chart.plot.datalabels.show = false : null;
        userdata.legend.enabled = false;
        
        overviewContext = chartObj.container.selectAll("div.zcoverview");
        overviewContext
            .style({
                "position": "absolute",
                "width": plotarea.width + "px",
                "height": (+scrollData.height) + "px",
                "pointer-events": "none",
                "box-sizing": "border-box"
            });
        $ZC.charts(overviewContext.node(), userdata);
    }
};