//$Id$
/**
 *
 * @returns {chart} sunburst renderer.
 */
svg_plot_renderer.sunburst = function() {
    var commonRendererProp, xscale, yscale, arc, radius, dataset, redraw = false,
        arcLabel, systemconf, oldData, yaxiscolumnorder;
    var chartwidth, chartheight, color, chartObj, notesArray, path, pathGroup, commonPlotOptions;
    var start, end, innerrad, rad,
        highlightGroup,
        highlightEle,
        highlightHierarchy,
        maxDepth;
    
    function chart(selection) {
        selection.each(function(data) {
            //renderer config values
            var rp = commonRendererProp.params;
            chartObj = rp.chartObj,
                commonPlotOptions = rp.commonPlotOptions,
                color = rp.color,
                notesArray = rp.notesArray,
                yaxiscolumnorder = rp.yaxiscolumnorder,
                dataset = rp.dataset;
            systemconf = chartObj.systemConf;
            var labelQueue = rp.labelQueue;
            
            chartwidth = chartObj.plotarea.width,
                chartheight = chartObj.plotarea.height;
            
            var gvpFn = $ZC.utils.getValueOfPlotOptions,
                dataLabels = commonPlotOptions.datalabels,
                outerPadding = (+commonPlotOptions.outerPadding || 0),
                padding;
            highlightGroup = null;
            maxDepth = 0;
            
            var showlabel = isDatalabelEnabled(dataLabels, chartObj)
            var boundBoxPadding = (showlabel && isBoundBoxEnabled(dataLabels, chartObj)) ? (getBoundBoxPadding(dataLabels, chartObj) + getBoundBoxStrokeWidth(dataLabels, chartObj)) : 0
            
            var eventDisabled = (validateNullVal(getEventConf(null, commonPlotOptions, chartObj, "enabled"), true) == false);
            var cursor = eventDisabled ? null : (commonPlotOptions.events) ? commonPlotOptions.events.cursor || "pointer" : systemconf.chart.plot.events.cursor || "pointer";
            highlightHierarchy = commonPlotOptions.events && commonPlotOptions.events.highlightHierarchy;
            
            //Animation variables
            var animationOptions = getAnimationConf(commonPlotOptions, null, chartObj, "sunburst");
            var isAnimationEnable = animationOptions.enabled,
                animationType = animationOptions.type,
                easingType = animationOptions.easingType,
                transitionDuration = redraw ? (isAnimationEnable ? 200 : 0) : animationOptions.duration;
            
            radius = (mathMin(chartwidth, chartheight) / 2) - outerPadding - (2 * boundBoxPadding);
            var innerRadius = mathMin(mathMax(0, perToPx(commonPlotOptions.innerRadius, radius)), perToPx("95%", radius)),
                fillColor = chart.fillColorFunc = getFillColor(chartObj, 0, [radius, radius]),
                strokeColor = getStrokeColor(chartObj, 0);

            var partition = d3_partition()
                .value(function(d, i) {
                    return data[getSeriesIndex(d)].disabled ? 0 : mathAbs(dataset.getY(d, yaxiscolumnorder));
                })
                .children(getChildren)
                .sort(null);
            
            start = function(d) {
                return mathMax(0, mathMin(2 * mathPI, xscale(d.x)));
            },
                end = function(d) {
                    return mathMax(0, mathMin(2 * mathPI, xscale(d.x + d.dx)));
                },
                innerrad = function(d) {
                    return yscale(d.y) + (padding / 2);
                };
            rad = function(scale, extra) {
                extra = extra || 0;
                return function(d) {
                    return yscale(d.y + d.dy) + extra - (padding / 2);
                };
            };
            
            arc = d3.d4_arc() //#ZC814 - d3.svg.arc has issue in this 
                .startAngle(start)
                .endAngle(end)
                .innerRadius(innerrad)
                .outerRadius(rad(yscale));
            
            arcLabel = d3.d4_arc() //#ZC814 - d3.svg.arc has issue in this 
                .startAngle(start)
                .endAngle(end)
                .innerRadius(innerrad)
                .outerRadius(rad(yscale));

            pathGroup = appendEle(selection, "g", [data], ".", "class", "hierarchychart");
            pathGroup
                .attr("transform", "translate(" + chartwidth / 2 + "," + chartheight / 2 + ")")
                .style("cursor", cursor);

            oldData = [];
            pathGroup.selectAll("path")
                .each(function(d, i) {
                    oldData.push({
                        x: d.x,
                        dx: d.dx
                    });
                });
            
            path = appendEle(pathGroup, "path", partition.nodes);
            var colorLighter = [],
                leafNodes = [],
                childColorIndex;
            data.forEach(function(d, i) {
                leafNodes[i] = [[]];
                leafNodes[i].disabled = d.disabled;
                var nonRGBAColor = rgb2hex(color[i % color.length]);
                colorLighter[i] = d3_scaleLinear()
                    .domain([0, getChildrenCount(d)])
                    .range([nonRGBAColor, d3_rgb(nonRGBAColor).brighter(1.5)])
            });

            path
                .attr("class", function(d, i) {
                    return (d.level != null) ? ("series_" + getSeriesIndex(d)) : null;
                })
                .attr("id", function(d) {
                    return "arcPath_" + d.level;
                })
                .each(function(d, i) {
                    maxDepth = mathMax(maxDepth, d.depth);
                    if (!d.depth) {
                        d.arcColor = "transparent";
                    } else {
                        if (d.depth === 1) {
                            childColorIndex = 0;
                        }

                        if (d.color) {
                            d.arcColor = d.color;
                        } else {
                            var plotoptions = data[getSeriesIndex(d)].plotoptions;
                            var chClr = (plotoptions != null) ? gvpFn(plotoptions, commonPlotOptions)("childrenColors") : commonPlotOptions.childrenColors;
                            if (chClr != null) {
                                d.arcColor = chClr[childColorIndex % chClr.length];
                            } else {
                                var c = colorLighter[getSeriesIndex(d)](childColorIndex);

                                var rgba = d.parent.arcColor.match(RGBA_PATTERN);
                                if (rgba && rgba.length === 5) { // if (rgba && rgba.length === 4s) {
                                    c = hex2rgba(c, rgba[4]); // return hex2rgba(newColor, rgba[3]);
                                }

                                d.arcColor = c;
                            }
                        }

                        if (!d.children) {
                            childColorIndex++;
                            leafNodes[getSeriesIndex(d)][0].push(d);
                        }
                    }
                });

            xscale = d3_scaleLinear().range([0, 2 * mathPI]);
            yscale = d3_scaleLinear().range([innerRadius, radius]).domain([data.dy, 1]);
            padding = mathMin(mathMax(0, perToPx(commonPlotOptions.padding, radius, 0)), (radius - innerRadius) * 0.9 / maxDepth);

            path
                .transition()
                .duration(transitionDuration)
                .ease(redraw ? "linear" : easingType)
                .attrTween("d", (redraw || (animationType === "fan")) ? fanTween : ((animationType === "wheel") ? wheelTween : innerTween));

            path
                .style({
                    "fill": function(d) {
                        return d.fillColor = fillColor(d.seriesPos, d, null, getBaseClr(d));
                    },
                    "stroke": function(d) {
                        if (!d.depth) {
                            return "white";
                        } else if (commonPlotOptions.strokeColor) {
                            return commonPlotOptions.strokeColor;
                        } else {
                            return strokeColor(getSeriesIndex(d), d, getBaseClr(d));
                        }
                    },
                    "stroke-opacity": function(d) {
                        return d.depth ? (gvpFn(data[getSeriesIndex(d)].plotoptions, commonPlotOptions)("strokeOpacity")) : 1;
                    },
                    "fill-opacity": function(d) {
                        return d.depth ? (gvpFn(data[getSeriesIndex(d)].plotoptions, commonPlotOptions)("fillOpacity")) : 0;
                    },
                    "stroke-width": function(d) {
                        return d.depth ? (gvpFn(data[getSeriesIndex(d)].plotoptions, commonPlotOptions)("strokeWidth")) : 0;
                    }
                });

            if (!eventDisabled) {
                var mousemove = function(d, i) {
                        var args = getEventArgs(d, i);
                        chartObj.eventHandler.mouseMove(args);
                    },
                    mouseout = function(d, i) {
                        chartObj.eventHandler.mouseOut(chart, d, getSeriesIndex(d));
                    },
                    click = function(d, i) {
                        var args = getEventArgs(d, i);
                        chartObj.eventHandler.click(chart, d, getSeriesIndex(d), d.itemPos, args);
                        //clickFunc(path,d);     //default click function
                    },
                    dblclick = function(d, i) {
                        if (!chart.chartRendered) {
                            return;
                        }
                        var dt = (d.children == null) ? (($ZC.annotation != null) ? generateDataForAnnotation(d, ((data.length > 1) ? d.parent.seriesname : null)) : d) : [d.seriesname, d.value];
                        var coordinates = arcLabel.centroid(d);
                        coordinates = [(coordinates[0] + chartwidth / 2), (coordinates[1] + (chartheight / 2))];
                        var args = {
                            "coordinate": coordinates,
                            "data": dt,
                            "basedOn": "plot"
                        };
                        chartObj.eventHandler.doubleclick(d, getSeriesIndex(d), args);
                    },
                    eventAttacherOptions = {
                        target: "plot",
                        touchleave: true,
                        chartObj: chartObj
                    };
                var mouseHandlers = {
                    "mousemove": mousemove,
                    "mouseout": mouseout,
                    "click": click,
                    "dblclick": dblclick
                },
                touchHandlers = {
                    "singletouchmove": mousemove,
                    "touchend": mouseout,
                    "tap": click,
                    "dbltap": dblclick
                };
                attachevents(path.filter(function(d) {
                    return d.depth != 0;
                }), mouseHandlers, touchHandlers, eventAttacherOptions);
            }
            
            chart.renderedTimer = setTimeout(function() {
                if ($ZC.labelRenderer != null && !chartObj.dataObject.hasHierarchyData) {
                    var outline = d3_arc()
                        .startAngle(start)
                        .endAngle(end)
                        .innerRadius(rad(yscale))
                        .outerRadius(rad(yscale));

                    var datalabelsPadding = commonPlotOptions.datalabels.padding + boundBoxPadding;
                    var outlineWithPad = d3_arc()
                        .startAngle(start)
                        .endAngle(end)
                        .innerRadius(rad(yscale, datalabelsPadding))
                        .outerRadius(rad(yscale, datalabelsPadding));
                    
                    data.forEach(function(d, i) {
                        var args = [leafNodes, i, null, radius, function(d) {
                            return d.parent.value
                        }, outlineWithPad, outline, arcLabel, [chartwidth / 2, chartheight / 2]];
                        labelQueue.push({
                            "renderer": chart,
                            "chartObj": chartObj,
                            "charttype": "pie",
                            "args": args
                        });
                        labelQueue.push({
                            "renderer": chart,
                            "chartObj": chartObj,
                            "charttype": "pieInnerLabels",
                            "args": args
                        });
                    });
                }
                
                //Annotation preprocessing
                if ($ZC.annotation != null) {
                    data.forEach(function(series, i) {
                        notesArray = $ZC.annotation.preprocessAnnotations(data, i, function(dt, k, j) {
                            var c = arcLabel.centroid(dt);
                            return [(c[0] + (chartwidth / 2)), (c[1] + (chartheight / 2))];
                        }, notesArray, chartObj, rp);
                    });
                }
                
                redraw = true;
                chart.chartRendered = true;
                
            }, (transitionDuration + 1000 / 60));

            function getEventArgs(d, i) {
                var ev = getEventObject(d3.event);
                return {
                    "renderer": chart,
                    "seriesPos": getSeriesIndex(d),
                    // "subseriesPos": 0,
                    // "itemPos": d.itemPos,
                    "coordinates": [ev.pageX, ev.pageY],
                    "element": this,
                    "color": getBaseClr(d),
                    "data": d,
                    "level" : d.level
                };
            }
        });
        return chart;
    }
    
    chart.commonRendererProp = function(_) {
        if (!arguments.length) {
            return commonRendererProp;
        }
        commonRendererProp = _;
        return chart;
    };
    chart.defaultMouseOverFunc = function(d, seriesPos, itemPos) {
        if (!highlightGroup) {
            highlightGroup = chartObj.staticPaneSVG.selectAll("g#zchierarchyhighlight")
                .attr("transform", "translate(" + (chartObj.plotarea.left + chartwidth / 2) + "," + (chartObj.plotarea.top + chartheight / 2) + ")");
            highlightEle = appendEle(highlightGroup, "path", d3_range(highlightHierarchy ? maxDepth : 1));
        }

        var highlightEffect = chartObj.legendHighlight.getLegendHighlightEffect(seriesPos, "selectedSeries", "plot"),
            elementIds = getHierarchyElementIds(d.level);

        if (highlightEffect !== "none") {
            highlightGroup
                .style("opacity", null);

            var refElements = pathGroup.selectAll(elementIds.join())[0];
            highlightEle
                .each(function(d, i) {
                    var ele = d3_select(this);
                    if (refElements[i]) {
                        var refEle = d3_select(refElements[i]),
                            arcColor = getRawColor(chartObj, seriesPos, refEle.datum(), seriesPos);
                        ele
                            .attr("d", refEle.attr("d"))
                            .style(chartObj.legendHighlight.getEffect(highlightEffect, "sunburst", (/(illuminate|moody|flip)/.test(highlightEffect) ? arcColor : "transparent"), {fill: "transparent", stroke:arcColor}, seriesPos));
                    } else {
                        ele
                            .attr("d", null);
                    }
                });
        }
    };
    chart.defaultMouseOutFunc = function(d, i) {
        if (!highlightGroup) { //Touch device -> tap on a data, then tap a legend item. Mouseout should be called and legend tap action also will get executed.
            highlightGroup = chartObj.staticPaneSVG.selectAll("g#zchierarchyhighlight");
        }

        highlightGroup
            .style("opacity", 0);
    };
    
    // function Tween(d, tweentype) {
    //     var xd = d3_interpolate(xscale.domain(), [d.x, d.x + d.dx]),
    //         yd = d3_interpolate(yscale.domain(), [d.y, 1]),
    //         yr = d3_interpolate(yscale.range(), [d.y ? 20 : 0, radius]);
    //     if (tweentype == "arc") {
    //         return function(d, i) {
    //             return i ?
                    
    //                 function(t) {
    //                     return arcLabel(d);
    //                 } :
    //                 function(t) {
    //                     xscale.domain(xd(t));
    //                     yscale.domain(yd(t)).range(yr(t));
    //                     return arcLabel(d);
    //                 };
    //         };
    //     } else {
    //         return function(d, i) {
    //             return i ?
                    
    //                 function(t) {
    //                     return "translate(" + arcLabel.centroid(d) + ")";
    //                 } :
    //                 function(t) {
    //                     xscale.domain(xd(t));
    //                     yscale.domain(yd(t)).range(yr(t));
    //                     return "translate(" + arcLabel.centroid(d) + ")";
    //                 };
    //         };
    //     }
    // }
    
    function fanTween(d, i) {
        var startAngle = d3_interpolate(((redraw && oldData[i]) ? start(oldData[i]) : 0), start(d));
        var endAngle = d3_interpolate(((redraw && oldData[i]) ? end(oldData[i]) : 0), end(d));
        return function(t) {
            arc.startAngle(startAngle(t)).endAngle(endAngle(t));
            return arc(d);
        };
    }
    
    function innerTween(d) {
        var inner = d3_interpolate(0, innerrad(d));
        var outer = d3_interpolate(0, rad(yscale)(d));
        return function(t) {
            arc.innerRadius(inner(t)).outerRadius(outer(t));
            return arc(d);
        };
    }
    
    function wheelTween(d) {
        var endAngle = d3_interpolate(start(d), end(d));
        return function(t) {
            arc.endAngle(endAngle(t));
            return arc(d);
        };
    }
    
    // function clickFunc(path, d) {
    //     path.transition()
    //         .duration(250)
    //         .attrTween("d", Tween(d, "arc"));
    //     // text.style("visibility", function(e) {return isParentOf(d, e) ? null : "hidden";})
    //     //  .transition()
    //     //  .duration(250)
    //     //  .attrTween("transform",Tween(d,"label"))
    //     //  .style("fill-opacity", function(e) { return isParentOf(d, e) ? 1 : 1e-6; });
    // }
    
    // function getDomainMax(parentElement) {
    //     return parentElement.children.length;
    // }
    
    // function isParentOf(p, c) {
    //     if (p === c) {
    //         return true;
    //     }
    //     if (p.children) {
    //         return p.children.some(function(d) {
    //             return isParentOf(d, c);
    //         });
    //     }
    //     return false;
    // }

    function getChildren(node, depth) {
        if (!chartObj.dataObject.hasHierarchyData && depth == 1 && node.length < 2 && node[0] && isArray(node[0][0])) {
            return node[0];
        } else {
            return isArray(node[0]) ? node : null;
        }
    }

    function getSeriesIndex(d) {
        return +d.level.split("_")[0];
    }

    function getHierarchyElementIds(id) {
        var hierarchyIds = [];
        var currentId = "path#arcPath";

        if (highlightHierarchy) {
            var levels = id.split("_");

            levels.forEach(function(d) {
                currentId = currentId + "_" + d;
                hierarchyIds.push(currentId);
            });
        } else {
            hierarchyIds.push(currentId + "_" + id);
        }

        return hierarchyIds;
    }

    function getBaseClr(d) {
        return d.children ? d.arcColor : null;
    }
    
    return chart;
};