//$Id$
$ZC.events.tooltip = function(element, data, chartObj) {
    "use strict";
    var hookPosition = {
        left: 50,
        top: 100,
        rotation: 0
    };
    var MAX_WIDTH = "40%",
        MAX_HEIGHT = "100%";
    
    var tooltip = {
            element: element
        },
        systemConf = chartObj.systemConf,
        dataset = chartObj.dataset,
        metadata = systemConf.metadata,
        metadata_axes = metadata.axes,
        metadata_columns = metadata.columns,
        tooltipIndices = metadata_axes.tooltip || d3.range(metadata_columns.length),
        tooltip_data = systemConf.tooltip,
        themeMaxWidth = null,

        // data processor helpers

        dataProcessorHelpers = $ZC.DataProcessor.helpers,
        getChartType = dataProcessorHelpers.getChartType,
        getParsedYAxisOrder = dataProcessorHelpers.getParsedYAxisOrder;
    
    if (tooltip_data.theme && tooltip_data.theme !== "") {
        var theme = isObjectPropDefined($ZC, "tooltip.getTheme") && $ZC.tooltip.getTheme(tooltip_data.theme, chartObj);
        if (theme !== null) {
            tooltip_data = chartObj.systemConf.tooltip = mergeJSON(tooltip_data, theme.borderStyle);
            chartObj.tooltip.content = [theme.body];
            themeMaxWidth = theme.maxWidth;
        }
    }
    
    var dist = 20,
        hookHeight = validateNullVal(tooltip_data.hookHeight, 6),
        borderWidth = (defined(tooltip_data.borderWidth) ? tooltip_data.borderWidth : 1),
        borderRadius = (defined(tooltip_data.borderRadius) ? tooltip_data.borderRadius : 5),
        borderStyle = defined(tooltip_data.borderStyle) ? tooltip_data.borderStyle : "solid",
        padding = defined(tooltip_data.padding) ? tooltip_data.padding + "" : 5 + "",
        isWithinChart = validateNullVal(tooltip_data.withinChart, true),
        hookBorderWidth = validateNullVal(tooltip_data.hookBorderWidth, (/ /g.test(borderWidth) ? 1 : +borderWidth)),
        parsedBorderRadius = chartObj.dataObject.getTransRectValue("radius", null, null, null, null, borderRadius);

    var maxWidth = perToPx((tooltip_data.maxWidth || themeMaxWidth || MAX_WIDTH), chartObj.chartarea.width),
        maxHeight = perToPx((tooltip_data.maxHeight || MAX_HEIGHT), chartObj.chartarea.height),
        minWidth = mathMax(chartObj.chartarea.width * 0.05, (2 * hookHeight));

    var headerContentFunc = tooltip_data.headerContent,
        footerContentFunc = tooltip_data.footerContent,
        isColumnView = (tooltip_data.view === "column"), //normal | column
        crosshair_data = tooltip_data.crosshair;
    
    // Tooltip elements
    var arrowbefore,
        arrowafter,
        arrowBeforeNAfter,
        contentBox;

    tooltip.initialize = function() {
        
        var fontSize = getFontSizeByHierarchy(tooltip_data, chartObj),
            defaultLineHeight = ((+fontSize) + (+fontSize) * 0.35),
            userdataCanvasFontColor = (chartObj.userdata.canvas != null) ? chartObj.userdata.canvas.fontColor : null,
            borderRadius_px = shorthand_px(borderRadius),
            filterDisabled = tooltip_data.useChartEffect === false, // enabled default
            filterEffect = null;

        if (!filterDisabled && chartObj.filterEffectsID) {
            filterEffect = "url(#" + chartObj.filterEffectsID + ")";
        }
        
        
        element.style({
            "width": "auto",
            "box-shadow": tooltip_data.shadow,
            "opacity": tooltip_data.opacity,
            "z-index": tooltip_data.zindex,
            "border-radius": borderRadius_px,
            "border-width": shorthand_px(borderWidth),
            "border-style": borderStyle,
            "-webkit-transform-style": "preserve-3d",
            "-moz-transform-style": "preserve-3d",
            "-ms-transform-style": "preserve-3d",
            "-o-transform-style": "preserve-3d",
            "transform-style": "preserve-3d",
            // "-webkit-backface-visibility": "hidden",
            // "-moz-backface-visibility": "hidden",
            // "-ms-backface-visibility": "hidden",
            // "-o-backface-visibility": "hidden",
            // "backface-visibility": "hidden",
            "pointer-events": "none",
            "filter": filterEffect
            // "padding": shorthand_px(padding)
        });
        var borderTopBox = appendEle(element, "div", tooltip_data.borderTop ? [tooltip_data.borderTop] : [], ".", "class", "borderTop");
        borderTopBox.style({
            "pointer-events":"none",
            "height":tooltip_data.borderTop+"px",
            "border-radius": borderRadius_px,
            "border-bottom-left-radius": "1px",
            "border-bottom-right-radius": "1px"
        })
        
        contentBox = appendEle(element, "div", [1], "#", "id", "tooltipcontent");
        contentBox.style({
            "max-width": maxWidth + "px",
            "min-width": minWidth + "px",
            // "max-height" : maxHeight + "px", // Is it necessary?
            "line-height": (tooltip_data.lineHeight || defaultLineHeight) + "px",
            "word-wrap": "break-word",
            "white-space": "normal",
            "pointer-events": "none",
            "padding": shorthand_px(padding)
            //		"border-top":"1px solid red"
        });
        applyFont(contentBox, [{
            fontColor: (tooltip_data.fontColor == null && userdataCanvasFontColor == null) ? "white" : tooltip_data.fontColor || userdataCanvasFontColor
        }, tooltip_data, chartObj.systemConf.canvas, $ZCG], true);
        
        // arrow before
        arrowbefore = appendEle(element, "div", [1], "#", "id", "zctooltipbefore");
        arrowbefore
            .style({
                "margin-left": -(hookHeight + (2 * hookBorderWidth)) + "px",
                "border": (hookHeight + (2 * hookBorderWidth)) + "px solid transparent"
            });
        
        
        // arrow after
        arrowafter = appendEle(element, "div", [1], "#", "id", "zctooltipafter");
        arrowafter
            .style({
                "margin-left": -hookHeight + "px",
                "border": hookHeight + "px solid transparent"
            });

        // Properties for arrow before and after
        arrowBeforeNAfter = element.selectAll("div#zctooltipbefore,div#zctooltipafter");
        arrowBeforeNAfter
            .style({
                "border-color": "transparent",
                "border-top-color": "white",
                "position": "absolute",
                "top": "100%",
                "left": "50%",
                "pointer-events": "none",
                "width": 0,
                "height": 0,
                "content": " "
            });
    };
    
    tooltip.show = function(chartObj, position, content, useDefault, defaultArgs) {
        if (!defined(defaultArgs)) {
            defaultArgs = {}
        }

        var color = defaultArgs.color,
            svgFillFunc = defaultArgs.svgFillFunc,
            seriesindex = defaultArgs.seriesindex,
            subseriespos = defaultArgs.subseriespos,
            itempos = defaultArgs.itempos,
            backgroundColor = defaultArgs.backgroundColor || tooltip_data.backgroundColor,
            borderColor = defaultArgs.borderColor || tooltip_data.borderColor,
            tooltipSharedInfo = defaultArgs.tooltipSharedInfo,
            tooltipShared = defaultArgs.tooltipShared,
            chartType = (seriesindex != null) && chartObj.chartTypes[seriesindex].name,
            hasMultiColoring,
            hasDefaultMultiColoring;
        useDefault = content == null && useDefault === true;

        // getting color
        if (useDefault) {
            hasMultiColoring = /(bar|scatter|bubble$|bullet|gantt)/.test(chartType) && !chartObj.legendarea.show && systemConf.chart.plot.plotoptions[chartType].multiColoring;
            hasDefaultMultiColoring = /(^pie|funnel|pyramid)/.test(chartType);
        }
        var clr = color;
        if (useDefault) {
            var idx = (hasMultiColoring || hasDefaultMultiColoring) ? itempos : seriesindex;
            var dataPoint = dataset.getDataPoint(chartObj, null, seriesindex, subseriespos, itempos, defaultArgs.d);
            clr = getRawColor(chartObj, idx, dataPoint, seriesindex);
        } else if (isArray(clr)) {
            clr = clr[seriesindex % clr.length];
        }

        clr = (tooltipShared) ? "black" : clr;
        borderColor = (borderColor != null) ? borderColor : clr;
        backgroundColor = (backgroundColor != null && backgroundColor != "") ? backgroundColor : clr;

        element
            .style({
                "visibility": "visible",
                "background-color": backgroundColor,
                "border-color": borderColor
            })
        //***** TOOLTIP INNERHTML
        var tooltipcontent = "",
            toUserInfo;

        //custom header content
        if (useDefault && headerContentFunc != null) {
            tooltipcontent = invokeFunction(headerContentFunc, d3.event, (toUserInfo = getInfoToUser()), chartObj);
        }

        //content
        if (useDefault) {
            if (tooltip_data.content) {
                tooltipcontent += invokeFunction(tooltip_data.content, d3.event, (toUserInfo = toUserInfo || getInfoToUser()), chartObj);
            } else if (tooltipShared) {
                tooltipcontent += getTooltipSharedContent(seriesindex, subseriespos, itempos, defaultArgs.d, tooltipSharedInfo, color, hasMultiColoring);
            } else {
                tooltipcontent += getDefaultContent(seriesindex, subseriespos, itempos, clr, defaultArgs.d);
            }
        } else {
            tooltipcontent += content;
        }

        //custom footer content
        if (useDefault && footerContentFunc != null) {
            tooltipcontent += invokeFunction(footerContentFunc, d3.event, (toUserInfo = toUserInfo || getInfoToUser()), chartObj);
        }

        contentBox
            .html(tooltipcontent);
        // TOOLTIP INNERHTML *****

        var elementNode = element.node(),
            eleWidth = elementNode.offsetWidth,
            eleHeight = elementNode.offsetHeight,
            curx = parseFloat(elementNode.style.left),
            cury = parseFloat(elementNode.style.top);

        var coordinate = getTooltipXY(position, eleWidth, eleHeight, chartObj);

        //borderTop
        if(tooltip_data.borderTop){
            var svgFillEle = (svgFillFunc) ? svgFillFunc((hasDefaultMultiColoring || hasMultiColoring ? itempos : seriesindex), null,null, clr, null,null,{"width":eleWidth},true) : borderColor;
            var svgFillUrl = (typeof svgFillEle === 'object') ? "url(" + getSVGImage(svgFillEle,true).url + ")" : svgFillEle;
            var borderTop = element.selectAll("div.borderTop");
            borderTop.style({
                "background":svgFillUrl
            })
        }

        if (crosshair_data.type != "none" && chartObj.isAxisCategory) {
            showCrossshair(chartObj, position.actLeft || position.left, position.actTop || position.top, crosshair_data);
        }

        // Transition will start after ~4ms only. If the tooltip is called at the first time no transition
        if (!curx || isNaN(curx) || !cury || isNaN(cury)) {
            element.style({
                "left": coordinate.x + "px",
                "top": coordinate.y + "px"
            });
        }

        curx = !curx || isNaN(curx) ? coordinate.x : (2 * curx + coordinate.x) / 3;
        cury = !cury || isNaN(cury) ? coordinate.y : (cury + coordinate.y) / 2;

        element
            .interrupt()
            .transition();

        element
            .transition()
            .delay(0);

        element
            .transition()
            .duration(100)
            .ease("linear")
            .styleTween("left", positionTween(curx, coordinate.x)) //To avoid sub pixel rendering issue, d3.interpolateRound has been used
            .styleTween("top", positionTween(cury, coordinate.y));
        
        
        // Tooltip arrow
        if (tooltipShared) {
            arrowBeforeNAfter
                .style("display", "none");
        } else {
            var arrowTransform = "rotate(" + hookPosition.rotation + "deg)",
                arrowTransformOrigin = "50% 0%";

            arrowBeforeNAfter
                .style({
                    "display": "",
                    "left": hookPosition.left + "%",
                    "top": hookPosition.top + "%",
                    "transform": arrowTransform,
                    "-webkit-transform": arrowTransform,
                    "-ms-transform": arrowTransform,
                    "transform-origin": arrowTransformOrigin,
                    "-webkit-transform-origin": arrowTransformOrigin,
                    "-ms-transform-origin": arrowTransformOrigin
                });

            arrowbefore
                .style("border-top-color", borderColor);

            arrowafter
                .style({
                    "border-top-color": backgroundColor,
                    "opacity": 1
                });
        }

        function getInfoToUser() {
            if (tooltipShared) {
                return tooltipSharedInfo.keys().map(function(d) {
                    var curSerpos = pInt(d),
                        curItempos = pInt(tooltipSharedInfo.get(d)),
                        clridx = dataset.getColumnIdx("clr");
                    var yaxiscolumnorder = chartObj.seriesdata[curSerpos].yaxiscolumnorder || [0, 0];
                    var validx = (chartObj.ismultiy || clridx == null) ? dataset.getColumnIdx("y", yaxiscolumnorder) : clridx;
                    var dp = dataset.getDataPoint(chartObj, validx, curSerpos, subseriespos, curItempos);
                    var metaInfoObj = dataset.getMetaInfoObj(chartObj, dp, curSerpos);
                    return {
                        "actData": dp,
                        "metaInfo": metaInfoObj
                    };
                });
            }
            var dp = dataset.getDataPoint(chartObj, null, seriesindex, subseriespos, itempos, defaultArgs.d);
            var metaInfoObj = chartObj.dataset.getMetaInfoObj(chartObj, dp, seriesindex);
            return {
                "actData": dp,
                "metaInfo": metaInfoObj,
                "seriesIndex": seriesindex,
                "subseriesIndex": subseriespos,
                "itemIndex": itempos
            };
        }
    };

    
    tooltip.hide = function(tooltipele) {
        tooltipele = (tooltipele == null) ? this.element : tooltipele;
        tooltipele
            .style("visibility", "hidden");
        if (chartObj.systemConf.tooltip.crosshair.type != "none") {
            chartObj.staticPaneSVG.selectAll("g.crosshair")
                .style("visibility", "hidden");
        }
    };

    tooltip.destroy = function() {
        this.element.remove();
    };

    tooltip.getTooltipContent = function() {
        /* arguments
         * 0 - seriesIndex
         * 1 - subseriesIndex
         * 2 - itemIndex
         */

        return getDefaultContent.apply(null,arguments);
    };
    
    function showCrossshair(chartObj, x, y, data) {
        chartObj.staticPaneSVG.selectAll("g.crosshair")
            .style("visibility", "visible");

        var type = data.type;
        if (type == "x" || type == "xy") {
            drawCrosshairLine(chartObj, x, y, data, "x")
        }
        if (type == "y" || type == "xy") {
            drawCrosshairLine(chartObj, x, y, data, "y")
        }
        
    }
    
    function drawCrosshairLine(chartObj, x, y, data, type) {
        var ele = chartObj.staticPaneSVG.selectAll("line." + type + "-crosshair");
        var isVerticalLine = (type == "x" && !chartObj.axes.rotated) || (type == "y" && chartObj.axes.rotated);
        ele
            .attr({
                "x1": (isVerticalLine) ? x : chartObj.plotarea.left,
                "y1": (isVerticalLine) ? chartObj.plotarea.top : y,
                "x2": (isVerticalLine) ? x : chartObj.plotarea.left + chartObj.plotarea.width,
                "y2": (isVerticalLine) ? chartObj.plotarea.height + chartObj.plotarea.top : y
            }).style({
            "shape-rendering": "crispEdges",
            "stroke": data.color,
            "stroke-width": data.strokeWidth,
            "stroke-dasharray": getDashArray(data.dashStyle, data.strokeWidth),
            "pointer-events": "none"
        })
    }
    
    function getTooltipXY(position, eleWidth, eleHeight, chartObj) {
        var marginRight = 2,
            canvasarea = chartObj.canvasarea,
            distance = dist + hookHeight + (position.distance || 0),
            dTop = position.dTop || 0,
            dLeft = position.dLeft || 0;
        hookPosition = {
            left: 50,
            top: 100,
            rotation: 0
        };
        
        var checkBounds = {
                left: mathAbs(mathMin(dLeft, 0)) + (chartObj.scrolledX || 0),
                top: mathAbs(mathMin(dTop, 0)) + (chartObj.scrolledY || 0),
                right: (canvasarea.width - marginRight - mathMax(dLeft, 0)) + mathAbs(mathMin(dLeft, 0)), // width + left
                bottom: (canvasarea.height - mathMax(dTop, 0)) + mathAbs(mathMin(dTop, 0)) // height + top
            },
            coordinate = {
                x: (position.left - (eleWidth / 2)),
                y: (position.top - eleHeight - distance)
            };
        
        if (!(!isWithinChart || chartObj.scrolledX || chartObj.scrolledY)) {
            var maxTop = position.top - eleHeight - distance,
                maxRight = position.left + eleWidth + distance,
                maxLeft = position.left - eleWidth - distance,
                maxBottom = position.top + eleHeight + distance
            
            if (maxTop > checkBounds.top) { // TOP
                positionHook(position, eleWidth, eleHeight, coordinate, checkBounds, "top");
                hookPosition.rotation = 0;
                hookPosition.top = 100;
            } else if (maxRight < checkBounds.right) { // RIGHT
                coordinate.x = position.left + distance;
                positionHook(position, eleWidth, eleHeight, coordinate, checkBounds, "right");
                hookPosition.rotation = 90;
                hookPosition.left = 0;
            } else if (maxLeft > checkBounds.left) { // LEFT
                coordinate.x = position.left - distance - eleWidth;
                positionHook(position, eleWidth, eleHeight, coordinate, checkBounds, "left");
                hookPosition.rotation = -90;
                hookPosition.left = 100;
            } else if (maxBottom < checkBounds.bottom) { // BOTTOM
                positionHook(position, eleWidth, eleHeight, coordinate, checkBounds, "bottom");
                coordinate.y = position.top + distance;
                hookPosition.rotation = 180;
                hookPosition.top = 0;
            }
        }

        return coordinate;
    }
    
    function positionHook(position, eleWidth, eleHeight, coordinate, checkBounds, mode) {
        var rad;
        if (/(top|bottom)/.test(mode)) {
            if ((position.left - (eleWidth / 2)) < checkBounds.left) { // Is within left?
                rad = getRad((mode == "top") ? "blx" : "tlx");
                coordinate.x = checkBounds.left;
                hookPosition.left = (mathMax(rad, (position.left - checkBounds.left)) / eleWidth) * 100;
            } else if ((position.left + (eleWidth / 2)) > checkBounds.right) { // Is within right?
                rad = getRad((mode == "top") ? "brx" : "trx");
                coordinate.x = checkBounds.right - eleWidth;
                hookPosition.left = (1 - (mathMax(rad, (checkBounds.right - position.left)) / eleWidth)) * 100;
            } else {
                hookPosition.left = 50;
            }
        } else {
            if ((position.top - (eleHeight / 2)) < checkBounds.top) { // Is within top?
                rad = getRad((mode == "left") ? "trY" : "tly");
                coordinate.y = checkBounds.top;
                hookPosition.top = (mathMax(rad, (position.top - checkBounds.top)) / eleHeight) * 100;
            } else if ((position.top + (eleHeight / 2)) > checkBounds.bottom) { // Is within bottom?
                rad = getRad((mode == "left") ? "bry" : "bly");
                coordinate.y = checkBounds.bottom - eleHeight;
                hookPosition.top = (1 - (mathMax(rad, (checkBounds.bottom - position.top)) / eleHeight)) * 100;
            } else {
                coordinate.y = position.top - (eleHeight / 2);
                hookPosition.top = 50;
            }
        }
    }
    
    function getRad(attr) {
        return hookHeight + parsedBorderRadius[attr] + mathSQRT(mathPow(hookBorderWidth, 2) + mathPow(hookBorderWidth, 2));
    }
    
    function getDefaultContent(seriesindex, subseriespos, itempos, color, d, customTooltipIndices) {
        
        if (isObjectPropDefined($ZC, "tooltip.getTheme") && defined(systemConf.tooltip.theme) && systemConf.tooltip.theme !== "") {
            if (defined(chartObj.tooltip.content)) {
                return dataset.getCustomizedVal(chartObj, chartObj.tooltip.content, seriesindex, subseriespos, itempos, d, "tooltip", color);
            }
        }
        
        var seriesdata = systemConf.seriesdata.chartdata,
            curTooltipIndices = customTooltipIndices || tooltipIndices,
            dataPoint;
        var tooltipcontent = isColumnView ? "<table cellspacing='0' cellpadding='0' border='0' style='border-spacing: 5px;'><tbody>" : "";
        curTooltipIndices.forEach(function(colidx, i) {
            var isRegexAvailable = metadata_columns[colidx] == null;
            var func = isRegexAvailable ? get_custom_content : get_default_col_val_content;
            tooltipcontent += func(seriesindex, subseriespos, itempos, d, colidx, i, color, curTooltipIndices);
        });
        tooltipcontent += isColumnView ? "</tbody></table>" : "";
        return tooltipcontent;
    }
    
    function get_default_col_val_content(seriesindex, subseriespos, itempos, d, colidx) {
        var columnname = dataset.getColumnName(colidx, false),
            yaxiscolumnorder = getParsedYAxisOrder(chartObj.seriesdata[seriesindex].yaxiscolumnorder),
            dataPoint = dataset.getDataPoint(chartObj, colidx, seriesindex, subseriespos, itempos, d),
            content = "";
        var yidx = dataset.getColumnIdx("y", yaxiscolumnorder);
        
        if (dataPoint != null) {
            var val = dataset.getDataPointValue(chartObj, dataPoint, colidx, seriesindex,false, "tooltip");
            val = (colidx == yidx && val instanceof Array) ? val[0] + " - " + val[1] : val
            content = isColumnView ?
                "<tr style='vertical-align: top;'><td style='max-width:" + (maxWidth * 0.4) + "px;'>" + columnname + ": </td><td style='max-width:" + (maxWidth * 0.6) + "px;'><b>" + val + "</b></td></tr>" :
                columnname + " : " + val + "<br>";
        }
        return content;
    }
    
    function get_custom_content(seriesindex, subseriespos, itempos, d, colidx, i, color, meta_tooltip) {
        var customizedString = dataset.getCustomizedVal(chartObj, colidx, seriesindex, subseriespos, itempos, d, "tooltip", color); //{{val(INDEX)}} {{col(INDEX)}} {{per(INDEX)}} - pattern will be replaced with the respective value , columnname and the percentage of value
        return isColumnView ?
            "<tr style='vertical-align: top;'><td style='max-width:" + maxWidth + "px;'>" + customizedString + "</td></tr>" :
            (customizedString + ((i == meta_tooltip.length - 1) ? "" : "<br>"));
    }
    
    function get_shared_tooltip_meta(tooltipSharedInfo) {
        var meta_tooltip = tooltipIndices;
        var has_custom_tooltip = meta_tooltip.some(function(d) {
            return (typeof d === "string") && CUSTOM_VALUE_PATTERN.test(d);
        });
        if (!has_custom_tooltip) {
            meta_tooltip = [];
            var x_pattern = "{{val(" + dataset.getColumnIdx("x") + ")}}",
                y_pattern = tooltipSharedInfo.y_val_pattern,
                clr_pattern = "{{clr}}",
                clr_val_pattern = tooltipSharedInfo.y_col_pattern;
            
            var x_val = x_pattern,
                y_val = isColumnView ?
                    "<tr style='vertical-align: top;'><td style='max-width:" + (maxWidth * 0.4) + "px;'><div style='margin-right:0.2em;display:inline-block;vertical-align:middle;border-radius: 50%;padding:0.4em;background:" + clr_pattern + "''></div>" + clr_val_pattern + ": </td><td style='max-width:" + (maxWidth * 0.6) + "px;'>" + y_pattern + "</td></tr>" :
                    "<div style='margin-right:0.2em;display:inline-block;vertical-align:middle;border-radius: 50%;padding:0.4em;background:" + clr_pattern + "''></div>" + clr_val_pattern + " : " + y_pattern;
            meta_tooltip.push(x_val, y_val);
        }
        return meta_tooltip;
    }
    
    function getTooltipSharedContent(seriesindex, subseriespos, itempos, d, tooltipSharedInfo, color, hasMultiColoring) {
        
        var tooltipcontent = isColumnView ? "<table cellspacing='0' cellpadding='0' border='0' style='border-spacing: 5px;'><tbody>" : "";
        var clridx = dataset.getColumnIdx("clr");
        var yaxiscolumnorder_series = chartObj.seriesdata[seriesindex].yaxiscolumnorder || [0, 0]; //yaxiscolumnorder is not mandatory
        var yidx = dataset.getColumnIdx("y", yaxiscolumnorder_series);
        var sharedKeys = tooltipSharedInfo.keys();
        
        var y_pattern = tooltipSharedInfo.y_val_pattern,
            meta_tooltip = get_shared_tooltip_meta(tooltipSharedInfo);
        
        meta_tooltip.forEach(function(pattern, i) {
            if (metadata_columns[pattern] != null) {
                //columnname : value
                tooltipcontent += get_default_col_val_content(seriesindex, subseriespos, itempos, d, pattern);
            } else if ((typeof pattern === "string") && (pattern.indexOf("{{val(" + yidx + ")}}") != -1)) {
                //y-values
                for (var i = 0, len = tooltipSharedInfo.size(); i < len; i++) {
                    var curSerpos = pInt(sharedKeys[i]),
                        curItempos = pInt(tooltipSharedInfo.get(sharedKeys[i])),
                        cur_pattern = pattern,
                        copy_y_pattern = y_pattern;

                    var clr = color;
                    if (isArray(clr)) {
                        var dataPoint = dataset.getDataPoint(chartObj, null, curSerpos, subseriespos, curItempos, d);
                        clr = getRawColor(chartObj, (hasMultiColoring ? curItempos : curSerpos), dataPoint, curSerpos);
                    }

                    if (chartObj.ismultiy || clridx == null) {
                        cur_pattern = pattern.replace("{{val(null)}}", (encodeHTML(chartObj.seriesdata[curSerpos].seriesname) || copy_y_pattern.replace(/val/gi, "col")))
                    }
                    var content = dataset.getCustomizedVal(chartObj, cur_pattern, curSerpos, subseriespos, curItempos, d, "tooltip", clr);
                    tooltipcontent += content + (isColumnView ? "" : "<br>");
                }
            } else {
                //Custom text with non y-values
                tooltipcontent += get_custom_content(seriesindex, subseriespos, itempos, d, pattern, i, color, meta_tooltip)
            }
        });
        
        tooltipcontent += isColumnView ? "</tbody></table>" : "";
        return tooltipcontent;
        
    }

    function positionTween(start, end) {
        return function(d, i) {
            var interpolate = d3.interpolateRound(parseInt(start), parseInt(end));
            return function(t) {
                return interpolate(t) + "px";
            };
        }
    }

    return tooltip;
};