//$Id$
/**
 *
 * @returns {chart} treemap renderer.
 */

 //hierarchy series name on the top like fusion chart
svg_plot_renderer.treemap = function() {
	var treemap = d3.layout.treemap(),
		isredraw = false,
		commonRendererProp,
		treemapGroup,
		path,
		chartObj,
		commonPlotOptions,
		highlightGroup,
		highlightEle,
		multiColoring,
		transform,
		borderRadius = {
		tlx: 0,
		tly: 0,
		trx: 0,
		trY: 0,
		brx: 0,
		bry: 0,
		blx: 0,
		bly: 0
	}

	function chart(selection) {
		selection.each(function(data) {
			//renderer config values
			var rp = commonRendererProp.params;
			chartObj = rp.chartObj;
			commonPlotOptions = rp.commonPlotOptions;
			var color = rp.color,
				notesArray = rp.notesArray,
				yaxiscolumnorder = rp.yaxiscolumnorder,
				dataset = rp.dataset,
				systemconf = chartObj.systemConf,
				chartWidth = chartObj.plotarea.width,
				chartHeight = chartObj.plotarea.height,
				labelQueue = rp.labelQueue;

			highlightGroup = null;

			var mode = commonPlotOptions.mode || "squarify",
				outerPaddingX = Math.min(chartWidth/2, perToPx(commonPlotOptions.outerPadding, chartWidth/2, 0)),
				outerPaddingY = Math.min(chartHeight/2, perToPx(commonPlotOptions.outerPadding, chartHeight/2, 0)),
				strokeWidth = commonPlotOptions.strokeWidth,
				strokeOpacity = commonPlotOptions.strokeOpacity,
				fillOpacity = commonPlotOptions.fillOpacity,
				negativeAsAbsolute = commonPlotOptions.negativeAs === "absolute",

				leafNodes = [],
				fillColor = chart.fillColorFunc = getFillColor(chartObj, 0, [chartWidth - (2 * outerPaddingX), chartHeight - (2 * outerPaddingY)]),
				strokeColor = getStrokeColor(chartObj, 0);

			//Animation variables
			var animationOptions = getAnimationConf(commonPlotOptions, null, chartObj, "treemap");
			var isAnimationEnable = animationOptions.enabled,
				animationType = animationOptions.type,
				easingType = animationOptions.easingType,
				transitionDuration = isredraw ? (isAnimationEnable ? 200 : 0) : animationOptions.duration;

			var eventDisabled = (validateNullVal(getEventConf(null, commonPlotOptions, chartObj, "enabled"), true) == false);
			var cursor = (commonPlotOptions.events) ? commonPlotOptions.events.cursor || "pointer" : systemconf.chart.plot.events.cursor || "pointer";
			transform = [outerPaddingX, outerPaddingY];

			treemap
				.size([chartWidth - (2 * outerPaddingX), chartHeight - (2 * outerPaddingY)])
				.value(function(d) {
					if (data[getSeriesIndex(d)].disabled) {
						return 0;
					} else {
						var y = dataset.getY(d, yaxiscolumnorder) || 0;
						return negativeAsAbsolute ? mathAbs(y) : mathMax(0, y);
					}
				})
				.sort(null)
				.children(getChildren)
				.padding(commonPlotOptions.padding)
				.mode(mode);

            var colorLighter = [],
                childColorIndex;
            data.forEach(function(d, i) {
                var nonRGBAColor = rgb2hex(color[i % color.length]);
                colorLighter[i] = d3_scaleLinear()
                    .domain([0, getChildrenCount(d)])
                    .range([nonRGBAColor, d3_rgb(nonRGBAColor).brighter(1.5)])
            });

			treemapGroup = appendEle(selection, "g", [data], ".", "class", "hierarchychart");
			treemapGroup
				.attr("transform", "translate("+ outerPaddingX+","+ outerPaddingY+")")
				.style({
					"stroke-width": strokeWidth,
					"stroke-opacity": strokeOpacity,
					"fill-opacity": fillOpacity
				});

			var path = appendEle(treemapGroup, "path", treemap.nodes);
			if (!data.value) {
				path.remove();
			}

			path
				.each(function(d, i) {
                    if (d.depth) {
                        if (d.depth === 1) {
                            childColorIndex = 0;
                        }

                        if (d.color) {
                            d.arcColor = d.color;
                        } else {
                            var c = colorLighter[getSeriesIndex(d)](childColorIndex);

                            var rgba = d.parent.arcColor.match(RGBA_PATTERN);
                            if (rgba && rgba.length === 5) { // if (rgba && rgba.length === 4s) {
                                c = hex2rgba(c, rgba[4]); // return hex2rgba(newColor, rgba[3]);
                            }

                            d.arcColor = c;  
                        }
                    }
                        if (!d.children) {
							var seriesIndex = getSeriesIndex(d);
							if (!leafNodes[seriesIndex]) {
								leafNodes[seriesIndex] = [];
							}
							leafNodes[seriesIndex].push(d);

                            childColorIndex++;
                        } else {
                        	d.arcColor = commonPlotOptions.parentColor || "transparent"
                        }
                })
				.style({
					"fill": function(d) {
						if (!d.children) {
							return d.fillColor = fillColor(getSeriesIndex(d), d);
						} else {
							return d.fillColor = d.arcColor;
						}
					},
					"stroke": function(d) {
                        if (commonPlotOptions.strokeColor) {
                            return commonPlotOptions.strokeColor;
                        } else if (d.children) {
                            return d.arcColor;
                        } else {
                            return strokeColor(getSeriesIndex(d), d);
                        }
					}
				})
				.attr("class", function(d, i) {
					return (d.level != null) ? ("series_" + getSeriesIndex(d)) : null;
				})
				.attr("id", function(d) {
					return "tree_" + d.level;
				});

			if (isredraw) {
				path
					.transition()
					.duration(transitionDuration)
					.ease("linear")
					.attr("d", function(d) {
						return roundedRect(d.x, d.y, d.dx, d.dy, borderRadius);
					});
			} else {
				path
					.attr("d", function(d) {
						return roundedRect(0, d.y, 0, d.dy, borderRadius);
					})
					.transition()
					.duration(transitionDuration)
					.ease(easingType)
					.attr("d", function(d) {
						return roundedRect(d.x, d.y, d.dx, d.dy, borderRadius);
					})
			}

			//Events
			if (!eventDisabled) {
				var mousemove = function(d, i) {
						var args = getEventArgs(d, i);
						chartObj.eventHandler.mouseMove(args);
					},
					mouseout = function(d, i) {
						chartObj.eventHandler.mouseOut(chart, d, getSeriesIndex(d));
					},
					click = function(d, i) {
						var args = getEventArgs(d, i);
						chartObj.eventHandler.click(chart, d, getSeriesIndex(d), args);
					},
					dblclick = function(d, i) {
						if (!chart.chartRendered) {
							return;
						}
						var dt = $ZC.annotation ? generateDataForAnnotation(d, ((data.length > 1) ? d.parent.parent.seriesname : null)) : d;
						var coordinates = [(d.x + outerPaddingX/2), (d.y + outerPaddingY/2)];
						var args = {
							"coordinate": coordinates,
							"data": dt,
							"basedOn": "plot"
						};
						chartObj.eventHandler.doubleclick(d, getSeriesIndex(d), args);
					},
					eventAttacherOptions = {
						target: "plot",
						touchleave: true,
						chartObj: chartObj
					};
				var mouseHandlers = {
						"mousemove": mousemove,
						"mouseout": mouseout,
						"click": click,
						"dblclick": dblclick
					},
					touchHandlers = {
						"singletouchmove": mousemove,
						"touchend": mouseout,
						"tap": click,
						"dbltap": dblclick
					};
				attachevents(path.filter(function(d) {
					return !d.children;
				}), mouseHandlers, touchHandlers, eventAttacherOptions);
			}

			chart.renderedTimer = setTimeout(function() {

                if ($ZC.labelRenderer) {
                    var totalFunc = function(d) {
                            return d.parent.value
                        };
                    data.forEach(function(d, i) {
                        labelQueue.push({
                            "renderer": chart,
                            "charttype": "hierarchy",
                            "chartObj": chartObj,
                            "args": [data[i], i, null, null, totalFunc, leafNodes[i], transform]
                        });
                    });
                }

                //Annotation preprocessing
                if ($ZC.annotation != null) {
                    data.forEach(function(series, i) {
                        notesArray = $ZC.annotation.preprocessAnnotations(data, i, function(dt, k, j) {
                            return [(dt.x + outerPaddingX/2), (dt.y + outerPaddingY/2)];
                        }, notesArray, chartObj, rp);
                    });
                }

				isredraw = true;
				chart.chartRendered = true;
			}, (transitionDuration + 17));

			function getEventArgs(d, i) {
				var ev = getEventObject(d3.event);
				return {
					"renderer": chart,
					"seriesPos": getSeriesIndex(d),
					// "subseriesPos": 0,
					// "itemPos": getItemPos(d),
					"coordinates": [ev.pageX, ev.pageY],
					"element": this,
					// "color": d.arcColor,
					"data": d,
					"level": d.level
				};
			}
		});
		return chart;
	}
	chart.commonRendererProp = function(_) {
		if (!arguments.length) {
			return commonRendererProp;
		}
		commonRendererProp = _;
		return chart;
	};
	chart.defaultMouseOverFunc = function(d, seriesPos, itemPos) {
		if (!highlightGroup) {
			highlightGroup = chartObj.staticPaneSVG.selectAll("g#zchierarchyhighlight")
				.attr("transform", "translate(" + (chartObj.plotarea.left + transform[0]) + "," + (chartObj.plotarea.top + transform[1]) + ")")
			highlightEle = appendEle(highlightGroup, "path", [1]);
		}
		var refElement = treemapGroup.selectAll("path#tree_" + d.level);
		var highlightEffect = chartObj.legendHighlight.getLegendHighlightEffect(seriesPos, "selectedSeries", "plot"),
			currentColor = getRawColor(chartObj, (multiColoring ? itemPos : seriesPos), d, seriesPos);

		highlightGroup
			.style("opacity", null);
		highlightEle
			.attr("d", function() {
				return roundedRect(d.x, d.y, d.dx, d.dy, borderRadius);
			})
			.style(chartObj.legendHighlight.getEffect(highlightEffect, "treemap", (/(illuminate|moody|flip)/.test(highlightEffect) ? currentColor : "transparent"), {
				fill: "transparent",
				stroke: currentColor
			}, (multiColoring ? itemPos : seriesPos)));
	};
	chart.defaultMouseOutFunc = function(d, i) {
		highlightGroup
			.style("opacity", 0);
	};

	function getChildren(node, depth) {
		if (!chartObj.dataObject.hasHierarchyData && depth == 1 && node.length < 2) {
			return node[0];
		} else {
			if (isArray(node[0])) {
				return node;
			} else {
				return null
			}
		}
	}

	function getSeriesIndex(d) {
		return +d.level.split("_")[0];
	}
	return chart;
};