//$Id$
"use strict";
var canvasUtils = (function() {
    var utils = {};
    utils.drawPath = function(ctx, pathString) {
        var relX=0, relY=0;
        parseSVGPath(pathString).forEach(function(d) {
            var moves, p;
            switch (d.function) {
                case "M":
                    ctx.moveTo(d.moves[0], d.moves[1]);
                    relX = d.moves[0], relY = d.moves[1];
                    break;
                case "L":
                    for (var i = 0; i < d.moves.length; i = i + 2) {
                        ctx.lineTo(d.moves[i], d.moves[i + 1]);
                    }
                    break;
                case "l": //for relative path line L
                    for (var i = 0; i < d.moves.length; i = i + 2) {
                        relX += d.moves[i], relY += d.moves[i+1];
                        ctx.lineTo(relX, relY);
                    }
                    break;
                case "A":
                    ctx.arcTo(d.moves[0], d.moves[1], d.moves[2], d.moves[3], d.moves[4]);
                    break;
                case "C":
                    moves = d.moves;
                    while (moves.length) {
                        p = moves.splice(0, 6);
                        ctx.bezierCurveTo(p[0], p[1], p[2], p[3], p[4], p[5]);
                    }
                    break;
                case "c": //for relative path cubic curve C
                    moves = d.moves;
                    while (moves.length) {
                        p = moves.splice(0, 6);
                        p[0] += relX, p[2] += relX, p[1] += relY, p[3] += relY;
                        relX += p[4] , relY += p[5];
                        ctx.bezierCurveTo(p[0], p[1], p[2], p[3], relX, relY);
                    }
                    break;
                case "Q":
                    moves = d.moves;
                    while (moves.length > 0) {
                        p = moves.splice(0, 6);
                        ctx.quadraticCurveTo(p[0], p[1], p[2], p[3]);
                    }
                    break;
                case "Z":
                    ctx.closePath();
                    break;
            }
        });
    };
    
    utils.getRadianAngle = function(degreeValue) {
        return degreeValue * mathPI / 180;
    };
    
    utils.constant = function(x) {
        return function constant() {
            return x;
        };
    };
    
    utils.pointX = function(p) {
        return p[0];
    };
    
    utils.pointY = function(p) {
        return p[1];
    };
    
    utils.rgba = function(color, opacity) {
        if (typeof color === "string" && (color.indexOf("rgba") > -1 || color === "transparent")) {
            return color;
        }
        var rgb = d3_rgb(color);
        opacity = (opacity === null || opacity === "" || opacity === undefined) ? 1 : opacity;
        return "rgba(" + rgb.r + "," + rgb.g + "," + rgb.b + "," + opacity + ")";
    };
    
    utils.roundedRect = function(context, x, y, width, height, radius) {
        radius = adjust(radius, width, height);
        context.moveTo(x + radius.tlx, y);
        context.lineTo(x + width - radius.trx, y);
        context.quadraticCurveTo(x + width, y, x + width, y + radius.trx);
        context.lineTo(x + width, y + height - radius.brx);
        context.quadraticCurveTo(x + width, y + height, x + width - radius.brx, y + height);
        context.lineTo(x + radius.blx, y + height);
        context.quadraticCurveTo(x, y + height, x, y + height - radius.blx);
        context.lineTo(x, y + radius.tlx);
        context.quadraticCurveTo(x, y, x + radius.tlx, y);
        context.closePath();
        
        function adjust(r, w, h) {
            if (w < 2 * r.tlx) {
                r.tlx = w / 2;
            }
            if (h < 2 * r.tlx) {
                r.tlx = h / 2;
            }
            if (w < 2 * r.trx) {
                r.trx = w / 2;
            }
            if (h < 2 * r.trx) {
                r.trx = h / 2;
            }
            if (w < 2 * r.blx) {
                r.blx = w / 2;
            }
            if (h < 2 * r.blx) {
                r.blx = h / 2;
            }
            if (w < 2 * r.brx) {
                r.brx = w / 2;
            }
            if (h < 2 * r.brx) {
                r.brx = h / 2;
            }
            return r;
        }
    };
    
    utils.boundBoxWithHook = function(context, pathDescription) {
        var controlIndex = 0;
        var points = parseSVGPath(pathDescription);
        points.forEach(function(d, i) {
            var moves = d.moves;
            switch (d.function.toLowerCase()) {
                case "m":
                    context.moveTo(moves[0], moves[1]);
                    break;
                case "l":
                    context.lineTo(moves[0], moves[1]);
                    break;
                case "a":
                    var prevMoves = points[i - 1].moves;
                    var controlPoint = (controlIndex == 0) ? [prevMoves[0], moves[6]] :
                        (controlIndex == 1) ? [moves[5], prevMoves[1]] :
                            (controlIndex == 2) ? [prevMoves[0], moves[6]] :
                                [moves[5], prevMoves[1]];
                    controlIndex++;
                    context.quadraticCurveTo(controlPoint[0], controlPoint[1], moves[5], moves[6]);
            }
        });
    };
    
    
    utils.getFontStyles = function(chartObj, dataLabels, color) {
        return applyDatalabelFonts(null, dataLabels, chartObj, color);
    };
    
    return utils
})();

