//$Id$
function rgb2hex(color){
   var rgb = color.match(RGBA_PATTERN);
   return (rgb && rgb.length === 5) ? "#" +
    ("0" + parseInt(rgb[1],10).toString(16)).slice(-2) +
    ("0" + parseInt(rgb[2],10).toString(16)).slice(-2) +
    ("0" + parseInt(rgb[3],10).toString(16)).slice(-2) : color;
}

function hex2rgba(hex, opacity){
    var hex = hex.toString().replace('#',''),
	    r = parseInt(hex.substring(0,2), 16),
	    g = parseInt(hex.substring(2,4), 16),
	    b = parseInt(hex.substring(4,6), 16);

    return 'rgba('+r+','+g+','+b+','+opacity+')';
}

function isRgba(color) {
    return /^rgba/.test(color);
}

function getColorCopy(color, k, copyType, colorMode) {
    if (color === "transparent") {
        return color;
    }

    copyType = pick(copyType, "darker");
    var convertToRGBA = false, d3Color, alpha;

    if (isRgba(color)) {
        alpha = color.match(RGBA_PATTERN)[4];
        color = rgb2hex(color);
        convertToRGBA = true;
    }

    d3Color = pick(colorMode, d3.rgb)(color);
    color = d3Color[copyType](k).toString();

    if (convertToRGBA) {
        color = hex2rgba(color, alpha);
    }

    return color;
}

function getFillColor(chartObj, seriesIndex, gradientDim, dontUsePatternPallete) {
	var defsLocation = chartObj.defsLocation,
        chartName = chartObj.chartTypes[seriesIndex].name,
		commonPlotOptions = chartObj.systemConf.chart.plot.plotoptions[chartName],
        plotoptions = chartObj.seriesdata[seriesIndex].plotoptions,
        gradientOptions = (plotoptions && plotoptions.gradients) ? mergeJSON(commonPlotOptions.gradients, plotoptions.gradients, true)  : commonPlotOptions.gradients,
		patternPallete = chartObj.patternPallete,
        imagePallete = chartObj.imagePallete,
		patternPalleteLength = !dontUsePatternPallete && patternPallete && patternPallete.length,
        rendererMode = chartObj.plot.renderer.mode,
        colorIndexScale = getColorIndexScale(chartObj);

    return function(i, d, colorGamma, customColor, fillOpacity, customGradientDim, customPatternDim, returnSVGimage) {

        var baseClr;
    	gradientDim = customGradientDim || gradientDim;
    	i = Math.max(i, 0) || 0; // check packed bubble and sunburst

    	if (customColor) {
            baseClr = customColor;
    	} else {
            baseClr = getRawColor(chartObj, i, d, seriesIndex);
            if (colorIndexScale && isUndefined(i = getColorIndex(chartObj, colorIndexScale, d, i))) {
                return baseClr;
            }
        }

        if(rendererMode == "canvas" && !gradientOptions.type && !(baseClr instanceof Object)){
            baseClr = canvasUtils.rgba(baseClr,fillOpacity);
        }

	    if (patternPalleteLength && /(url|data)/.test(baseClr)) {
	    	return baseClr;
	    }

	    if (defined(colorGamma)) {
	    	baseClr = getColorCopy(baseClr, colorGamma).toString();
	    }
        if(imagePallete && imagePallete.length){
            var prop = {
                name : chartObj.imagePallete[i],
                width : (customPatternDim && customPatternDim.width) || chartObj.patternImageSizes[i][0],
                height : (customPatternDim && customPatternDim.height) ||chartObj.patternImageSizes[i][1],
                mode :rendererMode,
                context: customPatternDim && customPatternDim.context
            }
            var patternEle = (rendererMode == "svg" || returnSVGimage) ? effects.get(svg_patterns,"imageFill",prop,{"returnSVGimage":returnSVGimage})
                                                                        : effects.get(canvas_patterns,"imageFill",prop,{"returnSVGimage":returnSVGimage});
            if(rendererMode == "canvas"){
                return patternEle;
            }
            return "url(" + defsLocation + "#" + patternEle + ")";
        }
	    if (patternPalleteLength) {
            var patternOptions = patternPallete[i % patternPalleteLength];
            var prop = getPatternProp(patternOptions,baseClr,fillOpacity,customPatternDim);
            
            var patternEle = (rendererMode == "svg" || returnSVGimage) ? effects.get(svg_patterns, "svgPatternFill", prop, {"returnSVGimage":returnSVGimage}) : 
                                                                         effects.get(canvas_patterns, "canvasPatternFill",prop,{"returnSVGimage":returnSVGimage});
            
            if(returnSVGimage || rendererMode == "canvas"){
                return patternEle;
            }
            return "url(" + defsLocation + "#" + patternEle + ")";
	    }

	    if (gradientDim && gradientOptions && gradientOptions.type) {
            var prop = getGradientProp(gradientOptions,gradientDim,baseClr,fillOpacity);
            var args = {"offset":chartName, "chartObj":chartObj};
	    	var gradientEle = (rendererMode == "svg") ?  effects.get(svg_gradients, "svgGradientFill",prop, args) : effects.get(canvas_gradients, "canvasGradientFill",prop, {"offset":chartName, "chartObj":chartObj,"returnSVGimage":returnSVGimage});

            if(returnSVGimage || rendererMode == "canvas"){
                return gradientEle;
            }
            return "url(" + defsLocation + "#" + gradientEle + ")";

	    	// if (gradientEle) {
	    	// 	return "url(" + defsLocation + "#" + gradientEle + ")";
	    	// }
	    }

	    return baseClr;
    }
}

/*
 * @param {Object} chartObj
 * @param {Number} seriesIndex
 * @returns {Color}
 */
function getStrokeColor(chartObj, seriesIndex) {
    return function(i, d, customColor, colorGamma) {
        if (customColor) {
            return customColor;
        } else {
            i = Math.max(i, 0) || 0; // check packed bubble and sunburst
            var baseClr = getRawColor(chartObj, i, d, seriesIndex);
            if (defined(colorGamma)) {
                baseClr = getColorCopy(baseClr, colorGamma).toString();
            }
            return baseClr;
        }
    }
}

/*
 * @param {Number} clr
 * @param {Object} chartObj
 * @param {Scale} colorScale
 * @param {Array} colorScale_domain
 * @returns {Boolean}
 */
function isWithinColorRange(clr, chartObj, colorScale, colorScale_domain) {
    if (!colorScale) {
        colorScale = chartObj.axes.clr.scale;
    }

    if (!colorScale) { //If clr index is not specified
        return true;
    }

    if (!colorScale_domain) {
        colorScale_domain = colorScale.domain();
    }

    if (clr === UNDEFINED){
        clr = null;
    }

    var filterRange = chartObj.systemConf.legend.filter.range,
        isWithinRange = !filterRange || ((chartObj.dataset.getDataType("clr") === ORDINAL_DATATYPE) ? (filterRange.indexOf(clr) > -1) : (clr >= filterRange[0] && clr <= filterRange[1])),
        isWithinScale = (chartObj.dataset.getDataType("clr") === ORDINAL_DATATYPE) ? (colorScale_domain.indexOf(clr) > -1) : (clr >= colorScale_domain[0] && clr <= colorScale_domain[colorScale_domain.length - 1]);
    return isWithinScale && isWithinRange && colorScale(clr) !== null && !(isHeatMapChartType(chartObj.chartTypes[0].value) && clr == null);
}

/*
 * @param {Object} chartObj
 * @param {Number} i
 * @param {Array} d
 * @param {Number} seriesIndex
 * @returns {Color} 1)Threshold 2)Color scale 3)Index based
 */
function getRawColor(chartObj, i, d, seriesIndex) {
    var baseClr = getColorByIndex(chartObj, i),
        rp = chartObj.isAxisCategory && defined(seriesIndex) && chartObj.rendererConf && chartObj.rendererConf[seriesIndex].params;

    if (rp && rp.colorOverlay) {
        return getColorBythreshold(chartObj, rp, d, baseClr);
    } else if (d && colorscale_helpers.isQuantileColorScale(chartObj)) {
        return getColorByScale(chartObj, d);
    } else {
        return (d && d.arcColor) || baseClr;
    }
}

/*
 * @param {Object} chartObj
 * @param {Number} i
 * @returns {Color} based on index
 */
function getColorByIndex(chartObj, i) {
    var seriesColor = chartObj.seriesColor;
    return seriesColor[i % seriesColor.length];
}

/*
 * @param {Object} chartObj
 * @param {Object} rp
 * @param {Array} d
 * @param {Color} baseClr
 * @returns {Color} color overlay
 */
function getColorBythreshold(chartObj, rp, d, baseClr) {
    var yaxiscolumnorder = rp.yaxiscolumnorder,
        threshold = rp.threshold,
        thresholdType = rp.thresholdType,
        dataset = chartObj.dataset;

    var thresholdValue = (thresholdType == "x") ? dataset.getX(d) : dataset.getY(d, yaxiscolumnorder);
    return rp.colorOverlay(threshold, threshold.color, thresholdValue, baseClr);
}

/*
 * @param {Object} chartObj
 * @param {Array} d
 * @returns {Color} based on color scale
 */
function getColorByScale(chartObj, d) {
    var hiddenColor = "transparent",
        clrValue = chartObj.dataset.getClr(d),
        colorScale = chartObj.axes.clr.scale;

    if (isWithinColorRange(clrValue, chartObj, colorScale)) {
        if (clrValue === UNDEFINED){
            clrValue = null;
        }
        return colorScale(clrValue);
    } else {
        return hiddenColor;
    }
}

/*
 * @param {Object} chartObj
 * @returns {Scale} index scale
 */
function getColorIndexScale(chartObj) {
    var hasColorScale = colorscale_helpers.isQuantileColorScale(chartObj),
        user_ranges = chartObj.systemConf.legend.colorBand.ranges,
        scale;

    if (hasColorScale && user_ranges && user_ranges.length) { //color index scale is only for user ranges
        var colorScale = chartObj.axes.clr.scale,
            colorScale_domain = colorScale.domain(),
            patternPallete = chartObj.patternPallete,
            imagePallete = chartObj.imagePallete,
            pallete;

        if (imagePallete && imagePallete.length) {
            pallete = imagePallete;
        } else if (patternPallete && patternPallete.length) {
            pallete = patternPallete;
        }
        if (pallete) {
            scale = colorScale.copy();
            scale.range(colorscale_helpers.getProcessedColorRange(d3_range(pallete.length), colorScale_domain, chartObj.systemConf.legend.colorBand, chartObj));
        }
    }
    return scale;
}

/*
 * @param {Object} chartObj
 * @param {Scale} colorIndexScale
 * @param {Array} d
 * @param {Number} i
 * @returns {Number} color index
 */
function getColorIndex(chartObj, colorIndexScale, d, i) {
    var clrValue = chartObj.dataset.getClr(d);
    if (isWithinColorRange(clrValue, chartObj)) {
        if (colorIndexScale) {
            return colorIndexScale(clrValue);
        } else {
            return i;
        }
    }
}