//$Id$
$ZC.utils = function() {};

/*
 * element - to which the bound box is drawn
 * component1 - node from which we start to draw the box
 * component2 - where to end the box
 * properties - outer box configuration
 */

function drawBorderBox(element, properties) {
    
    element
        .attr('x', properties.left)
        .attr('y', properties.top)
        .attr('width', properties.width)
        .attr('height', properties.height)
        .style('fill-opacity', 0)
        .style('stroke', "transparent");
    
}
$ZC.utils.getActualTextBounds = function(fontSize,width,rotation)
{   
    var width = width,
    height = fontSize,
    radian = rotation *(Math.PI * 2 / 360)
    var bbox = {}
    bbox.width = Math.abs(height * Math.sin(radian)) + Math.abs(width * Math.cos(radian))
    bbox.height = Math.abs(height * Math.cos(radian)) + Math.abs(width * Math.sin(radian))
    return bbox;
}
var mergeJSON = $ZC.utils.mergeJSON = function(source1, source2, mergeConflicts, mergeIntoExists) {
    /*
     * Properties from the Souce2 object will be copied to Source1 Object.
     * Note: mergeIntoExists : true -- Here source 1 and source2 will get merged into source1.
     * mergeIntoExists : false -- This method will return a new merged object, Source1 and Source2 original values will not be replaced.
     * */
    source1 = source1 || {};
    source2 = source2 || {};
    
    var mergedJSON = (mergeIntoExists != null && mergeIntoExists == true) ? source1 : simpleClone(source1); // Cloning the source1
    for (var attrname in source2) {
        if (source2.hasOwnProperty(attrname)) {
            if (mergedJSON[attrname] != undefined) {
                if (source2[attrname] != null && (source2[attrname].constructor == Object || source2[attrname].constructor == Array)) {
                    /*
                     * Recursive call if the property is an object,
                     * Iterate the object and set all properties of the inner object.
                     */
                    mergedJSON[attrname] = mergeJSON(mergedJSON[attrname], source2[attrname], mergeConflicts, mergeIntoExists);
                } else if (mergeConflicts == true && typeof mergedJSON === "object") { //Merge the property from source 2 to source1
                    mergedJSON[attrname] = source2[attrname];
                }
                
            } else { //else Add the property from source 2 to source1
                mergedJSON[attrname] = source2[attrname];
                
            }
        }
    }
    
    return mergedJSON;
};

$ZC.utils.getValueOfPlotOptions = function(obj1, obj2) {
    /*
     * obj1 -->high priority
     * obj2 -->Low priority
     */
    return function(key) {
        if (obj1 != null && obj1[key] != null) {
            return obj1[key];
        } else if (obj2 != null && obj2[key] != null) {
            return obj2[key];
        } else {
            return null
        }
    }
};

$ZC.utils.getVal = function(value) {
    /*
     * If the value is function, execute it and return the value
     */
    return (typeof value === "function") ? value() : value;
};


function generateIdByPropValue(properties, name) {
    if (properties == null || properties.constructor != Object) {
        if (name != null) {
            return name;
        }
        return;
    }
    var id = "",
        val;
    for (var prop in properties) {
        if (properties.hasOwnProperty(prop)) {
            val = properties[prop];
            id += (val == null || val == "") ? "$$" : (val + "_");
        }
    }
    id = name + id;
    id = id.replace(/[.-]/g, '_');
    id = id.replace(/[`~!@#$%^&*( )|+\-=?;:'",.<>{}[\]\\/\s]/gi, '_');
    return id;
}


function getMonochromeColor(baseClr, domain, range) {
    var hsl = d3_hsl(baseClr);
    
    var interpolateClr = d3_scaleLinear()
        .domain([-1, 0, 1])
        .range([d3_hsl(hsl.h, hsl.s, 0.1), hsl, d3_hsl(hsl.h, hsl.s, 0.9)])
        .interpolate(d3_interpolateRgb);
    var clrScale = d3_scaleLinear()
        .domain(domain)
        .range(range);
    return function(clr) {
        return interpolateClr(clrScale(clr));
    };
}

function bbox(ele) {
    //if element is removed from DOM, firefox will throw NS_ERROR_FAILURE on getBBox(). This function returns getBBox() of the given element if that element is present in DOM, else returns {x:0, y:0, height:0, width:0}
    if (ele) {
        if (document.body.contains(ele) && (ele.style.display != "none")) {
            return ele.getBBox();
        }
    }
    return {
        x: 0,
        y: 0,
        height: 0,
        width: 0
    };
}
var simpleClone = $ZC.utils.simpleClone = function(o) {
    var copy = o,
        k;
    
    if (o && typeof o === 'object') {
        copy = isArray(o) ? [] : {};
        for (k in o) {
            // if(o.hasOwnProperty(k)){
            copy[k] = simpleClone(o[k]);
            // }
        }
    }
    
    return copy;
};

function decodeHTML(text) {
    if (!$ZC.dataEncoded || text == null) {
        return text;
    }
    var tempDiv = document.createElement("div");
    tempDiv.innerHTML = encodeURI(text);
    var decodedText = tempDiv.textContent;
    return decodeURI(decodedText);
}

function encodeHTML(text) {
    if ($ZC.dataEncoded || text == null) {
        return text;
    }
    var tempDiv = document.createElement("div");
    tempDiv.textContent = decodeHTML(text);
    var encodedText = tempDiv.innerHTML;
    return encodedText;
}

function validateURL(url) {
    var exp_http = /https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)/g,
        exp_nonHttp = /[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)?/gi;
    var regExp_http = new RegExp(exp_http),
        regExp_nonHttp = new RegExp(exp_nonHttp);
    return (url.match(regExp_http) || url.match(regExp_nonHttp));
}

function d3_values(map) {
    return map.slice(0);
}

function getDefsLocation() {
    return d3_selectAll("base").node() ? location.href.split(location.search || location.hash || /[?#]/)[0] : "";
}

function isObjectPropDefined(obj, key, self) {
    var keySet = key.replace(/\[(\w+)\]/g, '.$1'); // convert indexes to properties
    keySet = keySet.replace(/^\./, ''); // strip a leading dot
    var keys = keySet.split('.');

    var childObj = keys.reduce(function(o, x) {
        return !defined(o) ? o : o[x];
    }, obj);
    return self ? childObj : defined(childObj);
    
    // return keys.every(function (x) {
    // 	if (typeof obj != "object" || obj === null || !x in obj)
    // 		return false;
    // 	obj = obj[x];
    // 	return true;
    // });
}

/**
 *
 * @param {Object|*} variable
 * @return {boolean}
 */
function isUndefined(variable) {
    return !defined(variable);
}

function toArray(objectLikeArray) {
    return Array.prototype.slice.call(objectLikeArray);
}

/**
 * It invokes a function with the given arguments.
 * If the first argument is true, then corresponding function will have
 * access to 'this' keyword with the value arguments[2]
 */

function invokeFunction() {
    var args = toArray(arguments),
        advanced = args[0] === true && args.shift(),
        callback = args.shift();
    
    
    if (typeof callback === "string") {
        var temp = getPropVal(window, callback);
        // callback = defined(temp) ? temp : callback; // for both global variable and global function support.
        callback = typeof temp === "function" ? temp : callback; // for only global function support.
    }
    return typeof callback === "function" ? advanced ? callback.apply(args[0], args.splice(1)) : callback.apply(undefined, args) : callback;
}

function getValByPriority(array, attr) {
    var i = 0,
        len = array.length,
        val;
    for (; (i < len && (val == null || val === "")); i++) {
        if(array[i]) {
            val = array[i][attr];
        }
    }
    return val;
}

function validateNullVal(actualVal, defaultVal) {
    var val = (actualVal == null || actualVal === "") ? defaultVal : actualVal;


    var parsedVal = JSON.parse(val);
    val = (typeof parsedVal == "number") ? parseFloat(parsedVal) : (typeof parsedVal == "boolean") ? JSON.parse(parsedVal) : parsedVal;
    return val;
}

function validateValWithinLimit(data, prop, minPercent, maxPercent, width, height) {
    var val;
    var refLevel = mathMin(width, height);
    var maxVal = (maxPercent * refLevel) / 100,
        minVal = (minPercent * refLevel) / 100;
    val = data[prop]
    val = (val == null || val === "") ? null : (val < minVal) ? minVal : (val > maxVal) ? maxVal : parseFloat(val);
    return val;
}

/**
 * Returns true if the object is not null or undefined.
 * @param {Object} obj
 */

function defined(obj) {
    return obj !== undefined && obj !== null;
}

/**
 * Returns true if the value is null object or null string
 * @param {Object} obj
 */

function isNull(value) {
    return value == null || value == "null";
}

/**
 *
 * @param e
 * @returns {boolean}
 */

function isTrue(e) {
    return e + "".toLocaleLowerCase() === "true";
}

function shorthand_px(val) {
    /*
     * Input : 10 5 5 10
     * Output : 10px 5px 5px 10px
     */
    val += "";
    return val.replace(/ /g, "px ") + "px";
}

/**
 *
 * @param {Object} obj
 * @param {String} key
 * @returns {*}
 */
var getPropVal = $ZC.utils.getPropVal = function(obj, key) {
    return isObjectPropDefined(obj, key, true);
};

function intersectObject(obj1, obj2) {
    if (!obj1 || !obj2) {
        return null;
    }
    var output = {},
        intersected;
    for (var prop in obj1) {
        if (obj1.hasOwnProperty(prop) && obj2.hasOwnProperty(prop)) {
            var val1 = obj1[prop],
                val2 = obj2[prop];
            if (val1 && (val1.constructor == Object || val1.constructor == Array)) {
                var i = intersectObject(val1, val2);
                if (i != null) {
                    output[prop] = i;
                    intersected = true;
                }
            } else if (val1 == val2) {
                output[prop] = val1;
                intersected = true;
            }
        }
    }
    return intersected ? output : null;
}

/**
 *
 * @param {String} color
 * @returns {boolean}
 */

function isDarkColor(color) {
    // http://stackoverflow.com/questions/2509443/check-if-uicolor-is-dark-or-bright/2509596#2509596
    // https://stackoverflow.com/questions/12043187/how-to-check-if-hex-color-is-too-black
    // https://stackoverflow.com/questions/24260853/check-if-color-is-dark-or-light-in-android
    // https://stackoverflow.com/questions/635022/calculating-contrasting-colours-in-javascript
    
    var rgb = d3_rgb(color);
    
    // return 1 - (0.299 * rgb.r + 0.587 * rgb.g + 0.114 * rgb.b) / 255 > 0.45;
    
    // return 0.2126 * rgb.r + 0.7152 * rgb.g + 0.0722 * rgb.r < 165; // zoho-reports version
    
    return (rgb.r * 299 + rgb.g * 587 + rgb.b * 114) / 1000 < 128; // tiny color
}

/**
 *
 * @param {String} color
 * @returns {boolean}
 */

function isLightColor(color) {
    return !isDarkColor(color);
}

/**
 *
 * @param {String} color
 * @returns {string}
 */

function getContrastColor(color) {
    return isDarkColor(color) ? "#f8f8f8" : "#444444";
}

/**
 *
 * @param {*} obj
 * @returns {boolean}
 */

function isWindow(obj) {
    return obj != null && obj === obj.window;
}

/**
 *
 * @param elem
 * @returns {boolean|DocumentView}
 */

function getWindow(elem) {
    return isWindow(elem) ? elem : elem.nodeType === 9 && elem.defaultView;
}

/**
 *
 * @param {Element} elem
 * @returns {{top: number, left: number}}
 */

function offset(elem) {
    var box = elem.getBoundingClientRect(),
        doc = elem && elem.ownerDocument,
        docElem = doc.documentElement,
        win = getWindow(doc);
    return {
        top: box.top + win.pageYOffset - docElem.clientTop,
        left: box.left + win.pageXOffset - docElem.clientLeft
    };
}

/**
 *
 * @param {String} str
 * @returns {Array}
 */
function stringifiedArrayReverse(str) {
    if (!str) {
        return str;
    }

    return str
        .split(",")
        .reverse()
        .join();
}

/**
 *
 * @returns {Boolean}
 */
function isHiddenDocument() {
    return defined(document.visibilityState) && document.visibilityState !== "visible"; //window blur state
}

/**
 *
 * @param {Element} element
 * @param {Element} container
 * @param {Boolean|null} partial
 * @returns {boolean}
 */

function elementIsVisible(element, container, partial) {
    var contHeight = container.offsetHeight,
        elemTop = offset(element).top - offset(container).top,
        elemBottom = elemTop + element.offsetHeight;
    return (elemTop >= 0 && elemBottom <= contHeight) || (partial && ((elemTop < 0 && elemBottom > 0) || (elemTop > 0 && elemTop <= contHeight)))
}

$ZC.utils.elementIsVisible = elementIsVisible;

/**
 * Return the first value that is defined.
 */

var pick = function() {
    var args = arguments,
        i, arg, length = args.length;
    for (i = 0; i < length; i++) {
        arg = args[i];
        if (defined(arg)) {
            return arg;
        }
    }
    return args[length - 1];
};

/**
 *
 * joins one or multiple arrays.
 * if the first argument is true, then remaining arrays will copied to the first array
 * else it copies all arrays to a new array.
 * @return {Array}
 */

function arrayJoin() {
    // performance https://jsperf.com/array-prototype-push-apply-vs-concat/5
    // https://jsperf.com/arrayconcatvsarraypushapply
    // return target.concat(newObject)
    
    var args = toArray(arguments),
        copyToExisting = args[0] === true && args.shift(),
        target = copyToExisting ? args.shift() : [];
    
    if ((copyToExisting && !args.length)) {
        return target;
    }
    
    args.forEach(function(newObject) {
        var i = 0,
            c = newObject.length;
        for (; i < c; ++i) {
            target.push(newObject[i]);
        }
    });
    return target;
}

// for identifying d3 selection
d3.selection.prototype.isD3Selection = true;

function formatArray(array, formatter, formatConf, chart, dataType, type,specifierIndex) {
    return array.map(function(value) {
        return formatter(formatConf, value, dataType, chart, type,null,null,specifierIndex);
    })
}

function sortByLength(array) {
    return array.slice(0).sort(function (a, b) {
        // ASC  -> a.length - b.length
        // DESC -> b.length - a.length
        if (!defined(a)) {
            a = "";
        }
        if (!defined(b)) {
            b = "";
        }
        return b.length - a.length;
    })
}

function isHidden(el) {
    return (el.offsetParent === null)
}

function each(obj, fn) {

    if (typeof obj !== "object") {
        return false;
    }

    if (isArray(obj) || typeof (obj.length) === "number") {
        Array.prototype.forEach.call(obj, fn)
    } else {
        for (var attr in obj) {
            if (obj.hasOwnProperty(attr)) {
                fn(obj[attr], attr, obj)
            }
        }
    }
}

/**
 * Extend one or more objects with the members of another
 * @param target
 * @returns {Object}
 */

function extend(target) {
    if (!arguments[1]) {
        return;
    }
    for (var ii = 1, ll = arguments.length; ii < ll; ii++) {
        var source = arguments[ii];
        for (var prop in source) {
            if (!target[prop] && source.hasOwnProperty(prop)) {
                target[prop] = source[prop];
            }
        }
    }
    return target;
}

/**
 * Inherits an Object
 * @param {Object|*} subClass
 * @param {Object|*} superClass
 */

function inherit(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
    }
}

/**
 *
 * @param {Array} array
 * @param {*} element
 * @returns {Array}
 */
function removeElementFromArray(array, element) {
    var index = array.indexOf(element);
    if (index > -1) {
        array.splice(index, 1);
    }
    return array;
}