//$Id$
//This is a Semantic Zooming Interface
//The zoom options can be configured in systemconf.chart.zoom.type
//Expected values are "x","y","xy"
//Multi yaxes - zoomY is not implemented
$ZC.events.zoom = function(chartObj, axisrotated, systemconf, dataset) {
    "use strict";
    var mdwn = false,
        startPt, endPt, zmevent = {},
        zmarea,
        svg = chartObj.svg,
        seriesdata = systemconf.seriesdata.chartdata,
        zmConf = systemconf.chart.zoom,
        mindist = 2,
        plotarea = chartObj.plotarea;
    var zmareaConf = zmConf.selectionArea,
        zmbtnConf = zmConf.resetButton;
    var btnBorderConf = zmbtnConf.border,
        btnShadowConf = zmbtnConf.shadow,
        btnLabelConf = zmbtnConf.label;
    var zmtype = zmConf.type,
        showBtnBorder = btnBorderConf.show,
        x = zmbtnConf.x,
        y = zmbtnConf.y;
    var btnRadius = (showBtnBorder ? btnBorderConf.radius : 0),
        filterID = null,
        rectBound;
    if (btnShadowConf.show) {
        filterID = effects.get(svg_filters, "dropShadow",btnShadowConf);
        filterID = filterID ? "url(" + chartObj.defsLocation + "#" + filterID + ")" : null;
    }
    
    var callbacks = zmConf.events || {};
    
    zmevent.zmInit = function() {
        zmarea = appendEle(svg.selectAll(".seriesholder"), "rect", [1], ".", "class", "zmarea");
        zmarea
            .style({
                "fill": zmareaConf.fillColor || "grey",
                "fill-opacity": zmareaConf.fillOpacity,
                "visibility": "hidden"
            });
        
        /*
         * reset button added
         * */
        var zoomBtn = appendEle(chartObj.staticPaneSVG.select(".eventgroup"), "rect", [1], ".", "class", "zmbtn");
        var dashStyle = getDashArray(btnBorderConf.style, btnBorderConf.size);
        
        zoomBtn
            .attr({
                "rx": btnRadius,
                "ry": btnRadius,
                "filter": filterID
            })
            .style({
                "fill": zmbtnConf.fillColor,
                "fill-opacity": zmbtnConf.fillOpacity,
                "stroke": btnBorderConf.color,
                "stroke-width": (showBtnBorder ? btnBorderConf.size : 0),
                "stroke-dasharray": dashStyle,
                "stroke-linecap": getLineCap(btnBorderConf.lineCap, dashStyle),
                "visibility": "hidden",
                "cursor": "pointer"
            });
        
        var handlers = {
            "click": zmevent.reset
        };
        var touchhandlers = {
            "tap": zmevent.reset
        };
        attachevents(zoomBtn, handlers, touchhandlers);
        
        //$$COMMENT$$ - Create a group and append the rect and text to the group and position the group - Just for the maintenance.
        //$$COMMENT$$ - It is also better to add one event tracker rect to the group and listen the reset event. Not sure how many events trigger happen in different devices / browsers.
        
        var zoomBtnText = appendEle(chartObj.staticPaneSVG.select(".eventgroup"), "text", [1], ".", "class", "zmbtn");
        
        zoomBtnText
            .attr("dy", "0.9em")
            .style({
                "visibility": "hidden",
                "pointer-events": "none"
            })
            .text(btnLabelConf.text);
        applyFont(zoomBtnText, [btnLabelConf, systemconf.chart, systemconf.canvas, {
            fontColor: "white",
            fontSize: 11,
            fontFamily: "verdana"
        }, $ZCG]);
        
        var textBound = zoomBtnText.node().getBBox();
        zoomBtn
            .attr({
                "width": (textBound.width + btnLabelConf.marginLeft + btnLabelConf.marginRight),
                "height": (textBound.height + btnLabelConf.marginTop + btnLabelConf.marginBottom)
            });
        rectBound = zoomBtn.node().getBBox();
        x = getVal(x, (plotarea.left + plotarea.width - rectBound.width));
        y = getVal(y, 10);
        zoomBtn
            .attr({
                "x": x,
                "y": y
            });
        zoomBtnText
            .attr({
                "x": (x + btnLabelConf.marginLeft),
                "y": (y + btnLabelConf.marginTop)
            });
    };
    
    zmevent.zoomInitialized = false;
    
    zmevent.zmStart = function(mouse) {
        zmtype = zmConf.type;
        mdwn = true;
        startPt = getCoordinate(mouse);
        var prop = {};
        zmarea.style("visibility", "visible");
        var horiZoom = ((zmtype.indexOf("x") > -1) && !axisrotated) || ((zmtype.indexOf("y") > -1) && axisrotated),
            verZoom = ((zmtype.indexOf("y") > -1) && !axisrotated) || ((zmtype.indexOf("x") > -1) && axisrotated);

        if (horiZoom) {
            prop.x = startPt.x;
        } else {
            prop.width = plotarea.width;
        }
        if (verZoom) {
            prop.y = startPt.y;
        } else {
            prop.height = plotarea.height;
        }
        zmarea.attr(prop);
        
        zmevent.zoomInitialized = true;
        // invoke the callback
        
        invokeFunction(true, callbacks.zoomstart, startPt, {
            "x": zmConf.x,
            "y": zmConf.y
        }, chartObj);
    };
    
    zmevent.zmMove = function(mouse) {
        zmtype = zmConf.type
        if (mdwn) {
            var point = getCoordinate(mouse);
            var prop = {};
            var horiZoom = ((zmtype.indexOf("x") > -1) && !axisrotated) || ((zmtype.indexOf("y") > -1) && axisrotated),
                verZoom = ((zmtype.indexOf("y") > -1) && !axisrotated) || ((zmtype.indexOf("x") > -1) && axisrotated);

            if(isTouchEvent(d3.event)) {
                startPt = getCoordinate(d3_mousePoint(chartObj.eventHandler.trackerElement.node(), 0)),
                point = getCoordinate(d3_mousePoint(chartObj.eventHandler.trackerElement.node(), 1));
            }
            endPt = point;

            if (horiZoom) {
                prop.x = ((point.x < startPt.x) ? point : startPt).x;
                prop.width = mathAbs(point.x - startPt.x); //Reverse area selection (Right -> Left) also possible by Math.abs
            }
            if (verZoom) {
                prop.y = ((point.y < startPt.y) ? point : startPt).y;
                prop.height = mathAbs(point.y - startPt.y); //Reverse area selection (Bottom -> Top) also possible by Math.abs
            }
            zmarea.attr(prop);
        }
        invokeFunction(true, callbacks.zoom, point, {
            "x": zmConf.x,
            "y": zmConf.y
        }, chartObj);
    };
    
    zmevent.zmEnd = function(mouse) {
        if (!zmevent.zoomInitialized) {
            return false
        }

        if(!isTouchEvent(d3.event)) {
            endPt = getCoordinate(mouse);
        }

        zmarea
            .attr({
                "height": 0,
                "width": 0
            })
            .style("visibility", "hidden");
        mdwn = false;
        zmevent.zoomInitialized = false;
        return {
            "startPt": startPt,
            "endPt": endPt
        };
    };
    
    zmevent.zoomed = function(zmdata, seriesidx, xdomain, ydomain) {
        zmtype = zmConf.type;
        var rangeDefined = (seriesidx != null), //chartObj.zoomX, chartObj.zoomY, chartObj.zoomXY will also this function
            xtype = (axisrotated ? "y" : "x"),
            ytype = (axisrotated ? "x" : "y");
        var zoomedX = ((zmtype.indexOf("x") > -1) && (rangeDefined || mathAbs(zmdata.startPt[xtype] - zmdata.endPt[xtype]) > mindist)),
            zoomedY = ((zmtype.indexOf("y") > -1) && (rangeDefined || mathAbs(zmdata.startPt[ytype] - zmdata.endPt[ytype]) > mindist));
        
        var isZoomed = ((zmtype.indexOf("x") > -1) && (rangeDefined ? (xdomain[0] != xdomain[1]) : (zmdata.startPt[xtype] != zmdata.endPt[xtype]))) || ((zmtype.indexOf("y") > -1) && (rangeDefined ? (ydomain[0] != ydomain[1]) : (zmdata.startPt[ytype] != zmdata.endPt[ytype])));
        if (!isZoomed) { //If zoom is started but not moved, dont process zoom operation
            return;
        }
        stateChangeListener(chartObj, systemconf.chart.axes.events, null, "init");
        if (zoomedX) {
            var datatype = chartObj.dataset.getDataType("x");
            if (!rangeDefined) {
                var xscale = chartObj.axes.x.scale,
                    axisReversed = systemconf.chart.axes.xaxis.reversed;
                var x1 = $ZC.scale.invert(datatype, zmdata.startPt, xscale, axisrotated, "x", axisReversed),
                    x2 = $ZC.scale.invert(datatype, zmdata.endPt, xscale, axisrotated, "x", axisReversed);
                if (x1 > x2) {
                    var temp = x1;
                    x1 = x2;
                    x2 = temp;
                }
                if (datatype == ORDINAL_DATATYPE) {
                    xdomain = xscale.domain().slice(x1, x2 + 1, 0);
                } else {
                    xdomain = [x1, x2];
                }
            }
            updateZoom("x", (rangeDefined ? seriesidx : 0), xdomain, datatype);
        }
        if (zoomedY) {
            var i = (rangeDefined ? seriesidx : 0),
                len = (rangeDefined ? seriesidx + 1 : chartObj.axes.y.groups.length);
            for (; i < len; i++) {
                var datatype = chartObj.dataset.getDataType("y", i);
                if (!rangeDefined) {
                    var yscale = chartObj.axes.y.groups[i].scale,
                        axisReversed = systemconf.chart.axes.yaxis[i].reversed;
                    var y1 = $ZC.scale.invert(datatype, zmdata.startPt, yscale, axisrotated, "y", axisReversed),
                        y2 = $ZC.scale.invert(datatype, zmdata.endPt, yscale, axisrotated, "y", axisReversed);
                    if (y1 > y2) {
                        y1 = y1 + y2 - (y2 = y1);
                    }
                    if (datatype == ORDINAL_DATATYPE) {
                        ydomain = yscale.domain().slice(y1, y2 + 1, 0);
                    } else {
                        ydomain = [y1, y2];
                    }
                }
                updateZoom("y", i, ydomain, datatype);
            }
        }
        stateChangeListener(chartObj, systemconf.chart.axes.events, null, "onreload");
        
        chartObj.updatePlotareaProp(); //Test_548.json
        chartObj.plot.renderer.draw(true);
        chartObj.eventHandler.updateParams(chartObj); //Need to update the event params. Becoz axes.eventscale is a new scale now
        
        if (!zmevent.clipMarker) {
            //On zooming, update the plot area clip path once (using clipMarker boolean) which doesn't consider the marker size
            chartObj.drawPlotareaClippath();
            zmevent.clipMarker = true;
        }
        
        chartObj.drawThreshold(true);
        chartObj.staticPaneSVG.selectAll(".zmbtn").style("visibility", "visible");
        
        // call the zoom end callback here
        invokeFunction(true, callbacks.zoomend, endPt, {
            "x": zmConf.x,
            "y": zmConf.y
        }, chartObj);
    };
    
    zmevent.reset = function() {
        delete zmConf.x;
        delete zmConf.y;
        
        stateChangeListener(chartObj, systemconf.chart.axes.events, null, "init");
        zmtype = zmConf.type
        if (zmtype.indexOf("x") > -1) {
            systemconf.chart.axes.xaxis.disableBaseValue = false;
            chartObj.updateScale(seriesdata[0], "x", null);
            
        }
        if (zmtype.indexOf("y") > -1) {
            for (var i = 0; i < chartObj.axes.y.groups.length; i++) {
                systemconf.chart.axes.yaxis.disableBaseValue = false;
                chartObj.updateScale(seriesdata[i], "y", null);
            }
        }
        stateChangeListener(chartObj, systemconf.chart.axes.events, null, "onreload");
        
        //Reset the plot area clip path as soon as chart rendered, so that the marker size will be considered
        chartObj.updatePlotareaProp(); //Test_548.json
        chartObj.plot.renderer.draw(true);
        chartObj.eventHandler.updateParams(chartObj); //Need to update the event params. Becoz axes.eventscale is a new scale now
        chartObj.drawThreshold(true);
        var rendered = setInterval(function() {
            if (chartObj.chartRendered) {
                chartObj.drawPlotareaClippath();
                zmevent.clipMarker = false;
                clearInterval(rendered);
            }
        }, 10);
        chartObj.staticPaneSVG.selectAll(".zmbtn").style("visibility", "hidden");
        invokeFunction(callbacks.reset, chartObj);
    };
    
    function updateZoom(type, idx, domain, datatype) {
        /*
         * disableBaseValue is for not to take the base-
         * value while the server generation plot image.
         */
        systemconf.chart.axes[type + "axis"].disableBaseValue = true;
        var z = zmConf[type] = {};
        if (datatype == ORDINAL_DATATYPE) {
            z.categories = domain;
        } else {
            z.minRange = domain[0];
            z.maxRange = domain[1];
        }
        chartObj.updateScale(seriesdata[idx], type, domain); //For multi Yaxis, dont update the scale if the axis is disabled
    }
    
    function getVal(val, defaultVal) {
        val = invokeFunction(val, chartObj, rectBound);
        val = (val != null) ? val : defaultVal;
        return val;
    }
    
    function getCoordinate(mouse) {
        return {
            x: mathMin(mathMax(mouse[0] - plotarea.left, 0), plotarea.width), //range should be within plotarea dimension ISSUE:ZC429
            y: mathMin(mathMax(mouse[1] - plotarea.top, 0), plotarea.height)
        };
    }
    
    return zmevent;
};

function d3_mousePoint(container, i) {
    var e = d3_eventSource();
    if (e.changedTouches) {
        e = e.touches[i];
    }
    var svg = container.ownerSVGElement || container;
    if (svg.createSVGPoint) {
        var point = svg.createSVGPoint();
        point.x = e.clientX, point.y = e.clientY;
        point = point.matrixTransform(container.getScreenCTM().inverse());
        var mouse = [point.x, point.y];
        if (isTouchEvent(d3.event) && !$Browser.ANDROID && $Browser.uiwebview) { //d3.mouse(node) considers pageXOffset and pageYOffset in android devices
            mouse[0] += window.pageXOffset;
            mouse[1] += window.pageYOffset;
        }
        return mouse;
    }
    var rect = container.getBoundingClientRect();
    return [e.clientX - rect.left - container.clientLeft, e.clientY - rect.top - container.clientTop];
}

function d3_eventSource() {
    var e = d3.event,
        s;
    while (s = e.sourceEvent) {
        e = s;
    }
    return e;
}

function isTouchEvent(event) {
    return event.touches;
}
