/* $Id$ */
var ZDOMUtil = (function(window){
	var elements = undefined;
	return {
		init: function(elementName){
			elements = {
				div: document.createElement('div')	// No I18N
			}
		},
		clone: function(elementName){
			return elements[elementName].cloneNode();
		}
	}
})(window);
var ZComponents = (function($){
	var needToInit = true,
		userAgent = navigator.userAgent.toLowerCase(),
		sourceDiv = document.createElement('div'),
		sourceText = document.createTextNode(" ");
	function getAttrSupport(){
		var ele = document.createElement("div"), supported = false;
		ele.addEventListener("DOMAttrModified", function(){ 
			supported = true; 
		}, false);
		ele.setAttribute("class","zcomponents");
		return supported;
	}
	return {
		version: "v3_1_1_hotfix_1",	// No I18N
		isRTL : false,
		selector: "data-ctype",	// No I18N
		userAgent : userAgent,
		DOMUtil: $,
		locale: navigator.language,
		zIndex : 1000, // ZIndex can be used for components like dialog.
		$document : $(document),
		$window : $(window),
		hasHandlebars : window.Handlebars ? true : false,
		keys : {},
		keyCode : {
			BACKSPACE: 8,
			COMMA: 188,
			DELETE: 46,
			DOWN: 40,
			END: 35,
			ENTER: 13,
			ESCAPE: 27,
			HOME: 36,
			LEFT: 37,
			NUMPAD_ADD: 107,
			NUMPAD_DECIMAL: 110,
			NUMPAD_DIVIDE: 111,
			NUMPAD_ENTER: 108,
			NUMPAD_MULTIPLY: 106,
			NUMPAD_SUBTRACT: 109,
			PAGE_DOWN: 34,
			PAGE_UP: 33,
			PERIOD: 190,
			RIGHT: 39,
			SPACE: 32,
			TAB: 9,
			UP: 38,
			ALT: 18
		},
		initDeferredComponents : function(componentsMap, timeout, callback){
			var base = this;
			setTimeout(function(){
				$.each(componentsMap, function(ctype, componentGroup){
					$(componentGroup).each(function(i, component){
						base._initComponent($(component), ctype);		
					});
				});
				if(callback){
					callback(componentsMap);
				}
			},(timeout ? timeout : 10000)); // Default is 10 seconds
		},
		init : function(container,callback, attr){
			needToInit && this._init();
			container = container ? $(container) : $("body");
			var attr1 = attr ? attr : this.selector,
				attrSelector = "["+ attr1 +"]",	// No I18N
				uiComponents = document.querySelectorAll ? container[0].querySelectorAll(attrSelector) : $(attrSelector, container);
			if(container[0].attributes[attr1] || (container[0].dataset && container[0].dataset[attr1])){
				uiComponents = $(container).add(uiComponents);
			}			
			if(uiComponents.length){
				var base = this;
				this.eachAsync(uiComponents, {
					delay: 20,
					loop: function(i, ele){
						base._initComponent(ele, undefined, attr);
					},
					end: function(){
						base._end(container, uiComponents, callback);
					}
				});
			}else{
				this._end(container, uiComponents, callback);
			}
		},
		destroy: function( container,callback,attr ){
			container = container ? $(container) : $("body");
			attr = attr ? attr : this.selector;
			var attrSelector = "["+ attr +"]", // No I18N
				uiComponents = $(attrSelector, container); 
			if(container.is(attrSelector)){
				uiComponents = $(container).add(uiComponents);
			}
			if(uiComponents.length){
				var base = this;
				this.eachAsync(uiComponents, {
					delay: 20,
					loop: function(i, ele){
						$(ele)[ ele.getAttribute(attr) ]('destroy');
					},
					end: function(){
						base._end(container, uiComponents, callback, true);
					}
				});
			}else{
				this._end(container, uiComponents, callback, true);
			}
		},
		eachAsync : function(array,opts){ // will be modified later
			var i = 0, 
			l = array.length, 
			loop = opts.loop || function(){};
		
			this.whileAsync(
				$.extend(opts, {
					test: function(){ return i < l; },
					loop: function()
					{ 
						var val = array[i];
						return loop.call(val, i++, val);
					}
				})
			);
		},
		whileAsync : function(opts){ // async utility from jquery
			var delay = Math.abs(opts.delay) || 10,
				bulk = isNaN(opts.bulk) ? 500 : Math.abs(opts.bulk),
				test = opts.test || function(){ return true; },
				loop = opts.loop || function(){},
				end  = opts.end  || function(){};
			
			(function executeLoop(){
				var t = false, 
					begin = new Date();
				while( t = test() ){
					loop();
					if( bulk === 0 || (new Date() - begin) > bulk ){
						break;
					}
				}
				if( t ){
					setTimeout(executeLoop, delay);
				}else{
					end();
				}
			})();
		},
		_initComponent : function(element,ctype,attr){
			attr = attr ? attr : this.selector;
			element = $(element);
			element[!ctype ? element.attr(attr) : ctype]();
		},
		_end : function(mainDiv, uiComponents, callback, isdestroy){
			this.$document.trigger(isdestroy?"zcomponentdestroyed" : "zcomponentinitialized", mainDiv); // No I18N
			if(callback){
				callback(uiComponents);
			}
		},
		clearMetadata: function( elements ){
			if( elements ){
				$(elements).removeData('metadata');
			}
		},
		// using element.attributes because element.dataset is not supported in IE 9 and 10.
		_getDataAttributes : function(element, object){
			var data = element.attributes,
				len = data.length;
			for(var i = 0; i < len; i++){
				var name = data[i].nodeName,
					value = data[i].nodeValue === "" ? "true" : data[i].nodeValue ; // No I18N
				if(name !== "data"){
					// data-* attributes and other attributes
					name = name.match(/^data-/) ? name.replace(/^data-/,'') : name; // No I18N
					name = name.replace(/-([a-zA-Z])/g, this._stringReplace.bind(this));   // replacing the hyphenated names to camelCase
					/* modified the optionName since providing data-inner-html attribute in html element will be converted to innerHtml only. */
					/* replace svg and Html as Caps */
					name = name.replace("Html","HTML").replace(/Svg/g,"SVG");	// No I18N
					object[name] = (value === "true" || value === "false") ? (value === "true") : ((value.indexOf("{") !== -1) ? ZComponents._getObject(value) : value);	// No I18N
				}
			}
			return object;
		},
		_stringReplace: function(g){ 
			return g[1].toUpperCase(); 
		},
		_getDataSet : function(element, object){
			var data = element.dataset, value;
			for(var i in data){
				if(data.hasOwnProperty(i)){
				 	/* replace svg and Html as Caps */
					value = data[i];
					i = i.replace(/Html/g,"HTML").replace(/Svg/g,"SVG");	// No I18N
					object[i] = (value === "true" || value === "false") ? (value === "true") : ((value.indexOf("{") !== -1) ? ZComponents._getObject(value) : value);	// No I18N
				}
			}
			return object;
		},
		_getOptions: undefined,
		getMetadata: function(element){
			return this.getOptions(element);
		},
		getOptions: function(element){
			element = element instanceof jQuery ? element[0] : element;
			var object = $.data(element,"metadata");	// No I18N
			if(!object && element.attributes){
				var dataAttr = element.getAttribute('data');	// No I18N
				if(dataAttr){
					// Retrieving options present in data attribute as a json.
					object = ZComponents._getObject(undefined, element);
				}
				object = this._getOptions(element, object ? object : {});
				$.data(element,"metadata",object);	// No I18N
			}
			return object ? object : {};
		},
		getTemplate: function(handleBarsFileName){
			var template;
			if( this.hasHandlebars && Handlebars.templates ){
				template = Handlebars.templates[handleBarsFileName];
			}
			return template;
		},
		_getObject : function(data, element) {
	        if(typeof data === "object"){	// No I18N
	        	return data;
	        }
	        data = data || element.getAttribute('data');	// No I18N
	        data = data.indexOf("{") < 0 ? "{" + data + "}" : data;	// No I18N
	        try{
	        	data = JSON.parse( data.replace(/'/g,'"') ); // No I18N
	        }
	        catch(error){
	        	return $.metadata ? $(element).metadata() : {};
	        }
	        return data;
	    },
		_extend : function(target,base){
			target.prototype = $.extend({},base.prototype,target.prototype);
			$.each(target.prototype,function(prop,value){
				if ($.type(value) === "function") { // No I18N
					target.prototype[ prop ] = (function(){
						var _super = function() {
							return base.prototype[ prop ].apply( this, arguments );
						};
						return function(){
							this._super = _super;
							return value.apply( this, arguments );
						};
					})();
				}
			});
		},
		OS: {
			isLinux: function(){
				return navigator.appVersion.indexOf("Linux") != -1;
			}
		},
		Browser : {
			isIE :  function(){
				return (userAgent.indexOf("msie") !== -1 || userAgent.indexOf(".net") !== -1); // No I18N
			},
			isSafari : function(){
				return (userAgent.indexOf("safari") !== -1 && userAgent.indexOf("chrome") === -1); // No I18N
			},
			isChrome : function(){
				return window.chrome;
			},
			isFirefox: function(){
				return userAgent.indexOf('firefox') > -1 ? true : false;
			},
			isMac: function(){
				return navigator.platform.match(/(Mac|iPhone|iPod|iPad)/i) ? true : false;
			},
			isWindows: function(){
				return navigator.platform.indexOf("Win") !== -1; // No I18N
			},
			getIEVersion : function(){
				var rv = -1; // Return value assumes failure.
				  if (navigator.appName == 'Microsoft Internet Explorer'){ //No I18N
				    var ua = navigator.userAgent;
				    var re  = new RegExp("MSIE ([0-9]{1,}[.0-9]{0,})"); //No I18N
				    if (re.exec(ua) != null){
				      rv = parseFloat( RegExp.$1 );
				    }
				  }
				  return rv;	
			}
		},
		AJAX : {
			transport: function(params, successCallback, failureCallback){
				var request = $.ajax({
					url: params.sourceURL,
					data: params.data,
					type: params.type,
					dataType: params.dataType,
					success: function(data){
						successCallback(data)
					},
					fail: function(error, exception){
						failureCallback(error, exception);
					},
					error: function(error, exception){
						failureCallback(error, exception);
					}
				})
				return request;
			}
		},
		// Component Users can set I18N for all Components at once or individual components separately.
		setI18NKeys : function(moduleName, values, replace){ 
			if(typeof moduleName === "object"){ // No I18N 
				// {"zdialog" : {"close":"Close the Dialog"},"ztooltip":{"close": "Close Text"} }
				for(var i in moduleName){
					this.keys[i] = $.extend({},this.keys[i],moduleName[i]);
				}
			}else{
			// module name is a string. Format: "zdialog",{ "close" : "Close the Dialog"}, true or false
				this.keys[moduleName] = replace ? values : $.extend({},this.keys[moduleName],values);
			}
		},
		getI18NText : function(moduleName, key, stringArray){
			var defaultKeys = undefined;
			if(arguments.length === 4){
				defaultKeys = stringArray;
				stringArray = arguments[3];
			}
			var keyObject = ZComponents.keys[moduleName] || {};
			var ii8nText = keyObject[key] || "";
	        if(ii8nText === "" && defaultKeys){	// No I18N
	            ii8nText = defaultKeys[ key ];           
	        }
	        if(stringArray){
                for (var i = 0; i < stringArray.length; i++) {
                    ii8nText = ii8nText.replace('{'+ i +'}',stringArray[i]);	// No I18N
                }
            }
	        return ii8nText;
		},
		create : function(initObject){
			var componentName = initObject.ctype.replace(initObject.ctype.charAt(0),""), // No I18N
				methodName = "create"+componentName.charAt(0).toUpperCase()+componentName.substring(1); // No I18N
			return this[methodName](initObject);	
		},
		setProperties : function(){
			var doc = document.documentElement || {},
				body = document.body || {};
			ZComponents.documentObject = {
				height: Math.max(doc.clientHeight,doc.scrollHeight,doc.offsetHeight,body.offsetHeight,body.scrollHeight),
				width: Math.max(doc.clientWidth,doc.scrollHeight,doc.offsetHeight,body.offsetWidth,body.scrollWidth)
			};
			ZComponents.windowObject = {
				height: window.innerHeight,
				width: window.innerWidth
			};
		},
		createSVGElement : function(tagName,attributes,childElements) {
    		var element =  document.createElementNS("ht"+"tp://www.w3.org/2000/svg", tagName); // No I18N
    		for (var i in attributes){
                element.setAttribute(i, attributes[i]);
    		}
    		if(childElements){
    			for(var a = 0; a < childElements.length; a++){
    				for(var b in childElements[a]){
    					var childEle = document.createElementNS("ht"+"tp://www.w3.org/2000/svg", b), // No I18N
    						childData = childElements[a][b];
		    			if(childData.nodes){
		    				childEle = this.createSVGElement(b, childData.props, childData.nodes);
		    			}else{
		    				for (var k in childData){
		               			childEle.setAttribute(k, childData[k]);
		    				}
		    			}
    					element.appendChild(childEle);
    				}
    			}
    		}
    		return element;
    	},
		createIcon: function(element, icon, svgId, iconClass, cname){
			icon = svgId ? (svgId.indexOf("#") === -1 ? "#"+svgId: svgId ) : (iconClass ? iconClass : icon); // No I18N
			if(icon){
				if(icon.indexOf("#") === -1){ // No I18N
					element.addClass(" "+icon+""); // No I18N
				}else{
					var svgId = icon.split(" ")[0], // No I18N
						customClass = icon.slice(svgId.length).trim() +" "+(cname ? cname+"__svg": "");	// No I18N
					element.empty().append($("<svg class='"+customClass+"'><use xlink:href='"+svgId+"'></use></svg>"));
				}
			}
			return element;
		},
		_handleIcon: function(iTag, svgData, defaultSVGId, iconClass, svgIconId, v1IconClass, isUpdate, prevIconClass, prevSVGIconId){
			if(isUpdate){
				iTag.removeClass(prevIconClass);
			}
			var iconIsSVG = false;
			if(v1IconClass !== undefined || iconClass !== undefined || svgIconId !== undefined){
				if((v1IconClass && v1IconClass.indexOf("#") === 0) || svgIconId !== undefined){
					var svgIcon = svgIconId || v1IconClass;
					if(svgIcon){
						var iconInfo = svgIcon.split(" ");	// No I18N
						svgIconId = iconInfo[0];
						iconClass = iconInfo[1] || "";
						iconIsSVG = true;
					}
				}
				else{
					iconClass = iconClass || v1IconClass;
				}
			}
			else if(defaultSVGId !== undefined && svgData !== undefined){
				var svgIcon = this._svgSprite.querySelector("#"+defaultSVGId);
				if(!svgIcon){
					var props = svgData.props || {};
					props.id = defaultSVGId;
					var element = this.createSVGElement("symbol", props, svgData.nodes);	// No I18N
					this._svgSprite.appendChild(element);
				}
				svgIconId = "#"+ defaultSVGId;
				iconClass = svgData.props && svgData.props['class'] ? svgData.props['class'] : defaultSVGId;
				iconIsSVG = true;
			}
			if(iconIsSVG){
				svgIconId = svgIconId.indexOf("#") !== -1 ? svgIconId : "#"+svgIconId; // No I18N
				iTag.empty().append($("<svg class='"+ iconClass +"'><use xlink:href='"+ svgIconId +"'></use></svg>"));	// No I18N
			}
			else{
				iTag.addClass(iconClass);
			}
			return iTag;
		},
		encodeHTML : function(str){
			sourceText.nodeValue = str;
			return sourceDiv.innerHTML;
		},
		_init: function(str){
			ZDOMUtil.init();
			var base = ZComponents;
			if(!needToInit){
				return false;
			}
			needToInit = false;
			base._getOptions = ZDOMUtil.clone('div').dataset ? base._getDataSet : base._getDataAttributes;	// No I18N
			base.isDOMAttrModifiedSupported = getAttrSupport();
			sourceDiv.appendChild(sourceText);
			if(base.Browser.isIE() && base.isDOMAttrModifiedSupported){				
				base.core._attachObserver = function(element){
					var base = this;
					this.observer = function(e){
						if(base.attributeList.indexOf(e.attrName) !== -1){
							// DOMAttrModified triggered for each and every property change.
							base._handleObserver(e.attrName, e.newValue);
						}
					}
					element.addEventListener("DOMAttrModified", this.observer, false);
				};
				base.core.destroy = function(){
					this.element[0].removeEventListener("DOMAttrModified", this.observer);
				};
			};
			base.setProperties();
			base._svgSprite = base.createSVGElement('svg',{id:"zcomponents__svg"});	// No I18N
			$(base._svgSprite).appendTo('body').hide();
		},
		registerComponent: function(componentName, parentComponent,  prototype){
			if(!prototype){
				prototype = parentComponent;
				parentComponent = this.core;
			}
			prototype.componentName = componentName;
			$[componentName] = function(options, element, userOptions){  // creating the constructor method for child.
				if(arguments.length){
					this._init(options, element, userOptions);
				}
			}
			var baseClass = new parentComponent();
			baseClass.options = $.extend( true, {}, baseClass.options );
			$[componentName].prototype = $.extend(true, {}, baseClass, prototype);
			if(!this[componentName]){
				this[componentName] = {
					DEFAULTS: {}
				};
			}
			this[componentName]._DEFAULTS = $[componentName].prototype.options;
			$.fn[componentName] = function(options){ // Assigning the childComponent to global scope... [ Alternative to JQuery's Method ]
				needToInit && ZComponents._init();
				var returnValue = this,
					length = this.length;
				if(!options || typeof options === "object"){ // component is initialized
					options = options ? options : {};
					for(var i = 0; i < length; i++){
						var componentInstance = $(this[i]).data(componentName);
						if(componentInstance){
							return returnValue;
						}else{
							var metadata = ZComponents.getOptions(this[i],{});
							if(metadata.override){ // handling the case where two or more components share the same set of attributes.
								var newArray = {};
								for(var obj in metadata){
									if(obj.indexOf(componentName) === 0){
										var result = obj.replace(componentName,""), // No I18N
											keyName = result[0].toLowerCase()+result.substring(1);
										newArray[keyName] = metadata[obj];
									}
								};
								metadata = newArray;
							}
							var data = $.extend(true, {},ZComponents[componentName]._DEFAULTS, ZComponents[componentName].DEFAULTS, metadata, options);
							$(this[i]).data(componentName, new $[componentName](data, this[i], options));
							$(this[i]).data("componentName",componentName);	// No I18N
						}
					}
				}else if(typeof options === "string" && options.indexOf("_") === -1){ // No I18N
					// Public Method Invocation
					var params = Array.prototype.slice.call(arguments,1);
					for(var i = 0; i < length; i++){
						var componentInstance = $(this[i]).data(componentName);
						if(componentInstance){
							if(options === "option"){ // No I18N
								options = params && params.length === 2 ? "setAttribute" : "getAttribute"; // No I18N
							}else if(options === "options"){ // No I18N
								options = params && params.length === 0 ? "getAttributes" : "setAttributes"; // No I18N
							}
							returnValue = componentInstance[options].apply(componentInstance, params);
						}
					}
				}else{
					return false;
				}
				return returnValue;
			}
			this._extend($[componentName], parentComponent);
		}
	}
})(jQuery);
(function(ZC){
    var $ = ZC.DOMUtil;
	ZC.core = function(options, element){
		if(arguments.length){
			this._init(options, element);
		}
	}
	ZC.core.prototype = {
		_init: function(options, element, userOptions){
			this.element = $(element);
			this.options = options;
			// this._attachObserver(element);
			this._create($(element), options, userOptions);
		},
		_create: function(){},
		_trigger: function(eventName, event, data, element, isFromUI){
			return this._dispatchEvent(eventName,event, data, element, isFromUI);
		},
		_dispatchEvent: function(eventName, event, data, element, isFromUI){
			var type = this._EVENTS[eventName] || eventName;
			element = element ? element : this.element;
			var prop, orig,
				callback = this.options[ type ], value;

			data = data || {};
			data.fromUI = isFromUI;
			data.fromui = isFromUI;
			data.element = element;
			data.options = this.options;

			var event = $.Event( event );
			event.type = (this.componentName ? (this.componentName + type) : ""+type).toLowerCase(); // No I18N
			// the original event may come from any element
			// so we need to reset the target on the new event
			event.originalTarget = event.target; // saving the original target.
			event.target = element;
			orig = event.originalEvent;
			if ( orig ) {
				for ( prop in orig ) {
					if ( !( prop in event ) ) {
						event[ prop ] = orig[ prop ];
					}
				}
			}
			element.trigger( event, data );
			data = [ event ].concat( data );
			if(callback && typeof callback === "string"){ // No I18N
				value = this._triggerFunction(callback, element[0], data);
			}else{
				value = $.isFunction(callback) ? callback.apply(element, data) : undefined;
			}
			return value !== undefined ? value : !event.isDefaultPrevented();
		},
		_handleIcon: function(iTag, svgData, defaultSVGId, iconClass, svgIconId, v1IconClass, isUpdate, prevIconClass, prevSVGIconId, svgIconName){
			svgIconName = svgIconName || (this.componentName +"__"+ defaultSVGId);
			return ZC._handleIcon(iTag, svgData, svgIconName , iconClass, svgIconId, v1IconClass, isUpdate, prevIconClass, prevSVGIconId);
		},
		_triggerFunction: function(functionName, context, params){ 
			// Invoking functions which are passed as strings in HTML Elements.
			var namespaces = functionName.split("."),executionContext = window;
		    for(var i = 0; i < namespaces.length && executionContext; i++){
		        executionContext = executionContext[namespaces[i]];
		    }
		    return executionContext && executionContext.apply(context, params);
		},
		_handleObserver: function(attrName, attrValue){
			var formattedAttribute = attrName.replace(/^data-/,''); // No I18N
			formattedAttribute = formattedAttribute.replace(/-([a-zA-Z])/g, function(g){ 
				return g[1].toUpperCase(); // replacing the hyphenated names to camelCase
			});
			this.setAttribute(formattedAttribute, attrValue);
		},
		_attachObserver: function(element){
			var base = this;
			this.observer = new MutationObserver(function(props){
				var attrName = props[0].attributeName,
					attrValue = props[0].target.getAttribute(attrName);
				base._handleObserver(attrName, attrValue);
			});
			this.observer.observe(element,{
				attributes: true,
				attributeFilter: this.attributeList
			});
		},
		getData: function(element){
			return ZC.getOptions(element ? element : this.element[0]);
		},
		_setAttribute: function(){
			return true;
		},
		setAttribute: function(optionName, value){
			// calling "option" method with key and value sets the option with key to the value specified.(Setter Method)
			// This method inturn calls the "option" method of components to reflect the change immediately.
			// Example: element.zbutton("option","text","Save");
			if(typeof this.options[optionName] === "string" && this.options[optionName] === value){  // No I18N
			// checking whether the value is actually modified or not.
				return;
			}
			var valueChanged = this._setAttribute(optionName, value);
			if(optionName === "disable"){
				this._handleDisable(value);
				valueChanged = true;
			}
			if(valueChanged){
				this.options[optionName] = value;
			}
		},
		setAttributes: function(options){
			// calling "setAttributes" method will sets one or more options.(Setter Method) 
			// Example: element.zbutton("setAttributes",{"text":"Save","disabled":true});
			for(var i in options){
				this.setAttribute(i, options[i]); 
			}
		},
		getAttribute: function(optionName){
			// calling "getAttribute" method will return the value of the key.(Getter Method)
			// Example: element.zbutton("getAttribute","disabled");
			return this.options[ optionName ];
		},
		getAttributes: function(){
			// calling "getAttributes" method will return the entire options object.(Getter Method)
			// Example: element.zbutton("getAttributes")
			return $.extend(true, {}, this.options); // extending in order to avoid object overriding.
		},
		_localize: function(key, stringArray, optionKey){
			var	defaultsKeys = (this.options.labels || this.options.messages),
				i18nText = optionKey ? this.options[optionKey][key] : undefined;
			return i18nText || this._getI18N(this.componentName, key, defaultsKeys, stringArray);
		},
		_getI18N: function(moduleName, key, defaultKeys, stringArray){
			var keyObject = ZC.keys[moduleName] || {};
			var ii8nText = keyObject[key] || "";	// No I18N
			if(ii8nText === "" && defaultKeys){	// No I18N
	            ii8nText = defaultKeys[ key ];
	        }
	        if(stringArray){
                for (var i = 0; i < stringArray.length; i++) {
                    ii8nText = ii8nText.replace('{'+ i +'}',stringArray[i]);	// No I18N
                }
            }
	        return ii8nText;
		},
		enable: function(){
			this._handleDisable(true);
		},
		disable: function(){
			this._handleDisable(false);
		},
		_handleDisable: function(enable){
			var ele = $(this.container || this.element);
			ele[enable ? "addClass" : "removeClass"]("is-disabled"); // No I18N
			enable ? ele.attr({"aria-disabled": enable, "disabled": enable}) : ele.removeAttr("aria-disabled disabled");	// No I18N
			this.setAttribute( 'disabled', !enable ); //No I18N
		},
		getElement: function(){
			return (this.container || this.element);
		},
		_destroy: function(){},
		destroy: function(){
			// this.observer.disconnect();
			// this.observer = undefined;
			this.element.off("."+this.eventPrefix);	//No I18N
			this.element.removeData(this.componentName).removeData('metadata');	// No I18N
			this._destroy();
			var ele = this.container;
			if(ele && ele[0] !== this.element[0]){ // sometimes element will be stored in base element
				ele.remove();
			}
		}
	};
	var bindedElements = [];
	var closeHandlerDatas = [];
	var getHandlerData = function( element ){
		return closeHandlerDatas[ bindedElements.indexOf( element ) ];
	}
	ZC.bindDocumentEvents = function( options ){
		var base = this;
		var uniqueNamespace = '.zclosehandler-'+( Math.floor( Math.random() * 1000000 ) ); //No I18N
		var ctype = options.ctype;
		var element = $(options.element);
		var container = $(options.container || options.element);
		var forElement = $(options.forElement);
		var closeFn =  options.closeMethodName;
		var metadata = {};
		var data = getHandlerData( element[0] );
		if( data && forElement.is( data.forElement ) ){
			return;
		}
		ZC.unBindDocumentEvents( element );
		bindedElements.forEach( function( bindedEle ){
			var data = getHandlerData( bindedEle );
			if( forElement.closest( data.contextElements ).length ){
				data.contextElements = data.contextElements.add( forElement ).add( container );
			}
		});
		bindedElements.push( element[0] );
		metadata.contextElements = container.add( forElement );
		metadata.uniqueNamespace = uniqueNamespace;
		metadata.forElement = forElement;
		closeHandlerDatas.push( metadata );
		element.on( ctype+closeFn + '.zclosehandler', function(){ //No I18N
					ZC.unBindDocumentEvents( element );
				});	
		ZC.$document.on('mousedown'+uniqueNamespace+' contextmenu'+uniqueNamespace, function( origEvent ){//No I18N
			var metadata1 =  getHandlerData( element[0] );
			if( !metadata1 || !$.contains( document.documentElement, element[0] ) ){
				ZC.unBindDocumentEvents( element );
				return;
			}
			if( metadata1.uniqueNamespace !== metadata.uniqueNamespace ){
				return;
			}
			var target = $( origEvent.target );
			if( target.closest( metadata.contextElements ).length ){
				return;
			}
			element[ctype]( closeFn );
		}); 
	}
	ZC.unBindDocumentEvents = function( element ){
		var index = bindedElements.indexOf( element[0] );
		if( index !== -1 ){
			var data = closeHandlerDatas[ index ];
			element.off( '.zclosehandler' ); //No I18N
			ZC.$document.off( data.uniqueNamespace );
			bindedElements.splice( index, 1 );
			closeHandlerDatas.splice( index, 1 );
		}
	};
}(ZComponents));
document.addEventListener("DOMContentLoaded", ZComponents._init); // No I18N
window.addEventListener("resize", ZComponents.setProperties); // No I18N
/* $Id$ */
var ZPosition = (function($, $window, $document){
	var target, element, targetHeight, targetWidth, targetOffset, options, within, elemWidth,
	elemHeight, elemOffset, availableHeight, availableWidth, win , element, callout, recalculated, hiddenParents, prevStyles, correctionOffset;
	var defaultOptions = {
		direction: 'top-left', //No I18N
		showWithinTarget: false,
		within: $window,
		arrow:{
			minSpace:0,
			margin:{
				left:0,
				top:0
			},
			callback:undefined
		},
		callbacks:{
			dimensionModifier: $.noop,
			directionModifier: $.noop
		},
		customDimensions: false,
		positionAlterable: 'flipallfit' //'flipside'|flipsidefit'|flipall'|flipfit'|'fit'|false //No I18N
		/*newly added flipside, flipsideallfit*/
	};
	var calloutPositions = {
		'top-left' : 'bottom-right',	//No I18N
		'top-right' : 'bottom-left',	//No I18N
		'bottom-left' : 'top-right',	//No I18N
		'bottom-right' :'top-left',	//No I18N
		'top' :'bottom',	//No I18N
		'bottom' :'top',	//No I18N
		'left' :'right',	//No I18N
		'right' : 'left',    //No I18N
		'left-top' : 'right-bottom',	//No I18N
		'left-bottom' : 'right-top',	//No I18N
		'right-top' : 'left-bottom',	//No I18N
		'right-bottom' : 'left-top'	//No I18N
	};
	var regex = {
		relDirections: /^(top|top-right|top-left|bottom|bottom-right|bottom-left|left|left-top|left-bottom|right|right-top|right-bottom|top-left-corner|top-right-corner|bottom-left-corner|bottom-right-corner)$/,
		withinDirections: /^(top|bottom|left|right|top-left|top-right|bottom-left|bottom-right|center)$/,
		fit: /^(fit|flipfit|flipallfit|flipsidefit|flipsideallfit|true)$/,
		flip: /^(flip|flipfit|flipall|flipallfit|flipside|flipsidefit|flipsideall|flipsideallfit|true)$/,
		flipside : /^(flipside|flipsidefit|flipsideall|flipsideallfit|true)$/,
		flipall: /^(flipall|flipallfit|flipsideall|flipsideallfit|true)$/
	};
	function getCorrectionOffset(){
		var $body = $('body'); //No I18N
		if( correctionOffset ){
			return correctionOffset;
		}
		correctionOffset = {left:0, top: 0};
		//Correction in mobile browsers. --> Issue : https://github.com/jquery/jquery/issues/3187
		if( /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent) ) {
			var temp = $('<span>').css({
				position:'absolute', //No I18N
				height:'20px', //No I18N
				width:'20px', //NO I18N
				top:'0px', //No I18N
				left:'0px' //No I18N
			}).appendTo( $body );
			correctionOffset = temp.offset();
			correctionOffset.left = correctionOffset.left - parseFloat( $body.css('margin-left') ) - parseFloat( $body.css('border-left-width') ) ; //No I18N 
			correctionOffset.top = correctionOffset.top - parseFloat( $body.css('margin-top') ) - parseFloat( $body.css('border-top-width') ) ; //No I18N
			temp.remove();
		}
		return correctionOffset;
	}
	function getDimensions(element){
		if( element.preventDefault ){
			return{
				width:0, height:0, offset: {left: element.pageX, top: element.pageY}
			}
		}
		var element = $(element);
		if( element.nodeType === 9 ){
			return{
				width: element.width(), height: element.height(), offset: {left:0, top:0}
			}
		}
		if( $.isWindow(element) ){
			return{
				width: element.width(), height: element.height(), offset: {left: element.scrollLeft(), top: element.scrollTop()}
			}
		}
		var dimension = {
			width: getDimension( element[0], 'outerWidth' ), //No I18N
			height: getDimension( element[0], 'outerHeight' ), //No I18N
			offset: getOffset( element )
		}
		return dimension;
	}
	function getOffset( element ){
		// element.addClass('no-transform');
		var offset = element.offset();
		// console.log( offset );
		// var styleset = window.getComputedStyle( element[0], null );
		// console.log('transform', styleset.transform)
		// element.removeClass('no-transform');
		// element[0].style.transform = transform;
		var excessoffset = getCorrectionOffset();
		offset.left -= excessoffset.left;
		offset.top -= excessoffset.top;

		// console.log( 'ofset ', offset );
		//Incase it is transformed
		if( (ZComponents.Browser.isFirefox() || ZComponents.Browser.isChrome() ||  ZComponents.Browser.isSafari()) && window.WebKitCSSMatrix ){
			if( ZComponents.Browser.isSafari() ){
				var version = /version\/(\d+)/.exec( ZComponents.userAgent );
				if( version ){
					version = parseFloat( version[1] );
					if( version < 11 ){
						return offset;
					}
				}
			}
			var styleset = window.getComputedStyle( element[0], null );
			if( styleset.webkitTransform && styleset.webkitTransform !== 'none' ){ //No I18N
				 // console.log( styleset.transform );
				 var curTransform = new  WebKitCSSMatrix(window.getComputedStyle(element[0]).webkitTransform);
				 // console.log( curTransform );
				 offset.left -= curTransform.m41;
				 offset.top -= curTransform.m42;
			}	
		}
		return offset;	
	}
	/* Direction if alterable is fixed based on within element*/
	function fixDirection(direction){
		var dir = direction.split('-'); //No I18N
		var changedir = false;
		/* To filp in case of insufficient space
			top-left =>  bottom-left
			bottom-left => top-left
			left-top => right-top
			right-top => left-top
		 */
		if(/^(top|bottom)$/.test(dir[0]) || /^corner$/.test(dir[2])){
			callout && callout.callback.call(this, callout.element, calloutPositions[dir[0]]);
			var collisionWidth = elemWidth + ( callout ? (callout.width ? callout.width : callout.element.outerWidth()) : 0 );
			var collisionHeight = elemHeight + ( callout ? (callout.height ? callout.height : callout.element.outerHeight()) : 0 );
			var dirInfo = flip.top(direction, collisionHeight);
			direction = dirInfo.direction;
			if(dirInfo.overTop > 0 && regex.flipall.test(options.positionAlterable)){
				var	newDirInfo = flip.left("right", collisionWidth);	//No I18N
				/* check - if space in top/bottom is inefficient left/right space is checked, if available left/right position is taken, else top/bottom position is fixed */
				if(newDirInfo.overLeft <= 0){
					direction = newDirInfo.direction;
				}
			}
		}
		dir = direction.split('-'); //No I18N

		if(/^(left|right)$/.test(dir[0]) || /^corner$/.test(dir[2])){
			callout && callout.callback.call(this, callout.element, calloutPositions[dir[0]]);
			var collisionWidth = elemWidth + ( callout ? (callout.width ? callout.width : callout.element.outerWidth()) : 0 );
			var collisionHeight = elemHeight + ( callout ? (callout.height ? callout.height : callout.element.outerHeight()) : 0 );
			var dirInfo = flip.left(direction, collisionWidth);
			direction = dirInfo.direction;
			if(dirInfo.overLeft > 0 && regex.flipall.test(options.positionAlterable)){
				var newDirInfo = flip.top("bottom", collisionHeight);	//No I18N
				/* check - if space in left or right is inefficient, element is displayed in top/bottom */
				direction = newDirInfo.direction;
			}
		}
		return direction;
	}
	/* To filp in case of insufficient space
		top-left =>  top-right
		bottom-left => bottom-right
		left-top => left-bottom
		right-top => right-bottom
	 */
	function fixAlternateDirection(direction){
		var dir = direction.split('-'), //No I18N
		calloutWidthLeft = 0, calloutHeightLeft = 0;
		if(target.preventDefault && callout){
			/* If target is an event, callout must point the mouse pointer, so collisionWidth is reduced*/
			calloutWidthLeft = 2*callout.width + callout.width/2;
			calloutHeightLeft = 2*callout.height + callout.height/2;	
		}
		if(/^(left|right)$/.test(dir[1])){
			// var dirInfo = flip.left(dir[1], elemWidth - targetWidth - calloutHeightLeft);
			var dirInfo = flip.left(dir[1], elemWidth - (dir[0]==='corner'? 0: targetWidth) - calloutHeightLeft);
			direction = direction.replace(dir[1],dirInfo.direction);
		}else if(/^(top|bottom)$/.test(dir[1])){
			var dirInfo = flip.top(dir[1], elemHeight - targetHeight - calloutWidthLeft);
			direction = direction.replace(dir[1], dirInfo.direction);
		}
		return direction;
	}
	/* To initiate callout element*/
	function initCallout(elementDir){
		if( options.arrow && options.arrow.element ){
			var	callout = {};
			var element = options.arrow.element;
			if(typeof element === 'string' || element.jquery || element.nodeType){	//No I18N
				element = $(element);
				if(element.length){
					callout.element = element;
				}
			}
			if(callout.element && callout.element.css('position') !== 'static'){	//No I18N
				callout.centeredToTarget = (/^(center|undefined)$/.test(options.arrow.direction)) && !target.preventDefault ? true : false;
				callout.callback =  $.isFunction(options.arrow.callback)? options.arrow.callback : ($.isFunction(options.arrowCallback)? options.arrowCallback: $.noop );
				callout.direction = calloutPositions[elementDir];
				callout.callback.call(this, callout.element, callout.direction.split('-')[0]);	//No I18N
				callout.height = (typeof options.arrow.height === 'number' ? options.arrow.height : 0);	//No I18N
				callout.width = (typeof options.arrow.height === 'number' ? options.arrow.width : 0); //No I18N
				callout.minSpace = options.arrow.minSpace;
				callout.margin = options.arrow.margin || {left:0,top:0};
				return callout;	
			}else{
				options.arrow = undefined;
			}
		}	
	}
	function getElementHeight( direction ){
		if( elemHeight > getAvailableHeight( options.direction ) ){
			return  getAvailableHeight( options.direction );
		}else{
			return elemHeight;
		}
	}
	function getElementWidth(){
		if( elemWidth > getAvailableWidth( options.direction ) ){
			return  getAvailableWidth( options.direction );
		}else{
			return elemWidth;
		}
	}
	function fitCalloutPosition( callout, calloutP ){
		var dir = callout.direction.split('-'); //No I18N
		var maxValue, tempDir = dir[1], minValue = 0;
		var elemWidth1 = getElementWidth();
		var elemHeight1 = getElementHeight();
		if( /^(left|right)$/.test( dir[1] ) ){ 
			maxValue = elemWidth1 - callout.width;
		}else if( /^(top|bottom)$/.test( dir[1] ) ){ 
			maxValue = elemHeight1 - callout.height;
		}else{
			if( /^(top|bottom)$/.test( dir[0] ) ){
				maxValue = elemWidth1 - callout.width;
				tempDir = 'left'; //No I18N
			}else if( /^(left|right)$/.test( dir[0] ) ){
				maxValue = elemHeight1 - callout.height;
				tempDir = 'top'; //No I18N
			}
		}	
		if( calloutP[tempDir] > maxValue ){
			calloutP[ tempDir ] = maxValue;
		}
		if( calloutP[tempDir] < minValue ){
			calloutP[ tempDir ] = minValue;
		}
	}
	/* Function to get the callout position `for a given direction*/
	function getCalloutPosition( callout , correctionPixel ){
		callout.element.css({left:'auto', top:'auto', right:'auto', bottom:'auto'}); //No I18N
		var pos = {};
		var dir = callout.direction.split('-'); //No I18N
		if( /^(top|bottom)$/.test( dir[0] ) ){
			pos[ dir[0] ] = -(callout.height);	
			if( !dir[1] ){
				pos.left = elemWidth/2 - callout.width/2 - correctionPixel.left;
			}
		}else if(/^(left|right)$/.test( dir[0] )){
			pos[dir[0]]  = -(callout.width);
			if( !dir[1] ){
				pos.top = elemHeight/2 - callout.height/2 - correctionPixel.top;
			}
		}
		if( /^(left|right)$/.test( dir[1] ) ){
			var cPix = ( correctionPixel.left );
			if( dir[1] === 'left' ){ //No I18N	
				cPix = -( cPix );
			}
			if( targetWidth && elemWidth > targetWidth ){
				pos[dir[1]] = targetWidth/2 - callout.width/2 + cPix;
			}else{
				pos.left = elemWidth/2 - callout.width/2;
			}
		}else if( /^(top|bottom)$/.test( dir[1] ) ){
			var cPix = 	( correctionPixel.top );
			if( dir[1] === 'top' ){ //No I18N
				cPix = -( cPix );
			}
			if( targetHeight && elemHeight > targetHeight ){
				pos[dir[1]] = targetHeight/2 - callout.height/2 + cPix;
			}else{
				pos.top = elemHeight/2 - callout.height/2;
			}
		}
		var doesHaveBoxShadow = element.css('box-shadow') !== 'none'; //No I18N
		/*This correction is done because the pointer always sits on the element rather than attached to the element*/
		if( callout.direction.split("-")[0] === "top" ){ //No I18N
			pos.top =  pos.top + ( ( parseFloat( element.css('border-top-width') ) || doesHaveBoxShadow )? 1:0); //No I18N
		}else if( callout.direction.split("-")[0] === 'bottom' ){ //No I18N
			pos.bottom = pos.bottom +( ( parseFloat( element.css('border-bottom-width') ) || doesHaveBoxShadow )? 1:0 ); //No I18N
		}else if( callout.direction.split("-")[0] === 'right' ){ //No I18N
			pos.right = pos.right + ( ( parseFloat( element.css('border-right-width') ) || doesHaveBoxShadow )? 1:0); //No I18N
		}else if( callout.direction.split("-")[0] === 'left' ){ //No I18N
			pos.left = pos.left + ( ( parseFloat( element.css('border-left-width') ) || doesHaveBoxShadow )? 1:0);  //No I18N
		}
		return pos;
	}
	var flip = {
		/* Filp north south position incase of insufficient height in the given direction */
		top: function(direction, collisionHeight){
			var dir = direction.split('-')[0], 
				response = {direction: direction},
				availHeight = getAvailableHeight(direction);
			response.overTop = collisionHeight -  availHeight;
			if(response.overTop > 0){
				var newdir = dir === 'top' ? 'bottom' : 'top'; //No I18N
				var newAvailableHeight = getAvailableHeight(direction.replace(dir, newdir));
				if(newAvailableHeight > availHeight){
					response.direction = direction.replace(dir, newdir);
					response.overTop = collisionHeight - newAvailableHeight;
				}
			}
			return response;
		},
		/* Flip the left right position incase of insufficient width in the given direction*/
		left: function(direction, collisionWidth){
			var dir = direction.split('-'), response = {direction: direction}, finalDir, newdir;
			var availWidth = getAvailableWidth(direction);
			response.overLeft = collisionWidth -  availWidth;
			if(response.overLeft > 0){
				finalDir = dir[0];
				if(dir[2] === 'corner'){
					finalDir = dir[1];
				}
				newdir = finalDir === 'left' ? 'right' : 'left';	//No I18N
				var newAvailableWidth = getAvailableWidth( direction.replace(finalDir, newdir) );
				if(newAvailableWidth > availWidth){
					response.direction = direction.replace(finalDir, newdir);
					response.overLeft = collisionWidth - newAvailableWidth;
				}
			}
			return response;	
		}
	}
	/* Fit the element position based on the viewport*/
	function fitPosition( position ){
		var newPosition = $.extend({}, position),
		direction = options.direction,
		collisionWidth = elemWidth + callout ? callout.width : 0,
		collisionHeight = elemHeight + callout ? callout.height : 0,
		dir = direction.split('-');	//No I18N
		dir[1] = dir[1] === undefined? (/^(top|bottom)$/.test(dir[0])? 'left' : 'top') : dir[1];	//No I18N
		var offset = within.isWindow? {left: win.scrollLeft, top: win.scrollTop} : within.offset;
		if(/^(top|bottom)$/.test(dir[1]) || target.preventDefault){
			var overTop = within.scrollTop - newPosition.top;
			if( overTop>0 ){
				newPosition.top += overTop;
			}
			var overBottom = ( newPosition.top + elemHeight) - (within.scrollTop + getAvailableHeight().complete);
			if(overBottom > 0){
				newPosition.top -= overBottom;
			}
		}
		if(/^(left|right)$/.test(dir[1]) || target.preventDefault){
			var overLeft = within.scrollLeft - newPosition.left;
			if( overLeft>0 ){
				newPosition.left += overLeft;
			}
			var overRight = ( newPosition.left + elemWidth) - (within.scrollLeft + getAvailableWidth().complete)+1;
			if(overRight > 0){
				newPosition.left -= overRight;
			}
		}
		return newPosition;
	}
	function calculatePosition(direction){
		var newPosition, position;
		options.direction = direction;
		if( !/corner/.test( options.direction )){
			callout = initCallout( options.direction );
		}
		if( regex.flip.test( options.positionAlterable ) && !recalculated){
			options.direction = fixDirection( options.direction );
		}
		/*Setting Callout properties*/
		if( callout ){
			callout.direction = calloutPositions[ options.direction ];
			callout.callback.call(this, callout.element, callout.direction.split('-')[0]);	//No I18N
			if( !callout.height ){
				callout.height = callout.element.outerHeight();
			}
			if( !callout.width ){
				callout.width = callout.element.outerWidth();
			}
		}
		if( regex.flipside.test(options.positionAlterable) ){
			options.direction = fixAlternateDirection( options.direction );
		}
		position = fixPosition( options.direction );
		newPosition = $.extend( {}, position);
		if(regex.fit.test(options.positionAlterable)){
			newPosition = fitPosition( position );
		}
		if( newPosition.top < within.scrollTop ){
			newPosition.top = within.scrollTop;
		}
		if( newPosition.left < within.scrollLeft ){
			newPosition.left = within.scrollLeft;
		}
		var newdir;
		if( callout && regex.flipall.test( options.positionAlterable ) && !recalculated && !target.preventDefault ){
			/* Calculate whether calloutminspace is available*/
			var CALLOUTMINSPACE = callout.minSpace;
			if( CALLOUTMINSPACE !== undefined ){
				var dir = options.direction.split('-'); //No I18N
				var minElemLeft, minElemTop, maxEleLeft, maxEleTop;
				var elemWidth1 = getElementWidth();
				var elemHeight1 = getElementHeight();
				minElemLeft = targetOffset.left - elemWidth1 + callout.width;
				minElemTop = targetOffset.top - elemHeight1 + callout.height;
				maxEleLeft = targetOffset.left + targetWidth - callout.width;
				maxEleTop = targetOffset.top + targetHeight - callout.height;
				if( /^(top|bottom)$/.test( dir[0] ) ){
					if( newPosition.left < minElemLeft ){
						newdir = 'left'; //No I18N
					}
					if( newPosition.left > maxEleLeft ){
						newdir = 'right'; //No I18N
					}
				}else if( /^(left|right)$/.test( dir[0] ) ){
					if( newPosition.top < minElemTop ){
						newdir = 'top'; //No I18N
					}
					if( newPosition.top > maxEleTop ){
						newdir = 'bottom'; //No I18N
					}	
				}
			}
		}
		if( !newdir || recalculated ){
			var correctedPosition = getCorrectedPosition(newPosition, element);
			if( callout ){
				callout.direction = calloutPositions[options.direction];
				var correctionPixel = {
					left: ( newPosition.left - position.left ),
					top: ( newPosition.top - position.top )
				}
				callout.position = getCalloutPosition( callout, correctionPixel );
				fitCalloutPosition( callout, callout.position );
			}	
		}else{
			recalculated = true;
		 	correctedPosition = calculatePosition( newdir );
		}
		return correctedPosition;
	}
	/* Function to get the available height in the given direction*/
	function getAvailableHeight( direction ){
		if( !availableHeight ){
			if( within.isWindow || within.isDocument ){
				availableHeight = { 
									top: targetOffset.top - within.scrollTop,
									bottom: within.height - ( targetOffset.top + targetHeight - within.scrollTop ),
									complete: within.height
								  }
			}else{
				availableHeight = {};
				if( within.offset.top < win.scrollTop ){
					availableHeight.top = targetOffset.top - win.scrollTop;
				}else{
					availableHeight.top = targetOffset.top - within.scrollTop;
				}
				availableHeight.bottom = ( within.offset.top + within.height ) - ( targetOffset.top + targetHeight );
				var diffHeight = ( win.height + win.scrollTop ) - ( within.offset.top + within.height );
				if( diffHeight < 0 ){
					availableHeight.bottom += diffHeight;
				}	
				availableHeight.complete = 	availableHeight.top + availableHeight.bottom + targetHeight;
			}
		}
		if(!direction){
			return availableHeight;
		}
		direction = direction.split('-')[0];	//No I18N
		if(direction === 'top'){	//No I18N
			return availableHeight.top;
		}else if(direction === 'bottom'){	//No I18N
			return availableHeight.bottom;
		}else{
			return availableHeight.complete;
		}	
	}
	/* Function to get the available width in the given direction*/
	function getAvailableWidth( direction ){
		if( !availableWidth ){
			if( within.isWindow || within.isDocument ){
				availableWidth = { 
									left: targetOffset.left - within.scrollLeft,
									right: within.width - ( targetOffset.left + targetWidth - within.scrollLeft ),
									complete: within.width
								  }
			}else{
				availableWidth = {};
				if( within.offset.left < win.scrollLeft ){
					availableWidth.left = targetOffset.left - win.scrollLeft;
				}else{
					availableWidth.left = targetOffset.left - within.scrollLeft;
				}
				availableWidth.right = ( within.offset.left + within.width ) - ( targetOffset.left + targetWidth );
				var diffWidth = ( win.width + win.scrollLeft ) - ( within.offset.left + within.width );
				if( diffWidth < 0 ){
					availableWidth.right += diffWidth;
				}	
				availableWidth.complete = 	availableWidth.left + availableWidth.right + targetWidth;
			}
		}
		if(!direction){
			return availableWidth;
		}
		var dir = direction.split('-'),	//No I18N
			finaldir = dir[2] === 'corner'? dir[1]:dir[0];
		if(finaldir === 'left'){	//No I18N
			return availableWidth.left;
		}else if(finaldir === 'right'){	//No I18N
			return availableWidth.right;
		}else{
			return availableWidth.complete;
		}
	}
	/* Function to get element position for the given direction*/
	function fixPosition(direction){
		direction = direction || options.direction;
		if(!regex.relDirections.test(direction)){
			options.direction = direction = 'top-left';	//No I18N
		}
		var dir = direction.split('-'); //No I18N
		var basePosition = $.extend({}, targetOffset);
		if(dir[0] === 'top'){ //No I18N
			basePosition.top -=  (elemHeight + (callout? callout.height : 0));
		}else if(dir[0] === 'bottom'){ //No I18N
			basePosition.top += (targetHeight + (callout? callout.height : 0));
		}else if(dir[0] === 'left'){ //No I18N
			basePosition.left -= (elemWidth + (callout? callout.width : 0));
		}else if(dir[0] === 'right'){ //No I18N
			basePosition.left += (targetWidth + (callout? callout.width : 0));
		}	
		if(dir[1] === undefined){
			if(dir[0].match(/^left|right$/)){
				basePosition.top = basePosition.top - elemHeight/2 + targetHeight/2; 
			}else{
				basePosition.left = basePosition.left - elemWidth/2 + targetWidth/2;
			}
		}else if(dir[1] === 'top'){	//No I18N
			basePosition.top = basePosition.top + targetHeight - elemHeight;
		}else if(dir[1] === 'left'){	//No I18N
			basePosition.left = basePosition.left - elemWidth + (dir[2] === 'corner'? 0: targetWidth);	//No I18N
		}else if(dir[1] === 'right' && dir[2] === 'corner'){	//No I18N
			basePosition.left = basePosition.left + targetWidth; 
		}
		return basePosition;
	}
	function showHiddenParent( element ){
		element = $(element);
        if( element.is(':hidden') ){ //No I18N
            var i = 1;
            var parent = element;
            while( i && parent && !parent.is('body') ){ //No I18N
                if( parent.css('display') === 'none' ){ //No I18N
                    hiddenParents.push( parent );
                    prevStyles.push( parent.attr('style')||''); //No I18N
                    parent.css({
                        display:'block', //No I18N
                        position:'absolute', //No I18N
                        opacity:1,
                        visibility:'hidden' //No I18N
                    });  
                    i = parent.is(':hidden'); //No I18N
                }
                parent = parent.parent();
            }
        }
    }
    function resetParent(){
        if( hiddenParents && hiddenParents.length ){
            hiddenParents.forEach(function( parent, i ){
                parent.attr('style', prevStyles[i] ); //No I18N
            });
            hiddenParents = [];
            prevStyles = [];
        }
    }
	/* Function to get the corrected element position based on the element positioning and relative parent positioning*/
	function getCorrectedPosition( calculatedOffset, element ){
		var offsetNew = $.extend({}, calculatedOffset);
		var position = element.css('position'); //No I18N
		var curOffset = elemOffset || element.offset();
		var curTopCss = parseFloat( element.css('top') ); //No I18N
		var curLeftCss = parseFloat( element.css('left') ); //No I18N
		if( /^(absolute|fixed)$/.test( position ) && (isNaN( curTopCss ) || isNaN( curLeftCss ) )){
			var p = element.position();
			curTopCss = p.top;
			curLeftCss = p.left;
		}else{
			curTopCss = parseFloat( curTopCss ) || 0;
			curLeftCss = parseFloat( curLeftCss ) || 0;
		}
		offsetNew.top = offsetNew.top - (curOffset.top - parseFloat( curTopCss ) );
		offsetNew.left = offsetNew.left - (curOffset.left - parseFloat( curLeftCss ) );
		return offsetNew;	
	}
	function getScrollBarWidth(){
		var scrollEle = document.createElement("div");
		var styleProps = "width:100px;height:100px;overflow:scroll;";  	 // No I18N
		document.querySelector("body").appendChild(scrollEle);	 // No I18N
		scrollEle.style.cssText = styleProps; 
		var scrollWidth = $(scrollEle).width() - scrollEle.scrollWidth;			
		document.querySelector("body").removeChild(scrollEle);	 // No I18N
		return scrollWidth || 0;
	}
	/* Function to get the dimensions of within container*/
	function getWithinDimensions( element ){
		element = element.length? element : $window;
		var isWindow =  $.isWindow(element[0]);
		var isDocument = !!element[0] && element[0].nodeType === 9;
		var offset = (isWindow || isDocument)? { left:0, top:0 } : (getOffset( element ) || {left:0, top:0});
		var scrollLeft = isWindow || isDocument? win.scrollLeft : offset.left < win.scrollLeft? win.scrollLeft : offset.left;
		var scrollTop = isWindow || isDocument? win.scrollTop : offset.top < win.scrollTop? win.scrollTop : offset.top;
		/* For window, documentElements.clienWidth is used instead of window.innerWidth ==> because
		   When mouse is scrollBar is shown permanently, window.innerWidth gives (width + scrollbarwidth)
		   documentElement.clientWidth gives width excluding scrollbar width. 	
		*/
		var width, height, scrollEle;
		if( isWindow ){
			var d = document.documentElement;
			var w = window;
			width =  w.innerWidth || d.clientWidth;
			height =  w.innerHeight || d.clientHeight;
			/*width = window.innerWidth;
			height = window.innerHeight;*/
			scrollEle = $('body'); //No I18N
		}else if( isDocument ){
			width = document.documentElement.scrollWidth; 
			height = document.documentElement.scrollHeight;
		}else{
			width = element.outerWidth();
			height = element.outerHeight(); 
			scrollEle = element;
		}
		if( scrollEle ){
			var overflowX = scrollEle.css('overflow-x'); //No I18N
			var overflowY = scrollEle.css('overflow-y'); //No I18N
			var hasOverflowX = overflowX === "scroll" || ( (overflowX === "auto" || ( scrollEle.is('body') && overflowX === 'visible') ) && width < scrollEle[0].scrollWidth); //No I18N
			var hasOverflowY = overflowY === "scroll" || ( ( overflowY === "auto" || ( scrollEle.is('body') && overflowY === 'visible') ) && height < scrollEle[0].scrollHeight); //No I18N
			var scrollWidth;
			if( hasOverflowX ){
				height -= ( scrollWidth === undefined? getScrollBarWidth() : scrollWidth );
			}
			if( hasOverflowY ){
				width -= (scrollWidth = getScrollBarWidth() );
			}
		}
		/*
			For window|document, to find with and height, javascript functions are used, becos jquery produces buggy results in some cases
		*/
		return{
			element : element,
			isWindow:  isWindow,
			isDocument: isDocument,
			offset: offset,
			scrollLeft: scrollLeft, 
			scrollTop: scrollTop,
			width: width,
			height: height
		}
	}
	function changeDirection(  direction ){
		if( direction === 'top-left' ){	//No I18N
			direction = 'top-right';	//No I18N
		}else if( direction === 'bottom-left'){	//No I18N
			direction = 'bottom-right';	//No I18N
		}else if( direction === 'top-right'){	//No I18N
			direction = 'top-left';	//No I18N
		}else if( direction === 'bottom-right' ){	//No I18N
			direction = 'bottom-left'	//No I18N
		}
		return direction;
	}
	function getDimension( element, dimension ){
		var value;
		var styles = element.getBoundingClientRect && element.getBoundingClientRect();
		if( styles ){
			switch( dimension ){
				case 'outerWidth':	//No I18N
					value = styles.width;
					break;
				case 'outerHeight':	//No I18N
					value = styles.height;
					break;
			}
		}
		if( isNaN( value ) ){
			value = $(element)[dimension]();
		}
		return  value;
	}
	function getPosition(targetElement, elementPositioned ,opt){
		options = $.extend(true, {}, defaultOptions, opt);  //make a copy of options
		/* target elements dimensions*/
		target = targetElement;
		element = elementPositioned;
		if(!options.showWithinTarget){
			var targetDimension, elementDimension, position, oldDirection = options.direction;
			win = {
				scrollLeft : $window.scrollLeft(), scrollTop : $window.scrollTop(), height: document.documentElement.clientHeight, width: document.documentElement.clientWidth
			},
			/* element to be positioned dimensions*/
			elementDimension = getDimensions( element );
			targetDimension = opt.customDimensions ? target : getDimensions( target );
			
			within = getWithinDimensions($(options.within));
			if( opt.callbacks && typeof opt.callbacks.dimensionModifier === 'function' ){ //No I18N
				opt.callbacks.dimensionModifier( targetDimension, elementDimension, within );
			}

			elemWidth = elementDimension.width;
			elemHeight = elementDimension.height;
			elemOffset = elementDimension.offset;

			targetWidth = targetDimension.width;
			targetHeight = targetDimension.height;
			targetOffset = targetDimension.offset;

			if(!regex.relDirections.test(options.direction)){
				options.direction = 'top-right';	//No I18N
			}else{
				options.direction = changeDirection( options.direction );
			}
			position = calculatePosition(options.direction);
			var response = {
				elementPosition: position || {left:0, top:0}, 
				oldDirection: oldDirection,
				direction: changeDirection( options.direction ),
				arrowPosition: callout? callout.position : {left:0,top:0},
				arrowDirection: calloutPositions[options.direction] && calloutPositions[options.direction].split('-')[0]	//No I18N
			}
			var dir = options.direction.split('-'); //No I18N
			if( /^(top|bottom)$/.test( dir[0] ) ){
				response.availableHeight =  getAvailableHeight( dir[0] ) - (callout? callout.height : 0);
				response.availableWidth = getAvailableWidth( dir[0] );
			}else if( /^(left|right)$/.test( dir[0]) ){
				response.availableHeight =  getAvailableHeight( dir[0] );
				response.availableWidth = getAvailableWidth( dir[0] ) - (callout? callout.width : 0);
			}
			return response;
		}else{
			return getWithinTargetPosition();			
		}
	}
	function getWithinTargetPosition(){
		/*Elements with position 'absolute'|'fixed' are preferred to be  */
		target = $(target);
		var isWindow, position = {}, containerProps = {};
		if( !target.length || ( target[0] === window || target[0].nodeType === 9 /*document*/ ) ){
			target = $(window);
			isWindow = true;
		}
		if( !regex.withinDirections.test(options.direction) ){
			options.direction = 'top-left';	 //No I18N
		}
		var containerProps = { 
							   width : target.outerWidth() - (isWindow ? 0 :  parseFloat(target.css('border-right-width')) + parseFloat(target.css('border-left-width'))), //No I18N
							   height : target.outerHeight() - (isWindow ? 0 :  parseFloat(target.css('border-top-width')) - parseFloat(target.css('border-bottom-width')))	//No I18N
							};
		/*Container Properties*/
		containerProps.offset = isWindow ? {left:window.scrollX, top:window.scrollY} : (getOffset( target ) || {left:0, top:0});
		/*Element Properties*/
		var elementProps = { width:element.outerWidth(), height:element.outerHeight() };
		var centerDirections = ['center','left','right','top','bottom']; // No I18N
		var directionPropertObj = {left:'width',top:'height'}; // No I18N
		for(var dir in directionPropertObj){
			var oppDir = dir === 'left'?'right':'bottom'; // No I18N
			var prop = directionPropertObj[dir];
			if(options.direction.indexOf(dir) !== -1){
				position[dir] = containerProps.offset[dir];
			}else if(options.direction.indexOf(oppDir) !== -1){
				position[dir] = containerProps.offset[dir] + containerProps[prop] - elementProps[prop];
			}else if(centerDirections.indexOf(options.direction) !== -1){
				position[dir] = containerProps.offset[dir] + containerProps[prop]/2 - elementProps[prop]/2;	
			}
		}
		position = getCorrectedPosition(position, element);
		return {
			elementPosition : position
		}
	}
	return {
		get: function(targetElement, elementPositioned, opt){
			if( targetElement && $(elementPositioned).length){
				if( targetElement instanceof jQuery ){
					targetElement = targetElement[0];
				}
				if( elementPositioned instanceof jQuery ){
					elementPositioned = elementPositioned[0];
				}
				if( elementPositioned.nodeType === 1 && ( targetElement.nodeType === 1 || ( opt.showWithinTarget && (targetElement === window || targetElement === document) ) || targetElement.preventDefault || opt.customDimensions ) ){
					hiddenParents = [], prevStyles = [];
					showHiddenParent( targetElement );
					showHiddenParent( elementPositioned );
					opt = opt || {};
					var positionObject = getPosition( targetElement, $(elementPositioned), opt);
					resetParent();
					target = element   = recalculated = targetHeight = targetWidth = callout  = targetOffset   = options  = within = elemWidth = elemHeight = elemOffset = availableHeight = availableWidth = win = correctionOffset = hiddenParents = prevStyles = undefined;
					return positionObject;		
				}
			}
		}
	}
})(jQuery, jQuery(window), jQuery(document));
/* $Id$ */
jQuery(function($){
	// _lastVisited = undefined;
	_lastVisitedMenu = undefined;
	_lastVisitedPopover = undefined;
	$.ZContextMenu = function(ele){
		bindEvents($(ele));
		initScrollWidth();
	}
	var components = {
		zcolorpicker:{
			events:'zcolorpickercolorpanmouseover zcolorpickercolorpanmouseout zcolorpickerbeforeopen zcolorpickeropen zcolorpickerbeforeclose zcolorpickerbeforeotherusedcoloradd zcolorpickerotherusedcoloradd zcolorpickerbeforefocus zcolorpickerfocus zcolorpickerblur zcolorpickerclose', //No I18N
			selectEvent: 'zcolorpickerchange' //No I18N
		}
	}
	$.ZContextMenu.hidingMenu = true;
	bindEvents = function(jele) {
		jele.on("click.zcontextmenu", function(ev, data){ // No I18N
			var tar = $(ev.target);
			if(tar.closest(".zmenu").length || tar.closest(".zpopover__content").length){	// No I18N
				return;
			}
			$.ZContextMenu.isColorpaletteShowing = false;
			$.ZContextMenu.hidingMenu = true;
			setTarget(ev);
			mouseClicked(ev, data);
		});
		jele.on("contextmenu.zcontextmenu", function(ev, data){ // No I18N
			setTarget(ev);
			rightClicked(ev, data);
		});
		$(".zdialog").on("zdialogdragstart.zcontextmenu", function(ev){	// No I18N
			mouseClicked(ev);
		});
		ZComponents.$document.on("zbuttonsetselectionchange", function(e, ui){ // No I18N
			if(canAssociationHappen(ui)){
				handleAssociatedObjects($(e.target), ui);
			}
		}).on("zmenuitemclick", function(e, ui){ //No I18N
			if(canAssociationHappen(ui)){
				handleAssociatedObjects(ui.menuitem);
			}
		}).on("tabselect", function(e, ui){ //No I18N
			if(canAssociationHappen(ui)){
				handleAssociatedObjects(ui.tab);
			}
		}).on("zselectboxitemclick", function(e, ui){ //No I18N
			// select box item click is binded separately since menu item click event will be prevented from select box component.
			if(canAssociationHappen(ui)){
				handleAssociatedObjects(ui.menuitem);
			}
		});
	}
	setTarget = function(ev){
		if(ev.target.correspondingElement) { // If the target is SVGElementInstance object.
			ev.target = ev.target.correspondingElement;	
		}
	}
	function initScrollWidth(){
		var scrollEle = $("<div>").css({"width": "100px", "height": "100px","overflow": "scroll"}).appendTo("body"); 
		var scrollWidth = scrollEle.width()-scrollEle[0].scrollWidth;
		scrollEle.remove();
		$.ZContextMenu.scrollWidth = scrollWidth;
	}
	function canAssociationHappen(data){
		return !data.fromDataBinding;
	}
	function mouseClicked(ev, data){
		if(!data){
			data = {};
		}
		if(ev.button && ev.button != 0){
			return;
		}
		var ele = _getClickedElement(ev);
		if(ele.is("input[type=file]")){	//No I18N
			return;
		}
	    
		if(canAssociationHappen(data)){
			handleAssociatedObjects(ele);
		}
		var res = performMenuAction(ev, "data-menu-id");	// No I18N
		if(!res){ // got a popup...
			return res;
		}
		else{
			var res = performMenuAction(ev, "data-popover-id");	// No I18N
			if(!res){// got a popup...
				return res;
			}
			res = performMenuAction(ev, "toolbar"); // No I18N
			if(!res){
				return res;	
			}
		}
		if( $.ZContextMenu.colorpaletteRegistered ){
			return performMenuAction(ev, "colorpalette"); // No I18N
		}
		return performMenuAction(ev, "data-colorpicker-id"); // No I18N
	}
	function rightClicked(ev, data){
		ev.data = data;
		if(data && data.target){
			ev.target = data.target;
		}
		$(".zmenu").length && $(".zmenu").zmenu("hide");
		return performMenuAction(ev, "data-context",undefined, true); //No i18n
	}
	function _getButtonIfClickedInsideButton(ele){
		if(ele.parent().is("button")){
			return ele.parent();
		}
		return ele;
	}
	function _isObjInArray(processedArray, ele){
		var isInArray = false;
		$.each(processedArray, function(i, pele){
			if(pele == ele[0]){
				isInArray = true;
				return false;
			}
		});
		return isInArray;
	}
	function handleAssociatedObjects(ele, data, processedArray){
		setTimeout(function(){
			_handleAssociatedObjects(ele, data, processedArray);
		}, 10);
	}
	function _handleAssociatedObjects(ele, data, processedArray){
		var click = !data? true: false;
		data = data? data: {};
		var associationAttr = "data-association"; // No I18N
		var selector = "["+associationAttr+"]"; // No I18N

		ele = ele.is(selector)? ele: $(ele.parents(selector).get(0));
		// returning back if disabled for click
		if(!ele.length || ((data && data.disabled == undefined) || ele.is(':disabled'))){ // No I18N
			return;
		}
		if(processedArray){
			if(_isObjInArray(processedArray, ele)){
				return;
			}
		}else{
			processedArray = [];
		}

		processedArray.push(ele[0]);

		var associatedSelectors = ele.attr(associationAttr);
		var associatedObjs = $(associatedSelectors);
		associatedObjs.each(function(i, obj){
			if(obj == ele[0]){// Same Element...
				return;
			}
			if(_isObjInArray(processedArray, $(obj))){
				return;
			}
			obj = $(obj);
			var edata = {"fromDataBinding": true}; // No I18N
			if(typeof data.visible !== undefined){ 
				if(obj.is("input, button") && obj.zbutton("getElement").hasClass("zbutton")){
					obj = obj.parent().hasClass('zbutton--split')? obj.parent(): obj.zbutton('getElement'); // No I18N
					obj[data.visible? 'show': 'hide']();  // No I18N
				}else{
					var menu = obj.closest('.zmenu'); // No I18N
					if(menu.length){
						menu.zmenu("updateMenuItems",['#'+obj.attr('id')],"hidden",(data.visible ? true: false)); // No I18N
					}else{
						obj[data.visible ? 'show': 'hide'](); // No I18N
					}
				}
			}
			if(typeof data.disabled !== undefined){
				if(obj.is('input, button') && obj.zbutton('getElement').hasClass('zbutton')){
					obj.zbutton("option","disabled",data.disabled ? true : false); // No I18N
				}else{
					var menu = obj.closest('.zmenu'); // No I18N
					if(menu.length){
						menu.zmenu("updateMenuItems",['#'+obj.attr('id')],"disabled",(data.disabled ? true: false)); // No I18N
					}else{
						obj[data.disabled? 'disable': 'enable'](false, true, edata); // No I18N
					}
				}
			}
			if(typeof data.checked !== undefined){
				//assuming its a button getElement
				if(ele.is('[checked]') || ele.is('input[type=checkbox]') || ele.is('input[type=radio]')){
					//Extra check to ensure consistant states
					var isChecked = ele.is(':checked'); // No I18N
					if(obj.is('.zmenu__item')){
						obj.data('checked', isChecked); // No I18N
						var menu = obj.closest('.zmenu'); // No I18N
						menu.zmenu("updateMenuItems",[obj],"checked",isChecked); // No I18N
					}else{
						obj.attr('checked', isChecked); // No I18N
					}
				}
				if(obj.is('input, button')){
					if(obj.data('zbuttonset')){	// No I18N
						obj.buttonset('destroy'); // No I18N
						obj.buttonset();
					}
					else if(obj.data('zbutton')) {	// No I18N
						obj.button('destroy');	// No I18N
						obj.button();
					}
				}
			}
			if(click){
				obj.trigger('click', edata); // No I18N
			}

			handleAssociatedObjects(obj, data, processedArray);
		});
	}
	function _getClickedElement(ev){
		var ele = (ev.data && ev.data.target)? ev.data.target: ev.srcElement;
		if(!ele){
			ele = $(ev.target);
		}else{
			var cele = ele.correspondingElement;
			if(cele){
				ele = $(cele);
			}
		}
		return $(ele);
	}
	function stopEvent(ev){
			ev.stopPropagation();
			ev.preventDefault();
	}
	function isDisabled(ele){
		if(ele.is('input,button,textarea,select')){
			return ele.is(':disabled')  // No I18N
		}else{
			return ele.hasClass('is-disabled'); // No I18N
		}
	}
	function performMenuAction(ev, attr, pos, isRightClick){
		var lastMenu = $.ZContextMenu.currentMenu, lastPopover = $.ZContextMenu.currentPopover;
		if(!$.ZContextMenu.hideMenu(ev)){
			return false;
		}
		var ele = _getClickedElement(ev);
		ele = _getButtonIfClickedInsideButton($(ele));
		var obj = traverseAndGetContextElement(ele, attr);
		/* hide opened colorpalette */
		if($.ZContextMenu.colorpaletteRegistered && attr != "colorpalette"){	//No I18N
			$.ZContextMenu.colorpalette.zcolorpalette('hide');	//No I18N
		}
		// {'element' : targetElement, 'attr' :  menu/colorpallette/popover}
		var cmenu = obj.attr;
		ele = obj.element;
		
		if(isDisabled(ele)){	// If the target element is disabled, Do nothing - continue with the next work (return true).
			return true;
		}
		/*Display colorpalette*/
		if(cmenu && attr === 'colorpalette'){	// No I18N
			if(ele.length){
				$.ZContextMenu.displayColorPalette(ele, true);
			}
			return;
		}
		if(cmenu && attr === 'data-colorpicker-id'){	// No I18N
			if( ele.length ){
				$.ZContextMenu.toggleColorPicker( ele );
			}
			return;
		}

		$.ZContextMenu.closePopover(ev);
		if(cmenu){
			var menu = $("#"+cmenu); 
			if(attr === 'toolbar' && !menu.length){ // No I18N
				var toolbar = $(ele.parent().metadata().relatedTabPane);
				if(toolbar.hasClass('ui-ztoolbar-panel')){ 
					menu = toolbar.ztoolbar('getToolbarAsMenu'); // No I18N
					menu.attr('relativeto', 'parent'); // No I18N
					$(ele).attr('data-menu-id', $(ele).attr('toolbar')).removeAttr('toolbar'); // No I18N
				}
			}

			if(lastMenu && lastMenu[0] == menu[0] && this._lastVisitedMenu[0] === ele[0]){	// (Same Menu for Same Button && Same Menu for Different Button)
				if(menu.data("toggle")){
					stopEvent(ev);
					return false;
				}
			}
			this._lastVisitedMenu = ele;
		
			if(attr == 'data-popover-id'){	//No I18N
				menu.off("zpopoverclose.zeventhandler").on("zpopoverclose.zeventhandler", function(orgEvent){  //No I18N
					var target;
					orgEvent = orgEvent.originalEvent;
					if(orgEvent && (target = orgEvent.target)){
						if(target !== $.ZContextMenu._lastVisitedPopover[0]){
							$.ZContextMenu._lastVisitedPopover = undefined;
						}
					}					
				});
				if(lastPopover && lastPopover[0] == menu[0] && $.ZContextMenu._lastVisitedPopover && $.ZContextMenu._lastVisitedPopover[0] === ele[0] ){	// (Same Menu for Same Button && Same Menu for Different Button)
					$.ZContextMenu._lastVisitedPopover = undefined;
					stopEvent(ev);
					return false;
				}
				options = $.extend({}, ZComponents.getOptions(menu));
				if($.ZContextMenu._lastVisitedPopover){
					if($("#"+ $.ZContextMenu._lastVisitedPopover.attr('data-popover-id')).is(":visible")){ // No I18N
						$("#"+ $.ZContextMenu._lastVisitedPopover.attr('data-popover-id')).zpopover( 'close' ); // No I18N
					}
				}
				if($.ZContextMenu._lastVisitedPopover === undefined || $.ZContextMenu._lastVisitedPopover && $.ZContextMenu._lastVisitedPopover[0] !== ele[0]){
					if(!menu.data('zpopover')){	//No I18N
						if(options.forElement === '' || options.forElement === undefined ){	//No I18N
							options.forElement = ele;
						}
						options.content = menu;
						options.popOn = "none"; // No I18N
						menu.zpopover( options );
					}
					menu.zpopover("setAttribute","forElement",ele); // No I18N
					menu.zpopover('open');	//No I18N
					$.ZContextMenu.currentPopover = menu;
					$.ZContextMenu._lastVisitedPopover = ele; 					
				}
			}
			else{
				$.ZContextMenu.showMenu(menu, ele, pos, ev, isRightClick);
				$.ZContextMenu._lastVisitedPopover = undefined;
			}
			stopEvent(ev);
			return false;	// Do not go further, there is a menu/colorpallete/popover to be opened.
		}
		return true; // Nothing found - continue with your job
	}


	function getRelatedMenu(ele, attr){
		var cmenu = ele.metadata();
		cmenu = cmenu[attr];
		cmenu = (cmenu)? cmenu: ele.attr(attr);
		return cmenu;
	}
	function traverseAndGetContextElement(ele, attr){
		var selector = "["+attr+"]";
		ele = ele.is(selector)? ele: $(ele.parents(selector).get(0));
		return {"element": ele, "attr": ele.attr(attr)}; //No i18n
	}
	function getRelativeToEle(menu, targetElement){
		var relativeTo = menu.attr("relativeto"); // No I18N
		if(relativeTo){
			var route = relativeTo.split(".");
			var base = targetElement;
			for(var i=0; i<route.length; i++){
				try{
					var _base = base[route[i]]();
					if(!_base || !_base.length){
						break;
					}
				}catch(e){
					break;
				}
				base = _base;
			}
			if(base){
				targetElement = base;
			}
		}
		return targetElement;
	}
    $.ZContextMenu.highlightItem = function(tar, remove){
    	var meta = ZComponents.getOptions(tar);
		if(tar.is('.zcolorchooser') || (meta.highlightOnMenuShow === false)){
			return;
		}
		var fn = remove? 'removeClass': 'addClass'; // No I18N
		var classNm = meta.activeClass ? meta.activeClass : 'ui-state-highlight';	// No I18N
		var classOp = 'ui-state-open';	// No I18N
		tar[fn](classNm);
		var targetParent = tar.parent();
		if(targetParent){
			targetParent[fn](classOp);
			if(targetParent.is('.zbutton--split')){	// No I18N
				targetParent[fn](classNm);
			}
		}
	}
	$.ZContextMenu.showMenu =  function(cmenu, ele, pos, ev, isRightClick){
		if(!cmenu || !cmenu.length || $(ele).data("zmenubutton")){  // No I18N
			return false;
		}

		var options = {};
		var _data = ZComponents.getOptions(cmenu);
		if(_data){
			options = _data;
		}
		if(!isRightClick){
			cmenu.one("zmenushow.zcontext", function(e, ui){ // No I18N
				if(!ui.skipHighlightTarget){
					$.ZContextMenu.highlightItem($(ui.forElement));
				}
			});
			cmenu.one("zmenuhide.zcontext",function(e,ui){ // No I18N
				if(!ui.skipHighlightTarget){
					$.ZContextMenu.highlightItem($(ui.forElement), true);
				}
			});
		}else{
			options.direction = "after-pointer"; // No I18N
		}

		cmenu.zmenu(options);
		$.ZContextMenu.currentMenu = cmenu;
		cmenu.zmenu("show", { // No I18N
			'currentTarget': ev.target, // No I18N
			'forElement': ele, // No I18N
			'event': ev, // No I18N
            'isRightClick': isRightClick // No I18N
		});

		$.ZContextMenu._isMenuShowing = true;
		$.ZContextMenu.hidingMenu = false;

		cmenu.one("zmenuhide.zcontext", function(e, ui){ // No I18N
			var tar = $(ui.currentTarget).closest('.zbutton').length? $(ui.currentTarget).closest('.zbutton'): $(ui.forElement); // No I18N
			$.ZContextMenu.highlightItem(tar, true);

			$.ZContextMenu.currentMenu = undefined;
			$.ZContextMenu._isMenuShowing = false;
		});
		return true;
	}
	$.ZContextMenu.hideMenu = function(ev){
		if(($.ZContextMenu.currentMenu && $.ZContextMenu._isMenuShowing) || ($.ZContextMenu.colorpalette && $.ZContextMenu.colorpalette.is(':visible')) || ($.ZContextMenu.colorpicker && $.ZContextMenu.colorpicker.is(':visible'))){
			if($.ZContextMenu.currentMenu && $.ZContextMenu.currentMenu.hasClass('zmenu')){
				$.ZContextMenu.currentMenu.zmenu("hide"); // No I18N
			}else{
				$.ZContextMenu.isColorpaletteShowing=true;
			}
			$.ZContextMenu._isMenuShowing = false;
			$.ZContextMenu.currentMenu = undefined;
			if(ev && $.ZContextMenu.consumeEventOnMenuHide){
				ev.stopPropagation();
				ev.preventDefault();
				return false;
			}
		}
		return true;
	}
	
	$.ZContextMenu.currentMenu = undefined;
	$.ZContextMenu._colorpaletteHoveredMenuItem = undefined;
	$.ZContextMenu._isMenuShowing = false;
	$.ZContextMenu.colorpalette = undefined;
	$.ZContextMenu.colorpicker = undefined;
	$.ZContextMenu.windowElement = $(window);
	$.ZContextMenu._documentElement = $(document);
	$.ZContextMenu.windowWidth = $.ZContextMenu.windowElement.width();
	$.ZContextMenu.windowHeight = $.ZContextMenu.windowElement.height();
	$.ZContextMenu.colorpaletteRegistered = false;

	$.ZContextMenu.displayMenuComponents = function(event, ui){
		if($.ZContextMenu._colorpaletteHoveredMenuItem){
			$.ZContextMenu.displayColorPalette(undefined, undefined, false);
			$.ZContextMenu._colorpaletteHoveredMenuItem = undefined;
		}
		var itemData = ui.menuitem.metadata();
		if(itemData.colorpalette){
			var itemObj = $(ui.menuitem);
			var targetEle = $(itemData.relatedButton);
			targetEle = targetEle.data('menuButton') ? targetEle.data('menuButton') : targetEle;	// No I18N
			itemObj.addClass("on-hover");	// No I18N
			var _options = (itemData.colorpaletteOptions)? itemData.colorpaletteOptions: {};
			var selectedColor = $.data($(targetEle)[0], 'selectedColor');	// No I18N
			selectedColor = (selectedColor && selectedColor != 'transparent')? selectedColor: false;	// No I18N
			var colorpalette = $.ZContextMenu.colorpalette;
			(_options.relativeto)? colorpalette.attr('relativeto',_options.relativeto): colorpalette.removeAttr('relativeto');	//No I18N
			options = $.extend({}, _options, {'relativeToEle': targetEle, 'selectedColor': selectedColor});	// No I18N
			$.ZContextMenu.colorpalette.one('colorpaletteshow', function(e, ui){	// No I18N
				var position = $.ZContextMenu.adjustElementPosition({bufferWidth: (itemObj.width() + colorpalette.width()), bufferHeight: (colorpalette.height() - itemObj.outerHeight()), top: itemObj.offset().top, left: (itemObj.offset().left + itemObj.width()), width : colorpalette.width(), height : colorpalette.height()});
				$.ZContextMenu.colorpalette.addClass('ui-zcolorpalette-menu-popover');	// No I18N
				$.ZContextMenu.colorpalette.css({ top: position.top, left: position.left});
			});
			$.ZContextMenu._colorpaletteHoveredMenuItem = itemObj;
			$.ZContextMenu.colorpalette.one('mouseenter', function(e, ui){	// No I18N
				if($.ZContextMenu._colorpaletteHoveredMenuItem){
					$.ZContextMenu._colorpaletteHoveredMenuItem.addClass('ui-state-hover');	// No I18N
				}
			});
			itemObj.one('mouseleave', function(e, ui){	// No I18N
				if(colorpalette.is(":visible") && $.ZContextMenu._colorpaletteHoveredMenuItem){
					$.ZContextMenu._colorpaletteHoveredMenuItem.addClass('ui-state-hover');	// No I18N
				}
			});
			colorpalette.zcolorpalette('show', targetEle, options);	// No I18N
		}
	}
	$.ZContextMenu.adjustElementPosition = function(eleOffet){
        if(($.ZContextMenu.windowWidth - (eleOffet.left + eleOffet.width + $.ZContextMenu.scrollWidth)) < 5){
            eleOffet.left = eleOffet.left - eleOffet.bufferWidth;
        }
        if(($.ZContextMenu.windowHeight - (eleOffet.top + eleOffet.height + $.ZContextMenu.scrollWidth)) < 5){
            eleOffet.top = eleOffet.top - eleOffet.bufferHeight;
            if(eleOffet.top < 0){

            	eleOffet.top = ($.ZContextMenu.windowHeight - eleOffet.height) - 5;
            }
        }
        return eleOffet;
	}
	$.ZContextMenu.toggleColorPicker = function( target, colorpicker, options ){
		var colorpicker = $('#'+target.attr('data-colorpicker-id')); //No I18N
		var events = components.zcolorpicker.events.replace(/  +/g, ' ').split(' ').join('.zcolorfield '); //No I18N
		var selectEvent = components.zcolorpicker.selectEvent+'.zcolorfield'; //No I18N
		colorpicker.off('.zcolorfield'); //No I18N
		if( colorpicker.is(':visible') && target.is( $.ZContextMenu.lastPickerTarget ) ){ //No I18N
			colorpicker.zcolorpicker('close'); //No I18N
		}else{
			target = $(target);
			options = ZComponents.getOptions( target ).colorpickerOptions || {};
			$.ZContextMenu.lastPickerTarget = options.forElement = target;
			options.value = target.data('selectedColor') || $.zcolorpicker.prototype.options.defaultValue; //No I18N
			colorpicker.zcolorpicker('setAttributes', options); //No I18N
			colorpicker.zcolorpicker( 'open' ); //No I18N
			colorpicker.on( selectEvent,function(origEvent, customData){
							target.trigger( origEvent.type, customData );
							target.data( 'selectedColor', customData.color );	// No I18N
						}).on( events,function(origEvent, customData){
							target.trigger( origEvent.type, customData);
						}).on('zcolorpickerclose.zcolorfield', function(){ //No I18N
							colorpicker.off('.zcolorfield'); //No I18N
						});
		}
	},
	$.ZContextMenu.registerColorpalette = function(options){
		if(!$.ZContextMenu.colorpalette){
			options = options || {};
			options.closeOnBodyClick = false;
			$.ZContextMenu.colorpalette = $("<div>").css('display','none').appendTo('body')
										.zcolorpalette( options ) 
										.on('zcolorpalettecolorselected',function(origEvent, customData){ // No I18N
											$.ZContextMenu.lastPaletteTarget.trigger( 'zcolorpalettecolorselected', customData ).data( 'selectedColor', customData.color );	// No I18N
										}).on('zcolorpalettehover zcolorpaletteshow',function(origEvent, customData){	// No I18N
											$.ZContextMenu.lastPaletteTarget.trigger( origEvent.type, customData);
										}).on('zcolorpalettehide', function(origEvent, customData){
											$.ZContextMenu.lastPaletteTarget.trigger( origEvent.type, customData);
											if( origEvent.originalEvent && $( origEvent.originalEvent.currentTarget ).closest( $.ZContextMenu.colorpalette  ).length ){
												$.ZContextMenu.lastPaletteTarget && $.ZContextMenu.lastPaletteTarget.focus();
											}
										});
			$.ZContextMenu.defaultPaletteOptions = $.ZContextMenu.colorpalette.zcolorpalette('options');//No I18N
			$.ZContextMenu.colorpaletteRegistered = true;
		}
	}
	$.ZContextMenu.destroyColorPalette = function(){
		if( $.ZContextMenu.colorpalette ){
			$.ZContextMenu.colorpalette.zcolorpalette('destroy');	//No I18N
			$.ZContextMenu.colorpalette = $.ZContextMenu.colorpaletteRegistered = null;
		}
	}
	$.ZContextMenu.hideColorPalette = function(){
		clearTimeout( $.ZContextMenu.colorpaletteHideTimer );
		// $.ZContextMenu.colorpalette && $.ZContextMenu.colorpalette.zcolorpalette('close', true);	//No I18N
		$.ZContextMenu.colorpalette && $.ZContextMenu.colorpalette.zcolorpalette('hide', true);	//No I18N
		// this._lastVisited = undefined;
	}	
	$.ZContextMenu.displayColorPalette = function(target, options){
		clearTimeout( $.ZContextMenu.colorpaletteHideTimer );
		target = $(target);
		var colorpalette = $.ZContextMenu.colorpalette;
		if( colorpalette && target.length ){
			var initElement = target.hasClass('ui-zcolorchooser')? target.prev(): target;	//No I18N
			$.ZContextMenu.lastPaletteTarget = target;
	 		var options = ZComponents.getOptions( $(initElement) );
	 		options = options.colorpaletteOptions || {};
			var selectedColor = target.data('selectedColor');//No I18N
			options.selectedColor = selectedColor!== 'transparent' && selectedColor? selectedColor : false; //No I18N
	 		options.relativeToEle = $(target);
	 		var display = true;
			if($.ZContextMenu.isColorpaletteShowing){
				if(this._lastVisited && this._lastVisited[0] === target[0] ){
					display=false;
				}
			}
			this._lastVisited=target;
			colorpalette.zcolorpalette(display? 'show': 'hide', $(target), options ); // No I18N
		}
	},
	$.ZContextMenu.closePopover = function (ev){
		var target = $(ev.target);
		if($.ZContextMenu.currentPopover && target.closest(".zpopover").length === 0){  // No I18N
			var metadata = $.ZContextMenu.currentPopover.data().zpopover, bodyClick = metadata.options.closeOnBodyClick,
				forElement = metadata._forElement;
			if(bodyClick  || target.closest(forElement).length){
				//$.ZContextMenu._isPopoverClosed = true;
				$.ZContextMenu.currentPopover.zpopover( 'close' );	// No I18N

				if(target.closest($.ZContextMenu._lastVisitedPopover).length === 0){
					$.ZContextMenu.currentPopover = $.ZContextMenu._lastVisitedPopover = undefined;
				}

			}
		}
	}
	$.fn.zcontextmenu = function(){
		$.ZContextMenu(this);
		ZComponents.$window.on('resize.zcontextmenu', function(ev){ // No I18N
			window.setTimeout(function(){
				$.ZContextMenu.hideMenu();
				$.ZContextMenu.closePopover(ev);	
			},300);
		});
    }
	$(document).zcontextmenu();
});
/* $Id$ */
var ZShortCut = (function($, $Z){
	var broswerObj = $Z.Browser,
		allowedModifiers = ["ctrl","cmd","shift","alt"],	 // No I18N
		isMac = broswerObj.isMac(),
		isWindows = broswerObj.isWindows(),
		allKeys = {},
		keyMap = {},
		options = {},
		isShortcutEnabled = true,
		keyUpBinded = false,
		spCharArr= {
			"<F1>": "112",	// No I18N
			"<F2>": "113",	// No I18N
			"<F3>": "114", 	// No I18N
			"<F4>": "115", 	// No I18N
			"<F5>": "116", 	// No I18N
			"<F6>": "117", 	// No I18N
			"<F7>": "118", 	// No I18N
			"<F8>": "119", 	// No I18N
			"<F9>": "120",	// No I18N
			"<F10>": "121", // No I18N	
			"<F11>": "122", // No I18N
			"<F12>": "123",	// No I18N
			"PRINTSCREEN": "44", // No I18N
			"NUMLOCK": "144", // No I18N
			"PAUSE": "19", // No I18N
			"INSERT": "45", // No I18N
			"HOME": "36", // No I18N
			"PAGEUP": "33",	// No I18N
			"PAGEDOWN": "34", // No I18N
			"DELETE": "46", // No I18N
			"END": "35",// No I18N
			"TAB": "9", 	// No I18N
			"CAPSLOCK": "20", // No I18N
			"WINDOWS": "91",// No I18N
			"ARROW_UP": "38", // No I18N
			"ARROW_DOWN": "40", // No I18N
			"ARROW_LEFT": "37", // No I18N
			"ARROW_RIGHT": "39", // No I18N
			"ENTER": "13", // No I18N
			"BACKSPACE": "8", // No I18N
			'SPACE': "32", // No I18N
			"ESCAPE": "27",// No I18N
			"DOT": "190", //key - .  Dot, Delete  keys have the same keycode. '.'.charCodeAt(0) = 46, keyCode for delete = 46. In this case we have to consider dot as a special char like TAB, SPACE, DELETE ....    // No I18N
			".": "190", // User can register either (.) or (dot/DOT).
			"Ã‚": "226", //To handle (<)&(>) keys in same button in GERMAN keyboard.	//No I18N			
			"91": "219", //Key - \  // No I18N
			"92": "220", //key - ] // No I18N
			"93": "221", //Key - [ // No I18N
			"61": "187", //key - = // No I18N
			"45": "189", //key - - // No I18N
			"96": "192", //key - ` // No I18N
			"44": "188" ,//key - , // No I18N
			"47": "191", //key - / // No I18N
			"39": "222", //key - ' // No I18N
			"59": "186",  //key - ; // No I18N
			"123": "219", //key - |	 // No I18N
			"124": "220", //Key - }	 // No I18N
			"125": "221", //Key - {   // No I18N
			"43": "187", //Key - +    // No I18N
			"95": "189", //Key - _    // No I18N
			"126": "192", //Key - ~   // No I18N
			"60": "188" ,//Key - <    // No I18N
			"62": "190", //Key - >    // No I18N
			"63": "191", //Key - ?   // No I18N
			"34": "222", //Key - "   // No I18N
			"58": "186",//Key - :     // No I18N
			"33" : "49",  //Key - !  // No I18N
			"64" : "50", //key - @    // No I18N
			"35" : "51", //key - #   // No I18N
			"36" : "52", //key - $   // No I18N
			"37" : "53", //key - %    // No I18N
			"94" : "54", //key - ^    // No I18N
			"38" : "55", //key - &   // No I18N
			"42" : "56", //key - *    // No I18N
			"40" : "57", //key - (  // No I18N
			"41" : "48" //key - )    // No I18N
	};
	var tagNames = ["BUTTON","INPUT","TEXTAREA","OPTION","SELECT","TEXTAREA","OPTGROUP","FIELDSET"];	// No I18N
	$Z.$document.on("keydown.shortcut", function(orgEvent){	 // No I18N
		triggerShortcut(orgEvent, true);
	});
	function triggerShortcut(eventObj, isKeyDown){
		if(isShortcutEnabled && isInputElement(eventObj)){
			var keyCode = eventObj.keyCode, key ="";	// No I18N
			if(eventObj.ctrlKey){
				key = "ctrl+";	 // No I18N
			}
			if(broswerObj.isMac() && eventObj.metaKey){
				key += "cmd+";	 // No I18N
			}
			if(eventObj.shiftKey){
				key += "shift+";	 // No I18N
			}
			if(eventObj.altKey){
				key += "alt+";	 // No I18N
			}
			var charCode = String.fromCharCode(keyCode);
			if(keyCode >= 65 && keyCode <= 90){
				keyCode = charCode.charCodeAt();
			}
			var tempKey = key;
			key+=keyCode;
			var eventObject = allKeys[key];
			if(!eventObject){
				for(var i in spCharArr){
					if(spCharArr[i] === keyCode.toString()){
						key = tempKey+(i.toLowerCase());
						eventObject = allKeys[key];
						if(eventObject){
							break;
						}
					}
				}
			}
			if(eventObject){
				var shortcutKey = key,
					keyInfo = (eventObject.keyInfo || {}),
					element = (eventObject.element || ""),	// No I18N
					handleOn = eventObject.handleOn,
					isDirectInit = eventObject.isDirectInit;
					element = $(element);
				if((isDirectInit && keyInfo.disabled !== true) || 
					(element && ((isKeyDown && handleOn !== "keyup") ||	(!isKeyDown && handleOn === "keyup"))))// No I18N
				{
					if(isDirectInit){
						var elementId = keyInfo.elementIds;
						elementId = (elementId && elementId[0]);
						element = $("#"+elementId);
					}
					var isSupportDisabledAttr = element && (tagNames.indexOf(element[0].tagName) !== -1 );
					if( (isDirectInit && !element.length) ||
						(!element.is(":disabled") && isSupportDisabledAttr) || // No I18N
						(!isSupportDisabledAttr && !element.is("[disabled]"))	// No I18N
					){
						var triggerData = {
							ele: element,
							shortcutKey: eventObject.shortcutKey,
							fromShortCut: true
						};
						if(isDirectInit){
							executionFunction(keyInfo.action, element, eventObj, keyInfo.params)
						}
						else{
							if(element.is(":checkbox") && eventObject.event === "click"){	 // No I18N
								element.each(function(){
									triggerData.ele = $(this);
									$(this).triggerHandler(eventObject.event, triggerData);
								})
							}else{
								element.trigger(eventObject.event, triggerData);
							}
						}
					}
					eventObj.stopPropagation();
					eventObj.preventDefault();
				}
			}
		}
	}
	function executionFunction(callback, element, eventObj, data){
		var value = undefined;
		if(callback && typeof callback === "string"){ // No I18N
		    var executableFnc = _getAsFunction(callback);
			value = executableFnc.callable(data);
		}else{
			value = $.isFunction(callback) ? callback.apply(element, data) : undefined;
		}
		return value !== undefined ? value : !eventObj.isDefaultPrevented();
	}
	function _getAsFunction(strValue){
	    if (!strValue) { return; }

	    var _action = strValue.split("."), fn = window, base = "", methodIdx;
	    $(_action).each(function(i, action){
	        methodIdx = action.indexOf('()');
	        fn = methodIdx>0?fn[action.substring(0, methodIdx)]():fn[action];
	        if (i == _action.length-2) { base = fn; }
	    });
	    //closure issue with callable needs to be addressed
	    return {'fn': fn, 'base': base, 'callable': function(){ fn && fn.call(base,arguments[0]); }}; //No I18N
	}
	function getKey(element, keyInfo){
		keyInfo = keyInfo || $Z.getOptions(element);
		var shortcut = keyInfo.shortcutkey || keyInfo.shortcutKey || keyInfo.key;
		if(shortcut){
			var keys= (typeof shortcut === 'object' && $.isArray(shortcut)) ? shortcut : [shortcut];	 // No I18N
			for(var i=0;i<keys.length;i++){
				var key = keys[i];
				if(typeof keys[i] === 'object'){	 // No I18N
					key = !isMac ? keys[i].key : (keys[i].macKey ? keys[i].macKey : keys[i].key);
					if(!keys[i].key && !isMac){
						return false;
					}
				}
				if(key && !key.macKey && isMac){
					key = key.replace(/ctrl/i, keyInfo.skipShortcutFormat ? "Cmd": 'cmd');	 // No I18N
				}
				if(!keyInfo.skipShortcutFormat){
					key = key.toString().toLowerCase();
				}
				keys[i] = key;
			}
			return keys;
		}
	}
	function isInputElement(orgEvent){
		if(ZShortCut.discardInputElement){
			var element = $(orgEvent.srcElement);
			if(element.length === 0){
				element = $(orgEvent.target);
			}
			if(element.is("input") || element.is("textarea")){
				return false;
			}
		}
		return true;
	}
	function sortKeyCombination(key){
		var splittedKeys = key.split("+"),	 // No I18N
			arrangedKeys = "",splittedKey;	// No I18N
			key = "";	// No I18N	
		for(var i=0;i<allowedModifiers.length;i++){
			for(var j=0;j<splittedKeys.length;j++){
				splittedKey = splittedKeys[j].trim();
				if(allowedModifiers[i] === splittedKey){
					arrangedKeys += splittedKey+"+";	 // No I18N
				}else if($.inArray(splittedKey, allowedModifiers) === -1){
					key = splittedKey;
				}
				if(splittedKey === ''){	 // No I18N
					key = "+";	 // No I18N
				}
			}
		}
		if(!spCharArr[key.toUpperCase()]){
			key = key.toUpperCase().charCodeAt(0);
		}
		return (arrangedKeys += key);
	}
	function registerKey(element, key, orgEvent, handleOn, isDirectInit, keyInfo){
		var arrangedKeys = sortKeyCombination(key);
		if(arrangedKeys){
			var registerObj = {element: element, event: orgEvent, handleOn: handleOn };
			if(isDirectInit){
				registerObj.isDirectInit = true;
				registerObj.keyInfo = keyInfo;
			}
			allKeys[arrangedKeys] = registerObj;
		}
	}
	function getFormattedKey(key){
		var splitKeys = key.split("+"),	 // No I18N
			dkey = "",
			base = this;
		for(var i=0;i<splitKeys.length;i++){
			if(splitKeys[i].charAt(0) == splitKeys[i].charAt(0).toUpperCase()){
				splitKeys[i] = splitKeys[i].toUpperCase();
			}else{
				splitKeys[i] = splitKeys[i].charAt(0).toUpperCase()+splitKeys[i].substring(1, splitKeys[i].length);
			}
			if(splitKeys.length - 1 !== i){
				splitKeys[i]+="+";	 // No I18N
			}
			dkey+=splitKeys[i]
		}
		return dkey;
	}
	function bindKeyUpEvent(){
		keyUpBinded = true;
		$Z.$document.on("keyup.shortcut", function(orgEvent){	 // No I18N
			triggerShortcut(orgEvent, false);
		});
	}
	function deregisterKey(key){
		var arrangedKeys = sortKeyCombination(key);
		arrangedKeys && delete(allKeys[arrangedKeys]);
	}
	function _handleRegister(keyInfo, element, isDirectInit){
		var eventName = keyInfo.eventName || "click",	// No I18N
			handleOn = keyInfo.handleOn,
			keys = getKey(null, keyInfo);
		if(!isWindows && !keyUpBinded && handleOn === "keyup"){	// No I18N
			bindKeyUpEvent();
		}
		if(keys){
			var skipRegister =  keyInfo.skipshortcutregister || keyInfo.skipRegistration;
			for(var i=0;i<keys.length;i++){
				if(!skipRegister){
					registerKey(element, keys[i], eventName, handleOn, isDirectInit, keyInfo);
				}
				if(!keyInfo.skipShortcutFormat){
					keys[i] = getFormattedKey(keys[i]);
				}
			}
		}
		return keys ? keys : false;
	};
	return {
		discardInputElement: true,
		disableAction: function(actionName){
			var keyInfo = keyMap[actionName];
			if(keyInfo){
				keyInfo.disabled = true
			}
		},
		enableAction: function(actionName){
			var keyInfo = keyMap[actionName];
			if(keyInfo){
				keyInfo.disabled = false;
			}
		},
		addShortcuts: function(keysList){
			var keysList = (keysList instanceof Array)? keysList : [keysList],keyInfo;
			for(var i=0;i<keysList.length;i++){
				keyInfo = keysList[i];
				var newKey = _handleRegister(keyInfo, keyInfo.elements, true);
				keyMap[keyInfo.actionName] = keyInfo;
				keyMap[keyInfo.actionName].shortcutKey = newKey[0];
			}
		},
		register: function(element){
			var keyInfo = $Z.getOptions(element),
				shortcut = keyInfo.shortcutkey || keyInfo.shortcutKey;
				keyInfo.eventName = shortcut && shortcut.event ? shortcut.event : "click";	// No I18N
			return _handleRegister(keyInfo, element);
		},
		getShortcutKey: function(element){
			var keys = getKey(element);
			if(keys){
				for(var i=0;i<keys.length;i++){
					keys[i] = getFormattedKey(keys[i])
				}
				return keys;
			}
			return false;
		},
		getInfo: function(actionName, element, registerElement){
			if(actionName){
				var keyInfo = keyMap[actionName] || {};
				return keyInfo.shortcutKey || keyInfo.key || "";	// No I18N
			}
			else if(registerElement !== false && element){
				return this.register(element);
			}
			else if(element){
				return this.getShortcutKey(element);
			}
		},
		enable: function(){
			isShortcutEnabled = true;
		},
		disable: function(){
			isShortcutEnabled = false;
		},
		deregister: function(element){
			var keys = getKey(element);
			for(var i=0;i<keys.length;i++){
				deregisterKey(keys[i]);
			}
		}
	}
})(jQuery, ZComponents);
/*
	{
	action: "bold",	// No I18N
	disabled: true,
	hidden: false,
	checked: true,
	group: "",	// No I18N
	key: ""	// No I18N
	eventName: "keyup|keydown|click"	// No I18N
	handleOn: "keyup|keydown"	// No I18N
	elements:[]
}
*/
/* $Id$ */
(function(ZC){
	/** @components menu - Menu allows users to perform an action from the list of choices.
		@eventPrefix zmenu
	*/
	/*
	ARIA attributes
		role - menu
		aria-disabled
		aria-hidden
		aria-checked
		role of menu item - menuitem | menuitemcheckbox | menuitemradio | separator | group
	*/
    var $ = ZC.DOMUtil;
    var menuItemTemplate = ZC.getTemplate("zmenu.hbs"); //No I18N
    var menuItemJsonTemplate = ZC.getTemplate("zmenujson.hbs"); //No I18N
    var hasHandlebars = false;
	if( !ZC.Browser.isIE() && menuItemTemplate && menuItemJsonTemplate  ){
		Handlebars.partials = Handlebars.templates;
        hasHandlebars = true;
	}
	var menuPrototype = {
		options : {
			"displayType"  : "box", // No I18N
			"direction"    : "auto",  // No I18N
			"type"         : "list", // No I18N
			"toggle"       : true, // No I18N
			"contentType"  : "text", // No I18N
			"renderInPlace": false,  // No I18N
			"columnCount"  : undefined,  // No I18N
			"columns"      : 3,  // No I18N
			"search"       : false, // No I18N
			"forElement"   : undefined, // No I18N
			"width"        : undefined, // No I18N
			"height"       : undefined, // No I18N
			"menuClass"    : "", // No I18N
			"className"    : "", // No I18N
 			"animation"    : undefined, // No I18N
			"pointer"      : false, // No I18N
			"checkedRadioItemIconClassName": undefined, // No I18N
			"checkedRadioItemSVGIconId": undefined, // No I18N
			"checkedCheckboxItemIconClassName": undefined, // No I18N
			"checkedCheckboxItemSVGIconId": undefined, // No I18N
			"isCustomHTMLEncoded"  : true, // No I18N
			"htmlSafe" : true, // No I18N
			"offset"       : undefined, // No I18N
			"scroll"	   : "slider", // No I18N
			"constructMenu": true, // No I18N
			"EFFECTS"      : {   // No I18N
				'slide' : { // No I18N
					"open" : "slideDown", // No I18N
					"close": "slideUp" // No I18N
				},
				'fade' : { // No I18N
					"open" : "fadeIn", // No I18N
					"close" : "fadeOut" // No I18N
				}
			},
            "minWidth"      : 200, // No I18N
			"handleAccessKeyNavigation" : true, // No I18N
		},
        ANIMATION_EVENTS: "webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend webkitTransitionEnd msTransitionEnd oTransitionEnd transitionEnd transitionend", // No I18N
		ARIA_ROLES :{
			"default"  : "menuitem", // No I18N
			"custom"   : "menuitem", // No I18N
			"radio"    : "menuitemradio", // No I18N
			"checkbox" : "menuitemcheckbox", // No I18N
			"itemgroup": "group", // No I18N
			"separator": "separator" // No I18N
		},
		/** @member {String} [menu.type = "list"] - The type of menu which specifies whether the menu should be shown as a list or grid.
			@allowedValues {('list'|'grid')}
			@example <caption> Initialize the menu with the "type" option specified: </caption>
    $("menuElement").zmenu({
      type: "grid"
    });
    @example <caption> Get or set the "type" option, after initialization:</caption>
    Getter Method
        var type = $("menuElement").zmenu("getAttribute","type");
    Setter Method
        $(menuElement).zmenu("setAttribute","type","grid");
		 */
		/** @member {String} [menu.contentType = "text"] - The types of all the menu items present in the menu. 
			@allowedValues {('icon'|'text'|'icon-text')}
		@example <caption> Initialize the menu with the "contentType" option specified: </caption>
    $("menuElement").zmenu({
      contentType: "icon"
    });
    @example <caption> Get or set the "contentType" option, after initialization:</caption>
    Getter Method
        var contentType = $("menuElement").zmenu("getAttribute","contentType");
    Setter Method
        $(menuElement).zmenu("setAttribute","contentType","icon");
		*/
		/* @member {Boolean} [menu.renderInPlace = false] - Specifies whether to position menu as an absolute element nearer to the "forElement" or not. If true, the menu will be opened in its current location. (ie. opened as a relative element). 
	@example <caption> Initialize the menu with the "renderInPlace" option specified: </caption>
    $("menuElement").zmenu({
      renderInPlace: true
    });
    @example <caption> Get or set the "renderInPlace" option, after initialization:</caption>
    Getter Method
        var inPlaceMenu = $("menuElement").zmenu("getAttribute","renderInPlace");
    Setter Method
        $(menuElement).zmenu("setAttribute","renderInPlace",true);
		*/
		/** @member {String} [menu.direction = "auto"] - Opening direction of the menu.
			@allowedValues {('top'|'left'|'right'|'bottom'|'top-left'|'top-right'|'bottom-left'|'bottom-right'|'at-cursor'|'auto')}
	@example <caption> Initialize the menu with the "direction" option specified: </caption>
    $("menuElement").zmenu({
      direction: "top"
    });
    @example <caption> Get or set the "direction" option, after initialization:</caption>
    Getter Method
        var direction = $("menuElement").zmenu("getAttribute","direction");
    Setter Method
        $(menuElement).zmenu("setAttribute","direction","top");
		*/
		/** @member {Number} [menu.columns = 3] - The number of columns that the "grid" type menu will have. 
	@example <caption> Initialize the menu with the "columns" option specified: </caption>
    $("menuElement").zmenu({
      columns: 4
    });
    @example <caption> Get or set the "columns" option, after initialization:</caption>
    Getter Method
        var columns = $("menuElement").zmenu("getAttribute","columns");
    Setter Method
        $(menuElement).zmenu("setAttribute","columns",4);
		*/
		/** @member {String | Object} [menu.forElement] - An element or its selector, which will be used as the target for the menu. 
	@example <caption> Initialize the menu with the "forElement" option specified: </caption>
    $("menuElement").zmenu({
      forElement: "#elementID"
    });
    @example <caption> Provide the "forElement" option while showing the menu:</caption>
    $(menuElement).zmenu("show",{"forElement":"#elementID"});
		*/
		/** @member {String} [menu.className] - The CSS class (or the space separated string of classes) to be added to the menu element for additional theming. This option is mostly used while creating menu using JSON. 
	@example <caption> Initialize the menu with the "className" option specified: </caption>
    $("menuElement").zmenu({
      className: "custom-css-class"
    });

    (or)

	ZComponents.createMenu("fileMenu",[
		{"label": "Open"}, {"label": "Save"}
	],{
		className: "custom-css-class",
		displayType: "callout"
	});
		*/
		/** @member {Number} [menu.height] - The height of the menu. 
	@example <caption> Initialize the menu with the "height" option specified: </caption>
    $("menuElement").zmenu({
      height: 400
    });
    @example <caption> Get or set the "height" option, after initialization:</caption>
    Getter Method
        var height = $("menuElement").zmenu("getAttribute","height");
    Setter Method
        $(menuElement).zmenu("setAttribute","height",400);
		*/
		/** @member {Number} [menu.width] - The width of the menu.
	@example <caption> Initialize the menu with the "width" option specified: </caption>
    $("menuElement").zmenu({
      width: 400
    });
    @example <caption> Get or set the "width" option, after initialization:</caption>
    Getter Method
        var width = $("menuElement").zmenu("getAttribute","width");
    Setter Method
        $(menuElement).zmenu("setAttribute","width",400); 
		*/	
		/* @member {Boolean} [menu.toggle = true] - Toggling nature of the menu. */
		/** @member {Boolean} [menu.displayType = "box"] - The display type of the menu. On setting displayType as "callout", the menu will have a pointing arrow towards the forElement.
		@allowedValues {("callout"|"box")}
	@example <caption> Initialize the menu with the "displayType" option specified: </caption>
    $("menuElement").zmenu({
      displayType: "callout"
    });
    @example <caption> Get or set the "displayType" option, after initialization:</caption>
    Getter Method
        var hasPointingArrow = $("menuElement").zmenu("getAttribute","displayType");
    Setter Method
        $(menuElement).zmenu("setAttribute","displayType","callout"); 
		*/
		/* @member {Boolean} [menu.search = false] - Specifies whether the menu should have a search box or not. */
		/** @member {String | Object} [menu.appendTo = "body"] - An element or its selector, which will be used as the container for the menu.
	@example <caption> Initialize the menu with the "appendTo" option specified: </caption>
    $("menuElement").zmenu({
      appendTo: "containerElementID"
    });
    @example <caption> Provide the "appendTo" option while showing the menu:</caption>
    $(menuElement).zmenu("show",{"appendTo":"containerElementID"});
		*/			
		/** @member {String} [menu.checkedRadioItemIconClassName] - The icon to be shown in the radio type menu item whenever it is in checked state. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedRadioItemIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      checkedRadioItemIconClassName: "zicon zicon-bullet"
    });
    @example <caption> Get or set the "checkedRadioItemIconClassName" option, after initialization:</caption>
    Getter Method
        var checkedRadioItemIconClassName = $("menuElement").zmenu("getAttribute","checkedRadioItemIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedRadioItemIconClassName","zicon zicon-bullet");
		*/
		/** @member {String} [menu.checkedRadioItemSVGIconId] - The icon to be shown in the radio type menu item whenever it is in checked state. It contains SVG element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedRadioItemSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      checkedRadioItemSVGIconId: "bulletIconID custom__class"
    });
    @example <caption> Get or set the "checkedRadioItemSVGIconId" option, after initialization:</caption>
    Getter Method
        var checkedRadioItemSVGIconId = $("menuElement").zmenu("getAttribute","checkedRadioItemSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedRadioItemSVGIconId","bulletIconId custom__class");
		*/

		/** @member {String} [menu.checkedCheckboxItemIconClassName] - The icon to be shown in the checkbox type menu item whenever it is in checked state. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedCheckboxItemIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      checkedCheckboxItemIconClassName: "zicon zicon-tick"
    });
    @example <caption> Get or set the "checkedCheckboxItemIconClassName" option, after initialization:</caption>
    Getter Method
        var checkedCheckboxItemIconClassName = $("menuElement").zmenu("getAttribute","checkedCheckboxItemIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedCheckboxItemIconClassName","zicon zicon-tick");
		*/
		/** @member {String} [menu.checkedCheckboxItemSVGIconId] - The icon to be shown in the checkbox type menu item whenever it is in checked state. It contains SVG icon element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "checkedCheckboxItemSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      checkedCheckboxItemSVGIconId: "tickIconID custom__class"
    });
    @example <caption> Get or set the "checkedCheckboxItemSVGIconId" option, after initialization:</caption>
    Getter Method
        var checkedCheckboxItemSVGIconId = $("menuElement").zmenu("getAttribute","checkedCheckboxItemSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","checkedCheckboxItemSVGIconId","tickIconID custom__class");
		*/
		/** @member {String} [menu.arrowIconClassName] - The icon to be shown in the submenu or split menu item. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "arrowIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      arrowIconClassName: "zicon zicon-arrow"
    });
    @example <caption> Get or set the "arrowIconClassName" option, after initialization:</caption>
    Getter Method
        var arrowIconClassName = $("menuElement").zmenu("getAttribute","arrowIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","arrowIconClassName","zicon zicon-arrow");
		*/
		/** @member {String} [menu.arrowSVGIconId] - The icon to be shown in the submenu or split menu item. It contains SVG icon element id followed by space separated custom CSS class name for the SVG element. 
		@newlyAdded
		@example <caption> Initialize the menu with the "arrowSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      arrowSVGIconId: "arrowIconID custom__class"
    });
    @example <caption> Get or set the "arrowSVGIconId" option, after initialization:</caption>
    Getter Method
        var arrowSVGIconId = $("menuElement").zmenu("getAttribute","arrowSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","arrowSVGIconId","arrowIconID custom__class");
		*/

		/** @member {String} [menu.scrollUpArrowIconClassName] - The icon to be shown in the scroller present in the top of the menu when the menu has scrollable content. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollUpArrowIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      scrollUpArrowIconClassName: "zicon zicon__top-scroll"
    });
    @example <caption> Get or set the "scrollUpArrowIconClassName" option, after initialization:</caption>
    Getter Method
        var scrollUpArrowIconClassName = $("menuElement").zmenu("getAttribute","scrollUpArrowIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollUpArrowIconClassName","zicon zicon__top-scroll");
		*/
		/** @member {String} [menu.scrollUpArrowSVGIconId] - The icon to be shown in the scroller present in the top of the menu when the menu has scrollable content. It contains SVG icon element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollUpArrowSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      scrollUpArrowSVGIconId: "scrollIconID custom__class"
    });
    @example <caption> Get or set the "scrollUpArrowSVGIconId" option, after initialization:</caption>
    Getter Method
        var scrollUpArrowSVGIconId = $("menuElement").zmenu("getAttribute","scrollUpArrowSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollUpArrowSVGIconId","scrollIconID custom__class");
		*/
		/** @member {String} [menu.scrollDownArrowIconClassName] - The icon to be shown in the bottom scroller of the menu when the menu has scrollable content. It can contain one or more space separated sprite CSS classes.
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollDownArrowIconClassName" option specified: </caption>
    $("menuElement").zmenu({
      scrollDownArrowIconClassName: "zicon zicon__bottom-scroll"
    });
    @example <caption> Get or set the "scrollDownArrowIconClassName" option, after initialization:</caption>
    Getter Method
        var scrollDownArrowIconClassName = $("menuElement").zmenu("getAttribute","scrollDownArrowIconClassName");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollDownArrowIconClassName","zicon zicon__bottom-scroll");
		*/
		/** @member {String} [menu.scrollDownArrowSVGIconId] - The icon to be shown in the bottom scroller of the menu when the menu has scrollable content. It contains SVG icon element id followed by space separated CSS class names. 
		@newlyAdded
		@example <caption> Initialize the menu with the "scrollDownArrowSVGIconId" option specified: </caption>
    $("menuElement").zmenu({
      scrollDownArrowSVGIconId: "scrollIconID custom__class"
    });
    @example <caption> Get or set the "scrollDownArrowSVGIconId" option, after initialization:</caption>
    Getter Method
        var scrollDownArrowSVGIconId = $("menuElement").zmenu("getAttribute","scrollDownArrowSVGIconId");
    Setter Method
        $(menuElement).zmenu("setAttribute","scrollDownArrowSVGIconId","scrollIconID custom__class");
		*/
		EFFECTS : {   
			'slide' : { // No I18N
				"open" : "slideDown", // No I18N
				"close": "slideUp" // No I18N
			},
			'fade' : { // No I18N
				"open" : "fadeIn", // No I18N
				"close": "fadeOut" // No I18N
			}
		},
		_MENU_CONTENT_TYPES : {
			"text"      : "", // No I18N
			"icon"      : "zmenu--icononly", // No I18N
			"icon-text" : "zmenu--iconwithtext" // No I18N
		},
		_MENU_ITEM_TYPES : {
			"default"   : "zmenu__item", // No I18N
			"separator" : "zmenu__separator", // No I18N
			"splitmenu" : "zmenu__item zmenu--split", // No I18N
			"itemgroup" : "zmenu__title" // No I18N
		},
		_DIRECTIONS : { 
			"menu"    : "bottom-left", // No I18N
			"submenu" : "right-bottom", // No I18N
			"pointer" : "bottom" // No I18N
		},
		_attrNames : ["id","name","data-item-type","disabled","checked","hidden","data-fixed","data-is-footer","data-item-value","data-label","data-icon","data-icon-class-name","data-svg-icon-id","data-text-as-icon","data-shortcutkey","data-shortcut-key","data-hint","data-description","data-menu-id"], // No I18N
		_EVENTS : { 
			/**
					Triggered before showing the menu. To prevent menu show, return false from an event handler.
					@event menu.beforeshow
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu and forElement.
			*/
			"ONBEFORESHOW"         : "beforeshow", // No I18N
			/**
					Triggered before hiding the menu. To prevent hiding the menu, return false from an event handler.
					@event menu.beforehide
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, and left and top position of the menu.
			*/
			"ONBEFOREHIDE"         : "beforehide", // No I18N
			/**
					Triggered after showing the menu.
					@event menu.show
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, and left and top position of the menu.
			*/
			"ONSHOW"               : "show",       // No I18N
			/**
					Triggered after hiding the menu.
					@event menu.hide
					@param {Object} event originalEvent
					@param {Object} data The object, which contains instance of menu and forElement.
			*/
			"ONHIDE"               : "hide",	   // No I18N
			/**
					Triggered while clicking the item present in the sub menu of the main menu.
					@event menu.submenuitemclick
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, clicked menu item of submenu and left and top position of the submenu.
			*/
			"ONSUBMENUMENUITEMCLICKED" : "submenuitemclick", 		// No I18N
			/**
					Triggered before showing the submenu of the currently hovered menu item. To prevent opening the submenu, return false from an event handler.
					@event menu.beforesubmenushow
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONBEFORESUBMENUSHOW"  : "beforesubmenushow",	// No I18N
			/**
					Triggered after showing the submenu of the currently hovered menu item.
					@event menu.submenushow
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONSUBMENUSHOW"        : "submenushow",         // No I18N
			/**
					Triggered before closing the submenu. To prevent closing the submenu, return false from an event handler.
					@event menu.beforesubmenuhide
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONBEFORESUBMENUHIDE"  : "beforesubmenuhide",     // No I18N
			/**
					Triggered after hiding the submenu of the currently hovered menu item.
					@event menu.submenuhide
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of submenu, menu item which opens the submenu, and left and top position of the submenu.
			*/
			"ONSUBMENUHIDE"        : "submenuhide", // No I18N
			/**
					Triggered while hovering(placing mouse over) an menu item.
					@event menu.itemhover
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, currently hovered menu item, and left and top position of the menu.
			*/
			"ONMENUITEMHOVERED"    : "itemhover", // No I18N
			/**
					Triggered while clicking a menu item.
					@event menu.itemclick
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu, forElement, clicked menu item, and left and top position of the menu.
			*/
			"ONMENUITEMCLICKED"    : "itemclick",	 // No I18N
			/**
					Triggered before setting focus on the menu while opening the menu. To prevent setting focus, return false from an event handler.
					@event menu.beforefocus
					@param {Object} event originalEvent.
					@param {Object} data The object, which contains instance of menu and forElement.
			*/
			"ONBEFOREFOCUS" : "beforefocus" // No I18N

		},
		_lastOverlayedMenuItem: undefined,
		_lastOverlayedMenu: undefined,
		_create: function(element,options){
			this.scrollWidth = menuPrototype._scrollWidth === undefined ? menuScrollWidth() : menuPrototype._scrollWidth;
			this.options.appendTo = (options.appendTo !== undefined && options.appendTo.length) ? $(options.appendTo) : undefined;
			if(!options.usingJSON){  	// usingJSON option is used to build the menu using JSON Object.
				this._buildMenu(element);
			}
			if(this.element[0].tagName === "DIV"){ // No I18N
				// this.element.find("ul:first").attr("role","menu"); // No I18N
				element.find("ul:first").addClass("zmenu__list"); // No I18N
			}
			// else{
			// 	this.element.attr("role","menu"); // No I18N
			// }
			element[0].setAttribute("role","menu"); // No I18N
			this._initEvents();
			this._menuTextArray = [];
			if(options.search){
				this._createSearchContainer(element);
			}
			if(options.renderInPlace){
				this._resetMenuTextArray(element);
				this.show();
			}else{
				element[0].style.display = "none"; // No I18N
			}
			element.data("toggle",options.toggle); // No I18N
			// Replacement for focusable method in ui.focusable.js
			element[0].setAttribute("tabindex",0);
			element[0].style.outline = 0; 
			if(options.height){
				// Height is stored in data attribute to support submenu height fixing
				element.data("height", options.height); // No I18N
			}
			if(options.width){
				// width is stored in data attribute to support submenu width fixing
				element.data("width", options.width); // No I18N
			}
			// check for the rtl direction
			if(ZC.isRTL){
				// this.element.addClass("h-rtl");
				this._DIRECTIONS.submenu = "left-bottom"; // No I18N
			}
		},
		/* Building the menu by adding component specific classes and scope classes */
		_buildMenu: function(menu){ 
			var items = menu[0].querySelectorAll("li"); // No I18N
			if(this.options.constructMenu){
				for(var i = 0; i < items.length; i++){
					this._buildMenuItem($(items[i]),menu);
				}
				menu[0].className += " zmenu "+(this.options.type === "grid" ? "zmenu--thumbnail" : this._MENU_CONTENT_TYPES[this.options.contentType])+" "+(this.options.menuClass ? this.options.menuClass : "")+" "+(this.options.className ? this.options.className : ""); // No I18N
			}else{
				for(var i = 0; i < items.length; i++){
					this._handleCheckedandDisabled($(items[i]),this.getData(items[i])); // initial state maintainance for checked,hidden and disabled.
					this._addEventsForMenuItems($(items[i])); // binding events for each menu item.
				}
			}
		},
		_createSearchContainer: function(menu){
			var header = $("<div class='zmenu__header'></div>"), // No I18N
				textBox = $("<input type='text' class='zmenu__textbox' placeholder='Search' />"), // No I18N
				searchIcon = $("<i class='zmenu__icon'></i>"),
				svgEle = ZC.createSVGElement("svg",{ // No I18N
					"class":"zmenu__search","viewBox":"0 0 16 16","width":"100%","height":"100%" // No I18N
				},[
				{"path":{"d":"M15.6,15.6c-0.6,0.6-1.5,0.6-2.1,0l-2.7-2.7C9.7,13.6,8.4,14,7,14c-3.9,0-7-3.1-7-7s3.1-7,7-7c3.9,0,7,3.1,7,7c0,1.4-0.4,2.7-1.1,3.8l2.7,2.7C16.1,14,16.1,15,15.6,15.6z M7,2C4.2,2,2,4.2,2,7s2.2,5,5,5c2.8,0,5-2.2,5-5C12,4.2,9.8,2,7,2z"}} // No I18N
				]),
				base = this;
			searchIcon[0].appendChild(svgEle);
			header.append(textBox,searchIcon);
			this.element.prepend(header);
			menu.data('_styleElement',$("<style id='zmenu-search-filter' type='text/css'>").appendTo("head")); // No I18N
			textBox.on("keydown."+this.componentName,function(event){ // No I18N
				if(event.keyCode === ZC.keyCode.TAB){ // keycode is 9 and 13
					event.preventDefault();
				}
			});
		},
		/* Building menu items present inside the menu */
		_buildMenuItem: function(menuItem,menu){

			var data = this.getData(menuItem),
				hasSubmenu = data.menuId || (menuItem[0].firstElementChild && typeof menuItem[0].firstElementChild === "object" && menuItem[0].firstElementChild.tagName ? menuItem[0].firstElementChild.tagName === "UL" : false), // No I18N
				itemClass = data.className ? data.className : (data.itemCssClass ? data.itemCssClass : ""); // No I18N
			data.isGrid = this.options.type === "grid"; // No I18N
			menuItem[0].className += " "+(this._MENU_ITEM_TYPES[data.itemType] ? this._MENU_ITEM_TYPES[data.itemType] : "zmenu__item")+" "+itemClass; // No I18N
			data.parentMenu = menuItem.parent();
			data.disabled = data.disabled !== undefined ? data.disabled : (menuItem[0].getAttribute("disabled") != null);
			data.hidden   = data.hidden !== undefined ? data.hidden : (menuItem[0].getAttribute("hidden") != null);
			data.checked  = data.checked !== undefined ? data.checked : (menuItem[0].getAttribute("checked") != null);
			if(hasSubmenu){
				data.hasSubmenu = true;
				this._buildSubmenu(menuItem,data);
			}
            menuItem[0].setAttribute("role", this.ARIA_ROLES[data.itemType ?  data.itemType : "default"]);  // No I18N
			if(data.itemType === "separator"){ // No I18N
				this._handleCheckedandDisabled(menuItem,data);
				return;
			}else if(data.itemType === "custom" || data.custom){ // No I18N
				menuItem[0].className += " zmenu__custom"; // No I18N
				if(data.innerHTML){
					var content = data.innerHTML.content,
						replaceArray = data.innerHTML.replaceArray;

					if(content && replaceArray){
						for (var i = 0; i < replaceArray.length; i++) {
	                    	content = content.replace('{'+ i +'}',ZC.encodeHTML(replaceArray[i]));	// No I18N
	                	}
	                	menuItem.html(content);
					}else{
						if(this.options.isCustomHTMLEncoded || this.options.htmlSafe){
							menuItem.html(data.innerHTML); 
						}else{
							menuItem.html(ZC.encodeHTML(data.innerHTML));
						}
					}
					// menuItem.html(data.innerHTML);
				}
				if(data.hasSubmenu){
					this._createArrowIcon(menuItem);
				}
				this._handleCheckedandDisabled(menuItem,data);
				return  this._addEventsForMenuItems(menuItem);
			}
			data.label = data.label ? data.label : menuItem.text().trim();
            data.label = data.ellipsis ? data.label+"..." : data.label; // No I18N
            var shortcutkey = data.shortcutkey || data.shortcutKey;
            data.shortcutkey = window.ZShortCut ?  ZShortCut.getInfo(data.actionName, menuItem, true) : shortcutkey;
			menuItem[0].innerHTML = ""; // No I18N
			var actualItem = menuItem;
			// actualItem is the item to append.
			if(data.action && typeof data.action === "string" && data.action.indexOf("URL:") !== -1){ // No I18N
				data.target = data.target ? data.target : "_blank"; // No I18N
				menuItem.addClass("zmenu--withlink").append("<a class='zmenu__link' href='"+data.action.replace("URL:","")+"' target='"+data.target+"'>");  // No I18N
				actualItem = menuItem.find(".zmenu__link");
			}
			if(!hasHandlebars){
				if(this.options.contentType !== "text"){ // No I18N
					if(data.itemType !== "itemgroup"){ // No I18N
						var isThumbnail = data.isGrid,
							iconContainer = isThumbnail ? undefined : ZM_DOM._iconDiv.cloneNode(),
							iconEle = ZM_DOM._icon.cloneNode();
						iconEle.className = isThumbnail ? "zmenu__image " :"zmenu__icon ";	// No I18N
                        this._handleIcon($(iconEle), undefined, undefined, data.iconClassName, data.SVGIconId, data.icon);
						!isThumbnail && iconContainer.appendChild(iconEle);
						actualItem[0].appendChild(isThumbnail ? iconEle : iconContainer);
					}else if(this.options.contentType === "icon"){ // No I18N
						actualItem[0].appendChild(this._domUpdate(ZM_DOM._text, data.label));
					}
				}
				if(this.options.contentType !== "icon"){ // No I18N
					actualItem[0].appendChild(this._domUpdate(ZM_DOM._text, data.label));
					if(data.hint || data.description){
						actualItem[0].appendChild(this._domUpdate(ZM_DOM._hint, (data.hint ? data.hint : data.description)));
					}else if(data.shortcutkey){
						actualItem[0].appendChild(this._domUpdate(ZM_DOM._key, data.shortcutkey));
					}
				}
				if(data.hasSubmenu){
					this._createArrowIcon(actualItem);
				}
			}else{ // MenuItem using handlebar templates
				data.icon = data.SVGIconId ? "#"+data.SVGIconId : (data.iconClassName ? data.iconClassName : data.icon); // No I18N
				if(data.icon && data.icon.indexOf("#") !== -1){ // No I18N
					data.svgId = data.icon.split(" ")[0]; // No I18N
					data.svgClass = data.icon.slice(svgId.length).trim();
				}
				if(data.itemType !== "itemgroup"){ // No I18N
					data[this.options.contentType === "icon" ? "icon-only" : this.options.contentType ] = this.options.contentType; // No I18N
				}
				actualItem.append(menuItemTemplate(data));
			}
			if(data.icon && data.icon.indexOf("text:") === 0 && this.options.contentType !== "text"){	// No I18N
				menuItem.find("i:first").removeClass(data.icon).text(data.icon.replace("text:",""));	// No I18N
			}
			if(data.fixed || data.isFooter){
				this._addFixedItem(menuItem,menu,data);
			}
            menuItem[0].setAttribute("data-item-type",data.itemType);
			this.options.contentType !== "icon" && menuItem[0].setAttribute("data-label",data.label); // No I18N
			this._handleCheckedandDisabled(menuItem,data);
			this._addEventsForMenuItems(menuItem,data);
		},
		_domUpdate: function(node, text){
			var itemText = node.cloneNode();
			itemText.textContent = text;
			return itemText;
		},
		_createArrowIcon: function(menuItem){
			var iconEle = $("<i class='zmenu__icon h-alignright'/>"),
                svgData = {
                    props: {"class":"zmenu__arrow","viewBox":"0 0 48 48","width":"100%","height":"100%"}, // No I18N
                    nodes: [{
                        "path":{"d":"M17,6l-4,5L27,24,13,38l4,4L36,24Z","class":"cls-1"} // No I18N
                    }]
                };
            this._handleIcon(iconEle, svgData, "arrow", this.options.arrowIconClassName, this.options.arrowSVGIconId);
			menuItem.append(iconEle);
			return iconEle;
		},
		/* Adding the fixed item in the fixed container. */
		_addFixedItem: function(menuItem,menu,data){
			if(data.label && this.options.contentType === "icon" && this.options.type === "grid" && !menuItem.find(".zmenu__text").length){
				menuItem.append($("<span class='zmenu__text'>"+data.label+"</span>"));
			}
			if(menu.find(".zmenu__footer").length){ // No I18N
				menu.find(".zmenu__footer ul").append(menuItem); // No I18N
			}else{
				var fixedArea = $("<div class='zmenu__footer'><ul class='zmenu__list'></ul></div>"); 
				menu.append(fixedArea);
				fixedArea.find("ul").append(menuItem.addClass("is-fixed")); // No I18N
				menu.data("fixedArea",fixedArea.find("ul")); // No I18N
			}			
		},
		_buildSubmenu: function(menuItem,data){
			var menu = this.getSubmenu(menuItem);
			if(!menu){
				return;
			}
			// MenuItem with itemType as "radio" and "checkbox" can also contains splitmenu or submenu.
			menuItem.addClass(data.splitmenu || data.itemType === "splitmenu" ? "zmenu--split": "zmenu--submenu"); // No I18N
			menuItem.data("submenu",menu); // No I18N
			if(!data.menuId){
				$("body").append(menuItem.find(menu)); // No I18N
				this.getData(menuItem).menuId = menu.attr("id"); // No I18N
				menu.zmenu();
			}
		},
		getSubmenu: function(menuItem){
			if(menuItem.data("submenu")){ // No I18N
				return menuItem.data("submenu"); // No I18N
			}
			var submenu = this.getData(menuItem).menuId;
			if(submenu){
				submenu = $("#"+submenu); // No I18N
				if(!submenu.length){
					return undefined;
				}
				if(!submenu.data("zmenu")){ // No I18N
					submenu.zmenu();
				}
				submenu.data("isExternalSubmenu",true);  // No I18N
				return submenu;
			}
			return (menuItem.children("ul:first")); // No I18N
		},
		_handleCheckedandDisabled: function(menuItem,data,optionName){
            var fromSetAttribute = optionName !== undefined; // check is included because disabling the hidden items(made hidden by hide() method of jquery) make the items visible.
			if(fromSetAttribute ? (optionName === "hidden") : (data.hidden !== undefined)){ // No I18N
				this._hideMenuItem(menuItem,!data.hidden);
			}
			if(fromSetAttribute ? (optionName === "disabled") : (data.disabled !== undefined)){ // No I18N
				menuItem[data.disabled ? "addClass" : "removeClass"]("is-disabled"); // No I18N
				menuItem.attr("aria-disabled",data.disabled); // No I18N
				data.disabled ? menuItem.attr("disabled",true) : menuItem.removeAttr("disabled"); // No I18N
			}
            var condition = fromSetAttribute ? (optionName === "checked") : true; // No I18N
			if(condition && (this.options.contentType !== "text" && data.checked !== undefined && (data.itemType === "checkbox" || data.itemType === "radio"))){ // No I18N
				this._checkMenuItem(menuItem,data.checked,data);
			}	
		},
		_initEvents: function(menu){
			var base = this, componentName = this.componentName;
			menu = menu ? $(menu) : $(this.element);
			if((menu.data('initialized') && !menu) || menu.data("isExternalSubmenu")){ // No I18N
				return;
			}
			menu.data('initialized', true);  // No I18N
			menu.on("mouseup."+componentName, function (event) { // No I18N
				base._handleMenuItemClick(event, menu);
			}).on("contextmenu."+componentName,function(event){ // No I18N
				// binding the context menu event to prevent closing the menu while clicking the link item.
				var item = $(event.target).closest(".zmenu__item");
				if(item.length && item.hasClass("zmenu--withlink")){
					event.stopPropagation();
				}
			});
		},
        _handleMenuItemClick: function(event, menu){
            menu = menu || this.element;
            var menuItem = $(event.target);
            if(!menuItem.is("li")){ // No I18N
                menuItem = menuItem.closest("li"); // No I18N
            }
            if(!menuItem || menuItem.length === 0 || menuItem.hasClass("separator")){
                return;
            }
            var data = this.getData(menuItem),
                hasSubmenu = menuItem.data("submenu") && menuItem.data("submenu").length && !menuItem.hasClass("zmenu--split") ? true : false; // No I18N
            if(data.disabled){
                return;
            }
            if(hasSubmenu && data.openOn === "click"){ // No I18N
                var submenu = this.getSubmenu(menuItem), base = this;
                if(submenu.is(":visible")){ // No I18N
                    this.hide(submenu,function(){
                        if(submenu.data("forElement")[0] !== menuItem[0]){ // No I18N
                        // same submenu is used for two or more items
                            base._showSubmenu(menuItem,{"event":event},false,event); // No I18N
                        }
                    });
                }else{
                    this._checkAndHideAllOpenedSubMenus(menu,true);
                    this._showSubmenu(menuItem,{"event":event},true,event); // No I18N
                }
                event.preventDefault(); // preventing the events here since closehandler hide the menu
                event.stopPropagation();
            }
            var click = data.action;
            if(click !== undefined){
                if(typeof click !== "function" && typeof click === "string" && click.indexOf("URL:") === -1){ // No I18N
                    // click = new Function(click);
                    // click.call(event,menuItem,data);
                    this._triggerFunction(click, menuItem, [event, {"menuitem": menuItem, "options": data}]); // No I18N
                    // function names given as strings will be handled here
                }else if(typeof click === "function"){ // No I18N
                    click.call(menuItem,event,{"menuitem": menuItem,"options":data});  // No I18N
                }
            }
            if(this._triggerEvent("beforeitemchecked", event, menu, menuItem, {})){
                if(data.itemType === "checkbox" || data.itemType === "radio"){ // No I18N
                // Uncheck the menuitem if it is of checkbox type and already in checked state.
                    this._checkMenuItem(menuItem,(data.itemType === "checkbox" ? !menuItem.prop("checked") :true),data); // No I18N
                }
            }
            if(this._triggerEvent((menu.data("parent") !== undefined) ?  "ONSUBMENUMENUITEMCLICKED" : "ONMENUITEMCLICKED", event, menu, menuItem, {})){ // No I18N
            // if(!this.options.renderInPlace && !data.doNotTriggerClickEvent && data.itemType !== "checkbox"){ // No I18N
                if(!this.options.renderInPlace){ 
                    !hasSubmenu && this.hide(undefined,undefined,event); // Clicking the item containing the submenu should not close the menu.
                    if(menuItem){
                        if(!menuItem.is(".zmenu--withlink")){ // No I18N
                            event.stopImmediatePropagation();
                            event.preventDefault();
                            event.stopPropagation();
                        }else{
                            menuItem.find(".zmenu__link")[0].click(); // No I18N
                        }
                    }
                }
            }
        },
		/** 
			Opens the menu in the specified direction nearer to the forElement. If direction is at-cursor, original event object should be passed.
			@function menu.show
			@param {Object} [options] - It can contain forElement,direction and event object.
			@param {Object} [menu] -  The menu to show.
			@example <caption> Example: </caption>
$("menuElement").zmenu("show",{"forElement": $("#fileButton")});
		*/
		show: function(options,menu){
            
			options = options ? options : {};
			var appendedTo = $(this.options.appendTo),
				isSubMenu = options.isSubMenu = menu ? true : false,
				menu = menu ? menu : this.element, // Menu is undefined - i.e Its the base menu
				forElement = options.forElement ? $(options.forElement) : $(this.options.forElement),
				isFloating = this._isGridMenu(menu),
				offsetValue = options.offset ? options.offset : (this.options.offset ? this.options.offset : undefined); 
			this._uninitSliders(menu);
            if(!isSubMenu){
                this.options.forElement = options.forElement = forElement;
            }
			if(menu.is(":visible") && !options.mustBeShown){  // No I18N
				if(this.options.toggle && !isSubMenu){ // toggling the menu whenever toggle option is set to true.
					this.hide();
				}
				return;
			}
			if(!isSubMenu){
				if(this.options.handleAccessKeyNavigation){
					this._initRuntimeEvents(); // Keypress events	
				}
				this.element.data("forElement",forElement); // No I18N
			}else{
				menu.data({"forElement":forElement,"parent":forElement}); // No I18N
			}
			if((!this.options.renderInPlace || isSubMenu) && !options.mustBeShown){
				if(appendedTo !== undefined && appendedTo.length ){
					appendedTo.append(menu); 
				}else{
					// submenu is also appended to body in order to support using same submenu for more than one menu items.
					$("body").append(menu);	// No I18N
				}
			}
			options.within = this.options.viewport && $(this.options.viewport).length ? $(this.options.viewport) : appendedTo;
			this.updateLastMenuChain(menu);
			if(menu.data("modified")){ // No I18N
				menu.find("#noresultFound").remove();  // No I18N
				menu.data("_styleElement").empty(); // No I18N
			}
			if(!isFloating && !this.options.renderInPlace){
				menu.width("auto");  // No I18N
			}
			if(!this.options.renderInPlace){
                ZC.bindDocumentEvents && ZC.bindDocumentEvents({
                    element: menu,
                    forElement: forElement,
                    ctype: this.componentName,
                    closeMethodName: "hide" // No I18N
                });
				var originalEvent = options.event ? options.event : {};
                options.isRightClick = options.isRightClick ? options.isRightClick : (options.direction ? ( options.direction === "after-pointer" || options.direction === "at-cursor") : ( this.options.direction === "after-pointer" || this.options.direction === "at-cursor")); // No I18N
				if(!this._triggerEvent(isSubMenu ? "ONBEFORESUBMENUSHOW" : "ONBEFORESHOW", originalEvent, menu, undefined, {"currentTarget": options.isRightClick ? originalEvent.target : forElement})){ // No I18N
					return;
				}
				var base = this;
				menu.css("opacity",0); // No I18N
				menu.css("display","block"); // No I18N
				this._setFocus();
				if(menu.data("width")){ // No I18N
					var menuWidth = parseInt(menu.outerWidth(true)),
						optionWidth = parseInt(menu.data("width")); // No I18N
					if(menuWidth > optionWidth){
						// finding the maximum shortcut key's width
						var maxWidth = menu.find("li").map(function(index,item){
							return ($(item).find(".zmenu__shortcut").length ? parseInt($(item).find(".zmenu__shortcut").outerWidth(true)) : 0);
						});
						maxWidth = Math.max.apply(null, maxWidth);
						if(maxWidth){ // should not set 100% to text span
							menu.find("li:not(li.zmenu__title) .zmenu__text").css("width","calc(100% - "+maxWidth+"px)"); // No I18N			
						}
					}
					menu.width(optionWidth);
				}else if(menu.width() < parseInt(this.options.minWidth) && menu.data("zmenu").options.contentType !== "icon"){ // No I18N
					// min-width for menu. 
                    // Subtract border width if box-sizing is menu's box-sizing is border-box. 1px is added extra by default while setting width. So subtracted 1px.
                    if(menu.css("box-sizing") === "border-box"){ // No I18N
                        menu.width(parseInt(this.options.minWidth) - parseInt(menu.css("border-left-width")) - parseInt(menu.css("border-right-width")) - 1); // No I18N
                    }else{
                        menu.width(parseInt(this.options.minWidth) - 1);
                    }
				}

				// finding the maximum width of li
				/* var maxWidth = menu.find("li").map(function(index,item){
					return ($(item).find(".zmenu__text").length ? parseInt($(item).find(".zmenu__text").outerWidth(true)) : 0) + ($(item).find(".zmenu__shortcut").length ? parseInt($(item).find(".zmenu__shortcut").outerWidth(true)) : 0) + ($(item).find(".zicon--arrow h-alignright").length ? parseInt($(item).find(".zicon--arrow h-alignright").outerWidth(true)) : 0) ;
				});
				maxWidth = Math.max.apply(null, maxWidth)); */
				var hasScroll = false;
				if(isFloating){
					this._fixFloatingMenuWidth(menu); // Fixing the width of grid type menu.
				}
				menu.css("height","auto"); // No I18N 	
	 			menu.find("ul").length && menu.find("ul").css("height","auto"); // No I18N
				if(menu.data("height") <= menu.outerHeight()){ // No I18N
					// Fixing the menu height,which is provided in the option only if it is less than actual height of the menu
					// menu.height(menu.data("height")); // No I18N 
					menu.outerHeight(parseInt(menu.data("height"))); // No I18N 
					hasScroll = true;
				}
				if(!isFloating){ // fixing the width of list type menu.
					var mwidth = menu.innerWidth() + 1 + ( hasScroll ? this.scrollWidth : 0 ); // To consider width in floating numbers, added 1px to innerWidth value.
					menu.width(mwidth);
				}
	            this.refresh(menu);
	            if(options && !options.position){
					options.direction = options.direction ? ( (options.direction === "auto" || options.isRightClick) ? ( (this.options.pointer || this.options.displayType === "callout") ? this._DIRECTIONS.pointer : this._DIRECTIONS.menu) : options.direction) : (isSubMenu ? this._DIRECTIONS.submenu : ( (this.options.direction === "auto" || options.isRightClick) ? ((this.options.pointer || this.options.displayType === "callout") ? this._DIRECTIONS.pointer : this._DIRECTIONS.menu) : this.options.direction)); // No I18N
					options.eventObj = options.event;
	                options.positionAlterable = options.positionAlterable !== undefined ? options.positionAlterable : (isSubMenu ? "flipfit" : (this.options.direction === "auto" || this.options.direction === "after-pointer" || this.options.direction === "at-cursor")); // No I18N
	                /* if direction is after-pointer, we should not place the menu to the left or right of the target */
	                options.positionAlterable = ((this.options.direction === "after-pointer" || this.options.direction === "at-cursor" || options.isRightClick) && !isSubMenu) ? "fit" : options.positionAlterable; // No I18N
	                if(typeof options.position !== "object"){ // No I18N
	                	if((this.options.pointer || this.options.displayType === "callout") && !isSubMenu){ // No I18N
	                		this._createPointerElement();
	                		var base = this; 
	                		options.arrowCallback = function(element,direction){
	                			base._arrowDirection = direction;
	                			element.removeClass("h-top h-bottom h-right h-left").addClass('h-'+direction); // No I18N
	                		};
	                	}
						menu.data("isRightClick",options.isRightClick);  // No I18N
	                	var pos = ZPosition.get( (options.isRightClick && !isSubMenu ? options.event : forElement), menu, $.extend({},{"arrow": {"element": !isSubMenu ? this._pointerElement : null,"callback":options.arrowCallback}},options),this._pointerElement ? this._pointerElement : undefined); // No I18N
						options.availableHeight = pos.availableHeight;
						options.position = pos.elementPosition;
						options.position.direction = pos.direction;
						menu.data("direction",pos.direction);  // No I18N
						if(this._pointerElement && !isSubMenu){
							if(offsetValue){
								if(pos.arrowDirection.match(/^top|bottom$/)){ 
									// adjust left position.
									pos.arrowPosition.left = pos.arrowPosition.left - (offsetValue.left);
								}else{ // adjust top position.
									pos.arrowPosition.top = pos.arrowPosition.top - (offsetValue.top);
								}
							}
							this._pointerElement.css(pos.arrowPosition);
							
						}else if(!this._pointerElement && !offsetValue){
							// Resetting the border radius which was set previously.
							menu.removeClass("zmenu--bottomleft zmenu--bottomright zmenu--topleft zmenu--topright zmenu--top zmenu--bottom zmenu--left zmenu--right"); // No I18N
							if(isSubMenu){
								if(pos.direction === "right-top"){ // No I18N
									menu.addClass("zmenu--bottomleft"); // No I18N
									options.position.left = options.position.left - 1;
								}else if(pos.direction === "right-bottom"){ // No I18N
									menu.addClass("zmenu--topleft"); // No I18N
									options.position.left = options.position.left - 1;
								}else if(pos.direction === "left-top"){ // No I18N
									menu.addClass("zmenu--bottomright"); // No I18N
									options.position.left = options.position.left + 1;
								}else if(pos.direction === "left-bottom"){ // No I18N
									menu.addClass("zmenu--topright"); // No I18N
									options.position.left = options.position.left + 1;
								}
							}else{
								// setting the border radius based on the menu opening direction.
								var topLeft = parseInt(forElement.css("border-top-left-radius")), // No I18N
									topRight = parseInt(forElement.css("border-top-right-radius")), // No I18N
									bottomLeft = parseInt(forElement.css("border-bottom-left-radius")), // No I18N
									bottomRight = parseInt(forElement.css("border-bottom-right-radius")); // No I18N
								if(topLeft <= 5 && topRight <= 5 && bottomLeft <=5 && bottomRight <= 5){
									if(options.isRightClick){
										menu.removeClass("zmenu--bottomleft zmenu--bottomright zmenu--topleft zmenu--topright zmenu--top zmenu--bottom zmenu--left zmenu--right"); // No I18N
									}else if(pos.direction.indexOf("bottom") === 0){ // No I18N

										if(pos.direction === "bottom-right"){ // No I18N
											if(forElement.outerWidth(true) < menu.width()){ 
												// menu.addClass("zmenu--topleft");  // No I18N
												menu.addClass("zmenu--topright");  // No I18N
												forElement.css({"border-bottom-left-radius":"0px","border-bottom-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--top"); // No I18N
												forElement.css({"border-bottom-right-radius":"0px"});	 // No I18N							
											}
										}else if(pos.direction === "bottom-left"){  // No I18N
											if(forElement.outerWidth(true) < menu.width()){
												// menu.addClass("zmenu--topright"); // No I18N
												menu.addClass("zmenu--topleft"); // No I18N
												forElement.css({"border-bottom-left-radius":"0px","border-bottom-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--top"); // No I18N
												forElement.css({"border-bottom-left-radius":"0px"});	 // No I18N	
											}
										}
										options.position.top = options.position.top - 1;
									}else if(pos.direction.indexOf("top") === 0){ // No I18N
										if(pos.direction === "top-right"){ // No I18N
											if(forElement.outerWidth(true) < menu.width()){ 
												// menu.addClass("zmenu--bottomleft"); // No I18N
												menu.addClass("zmenu--bottomright"); // No I18N
												forElement.css({"border-top-left-radius":"0px","border-top-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--bottom"); // No I18N
												forElement.css({"border-top-right-radius":"0px"});	 // No I18N							
											}
										}else if(pos.direction === "top-left"){ // No I18N
											if(forElement.outerWidth(true) < menu.width()){
												// menu.addClass("zmenu--bottomright"); // No I18N
												menu.addClass("zmenu--bottomleft"); // No I18N
												forElement.css({"border-top-left-radius":"0px","border-top-right-radius":"0px"}); // No I18N
											}else{
												menu.addClass("zmenu--bottom"); // No I18N
												forElement.css({"border-top-left-radius":"0px"});	 // No I18N	
											}
										}
										options.position.top = options.position.top + 1;
									}else if(pos.direction.indexOf("left") === 0){ // No I18N
										menu.addClass("zmenu--right"); // No I18N
										forElement.css({"border-top-left-radius":"0px","border-bottom-left-radius":"0px"});	 // No I18N	
									}else if(pos.direction.indexOf("right") === 0){ // No I18N
										menu.addClass("zmenu--left"); // No I18N
										forElement.css({"border-top-right-radius":"0px","border-bottom-right-radius":"0px"}); // No I18N
									}
								}else{ // adjust the top value of the menu by 2 pixels.
									options.position.top = options.position.top + 2;
								}
							}
						}
	                }
	                if(menu.find(".zmenu__textbox").length){ // No I18N
						menu.data("initialDirection",options.position.direction); // No I18N
						var searchDiv = menu.find('.zmenu__search').parent(); // No I18N
						if((options.position.top < $(forElement).position().top)){
							menu.append(searchDiv.addClass("zmenu__footer").removeClass("zmenu__header")); // No I18N
							var pos = ZPosition.get( (options.isRightClick && !isSubMenu ? options.event : forElement), menu, options,this._pointerElement ? this._pointerElement : undefined); 
							options.position = pos.elementPosition;
							options.position.top = options.position.top + 1;
						}else{
							menu.prepend(searchDiv.addClass("zmenu__header").removeClass("zmenu__footer")); // No I18N
						}
					}
					if(offsetValue){
						options.position.top = options.position.top + (parseInt(offsetValue.top) || 0);
						options.position.left = options.position.left + (parseInt(offsetValue.left) || 0);
					}
	            }
               
                if(options.position){
                    if(options.isRightClick){ /* Adjusting two pixel for right click menu's since mouseup events will not be triggered for the clicked portions if we open the menu in the same location as that of cursor. */
                        options.position.top = options.position.top + 2;
                        options.position.left = options.position.left + 2;
                    }
                    menu.css(options.position)
                }
				this._fixMenuUI(menu, options, isSubMenu);
				if(menu.find("ul:first .zmenu__item").length === 0 && menu.find(".zmenu__footer").length){ // No I18N
					menu.find(".zmenu__footer").css("border-top","0px"); // No I18N
				}
				menu.hide();
				menu.css("opacity",""); // No I18N
				this._pointerElement && this._pointerElement.hide();
				this._openWithAnimation(menu,options,originalEvent);
			}else{
				menu.css({"width":"inherit","height":"inherit","position":"relative"}); // No I18N
				this._openWithAnimation(menu,options,originalEvent);
			}
			if(!isSubMenu){
				if(ZC.currentMenu && ZC.currentMenu.get(0) != menu.get(0)){
					ZC.prevMenu = ZC.currentMenu;
				}
				ZC.currentMenu = menu;
			}
		},
		_openWithAnimation: function(menu,options,originalEvent){
			var animationObject = this.options.animation;
			if(options.isSubMenu){
				animationObject = menu.data("zmenu").options.animation; // No I18N
			}
			if(animationObject || options && options.animation){
				var base = this,
					animation = animationObject ? animationObject.open : options.animation.open,
					animationClass = animation.className,
					effect = animation.name,
					duration = animation.duration ? animation.duration : 0,
					completeFunction = function(){
						base._handleMenuOpen(menu,options,originalEvent);
					};
				if(animationClass){
					this._pointerElement && this._pointerElement.show().addClass(animationClass);
					menu.show().addClass(animationClass).on(this.ANIMATION_EVENTS, function(){
                        menu.removeClass(animationClass); // removing the animation class.
						completeFunction && completeFunction();
                        completeFunction = undefined;
						menu.off(base.ANIMATION_EVENTS);
					});
				}else if(typeof effect === "object"){  // No I18N
					// custom animation properties
					this._pointerElement && this._pointerElement.show().animate(effect,duration,completeFunction);
					menu.show().animate(effect,duration,completeFunction);
				}else{ // predefined effects like slide and fade.
					if(this.options.EFFECTS[effect]){
						this._pointerElement && this._pointerElement[this.options.EFFECTS[effect].open](duration,completeFunction);
						menu[this.options.EFFECTS[effect].open](duration,completeFunction);
					}
				}
			}else{
				menu.show();
				this._pointerElement && this._pointerElement.show();
				this._handleMenuOpen(menu,options,originalEvent);
			}
		},
		_handleMenuOpen: function(menu,options,originalEvent){
			this._resetMenuTextArray(menu);
			// if(!options.preventHover && !this.options.preventHover && menu.find("li[data-item-type='radio'],li[data-item-type='checkbox']").length){ // No I18N
			// 	var checkedItems = menu.find("li[data-item-type='radio'][checked],li[data-item-type='checkbox'][checked]");
			// 	checkedItems.length && this._handleMouseOverOnMenuItems($(checkedItems[checkedItems.length - 1]),{},true,true);
			// }
			!this.options.renderInPlace && this._triggerEvent(options.isSubMenu ? "ONSUBMENUSHOW" : "ONSHOW", originalEvent, menu, undefined, {"forElement":options.forElement});  // No I18N
		},
		_createPointerElement: function(){
			if(this._pointerElement){
				return this._pointerElement.removeAttr("class").addClass("zmenu__pointer"); // No I18N
			}
			this._pointerElement = $("<div class='zmenu__pointer' />");
			this.element.prepend(this._pointerElement);
		},
		_isGridMenu: function(menuOrMenuItem, isMenuItem){
			var menu = isMenuItem ? menuOrMenuItem.closest(".zmenu") :  // No I18N
										(menuOrMenuItem? menuOrMenuItem: this.element);
			if(menu.length){
				var type = this.getMenuType(menu);
				return this._isTypeFloating(type, menu);
			}
		},
		/* Binding hover events on menu items */
		_addEventsForMenuItems: function(menuItem){
			var base = this,
				metaData = this.getData(base.element);
			menuItem.hover(function(event, data){
				event.data = data;
				var isdirect = base._isDirectEvent(event),
					mouseOverFunc = function(){
						base._handleMouseOverOnMenuItems(menuItem, event,undefined,true);
					};
				base._clearMouseHoverTimeout();
				if(!isdirect || !base._isToBeDelayed(event)){
					mouseOverFunc();
				}else{
					metaData.mouseOverTimeout = setTimeout(mouseOverFunc,300);
				}
			}, function(event, data) {
				event.data = data;
				var isdirect = base._isDirectEvent(event),
					mouseOutFunc = function(){
						base._handleMouseOutOnMenuItems(menuItem, event);
					};
				if(!isdirect || ( !menuItem.data("submenu") && !metaData.mouseOverTimeout)){
					mouseOutFunc();
				}else{
					metaData.mouseOutTimeout = setTimeout(mouseOutFunc,300);
				}
			});
		},
		/** 
			Hides the menu.
			@function menu.hide
			@param {Object} [menu] - The menu to hide.
			@example <caption> Example: </caption>
$("menuElement").zmenu("hide",menu);
		*/
		hide: function(menu,callback,event){
			if(this.options.renderInPlace){
				return;
			}
			var isSubMenu = menu && menu.length ? true : false,base = this;
			menu = menu ? $(menu) : this.element;
			if(menu.is(":hidden")){ // No I18N
				return;
			}
			var result = this._triggerEvent(isSubMenu ? "ONBEFORESUBMENUHIDE" : "ONBEFOREHIDE", event, menu); // No I18N
			if(result){			
				if(!isSubMenu){
					this._hideAllSubMenus(menu);
					this._destroyRuntimeEvents();	
				}
				$("li.on-hover",menu).each(function(i, menuItem){ // No I18N
					if($(menuItem).is(".zmenu--split")){ // No I18N
						$(menuItem).find(".zmenu__icon.h-alignright").removeClass("on-hover");
					}
					$(menuItem).removeClass("on-hover");  // No I18N
				});
				if(menu.get(0) !== this.element.get(0)){
					this.updateLastMenuChain(menu.data("parent").parent());  // No I18N
				}else{
					this._lastOverlayedMenuItem = undefined;
				}
				this._uninitSliders(menu);	
				this._closeWithAnimation(menu,isSubMenu,callback,event);	
			}
		},
		_closeWithAnimation: function(menu,isSubMenu,callback,event){
			var animation = this.options.animation;
			if(isSubMenu){
				animation = menu.data("zmenu").options.animation; // No I18N
			}
			if(animation && animation.close){
				animation = animation.close;
				var base = this,
					animationClass = animation.className,
					effect = animation.name,
					duration = animation.duration ? animation.duration : 0,
					completeFunction = function(){
						base._handleMenuClose(menu,isSubMenu,callback,event); // hiding the menu on animation complete function
					};
				if(animationClass){
					menu.addClass(animationClass).on(this.ANIMATION_EVENTS, function(){
						menu.removeClass(animationClass);
						menu.hide();
						completeFunction && completeFunction();
						completeFunction = undefined;
						menu.off(base.ANIMATION_EVENTS);
					});
				}else if(typeof effect === "object"){  // No I18N
					// custom animation properties
					var base = this;
					menu.animate(effect,duration,completeFunction);
				}else{ // predefined effects like slide and fade. 
					if(this.options.EFFECTS[effect]){
						menu[this.options.EFFECTS[effect].close](duration,completeFunction);
					} 
				}
				if(isSubMenu){
					menu.data("parent").closest(".zmenu").data("waitingTime",duration); // No I18N
				}
			}else{
				var base = this;
				if(menu.data("waitingTime") > 0){
					window.setTimeout(function(){
						menu.hide();
						base._handleMenuClose(menu,isSubMenu,callback,event);
						menu.data("waitingTime",0);  // No I18N
					},menu.data("waitingTime"));  // No I18N
				}else{
					menu.hide();
					base._handleMenuClose(menu,isSubMenu,callback,event);
				}
				
			}
		},
		_handleMenuClose: function(menu,isSubMenu,callback,event){
			if(isSubMenu){
				var __menu = $(menu.data('parent')).closest(".zmenu");  // No I18N
				this._resetMenuTextArray(__menu);
			}else{
				this._resetMenuTextArray(menu);				
			}
			menu.css({top:"", left:"",height:"", width:""}); // No I18N
			menu.removeClass("zmenu--bottomleft zmenu--bottomright zmenu--topright zmenu--topleft zmenu--bottom zmenu--top zmenu--left zmenu--right"); // No I18N
			$(menu.data("forElement")).css({"border-radius":""}); // No I18N
			if(menu.data("_styleElement") !== undefined){ // No I18N
				menu.data("_styleElement").empty(); // No I18N
			}
			this._triggerEvent(isSubMenu? "ONSUBMENUHIDE": "ONHIDE", event, menu); // No I18N
			callback && callback();
		},
		/* Clearing the preserved mouseover and mouseout timeout values */
		_clearMouseHoverTimeout: function(){
			var metaData = this.getData(this.element);
			clearTimeout(metaData.mouseOverTimeout);
			clearTimeout(metaData.mouseOutTimeout); 
			if(this.element.data("parent") !== undefined){ // No I18N
				var parData = this.getData( this.element.data("parent").closest(".zmenu") ); // No I18N
				clearTimeout(parData.mouseOverTimeout);
				clearTimeout(parData.mouseOutTimeout);
				parData.constantLoc = undefined;
			}
		},
		_isElementInMenu: function(element, menu){
			if(!menu){
				menu = this.element;
			}
			element = $(element)[0];
			return (menu.has(element).length === 1);
		},
		/* Checking whether the submenu hide should be delayed in order to provide inclined mouseover on submenu items */
		_isToBeDelayed: function(e){
			var metaData = this.getData(this.element);
			if(metaData.constantLoc !== undefined){
				var cursorPosition = {x:e.pageX,y:e.pageY},
					decreasingSlope = this._slope(cursorPosition, this.decreasingCorner),
	                increasingSlope = this._slope(cursorPosition, this.increasingCorner);
	            if(decreasingSlope < this.prevDecreasingSlope && increasingSlope > this.prevIncreasingSlope){
	            	return true;
	            }
			}
			return false;
		},
		_scrollToView: function(menuItem, menu, isTop){
			// menu = menu.is("div") ? menu.find("ul:first") : menu; // No I18N
	        var listScrollTop = menu.scrollTop(),
	        	currentItemHeight = menuItem.height(),
	        	currentTopPosition = menuItem.position().top,
	        	listHeight = menu.outerHeight(),sval = listScrollTop;
	        if((listHeight - currentTopPosition) < currentItemHeight) { // scrolling forward in a menu
	         	// sval = listScrollTop + (currentTopPosition - listHeight/2 - currentItemHeight);
	         	sval = listScrollTop + currentTopPosition; // considering the currentTopPosition & listScrollTop alone
	        }
	        else if (currentTopPosition < 0) {  // Scrolling backward
	           	sval = listScrollTop + currentTopPosition - currentItemHeight; // fix for setting zero to scrolltop because of negative values in currentTopPosition.
	        }
	        // menu.animate({scrollTop: sval},{"duration": 100,"easing": "linear"});	// No I18N
	        menu.scrollTop(sval); // animation is removed because in fast repeat keyboards animation produces incorrect results.
		},
		_handleMouseOverOnMenuItems: function(menuItem,event,scrollToView,triggerEvent,fromLoad){
			if(menuItem.is(".is-disabled") || menuItem.is(".zmenu__title")){ // No I18N
				this._lastOverlayedMenuItem && this._lastOverlayedMenuItem.removeClass("on-hover");
				return;
			}
			if(scrollToView){
				this._scrollToView(menuItem, menuItem.closest(".zmenu").is("ul") ? menuItem.closest(".zmenu") : menuItem.closest(".zmenu").find("ul:first")  );	// No I18N
			}
			if(menuItem.is(".on-hover") && this._isMenuOfMenuItemShown(menuItem)){ // No I18N
				this._setLastOverlayedMenuItem(menuItem);
				return;
			}
			if(fromLoad || (triggerEvent && this._triggerEvent("ONMENUITEMHOVERED", event, menuItem.closest(".zmenu"), menuItem))){ // No I18N
				this._checkAndHideAllOpenedSubMenus(menuItem.closest(".zmenu")); // No I18N
				this._setLastOverlayedMenuItem(menuItem);
				this._clearMenuTimeout(); 
				menuItem.addClass("on-hover");
				var isDirectlyFromMouseEvent = event ? this._isDirectEvent(event) : false,
					base = this,
					showSubMenuAndTriggerEvent = function(){
						if(this.getData(menuItem).openOn !== "click" && menuItem.data("submenu") && isDirectlyFromMouseEvent){ // No I18N
							this._checkAndHideAllOpenedSubMenus(menuItem.closest(".zmenu"),true); // No I18N
							this._showSubmenu(menuItem,{"event":event},false,event); // No I18N
						}else{
							this.getData(this.element).constantLoc = undefined; // fix - Not clearing constantLoc creates problem in hoverOnMenuItem after menu is destroyed.
						}
					};
				if(menuItem.is(".zmenu--split")){ // No I18N
					window.setTimeout(function(){ // Hover delay for split menu item
						if(menuItem.hasClass("on-hover")){
							menuItem.find(".zmenu__icon.h-alignright").addClass("on-hover"); // No I18N
							showSubMenuAndTriggerEvent.bind(base)();		
						}
					},150);
				}else{
					showSubMenuAndTriggerEvent.bind(this)();
				}	
			}	
		},
		_showSubmenu: function(menuItem, options, showImmediately,eventObj){
			var base = this;
			if(!options.forElement){
				options.forElement = menuItem;
			}
			var f = function(){
				if(menuItem.is(":visible")){
					var submenu = base.getSubmenu(menuItem);
					options.direction = submenu.data("zmenu").options.submenuDirection ? submenu.data("zmenu").options.submenuDirection : base._DIRECTIONS.submenu; // No I18N
					base.show(options,submenu,false);
					base._saveMenuPoints(eventObj, menuItem); // Save Menu Points when the last menu chain is updated. ie. after show().
				}			
			};
			if(showImmediately){
				f();
			}else{
				base._menuTimeout = setTimeout(f, 100);
			}
		},
		_clearMenuTimeout: function(){
			if(this._menuTimeout){
				clearTimeout(this._menuTimeout);
				this._menuTimeout = undefined;
			}
		},
		_handleMouseOutOnMenuItems: function(menuItem,e){
			var isDirectlyFromMouseEvent = this._isDirectEvent(e);
			if(isDirectlyFromMouseEvent){
				if(!menuItem.parent().has(e.relatedTarget).length){
					if(menuItem.data("submenu") && this._isMenuOfMenuItemShown(menuItem)){
						return;
					}
					this._setLastOverlayedMenuItem(undefined);
				}			
				return;
			}
			var relatedEle = e.data;
			if(this._isMenuOfMenuItemShown(menuItem) && this._isElementInMenu(relatedEle, $(this.getSubmenu(menuItem)))){
				return;
			}
			this._clearMenuTimeout();
			menuItem.removeClass("on-hover"); // No I18N
			if(menuItem.is(".zmenu--split")){ // No I18N
				menuItem.find(".zmenu__icon.h-alignright").removeClass("on-hover"); // No I18N
			}
			if(menuItem.data("submenu")){ 
				var smenu = this.getSubmenu(menuItem);
				if(!smenu || this.getData(menuItem).openOn === "click"){
					return;
				}
				if(!this._isElementInMenu(relatedEle, $(smenu))){
					this.hide(smenu); // hiding the sub menu.
				}
			}
		},
		_isMenuOfMenuItemShown: function(menuItem){
			return (menuItem.data("submenu") && $(this.getSubmenu(menuItem)).is(":visible")); // No I18N
		},
		_isDirectEvent: function(e){
			return !!e.originalEvent;
		},
		_hideMenuItem: function(menuItem,show){
			var menuItem = (typeof menuItem === "string") ? $("#"+menuItem, this.element) : $(menuItem); // No I18N
			if(menuItem.length){
				menuItem[show ? 'show': 'hide'](); // No I18N
				menuItem.attr({"hidden": show ? false : true,"aria-hidden": show ? false : true}); // No I18N
			}
		},
		_checkMenuItem: function(menuItem,checked,data){
			if(data.itemType === "radio"){ // No I18N
				this._handleRadioChecked(menuItem,checked,data);
			}else if(data.itemType === "checkbox"){   // No I18N
				this._handleCheckboxChecked(menuItem,checked,data);
			}
		},
		_checkAndHideAllOpenedSubMenus: function(menu,hideAll,callback){

			var items = menu.find("li");
			for(var i = 0; i < items.length; i++){
				var item = $(items[i]),
					subMenu = this.getSubmenu(item),
					openOnClick = subMenu && this.getData(item).openOn == "click"; // No I18N
				if(subMenu && subMenu.is(":visible") && (!openOnClick || hideAll)){ // No I18N
					this._checkAndHideAllOpenedSubMenus(subMenu,hideAll,callback); // hiding all the submenus of submenu.
					if(subMenu.data("parent") && this.getData(subMenu.data("parent")).openOn === "click"){ // No I18N
						menu.zmenu("hide",subMenu,callback);  // No I18N
						// item.trigger("click"); // No I18N
					}else{
						item.trigger("mouseleave"); // No I18N
					}
				}
			}
		},
		_setLastOverlayedMenuItem: function(menuItem, noTrigger){
			if(this._lastOverlayedMenuItem && menuItem && (this._lastOverlayedMenuItem.get(0) === menuItem.get(0))){
				return;
			}
			if(this._lastOverlayedMenuItem && menuItem && menuItem.length && !this._lastOverlayedMenu.has(menuItem.get(0)).length){ // Split menu case
				var submenu = menuItem.data("submenu"); // No I18N
				if(!submenu || submenu[0] !== this._lastOverlayedMenu[0]){
					var par = this._lastOverlayedMenu.data('parent'); // No I18N
					if(par && par.parent().has(menuItem[0]).length){
						par.trigger("mouseleave", menuItem); // No I18N
					}
				}
			}
			if(this._lastOverlayedMenuItem && !noTrigger){
				this._lastOverlayedMenuItem.trigger("mouseleave", menuItem); // No I18N
			}
			this._lastOverlayedMenuItem = menuItem;
			if(menuItem && menuItem.length){
				var __menu = menuItem.closest(".zmenu"),  // No I18N
					currentMenu = this.element;
				this.updateLastMenuChain(__menu);			
				this.getData(currentMenu)._lastOverlayedMenuItem = menuItem;
				while(currentMenu.data("isSubMenu"))	// No I18N
				{
					var par = this.getData(currentMenu).parent;
					currentMenu = par.closest(".zmenu"); // No I18N
				}
				this.getData(currentMenu)._lastOverlayedMenuItem = menuItem;	
			}
		},
		_fixFloatingMenuWidth: function(menu){
			var options = menu.data("zmenu").options, // No I18N
				type = options.type,
				columns = options.columnCount || options.columns;
			if(!columns || columns === -1){
				var _menu = menu.closest('.zmenu'); // No I18N
				columns = (_menu.data("isExternalSubmenu")) ? (this.getData(_menu).columns ? this.getData(_menu).columns :  this.getData(_menu).columnCount ): this.options.columns; // No I18N
				if(!columns || columns === -1){
					return;
				}
			}
			var items = this._getAllMenuItems(menu, false, false, true),
				itemWidth = $(items[0]).outerWidth(true),
				floatingMenuWidth = 0;
	        	// floatingMenuWidth = itemWidth * columns;

			for (var i = 0; i < columns; i++) {    
				floatingMenuWidth += $(items[i]).outerWidth(true);  
			}
			menu.width(floatingMenuWidth); // setting width here in order to retrieve correct height before menu show.
			if((menu.data("height") <= menu.outerHeight()) || (menu.hasClass("h-scroll_y") || menu.find(".h-scroll_y").length)){ // No I18N
				floatingMenuWidth = floatingMenuWidth + this.scrollWidth;  // adding scroll bar width
				menu.width(floatingMenuWidth);
			}
	        menu.data('width', floatingMenuWidth);	// No I18N
	        if(this.getData(menu).search){
	        	menu.data('container').width(floatingMenuWidth).find('input').width(floatingMenuWidth); // No I18N
	        }
		},
        /*
            Below functions added for Docs backward compatibility
        */
        getItemCount: function(){
            return $(this.element).children('.zmenu__item').length;
        },
		/** 
			Removes the given menu items from a menu.
			@function menu.removeMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("removeMenuItems",["#menuItemId1","#menuItemId2",...]);
		*/
		removeMenuItems: function(menuItems){ // menuItems can be an array of id's or selector containing elements.
			var base = this, item;
			if(!menuItems){
				this.element.empty();
				if(this.element.is("div")){ // No I18N
                    if(this._pointerElement){
                        this.element.append(this._pointerElement);
                    }
					this.element.append($("<ul class='zmenu__list'>"));
				}
			}else{
				for(var i = 0;i < menuItems.length; i++){
					item = $(menuItems[i],this.element);
                    if(item.length){
                        base.removeMenuItem(item);
                    }
				}				
			}
		},
		/** 
			Removes the given menu item from a menu.
			@function menu.removeMenuItem
			@param {(String|Object)} menuItem - Identifier of the item or the item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("removeMenuItem","#menuItemId");
		*/
		removeMenuItem: function(menuItem){
			menuItem = $(menuItem,this.element); 
            if(!menuItem.length){
                return;
            }
			menuItem.remove();
			// Removing the group label if no element of that particular group is visible
			var groups = this.element.find("li.zmenu__title"), // No I18N
				base = this;
			for(var i = 0; i < groups.length; i++){
				var id = $(groups[i]).attr("id"),
					items = base.element.find("li[name="+id+"]:not(.zmenu__title):visible"); // No I18N
				if(!items.length){
					$(groups[i]).remove();
				}
			}
			if(this.element.find(".zmenu__footer") && !this.element.find(".zmenu__footer li").length){ // No I18N
				this.element.find(".zmenu__footer").remove(); // No I18N
			}
			this._fixMenuUI(this.element);
			this._resetMenuTextArray(this.element);
		},
		/** 
			Returns the menu item present in the specified index of the menu.
			@function menu.getMenuItemAt
			@param {Number} index - The index of the menu item to retrieve.
			@param {Object} [menu] - The menu which contains the menu item.
			@param {Boolean} [returnAsObject] - Specifies whether to return the menu item details as a JSON object or not.
			@example <caption> Example: </caption>
var item = $("menuElement").zmenu("getMenuItemAt",2);
		*/
		getMenuItemAt: function(index, menu, returnAsObj){
			menu = menu ? menu : this.element;
			if(returnAsObj){
				return this._getMenuItemAsObject(menu.find(".zmenu__item:eq("+index+")")); // No I18N
			}else{
				return menu.find(".zmenu__item:eq("+index+")"); // No I18N
			}
		},
		_getMenuItemAsObject: function(menuItem){
			var mdata = {},
				data = this.getData(menuItem),
				base = this;
			$(this._attrNames).each(function(i, name){
				var attrVal = menuItem.attr(name) || data[name];
				if(attrVal !== undefined){
					mdata[name] = attrVal;
				}
			});		
			for(var i in data){
				if($.inArray(i, base._attrNames) === -1 && i !== "menuItems"){ // No I18N
					if(data[i] !== undefined){
						mdata[i] = data[i];
					}
				}
			}
			if(mdata.itemType == "custom" || mdata.custom){ // No I18N
				mdata.innerHTML = menuItem[0].innerHTML;
			}else if(mdata.label === undefined){
				mdata.label = menuItem.find(".zmenu__text").length && menuItem.find(".zmenu__text").text();
			}
			mdata.text = undefined; // text is made undefined because text variable affects handlebars template rendering.
			if(menuItem.data("submenu")){
				var subMenu = this.getSubmenu(menuItem);
				if(subMenu){
					mdata.menuItems = this._getMenuAsObject(subMenu);
				}
			}
			return mdata;
		},
		_getMenuAsObject: function(menu){
			menu = menu ? menu : this.element;
			var base = this,menuArr = [],
				menuItems = menu.find("li"); // No I18N
			menuItems.each(function(index, menuItem){
					menuArr.push(base._getMenuItemAsObject($(menuItem)));
			});
			return menuArr;	
		},
		/** 
			Add the menu items whose details are given in the first argument and executes the callback function after adding the menu item into the menu.
			@function menu.addMenuItems
			@param {Array} menuItems - An array containing details of all the menu items to add.
			@param {Object} callbackFunction - Callback function to execute after adding each menu item.
			@example <caption> Example: </caption>
$("menuElement").zmenu("addMenuItems",[{"label":"Save As","icon":"icon-save-as"}], js-function);
		*/
		addMenuItems: function(menuItemArray,itemCallback, appendOnce){ // menuItem details are given as json object.
			var items = [],
				base = this,
				menuItemArr = Object.create(menuItemArray),
				menuItem = undefined, len = menuItemArr.length;
				for(var i = 0;i < len; i++){
					menuItem = base.addMenuItem(menuItemArr[i],itemCallback,true,appendOnce);
					items.push(menuItem);
				}
				if(appendOnce){
					this.element.find("ul.zmenu__list:first").append(items);
				}
			if(this.element.is(":visible") || this.options.renderInPlace){ // No I18N
				this._fixMenuUI(this.element);
				this._resetMenuTextArray(this.element);
			}
			return items;
		},
		/** 
			Adds the menu item using the details given in the first argument and executes the callback function after adding the menu item into the menu.
			@function menu.addMenuItem
			@param {Object} menuItemObject - Object containing details of the menu item.
			@param {Object} callbackFunction - Callback function to execute after adding the item.
			@example <caption> Example: </caption>
$("menuElement").zmenu("addMenuItem",{"label":"Save As","icon":"icon-save-as"}, js-function);
		*/
		addMenuItem: function(menuItemObj,itemCallback,isInternal,appendOnce){
			var menuItem = undefined,
				menu = $(this.element);
			if(typeof menuItemObj === "object"){ // No I18N
				menuItemObj = Object.create(menuItemObj);
			}
			if((menuItemObj.menu && menuItemObj.menu.length && typeof menuItemObj.menu === 'object')){   // No I18N
			// Adding submenu items - menuItemObj.menu will hold the submenu
				if(!$(menuItemObj.menu)[0].tagName === "UL"){ // No I18N
	                menu = $(menuItemObj.menu).children('ul'); // No I18N
	            }else{
			        menu = menuItemObj.menu;
	            }
	        }
			var container = menu[0].tagName === "UL" ? false : true; // No I18N
	        if(!hasHandlebars){ // For applications which doesnot use handlebars.
	        	menuItem = $(this._menuItemUsingElementCreation(menu,menuItemObj,itemCallback));
	        }else{
	      	  	menuItem = this._menuItemUsingTemplate(menu,menuItemObj,itemCallback);
	        }
	        if(!appendOnce){			
				var allItems = menu[0].querySelectorAll("li"); // No I18N
				if(typeof menuItemObj.index !== "undefined" && allItems.length){ // No I18N 
				// Menu is not empty and we are adding a single item at the specified position
					var insertAfter = false;
					if(menuItemObj.index >= allItems.length){
						menuItemObj.index = allItems.length-1;
						insertAfter = true;
					}
					if(menuItemObj.index < 0){
						menuItemObj.index = 0;
					}
					menuItem[insertAfter? "insertAfter": "insertBefore"]($(allItems[menuItemObj.index]));	// No I18N
				}else{
					container ? menu[0].querySelector("ul").appendChild(menuItem[0]) : menu[0].appendChild(menuItem[0]); // No I18N
				}
			}
	        if(typeof menuItemObj.menuItems !== 'object' && itemCallback && menuItemObj.mtype !== "separator"){ // No I18N
	        	itemCallback(menuItem,menuItemObj);	
			}
			var attrName = undefined;
			for(var i in this._attrNames){
				attrName = this._attrNames[i];
				if(menuItemObj[attrName]){
					menuItem[0].setAttribute(attrName, menuItemObj[attrName]+""); // No I18N
				}
			}
			var data = this.getData(menuItem);
			for(var i in menuItemObj){  // Placing Other attributes in data attribute
				if($.inArray(i, this._attrNames) === -1 && i !== "menuItems" && i !== "action"){ // No I18N
					if(menuItemObj[i]){
						data[i] = menuItemObj[i];
					}
				}
			}
			if(menuItemObj.action && typeof menuItemObj.action === "string" && menuItemObj.action.indexOf("URL:") !== -1){ // No I18N
				data.action = menuItemObj.action; 
			}
			if(menuItemObj.checked){
				menuItem.data('checked', menuItemObj.checked); // No I18N
				menuItem.attr({"data-checked":menuItemObj.checked,"aria-checked":menuItemObj.checked}); // No I18N
			}
			if(menuItemObj.hidden){
				menuItem.attr({"data-hidden":menuItemObj.hidden,"aria-hidden":menuItemObj.hidden}); // No I18N
			}
			if(menuItemObj.disabled){
				menuItem.attr({"data-disabled":menuItemObj.disabled,"aria-disabled":menuItemObj.disabled}); // No I18N
			}
			if(menuItemObj.customAttributes){
                if(menuItemObj.customAttributes.data){
                    var attrVal = menuItemObj.customAttributes.data;
                    if(typeof attrVal === "string"){ // No I18N
                        attrVal = JSON.parse(attrVal);
                    }
                    if(attrVal.appData){
                        this.getData(menuItem).appData = attrVal.appData;
                    }
                }
				menuItem.attr(menuItemObj.customAttributes);
			}
			if(!menuItemObj.internal && !hasHandlebars){ // Build the menu items of menu and not submenu items
				this._buildMenuItem(menuItem, menu);
			}else{
				menuItem.attr("role", this.ARIA_ROLES[menuItemObj.itemType ?  menuItemObj.itemType : "default"]); // No I18N
				if(menuItemObj.innerHTML){
					var content = menuItemObj.innerHTML.content,
						replaceArray = menuItemObj.innerHTML.replaceArray;
					if(content && replaceArray){
						for (var i = 0; i < replaceArray.length; i++) {
	                    	content = content.replace('{'+ i +'}',ZC.encodeHTML(replaceArray[i]));	// No I18N
	                	}
	                	menuItem.html(content);
					}else{
						if(this.options.isCustomHTMLEncoded || this.options.htmlSafe){
							menuItem.append(data.innerHTML); 
						}else{
							menuItem.html(ZC.encodeHTML(data.innerHTML));
						}
					}
					if(menuItemObj.hasSubmenu){
						this._createArrowIcon(menuItem);
					}
				}
				this._handleCheckedandDisabled(menuItem,menuItemObj);
			}
			var extAsso = menuItemObj.itemValue;
			if(extAsso){
				menuItem.data('itemValue',extAsso); // No I18N
			}
			var splitButtonData = menuItemObj.splitButtonData;
			if(splitButtonData){
				menuItem.data('relatedButton', splitButtonData); // No I18N
			}
			this.getData(menuItem).action = menuItemObj.action;
			if(menuItemObj.fixed || menuItemObj.isFooter){
				this._addFixedItem(menuItem,menu,data);
			}
			if(!isInternal){
				this._fixMenuUI(menu);
			}
			var className = menuItemObj.className ? menuItemObj.className : (menuItemObj.itemCssClass ? menuItemObj.itemCssClass : ""); // No I18N
			return menuItemObj.isFooter ? null : menuItem.addClass(className);
		},
		_menuItemUsingTemplate: function(menu,menuItemObj,itemCallback){
			menuItemObj.listClass = (menuItemObj.itemType === "itemgroup") ? "zmenu__title" : "zmenu__item"; // No I18N
			menuItemObj.icon = menuItemObj.SVGIconId ? menuItemObj.SVGIconId : (menuItemObj.iconClassName ? menuItemObj.iconClassName : menuItemObj.icon);
			if(menuItemObj.icon){
				if(menuItemObj.icon.indexOf("#") !== -1){ // No I18N
					menuItemObj.svgId = menuItemObj.icon.split(" ")[0]; // No I18N
					menuItemObj.svgClass = menuItemObj.icon.slice(svgId.length).trim();
				}
			}
			menuItemObj.isGrid = this._isGridMenu(menu);
			menuItemObj.isSeparator = (menuItemObj.itemType === "separator") ? true : false; // No I18N
			menuItemObj.isCustom = (menuItemObj.itemType === "custom" || menuItemObj.custom) ? true : false; // No I18N
			if(menuItemObj.itemType === "splitmenu"){ // No I18N
				menuItemObj.listClass += " zmenu--split"; // No I18N
			}
			if(typeof menuItemObj.menuId === "string"){ // No I18N
				menuItemObj.hasSubmenu = true;
				menuItemObj.submenuId = menuItemObj.menuId.substr(1);
			}else if(menuItemObj.menuItems && menuItemObj.menuItems.length){
				menuItemObj.submenuId = menuItemObj.id+"submenu"; // No I18N
				menuItemObj.submenu = $("<ul>").attr('id',menuItemObj.submenuId); // No I18N
				menuItemObj.hasSubmenu = true;
				menuItemObj.contentType = menuItemObj.contentType ? menuItemObj.contentType : "text"; // No I18N
				this.getData(menuItemObj.submenu).contentType = menuItemObj.contentType;		
				var base = this;
				$(menuItemObj.menuItems).each(function(i, _menuItemObj){ // menuItems contain array of menu items details.
					_menuItemObj.menu = menuItemObj.submenu;
					_menuItemObj.internal = true;
					base.addMenuItem(_menuItemObj, itemCallback, undefined);
				});
				menuItemObj.submenu.addClass("zmenu "+this._MENU_CONTENT_TYPES[menuItemObj.contentType ? menuItemObj.contentType : "text"]).zmenu({"usingJSON":true}); // No I18N
				$('body').append(menuItemObj.submenu);
			}
			var menuType = menu.data('zmenu') ? menu.data('zmenu').options.contentType : this.getData(menu).contentType;  // No I18N
			menuItemObj[menuType === "icon" ? "icon-only" : menuType] = menuType; // No I18N
			var menuItem = $(menuItemJsonTemplate(menuItemObj));
			if(menuItemObj.isGroup){
				menuItem.append(menuItemObj.submenu);
			}else if(menuItemObj.menuItems && menuItemObj.menuItems.length){
				menuItem.attr("data-menu-id",menuItemObj.submenuId); // No I18N
				menuItemObj.submenu.data("parent",menuItem); // No I18N
			}
			menuItemObj.label && menuItem.attr("data-label",menuItemObj.label); // No I18N
			this.getData(menuItem).parentMenu = menu;
			if(menuItemObj.icon && menuItemObj.icon.indexOf("text:") === 0 && this.options.contentType !== "text"){  // No I18N
				menuItem.find("i:first").removeClass(menuItemObj.icon).text(menuItemObj.icon.replace("text:","")); // No I18N
			}
			if(!menuItem.is(".zmenu__separator") && !menuItem.is(".zmenu__title")){ // No I18N 
				// Binding events for menu items
				this._addEventsForMenuItems(menuItem);
			}
			if(menuItemObj.action && typeof menuItemObj.action === "string" && menuItemObj.action.indexOf("URL:") !== -1){ // No I18N
				var menuItemContents = menuItem.html();
				menuItemObj.target = menuItemObj.target ? menuItemObj.target : "_blank"; // No I18N
				menuItem.empty().addClass("zmenu--withlink").append($("<a class='zmenu__link' href='"+menuItemObj.action.replace("URL:","")+"' target='"+menuItemObj.target+"'></a>")); // No I18N
				menuItem.find(".zmenu__link").append(menuItemContents);
			}
			if(menuItemObj.hasSubmenu){
				menuItem.data("submenu", menuItemObj.submenu ? menuItemObj.submenu : $(menuItemObj.submenuId) ); // No I18N 
			}
			return menuItem;
		},
		_menuItemUsingElementCreation: function(menu,menuItemObj,itemCallback){
			var menuItem = document.createElement("li");
			menuItemObj.itemType && menuItem.setAttribute("data-item-type",menuItemObj.itemType);
			menuItemObj.name && menuItem.setAttribute("name",menuItemObj.name);
			if(typeof menuItemObj.menuItems === "object"){ // No I18N
				var className = menuItemObj.className ? menuItemObj.className : ( menuItemObj.menuClass ?  menuItemObj.menuClass : ''),
					subMenu = $("<ul>").attr({"id":menuItemObj.id+"submenu","class":className}),
					base = this;
				// $("body").append(subMenu);
				menuItem.appendChild(subMenu[0]);
				// menuItem.data("menuId",menuItemObj.id+"submenu").attr("data-menu-id",menuItemObj.id+"submenu"); // No I18N
				subMenu.data("usingJSON",true); // No I18N
				this.getData(subMenu).contentType = menuItemObj.contentType ? menuItemObj.contentType : this.getData(menu).contentType;
				$(menuItemObj.menuItems).each(function(i, _menuItemObj){ // menuItems contain array of menu items details.
					_menuItemObj.menu = subMenu;
					base.addMenuItem(_menuItemObj, itemCallback, undefined);
				});
			}
			return menuItem;
		},
		/** 
			Returns a jQuery object containing the element visually representing the menu.
			This method does not accept any arguments.
			@function menu.getElement
			@return {Object} element.
			@example <caption> Example: </caption>
var element = $("menuElement").zmenu("getElement");
		*/
		/** 
			Removes the menu component functionality. This will return the element back to its pre-init state.
			This method does not accept any arguments.
			@function menu.destroy
			@example <caption> Example: </caption>
$("menuElement").zmenu("destroy");
		*/
		destroy: function(menu){	
			var menu = menu ? menu : this.element;
			this._clearMenuTimeout();
			this._uninitSliders(menu);
			this.hide();
			if(menu.data("usingJSON") ||  this.options.usingJSON){ // No I18N
				var submenuItems = menu.find("li[submenuitem=true]"); // No I18N
				for(var i = 0; i < submenuItems.length; i++){
					$(submenuItems[i]).data("submenu").zmenu("destroy"); // No I18N
					$(submenuItems[i]).data("submenu").remove(); 
				}
				menu.empty(); // Json menu contents are removed.
			}
            var componentName = this.componentName;
			menu.off("mouseup."+componentName+" keydown.run."+componentName).removeClass('zmenu zmenu--thumbnail '+this._MENU_CONTENT_TYPES[this.options.contentType] +' '+this.options.menuClass+' '+this.options.className).removeData('zmenu').data('initialized',false); // No I18N
			// Destroying separator items
			menu.find('.h-scroll_y').removeClass('h-scroll_y'); // No I18N
			// pointer element is retained in order to avoid pointer element creation each item. 
			if(menu.find(".zmenu__pointer").length){ // No I18N
				menu.find(".zmenu__pointer").remove();
				this._pointerElement = undefined;
			}
			var listEle = menu.find("li");
			for(var i = 0;i < listEle.length; i++){
				var item = listEle[i],
					itemData = this.getData(listEle[i]),
					text = $(item).find(".zmenu__text").text() || itemData.label, // No I18N
					isFixed = $(item).is(".is-fixed"); // No I18N
				$(item).removeData("metadata").empty().removeClass("zmenu__item zmenu__title zmenu__custom zmenu__separator is-disabled is-fixed is-selected").empty().removeAttr('data-label').off('mouseenter mouseleave'); // No I18N
				$(item).text(text);
				isFixed && $(item).appendTo(menu.find("ul:first")); // No I18N
			}
			menu.find(".zmenu__header,.zmenu__footer").remove(); // No I18N
			menu.removeData("fixedArea width height"); // No I18N
			this._menuTextArray = [];
		},
		updateLastMenuChain: function(menu, isExternal){
			this._lastOverlayedMenu = menu;
			if(!isExternal && menu[0] !== this.element[0]){
				var bmenu = this._traverseAndFindBaseMenu();
				bmenu.zmenu("updateLastMenuChain", menu, true); // No I18N
			}
		},
		_traverseAndFindBaseMenu: function(menu){
			var _menu = menu;
			menu = menu ? menu : this.element;
			// if(menu.data("isExternalSubmenu") && !retainParent){  // No I18N
			if(menu.data("parent")){  // No I18N
				var par = menu.data('parent'); // No I18N
				while(par){
					var _menu = par.closest(".zmenu"); // No I18N
					if(_menu.is(":visible")){  // No I18N
						menu = _menu;
						par = menu.data('parent'); // No I18N
					}else{
						par = false;
					}
				};
			}
			return menu.closest(".zmenu"); // No I18N
		},
		/* 
			Triggers the click event on the menu item, which is currently hovered. 
			@function menu.selectHoveredMenuItem
			@example <caption> Example: </caption>
$("menuElement").zmenu("selectHoveredMenuItem");
		*/
		selectHoveredMenuItem: function(){
			var item = this._lastOverlayedMenuItem;
			if(item){
                $(item).trigger("mouseup"); // Trigger method is used since event object is not present. Event object is mandatory in this case to use event.preventDefault and event.stopPropagation methods.
			}
			// this.hide(); Commented because hiding the menu can be prevented using itemclick event.
			return item ? item : false;
		},
		/** 
			Triggers the mouseover event on the menu item. 
			@function menu.hoverMenuItem
			@param {String} menuItem -  Specifies the id of the item or html element to hover.
			@example <caption> Example: </caption>
$("menuElement").zmenu("hoverMenuItem",$("selector"));
		*/
		hoverMenuItem: function(item){
            item = $(item);
            if(!item.length){
                this._lastOverlayedMenuItem && this._lastOverlayedMenuItem.removeClass("on-hover"); // No I18N
                this._lastOverlayedMenuItem = undefined;
                return;
            }
			this._handleMouseOverOnMenuItems($(item),{},true,false,true);
			// item.trigger("mouseover"); // No I18N
		},
		_getAllMenuItems: function(menu, excludeFloatingItems, excludeSubMenuItems, isOnlyVisible, excludeseparator){
			if(!menu || !menu.length){
				menu = this.element;
			}
			return menu.find('.zmenu__item:not([itemType=group], .zc-splitmenu-list-rht'+(excludeseparator? ', .zmenu__separator': '') +(excludeFloatingItems? ', .zmenu--thumbnail': '')+(excludeSubMenuItems? ', li ul[mtype=submenu] li': '')+')'+(isOnlyVisible? ':visible': ''));  // No I18N		
		},
		_handleCheckboxChecked: function(menuItem,checked,data){
			if(this._isGridMenu(menuItem, true)){ 
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else if((data.icon || data.iconClassName || data.SVGIconId) && this.options.contentType !== "text"){  // No I18N
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else{
				var iconEle = menuItem.find("i:first");
				if(iconEle.length){
                    this.options.checkedCheckboxItemIconClassName && iconEle.removeClass(this.options.checkedCheckboxItemIconClassName);
                    if(checked){
                        var svgData = {
                            props: {"class":"zmenu__tick","viewBox":"0 0 16 16","width":"100%","height":"100%"}, // No I18N
                            nodes:[{
                                "polygon": {"points":"14.6,2.4 4.9,12.1 1.2,8.5 0.4,9.3 4.9,13.8 15.4,3.2 "} // No I18N
                            }]
                        };
                        this._handleIcon(iconEle, svgData, "checkedcheckbox", this.options.checkedCheckboxItemIconClassName, this.options.checkedCheckboxItemSVGIconId);
                    }else{
                       iconEle.find("svg").remove(); // No I18N
                    }
				}
			}
			menuItem.prop("checked",checked).attr({"checked":checked,"aria-checked":checked}); // No I18N
		},
		_handleRadioChecked: function(menuItem,checked,data){
			var name = $(menuItem).attr("name"), // No I18N
			    menu = menuItem.closest('ul'), // No I18N
			    base = this,metaInfo = undefined,
			    radios = menu.find("li[role=menuitemradio][name="+name+"]"); // No I18N
		    if(checked){
				radios = radios.filter(function(){
					return $(this).prop('checked');
				});
				radios.each(function(i, radio){
					radio = $(radio);
					metaInfo = base.getData(radio);
					base._fixRadioIcon(radio, false, metaInfo);
				});
		    }		
			base._fixRadioIcon(menuItem, checked, data);
		},
		_fixRadioIcon: function(menuItem,checked,data){
			var isGrid = this._isGridMenu(menuItem, true);
			if(isGrid){
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else if(data.icon || data.iconClassName || data.SVGIconId){
				menuItem[checked ? "addClass" : "removeClass"]("is-selected"); // No I18N
			}else{
                var iconEle = menuItem.find("i:first");
                if(iconEle.length){
                    this.options.checkedRadioItemIconClassName && iconEle.removeClass(this.options.checkedRadioItemIconClassName);
                    if(checked){
                        var svgData = {
                            props: {"class":"zmenu__radio","viewBox":"0 0 48 48","width":"100%","height":"100%"}, // No I18N
                            nodes:[{
                               "circle":{"cx":"24.5","cy":"24.5","r":"15.5"} // No I18N
                            }]
                        };
                        this._handleIcon(iconEle, svgData, "checkedradio", this.options.checkedRadioItemIconClassName, this.options.checkedRadioItemSVGIconId);
                    }else{
                       iconEle.find("svg").remove(); // No I18N
                    }
                }
			}
			menuItem.prop("checked",checked).attr({"checked":checked,"aria-checked":checked}); // No I18N
		},
		_initRuntimeEvents: function(){
			var base = this, componentName = this.componentName;
			this.element.off("keydown.run."+componentName).on("keydown.run."+componentName,function(event){ // No I18N
				base.handleKeyCode(event.keyCode,event,undefined);
				if((!event.ctrlKey && !event.metaKey && !event.shiftKey && !event.altKey) || (event.keyCode === ZC.keyCode.UP || event.keyCode === ZC.keyCode.DOWN || event.keyCode === ZC.keyCode.LEFT || event.keyCode === ZC.keyCode.RIGHT)){
					// checking whether the key pressed is a short cut key or not. Shortcutkey events should not be prevented. 
					event.preventDefault();
					event.stopPropagation();
				}		
			});
		},
		handleKeyCode: function(code,event,showSubMenu){
			var _isFloatingMenu = this._isGridMenu();
			event = event ? event : {};
			switch(code){
				case ZC.keyCode.UP:
						if(event.altKey || event.ctrlKey || event.metaKey){ // page up
							var item = this._lastOverlayedMenu.find("li:visible:not('.zmenu__title,.is-disabled'):first"); // No I18N
							this.hoverMenuItem(item,true);
						}else{
							var nextItem = this._getNextMenuItem(true);
							if(nextItem.length){
								this._scrollToView(nextItem,nextItem.closest(".zmenu").is("ul") ? nextItem.closest(".zmenu") : nextItem.closest(".zmenu").find("ul:first") ); // No I18N
								nextItem.trigger('mouseover'); // No I18N
							}
						}					
					break;
				case ZC.keyCode.DOWN:
						if(event.altKey || event.ctrlKey || event.metaKey){ // page down
							var item = this._lastOverlayedMenu.find("li:visible:not('.zmenu__title,.is-disabled'):last"); // No I18N
							this.hoverMenuItem(item,true);
						}else{
							var nextItem = this._getNextMenuItem();
							if(nextItem.length){
								this._scrollToView(nextItem,nextItem.closest(".zmenu").is("ul") ? nextItem.closest(".zmenu") : nextItem.closest(".zmenu").find("ul:first")); // No I18N
								nextItem.trigger('mouseover'); // No I18N
							}
						}
						break;
				case ZC.keyCode.RIGHT:
					if(this._lastOverlayedMenuItem !== undefined){
						var subMenu = this.getSubmenu(this._lastOverlayedMenuItem);
						var menu = this._lastOverlayedMenuItem.closest(".zmenu"); // No I18N
						if((_isFloatingMenu || !subMenu.length && menu.hasClass("zmenu--thumbnail")) && !showSubMenu){ // No I18N - Traversing floating menu items which doesn't have any submenus
							var f = undefined,
								item = this._getNextMenuItem(f, f, f, code);
							this._scrollToView(item,item.closest(".zmenu").is("ul") ? item.closest(".zmenu") : item.closest(".zmenu").find("ul:first") ); // No I18N
							item.trigger("mouseover"); // No I18N
							break;
						}
						if(this._lastOverlayedMenuItem.is(".is-disabled")){ // No I18N
							return;
						}
						if(subMenu && subMenu.length){
							var menu = this.getSubmenu(this._lastOverlayedMenuItem); // Submenu was already extracted -- check again
							if(!menu){
								return;
							}
							this.show({"forElement": this._lastOverlayedMenuItem}, menu); // No I18N
							this._lastOverlayedMenuItem = undefined;
							this._getNextMenuItem(false, true).trigger("mouseover"); // No I18N
						}
					}else if(_isFloatingMenu){
						this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):not(':hidden'):first").trigger("mouseover"); // No I18N
					}
					break;
				case ZC.keyCode.LEFT:
					if(this._lastOverlayedMenuItem !== undefined){ 
						// Fix for split menu left click
						menu = this._lastOverlayedMenuItem.closest(".zmenu"); // No I18N
						// Fix to hide the submenu of floating menu item.
						var parentItem = this._lastOverlayedMenu.data('parent'),  // No I18N
							subMenuShown = parentItem && this._isMenuOfMenuItemShown(parentItem),
							firstMenuItem = menu.find(":visible.zmenu__item").not(".zmenu__separator,.is-disabled,.zmenu__title").filter(":first"), // No I18N
							// _isFirstItem = this._lastOverlayedMenuItem.index() === 0,
                            _isFirstItem = firstMenuItem[0] == this._lastOverlayedMenuItem[0],
							currentMenuFloating = this._isGridMenu(menu);
						if(currentMenuFloating && !_isFirstItem){
							if(!subMenuShown || !_isFirstItem){ 
								var item = this._getNextMenuItem(true, undefined, undefined, code);				
								this._scrollToView(item,item.closest(".zmenu").is("ul") ? item.closest(".zmenu") : item.closest(".zmenu").find("ul:first") ); // No I18N
								item.trigger("mouseover"); // No I18N
								break;
							}
						} 
						if(!this._isMenuMainMenu(this._lastOverlayedMenu)){ // Pressing left from submenu will hide the submenu.
							this._hideLastShownMenu();
						}
					}else if(_isFloatingMenu){
						this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):not(':hidden'):last").trigger("mouseover"); // No I18N
					}
					break;
				case ZC.keyCode.ESCAPE:
					if(!this.options.renderInPlace){
						if(!this._isMenuMainMenu(this._lastOverlayedMenu)){
							this._hideLastShownMenu();
						}else{
							this.hide();
						}
					}
					break;
				case ZC.keyCode.ENTER:
				case ZC.keyCode.SPACE:
					if(this._lastOverlayedMenuItem && this._lastOverlayedMenuItem.data("submenu") && !this._lastOverlayedMenuItem.hasClass("zmenu--split")){ // No I18N
						this.handleKeyCode(ZC.keyCode.RIGHT,event,true);
					}else{	
						if(this._lastOverlayedMenuItem){
                            if(this._lastOverlayedMenuItem.is(".zmenu--withlink")){ // No I18N
                                $(this._lastOverlayedMenuItem).find(".zmenu__link")[0].click(); // No I18N
                            }else{
                                event.target = $(this._lastOverlayedMenuItem); // target is modified since actual target will be menu element.
                                this._handleMenuItemClick(event);
                            }
  						}else{
           					this.hide();
  						}
						// if(this.selectHoveredMenuItem() && !this._lastOverlayedMenuItem  || (this._lastOverlayedMenuItem && !this._lastOverlayedMenuItem.is("[data-item-type='checkbox']"))){ // No I18N
						// 	// checkbox menu should not be closed.
						// 	this.hide();
						// }
					}
					break;
				case ZC.keyCode.PAGE_UP:   var item = this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):first"); // No I18N
													this.hoverMenuItem(item,true);
													break;
				case ZC.keyCode.PAGE_DOWN: var item = this._lastOverlayedMenu.find("li:not('.zmenu__title,.is-disabled'):last"); // No I18N
													this.hoverMenuItem(item,true);
												    break;
				default:
		            var alphaNumericKey = String.fromCharCode(code);
		            if(this._menuTextArray.length && this.options.handleAccessKeyNavigation){
						this._callMenuitemSearch(alphaNumericKey,(new Date()).getTime());
					}
				break;
			}
		},
		_isDisabled : function(menuItem){
			var disabled = menuItem.attr("disabled")? menuItem.attr("disabled"): false; // No I18N
			return disabled;
		},
		_isMenuMainMenu : function(menu){
			return ($(menu).get(0) === this.element.get(0));
		},
		_getNextFloatingMenuItem : function(currentItemPos, items, isPrev, compareTop, isReverse){
			var next = undefined,
				key = compareTop ? "top" : "left", // No I18N
				total = items.length;
			$.each(items, function(i, item){
				item = items[isReverse ? total-i-1 : i];
				var ipos = $(item).position();
				if(ipos[key] === currentItemPos[key]){
					next = $(item);
					return false;
				}else{
					// Navigation to next row items (example: open office application)
					if(key === "top"){ // No I18N
						next = $(item);
						return false;
					}
				}
			});
			return next;
		},
		_hideLastShownMenu: function(){
			var nextItem = this._lastOverlayedMenu.data('parent'); // No I18N
			this._lastOverlayedMenuItem = nextItem;
			this.hide(this._lastOverlayedMenu);
			var __menu = nextItem.closest(".zmenu"); // No I18N
			this.updateLastMenuChain(__menu);
			this.getData(this.element)._lastOverlayedMenuItem = nextItem;
		},
		_resetMenuTextArray: function(menu){
			this._menuTextArray = [];
	        this._prevSearchKey = ''; // No I18N
	        this._prevKeydownTime = ''; // No I18N
	        this._prevSearchIndex = ''; // No I18N
	        this._prevSearchTimer = undefined; 
			var menuItemSelector = ".zmenu__item:visible:not(.zmenu__title,.is-disabled,.zmenu__separator)";	// No I18N
			if(this.options.renderInPlace){
				menuItemSelector = ".zmenu__item:not(.zmenu__title,.is-disabled,.zmenu__separator)"; 	// No I18N
			}
			var menuItems = menu.find(menuItemSelector);
			for (var i = 0; i < menuItems.length; i++) {
				this._menuTextArray[this._menuTextArray.length] = {"text": $(menuItems[i]).find(".zmenu__text").text() || "","menuitem":$(menuItems[i])}; // No I18N
			}
		},
		_callMenuitemSearch: function(alphaNumericKey,currentTime){
			var sameKeySearch = false;
		    if(this._prevKeydownTime !== "" && (currentTime-this._prevKeydownTime)>700){ // No I18N
		    	sameKeySearch = (this._prevSearchKey == alphaNumericKey) ? true :false;
		        this._prevSearchKey = alphaNumericKey;
		    }
		    else{
		        this._prevSearchKey =  this._prevSearchKey + alphaNumericKey;
		    }
			this._prevKeydownTime = currentTime;    
		   	this._menuitemSearch(sameKeySearch);
		},
		_menuitemSearch: function(sameKeySearch){
			var regExp = new RegExp("^"+(this._prevSearchKey).replace(/[|()\[{.+*?$\\]/g, "\\$0"),"i");	// No I18N 
			// i is used to ignore case
			var searchFromIndex = (sameKeySearch && this._prevSearchIndex < this._menuTextArray.length) ? this._prevSearchIndex + 1 : 0 ;
		    searchFromIndex = (searchFromIndex > 0 && searchFromIndex < this._menuTextArray.length) ? searchFromIndex : 0 ;
	        for(var x = searchFromIndex; x < this._menuTextArray.length; x++){
				if((this._menuTextArray[x].text).match(regExp)){                
					this._prevSearchIndex = x;
					var checkNextMatch = false;
					for(var y = ( x+1 ); y < this._menuTextArray.length; y++){
						if((this._menuTextArray[y].text).match(regExp)){ 
							checkNextMatch = true;
						}
				  	}
				  	if(!checkNextMatch){
				  		 this._prevSearchIndex = -1;
				  	}
				  	this._handleMouseOverOnMenuItems($(this._menuTextArray[x].menuitem),{},true,true);
				  	return;
				}
		  	}
		},
		_getNextMenuItem: function(isPrev, noTrigger, prevItem, keyCode){
			var menu = (this._lastOverlayedMenu && this._lastOverlayedMenu.length !== 0) ? this._lastOverlayedMenu: this.element,
				_isFloatingMenu = this._isGridMenu(menu),	
				item = undefined,
				actualMenu = menu.is("ul") ? menu : menu.find("ul:first"); // No I18N
			prevItem = (prevItem !== undefined)? prevItem : this._lastOverlayedMenuItem;
			if(!prevItem || !prevItem.length){
				if(menu.find(".zmenu__footer").length && !this._lastOverlayedMenuItem){
					item = menu.find(".zmenu__item:visible:not(.zmenu__title,.is-disabled,.zmenu__separator):"+(isPrev? "last": "first")); // No I18N
				}else{
					item = actualMenu.find(".zmenu__item:visible:not(.zmenu__title,.is-disabled,.zmenu__separator):"+(isPrev? "last": "first")); // No I18N
				}
			}else{
				item = prevItem[isPrev? "prevAll" : "nextAll"](".zmenu__item:visible").not(".zmenu__separator,.is-disabled,.zmenu__title"); // No I18N	
				if(_isFloatingMenu){
					var prevPos = prevItem.position(),
						compareTop = (keyCode === ZC.keyCode.LEFT || keyCode === ZC.keyCode.RIGHT);
					item = this._getNextFloatingMenuItem(prevPos, item, isPrev, compareTop);
					if(!item){ // Reached end of row or column in menu
						if(!(menu.find(".zmenu__footer").length && compareTop)){ // No I18N
							item = prevItem[!isPrev? "prevAll" : "nextAll"](".zmenu__item:not(.zmenu__separator,.is-disabled,.zmenu__title):visible"); // No I18N
							prevPos[compareTop ? "left" : "top"] = 0; // No I18N
							item = this._getNextFloatingMenuItem(prevPos, item, isPrev, compareTop, true);	
						}					
					}
				}else{
					item = $(item[0]);
				}
				if(!item || !item.length){
					if(menu.find(".zmenu__footer").length){ // No I18N
						if(!prevItem.closest(".zmenu__footer").length){ // No I18N
							item = menu.find(".zmenu__footer").find(".zmenu__item:visible")[isPrev ? "last" : "first"](); // No I18N
						}else if(prevItem[0] === menu.find(".zmenu__footer").find(".zmenu__item:visible").first()[0]){ // No I18N
							// this._lastOverlayedMenuItem = undefined;
							prevItem.removeClass("on-hover"); // No I18N
							item = this._getNextMenuItem(isPrev, noTrigger, false);
						}else{
							item = this._getNextMenuItem(isPrev, noTrigger, false);
						}
					}else{
						item = this._getNextMenuItem(isPrev, noTrigger, false);
					}
				}
			}
			this._setLastOverlayedMenuItem(item, noTrigger);
			return this._lastOverlayedMenuItem;
		}, 
		_setFocus: function(){
			var base = this;
			var f = function(){
				base.element.focus();
				if(base.element.find(".zmenu__textbox").length){ // No I18N
					base.element.find(".zmenu__textbox").focus(); // No I18N
				}
			};
			if(this._triggerEvent("ONBEFOREFOCUS", {},this.element)){ // No I18N
				setTimeout(f, 100);
			}
		},
		refresh: function(menu){
			menu = menu? menu: this.element;
			this._fixMenuItemsWidth(menu);
		},
		_fixMenuItemsWidth: function(menu, type, referenceMenu){
			if(!menu){
				return;
			}
			type = !type ? this.getMenuType(menu) : type;
			if(this._isTypeFloating(type)){
				return this._fixFloatingMenuWidth(menu);
			}
		},
		getMenuType: function(menu){
			if(menu && menu.data("isExternalSubmenu")){ // No I18N
				return menu.zmenu("getMenuType"); // No I18N
			}else{
				menu = !menu ? this.element : menu;
				var type = this._getMenuTypeFromData(menu);
				if(!type){	
					type = this.options.type;
				}
				return type;
			}
		},
		_getMenuTypeFromData: function(menu){
			var mdata = this.getData(menu);
			return mdata.componentOptions ? mdata.componentOptions.type : mdata.type; 
		},
		_isTypeFloating: function(type, menu){
			if((type && type === "grid") || (menu && (menu.hasClass('zmenu--thumbnail')))){ // No I18N
				return true;
			}else{
				return false;
			}
		},
		_fixMenuUI: function(menu, options, isSubMenu){
			if(!menu.is(":visible")){  // No I18N
				return;
			}
			this._fixMenuHeight(menu,options);
		},
		_fixMenuHeight: function(menu, options){
			var menuHeight = "auto",  // No I18N
				actualMenu = menu.is("ul") ? undefined : menu.find("ul:first");  // No I18N
			menu.height(menuHeight);
			actualMenu && actualMenu.height(menuHeight);
			menu.removeClass("h-scroll_y"); // No I18N
			var positionAlterable = this.options.direction === "auto" || this.options.direction === "after-pointer" || this.options.direction === "at-cursor", // No I18N
				height = menu.outerHeight(),
				top = menu.css('top').split('px')[0] * 1 - $(window).scrollTop(), // No I18N
				windowHeight = ZC.windowObject ? ZC.windowObject.height : $(window).height(), // window object stored on the ZC is undefined sometimes while trying to show the menu on load events.So included $(window).height()
				openingDirection = menu.data("direction"),  // No I18N
				limit = (openingDirection && openingDirection.indexOf("bottom") === 0 || openingDirection && openingDirection.indexOf("top") === 0 ) ? (options && !options.isRightClick ? options.availableHeight : windowHeight ) : windowHeight, // No I18N
				space = limit - (height),
				top = top ? top : 0;
			if(top + height >= limit && (space < 0) || !positionAlterable){
				menuHeight = (options && options.availableHeight && !options.isRightClick) ? options.availableHeight : (windowHeight - top);
			}
			if(menu.data('height')){ // No I18N
				if(options && options.availableHeight){
					menuHeight = (parseInt(menu.data('height')) >= options.availableHeight) ? options.availableHeight : parseInt(menu.data('height')); // No I18N
				}else{
					menuHeight = parseInt(menu.data('height')); // No I18N
				}
			}else if(space > 0 && positionAlterable){
				menuHeight = height;
			}
			// if(menu.is("div")){
			// 	var headerFooter = menu.find(".zmenu__header").outerHeight() + menu.find(".zmenu__footer").outerHeight();
			// 	menu.height(menuHeight);
			// 	menuHeight = menuHeight - headerFooter;
			// 	menu = menu.find("ul:first");
			// }
			var scrollWidth = this._findScrollWidth(),
				menudata = this.getData(menu);
			if(height > menuHeight){
				// menu.height(menuHeight);
				menu.outerHeight(menuHeight);
				if(actualMenu){
					var headerHeight = menu.find(".zmenu__header").length ? (parseInt(menu.find(".zmenu__header").outerHeight(true)) + parseInt(menu.find(".zmenu__header").css("margin-top"))) + parseInt(menu.find(".zmenu__header").css("margin-bottom")) : parseInt(menu.css("padding-top")), // No I18N
						footerHeight = menu.find(".zmenu__footer").length ? (parseInt(menu.find(".zmenu__footer").outerHeight(true)) + parseInt(menu.find(".zmenu__footer").css("margin-top"))) + parseInt(menu.find(".zmenu__footer").css("margin-bottom")): parseInt(menu.css("padding-bottom")),// No I18N
						pointerHeight = menu.find(".zmenu__pointer").length ? parseInt(menu.find(".zmenu__pointer").outerHeight(true)) : 0, // No I18N
						remaining =  headerHeight + footerHeight + pointerHeight; 
					actualMenu.innerHeight(menuHeight - remaining);
					if(this.options.scroll === "slider"){
						this._initSliders(menu);
						actualMenu.addClass("h-overflowhidden");
					}else{
						actualMenu.addClass("h-scroll_y");
					}
					this.options.scroll === "slider" && this._initSliders(menu);  // No I18N
					if(remaining !== 0){
						// menu.height(menuHeight);
						menu.outerHeight(menuHeight);
					}
				}else{
					if(this.options.scroll === "slider"){
						this._uninitSliders(menu);
						this._initSliders(menu);
						menu.addClass("h-overflowhidden");
					}else{
						this._uninitSliders(menu);
	                	menu.addClass("h-scroll_y"); // No I18N
					}
				}
			}else{
				menu.removeClass("h-scroll_y"); // No I18N
			}
		},
		_findScrollWidth: function(){
			var scrollEle = document.createElement("div"),
				styleProps = "width:100px;height:100px;overflow:scroll;";  	 // No I18N
			document.querySelector("body").appendChild(scrollEle);	 // No I18N
			scrollEle.style.cssText = styleProps; 
			var scrollWidth = $(scrollEle).width() - scrollEle.scrollWidth;
			document.querySelector("body").removeChild(scrollEle);	 // No I18N
			return scrollWidth;
		},
		_initSliders: function(menu){
			this._createSlider(menu);
			this._createSlider(menu,true); // create bottom slider.
		},
		_uninitSliders: function(menu){
			menu = menu.is("div") ? menu.find("ul:first") : menu; // No I18N
			var data = this.getData(menu);
			if(data && data.sliders){
				if(data.sliders.top){
					data.sliders.top.remove();
				}
				if(data.sliders.down){
					data.sliders.down.remove();
				}
				data.sliders = null;
			}
			menu.off("mousewheel");	 // No I18N
		},
	/*	Case 1: For menu containing div element as parent without header and footer cases.
		_createSlider: function(menu,isTop){
			var direction = isTop ? 'top': 'bottom',  // No I18N
				data = this.getData(menu), base = this;
			menu = menu.is("ul") ? menu : menu.find("ul");  // No I18N
			if(!data.sliders){
				data.sliders = {};
			}
			var slider = data.sliders[direction];
			if(slider){
				slider.remove();
			}
			slider = $('<li>').addClass('zmenu__scroll zmenu__scroll-'+direction);
			data.sliders[direction] = slider;
			slider.hover(function(e){
				base._startScrolling(menu, isTop);
			}, function(e){
				base._stopScrolling(menu);
			}).hide();
	
			slider.click(function(e){
				var top = menu.scrollTop();
				base._stopScrolling(menu);
				menu.scrollTop(top + (isTop ? -1: 1) * 100);
				base._startScrolling(menu, isTop);
				return false;
			});
	
			var offset = menu.position(),
				width = menu.outerWidth()-2,
				buffer = isTop ? 0 : (menu.height() - 6);
				if(!isTop){
					slider.css({
						'top': offset.top+1+buffer  // No I18N
					}).fadeTo(0, 1);
				}
			var sliderTop = (isTop)? '-3px': '-1px',  // No I18N
				button = $('<button class="zbutton"></button>').css({'top': sliderTop}),
				arrow = (direction === 'top') ? 'n' : 's',   // No I18N
				iconSpan = $('<i class="zbutton__icon zicon--arrow h-floatright arrow-direction-'+arrow+'"></i>');
			$(iconSpan).appendTo($(button));
			$(button).appendTo(slider);
			// slider[isTop ? "insertBefore" : "insertAfter"](menu);
			slider[isTop ? "prependTo" : "appendTo"](menu);   // No I18N
			isTop && slider.hide();
			return slider;
		}, */
		/* Case 2: For Menu which doesn't contain div element as parent. */
		_createSlider: function(menu,isTop){
			menu = menu.is("div") ? menu.find("ul:first"): menu; // No I18N
			var direction = isTop ? 'top': 'down',  // No I18N
				data = this.getData(menu), base = this;
			if(!data.sliders){
				data.sliders = {};
			}
			var slider = data.sliders[direction];
			if(slider){
				slider.remove();
			}
			slider = $('<div>').addClass('zmenu--scroller zmenu--scroller'+direction);
			data.sliders[direction] = slider;
	
			var base = this;
			slider.hover(function(e){
				base._startScrolling(menu, isTop);
			}, function(e){
				base._stopScrolling(menu);
			}).hide();
			slider.click(function(e){
				var top = menu.scrollTop();
				base._stopScrolling(menu);
				menu.scrollTop(top + (isTop ? -1 :  1));
				base._startScrolling(menu, isTop);
				return false;
			});
			/* Binding mousewheel event for the menu to scroll the menu using mouse eventhough sliders are present. */
            var componentName = this.componentName;
			menu.off("mousewheel."+componentName).on("mousewheel."+componentName,function(event){ // No I18N
				var top = menu.scrollTop();
				if(event.originalEvent.wheelDelta > 0){
					menu.scrollTop(top-6);
				}else{
					menu.scrollTop(top+6);
				}
				base._displayScrollElements(menu);
				event.stopPropagation();
				event.preventDefault();
			});
			var	width = menu.width();
			slider.css({"width":width}); // No I18N
			if(!isTop){
				slider.show();
			}
			var	arrow = (direction === 'top') ? 'n' : 's',   // No I18N
				iconSpan = $('<i class="zmenu__icon"></i>'),svgData = undefined;
			if(direction === "top"){ // No I18N
                svgData = {
                    props: {"viewBox":"0 0 16 16","width":"100%","height":"100%"}, // No I18N
                    nodes: [{
                        "path":{"d":"M0,12l8-8l8,8H0z","class":"st0"} // No I18N
                    }]
                };
            }else{
                svgData = {
                    props: {"viewBox":"0 0 16 16","width":"100%","height":"100%"}, // No I18N
                    nodes: [{
                        "path":{"d":"M16,4l-8,8L0,4H16z","class":"st0"} // No I18N
                    }]
                };
            }
            this._handleIcon(iconSpan, svgData, isTop ? "topscroller" : "bottomscroller", isTop ? this.options.scrollUpArrowIconClassName : this.options.scrollDownArrowIconClassName, isTop ? this.options.scrollUpArrowSVGIconId : this.options.scrollDownArrowSVGIconId );
			$(iconSpan).appendTo(slider);
			if(menu.hasClass("zmenu__list")){
				slider.appendTo(menu.parent()); 
				var topValue = menu.height() - slider.outerHeight(true)/2;
				if(!menu.parent().find(".zmenu__footer").length){
					// footer element's margin top is set as 5
					if(this._pointerElement){
						topValue = topValue + Math.ceil(this._pointerElement.outerHeight(true)/2) + parseInt(menu.parent().css("border-bottom-width"));  // No I18N
					}else{
						// without pointer bottom border should be taken into account.
						topValue = topValue - parseInt(menu.parent().css("border-bottom-width")) - parseInt(menu.parent().css("border-top-width"));  // No I18N
					}
				}
				slider.css({"left":"0px","top": isTop ? "0px" : topValue  });  // No I18N	
			}else{
				slider.insertAfter(menu);
				var offset = menu.position(),
				buffer = isTop ? 1 : menu.outerHeight() - slider.outerHeight(true) - 1;
				slider.css({
					'top': offset.top+buffer, // No I18N
					'left': offset.left+1 // No I18N
				});
			}
			return slider;
		},
		_displayScrollElements: function(menu){
			var hasParent = menu.hasClass("zmenu") ? false : true; // No I18N
			// var scrollElement = this.getData(menu.parent().is(".zmenu") ? menu.parent() : menu ).sliders,   // No I18N
			var scrollElement = this.getData( menu ).sliders,  
			// var scrollElement = this.getData( hasParent ? menu.closest(".zmenu") : menu ).sliders,   // No I18N
				scrollTop = menu.scrollTop();
			scrollElement.top[(scrollTop === 0) ? 'hide': 'show']();   // No I18N
			scrollElement.down[(menu[0].scrollHeight-scrollTop <= menu.outerHeight())? 'hide': 'show']();   // No I18N
		},
		_startScrolling: function(menu, isTop){
			var stop = isTop ? 0 : (menu[0].scrollHeight - menu.innerHeight()),
				base = this,
				distance = menu.scrollTop() - stop,
				duration = Math.abs(distance*1000/50); //1000 ms for 50px
			menu.animate({scrollTop: stop}, {
				'duration' : duration,   // No I18N
				'easing'   : 'linear',   // No I18N
				complete   : function(){
					base._displayScrollElements(menu);
				}
			});
		},
		_stopScrolling: function(menu){
			menu.stop();
			this._displayScrollElements(menu);
		},
		_hideAllSubMenus: function(menu,callback){
			var menu = this._traverseAndFindBaseMenu(menu);
			if(menu[0] != this.element[0]){
				 menu.zmenu("hide",undefined,callback); // No I18N
			}else{
				this._checkAndHideAllOpenedSubMenus(menu,true,callback);
			}
		},
		_destroyRuntimeEvents: function(){
			$(this.element).off(".run."+this.componentName); 
		},
		_triggerEvent: function(type, orgEvent, menu, menuItem, dataObj){
			orgEvent = orgEvent ? orgEvent : {};
			menuItem = menuItem ? menuItem : $(orgEvent.target).closest(".zmenu__item");
			var isBase = !menu || (menu.get(0) == this.element.get(0));
			var data = {
				"originalEvent": orgEvent,	// No I18N
                "target": menu.data("forElement"), // No I18N
				"forElement": menu.data("forElement"), // No I18N
				"offset": menu.offset(),  // No I18N
				"menu": menu, // No I18N
				"isBaseMenu": isBase, // No I18N
				"menuitem":menuItem, // No I18N
				"data": menuItem.length ? this._getMenuItemAsObject(menuItem): undefined,  // No I18N
				"itemValue": menuItem.length && menuItem.data("itemValue") // No I18N
			};
			if(dataObj){
				data = $.extend({}, data, dataObj);
			}
			if(menu.data("parent") && type.indexOf("SUBMENU") !== -1){  // No I18N 
				// if external submenu has menu events binded on it
				menu.zmenu("triggerEvent",type.replace("SUBMENU", ""), orgEvent, data);	// No I18N
				var parentMenu = menu.data('parent').closest(".zmenu"); // No I18N
				return parentMenu.zmenu("triggerEvent",type,orgEvent,data); // No I18N
			}
			return this._dispatchEvent(type,orgEvent,data, menu);
		},
		triggerEvent: function(type, orgEvent, data){
			return this._dispatchEvent(type,orgEvent,data);
		},
		/* Computing the slope of given points */
		_slope: function(a,b){
			return (b.y - a.y) / (b.x - a.x);
		},
		/* Computing the triangular region for slanted mouseover on menu items */
		_saveMenuPoints: function(e, menuItem){
			var metaData = this.getData(this.element);
			// metaData.constantLoc = {x:e.pageX, y:e.pageY}; // Current mouse position
			var subMenu = this._lastOverlayedMenu,
				baseMenuOffset = this.element.offset(),
				subMenuOffset = subMenu.offset();
			metaData.constantLoc = {x:baseMenuOffset.left, y:baseMenuOffset.top+menuItem.position().top}; // Current mouse position
			if(baseMenuOffset.left >= subMenuOffset.left){ // Properties of submenu to decide decreasing corner and increasing corner
				var upperRight = {
					x:subMenuOffset.left + subMenu.outerWidth(),
					y:subMenuOffset.top
				};
				var lowerRight = {
					x:subMenuOffset.left + subMenu.outerWidth(),
					y:subMenuOffset.top + subMenu.outerHeight()
				};
				this.increasingCorner = upperRight;
				this.decreasingCorner = lowerRight;
			}
			else{
				var upperLeft = {
					x:subMenuOffset.left,
					y:subMenuOffset.top
				};
				var lowerLeft = {
					x:subMenuOffset.left,
					y:subMenuOffset.top + subMenu.outerHeight()
				};
				this.decreasingCorner = upperLeft;
	            this.increasingCorner = lowerLeft;
	        }
	        this.prevDecreasingSlope = this._slope(metaData.constantLoc, this.decreasingCorner);
	        this.prevIncreasingSlope = this._slope(metaData.constantLoc, this.increasingCorner);
		},
		/** 
			Returns the value currently associated with the specified optionName.
			@function menu.getAttribute
			@param {String} optionName - The name of the option.
			@return {String} value.
			@example <caption> Example: </caption>
var type = $("menuElement").zmenu("getAttribute","type");
		*/
		/** 
			Sets the value for the option with the specified optionName.
			@function menu.setAttribute
			@param {String} optionName - The name of the option.
			@param {String} value - The value for the option.
			@example <caption> Example: </caption>
$("menuElement").zmenu("setAttribute","type","grid");
		*/
		/** 
			Returns an object containing key/value pairs representing the current menu options hash.
			This method does not accept any arguments.
			@function menu.getAttributes
			@return {Object} options object.
			@example <caption> Example: </caption>
var options = $("menuElement").zmenu("getAttributes");
		*/
		/** 
			Sets one or more options for the menu.
			@function menu.setAttributes
			@param {Object} options - A map of option-value pairs to set.
			@example <caption> Example: </caption>
$("menuElement").zmenu("setAttributes",{"type":"grid","columns":4});
		*/
		setAttribute: function(optionName,value){
			if(optionName === "height" && this.options.height !== value){ // No I18N
				this.options.height = parseInt(value);
				this.element.data("height",parseInt(value)); // No I18N
				this._fixMenuHeight(this.element,{});
			}else if(optionName === "width" && this.options.width !== value){ // No I18N
				this.options.width = parseInt(value);
				this.element.data("width",parseInt(value)).width(this.options.width); // No I18N
			}else if(optionName === "contentType" && this.options.contentType !== value){ // No I18N
				if(this.options.type !== "grid"){ // No I18N
					this.element.removeClass(this._MENU_CONTENT_TYPES[this.options.contentType]).addClass(this._MENU_CONTENT_TYPES[value]);
				}
				this.options.contentType = value;
				var items = this._getMenuAsObject(this.element);
				this.element.is("ul") ? this.element.empty() : this.element.find("ul").empty(); // No I18N
				this.addMenuItems(items);
			}else if(optionName === "type" && this.options.type !== value){ // No I18N
				this.element.zmenu('destroy'); // No I18N
				this.options.type = value;
				this.element.zmenu(this.options);
			}else if(optionName === "search" && this.options.search !== value){ // No I18N
				if(value){
					this._createSearchContainer(this.element);
				}else{
					var searchBoxPosition = this.element.find(".zmenu__textbox").parent().attr("class"); // No I18N
					this.element.find("."+searchBoxPosition).remove(); // No I18N
				}
				this._fixMenuHeight(this.element,{});
			}else if((optionName === "pointer" || optionName === "displayType") && this.options.pointer !== value){ // No I18N
				if(value && value !== "box"){ // No I18N
					this._createPointerElement();
				}else{
					this._pointerElement.remove();
					this._pointerElement = undefined;
				}
			}else if(optionName === "renderInPlace" && this.options.renderInPlace !== value){ // No I18N
				if(!value){
					this.element.css({"position":"absolute","width":"auto","height":"auto"}); // No I18N
				}
			}else if((optionName === "menuClass" || optionName === "className") && (this.options.menuClass !== value || this.options.className !== value)){ // No I18N
				this.element.removeClass(this.options.menuClass).addClass(value);
				this.element.removeClass(this.options.className).addClass(value);
			}else if(optionName === "handleAccessKeyNavigation" && this.options.handleAccessKeyNavigation !== value){ // No I18N
				value ? this._initRuntimeEvents() : this._destroyRuntimeEvents(); // Keypress events	
			}else if(optionName === "appendTo"){ // No I18N
                value !== undefined && $(value).length ? this.element.appendTo($(value)) : $("body").append(this.element);
            }else if(optionName === "checkedCheckboxItemSVGIconId" || optionName === "checkedCheckboxItemIconClassName" || optionName === "checkedRadioItemSVGIconId" || optionName === "checkedRadioItemIconClassName"){ // No I18N
                this.options[optionName] = value;
                var childElements = this.element.find("li"), length = childElements.length, menuItem;
                for(var i = 0; i < length; i++){
                    menuItem = $(childElements[i]);
                    this._checkMenuItem(menuItem, menuItem.prop("checked"), this.getData(menuItem)); // No I18N
                }
            }
			this.options[optionName] = value;
		},
		/* 
			Sets the checked state of the given radio or checkbox menu items to true.
			@function menu.checkMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("checkMenuItems",["#menuItemId1","#menuItemId2",...]);
		*/
		checkMenuItems: function(items){
			items = items ? items : this.element.find('[role=menuitemcheckbox],[role=menuitemradio]');
			this.setMenuItemsAttribute(items,"checked",true); // No I18N
		},
		/*
			Sets the checked state of the given radio or checkbox menu items to false.
			@function menu.uncheckMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("uncheckMenuItems",["#menuItemId1","#menuItemId2",...]);
		*/
		uncheckMenuItems: function(items){
			items = items ? items : this.element.find('[role=menuitemcheckbox],[role=menuitemradio]');
			this.setMenuItemsAttribute(items,"checked",false); // No I18N
		},
		/**
			Hides the given menu items by setting its hidden property to true.
			@function menu.hideMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("hideMenuItems",["#menuItemId1","#menuItemId2",...]);	
		*/
		hideMenuItems: function(items){
			items = items ? items : this.element.find('li').filter(function(){  // No I18N
											return $(this).css("display") !== "none"; // No I18N
									});
			this.setMenuItemsAttribute(items,"hidden",true); // No I18N
		},
		/**
			Shows the given menu items by setting its hidden property to false.
			@function menu.showMenuItems
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@example <caption> Example: </caption>
$("menuElement").zmenu("showMenuItems",["#menuItemId1","#menuItemId2",...]);	
		*/
		showMenuItems: function(items){
			items = items ? items : this.element.find('li:not(li[data-hidden=true])'); // No I18N
			this.setMenuItemsAttribute(items,"hidden",false); // No I18N
		},
		/** 
			Set the properties of given menu items.
			@function menu.setMenuItemsAttribute
			@param {Array} menuItems - An array of menu item's identifier or the menu item itself.
			@param {String} propName - The name of the property.
			@param {String} value - The value for the property.
			@example <caption> Example: </caption>
$("menuElement").zmenu("setMenuItemsAttribute",["#menuItemId1","#menuItemId2",...],"hidden",true);
		*/
		setMenuItemsAttribute: function(menuItems,optionName,value){
			var base = this;
			$.each(menuItems,function(i,menuItem){
				menuItem = $(menuItem,base.element);
                if(menuItem.length){
    				var data = base.getData(menuItem);
    				if(data.itemType === "itemgroup"){
    					var groupItems = $("[name="+menuItem.attr('name')+"]:not(.zmenu__title)",base.element); // No I18N
    					groupItems.length && base.setMenuItemsAttribute(groupItems,optionName,value);
    				}
    				if((optionName === "icon" || optionName === "iconClassName" || optionName === "SVGIconId") && data[optionName] !== value){ // No I18N
    					if(data.icon || data.iconClass || data.SVGIconId){ // Removing the previous icon
    						menuItem.find(".zmenu__iconcontainer:first").remove();  // No I18N
    					}
    					var iconEle = $("<i class='zmenu__icon'/>");
    					if(value.indexOf("text:") === 0){
    						// giving text as icon
    						menuItem.prepend($("<div class='zmenu__iconcontainer'>").append(iconEle));
    						menuItem.find(".zmenu__icon:first").text(value.replace("text:","")); // No I18N
    					}else{
    						data[optionName] = value;
    						menuItem.prepend($("<div class='zmenu__iconcontainer'>").append(base._handleIcon(iconEle, undefined, undefined, data.iconClassName, data.SVGIconId, data.icon)));
    					}
    				}else if(optionName === "label" && data.label !== value){ // No I18N
                        if(data.ellipsis){
                            value = value+"..." // No I18N
                        }
    					menuItem.find(".zmenu__text").text(value); // No I18N
    				}else if((optionName === "shortcutkey" || optionName === "shortcutKey") && (data.shortcutkey !== value || data.shortcutKey !== value)){ // No I18N
    					if(data.shortcutKey || data.shortcutkey){
    						menuItem.find(".zmenu__shortcut").text(value);  // No I18N
    					}else{
                            value = window.ZShortCut ?  ZShortCut.getInfo(undefined, value) : value;
                            base.getData(menuItem).shortcutkey = base.getData(menuItem).shortcutKey = value;
                            menuItem.append($("<span class='zmenu__shortcut'/>").text(value));    // No I18N
    					}
    				}else if(optionName === "hint" || optionName === "description" && (data.hint !== value || data.description !== value)){ // No I18N
    					if(data.hint || data.description){
    						menuItem.find(".zmenu__hint").text(value);  // No I18N
    					}else{
    						menuItem.append($("<span class='zmenu__hint'/>").text(value));  // No I18N
    					}
    				}else if(optionName === "disabled" || optionName === "checked" || optionName === "hidden"){ // No I18N
    					data[optionName] = value;				
    					base._handleCheckedandDisabled(menuItem,data,optionName);
    				}else if(optionName === "menuId" && data.menuId !== value){ // No I18N
    					data[optionName] = value;
    					if(!value){
    						menuItem.find(".zmenu__icon.h-alignright").remove();
    						menuItem.removeData("submenu"); // No I18N
    						menuItem.off("mouseup.submenuitem.zmenu"); // No I18N
    					}else{
    						menuItem.removeData("submenu"); // No I18N
    						base._buildSubmenu(menuItem,data);
    						base._createArrowIcon(menuItem);
    					}	
    				}else if(optionName === "innerHTML" && data.itemType === "custom"){   // No I18N
    					menuItem.empty().html(value); // should be used only in custom menu item
    				}else if(optionName === "itemType" && data.itemType !== value){ // No I18N
    					if(value === "separator"){ // No I18N
    						menuItem.removeClass("zmenu__item").addClass("zmenu__separator").empty(); // No I18N
    					}else if(value === "default"){ // No I18N
    						menuItem.removeClass("zmenu__title zmenu__custom zmenu__separator").addClass("zmenu__item"); // No I18N
    					}else if(value === "custom"){ // No I18N
    						menuItem.addClass("zmenu__item zmenu__custom"); // No I18N
    					}else if(value === "itemgroup"){ // No I18N
    						menuItem.removeClass("zmenu__item zmenu__custom zmenu__separator").addClass("zmenu__title"); // No I18N
    					}else if(value === "splitmenu"){ // No I18N
    						menuItem.removeClass("zmenu__title zmenu__custom zmenu__separator").addClass("zmenu--split"); // No I18N
    						base._createArrowIcon(menuItem);
    						menuItem.find(".zmenu__shortcut").remove();
    					}
                        menuItem.attr("role", base.ARIA_ROLES[value ?  value : "default"]); // No I18N
    					menuItem.attr("data-item-type",value); // No I18N
                        menuItem[0].setAttribute("role", this.ARIA_ROLES[value ?  value : "default"]); // No I18N           
    				}
    				data[optionName] = value;
                }
    		});
		}
	}
    menuPrototype.updateMenuItems = menuPrototype.setMenuItemsAttribute;
	ZC.registerComponent("zmenu", menuPrototype); // No I18N
	$.extend($.expr[":"],{	// No I18N
		visibleElementsOfGroup:function(element){ // Filter to find the elements that are hidden with display='none'    // No I18N
			return $(element).css("display") !== "none";	// No I18N
		}
	});
	ZC.createMenu = function(menuId,menuDetails,options){ // Menu Created Using JSON - assigning it to global namespace
		if(typeof menuId === "object"){ // No I18N
			menuDetails = menuId.items,
			options = menuId;
			menuId = menuId.id;
		}
		if($("#"+menuId).data("zmenu")){ // No I18N
			return $("#"+menuId); // No I18N
		}
		var	menu = undefined;
		if(document.getElementById(menuId)){
			menu = $(document.getElementById(menuId));
			menu[0].tagName === "DIV" && menu[0].appendChild(document.createElement("ul")); // No I18N
		}else{
			menu = $("<div><ul></ul></div>").attr("id",menuId);	 // No I18N
		}
		var appendTo = options ? options.appendTo : undefined;	
		menu[0].className += " zmenu "+menuPrototype._MENU_CONTENT_TYPES[options.contentType ? options.contentType : "text"]+" "+(options.menuClass ? options.menuClass :"")+" "+(options.className ? options.className :""); // No I18N
		if(options.type && options.type.indexOf("grid") !== -1){ // No I18N
			menu[0].className += " zmenu--thumbnail"; // No I18N
		}
		options.usingJSON = true;
		var appendedTo = document.querySelector("body");	// No I18N
		if( appendTo !== undefined && appendTo.length ){
			appendedTo = $(appendTo)[0];
		}
		appendedTo.appendChild(menu[0]);
		ZC.getOptions(menu).type = options.type;
		menu.zmenu(options);
		menuDetails.length ? menu.zmenu("addMenuItems",menuDetails,options ? options.addItemCallback : undefined) : ""; // No I18N
		return menu;
	}

	// ZC.$document.on("click.zmenu",function(event){ // No I18N
	// 	if(event.which === 3){
	// 		return;
	// 	}
	// 	var target = $(event.target),
	// 		forElement = ZC.currentMenu && ZC.currentMenu.data("forElement"), // No I18N
	// 		inPlaceMenu = ZC.currentMenu && ZC.currentMenu.data("zmenu") && ZC.currentMenu.data("zmenu").options.renderInPlace; // No I18N
	// 	if(!inPlaceMenu){
	// 		if(!target.closest(".zmenu").length && forElement && forElement.length && !target.closest(forElement).length){
	// 			ZC.currentMenu.is(":visible") && ZC.currentMenu.zmenu("hide"); // No I18N
	// 		}else if(!target.closest(".zmenu").length && ZC.prevMenu && ZC.prevMenu.is(":visible")){ // No I18N
	// 			ZC.prevMenu.zmenu("hide"); // No I18N
	// 			ZC.prevMenu = undefined;
	// 		}
	// 	}
	// }).on("zmenuhide",function(event,ui){	// No I18N
	// 	if(ZC.prevMenu && ZC.prevMenu.get(0) === ui.menu.get(0)){
	// 		ZC.prevMenu = undefined;
	// 	}else if(ZC.currentMenu && ZC.currentMenu.get(0) === ui.menu.get(0)){
	// 		ZC.currentMenu = undefined;
	// 	}
	// });
	var ZM_DOM = {};
	function menuScrollWidth(){
		var scrollEle = document.createElement("div");
		ZM_DOM._iconDiv = scrollEle.cloneNode();
		ZM_DOM._iconDiv.className = 'zmenu__iconcontainer';
		ZM_DOM._icon = document.createElement("i");
		document.querySelector("body").appendChild(scrollEle);	// No I18N
		scrollEle.style.cssText = "width:50px;height:50px;overflow:scroll;";	// No I18N
		menuPrototype._scrollWidth = $(scrollEle).width() - scrollEle.scrollWidth;
		$(scrollEle).remove();
		ZM_DOM._text = document.createElement("span");
		ZM_DOM._hint = ZM_DOM._text.cloneNode();
		ZM_DOM._key = ZM_DOM._text.cloneNode();
		ZM_DOM._text.className = 'zmenu__text';
		ZM_DOM._hint.className = 'zmenu__hint';
		ZM_DOM._key.className = 'zmenu__shortcut';
		return menuPrototype._scrollWidth;
	};
})(ZComponents);

/** @member {String} [menu.item-options.icon] - The icon to be shown in the menu item. It can be a sprite css class name or svg element identifier followed by class name to be used in svg element. This option will be deprecated in the future version. So use "iconClassName" and "SVGIconId" instead.
								   Example : "settings-icon" or "#svg-id custom-class"
	@deprecated
*/
/** @member {String} [menu.item-options.iconClassName] - The icon to be shown in the menu item. It can contain one or more space separated sprite css classes.
	@newlyAdded
*/
/** @member {String} [menu.item-options.SVGIconId] - The icon to be shown in the menu item. It contains svg element id followed by space separated css class names to customize SVG icon.
	@newlyAdded
*/

/** @options menu.item-options */
/** @member {String} [menu.item-options.label] - The text to be displayed in the menu item.*/
/** @member {String} [menu.item-options.shortcutKey] - The shortcut key to be displayed in the menu item.*/
/** @member {String} [menu.item-options.description] - The additional description text to be displayed in the menu item.*/
/** @member {String} [menu.item-options.itemType] - The type of the menu item.
	@allowedValues {("default"|"separator"|"radio"|"checkbox"|"custom"|"itemgroup"|"splitmenu")}
*/
/** @member {String | Object} [menu.item-options.itemValue] - The custom value for the menu item.*/
/** @member {String} [menu.item-options.innerHTML] - The content HTML of the menu item of type "custom"(ie. menu item with itemType as "custom").*/
/** @member {String} [menu.item-options.menuId] - The identifier of the menu item's submenu.*/
/** @member {Array} [menu.item-options.menuItems] - Array of menu items object to create the submenu for the menu item.*/
/** @member {Boolean} [menu.item-options.disabled] - Specifies whether the menu item is in disabled state or not.*/
/** @member {Boolean} [menu.item-options.checked] - Specifies whether the menu item is in checked state or not.*/
/** @member {Boolean} [menu.item-options.hidden] - Specifies whether the menu item is in hidden state or not.*/
/** @member {Function} [menu.item-options.action] - The callback to be executed whenever the menu item is clicked.*/
/** @member {Number} [menu.item-options.index] - The index to insert the menu item.*/
/** @member {Boolean} [menu.item-options.isFooter] - Indicates whether to place the menu item as the footer item to the menu or not.*/
/** @member {String} [menu.item-options.className] - The CSS class (or the space separated string of classes) to be added to the menu item for additional theming. This option is mostly used while creating menu using JSON or adding menu items using API methods. 
	@example <caption> Add the menu item with the "className" option specified: </caption>
	$("menuElement").zmenu("addMenuItem",{
	    "label"     : "Print",
	    "icon"      : "printer-icon",
	    "className" : "custom-css-class"
	});
*/
/*	  "icon"  : <#svg-id svg-class|icon-class-name>,  
      "iconText" : <string>,
      "label" : <string>,
      "shortcutKey": <string>,
      "description" : <string>,
      "itemValue" : <string>,
      "itemType": "<default|separator|radio|checkbox|custom|itemgroup|splitmenu|submenu>",
      "innerHTML": <DomElements>,
      "menuId" : <submenuId>,      
      "menuItems" : <[menuItemObject1,menuItemObject2,...]>,      
      "disabled" : <true|false>,
      "checked" : <true|false>,
      "hidden" : <true|false>,
      "action" : <js-function>, */ 
	/* $Id$ */ 
	(function( ZC ){
		var $ = ZC.DOMUtil;
		/** @components combo-box Combobox is the combination of selectbox and textbox that allows the user to search for an item by typing inside the textbox.
			@eventPrefix zcombobox
		*/
		/** @components select-box Selectbox enables the user to select items from the dropdown list with additional visual effects.
			@eventPrefix zselectbox
		*/
		var selectboxPrototype = {
			options: {
				/** @member {Boolean} [select-box.multiple=false] Specifies whether multiple values can be selected. When multiple  is true, always contentType is set to 'text' */
				multiple:false,
				selectionLimit: 0,
				dataSource: undefined,
		        selectedIndex:undefined,
		        selectedValue:undefined,
				placeholder: 'Select',	//No I18N
		        dropdownList:{
		        	height : undefined,
		        	width :undefined,
		        	appendTo : undefined,
		        	direction : 'auto',	//No I18N
		    		className : undefined,
		    		viewport: undefined
		        },
				autoFill: false,
				searchable: false,
				selectOnClose: false,
				clearSelection: false,
		        searchSettings: {
			       delay: 150,  
			       searchCriteria: 'contains',  //No I18N  
			       caseSensitive: false,             
			       // minKeywordLength: 0,
			       searchByField:'label' //No I18N
			     },
		        disabled: false,
		        contentType: 'text',	//No I18N
		        className:undefined,
		        messages:{
		        	allOptionsSelected: 'All Selected',	//No I18N
		        	noResults:'No Results'	//No I18N
		        },
				eventPrefix:'zselectbox'	//No I18N
			},			
			_EVENTS: {
				/** 
					Triggered before the dropdown list is shown. In order to prevent dropdown list from opening, return 'false' in the callback.
					@event combo-box.beforedropdownlistopen
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combo.
				*/
				'ONBEFORESHOW': 'beforedropdownlistopen',	//No I18N
				/**
					Triggered after the dropdown list is shown.
					@event combo-box.dropdownlistopen
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combo.
				*/
				'ONSHOW': 'dropdownlistopen',	//No I18N
				/**
					Triggered when the selection is changed.
					@event combo-box.change
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combo.
				*/
				'ONCHANGE': 'change',	//No I18N
				/**
					Triggered before the dropdown list is closed. In order to prevent dropdown list from closing, return 'false' in the callback.
					@event combo-box.beforedropdownlistclose
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				'BEFOREHIDE':'beforedropdownlistclose', //No I18N
				/**
					Triggered after the dropdown list is closed.
					@event combo-box.dropdownlistclose
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				'ONHIDE': 'dropdownlistclose',	//No I18N
				/**
					Triggered after an item is clicked.
					@event combo-box.itemclick
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONITEMCLICK: 'itemclick',	//No I18N
				/**
					Triggered when the component is ready for use.
					@event combo-box.init
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONINIT:'init',	//No I18N
				ONNEWVALUE:'newvalue', //No I18N
				/**
					Triggered when the component loses focus.
					@newlyAdded true
					@event combo-box.blur
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONBLUR:'blur', //No I18N
				/**
					Triggered when the component receives focus.
					@newlyAdded true
					@event combo-box.focus
					@param {Object} event Contains properties of the occurred event.
					@param {Object} data Contains additional properties of the combobox.
				*/
				ONFOCUS:'focus', //No I18N
				ONBEFOREOPTIONADD:'beforeoptionadd', //No I18N
				ONOPTIONADD: 'optionadd' //No I18N
			},
			_CLASS : {
				ICON:'zselectbox__icon',	//No I18N
				ICONTEXT: 'zselectbox--icontext', //No I18N
				ICONONLY: 'zselectbox--icononly',	//No I18N
				PLACEHOLDER: 'zselectbox__placeholder', //No I18N
				DROPUP: 'zselectbox--top', //No I18N
				DROPDOWN: 'zselectbox--bottom',	//No I18N
				ACTIVE: 'is-active', //No I18N
				FOCUS: 'has-focus', //No I18N
				DISABLED: 'is-disabled', //No I18N
				EMPTY:'zselectbox__emptyresult', //No I18N
				TEXTBOX:'zselectbox__text' //No I18N
			},
			_create: function( element, options){
				this.frominit = true;
				this._elementWidth = options.width || this.element[0].offsetWidth;
				this.element[0].style.display = 'none';
				this._namespace = '.' + options.eventPrefix; //No I18N
				var base = this;
				['disabled','multiple','title'].forEach( function( attr){ //No I18N
					options[attr] = options[attr] || base.element[0][attr];
				});	
				this._initialize(element, options);
				this._constructComponent();
				this._handleDS();
				this._setInitial();
				this._bindEvents();
				this._trigger( "ONINIT" ); 	//No I18N
				this.frominit = false;
			},
			_initialize: function(element, options){
				/*To handle keydown selection when menu is not opened*/
				this._textArray = [];
				this._prevKey = this._prevTimeStamp = ''; // No I18N
				if( !/^(text|icon|icon-text|custom)$/.test( options.contentType ) ){
					options.contentType = $.zselectbox.prototype.options.contentType;
				}
				if( options.multiple ){
					var limit = +options.selectionLimit;
					options.selectionLimit =  ( isNaN(limit) || limit < 0 ) ? 0 : limit;
					// if( options.contentType === 'icon' ){ //No I18N
						options.contentType = 'text'; //No I18N
					// }
					if( options.selectedValue !== undefined ){
						options.selectedValue = String(options.selectedValue).split(','); //No I18N
					}else if( options.selectedIndex !== undefined ){
						options.selectedIndex = String(options.selectedIndex).split(',') || 0; //No I18N
					}
					this.element[0].setAttribute('multiple', true); //No I18N
				}
			},
			/* Function to construct layout of combo component*/
			_constructComponent: function(){
				if( !this.element[0].id ){
					this.element[0].id = (this._isEditable ?'zcombo_':'zselect_')+Math.floor(Math.random() * 1000000); //No I18N
				}
				if( typeof this.options.menuId === 'string' ){ //No I18N
					this._menuID = this.options.menuId;
				}else{
					this._menuID = this.element[0].id + "Menu"; //No I18N
					$(document.getElementById(this._menuID )).remove();
				}
				var containerId = this.element[0].id+'Container'; //No I18N
				this.container = document.getElementById( containerId );
				if( this.container ){
					this._closeIcon = this.container.querySelector('.zselectbox__close'); //No I18N
					this._arrow = this.container.querySelector('.zselectbox__arrowicon'); //No I18N	
					this._optArea = this.container.querySelector('.zselectbox__selected'); //No I18N
					this.textspan = this._optArea.querySelector('.zselectbox__text'); //No I18N
					this.iconspan = this._optArea.querySelector('.zselectbox__icon'); //No I18N
				}else{
					var baseClass = 'zselectbox ' + ( typeof this.options.className === 'string'? this.options.className : '') + ' ' + (this.element[0].getAttribute('class') ? this.element[0].getAttribute('class') : '');	//No I18N
					this.container = document.createElement('div');	//No I18N
					$(this.container).attr({
						'class':baseClass, //No I18N
						'aria-haspopup': true, //No I18N
						'title': this.element[0].getAttribute('title'), //No I18N
						'aria-owns': this._menuID, //No I18N
						id: containerId,
						'style': this.element[0].getAttribute('style') //No I18N
					});
					this.container.style.display = ''; //No I18N
					if( !this.element[0].style.width && this._elementWidth ){
						this.container.style.width = isNaN( this._elementWidth )? this._elementWidth : (this._elementWidth+'px'); //No I18N
					}
					this.container.innerHTML = "<div class=zselectbox__selected></div> <i class='zselectbox__close'></i> <i class='"+ this._CLASS.ICON +" zselectbox__arrowicon'></i>";
					this._optArea = this.container.children[0];
					this._closeIcon = this.container.children[1];
					this._arrow = this.container.children[2];
					if( this.options.arrowSVGIconId || this.options.arrowIconClassName ){
						this._arrow = ZC.createIcon( $(this._arrow), undefined, this.options.arrowSVGIconId, this.options.arrowIconClassName )[0];
					}else{
						this._arrow.innerHTML = "<svg class='zselectbox__arrow' viewBox='0 0 48 48'>"+ //No I18N
									              "<path class='cls-1' d='M12,19.478L15.571,16l9.286,9.739L34.143,16,37,19.478,24.857,32Z'></path>"+ //No I18N
									            "</svg>";
					}
					if( this.options.closeSVGId || this.options.closeIconClass ){
						this._closeIcon = ZC.createIcon( $(this._closeIcon), undefined, this.options.closeSVGId, this.options.closeIconClass )[0];
					}else{
						this._closeIcon.innerHTML = '<svg viewBox="0 0 16 16"> <path d="M15.1 3.1l-2.2-2.2-4.9 5-4.9-5-2.2 2.2 5 4.9-5 4.9 2.2 2.2 4.9-5 4.9 5 2.2-2.2-5-4.9z"/> </svg>'; //No I18N
					}
					this._closeIcon.style.display = 'none'; //No I18N
					this._constructOptArea();
					this.element[0].parentElement.insertBefore( this.container, this.element[0].nextSibling );
				}
			},
			/*Function to create optArea*/
			_constructOptArea: function(){
				this.container.setAttribute('tabindex', this.element[0].getAttribute('tabindex') || 0 );
				this.container.setAttribute('role','select');
				var html = '';
				if( this.options.contentType.match('icon') && !this.options.multiple ){
					html = html + '<i class='+ this._CLASS.ICON +'></i>';
				}
				if( !/^(icon)$/.test( this.options.contentType) || this.options.multiple ){
					html = html + '<span class=zselectbox__text></span>';
				}
				this._optArea.innerHTML = html;
				this.textspan = this._optArea.querySelector('.zselectbox__text'); //No I18N
				this.iconspan = this._optArea.querySelector('.zselectbox__icon'); //No I18N
				var specialClass = (this.options.contentType === 'icon'? this._CLASS.ICONONLY : this.options.contentType === 'icon-text' && !this.options.multiple ? this._CLASS.ICONTEXT :' '  ).trim();  //No I18N
				this._addClass( this.container, specialClass );
			},
			_createMenu: function(){
				var o = this.options;
				var base = this;
				var menu = document.getElementById( this._menuID );
				var menu_opt = {
					position : o.position,
					contentType: o.multiple ? 'icon-text': o.dropdownList.contentType || o.contentType, //No I18N
					tagName : 'div', //No I18N
					height : o.dropdownList.height,
					width :  o.dropdownList.width,
					appendTo : $(o.dropdownList.appendTo),
					className: (o.dropdownList.className || "") + " zmenu--selectmenu",	//No I18N
					direction : o.dropdownList.direction,
					handleAccessKeyNavigation: o.searchable? false: true,
					scroll: 'default', //No I18N
					viewport: $(o.dropdownList.viewport),
					toggle: false
				}
				if( menu ){
					this._menu = $(menu);
					this._completeMenuLoaded = true;
					menu_opt.constructMenu = false;
					this._menu.zmenu( menu_opt  );
				}else{
					this._menu = ZC.createMenu(this._menuID, {}, menu_opt);
					this._createSearchBox();
				}
				this._menu.attr({'tabindex':-1, 'role':'listbox'}) //No I18N
				  		  .on({
								zmenubeforeshow: function( origEvent ){
									// origEvent.stopImmediatePropagation();
									return base._trigger( "ONBEFORESHOW", origEvent ); 	//No I18N
								},
								zmenubeforehide: function( origEvent ){
									origEvent.stopImmediatePropagation();
									return base._trigger( "BEFOREHIDE", origEvent ); 	//No I18N
								},
								zmenushow: this._onMenuShow.bind(this),
								zmenuhide: this._onMenuHide.bind(this),
								zmenuitemclick: this._onItemClick.bind(this),
								zmenuitemhover: this._onMenuItemHover.bind(this),
								zmenubeforefocus: function(){
									return false;
								}
							}).on('focusout'+this._namespace, this._onBlur.bind(this)) //No I18N
				  		  	  .on('keydown'+this._namespace, function( origEvent ){ //No I18N
				  		  	  	 base._keyPress.call( base, origEvent );
				  		  	  });
			},
			_createSearchBox: function(){
				if( !this._isEditable && this.options.searchable && this._menu ){
					var header = $("<div class='zmenu__header'></div>"); // No I18N
					var textbox = $("<input type='text' class='zmenu__textbox' placeholder='Search' />"); // No I18N
					var searchIcon = $("<i class='zmenu__icon'></i>");
					var svgEle = ZC.createSVGElement("svg",{ // No I18N
						"class":"zmenu__search","viewBox":"0 0 16 16","width":"100%","height":"100%" // No I18N
					},[
					{"path":{"d":"M15.6,15.6c-0.6,0.6-1.5,0.6-2.1,0l-2.7-2.7C9.7,13.6,8.4,14,7,14c-3.9,0-7-3.1-7-7s3.1-7,7-7c3.9,0,7,3.1,7,7c0,1.4-0.4,2.7-1.1,3.8l2.7,2.7C16.1,14,16.1,15,15.6,15.6z M7,2C4.2,2,2,4.2,2,7s2.2,5,5,5c2.8,0,5-2.2,5-5C12,4.2,9.8,2,7,2z"}} // No I18N
					]);
					searchIcon[0].appendChild(svgEle);
					header.append(textbox, searchIcon);
					this._menu.prepend(header);
					this._bindSearchbox(textbox[0]);
				    // textbox.on('keydown'+this._namespace, this._keyPress.bind(this)) //No I18N
				}
			},
			_bindSearchbox:function( textbox ){
				var eventName = isIE9? 'keyup':'input'; //No I18N
				var base = this;
				$(textbox).on( eventName +this._namespace,  function( origEvent ){
					if( origEvent.type === 'input' || !(origEvent.metaKey || origEvent.ctrlKey) && ((origEvent.keyCode >= 186 && origEvent.keyCode <= 192) || (origEvent.keyCode >= 219 && origEvent.keyCode <= 222) || (origEvent.keyCode >= 48 && origEvent.keyCode <= 57) || (origEvent.keyCode >= 65 && origEvent.keyCode <= 90) || (origEvent.keyCode ===  ZC.keyCode.SPACE) || (origEvent.keyCode === ZC.keyCode.DELETE) || (origEvent.keyCode === ZC.keyCode.BACKSPACE))){	//No I18N
						origEvent.type = 'input'; //No I18N
						base._onInput.call( base, textbox, origEvent );
					}
				});
			},
			_onInput:function( textbox, origEvent ){
				var base = this;
				var value = textbox.value;
				var setting = this.options.searchSettings;
				this._activated = true;
				if( this._closeIcon ){
					this._closeIcon.style.display = (($.trim(value) === '' || !this.options.clearSelection) ? 'none':''); //No I18N
				}
				value = value.replace(new RegExp(/\\/gi),"\\\\").replace(new RegExp("'","gi"),"\\'"); //No I18N
				value = setting && setting.caseSensitive ? value : value.toLowerCase();
				clearTimeout( this.inputtextTimer );
				if( setting && setting.delay ){
					this.inputtextTimer = setTimeout( this._searchQuery.bind( this, origEvent, value ), setting.delay );
				}else{
					this._searchQuery( origEvent, value );
				}
			},
			_searchQuery: function( origEvent, query ){
				this._filterList( query );
				this._open( origEvent );
			},
			_filterList: function( query ){
				var base = this;
				var s = this.options.searchSettings || {};
				this._loadDropdown();
				this._clearStyle();
				if( s.searchByField === 'label' ){ //No I18N
					var metadata = ZC.getOptions( this.element[0] );
					/*creating style tag*/
					var styletag = metadata.styletag = document.getElementsByTagName('head')[0].appendChild( document.createElement('style') );
					if( query.length ){
						styletag.textContent = "#"+ this._menuID +" [data-filter-label]:not([data-filter-label"+ (s.searchCriteria === 'startswith'? '^=':'*=') +"'"+ query +"']) {display:none !important;}";	//No I18N
						var g_hidden = [];
						this._queryAll( this._menu[0], '.zmenu__title' ).forEach( function( group ){
							var visibleitems = base._getFilteredNodes(function( node ){
								return $(node).css('display') !== 'none'; //No I18N
							}, base._queryAll( base._menu[0], '.zmenu__item[data-group='+group.id+']' )); //No I18N
							if( !visibleitems.length ){
								g_hidden.push('#'+ group.id);
							}
						});
						if(g_hidden.length){
							styletag.textContent = styletag.textContent +" " +g_hidden.join(',') + " {display:none !important;}"; //No I18N
						}	
					}	
				}else{
					this._hiddenitems = [];
					this._queryAll( this._menu[0],'li:not([hidden])').forEach(function( node ){ //No I18N
						var menuData = ZC.getOptions( node );
						if(!( base._menuitemMatcher( menuData , query )  ) ){
							node.style.display = 'none'; //No I18N
							base._hiddenitems.push( node );
						}	
					});
				}
			},
			_menuitemMatcher: function( menuData, query ){
				if( !menuData || $.trim(query) === ''){ //No I18N
					return true;
				}
				if( menuData.itemType === 'itemgroup' ){ //No I18N
					var childDatas = this._menu.find('[data-group = ' + menuData.customAttributes.id + ']'); //No I18N
					for( var i=0; i< childDatas.length; i++ ){
						var child = childDatas[i];
						if( this._menuitemMatcher( ZC.getOptions( child ), query ) ){
							return true;
						}
					}
				}else{
					var s = this.options.searchSettings;
					var searchKeys = s.searchByField.split(','); //No I18N
					for( var i =0; i< searchKeys.length; i++ ){
						var key = searchKeys[ i ];
						var regex = new RegExp( s.searchCriteria === 'startswith'? '^('+query+')':query , 'g'+(s.caseSensitive?'':'i')); //No I18N
						if(/* menuData.properties && */typeof menuData.properties[key] === 'string' && regex.test( menuData.properties[key] ) ){ //No I18N
							return true;
						}
					}
				}
			},
			_clearStyle: function(){
				var metadata = ZC.getOptions( this.element[0] );
				if( metadata.styletag ){
					$(metadata.styletag).remove();
					metadata.styletag = undefined;
				}
				if( this._hiddenitems && this._hiddenitems.length ){
					$(this._hiddenitems).show();
					this._hiddenitems = undefined;
				}	
				$('.'+ this._CLASS.EMPTY, this._menu ).remove();
			},
			_onMenuHide: function( origEvent ){
				this._removeClass( this.container, this._CLASS.DROPUP + ' '+ this._CLASS.DROPDOWN );
				this.container.setAttribute('aria-expanded', false);
				clearTimeout( this.inputtextTimer );
				this._clearStyle();
				this._updateSTATE();		
				if( this.options.searchable && !this._isEditable ){
					this._menu.find('input.zmenu__textbox').val(''); //No I18N
				}
				this._trigger( "ONHIDE", origEvent );		//No I18N
				// origEvent.stopImmediatePropagation();
			},
			_onMenuShow: function( origEvent ){
				this._activated = true;
				var hoveredItem;
				var visibleItems = this._getFilteredNodes(function( node ){
					return $( node ).css( 'display' ) !== 'none'; //No I18N
				}, this._queryAll( this._menu[0], 'li[data-filter-label]:not([hidden])' )); 
				if( visibleItems.length ){
					visibleItems = this._getFilteredNodes( ':not(.is-disabled)', visibleItems ); //No I18N
					if( origEvent.originalEvent && origEvent.originalEvent.type === 'input' ){ //No I18N
						/* Open dropdown if it contains visible items and hover the first matched items */
						hoveredItem = visibleItems[0]; 
						this._isEditable && this._autoComplete( this.textspan.value, visibleItems[0] );
					}else{
						if( this.options.multiple || !this._selected.length || typeof this._selected === 'string'){
							// hoveredItem = undefined;
							hoveredItem = this._menu[0].querySelector('li.zmenu__item:not([disabled]):not([hidden])');
						}else{ 
							hoveredItem = this._getMenuItemForOption( this._selected )[0];
						}
					}
				}else{
					if( this.options.acceptNewValues ){
						this._menu.hide();
						return;
					}
					$(this._menu[0].querySelector('ul')).append( "<li class='zmenu__item  "+ this._CLASS.EMPTY +"'><span>"+ this._localize('noResults')+"</span></li>" );  //No I18N
				}
				hoveredItem && this._menu.zmenu('hoverMenuItem', hoveredItem); //No I18N
				this.container.setAttribute('aria-expanded', true);
				this._setMenuWidth();	
				this._setState();
				this._menu.find('>ul.zmenu__list').height( this._menu.height() - this._menu.find('.zmenu__header').outerHeight() - this._menu.find('.zmenu__footer').outerHeight( true ) ); //No I18N
				if( this.options.searchable && !this._isEditable ){
					this._menu.find('input.zmenu__textbox').focus(); //No I18N
				}
				this._trigger( "ONSHOW", origEvent ); 	//No I18N
				// origEvent.stopImmediatePropagation();
			},
			_updateSTATE: function(){
				if( !this._activated ){ 
					var o = this.options;
					var itemToBeSelected = o.selectOnClose? this._lastHighlightedItem : ( (o.autoFill || o.autoComplete)? this._lastHoveredItem: undefined );
					if( !o.multiple && itemToBeSelected && itemToBeSelected !== this._selected[0] ){
						this._selectOption( itemToBeSelected, true );
					}else{
						!/(string|number)/.test( typeof this._selected ) && this._updateCombobox();
					}
				}
				if( this._focussed ){
					this.setFocus();
				}
				this._setState();
			},
			_onMenuItemHover: function( origEvent, userdata ){
				var o = this.options;
				this._lastHighlightedItem = this._getOptionForMenuItem( userdata.menuitem[0].id );
						
				if( !o.multiple && origEvent.originalEvent && !origEvent.originalEvent.originalEvent && !userdata.data.isFooter && (o.autoFill || o.autoComplete ) && (!o.searchable || !this._isEditable) ){
					this._lastHoveredItem = this._lastHighlightedItem;
					this._updateCombobox( [this._lastHoveredItem], false, false );
					return true;
				}
				// origEvent.stopImmediatePropagation();
			},
			_setInitial: function( ){
				this._selected = [];
				var o = this.options;
				var defaultOption;
				if( this.element[0].options && this.element[0].options.length ){
					if( o.selectedValue ){
						defaultOption = o.selectedValue;
					}else if( o.selectedIndex !== undefined ){
						if( parseInt( o.selectedIndex ) !== -1 ){
							defaultOption = this._getOptionAt( o.selectedIndex );
						}
					}else{
						defaultOption = $(this._getOptions()).filter(':selected,[data-selected]').toArray() ; //No I18N
					}
					o.selectedIndex = this.element[0].selectedIndex = -1;
					o.selectedValue = undefined;
					this._includePlaceholder();
				}
				this.setValue( defaultOption );
				o.disabled && this._setAttribute('disabled', o.disabled ); //No I18N
			},
			_getOptionForMenuItem: function( itemid ){
				return this.element[0].querySelector('[data-zitemid="'+ itemid +'"]'); //No I18N
			},
			_handleDS: function(){
				var ds = this.options.dataSource;
				if( ds && typeof ds === 'object' ){
					this._addOption( ds );
				}
			},
			_populateMenu: function( options ){
				var results = this._constructResultSet( options );
				if( results.length ){
					//We can part this and show
					var menuObjects = this._constructMenuObject( results );
					if( menuObjects.length ){
						var menuitems = [];
						this._menu.zmenu('addMenuItems', menuObjects, function( item ){ //No I18N
							menuitems.push( item[0] );
						}); 
						return menuitems;
					}
				}
			},
			_isSelected: function( data ){
				var element = this._getOptionForMenuItem( data.id );
				if( element ){
				 	return element.selected;
				}
			},
			_constructMenuObject: function( results ){
				var menuObjects = [];
				var o = this.options;
				var contentType = o.dropdownList.contentType || o.contentType;
				if( results ){
					results = Array.isArray( results ) ? results : [ results ];
					for( var i=0; i< results.length ; i++ ){
						var result = results[i];
						if( typeof result === 'string' ){ //No I18N
							result = { label: result }
						}
						var m_obj = {};
						var attr = m_obj.customAttributes = {};
						if( typeof result.customAttributes === 'object' ){ //No I18N
							attr = $.extend( attr, result.customAttributes );
						}
						var label = String( result.label );
						m_obj.label = label;
						m_obj.hidden = result.hidden;
						m_obj.disabled = result.disabled;
						attr.id = result.id;
						var selected = this._isSelected( result );
						if( !result.options ){
							if( result.isFooter ){
								m_obj.isFooter = true;
								m_obj.innerHTML = result.innerHTML || label;
								m_obj.itemType = 'custom'; //No I18N
							}else if( result.separator ){
								m_obj.itemType = attr.role  = 'separator'; // No I18N
							}else{	
								m_obj.description = result.description;
								m_obj.shortcutkey = result.shortcutkey;
								attr.style = attr.style || result.style;
								attr['data-filter-label'] = o.searchSettings && o.searchSettings.caseSensitive ? label : label.toLowerCase(); //No I18N
								/*Include all supported attributes*/
								attr.title = attr.title || result.title || ( contentType === 'icon'? label : undefined ); //No I18N
								m_obj.className = ( result.className || '' )+( selected? ' is-selected':'' ); //No I18N
								// attr.role = 'option'; //No I18N
								if( contentType === 'custom' ){ //No I18N
								    m_obj.itemType = 'custom'; //No I18N
								    m_obj.replaceWithText = result.replaceWithText;
								    m_obj.innerHTML = this._getCustomContent( result, o.dropdownList.template );
								}
								if( o.multiple ){
									var html = "<div class='zmenu__iconcontainer'><i class='zmenu__icon'></i></div>";
									if( contentType.indexOf('icon') !== -1 ){
										html = html+ "<div class='zmenu__iconcontainer'><i class='zmenu__icon "+ result.icon +"'></i></div>";
									}
									if( contentType === 'custom' ){ //No I18N
										html = html + m_obj.innerHTML;
									}else{
										html = html+ "<span class='zmenu__text'>"+ label +"</span>";
									}
									m_obj.innerHTML = html;
									m_obj.custom = true;
									m_obj.itemType = 'checkbox'; //No I18N
									m_obj.checked = selected;
								}else{
									m_obj.SVGIconId = result.SVGIconId;
									m_obj.iconClassName = result.iconClassName;
									//deprecated
									m_obj.icon = result.icon;
									m_obj.textAsIcon = result.textAsIcon;
								}
								if( this._textArray ){
									this._textArray.push({
										text: label,
										menuid: attr.id
									});	
								}
							}
						}else if( result.options ){
							// attr.role = 'group'; //No I18N
							m_obj.itemType = 'itemgroup'; //No I18N
						}
						menuObjects.push( m_obj );
						if( m_obj.itemType === 'itemgroup' ){ //No I18N
							for( var j=0 ;j< result.options.length;j++ ){
								var suboption = result.options[j];
								var subItemObj = this._constructMenuObject( suboption )[0];
								if( m_obj.disabled ){
									subItemObj.disabled = true;
								}
								if( m_obj.hidden ){
									subItemObj.hidden = true;
								}
								subItemObj.customAttributes['data-group'] = m_obj.customAttributes.id; //No I18N
								menuObjects.push( subItemObj );	
							}
						}
						m_obj.properties = result;
					}
				}	
				return menuObjects;
			},
			/* Function to construct the menu object with which menu can be created*/
			_constructResultSet: function( option ){
				if( option ){
					var results = [];
					option = Array.isArray( option ) ? option : [ option ];
					for( var i=0; i< option.length ; i++ ){
						var opt = option[i];
						var optData = ZC.getOptions( opt );
						if( opt.tagName === 'OPTION' ){ //No I18N
							optData.id = opt.getAttribute('data-zitemid') || 'zoption-'+ (  opt.id || Math.round(Math.random()*100000000) ); //No I18N
							optData.value = opt.value.trim();
						}else if( opt.tagName === 'OPTGROUP' ){ //No I18N
							optData.id = opt.getAttribute('data-zitemid') || 'zoptgroup-'+ (  opt.id || Math.round(Math.random()*100000000) ); //No I18N
							optData.options = this._constructResultSet( this._childs( opt ) );
						}else{
							continue;
						}
						optData.label = optData.label === undefined?  (opt.label || opt.text) : optData.label;
						optData.title = opt.title;
						optData.style = opt.getAttribute('style'); //No I18N
						optData.className = opt.getAttribute('class') || optData.className; //No I18N
						optData.hidden = optData.hidden || opt.hidden;
						optData.disabled = optData.disabled || opt.disabled;
						opt.setAttribute('data-zitemid', optData.id); //No I18N
						optData.element = opt;
						results.push( optData );
					}
					return results;
				}
			},
			_setIcon: function( mData , display ){
				if( this.iconspan ){
					this.iconspan.innerHTML = ''; //No I18N
					this.iconspan.className = this._CLASS.ICON;
					ZC.createIcon( $(this.iconspan), mData.icon, mData.SVGIconId, mData.iconClassName );
					if( display ){
						this.iconspan.style.display = ''; 
					}
				}
			},
			_setText: function( text ){
				if( this.textspan ){
					this._removeClass( this.textspan, this._CLASS.PLACEHOLDER );
					this.textspan.textContent = text;
				}
			},
			/*Function to find whether an element is disabled*/
			_isDisabled: function( element ){	
				return this._matches( element, '.'+this._CLASS.DISABLED );
			},	
			_setMenuWidth: function(){
				var elementWidth = this.container.offsetWidth;
				var direction = this._menu.data('direction') ? this._menu.data('direction').split('-') : undefined; //No I18N
				/* To add class to remove the border radius */
				if( direction ){
					this._addClass( this.container, direction[0] === 'top'? this._CLASS.DROPUP : this._CLASS.DROPDOWN );
				}
				/* To set the width of the menu to the combo width if (menuwidth < combowidth)*/
				if( this._menu.width() <= elementWidth && !/^(left|right)$/.test( direction ) ){
					if( direction ){
						direction = this._menu.data('direction'); //No I18N
						if( /^(bottom-right|top-right|top|bottom)$/.test(direction)  ){
							var left = parseFloat( this._menu.css('left') ); //No I18N
							left = left - ( elementWidth - this._menu.outerWidth() )*( /^(top|bottom)$/.test( direction )? 0.5:1 );
							this._menu.css('left', left); //No I18N
						}
					}
					this._menu.width(elementWidth - 2); //2 is the correction pixel.
				}	
			},
			_bindEvents: function(){
				$(this.container)
					.on('focusin'+this._namespace, this._onFocus.bind(this) ) //No I18N
				  	.on('click'+this._namespace, this._onClick.bind(this) )	//No I18N
				  	.on('keydown'+this._namespace, this._onKeydown.bind(this) ) //No I18N
				  	.on('focusout'+this._namespace, this._onBlur.bind(this) ); //No I18N
				$(this._closeIcon)
					.on('click'+this._namespace, this._onCloseButtonClick.bind(this) ); //No I18N
			},
			_onCloseButtonClick: function( origEvent ){
				this.deselectAllOptions( origEvent );	
				this._includePlaceholder( );	
				this._lastHoveredItem = undefined;
				return false;	
			},
			_onFocus: function( origEvent ){
				if( !this._focussed ){
					this._focussed = true;
					this._setState();
					this._trigger( "ONFOCUS", origEvent ); 	//No I18N
				}
			},
			_onKeydown: function( origEvent ){
				if( !this._isDisabled( this.container ) ){
					this._keyPress( origEvent );
				}
			},
			_onClick: function( origEvent ){
				if( !this._isDisabled( this.container ) ){
					if( this._menu && this._menu.is(':visible') ){
						this._close( origEvent );
					}else{
						this._open( origEvent );
					}
				}
			},
			_onBlur: function( origEvent ){
				clearTimeout( this.focusouttimer );
				this._focussed = false;
				this.focusouttimer = setTimeout( this._closeHandler.bind(this, origEvent), 1 );
			},
			_closeHandler: function( origEvent ){
				var activeElement = $(document.activeElement);
				if( /*!document.hasFocus()||*/!activeElement.closest( $(this.container).add( this._menu )).length ){
					this._focussed = this._activated = false;
					this._close( origEvent );
					this._lastHoveredItem = this._lastHighlightedItem = false;
					this._trigger( "ONBLUR", origEvent ); 	//No I18N
				}else{
					this._focussed = true;
				}
			},
			_onItemClick: function( origEvent, userdata  ){
				if( userdata.data ){
					userdata.data.menuitem = userdata.menuitem;
				}
				var menuData = userdata.data;
				var option = this._getOptionForMenuItem( menuData.menuitem[0].id );
				var trigger = false;
				if( option ){
					if( option.tagName === 'OPTION' ){
						var previousValue = this.options.selectedValue;
						if( menuData.isFooter ){ 
							this._activated = false;
							this._updateCombobox();
						}else if( this.options.multiple ){
							if( menuData.menuitem.prop('checked') ){ 
								if( !this.options.selectionLimit || this._selected.length < this.options.selectionLimit ){
									this._selectOption( option );
									trigger = true;
								}else{
									this._updateMenuItem( menuData.menuitem , false );
								}
							}else{
								option.selected = false;
								this._remove( this._selected, option );
								this._updateCombobox();
								trigger = true;
							}
						}else{
							this._activated = false;
							trigger = this._selected[0] !== option;
							this._selectOption( option );
							this._showDropdown( false, origEvent ); //added -- The menu close is controlled here.
						}
						var metadata = menuData.properties;
						metadata.selected = option.selected;
						trigger && this._trigger("ONCHANGE", origEvent, {previousValue: previousValue, optionData : metadata, menuitem: menuData.menuitem }); 	//No I18N
						this._trigger("ONITEMCLICK", origEvent, {previousValue: previousValue, optionData : metadata, menuitem: menuData.menuitem }); 	//No I18N
					}else{
						this._updateCombobox();
					} 
				}
				return false;
				// else{
					// return false;
				// }
				// this.setFocus();
			},
			_showDropdown: function(show, origEvent){
				show = show === undefined? true: show;
				show && this._loadDropdown();
				if( !show ){
					this._menu.zmenu('hide', undefined, undefined, undefined, origEvent); // No I18N
				}else{
					this._menu.zmenu('show',{ //No I18N
						forElement: this.container,
						mustBeShown:true,
						event:origEvent,
						positionAlterable: 'flipsidefit' //No I18N
					}, undefined);
				}
			},
			_reloadDropdown: function(){
				this._menu && $(this._queryAll(this._menu[0], 'li, .zmenu__footer')).remove(); //No I18N
				this._optArea.innerHTML = "";
				this._completeMenuLoaded = this.textspan = this.iconspan = undefined;
				//nulling textArray for keyAccess
				this._textArray = [];
				this._prevKey = this._prevTimeStamp = ''; // No I18N
				this._removeClass( this.container, this._CLASS.ICONONLY +' ' + this._CLASS.ICONTEXT );
				this._constructOptArea();
				this.setValue( this.options.selectedValue );
				// this._selected = $( this._selected, this.element[0] ).toArray();
				this._updateCombobox();
				this._close();
			},
			_loadDropdown: function(){
				if( !this._menu ){
					this._createMenu();
				}
				if( !this._completeMenuLoaded ){
					this._populateMenu( this._childs( this.element[0] ) );
					this._completeMenuLoaded = true;
				}
			},
			_updateMenuItem: function(menuitems, checked){
				if( checked ){
					menuitems = menuitems.filter(function( item ){
						return !item.checked;
					});
				}
				this._menu && this._menu.zmenu('setMenuItemsAttribute', menuitems, 'checked', checked);	//No I18N		
			},
			_selectOption: function( option, origEvent ){
				if( option ){
					var selected = $(option).filter('option'); //No I18N
					if( this.options.multiple ){
						selected = selected.add( this._selected );
					}
					this._selected = selected.prop('selected', true).toArray(); //No I18N
				}

				this._updateCombobox( this._selected, origEvent, true );
				this._lastHighlightedItem = this._lastHoveredItem = undefined;
			},
			/* Function to update the value in combobox*/
			_updateCombobox: function( selected, origEvent, updatedata ){
				var o = this.options;
				var base = this;
				selected = selected || this._selected;
				if(this._closeIcon){
					this._closeIcon.style.display = 'none'; //No I18N
				} 

				if( selected && selected.length ){
					var prevValue = o.selectedValue;
					var selectedText = ''; //No I18N
					this._removeClass( this.textspan, this._CLASS.PLACEHOLDER  );
					if( o.multiple ){
						selectedText = [];
						selected.forEach( function( opt ){
							var metaData = ZC.getOptions( opt );
							selectedText.push( metaData.label !== undefined ? metaData.label : opt.label.trim() );
						});
						var options = this._queryAll( this.element[0], 'option:not([hidden]):not([data-separator=true]):not([data-is-footer=true])');
						var unselectedOption = this._getFilteredNodes( function(){
							return !this.selected;
						} , options );
						this._setText( unselectedOption.length ? selectedText.join(', ') : this._localize('allOptionsSelected') ); //No I18N
					}else{
						var option = selected[0];
						var metaData = ZC.getOptions( option );
						metaData.label =  metaData.label !== undefined ? metaData.label : option.label.trim();
						selectedText = [ metaData.label ];
						if( o.contentType === 'custom' && !this._isEditable ){ //No I18N
							this.textspan.innerHTML = this._getCustomContent( metaData, o.template );
						}else{
							this._setText( selectedText[0] );
							this._setIcon( metaData , true );
						}
						this.container.setAttribute('aria-activedescendant', option.getAttribute('data-zitemid')); //No I18N	
					}
					if( o.contentType === 'icon' || this.textspan.clientWidth < this.textspan.scrollWidth ){	//No I18N
						this._optArea.setAttribute('title', selectedText.join(', ') );
					}else{
						this._optArea.removeAttribute('title');
					}	
					if( o.clearSelection ){
						this._closeIcon.style.display = ''; //No I18N
					}
				}else{
					this.container.removeAttribute('title');
					this.container.removeAttribute('aria-activedescendant');
					this._setText('');
					this._includePlaceholder();
				}
				if( updatedata || updatedata === undefined ){
					var prevValue = o.selectedValue;
					this._menu && this._menu.find('li').removeClass('is-selected'); //No I18N
					if( selected && selected.length ){
						o.selectedValue = [];
						o.selectedIndex = [];
						selected.forEach(function( opt ){
							o.selectedValue.push( base._getOptionValue( opt ) );
							o.selectedIndex.push( base._getOptionIndex( opt ) );
						});
						o.selectedValue = o.multiple? o.selectedValue : o.selectedValue[0];
						o.selectedIndex = o.multiple? o.selectedIndex : o.selectedIndex[0];
						if( this._menu ){
							var menuitems = this._getMenuItemForOption( this._selected );
							$(menuitems).addClass('is-selected'); //No I18N
						}
					}else{
						o.selectedValue = this.options.multiple? []:''; //No I18N
						o.selectedIndex = this.element[0].selectedIndex  = -1;
					}
					if( origEvent ){
						this._trigger("ONCHANGE",  origEvent.preventDefault? origEvent : undefined, { 'previousValue':prevValue });	//No I18N
					}
				}
			},
			_getOptionValue: function( opt ){
				if( opt ){
					if( opt.value !== '' ){ //No I18N
						return opt.value;
					}else{
						var optionData = ZC.getOptions( opt );
						if( optionData.value !== undefined ){
							return optionData.value;
						}else{
							return optionData.label;
						}
					}	
				}
			},
		    _getCustomContent: function( data, template ){
		   		return ( this._populateTemplate( template || data.innerHTML, data ) || data.label );
			},
			/*
				#prop# is replaced by actual property value
				previously ##prop## was supported, it will be deprecated
			*/
			_populateTemplate: function( template, data ){
				if( typeof template === 'string' ){ //No I18N
					var props = template.match(/#?#[^# ]+#?#/g);
					if( props ){
						props.forEach( function( prop ){
							template = template.replace( new RegExp( prop, 'g' ) ,function( match ){ //No I18N
								return data[match.replace(/#/g,'')]; //No I18N
							});
						});
					}
					return template;
				}
			},
			_insertOption: function( option, menuitem, action, refEle ){
				if( refEle ){
					if( typeof refEle === 'string' &&  !/^(#)/.test( refEle ) ){ //No I18N
						refEle = this._getOption( refEle )[0];
					}else{
						refEle = $(refEle, this.element)[0];
					}
				}
				if( !action || !refEle ){
					action = 'append'; //No I18N
					refEle = this.element;
				}
				$(refEle)[ action ](option);
				var parent = option[0].parentElement;
				if( !parent || !(/^(OPTGROUP|SELECT)$/.test(parent.tagName) ) ){
					action = 'append'; //No I18N
					this.element.append( option );
					parent = option[0].parentElement;
				}
				if( menuitem && menuitem.length ){
					var refItem = this._menu[0].querySelector('ul');
					if( parent.tagName === 'OPTGROUP' ){
		 				this._menu.zmenu( 'setMenuItemsAttribute', menuitem, 'disabled', parent.disabled );	//No I18N
		 				this._menu.zmenu( 'setMenuItemsAttribute', menuitem, 'hidden', parent.hidden  );	//No I18N   
						for( var i=0; i<menuitem.length; i++ ){
		 					menuitem[i].setAttribute( 'data-group', parent.id );
						} 	
					}
					if( action !== 'append' || refEle.tagName === 'OPTGROUP' ){
						if( refEle.tagName === 'OPTGROUP' && (action === 'after' || action === 'append') ){
							var siblings = $(refEle.children).not( option );
							refEle = siblings.length? siblings[siblings.length - 1] : refEle;
							action = 'after'; //No I18N
						}
						refItem = this._getMenuItemForOption( refEle )[0];
					}
					$(refItem)[action]( menuitem );
				}
			},
			_insertOptionAt: function( option, menuitem, index , group ){
				var baseEle = $(group, this.element).filter('optgroup')[0] || this.element[0];
				var refEle = this._queryAll( baseEle, 'option:not([data-separator=true]):not([data-is-footer=true])')[ index ];
				if( !refEle ){
					this._insertOption( option, menuitem, 'append', baseEle );	//No I18N
				}else{
					this._insertOption( option, menuitem, 'before', refEle );	
				}
			},
			_createOption: function( obj ,isGroup){
				var newOptions = [];
				var base = this;
				$.each( (!$.isArray(obj) ? [obj] : obj), function(i, o){
					var option;
					try{
						option = $(o);
					}catch(e){
						option = undefined;
					}
					if( option && option.length && option[0].nodeType ){
						if( /^(OPTGROUP|OPTION)$/.test( option[0].tagName ) ){
							newOptions.push( option[0] );	
						}
					}else{
						o = typeof o === 'string'? { value:o } : o;	//No I18N
						if( o.label === undefined && o.value === undefined && !o.isFooter && !o.separator ){
							return;
						}
						o.label = o.label === undefined? String( o.value ) : String( o.label );
						if( isGroup || o.options ){
							option = $('<optgroup />',{ id: o.id || undefined, label: o.label }); //No I18N	
							if( Array.isArray( o.options ) ){
								option.append( base._createOption( o.options ) );
							}
						}else{
							option = $('<option />',{   value: o.value, 
														id: o.id || undefined
							}).text( o.label );
							if( o.separator ){
								option.attr('data-separator', true); //No I18N
							}else if( o.isFooter ){
								option.attr('data-is-footer', true); //No I18N
							}
						}
						if( o.title !== undefined ){
							option[0].title = o.title;
						}
						if( o.style ){
							option[0].style = o.style;
						}
						option.prop({disabled:o.disabled, hidden: o.hidden, selected: o.selected});
						$.extend( ZC.getOptions( option ), o);
						newOptions.push( option[0] );
					}
				});
				return newOptions;
			},
			_trigger: function(customEvent, originalEvent, data){
			   data = data || {};
			   data.dropdownList = this._menu;
			   data.selectedValue = data.value = this.options.selectedValue || ''; //No I18N
	       	   data[ this._isEditable? 'combobox':'selectbox' ] = $(this.container); //No I18N
		       return this._dispatchEvent( customEvent, originalEvent, data );
			},
			_getOptionIndex: function( option ){
				return  this._getOptions().indexOf( option );
			},
			_getMenuItemForOption: function( option ){
				var menuitems = [];
				if( this._menu ){
					option = Array.isArray( option ) ? option : [option];
					for( var i = 0; i< option.length; i++ ){
						var menuitem = this._menu[0].querySelector('#' + option[i].getAttribute('data-zitemid') );		
						menuitem && this._add( menuitems, menuitem );
					}
				}
				return menuitems;
			},
			_keyPress: function( origEvent ){
				var code = this.currentKeyCode = origEvent.charCode || origEvent.keyCode;
				var isDropdownVisible = this._menu &&  this._menu.is(':visible'); //No I18N
				if( code === ZC.keyCode.ENTER || code === ZC.keyCode.SPACE ){
					if( !isDropdownVisible){
						origEvent.preventDefault();	
						origEvent.stopPropagation();
						this._open();
						return;
					}
					if( this.options.searchable ){
						if( code === ZC.keyCode.SPACE ){
							return;
						}else{
							if( this._menu[0].querySelector('.zselectbox__emptyresult') ){ //No I18N
								this._close();
								return;
							}
						}
					}
					origEvent.preventDefault();	
				}else if( code === ZC.keyCode.DOWN || code === ZC.keyCode.UP ){
					origEvent.preventDefault();
					origEvent.stopPropagation();
					if( !isDropdownVisible ){
						this._open();
						return;
					}
				}else if( code === ZC.keyCode.ESCAPE || (code === ZC.keyCode.TAB && isDropdownVisible) ){
					//In Firefox, pressing ESC shows old value bug --> https://bugzilla.mozilla.org/show_bug.cgi?id=524360 and the event is prevented.
					origEvent.preventDefault();
					if( isDropdownVisible ){
						origEvent.stopPropagation();
					}
					this._close( origEvent );
					return;
				}else if( !isDropdownVisible ){
					if( !this._isEditable && !this.options.multiple && /(text)/.test( this.options.contentType ) ){
						this._loadDropdown();
						var alphaNumericKey = String.fromCharCode(code);
			            if( this._textArray.length ){
							this._selectItemOnKeydown(alphaNumericKey,(new Date()).getTime());
						}
					}
					return;
				}
				this._handleDropdownKeycode(code);
			},
			_reloadTextArray: function(){
				if( this._completeMenuLoaded ){
					var options = this._queryAll( this.element[0] ,'option'); //No I18N
					var t = this._textArray = [];
					options.forEach(function(p){
						t.push({
							text: p.text.trim(),
							menuid: p.getAttribute('data-zitemid')
						});
					});
				}	
			},
			_selectItemOnKeydown: function(searchKey, currentTime){
				var search = function( start, end, pattern ){
					for( var x = start; x < end; x++){
						if((this._textArray[x].text).match(pattern)){
							menuitem = this._menu[0].querySelector('#' + this._textArray[x].menuid  );		
							if(menuitem && !this._matches(menuitem,'.is-disabled,[hidden]') ){
							  	break;
						  	}
						}
				  	}
				  	return x === end ? undefined : menuitem;	
				}.bind(this);
		    	var startIndex = this.options.selectedIndex + 1;
				var timeDiff = currentTime - this._prevTimeStamp;
			    if( !isNaN( timeDiff ) && timeDiff < 300 && this._prevKey !== searchKey ){
		        	searchKey =  (this._prevKey || '') + searchKey;
		        	startIndex--;
			    }
			    var pattern = new RegExp("^"+(searchKey).replace(/[|()[{.+*?$\\]/g, "\\$0"),"i");
		    	if( isNaN(startIndex) || startIndex >= this._textArray.length ){
		    		startIndex = 0;
		    	}
			    var menuitem = search( startIndex, this._textArray.length, pattern );
			    if( !menuitem && startIndex !== 0 ){
			    	menuitem = search( 0, startIndex, pattern );
			    }
			  	if( menuitem ){
					$(menuitem).trigger('mouseup'); //No I18N
			  	}
			  	this._prevKey = searchKey;
				this._prevTimeStamp = currentTime;  
			},
			_handleDropdownKeycode: function(keyCode){
				this._menu.zmenu('handleKeyCode', keyCode);	//No I18N
			},
			_remove: function( array, item ){
				var index = array.indexOf( item );
				if( index > -1 ){
					array.splice(index, 1);
				}	
			},
			_add: function(array, item){
				var index = array.indexOf( item );
				if( index < 0 ){
					array.push( item );
				} 
			},
			//options == should be an array of option nodes
			_removeOption : function( options ){
				if( options && options.length ){
					options = this._getFilteredNodes(':not([data-readonly=true])', options); //No I18N
					if( options.length ){
						this._selected =  $(this._selected).not( options ).toArray();
						$( this._getMenuItemForOption( options ) ).remove();
						$( options ).remove();
						this._updateCombobox();
						this._reloadTextArray();
					}
					return options;
				}
			},
			/*To include placeholder as a span tag for non-editable combo*/
			_includePlaceholder: function(){
				if( this.iconspan ){
					this.iconspan.style.display = 'none';
				}
				if( this.textspan && !this._isEditable ){
					this._addClass( this.textspan, this._CLASS.PLACEHOLDER );
					this.textspan.textContent = this.options.placeholder;
					this.container.setAttribute( 'aria-placeholder', this.options.placeholder ); 
				}
			},
			_getMatchedOption: function( val, groupselector){
				var base = this;
				val = String( val );
				return this._getFilteredNodes( function(){
					return val === String(base._getOptionValue( this ));
				}, this._getOptions( groupselector ));	
			},
			_getOption: function( value, groupselector ){
				var filteredOps = [];
				if( value !== undefined ){
					value = Array.isArray( value ) ? value : [value];
					for( var i=0; i<value.length; i++ ){
						var v = value[i];
						if( typeof v === 'string' || typeof v === 'number' ){ 
							if( /^(#)/.test( v ) ){
								v = this.element[0].querySelector( v );	
							}else{
							   filteredOps = filteredOps.concat( this._getMatchedOption( v , groupselector ) );
							}	
						}else if( v instanceof jQuery ){
							v = v[0];
						}
						if( v && v.nodeType === 1 && ( v.tagName === 'OPTION' || v.tagName === 'OPTGROUP' ) ){
							this._add( filteredOps, v );
						}
					}
				}else if( groupselector !== undefined ){
					return this._getOptions( groupselector );	 		
				}
				return filteredOps;
			},
			_getOptions: function( group ){
				var options = [];
				var selector = 'option:not([data-separator=true]):not([data-is-footer=true])'; //No I18N
				if( group ){
					group = $(group, this.element)[0];
					if( group ){
						options = this._queryAll( group, selector );
					}
				}else{
					options = this._queryAll( this.element[0], selector );
				}
				return options;
			},
			_getOptionAt: function( index, groupselector ){
				index = $.isArray(index)? index : [index];
				var filteredOps = [];
				var options = this._getOptions( groupselector );
				var base = this;
				index.forEach(function( i ){
					base._add( filteredOps, options[i] );
				});
				return filteredOps;
			},
			_reconstructOptArea: function(){
				this._optArea.innerHTML = "";
				this.textspan = this.iconspan = undefined;
				this._removeClass( this.container, this._CLASS.ICONTEXT+' '+ this._CLASS.ICONONLY  );
				this._constructOptArea();
				if( this.options.disabled ){
					this.option('disabled', true, true); //No I18N
				}
				this._updateCombobox();
			},
			_setState: function(){
				if( this._focussed ){
					// this.setFocus();
					if( this._activated ){
						this._addClass( this.container, this._CLASS.ACTIVE );
						this._removeClass( this.container, this._CLASS.FOCUS );
					}else{
						this._removeClass( this.container, this._CLASS.ACTIVE );
						this._addClass( this.container, this._CLASS.FOCUS );
					}	
				}else{
					this._removeClass( this.container, this._CLASS.FOCUS + ' '+  this._CLASS.ACTIVE );
				}
			},
			_setValue: function( value, groupselector, trigger ){
				if( value !== undefined ){
					var options = this._getFilteredNodes( function(){
										if( !this.hidden && !this.disabled ){
											var parent = this.parentElement;
											if( parent && parent.tagName === 'OPTGROUP' && parent.disabled && parent.hidden ){ //No I18N
												return false;
											}
											return true;	
										}
									}, this._getOption( value, groupselector ));
					if( !options.length ){
						!this.options.multiple && this.deselectAllOptions();
						// this._updateCombobox();
						return false;
					}
					if( this.options.multiple ){
						if( this.options.selectionLimit ){
							options = $(options).not( this._selected ).toArray().slice( this.options.selectionLimit - this._selected.length );
						}
						this._updateMenuItem( this._getMenuItemForOption(options), true );	
					}
					this._selectOption( options, trigger );
				}else{
					this._updateCombobox();
				}
			},
			_deselectOption:function( values, groupselector, trigger ){
				if( this._selected.length ){
					var options = $( this._selected ).filter( this._getOption(values, groupselector) );
					if( options.length ){
						options.prop('selected', false); //No I18N
						if( this.options.multiple ){
							this._updateMenuItem( this._getMenuItemForOption( options.toArray() ) , false);
						}
						this._selected = $(this._selected).not(options).toArray();
						this._updateCombobox( undefined, trigger );
					}
				}
			},
			_close: function( origEvent, deactivate){			
				if( deactivate || deactivate === undefined ){
					this._activated = false;
				}
				if( this._menu && this._menu.is(':visible') ){	//No I18N
					this._showDropdown(false, origEvent);
				}else{
					this._updateSTATE();
				}
			},
			_open: function(  origEvent  ){
				this._showDropdown(true, origEvent );
			},
			_addOption: function( obj, arg1, arg2 , isIndex ){
				if( obj ){
					var options = this._createOption(obj);
					if( options.length ){
						var menuitems = this._completeMenuLoaded ? this._populateMenu( options ) : undefined;
						if( menuitems && menuitems.length ){
							menuitems = this._getFilteredNodes(function( n ){
								return !n.closest( '.zmenu__footer' ); //No I18N
							}, menuitems );
						}
						if( isIndex ){
							this._insertOptionAt( options, menuitems, arg1, arg2 );	
						}else{
							this._insertOption( options, menuitems, arg1, arg2 );	
						}
						if( !this.frominit ){
							// this._updateCombobox();
							this._reloadTextArray();
						}
						return options;
					}
				}
			},
			_setOptionAttribute: function( option, optionName, value ){
				if( option ){
					var menuitem = this._getMenuItemForOption( option );
					var optionData = ZC.getOptions( option );
					var menu = $(this._menu);
					menuitem = menuitem.length? $( menuitem[0] ):undefined;
					var oldValue = optionData[optionName];
					optionData[optionName] = value;
					if( option.tagName === 'OPTGROUP' ){ 
						if( /^(label|disabled|hidden)$/.test( optionName ) ){
							menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
							if(/^(disabled|hidden)$/.test( optionName ) ){
								option[optionName] = value;
								this.setOptionAttributes( this._childs(option), optionName, value );
							}
						}
					}else{
						var selected = option.selected;
						if( optionName === 'value' ){	//No I18N
							option.value = value;
							selected && this._updateCombobox();
						}else if( optionName === 'label' ){	//No I18N
							if( typeof value === 'string' ){
								option.innerHTML = value;
								selected && this._updateCombobox();
								if( menuitem ){
									menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
									menuitem[0].setAttribute('data-filter-label', this.options.searchSettings.caseSensitive? value : value.toLowerCase());	//No I18N
								}
							}else{
								value = option.innerHTML;
							}
						}else if( optionName === 'disabled'  || optionName === 'hidden' ){	//No I18N
							option[optionName] = value;
							menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
							value && this.deselectOption( option, undefined, true );
						}else if( optionName === 'readonly' ){ //No I18N
							option.setAttribute('data-readonly', !!value);
						}else if( optionName === 'selected' ){ //No I18N
							if( value ){
								this._setValue( option );
							}else{
								this.deselectOption( option );
							}
							return;
						}else if( optionName === 'customAttributes' ){ //No I18N
							if( typeof value === 'object' ){ //No I18N
								Object.keys( value ).forEach( function( key ){
									if( menuitem && menuitem.length ){
										if( key === 'class' ){ //No I18N
											option['class'] && menuitem.removeClass( option['class'] ); //No I18N
											if( oldValue && oldValue['class'] ){	 //No I18N
												menuitem.removeClass( oldValue['class'] ); //No I18N
											}
											menuitem.addClass( value[key] );
										}else{
											menuitem[0].setAttribute( key, value[key] );
										}
									}
									option.setAttribute( key, value[ key ] );
								});
								optionData.customAttributes = $.extend( oldValue || {}, value );
							}
						}else{
							menu.zmenu( 'setMenuItemsAttribute', menuitem, optionName, value );	//No I18N
							selected && this._updateCombobox();
						}
					}			
				}
			},
			_setAttribute: function( optionName, value ){
				var menu = $(this._menu);
				var o = this.options;
				switch(optionName){
					case 'disabled': // No I18N
						this._close();
						o[optionName] = value;
						if( value ){
							 $( this.container ).addClass( this._CLASS.DISABLED ).attr('aria-disabled', true).removeAttr('tabindex').removeClass( this._CLASS.ACTIVE ).removeClass( this._CLASS.FOCUS ); //No I18N
						}else{
							 $( this.container ).removeClass( this._CLASS.DISABLED ).removeAttr('aria-disabled').attr('tabindex', this.element.attr('tabindex') || 0); //No I18N
						}
						break;
					case 'selectedIndex': //No I18N
						this._setAttribute('selectedValue', this._getOptionAt( value )); //No I18N
						break;
					case 'selectedValue': //No I18N
						this.deselectAllOptions();
						this.setValue( value );
						break;
					case 'dataSource': //No I18N
						var selectedValue = this.options.selectedValue; // preserving the selected value since same dataSource can be given with some modifications.
						o[optionName] = value;
						this.removeOption();
						this._selected = [];
						this._handleDS();
						this.setValue(selectedValue);
						break;
					case 'placeholder': //No I18N
						o[ optionName ] = value;
						if( this.textspan && o.selectedIndex === -1 ){
							this._includePlaceholder();
						}
						break;
					case 'contentType': //No I18N
						if( /^(text|icon|icon-text|custom)$/.test( value ) ){
							if( !o.multiple || value === 'text' ){
								o[ optionName ] = value;
								menu.zmenu('setAttribute', 'contentType', o.multiple? 'icon-text': value );	//No I18N
								this._reloadDropdown();	
							}
							break;
						}
					case 'dropdownList': //No I18N
						var keys = Object.keys(value);
						keys.forEach(function(key, index){
							if(/^(width|height|appendTo|className)$/.test(key)){
								o.dropdownList[key] = value[key];
								menu.zmenu('setAttribute', key, value[key]); //No I18N
							}
						});	
						break;
					case 'multiple': //No I18N
						value = !!value;
						if( value !== o[optionName] ){
							o[optionName] = value;
							if( value ){
								this._removeClass( this.container, this._CLASS.ICONONLY );
								this.element[0].setAttribute('multiple', true);
								o.contentType = 'text'; //No I18N
								menu.zmenu('setAttribute', 'contentType', 'icon-text');	//No I18N
							}else{
								this._selected = this._selected.slice(-1);
								this.element[0].removeAttribute('multiple');  //No I18N
								menu.zmenu('setAttribute','contentType',o.dropdownList.contentType || o.contentType ); //No I18N
							}
							this._reloadDropdown();
						}
						break;
					case 'selectionLimit': //No I18N
						value = parseInt( value );
						if( !isNaN( value )  || value >= 0 ){
							o[optionName] = value;
							if( o.multiple && value && this._selected.length > value ){
								this.deselectOption( this._selected.slice( 0, this._selected.length - value ) );
							}
						}
						break;
					case 'autoFill': //No I18N
						o[optionName] = !!value;
						break;
					case 'clearSelection': //No I18N
						o[optionName] = !!value;
						this._updateCombobox();
						break;
					case 'selectOnClose': //No I18N
						o[optionName] = !!value;
						break;
					case 'searchable': //No I18N
						o[optionName] = !!value;
						if( this._menu ){
							this._menu.zmenu('setAttribute','handleAccessKeyNavigation', !value ); //No I18N
							if( value ){
								this._createSearchBox();
							}else{
								this._menu.find('.zmenu__header').remove(); //No I18N
							}
						}
						break;
					case 'searchSettings': //No I18N
						var base = this;
						var keys = Object.keys(value);
						keys.forEach(function(key, index){
							if(/^(delay|searchCriteria|caseSensitive|minKeywordLength)$/.test(key)){
								if(key === 'searchCriteria'){ //No I18N
									if(!/^(startswith|contains)$/.test(value[key]) || ( o.autoComplete ) ){
										return true;
									}
								}else if(key === 'caseSensitive'){//No I18N
									if( base._menu ){
										base._getFilteredNodes( function( n ){
											var label = ZC.getOptions( n ).label;
											n.setAttribute('data-filter-label', value[key]? label : label.toLowerCase());
										}, base._queryAll( base._menu[0], 'li[data-filter-label]' ));
									}
								}
								o.searchSettings[key] = value[key];
							}
						});	
						break;
					default:
				}		
			},
			 /*JQUERY HELPER CLASSES*/
		 	_getFilteredNodes: function( filter, nodeList ){
				nodeList = Array.isArray( nodeList ) ?  nodeList : [nodeList];
				var filteredNodes = [];
				for( var i=0; i<nodeList.length; i++ ){
					var isMatched, n = nodeList[ i ];
					if( n && n.nodeType ){
						if( typeof filter === 'string' ){
							isMatched = this._matches( n, filter );	
						}else if(  typeof filter === 'function' ){ //No I18N
							isMatched = filter.call( n, n );
						}
						if( isMatched ){
							this._add( filteredNodes, n );
						}
					}
				}
				return filteredNodes;
		   },
		   _matches: function(node, selector){
		   		var p = Element.prototype;
				var f = p.matches || p.webkitMatchesSelector || p.mozMatchesSelector || p.msMatchesSelector || function(s) {
					return [].indexOf.call(document.querySelectorAll(s), this) !== -1;
				};
				return f.call(node, selector);
		   },
		   _queryAll: function( node, selector ){
		   	 	return this._toArr(  node.querySelectorAll( selector ) );
		   },
		   _childs: function( node ){
		    	return this._toArr(  node.children );
		   },
		   _toArr: function( list ){
		   		return Array.prototype.slice.call( list, 0 );
		   },
	      _removeClass: function( node, className ){
				if( node && node.nodeType === 1 && typeof className === 'string'  ){ //No I18N
					var nowhitespace = /[^\x20\t\r\n\f]+/g;
					var clsList = className.match( nowhitespace );
					clsList.forEach( function( clazz ){
			   			node.className = node.className.replace( new RegExp("(([ ]|^)"+ clazz +"([ ]|$))" ,'g'), ' '); //No I18N
					});
				}
				return node;
		   },
		  	_addClass: function( node, className ){
				if( node && node.nodeType === 1 && typeof className === 'string' && className ){ //No I18N
					var nowhitespace = /[^\x20\t\r\n\f]+/g;
					var clsList = className.match( nowhitespace );
					var curClassList = node.className.match( nowhitespace ); 
					clsList.forEach( function( clazz ){
						if( curClassList.indexOf( clazz ) === -1 ){
							curClassList.push( clazz );
						}
					});
					node.className = curClassList.join(" ");
				}
				return node;
		   },
				/****PUBLIC FUNCTIONS****/
			/***********************/
			/** Method to set focus to the component.
				@function combo-box.setFocus
				@newlyAdded true
			*/
			setFocus: function(){
				this.container.focus();
				!this._isDisabled( this.container ) && this._onFocus();
			},
			/** Method to close the dropdown list.
				@function combo-box.close
			*/
			close: function(){
				this._close();
			},
			/** Method to open the dropdown list.
				@function combo-box.open
			*/
			open: function(){
				this._open();
			},
			/** 
				Method to re-render the dropdown list.
				@function combo-box.refresh
			*/
			refresh: function(){
				ZC.clearMetadata( this._queryAll( this.element[0], 'option,optgroup' ) ); //No I18N
				this._reloadDropdown();
			},
			/** This method adds item/items to the dropdown.
				@function combo-box.addOption
				@param {Object} items Array of item Properties or HTMLOptions that is/are to be added. 
				@param {String} action 
				Specifies whether to insert the new option after or before or to be appended to the element(option/optgroup) given in the third argument .
				Valid values are 'after', 'before' and 'append'.
				@param {Object|String} selector Selector or DOMElement of the option or optgroup relative to which the new option is placed based on the argument 'action'.
				@returns {Object} Newly added options.
				@example <caption> Option added using item-properties </caption>
	$(selector).zcombobox('addOption',[{
		label:'India',
		value:'IND',
	}], 'after', 'AF')
				@example <caption> Option added using HTML </caption>
	$(selector).zcombobox('addOption',$('<option value='IND' data-icon='flags ind'>India</option>'), 'after', 'AF');
				@example <caption> To insert an option to a group </caption>
	$(selector).zcombobox('g',[{
		label:'FastTrack',
		value:'FT'
	},{
		label:'Titan',
		value:'TT'
	}],'append','#watchBrands')
			*/
			addOption: function(obj, action, refElement){
				return this._addOption.apply( this, arguments );
			},
			/** This method adds option/options to the dropdown at the specified index( Index is Zero based excluding the separators).
				@function combo-box.addOptionAt
				@param {Object} items Array of item Properties or HTMLOptions that is/are to be added. 
				@param {Number} [Index] Specifies the location at which the new item/items is/are inserted.
				@returns {Object} Newly added items.
				@example <caption> Option added using item Properties </caption>
	$(selector).zcombobox('addOption',{
		label:'India',
		value:'IND'
	},4)
				@example <caption> Option added using HTML </caption>
	$(selector).zcombobox('addOptionAt',$('<option value='IND' data-icon='flags ind'>India</option>'),4);
			*/
			addOptionAt: function(obj, index, group){
				return this._addOption( obj, index, group, true );
			},
			/** This method adds groups to the dropdown list.
				@function combo-box.addOptGroup
				@param {Object} group
				Array of Group properties or HTMLOptGroups to be added to the dropdown list. Refer 'group Properties' for the available options.
				@param {String} Value/#option-id Selector of the item or group before which the new group is added. If not mentioned, the new group is appended to the dropdown list.
				@returns {Object} Newly added groups.
				@example <caption> Group added using group-properties </caption>
	$(selector).zcombobox('addOptGroup',{
		id:'BagBrands',
		label:'Bag Brands',
		options:['WildCraft','Fastrack']
	},'#WatchBrands')
			*/
			addOptGroup: function(obj, refElement){
				if(obj){
					var groups = this._createOption( obj , true );
					if( groups.length ){
						var menuitems = this._completeMenuLoaded ? this._populateMenu( groups ) : undefined;
						this._insertOption( groups	,menuitems, 'before', refElement );	//No I18N
					}
					return groups;
				}
			},
			/** This method removes the item/items from the dropdown list given the value/#optionid of the items.
				@function combo-box.removeOption
				@param {String|Object} value Specifies the value/#optionid or array of values/#optionids whose items are to be removed. If not mentioned, all items are removed.
			*/
			removeOption: function(value, groupselector){
				return this._removeOption( ( value === undefined && groupselector === undefined )? this._queryAll( this.element[0], 'option,optgroup' ) : this._getOption(value, groupselector)  );
			},	
			/** This method removes the item/items located at the specified index/indexes.
				@function combo-box.removeOptionAt
				@param {Number|Object} index Specifies position or array of positions of item/items that is/are to be removed. Index is 0 based and the separators are excluded.
			*/
			removeOptionAt: function(index, groupselector){
				return this._removeOption( this._getOptionAt(index, groupselector) );
			},
			/** This method removes the group given the groupselector.
				@function combo-box.removeOptGroup
				@param {String} group-selector Selector of the group to be removed.
			*/
			removeOptGroup: function( groupselector ){
				var g = this._getOption( groupselector );
				var base = this;
				g.forEach(function( _g ){
					base._removeOption( base._childs( _g ) );
					if( !base._childs(_g).length ){
						base._removeOption( [_g] );
					}
				});
			},
			/** Method to select the item that matches given the value/#option-id. This method is valid only when multiple is not set to true.
				@function select-box.setValue
				@param {String} Value  Value/#option-id of the item that is to be selected.
			*/
			/** Method to select the item that matches given the value/#option-id of the option.
				@function combo-box.setValue
				@param {String} Value  Value/#option-id of the item that is to be selected.
			*/
			setValue: function(value, groupselector, trigger){
				// !this.options.multiple && this.deselectAllOptions();
				this._setValue( value, groupselector, trigger );
				return this.options.selectedValue;
			},
			/** Method to select the items that matches the given array of values/#option-ids. Method is valid only when multiple is set to true.
				@function select-box.setValues
				@param {String} Values  Array of values or #option-ids of the items that are to be selected.
			*/
			setValues: function( values, groupselector,trigger ){
				return this.setValue.apply( this, arguments );
			},
			/**
			Method to clear the selection.
				@function combo-box.deselectAllOptions
			*/
			deselectAllOptions: function( trigger ){
				this._deselectOption( this._selected, undefined, trigger );	
			},
			/** Method to deselect a particular item given the value/#option-id.
				@function combo-box.deselectOption
				@param {Object} value  Value/#option-id or array of Values/#option-ids  is/are to be deselected.
			*/
			deselectOption: function( value, groupselector, trigger ){
				this._deselectOption.apply( this, arguments );
			},
			/** This method destroys the component instance.
				@function combo-box.destroy
			*/
			destroy: function(){
				if( this._menu ){
					this._menu.zmenu('destroy'); //No I18N
					this._menu.remove();	
				}
				$(this.container).remove();
				this.element.show().removeData('zcombobox').removeData('zselectbox').removeData('metadata') //No I18N
							.children().removeData('metadata'); //No I18N
			},
			setOptionAttributes: function( options, optionName, value ){
				if( typeof optionName === 'object' && !$.isEmptyObject(optionName) ){	// No I18N
					var base = this;
					$.each(optionName, function( o,v ){
						base.setOptionAttributes(options, o ,v );
					});
				}else if( optionName && value !== undefined ){
					options = this._getOption( options );
					for( var i=0; i<options.length; i++ ){
						if(   this._matches( options[i], ':not([data-readonly=true])' ) ){
							this._setOptionAttribute( options[i], optionName, value );
						}
					}
				}
			},
			setAttribute: function( optionName, value , force){
				if( value !== this.options[optionName] || optionName === 'dataSource'  || force ){
					value = typeof value === 'string'? value.trim() : value; //No I18N
					this._setAttribute( optionName, value );
				}
			},
			/** Method to return the selected value.
				@function combo-box.getValue
				@returns {string} Currently selected value.
			*/
			getValue: function(){
				return this.options.selectedValue;
			},	
			/** Method to return the selected values. Valid only when multiple is set to true.
				@function select-box.getValues
				@returns {string} Currently selected values.
			*/
			getValues: function(){
				if( this.options.multiple ){
					return this.getValue();
				}
			},
			/** Method to return the created component.
				@function select-box.getValues
				@returns {Object} Currently the component.
			*/
		};
		ZC.registerComponent("zselectbox", selectboxPrototype); //No I18N
		var isIE9 = ( ZC.Browser.getIEVersion() > 9 || ZC.Browser.getIEVersion() === -1 ) ? false : true;
		ZC.zcombobox = { }

			/** @member {Object} [combo-box.dataSource = undefined]

				To specify the data source from which dropdown list has to be populated. DataSource is an array of itemProperties. Refer 'itemProperties' list for the list of properties that can be applied to an item.

				@example <caption> To populate using dataSource</caption>

	$(selector).zcombobox({
		contentType : 'custom',
		dataSource:[{
			label:'Mathew',
			icon:'employeeicon mathew_img'
		},{
			label:'James',
			icon:'employeeicon james_img'
		},{
			label:'Rose',
			icon:'employeeicon james_img'
		}]
	});
			*/
	        /** @member {number} [combo-box.selectedIndex = 0] 
	    		Specifies the index of the initially selected option. Index is zero based excluding separators( items with property 'separator' = true ). By default, first option is selected.
	        */
	        /** @member {String} [combo-box.selectedValue = undefined] 
	        	Specifies the initially selected value. SelectedValue is given priority over selectedIndex.
	        */
			/** @member {String} [combo-box.placeholder = "Select"]
				The text displayed when none of the values are selected. 
			*/
			/** @member {Object} 
				combo-box.searchSettings Specifies the search settings. Search is done based on the item label.
				@example <caption>  Search Settings </caption>
	$(selector).zcombobox({
		searchSettings:{
			delay:20,
			searchCriteria: 'contains',
			caseSensitive: true,
			minKeywordLength:3
		}
	});
			*/
			   /**  @member {Number} [combo-box.searchSettings.delay = 150] 
			   		Specifies the search delay.
			   	*/
		       /** @member {String} [combo-box.searchSettings.searchCriteria = 'startswith'] 
		       	   Determines the search method that can be used to match the typed text.
		       	   @allowedValues {('contains'|'startswith')}
		       */              
		       /** @member {Boolean} [combo-box.searchSettings.caseSensitive = false]  
		       	   Specifies whether searching is case sensitive or not. 
		       	*/          
		       /** @member {Number} [combo-box.searchSettings.minKeywordLength = 0] 
		           Specifies the minimum character the user must type in the combobox, before the filtered list is displayed. 
		       */  
	        /** @member {Boolean} [combo-box.autoComplete = 'false'] 
	        	Specifies whether suggestion text is highlighted in the combobox when options are visited( hovered ) using arrow keys.
	        */
	        /** @member {object} combo-box.dropdownList Configurations for the dropdown list. */
	        	/** @member {Number} [combo-box.dropdownList.height = 'auto'] Determines the height of the list. */
	        	/** @member {Number} [combo-box.dropdownList.width = 'auto'] Determines the width of the list. */
	        	/** @member {(Object|String)} [combo-box.dropdownList.appendTo = 'body'] Selector or HTMLElement of the container to which the dropdown list is appended. */
	    		/** @member {String} combo-box.dropdownList.className Custom classname that is added to the dropdown list. */
	    		/** @member {String} [combo-box.dropdownList.direction='auto'] The direction in which the dropdown list opens. If the value is 'auto', it opens in bottom-left direction.
	    			@allowedValue {('top'|'bottom'|'left'|'right'|'bottom-left'|'bottom-right'|'top-left'|'top-right'|'auto')}
	    		*/
	        /** @member  {Boolean} [combo-box.disabled = 'false'] Determines whether the combobox is disabled. */
	        /** @member {String} [combo-box.contentType = 'text'] 
	Determines whether the combobox and the dropdown list has only text or both icon and text. If set to 'custom', the dropdown list can have custom HTML, but the combobox is populated with the label of the selected item.
	If contentType is set to 'custom', label must be given,and the HTML content is given in 'innerHTML' option of each item and any '#label#' in the html is replaced by the item label.
	        	@allowedValues {('text'|'icon-text'|'custom')}
	        	@example <caption> Example using contentType = 'custom' </caption>
	$('#employees').zcombobox({
		contentType:'custom',
		dataSource: [{
			label: 'Mathew',
			innerHTML:'<span class="employee_name">#label#</span><span class='employee_designation'>Designer</span>'
		},{
			label: 'James',
			innerHTML:'<span class='employee_name'>#label#</span><span class='employee_designation'>Developer</span>'
		},{
			label: 'Rose',
			innerHTML:'<span class='employee_name'>#label#</span><span class='employee_designation'>Designer</span>'
		}]	
	});
	        	@example <caption> Example using contentType='icon-text' </caption>
	<select id='employees' data-content-type='icon-text'>
		<option data-icon ='employeeicon mathew_img' >Mathew</option>
		<option data-icon = 'employeeicon james_img'>James</option>
		<option data-icon = 'employeeicon james_img'>Rose</option>
	</select>

	        */
    	/** @member {String} [combo-box.arrowIconClassName] - One or more space separated CSS classes, which contains configuration about the arrow icon to be shown in the menu button. 
			@newlyAdded true
		@example <caption> Example: </caption>
	    $("#country").zcombobox({
	        "arrowIconClassName" : "zicon zicon-arrow-right"
	    });
			*/
	        /** @member {String} [combo-box.arrowSVGIconId] - The SVG element Id followed by space separated CSS class names to replace the default arrow icon. 
			@newlyAdded true
		@example <caption> Initialize the menu button with the "arrowSVGIconId" option specified: </caption>
	    $("#country").zcombobox({
	        "arrowSVGIconId" : "arrowIconID custom__class"
	    });
	        */
			/** @member {String} combo-box.className Custom classname that is added to the combobox component.*/
		var comboboxPrototype =  {
			options: $.extend( {}, selectboxPrototype.options, {
				searchSettings: {
			       delay: 150,  
			       searchCriteria: 'startswith',  //No I18N  
			       caseSensitive: false,             
			       minKeywordLength: 0,
			       searchByField: 'label' //No I18N
			     },
			     autoComplete: false,
		         autoFill: true, 
		         messages:{
		        	noResults:'No Results'	//No I18N
		         },
				 eventPrefix:'zcombobox'	//No I18N
			}),
			_initialize: function( element, options ){
				this._isEditable = true;
				options.searchable = options.multiple = false;
				if( options.autoComplete ){
					options.searchSettings.searchCriteria = 'startswith'; //No I18N
				}
				if( !/^(text|icon-text|custom)$/.test( options.contentType ) ){
					options.contentType = $.zcombobox.prototype.options.contentType;
				}
			},
			_constructComponent: function(){
				this._super();
				this._addClass( this.container, 'zcombobox' ); //No I18N
			},
			_constructOptArea: function(){
				var o = this.options;
				if( this.options.contentType.match('icon') ){
					var i = this.iconspan = document.createElement('i');
					i.className = this._CLASS.ICON;
					this._optArea.appendChild( i );
				}
				var t = this.textspan = document.createElement('input');
				t.className = 'zselectbox__textbox';
				t.setAttribute('placeholder', o.placeholder );
				var tabindex = this.element[0].getAttribute('tabindex');
				if( tabindex !== undefined ){
					t.setAttribute('tabindex', tabindex );
				}
				this._optArea.appendChild( this.textspan );
				this.container.setAttribute('tabindex', -1);
				this._bindSearchbox( this.textspan );
			},
			_onMenuItemHover: function( origEvent, data ){
				if( this._super.apply( this, arguments ) ){
					this.textspan.select();
				}
			},
			_onFocus: function( origEvent ){
				if( !this._focussed ){
					this.textspan.select();
				}
				this._super( origEvent );
			},
			_onClick: function( origEvent ){
				var _super = this._super;
				if( !this._isDisabled( this.container ) && $(origEvent.target).closest( this._arrow ).length  ){
					_super.call( this );
					this.textspan.select();
					this._clearStyle();	
				}
			},
			_bindEvents: function(){
				var base = this;
				this._super();
				var $textspan = $(this.textspan);
				$textspan.on('mousedown'+this._namespace, function( origEvent ){ //No I18N
					if( !base._focussed ){
						$textspan.on('mouseup'+base._namespace, function( origEvent ){ //No I18N
							$textspan.off('mouseup'+base._namespace).select(); //No I18N
							origEvent.preventDefault();
						});
					}
					base._onFocus( origEvent );
				}); 	
			},
			_onCloseButtonClick: function( origEvent ){
				this.deselectAllOptions( origEvent );	
				this._lastHoveredItem = undefined;
				this._setText(''); //No I18N
				if( this._activated ){
					this._onInput( this.textspan, origEvent );
				}
				this.textspan.focus();
				return false;
			},
			_setText: function( text ){
				if( this.textspan ){
					this.textspan.value = text;
				}
			},
			_onInput: function(){
				if( this.iconspan ){
					this.iconspan.style.display = 'none'; //No I18N
				}
				this._super.apply(this, arguments);
				// this._removeClass( this.container, this._CLASS.FOCUS );
				// this._addClass( this.container, this._CLASS.ACTIVE );
			},
			_searchQuery: function( origEvent, query ){
				var s = this.options.searchSettings || {};
				var minKeywordLength = Number(s.minKeywordLength)!=='NaN'? Number(s.minKeywordLength) : 1;	//No I18N
				if( !minKeywordLength || query.length >= minKeywordLength ){
					this._super( origEvent, query );
				}else{
					this._close( origEvent, false );
				}
			},
			_setValue: function( value, groupselector, origEvent ){
				if( this._super( value, groupselector, origEvent ) === false && /(string|number)/.test( typeof value ) ){
					if( this.options.acceptNewValues && value !== this._selected ){
						var previousValue = this._selected;
						this._selected = this.options.selectedValue = value;
						this.options.selectedIndex = this.element.selectedIndex = -1;
						this._setText( value );
						origEvent && this._trigger( "ONCHANGE", origEvent, { previousValue: previousValue } ); 	//No I18N
					}
				}
			},
			_keyPress: function( origEvent ){
				var code = this.currentKeyCode = origEvent.charCode || origEvent.keyCode;
				if( code === ZC.keyCode.ENTER && this.options.acceptNewValues ){
					this._selectMenuItemByContent( this.textspan.value, origEvent );
				}else if( code !== ZC.keyCode.SPACE ){
					this._super( origEvent );
				}
			},
			_selectMenuItemByContent: function( label, origEvent ){
				if( this._menu ){
					var menuitem = this._getItemByLabel( label );
					if( menuitem ){
						$(menuitem).trigger('mouseup'); //No I18N
					}else{
						this._setValue( this.textspan.value, undefined, origEvent );
						!this.options.multiple && this._close();
					}	
				}
				return;
			},
			_getItemByLabel: function( query ){
				if( this.options.searchSettings.searchByField === 'label' ){ //No I18N
					return this._menu.find('li:not([hidden])[data-filter-label="'+ (this.options.searchSettings.caseSensitive? query: query.toLowerCase()) +'"]')[0]; //No I18N	
				}else{
					var base = this;
					this._queryAll( this._menu[0],'li:not([hidden])[data-filter-label]').forEach(function( node ){ //No I18N
						var menuData = ZC.getOptions( node );
						if(!( base._menuitemMatcher( menuData , query )  ) ){
							return node;
						}	
					});
				}
			},
			/*Function to autoComplete Combo*/
			_autoComplete: function( currentText, matchedItem ){			
				if( this.options.autoComplete && currentText.length >= this.options.searchSettings.minKeywordLength && currentText ){
					var keyCode = this.currentKeyCode;
					if( currentText && currentText !== '' && matchedItem ){
						var mData = ZC.getOptions( matchedItem );
						this._setIcon( mData, true );
						if(keyCode !== ZC.keyCode.BACKSPACE && keyCode !== ZC.keyCode.DELETE){
							this._setText( mData.label );
							var start = mData.label.toLowerCase().indexOf(currentText.toLowerCase()) + currentText.length;
							this.textspan.setSelectionRange(start, mData.label.length);
						}
						this._lastHoveredItem = this._getOptionForMenuItem( matchedItem.id );
					}else if( this.iconspan ){
						this.iconspan.style.display = 'none';
					}	
				}
			},
			setFocus: function(){
				if( this._focussed && this._activated ){
					this.textspan.focus();
				}else{
					this.textspan.select();
				}
			},
			_setAttribute: function( optionName, value ){
				if( optionName === 'disabled' ){
					this._close();
					this.options[optionName] = value;
					if( value ){
						$(this.container).addClass( this._CLASS.DISABLED ).attr('aria-disabled', true).removeAttr('tabindex'); //No I18N
						$(this.textspan).prop('disabled', true).removeClass( this._CLASS.ACTIVE ).removeClass( this._CLASS.FOCUS ); //No I18N
					}else{
						$(this.container).removeClass( this._CLASS.DISABLED ).removeAttr('aria-disabled').attr('tabindex', -1); //No I18N
						$(this.textspan).prop('disabled', false); //No I18N
					}
				}else if( optionName === 'placeholder' ){ //No I18N
					this.options[optionName] = value;
					this.textspan.setAttribute('placeholder', value); //No I18N		
				}else if( optionName === 'autoComplete' ){ //No I18N
					this.options[optionName] = !!value;
					if( value ){
						this.options.searchSettings.searchCriteria = 'startswith'; //No I18N
					}
				}else{
					if( /^(multiple|selectionLimit|searchable)$/.test( optionName ) || ( optionName === 'contentType' && !/^(text|icon-text|custom)$/.test(value) ) ){
						return;
					}
					this._super( optionName, value );
				}
			}
		};
		ZC.zcombobox = {
			DEFAULTS: comboboxPrototype.options
		};
	   	ZC.registerComponent("zcombobox", $.zselectbox, comboboxPrototype); //No I18N
	   	var createSelectbox = function( options, baseFunction ){
	  		if(typeof options === 'object'){ //No I18N
				var element = $('#' + options.id); //No I18N
				if( element.length ){
					if(element.data( baseFunction )){
						return element;
					}
				}else{
					element = $('<select>').appendTo( $(options.appendTo).length? options.appendTo:'body' ) // No I18N
										   .attr({
											   	id: options.id,
												style: options.style,
												title: options.title   
											}); 
				}
				element[ baseFunction ](options);
				return element;
			} 	
	   }
	   ZC.createCombobox = function(options){
			createSelectbox( options, 'zcombobox' ); //No I18N
	   }
	   ZC.createSelectbox = function(options){
			createSelectbox( options, 'zselectbox' ); //No I18N
	   }
	})( ZComponents );
	// API
	/** @options combo-box.item-properties This lists properties of each items available in the dropdown list.
		These properties can be set to HTMLOptions as attributes and the custom attributes are prefixed with 'data-'.
		@example <caption> Setting Properties to HTMLOptions </caption>
	<select id='country'>
		<option data-icon='af' value='AF'>Afghanistan</option>
		<option selected data-icon='al' value='AL'>Algeria</option>
		<option data-separator = true></option>
		<option disabled data-icon='Ind' selected=true value='IND'>India</option>
		<option hidden data-icon='us' value='US'>United States</option>
	</select>
	$('#country').zcombobox();
			@example <caption> Setting properties in the datasource </caption>
	<select id='country'></select>
	$('#country').zcombobox({
		dataSource:[{
			label:'Afghanistan',
			value:'AF',
			icon:'af'
		},{
			label:'Algeria',
			value:'AL',
			icon:'al'
		},{
			separator:true
		},{
			label:'India',
			value:'IND',
			icon:'ind'
		},{
			label:'United States'
			value:'US'
			icon:'us'
		}]
	});
		*/
		/** @member {Boolean} [combo-box.item-properties.separator = false] 
			Specifies whether the item is a separator. Separators are not selectable.
		*/
		/** @member {String} [combo-box.item-properties.value = null] 
			Specifies the value.
		*/
		/** @member {String} [combo-box.item-properties.label = null]
			Specifies the label.
		*/
		/** @member {String} [combo-box.item-properties.icon = undefined] 
			Specifies the icon. Use iconClassName / SVGIconId instead.
		    @deprecated
		*/
		/** @member {String} [combo-box.item-properties.SVGIconId = undefined] 
			Specifies SVG element Id followed by space separated CSS class names.
		    @newlyAdded
		*/
		/** @member {String} [combo-box.item-properties.iconClassName = undefined] 
			One or more space separated CSS classes, which configures the icon.
		    @newlyAdded
		*/
		/** @member {String} [combo-box.item-properties.description = undefined]
			Specifies the secondary text for the item. 
		*/
		/** @member {String} [combo-box.item-properties.innerHTML = undefined]
			Specifies the html content when contentType='custom' or when isFooter=true for a particular item. When used as customHTML for contentType='custom, any "#label#" in the content is replaced by item label and items are preferred to be given in datasource rather than in HTML. Refer option 'contentType' for example.
		*/
		/** @member {(String|Object)} [combo-box.item-properties.customData = undefined]
			User required information can be given here. It is passed on to the user when the particular item is selected.	
		*/
		/** @member {String} [combo-box.item-properties.id = undefined]
			Unique id of the item.
		*/
		/** @member {Boolean} [combo-box.item-properties.readonly = false]
		   If set to true , removal and updation of this particular option is prevented.
		*/
		/** @member {Boolean} [combo-box.item-properties.disabled = false]
			If set to true, the item is disabled.
		*/
		/** @member {Boolean} [combo-box.item-properties.hidden = false]
			If set to true, the item is hidden.
		*/
		/** @member {Boolean} combo-box.item-properties.isFooter
			Specifies whether this item is added as footer and the html content of the footer is given in 'innerHTML' property.
		*/
		
	// OPTGROUP OPTIONS
	/** @options combo-box.group-properties This lists properties of each group available in the dropdown list.
		These properties can be set to HTMLOptGroup as attributes inside select tag and the custom attributes are prefixed with 'data-'.
		@example <caption> Groups represented in HTML </caption>
	<select id='laps'>
		<optgroup label="Lenovo" id="lenovolaps">
			<option selected=true value="Lenovo IdeaPad">Lenovo IdeaPad</option>
			<option value="Lenovo UltraBook">Lenovo UltraBook</option>
		</optgroup>
		<optgroup label="HP" id="hplaps">
			<option value="HP Pavilion Notebook">HP Pavilion Notebook</option>
			<option value="HP ENVY Notebook">HP ENVY Notebook</option>
		</optgroup>
	</select>
	$('#laps').zcombobox();
		@example <caption> Groups added using JSON </caption>
	<select id='laps'></select>
	$('#laps').zcombobox({
		dataSource:[{
			label: 'Lenovo',
			id:'lenovolaps',
			options:[{
				label:'Lenovo IdeaPad',
				value:'Lenovo IdeaPad'
			},{
				label:'Lenovo UltraBook',
				value:'Lenovo UltraBook'
			}]
		},{
			label:'HP',
			id:'hplaps',
			options:[{
				label:'HP Pavilion Notebook',
				value:'HP Pavilion Notebook'
			},{
				label:'HP ENVY Notebook',
				value:'HP ENVY Notebook'
			}]
		}]
	});
		*/
		/** @member {String} [combo-box.group-properties.label = null]
			Specifies the group label.
		*/
		/** @member {String} [combo-box.group-properties.id = undefined]
			Unique id of the group.
		*/
		/** @member {Boolean} [combo-box.group-properties.disabled = false]
			If set to true, the group is disabled.
		*/
		/** @member {Boolean} [combo-box.group-properties.hidden = false]
			If set to true, the group is hidden.
		*/
		/** @member {Object} combo-box.group-properties.options 
			Array of items that are to be added to this group.This property is used only when the group is added using JSON.
		*/
	//Updation option
			/** This method is used to update dropdown list items.			
				Properties that can be updated for an item are value, readonly, icon, textAsIcon, label, innerHTML, description, hidden, disabled, customData.
				Properties that can be updated for an item group are hidden, disabled, label.
				@function combo-box.setOptionAttributes
				@param {Object|String} values Specifies the value/#option-id/#optgroup-id of item/group whose properties are to be changed. Inorder to update more than one item, the values are given in array.
				@param property {Object|String} Property to be updated. If more than one property is changed, this param is given as object with <option, value> pair and the third param is omitted.
				@param value {Boolean|String|Object} New property value.
				@example <caption>To update single item </caption>
	$(selector).zcombobox('setOptionAttributes', 'IN','disabled', true)
				@example <caption>To update multiple items </caption>
	$(selector).zcombobox('setOptionAttributes', ['IN','AF'],'disabled', true)
				@example <caption>To update multiple properties for a single item </caption>
	$(selector).zcombobox('setOptionAttributes', IN ,{
								disabled:true,
								label: 'India',
								icon:'flag flag-in'
							});
			*/
	//Update components
	/** Method to set the combobox property.
		@function combo-box.setAttribute
		@param {String} property Name of the property that is to be set.
		@param {String} value New value for the property.
		@example <caption> Example: </caption>
	$(selector).zcombobox('setAttribute','contentType','icon-text');
	*/	
	/** This method sets one or more combobox property.
		@function combo-box.setAttributes
		@param {Object} properties Object contains property-value pairs to be set.
		@example <caption> Example: </caption>
	$(selector).zcombobox('setAttributes',{
						'placeholder':'Select Country',
						'disabled':true 
					});
	*/
	/** This method returns the value of the given property.
		@function combo-box.getAttribute
		@param {String} property Name of the property whose value is to be returned.
		@returns {Object|String} Value of the given property.
		@example <caption> Example: </caption>
	$(selector).zcombobox('getAttribute','disabled')
	*/
	/** This method returns current combobox properties.
		@function combo-box.getAttributes
		@returns {Object} Combobox properties.
		@example <caption> Example: </caption>
	$(selector).zcombobox('getAttributes');
	*/
/* $Id$ */
(function(ZC){
	/** @components tooltip
		Tooltip is a short description that appears while hovering over toolbar icons, truncated text and other elements.This replaces the native tooltip with rich configurations.Tooltip is shown only for elements with title attribute. It is not shown for disabled form elements(button, input, textarea, select, fieldset, keygen, output).
	*/
    var $ = ZC.DOMUtil;
	var tooltipPrototype = {
		options:{
			/** @member {String} tooltip.title
				Content to be displayed as tooltip. This is given in title attribute.
				@example <caption> Example: </caption>
	<button title='Align Left'><i class='icon alignleft'></i></button>
	$(selector).ztooltip()
			*/
			title: undefined,
			/** @member {Boolean} [tooltip.isMessageHTMLEncoded = false] 
				Specifies whether the tooltip content is encoded. By default, the tooltip content is encoded and displayed as tooltip. When applied in element level, it is prefixed with data-title-*.		
			*/
			isMessageHTMLEncoded: false,
			/** @member {Object|String} [tooltip.selector = undefined]
				Value can be dom-selector|dom-element. Tooltip is shown only for the matched elements. This is not an element level configuration.
				If not mentioned, tooltip is shown to all the elements with title attribute.
				@example <caption>  Example: </caption>
	$(selector).ztooltip({
		selector:':not(.is-disabled)'
	});
			*/
			selector: undefined,
			/** @member {Boolean} [tooltip.displayType = 'box'] Specifies whether the callout is attached to the tooltip. When set to 'box', tooltip will not have a callout. When position = 'at-cursor'|'follow-cursor', displayType is always set to 'box'.
			@allowedValues {('callout'|'box')}
			@example <caption> Property when applied in element level </caption>
	<button title='Align Left' data-title-display-type='callout'><i class='icon alignleft'></i></button>
	$(selector).ztooltip();	
			*/ 
			displayType: "box", //No I18N
			/** @member {String} [tooltip.position = 'at-cursor'] 
	Position at which tooltip is shown.When applied in element level, it is prefixed with data-title-*.	
	'top' - Tooltip is shown to the top of the target.
	'bottom' - Tooltip is shown to the bottom of the target.
	'right' -Tooltip is shown to the right of the target.
	'left' - Tooltip is shown to the left of the target.
	'top-left' - Tooltip is shown to the top of the target, with its left aligned to the left of the target.
	'top-right' - Tooltip is shown to the top of the target, with its right aligned to the right of the target.
	'bottom-left' - Tooltip is shown to the bottom of the target, with its left aligned to the left of the target.
	'bottom-right' - Tooltip is shown to the bottom of the target, with its right aligned to the right of the target.
	'at-cursor' - Tooltip appears at the bottom right position of the cursor.
	'follow-cursor' - Tooltip appears at the bottom right position of the cursor and moves along the cursor. 
	'auto' - Tooltip is shown to the bottom of the target, if space not available, it is shown in the direction with required space.
	 @allowedValues {('auto'|'top'|'bottom'|'right'|'left'|'top-left'|'top-right'|'bottom-left'|'bottom-right'|'at-cursor'|'follow-cursor')}
	 		@example <caption> Property when applied in element level </caption>
	<button title='Align Left' data-title-position='right'><i class='icon alignleft'></i></button>
	$(selector).ztooltip();	
			*/	
			position:'at-cursor',	//No I18N
			/** @member {Object} 
			        tooltip.margin - If the value is specified the tooltip will be relocated from the calculated position based on the given value and the direction of the top. This is not an element level configuration.
	@example <caption> Example: </caption>  
	$(selector).ztooltip({
		margin: {top: 2, left: 10}
	}) */	
			/** @member {Number}
					[tooltip.margin.top=0] - If the tooltip is displayed above the target, its top position is reduced by this value and vice-versa. 
					*/			
			/** @member {Number}
					[tooltip.margin.left=0] - If the tooltip is displayed left to the target, its left position is reduced by this value and vice-versa. 	*/
			margin: {left: 0, top: 0},
			/** @member {Number} [tooltip.showDelay = 500]
				Time interval between mousepointer entering the target and the display of the tooltip.When applied in element level, it is prefixed with data-title-*.	
	        */
	        showDelay: 500,
			/** @member {Number} [tooltip.hideDelay = 300] 
				Time, the tooltip takes to hide when the cursor leaves the target. If set to 0, the tooltip is hidden only for maxDisplayTime.When applied in element level, it is prefixed with data-title-*.	
			*/
			hideDelay: 300,
			/** @member {Number} [tooltip.maxDisplayTime = 5000]
				 Specifies the maximum duration, the tooltip can be shown on the page. Assigning it to 0 disables this feature.When applied in element level, it is prefixed with data-title-*.	
			*/
			maxDisplayTime: 5000,
	        /** @member {Number} [tooltip.reshowDelay = 100]
	    		Specifies the duration before which tooltip is shown when the cursor moves from one target to another within the duration mentioned in hideDelay.By default it is calculated as showDelay/5. If showDelay = 500, reshowDelay = 500/5 = 100ms.When applied in element level, it is prefixed with data-title-*.	
			*/
	        reshowDelay: undefined,
	        /** @member {Number} [tooltip.showShortcutKey = true]
	        	Specifies whether shortcut key registered for the target has to be shown.
			*/
	        showShortcutKey: true,
	        /** @member {Object} tooltip.animation Sets animation for showing and hiding the tooltip. If false, disables this feature.
	        */
	        /** @member {Object} tooltip.animation.show Animation to show the tooltip.
	        */
	        /** @member {String} [tooltip.animation.show.name = 'fadeIn']  Animation effect for showing the tooltip.
	        	@example <caption> To use 'slideDown' effect for tooltip show </caption>
	$(selector).ztooltip({
		animation: {
			show: {
				name: 'slideDown'
			}
		}
	}); */
	        /** @member {String} tooltip.animation.show.className  Animation class for showing the tooltip.
	        	@example <caption> To use animation class for tooltip show</caption>
	$(selector).ztooltip({
		animation:{
			show:{
				className:"animated flipInY"
			}	        	
		}
	});
	        	
	        */
	        /** @member {Number} [tooltip.animation.show.duration=200]  Duration for showing the tooltip. This is not used incase className is used.
	        	@example <caption> SlideDown effect with duration </caption>
	$(selector).ztooltip({
		animation: {
			show: {
				name:'slideDown',
				duration: 300
			}
		}
	});
	        */
	        /** @member {Object} tooltip.animation.hide Animation to hide the tooltip.
	        */
	        /** @member {String} [tooltip.animation.hide.name = 'fadeOut']  Animation effect for hiding the tooltip.
	        	@example <caption> To use 'slideUp' effect for tooltip hide </caption>
	$(selector).ztooltip({
		animation: {
			hide: {
				name: 'slideUp'
			}
		}
	}); 
	*/
	        /** @member {String} tooltip.animation.hide.className  Animation class for hiding the tooltip.
	        	@example <caption> To use animation class for tooltip hide</caption>
	$(selector).ztooltip({
		animation:{
			hide:{
				className:"animated flipDownY"
			}	        	
		}
	});
	        	
	        */
	        /** @member {Number} [tooltip.animation.hide.duration=200]  Duration for hiding the tooltip.
	        	@example <caption>  Example:  </caption>
	$(selector).ztooltip({
		animation: {
			hide: {
				name: 'slideUp',
				duration: 500
			}
		}
	});
	        */
			animation:{
				show:{
					name:'fadeIn' //No I18N
				},
				hide:{
					name:'fadeOut' //No I18N
				}
			},
			/** @member {String} tooltip.className
				Custom class that can be added to the tooltip to give additional styling to the tooltip.When applied in element level, it is prefixed with data-title-*.	
		 		@example <caption> Property applied in element level </caption>
	<button title='Align Left' data-title-class-name='customClass'><i class='icon alignleft'></i></button>
	$(selector).ztooltip();
			*/
			className: undefined,
			/** @member {String} tooltip.innerHTML
				If HTML content is needed in the custom tooltip, HTML String is given here. Every ##title## in the string is replaced with the title content of the element.When applied in element level, it is prefixed with data-title-*.	
				@example <caption> Tooltip with custom HTML -container level configuration</caption>
	$(selector).ztooltip({
		innerHTML: "<span class='customClass'>##title##</span>"
	});
			*/
			innerHTML:undefined,
			/** @member {String} tooltip.pauseOnHover
				This option is currently deprecated. Use 'pauseOnMouseOver' instead.
				@deprecated
			*/
			/** @member {String} tooltip.pauseOnMouseOver
				If the tooltip should say still on hover, or if it has to be made clickable, set pauseOnMouseOver = true. This works only when the user leaves the target and hovers the tooltip before the hideDelay time and the tooltip will be removed only when the mousepointer leaves the tooltip.When applied in element level, it is prefixed with data-title-*.	
			*/
			pauseOnMouseOver:false
		},
		_EVENTS:{
			/** Triggered after the tooltip is shown.
				@event tooltip.show
				@param {Object} event eventObject*/
			ONSHOW:'show',	//No I18N
			/** Triggered after the tooltip is hidden.
				@event tooltip.hide
				@param {Object} event eventObject*/
			ONHIDE:'hide'	//No I18N
		},	
		_ANIMATION: {
			slide:{
				show:'slideDown',	//No I18N
				hide:'slideUp'	//No I18N
			},
			fade:{
				show:'fadeIn',	//No I18N
				hide:'fadeOut'	//No I18N
			},
			end: 'webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend'	//No I18N
		},
		_create: function(){
	    	this._namespace = '.'+this.componentName;	//No I18N
	    	this._tempNmspace = this._namespace+'-temp'; //No I18N
			this._bindEvents();
	    },
	    /* Binding events */
	    _bindEvents: function(){
	    	var base = this;
	    	if( !ZC.ztooltipBinded ){
		    	ZC.ztooltipBinded = true;
	    		ZC.$document.off('keydown'+this._namespace).on('keydown'+this._namespace, function(event){	//No I18N
					if( ZC.ztooltipLastInstance ){
						ZC.ztooltipLastInstance.hide();
		    		}
		    	}).off('mousedown'+this._namespace).on('mousedown'+this._namespace, function( event ){ //No I18N
		    		if( ZC.ztooltipLastInstance && ZC.ztooltipLastInstance.target && !$(event.target).closest( ZC.ztooltipLastInstance.target ).length && !$(event.target).closest( ZC.ztooltipLastInstance.tooltip ).length ){
			    		ZC.ztooltipLastInstance.hide();
		    		}
		    	});	
		    	window.addEventListener('scroll', function(){ //No I18N
		    		if( ZC.ztooltipLastInstance ){
		    			ZC.ztooltipLastInstance.hide();
		    			ZC.ztooltipLastInstance = undefined;
		    		}
		    	}, true);
	    	}
	    	/* Handler to observe enable and disable of buttons */
	    	var stateChangeHandler = function(event, ui){ 
	    		var element = ui.button,
					title = element.attr("title");	// No I18N
				if(element.is("input")){
					element = element.next("label"); // No I18N
				}
				if(ui.disabled){
					element.removeAttr("title");	// No I18N
					element.data("ztooltip", title);	// No I18N
				}else{
					element.attr("title",element.data("ztooltip"));	// No I18N
				}
	    	}
	    	this.element.on('mouseenter'+this._namespace+' mousemove'+this._namespace, function(event){ //No I18N
	    		base._targetEnter.call(base, event);
	    	}).on("zbuttonstatechange", stateChangeHandler).on("zmenubuttonstatechange", stateChangeHandler).on("zsplitbuttonstatechange", stateChangeHandler).on("zuploadbuttonstatechange", stateChangeHandler); // No I18N
	    },	
	    /* Displaying the tooltip when the element is hovered */
	    _targetEnter: function( event , customTarget, opt ){
	    	/* Used to track current mouse pointer position ---> used for tracking tooltip */
	    	var target, custom = false;
	    	var disabledSelector = 'button[disabled], input[disabled], textarea[disabled], fieldset[disabled], select[disabled], keygen[disabled] , output[disabled], option[disabled]';
	    	if( this.target && (this.target.is(':hidden') || this.target.is( disabledSelector ) )){
				this.hide();
			}
	    	if( event ){
	    		target = $(event.target).closest('[title]');
	    		if( !event.originalEvent ){
	    			custom = true;
	    		}else{
		    		this.pageX = event.pageX;
		    		this.pageY = event.pageY;
	    		}
	    	}else{
	    		custom = true;
	    		target = customTarget.closest('[title]');
	    	}
	 		//To check whether the target is eligible for tooltip
	 		//Disabled form elements are omitted becos some browsers doesnt trigger events for disabled elements.
			if( !target.length || !target.attr('title') || ( target.is( disabledSelector ) ) ){
				if( target.is( this.target ) && this.tooltip && !this.tooltip.hasClass('hiding') ){
					this.hide();
				}
				return;
			}
			this.custom = custom;
			var showTip = true;
			if( ( this.options.selector && !target.is( $(this.options.selector, this.element) ) ) ){
				showTip = false;
			}
			//clear previous title.
			this.target && this._clearTitle( true, this.target );
			var titleOptions, base = this;
			this.target = target;
			titleOptions = this._getOptions( this.target );
			//Saving the options of the target so retrival will be easy
			this.targetOpt = $.extend(true, {}, this.options, typeof titleOptions === 'object' && titleOptions, opt);	//No I18N
			//@decprecated
			this.targetOpt.pauseOnMouseOver = this.targetOpt.pauseOnMouseOver || this.targetOpt.pauseOnHover;
	    	!custom && this._clearTitle();
	    	this.target.off( this._tempNmspace )
					   .on( 'mouseleave'+ this._tempNmspace, function(){
					   		base._targetLeave.call(base);
					   });
			if( showTip ){
		    	this._clearHideTimer();
				this._show( event );
			}
	    },	
	    /* To hide the tooltip when the target is left*/
	    _targetLeave: function( immediate ){
	    	this._clearTitle( true );
			this._clearShowTimer();
			if( this.tooltip ){
				this._completeAnimation( this.tooltip );
				this._removeTip( immediate );	
			}
		},
		/* To show the tooltip */
	    _show: function(eventObj){
			/*calculating delay*/
			var targetOpt = this.targetOpt;
			if( targetOpt.showDelay && targetOpt.reshowDelay === undefined){
				this.targetOpt.reshowDelay = targetOpt.showDelay / 5 ;
			}
			//Binding the elements
			var base = this;
			this.target.on('mousedown'+ this._tempNmspace, function(){	
				base._clearShowTimer();
		   		base._removeTip( true ,base.target);
		   	});
	    	this._showTooltip(eventObj);	
	    },
	    _completeAnimation: function( element ){
			element && ( element.finish ? element.finish() : element.stop( true, true ) );
	    	return element;
		},
	    /* Function to stop the show timer */
	    _clearShowTimer: function(){
	    	this._completeAnimation( this.tooltip );
			clearTimeout(this.showTimer)
	    },
	    /* Function to stop the hide timer */
	    _clearHideTimer: function(){
	    	this._completeAnimation( this.tooltip );
	    	clearTimeout(this.hideTimer);
	    },
	    _setTimer: function(target, opt){
	    	if(opt.maxDisplayTime){
	    		var base = this;
				this._clearHideTimer();
				this.hideTimer = setTimeout(function(){
					base._removeTip( true );
				}, opt.maxDisplayTime);
			}	
	    },
		/* Function to show the tooltip*/
	    _showTooltip: function(eventObj){
	    	var base = this,
				opt = this.targetOpt;
			var showFunc = function() {	
				/****CLEAR TITLE FOR LAST TOOLTIP*/
				/*If the current target is not equal to last target, the tite attribute of last target is reset*/
				var animation = opt.animation;
				if( base._lastTip ){
					if( base.tooltip && base.tooltip.is(':visible') ){
						// animation = false;     discuss - whether animation is necessary when the previous tooltip is not yet destroyed.( Becos of quick movement )
						if( base.target.is(base._lastTip) ){
							opt.reshowDelay = undefined;
							animation = false;
						}else{
						    base._trigger( "ONHIDE" , undefined, base.target); //No I18N
						}
					}else{
					    base._trigger( "ONHIDE", undefined, base.target ); //No I18N
					}
				}
				/*When the tooltip is about to be displayed, current target is set as last target*/
				base._lastTip = base.target;
		    	// base._clearHideTimer();
		    	/* validating the position*/
				if( !/^(auto|top|bottom|right|left|top-left|top-right|bottom-left|bottom-right|at-cursor|follow-cursor)$/.test(opt.position)){
					opt.position = $.ztooltip.prototype.options.position;
				}
		    	/*if( opt.pointer === undefined ){
		    		if( !/^(at-cursor|follow-cursor)$/.test(opt.position) || opt.isTargetCustom ){
		    			opt.pointer = true;
		    		}
		    	}*/
		    	/*set displayType = 'box' for at-cursor/follow-cursor*/
		    	if( /^(at-cursor|follow-cursor)$/.test(opt.position) ){
		    		if( !base.custom ){
			    		opt.displayType = 'box'; //No I18N
		    		}else{
		    			opt.position = 'top'; //No I18N
		    		}
		    	}
		        base._createTooltip();
				if(base.tooltip && base.tooltip.length &&(base.target.is(':visible') || base.target.is('option'))){//Chrome & IE says option element is always hidden. So we have to skip option element.//No I18N 
					base._includeContent(opt);
					base._setTimer(base.target, opt);
					if(  opt.position === 'follow-cursor' ){	//No I18N
						var mousemoveDelegate = function(){
							base._positionElement.apply(base, arguments);
							/*When the tooltip is on move, its hidetimer is reset*/
							base._setTimer(base.target, opt);
						}
						base.target.off('mousemove'+base._tempNmspace)	//No I18N
								   .on('mousemove'+base._tempNmspace, mousemoveDelegate); //No I18N
					}
					base._positionElement(eventObj);
					ZC.ztooltipLastInstance = base;
				}
				base._completeAnimation( base.tooltip.hide() );
				base._animate( base.tooltip, animation, 'show','show', function(){	//No I18N
					base._trigger( "ONSHOW", eventObj); //No I18N
				});
			}
			if( this.custom || opt.reshowDelay || opt.showDelay !== undefined ){
				// this.tooltip && this.tooltip.hide(); //removed as it prevents reshowDelay behavior
				this._clearShowTimer();
				this.showTimer = setTimeout(function(){
					showFunc();
				}, ( !this.custom && this.tooltip ) ? opt.reshowDelay : opt.showDelay);
			}else{
				showFunc();
			}
	    },
	    _animate: function( element, animation ,action , alterFn, callback ){
	    	if( animation && typeof animation === 'object'){ //No I18N
	    		var effect, duration, className;
				if( typeof animation === 'string' ){ //No I18N
					effect = this._ANIMATION[ animation ][action];
				}else if( animation.name ){
					if( this._ANIMATION[ animation.name ] ){
						effect = this._ANIMATION[ animation.name ][action];
					}
				}else if( animation[action] ){
					if( typeof animation[action] === 'string'){ //No I18N
						effect = animation[action];
					}else if( animation[action].name ){
						effect = animation[action].name;
					}else if( animation[action].className ){
						className = animation[action].className;
					}
				}
				duration = animation.duration? animation.duration :(animation[action] && animation[action].duration? animation[action].duration : undefined);		
				if(typeof effect === 'object'){	//No I18N
					element.animate( effect, duration ,callback );
				}else if(/^(fadeIn|fadeOut|slideDown|slideUp)$/.test(effect) ){
					element[effect](duration, callback);
				}else if(typeof className === 'string'){ //No I18N
					if(alterFn === 'show'){	//No I18N
						element.show();
					}
				    element.removeClass( element.data('css-effect') ).addClass( className ).data('css-effect', className)	//No I18N
							.off( this._ANIMATION.end )
							.on( this._ANIMATION.end, function(){
									if(alterFn === 'hide'){	//No I18N
										element.hide();
									}
									element.removeClass( className ).removeData('css-effect');	//No I18N
									callback();
							});	
				}else{
					element[ alterFn ]() && callback();	
				}
			}else{
				element[ alterFn ]() && callback();	
			}
	    },
	    _includeContent: function(opt){
	    	var o = this.targetOpt;
			if(typeof o === 'object' && this.tooltipContent && this.tooltipContent.length){	//No I18N
				var content = o.title = o.title || o.content || ''; //No I18N
				if( /*/^(\.|\#)/.test(content) || */content.nodeType || content.jquery ){
					content = $(content).clone().show()[0].outerHTML;
					o.isMessageHTMLEncoded = true;	
				}else if( typeof o.innerHTML === 'string' ){ //No I18N
					content = this._populateTemplate( o.innerHTML, o );
					o.isMessageHTMLEncoded = true;	
				}
				if( o.showShortcutKey && window.ZShortCut ){
					var target = this.target;
					target = target.is('label.zbutton')? target.prev('input') : target;	//No I18N
				    var data = ZC.getOptions(target);
				    var key = ZShortCut.getInfo(data.actionName, target , false);
				    if( key ){
				    	if( o.shortcutKeyCustomHTML ){
				    		key = o.shortcutKeyCustomHTML.replace(/(#key#)/gi, key );
				    	}else{
				    		key = " ("+key+")"; //No I18N
				    	}
				    	content = content + key;
				    }
				}
				this.setContent(o, content );
			}
	    },
	    setContent: function(options, content ){
			if( options.isMessageHTMLEncoded ){
				this.tooltipContent.html(content);
			}else{
				this.tooltipContent.html($('<div>').addClass('ztooltip__text').text(content)); //No I18N
			}
		},
	    _populateTemplate: function( template, data ){
			if( typeof template === 'string' ){ //No I18N
				var props = template.match(/#?#[^# ]+#?#/g);
				if( props ){
					props.forEach( function( prop ){
						template = template.replace( new RegExp( prop, 'g' ) ,function( match ){ //No I18N
							return data[match.replace(/#/g,'')]; //No I18N
						});
					});
				}
				return template;
			}
		},
	    //This function is to position the tooltip and the respective arrow.
		_positionElement: function(eventObj){
			if(this.tooltip){	
				var positionObject = this._getPosition(eventObj);
  				var direction = positionObject.direction.split('-')[0]; //No I18N
				var elementPosition = positionObject.elementPosition;
				this.tooltip.removeClass( 'class' ).addClass('ztooltip ztooltip--'+ direction); //No I18N
	  			if( typeof this.targetOpt.margin === 'object' ){ //No I18N
	  				var margin = {};
	  				margin.top = parseInt( this.targetOpt.margin.top );
	  				margin.left = parseInt( this.targetOpt.margin.left );
	  				if( margin.top === 'NaN' ){ //No I18N
	  					margin.top = 0;
	  				}
	  				if( margin.left === 'NaN' ){ //No I18N
	  					margin.left = 0;
	  				}
	  				if( direction === 'top' ){ //No I18N
						elementPosition.top -= margin.top;	
	  				}else if( direction === 'bottom' ){ //No I18N
						elementPosition.top += margin.top;	
	  				}else if( direction === 'right' ){ //No I18N
						elementPosition.left += margin.left;
	  				}else if( direction === 'left' ){ //No I18N
						elementPosition.left -= margin.left;
	  				}
					// elementPosition.left += parseInt(this.targetOpt.margin.left);
					// elementPosition.top += parseInt(this.targetOpt.margin.top);	
					if(/^(at-cursor|follow-cursor)$/.test( this.targetOpt.position )){
						var maxCorrection = positionObject.availableHeight - this.tooltip.outerHeight();
						elementPosition.left += parseInt(this.targetOpt.margin.left);
						//@constant 18
						elementPosition.top += (parseInt(this.targetOpt.margin.top) + Math.min( 18, maxCorrection)/*There must be some distance between cursor and the tooltip by default*/ );
					}
	  			}
				this.tooltip.css(elementPosition);
				if(this.tooltipCallout){
					this.tooltipCallout.css(positionObject.arrowPosition);
				}	
			}
		},
		//This function is to get the position for the tooltip
		//EventObject can be passed, only when the position is based on the mouse pointer position
		_getPosition: function(eventObj){
			var opt = this.targetOpt;
			var eventTarget, position = this.targetOpt.position, positionAlterable =false;
			var arrowCallback = function(element, direction){
				$(element).removeAttr('class').addClass('ztooltip__pointer h-' + direction); //No I18N
			}
			if(ZPosition && this.tooltip){
				this.tooltip.css({
					left:'auto', //No I18N
					right:'auto'//No I18N
				});
				/* Get the current position of mouse pointer */
				if( /^(at-cursor|follow-cursor)$/.test( opt.position ) || opt.offset ){
					if( opt.offset ){
						this.pageX = parseInt( opt.offset.left );
						this.pageY = parseInt( opt.offset.top );
					}else{
						position = 'bottom-left';	//No I18N
					}
					eventTarget = eventObj;
					eventTarget.pageX = this.pageX;
					eventTarget.pageY = this.pageY;
					positionAlterable = 'flipallfit'; //No I18N
				}else{
					if( opt.position === 'auto' ){ //No I18N
						position = 'bottom'; //No I18N
						positionAlterable = 'flipallfit'; //No I18N
					}
				}
				var tipPosition = ZPosition.get((eventTarget || this.target), this.tooltip,
										{
											direction : position,
											eventObj : eventObj,
											bufferSpace : this.tooltipCallout? 10:0,
											positionAlterable :positionAlterable,
											arrowCallback: arrowCallback,
											arrow: opt.displayType === 'callout'? { //No I18N
														element : this.tooltipCallout,
														direction:true,
														minSpace:8
													} : undefined
										});
			}
			if( !tipPosition ){
				tipPosition = {
					elementPosition: {left:0, top:0},
					arrowPosition: {left:0, top:0}
				}
			}
			return tipPosition;
		},
		/* Function to remove the tip*/
	    _removeTip: function(immediate, target, opt){
	    	opt = opt || this.targetOpt;
	    	if(this.tooltip){
	    		target = target || this.target;
				var base = this;
				var remove = function(){
					base._animate( base._completeAnimation( base.tooltip ), opt.animation, 'hide','hide', function(){	//No I18N
						if( base.tooltip ){
							base.tooltip.remove();
						    base.tooltip = base.tooltipCallout = base.tooltipTitle = base.tooltipContent = null;
						    if( ZC.ztooltipLastInstance && ZC.ztooltipLastInstance.element.is( base.element ) ){
						    	ZC.ztooltipLastInstance = undefined;
						    }
						    base._trigger( "ONHIDE" ); //No I18N
						}	
					});	
				}
				var removeFunc = function(){
					if(base.tooltip){
						base.tooltip.off('mouseleave'); //No I18N
						/* To empty lasttip if the current target is different from lasttip*/
						if(!base.target.is(base._lastTip)){
							base._lastTip = undefined;
						}else if( !immediate && base.targetOpt.pauseOnMouseOver && base.tooltip.hasClass('on-hover') ){  //No I18N
							base.tooltip.on('mouseleave', remove); //No I18N
							return;
						}
						remove();
					}
				}
				/*This has been incuded becos, for all tip targets, its title is reset on targetleave but incase of tip shown using show method, there wont be any targetleave method to clear its title. 
				  This cannot be done for all case becos removeing tip on target click, its title should not be cleared.
				*/
				this.isTargetCustom && this._clearTitle( true, this.target );
				if(immediate || !this.targetOpt.hideDelay){
					removeFunc();
				}else{
					this._clearHideTimer();
					this.hideTimer = setTimeout(function(){
						removeFunc();
					}, opt.hideDelay);
				}	
	    	}
		},

		//To get data by parsing JSON of title attribute
		_getOptions: function(target){
			var titledata = target.attr('title');	//No I18N
			var tooltipdata = {};
			$.each(target[0].attributes,function( ){
				var name = this.nodeName;
				var value = this.value;
				if( name.match(/^data-title-/) ){
					name = name.replace(/^data-title-/,''); 
					name = name.replace(/-([a-z])/g, function(g){
								return g[1].toUpperCase(); // replacing the hyphenated names to camelCase
							});
					name = name.replace(/(html)/gi, "HTML"); //No I18N
					tooltipdata[name] = (value === "true" || value === "false") ? (value === "true") : ((value.indexOf("{") !== -1) ? ZC._getObject(value) : value); // No I18N
				}
			});
			var titleObject;
			if( !isNaN( titledata ) ){
				titleObject = {"content":  titledata };	//No I18N
			}else{
				try{
					titleObject = JSON.parse( titledata );
				}catch(e){
					if( titledata ){
						titleObject = {"content":  titledata };	//No I18N
					}
				}	
			}
			tooltipdata.targetElement = target;
			$.extend(titleObject, tooltipdata);
			return titleObject;
		},
	    //Creating the tooltip
	    _createTooltip: function(){
				if(!this.tooltip){
					this.tooltip = $("<div>").appendTo('body')
											 .hover(function(){
											 	$(this).addClass('on-hover');
											 }, function(){
											 	$(this).removeClass('on-hover');
											 });
					this.tooltipContent = $("<div>").addClass("ztooltip__content").appendTo(this.tooltip);	//No I18N
				}
				this.tooltip.css('pointer-events', this.targetOpt.pauseOnMouseOver? 'auto':'none'); //No I18N
				this.tooltip.attr('class', 'ztooltip '+ (this.targetOpt.className ||''));	//No I18N
				/*	if(this.targetOpt.title){
					if( !this.tooltipTitle ){
						this.tooltipTitle = $("<div>").addClass("ztooltip__title").append($("<span>").addClass("ztooltip__text")).insertBefore( this.tooltipContent ); //No I18N
					}
					this.tooltipTitle.find('.ztooltip__text').html( this.options.isMessageHTMLEncoded ? this.targetOpt.title : ZC.encodeHTML( this.targetOpt.title )); //No I18N
				}else if( this.tooltipTitle ){
					this.tooltipTitle.remove();
					this.tooltipTitle = undefined;
				}*/
				if( this.targetOpt.displayType === 'callout' ){ //No I18N
					if(!this.tooltipCallout){
						this.tooltipCallout = $("<div>").addClass('ztooltip__pointer').appendTo(this.tooltip);	//No I18N
					}
				}else if( this.tooltipCallout ){
					this.tooltipCallout.remove();
					this.tooltipCallout = undefined;	
				}
				return this.tooltip;
	    },
	    //Clearing the title of the target
	    _clearTitle: function(reset, target){
			target = target || this.target;
			if( target && target.length ){
		    	if( reset ){
		    		target.off( this._tempNmspace );
		    		var parents = $( target.data('zparents') );	//No I18N
	    			parents.add( target ).each( function( i, tar ){
	    				tar = $(tar);
	    				var titleData = tar.attr('title') !== undefined ? tar.attr('title') : tar.data('ztitle');	//No I18N
	    				if( titleData !== undefined ){
	    					tar.attr('title', titleData).removeData('ztitle');	//No I18N
		    				tar.removeAttr('aria-label');	//No I18N
	    				}
	    			});
		    		target.removeData('zparents');	//No I18N
		    	}else if( !reset ){
		    		/*Fix to remove title attributes of parent elements having title attribute*/
		    		var parents = target.parents('[title]'); //No I18N
	    			parents.add( target ).each(function( i, tar ){
		    			tar = $(tar);
			    		tar.data('ztitle', tar.attr('title')).removeAttr('title');	//No I18N
		    		});
		    		target.data('zparents', parents); //No I18N	
		    	}
			}
	    },
		/* Function to know the visiblity of the element*/
		_isVisible: function(element){
			if(element && element.length){
				return element.is(':visible') ? true:false;	//No I18N
			}
		},
		_trigger: function( customEvent , origEvent, target){
			var data = {
				target : target || this.target,
				tooltip : customEvent === "ONSHOW"? this.tooltip : undefined  //No I18N
			}
	       return this._dispatchEvent( customEvent, origEvent, data );
		},
		/** Shows the Tooltip for the given target.
			@function tooltip.show
			@param {Object} target Element for which tooltip has to be shown. This 
			element should have title attribute.
			@returns {Object} Returns the element.
		*/
		show: function(target, opt){
			/*To clear previous target*/
			target = $(target);
			if( target.length && target[0].nodeType && target.is(':visible') ){
				this._clearTitle( true , target);
				opt = typeof opt === 'object'? opt : {}; //No I18N
				opt.showDelay = 0;
				this._targetEnter( undefined, target, opt );
			}
			return target;
		},
		/** Hides the current tooltip.
			@function tooltip.hide
			@param {Boolean} immediate Whether tooltip has to be hidden immediately without animation.
		*/
		hide: function( immediate ){
			this._targetLeave( immediate );
		},
		/** Destroys the Tooltip instance.
			@function tooltip.destroy
		*/
		/* Function to destroy the tooltip */
		_destroy: function(){
			if( ZC.ztooltipLastInstance && ZC.ztooltipLastInstance.element.is( this.element ) ){
				ZC.ztooltipLastInstance.target && ZC.ztooltipLastInstance.target.off( this._tempNmspace );
				ZC.ztooltipLastInstance = undefined;
			}
			this.element.off( this._namespace );
			this.hide( true );
			$(this.tooltip).remove();
		},
		option: function(optionName, value){
			var base =  this;
			if(typeof optionName === 'object' && !$.isEmptyObject(optionName)){	// No I18N
				$.each(optionName, function(o,v){
					base.option(o,v);
				});
			}else if(optionName !== undefined && value === undefined){
				return this.options[ value ];
			}else if( optionName !== undefined && value !== undefined ){
				if( typeof value === 'object' ){ //No I18N
					$.extend( this.options[optionName], value );
				}else{
					this.options[optionName] = value;
				}
			}else{
				return Object.create( this.options );
			}
		}	
	}
   ZC.registerComponent("ztooltip", tooltipPrototype); //No I18N
})(ZComponents);
/** Method to set property for tooltip instance property.
	@function tooltip.setAttribute
	@param {String} property Property name to set.
	@param {String} value New value for the property.
	@example <caption> Example: </caption>
$(selector).ztooltip('setAttribute','position','top');
*/	
/** This method sets one or more tooltip instance property.
	@function tooltip.setAttributes
	@param {Object} properties Object contains property-value pairs to be set.
	@example <caption> Example: </caption>
$(selector).ztooltip('setAttributes',{
	position:'top',
	showDelay:100,
	reshowDelay:30
});
*/
/** This method returns the value of the given property.
	@function tooltip.getAttribute
	@param {String} property Name of the property whose value is to be returned.
	@returns {Object|String} Value of the given property.
	@example <caption> Example: </caption>
$(selector).ztooltip('getAttribute','position')
*/
/** This method returns current tooltip instance properties.
	@function tooltip.getAttributes
	@returns {Object} Tooltip instance properties.
	@example <caption> Example: </caption>
$(selector).ztooltip('getAttributes');
*/
/* $Id$ */
(function(ZC){
	/** @components suggest-field -  Assists the user to quickly find the required value from the pre-populated suggestions. 
	    @eventPrefix zsuggestfield 
	*/
	var $ = ZC.DOMUtil,
		suggestfieldPrototype = {
		options: {
			/** @member {(array|object)} 
                 [suggest-field.dataSource] - Specifies the input for the suggestions that needs to be loaded.

                JSONArray - If JSON array is passed as input, it is mandatory to provide data mapping (or)

                 Array - String / Number array can also be passed as input. 	 (or)

                 object - {URL: &lt;string|function&gt;, queryParam: &lt;object|function&gt;, cacheResults: (true|false)} 
	             @example <caption> Example: </caption>  
   dataSource: {
         "URL": "https://api.github.com/search/repositories",
         "type": "GET",
         queryParam: function(params){
                return {
                    "q" : params.query,
                    "page": params.pages  	
                    //If the API has paging for data retrieval, dynamically data is retrieved upon scroll.
                };
          },
          success: function(data, params){
                return {
                    "results": data.items,
                    "more": (params.pages * 30) < data.total_count
                    //Indicates whether data is available for next server request.
                }
            },		    
    } */
			/** @member {string|function}
				[suggest-field.dataSource.URL] - The path from where the suggestion data need to be fetched.*/
			/** @member {object|function}
				[suggest-field.dataSource.queryParam] - The query Parameter that need to be passed to the API to retrieve the associated data. */		
			/** @member {object|function}
				[suggest-field.dataSource.success] - Callback for AJAX request success  . */	
			/** @member {object|function}
				[suggest-field.dataSource.fail] - Callback for AJAX request failure  . */	
			"dataSource": {	// No I18N
				"dataType": "json",	// No I18N
				"type": "GET" // No I18N
			},	
			"searchIcon": false,	// No I18N
			/** @member {object} 
			    [suggest-field.dataMapping] - The mapping for the JSON Data passed should be given here. This field is mandatory for both JSON and API input. The name of the field in the input data should be specified for each of the fields provided below.
		        @example <caption> Example: </caption>  

Data mapping for the sample JSON below:

    object = {
        "id": 3167891,
        "name": {
            "first_name": "Edward",
            "last_name": "Cullen"
        },
        "email": "edward@gmai.com",
        "avatarURL": "http://www.github.com?page=1&q=edward",
        "department": "Training",
        "gender": "Male",
        "country": "India"
    }

    dataMapping: {
        value: "id",
        text: "name.first_name", 
        // When the value should be retrieved from a nested object. The value should be seperated by "." operator
       	informativeText: "email",  (optional)
        image: "avatarURL",   (optional)
        groupLabel: "department" (optional)
    }			
			*/
			/** @member {string}
				[suggest-field.dataMapping.value] - Any unique data field in the JSON object can be given here. */
			/** @member {string}
				[suggest-field.dataMapping.text] - The name(Primary Text) that need to be displayed in each search list item. */
			/** @member {string}
				[suggest-field.dataMapping.informativeText] - The description / any addition information you need to provide for each search list item. */
			/** @member {string}
				[suggest-field.dataMapping.image] - If the user wants to display Image/icon in the suggestions, the JSON key which has image URL / icon should be assigned to the key "image"  */
			/** @member {string}
				[suggest-field.dataMapping.groupLabel] - The field in the JSON that need to be displayed as a group header. Set this value only when you want group header in the suggestions list. */
			"dataMapping": undefined,	// No I18N
			/** @member {boolean}
				[suggest-field.cacheServerResponse = true] - Whether response received from the server/API/JSON file should be cached */
			"cacheServerResponse": true,	// No I18N
			/** @member {boolean}
				[suggest-field.searchKeywordSubsetsInCache = false] - Denotes whether result subset can be retrieved from the previous cached results. 	*/
			"searchKeywordSubsetsInCache": false,	// No I18N
			/** @member {boolean}
				[suggest-field.isResponseFiltered] - The value should be set to true if the user does not want search operation to be handled by the component. If the search result is retrieved from the API, then this value can be set to true. */
			"isResponseFiltered": false,	// No I18N
			/** @member {boolean}
				[suggest-field.isResponsePaginated] - This can be used to load content when paging is available in API. 
		        @example <caption> Example: </caption>  
    $("#input1").zsuggestfield({
        dataMapping: {
            "value": "id",
            "text": "full_name",
            "informativeText": "git_url",
            "image": "owner.avatar_url"
        },
        dataSource: {
            "URL": "https://api.github.com/search/repositories",
            "type": "GET",
            "dataType": "json",
            queryParam: function(params){
               return {
                   "q" : params.query,
                   "page": params.pages
                };
            },
            success: function(data, params){
                return {
                    "results": data.items,
                    "more": (params.pages * 30) < data.total_count
                }
            }	    
        },
        isResponseFiltered: true,
        isResponsePaginated: true,
        lazyLoadData: true,
        className: "container"
    })
			*/
			"isResponsePaginated" : false,	// No I18N
			/** @member {(object)} 
			       [suggest-field.search] - Options associated to search can be specified here. It is MANDATORY to provide "by" field in search for search to be triggered in the component.
			       @example <caption> Example: </caption>  
   $("#demoInput").zsuggestfield({
        search: {
            minKeywordLength: 1,
            by: "name",
            criteria: "startsWith"
        }
   })
			*/
			"search": {			// No I18N
				/** @member {(number)} 
			        	[suggest-field.search.minKeywordLength = 0] - Minimum length of the text that need to be entered to initiate the autocomplete action.*/
				"minKeywordLength": 0,	// No I18N
				"maxKeywordLength": 50,	// No I18N
				/** @member {(string|array)} 
			        	[suggest-field.search.by] - The field name with which search should be carried out. This field can be set if the source data is JSON. If you want search action to be processed for multiple fields, the field names can be passed as "," seperated values. The search will be carried out in the order it appears. It is mandatory to provide "by" if the source data is JSON 
						@example <caption> Example: </caption> 
If search operation has to be carried out on multiple fields in the input JSON, then value should be passed as follows: 

	search: {
	   by: "name,email"
	}


If the input JSON structure contains nested JSON object, then the value should be given as follows: 

	search: {
	   by: "name.first_name,product"
	}
			    */
				"by": undefined,	// No I18N
				/** @member  {{string}}
				  	@allowedValues {("startsWith"|"contains")}
			        	[suggest-field.search.criteria = "contains"] - Filtering mode */	
				"criteria": "contains",	// No I18N
				/** @member {number} 
			        	[suggest-field.search.delay = 100] - The delay after which the search need to be carried out. */
				"delay": 100	// No I18N
			},
			/** @member {string} 
			    [suggest-field.autoComplete] - The value gets auto filled on hover of each suggestion. */
			"autoComplete": false,	// No I18N
			/** @member {string} 
			        [suggest-field.placeholder] - Add the text that need to be displayed as placeholder. */
			"placeholder": false,	// No I18N
			/** @member	{(string)} 
				[suggest-field.isImageSourceSprited = false] -  Indicates whether the image that need to be displayed would be image or sprite CSS class  */	
			"isImageSourceSprited": false,	// No I18N
			/** @member {object} 
			    [suggest-field.dropdownList] - The dropdown options are provided here. 
			    	@example <caption> Example: </caption>
    $("#demoInput").zsuggestfield({
        dataMapping: {
           value: "id",
           name: "name, 
   	    informativeText: "email",
           image: "imageURL",
           groupLabel: "department"
        },			
        search: {
           by: "name"
        },
        drodownList: {
            contentType: "image-text",
            width: 300,
            sortResults: false
        },
        lazyLoadData: false
    })
			*/
			"dropdownList": {	// No I18N
				/** @member {(string)}
			        [suggest-field.dropdownList.contentType = "text"] - The type of menu that need to be displayed. If "image-text-desc" menu type is selected then "informativeText" field in "dataMapping" option need to be given.
			        @allowedValues {("text"|"image-text"|"image-text-desc"|"text-desc")} */
				"contentType": "text",	// No I18N	
				/** @member {String} 
			        [suggest-field.dropdownList.progressIndicatorSVGIconId] - The default SVG Icon provided by the component for the loader can be changed. ID of the user's SVG element should be provided. 
					@newlyAdded true
			    */
				"progressIndicatorSVGIconId": undefined, // No I18N	
				/** @member {String} 
			        [suggest-field.dropdownList.progressIndicatorIconClassName] -   Sprite CSS class can also be provided for the loader icon.
			        @newlyAdded true
			    */
				"progressIndicatorIconClassName": undefined, // No I18N	
				/** @member {number} 
			        	[suggest-field.dropdownList.height = 350 ] - Height of the suggestion menu. */
				"height": 350,	// No I18N
				/** @member {number} 
			        	[suggest-field.dropdownList.width] - Width of the suggestion menu. */
				"width": undefined,	// No I18N
				/** @member {boolean} 
			        	[suggest-field.dropdownList.moreResultsAction = false] - Set this to true if you want more action to be included in the suggestions. If maxDisplayResults option is not set by the user, value for maxDisplayResults will be taken as 10 in this case. 
						@example <caption> Example: </caption>
    $("#demoInput").zsuggestfield({
        drodownList: {
            moreResultsAction: function(){
            	alert("More action clicked")
            },
        },
        lazyLoadData: false
    })
			    */
				"moreResultsAction": false,	// No I18N
				/** @member {(id-string-selector | dom-selector)} 
			        	[suggest-field.dropdownList.appendTo] - Enable this if the suggestions menu need to be appended to a DOM element. */
				"appendTo": undefined,	// No I18N
				/** @member {boolean} 
			        	[suggest-field.dropdownList.sortResults = false] - Enable it when you want to sort the results retrieved for the input. Customize the default text by passing a string if it is needed. */
				"sortResults": true,	// No I18N
				/** @member {(string)} 
			        	[suggest-field.dropdownList.position = "bottom-right"] - The position where the menu need to be placed.				
					@allowedValues {("top"|"bottom"|"left"|"right"|"after-cursor")} */
				"position": 'auto',// No I18N
				/** @member {number} 
			        	[suggest-field.dropdownList.maxDisplayResults] - Number of results that need to be displayed for each matched input */
				"maxDisplayResults": undefined,	// No I18N
				"animation": undefined,	// No I18N
				/** @member {boolean}
			        	[suggest-field.dropdownList.focusFirstItem = true] - First menu item will be focused if set. */
				"focusFirstItem": true,	// No I18N
				/** @member {boolean}
			        [suggest-field.dropdownList.highlightKeyword = false] - Highlight the keyword typed in the input field in the menu displayed. */
				"highlightKeyword": false,	// No I18N
				/** @member {boolean}
			        [suggest-field.dropdownList.menuClassName] - Custom CSS for the menu. */
				"menuClassName": undefined,	// No I18N
				/** @member {boolean} 
		        	[suggest-field.dropdownList.showNoResultsMessage = false] - Enable it when you want to display no results message.
                */	
				"showNoResultsMessage": false	// No I18N
			},
			/** @member {boolean}
			    [suggest-field.showDropDownListOnFocus = false] - Shows Menu upon focus of input field. This option can be enabled when the user wants to show suggestions upon focus of the input field when minKeywordLength is 0.  */
			"showDropDownListOnFocus": false,	// No I18N
			 /** @member {boolean} 
			        [suggest-field.lazyLoadData = false] - Indicates whether data need to be loaded as you scroll.  (On demand) */
			"lazyLoadData": true,	// No I18N
			/** @member {boolean} 
			        [suggest-field.className] - The class name that need to be applied for the input field for which you initialize the component. */
			"className": undefined,	// No I18N
			/** @member {boolean} 
			        [suggest-field.disabled = false] - Enable this option to disable the input field. */
			"disabled": false,	// No I18N
			/**	@member {object}
					[suggest-field.messages] - Text messages that need to be displayed can be provided here.*/
			"messages": {	// No I18N
				/** @member {string}
					[suggest-field.messages.noResults] - Message that need to be displayed when there are no results for the input keyword typed.

	                @example <caption> Example: </caption>

To append typed query at a specified position, #query# acts as a placeholder to append query. Eg. "No results for #query# found." 
    $("#demoInput").zsuggestfield({
        drodownList: {
            showNoResultsMessage: true
        },
        messages: {
             noResults: "No results for #query# found."
        }
    })
				*/
				"noResults": "No results found.",	// No I18N
				/** @member {string}
					[suggest-field.messages.moreAction] - More action text. 
					@example <caption> Example: </caption>
    $("#demoInput").zsuggestfield({
        drodownList: {
            showNoResultsMessage: true,
            moreResultsAction: function(){
                alert("More Action Clicked");
            }
        },
        messages: {
            noResults: "No results for #query# found.",
            moreAction: "See more results for #query#"
        }
    })
				*/
				"moreAction": "See more results",	// No I18N
				/** @member {string} 
					[suggest-field.messages.serverError] - Error message that need to be displayed in case of server output failure. */
				"serverError": "Server request failed. Please try again later"	// No I18N
			}
		},
		_EVENTS: {
			/** Triggered when a suggestion is selected.
				@event suggest-field.listitemselect
				@param {object} event eventObject
				@param {object} customObject containing additional information about the selected item.
			*/
			"ITEMSELECT": "listitemselect",	// No I18N
			/** Triggered when a suggestion is focuse.
				@event suggest-field.listitemfocus
				@param {object} event eventObject
				@param {object} customObject containing additional information about the focused item. */
			"ITEMFOCUS": "listitemfocus",	// No I18N
			/** Triggered before the menu is shown.
				@event suggest-field.dropdownlistbeforeopen
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu. */
			"MENUBEFOREOPEN": "dropdownlistbeforeopen",	// No I18N
			/** Triggered after the menu is shown.
				@event suggest-field.dropdownlistopen
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu */
			"MENUAFTEROPEN": "dropdownlistopen",	// No I18N
			/** Triggered before the menu is closed.
				@event suggest-field.dropdownlistbeforeclose
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu. */
			"MENUBEFORECLOSE": "dropdownlistbeforeclose",	// No I18N
			/** Triggered after the menu is closed.
				@event suggest-field.dropdownlistclose
				@param {object} event eventObject
				@param {object} customObject containing additional information about the menu. */
			"MENUAFTERCLOSE": "dropdownlistclose",	// No I18N
			/** Triggered when the value is changed and input field loses focus.
				@event suggest-field.change
				@param {object} event eventObject
				@param {object} customObject containing additional information about the changed values. */
			"CHANGE": "change",	// No I18N
			/** Triggered before the search is performed.
				@event suggest-field.beforesearch
				@param {object} event eventObject
				@param {object} Contains information about the query passed and the associated data.
	            @example <caption> Example </caption>

    $("#input1").on("zsuggestfieldbeforesearch", function(orgEvent, ui){
        console.log(ui.value);
    })
		
Events can be listened through callbacks 

    $("#input1").zsuggestfield({
        dataSource: sourceData, 
        search: {
            by: "name"
        },
        dataMapping: {
            "value": "id",
            "text": "value",
            "informativeText": "email",
            "image": "imageURL"
        },
        beforesearch: function(orgEvent, ui){
            console.log(ui.value);
        }
    })
			*/
			"BEFORESEARCH": "beforesearch",	// No I18N
			/** Triggered after the search is done. This event will be triggered only when the dataSource requires AJAX request.
				@event suggest-field.searchsuccess
				@param {object} event eventObject
				@param {object} Contains information about the query passed and the response received for the query. */
			"SEARCHSUCCESS": "searchsuccess",	// No I18N
			/** Triggered after the search is done. This event will be triggered only when the dataSource requires AJAX request.
				@event suggest-field.searcherror
				@param {object} event eventObject
				@param {object} Contains information about data retrieval failure */
			"SEARCHERROR": "searcherror",	// No I18N
			/** Triggered when the more action is selected.
				@event suggest-field.moreactionclick
				@param {object} event eventObject
				@param {object} customObject containing additional information about the clicked action. */
			"MOREACTIONSELECT": "moreactionclick"	// No I18N
		},
		_CONSTANTS: {
			minKeywordLength: 20,
			maxResults: 10
		},
		_create: function(element, options){
		    this._constructContainer();
			this._initialize(element,options);
			this._bindEvents();
			this._bindMenuEvents();
			this._initSource();
			this._cachedData = {};		
			this.options.isResponsePaginated && (this._params = {pages: undefined});
		},
		_constructContainer: function(){
			this.options.dropdownList.forElement = this.element;
			this._element = this.element;
			this._element.addClass("zsuggestfield-element");  // No I18N
			this._input = this.element;
			this._elementId = (this.element.attr("id") ? this.element.attr("id") : Math.floor(Math.random() * 1000000))+this.componentName;  // No I18N
			this._input.attr({
				"aria-haspopup": "true",	//No I18N
				"aria-expanded": "false",	//No I18N
				"aria-owns": this._elementId+"-menu",	//No I18N
				"role": "search"	//No I18N
			});		
			this.options.className && this._element.addClass(this.options.className);
		},
		_initialize: function(){	
			this._dataMapping = this.options.dataMapping;
			var menu = this.options.dropdownList, isImageSprite = this.options.isImageSourceSprited;
			this.options.placeholder && this.element.attr("placeholder", this.options.placeholder);		// No I18N
			switch(menu.contentType){
				case "image-text-desc": // No I18N
				case "image-text": // No I18N
					this._type = "icon-text"; // No I18N
				break;
				case "text":  // No I18N
				case "text-desc": // No I18N
					this._type = menu.itemType === "checkbox" ? "icon-text" : "text";	// No I18N
				break;
			}
			if(this.options.dataSource instanceof Array && typeof this.options.dataSource[0] === "string"){	// No I18N
				this._type = "text";	// No I18N
			}
			var menuId = (this.element.attr("id") || Math.floor(Math.random() * 1000000))+"-"+this.componentName+"-menu", menuElement = $("#"+menuId);  // No I18N
			if(menuElement.length){
				menuElement.data("zmenu") && menuElement.zmenu("destroy"); // No I18N
				menuElement.remove();
			}
			this._menu = ZC.createMenu(menuId, {}, {forElement: menu.forElement, scroll: "default", contentType: this._type, direction: menu.position !== "after-cursor" ? menu.position : "auto"});	// No I18N
			if(this._type === "icon-text" && menu.contentType !== "text" && menu.contentType !== "text-desc" && !isImageSprite){ // No I18N
				this._menu.addClass("zsuggestfield"); // No I18N
				menu.contentType === "image-text-desc" && this._menu.addClass("zmenu--withhint");	//No I18N
			}
			if(menu.contentType === "image-text" && menu.itemType === "checkbox" && isImageSprite){		// No I18N
				this._menu.addClass("zsuggestfield--withicon"); // No I18N
			}
			if(menu.contentType === "image-text-desc" && menu.itemType === "checkbox" && isImageSprite){	// No I18N
				this._menu.addClass("zsuggestfield zsuggestfield--withiconandhint"); // No I18N
			}
			this._dataMapping.groupLabel && (this.options.lazyLoadData = false);
			(this.options.search.minKeywordLength > this._CONSTANTS.minKeywordLength) && (this.options.search.minKeywordLength = this._CONSTANTS.minKeywordLength);
			(menu.maxDisplayResults === undefined && this.options.lazyLoadData) && (menu.maxDisplayResults = this._CONSTANTS.maxResults);
			menu.appendTo && this._menu.zmenu("option", "appendTo", menu.appendTo);	// No I18N
			menu.menuClassName && this._menu.addClass(menu.menuClassName);
			menu.height !== undefined && this._menu.zmenu("option", "height", menu.height); 	// No I18N
			menu.width !== undefined && this._menu.zmenu("option", "width", menu.width); 	// No I18N
		},
		_bindEvents: function(){
			var componentName = this.componentName;
			this._element.on('keydown.'+componentName, this._keydowHandler.bind(this))	  // No I18N
						 .on('keyup.'+componentName, this._keyupHandler.bind(this)) // No I18N
						 .on('mouseup.'+componentName, this._mouseupHandler.bind(this)) // No I18N
						 .on('paste.'+componentName, this._pasteHandler.bind(this)); // No I18N
			ZC.$document.on("click."+componentName, this._documentClickHandler.bind(this));  // No I18N
		},
		_documentClickHandler: function(orgEvent){
			var target = $(orgEvent.target), value = this._element.val();
			if(target.closest(".zsuggestfield-element")[0] !== this._element[0] && target.closest(".zmenu").attr("id") !== this._menu.attr("id")  && value !== this._oldVal){  // No I18N
				var data = this._currentData ? {} : undefined;
				data = this._getData(this._currentData);				
				this._dispatchEvent("CHANGE", {value: value, data: data}, orgEvent); // No I18N
				this._oldVal = this._element.val();
			}	
		},
		_pasteHandler: function(orgEvent){
			setTimeout(this._searchTimeout.bind(this, this._element.val(), this._element, orgEvent), 200);
		},
		_mouseupHandler: function(orgEvent){	
			var query = this._element.val();
			query = query.substring(0 , this._element[0].selectionStart);
			this._searchTimeout(query, this._element, orgEvent);
		},
		_keydowHandler: function(orgEvent){
			var keyCode = ZC.keyCode;
			if(this._menu !== undefined && !this._menu.is(":visible")){	// No I18N
				if(orgEvent.keyCode === keyCode.DOWN && !this._ajaxSource){
					this._search('', this._element, orgEvent);	// No I18N
				}
				if(orgEvent.keyCode === keyCode.ENTER){
                	ZC.$document.trigger("click");  // No I18N
				}
				return true;
			}
			switch(orgEvent.keyCode){
				case keyCode.UP:
			    case keyCode.DOWN:
				case keyCode.TAB:
                case keyCode.ESCAPE:
             	    orgEvent.preventDefault();
					!this._isLoaderInited && this._menu.zmenu('handleKeyCode',orgEvent.keyCode); // No I18N	
                break;
                case keyCode.ENTER:
                	orgEvent.preventDefault();
					!this._isLoaderInited && this._menu.zmenu('handleKeyCode',orgEvent.keyCode); // No I18N	
                	ZC.$document.trigger("click");  // No I18N
                break;
			}
		},
		_keyupHandler: function(orgEvent){
			var query = this._element.val(), keyCode = ZC.keyCode;
			if((orgEvent.keyCode >= 186 && orgEvent.keyCode <= 192) || (orgEvent.keyCode >= 219 && orgEvent.keyCode <= 222) || (orgEvent.keyCode >= 48 && orgEvent.keyCode <= 57) || (orgEvent.keyCode >= 65 && orgEvent.keyCode <= 90) || (orgEvent.keyCode ===  keyCode.SPACE) || (orgEvent.keyCode ===  keyCode.BACKSPACE) || (orgEvent.keyCode ===  keyCode.DELETE)){		
				this._searchTimeout(query, this._element, orgEvent);
			}
			if(orgEvent.keyCode === keyCode.TAB){
				this._menu.is(":visible") && this._menu.zmenu("selectHoveredMenuItem");	// No I18N
			}
		},
		_bindMenuEvents: function(){
			var base = this;
			this._menu.off("zmenuitemclick").on("zmenuitemclick", this._handleItemClick.bind(this)) 	// No I18N
					  .off('zmenubeforefocus').on('zmenubeforefocus', function(orgEvent, ui){	// No I18N
						return false; })
				  	  .off("zmenuitemhover").on("zmenuitemhover", this._handleItemHover.bind(this))  // No I18N
					  .off("zmenubeforeshow").on("zmenubeforeshow", this._handleMenuBeforeShow.bind(this))  // No I18N
					  .off("zmenushow").on("zmenushow", this._handleMenShow.bind(this))  // No I18N
					  .off("zmenubeforehide").on("zmenubeforehide", this._handleMenuHide.bind(this))  // No I18N
					  .off("zmenuhide").on("zmenuhide", this._handleMenuHide.bind(this)) // No I18N
		},
		_handleMenuBeforeHide: function(orgEvent, ui){
			this._dispatchEvent("MENUBEFORECLOSE", {value: ui}, orgEvent);  // No I18N
		},
		_handleMenuHide: function(orgEvent, ui){
			this._input && this._input.attr("aria-expanded", "false");	//No I18N
			this._dispatchEvent("MENUAFTERCLOSE", {value: ui}, orgEvent);  // No I18N
			orgEvent.stopImmediatePropagation(); 
			//Prevent zmenuhide from triggering when event binded through the element selector
		},
		_handleMenShow: function(orgEvent, ui){
			this._input && this._input.attr("aria-expanded", "true");	//No I18N
			this._dispatchEvent("MENUAFTEROPEN", {value: ui}, orgEvent);  //No I18N
		},	
		_handleMenuBeforeShow: function(orgEvent, ui){
			if(this.options.isImageSourceSprited && this.options.dropdownList.itemType === "checkbox" && (this.options.dropdownList.contentType === "image-text" || this.options.dropdownList.contentType === "image-text-desc")){	// No I18N
				this._menu.removeClass("zmenu--iconwithtext");
			}else if(!this.options.isImageSourceSprited && (this.options.dropdownList.contentType === "image-text" || this.options.dropdownList.contentType === "image-text-desc")){	// No I18N
				this._menu.removeClass("zmenu--iconwithtext");	
			}
			this._dispatchEvent("MENUBEFOREOPEN", {value: ui}, orgEvent);  // No I18N
		},
		_handleItemHover: function(orgEvent, ui){
			var componentName = this.componentName
			if(ui.menuitem.attr("id") === componentName+"noresults" || ui.menuitem.attr("id") === componentName+"loaderror" || ui.menuitem.attr("id") === componentName+"loader"){	// No I18N
				return false;
			}else{
				if(this.options.autoComplete){
					this._handleHover(orgEvent, ui);
				}
				this._dispatchEvent("ITEMFOCUS", {value: ui.menuitem}, orgEvent); // No I18N
			}
		},
		_handleItemClick: function(orgEvent, ui){
			this._handleSelection(orgEvent, ui);
		},
		_mapJSONToData: function(data, valueObj){
			if(valueObj !== undefined && data !== undefined){
				var valueObj = valueObj.split(".");	// No I18N
				for(var i=0;i<valueObj.length;i++){
					data =  data[valueObj[i]];
				}
				return data;
			}	
		},
		_getData: function(currentData){
			var data = {};
			if(currentData){
				for(var key in currentData){
					if(this._dataMapping[key]){
						data[this._dataMapping[key]] = this._mapJSONToData(currentData, key);
					}
				}
			}
			return data;
		},
		_handleHover: function(orgEvent, ui){
			var componentName = this.componentName;
			if(!(ui.menuitem.attr("id") === componentName+"noresults") && !(ui.menuitem.hasClass("zmenu__title")) && !(ui.menuitem.attr("id") === componentName+"moreaction")){	// No I18N
				if(!this._isJSONSource){
					this._element.val(ui.menuitem.data().metadata.label);
				}else{
					this._element.val(ui.menuitem.data().itemValue.text);
				}
			}
		},
		_initSource: function(){
			var dataSource = this.options.dataSource
			if($.isArray(dataSource)){
				this._isJSONSource = typeof dataSource[0] === "object"; 	// No I18N
				this._responseData = this.options.dataSource;
			}else if(typeof dataSource === "object"){	// No I18N
				if(dataSource.URL){
					this._ajaxSource = true;
					this._isJSONSource = true;
				}
			}

			if(!this._isJSONSource){
				var options = this.options;
				this._initialContentType = this.options.dropdownList.contentType;
				options.isImageSourceSprited = options.isResponsePaginated = options.isResponseFiltered = false, options.dropdownList.contentType = "text"; // No I18N
			}
		},
		_searchTimeout: function(query, container, orgEvent){
			this._newQuery = true;
			clearTimeout(this._searchTimer);
			if(this.options.search.delay){
				this._searchTimer = setTimeout(this._search.bind(this, query, container, orgEvent), this.options.search.delay);
			}
		},
		_addLoaderIcon: function(iconElement){
			var dropdown = this.options.dropdownList;
			if(dropdown.progressIndicatorIconClassName || dropdown.progressIndicatorSVGIconId){
				if(dropdown.progressIndicatorIconClassName){
					iconElement.addClass(dropdown.progressIndicatorIconClassName);
				}else{
					var svgConfig = dropdown.progressIndicatorSVGIconId, svgId = svgConfig.split(" ")[0], customClass = svgConfig.slice(svgId.length).trim();
					iconElement.append($("<svg class='"+customClass+"'><use xlink:href='"+svgId+"'></use></svg>"))
				}
			}else{
				var svgElement = ZC.createSVGElement("svg", {"class":"zmenu__svg zmenu__loader zeffects--rotate"}),	// No I18N
				circleElement1 = ZC.createSVGElement("circle", {"class": "circle__track"}),	// No I18N
				circleElement2 = ZC.createSVGElement("circle", {"class": "circle__load"});	// No I18N
				svgElement.appendChild(circleElement1);
				svgElement.appendChild(circleElement2);
				iconElement[0].appendChild(svgElement);
			}
			return iconElement
		},
		_initLoader: function(container, orgEvent){
			this._isLoaderInited = true;
			var iconElement = $("<i class='zmenu__icon h-loading'>");	// No I18N
			iconElement = this._addLoaderIcon(iconElement);
			this._loader = this._menu.zmenu("addMenuItem", {id: this.componentName+"loader",fixed: true, innerHTML: iconElement, itemType: "custom", itemCssClass: "zsuggestfield__nonselectable"});	// No I18N
			this._loader.closest(".zmenu__footer").removeClass("zmenu__footer").addClass("zmenu--loading");		// No I18N
			this._menu.zmenu("show", {"mustBeShown":true, preventHover: true});	// No I18N
			this._makeAjaxRequest(container, orgEvent);
		},
		_search: function(query, container, orgEvent){
			this._filteredResult = [];
			var def = $.Deferred();
			this._currentData = undefined;
			if(this._request){
				this._request.abort();
				this._removeLoader();
				// if(this._filteredResult && this._filteredResult.length){
				// 	this._responseData = this._filteredResult;
				// 	this._filterSuggestions(query);
				// 	this._menu.zmenu("removeMenuItems"); // No I18N
				// 	this._renderMenu(container);
				// }		
				if(this.options.isResponsePaginated){
					this._params && (this._params.pages = undefined);
				}
			}
			if(this.options.lazyLoadData){
				this._pageIndex = 0;
				if(this.options.isResponsePaginated){
					this._params && (this._params.pages = undefined);
				}
			}
			var dataSource = this.options.dataSource; 
			this._params = $.extend(true, {}, this._params, {query: query});
			if(query !== undefined && (query.length >= this.options.search.minKeywordLength) && query.length < this.options.search.maxKeywordLength){
				var returnValue = this._dispatchEvent("BEFORESEARCH", {data: this._params}, orgEvent); // No I18N
				if(this._ajaxSource){
					if(returnValue){
						if(query in this._cachedData){
							this._responseData = this._cachedData[query];	
							if(this.options.isResponseFiltered){
								this._filteredResult = this._responseData;
							}else{
								this._filterSuggestions(query);
							}
							this._renderSuggestions(container, orgEvent);
						}else{
							if(dataSource.queryParam && dataSource.searchKeywordSubsetsInCache && this.options.cacheServerResponse && !this.options.isResponsePaginated){
								var flag = false;
								for(var i=this._params.query.length-1; i>=1; i--){
									var query = this._params.query.slice(0, i);
									if(query in this._cachedData){
										this._responseData = this._cachedData[query];
										flag = true;
										break;
									}
								}
								if(flag){
									this._filterSuggestions(this._params.query);
									this._cachedData[this._params.query] = this._filteredResult;
									this._renderSuggestions(container, orgEvent);
								}else{
									this._menu.zmenu("removeMenuItems");	// No I18N
									this._callLoader(container, orgEvent);
								}
							}
							else{
								this._menu.zmenu("removeMenuItems");	// No I18N
								if(dataSource.queryParam){
									this._callLoader(container, orgEvent);
								}else{
									if(!this._requestInitialized){
										this._callLoader(container, orgEvent);
										this._requestInitialized = true;
									}else{
										this._filterSuggestions(query);
										this._renderSuggestions(container, orgEvent);
									}
								}
							}			
						}
					}else{
						this._menu.is(":visible") && this._hideSuggestions();	 // No I18N
					}
				}else{
					this._responseData = this.options.dataSource;
					this._filterSuggestions(query);
					this._renderSuggestions(container, orgEvent);
				}		
			}else{
				if(typeof query === "string" && query.length === 0 && this.options.search.minKeywordLength === 0){  // No I18N
					if(!this._ajaxSource){
						this._params = {query: query};
						this._filteredResult = this.options.dataSource.slice();	// To prevent mutation of the original array
						this._sortResults();
						this._renderSuggestions(container, orgEvent);
					}else{
						 if(this.options.showDropDownListOnFocus){
						 	this._menu.zmenu("removeMenuItems");	// No I18N
							if(dataSource.queryParam){
								this._callLoader(container, orgEvent);
							}else{
								if(!this._requestInitialized){
									this._callLoader(container, orgEvent);
									this._requestInitialized = true;
								}else{
									this._filteredResult = this._responseData;
									this._renderSuggestions(container, orgEvent);
								}
							}
						 }else{
							this._removeLoader();
							this._menu.is(":visible") && this._hideSuggestions();	// No I18N
						}	
					}
				}else{
					this._removeLoader();
					this._menu.is(":visible") && this._hideSuggestions();	// No I18N
				}			
			}
			this._newQuery = false;
			return def.promise();
		},
		_callLoader: function(container, orgEvent){
			this._initLoader(container, orgEvent);
			this._menu.css("height", "70px");	// No I18N
			this._menu.find(".zmenu--loading").find("ul").css("height", "100%");	 // No I18N
		},
		_handleSelection: function(orgEvent, ui){
			this._oldVal = this.element.val();
			if(!(ui.menuitem.attr("id") === this.componentName+"noresults") && !(ui.menuitem.hasClass("zmenu__title")) && !(ui.menuitem.hasClass("zsuggestfield__nonselectable"))){	// No I18N
				if(ui.menuitem.attr("id") === this.componentName+"moreaction"){	// No I18N
					var action = this.options.dropdownList.moreResultsAction;
					this._dispatchEvent("MOREACTIONSELECT", {value: ui.menuitem, query: this._params.query});  // No I18N
					if(action && $.isFunction(action)){
						action();
					}
				}else{
					//this.options.multiselect && !this.options.multiselect.closeOnSelect && orgEvent.preventDefault();
					if(!this._isJSONSource){
						this._element.val(ui.menuitem.data().metadata.label);
					}else{	
						var itemData = ui.menuitem.data().metadata;
						var result =itemData.itemValue !== undefined ? itemData.itemValue.text : this._mapJSONToData(itemData, this._dataMapping.text);
						this._currentData = itemData.itemValue;
						this._element.val(result);
					}
					var data = {};
					data = this._getData(this._currentData)
					this._dispatchEvent("ITEMSELECT", {value: ui.menuitem, data: data});  // No I18N
					this._element.focus();
				}
			}else{
				this._element.focus();
			}
		},
		_makeAjaxRequest: function(container, orgEvent){
			var dataSource = this.options.dataSource;
			dataSource.sourceURL = typeof dataSource.URL === "function" ? dataSource.URL.call(this.element, this._params) : dataSource.URL;	// No I18N
			dataSource.data = typeof dataSource.queryParam === "function" ? dataSource.queryParam.call(this.element, this._params) : dataSource.queryParam;	// No I18N
			this._requestData(container, orgEvent);
		},
		_successCallback: function(container,orgEvent, data){
			var dataSource = this.options.dataSource;
			this._filteredResult = [];
			this._request = undefined;
			this._isLoaderInited = false;
			this._response = dataSource.success(data, this._params);
			this._responseData = this._response.results;
			this._dispatchEvent("SEARCHSUCCESS", this._responseData, orgEvent);  // No I18N
			if(this._response.more && this._params.pages !== undefined && this.options.lazyLoadData){
				this._cacheResponse(this._responseData);
				this._filteredResult = this._responseData;
				this._totalResponse = this._filteredResult;
				this._spliceResults();
				this._pageIndex = 0;
				var scrollTop = this._menu.find("ul").scrollTop();	// No I18N
				this._removeLoader();
				this._renderMenu(container);
				this._menu.find("ul").scrollTop(scrollTop);	// No I18N
			}else if (this._params.pages === undefined){
				this._cacheResponse(this._responseData);
				if(this.options.isResponseFiltered){
					this._filteredResult = this._responseData;
				}else{
					this._filterSuggestions(this._params.query);
				}
				this._renderSuggestions(container, orgEvent);
			}else{
				this._removeLoader();
				return;
			}
		},
		_spliceResults: function(){
			var maxResults = this.options.dropdownList.maxDisplayResults;
			if(maxResults){
				this._totalPages = Math.ceil(this._filteredResult.length / maxResults);
				this._filteredResult = this._filteredResult.slice(0, maxResults);
			}
		},
		_cacheResponse: function(response){
			var data = this._cachedData[this._params.query];
			if(this.options.cacheServerResponse){
				this._cachedData[this._params.query] = data && data.length ? data.concat(response) : response;
			}
		},
		_removeLoader: function(){
			if(this._loader){
				this._menu.find(".zmenu--loading").remove();	// No I18N
				this._loader = null;
			}
			this._menu.removeData("fixedArea");	// No I18N
		},
		_failureCallback: function(orgEvent, jqXHR, exception){
			if(jqXHR.statusText !== "abort"){	// No I18N
				var dataSource = this.options.dataSource;
				this._isLoaderInited = false;
				this._removeLoader();
				this._menu.zmenu("removeMenuItems");	// No I18N
				this._menu.zmenu("addMenuItem", {"label" : this._localize('serverError'), "index" : 0, id: this.componentName+"loaderror", itemCssClass: "zsuggestfield__nonselectable"}); // No I18N
				this._menu.zmenu("show", {mustBeShown: true, preventHover: true, positionAlterable: true});	// No I18N
				if(dataSource.fail && $.isFunction(dataSource.fail)){
					dataSource.fail(jqXHR, exception);
				}
				this._dispatchEvent("SEARCHERROR", {jqXHR: jqXHR, exception: exception}, orgEvent);		  // No I18N			
			}	
		},
		_requestData: function(container, orgEvent){
			this._request = ZC.AJAX.transport(this.options.dataSource, this._successCallback.bind(this, container, orgEvent), this._failureCallback.bind(this, orgEvent));
		},
		_checkIfItemExists: function(valueToCheck){
			var validPush = false, sourceData = this._filteredResult;
			if(!sourceData.length){
				sourceData.push(valueToCheck);
			}else{
				for(var i=0;i<sourceData.length;i++){
					var sourceKey = this._mapJSONToData(sourceData[i], this._dataMapping.value), valueKey = this._mapJSONToData(valueToCheck, this._dataMapping.value);
					if(sourceKey !== valueKey){
						validPush = true;
					}else{
						validPush = false;
						break;
					}
				}
				validPush && sourceData.push(valueToCheck);
			}			
		},
		_filterSuggestions: function(query){
			//this._filteredResult = [];
			var searchByField, base = this;
			this._isJSONSource && typeof this.options.search.by === "string" && (searchByField = this.options.search.by.split(","));	// No I18N
			if(searchByField){
				switch(this.options.search.criteria){
					case "contains":   	// No I18N
						if(this._isJSONSource){ 
							for(var i=0;i<this._responseData.length;i++){
								for(var j=0;j<searchByField.length;j++){
									var value = this._mapJSONToData(this._responseData[i], searchByField[j]);
									if(value !== undefined){
										value = value.toString();
										if(value.toLowerCase().indexOf(query.toLowerCase()) > -1){
											this._checkIfItemExists(this._responseData[i]);							
										}	
									}							
								}
							}
						}else{
							this._filteredResult = this._responseData.filter(function(item){
								return (item !== undefined && item.toLowerCase().indexOf(query.toLowerCase())) > -1;
							});
						}	
					break;
					case "startsWith":   // No I18N	
						this._getFilteredResult(new RegExp('^'+query, 'gi')); // No I18N
						break;
					case "endsWith":   	// No I18N 	
						this._getFilteredResult(new RegExp(query+'$', 'gi'));  // No I18N
						break;
					case 'equals':   // No I18N
						if(this._isJSONSource){ 
							for(var i=0;i<this._responseData.length;i++){
								for(var j=0;j<searchByField.length;j++){
									var value = this._mapJSONToData(this._responseData[i], searchByField[j]).toString().toLowerCase();
									if(value !== undefined && value === query.toLowerCase()){
										this._checkIfItemExists(this._responseData[i]);							
									}
								}
							}
						}else{
							this._filteredResult = this._responseData.filter(function(item){
								return item !== undefined && item.toString().toLowerCase() === query.toString().toLowerCase();
							});
						}	
					break;
				}
			}
			this._sortResults();
		},
		_getFilteredResult: function(regEx){
			var searchByField, base = this;
			this._isJSONSource && typeof this.options.search.by === "string" && (searchByField = this.options.search.by.split(","));	// No I18N
			if(this._isJSONSource){	
				for(var i=0;i<this._responseData.length;i++){
					for(var j=0;j<searchByField.length;j++){
						var value = this._mapJSONToData(this._responseData[i], searchByField[j]).toString().toLowerCase();
						if(value !== undefined && value.match(regEx) && value.match(regEx).length){
							this._checkIfItemExists(this._responseData[i]);	
						}
					}
				}
			}else{
				this._filteredResult = this._responseData.filter(function(item){
					var value = item.toString().toLowerCase();
					return (value !== undefined && value.match(regEx) && value.match(regEx).length);					
				})
			}
		},
		_sortResults: function(){
			if(this.options.dropdownList.sortResults){
				var base = this;
				if(this._isJSONSource){
					this._filteredResult.sort(function(itemA, itemB) {
						var valueA = base._mapJSONToData(itemA, base.options.dataMapping.text).toLowerCase(), valueB = base._mapJSONToData(itemB, base.options.dataMapping.text).toLowerCase();
				    	return (valueA > valueB) ? 1 : (valueA < valueB) ? -1 : 0;	
					});
				}else{
					this._filteredResult.sort();
				}		
			}
		},
		_lazyLoadContent: function(container, orgEvent){
			if(this._totalPages > 1){
				var base = this;
				this._menu.find("ul").on("scroll."+this.componentName, function(orgEvent){	  // No I18N
					if(!base._request){
						if(base._menu.find("ul").scrollTop() >= (base._menu.find("ul")[0].scrollHeight - base._menu.find("ul").outerHeight())){
							var scrollTop = base._menu.find("ul").scrollTop();
							++base._pageIndex;
							if(base._pageIndex >= base._totalPages){
								if(base.options.isResponsePaginated && base._response && base._response.more !== false){
									base._params.pages++;
									base._initLoader(container, orgEvent);
									base._menu.find(".zmenu--loading").find("ul").css("height", "100%");	 // No I18N
								}
							}else{
								base._filteredResult = base._totalResponse.slice(base._pageIndex * base.options.dropdownList.maxDisplayResults,  base._pageIndex * base.options.dropdownList.maxDisplayResults + base.options.dropdownList.maxDisplayResults);
								base._renderMenu(container);
							}			
							base._menu.find("ul").scrollTop(scrollTop);
						}
					}
				})
			}
		},
		_renderSuggestions: function(container, orgEvent){
			this._focusSet = false;
			if(this.options.isResponsePaginated){
				this._params.pages = 1;
			}		
			this._menu.zmenu("removeMenuItems"); // No I18N
			if(!this.options.allowDuplicateValues && this.options.dropdownList.itemType === "none"){
				this._removeAddedItem && this._removeAddedItem();
			}
			if(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel && this._filteredResult.length){
				this._groupedResult = {};
				for(var i=0; i<this._filteredResult.length;i++){
					var title = this._mapJSONToData(this._filteredResult[i], this._dataMapping.groupLabel);
					if(title in this._groupedResult){
						this._groupedResult[title].push(this._filteredResult[i]); 
					}else{
						this._groupedResult[title] = [];
						this._groupedResult[title].push(this._filteredResult[i]); 
					}
				}
			}
			this._totalResponse = this._filteredResult;
			this._spliceResults();
			this.options.lazyLoadData && this._lazyLoadContent(container); 
			this._renderMenu(container, orgEvent).done(this._handleEnterPressed && this._handleEnterPressed(orgEvent));
		},
		_getItemJSON: function(data){
			var itemImage = this._mapJSONToData(data, this._dataMapping.image), 
				itemText = this._mapJSONToData(data, this._dataMapping.text), 
			    itemValue = this._mapJSONToData(data, this._dataMapping.value),
			    itemSecondaryText = this._mapJSONToData(data, this._dataMapping.informativeText),
			    itemIdentifier = this._mapJSONToData(data, this._dataMapping.itemIdentifier),
			    itemTitle = this._mapJSONToData(data, this._dataMapping.titleText),
			    customValues = this._mapJSONToData(data, this._dataMapping.customValues);
			return {icon: itemImage, name: itemText, key: itemValue, description: itemSecondaryText, identifier: itemIdentifier, customValues: customValues, title: itemTitle};
		},
		_renderMenu: function(container){
			this._lastSelectedMenuItem = null;
			var def = $.Deferred();
			var menuJSON = [], numId = 0, content;
			if((this.options.isImageSourceSprited && (this.options.dropdownList.contentType === "image-text" || this.options.dropdownList.contentType === "image-text-desc"))		// No I18N
				|| (this.options.isImageSourceSprited && this.options.dropdownList.itemType === "checkbox") || (!this.options.isImageSourceSprited && this.options.dropdownList.contentType === "text" && this.options.dropdownList.itemType === "checkbox")){		// No I18N
				if(!this._menu.hasClass("zmenu--iconwithtext")){
					this._menu.addClass("zmenu--iconwithtext");
				}
			}
			if(this._filteredResult && this._filteredResult.length){
				if(!this._isJSONSource){
					for(var i=0; i<this._filteredResult.length;i++){
						var object = {"id": this._filteredResult[i], "label": this._filteredResult[i]};	// No I18N
						this._menu.zmenu("addMenuItem", object);	// No I18N
					}
				}else{		
					if(this.options.dropdownList.contentType !== "text"){	// No I18N						
						if(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel){
							for(var key in this._groupedResult){
								numId++;
								var jsonObj = {"label": key, "itemType": "itemgroup", "id": numId};	// No I18N	
								menuJSON.push(jsonObj);
								for(var i=0; i<this._groupedResult[key].length;i++){
									var itemObj = {};
									var jsonItem = this._getItemJSON(this._groupedResult[key][i]);
									var checkbox = this.options.dropdownList.itemType === "checkbox" ? true : false, type = this.options.dropdownList.contentType;	//No I18N
									if(this.options.dropdownList.contentType === "text-desc"){	// No I18N
										content = "<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>";	// No I18N
									}else{
										content = !this.options.isImageSourceSprited ? "<img src='"+jsonItem.icon+"' class='zsuggestfield__avatar'>" : "<div class='zmenu__iconcontainer'><i class='zmenu__icon "+jsonItem.icon+"'></i></div>";
								    content = type === "image-text" ? content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span></div>" : 
								                     content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>"; // No I18N	
									}				
								    content = checkbox ? "<div class='zmenu__iconcontainer'><i class='zmenu__icon'></i></div>"+content : content;  // No I18N
									itemObj = {
										"itemType": checkbox ? "checkbox" : "custom",// No I18N
										"custom": true,	// No I18N
									    "innerHTML": {content:content, replaceArray:[jsonItem.name, jsonItem.description]}, 	// No I18N
										"isCustomHTMLEncoded": false,	 // No I18N
									    "id": ZC.encodeHTML(jsonItem.key),	// No I18N	
									    "name": jsonObj.id,// No I18N	
										"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}		// No I18N
									}
									menuJSON.push(itemObj);
								}
							}	
							this._menu.zmenu("addMenuItems", menuJSON);	// No I18N
						}
						else{
							var jsonData = {};
							for(var i=0; i<this._filteredResult.length;i++){
								var jsonItem = this._getItemJSON(this._filteredResult[i]);
								var checkbox = this.options.dropdownList.itemType === "checkbox" ? true : false, type = this.options.dropdownList.contentType;	//No I18N
								if(this.options.dropdownList.contentType === "text-desc"){	// No I18N
									content = "<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>";	// No I18N
								}else{
									content = !this.options.isImageSourceSprited ? 
		                       	 		"<img src='"+jsonItem.icon+"' class='zsuggestfield__avatar'>" : "<div class='zmenu__iconcontainer'><i class='zmenu__icon "+jsonItem.icon+"'></i></div>";
									content = type === "image-text" ? content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span></div>": content+"<div class='zmenu__content'><span class='zmenu__text'>{0}</span><span class='zmenu__hint'>{1}</span></div>"; // No I18N
								}
								content = checkbox ? "<div class='zmenu__iconcontainer'><i class='zmenu__icon'></i></div>"+content : content;  // No I18N
								jsonData = {
									"itemType" : checkbox ? "checkbox" : "custom", 	// No I18N
									"custom": true,	// No I18N
									"innerHTML": {content: content, replaceArray: [jsonItem.name, jsonItem.description]}, // No I18N
									"id": ZC.encodeHTML(jsonItem.key),	// No I18N
									"isCustomHTMLEncoded": false,  // No I18N
									"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}		// No I18N
								};
								this._menu.zmenu('addMenuItem', jsonData); // No I18N
							}
						}
					}else{
						if(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel){
							for(var key in this._groupedResult){
								numId++;
								var jsonObj = {"label": key, "itemType": "itemgroup", "id": numId};	// No I18N
								menuJSON.push(jsonObj);
								for(var i=0; i<this._groupedResult[key].length;i++){
									var itemObj = {};
									var jsonItem = this._getItemJSON(this._groupedResult[key][i]);
									itemObj = {
										"label": ZC.encodeHTML(jsonItem.name),	// No I18N
										"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}, // No I18N
										"name": jsonObj.id, // No I18N
										"itemType": this.options.dropdownList.itemType === "checkbox" ? "checkbox" : "text", // No I18N
										"id": ZC.encodeHTML(jsonItem.key) // No I18N
									}
									menuJSON.push(itemObj);
								}
							}
							this._menu.zmenu("addMenuItems", menuJSON);	// No I18N
						}
						else{
							var items = [], itemObj = {};
							for(var i=0; i<this._filteredResult.length;i++){
								var jsonItem = this._getItemJSON(this._filteredResult[i]);
								itemObj = {
									"label": ZC.encodeHTML(jsonItem.name), // No I18N
									"itemValue": {text: jsonItem.name, image: jsonItem.icon, informativeText: jsonItem.description, value: jsonItem.key, itemIdentifier: jsonItem.identifier, customValues: jsonItem.customValues, titleText: jsonItem.title}, // No I18N
								    "itemType": this.options.dropdownList.itemType === "checkbox" ? "checkbox" : "text", // No I18N
								    "id": ZC.encodeHTML(jsonItem.key) // No I18N
								}
								this._menu.zmenu('addMenuItem', itemObj); 	// No I18N
							}
						}
					}						
				}
				var moreAction = this.options.dropdownList.moreResultsAction, text;
				if(moreAction && !this.options.lazyLoadData){
					var moreActionText = this._localize('moreAction'); 	// No I18N
					moreActionText = ZC.encodeHTML(moreActionText);
					 if(typeof moreActionText === "string"){	// No I18N
						if(moreActionText.match(/#query#/g)){	
							text = moreActionText.replace(/#query#/g, this._params.query);
							moreActionText = moreActionText.replace(/#query#/g, "<b class='h-bold'>"+this._params.query+"</b>")	// No I18N										
						}else{
							text = moreActionText;
						}
						var container = $("<span class='zmenu__text'>").html(moreActionText);	
					}
					var item = this._menu.zmenu("addMenuItem", {id: this.componentName+"moreaction", "label": ZC.encodeHTML(text), fixed: true, innerHTML: container, itemType: "custom"});	// No I18N
					item.css("min-height", "inherit");	// No I18N
				}
				this._menu.zmenu("show", {"mustBeShown":true, preventHover: true, positionAlterable: true});	// No I18N
				if(!(this.options.isImageSourceSprited && (this.options.dropdownList.contentType === "image-text-desc") && this.options.dropdownList.itemType === "none") && this.options.dropdownList.contentType !== "text-desc"){		// No I18N
					var totalWidth = 0;
					if(!this.options.isImageSourceSprited){	
						var avatar = $(this._menu.find("li")[this._dataMapping && this._isJSONSource && this._dataMapping.groupLabel ? 1 : 0]).find(".zsuggestfield__avatar");	// No I18N
						if(avatar.length){
							totalWidth = avatar.outerWidth(true);
						}
					}
					var icon = $(this._menu.find("li")[this._dataMapping && this._isJSONSource && this._dataMapping.groupLabel ? 1 : 0]).find(".zmenu__iconcontainer");	// No I18N
					if(icon.length){
						for(var i=0; i<icon.length;i++){
							totalWidth = totalWidth + $(icon[i]).outerWidth(true);
						}		
					}
					totalWidth = totalWidth && totalWidth.toString();
					if(!(this._type === "icon-text" && this.options.dropdownList.itemType === "checkbox" && this.options.isImageSourceSprited && !this.options.dropdownList.contentType === "image-text-desc")){	// No I18N
						totalWidth && this._menu.find(".zmenu__content").css("width", "calc(100% - "+totalWidth+"px)");	// No I18N
					}		
				}
				if(this.options.dropdownList.position === "after-cursor"){	// No I18N
					var pos = this._positionMenuAtCursorPosition(container);
					this._menu.css({top: (pos.top + 15), left: pos.left});
				}
				this.options.dropdownList.highlightKeyword && this._highlightSearchQuery();
				var id,element;
				this._lastSelectedMenuItem && (id = this._lastSelectedMenuItem.attr("id"));		// No I18N
				this._lastSelectedMenuItem && (element = $("#"+id));
				if(element && element.length){
					this._menu.zmenu("hoverMenuItem", element);	// No I18N
					this._lastSelectedMenuItem = null;
				}else{
					if(this.options.dropdownList.focusFirstItem && !this.options.autoComplete){
						if(!this._focusSet){
							this._focusSet = true;
							this._menu.zmenu("hoverMenuItem", this._menu.find("li").eq(this._isJSONSource && this._dataMapping && this._dataMapping.groupLabel ? 1 : 0));	// No I18N
						}
					}
				}
				this._removeLoader();			
			}else{
				this._removeLoader();
				if(this.options.dropdownList.showNoResultsMessage){
					var noResultsText = this._localize('noResults');	// No I18N
					noResultsText = ZC.encodeHTML(noResultsText);
					 if(typeof noResultsText === "string"){	// No I18N
						if(noResultsText.match(/#query#/g)){	
							noResultsText = noResultsText.replace(/#query#/g, "<b class='h-bold'>"+this._params.query+"</b>")	
						}
					}
					this._displayNoresults(noResultsText);
					if(this.options.dropdownList.position === "after-cursor"){	// No I18N
						var pos = this._positionMenuAtCursorPosition(container);
						this._menu.css({top: (pos.top+15), left: pos.left});
					}
				}else{
					this._menu.is(":visible") && this._hideSuggestions();	// No I18N
				}
			}	
			this._newQuery = false;	
			return def;
		},
		_highlightSearchQuery: function(){
			var searchBy, menuItems, informativeText, text, groupLabel, componentName = this.componentName;
			if(this._isJSONSource){
				typeof this.options.search.by === "string" && (searchBy = this.options.search.by.split(","));  // No I18N
				informativeText = this._dataMapping.informativeText, text = this._dataMapping.text,
				groupLabel = this._dataMapping.groupLabel; 	
			}
			if(this._isJSONSource && searchBy){
				if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) > -1 && searchBy.indexOf(informativeText) === -1){
					menuItems = this._menu.find(".zmenu__text:not('#"+componentName+"moreaction')");
				}else if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) === -1){
					menuItems = this._menu.find(".zmenu__title").find(".zmenu__text");
				}else if(searchBy.indexOf(groupLabel) === -1 && searchBy.indexOf(text) > -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__item:not('#"+componentName+"moreaction')").find(".zmenu__text,.zmenu__hint")
				}else if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) > -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__text,.zmenu__hint:not('#"+componentName+"moreaction')");
				}else if(searchBy.indexOf(groupLabel) > -1 && searchBy.indexOf(text) === -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__hint, .zmenu__title > .zmenu__text")
				}else if(searchBy.indexOf(text) === -1 && searchBy.indexOf(informativeText) > -1){
					menuItems = this._menu.find(".zmenu__hint")
				}else if(searchBy.indexOf(text) >  -1 && searchBy.indexOf(informativeText) === -1 && searchBy.indexOf(groupLabel) === -1){
					menuItems =this._menu.find(".zmenu__item:not('#"+componentName+"moreaction')").find(".zmenu__text");
				}	
			}else{
				menuItems = this._menu.find(".zmenu__item:not('#"+componentName+"moreaction')").find(".zmenu__text");
			}
			if(menuItems){
				for(var i=0; i<menuItems.length; i++){
					var text = $(menuItems[i]).text();
					text = ZC.encodeHTML(text);
					switch(this.options.search.criteria){
						case "contains":   	// No I18N
						$(menuItems[i]).html(text.replace(new RegExp("(?![^&;]+;)(?!<[^<>]*)("+this._params.query+")(?![^<>]*>)(?![^&;]+;)", "gi"), "<b class='h-bold'>$1</b>")); 	// No I18N
						break;
						case "startsWith":   	// No I18N
						$(menuItems[i]).html(text.replace(new RegExp("(^"+this._params.query+")", "gi"), "<b class='h-bold'>$1</b>"));  	// No I18N
						break;
						case "endsWith":  // No I18N
						$(menuItems[i]).html(text.replace(new RegExp("("+this._params.query+"$)", "gi"), "<b class='h-bold'>$1</b>")); 	// No I18N
						break;
					}
				}
			}
		},
		_positionInputElement : function(domElement){
			var offsetCaretPosition, domElementOffset, pos;
			offsetCaretPosition = domElement.offset(), pos = this._getPositon(domElement);
			offsetCaretPosition = {
				left: offsetCaretPosition.left + pos.left - domElement.scrollLeft(),
				top: offsetCaretPosition.top + pos.top - domElement.scrollTop(),
				height: pos.height
			};
			return offsetCaretPosition;
		},
		_removeSpecialCharacters: function(value){
			value = value.replace(/<|>|`|"|&/g, '?').replace(/\r\n|\r|\n/g, "<br/>");	 // No I18N
		    if(/firefox/i.test(navigator.userAgent)){	
		        value = value.replace(/\s/g, '&nbsp;');	 // No I18N
		    }
	     	return value;
		},
		_duplicateElement : function(domElement){
			var simulatedCss;
			this._duplicateDiv = $("<div>");	
			var cssProperties = ["borderBottomWidth", "borderLeftWidth", "borderRightWidth", "borderTopStyle", "borderRightStyle", "borderBottomStyle", "borderLeftStyle", "borderTopWidth", "boxSizing", "fontFamily", "fontSize", "fontWeight", "height", "letterSpacing", "lineHeight", "marginBottom", "marginLeft", "marginRight", "marginTop", "outlineWidth", "overflow", "overflowX", "overflowY", "paddingBottom", "paddingLeft", "paddingRight", "paddingTop", "textAlign", "textOverflow", "textTransform", "whiteSpace", "wordBreak", "wordWrap"];  // No I18N
	        simulatedCss = {
	        	position: 'absolute',	 // No I18N
	        	left: -9999,
	        	top: 0,
	        	zIndex: -20000
	        };
	        for(var i=0; i<cssProperties.length; i++){
	        	simulatedCss[cssProperties[i]] = domElement.css(cssProperties[i]);
	        }
			this._duplicateDiv.css(simulatedCss);
			return this._duplicateDiv;
		},
		_getPositon : function(domElement){
			var pos = domElement[0].selectionStart, startPos = domElement.val().slice(0, pos), endPos = domElement.val().slice(pos), spanElement,spanRelativePosition, elementWithCaret, dimensions;
			spanElement = "<span style='position: relative; display: inline;'>" +this._removeSpecialCharacters(startPos)+ "</span>";	 // No I18N
			spanElement += "<span id='caret' style='position: relative; display: inline;'>|</span>";	 // No I18N
	  		spanElement += "<span style='position: relative; display: inline;'>" + this._removeSpecialCharacters(endPos) + "</span>";	 // No I18N
	  		this._duplicateElement(domElement);
	  		this._duplicateDiv.html(spanElement);
	  		domElement.after(this._duplicateDiv);
	  		elementWithCaret = this._duplicateDiv.find("#caret");	 // No I18N
	 		spanRelativePosition = elementWithCaret.position();
	 		dimensions = {
	 			left: spanRelativePosition.left,
	 			top: spanRelativePosition.top,
	 			height: elementWithCaret.height()
	 		}
	 		this._duplicateDiv.remove() && (this._duplicateDiv = null);
	 		return dimensions;
		},
		_positionEditableDiv :  function(domElement){
			var sel = window.getSelection(), range = sel.getRangeAt(0), rangeCopy, boundingRect, offsetCaretPosition;
			if(sel){
				if(range.endOffset > -1 && range.endContainer != domElement[0]){
					rangeCopy = range.cloneRange();
					rangeCopy.setStart(range.endContainer, range.endOffset - 1);
					rangeCopy.setEnd(range.endContainer, range.endOffset);
					boundingRect = rangeCopy.getBoundingClientRect();
					offsetCaretPosition = {
						height : boundingRect.height,
						left: boundingRect.left,
						top: boundingRect.top
					};
					rangeCopy.detach();
				}
				if(offsetCaretPosition){
					offsetCaretPosition.top += ZC.$window.scrollTop();
					offsetCaretPosition.left += ZC.$window.scrollLeft();
				}
			}
			return offsetCaretPosition;
		},
		_positionMenuAtCursorPosition: function(container){
		    var offsetCaretPosition;
			offsetCaretPosition = container.is("input") ? this._positionInputElement(container) : this._positionEditableDiv(container)	 // No I18N
			return offsetCaretPosition;
		},
		_hideSuggestions: function(){		
			this._menu !== undefined && this._menu.zmenu("hide");	// No I18N
		},
		_displayNoresults: function(text){
			if(!(this.options.search.minKeywordLength === 0 && this._params.query.length === 0)){
				var container = $("<span class='zmenu__text'>").html(text);	//No I18N
				this._menu.zmenu("removeMenuItem", this.componentName+"moreaction"); // No I18N
				this._menu.zmenu("addMenuItem", {	// No I18N
					label: this._params.query,
					index: 0,
					innerHTML: container,
					itemType: "custom",	// No I18N
					id: this.componentName+"noresults",	// No I18N
					itemCssClass : "zsuggestfield__nonselectable"	// No I18N
				})
				this._menu.zmenu("show", {mustBeShown: true, preventHover: true, positionAlterable: true});	// No I18N
				if(this._menu.hasClass("zmenu--iconwithtext")){
					this._menu.removeClass("zmenu--iconwithtext");
				}
			}else{
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}
		},
		_dispatchEvent: function(customEvent, ui, orgEvent){
			var content = {
				'value': ui  	// No I18N
			};	
			var value = this._super(customEvent, orgEvent, content);
			return content.value.modified !== undefined ? content.value : value;
		},
		/** Opens the suggestion list.
			@function suggest-field.openDropdownList
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("openDropdownList");	*/
		openDropdownList: function(){
			this._menu.zmenu("show", {"mustBeShown":true, preventHover: true, positionAlterable: true});	// No I18N
			this._input.attr("aria-expanded", "true");	//No I18N
		},
		/** Closes the suggestion list.
			@function suggest-field.closeDropdownList
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("closeDropdownList");	*/
		closeDropdownList: function(){
			this._hideSuggestions();
		},
		getValue: function(){
			var data = this._currentData ? {} : undefined;
			data = this._getData(this._currentData);
			return data;
		},
		/** Performs search for the specified input passed as the parameter.
			@function suggest-field.search
			@param {Object} value - Query input for which the search need to be carried out.
			@return {object} Returns the list of results obtained for the specified query.
			@example <caption> Example </caption>
	var results = $(".selector").zsuggestfield("search", "a") */
		search: function(query){
			this._searchTimeout(query, this._element);
		},
		/** Closes the suggestion list.
			@function asuggest-field.clearCache
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("clearCache");	*/
		clearCache: function(){
			this._cachedData = {};
		},
		/** Gets the value currently associated with the specified optionName.
			This signature doesn't accept any arguments.
			@function suggest-field.getAttribute
			@param {string} optionName - The name of the option to get.
			@return {string|object} value.
			@example <caption> Example </caption>
	var position = $(".selector").zsuggestfield("getAttribute","placeholder") */
		/** Sets the value of the option associated with the specified optionName.
			@function suggest-field.setAttribute
			@param {string} optionName - The name of the option to get.
			@param {string} value - A value to set for the option.
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("setAttribute","placeholder","Type a name");	*/
		/** Gets an object containing key/value pairs representing the current autosuggest options.
			This signature doesn't accept any arguments.
			@function suggest-field.getAttributes
			@return {object} options object.
			@example <caption> Example </caption>
	var options = $(".selector").zsuggestfield("getAttributes");*/

		/** Sets one or more options for the autocomplete.
			@function suggest-field.setAttributes
			@param {Object} options - A map of option-value pairs to set.
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("getAttributes",{"placeholder": "Type a name","lazyLoadData":true});*/
		setAttribute: function(optionName, value){
			if(typeof value === "object" && !(value instanceof Array)){  // No I18N
				value = $.extend(true, {}, this.options[optionName], value);
			}
			var oldValue = this.options[optionName];
			this.options[optionName] = value;
			if(optionName === 'className'){	//No I18N
				oldValue && this.element.removeClass(oldValue);
				this.element.addClass(value);
			}else if(optionName === "dataSource"){	// No I18N
				if(typeof value === "object" || value instanceof Array){	// No I18N
					this._cachedData = {};
					var oldSource = this._isJSONSource;
					if(value instanceof Array && typeof oldValue === "object"){	// No I18N
					  	this._isJSONSource =  (typeof value[0] === "object");	// No I18N
						this._ajaxSource = false;
					}else if(typeof value === "object" && oldValue instanceof Array || typeof value === "object"){	// No I18N
						this._ajaxSource = this._isJSONSource = true;
						this._requestInitialized = false;
					}
					if(this._isJSONSource && !oldSource){
						this.options.dropdownList.contentType = this._initialContentType;
					}
				}
			}else if(optionName === "autoComplete"){	//No I18N
				if(value){
					var base = this;
					this._menu.on("zmenuitemhover", function(orgEvent, ui){	// No I18N
						base._handleHover(orgEvent, ui);
						base._dispatchEvent("ITEMFOCUS", {value: ui.menuitem});  // No I18N
					})
				}else{
					this._menu.off("zmenuitemhover");	//No I18N
				}
			}else if(optionName === "search"){	// No I18N
				if(typeof value === "object"){	// No I18N
					if(value.minKeywordLength && value.minKeywordLength < 20){
						this.options.search.minKeywordLength = value.minKeywordLength;
					}else{
						this.options.search.minKeywordLength = oldValue.minKeywordLength;
					}	
				}
			}else if(optionName === "placeholder"){	// No I18N
				value ? this.element.attr("placeholder", value) : this.element.removeAttr("placeholder");	// No I18N
			}else if(optionName === "dropdownList" || optionName === "isImageSourceSprited"){	// No I18N
				if((optionName === 'dropdownList' && value.contentType !== undefined || value.itemType === "checkbox") || (optionName === "isImageSourceSprited")){
					this._menu.removeClass("zsuggestfield zmenu--withhint zmenu--withicon zmenu--withiconandhint zmenu--iconwithtext");
					switch(this.options.dropdownList.contentType){
							case "image-text-desc": // No I18N
							case "image-desc": // No I18N
								this._type = "icon-text"; // No I18N
							break;
							case "text":  // No I18N
							case "text-desc": // No I18N
								if(this.options.dropdownList.itemType === "checkbox"){
									this._type = "icon-text";	// No I18N
									this._menu.addClass("zmenu--iconwithtext");
								}else{
									this._type = "text";	// No I18N
								}
							break;
					}
					this._menu.zmenu("option", "contentType", this._type);	// No I18N
					if(this._type === "icon-text" && this.options.dropdownList.contentType !== "text" && this.options.dropdownList.contentType !== "text-desc" && !this.options.isImageSourceSprited){ // No I18N
						this._menu.addClass("zsuggestfield"); // No I18N
						this.options.dropdownList.contentType === "image-text-desc" && this._menu.addClass("zmenu--withhint");	//No I18N
					}
					if(this.options.dropdownList.itemType === "checkbox" && this.options.isImageSourceSprited){		// No I18N
						if(this.options.dropdownList.contentType === "image-text"){	// No I18N
							this._menu.addClass("zmenu--withicon"); // No I18N
						}
						if(this.options.dropdownList.contentType === "image-text-desc"){	// No I18N
							this._menu.addClass("zsuggestfield zmenu--withiconandhint"); // No I18N
						}
					}
				}
				if(value.appendTo !== undefined){
					this._menu.zmenu("option", "appendTo", value.appendTo);	// No I18N
				}
				if(value.position !== undefined){	
					if(value.position !== "after-cursor"){	// No I18N
						this._menu.zmenu("option", "direction", value.position)	// No I18N
					}
				}
				if(value.height !== undefined){
					this._menu.zmenu("option", "height", value.height)	// No I18N
				}
				if(value.width !== undefined){
					this._menu.zmenu("option", "width", value.width)	// No I18N
				}
			}else if(optionName === "disabled"){	// No I18N
				this.element.attr("disabled", value); // No I18N
				this.element[value ? "addClass" : "removeClass"]("is-disabled");	//No I18N
				this.element.focus();
			}else if(optionName ==="dataMapping"){	//No I18N
				this._dataMapping = value;
			}else if(optionName ==="lazyLoadData"){		//No I18N
				(this.options.dropdownList.maxDisplayResults === undefined && this.options.lazyLoadData) && (this.options.dropdownList.maxDisplayResults = this._CONSTANTS.maxResults);
			}
		},
		updateDataSource: function(newData){
			if(this.options.dataSource instanceof Array && newData instanceof Array){
				var len = this.options.dataSource.length;
				for(var i=len, j=0; i<len+(newData.length);i++){
					this.options.dataSource[i] = newData[j++];
				}
			}
		},
		/** Removes autosuggest functionality completely. This will return the element back to its pre-init state.
			This method does not accept any arguments.
			@function suggest-field.destroy
			@example <caption> Example </caption>
	$(".selector").zsuggestfield("destroy"); */
		destroy: function(){
			var componentName = this.componentName;
			this._element.val("");	//No I18N
			this._oldVal = "";  // No I18N
			this._currentData = {};
			this.element.removeClass("zsuggestfield-element");
			this._isLoaderInited = false;
			this.element.removeData(componentName);
			this._element.off("."+componentName);	//No I18N
			this._menu.zmenu("destroy");	//No I18N
			this._menu.remove();
			this.element.show();
		}
	}
	ZC.registerComponent("zsuggestfield", suggestfieldPrototype); // No I18N
})(ZComponents);
/* $Id$ */
(function(ZC){
	var $ = ZC.DOMUtil,
		sortablePrototype = {
		_create: function(element, options){
			this._initialize(element,options);
			this._constructComponent();
		},
		options: {
			'sortableItems': '> *',	// No I18N
			'disabled': false,	// No I18N
			'sortAxis': false,	// No I18N
			'zIndex': 1000,	// No I18N
			'scrollSensitivity': 20,	// No I18N	
			'scrollSpeed': 20,	// No I18N
			'cancel': "input,textarea,button,select,option",	// No I18N
			'leave': null	// No I18N
		},
		_EVENTS: {
			"DRAGSTART": "dragstart",// No I18N
			'MOUSEDRAG': 'drag',	// No I18N
			"DRAGSTOP": "dragstop"	// No I18N
		},
		_initialize: function(element, options){
			this.document = $(this.element[0].ownerDocument);
			this.window = $(this.document[0].defaultView || this.document[0].parentWindow);
		},
		_constructComponent: function(){
			this.saveContainerData = {};
			this.element.addClass("zc-sortable");	// No I18N
			this.refresh();
			this._offsetPos = this.element.offset();
			this._bindMouseEvents();
			this._setHandleClassName();
			this.ready = true;	
		},
		refresh: function(orgEvent){
			this._refreshItems(orgEvent);
			this._setHandleClassName();
			this._refreshPositions();
			return this;
		},
		_handleMouseDown: function(orgEvent){
			var componentName = this.componentName;
			this._dispatchEvent("DRAGSTART", orgEvent.target, orgEvent);  // No I18N
			this._isMouseMoved = false;
			(this._mouseStarted && this._handleMouseUp(orgEvent));
			this._mouseDownEvent = orgEvent;
			var base = this, leftButtonClicked = (orgEvent.which === 1), elementToNeglect = (typeof this.options.cancel === "string" && orgEvent.target.nodeName ? ($(orgEvent.target).closest(this.options.cancel).length) : false); 	// No I18N
			var leaveElem = typeof this.options.leave === "string" ? $(orgEvent.target).hasClass(this.options.leave) ? true : false : false; 	// No I18N
			var isInValidElement = this.options.sortWith ? $(orgEvent.target).closest(this.options.sortWith).length : true;
			if (!isInValidElement || !leftButtonClicked || elementToNeglect || leaveElem || !this._captureMouseAction(orgEvent)) {
				return true;
			}
			if (true === $.data(orgEvent.target, componentName + ".preventClickEvent")) { 	// No I18N
				$.removeData(orgEvent.target, componentName + ".preventClickEvent"); 	// No I18N
			}
			var base = this;
			this.document.on("mousemove." +componentName, function(orgEvent){	// No I18N
				base._handleMouseMove(orgEvent);
			}).on("mouseup." +componentName, function(orgEvent){	// No I18N
				base._handleMouseUp(orgEvent);
			});
			orgEvent.preventDefault();
			return true;
		},
		_handleMouseMove: function(orgEvent){
			this._dispatchEvent("MOUSEDRAG", orgEvent.target, orgEvent);  // No I18N
			if(this._isMouseMoved){
				if(ZC.Browser.isIE() && (!document.documentMode || document.documentMode < 9) && !orgEvent.button) {
					return this._handleMouseUp(orgEvent);
				}
			}
			if(orgEvent.which || orgEvent.button){
				this._isMouseMoved = true;
			}
			if(this._mouseStarted){
				this._onMouseDrag(orgEvent);
				return orgEvent.preventDefault();
			}
			ZC.$document.on("keydown."+this.componentName, this._keydownHandler.bind(this, orgEvent)); // No I18N
			this._mouseStarted = (this._handleMouseMoveStart(this._mouseDownEvent, orgEvent) !== false);
			(this._mouseStarted ? this._onMouseDrag(orgEvent) : this._handleMouseUp(orgEvent));
			return !this._mouseStarted;
		},
		_keydownHandler: function(mouseMoveEvent, orgEvent){
			if(orgEvent.keyCode === ZC.keyCode.ESCAPE){
				this._handleMouseUp(this._mouseDownEvent, true);
			}
			ZC.$document.off("keydown."+this.componentName);  // No I18N
		},
		_dispatchEvent: function(customEvent, ui, orgEvent){
			var content = {
				'value': ui 	// No I18N
			}
			return this._super(customEvent, orgEvent, content);
		},
		_handleMouseMoveStart: function(orgEvent){
			this.currentContainer = this; //revisit
			this._refreshPositions();
			this._draggableElement= this._createDraggableElement();  //revisit
			this._saveDraggableElementSize();
			this._saveMargins();
			this._scrollParent = this._getScrollParent(this._draggableElement);
			this._offsetPos = this._selectedItem.offset();
			this._offsetPos = {
				top: this._offsetPos.top - this._margins.top,
				left: this._offsetPos.left - this._margins.left
			}
			var base = this;
			$.extend(this._offsetPos,{
				click:{
						left: orgEvent.pageX - base._offsetPos.left,
						top: orgEvent.pageY - base._offsetPos.top
					},
				parent: base._getParentOffset(),
				relative: base._getRelativeOffset()
			})
			this._draggableElement.css("position","absolute");	// No I18N
			this._cssPosition = this._draggableElement.css("position");	// No I18N
			this._elementPosition = this._fixPosition(orgEvent);
			this._eventPageX = orgEvent.pageX;
			this._eventPageY = orgEvent.pageY;
			this._formerDomElements = {prev: this._selectedItem.prev()[0], parent: this._selectedItem.parent()[0]};
			this._createPlaceHolder();
			if(this.options.containment){
				this._setContainment();
			}
			if(this.options.zIndex){ //revisit
				if(this._draggableElement.css("zIndex")){	// No I18N
					this._savedZIndex = this._draggableElement.css('zIndex');	// No I18N
				}
				this._draggableElement.css("zIndex", this.options.zIndex);	// No I18N
			}
			if(!this._preserveDraggableElementSize){
				this._saveDraggableElementSize();
			}
			this._draggableElement.addClass("zc-sortable-helper");	// No I18N
			this._onMouseDrag(orgEvent);
			return true;	
		},
		_setContainment: function(){
			if(this.options.containment === "parent"){   // No I18N
				this.options.containment = this._draggableElement[0].parentNode;
			}
			var contaimentElement = $(this.options.containment)[0],
				containmentOffset = $(this.options.containment).offset(),
				isOverFlowHidden = ($(contaimentElement).css("overflow") !== "hidden");  // No I18N 
			this._containment = [containmentOffset.left +  parseInt($(contaimentElement).css("borderLeftWidth")) + parseInt($(contaimentElement).css("paddingLeft")) - this._margins.left,  // No I18N
								containmentOffset.top + parseInt($(contaimentElement).css("borderTopWidth")) + parseInt($(contaimentElement).css("paddingTop")) - this._margins.top,   // No I18N
								containmentOffset.left + (isOverFlowHidden ? Math.max(contaimentElement.scrollWidth, contaimentElement.offsetWidth) : contaimentElement.offsetWidth) - parseInt($(contaimentElement).css("borderLeftWidth")) - parseInt($(contaimentElement).css("paddingRight")) - this._draggableElementSize.width - this._margins.left,  // No I18N
								containmentOffset.top + (isOverFlowHidden ? Math.max(contaimentElement.scrollHeight, contaimentElement.offsetHeight) : contaimentElement.offsetHeight) - parseInt($(contaimentElement).css("borderTopWidth")) - parseInt($(contaimentElement).css("paddingBottom")) - this._draggableElementSize.height - this._margins.top  // No I18N
								];		
		},
		_onMouseDrag: function(orgEvent){
			var scrolled = false, item, intersection, itemElement;
			this._position = this._fixPosition(orgEvent);
			this._changeToAbsolute = this._changePosition("absolute");	// No I18N
			if(!this._lastAbsolutePos){
				this._lastAbsolutePos = this._changeToAbsolute;
			}
			if(orgEvent.pageY - this.document.scrollTop() < this.options.scrollSensitivity) {
				scrolled = this.document.scrollTop(this.document.scrollTop() - this.options.scrollSpeed);
			} else if(this.window.height() - (orgEvent.pageY - this.document.scrollTop()) < this.options.scrollSensitivity) {
				scrolled = this.document.scrollTop(this.document.scrollTop() + this.options.scrollSpeed);
			}
			if(orgEvent.pageX - this.document.scrollLeft() < this.options.scrollSensitivity) {
				scrolled = this.document.scrollLeft(this.document.scrollLeft() - this.options.scrollSpeed);
			}else if(this.window.width() - (orgEvent.pageX - this.document.scrollLeft()) < this.options.scrollSensitivity) {
				scrolled = this.document.scrollLeft(this.document.scrollLeft() + this.options.scrollSpeed);
			}
			this._changeToAbsolute = this._changePosition("absolute");	// No I18N
			if(!this.options.axis || this.options.axis !== "y"){	// No I18N
				this._draggableElement[0].style.left = this._position.left+"px";	// No I18N
			}
			if(!this.options.axis || this.options.axis !== "x"){	// No I18N
				this._draggableElement[0].style.top = this._position.top+"px";	// No I18N
			}
			for(var i= this.items.length - 1;i>=0;i--){
				item = this.items[i];
				itemElement = item.item[0];
				intersection = this._intersectsWithPointer(item);
				if(!intersection){
					continue;
				}
				if(itemElement !==  this._selectedItem[0] && this._placeHolder[intersection === 1 ? "next" : "prev"]()[0] !== itemElement && !$.contains(this._placeHolder[0], itemElement)){	// No I18N
					this._direction = intersection === 1 ? "down" : "up";	// No I18N
					if(this._intersectsWithSides(item) && item.item[0].nodeName !== "INPUT"){	// No I18N
						this._reArrangeItems(orgEvent, item);
					}else{
						break;
					}
					break;
				}
			}
			this._lastAbsolutePos = this._changeToAbsolute;
			return false;	
		},
		_reArrangeItems: function(orgEvent, item, appendTo, hardRefresh){
			if(appendTo){
				appendTo[0].appendChild(this._placeHolder[0]);
			}else{
				var position;
				if(this._direction === "down"){
					position = item.item[0];
				}else{
					position = item.item[0].nextSibling;
				}
				item.item[0].parentNode.insertBefore(this._placeHolder[0], position);
			}
			this._counter = this._counter ? ++this._counter : 1;
			var counter = this._counter, base = this;
			setTimeout(function(){
				base._refreshPositions(!hardRefresh);
			},0);
		},
		_intersectsWithSides: function(item){
			var isOverBottomHalf = this._isOverAxis(this._changeToAbsolute.top + this._offsetPos.click.top, item.top + (item.height/2), item.height),
				isOverRightHalf = this._isOverAxis(this._changeToAbsolute.left + this._offsetPos.click.left, item.left + (item.width/2), item.width),
				verticalDirection = this._getDragVerticalDirection(),
				horizontalDirection = this._getDragHorizontalDirection(), 
				intersectsContainment = false;
			if(!horizontalDirection && !verticalDirection){
				return false;	
			} 
			if(this.floating && horizontalDirection){
				if(this._containment){
					intersectsContainment = ((this._changeToAbsolute.left <= this._containment[0]) || (this._changeToAbsolute.left >= this._containment[2]));	
				}  
	            var value = intersectsContainment || ((horizontalDirection == "right" && isOverRightHalf) || (horizontalDirection == "left" && !isOverRightHalf));  // No I18N
			}else{
				if (this._containment){
					 intersectsContainment = ((this._changeToAbsolute.top <= this._containment[1]) || (this._changeToAbsolute.top >= this._containment[3]));	
				}
	            var value = intersectsContainment || (verticalDirection && ((verticalDirection == "down" && isOverBottomHalf) || (verticalDirection == "up" && !isOverBottomHalf)));  // No I18N
				return value;
			}
		},
		_intersectsWithPointer: function(item){
			var isOnElementHeight = (this.options.axis === "x") || this._isOverAxis(this._changeToAbsolute.top + this._offsetPos.click.top, item.top, item.height, "x"),	// No I18N
				isOnElementWidth = (this.options.axis === "y") || this._isOverAxis(this._changeToAbsolute.left + this._offsetPos.click.left, item.left, item.width, "y"),	// No I18N
				intersectsContainmentL = false,
				intersectsContainmentT = false,
				isOverElement = isOnElementHeight && isOnElementWidth,
				verticalDirection = this._getDragVerticalDirection(),
				horizontalDirection = this._getDragHorizontalDirection();
				if(!isOverElement){
					return false;
				}
				return this.floating ?
				(((horizontalDirection && horizontalDirection === "right") || verticalDirection === "down") ? 2 : 1)	// No I18N
				: (verticalDirection && (verticalDirection === "down" ? 2 : 1));	// No I18N
		},
		_isOverAxis: function(x, reference, size, axis){
			var value = axis === "x" ? this._selectedItem.height() : 0;  // No I18N
			return ((x + value) >= reference) && (x < (reference + size ));
		},
		_getDragVerticalDirection: function(){
			var delta = this._changeToAbsolute.top - this._lastAbsolutePos.top;
			return delta !== 0 && (delta > 0 ? "down" : "up");	// No I18N
		},
		_getDragHorizontalDirection: function(){
			var delta = this._changeToAbsolute.top - this._lastAbsolutePos.top;
			return delta !== 0 && (delta > 0 ? "right" : "left");	// No I18N
		},
		_changePosition: function(pos){
			var mod = pos === "absolute" ? 1 : -1, scroll = this._cssPosition === "absolute" && !(this._scrollParent[0] !== this.document[0] && $.contains(this._scrollParent[0],this._offsetParent[0])) ? this._offsetParent : this._scrollParent, isScrollRootNode;	// No I18N
			if(scroll[0].tagName.toLowerCase() === "html" || scroll[0].tagName.toLowerCase() === "body"){	// No I18N
				isScrollRootNode = true;
			}
			return {
				top: (this._position.top + this._offsetPos.relative.top * mod + this._offsetPos.parent.top*mod - ((this._cssPosition === "fixed" ? - this._scrollParent.scrollTop() : (isScrollRootNode ? 0 : scroll.scrollTop())) * mod)),	// No I18N
				left: (this._position.left + this._offsetPos.relative.left * mod + this._offsetPos.parent.left*mod - ((this._cssPosition === "fixed" ? - this._scrollParent.scrollLeft() : (isScrollRootNode ? 0 : scroll.scrollLeft())) * mod))	// No I18N
			};
		},
		_createPlaceHolder: function(){
			var selectedNodeName = this._selectedItem[0].nodeName.toLowerCase();
			this._placeHolder = $("<"+ selectedNodeName + ">", this.document[0]).addClass(this._selectedItem[0].className+" zc-sortable-placeholder");	// No I18N
			this._placeHolder.css("visibility","hidden");	// No I18N
			this._selectedItem.after(this._placeHolder);
			if(!this._placeHolder.width()) { 
				this._placeHolder.width(this._selectedItem.innerWidth() - parseInt(this._selectedItem.css("paddingLeft")||0, 10) - parseInt(this._selectedItem.css("paddingRight")||0, 10)); 	// No I18N
			}
			if(!this._placeHolder.height()){
				this._placeHolder.height(this._selectedItem.height());
			}
		},
		_fixPosition: function(orgEvent){
			var top, left, pageX = orgEvent.pageX, pageY = orgEvent.pageY, scroll = this._cssPosition === "absolute" && !(this._scrollParent[0] !== this.document[0] && $.contains(this._scrollParent[0],this._offsetParent[0])) ? this._offsetParent : this._scrollParent, isScrollRootNode;	// No I18N
			if(this._containment){
				if(orgEvent.pageX - this._offsetPos.click.left < this._containment[0]){
					pageX = this._containment[0] + this._offsetPos.click.left;
				}
				if(orgEvent.pageY - this._offsetPos.click.top < this._containment[1]){
					pageY = this._containment[1] + this._offsetPos.click.top;
				}
				if(orgEvent.pageX - this._offsetPos.click.left > this._containment[2]){
					pageX = this._containment[2] + this._offsetPos.click.left;
				}
				if(orgEvent.pageY - this._offsetPos.click.top > this._containment[3]){
					pageY = this._containment[3] + this._offsetPos.click.top;
				}
			}
			if(scroll[0].tagName.toLowerCase() === "html" || scroll[0].tagName.toLowerCase() === "body"){	// No I18N
				isScrollRootNode = true;
			}
			return {
				top: pageY - this._offsetPos.click.top - this._offsetPos.relative.top - this._offsetPos.parent.top + (this._cssPosition === "fixed" ? - this._scrollParent.scrollTop() : (isScrollRootNode ? 0 : scroll.scrollTop())),	// No I18N
				left: pageX - this._offsetPos.click.left - this._offsetPos.relative.left - this._offsetPos.parent.left + (this._cssPosition === "fixed" ? - this._scrollParent.scrollLeft() : (isScrollRootNode ? 0 : scroll.scrollTop()))	// No I18N
			}
		},
		_getScrollParent: function(element, includeHidden){
			var position = element.css("position"),	// No I18N
			excludeStaticParent = position === "absolute",  // No I18N	
			overflowRegex = includeHidden ? /(auto|scroll|hidden)/ : /(auto|scroll)/,	
			scrollParent = element.parents().filter(function(){
				var parent = $(this);
				if(excludeStaticParent && parent.css("position") === "static"){	// No I18N
					return false;
				}
				return overflowRegex.test( parent.css( "overflow" ) + parent.css("overflow-y") + parent.css("overflow-x"));	// No I18N
			}).eq(0);
			return position === "fixed" || !scrollParent.length ? $(element[ 0 ].ownerDocument || document) : scrollParent;	// No I18N
		},
		_getParentOffset: function(){
			this._offsetParent = this._draggableElement.offsetParent();
			var po = this._offsetParent.offset();
			if(this.cssPosition === "absolute" && this._scrollParent[0] !== this.document[0] && $.contains(this._scrollParent[0], this._offsetParent[0])) {	// No I18N
				po.left += this._scrollParent.scrollLeft();
				po.top += this._scrollParent.scrollTop();
			}
			if(this._offsetParent[0] === this.document[0].body || (this._offsetParent[0].tagName && this._offsetParent[0].tagName.toLowerCase() === "html" && ZC.Browser.isIE())) {	// No I18N
				po = { top: 0, left: 0 };
			}
			return {
				top: po.top + (parseInt(this._offsetParent.css("borderTopWidth"),10) || 0),	// No I18N
				left: po.left + (parseInt(this._offsetParent.css("borderLeftWidth"),10) || 0)	// No I18N
			};
		},
		_getRelativeOffset: function(){
			if(this.cssPosition === "relative"){	// No I18N
				var p = this._selectedItem.position();
				return {
					top: p.top - (parseInt(this._draggableElement.css("top"),10) || 0) + this._scrollParent.scrollTop(),	// No I18N
					left: p.left - (parseInt(this._draggableElement.css("left"),10) || 0) + this._scrollParent.scrollLeft()	// No I18N
				};
			}else {
				return { top: 0, left: 0 };
			}
		},
		_saveMargins: function(){
			this._margins = {
				left: (parseInt(this._selectedItem.css("marginLeft"),10) || 0),	// No I18N
				top: (parseInt(this._selectedItem.css("marginTop"),10) || 0)	// No I18N
			}
		},
		_createDraggableElement: function(orgEvent){
			var draggableItem = this._selectedItem;
			this._savedCSS = {width: this._selectedItem[0].style.width, height: this._selectedItem[0].style.height, position: this._selectedItem.css("position"), top: this._selectedItem.css("top"), left: this._selectedItem.css("left")};	// No I18N
			draggableItem.width(this._selectedItem.width()).height(this._selectedItem.height());
			return draggableItem;
		},
		_saveDraggableElementSize: function(){
			this._draggableElementSize = {
				width: this._draggableElement.outerWidth(),
				height: this._draggableElement.outerHeight()
			};
		},
		_captureMouseAction: function(orgEvent){
			var _selectedItem = null, base = this;
			this._isPrev = false;
			if(this.reverting){	// revisit
				return false;
			}
			if(this.options.disabled){
				return false;
			}
			this._refreshItems(orgEvent);
			$(orgEvent.target).parents().each(function(){
				if($.data(this,base.componentName+"-item") === base){	// No I18N
					_selectedItem = $(this);
					return false;
				}
			})
			if($.data(orgEvent.target, this.componentName+"-item") === base){	// No I18N
				_selectedItem = $(orgEvent.target);
			}
			if(!_selectedItem){
				return false;
			}
			this._selectedItem = _selectedItem;
			if(this._selectedItem.prev().length){
				this._adjacentElement = this._selectedItem.prev();
				this._isPrev = true;
			}else{
				this._adjacentElement = this._selectedItem.next()
			}
			//this._selectedItem.css("cursor","move");	
			return true;
		},
		_handleMouseUp: function(orgEvent, escKeyPressed){
			var componentName = this.componentName;
			this.document.off("mousemove."+componentName);	// No I18N
			this.document.off("mouseup."+componentName);	// No I18N
			if(this._mouseStarted){
				this._mouseStarted = false;
				if (orgEvent.target === this._mouseDownEvent.target) {
					$.data(orgEvent.target, componentName + ".preventClickEvent", true);	// No I18N
				}
				this._dropDraggedElement(orgEvent, escKeyPressed);
			}
			return false;
		},
		_dropDraggedElement: function(orgEvent, escKeyPressed){
			if(!orgEvent){
				return;
			}
			this._placeElement(orgEvent, escKeyPressed);
		},
		_placeElement: function(orgEvent, escKeyPressed){
			var i;
			this.reverting = false;
			if(escKeyPressed){
				if(this._isPrev){
					this._selectedItem.insertAfter(this._adjacentElement);
				}else{
					this._selectedItem.insertBefore(this._adjacentElement);
				}
			}else{
				if(!this._noFinalSort && this._selectedItem.parent().length){
					this._placeHolder.before(this._selectedItem);
				}	
			}
			
			this._noFinalSort = null;
			if(this._draggableElement[0] === this._selectedItem[0]) {
				for(i in this._savedCSS) {
					if(this._savedCSS[i] === "auto" || this._savedCSS[i] === "static") {	// No I18N
						this._savedCSS[i] = "";	// No I18N
					}
				}
				this._selectedItem.css(this._savedCSS).removeClass("zc-sortable-helper");	// No I18N
			}else {
				this._selectedItem.show();
			}
			if(this._savedZIndex) {
				this._draggableElement.css("zIndex", this._savedZIndex === "auto" ? "" : this._savedZIndex);	// No I18N
			}
			this.dragging = false; // revisit
			this._placeHolder[0].parentNode.removeChild(this._placeHolder[0]);
			this.fromOutside = false;
			this._dispatchEvent("DRAGSTOP", orgEvent.target, orgEvent);  // No I18N
			return !this.cancelHelperRemoval;
		},
		_bindMouseEvents: function(){
			var base = this, componentName = this.componentName;
			this.element.on("mousedown."+componentName, function(orgEvent){	// No I18N
				return base._handleMouseDown(orgEvent);
			})
			this.element.on("click."+componentName, function(orgEvent){	// No I18N
				if($.data(orgEvent.target, componentName+".preventClickEvent") ===  true){	// No I18N
					$.removeData(orgEvent.target, componentName+".preventClickEvent");	// No I18N
					orgEvent.stopImmediatePropagation();
					return false;
				}
			})
			this.started = false; // Revisit
		},
		_refreshItems: function(orgEvent){
			this.items = [];
			this.containers = [this];
			var targetData, item, query, queries = [[$(this.options.sortableItems, this.element), this]];
			for(var i=queries.length - 1;i>=0; i--){
				targetData = queries[i][1];
				query = queries[i][0];
				for(var j=0; j<query.length;j++){
					item = $(query[j]);
					item.data(this.componentName+"-item", targetData);	// No I18N
					this.items.push({
						item: item,
						width: 0, height: 0,
						left: 0, top: 0
					});
				}
			}
		},
		_setHandleClassName: function(){
			this.element.find('.zc-sortable-handle').removeClass('zc-sortable-handle');	// No I18N
			for(var i=0;i< this.items.length; i++){
				var elementsToNeglect = this.options.cancel.split(",");	// No I18N
				var elementPosition = elementsToNeglect.indexOf(this.items[i].item[0].nodeName.toLowerCase());
				if(elementPosition == -1){
					(this.items[i].item).addClass('zc-sortable-handle');	// No I18N
				}
			}
		},
		_refreshPositions: function(fast){
			this.floating = this.items.length ? (this.options.sortAxis === 'x' || this._isFloating(this.items[0].item)) : false;	// No I18N
			var item, itemSize, offsetPos;
			for(var i=this.items.length - 1; i>=0; i--){
				item = this.items[i];
				itemSize = item.item;
				if (!fast) {
					item.width = itemSize.outerWidth();
					item.height = itemSize.outerHeight();
				}
				offsetPos = itemSize.offset();
				item.left = offsetPos.left;
				item.top = offsetPos.top;
			}
			for(var i= this.containers.length - 1;i>=0;i--){
				offsetPos = this.containers[i].element.offset();
				this.containers[i].saveContainerData.left = offsetPos.left;
				this.containers[i].saveContainerData.top = offsetPos.top;
				this.containers[i].saveContainerData.width = this.containers[i].element.outerWidth();
				this.containers[i].saveContainerData.height = this.containers[i].element.outerHeight();
			}
			return this;
		},
		_isFloating: function(item){
			return (item.css('float').toLowerCase() === "left" || item.css('float').toLowerCase() === "right" || item.css('display').toLowerCase() === "table-cell" || item.css("display").toLowerCase() === "inline");	// No I18N
		},
		destroy: function(){
			var componentName = this.componentName;
			this.element.removeData(componentName);	
			this.element.off("mousedown."+componentName).off("click."+componentName);	// No I18N
			this.document.off("mousemove." +componentName).off("mouseup."+componentName);	// No I18N
		},
		disable: function(){
			var componentName = this.componentName;
			this.element.off("mousedown."+componentName).off("click."+componentName);	// No I18N
			this.document.off("mousemove." +componentName).off("mouseup."+componentName);	// No I18N
		},
		enable: function(){
			this._bindMouseEvents();
		}
	}
	ZC.registerComponent("zsortable", sortablePrototype); // No I18N
})(ZComponents);
/* $Id$ */
(function(ZC){
	/** @components token-field - Enables the user to make multiple selections by typing inside a textfield. A token will be added for each selected value. All options of Suggest field component can be accessed here.
	    @eventPrefix ztokenfield 
	*/
	var $ = ZC.DOMUtil,
		tokenfieldPrototype = {
		options: {
			/** @member {object} 
			    [token-field.dataMapping] - The mapping for the JSON Data passed should be given here. This field is mandatory for both JSON and API input. The name of the field in the input data should be specified for each of the fields provided below.
		        @example <caption> Example: </caption>  

Data mapping for the sample JSON below:

    object = {
        "id": 3167891,
        "name": {
            "first_name": "Edward",
            "last_name": "Cullen"
        },
        "email": "edward@gmai.com",
        "avatarURL": "http://www.github.com?page=1&q=edward",
        "department": "Training",
        "gender": "Male",
        "country": "India"
    }

    dataMapping: {
        value: "id",

        // When the value should be retrieved from a nested object. The value should be seperated by "." operator
        text: "name.first_name", 

        itemIdentifier: "email", 

       	informativeText: "email", (optional)

        image: "avatarURL", (optional, only if needed)
        //groupLabel should be used when the user wants a group header for the individual values.

        groupLabel: "department", (optional, only if needed)

        titleText: "email", (optional, only if needed)

        customValues: "gender" (optional, only if needed)
    }			
			*/
			/** @member {string}
				[token-field.dataMapping.titleText] - If the user wishes to have title text for the individual tokens, this value has to be provided. */
			/** @member {string}
				[token-field.dataMapping.itemIdentifier] - The value user provide here will be used for retrieving and removing individual tokens. */
			/** @member {string}
				[token-field.dataMapping.customValues] - If the user has any additional info that should be passed when the selected values are obtained through API. Then in such cases, the JSON key should be passed to the customValues key */
			/** @member {Object} 
			    [token-field.dropdownList] - Dropdown related options are specified here.	*/
			'dropdownList': {	//No I18N
				/** @member {Boolean} 
			    [token-field.dropdownList.itemType] - Indicates whether checkbox kind of menu need to be enabled. 
				@allowedValues 	{("checkbox"|"none")} */				
				"itemType": "none",	// No I18N
				/** @member {Boolean}
					[token-field.dropdownList.closeOnSelect = true] - Whether menu need to be closed once data/value is selected. */
				'closeOnSelect': true	// No I18N
			},
			/** @member {Boolean}
			[token-field.allowDuplicateValues = false] -  Indicates whether duplicate tokens can be included in the token field container. */
			'allowDuplicateValues': false,	// No I18N
			/** @member {Boolean} 
		    [token-field.dataValidation = false] - Performs data validation on the typed input keyword. If the data is invalid, error token is added for the typed input rather than normal token. */
			'dataValidation': false,	// No I18N
			/** @member {Object} 
			    [token-field.tokens] - Token related options are specified here.	*/
			'tokens': {	// No I18N
				/** @member {String} 
			        [token-field.tokens.contentType] - The type of token that need to be created for each selection.
			        @allowedValues {("text"|"image-text")} */
				'contentType': 'text',	// No I18N
				/** @member {String} 
			        [token-field.tokens.removeSVGIconId] - The default SVG Icon provided by the component for token's close icon can be changed. ID of the user's SVG element should be provided. 
					@newlyAdded true */
				"removeSVGIconId": undefined, // No I18N
				/** @member {String} 
			        [token-field.tokens.removeIconClassName] - Sprite CSS class can also be provided for the token's close icon.  
			        @newlyAdded true
			    */
				"removeIconClassName": undefined, // No I18N
				/** @member {Boolean} 
			        [token-field.tokens.sortable=false] - Indicates whether the tokens can have drag and drop functionality */
				'sortable': false,	// No I18N
				/** @member {Boolean} 
			        [token-field.tokens.editable = false] - Indicates whether token value can be edited and updated */
				'editable': false,	// No I18N
				/** @member {String} 
			        [token-field.tokens.editable.pattern = '"#value#" <#id#>'] - The pattern that is used to render the text when a user tries to edit the tokens  */
				/** @member {Boolean} 
					@deprecated true
			        [token-field.tokens.animateOnClose = false] - Denotes whether animation need to be provided for token removal. tokens.animateOnRemove is the replaceement for this option. */
				/** @member {Boolean} 
			        [token-field.tokens.animateOnRemove = false] - Denotes whether animation need to be provided for token removal. 
			        @newlyAdded true
			    */
				'animateOnRemove': false,	// No I18N
				/**	@member {Boolean}
					[token-field.tokens.showCloseButtonOnHover = false] - Denotes whether the remove button on the token should be there by default or should appear as user hovers the token. tokens.showRemoveButtonOnHover is the replacement for this option 
					@deprecated true
				*/
				/**	@member {Boolean}
					[token-field.tokens.showRemoveButtonOnHover = false] - Denotes whether the remove button on the token should be there by default or should appear as user hovers the token. 
					@newlyAdded true
				*/
				'showRemoveButtonOnHover': false	// No I18N
			},
			/** @member {Character}
				[token-field.delimiter = ','] - The delimiter to use for seperating the typed input. */
			'delimiter' : ',',		// No I18N
			/** @member {Number} 
		        [token-field.maxAllowedValues] - Maximum Number of allowed tokens on the tokenfield container. */
			'maxAllowedValues': undefined,	// No I18N
			/**	@member {Array} 
				[token-field.values = []] - Indicates the set of selected data/tokens that appears in the tokenfield. */
			'values': [],	// No I18N
			/** @member {Boolean}
				[token-field.acceptNewValues = false] - Indicates whether the user typed values should be added as a token if the value is not available in the suggestions list. */
			'acceptNewValues': false,	// No I18N
			/** @member {regExp}
				[token-field.newValuePattern = false] - "acceptNewValues" has to be set to true for this option to be effective. A valid regular expression object should be passed to indicate the pattern of input to be accepted. 
				@example <caption> Example </caption>	  
Following Snippet shows how regex should be provided as an option   
	                                    
newValuePattern: new RegExp(/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/) */
			'newValuePattern': undefined,	//No I18N
			/** @member {Object}
				[token-field.messages] - Default text messages */
			'messages': {	// No I18N
				/**  @member {String}
					[token-field.messages.close = 'Close'] - Title text for close button on token. messages.remove is the replacement for this option 
					@deprecated true
				*/
				/** @member {String}
					[token-field.messages.remove = 'Remove'] - Title text for remove button on token 
					@newlyAdded true
				*/
				'remove': 'Remove',	// No I18N
				'maxTokenLimitReached': "Max token limit reached"	// No I18N
			}
		},
		_TOKENFIELDEVENTS: {
			/** Triggered before the token is added.
				@event token-field.beforetokenadd
				@param {Object} event eventObject 
				@param {Object} customObject containing information about the token being added. 
				@example <caption> Example </caption>

    $("#input1").on("ztokenfieldbeforetokenadd", function(orgEvent, ui){
        console.log(ui.value);
    })
		
    Events can be listened through callbacks 

    $("#input1").ztokenfield({
        dataSource: sourceData, 
        search: {
            by: "name"
        },
        dataMapping: {
            "value": "id",
            "text": "value",
            "informativeText": "email",
            "image": "imageURL"
        },
        beforetokenadd: function(orgEvent, ui){
            console.log(ui.value);
        }
    })
			*/
			"BEFORETOKENADD" : "beforetokenadd",	// No I18N
			/** Triggered before the token is removed.
				@event token-field.beforetokenremove
				@param {Object} event eventObject
				@param {Object} customObject containing information about the token being removed. */
			"BEFORETOKENREMOVE": "beforetokenremove",	// No I18N
			/** Triggered after the token is added.
				@event token-field.tokenadd
				@param {Object} event eventObject
				@param {Object} customObject containing information about the token that is added. */
			"AFTERTOKENADD": "tokenadd",	// No I18N
			/** Triggered after the token is removed.
				@event token-field.tokenremove
				@param {Object} event eventObject
				@param {Object} customObject containing information about the token that is removed. */
			"AFTERTOKENREMOVE": "tokenremove",	// No I18N
			/** Triggered when a editable token's value change is submitted or aborted.
				@event token-field.tokenedit
				@param {Object} event eventObject
				@param {Object} customObject containing information about the editable token and the associated value in the token. 
				@example <caption> Example </caption>
	Below example shows how to validate the edited value to a EMAIL ID

	$(<selector>).on("ztokenfieldtokenedit", function(orgEvent, ui){
		var value = ui.value.textboxValue, isInvalid;
		//This value has to be set to true for the modification to be effective on the token
		ui.value.modified = true;
		var name = value.match(/\"(.*)\"/) && value.match(/\"(.*)\"/).pop();
		var value = value.match(/\<(.*)\>/) && value.match(/\<(.*)\>/).pop();
		if(name === null && value === null){
			name = ui.value.textboxValue;
			value = ui.value.textboxValue;
		}
		var regexp = new RegExp(/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/);
		if(ui.value.modified){
			if(value && value.length && !(regexp.test(value))){
				ui.value.itemContent.invalidToken = true;
				isInvalid = true;
			}
			if(!isInvalid){
				if(value !== null){
					if(ui.value.itemContent.invalidToken){
						ui.value.itemContent.itemValue = {};
					}
					ui.value.itemContent["email"] = value;	// No I18N
					ui.value.itemContent["email"] = value;	// No I18N
					ui.value.itemContent["name"] = name ? name : value;	// No I18N
					ui.value.itemContent.invalidToken = false;
				}else{
					ui.value.itemContent.invalidToken = true;
				}
			}	
		}	
	})
			*/
			"TOKENUPDATE": "tokenedit",	// No I18N
			"TOKENDRAG": "tokendrag",	// No I18N
			/** Triggered when token addition fails.
				@event token-field.tokenaddfail
				@param {Object} event eventObject
				@param {Object} customObject containing information about the reason for failure. The failure information is available in a variable called "failureType" */
			"TOKENADDFAIL": "tokenaddfail"	// No I18N	
		},
		_CONSTANTS: {
			'pattern': '"#value#" <#id#>',	// No I18N
			'patterFormatError': "Invalid pattern for token. Pattern should contain #id# or #value# or both",	// No I18N
			'minKeywordLength': 20,	// No I18N
			'maxResults': 10// No I18N
		},
		_create: function(element, options){
			//this._super();
			this._keyCode = ZC.keyCode;
			this._extendEvents();
			this._constructContainer();
			this._initialize();
			this._initSource();	
			this._tokensCollection = [], this._performedStack = [], this._recallStack = [], this._tokensCount = 0, this._cachedData = {};
			!this.options.disabled && this._bindEvents();
			this._uniqueIndex = 0;
			!this.options.disabled && this._bindMenuEvents();
			this.options.isResponsePaginated && (this._params = {pages: undefined});
			if(this.options.values && this.options.values.length){
				this._defaultSelected = true;
				this.addTokens(this.options.values);
				this._defaultSelected = false;
			}
			if(this.options.disabled){
				this.element.attr("disabled", true);	//No I18N
				this.container.addClass("is-disabled");	//No I18N
				this.container.attr("aria-disabled", "true");		//No I18N	
				this._input.attr("disabled", true);	//No I18N
			}
		},
		_constructContainer: function(){
			this._elementId = (this.element.attr("id") ? this.element.attr("id") : Math.floor(Math.random() * 1000000))+"-"+this.componentName;  // No I18N
			this.container = $("<div id='"+this._elementId+"-container' tabindex='-1'>").addClass("ztokenfield");	// No I18N
            this._input = $("<input type='text' role='textbox' aria-haspopup='true' role='textbox' aria-expanded='false' aria-owns='"+this._elementId+"-menu' id='"+this._elementId+"-input' autocomplete='off' tabindex='"+(this.element.attr("tabindex") || 0)+"'>").addClass('ztokenfield__textbox'); // No I18N
			this._input.appendTo(this.container).blur();
			this.container.insertAfter(this.element);
			this.options.className && this.container.addClass(this.options.className);
			this._pasteHelperTextBox = $("<input type='text' style='position:absolute; left: -10000000px' tabindex='-1' id='"+this._elementId+"-copyhelper'>").appendTo(this.container);	// No I18N
			this._pasteRegExp = this._CONSTANTS.pattern;	
			if(this.options.placeholder){	
				this._resizeSearchArea(this._input, this.options.placeholder);	
				this._input.attr("placeholder", this.options.placeholder);	// No I18N	
			}
			this.options.dropdownList.forElement = this.container;	
			!this.options.tokens.showRemoveButtonOnHover && this.container.addClass("ztokenfield--closeshown");		// No I18N
			this._prevContainerHeight = this.container.css("height");	// No I18N
			this.element.hide();
		},
	    _resizeSearchArea: function(textbox, value){
			var duplicateDiv, containerWidth, properties, inputWidth, styleBlock, style, width;
			//Simulate the textbox to calculate width
			styleBlock = "position:absolute; left: -1000px; top: -1000px; display:none;";	// No I18N
			properties = ['font-size', 'font-style', 'font-weight', 'font-family', 'line-height', 'text-transform', 'letter-spacing'];	// No I18N
			for(var i=0; i <properties.length; i++){
				style = properties[i];
				styleBlock += style + ":" + textbox.css(style) + ";"; 	// No I18N
			}
			duplicateDiv =$('<div />', {'style': styleBlock }); 	
			duplicateDiv.text(value !== undefined ? value : textbox.val());
			$('body').append(duplicateDiv);
			var inputWidth;
			containerWidth = this.container.outerWidth();

			if(textbox.attr("id") === "dblclick-input-"+this.componentName){	// No I18N
				inputWidth = duplicateDiv.width() + 30;
				width = inputWidth;
			}else{
				inputWidth = duplicateDiv.width() + 50;
				duplicateDiv.text(textbox.val());
				var inputWidth1 = duplicateDiv.width() + 50;
				duplicateDiv.remove();
				width = inputWidth > inputWidth1 ? inputWidth : inputWidth1; //Checking whether input width is greater or placeholder 
			}			
			if(this.container.is(":visible") && width > (containerWidth - 10)){	// No I18N
				width = containerWidth - 10;
			}
			return textbox.css({'width' : width + 'px'}); 	// No I18N
		},
		_activateInputArea: function(){
			this._inputArea.val(this._inputArea.val());
			this.container.addClass('has-focus');	// No I18N
		},
		_addCustomCloseIcon: function(){
			this._closeButton = $('<button class="ztokenfield__close" tabindex="-1" title='+this._localize('remove')+'></button>');		// No I18N
			var iconElement = $('<i class="ztokenfield__icon"></i>'), tokens = this.options.tokens, svgProps;  // No I18N
			svgProps = {
				props: {"viewBox":"0 0 16 16", "class": "ztokenfield__svg", "width": "100%", "height": "100%"},  // No I18N
				nodes: [
						{"rect": {"x":"1.5","y":"7.4","transform":"matrix(0.7071 -0.7071 0.7071 0.7071 -3.3137 8)","class":"st0","width":"13","height":"1.1"}},  // No I18N
						{"rect": {"x":"1.5","y":"7.4","transform":"matrix(0.7071 0.7071 -0.7071 0.7071 8 -3.3137)","class":"st0","width":"13","height":"1.1"}} // No I18N
					]
			};
			this._handleIcon(iconElement, svgProps, "closebutton", tokens.removeIconClassName, tokens.removeSVGIconId);  // No I18N
			iconElement.appendTo(this._closeButton);
		},
		_addErrorToken: function(value, orgEvent, index, tokenId){
			value = value.trim();
			if(value.length){
				var textbox = this._editTextBox !== undefined ? this._editTextBox : this._input;	
				this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>");	
				this._addCustomCloseIcon();
				this._tokenText = $("<span class ='ztokenfield__text' title='"+value+"'>").text(value);	// No I18N
				this._tokenContainer.addClass("has-error").data("invalidToken", true);	// No I18N
				this._insertToken({"errorValue": value}, textbox, orgEvent, index, tokenId);	// No I18N
				textbox.val("");	// No I18N
			}			
		},
		_checkTokenChange: function(orgEvent){
			if(this._tokensCollection.length > 0 && this._tokensCollection.length === this._lastTokenUpdate.length){
				var current = this._tokensCollection, prev = this._lastTokenUpdate, flag = false;
				for(var i=0; i<current.length;i++){
					for(var j=0; j<prev.length;j++){
						var currentData = $(current[i]).data().metadata, currentKey, prevKey;
						if(currentData !== undefined){
							currentKey = this._mapJSONToData(currentData, this._dataMapping.itemIdentifier);
						}
						var prevData = prev[j];
						if(prevData !== undefined){
							prevKey = this._mapJSONToData(prevData, this._dataMapping.itemIdentifier);
						}
						if(currentKey !== prevKey){
							flag = false;
						}else{
							flag = true;
							break;
						}
					}
				}
				if(!flag){
					var token = {prevTokenList: prev, currentTokenList: current};
					this._dispatchEvent("CHANGE", token, orgEvent);  // No I18N
					this._lastTokenUpdate = [];
					for(var i=0; i<this._tokensCollection.length; i++){
						this._lastTokenUpdate.push(this._tokensCollection[i].data().metadata);
					}
				}
			}else{
				if(this._tokensCollection.length !== this._lastTokenUpdate.length ){
					var token = {prevTokenList: this._lastTokenUpdate, currentTokenList: this._tokensCollection};
					this._dispatchEvent("CHANGE", token, orgEvent);  // No I18N
					this._lastTokenUpdate = [];
					for(var i=0; i<this._tokensCollection.length; i++){
						this._lastTokenUpdate.push(this._tokensCollection[i]);
					}
				}
			}				
		},
		_checkIsContentValid: function(value, keyCode){
			var acceptNewValues = this.options.acceptNewValues,
				pattern = this.options.newValuePattern;
			if(value !== undefined && value.length && acceptNewValues){
				var tokenJSON = {};
				tokenJSON.itemValue = {};
				if(pattern !== undefined && pattern instanceof RegExp && pattern.test(value) || pattern === undefined){
					tokenJSON.itemValue.text = value;
					tokenJSON.itemValue.itemIdentifier = value;
					tokenJSON.itemValue.titleText = value;
					this._addToken(tokenJSON, keyCode, undefined, undefined, true);
					var rawJSON = this._getData(tokenJSON.itemValue);
					if(this.options.dropdownList.acceptNewValues && this.options.dataSource instanceof Array){
						this.options.dataSource.push(rawJSON);
					}
				}else{
					this.options.dataValidation && this._addErrorToken(value, keyCode);
				}
			}
		},
		_handleEnterPressed: function(orgEvent){
			var noResultExists = this._menu.find("#ztokenfieldnoresults").length;
			if(this._enterKeyPressed && this._menu !== undefined && (!this._menu.is(":visible") || noResultExists)){	// No I18N	
				var val = this._input.val();	
				if(this.options.tokens.editable && this._pasteHelperTextBox && $(orgEvent.target).attr("id") === this._pasteHelperTextBox.attr("id")){	// No I18N	
					var activeToken = this.container.find(".is-selected");	 // No I18N
					activeToken.length && activeToken.length === 1 && this._editToken(activeToken, orgEvent);
				}else if(!this.options.acceptNewValues && this.options.dataValidation){
					$(orgEvent.target).attr("id") === this._input.attr("id") && val !== undefined  && val.length && this._addErrorToken(val);		 // No I18N	
					if(this._editTextBox !== undefined){
						this._reformToken(orgEvent);
						this._input.focus();
					}
				}else{
					if(this.options.acceptNewValues){
						if(this._editTextBox && $(orgEvent.target).attr("id") === this._editTextBox.attr("id")){	// No I18N
							this._reformToken(orgEvent);
							this._input.focus();
						}else{
							this._checkIsContentValid(val);
						}
					}else{	
						if(this._editTextBox && $(orgEvent.target).attr("id") === this._editTextBox.attr("id")){	// No I18N
							this._reformToken(orgEvent);
							this._input.focus();
						}	
					}					
				}
				if(noResultExists){
					this._hideSuggestions();
				}
			}
			this._enterKeyPressed = false;				
		},
		_docClickHandler: function(orgEvent){
			var target = $(orgEvent.target), item = target.closest(".zmenu__item");	// No I18N
			if(target.closest(".ztokenfield").attr("id") !==  this.container.attr("id") && target.closest(".zmenu").attr('id') !== this._menu.attr('id')){	// No I18N
				var val = this._input.val();					
			 	this.container.removeClass("has-focus");	// No I18N
				if(!this._lastTokenUpdate.length && this._tokensCollection.length){
					var token = {prevTokenList: this._lastTokenUpdate, currentTokenList: this._tokensCollection};
					this._dispatchEvent("CHANGE", token);  // No I18N
					for(var i=0; i<this._tokensCollection.length; i++){
						this._lastTokenUpdate.push(this._tokensCollection[i].data().metadata);
					}	
				}else{
					this._checkTokenChange(orgEvent);
				}
				this._clickReset = true;			
				if(!this.options.dropdownList.closeOnSelect){					
					!(this._menu[0].contains(item[0])) ? (this._menu.is(":visible") && this._hideSuggestions()) : this._searchTimeout(this._params.query, this._input, orgEvent);	// No I18N
				}else{
					this._menu.is(":visible") && this._hideSuggestions();	// No I18N
				}
				if(this._editTextBox !== undefined && $(orgEvent.target).attr("id") !== this._editTextBox.attr("id")){	// No I18N
					this._reformToken(orgEvent);
					this._menu.is(":visible") && this._hideSuggestions(); // No I18N
				}else{
					if(!(this._menu[0].contains(item[0])) && !this.options.acceptNewValues && this.options.dataValidation && val !== undefined && val.length){
						this._addErrorToken(val);
					}else if(this.options.acceptNewValues){
						this._checkIsContentValid(val);
					}else{
						this._input[0].value = "";
						this._resizeSearchArea(this._input, this.options.placeholder);
					}
				}	
				this.container.find(".is-selected").removeClass("is-selected");	// No I18N		
			}
			if(!this.options.dropdownList.closeOnSelect && this._menu[0].contains(item[0])){
				this._searchTimeout(this._params.query, this._input, orgEvent);
			}
		},
		_keydownHandler: function(orgEvent){
			this._metaKey = false;
			this._checkMaxLimitReach(orgEvent);
			if(this._input.val().length < 1 && !this._editTextBox){	
				var allElements = this.container.find('.is-selected');	// No I18N
				switch(orgEvent.keyCode){
					//Move across tokens left/right when moved with arrow keys
					case this._keyCode.LEFT:		
						var element = $(allElements[0]), ele;
						if(allElements.length && !(orgEvent.shiftKey || orgEvent.metaKey || orgEvent.ctrlKey)){
							allElements.removeClass('is-selected'); // No I18N
							this._clickReset = true;
						}
						if(orgEvent.shiftKey){
							if(!this._clickReset){
								element.length && (ele = element.prev());
							}else{
								if(this._element){
									var prevSelected;
									prevSelected = this._element.prev();
									if(allElements.length > 1){		
										if(prevSelected.hasClass("is-selected")){	// No I18N
											this._element.removeClass("is-selected");	// No I18N
											this._element = prevSelected;
										}else{
											ele = prevSelected;
										}											
									}else{
										element.length && (ele = element.prev());
										prevSelected = undefined;
									}	
								}else{
									element.length && (ele = element.prev());
									prevSelected = undefined;
								}
							}	
							this._clickReset = true;
						}else{
							element.length && (ele = element.prev());
						}
						if(ele !== undefined && ele.length){
							//Gets the latest active element after arrow key operation
							if(ele.hasClass('ztokenfield__token')){ // No I18N   
								this._element = ele.addClass('is-selected'); // No I18N
								this._element.focus();
								this._triggerPasteHelper();						
							}
							else{
								this._input.focus();
							}
						}else{
							if(!prevSelected){
								var ele = this._input.prev();
								if(ele.hasClass('ztokenfield__token')){ // No I18N
									this._element = ele.addClass('is-selected'); // No I18N
									this._element.focus();
									this._triggerPasteHelper();	
								}	
							}				
						}
						this._menu.is(":visible") && this._hideSuggestions();	// No I18N
                        orgEvent.preventDefault();
					break;
					case this._keyCode.RIGHT:
						var length = allElements.length;	
						var element = $(allElements[length - 1]), ele;	
						if(allElements.length && !(orgEvent.shiftKey || orgEvent.metaKey || orgEvent.ctrlKey)){
							allElements.removeClass('is-selected'); // No I18N
							this._clickReset = true;
						}
						if(orgEvent.shiftKey){
							if(!this._clickReset){
								element.length && (ele = element.next());
							}else{
								if(this._element){
									var prevSelected;
									if(allElements.length > 1){
										prevSelected = this._element.next();
										if(prevSelected.hasClass("is-selected")){	// No I18N
											this._element.removeClass("is-selected");	// No I18N
											this._element = prevSelected;
										}else{
											ele = prevSelected;
										}											
									}else{
										element.length && (ele = element.next());
									}	
								}else{
									element.length && (ele = element.next());
								}
							}	
							this._clickReset = true;
						}else{
							element.length && (ele = element.next());
						}
						if(ele !== undefined && ele.length){
							if(ele.hasClass('ztokenfield__token')){ // No I18N
								this._element = ele.addClass('is-selected'); // No I18N		
								this._triggerPasteHelper();			
							}else{
								this._input.focus();
								orgEvent.shiftKey && this.container.find(".is-selected").removeClass("is-selected");		// No I18N
							}
						}
						this._menu.is(":visible") && this._hideSuggestions();	// No I18N
						orgEvent.preventDefault();	
					break;
				}
			}else{
				var query = this._input.val(), cursorPos = this._input[0].selectionStart;
				if(query.length < this.options.search.maxKeywordLength){
						query = orgEvent.keyCode === this._keyCode.LEFT ?
					query.substring(0, cursorPos-1) : 
						orgEvent.keyCode === this._keyCode.RIGHT ? query.substring(0, cursorPos+1) : undefined;
					query !== undefined && this._searchTimeout(query, this._input, orgEvent);
					if(query !== undefined && query.length === cursorPos){
						orgEvent.preventDefault();	
					}
				}
			}
			if(orgEvent.keyCode === this._keyCode.BACKSPACE || orgEvent.keyCode ===  this._keyCode.DELETE){ //Added to prevent default action for backspace when focus is on token
				this._hasInput = false;
				if(this.container.find(".is-selected").length){	// No I18N
					orgEvent.preventDefault();
					return false;
				}else{
					var target = $(orgEvent.target);
					var query = target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && target.val();		// No I18N
					query !== undefined && query.length && (this._hasInput = true);
				}
			}
			if(orgEvent.metaKey || orgEvent.ctrlKey){
				this._metaKey = true;
				var target = $(orgEvent.target);
				if(orgEvent.keyCode === 65 && ($(orgEvent.target).attr("id") === this._pasteHelperTextBox.attr("id") || (target.attr("id") === this._input.attr("id") && this._input.val().length === 0))){	// No I18N
					orgEvent.preventDefault();
				}
				if(this._menu.is(":visible")){
					if(this.options.dropdownList.itemType === "none"){ 
						this._hideSuggestions(); 
					}else{
						var query = target.val();
						this._searchTimeout(query, target, orgEvent);
					}	
				}
				switch(orgEvent.keyCode){
					case 65:
						if(target.attr("id") === this._pasteHelperTextBox.attr("id") || (target.attr("id") === this._input.attr("id") && this._input.val().length === 0)){ 	// No I18N
							var allTokens = this.container.find(".ztokenfield__token"); // No I18N
							for(var i=0; i<allTokens.length;i++){
								$(allTokens[i]).addClass("is-selected").focus();	
							}
							this._triggerPasteHelper();
						}
					break;
					case 90: 
						if(!this._input.val().length && !this._editTextBox){
							orgEvent.preventDefault();
							if(this._performedStack.length){
								var undoAction = this._performedStack.pop();
								if(undoAction && undoAction.action === "Insert"){	
									this._removeToken($("#"+undoAction.tokenId).find(".ztokenfield__close"), orgEvent, undoAction.index);		
								}else if(undoAction && undoAction.action === "Remove"){	// No I18N
									this._addToken(undoAction.tokenData, orgEvent, undoAction.index, undoAction.tokenId);
								}
							}
						}else{
							this._resizeSearchArea(target);
						}							
					break;
					case 89:
						if(!this._input.val().length && !this._editTextBox){
							orgEvent.preventDefault();
							if(this._recallStack.length){
								var redoAction = this._recallStack.pop();
								if(redoAction && redoAction.action === "Remove"){	// No I18N
									this._removeToken($("#"+redoAction.tokenId).find(".ztokenfield__close"), orgEvent, redoAction.index);	
								}else if(redoAction && redoAction.action === "Insert"){	// No I18N
									this._addToken(redoAction.tokenData, orgEvent, redoAction.index, redoAction.tokenId);
								}
							}
						}else{
							this._resizeSearchArea(target);
						}
					break;
					case 88:
						if(target.attr("id") !== this._input.attr("id") && this._activeTokens !== undefined){	// No I18N
							for(var i=0; i<this._activeTokens.length;i++){							
								this._removeToken($("#"+this._activeTokens[i]).find(".ztokenfield__close"));			
							}												
						}
						setTimeout(this._undoTimeoutHandler.bind(this), 500);
					break;
				}	
			}
			if(orgEvent.keyCode === this._keyCode.ESCAPE){
				if(this._editTextBox !== undefined){
					this._escKeyPressed = true;
					this._reformToken(orgEvent);
					this._input.focus();
				}
			}
			if(orgEvent.keyCode === this._keyCode.TAB && this._editTextBox !== undefined){
				this._reformToken(orgEvent);
				this._menu.is(":visible") && this._hideSuggestions(); // No I18N
			}
			if(this._menu !== undefined && (!this._menu.is(":visible") || this._menu.find("#ztokenfieldnoresults").length) && this._tokensCount !== this.options.maxAllowedValues){	// No I18N
				if(orgEvent.keyCode === this._keyCode.DOWN && !this._ajaxSource && !this.container.find(".ztokenfield__token").hasClass("is-selected")){	// No I18N
					this._searchTimeout(this._params.query, this._input, orgEvent);
				}
				if(orgEvent.keyCode === this._keyCode.ENTER){
					orgEvent.preventDefault();
					this._enterKeyPressed = true;	
					if(!this._newQuery){
						this._handleEnterPressed(orgEvent);
					}
				}
				return true;
			}
			if(this._tokensCount !== this.options.maxAllowedValues){
				switch(orgEvent.keyCode){
					case this._keyCode.UP:
				    case this._keyCode.DOWN:
					case this._keyCode.ENTER:
	                case this._keyCode.ESCAPE:
	             	    orgEvent.preventDefault();
	             	    if(!this._newQuery){
							!this._isLoaderInited && this._menu.zmenu('handleKeyCode',orgEvent.keyCode); // No I18N
						}
						if(!this._menu.find(".on-hover").length){
							if(this._editTextBox !== undefined){
								this._reformToken(orgEvent);
							}
						}
	                break;
				}
			}
		},
		_checkMaxLimitReach: function(orgEvent){
			var charInput = $(orgEvent.target).is("input") && $(orgEvent.target).attr("id") !== this._pasteHelperTextBox.attr("id") && !(orgEvent.metaKey || orgEvent.ctrlKey) && ((orgEvent.keyCode >= 186 && orgEvent.keyCode <= 192) || (orgEvent.keyCode >= 219 && orgEvent.keyCode <= 222) || (orgEvent.keyCode >= 48 && orgEvent.keyCode <= 57) || (orgEvent.keyCode >= 65 && orgEvent.keyCode <= 90) || (orgEvent.keyCode ===  this._keyCode.SPACE));  // No I18N
			if(charInput && this.options.maxAllowedValues && this._tokensCount === this.options.maxAllowedValues){
				orgEvent.preventDefault();
			}
		},
		_keyUpHandler: function(orgEvent){
			if(this.element.attr("disabled")){	// No I18N
				return false;
			}
			if($(orgEvent.target).is("input") && $(orgEvent.target).attr("id") !== this._pasteHelperTextBox.attr("id") && !(orgEvent.metaKey || orgEvent.ctrlKey) && ((orgEvent.keyCode >= 186 && orgEvent.keyCode <= 192) || (orgEvent.keyCode >= 219 && orgEvent.keyCode <= 222) || (orgEvent.keyCode >= 48 && orgEvent.keyCode <= 57) || (orgEvent.keyCode >= 65 && orgEvent.keyCode <= 90) || (orgEvent.keyCode ===  this._keyCode.SPACE)) && this._tokensCount !== this.options.maxAllowedValues){		// No I18N	
				this.container.find(".is-selected").removeClass("is-selected");	// No I18N
				this._clickReset = true;
				this._resizeSearchArea($(orgEvent.target), this.options.placeholder ? $(orgEvent.target).attr("placeholder") : undefined);	// No I18N
				var query = $(orgEvent.target).val();
				if(query.charCodeAt(query.length-1) === this.options.delimiter.charCodeAt(0)){
					var isMenuVisible = this._menu.is(":visible"), menuitem;	// No I18N
					if(isMenuVisible){
						menuitem = this._menu.find("li:not(.zmenu__title)");	// No I18N
					}
					if(menuitem && menuitem.length){
						this.options.autoComplete && this._menu.zmenu("hoverMenuItem", $(menuitem[0]));	// No I18N
						this._menu.zmenu("selectHoveredMenuItem");	// No I18N
					}else{
						var query = query.substring(0, query.length - 1);
						if(!this.options.acceptNewValues && this.options.dataValidation){
							query !== undefined && query.length && this._addErrorToken(query);		
							this._editTextBox !== undefined && this._reformToken(orgEvent);
						}else{
							if(this.options.acceptNewValues){
								this._editTextBox ? this._reformToken(orgEvent) : this._checkIsContentValid(query);	
							}else{
								this._editTextBox && this._reformToken(orgEvent);	
							}					
						}
					}
					this.options.dropdownList.closeOnSelect && this._menu.is(":visible") && this._hideSuggestions(); 	// No I18N
				}else{
					if(query.length < this.options.search.maxKeywordLength && !this._metaKey){
						this._newQuery = true;
						this._searchTimeout(query, $(orgEvent.target), orgEvent);
					}
				}
			}
			if(orgEvent.keyCode === this._keyCode.BACKSPACE || (orgEvent.keyCode ===  this._keyCode.DELETE)){
				var target = $(orgEvent.target), query;
				var query = target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && target.val();	// No I18N
				target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && this._resizeSearchArea(target);	// No I18N
				if(query.length < this.options.search.maxKeywordLength){		
					!this._hasInput && query.length === 0 && this._handleTokenRemove($(orgEvent.target), orgEvent); 
					if(query.length >=0 && this._hasInput){
						this._searchTimeout(query, target, orgEvent);
					}
				}else{
					this.container.find(".is-selected").length && this._handleTokenRemove(this._input, orgEvent); 		// No I18N
				}
				if(this.options.values.length === 0 && target.attr("id") === this._input.attr("id") && target.val().length === 0){	// No I18N
					if(this.options.placeholder){
						this._input.attr("placeholder", this.options.placeholder);	// No I18N
						this._resizeSearchArea(this._input, this.options.placeholder);
					}
				}
			}
		},
		_mousedownHandler: function(orgEvent){
			var val = this._editTextBox ? this._editTextBox.val() : this._input.val();
			var token = $(orgEvent.target).closest(".ztokenfield__token"); 
		//	if($(document.activeElement).attr("id") !== this._input.attr("id")){	// No I18N
				if(!(this._editTextBox !== undefined && this._editTextBox.attr("id") !== $(document.activeElement).attr("id"))){	// No I18N
					if(!token.length && val.length < this.options.search.maxKeywordLength){
						this._searchTimeout(val, this._input, orgEvent);
					}
				}
			//}
			if(this._editTextBox !== undefined && $(orgEvent.target).attr("id") !== this._editTextBox.attr("id")){	// No I18N
				this._reformToken(orgEvent);
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}
			if(!(orgEvent.metaKey || orgEvent.ctrlKey || orgEvent.shiftKey)){
				this.container.find(".is-selected").removeClass("is-selected");	// No I18N
				this._clickReset = true;
			}	
			if(token.length){
				token.addClass("is-selected").focus();	// No I18N 	
				this._menu.is(":visible") && this._hideSuggestions(); // No I18N 
			} 
		},
		_focusHandler: function(orgEvent){
			this._reverseTab = false;

			clearTimeout(this._timeout);

			ZC.$window.off("keyup."+this.componentName).on("keyup."+this.componentName, this._windowKeyUpHandler.bind(this)) // No I18N

			this._timeout = setTimeout(this._focusTimeoutHandler.bind(this),200);
			this.container.addClass('has-focus');	// No I18N
		},
		_windowKeyUpHandler: function(orgEvent){
			if(orgEvent.keyCode === this._keyCode.TAB && orgEvent.shiftKey){
				this._reverseTab = true;
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}else if(orgEvent.keyCode === this._keyCode.TAB){
				this._menu.is(":visible") && this._hideSuggestions();	// No I18N
			}
		},
		_pasteEventHandler: function(orgEvent){
			this._pasteTimeout && clearTimeout(this._pasteTimeout);
			this._pasteTimeout = setTimeout(this._pasteHandler.bind(this, orgEvent), 100);
		},
		_contextMenuHandler: function(orgEvent){
			orgEvent.preventDefault();
		},
		_inputFocusHandler: function(orgEvent){
			this.container.find(".is-selected").removeClass("is-selected");		// No I18N
			this.container.addClass('has-focus'); // No I18N
		},
		_inputBlurHandler: function(orgEvent){
			var val = this._input.val();
			if(!this._menu.find("li").first().length){		// No I18N
				if(!this.options.acceptNewValues && this.options.dataValidation && val && val.length){
					this._addErrorToken(this._input.val(),{keyCode: this._keyCode.TAB});
				}else if(this.options.acceptNewValues){
					this._checkIsContentValid(this._input.val(), {keyCode: this._keyCode.TAB});
				}
			}
			this.container.removeClass("has-focus");	// No I18N
		},
		_bindEvents: function(){
			var componentName = this.componentName;

			this._lastTokenUpdate = [];

			ZC.$document.on("click."+componentName, this._docClickHandler.bind(this));	// No I18N

			this.container.on('keydown.'+componentName, this._keydownHandler.bind(this))  // No I18N

			.on('keyup.'+componentName, this._keyUpHandler.bind(this)) // No I18N

			.off('mousedown.'+componentName).on('mousedown.'+componentName, this._mousedownHandler.bind(this))  // No I18N

			.off('focus.'+componentName).on('focus.'+componentName, this._focusHandler.bind(this)) // No I18N

			.off('paste.'+componentName).on('paste.'+componentName, this._pasteEventHandler.bind(this))	// No I18N
				
			.off('contextmenu.'+componentName).on('contextmenu.'+componentName, this._contextMenuHandler.bind(this))	// No I18N
				
			this._input.off('focus.'+componentName).on('focus.'+componentName, this._inputFocusHandler.bind(this)) // No I18N			
				
			.off('blur.'+componentName).on("blur."+componentName, this._inputBlurHandler.bind(this))	// No I18N
				
		},
		_undoTimeoutHandler: function(){
			this._triggerPasteHelper();
			this._input.focus();
		},
		_focusTimeoutHandler: function(){
			!this._reverseTab && this._input.focus();
		},
		_pasteHandler: function(orgEvent){
			var target = $(orgEvent.target);
			var values = target.is("input") && target.attr("id") !== this._pasteHelperTextBox.attr("id") && target.val();	// No I18N
			values && values.length && this._resizeSearchArea(target, values); 
			var tokenValues = values.split(this.options.delimiter);
			for(var i=0; i<tokenValues.length;i++){
				//if(base._pasteHelperTextBox.val().length){
				var itemName = tokenValues[i].match(/\"(.*)\"/) && tokenValues[i].match(/\"(.*)\"/).pop(),
					itemKey = tokenValues[i].match(/\<(.*)\>/) && tokenValues[i].match(/\<(.*)\>/).pop(), 
					jsonObj = {};
				if(itemKey !== null  && itemName !== null){
					jsonObj.itemValue = {};
					jsonObj.itemValue.itemIdentifier = itemKey;
					jsonObj.itemValue.text = itemName;
					jsonObj.itemValue.titleText = itemKey;
				 	this._addToken(jsonObj, orgEvent, undefined, undefined, true);
				}else{
					this._newQuery = true;
					var value = this._search(tokenValues[i], target).done(this._convertPastedTextToTokens(tokenValues[i].trim()));
					if(value == false){
						return false;
					}
				}
			}	
			if(this._editTextBox !== undefined){
				this._editTextBox.remove();
				this._editTextBox = undefined;
			}
		},
		_extendEvents: function(){
			this._EVENTS = $.extend(true, {}, this._TOKENFIELDEVENTS, this._EVENTS);
		},
		_bindMenuEvents: function(){
			this._super();
			var base = this;
			this._menu.off("zmenubeforeitemchecked").on("zmenubeforeitemchecked", function(orgEvent, ui){	//No I18N
				if(base.options.dropdownList.itemType === "checkbox" && ui.data.checked !== "checked" && base.options.maxAllowedValues && base._tokensCount >= base.options.maxAllowedValues){	//No I18N
					orgEvent.preventDefault();
					base._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"}, orgEvent);	//No I18N
				}
			})
		},
		_convertPastedTextToTokens: function(tokenValue){
			!this._menu.is("visible") && this.options.acceptNewValues && this._checkIsContentValid(tokenValue);	// No I18N
		},
		_handleTokenRemove: function(textbox, orgEvent){
			var element = this.container.find('.is-selected');	// No I18N
			if(!element.length){
				this._element = textbox.prev()
				if(this._element.hasClass('ztokenfield__token')){	// No I18N
					this._element.addClass('is-selected');	// No I18N
					this._removeToken(this._element.find('.ztokenfield__close'), orgEvent);	// No I18N
					textbox.focus();
				}
				if(this._editTextBox !== undefined){
					this._editTextBox.remove();
					this._editTextBox = undefined;
					this._input.focus();
				}
			}else{
				for(var i=0; i<element.length;i++){
					this._removeToken($(element[i]).find('.ztokenfield__close'), orgEvent);	
					textbox.focus();
				}
			}
			this.options.dropdownList.itemType === "none" && this._hideSuggestions();	//No I18N
			//this._searchTimeout('', this._input, orgEvent);	//No I18N
			if(textbox.attr("id") === this._input.attr("id")){	//No I18N
				this._resizeSearchArea(textbox, this.options.placeholder ? textbox.attr("placeholder") : undefined);	//No I18N
			}
		},
		_handleSelection: function(orgEvent, ui){
			if(!(ui.menuitem.attr("id") === this.componentName+"noresults" || ui.menuitem.hasClass("zmenu__title") || ui.menuitem.attr("id") === this.componentName+"loaderror")){	// No I18N
				var textbox = this._editTextBox !== undefined ? this._editTextBox : this._input;						
				if(ui.menuitem.attr("id") === this.componentName+"moreaction"){	// No I18N
					var action = this.options.dropdownList.moreResultsAction;
					if(action && typeof action === "function"){  // No I18N
						action();
					}
					this._dispatchEvent("MOREACTIONSELECT", {value: ui.menuitem, query: this._params.query}, orgEvent);  // No I18N
				}else{
					var itemData = ui.menuitem.data().metadata;
					if(!this.options.dropdownList.closeOnSelect || this.options.dropdownList.itemType === "checkbox"){  // No I18N
						orgEvent.preventDefault();
					}
					this._lastSelectedMenuItem = ui.menuitem;
					if(!($(ui.menuitem).prop("checked")) && this.options.dropdownList.itemType === "checkbox"){	// No I18N
						var itemKey = itemData.itemValue !== undefined ? itemData.itemValue.itemIdentifier : this._mapJSONToData(itemData, this._dataMapping.itemIdentifier);
						this.removeToken(itemKey, orgEvent);
					}else{
						if(!this._isJSONSource){
							return;
						}else{	
							this._dispatchEvent("ITEMSELECT", {value: ui.menuitem}, orgEvent);  // No I18N
							this._selectedItem = this.getData(ui.menuitem);
							if(this._editTextBox){
								var index = this._currentTokenData.metadata ? this._currentTokenData.metadata.index : this._currentTokenData.index;
								var id = this._currentTokenData.metadata ? this._currentTokenData.metadata.id : this._currentTokenData.tokenId;
								this._buildToken(ui.menuitem, textbox, orgEvent, index, id);
							}else{
								this._buildToken(ui.menuitem, textbox, orgEvent);
							}
							if(this._editTextBox !== undefined){
								this._editTextBox.remove();
								this._editTextBox = undefined
							}
						}						
					}	
					if(this.options.dropdownList.itemType === "checkbox" && this.options.dropdownList.position === "after-cursor"){		// No I18N
						this._input.focus();
						var pos = this._positionMenuAtCursorPosition(this._input);
						this._menu.css({top: (pos.top + 15), left: pos.left});	
					}
				}
			}else{
				this._input.focus();
			}
		},
		_handleHover: function(orgEvent, ui){
			if(!(ui.menuitem.attr("id") === this.componentName+"noresults") && !(ui.menuitem.hasClass("zmenu__title")) && !(ui.menuitem.attr("id") === this.componentName+"moreaction")){	// No I18N
				if(!this._isJSONSource){
					this._input.val(ui.menuitem.data().metadata.label);
				}else{
					var value = ui.menuitem.data().itemValue !== undefined ? ui.menuitem.data().itemValue.text : this._mapJSONToData(ui.menuitem.data().metadata, this._dataMapping.text);
					this._input.val(value);
				}
			}else{
				this._input.val(this._params.query);
			}
			this._resizeSearchArea(this._input);
		},
		_removeAddedItem: function(){
			if(this._tokensCollection.length){
				var base = this, addedTokens;
				 addedTokens = this._tokensCollection.map(function(item){
				 	if(item.data("invalidToken") !== true){	// No I18N
					 	var value = item.data().metadata[base._dataMapping.value]; 
						return value;	
					}
				})
				var base = this;
				//Removes already added tokens
				$.each(addedTokens, function(index,item){
					base._filteredResult = $.grep(base._filteredResult , function(val){
				 		var value = base._mapJSONToData(val, base._dataMapping.value);
				 		if(item === undefined || (item !== undefined && value !== undefined && value.toString() !== item.toString())){
				 			return true;
				 		}
					})
				})			
			}	
		},
		_search: function(query, container, orgEvent){
			return	this._super(query, container, orgEvent);	
		},
		_updateCheckedState: function(){
			if(this.options.dropdownList.itemType === "checkbox"){
				var selectedId = [];
				if(this._tokensCollection.length){
					for(var i=0; i<this._tokensCollection.length;i++){
						var tokenData = this._tokensCollection[i].data().metadata, id;
						if(tokenData !== undefined && !tokenData.isNewValue){
							id = this._mapJSONToData(tokenData, this._dataMapping.value);
						}
						this._menu.find("#"+id).length && selectedId.indexOf("#"+id) === -1 && selectedId.push("#"+id); // No I18N 
					}
					selectedId.length && this._menu.zmenu("setMenuItemsAttribute",selectedId,"checked",true); // No I18N
				}
			}
		},
		_renderSuggestions: function(container, orgEvent){
			this._super(container, orgEvent);
			this._updateCheckedState();
			if(this._loader){
				this._loader.closest(".zmenu--loading").remove();	// No I18N
				this._loader = null;
			}
			this._menu.removeData("fixedArea");	// No I18N
		},
		_buildToken: function(menuItem, textbox, orgEvent, currentIndex, currentId){
			var itemKey, itemName, itemTitle, itemImage, maxAllowed = this.options.maxAllowedValues, values;
			if(values = this._selectedItem.itemValue){
				itemKey = values.itemIdentifier;
				itemName = values.text;
				itemTitle = values.titleText;
				itemImage = this.options.tokens.contentType === "image-text" ? values.image : false; // No I18N
			}			
			if(!maxAllowed || (maxAllowed && (this._tokensCount < maxAllowed))){
				this._addCustomCloseIcon();
				this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>"); 
				this._tokenText = $('<span class ="ztokenfield__text">').text(itemName);	// No I18N
				itemTitle !== undefined && this._tokenText.attr("title", itemTitle); // No I18N
				if(itemImage !== false && itemImage !== undefined){
					var tokenImage = !this.options.isImageSourceSprited ? $("<img class='ztokenfield__image' src='"+itemImage+"'>") : 	// No I18N
				    $("<i class='ztokenfield__image'>").addClass(itemImage);	
					tokenImage.appendTo(this._tokenContainer);
				}
				this._insertToken(menuItem, textbox, orgEvent, currentIndex, currentId);
			}else if(maxAllowed && this._tokensCount >= maxAllowed){				
				this._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"}, orgEvent);	//No I18N
			}
		},
		_animateHandler: function(token, orgEvent){
			var base = this;
			token.addClass('on-remove').animate({width: 0}, "fast", function(){	// No I18N
				var tokenToRemove = token.data().metadata;
				var selectedItems = base.options.values;		
				for(var i=0; i < selectedItems.length;i++){
					var itemKey = !tokenToRemove.invalidToken ? base._mapJSONToData(tokenToRemove, base._dataMapping.itemIdentifier) : tokenToRemove.id, 
							tokenToCompare = !tokenToRemove.invalidToken ? base._mapJSONToData(selectedItems[i], base._dataMapping.itemIdentifier) : selectedItems[i].id; 
					if(itemKey === tokenToCompare){
						selectedItems.splice(i,1);
						break;
					}
				}
				token.remove();
				token = null;
				base._checkContainerHeight();
				if(base.options.dropdownList.position === "after-cursor"){	// No I18N	
					base._input.focus();
					var pos = base._positionMenuAtCursorPosition(base._input);
					base._menu.css({top: (pos.top + 15), left: pos.left});
				}	
				if(base.options.placeholder && base.options.values.length === 0 && base._input.val().length === 0){
					base._input.attr("placeholder", base.options.placeholder);	// No I18N
					base._resizeSearchArea(base._input, base.options.placeholder);
				}
				base._dispatchEvent("AFTERTOKENREMOVE", tokenToRemove, orgEvent);  // No I18N				
			});
		},
		_removeToken: function(element, orgEvent, index){
			var canBeRemoved = this._dispatchEvent("BEFORETOKENREMOVE", $(element).parent(), orgEvent);  // No I18N
			var data, invalidToken, token, tokenEncountered = false, matchedId, id;
			if(canBeRemoved && element){
				var tokenToRemove = $(element).parent();
				for(var i=0; i < this._tokensCollection.length; i++){
					if(this._tokensCollection[i].attr("id") === tokenToRemove.attr("id")){	// No I18N
						tokenEncountered = true, matchedId = tokenToRemove.attr("id");	//No I18N
						data = this._tokensCollection[i].data().metadata, invalidToken = this._tokensCollection[i].data("invalidToken"), id = tokenToRemove.attr("id");	// No I18N
						if(this._menu.is(":visible")){		// No I18N
							if(this.options.dropdownList.itemType === "none"){ 
								this._hideSuggestions(); 
							}else{
								if($("#"+id).length && !invalidToken && !data.isNewValue){
									var id = this._mapJSONToData(data, this._dataMapping.value);
									this._menu.zmenu("setMenuItemsAttribute",["#"+id],"checked",false);	// No I18N
									this._lastSelectedMenuItem = $("#"+id);
									this._menu.zmenu("hoverMenuItem", this._lastSelectedMenuItem);	// No I18N
								}
							}
						}		
						invalidToken && (data.invalidToken = invalidToken);
						var token = this._tokensCollection[i];
						if(this.options.tokens.animateOnRemove && ((orgEvent && orgEvent.type !== "dblclick" && orgEvent.key !== "Enter") || orgEvent === undefined)){	// No I18N
							token.animate({ opacity: 0.01 }, "fast", this._animateHandler.bind(this, token, orgEvent));	// No I18N								
						}else{
							this._tokensCollection[i].remove();
							this._tokensCollection[i] = null, tokenToRemove = null, this._element = null;	
							this._checkContainerHeight();
							if(this.options.dropdownList.position === "after-cursor"){	// No I18N	
								this._input.focus();
								var pos = this._positionMenuAtCursorPosition(this._input);
								this._menu.css({top: (pos.top + 15), left: pos.left});
							}							
						}		
						this._tokensCollection.splice(i, 1);
						for(var j = i; j< this._tokensCollection.length; j++){
							this._tokensCollection[j].data().metadata.index = j;
						}
						if(this._editTextBox === undefined && (index === undefined || (orgEvent && (orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 89))){
							this._performedStack.splice(i, 1);		
							this._performedStack.push({"action":"Remove", tokenData: data, tokenId: id, index: i});	// No I18N
						}else if(this._editTextBox === undefined && index !== undefined && (orgEvent && (orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 90)){
							this._recallStack.push({"action": "Insert", tokenData: data, tokenId: id, index: i});		// No I18N
						}
						!invalidToken && this._tokensCount--;
						break;
					}
				}
				if(!this.options.tokens.animateOnRemove || (this.options.tokens.animateOnRemove && orgEvent && (orgEvent.type === "dblclick" || orgEvent.key === "Enter"))){
					var selectedItems = this.options.values, tokenToCompare,
						itemKey = !invalidToken ? this._mapJSONToData(data, this._dataMapping.itemIdentifier) : matchedId;
					for(var i=0; i < selectedItems.length;i++){
							tokenToCompare = selectedItems[i].errorValue === undefined ? this._mapJSONToData(selectedItems[i], this._dataMapping.itemIdentifier) : (data ? data.id : undefined); 
						if(itemKey && tokenToCompare && itemKey === tokenToCompare){
							this.options.values.splice(i,1);
							break;
						}
					}
					if(this.options.placeholder && this.options.values.length === 0 && this._input.val().length === 0){
						this._input.attr("placeholder", this.options.placeholder);	// No I18N
						this._resizeSearchArea(this._input, this.options.placeholder);
					}
					tokenEncountered && (this._dispatchEvent("AFTERTOKENREMOVE", data, orgEvent));	  // No I18N			
				}				
			}
		},
		_bindTokenEvents: function(){
			var base = this;
			if(!this.options.disabled){
				this._closeButton.off('click.'+this.componentName).on('click.'+this.componentName, function(orgEvent){	// No I18N
					orgEvent.stopPropagation();
					base._removeToken(orgEvent.currentTarget, orgEvent);
					base._input.focus();
				})
				this._tokenContainer.off("focus."+this.componentName)	// No I18N
			    .on("focus."+this.componentName, function(orgEvent){	// No I18N
					base.container.addClass("has-focus");	// No I18N
				})
				this._tokenContainer.off("click."+this.componentName).on("click."+this.componentName, function(orgEvent){	// No I18N
								
				})
				this.options.tokens.editable && this._tokenContainer.off("dblclick."+this.componentName).on("dblclick."+this.componentName, function(orgEvent){	// No I18N
					base._editToken($(orgEvent.currentTarget), orgEvent);
				})
			}
		},
		_tokenClickHandler: function(orgEvent){
			var currentTarget = $(orgEvent.currentTarget);
			base._clickReset = false;
			base.container.addClass("has-focus");	// No I18N
			if(!(orgEvent.metaKey || orgEvent.ctrlKey || orgEvent.shiftKey)){
				base.container.find(".is-selected").removeClass("is-selected");	// No I18N
				base._clickReset = true;
			} 
			if(orgEvent.shiftKey){
				var elementsLength = base.container.find(".is-selected").length;	// No I18N
				var firstIndex = $(base.container.find(".is-selected")[0]).data().metadata.index,	
					 lastIndex = $(base.container.find(".is-selected")[elementsLength - 1]).data().metadata.index;	
				for(var i = firstIndex; i<=lastIndex; i++){
					base.container.find("#token-"+i).addClass("is-selected");	// No I18N
				}
				base._clickReset = false;
			}
			currentTarget.addClass("is-selected"); // No I18N
			currentTarget.focus();	
			base._triggerPasteHelper();	
		},
		_triggerPasteHelper: function(){
			var activeTokens = this.container.find(".ztokenfield__token.is-selected");	// No I18N
			var base = this;
			base._activeTokens = [];
			var finalValue = $.map(activeTokens, function(token, j){
				var key, value, data;
				base._activeTokens[j] = $(token).attr("id");	
				data = $(token).data().metadata;
				if($(token).data("invalidToken") !== true && data){	// No I18N
					key =  base._mapJSONToData(data, base._dataMapping.itemIdentifier), value = base._mapJSONToData(data, base._dataMapping.text);				
					var finalValue = base._pasteRegExp.replace(/#value#/g, value).replace(/#id#/g, key);
					return finalValue;	
				}			
			}).join(this.options.delimiter);
			this._pasteHelperTextBox.val(finalValue).select();
		},
		_editToken: function(tokenContainer, orgEvent){
			if(tokenContainer.hasClass("ztokenfield__token")){	// No I18N
				this._editTextBox = $("<input type='text' id='dblclick-input-"+this.componentName+"'>").addClass("ztokenfield__textbox");	// No I18N
				var value, matchFound, itemData, dataValue = tokenContainer.data();	
				dataValue = dataValue.invalidToken;	
				this._tokenOption = this.options.tokens;
				if(dataValue !== true){	
				    itemData = tokenContainer.data().metadata;
				    this._currentTokenData = itemData;
				    this._currentTokenData.tokenId = tokenContainer.attr("id");	// No I18N
				    var tokenKey, tokenValue
				    if(itemData !== undefined){
				    	tokenValue = this._mapJSONToData(itemData, this._dataMapping.text),
				    	tokenKey = this._mapJSONToData(itemData, this._dataMapping.itemIdentifier);
				    }
					try{
						var pattern = this._tokenOption.editable.pattern ? this._tokenOption.editable.pattern : this._CONSTANTS.pattern;
						if(pattern.match(/#value#/g) || pattern.match(/#id#/g)){
							value = pattern.replace(/#value#/g, tokenValue).replace(/#id#/g, tokenKey);
							matchFound = true;
							this._editTextBox.insertBefore(tokenContainer);					
							this._removeToken(tokenContainer.find(".ztokenfield__close"), orgEvent);	// No I18N	
							this._editTextBox.focus()
						}else{
							throw(this._CONSTANTS.patterFormatError);	
						}
					}catch(error){
						throw error;
					}
				}else{
					this._editTextBox.insertBefore(tokenContainer).addClass("invalid"); // No I18N
					value = tokenContainer.find(".ztokenfield__text").text();	// No I18N
					matchFound = true;
					itemData = tokenContainer.data();
					this._currentTokenData = itemData;
					this._currentTokenData.tokenId = tokenContainer.attr("id");	// No I18N
					this._removeToken(tokenContainer.find(".ztokenfield__close"), orgEvent); 	// No I18N
					this._editTextBox.focus()
				}
				if(this._input.attr("placeholder") !== undefined){		// No I18N
					this._input.removeAttr("placeholder");		// No I18N
					this._resizeSearchArea(this._input, undefined);
				}
				if(matchFound){
					matchFound = false;
					this._editTextBox.val(value).select();
					this._resizeSearchArea(this._editTextBox);
					this._input.val("");	// No I18N
					var base = this;
					this._editTextBox.off("focus."+this.componentName).on("focus."+this.componentName, function(orgEvent){	// No I18N
						base.container.addClass("has-focus");	// No I18N
					})
				}

			}
		},
		_insertToken: function(menuItem, textbox, orgEvent, index, tokenId, isNewValue){
			var base = this, metadata, isDuplicate, value = textbox.val();
			if(menuItem !== undefined){
				//Check whether the passed parameter is DOM Element
				metadata = (menuItem[0] && menuItem[0].nodeName) ? menuItem.data().metadata : menuItem;
			}
			metadata && metadata.errorValue === undefined && !this._defaultSelected && !this.options.allowDuplicateValues && (isDuplicate = this._isDuplicateToken(metadata.itemValue !== undefined ? metadata.itemValue : metadata));
			!isDuplicate && (this.options.dropdownList.itemType !== "checkbox" && this.options.dropdownList.closeOnSelect) && textbox.val(""); //No I18N	
			if(this._tokensCollection.length === 0){
				this._input.attr("placeholder") && this._input.removeAttr("placeholder");	 // No I18N
				this._resizeSearchArea(this._input, undefined); 		
			}
			if(!isDuplicate){
				var returnValue = this._dispatchEvent("BEFORETOKENADD", {token: this._tokenContainer, tokenData: metadata ? metadata.itemValue : undefined}, orgEvent); // No I18N
				if(returnValue){
					this._closeButton.appendTo(this._tokenContainer);
					this._tokenText.appendTo(this._tokenContainer);
					tokenId !== undefined ? this._tokenContainer.attr("id", tokenId) :  // No I18N
					this._tokenContainer.attr("id","token-"+this._uniqueIndex++);	// No I18N
					(index || index === 0) ? (index < this._tokensCollection.length ? this._tokenContainer.insertBefore(this._tokensCollection[index]) : this._tokenContainer.insertBefore(textbox)) : this._tokenContainer.insertBefore(textbox);
					if(this.options.tokens.sortable){
						//Hides menu when token is dragged with mouse
						this.container.off("zsortablemousedrag").on("zsortablemousedrag", function(orgEvent){	// No I18N
							base._menu.is(":visible") && base._hideSuggestions();	// No I18N
						})
					}
					var data = this.getData(this._tokenContainer);
					//Adding the remaining metadata to the data attribute of token
					if(metadata){
						if(!metadata.errorValue){
							if(metadata.itemValue){
								for(var i in metadata.itemValue){
									if(i != "id"){
										if(i in this._dataMapping){
											data[this._dataMapping[i]] = metadata.itemValue[i]
										}		
									}
								}	
							}else{
								for(var key in metadata){
									if(i != "id"){
										data[key] = metadata[key];
									}
								}
							}
							if(isNewValue){
								data.isNewValue = isNewValue;
							}
						}else{
							data.errorValue = metadata.errorValue
						}
					}
					data.index = index !== undefined ? index : this._uniqueIndex - 1;
					if(index !== undefined){
						for(var i=index; i<this._tokensCollection.length;i++){
							this._tokensCollection[i].data().metadata.index = (i+1);
						}
					}
					if(this.options.tokens.sortable){
						var data = this.container.data("zsortable");	// No I18N
						data && this.container.zsortable("destroy");	// No I18N
						this.container.zsortable({leave: "ztokenfield__close"});	// No I18N
					}
					this.options.maxAllowedValues && metadata && !metadata.errorValue && this._tokensCount++;
					if(!this._defaultSelected){
						var data = this._tokenContainer.data().metadata, jsonData = {};
						if(data && data.errorValue === undefined){
							for(var key in this._dataMapping){
								this._dataMapping[key] !== undefined && (jsonData[this._dataMapping[key]] = data[this._dataMapping[key]]);
							}
						}else{
							jsonData = data;
							jsonData.type = "error";		// No I18N
						}			
						if(!this.options.values.length){
				    		this.options.values.push(jsonData);
				    	}else{   	
				    		index === undefined ? this.options.values.push(jsonData) : this.options.values.splice(index, 0, jsonData);
				    	}
						this.element.data("selectedTokens", this.options.values);	// No I18N
					}		
					this._dispatchEvent("AFTERTOKENADD", {token: this._tokenContainer}, orgEvent);  // No I18N
					this._checkIfTokenAdded(this._tokenContainer, index);
					this._resizeSearchArea(this._input, undefined); 	
					this._bindTokenEvents();
					orgEvent && orgEvent.keyCode !== this._keyCode.TAB && this._input.focus();
					if(index !== undefined && ((orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 90)){ 
						this._recallStack.push({"action": "Remove", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: index});	// No I18N
						this._performedStack.splice(index, 0, {"action" : "Insert", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: index});	// No I18N
					}else if(index !== undefined && ((orgEvent.metaKey || orgEvent.ctrlKey) && orgEvent.keyCode === 89)){
						this._performedStack.push({"action" : "Insert", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: index});	// No I18N
					}else{
						index === undefined && this._performedStack.push({"action" : "Insert", tokenData: data, tokenId: this._tokenContainer.attr("id"), index: this._tokensCollection.length - 1}) ;	// No I18N
					}
					this._checkContainerHeight();	
				}
				
				this._editTextBox !== undefined && (this._tokenContainer.focus().addClass("is-selected"));
				if(this.options.dropdownList.itemType === "checkbox"){
					if(this._menu.is(":visible")){	// No I18N
						value.length && this._searchTimeout(this._params.query, textbox, orgEvent);	
					}
				}
			}else{
				if(this.options.dataValidation){
					this._addErrorToken(menuItem.itemValue ? menuItem.itemValue.text : textbox.val(), orgEvent);
				}
				this._dispatchEvent("TOKENADDFAIL", {failureType: "Duplicate"}, orgEvent);	//No I18N
			}
			this._tokenContainer = null, this._tokenText = null, this._closeButton = null;
		},
		_checkContainerHeight: function(){
			var currentHeight = this.container.css("height");	// No I18N
			if(currentHeight !== this._prevContainerHeight && this._menu.is(":visible")){
				if(this._lastSelectedMenuItem || this._menu.find("li").length){
					this._menu.zmenu("show", {"mustBeShown":true, preventHover: true, positionAlterable: true});	// No I18N
					var item = this._lastSelectedMenuItem ? this._lastSelectedMenuItem : $(this._menu.find("li")[this._dataMapping.groupLabel ? 1 : 0]);
					this._menu.zmenu("hoverMenuItem", item);	// No I18N
				}
			}
			this._prevContainerHeight = currentHeight;
		},
		_reformToken: function(orgEvent){
			var itemData = this._currentTokenData;
			if(!this._escKeyPressed){
				var dataHandled = this._dispatchEvent("TOKENUPDATE", {textboxValue: this._editTextBox.val(), itemContent: this._currentTokenData}, orgEvent); // No I18N
				itemData = (typeof dataHandled === "object") ? dataHandled.itemContent : itemData;	// No I18N
				value = (typeof dataHandled === "object") ? dataHandled.itemContent.invalidToken : itemData.invalidToken;	// No I18N
			}else{
				if(!itemData.invalidToken){
					value = false;
				}else{
					value = true;
				}		
			}
			if(value !== true){	
				var itemKey =  this._mapJSONToData(itemData, this._dataMapping.itemIdentifier), 
					itemValue = this._mapJSONToData(itemData, this._dataMapping.text), 
					itemTitle =  this._mapJSONToData(itemData, this._dataMapping.titleText),
					itemIcon =  this._mapJSONToData(itemData, this._dataMapping.image);
				if(itemValue.length){
					this._addCustomCloseIcon();
					this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>");  
					this._tokenText = $('<span class ="ztokenfield__text">').text(itemValue);	// No I18N
					itemTitle !== undefined && this._tokenText.attr("title", itemTitle);	// No I18N
					if(this.options.tokens.contentType === "image-text" && itemIcon !== false && itemIcon !== undefined){	// No I18N
						var tokenImage = !this.options.isImageSourceSprited ? $("<img class='ztokenfield__image' src='"+itemIcon+"'>") : 	// No I18N
					    $("<i class='ztokenfield__image'>").addClass(itemIcon);	
						tokenImage.appendTo(this._tokenContainer);
					}
					var data = {};
					data.itemValue = {};
					for(var key in this._dataMapping){
						data.itemValue[key] = this._mapJSONToData(itemData, this._dataMapping[key]);
					}
					this._insertToken(data, this._editTextBox, orgEvent, itemData.index, itemData.tokenId);		
					this._performedStack.push({"action" : "Insert", tokenData: itemData, tokenId: itemData.tokenId, index: itemData.index});	// No I18N
				}
			}else{
				if(!this._escKeyPressed){
					var value = this._editTextBox.val();
					if(value.length){
						this._addErrorToken(value, orgEvent, itemData.index, itemData.tokenId);
					}
				}else{
					if(itemData.invalidToken){
						this._addErrorToken(itemData.metadata.errorValue, orgEvent, itemData.index, itemData.tokenId);
					}
				}			
			}	
			this._editTextBox.remove();
			this._editTextBox = undefined;	
			this._escKeyPressed = false;		
			//this._triggerPasteHelper();					
		},
		_checkIfTokenAdded: function(token, index){
			var isExists = false;
	    	if(!this._tokensCollection.length){
	    		this._tokensCollection.push(token);
	    	}else{   	
	    		index === undefined ? this._tokensCollection.push(token) : this._tokensCollection.splice(index, 0, token);
	    	}
		},
		_isDuplicateToken: function(tokenData){
			var isDuplicate = false;
			for(var i=0;i<this.options.values.length;i++){
				if(tokenData.itemIdentifier === this.options.values[i][this._dataMapping.itemIdentifier]){
					isDuplicate = true;
					break;
				}else{
					isDuplicate = false;
				}
			}
			return isDuplicate;
		},
		_addToken: function(value, orgEvent, index, tokenId, isNewValue, target){
			if(value.invalidToken !== true){
				var maxAllowed = this.options.maxAllowedValues;
				if(!maxAllowed || (maxAllowed && (this._tokensCount < maxAllowed))){
					var itemKey = value[this._dataMapping.itemIdentifier] ? value[this._dataMapping.itemIdentifier] : (value.itemValue && value.itemValue.itemIdentifier);
					var itemName = value[this._dataMapping.text] ?  value[this._dataMapping.text] :(value.itemValue && value.itemValue.text);
					var itemTitle = value[this._dataMapping.titleText] ?  value[this._dataMapping.titleText] :(value.itemValue && value.itemValue.titleText);
					var icon = value[this._dataMapping.image] ? value[this._dataMapping.image] : (value.itemValue && value.itemValue.image);
					this._addCustomCloseIcon();
					this._tokenContainer = $("<div class='ztokenfield__token' tabindex='-1'>"); 
					this._tokenText = $('<span class ="ztokenfield__text">').text(itemName);	// No I18N
					itemTitle !== undefined && this._tokenText.attr("title", itemTitle);	// No I18N
					if(this.options.tokens.contentType === "image-text" && icon !== undefined){	// No I18N
						var tokenImage = !this.options.isImageSourceSprited ? $("<img class='ztokenfield__image' src='"+icon+"'>") : 	// No I18N
					    $("<i class='ztokenfield__image'>").addClass(icon);	
						tokenImage.appendTo(this._tokenContainer);
					}
					var textbox = this._editTextBox !== undefined ? this._editTextBox : this._input;	
					this._insertToken(value, textbox, orgEvent, index, tokenId, isNewValue);
				}else if(maxAllowed && this._tokensCount >= maxAllowed){				
					this._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"}, orgEvent);	//No I18N
				}
			}else{
				this._addErrorToken(value.errorValue, orgEvent, index, tokenId);
			}	
		},
		_removeTokens: function(tokens, value){
			for(var j=0; j<tokens.length;j++){
				var metadata = this.getData($(tokens[j]));
				var itemKey = this._mapJSONToData(metadata, this._dataMapping.itemIdentifier);
				if(itemKey == value){
					var tokenItem = $(tokens[j]).find(".ztokenfield__close");	
					this._removeToken(tokenItem);
					if(!this.options.allowDuplicateValues){
						break;
					}
				}
			}
		},
		_getTokens: function(selectedItems, value){
			for(var j=0; j<selectedItems.length;j++){
				var itemKey = this._mapJSONToData(selectedItems[j], this._dataMapping.itemIdentifier);
				if(itemKey === value){
					return selectedItems[j];
				}
			}
		},
		/** Creates multiple tokens with the given data and add it to the container.
			@function token-field.addTokens
			@param {Array} tokenData - The related data for the tokens to be added should be passed as an array.
			@example <caption> Example </caption>
	var token = $(".selector").ztokenfield("addTokens",[{
	        countryName: "India", 
	        flag:"image.jpg", 
	        countryCode: 345, 
	        continentName: "Asia"
	    },
	    {	countryName: "Brazil", 
	        flag:"image1.jpg", 
	        countryCode: 457, 
	        continentName: "South America"
	    }]);*/
		addTokens: function(value){
			if(value && $.isArray(value)){
				for(var i=0; i<value.length;i++){
					var prevIndex = i;
					var returnValue = this.addToken(value[i], i);
					if(returnValue){
						value.splice(i, 1);
					}
					returnValue && (i = prevIndex - 1);
				}
			}
		},
		/** Creates the token with the given data and add it to the tokenfield container.
			@function token-field.addToken
			@param {Object} tokenData - The related data for the token need to be passed as an object
			@example <caption> Example </caption>
var token = $(".selector").ztokenfield("addTokens",{countryName: "India",flag:"image.jpg",countryCode: 345,continentName: "Asia"});*/                                              
		addToken: function(value, index){
			if(value && !$.isArray(value)){
				var tempObj = {};
				tempObj.itemValue = {};
				for(var key in this._dataMapping){				
					tempObj.itemValue[key] = value[this._dataMapping[key]]
				}	
				var shouldPreventAdd,isValidValue;
				if(this.options.dataSource && !this._ajaxSource && value.type !== "error"){
					for(var i=0;i<this.options.dataSource.length;i++){
						if(this.options.dataSource[i][this._dataMapping.itemIdentifier] === tempObj.itemValue.itemIdentifier){
							isValidValue = true;
							break;
						}else{
							isValidValue = false;
						}
					}
					if(!this.options.acceptNewValues){
						if(!isValidValue){
							shouldPreventAdd = true;
						}
					}
				}
				if(value.type !== "error"){	//No I18N
					var maxAllowed = this.options.maxAllowedValues
					if(!shouldPreventAdd && ((maxAllowed && this._tokensCount < maxAllowed) || !maxAllowed)){
						this._addToken(tempObj);
						if(!isValidValue && this.options.acceptNewValues){
							if(this.options.dropdownList.acceptNewValues && this.options.dataSource instanceof Array){
								this.options.dataSource.push(value);
							}
						}
					}else if(maxAllowed && this._tokensCount >= maxAllowed){
						this._defaultSelected && this.options.values.splice(index, 1);
						this._dispatchEvent("TOKENADDFAIL", {failureType: "limit exceeded"});	//No I18N
						return true;
					}else if(shouldPreventAdd){
						this._defaultSelected && this.options.values.splice(index, 1);
						this._dispatchEvent("TOKENADDFAIL", {failureType: "deny new value"});	//No I18N
						return true;
					}
				}else{
					value.type === "error" && this._addErrorToken(value.errorValue);	//No I18N
				}
			}
		},
		/** To retrieve multiple tokens, pass the unqiue key as an array.
			@function token-field.getValues
			@param {Array} searchKey - The unique keys whose associated token need to be retrieved 
			@return {Array} Array of JSON containing information about the passed token keys.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("getValues",["aishwaryalakshmi.p@zohocorp.com","ashwini@zohocorp.com"]);*/
		getValues: function(value){
			var tokens = [];
			//this.container.focus();
			if($.isArray(value) && this.options.values && this.options.values.length){
				for(var i=0;i<value.length;i++){
					tokens.push(this._getTokens(this.options.values, value[i]));
				}
				return tokens;
			}
		},
		getValue: function(value){
			//this.container.focus();
			if(value !== undefined && !$.isArray(value)){
				return this._getTokens(this.options.values, value);	
			}
		},
		/** Retrieves all the tokens in the container.
			This signature doesn't accept any arguments.
			@function token-field.getAllValues
			@return {Object} JSON data of all added tokens.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("getAllValues");*/
		getAllValues: function(){
			return this.options.values;
		},
		/** To remove multiple tokens, pass the unqiue key as an array.
			@function token-field.removeTokens
			@param {array} searchKey - The unique keys whose associated token need to be removed 
			@return {array} value.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("removeTokens",["aishwarya@zohocorp.com","ashwini@zohocorp.com"]);*/
		removeTokens: function(value){
			var tokens = [];
			var allTokens = this.container.find(".ztokenfield__token"); // No I18N 
			if($.isArray(value) && allTokens && allTokens.length){
				for(var i=0;i<value.length;i++){
					this._removeTokens(allTokens, value[i])
				}
			}
			//this._input.focus();		
		},
		removeToken: function(value){
			var tokens = [], allTokens = this.container.find(".ztokenfield__token"); // No I18N 
			if(value !== undefined && !$.isArray(value)){
				this._removeTokens(allTokens, value);
			}
			//this._input.focus();
		},
		/** Removes all the tokens in the container.
			This signature doesn't accept any arguments.
			@function token-field.removeAllTokens
			@return {object} value.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("removeAllTokens"); */
		removeAllTokens: function(){
			var tokens = [], allTokens = this.container.find(".ztokenfield__token"); // No I18N 
			var removedTokenData = []
			for(var i=0; i<allTokens.length;i++){
				var data = $(allTokens[i]).data().metadata, jsonData = {};
				for(var key in this._dataMapping){
					this._dataMapping[key] !== undefined && (jsonData[this._dataMapping[key]] = data[this._dataMapping[key]]);
				}
				removedTokenData[i] = jsonData;
			}
			this._dispatchEvent("BEFORETOKENREMOVE", {tokensToBeRemoved: removedTokenData});  // No I18N
			allTokens.remove();
			this._dispatchEvent("AFTERTOKENREMOVE", {tokensRemoved: removedTokenData});  // No I18N
			this._tokensCollection = [];
			this.options.values = [];
			this._tokensCount = 0;
			//this._input.focus();
		},
		/** Indicates whether any error token exists inside the tokenfield container.
			This signature doesn't accept any arguments.
			@function token-field.hasInvalidTokens
			@return {boolean} value.
			@example <caption> Example </caption>
	var tokens = $(".selector").ztokenfield("hasInvalidTokens"); */
		hasInvalidTokens: function(){
			var hasErrorToken;
			if(this.options.dataValidation){
				for(var i=0; i<this.options.values.length;i++){
					if(this.options.values[i].type !== undefined && this.options.values[i].type === "error"){
						hasErrorToken = true;
						break;		
					}else{
						hasErrorToken = false;
					}
				}
				if(hasErrorToken){
					return true;
				}else{
					return false;
				}
			}else{
				return false;
			}
		},
		_disableContainer: function(){
			this.element.attr("disabled", true);	//No I18N
			this.container.addClass("is-disabled").attr("aria-disabled", "true");	//No I18N
			this.element.off("."+this.componentName);	//No I18N
			this.container.off("."+this.componentName).off(".zsortable").find("*").off("."+this.componentName).off(".zsortable");	//No I18N
			this._input.attr("disabled", true);	//No I18N
		},
		search: function(query){
			this._input[0].value = query;
			this._resizeSearchArea(this._input);
			this._searchTimeout(query, this._input);
		},
		/** Gets the value currently associated with the specified optionName.
			This signature doesn't accept any arguments.
			@function token-field.getAttribute
			@param {string} optionName - The name of the option to get.
			@return {string|object} value.
			@example <caption> Example </caption>
	var position = $(".selector").ztokenfield("getAttribute","placeholder");*/
		/** Sets the value of the option associated with the specified optionName.
			@function token-field.setAttribute
			@param {string} optionName - The name of the option to get.
			@param {string} value - A value to set for the option.
			@example <caption> Example </caption>
	$(".selector").ztokenfield("setAttribute","placeholder","Type a country"); */
		/** Gets an object containing key/value pairs representing the current tokenfield options.
			This signature doesn't accept any arguments.
			@function token-field.getAttributes
			@return {object} options object.
			@example <caption> Example </caption>
	var options = $(".selector").ztokenfield("getAttributes");*/
		/** Sets one or more options for the tokenfield.
			@function token-field.setAttributes
			@param {Object} options - A map of option-value pairs to set.
			@example <caption> Example </caption>
	$(".selector").ztokenfield("setAttributes",{"placeholder": "Type a country", "lazyLoading":true }); */									
		setAttribute: function(optionName, value){
			var optionsList = ["dataMapping", "lazyLoadData", "search", "dataSource", "autoComplete", "dropdownList", "isImageSourceSprited"];  //No I18N
			if(typeof value === "object" && !(value instanceof Array)){  // No I18N
				value = $.extend(true, {}, this.options[optionName], value);
			}
			var oldValue = this.options[optionName];
			this.options[optionName] = value;
			if(optionName === 'className'){	//No I18N
				oldValue && this.container.removeClass(oldValue);
				this.container.addClass(value);
			}else if(optionName === "values"){  	// No I18N
				this._defaultSelected = true;
				this.addTokens(this.options.values);
				this._defaultSelected = false;
			}else if(optionName === "placeholder"){	// No I18N
				if(value){
					this._input.attr("placeholder", value);	// No I18N	
					this._resizeSearchArea(this._input, this._input.attr("placeholder"));	// No I18N
				}else{
					this._input.removeAttr("placeholder");	// No I18N
				}
			}else if(optionName === "tokens"){	// No I18N
				if(typeof value === "object"){	// No I18N
					if(value.showRemoveButtonOnHover){
						this.container.removeClass("ztokenfield--closeshown");
					}else if(!value.showRemoveButtonOnHover){
						this.container.addClass("ztokenfield--closeshown");
					}
					if(oldValue.sortable){
						var data = this.container.data("zsortable");	//No I18N
						data && this.container.zsortable("destroy");	// No I18N
					}
					if(value.sortable && $.fn.zsortable){
						this.container.zsortable({leave: "ztokenfield__close", "container": "parent"});	//No I18N
					}
				}
			}else if(optionName === "disabled"){	// No I18N
				if(value){
					this._disableContainer();
				}else{
					if(this.element.attr("disabled")){	// No I18N
						this.element.removeAttr("disabled");	//No I18N
						this.container.removeClass("is-disabled");	//No I18N
						this._bindEvents();
						var base = this;
						//Bind removed events
						this.container.on("click."+this.componentName, '.ztokenfield__close', function(orgEvent){	//No I18N
						  	base._removeToken($(orgEvent.currentTarget), orgEvent);
						})
						//Enable sortable if disabled
						if(this.options.tokens.sortable){
							var data = this.container.data("zsortable");	// No I18N
							data && this.container.zsortable("enable");	// No I18N
						}
						this._input.removeAttr("disabled");	//No I18N
						this.container.focus();
						this.container.attr("aria-disabled", "false");		// No I18N
					}
				}
			}else if(optionsList.indexOf(optionName) > -1){	
				this._super(optionName, value);
			}else{
				this._super(optionName, value);
			}		
		},
		/** Removes the tokenfield functionality for the element completely. This will return the element back to its pre-init state.
			This method does not accept any arguments.
			@function token-field.destroy
			@example <caption> Example </caption>
	$(".selector").ztokenfield("destroy");*/
		destroy: function(){
			var componentName = this.componentName;
			this.container.off("."+componentName).off(".zsortable").find("*").off("."+componentName).off(".zsortable").remove(); 	//No I18N
			this.removeAllTokens();
			this._isLoaderInited = false;
			this._menu.zmenu("destroy");	// No I18N
			this._menu.remove();
			this.element.off("."+componentName);	//No I18N
			ZC.$document.off("."+componentName);	//No I18N
			this.container.remove();
			this.options.disabled && this.element.removeAttr("disabled");	//No I18N
			this.element.removeData(componentName).show();
		}
	}
	ZC.registerComponent("ztokenfield", $.zsuggestfield, tokenfieldPrototype);  // No I18N
	ZC.createTokenfield = function(initObject){
		var element = $("<div>").attr({id: initObject.id});	
		element.appendTo("body");	// No I18N
		return element.ztokenfield(initObject);
	}
})(ZComponents);
